(* ::Package:: *)

Make2DTrajectory[ti_,Ri_List,Vi_List,segments___]:=IterateMake2DTrajectory[{},ti,Ri,Vi,segments]
IterateMake2DTrajectory[PVL_List,ti_,Ri_List,Vi_List]:=
(PositionVector[t_]=Piecewise[{#[[1]][t],#[[2,1]]<=t<=#[[2,2]]}&/@PVL];ti)
IterateMake2DTrajectory[PVL_List,ti_,Ri_List,Vi_List,StraightPath[L_,a_],segments___]:=
With[{vi=Norm[Vi]},
If[vi^2+2 a L<=0,
Print["Error: Speed becomes zero or negative."],
With[{UnitVi=Vi/vi,vf=Sqrt[vi^2+2 a L]},
With[{tf=ti+If[a==0,L/vi,(vf-vi)/a]},
IterateMake2DTrajectory[Append[PVL,{Ri+UnitVi With[{\[CapitalDelta]t=#-ti},vi \[CapitalDelta]t+1/2 a \[CapitalDelta]t^2]&,{ti,tf}}],tf,Ri+L UnitVi,vf UnitVi,segments]
]]]]
IterateMake2DTrajectory[PVL_List,ti_,Ri_List,Vi_List,CircularPath[Rad_,\[Theta]_,a_],segments___]:=
With[{vi=Norm[Vi]},With[{UnitVi=Vi/vi},
If[vi^2+2 a Rad \[Theta]<=0,
Print["Error: Speed becomes zero or negative."],
With[{UnitRadial=-Sign[Rad] {-UnitVi[[2]],UnitVi[[1]]},vf=Sqrt[vi^2+2 a Rad \[Theta]]},
With[{RCenter=Ri-Abs[Rad] UnitRadial,tf=ti+If[a==0,Abs[Rad \[Theta]]/vi,(vf-vi)/a]},
With[{pos=RCenter+With[{\[CapitalDelta]t=#-ti},Abs[Rad] With[{\[Phi]=1/Abs[Rad] (vi \[CapitalDelta]t+1/2 a \[CapitalDelta]t^2)},Cos[\[Phi]] UnitRadial+Sin[\[Phi]] UnitVi]]&},IterateMake2DTrajectory[Append[PVL,{pos,{ti,tf}}],tf,pos[tf],vf RotationMatrix[Sign[Rad] \[Theta]].UnitVi,segments]]
]]]]]


Options[Kinematics2D]={PlotRange->5,ArrowheadsScale->1,ArrowThicknessScale->1,VelocityVectorScale->1,AccelerationVectorScale->1,PositionPointSize->.02,PositionPointColor->Black,PositionVectorColor->Red,DeltaPositionVectorColor->RGBColor[.6,.6,0],VelocityVectorColor->Blue,DeltaVelocityVectorColor->RGBColor[.8,.4,0],AccelerationVectorColor->RGBColor[0,.6,0],AverageVelocityVectorColor->Black,AverageAccelerationVectorColor->Black,TimeClipping->.001};


ManipulateTrajectory[R_,ti_,tf_,OptionsPattern[Kinematics2D]]:=
Manipulate[With[{MainGraph=Graphics[{ParametricPlot[R[t],{t,0,tFinal},AxesOrigin->{0,0},AxesLabel->{x,y}]//First,If[ShowPositionVector,{OptionValue[PositionVectorColor],Arrowheads[.05 OptionValue[ArrowheadsScale]],Thickness[.01 OptionValue[ArrowThicknessScale]],
Arrow[{{0,0},R[t]}]},{}],
If[ShowVelocityVector,{OptionValue[VelocityVectorColor],Arrowheads[.05 OptionValue[ArrowheadsScale]],Thickness[.01 OptionValue[ArrowThicknessScale]],
Arrow[{R[t],R[t]+OptionValue[VelocityVectorScale]R'[t]}]},{}],
If[ShowAccelerationVector,{OptionValue[AccelerationVectorColor],Arrowheads[.05 OptionValue[ArrowheadsScale]],Thickness[.01 OptionValue[ArrowThicknessScale]],
Arrow[{R[t],R[t]+OptionValue[AccelerationVectorScale]R''[t]}]},{}],
If[ShowPoint,{OptionValue[PositionPointColor],PointSize[OptionValue[PositionPointSize]],Point[R[t]]},{}]
},
Axes->True,AxesLabel->{x,y},PlotRange->OptionValue[PlotRange]
]},
If[ShowComponentTimeGraphs,
GraphicsColumn[{MainGraph,
GraphicsColumn[{
Plot[R[t][[1]],{t,ti,tf},Epilog->{OptionValue[PositionPointColor],PointSize[OptionValue[PositionPointSize]],Point[{t,R[t][[1]]}]},AxesLabel->{"t","x"},AspectRatio->.3],Plot[R[t][[2]],{t,ti,tf},Epilog->{OptionValue[PositionPointColor],PointSize[OptionValue[PositionPointSize]],Point[{t,R[t][[2]]}]},AxesLabel->{"t","y"},AspectRatio->.3]}]}],
MainGraph]
],
{{ShowPoint,True,Style["Show Position?",OptionValue[PositionPointColor],Bold]},{True,False}},
{{ShowComponentTimeGraphs,False,Style["Show x vs. t and y vs. t Graphs?",Bold]},{True,False}},
{{ShowPositionVector,False,Style["Show Position Vector?",OptionValue[PositionVectorColor],Bold]},{True,False}},
{{ShowVelocityVector,False,Style["Show Velocity Vector?",OptionValue[VelocityVectorColor],Bold]},{True,False}},
{{ShowAccelerationVector,False,Style["Show Acceleration Vector?",OptionValue[AccelerationVectorColor],Bold]},{True,False}},
{t,ti+OptionValue[TimeClipping],tf-OptionValue[TimeClipping]}]


ManipulateAverageVelocity[R_,ti_,tf_,OptionsPattern[Kinematics2D]]:=
Manipulate[Graphics[{ParametricPlot[R[t],{t,0,tFinal},AxesOrigin->{0,0},AxesLabel->{x,y}]//First,If[ShowPositionVectors,{OptionValue[PositionVectorColor],Arrowheads[.05 OptionValue[ArrowheadsScale]],Thickness[.01 OptionValue[ArrowThicknessScale]],
Arrow[{{0,0},R[t]}],Arrow[{{0,0},R[t2]}]},{}],If[ShowDeltaPositionVector,{OptionValue[DeltaPositionVectorColor],Arrowheads[.05 OptionValue[ArrowheadsScale]],Thickness[.01 OptionValue[ArrowThicknessScale]],
Arrow[{R[t],R[t2]}]},{}],
If[ShowAverageVelocityVector,{OptionValue[AverageVelocityVectorColor],Arrowheads[.05 OptionValue[ArrowheadsScale]],Thickness[.01 OptionValue[ArrowThicknessScale]],
If[t2==t,Arrow[{R[t],R[t]+OptionValue[VelocityVectorScale]R'[t]}],Arrow[{R[t],R[t]+OptionValue[VelocityVectorScale](R[t2]-R[t])/(t2-t)}]]},{}],
If[ShowVelocityVector,{OptionValue[VelocityVectorColor],Arrowheads[.05 OptionValue[ArrowheadsScale]],Thickness[.01 OptionValue[ArrowThicknessScale]],
Arrow[{R[t],R[t]+OptionValue[VelocityVectorScale]R'[t]}]},{}],
If[ShowPoints,{OptionValue[PositionPointColor],PointSize[OptionValue[PositionPointSize]],Point[R[t]],Point[R[t2]]},{}]
},
Axes->True,AxesLabel->{x,y},PlotRange->OptionValue[PlotRange]
]
,
{{ShowPoints,True,Style["Show Positions?",OptionValue[PositionPointColor],Bold]},{True,False}},
{{ShowPositionVectors,False,Style["Show Both Position Vectors?",OptionValue[PositionVectorColor],Bold]},{True,False}},{{ShowDeltaPositionVector,False,Style["Show \[CapitalDelta]\[VeryThinSpace]\!\(\*OverscriptBox[\"r\", \"\[RightVector]\"]\) Vector?",OptionValue[AccelerationVectorColor],Bold]},{True,False}},
{{ShowAverageVelocityVector,False,Style["Show Average Velocity Vector?",OptionValue[AverageVelocityVectorColor],Bold]},{True,False}},
{{ShowVelocityVector,False,Style["Show Velocity Vector at t?",OptionValue[VelocityVectorColor],Bold]},{True,False}},
{{t,ti+(tf-ti)/3},ti+OptionValue[TimeClipping],tf-OptionValue[TimeClipping]},{{t2,tf-(tf-ti)/3,"t+\[CapitalDelta]t"},ti+OptionValue[TimeClipping],tf-OptionValue[TimeClipping]}]
