(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    588554,      18337]
NotebookOptionsPosition[    580562,      18099]
NotebookOutlinePosition[    581645,      18139]
CellTagsIndexPosition[    581602,      18136]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Chapter K", "Title",
 TextAlignment->Center,
 TextJustification->0],

Cell["Geometric Optics", "Subtitle",
 TextAlignment->Center,
 TextJustification->0],

Cell[CellGroupData[{

Cell["Properties of Light", "Section"],

Cell[CellGroupData[{

Cell["The Speed of Light", "Subsection"],

Cell[TextData[{
 "\tThe speed of light in a vacuum is approximately ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "\[TildeEqual]", 
    RowBox[{"3.0", "\[Times]", 
     SuperscriptBox["10", "8"], " ", 
     RowBox[{
      StyleBox["m",
       FontSlant->"Plain"], "/", 
      StyleBox["s",
       FontSlant->"Plain"]}]}]}], TraditionalForm]]],
 ".  Because of its most fundamental nature we choose our units to make ",
 StyleBox["c",
  FontSlant->"Italic"],
 " have the exact value"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "=", 
    RowBox[{"2.99792458", "\[Times]", 
     SuperscriptBox["10", "8"], " ", 
     RowBox[{
      StyleBox["m",
       FontSlant->"Plain"], "/", 
      StyleBox["s",
       FontSlant->"Plain"]}]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "The definition of a second is given in terms of our most accurate method of \
measuring time, with atomic clocks, and the above definition of ",
 StyleBox["c",
  FontSlant->"Italic"],
 " then gives a definition of the meter."
}], "Text"],

Cell[TextData[{
 "\tHistorically, the first good estimate of the speed of light was made by \
Roemer.  By plotting the period of Io, one of Jupiter's moons, Roemer was \
able to explain the lack of periodicity in Io's orbit in terms of the time \
difference for light from Io to reach the Earth as the relative distance \
between Io and Earth varies.  An accurate measure of ",
 StyleBox["c",
  FontSlant->"Italic"],
 " can be found by shining a light through a rapidly rotating toothed wheel \
and reflecting it off a distant mirror.  "
}], "Text"],

Cell[TextData[{
 "\tThe speed of light is a fundamental speed limit.  It is impossible to \
send any information faster than ",
 StyleBox["c",
  FontSlant->"Italic"],
 ".  "
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Wave Fronts and Rays", "Subsection"],

Cell["\<\
\tThere are two ways we can visualize waves, as wave fronts or as rays.  If \
we mark the crest of each wave then these form the wave fronts.  For a plane \
wave the crest corresponds to parallel planes moving at the wave speed  \
perpendicular to the plane.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Plane Waves", "Subsubsection"],

Cell[TextData[Cell[BoxData[
 FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.30081 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0625242 0.154859 0.108401 0.154859 [
[.44967 .03097 -37.875 -8 ]
[.44967 .03097 37.875 8 ]
[.97619 .57298 -15.9688 -8.03125 ]
[.97619 .57298 15.9688 8.03125 ]
[ 0 0 0 0 ]
[ 1 1.30081 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.02381 1.19241 m
.02551 1.19776 L
.02722 1.20308 L
.02892 1.20835 L
.03062 1.21355 L
.03233 1.21864 L
.03403 1.22361 L
.03573 1.22843 L
.03744 1.23308 L
.03914 1.23753 L
.04084 1.24177 L
.04255 1.24577 L
.04425 1.24952 L
.04595 1.25299 L
.04766 1.25618 L
.04936 1.25906 L
.05106 1.26162 L
.05277 1.26385 L
.05447 1.26575 L
.05617 1.26729 L
.05788 1.26847 L
.05958 1.26929 L
.06129 1.26974 L
.06299 1.26983 L
.06469 1.26954 L
.0664 1.26889 L
.0681 1.26787 L
.0698 1.26649 L
.07151 1.26476 L
.07321 1.26268 L
.07491 1.26026 L
.07662 1.25753 L
.07832 1.25448 L
.08002 1.25113 L
.08173 1.24751 L
.08343 1.24362 L
.08513 1.23948 L
.08684 1.23513 L
.08854 1.23056 L
.09024 1.22582 L
.09195 1.22092 L
.09365 1.21588 L
.09535 1.21072 L
.09706 1.20548 L
.09876 1.20018 L
.10046 1.19484 L
.10217 1.18949 L
.10387 1.18416 L
.10557 1.17886 L
.10728 1.17363 L
Mistroke
.10898 1.16848 L
.11069 1.16346 L
.11239 1.15857 L
.11409 1.15384 L
.1158 1.14929 L
.1175 1.14495 L
.1192 1.14084 L
.12091 1.13698 L
.12261 1.13338 L
.12431 1.13006 L
.12602 1.12704 L
.12772 1.12433 L
.12942 1.12194 L
.13113 1.1199 L
.13283 1.11819 L
.13453 1.11685 L
.13624 1.11586 L
.13794 1.11524 L
.13964 1.11499 L
.14135 1.11511 L
.14305 1.11559 L
.14475 1.11645 L
.14646 1.11766 L
.14816 1.11924 L
.14986 1.12116 L
.15157 1.12342 L
.15327 1.12601 L
.15497 1.12892 L
.15668 1.13214 L
.15838 1.13564 L
.16009 1.13941 L
.16179 1.14343 L
.16349 1.14769 L
.1652 1.15216 L
.1669 1.15683 L
.1686 1.16166 L
.17031 1.16664 L
.17201 1.17175 L
.17371 1.17695 L
.17542 1.18222 L
.17712 1.18755 L
.17882 1.1929 L
.18053 1.19824 L
.18223 1.20356 L
.18393 1.20883 L
.18564 1.21401 L
.18734 1.2191 L
.18904 1.22405 L
.19075 1.22886 L
.19245 1.23349 L
Mistroke
.19415 1.23792 L
.19586 1.24214 L
.19756 1.24612 L
.19926 1.24984 L
.20097 1.25329 L
.20267 1.25645 L
.20437 1.2593 L
.20608 1.26184 L
.20778 1.26404 L
.20949 1.2659 L
.21119 1.26741 L
.21289 1.26856 L
.2146 1.26935 L
.2163 1.26977 L
.218 1.26982 L
.21971 1.2695 L
.22141 1.26881 L
.22311 1.26776 L
.22482 1.26635 L
.22652 1.26458 L
.22822 1.26247 L
.22993 1.26003 L
.23163 1.25726 L
.23333 1.25418 L
.23504 1.25081 L
.23674 1.24716 L
.23844 1.24325 L
.24015 1.2391 L
.24185 1.23472 L
.24355 1.23014 L
.24526 1.22538 L
.24696 1.22046 L
.24866 1.21541 L
.25037 1.21025 L
.25207 1.205 L
.25377 1.1997 L
.25548 1.19436 L
.25718 1.18901 L
.25889 1.18367 L
.26059 1.17838 L
.26229 1.17316 L
.264 1.16802 L
.2657 1.16301 L
.2674 1.15813 L
.26911 1.15342 L
.27081 1.14889 L
.27251 1.14457 L
.27422 1.14048 L
.27592 1.13664 L
.27762 1.13306 L
Mistroke
.27933 1.12977 L
.28103 1.12678 L
.28273 1.1241 L
.28444 1.12174 L
.28614 1.11973 L
.28784 1.11806 L
.28955 1.11674 L
.29125 1.11579 L
.29295 1.1152 L
.29466 1.11498 L
.29636 1.11514 L
.29806 1.11566 L
.29977 1.11654 L
.30147 1.11779 L
.30317 1.1194 L
.30488 1.12135 L
.30658 1.12364 L
.30828 1.12627 L
.30999 1.1292 L
.31169 1.13244 L
.3134 1.13597 L
.3151 1.13976 L
.3168 1.14381 L
.31851 1.14809 L
.32021 1.15258 L
.32191 1.15726 L
.32362 1.16211 L
.32532 1.1671 L
.32702 1.17222 L
.32873 1.17743 L
.33043 1.18271 L
.33213 1.18804 L
.33384 1.19338 L
.33554 1.19873 L
.33724 1.20404 L
.33895 1.2093 L
.34065 1.21448 L
.34235 1.21955 L
.34406 1.2245 L
.34576 1.22929 L
.34746 1.2339 L
.34917 1.23832 L
.35087 1.24251 L
.35257 1.24647 L
.35428 1.25017 L
.35598 1.25359 L
.35768 1.25672 L
.35939 1.25955 L
.36109 1.26205 L
.3628 1.26422 L
Mistroke
.3645 1.26605 L
.3662 1.26753 L
.36791 1.26865 L
.36961 1.2694 L
.37131 1.26979 L
.37302 1.2698 L
.37472 1.26945 L
.37642 1.26873 L
.37813 1.26764 L
.37983 1.2662 L
.38153 1.2644 L
.38324 1.26226 L
.38494 1.25979 L
.38664 1.25699 L
.38835 1.25389 L
.39005 1.25049 L
.39175 1.24682 L
.39346 1.24288 L
.39516 1.23871 L
.39686 1.23431 L
.39857 1.22971 L
.40027 1.22494 L
.40197 1.22001 L
.40368 1.21495 L
.40538 1.20978 L
.40708 1.20452 L
.40879 1.19921 L
.41049 1.19387 L
.4122 1.18852 L
.4139 1.18319 L
.4156 1.1779 L
.41731 1.17269 L
.41901 1.16756 L
.42071 1.16256 L
.42242 1.15769 L
.42412 1.153 L
.42582 1.14849 L
.42753 1.14419 L
.42923 1.14012 L
.43093 1.1363 L
.43264 1.13275 L
.43434 1.12949 L
.43604 1.12652 L
.43775 1.12387 L
.43945 1.12155 L
.44115 1.11956 L
.44286 1.11792 L
.44456 1.11664 L
.44626 1.11572 L
.44797 1.11517 L
Mistroke
.44967 1.11498 L
.45137 1.11517 L
.45308 1.11572 L
.45478 1.11664 L
.45648 1.11792 L
.45819 1.11956 L
.45989 1.12155 L
.4616 1.12387 L
.4633 1.12652 L
.465 1.12949 L
.46671 1.13275 L
.46841 1.1363 L
.47011 1.14012 L
.47182 1.14419 L
.47352 1.14849 L
.47522 1.153 L
.47693 1.15769 L
.47863 1.16256 L
.48033 1.16756 L
.48204 1.17269 L
.48374 1.1779 L
.48544 1.18319 L
.48715 1.18852 L
.48885 1.19387 L
.49055 1.19921 L
.49226 1.20452 L
.49396 1.20978 L
.49566 1.21495 L
.49737 1.22001 L
.49907 1.22494 L
.50077 1.22971 L
.50248 1.23431 L
.50418 1.23871 L
.50588 1.24288 L
.50759 1.24682 L
.50929 1.25049 L
.51099 1.25389 L
.5127 1.25699 L
.5144 1.25979 L
.51611 1.26226 L
.51781 1.2644 L
.51951 1.2662 L
.52122 1.26764 L
.52292 1.26873 L
.52462 1.26945 L
.52633 1.2698 L
.52803 1.26979 L
.52973 1.2694 L
.53144 1.26865 L
.53314 1.26753 L
Mistroke
.53484 1.26605 L
.53655 1.26422 L
.53825 1.26205 L
.53995 1.25955 L
.54166 1.25672 L
.54336 1.25359 L
.54506 1.25017 L
.54677 1.24647 L
.54847 1.24251 L
.55017 1.23832 L
.55188 1.2339 L
.55358 1.22929 L
.55528 1.2245 L
.55699 1.21955 L
.55869 1.21448 L
.56039 1.2093 L
.5621 1.20404 L
.5638 1.19873 L
.56551 1.19338 L
.56721 1.18804 L
.56891 1.18271 L
.57062 1.17743 L
.57232 1.17222 L
.57402 1.1671 L
.57573 1.16211 L
.57743 1.15726 L
.57913 1.15258 L
.58084 1.14809 L
.58254 1.14381 L
.58424 1.13976 L
.58595 1.13597 L
.58765 1.13244 L
.58935 1.1292 L
.59106 1.12627 L
.59276 1.12364 L
.59446 1.12135 L
.59617 1.1194 L
.59787 1.11779 L
.59957 1.11654 L
.60128 1.11566 L
.60298 1.11514 L
.60468 1.11498 L
.60639 1.1152 L
.60809 1.11579 L
.60979 1.11674 L
.6115 1.11806 L
.6132 1.11973 L
.61491 1.12174 L
.61661 1.1241 L
.61831 1.12678 L
Mistroke
.62002 1.12977 L
.62172 1.13306 L
.62342 1.13664 L
.62513 1.14048 L
.62683 1.14457 L
.62853 1.14889 L
.63024 1.15342 L
.63194 1.15813 L
.63364 1.16301 L
.63535 1.16802 L
.63705 1.17316 L
.63875 1.17838 L
.64046 1.18367 L
.64216 1.18901 L
.64386 1.19436 L
.64557 1.1997 L
.64727 1.205 L
.64897 1.21025 L
.65068 1.21541 L
.65238 1.22046 L
.65408 1.22538 L
.65579 1.23014 L
.65749 1.23472 L
.65919 1.2391 L
.6609 1.24325 L
.6626 1.24716 L
.66431 1.25081 L
.66601 1.25418 L
.66771 1.25726 L
.66942 1.26003 L
.67112 1.26247 L
.67282 1.26458 L
.67453 1.26635 L
.67623 1.26776 L
.67793 1.26881 L
.67964 1.2695 L
.68134 1.26982 L
.68304 1.26977 L
.68475 1.26935 L
.68645 1.26856 L
.68815 1.26741 L
.68986 1.2659 L
.69156 1.26404 L
.69326 1.26184 L
.69497 1.2593 L
.69667 1.25645 L
.69837 1.25329 L
.70008 1.24984 L
.70178 1.24612 L
.70348 1.24214 L
Mistroke
.70519 1.23792 L
.70689 1.23349 L
.70859 1.22886 L
.7103 1.22405 L
.712 1.2191 L
.7137 1.21401 L
.71541 1.20883 L
.71711 1.20356 L
.71882 1.19824 L
.72052 1.1929 L
.72222 1.18755 L
.72393 1.18222 L
.72563 1.17695 L
.72733 1.17175 L
.72904 1.16664 L
.73074 1.16166 L
.73244 1.15683 L
.73415 1.15216 L
.73585 1.14769 L
.73755 1.14343 L
.73926 1.13941 L
.74096 1.13564 L
.74266 1.13214 L
.74437 1.12892 L
.74607 1.12601 L
.74777 1.12342 L
.74948 1.12116 L
.75118 1.11924 L
.75288 1.11766 L
.75459 1.11645 L
.75629 1.11559 L
.75799 1.11511 L
.7597 1.11499 L
.7614 1.11524 L
.7631 1.11586 L
.76481 1.11685 L
.76651 1.11819 L
.76822 1.1199 L
.76992 1.12194 L
.77162 1.12433 L
.77333 1.12704 L
.77503 1.13006 L
.77673 1.13338 L
.77844 1.13698 L
.78014 1.14084 L
.78184 1.14495 L
.78355 1.14929 L
.78525 1.15384 L
.78695 1.15857 L
.78866 1.16346 L
Mistroke
.79036 1.16848 L
.79206 1.17363 L
.79377 1.17886 L
.79547 1.18416 L
.79717 1.18949 L
.79888 1.19484 L
.80058 1.20018 L
.80228 1.20548 L
.80399 1.21072 L
.80569 1.21588 L
.80739 1.22092 L
.8091 1.22582 L
.8108 1.23056 L
.8125 1.23513 L
.81421 1.23948 L
.81591 1.24362 L
.81762 1.24751 L
.81932 1.25113 L
.82102 1.25448 L
.82273 1.25753 L
.82443 1.26026 L
.82613 1.26268 L
.82784 1.26476 L
.82954 1.26649 L
.83124 1.26787 L
.83295 1.26889 L
.83465 1.26954 L
.83635 1.26983 L
.83806 1.26974 L
.83976 1.26929 L
.84146 1.26847 L
.84317 1.26729 L
.84487 1.26575 L
.84657 1.26385 L
.84828 1.26162 L
.84998 1.25906 L
.85168 1.25618 L
.85339 1.25299 L
.85509 1.24952 L
.85679 1.24577 L
.8585 1.24177 L
.8602 1.23753 L
.8619 1.23308 L
.86361 1.22843 L
.86531 1.22361 L
.86702 1.21864 L
.86872 1.21355 L
.87042 1.20835 L
.87213 1.20308 L
.87383 1.19776 L
Mistroke
.87553 1.19241 L
Mfstroke
.7 0 0 r
.03 w
.06252 1.26984 Mdot
.21738 1.26984 Mdot
.37224 1.26984 Mdot
.5271 1.26984 Mdot
.68196 1.26984 Mdot
.83682 1.26984 Mdot
.01 w
.06252 .1084 m
.06252 1.03755 L
s
.21738 .1084 m
.21738 1.03755 L
s
.37224 .1084 m
.37224 1.03755 L
s
.5271 .1084 m
.5271 1.03755 L
s
.68196 .1084 m
.68196 1.03755 L
s
.83682 .1084 m
.83682 1.03755 L
s
0 0 .7 r
.02381 .17034 m
.87553 .17034 L
s
.43419 .18583 m
.46516 .17034 L
.43419 .15486 L
s
.02381 .43877 m
.87553 .43877 L
s
.43419 .45425 m
.46516 .43877 L
.43419 .42328 L
s
.02381 .70719 m
.87553 .70719 L
s
.43419 .72267 m
.46516 .70719 L
.43419 .6917 L
s
.02381 .97561 m
.87553 .97561 L
s
.43419 .9911 m
.46516 .97561 L
.43419 .96012 L
s
.7 0 0 r
gsave
.44967 .03097 -98.875 -12 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 16.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.700 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.700 0.000 0.000 setrgbcolor
(Wave) show
99.062 16.000 moveto
(Fronts) show
134.750 16.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.700 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 .7 r
gsave
.97619 .57298 -76.9688 -12.0312 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.0625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 16.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
63.000 16.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
(Rays) show
90.938 16.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 1.30081 L
0 1.30081 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
   GeneratedCell->False,
   CellAutoOverwrite->False,
   ImageSize->{169.6875, 220.625},
   ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
   ImageRangeCache->{{{0, 338.375}, {440.25, 0}} -> {-0.403781, -1.09284, 
   0.0411425, 0.0411425}}], TraditionalForm]]]], "Text",
 CellChangeTimes->{{3.401219803832612*^9, 3.401219807280984*^9}},
 TextAlignment->Center]
}, Open  ]],

Cell[CellGroupData[{

Cell["Spherical Waves", "Subsubsection"],

Cell[TextData[Cell[BoxData[
 FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.01997 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.10582 0.542804 0.10582 [
[.5 .02429 -37.875 -8 ]
[.5 .02429 37.875 8 ]
[.87037 .91317 -15.9688 -8.03125 ]
[.87037 .91317 15.9688 8.03125 ]
[ 0 0 0 0 ]
[ 1 1.01997 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
.7 0 0 r
.007 w
[ ] 0 setdash
newpath
.5 .5428 .10582 0 365.73 arc
s
newpath
.5 .5428 .21164 0 365.73 arc
s
newpath
.5 .5428 .31746 0 365.73 arc
s
newpath
.5 .5428 .42328 0 365.73 arc
s
0 0 .7 r
.5 .5428 m
.97619 .5428 L
s
.75397 .55339 m
.77513 .5428 L
.75397 .53222 L
s
.5 .5428 m
.88525 .8227 L
s
.69924 .70064 m
.72259 .70452 L
.71168 .68352 L
s
.5 .5428 m
.64715 .99569 L
s
.56842 .78761 m
.58502 .80447 L
.58854 .78107 L
s
.5 .5428 m
.35285 .99569 L
s
.41146 .78107 m
.41498 .80447 L
.43158 .78761 L
s
.5 .5428 m
.11475 .8227 L
s
.28832 .68352 m
.27741 .70452 L
.30076 .70064 L
s
.5 .5428 m
.02381 .5428 L
s
.24603 .53222 m
.22487 .5428 L
.24603 .55339 L
s
.5 .5428 m
.11475 .26291 L
s
.30076 .38496 m
.27741 .38108 L
.28832 .40209 L
s
.5 .5428 m
.35285 .08992 L
s
.43158 .298 m
.41498 .28114 L
.41146 .30454 L
s
.5 .5428 m
.64715 .08992 L
s
.58854 .30454 m
.58502 .28114 L
.56842 .298 L
s
.5 .5428 m
.88525 .26291 L
s
.71168 .40209 m
.72259 .38108 L
.69924 .38496 L
s
.7 0 0 r
gsave
.5 .02429 -98.875 -12 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 16.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.700 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.700 0.000 0.000 setrgbcolor
(Wave) show
99.062 16.000 moveto
(Fronts) show
134.750 16.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.700 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 .7 r
gsave
.87037 .91317 -76.9688 -12.0312 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.0625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 16.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
63.000 16.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
(Rays) show
90.938 16.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 1.01997 L
0 1.01997 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
   GeneratedCell->False,
   CellAutoOverwrite->False,
   ImageSize->{237.1875, 241.75},
   ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
   ImageRangeCache->{{{0, 473.375}, {482.5, 0}} -> {-4.77397, -5.22273, 
   0.0403399, 0.0403399}}], TraditionalForm]]]], "Text",
 CellChangeTimes->{{3.401219809805319*^9, 3.401219813111434*^9}},
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Light in a Medium", "Subsection"],

Cell[TextData[{
 "\tWhen we refer to ",
 StyleBox["c",
  FontSlant->"Italic"],
 " as the speed of light it is implied that it is the speed in a vacuum.  In \
a medium light slows down.  It slows down by a factor called the index of \
refraction; this is a material dependent constant ",
 StyleBox["n",
  FontSlant->"Italic"],
 " defined by"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{"v", "=", 
    FractionBox["c", "n"]}], TraditionalForm]]],
 ","
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "where ",
 StyleBox["v",
  FontSlant->"Italic"],
 " is the speed in the medium.  Clearly ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[GreaterEqual]", "1"}], TraditionalForm]]],
 " and the equality applies to a vacuum."
}], "Text"],

Cell[TextData[{
 "\tWhen light hits an interface between indices ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]]],
 " both the frequency and wavelength cannot remain unchanged, since"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", " ", "\[Lambda]"}], "=", 
    RowBox[{"v", "=", 
     FractionBox["c", "n"]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
The frequency is the same on both sides of the interface.  This is easy to \
see.  At the interface the incoming radiation is varying at some frequency \
and generally driving a system at some frequency induces oscillations at the \
same frequency.  Given that the frequencies are equal\
\>", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{"f", "=", 
   RowBox[{
    SubscriptBox["f", "1"], "=", 
    SubscriptBox["f", "2"]}]}], TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
the wavelengths must change.  Since wavelength is proportional to the wave \
speed we get\
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     SubscriptBox["\[Lambda]", "2"], 
     SubscriptBox["\[Lambda]", "1"]], "=", 
    RowBox[{
     FractionBox[
      SubscriptBox["v", "2"], 
      SubscriptBox["v", "1"]], "=", 
     FractionBox[
      SubscriptBox["n", "1"], 
      SubscriptBox["n", "2"]]}]}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{"   ", 
    RowBox[{"\[DoubleLongRightArrow]", "   "}]}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["n", "1"], " ", 
     SubscriptBox["\[Lambda]", "1"]}], "=", 
    RowBox[{
     SubscriptBox["n", "2"], " ", 
     SubscriptBox["\[Lambda]", "2"]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "To view this a different way, if we define the vacuum wavelength as  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Lambda]", "0"], "=", 
    RowBox[{"c", "/", "f"}]}], TraditionalForm]]],
 "  then the wavelength in a medium with index ",
 StyleBox["n",
  FontSlant->"Italic"],
 " is"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Lambda]", "=", 
    FractionBox[
     SubscriptBox["\[Lambda]", "0"], "n"]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Refraction and Reflection", "Section"],

Cell["\<\
\tAs a consequence of the speed of light changing across an interface light \
will bend as it hits the interface.  This is called refraction.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Snell's Law", "Subsection"],

Cell[TextData[{
 "\tConsider light passing from one medium to another through a flat \
interface.  Take it to move from medium 1 with index ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " to medium 2 with index ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]]],
 ".  We will, as usual, measure the angles of the rays relative to the normal \
to the surface."
}], "Text"],

Cell[TextData[{
 " ",
 Cell[BoxData[
  FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .8375 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.5 0.3875 0.5 [
[ 0 0 0 0 ]
[ 1 .8375 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .8375 L
0 .8375 L
closepath
clip
newpath
.9 g
.5 -0.0625 m
.5 .8375 L
1 .8375 L
1 -0.0625 L
F
1 g
0 -0.0625 m
0 .8375 L
.5 .8375 L
.5 -0.0625 L
F
0 0 .7 r
.009 w
[ ] 0 setdash
.5 .5625 m
0 .79565 L
s
.19274 .72233 m
.21359 .69605 L
.18007 .69514 L
s
.5 .5625 m
1 .14295 L
s
.69815 .41581 m
.71149 .38504 L
.67887 .39283 L
s
.5 .2125 m
0 .44565 L
s
.19274 .37233 m
.21359 .34605 L
.18007 .34514 L
s
.5 .2125 m
0 0 rlineto s
.5 .2125 m
.75325 0 L
s
.69815 .06581 m
.71149 .03504 L
.67887 .04283 L
s
.7 0 0 r
.008 w
.5 .2125 m
0 0 rlineto s
.5 .2125 m
.40091 0 L
s
.5 .5625 m
0 0 rlineto s
.5 .5625 m
.73075 .8375 L
s
.5 .5625 m
0 0 rlineto s
.5 .5625 m
.2377 0 L
s
.5 .2125 m
1 .80838 L
s
.46503 .8375 m
.07449 0 L
s
.61538 0 m
1 .45838 L
s
.30182 .8375 m
0 .19025 L
s
.90906 0 m
1 .10838 L
s
.13861 .8375 m
0 .54025 L
s
0 g
.009 w
.5 .8375 m
.5 0 L
s
.5 Mabswid
[ .025 .025 ] 0 setdash
.25 .5625 m
.725 .5625 L
s
.005 w
[ ] 0 setdash
newpath
.5 .5625 .195 155 180 arc
s
gsave
.27545 .61228 -69.8438 -11.625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.25 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.500 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(q) show
69.125 18.250 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
76.688 14.500 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
newpath
.5 .5625 .145 -115 -90 arc
s
gsave
.45563 .36236 -69.8438 -11.625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.25 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.500 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(q) show
69.125 18.250 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
76.688 14.500 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
newpath
.5 .5625 .125 -40 0 arc
s
newpath
.5 .5625 .14 -40 0 arc
s
gsave
.67854 .49752 -69.8438 -11.625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.25 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.500 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(q) show
69.125 18.250 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
76.688 14.500 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
newpath
.5 .2125 .125 50 90 arc
s
newpath
.5 .2125 .14 50 90 arc
s
gsave
.56498 .39104 -69.8438 -11.625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.25 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.500 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(q) show
69.125 18.250 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
76.688 14.500 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.4 .8 -70.2812 -11.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.75 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(n) show
70.000 17.750 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
77.562 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.615 .8 -70.2812 -11.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.75 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(n) show
70.000 17.750 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
77.562 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.007 w
.71473 .6434 m
.66303 .68678 L
s
.62856 .71571 m
.6558 .67817 L
.67026 .6954 L
F
.71473 .6434 m
.76643 .60002 L
s
.8009 .5711 m
.77366 .60864 L
.7592 .5914 L
F
gsave
.74365 .67787 -70.375 -11.625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.25 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.500 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(l) show
70.188 18.250 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
77.750 14.500 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.36958 .10781 m
.39582 .09557 L
s
.43661 .07655 m
.40058 .10577 L
.39107 .08538 L
F
.36958 .10781 m
.34333 .12005 L
s
.30255 .13907 m
.33858 .10985 L
.34809 .13024 L
F
gsave
.35082 .06759 -70.375 -11.625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.25 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.500 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(l) show
70.188 18.250 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
77.750 14.500 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
    GeneratedCell->False,
    CellAutoOverwrite->False,
    ImageSize->{266., 222.5625},
    ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
    ImageRangeCache->{{{0, 531}, {444.125, 0}} -> {-1.00132, -0.775, 
    0.00754292, 0.00754292}}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.401219817118326*^9, 3.4012198198427353`*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
The diagram shows wave fronts and rays on either side of the interface.  \
Since the rays are perpendicular to the wave fronts, the angle between the \
rays and the normal are the same as the angle between the wave fronts and the \
interface.  Enlarging the triangles at the center\
\>", "Text"],

Cell[TextData[{
 " ",
 Cell[BoxData[
  FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.1423 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.440502 1.55415 0.57115 1.55415 [
[.19329 .04101 -9.375 -7.625 ]
[.19329 .04101 9.375 7.625 ]
[.77577 .97071 -9.375 -7.625 ]
[.77577 .97071 9.375 7.625 ]
[.35641 .73578 -8.84375 -7.625 ]
[.35641 .73578 8.84375 7.625 ]
[.5787 .40691 -8.84375 -7.625 ]
[.5787 .40691 8.84375 7.625 ]
[.36279 .54007 -7.8125 -7.21875 ]
[.36279 .54007 7.8125 7.21875 ]
[ 0 0 0 0 ]
[ 1 1.1423 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
.9 g
.4405 1.1151 m
.4405 .0272 L
.97619 .66561 L
F
1 g
.4405 1.1151 m
.4405 .0272 L
.02381 .2215 L
F
0 0 .7 r
.018 w
[ ] 0 setdash
.23216 .12435 m
.4405 .0272 L
s
.4405 .0272 m
.4405 .0272 L
.4405 .0272 L
F
.23216 .12435 m
.02381 .2215 L
s
.02381 .2215 m
.02381 .2215 L
.02381 .2215 L
F
gsave
.19329 .04101 -70.375 -11.625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.25 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
63.000 14.500 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
(l) show
70.188 18.250 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
(1) show
77.750 14.500 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.70835 .89036 m
.4405 1.1151 L
s
.4405 1.1151 m
.4405 1.1151 L
.4405 1.1151 L
F
.70835 .89036 m
.97619 .66561 L
s
.97619 .66561 m
.97619 .66561 L
.97619 .66561 L
F
gsave
.77577 .97071 -70.375 -11.625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.25 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
63.000 14.500 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
(l) show
70.188 18.250 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
(2) show
77.750 14.500 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.7 0 0 r
.015 w
.4405 1.1151 m
.02381 .2215 L
s
.4405 .0272 m
.97619 .66561 L
s
0 g
.018 w
.4405 1.1151 m
.4405 .0272 L
s
.12945 .17224 m
.17871 .27788 L
.07307 .32714 L
s
.8869 .74053 m
.81198 .65124 L
.90127 .57632 L
s
.5 Mabswid
newpath
.4405 1.1151 .27198 -115 -90 arc
s
gsave
.35641 .73578 -69.8438 -11.625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.25 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.500 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(q) show
69.125 18.250 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
76.688 14.500 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
newpath
.4405 .0272 .27975 50 90 arc
s
newpath
.4405 .0272 .31083 50 90 arc
s
gsave
.5787 .40691 -69.8438 -11.625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.25 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.500 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(q) show
69.125 18.250 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
76.688 14.500 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.36279 .54007 -68.8125 -11.2188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.4375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.438 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.125 15.438 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 16.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
74.625 15.438 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 1.1423 L
0 1.1423 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
    GeneratedCell->False,
    CellAutoOverwrite->False,
    ImageSize->{135., 154.3125},
    ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
    ImageRangeCache->{{{0, 269}, {307.625, 0}} -> {-0.2877, -0.378077, 
    0.00484733, 0.00484733}}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.401219822894373*^9, 3.401219827327104*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "and labelling the common side as ",
 StyleBox["D",
  FontSlant->"Italic"],
 " gives"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"sin", " ", 
     SubscriptBox["\[Theta]", "1"]}], "=", 
    FractionBox[
     SubscriptBox["\[Lambda]", "1"], "D"]}], TraditionalForm]]],
 " and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"sin", " ", 
     SubscriptBox["\[Theta]", "2"]}], "=", 
    FractionBox[
     SubscriptBox["\[Lambda]", "2"], "D"]}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{"\[DoubleLongRightArrow]", "  "}]}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{"sin", " ", 
      SubscriptBox["\[Theta]", "2"]}], 
     RowBox[{"sin", " ", 
      SubscriptBox["\[Theta]", "1"]}]], "=", 
    RowBox[{
     FractionBox[
      SubscriptBox["\[Lambda]", "2"], 
      SubscriptBox["\[Lambda]", "1"]], "=", 
     FractionBox[
      SubscriptBox["n", "1"], 
      SubscriptBox["n", "2"]]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
We can now write the law of refraction, called Snell's law, as\
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["n", "1"], " ", "sin", " ", 
     SubscriptBox["\[Theta]", "1"]}], "=", 
    RowBox[{
     SubscriptBox["n", "2"], " ", "sin", " ", 
     SubscriptBox["\[Theta]", "2"]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]],

Cell[CellGroupData[{

Cell["Incident, Reflected and Refracted Rays", "Subsection"],

Cell["\<\
\tIn addition to refraction some of the light is reflected.  The so-called \
law of reflection states that the angle, measured from the normal, of the \
reflected ray is the same as the incident.\
\>", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .99254 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.496446 0.355366 0.496269 0.355366 [
[.31839 .54569 -7.90625 -7.0625 ]
[.31839 .54569 7.90625 7.0625 ]
[.66223 .41464 -7.90625 -7.0625 ]
[.66223 .41464 7.90625 7.0625 ]
[.31839 .44684 -7.90625 -7.0625 ]
[.31839 .44684 7.90625 7.0625 ]
[.42537 .92271 -8.28125 -7.0625 ]
[.42537 .92271 8.28125 7.0625 ]
[.57818 .92271 -8.28125 -7.0625 ]
[.57818 .92271 8.28125 7.0625 ]
[.31876 .8161 -21.5938 -7 ]
[.31876 .8161 21.5938 7 ]
[.31876 .74502 -11.6562 -7 ]
[.31876 .74502 11.6562 7 ]
[.31876 .19421 -24.9062 -7 ]
[.31876 .19421 24.9062 7 ]
[.31876 .12313 -11.6562 -7 ]
[.31876 .12313 11.6562 7 ]
[.81628 .31859 -25.2188 -7 ]
[.81628 .31859 25.2188 7 ]
[.81628 .24751 -11.6562 -7 ]
[.81628 .24751 11.6562 7 ]
[ 0 0 0 0 ]
[ 1 .99254 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
.9 g
.49645 .02363 m
.49645 .96891 L
.97619 .96891 L
.97619 .02363 L
F
1 g
.02381 .02363 m
.02381 .96891 L
.49645 .96891 L
.49645 .02363 L
F
0 0 .7 r
.012 w
[ ] 0 setdash
.49645 .49627 m
.02381 .78056 L
s
.15589 .73429 m
.18996 .68062 L
.12659 .68557 L
s
.49645 .49627 m
.02381 .21198 L
s
.20462 .28756 m
.14124 .28261 L
.17531 .33628 L
s
.49645 .49627 m
.85181 .03429 L
s
.7504 .21275 m
.76254 .15035 L
.70534 .17809 L
s
0 g
.49645 .95824 m
.49645 .03429 L
s
.5 Mabswid
[ .05 .025 ] 0 setdash
.28323 .49627 m
.70967 .49627 L
s
.01 w
[ ] 0 setdash
newpath
.49645 .49627 .12438 148.973 180 arc
s
gsave
.31839 .54569 -68.9062 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(q) show
68.250 17.125 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
74.812 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
newpath
.49645 .49627 .0995 -52.4314 0 arc
s
newpath
.49645 .49627 .12082 -52.4314 0 arc
s
gsave
.66223 .41464 -68.9062 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(q) show
68.250 17.125 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
74.812 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
newpath
.49645 .49627 .14215 180 211.027 arc
s
gsave
.31839 .44684 -68.9062 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(q) show
68.250 17.125 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
74.812 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.42537 .92271 -69.2812 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(n) show
69.000 17.125 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
75.562 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.57818 .92271 -69.2812 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(n) show
69.000 17.125 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
75.562 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 .7 r
gsave
.31876 .8161 -82.5938 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
(Incident) show
102.188 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.31876 .74502 -72.6562 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
(Ray) show
82.312 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.31876 .19421 -85.9062 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
(Reflected) show
108.812 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.31876 .12313 -72.6562 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
(Ray) show
82.312 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.81628 .31859 -86.2188 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
(Refracted) show
109.438 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.81628 .24751 -72.6562 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
(Ray) show
82.312 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.700 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .99254 L
0 .99254 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
   GeneratedCell->False,
   CellAutoOverwrite->False,
   ImageSize->{155.8125, 154.6875},
   ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
   ImageRangeCache->{{{0, 310.625}, {308.375, 0}} -> {-1.39701, -1.39682, 
   0.0181185, 0.0181185}}], TraditionalForm]]]], "Text",
 CellChangeTimes->{{3.4012198305063267`*^9, 3.401219838320435*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
Note that in this example the angle in medium 2 is larger than in medium 1.  \
This implies, by Snell's law, that the index in 2 is smaller than in 1.\
\>", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Theta]", "1"], "<", 
   RowBox[{
    SubscriptBox["\[Theta]", "2"], "    ", "\[DoubleLongRightArrow]", "   ", 
    SubscriptBox["n", "1"]}], ">", 
   SubscriptBox["n", "2"]}], TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]],

Cell[CellGroupData[{

Cell["Total Internal Reflection", "Subsection"],

Cell["\<\
\tConsider the case where light moves across an interface to a region with \
lower index.  \
\>", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{" ", 
   RowBox[{
    SubscriptBox["n", "1"], ">", 
    SubscriptBox["n", "2"]}]}], TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "It follows that the angle from the normal increases (",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Theta]", "1"], "<", 
    SubscriptBox["\[Theta]", "2"]}], TraditionalForm]]],
 ").  Moreover, there is some angle where there is no refracted ray.  In this \
case there is only the reflected ray.  This is what we call total internal \
reflection.  This occurs at incident angles above some critical angle"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Theta]", "1"], "\[GreaterEqual]", 
    SubscriptBox["\[Theta]", "crit"]}], TraditionalForm]]],
 ","
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "where the critical angle occurs when the refracted angle goes to \
90\[Degree].  That is, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Theta]", "1"], "=", 
    SubscriptBox["\[Theta]", "crit"]}], TraditionalForm]]],
 "  when  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"sin", " ", 
     SubscriptBox["\[Theta]", "2"]}], "=", "1"}], TraditionalForm]]],
 ", or"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"sin", " ", 
     SubscriptBox["\[Theta]", "crit"]}], "=", 
    FractionBox[
     SubscriptBox["n", "2"], 
     SubscriptBox["n", "1"]]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]],

Cell[CellGroupData[{

Cell["Dispersion", "Subsection"],

Cell["\<\
\tA prism splits into the visible spectrum: red, orange, yellow, green, blue, \
indigo and violet.  When light hits a prism it refracts twice.  Using Snell's \
law and knowing the apex angle \[CapitalPhi], the angle between the two \
refracting surfaces, we can uniquely solve for the position of the ray \
leaving the prism in terms of the index of refraction of the prism.  The fact \
that different colors end up at different positions implies that the index of \
refraction varies with color (wavelength).  This is known as dispersion.\
\>", "Text"],

Cell["\<\
\tIn a prism, red is the color that refracts the least; this means the index \
of red is the least.  Red has the longest visible wavelength.  It is \
generally the case that the index of refraction decreases with wavelength.  \
For instance in the case of crown glass the tabulated index is 1.52.  This is \
a mean value.  As the wavelength increases from 400 to 700 nm, the index \
decreases from about 1.53 to 1.51.\
\>", "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Images from Spherical and Parabolic Mirrors", "Section"],

Cell[CellGroupData[{

Cell["Concave Mirrors", "Subsection"],

Cell[CellGroupData[{

Cell["Spherical Mirrors and Spherical Aberration", "Subsubsection"],

Cell["\<\
\tWhen light from infinity (a plane wave) parallel to the central axis of a \
concave spherical mirror reflects off the mirror, the rays near the central \
axis converge toward a point called the focal point, which we will see is at \
a position half the radius from the mirror..\
\>", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .86275 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0798319 0.0186741 0.431373 0.0186741 [
[.35994 .39402 -5.9375 -7.28125 ]
[.35994 .39402 5.9375 7.28125 ]
[.64006 .39402 -6.34375 -7.0625 ]
[.64006 .39402 6.34375 7.0625 ]
[ 0 0 0 0 ]
[ 1 .86275 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .7 r
.5 Mabswid
[ ] 0 setdash
.92017 .05789 m
.22249 .05789 L
s
.83419 .03922 m
.79684 .05789 L
.83419 .07656 L
s
.22249 .05789 m
.29511 .70744 L
s
.2724 .6724 m
.29511 .70744 L
.30952 .66825 L
s
.92017 .15126 m
.15489 .15126 L
s
.82405 .13259 m
.7867 .15126 L
.82405 .16993 L
s
.15489 .15126 m
.48169 .71729 L
s
.44684 .69428 m
.48169 .71729 L
.47918 .67561 L
s
.92017 .24463 m
.11187 .24463 L
s
.8176 .22596 m
.78025 .24463 L
.8176 .26331 L
s
.11187 .24463 m
.62022 .65544 L
s
.57944 .64649 m
.62022 .65544 L
.60291 .61744 L
s
.92017 .338 m
.08767 .338 L
s
.81397 .31933 m
.77662 .338 L
.81397 .35668 L
s
.08767 .338 m
.70495 .55282 L
s
.66354 .55818 m
.70495 .55282 L
.67582 .52291 L
s
.92017 .43137 m
.07983 .43137 L
s
.81279 .4127 m
.77544 .43137 L
.81279 .45005 L
s
.07983 .43137 m
.73343 .43137 L
s
.69608 .45005 m
.73343 .43137 L
.69608 .4127 L
s
.92017 .52474 m
.08767 .52474 L
s
.81397 .50607 m
.77662 .52474 L
.81397 .54342 L
s
.08767 .52474 m
.70495 .30993 L
s
.67582 .33984 m
.70495 .30993 L
.66354 .30456 L
s
.92017 .61811 m
.11187 .61811 L
s
.8176 .59944 m
.78025 .61811 L
.8176 .63679 L
s
.11187 .61811 m
.62022 .2073 L
s
.60291 .2453 m
.62022 .2073 L
.57944 .21625 L
s
.92017 .71148 m
.15489 .71148 L
s
.82405 .69281 m
.7867 .71148 L
.82405 .73016 L
s
.15489 .71148 m
.48169 .14545 L
s
.47918 .18714 m
.48169 .14545 L
.44684 .16846 L
s
.92017 .80486 m
.22249 .80486 L
s
.83419 .78618 m
.79684 .80486 L
.83419 .82353 L
s
.22249 .80486 m
.29511 .15531 L
s
.30952 .1945 m
.29511 .15531 L
.2724 .19035 L
s
0 g
.02381 .43137 m
.97619 .43137 L
s
.35994 .42204 m
.35994 .44071 L
s
.64006 .42204 m
.64006 .44071 L
s
gsave
.35994 .39402 -66.9375 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.64006 .39402 -67.3438 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.188 14.125 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
71.688 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02 w
.35994 .43137 Mdot
.01 w
newpath
.64006 .43137 .56022 132.833 227.167 arc
s
0 0 m
1 0 L
1 .86275 L
0 .86275 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
   GeneratedCell->False,
   CellAutoOverwrite->False,
   ImageSize->{259.625, 223.875},
   ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
   ImageRangeCache->{{{0, 518.25}, {446.75, 0}} -> {-4.29719, -23.1001, 
   0.206829, 0.206829}}], TraditionalForm]]]], "Text",
 CellChangeTimes->{{3.401219842519307*^9, 3.401219846228126*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
The rays far from the central axis miss the focal point.  This is called \
spherical aberration.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parabolic Mirrors", "Subsubsection"],

Cell["\<\
\tThe geometrical shape that causes all rays to converge to a single focal \
point is a paraboloid, which is a parabola under rotation.\
\>", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .86275 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0798319 0.0186741 0.431373 0.0186741 [
[.35994 .36601 -5.9375 -7.28125 ]
[.35994 .36601 5.9375 7.28125 ]
[ 0 0 0 0 ]
[ 1 .86275 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .7 r
.5 Mabswid
[ ] 0 setdash
.92017 .05789 m
.20433 .05789 L
s
.83147 .03922 m
.79412 .05789 L
.83147 .07656 L
s
.20433 .05789 m
.45571 .66121 L
s
.42411 .63392 m
.45571 .66121 L
.45858 .61955 L
s
.92017 .15126 m
.14986 .15126 L
s
.8233 .13259 m
.78595 .15126 L
.8233 .16993 L
s
.14986 .15126 m
.54202 .67414 L
s
.50467 .65546 m
.54202 .67414 L
.53455 .63305 L
s
.92017 .24463 m
.11096 .24463 L
s
.81746 .22596 m
.78011 .24463 L
.81746 .26331 L
s
.11096 .24463 m
.63383 .63679 L
s
.59275 .62932 m
.63383 .63679 L
.61516 .59944 L
s
.92017 .338 m
.08761 .338 L
s
.81396 .31933 m
.77661 .338 L
.81396 .35668 L
s
.08761 .338 m
.70588 .54998 L
s
.66449 .55553 m
.70588 .54998 L
.67661 .5202 L
s
.92017 .43137 m
.07983 .43137 L
s
.81279 .4127 m
.77544 .43137 L
.81279 .45005 L
s
.07983 .43137 m
.73343 .43137 L
s
.69608 .45005 m
.73343 .43137 L
.69608 .4127 L
s
.92017 .52474 m
.08761 .52474 L
s
.81396 .50607 m
.77661 .52474 L
.81396 .54342 L
s
.08761 .52474 m
.70588 .31277 L
s
.67661 .34254 m
.70588 .31277 L
.66449 .30721 L
s
.92017 .61811 m
.11096 .61811 L
s
.81746 .59944 m
.78011 .61811 L
.81746 .63679 L
s
.11096 .61811 m
.63383 .22596 L
s
.61516 .26331 m
.63383 .22596 L
.59275 .23343 L
s
.92017 .71148 m
.14986 .71148 L
s
.8233 .69281 m
.78595 .71148 L
.8233 .73016 L
s
.14986 .71148 m
.54202 .18861 L
s
.53455 .22969 m
.54202 .18861 L
.50467 .20728 L
s
.92017 .80486 m
.20433 .80486 L
s
.83147 .78618 m
.79412 .80486 L
.83147 .82353 L
s
.20433 .80486 m
.45571 .20154 L
s
.45858 .24319 m
.45571 .20154 L
.42411 .22883 L
s
0 g
.02381 .43137 m
.97619 .43137 L
s
.35994 .42204 m
.35994 .44071 L
s
gsave
.35994 .36601 -66.9375 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02 w
.35994 .43137 Mdot
.01 w
.23047 .02054 m
.21866 .03697 L
.20733 .05341 L
.19649 .06984 L
.18612 .08627 L
.17624 .10271 L
.16684 .11914 L
.15792 .13557 L
.14949 .15201 L
.14153 .16844 L
.13406 .18487 L
.12707 .20131 L
.12056 .21774 L
.11454 .23417 L
.109 .25061 L
.10393 .26704 L
.09935 .28347 L
.09526 .29991 L
.09164 .31634 L
.08851 .33277 L
.08586 .34921 L
.08369 .36564 L
.082 .38207 L
.0808 .39851 L
.08007 .41494 L
.07983 .43137 L
.08007 .44781 L
.0808 .46424 L
.082 .48067 L
.08369 .49711 L
.08586 .51354 L
.08851 .52997 L
.09164 .54641 L
.09526 .56284 L
.09935 .57927 L
.10393 .5957 L
.109 .61214 L
.11454 .62857 L
.12056 .645 L
.12707 .66144 L
.13406 .67787 L
.14153 .6943 L
.14949 .71074 L
.15792 .72717 L
.16684 .7436 L
.17624 .76004 L
.18612 .77647 L
.19649 .7929 L
.20733 .80934 L
.21866 .82577 L
Mistroke
.23047 .8422 L
Mfstroke
0 0 m
1 0 L
1 .86275 L
0 .86275 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
   GeneratedCell->False,
   CellAutoOverwrite->False,
   ImageSize->{261.75, 225.625},
   ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
   ImageRangeCache->{{{0, 522.5}, {450.25, 0}} -> {-4.3072, -23.1003, 
   0.205223, 0.205223}}], TraditionalForm]]]], "Text",
 CellChangeTimes->{{3.401219849819317*^9, 3.401219852760928*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
As long as the distance from the central axis is small (compared to the \
sphere's radius) the sphere approximates a paraboloid well.  This is the \
basic assumption of our analysis of spherical mirrors.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plane Waves from off the Central Axis", "Subsubsection"],

Cell[TextData[Cell[BoxData[
 FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .4487 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0989975 0.0250627 0.237512 0.0250627 [
[.47494 .16734 -5.9375 -7.28125 ]
[.47494 .16734 5.9375 7.28125 ]
[.85088 .18739 -6.34375 -7.0625 ]
[.85088 .18739 6.34375 7.0625 ]
[ 0 0 0 0 ]
[ 1 .4487 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .7 r
.5 Mabswid
[ ] 0 setdash
.89974 .01068 m
.11584 .07927 L
s
.78899 .01283 m
.77466 .02163 L
.7903 .02781 L
s
.11584 .07927 m
.66225 .38589 L
s
.64546 .38509 m
.66225 .38589 L
.65282 .37198 L
s
.89974 .08587 m
.10348 .15553 L
s
.78713 .08818 m
.77281 .09698 L
.78844 .10316 L
s
.10348 .15553 m
.70099 .34414 L
s
.68438 .34679 m
.70099 .34414 L
.68891 .33245 L
s
.89974 .16106 m
.09902 .23111 L
s
.78647 .16342 m
.77214 .17222 L
.78778 .1784 L
s
.09902 .23111 m
.72219 .29634 L
s
.70645 .30225 m
.72219 .29634 L
.70801 .28729 L
s
.89974 .23625 m
.10213 .30603 L
s
.78693 .23857 m
.77261 .24737 L
.78824 .25355 L
s
.10213 .30603 m
.72585 .24644 L
s
.7116 .25536 m
.72585 .24644 L
.71017 .24039 L
s
.89974 .31144 m
.11268 .38029 L
s
.78851 .31362 m
.77419 .32242 L
.78982 .3286 L
s
.11268 .38029 m
.71221 .19821 L
s
.7 .20978 m
.71221 .19821 L
.69563 .19539 L
s
0 g
.01 w
newpath
.85088 .23751 .75188 164.534 195.466 arc
s
.5 Mabswid
.02381 .23751 m
.97619 .23751 L
s
.47494 .22498 m
.47494 .25004 L
s
.85088 .22498 m
.85088 .25004 L
s
gsave
.47494 .16734 -66.9375 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.85088 .18739 -67.3438 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.188 14.125 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
71.688 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02 w
.47494 .2704 Mdot
0 0 m
1 0 L
1 .4487 L
0 .4487 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
   GeneratedCell->False,
   CellAutoOverwrite->False,
   ImageSize->{283., 126.8125},
   ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
   ImageRangeCache->{{{0, 565}, {252.625, 0}} -> {-4.02032, -9.47671, 
   0.141737, 0.141737}}], TraditionalForm]]]], "Text",
 CellChangeTimes->{{3.4012198552193537`*^9, 3.401219858074545*^9}},
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]],

Cell[CellGroupData[{

Cell["Point Source on the Central Axis", "Subsubsection"],

Cell[TextData[Cell[BoxData[
 FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .34043 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0845998 0.0202634 0.170213 0.0202634 [
[.38855 .09726 -5.9375 -7.28125 ]
[.38855 .09726 5.9375 7.28125 ]
[.6925 .09929 -6.34375 -7.0625 ]
[.6925 .09929 6.34375 7.0625 ]
[.93566 .12969 -4.71875 -7 ]
[.93566 .12969 4.71875 7 ]
[.55741 .1155 -7.5 -8.34375 ]
[.55741 .1155 7.5 8.34375 ]
[ 0 0 0 0 ]
[ 1 .34043 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .7 r
.5 Mabswid
[ ] 0 setdash
.93566 .17021 m
.10261 .02332 L
s
.72611 .12709 m
.71308 .13097 L
.724 .13906 L
s
.10261 .02332 m
.64232 .19833 L
s
.33783 .10598 m
.35127 .10395 L
.34158 .09442 L
s
.93566 .17021 m
.08913 .09615 L
s
.72215 .14543 m
.70951 .15043 L
.72109 .15754 L
s
.08913 .09615 m
.64953 .18487 L
s
.33435 .14113 m
.34731 .13702 L
.33625 .12912 L
s
.93566 .17021 m
.0846 .17021 L
s
.72047 .16413 m
.70831 .17021 L
.72047 .17629 L
s
.0846 .17021 m
.65198 .17021 L
s
.33384 .17629 m
.346 .17021 L
.33384 .16413 L
s
.93566 .17021 m
.08913 .24428 L
s
.72109 .18288 m
.70951 .19 L
.72215 .195 L
s
.08913 .24428 m
.64953 .15556 L
s
.33625 .21131 m
.34731 .2034 L
.33435 .1993 L
s
.93566 .17021 m
.10261 .3171 L
s
.724 .20136 m
.71308 .20946 L
.72611 .21334 L
s
.10261 .3171 m
.64232 .14209 L
s
.34158 .24601 m
.35127 .23647 L
.33783 .23444 L
s
0 g
.008 w
newpath
.6925 .17021 .6079 164.534 195.466 arc
s
.5 Mabswid
.02381 .17021 m
.97619 .17021 L
s
.38855 .16008 m
.38855 .18034 L
s
.6925 .16008 m
.6925 .18034 L
s
gsave
.38855 .09726 -66.9375 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.6925 .09929 -67.3438 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.188 14.125 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
71.688 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.93566 .12969 -65.7188 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
68.438 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.55741 .1155 -68.5 -12.3438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
68.438 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(\\242) show
74.000 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.015 w
.93566 .17021 Mdot
.55741 .17021 Mdot
0 0 m
1 0 L
1 .34043 L
0 .34043 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
   GeneratedCell->False,
   CellAutoOverwrite->False,
   ImageSize->{398., 135.4375},
   ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
   ImageRangeCache->{{{0, 795}, {269.875, 0}} -> {-4.24537, -8.40011, 
   0.124505, 0.124505}}], TraditionalForm]]]], "Text",
 CellChangeTimes->{{3.401219860845348*^9, 3.401219864170898*^9}},
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]],

Cell[CellGroupData[{

Cell["Point Source off the Central Axis", "Subsubsection"],

Cell[TextData[Cell[BoxData[
 FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .34043 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0845998 0.0202634 0.170213 0.0202634 [
[.38855 .21277 -5.9375 -7.28125 ]
[.38855 .21277 5.9375 7.28125 ]
[.6925 .12969 -6.34375 -7.0625 ]
[.6925 .12969 6.34375 7.0625 ]
[.93566 .12969 -4.71875 -7 ]
[.93566 .12969 4.71875 7 ]
[.55741 .19656 -7.15625 -7.5 ]
[.55741 .19656 7.15625 7.5 ]
[ 0 0 0 0 ]
[ 1 .34043 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .7 r
.5 Mabswid
[ ] 0 setdash
.93566 .25127 m
.10224 .02483 L
s
.72643 .18812 m
.71311 .1908 L
.72325 .19985 L
s
.10224 .02483 m
.65622 .14737 L
s
.34428 .0846 m
.35747 .08129 L
.34691 .07273 L
s
.93566 .25127 m
.0888 .09889 L
s
.72254 .20674 m
.7095 .21057 L
.72039 .21871 L
s
.0888 .09889 m
.65525 .13131 L
s
.33728 .1192 m
.34977 .11383 L
.33798 .10706 L
s
.93566 .25127 m
.08461 .17381 L
s
.72099 .22563 m
.70834 .23058 L
.71989 .23773 L
s
.08461 .17381 m
.649 .1157 L
s
.33316 .15433 m
.34463 .14704 L
.33192 .14224 L
s
.93566 .25127 m
.08964 .24831 L
s
.7218 .24444 m
.70962 .25048 L
.72175 .2566 L
s
.08964 .24831 m
.63778 .10182 L
s
.332 .18983 m
.34217 .18082 L
.32886 .17809 L
s
.93566 .25127 m
.10363 .32113 L
s
.72488 .26287 m
.71328 .26994 L
.7259 .27498 L
s
.10363 .32113 m
.62216 .09082 L
s
.33388 .22551 m
.34252 .21502 L
.32895 .2144 L
s
0 g
.008 w
newpath
.6925 .17021 .6079 164.534 195.466 arc
s
.5 Mabswid
.02381 .17021 m
.97619 .17021 L
s
.55741 .16008 m
.55741 .18034 L
s
.93566 .16008 m
.93566 .18034 L
s
.38855 .16008 m
.38855 .18034 L
s
.6925 .16008 m
.6925 .18034 L
s
gsave
.38855 .21277 -66.9375 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.6925 .12969 -67.3438 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.188 14.125 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
71.688 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.93566 .12969 -65.7188 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
68.438 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.55741 .19656 -68.1562 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 15.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
68.438 10.188 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(\\242) show
73.312 15.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.015 w
.93566 .25127 Mdot
.55741 .12518 Mdot
0 0 m
1 0 L
1 .34043 L
0 .34043 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
   GeneratedCell->False,
   CellAutoOverwrite->False,
   ImageSize->{395., 134.375},
   ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
   ImageRangeCache->{{{0, 789}, {267.75, 0}} -> {-4.25352, -8.40011, 0.125493,
    0.125493}}], TraditionalForm]]]], "Text",
 CellChangeTimes->{{3.401219866370332*^9, 3.401219871482283*^9}},
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Images from Concave Mirrors", "Subsection"],

Cell[TextData[{
 "\tWe need to mathematically relate the image position ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["s",
     FontSlant->"Italic"], "\[Prime]"], TraditionalForm]]],
 " and image height ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["h", "\[Prime]"], TraditionalForm]]],
 " to the object position ",
 StyleBox["s",
  FontSlant->"Italic"],
 " and object height ",
 StyleBox["h",
  FontSlant->"Italic"],
 " for the case of a concave spherical mirror.  In the preceding diagram it \
is clear that ",
 StyleBox["every",
  FontSlant->"Italic"],
 " ray from the tip of the object converges to the tip of the image.  It \
suffices to draw just two rays to find the image position.  One ray we will \
draw is from the tip of the object to the point where the central axis hits \
the mirror; this will reflect back at an equal angle below the central axis.  \
The other ray we will consider passes through the center of the sphere; this \
will hit the surface normally and reflect straight backward."
}], "Text",
 CellChangeTimes->{{3.387802158807097*^9, 3.3878021753759823`*^9}}],

Cell[TextData[Cell[BoxData[
 FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .76842 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0739348 0.0100251 0.369173 0.0100251 [
[.21429 .75013 -14.3438 -6.71875 ]
[.21429 .75013 14.3438 6.71875 ]
[.87594 .60977 -24.3125 -7.3125 ]
[.87594 .60977 24.3125 7.3125 ]
[.43848 .21789 -22.5625 -7.3125 ]
[.43848 .21789 22.5625 7.3125 ]
[.61529 .32907 -19.0938 -6.0625 ]
[.61529 .32907 19.0938 6.0625 ]
[.57519 .39925 -6.34375 -6.5625 ]
[.57519 .39925 6.34375 6.5625 ]
[.32456 .39925 -5.9375 -7.21875 ]
[.32456 .39925 5.9375 7.21875 ]
[.91604 .46942 -5.5 -6.8125 ]
[.91604 .46942 5.5 6.8125 ]
[.87594 .32907 -4.71875 -6 ]
[.87594 .32907 4.71875 6 ]
[.37833 .32862 -11.3125 -7.84375 ]
[.37833 .32862 11.3125 7.84375 ]
[.43848 .40927 -7.5 -7.84375 ]
[.43848 .40927 7.5 7.84375 ]
[ 0 0 0 0 ]
[ 1 .76842 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.02381 .36917 m
.97619 .36917 L
s
0 0 .55 r
.87594 .56967 m
.07393 .36917 L
s
.49925 .45483 m
.47494 .46942 L
.48953 .49374 L
s
.07393 .36917 m
.94885 .15044 L
s
.58429 .26225 m
.59888 .23794 L
.57457 .22335 L
s
.93426 .17476 m
.94885 .15044 L
.92454 .13586 L
s
.87594 .56967 m
.15812 .09113 L
s
.72429 .44448 m
.69648 .45004 L
.70205 .47784 L
s
.15812 .09113 m
.94772 .61753 L
s
.28824 .20197 m
.31604 .19641 L
.31048 .1686 L
s
.91992 .62309 m
.94772 .61753 L
.94216 .58972 L
s
0 g
3 Mabswid
newpath
.57519 .36917 .50125 135.573 224.427 arc
s
gsave
.21429 .75013 -75.3438 -10.7188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 21.4375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.438 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.188 14.438 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
71.688 14.438 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(>) show
80.688 14.438 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
87.688 14.438 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
.87594 .36917 m
.87594 .56967 L
s
.85589 .54962 m
.87594 .56967 L
.89599 .54962 L
s
gsave
.87594 .60977 -85.3125 -11.3125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(object) show
107.625 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
.43848 .36917 m
.43848 .27804 L
s
.41843 .29809 m
.43848 .27804 L
.45853 .29809 L
s
gsave
.43848 .21789 -83.5625 -11.3125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
70.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(image) show
104.125 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 g
gsave
.61529 .32907 -80.0938 -10.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(center) show
97.188 13.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.57519 .39925 -67.3438 -10.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 21.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.188 14.125 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
71.688 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.32456 .39925 -66.9375 -11.2188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.4375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
gsave
.91604 .46942 -66.5 -10.8125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 21.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.625 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(h) show
70.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.87594 .32907 -65.7188 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(s) show
68.438 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
gsave
.37833 .32862 -72.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
63.000 16.688 moveto
(\\310) show
65.750 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.562 0.000 rmoveto
66.312 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(h) show
73.312 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
78.875 16.688 moveto
(\\310) show
81.625 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.43848 .40927 -68.5 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(s) show
68.438 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
74.000 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 g
.015 w
.32456 .36917 Mdot
.57519 .36917 Mdot
0 .5 0 r
.87594 .16867 Mdot
0 0 m
1 0 L
1 .76842 L
0 .76842 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
   GeneratedCell->False,
   CellAutoOverwrite->False,
   ImageSize->{313.5, 240.8125},
   ImageMargins->{{0., 0.}, {0., 0.75}},
   ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
   ImageRangeCache->{{{0, 626}, {480.625, 0}} -> {-7.63103, -36.8252, 
   0.320325, 0.320325}}], TraditionalForm]]]], "Text",
 CellChangeTimes->{{3.401219873057295*^9, 3.401219875802017*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell["\tThese two rays gives two pair of similar triangles.", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .46537 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0113379 0.182179 0.0113379 [
[.26397 .15497 -7.5 -7.84375 ]
[.26397 .15497 7.5 7.84375 ]
[.52268 .21052 -4.71875 -6 ]
[.52268 .21052 4.71875 6 ]
[.93084 .45429 -2 -6 ]
[.93084 .45429 2 6 ]
[.4361 .01108 -2 -6 ]
[.4361 .01108 2 6 ]
[.97619 .29556 -5.5 -6.8125 ]
[.97619 .29556 5.5 6.8125 ]
[.50412 .13631 -11.3125 -7.84375 ]
[.50412 .13631 11.3125 7.84375 ]
[ 0 0 0 0 ]
[ 1 .46537 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.02381 .18218 m
.93084 .18218 L
s
gsave
.26397 .15497 -68.5 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
68.438 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(\\242) show
74.000 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.52268 .21052 -65.7188 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
68.438 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 .55 r
.93084 .40894 m
.02381 .18218 L
s
.02381 .18218 m
.4361 .07911 L
s
0 .5 0 r
3 Mabswid
.93084 .18218 m
.93084 .40894 L
s
.90816 .38626 m
.93084 .40894 L
.95351 .38626 L
s
gsave
.93084 .45429 -63 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
.4361 .18218 m
.4361 .07911 L
s
.41342 .10178 m
.4361 .07911 L
.45877 .10178 L
s
gsave
.4361 .01108 -63 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
gsave
.97619 .29556 -66.5 -10.8125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 21.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.625 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(h) show
70.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
gsave
.50412 .13631 -72.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
63.000 16.688 moveto
(\\310) show
65.750 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.562 0.000 rmoveto
66.312 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(h) show
73.312 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
78.875 16.688 moveto
(\\310) show
81.625 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .46537 L
0 .46537 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
    GeneratedCell->False,
    CellAutoOverwrite->False,
    ImageSize->{254.25, 118.125},
    ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
    ImageRangeCache->{{{0, 507.5}, {235.25, 0}} -> {-4.95606, -17.3117, 
    0.370096, 0.370096}}], TraditionalForm]]],
 "   ",
 Cell[BoxData[
  FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .72881 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.515335 0.0177563 0.285311 0.0177563 [
[.63882 .23737 -13.5625 -6.5625 ]
[.63882 .23737 13.5625 6.5625 ]
[.25141 .32615 -16 -7 ]
[.25141 .32615 16 7 ]
[.90517 .71146 -2 -6 ]
[.90517 .71146 2 6 ]
[.13035 .01735 -2 -6 ]
[.13035 .01735 2 6 ]
[.97619 .46287 -5.5 -6.8125 ]
[.97619 .46287 5.5 6.8125 ]
[.02381 .21348 -11.3125 -7.84375 ]
[.02381 .21348 11.3125 7.84375 ]
[ 0 0 0 0 ]
[ 1 .72881 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.13035 .28531 m
.90517 .28531 L
s
gsave
.63882 .23737 -74.5625 -10.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 21.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.125 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
68.438 14.125 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
77.625 14.125 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
86.125 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.25141 .32615 -77 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 15.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.188 15.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
71.688 15.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
80.688 15.000 moveto
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
86.125 10.188 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(\\242) show
91.000 15.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 .55 r
.90517 .64044 m
.13035 .12389 L
s
0 .5 0 r
3 Mabswid
.90517 .28531 m
.90517 .64044 L
s
.86965 .60492 m
.90517 .64044 L
.94068 .60492 L
s
gsave
.90517 .71146 -63 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
.13035 .28531 m
.13035 .12389 L
s
.09483 .1594 m
.13035 .12389 L
.16586 .1594 L
s
gsave
.13035 .01735 -63 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
gsave
.97619 .46287 -66.5 -10.8125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 21.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.625 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(h) show
70.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
gsave
.02381 .21348 -72.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
63.000 16.688 moveto
(\\310) show
65.750 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.562 0.000 rmoveto
66.312 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(h) show
73.312 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
78.875 16.688 moveto
(\\310) show
81.625 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .72881 L
0 .72881 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
    GeneratedCell->False,
    CellAutoOverwrite->False,
    ImageSize->{202.5, 147.5},
    ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
    ImageRangeCache->{{{0, 404}, {294, 0}} -> {27.0868, -16.8337, 0.289635, 
    0.289635}}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.401219878945244*^9, 3.401219887018063*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell["These give the expressions", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[Cell[TextData[Cell[BoxData[
    FormBox[
     RowBox[{
      FractionBox[
       RowBox[{"\[LeftBracketingBar]", 
        SuperscriptBox["h", "\[Prime]"], "\[RightBracketingBar]"}], "h"], "=",
       
      FractionBox[
       SuperscriptBox[
        StyleBox["s",
         FontSlant->"Italic"], "\[Prime]"], "s"]}], TraditionalForm]]]]], 
   TraditionalForm]],
  FontSize->12],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{"\[LeftBracketingBar]", 
      SuperscriptBox["h", "\[Prime]"], "\[RightBracketingBar]"}], "h"], "=", 
    FractionBox[
     RowBox[{"R", "-", 
      SuperscriptBox[
       StyleBox["s",
        FontSlant->"Italic"], "\[Prime]"]}], 
     RowBox[{"s", "-", "R"}]]}], TraditionalForm]],
  FontSize->12]
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     SuperscriptBox[
      StyleBox["s",
       FontSlant->"Italic"], "\[Prime]"], "s"], "=", 
    FractionBox[
     RowBox[{"R", "-", 
      SuperscriptBox[
       StyleBox["s",
        FontSlant->"Italic"], "\[Prime]"]}], 
     RowBox[{"s", "-", "R"}]]}], TraditionalForm]],
  FontSize->12],
 Cell[BoxData[
  FormBox[
   RowBox[{"   ", 
    RowBox[{"\[DoubleLongRightArrow]", "   "}]}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["1", "s"], "+", 
     FractionBox["1", 
      SuperscriptBox[
       StyleBox["s",
        FontSlant->"Italic"], "\[Prime]"]]}], "=", 
    FractionBox["2", "R"]}], TraditionalForm]],
  FontSize->12]
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "If the object is at infinity ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"s", "\[Rule]", "\[Infinity]"}], ")"}], TraditionalForm]]],
 " the image is at the focal point ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       StyleBox["s",
        FontSlant->"Italic"], "\[Prime]"], "=", "f"}], ")"}], "."}], 
   TraditionalForm]]],
 "  This gives ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "=", 
    RowBox[{"R", "/", "2."}]}], TraditionalForm]]],
 "  Since the image is inverted we choose, by convention, that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["h", "\[Prime]"], "<", "0"}], TraditionalForm]]],
 " and thus ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[LeftBracketingBar]", 
     SuperscriptBox["h", "\[Prime]"], "\[RightBracketingBar]"}], "=", 
    RowBox[{"-", 
     RowBox[{
      SuperscriptBox["h", "\[Prime]"], "."}]}]}], TraditionalForm]]],
 "  We can rewrite the above expressions as"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["1", "s"], "+", 
     FractionBox["1", 
      SuperscriptBox[
       StyleBox["s",
        FontSlant->"Italic"], "\[Prime]"]]}], "=", 
    FractionBox["1", "f"]}], TraditionalForm]],
  FontSize->12],
 "   and   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "=", 
    RowBox[{
     FractionBox[
      SuperscriptBox["h", "\[Prime]"], "h"], "=", 
     RowBox[{"-", 
      FractionBox[
       SuperscriptBox[
        StyleBox["s",
         FontSlant->"Italic"], "\[Prime]"], "s"]}]}]}], TraditionalForm]],
  FontSize->12],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "These expressions will apply generally to spherical mirrors, either concave \
or convex, and to thin lenses, either converging or diverging.  ",
 StyleBox["m",
  FontSlant->"Italic"],
 " is defined as the magnification; it has the same sign convention as the \
image height ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["h", "\[Prime]"], "."}], TraditionalForm]]]
}], "Text"],

Cell[TextData[{
 "\tThe preceding image is called a ",
 StyleBox["real image",
  FontSlant->"Italic"],
 ".  A real image occurs when the light rays converge to a point.  If the \
object is inside the focal point, it turns out that the reflected rays do not \
converge.  We trace the same rays as before but to find the image we trace \
the diverging rays backward to see where the reflected light rays appear to \
originate; this is the image position.  Mathematically, when the object is \
inside the focal point  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", "<", "s", "<", "f"}], TraditionalForm]]],
 "  the image position is negative ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     StyleBox["s",
      FontSlant->"Italic"], "\[Prime]"], "<", "0"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .63373 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.433243 0.00571303 0.268247 0.00571303 [
[.48466 .50819 -14.6875 -6.71875 ]
[.48466 .50819 14.6875 6.71875 ]
[.57607 .42821 -26.0625 -7.3125 ]
[.57607 .42821 26.0625 7.3125 ]
[.59892 .32538 -5.5 -6.8125 ]
[.59892 .32538 5.5 6.8125 ]
[.05237 .61865 -19.0625 -7.3125 ]
[.05237 .61865 19.0625 7.3125 ]
[.57607 .24539 -4.71875 -6 ]
[.57607 .24539 4.71875 6 ]
[.89029 .31395 -19.0938 -6.0625 ]
[.89029 .31395 19.0938 6.0625 ]
[.05237 .23397 -7.5 -7.84375 ]
[.05237 .23397 7.5 7.84375 ]
[.05237 .18826 -15.5 -7.84375 ]
[.05237 .18826 15.5 7.84375 ]
[.02952 .42345 -8.28125 -7.84375 ]
[.02952 .42345 8.28125 7.84375 ]
[.89029 .23397 -6.34375 -6.5625 ]
[.89029 .23397 6.34375 6.5625 ]
[.66176 .23397 -5.9375 -7.21875 ]
[.66176 .23397 5.9375 7.21875 ]
[ 0 0 0 0 ]
[ 1 .63373 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.02381 .26825 m
.94742 .26825 L
s
0 0 .55 r
[ .02 .02 ] 0 setdash
.43324 .26825 m
.05237 .57294 L
s
.46076 .42444 m
.05237 .57294 L
s
[ ] 0 setdash
.57607 .38251 m
.43324 .26825 L
s
.52072 .32359 m
.50466 .32538 L
.50644 .34144 L
s
.43324 .26825 m
.74746 .01687 L
s
.74568 .03293 m
.74746 .01687 L
.7314 .01509 L
s
.57607 .38251 m
.46076 .42444 L
s
.53678 .38464 m
.52995 .39928 L
.54459 .40611 L
s
.46076 .42444 m
.97619 .23701 L
s
.48485 .42783 m
.49169 .41319 L
.47704 .40636 L
s
.71164 .34537 m
.71848 .33072 L
.70383 .32389 L
s
.96936 .25165 m
.97619 .23701 L
.96155 .23018 L
s
0 g
3 Mabswid
newpath
.89029 .26825 .45704 154.056 205.944 arc
s
gsave
.48466 .50819 -75.6875 -10.7188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 21.4375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.438 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.188 14.438 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
71.688 14.438 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(>) show
80.688 14.438 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.688 0.000 rmoveto
81.375 14.438 moveto
(0) show
88.375 14.438 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
.57607 .26825 m
.57607 .38251 L
s
.56464 .37108 m
.57607 .38251 L
.58749 .37108 L
s
gsave
.57607 .42821 -87.0625 -11.3125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
77.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(object) show
111.125 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.59892 .32538 -66.5 -10.8125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 21.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.625 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(h) show
70.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
.05237 .26825 m
.05237 .57294 L
s
.04095 .56152 m
.05237 .57294 L
.0638 .56152 L
s
gsave
.05237 .61865 -80.0625 -11.3125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(image) show
97.125 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
gsave
.57607 .24539 -65.7188 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(s) show
68.438 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 g
gsave
.89029 .31395 -80.0938 -10.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(center) show
97.188 13.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
gsave
.05237 .23397 -68.5 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(s) show
68.438 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
74.000 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.05237 .18826 -76.5 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(s) show
68.438 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
74.000 16.688 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(<) show
83.000 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(0) show
90.000 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.02952 .42345 -69.2812 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(h) show
70.000 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
75.562 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 g
gsave
.89029 .23397 -67.3438 -10.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 21.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.188 14.125 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
71.688 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.66176 .23397 -66.9375 -11.2188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.4375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.015 w
.89029 .26825 Mdot
.66176 .26825 Mdot
.57607 .15399 Mdot
0 0 m
1 0 L
1 .63373 L
0 .63373 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
   GeneratedCell->False,
   CellAutoOverwrite->False,
   ImageSize->{359.5, 227.75},
   ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
   ImageRangeCache->{{{0, 718}, {454.5, 0}} -> {-76.8032, -46.9541, 0.492361, 
   0.492361}}], TraditionalForm]]]], "Text",
 CellChangeTimes->{{3.401219889583322*^9, 3.401219893472437*^9}},
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]],

Cell[CellGroupData[{

Cell["Images from Convex Mirrors", "Subsection"],

Cell["\<\
\tThe center of a convex mirror is behind the mirror.  We take the radius to \
be negative.  An incident plane wave parallel to the central axis will \
reflect away from a focal point behind the mirror.  When the reflected rays \
are extrapolated backward they meet at the focal point.  We also take the \
focal length to be negative.\
\>", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .52465 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.568027 0.0170068 0.262327 0.0170068 [
[.31293 .21131 -5.9375 -7.28125 ]
[.31293 .21131 5.9375 7.28125 ]
[.31293 .14328 -13.9375 -7.28125 ]
[.31293 .14328 13.9375 7.28125 ]
[.05782 .21131 -6.34375 -7.0625 ]
[.05782 .21131 6.34375 7.0625 ]
[.05782 .14328 -14.3438 -7.21875 ]
[.05782 .14328 14.3438 7.21875 ]
[ 0 0 0 0 ]
[ 1 .52465 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .7 r
.5 Mabswid
[ ] 0 setdash
.97619 .16029 m
.55772 .16029 L
s
.901 .15178 m
.88399 .16029 L
.901 .16879 L
s
.55772 .16029 m
.90194 .01365 L
s
.88962 .02814 m
.90194 .01365 L
.88296 .01249 L
s
[ .01 .01 ] 0 setdash
.55772 .16029 m
.31293 .26457 L
s
[ ] 0 setdash
.97619 .21131 m
.56547 .21131 L
s
.90255 .2028 m
.88554 .21131 L
.90255 .21981 L
s
.56547 .21131 m
.93214 .13685 L
s
.91716 .14857 m
.93214 .13685 L
.91378 .1319 L
s
[ .01 .01 ] 0 setdash
.56547 .21131 m
.31293 .26259 L
s
[ ] 0 setdash
.97619 .26233 m
.56803 .26233 L
s
.90306 .25382 m
.88605 .26233 L
.90306 .27083 L
s
.56803 .26233 m
.94218 .26233 L
s
.92517 .27083 m
.94218 .26233 L
.92517 .25382 L
s
[ .01 .01 ] 0 setdash
.56803 .26233 m
.31293 .26233 L
s
[ ] 0 setdash
.97619 .31335 m
.56547 .31335 L
s
.90255 .30484 m
.88554 .31335 L
.90255 .32185 L
s
.56547 .31335 m
.93214 .3878 L
s
.91378 .39275 m
.93214 .3878 L
.91716 .37608 L
s
[ .01 .01 ] 0 setdash
.56547 .31335 m
.31293 .26207 L
s
[ ] 0 setdash
.97619 .36437 m
.55772 .36437 L
s
.901 .35586 m
.88399 .36437 L
.901 .37287 L
s
.55772 .36437 m
.90194 .511 L
s
.88296 .51216 m
.90194 .511 L
.88962 .49652 L
s
[ .01 .01 ] 0 setdash
.55772 .36437 m
.31293 .26009 L
s
0 g
.008 w
[ ] 0 setdash
newpath
.05782 .26233 .5102 -15.466 15.466 arc
s
.5 Mabswid
.02381 .26233 m
.97619 .26233 L
s
.31293 .25382 m
.31293 .27083 L
s
gsave
.31293 .21131 -66.9375 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.31293 .14328 -74.9375 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(<) show
79.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
86.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.05782 .25382 m
.05782 .27083 L
s
gsave
.05782 .21131 -67.3438 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.188 14.125 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
71.688 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.05782 .14328 -75.3438 -11.2188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.4375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.438 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.188 14.438 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
71.688 14.438 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(<) show
80.688 14.438 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
87.688 14.438 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .52465 L
0 .52465 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
   GeneratedCell->False,
   CellAutoOverwrite->False,
   ImageSize->{283.25, 148.4375},
   ImageMargins->{{0., 0.}, {0., 0.5}},
   ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
   ImageRangeCache->{{{0, 565.5}, {295.875, 0}} -> {-33.4812, -15.425, 
   0.208532, 0.208532}}], TraditionalForm]]]], "Text",
 CellChangeTimes->{{3.4012198958435287`*^9, 3.401219899091947*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
Light from an off-axis point source will reflect away from a point behind the \
mirror, the image position.  This will create a virtual image.\
\>", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .65126 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.419601 0.0232819 0.361089 0.0232819 [
[.07037 .31452 -5.9375 -7.28125 ]
[.07037 .31452 5.9375 7.28125 ]
[.88524 .39601 -4.71875 -7 ]
[.88524 .39601 4.71875 7 ]
[.22004 .31452 -7.5 -8.34375 ]
[.22004 .31452 7.5 8.34375 ]
[.22004 .24468 -16.5312 -8.34375 ]
[.22004 .24468 16.5312 8.34375 ]
[ 0 0 0 0 ]
[ 1 .65126 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .7 r
.5 Mabswid
[ ] 0 setdash
.88524 .43093 m
.4157 .28738 L
s
.58876 .33299 m
.57336 .33558 L
.58468 .34635 L
s
.4157 .28738 m
.90386 .01551 L
s
.89506 .02841 m
.90386 .01551 L
.88826 .0162 L
s
[ .01 .01 ] 0 setdash
.4157 .28738 m
.22004 .39635 L
s
[ ] 0 setdash
.88524 .43093 m
.41899 .33183 L
s
.59046 .36114 m
.57534 .36506 L
.58756 .3748 L
s
.41899 .33183 m
.9539 .17031 L
s
.94255 .18104 m
.9539 .17031 L
.93851 .16766 L
s
[ .01 .01 ] 0 setdash
.41899 .33183 m
.22004 .3919 L
s
[ ] 0 setdash
.88524 .43093 m
.41949 .37375 L
s
.59028 .38768 m
.57557 .39291 L
.58858 .40155 L
s
.41949 .37375 m
.97619 .32579 L
s
.96287 .33395 m
.97619 .32579 L
.96167 .32003 L
s
[ .01 .01 ] 0 setdash
.41949 .37375 m
.22004 .39093 L
s
[ ] 0 setdash
.88524 .43093 m
.41755 .4146 L
s
.58846 .41358 m
.57426 .42007 L
.58798 .42754 L
s
.41755 .4146 m
.9724 .48065 L
s
.9577 .48593 m
.9724 .48065 L
.95935 .47206 L
s
[ .01 .01 ] 0 setdash
.41755 .4146 m
.22004 .39109 L
s
[ ] 0 setdash
.88524 .43093 m
.41317 .45567 L
s
.585 .43967 m
.57142 .44738 L
.58573 .45362 L
s
.41317 .45567 m
.94285 .63358 L
s
.92739 .63576 m
.94285 .63358 L
.93183 .62251 L
s
[ .01 .01 ] 0 setdash
.41317 .45567 m
.22004 .39081 L
s
0 g
.008 w
[ ] 0 setdash
newpath
-0.27885 .36109 .69846 -9.59407 9.59407 arc
s
.5 Mabswid
.02381 .36109 m
.9318 .36109 L
s
.22004 .34945 m
.22004 .37273 L
s
.88524 .34945 m
.88524 .37273 L
s
.07037 .34945 m
.07037 .37273 L
s
gsave
.07037 .31452 -66.9375 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.5625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.88524 .39601 -65.7188 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
68.438 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.22004 .31452 -68.5 -12.3438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
68.438 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(\\242) show
74.000 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.22004 .24468 -77.5312 -12.3438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
68.438 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(\\242) show
74.000 16.688 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(<) show
83.000 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
2.062 0.000 rmoveto
85.062 16.688 moveto
(0) show
92.062 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.015 w
.88524 .43093 Mdot
.22004 .39102 Mdot
0 0 m
1 0 L
1 .65126 L
0 .65126 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
   GeneratedCell->False,
   CellAutoOverwrite->False,
   ImageSize->{285., 185.5},
   ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
   ImageRangeCache->{{{0, 569}, {370, 0}} -> {-18.0555, -15.5094, 0.151204, 
   0.151204}}], TraditionalForm]]]], "Text",
 CellChangeTimes->{{3.401219901096321*^9, 3.401219903868926*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
\tThe ray tracing is similar to the concave case except that the center is \
now behind the mirror.\
\>", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .60753 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.575813 0.00613338 0.310783 0.00613338 [
[.52061 .56838 -14.6875 -6.71875 ]
[.52061 .56838 14.6875 6.71875 ]
[.88248 .58678 -19.0625 -7.3125 ]
[.88248 .58678 19.0625 7.3125 ]
[.90702 .43345 -5.5 -6.8125 ]
[.90702 .43345 5.5 6.8125 ]
[.43952 .45049 -19.0625 -7.3125 ]
[.43952 .45049 19.0625 7.3125 ]
[.88248 .27398 -4.71875 -6 ]
[.88248 .27398 4.71875 6 ]
[.08514 .34758 -19.0938 -6.0625 ]
[.08514 .34758 19.0938 6.0625 ]
[.43952 .27398 -7.5 -7.84375 ]
[.43952 .27398 7.5 7.84375 ]
[.43952 .22492 -15.8438 -7.84375 ]
[.43952 .22492 15.8438 7.84375 ]
[.41498 .36224 -8.28125 -7.84375 ]
[.41498 .36224 8.28125 7.84375 ]
[.08514 .27398 -6.34375 -6.5625 ]
[.08514 .27398 6.34375 6.5625 ]
[.33048 .27398 -5.9375 -7.21875 ]
[.33048 .27398 5.9375 7.21875 ]
[ 0 0 0 0 ]
[ 1 .60753 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.02381 .31078 m
.94382 .31078 L
s
0 0 .55 r
[ .02 .02 ] 0 setdash
.57581 .31078 m
.43952 .41982 L
s
.55412 .45508 m
.08514 .31078 L
s
[ ] 0 setdash
.88248 .55612 m
.57581 .31078 L
s
.74639 .43153 m
.72915 .43345 L
.73106 .45069 L
s
.57581 .31078 m
.94382 .01638 L
s
.9419 .03362 m
.94382 .01638 L
.92657 .01447 L
s
.88248 .55612 m
.55412 .45508 L
s
.76647 .50759 m
.75114 .5157 L
.75925 .53104 L
s
.55412 .45508 m
.97619 .58495 L
s
.6232 .48917 m
.63853 .48106 L
.63041 .46572 L
s
.96086 .59307 m
.97619 .58495 L
.96807 .56962 L
s
0 g
3 Mabswid
newpath
.08514 .31078 .49067 -25.9445 25.9445 arc
s
gsave
.52061 .56838 -75.6875 -10.7188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 21.4375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.438 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.188 14.438 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
71.688 14.438 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(<) show
80.688 14.438 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.688 0.000 rmoveto
81.375 14.438 moveto
(0) show
88.375 14.438 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
.88248 .31078 m
.88248 .55612 L
s
.87022 .54385 m
.88248 .55612 L
.89475 .54385 L
s
gsave
.88248 .58678 -80.0625 -11.3125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(object) show
97.125 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.90702 .43345 -66.5 -10.8125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 21.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.625 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(h) show
70.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
.43952 .31078 m
.43952 .41982 L
s
.42725 .40755 m
.43952 .41982 L
.45178 .40755 L
s
gsave
.43952 .45049 -80.0625 -11.3125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(image) show
97.125 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
gsave
.88248 .27398 -65.7188 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(s) show
68.438 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 g
gsave
.08514 .34758 -80.0938 -10.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(center) show
97.188 13.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
gsave
.43952 .27398 -68.5 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(s) show
68.438 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
74.000 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.43952 .22492 -76.8438 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(s) show
68.438 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
74.000 16.688 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(<) show
83.000 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.688 0.000 rmoveto
83.688 16.688 moveto
(0) show
90.688 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.41498 .36224 -69.2812 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(h) show
70.000 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
75.562 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 g
gsave
.08514 .27398 -67.3438 -10.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 21.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.188 14.125 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
71.688 14.125 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.33048 .27398 -66.9375 -11.2188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.4375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.015 w
.08514 .31078 Mdot
.33048 .31078 Mdot
.88248 .06545 Mdot
0 0 m
1 0 L
1 .60753 L
0 .60753 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
   GeneratedCell->False,
   CellAutoOverwrite->False,
   ImageSize->{336.5, 204.25},
   ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
   ImageRangeCache->{{{0, 672}, {407.5, 0}} -> {-94.1817, -50.6712, 0.48703, 
   0.48703}}], TraditionalForm]]]], "Text",
 CellChangeTimes->{{3.401219906046321*^9, 3.4012199101449223`*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
An analogous analysis with similar triangles yields the same expressions\
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["1", "s"], "+", 
     FractionBox["1", 
      SuperscriptBox[
       StyleBox["s",
        FontSlant->"Italic"], "\[Prime]"]]}], "=", 
    FractionBox["1", "f"]}], TraditionalForm]],
  FontSize->12],
 "   and   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "=", 
    RowBox[{
     FractionBox[
      SuperscriptBox["h", "\[Prime]"], "h"], "=", 
     RowBox[{"-", 
      FractionBox[
       SuperscriptBox[
        StyleBox["s",
         FontSlant->"Italic"], "\[Prime]"], "s"]}]}]}], TraditionalForm]],
  FontSize->12],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "It is still true that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "=", 
    RowBox[{"R", "/", "2"}]}], TraditionalForm]]],
 " but now both ",
 StyleBox["R",
  FontSlant->"Italic"],
 " and ",
 StyleBox["f",
  FontSlant->"Italic"],
 " are negative.  It follows that since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"s", ">", "0"}], TraditionalForm]]],
 " we will always get ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     StyleBox["s",
      FontSlant->"Italic"], "\[Prime]"], "<", "0"}], TraditionalForm]]],
 " and thus a virtual image."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Flat MIrrors", "Subsection",
 CellChangeTimes->{{3.387625160233453*^9, 3.387625168032604*^9}}],

Cell[TextData[{
 "\tA flat mirror can be viewed as a special case of a spherical mirror with \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "\[Rule]", "\[Infinity]"}], TraditionalForm]]],
 ".  This implies that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "\[Rule]", "\[Infinity]"}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"1", "/", "f"}], "\[Rule]", "0"}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.387625227442422*^9, 3.387625262922559*^9}, {
  3.387625304901248*^9, 3.387625372817215*^9}}],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["1", "s"], "+", 
     FractionBox["1", 
      SuperscriptBox[
       StyleBox["s",
        FontSlant->"Italic"], "\[Prime]"]]}], "=", 
    RowBox[{
     FractionBox["1", "f"], "=", "0"}]}], TraditionalForm]],
  FontSize->12],
 "   \[DoubleLongRightArrow]  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["s", "\[Prime]"], "=", 
    RowBox[{"-", "s"}]}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.387625438002071*^9, 3.3876254995470047`*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "=", 
    RowBox[{
     FractionBox[
      SuperscriptBox["h", "\[Prime]"], "h"], "=", 
     RowBox[{"-", 
      FractionBox[
       SuperscriptBox[
        StyleBox["s",
         FontSlant->"Italic"], "\[Prime]"], "s"]}]}]}], TraditionalForm]],
  FontSize->12],
 "  \[DoubleLongRightArrow]  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "=", "1"}], TraditionalForm]]],
 " and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["h", "\[Prime]"], "=", "h"}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.387625438002071*^9, 3.3876254995470047`*^9}, {
  3.38762834427816*^9, 3.387628434482736*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
This is what we would expect:  There is an upright virtual image the same \
size as the object and equal distance behind the mirror.\
\>", "Text",
 CellChangeTimes->{{3.387628452018735*^9, 3.387628498387312*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Images from Refraction", "Section"],

Cell[CellGroupData[{

Cell["General Sign Conventions in Optics", "Subsection"],

Cell[TextData[{
 "\tIn the case above where images were formed by mirrors, the sign \
conventions were relatively simple.  ",
 StyleBox["s",
  FontSlant->"Italic"],
 " is positive when the object is on the front side of the mirror.  \
Similarly, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["s", "\[Prime]"], TraditionalForm]]],
 ", ",
 StyleBox["f",
  FontSlant->"Italic"],
 " and ",
 StyleBox["R",
  FontSlant->"Italic"],
 " are positive when the image, focal point and center, respectively, are on \
the mirror's front side.  "
}], "Text",
 CellChangeTimes->{3.387804931747167*^9}],

Cell[TextData[{
 "\tWe will now consider cases involving refraction, where the light emerges \
(ends up) on the opposite side where it originates.  The ray that comes into \
the optical component (mirror, lens or refracting interface) will be called \
the incident ray.  The ray leaving the optical component will be called the \
",
 StyleBox["emergent ray",
  FontSlant->"Italic"],
 "; this is the reflected ray for mirrors or the refracted ray for a \
refracting interface or lens."
}], "Text"],

Cell[TextData[{
 "\tA general sign convention can now be given.  ",
 StyleBox["s",
  FontSlant->"Italic"],
 " is positive when the object is on the side of the incident ray, the side \
where the light originates.   ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["s", "\[Prime]"], TraditionalForm]]],
 ", ",
 StyleBox["f",
  FontSlant->"Italic"],
 " and ",
 StyleBox["R",
  FontSlant->"Italic"],
 " are positive when the image, focal point and center, respectively, are on \
the side of the emergent ray, on the side where the light ends up."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Spherical Interface", "Subsection"],

Cell[TextData[{
 " ",
 Cell[BoxData[
  FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .39623 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.365229 0.00898473 0.189128 0.00898473 [
[.09569 .15319 -4.3125 -7 ]
[.09569 .15319 4.3125 7 ]
[.58985 .22507 -5.75 -7 ]
[.58985 .22507 5.75 7 ]
[.90431 .22507 -6.75 -7.5 ]
[.90431 .22507 6.75 7.5 ]
[.27924 .21516 -7.90625 -7.0625 ]
[.27924 .21516 7.90625 7.0625 ]
[.56181 .16851 -7.90625 -7.0625 ]
[.56181 .16851 7.90625 7.0625 ]
[.09569 .38679 -16.625 -7 ]
[.09569 .38679 16.625 7 ]
[.90431 .05681 -16.625 -7 ]
[.90431 .05681 16.625 7 ]
[.49102 .03639 -8.28125 -7.0625 ]
[.49102 .03639 8.28125 7.0625 ]
[.36523 .03639 -8.28125 -7.0625 ]
[.36523 .03639 8.28125 7.0625 ]
[.06873 .27898 -5 -7 ]
[.06873 .27898 5 7 ]
[.94025 .13645 -10.4688 -7.5 ]
[.94025 .13645 10.4688 7.5 ]
[ 0 0 0 0 ]
[ 1 .39623 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
.9 g
.58985 .18913 m
.58985 .18913 .22462 126.87 233.13 arc
F
.45508 .00943 m
.45508 .36882 L
.97619 .36882 L
.97619 .00943 L
F
0 g
.5 Mabswid
[ ] 0 setdash
newpath
.58985 .18913 .22462 126.87 233.13 arc
s
.02381 .18913 m
.97619 .18913 L
s
0 .5 0 r
5 Mabswid
.09569 .18913 Mdot
gsave
.09569 .15319 -65.3125 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(s) show
67.625 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 g
.58985 .18913 Mdot
gsave
.58985 .22507 -66.75 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.188 14.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
70.500 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
.90431 .18913 Mdot
gsave
.90431 .22507 -67.75 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 15.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(s) show
67.625 10.188 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
72.500 15.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 .55 r
.5 Mabswid
.09569 .36882 m
.90431 .07478 L
s
.48925 .24483 m
.5 .2218 L
.47697 .21105 L
s
.09569 .36882 m
.36523 .18913 L
s
.22547 .30389 m
.23046 .27898 L
.20554 .27399 L
s
.36523 .18913 m
.90431 .07478 L
s
.62092 .15326 m
.63477 .13195 L
.61346 .1181 L
s
0 g
newpath
.36523 .18913 .07188 146.31 180 arc
s
gsave
.27924 .21516 -68.9062 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(q) show
68.250 17.125 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
74.812 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
newpath
.36523 .18913 .15274 -11.9761 0 arc
s
newpath
.36523 .18913 .16173 -11.9761 0 arc
s
gsave
.56181 .16851 -68.9062 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(q) show
68.250 17.125 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
74.812 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
.005 w
.09569 .18913 m
.09569 .36882 L
s
.07772 .35085 m
.09569 .36882 L
.11366 .35085 L
s
gsave
.09569 .38679 -77.625 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(object) show
92.250 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
.90431 .18913 m
.90431 .07478 L
s
.88634 .09275 m
.90431 .07478 L
.92228 .09275 L
s
gsave
.90431 .05681 -77.625 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(image) show
92.250 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 g
gsave
.49102 .03639 -69.2812 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(n) show
69.000 17.125 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
75.562 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.36523 .03639 -69.2812 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(n) show
69.000 17.125 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
75.562 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
gsave
.06873 .27898 -66 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(h) show
69.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
gsave
.94025 .13645 -71.4688 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 15.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
63.000 15.000 moveto
(\\310) show
65.750 15.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.562 0.000 rmoveto
66.312 15.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(h) show
72.312 10.188 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
77.188 15.000 moveto
(\\310) show
79.938 15.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .39623 L
0 .39623 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
    GeneratedCell->False,
    CellAutoOverwrite->False,
    ImageSize->{421.625, 167.},
    ImageCache->GraphicsData["CompressedBitmap", "\<\
eJzNfX98lNWZ75T8JCioqLXrb+uvIYi/USNK8QdRrKCpaLFTUTEU9zafQjfu
QlNXdvqhBAxRmrHA9JYRy1A36YZLB5oUHdiBHcgdyUBmSUJCJpuBnSvCJS5I
m3svvfe555z3nOecd+Z5k4n+s3w+M7w573u+z/d5znOec97za55+pXpRZdUr
1a8veOW68h+/snjR6wv+5rqZP/oxS8r7msuV95TL9bXZ17n4Nbhc6ov9Gzqt
v2bzL5H8nzl1ubjOg3Q6zbXg/7Mb7Puf+NfVxpNvOT3ZnPnk3zs9uTXzyZ9m
PGnZkf21LfPJGifM32c++ROnJ0OZTy5zenJ75pNL+dcYUOz+kHn/b52QWjKf
fMPpydbMJ6udnvxj5pP3yye731+EVsyT33PeCMIQL3f2xF0E4q3iOw8i9RWY
t1DmrfRHxbMo+eeZkh8Q1wUAX3TC2qddFsL0lXCSpd5DSHNLaelUDGrK7NJc
7hpIDCutTCImGuYKxDl1u0gppeK7hEmJQrXQqQJa412QiMchzj+JOERbA1DO
7nlbu4SwlbkIu5sQdpv4zod0bxg8XJjHB73pdJ7UtDfsh9pgbJRaUSV1uxIU
b4QyLqgyAEl2nye/nQvonQTonYpma71IK/eGXDSiyn32004Y6O+B8+yvQkLK
HcMUe29XHGLxBKTYX0UyLeqvFM9VBaKW5NWZkt3iOp99p2FbVal4esaKHXnK
9z7dAVNc0vcmL4H2T0VqFrHbCWI3KJumkxCodIv0sppmq/AKIN3VDG4F7a6C
1q4001ll6YX6CutefaRXUF+VSX2SpJ7wz5W1awkM8EgCwnpn962UyaUwZzr/
fxH0fXE6n2A/hWB/o6QSxrpbBbF0eoxILmRl6pXJbigXla0Soqm0UiAZ8Vm3
y7zQJf2o1kmBoc4NKjxA4FBKKQCndsIcmRw81AFNC9j1vAClwG2EAjcpKlEf
ojNbKgXSvSFRR3myLxIDv4ddV/ixBJIhLB1VAo4KwBcHYLF8mDkPKjCUhPUy
cM1p2Mv+7oHEoR5KgcmEAjcrKizSVEp05j+WAsXsuwtqy63k8tow+zsB4UiC
BdtK5u+xfAwdhiLhYRXhjv0ZtMha4HLNhcTnp11jpJLtDbMwBA8OnVa1oL1u
GmpRSmhxi3L3dAqCshbwmBlOpl1Kldbacpu38PxdoRqW5uFOlYdOVY8lWdvq
oEqppJtuWYoPu97YqbQ4+8kamTwN9nx6WhdVGsJ1SyHYsgPWLyhVGk1yjDf5
EG+s1gKqQpYuF7CYXCuTy6C5y9IFS7C8BsKJpEiLh0UbMUY2UL5ocoTm0FKs
BODYVpih5LJPU2fa6nx9vstqI3jD09KjItjgoa2wbLp+nJeTm1BrkmrXErIF
kB+/IpZshQqZVt2YGKMibKASm/Z8qYu7KsijsNYlq/2zdBknasRagxyPUd2f
8w5FCgLzrLQpbzSK9kBVm5NcIVarZjBFbnVsoMeJ+uAtM7ErIZJM22KrZOpS
2sRCPqhwW9qokqmsbeRtrtYmy+Umy5JpXzFN1pD5sPhpq6akWcn0qQBt1Rwq
2t4mrd9aUyZrggcqy60aEWfPRVUUtmpIvuSb7O2Crq4u6E2lHVpWy4MvEBZt
+90a9BCXaxaEO9tg1WSrLsyZXqpL4IvTVECdLD2cGzAUqEVvcLnKoTEagRq3
5fXlZW5t8JQV+39B8ypmobMHwu83wNo3l4BHcpjz/gF2txPWV01ThSCdQGRS
sVLyMrixoJhKQKOvHrzVVVAheZT7eE8zDrWVZWOcPPQlFaWSvdDDenHd7NPT
1QvHWeqUTO5WARbB+VNJGDz1mVX72GeI/X3yWFJ4K2VAqz9UBClear0pRV6U
YiLRJTzRYJMHx7sirFNSAXWCtPp2w8bqWojFei2tTWavOHXrf4ll1wXbWKVg
WPLjBn9lFXxYUwObKz3wrkxf523kuk/KFPCyk4B1yoCsuWmQIJv9YWFAFb95
RyjRXCvubfRHs8DnO4GvR4AENJdb4NsiqUJppm6/B+rK6mFARWcT9CUn0LXK
yMku6GN1qD+l4sBFgmmfVfwy+EWhiQsuq4Z21sfsjkXhYCQC7eEwxFpDsCdQ
D343v18LfaqTZLV5/NFENAqHY3FB8IacCb4hCXIyO1nLWSdCrOUFv//NGlWG
6BlNwVZ8Dr2l3MsI+mx5W1h3pt1XJf4aI76LYSMr7z6m5Idu5RflsDPcJUi1
15YXi8cKoMHbil57PN4MG92mY7IGLyhe6lzXZSr5fScll8poPRALwjrllKw3
nEgqy0dgkz1dlhLr+Aaq0ZE3+cK6ZxfRerir4TCLQcdjAeRZVpZhOm6YGqZ+
q8XAzRTlyYXwblk5bGbhv1+5QaKZO3ehzOuvRWu4rsnU2OOk8TJ05l7YLp25
wRtSlWR/jaRXXsvlKl0HwrWo67aw7sUaIOtk4QxEfVaxXsQYhmQHJAEtlWVY
tPt7mUmSSZG9pULW1ua4SBtIsU8yBcdVx2uc8OPdlmdJq7LOftTyjoGuKGyX
0C3RpOvaTEN8z8kQPyEMwXRQhmivrbCEldkMwQOMcpTm1q7hDNEfrrUMUQSb
mnvRcfvRlOUQ61XuVCTGDZp4bKzwwn5WsfeHQhCLxI0QxtuHMPgxdjJ3TNnD
28celwpFWR6hDGG2W1/FEMxf/ZmGmEAagrd+TbJGNFQFOLthLFEs4qzZSjRU
lMtyZ/3xeBJ9MurDRzYH4wUqXiVjEG2OCLtlGeFFJ2+o+ZJGCGcbIQtE9c35
MMqHMrnO44WDrH9+PFpv2kDaKwUfV8rgOA6aGuOyYvllA+mB9qQyocpeAbGk
fGkzFZ7npPBPv5TCLJBhI+2ssOof9oSDsMkqSJeqCjZFmfk2SrR3K4MuXZu6
4ENdskp71Vo0BUPQzpo9XkNELWltZV1nft0KPcl0VgiY5+T5I9mgXtqgvN60
QT8VCx0KfTwcDvmh2VsL0Zj1ItMXD0NLdUV2ofe2YmDx10d0VIigiRpqQmg2
v7CEbHlDXajX1ZnKf9fJAd404gahPAZc1uq196LyPY1moydpjhUl1lyWWe1Z
mqz279aEXOpZ0+X3oye0mmEVn03Hg+hvHzYmjOphsdseTrrM7o5qTax7vdnN
wQvDm2MCdLPy2uatxl6k0uZ4ohWaqyotNkY3ZXtjGA5HgqJAVBvvr/ZBO+t4
fVxdLv2+kJVpWLJPIktWoMquB+s90tw1ugtXCNFq2Wp6/DJsss6Yx22BjmW1
sxKaanzQnUqLN2eLdCXE5Jt/d7NXEuAdDy9HBrK78LyTWf5exq6+SCM0VVZg
YWBXjL0jtNRUobm4SZqqK1RnbxPrg8s+t7TOWEHwoOpf8RgW8SPsJq8PdtZX
uZSCTY0xfM7qScSwC95QWQMfetwubYxyWFdexrpNVYg/ENX9uoZytzYG82pp
pGwfed4pXixXTUyqFw42+6GlMSab32LhfYfDjbCz0ZpY6Iu1wm6/D1p8PtjT
Gkcn7Y3D/mBAJO8MNEN30qqDIuwlExD117I3oUrYxD9VXtbP6S3AqJhiUgPQ
XFPNXpi8sN0fhIPx3kJ1O9UFsaAftvsCQvDHrA1kLnN9pnZznYr6SOaTzzk9
6VV2YIx4X43/9ZxT7iwr/oMKPCziqdC2ORDNAvjO8OKtxgj7CDUhR4AsBgrA
eq9yi3eHbaF4FkCFE4OANECdgfxAZu5nnXJv5F+ik++YJ4vyMHmeGVYOb3Fd
9zvlyZLj5188vY5XVNd9mRnnDPsGzofg6vBunahsJfx7aibObCecBktRlnsM
UrmQQ9zrBJGlQz1C5COEAJvAce7JxHnaicrbqFIh4gjEiznO3U44WXx+jnx4
mEiPQcSJHOeuTJxvD+v3AqcE+Yippss4zp1OOFl83kScC4zSZleqQ8Gfzny9
N9PEG+gdmfKeGvZNS9hxPMobh/LGoGePwbTbnbCzdHkDdbkIbXsh4uQhdh6m
ZY2nzXLivQSxL0He4xEnH7HzMe02J+ws3j9E7EsR+yLEKUDsAkzLGm170on3
62jvyxH7EsQpROxCTCt1ws7i/RryvgLtfSniFCF2EaZljbM94cT7ZcT+K+R9
GeIUI3YxpmUNtz3hxNuDNrkKsb/Br4Qvj0XssZiWxbvcifd3kffViH21Ja+O
xz+FXYJpWfYud+L9HcS+Du19vXVV5xJjIxb2OEzL8pOZTrznIPYNyPsORMxq
Z2Y6cXwKbXsj4tztjPO4E59y5HMT4kz9EjiPIM6taLMHRsbJ0uth1GsS8nnQ
GecxJz5lyKcUcR4eGSeLz72IMwX1muGM86gTnzsR5w7k8+iXwLkN7XMX4swc
GSdLr1uRz92I86QzziNOfG5EnKlon2+PjJPF51rEuR/5zP4SOFeifcoQ51ln
nBlOen0d+TyIOM+NjJPF5xLEeRjt84Izzrec+ExAvb6FfOZ9CZwS5DMDcTwj
45h6/bP4FjkxbQbCPoqw851hpxP09qB9piHYTEx7dXRg/4IZp6LhnkBmr40M
Ziq8j1B4CnJ8EmF/MDrY/cjRjWBPIdhfO4M9TCgcIzhej7CzUVQVworWS46n
bnLjKPEVdPJ4JxKmRp+gmKvR6s+gRj8iRMsRKGsO7Qo6OSfR7Sj6ctT6WRT9
Y1K0GEW25lyuoJOzRD9EmP4QYfrxSOI7SOKNkUgUS6D+RBwOxxNiNP3Cr0Cs
A21SgsXxPKYtzaBTnCkDxOyGHPRj7/pqQsScA5BjOmKaeTQ0zaL7V6SUh1Z7
Aa32kwyaYwmaaT107q6G7nQaR7X7W72SqxvWlVsDdYlU2qEAcqdbgPFXkHwz
J5IDOHHgho9jKT303tWMY3ktkRjs5EOVFYFcSU4jiv4gkjRGBviN7yLn5Tlx
NqcL/L6IbbpADz5bS9HaI4nRcs6KHvrVP6NGzUUv+VkG8RKauDHmywd4e/Rk
Qswr57bKRHKuvktx3od2Hktw1nV/xUh1Hye4fR6sXBv9MZz/aFXzH2WwJ/GV
KEeQ8oUEZR0za3OjXALHo34dD9hnv1oC16tnKzcH47lyfpBw513oGpcQnGcj
57dz4zxOLM/bZHDmqzz4ogFzQqnOExAhOFefpoj/EY19GUH8afTp+pyJGxMx
fDB+M6+DZV4e9LTv8BUQX4K46SXbkfg3COK6m/LuSMTzpJd0B6s0uS4VVFhU
LlOLDMRs6lfhvBU5X0Nw1t2/htw4F9pmwnbGrDVBzZ4ynNCoxHl1a5whB95l
hJP8Dr37BoL3TOT9y9x486GxOC6H2eiPqnnNgd4uscItjbHd6mdtC3VZ4ztf
kv9v0e43EfwfRyffMBJ/tYA5YwauQD2N87IePtlUNErKpqt8gJQnEZQfQZP/
KlfKPc010o0FN9WC7ZGaNFQ3unLga82eFMLgsQNw8sxncPZEJ/T1t0H30Q4Y
PHd6gNDk16jJFEIT/bK4MVdNzGnh7ZFetTrenIHfHu4tHFkZayanGPia+LUr
XOB6S374mllxXQqr9rb9G6HUBqwRdxFKPYxKbSKVMt4dVJ+1J+SDTWpJC++y
1IblzgvuXEZbIPuPBTRctnpjQSyg/8BSafFHfMn/aYATO8Aj1V3b0eNKErXm
PSy4qYSO07DWbCZ0jIi4v856NVPRqj8Whj3BAGyvrWG6lrHmQdQeVVK4yoz3
5GrDBTRYtjcWQyI0i2mynO/CsrQbSkPgHUu7GU07KeXeReXKCOX00MwWVO4a
WRoDkXqkKXo78ZTrcoLUwO654NqyUxI6DeePBWCKNPmMrSSpOvSqhwlSetzp
H7NIpdMx2Gx0F9bVthKkrM0U588pK/XAqtViUflbvLXwtncqsq6jhM+vRJM9
QrC7F9n9DtldK4sQX34sxzaJPS+JJUJzdf17pwHOstQkQUJPd80kSNyDTrk1
iwRfJbCtygMbq7wQSyRNEnMlifPpRiwi11uzYM+J065+goSeK1N9j5F2E76I
1tmGxB7IYlAkPdlisGyfVR5njwZg1e42LBPTY/QsWgbFKyWxkTYe6vfY7cMQ
K4b2rZLYhoDaXgntTaXg8om/ewlub2BhSG68pPhf+Vhymq+YgMl9C+NcpN0y
LO30viUWbV8D3+YgbDmD/716Df+bor0YC3cMwZPifrW6M/KOyAq0yc5heOcB
nOuB4AfC5Fg/p6xm5n5rPnSfO+3qJbzyvyDxohyJX6Pu2HdYPofG/Zgk+YKs
LkOneuDkiR44e8bab3B2ME0Gjh+gk15AkChwJGbfpalf/HaTrGw7g02wV9Eu
F+Uo/hIsUOZ00ShEI1FIJFPqDdrcwfk0stozOlbfR1aX5cjqYvFdBL0RP+6e
c1sbg8BjbQjSXTj7Zs1Z6HbR0bGch0V3JcGI8inLxtxUMblDsAIa49bGkog0
nIs1/EmDbcbOTP1Csd+Z7X1E1Z2LNr12lGyTYbmNj/USe40BGrEDkxd1MGZ0
j23bMPV8Qmx0bJ9BtjeOim0+hGtla+MRdlRDXmhGuclp+F2XM9An2kfH+ynk
7SY4Up47QTKI1HtkOQsr43JW9jZYLR2ZN5rGNkuXwz5L3cM+ODry5ejQU0ZJ
PqzIs9YzrocNQl5jn+jo9lQ+gEokRqeEnsu+a5RKxINV0ozl0JrEt48UWtcN
AVZZo4FK7C2Y+9LuQ8adIzM25T+EjO8bFeNiY+8qawYqq6GmuhqqxacGqiuk
XfPB47M6W3K3ZJ7qgxkbJfX875HRkX8AfeahHCvqBKmWuc9bKODxMMYVFRXs
XoWnQu+1L68X4Vv1OMw9lFPR5r2jo30P2nzGKGnzsfIat6JWK2K1qoit6O+i
E6lfSpKjY3c7spuZI7us6MecNZjAMSUraliuHepN6/eRgdERm4TEnholMb7Y
WpUo80cc7IoH5e5lVud603ejGx4fHbGb0Q2fyZHYxfJOY7UbQ1fCvlHF2jDr
4buQ9bqP9OiI6Wnc50ZJTDPgG7jjSCwRNIvyNiR2YnTErkJi83KMOhaxsSzq
1COD5oTsD6TC1ukM7mreALj0+p9To6Oll268lCMtq0NaBPFmL9Lyh8IQ9FZg
98SNfjU4OjoT0a8W5Ehnogpv8VaorfZAmdsNZWUV4A1GrLMVhFX+g6Qh3nAK
+SsN9aqgp6V/kCMX0YyLtqoYJX9hpdXZ1ziLtefFXDL19jQWJf8wR8nCLYvB
Jdd6Csl/stLq7IukxehLCZfcQ0guQPsvyVGyWM1XIpu4dNqaPOVv1kPWjTr7
2uqRxKup/1wlC52Fuv+blHeV5WkuvrhRCFWP/wGv/mLdFRnZVRbRjQZRNSY1
lX+b4wmK1Dbl+RY2V8j1f0awBCXgXvEtSgJH7dj9EFas4i+LfDdSzyNKoBnd
p8RZgPbeACHgLkxD6i5KVBOKyseS/Avps7qevk/IuwMVKiSkfIhSirF2/j+y
TgopBVzKJkLK7YRWlLwtWEJiCdnsnAXchmqMJWB/g2oI2Gdyhp1M8KYEvG8X
8J2cBUxC3hcQsL+2wz6fM6yb4E0J+JXd3vNIAe/JjCMd0pZPELkZ9ZtAiF9v
1+97pPhfigT6pLYiQuRNhO6U8Pfswl8iha+TGc3jlIoJod9EPS8hRP3CLuqV
ryTqBkI/Sui79rJ97SsJvQ71u4wQtcau30JS1NvKSU51Ql/nAeg7dABOfv5Z
McFgLMHgGkJtxSVtvFKvtnN5neSyWiQUwdlDATyBbcpk6/+FDVvFpAV1RBxF
7Co0zRUEnZV2Oj8k6azCwNIhj/qaC+F+ayy2e4usdVUBXtPIA+IoWlcS9qII
rrC7yY+GJTj0SYNF5uk13EYSOG2dnsbSl23rIAmWEASvQLtdSdCah7RusVtw
yTAE81Goa4GwljqYa+iQOgROHEBEHZJGUfw6YUOK7PeQ7A12stXDku32z5fR
TFhTkeWnE3mlMy7eciBXspehPa8lqqgHKV5tp/h3w9tTUZy8nJ82lSeT2+ps
x7WpDop5XNs4guJEwp4U2ZeQ4hV27/zJsGTT25bIUp4F7Z9LViVw/pB0Wlcp
bBO1ijyOjSJ8Mdr0BoLmK0huot2m1tBYIfTFotCTTIlltoloBA5GYtCXSk8f
Roti4xA5RqtqKax6cyl4xWc5eOfhONlCcY7VaXUEmxqWZIoge8pLJhBFQOm2
ADW60K6beAWUJwQ2u/VEdZ1bTfq7oSkQeXgYHQtshzIKPRfMZxp45s1l9zwL
5upTMZ+2JpDHSIbm8WwXENpdiOWlegFmNV1o9P+VTmrRQMTnyejcMMIVPmMO
gZ8a6MWOjxzoE7nF+7F470zB7irLDJt88kCARLM+QySUsMZPHe0iZvInK93X
iICv6lw71jmxPuJiQv0LiMKlDLEIDZGPV9ZChQLoivhtR/TVNMdduErNmEjh
H09tiA8pfiFdor2WL3sRK12lzfTZEw01IVr11ZmRm9XSlmMYaQYjy7FOt506
fQmh9VgsdOrEPrG7QbwKWoOn+RCscqMG3tZe88hUrXiVGBk6JzU77K+AuuoQ
esLxmN84K2EEzfiZisqhWa1VmkF/ozxqkUUrWrNiojwpHRehjo9KHRPNNahj
uT7/Kaq9HCrqw39SqqfFiUxSvQQ/qAjPUviwOY5B4cFhO3bhN0uxrThpNGef
75UHA87nxy9OJPQswBKkjt+rRB/V8/gZK+j4giWz+MpwXQTYhsD5QHM87fpf
Um9zOYs8Bsahgr6dpQw/B7MTdTzWaHrppYSOeURZUtq+iqEp31HbIjDPZ+XH
IjbKoc1kxIdNgRmi/iw1Pm6sM+WHeUQTcpFblsZ1sm6dVV0+ptqeY7LL98Ve
6/DcyUt5r8B1qTSDoS+W6O2EjvNRR2r9/WMy2ujpAT4uanlhczUes1jVqB3z
z9IsVgTCheDiPj+ApMkfsda5Zam5RiQwg368EtVsiuyFljorHPEe7eUZxckH
rqgjqT2oFrVE/zHZSpgToO5KPyQSIeO0ywo+4SsdtFjvH8ADYtIQq3FDXYX4
m66P9aqR7d8F69+cDzMml8KM6XNh7bY2o0frUp5IqvIi1rlxhCqPyzS+TFbW
BjH6KGYZKyqhPtgqZsH+LNXAFdkV9eKQGl4kYhuKtdaRVuMfiQpDnS6uFz5R
uwwex+5KCs/JFtHP18osH4NwcwCqPehUxiIHGfzzrPPtqkRBYFR8V6wl98DB
lFNU/C3WgKlEDdD7NSYQpMtluNaTvDxcR1RLFQ3oZHFydkqbeqArAT2JBPRb
B7dAf2/SXH2dNYAvuG4hTE2x1jvsqM0R5ao1SQT1fCgr4a502jjFqct2bm5t
uNd1XmqbseiaHl3cjGZ9gPAFvfvwUoLgE0jCbBLEScoGQd7FMTsIXda4qsu+
xJpm9xvCkBRPvavhcoLnk9IguCLBZZyKTNvwLzJLxtrprHFgPWIo979mc9O7
F64guM2SkdbsSfBlcolUCrriEfDjVH6e8k2R4zySz14CPZVk+T5hS2qX3GPI
90qC7xOqEicjUCWXTxZJfhVV9RC1jpJy/V9pwIy1yzS1jWhAatfyI+iEVzsW
rnW6klwTyczCv/MgnuiCpDQYSINRq5TvJVn9mjAYtTt7OhrsWoKfkPscllb2
amRa+K/QJNSBAtNQ5PVOIp+XBZCxxJiW5idUpQ5W0IsabhxJbsaqYlruetSS
OtJDr0u52UnauyTsOkId6jwSfYDCraMT8B7ypo7D0esFBMQuEsJHcKTO7Zls
B4uQYL9APtQxT5PQjgLiX0iItQQf6tipjOHDfSTYO8iHOtbrRjvEfych6gk+
FYRy19nBDpBgdchnrrqylr9rqxwkM75NsKCOhLvSzqKDBFuFLKgzNr9uhzhM
QtQSfCiwS+1g3STYz5EPdf7xJXb79JIQKwg+FNgEO58+EuxnyOdlQqVxdoh/
IyG8BJ+XCT7FdrBjJNhy5LOAgCi02ydNQrxF8HmNUG6Mnc+nJNibyOcHGXz4
G5SmcpLM/VOCykL+rcWeJjP+xC5WjhxkG0RAnCIhljmZQb6Yjwrs75CFhigm
bDqMLf6W4PMKgo1z4mOCqXNs+Ea/gbT1K0gXoYBq5PgqXo3PBdba8ih/4mhj
qdqppX4gZKh/g7Ebbwkf6LKGoIXU76KsbxAKzkcFL3ay1mcGkwclE9yAtEIM
TuAvFh1eiXsB54gtZYug79zp8UhGHB4tBtEmIi1N4VInY1AUho40oKxtxz7T
vzm0w9qzwj7Box3Q5OP7WYKawktIYTxhDw+SucLJHicIMnBmLyyUUj0ftZm/
H7R2tZU8Z6v8/aCjPZrMq0imGO3xfbyipsEcKBQI5wj6lBeI0UH9yz9qP9Jq
ka6dYyHKp4bOXkRjXJ0Lk2lSWt9H89EfF+7uwB/uObxGJk+D8AmDxF+jGC3w
eieBn2YJLIHz/QFdAdin7ZQcHhvchfs2l+3rpCTq2nFj7hJZLDh3ABYbEl2r
l8LAOfmzN3IvpWtDUAQAXdqvS0sYNsbuh2Bzi5PP/Q+Kw9BnOhpsXA7LuKOt
lj9Yo4qA1c8BG4fscG2W+LNIhfrhpCwqD8kCwI1jXNwpVRVYDFhdKk00F9rP
GHFgISGb4jMb+UxxMk06iw+rd8bWxCY+nnrmAKzdME3tDXNt1AtqLkROr6Fl
Cggm38a71FCeAxM5z7xC1YUDSsehwSScPJUSf2m3XEDIpbg8iVa5x8kq/25w
eVhy2bNlmrkrcSLKfQV1oyZDZqI0aneFo7ST7ctl2c+H7jOGtJcJGUVE2mMo
lzodQsg9TsiFM7tgjiz7wNGUlvsSallCSHsE71LncWVJe0gWzsmODbDYp7eh
zgntdWmR3ycEUcKno6qOJ88dI9z87LG9EN4XhEBoOSMxjUUBUbCXoPzvoVIX
ElKnoVTHcxfpPvCLBBgl4AEU4HioaIoUoKMyNZV+P951PC+Xhn2BAKME3Iu8
HQ9IHiAF4IuksUItY2BdJDmea03DPkeAUQJuRwGOJ5TTb0gVyPsyAlav+3T8
JQQa9lkC7FIirRR5O/7UTD8pYA4yo1YR3Yqw8/FuRjEmSdjZBBgl4CYU8KoT
b1qAbky+hVcLnTjS78VPEXweQj6LnPjQYE8iCw3xQyeIoyTEEwSfMgRb7KQc
PYIwE/k8iFd/48SHhnic4HM/8nH8ReYeEuxRZKEhHH8emoZ4hOCj48ybTmBH
SDDtNXqZ/z84QdDjPNMJPncjn5+NDuwhZKEhVjpBdJEQ0wg+dyDY205gnSRY
GfK5E6/ecXJBelDtAYLPFOSz1okPDXYfstAQ7zlB/CsJMZXgo+PmBiewBAl2
D/KZjFf/dXQQdxN83Mgn4ARGD4LqUtIQv3GCOERC3EHwuRnBfusERo/wTkE+
t+BVk5P/xEmI2wg+NyKff3LiQ4OVIgsNsc0Jop2EmETwuR7BdjiB0UPntyKf
G/Dqj04Qn5AQtxB8rkU+HzuBxUiwm5CFhtg9OogbCT5XIdheJzB6nkJb5Wq8
2u8E0UZCXE/w+SvkE3MC20+CXYssNETcCYKevaG2BujV5QknsCgJlrGcX6B2
OkHQU1PU0vvLEKzHCWwvCfYN5KMhkk4QewyIJ2TGt6yl2AV4lcevqFc9Pe6Z
chJgTueJGcp8BCuwXxUh2ETk/e9OsP9swIoOltivNhbBxFURv6LGHC5CASec
BOw2BIjul9iAV4wCijEtD3lr2P/pBLsrk3cxqq6NoGEzeI9HUf/hJCCcaW8B
VogCCjFtDIKNR95fOMF+nOknxWjbArzSsBm8x6GAIScBHxkCnkSwfBSQj2mW
ZQQ++xjY552wdxrYs+wGLsCrTGz+dAmaCDKw+XoQgf1HA/spuxEK8EqmueRm
wzxElL6TTmfuthXYfzCwv53F0RqkceURaYXqjlEGhXbJRcNK3mFIfpr/dxmB
mG9HvMDJ/CaYeBudiI6obXuRU+7tRm7xinwxuoOWPdEpd8jILX7kagIWis79
9YzcWLi/N3KL9/8LLXdRGa8c1ohbjdxiUGIcYUSBc40TfRNC7DMeK8qYX33T
iXSzkec5DIMiz02WWHb/v5FNB1j32dcXI956VvnAmSSkj3XAAHt9OXvmNFb/
t1x2Hc286ufGTravRJ9V49kLQ1txPFtn/JOZsQjam9Q5nS5Y374XtslDLhfu
bjMy/dnMVAxDLBCs37Icmg6L5hmGjqg5nCV8XiFL2HNSw7P43CxoG8x+DgeJ
j7VB++Fd4tN9rMcaBrYOXhw8xV+k+ftdj9iiUUhAtw+ezjOypI8dgL7+AzBw
rNOWhU+NnTzRKeAGBz9Tmw7PnrDQz54zzWYdd4Dvo0OsLi5bbY8brneWQvsJ
sXUQYzec64Tglvl82kE/91YpeHfvElzyafoq+3kmZuEKe7HyibK1+w4oAeJ7
ILZGTW/ic56moDiuT9MpgHRsOZY2/6zad8DQ0eoeyanDEnHOHz68Yil0D1pz
UwMx5WvTIJyWU6wl9mMXW7bCwIkks2UbBDbI2Z4NDXyWMU86XobGFokTYpZW
Rt188IRwRxV0fzTfSi5hhl5jHFPaCV7UqBQWf2BNca0/ktKKHbEUK0OnMPNY
R1RatMbq2RDr/EwBoae1VxpbonjRJSHgUyP8u/Jk65Ct2WfQsrHUKhcWdlYs
gj39SavinDoAgY2WxGB/mhlgq3AV69nxMGVjA3SfSMHQYA+07V5qSWqRx88O
6hmNKVvEqxqcPLITEtwFSc3zYbBDl/+Mph35Umk9QS8mR+xKvyN2SLrU3hn9
rNhvMUZBZCp9Zi9OsvKw0HdOQ4gFCxv4tGQDfBKaqxx2ijH/NkZ6yaCQVsqn
CcUUjp637TAc96hdS5zXttRRWpprALyxTruWq+1amrLWdiRV/cnUcqh/A9Y3
ddSorpcsopzpgD3tbXAQ6wbztlOnC+Tt9NEDlpMZ0tYfTjkq1o1T6GLiVClm
Hrk6o2nn8Iqd2oles/awVKw4S7GzuIxCSCqSuft2LzKdooiy1uGkI3/zgNrw
qdOU+y1saRuWvyYqCtG5YAKyYC5gQY5VisM7oa3D+oiWRVzvgpPMr9pV4Vgz
86NVSq9rEI2aUqrNaFjXHk4Nq5S5NAILhQgkVp2y+q4FsKojqT3ChlgM51lg
2RPbAX2nrK3sNg874uxh3D1UHVm8t0Pt4zWzc5doG8yIjIxhi2oKCuD8sUZE
kQtxXMrTzh9tsCtlrFcIHEmreMKTcd2KuJUyWIilIi79bA+sUmePq7htMF68
90B2sHhQRhhdp+ZD2wlpLQa43qfCcD4s3n0A26GEClj5QrVlWzfAtt0rubpW
MusWb2zkfiTa/qatK8G7YVqGxqdFoLcstIgvzBJp6faVEmOiWLEhG28Y2CuX
UbyzUq4m4T3PoD5lmaks4qZRtVUYOG/r+/WYnZhiSMgWRbQgq2fZimzVvjbl
nbqQp8FCX2m+asdMOkMp1j6tgYXvlOKjzEewpRzqb9TU3inFroy1LM2I+GOh
e+9yhFi2ZZFBirnYsTQPFq4CzL9SNdKGlt2Wlmo6/PypDgjv3QDrQ2sg0NIA
TSwcpHXw1O2cZcW+jkYIsEfXbl0DTbG9toMfeHPd8lEQBs4YIf5cEtr3BSDA
0lt2M2dg4X5oSIb4saIitny0BrwfLGLqrBTSTxpWa9kyS/fsfCuh/QgLTkc7
swutPLM0rNOXv2m2qbIazQntzXJ3KvtZftqSpdn5M2kYOmef0+RHYohuUtPO
7OozE/tSXIdptiA0VsrSvc25vI+lls6dT++CPUd6MkpNdrAfx5bT7D8gtoLu
/mgR3uJxEGH+lEHvXAoGjuyAAO+D+xps9AaPbJDnWMu+bQaGThBfs0ef6vra
/wem6LUp\
\>"],
    ImageRangeCache->{{{0, 842.25}, {333, 0}} -> {-41.8322, -21.0502, 
    0.269907, 0.269907}}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.401219912921299*^9, 3.401219915947401*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "\tThe underlying assumption is that all angle are small and all rays hit \
the interface at a small distance from the central axis, relative to the \
radius.  When \[Phi] is a small angle: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"tan", " ", "\[Phi]"}], "\[TildeEqual]", "\[Phi]", 
    "\[TildeEqual]", 
    RowBox[{"sin", " ", "\[Phi]"}]}], TraditionalForm]]],
 "; the \[TildeEqual] will be replaced with = in this discussion.  "
}], "Text",
 CellChangeTimes->{{3.3876355835084467`*^9, 3.387635672475912*^9}, {
   3.3876358827782*^9, 3.387635912808522*^9}, {3.387635967499041*^9, 
   3.387636005074247*^9}, {3.387636035525217*^9, 3.387636080626515*^9}, {
   3.3876361507563267`*^9, 3.387636156167385*^9}, {3.387652928812895*^9, 
   3.387653039591601*^9}, 3.387655801318891*^9}],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"sin", " ", 
     SubscriptBox["\[Theta]", "1"]}], "=", 
    RowBox[{
     RowBox[{"tan", " ", 
      SubscriptBox["\[Theta]", "1"]}], "=", 
     FractionBox["h", "s"]}]}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"sin", " ", 
     SubscriptBox["\[Theta]", "2"]}], "=", 
    RowBox[{
     RowBox[{"tan", " ", 
      SubscriptBox["\[Theta]", "2"]}], "=", 
     RowBox[{
      FractionBox[
       RowBox[{"\[LeftBracketingBar]", 
        SuperscriptBox["h", "\[Prime]"], "\[RightBracketingBar]"}], 
       SuperscriptBox["s", "\[Prime]"]], "=", 
      RowBox[{"-", 
       FractionBox[
        SuperscriptBox["h", "\[Prime]"], 
        SuperscriptBox["s", "\[Prime]"]]}]}]}]}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.38763623306329*^9, 3.387636365495026*^9}},
 TextAlignment->Center,
 TextJustification->0.],

Cell["\<\
Snell's law relates the heights to the distances and indices.\
\>", "Text",
 CellChangeTimes->{{3.387636401523995*^9, 3.387636412173048*^9}, {
  3.387653113655861*^9, 3.387653124181755*^9}}],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["n", "1"], " ", "sin", " ", 
     SubscriptBox["\[Theta]", "1"]}], "=", 
    RowBox[{
     SubscriptBox["n", "2"], " ", "sin", " ", 
     SubscriptBox["\[Theta]", "2"]}]}], TraditionalForm]]],
 "  ",
 Cell[BoxData[
  FormBox["\[DoubleLongRightArrow]", TraditionalForm]]],
 "  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["n", "1"], " ", 
     FractionBox["h", "s"]}], "=", 
    RowBox[{
     SubscriptBox["n", "2"], " ", 
     FractionBox[
      RowBox[{"\[LeftBracketingBar]", 
       SuperscriptBox["h", "\[Prime]"], "\[RightBracketingBar]"}], 
      SuperscriptBox["s", "\[Prime]"]]}]}], TraditionalForm]]],
 "  ",
 Cell[BoxData[
  FormBox["\[DoubleLongRightArrow]", TraditionalForm]]],
 "  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{"\[LeftBracketingBar]", 
      SuperscriptBox["h", "\[Prime]"], "\[RightBracketingBar]"}], "h"], "=", 
    RowBox[{
     FractionBox[
      SubscriptBox["n", "1"], 
      SubscriptBox["n", "2"]], 
     FractionBox[
      SuperscriptBox["s", "\[Prime]"], "s"]}]}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.387636439642475*^9, 3.387636459377553*^9}, {
  3.387636511105527*^9, 3.387636537879436*^9}, {3.387636570948657*^9, 
  3.387636633052931*^9}, {3.38765315027077*^9, 3.387653188260714*^9}},
 TextAlignment->Center,
 TextJustification->0,
 FontSize->12],

Cell[TextData[{
 "We then have a pair of similar triangles, one connecting the object and the \
center (at ",
 StyleBox["R",
  FontSlant->"Italic"],
 ") and the other between the image and center."
}], "Text",
 CellChangeTimes->{{3.387653210009096*^9, 3.387653280725605*^9}}],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"\[LeftBracketingBar]", 
     SuperscriptBox["h", "\[Prime]"], "\[RightBracketingBar]"}], "h"], "=", 
   FractionBox[
    RowBox[{
     SuperscriptBox["s", "\[Prime]"], "-", "R"}], 
    RowBox[{"s", "+", "R"}]]}], TraditionalForm]]]], "Text",
 CellChangeTimes->{{3.387636439642475*^9, 3.387636459377553*^9}, {
  3.387636511105527*^9, 3.387636537879436*^9}, {3.387636570948657*^9, 
  3.387636633052931*^9}, {3.38765315027077*^9, 3.387653188260714*^9}, {
  3.387653310040247*^9, 3.387653334526918*^9}},
 TextAlignment->Center,
 TextJustification->0,
 FontSize->12],

Cell[TextData[{
 "Combine the two preceding expressions to eliminate ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[LeftBracketingBar]", 
     SuperscriptBox["h", "\[Prime]"], "\[RightBracketingBar]"}], "/", "h"}], 
   TraditionalForm]]],
 ", cross multiply and then regroup."
}], "Text",
 CellChangeTimes->{{3.387653418526703*^9, 3.387653427533433*^9}, {
  3.387653529070068*^9, 3.387653555117638*^9}, {3.387653822365696*^9, 
  3.3876538795263453`*^9}, {3.38765390964633*^9, 3.387653933622676*^9}, {
  3.387655594072071*^9, 3.387655595301977*^9}, {3.3876556263501043`*^9, 
  3.387655665701779*^9}}],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox[
      SubscriptBox["n", "1"], 
      SubscriptBox["n", "2"]], 
     FractionBox[
      SuperscriptBox["s", "\[Prime]"], "s"]}], "=", 
    FractionBox[
     RowBox[{
      SuperscriptBox["s", "\[Prime]"], "-", "R"}], 
     RowBox[{"s", "+", "R"}]]}], TraditionalForm]]],
 "  \[DoubleLongRightArrow]  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["n", "1"], "s", " ", 
      SuperscriptBox["s", "\[Prime]"]}], "+", 
     RowBox[{
      SubscriptBox["n", "1"], 
      SuperscriptBox["s", "\[Prime]"], " ", "R"}]}], "=", 
    RowBox[{
     RowBox[{
      SubscriptBox["n", "2"], "s", " ", 
      SuperscriptBox["s", "\[Prime]"]}], "-", 
     RowBox[{
      SubscriptBox["n", "2"], "s", " ", "R"}]}]}], TraditionalForm]]],
 "  \[DoubleLongRightArrow]  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["n", "1"], 
     SuperscriptBox["s", "\[Prime]"], " ", "R"}], "+", 
    RowBox[{
     SubscriptBox["n", "2"], "s", " ", "R", 
     RowBox[{"=", 
      RowBox[{
       RowBox[{
        SubscriptBox["n", "2"], "s", " ", 
        SuperscriptBox["s", "\[Prime]"]}], "-", 
       RowBox[{
        SubscriptBox["n", "1"], "s", " ", 
        SuperscriptBox["s", "\[Prime]"]}]}]}]}]}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.387636439642475*^9, 3.387636459377553*^9}, {
   3.387636511105527*^9, 3.387636537879436*^9}, {3.387636570948657*^9, 
   3.387636633052931*^9}, {3.38765315027077*^9, 3.387653188260714*^9}, {
   3.387653310040247*^9, 3.387653334526918*^9}, 3.387653452399262*^9, {
   3.387653487656301*^9, 3.387653504153994*^9}, {3.387653560646818*^9, 
   3.387653654245204*^9}, {3.387653688015815*^9, 3.387653734596531*^9}, {
   3.387653794723002*^9, 3.3876538084131603`*^9}},
 TextAlignment->Center,
 TextJustification->0,
 FontSize->12],

Cell[TextData[{
 "Dividing by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"s", " ", 
    SuperscriptBox["s", "\[Prime]"], "R"}], TraditionalForm]]],
 " gives the expression we seek."
}], "Text",
 CellChangeTimes->{{3.387653941992021*^9, 3.387653986141631*^9}}],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox[
      SubscriptBox["n", "1"], "s"], "+", 
     FractionBox[
      SubscriptBox["n", "2"], 
      SuperscriptBox[
       StyleBox["s",
        FontSlant->"Italic"], "\[Prime]"]]}], "=", 
    FractionBox[
     RowBox[{
      SubscriptBox["n", "2"], " ", "-", " ", 
      SubscriptBox["n", "1"]}], "R"]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0,
 FontSize->12],

Cell[TextData[{
 "Since the image is inverted we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[LeftBracketingBar]", 
     SuperscriptBox["h", "\[Prime]"], "\[RightBracketingBar]"}], "=", 
    RowBox[{"-", 
     SuperscriptBox["h", "\[Prime]"]}]}], TraditionalForm]]],
 ".   This gives expressions for the image height and magnification."
}], "Text",
 CellChangeTimes->{{3.387655882616145*^9, 3.3876559006698723`*^9}}],

Cell[TextData[{
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "=", 
    RowBox[{
     FractionBox[
      SuperscriptBox["h", "\[Prime]"], "h"], "=", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        SubscriptBox["n", "1"], 
        SubscriptBox["n", "2"]]}], 
      FractionBox[
       SuperscriptBox["s", "\[Prime]"], "s"]}]}]}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.387636439642475*^9, 3.387636459377553*^9}, {
  3.387636511105527*^9, 3.387636537879436*^9}, {3.387636570948657*^9, 
  3.387636633052931*^9}, {3.387655931671739*^9, 3.387655944990653*^9}},
 TextAlignment->Center,
 TextJustification->0,
 FontSize->12],

Cell[TextData[{
 "\tIn the diagram ",
 StyleBox["s",
  FontSlant->"Italic"],
 ", ",
 Cell[BoxData[
  FormBox[
   FormBox[
    SuperscriptBox[
     StyleBox["s",
      FontSlant->"Italic"], "\[Prime]"],
    TraditionalForm], TraditionalForm]],
  FontFamily->"Times",
  FontSize->12,
  FontWeight->"Plain",
  FontSlant->"Plain"],
 " and ",
 StyleBox["R",
  FontSlant->"Italic"],
 " are all positive.  The sign convention of ",
 StyleBox["R",
  FontSlant->"Italic"],
 " is simple; if the center is on the side of the refracted light, where the \
index is ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]]],
 ", then it is positive.  If the center is on the side of the incident rays, \
where the index is ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 ", then it is negative.  Just as before a positive ",
 Cell[BoxData[
  FormBox[
   FormBox[
    SuperscriptBox[
     StyleBox["s",
      FontSlant->"Italic"], "\[Prime]"],
    TraditionalForm], TraditionalForm]],
  FontFamily->"Times",
  FontSize->12,
  FontWeight->"Plain",
  FontSlant->"Plain"],
 " implies a real image and negative means a virtual one."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Flat Interface", "Subsection",
 CellChangeTimes->{{3.3876340732188587`*^9, 3.38763407693578*^9}}],

Cell[TextData[{
 "\tFor a flat interface we let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "\[Rule]", "\[Infinity]"}], TraditionalForm]]],
 " as we did with mirrors."
}], "Text",
 CellChangeTimes->{{3.3876368078725*^9, 3.387636860098565*^9}}],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox[
      SubscriptBox["n", "1"], "s"], "+", 
     FractionBox[
      SubscriptBox["n", "2"], 
      SuperscriptBox[
       StyleBox["s",
        FontSlant->"Italic"], "\[Prime]"]]}], "=", 
    RowBox[{
     FractionBox[
      RowBox[{
       SubscriptBox["n", "2"], " ", "-", " ", 
       SubscriptBox["n", "1"]}], "R"], "=", "0"}]}], TraditionalForm]]],
 "  \[DoubleLongRightArrow]  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     SuperscriptBox["s", "\[Prime]"], "s"], "=", 
    RowBox[{"-", 
     FractionBox[
      SubscriptBox["n", "2"], 
      SubscriptBox["n", "1"]]}]}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.3876368804196444`*^9, 3.387636893574819*^9}, {
   3.3876369489275227`*^9, 3.38763694910182*^9}, {3.3876396435744553`*^9, 
   3.387639669931848*^9}, 3.387639830891551*^9},
 TextAlignment->Center,
 TextJustification->0,
 FontSize->12],

Cell["The magnification, then, becomes one.", "Text",
 CellChangeTimes->{{3.387639763494512*^9, 3.387639774770605*^9}}],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    FormBox[
     RowBox[{"m", "=", 
      RowBox[{
       FractionBox[
        SuperscriptBox["h", "\[Prime]"], "h"], "=", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           SubscriptBox["n", "1"], 
           SubscriptBox["n", "2"]]}], 
         FractionBox[
          SuperscriptBox["s", "\[Prime]"], "s"]}], "=", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           SubscriptBox["n", "1"], 
           SubscriptBox["n", "2"]]}], 
         RowBox[{"(", 
          RowBox[{"-", 
           FractionBox[
            SubscriptBox["n", "2"], 
            SubscriptBox["n", "1"]]}], ")"}]}]}]}]}],
     TraditionalForm]}], TraditionalForm]]],
 "  ",
 Cell[BoxData[
  FormBox["\[DoubleLongRightArrow]", TraditionalForm]]],
 "  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "=", 
    RowBox[{
     FractionBox[
      SuperscriptBox["h", "\[Prime]"], "h"], "=", "1"}]}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.387636439642475*^9, 3.387636459377553*^9}, {
  3.387636511105527*^9, 3.387636537879436*^9}, {3.387636570948657*^9, 
  3.387636633052931*^9}, {3.3876398002967653`*^9, 3.387639904272019*^9}},
 TextAlignment->Center,
 TextJustification->0,
 FontSize->12],

Cell[TextData[{
 Cell[BoxData[
  FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .33715 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.70861 0.0107032 0.11506 0.0107032 [
[.38751 .07225 -11.4375 -7 ]
[.38751 .07225 11.4375 7 ]
[.10944 .07225 -14.1562 -7.5 ]
[.10944 .07225 14.1562 7.5 ]
[.60444 .13965 -7.90625 -7.0625 ]
[.60444 .13965 7.90625 7.0625 ]
[.86778 .0941 -7.90625 -7.0625 ]
[.86778 .0941 7.90625 7.0625 ]
[.38751 .29701 -16.625 -7 ]
[.38751 .29701 16.625 7 ]
[.10944 .29701 -33.7188 -7 ]
[.10944 .29701 33.7188 7 ]
[.75142 .04014 -8.28125 -7.0625 ]
[.75142 .04014 8.28125 7.0625 ]
[.6658 .04014 -8.28125 -7.0625 ]
[.6658 .04014 8.28125 7.0625 ]
[.36611 .18463 -5 -7 ]
[.36611 .18463 5 7 ]
[.13619 .20069 -7.4375 -7.5 ]
[.13619 .20069 7.4375 7.5 ]
[ 0 0 0 0 ]
[ 1 .33715 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
.9 g
.70861 .00803 m
.70861 .32912 L
.97619 .32912 L
.97619 .00803 L
F
0 g
.5 Mabswid
[ ] 0 setdash
.02381 .11506 m
.97619 .11506 L
s
.70861 .32912 m
.70861 .00803 L
s
0 .5 0 r
5 Mabswid
.38751 .11506 Mdot
gsave
.38751 .07225 -72.4375 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(s) show
67.625 14.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(>) show
75.312 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.562 0.000 rmoveto
75.875 14.000 moveto
(0) show
81.875 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
.10944 .11506 Mdot
gsave
.10944 .07225 -75.1562 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 15.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(s) show
67.625 10.188 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
72.500 15.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(<) show
80.188 15.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.562 0.000 rmoveto
0.562 0.000 rmoveto
81.312 15.000 moveto
(0) show
87.312 15.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 .55 r
.5 Mabswid
.38751 .27561 m
.97619 .27561 L
s
.54271 .29701 m
.56412 .27561 L
.54271 .2542 L
s
.92535 .29701 m
.94676 .27561 L
.92535 .2542 L
s
.38751 .27561 m
.70861 .11506 L
s
.53849 .22405 m
.54806 .19533 L
.51934 .18576 L
s
.70861 .11506 m
.97619 .04336 L
s
.9343 .07675 m
.94943 .05053 L
.92321 .03539 L
s
[ .015 .015 ] 0 setdash
.10944 .27561 m
.70861 .11506 L
s
.10944 .27561 m
.70861 .27561 L
s
0 g
[ ] 0 setdash
newpath
.70861 .11506 .08563 153.435 180 arc
s
gsave
.60444 .13965 -68.9062 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(q) show
68.250 17.125 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
74.812 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
newpath
.70861 .11506 .11774 -15 0 arc
s
newpath
.70861 .11506 .12844 -15 0 arc
s
gsave
.86778 .0941 -68.9062 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(q) show
68.250 17.125 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
74.812 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
.005 w
.38751 .11506 m
.38751 .27561 L
s
.36611 .2542 m
.38751 .27561 L
.40892 .2542 L
s
gsave
.38751 .29701 -77.625 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(object) show
92.250 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
.10944 .11506 m
.10944 .27561 L
s
.08803 .2542 m
.10944 .27561 L
.13084 .2542 L
s
gsave
.10944 .29701 -94.7188 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(virtual) show
97.188 14.000 moveto
(image) show
126.438 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 g
gsave
.75142 .04014 -69.2812 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(n) show
69.000 17.125 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
75.562 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.6658 .04014 -69.2812 -11.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.125 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(n) show
69.000 17.125 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
75.562 14.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
gsave
.36611 .18463 -66 -11 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(h) show
69.000 14.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
gsave
.13619 .20069 -68.4375 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 15.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(h) show
69.000 10.188 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 10.812 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
73.875 15.000 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .33715 L
0 .33715 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
    GeneratedCell->False,
    CellAutoOverwrite->False,
    ImageSize->{437.5, 147.375},
    ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
    ImageRangeCache->{{{0, 874}, {293.75, 0}} -> {-66.3521, -10.7502, 0.21447,
     0.21447}}], TraditionalForm]]],
 " "
}], "Text",
 CellChangeTimes->{{3.401219918734283*^9, 3.401219922254046*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "Notice from the diagram above that it is clear that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "=", "1"}], TraditionalForm]]],
 ".  Also, the diagram shows this can only be a virtual image."
}], "Text",
 CellChangeTimes->{{3.387795581055193*^9, 3.387795646417973*^9}, {
  3.3877991009493933`*^9, 3.387799119770859*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Thin Lens Approximation", "Subsection"],

Cell[TextData[{
 "\tA lens is formed from two refracting surfaces.  The light passes from air \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"n", "=", "1"}], ")"}], TraditionalForm]]],
 " to a medium with index ",
 StyleBox["n",
  FontSlant->"Italic"],
 " and then back to air.  Take the two refracting surfaces to have radii ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["R", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["R", "2"], TraditionalForm]]],
 ".  Using the standard sign convention, take both radii to be positive; this \
means that their centers are on the side of the light leaving the lens."
}], "Text",
 CellChangeTimes->{{3.387802253404058*^9, 3.387802260259803*^9}}],

Cell["\tAt the first interface we have", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox[
     SubscriptBox["", "1"], 
     SubscriptBox["s", "1"]], "+", 
    FractionBox["n", 
     SubsuperscriptBox[
      StyleBox["s",
       FontSlant->"Italic"], "1", "\[Prime]"]]}], "=", 
   FractionBox[
    RowBox[{"n", " ", "-", " ", "1"}], 
    SubscriptBox["R", "1"]]}], TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0,
 FontSize->12],

Cell["and at the second", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["n", 
      SubscriptBox["s", "2"]], "+", 
     FractionBox["1", 
      SubsuperscriptBox[
       StyleBox["s",
        FontSlant->"Italic"], "2", "\[Prime]"]]}], "=", 
    FractionBox[
     RowBox[{"1", " ", "-", " ", "n"}], 
     SubscriptBox["R", "2"]]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0,
 FontSize->12],

Cell["\<\
Since there are two surfaces the image from the first refraction becomes the \
object for the second refraction.  Take \
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{"s", "=", 
    SubscriptBox["s", "1"]}], TraditionalForm]],
  FontSize->12],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     StyleBox["s",
      FontSlant->"Italic"], "\[Prime]"], "=", 
    SubsuperscriptBox[
     StyleBox["s",
      FontSlant->"Italic"], "2", "\[Prime]"]}], TraditionalForm]],
  FontSize->12]
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "to be the object and image positions for the overall refraction of the \
lens.  In the thin lens approximation we assume that the distance between the \
two refracting surfaces is small.  The image from the first would form a \
distance of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["s",
     FontSlant->"Italic"], "1", "\[Prime]"], TraditionalForm]],
  FontFamily->"Times",
  FontSize->12,
  FontWeight->"Plain",
  FontSlant->"Plain"],
 " from it but in the thin lens case, it will not have formed before reaching \
the second surface.  This gives what is called a ",
 StyleBox["virtual object",
  FontSlant->"Italic"],
 " for the second surface; the light hitting the second surface is not moving \
away from a point but toward a point beyond it.  For a virtual object the \
object distance ",
 StyleBox["s",
  FontSlant->"Italic"],
 " is negative.  Here, since the lens thickness is ignored, it is a distance \
of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["s",
     FontSlant->"Italic"], "1", "\[Prime]"], TraditionalForm]]],
 " beyond it.  Thus  "
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["s", "2"], "=", 
    RowBox[{"-", 
     SubsuperscriptBox[
      StyleBox["s",
       FontSlant->"Italic"], "1", "\[Prime]"]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0,
 FontSize->12],

Cell["\<\
\tCombining all these expressions gives an expression relating the object and \
image positions to the index of the lens' material and the radii of curvature \
of its surfaces.\
\>", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "s"], "+", 
    FractionBox["1", 
     SuperscriptBox[
      StyleBox["s",
       FontSlant->"Italic"], "\[Prime]"]]}], "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"n", "-", "1"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["1", 
       SubscriptBox["R", "1"]], "-", 
      FractionBox["1", 
       SubscriptBox["R", "2"]]}], ")"}]}]}], TraditionalForm]],
 FontSize->12]], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "The right-hand side can be identified with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", "f"}], TraditionalForm]]],
 ", where ",
 StyleBox["f",
  FontSlant->"Italic"],
 "  is defined as the focal length of the lens.  This gives what is called \
the ",
 StyleBox["lensmaker formula",
  FontSlant->"Italic"],
 ":"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox["1", "f"], "=", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"n", "-", "1"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", 
        SubscriptBox["R", "1"]], "-", 
       FractionBox["1", 
        SubscriptBox["R", "2"]]}], ")"}]}]}], TraditionalForm]],
  FontSize->12],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
With this identification the same expressions that applied to both the \
concave and convex spherical mirrors also apply to the case of a thin lens\
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["1", "s"], "+", 
     FractionBox["1", 
      SuperscriptBox[
       StyleBox["s",
        FontSlant->"Italic"], "\[Prime]"]]}], "=", 
    FractionBox["1", "f"]}], TraditionalForm]],
  FontSize->12],
 "   and   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "=", 
    RowBox[{
     FractionBox[
      SuperscriptBox["h", "\[Prime]"], "h"], "=", 
     RowBox[{"-", 
      FractionBox[
       SuperscriptBox[
        StyleBox["s",
         FontSlant->"Italic"], "\[Prime]"], "s"]}]}]}], TraditionalForm]],
  FontSize->12],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]],

Cell[CellGroupData[{

Cell["Converging and Diverging Lenses", "Subsection"],

Cell[TextData[{
 "\tA thin lens with a positive focal length is called a converging lens.  \
This is analogous to a concave mirror mathematically.  If the object distance \
is larger than the focal length ",
 Cell[BoxData[
  FormBox[
   RowBox[{"s", ">", "f"}], TraditionalForm]]],
 "  then there is a real image and when the object distance is less than the \
focal length ",
 Cell[BoxData[
  FormBox[
   RowBox[{"s", "<", "f"}], TraditionalForm]]],
 " there is a virtual image.  To trace the rays: Draw one ray that passes \
straight through where the central axis hits the lens.  Draw the other ray \
parallel to the central axis; this will bend through the focal point.  The \
two rays (as all rays do) will converge to the image.  Below are examples of \
ray tracing in both cases."
}], "Text",
 CellChangeTimes->{{3.387802452461236*^9, 3.387802456872891*^9}}],

Cell[TextData[{
 Cell[BoxData[
  FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .39092 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.522438 0.00664838 0.182166 0.00664838 [
[.52244 .38162 -14.2812 -7.21875 ]
[.52244 .38162 14.2812 7.21875 ]
[.05705 .35502 -19.0625 -7.3125 ]
[.05705 .35502 19.0625 7.3125 ]
[.87148 .04255 -26.0625 -7.3125 ]
[.87148 .04255 26.0625 7.3125 ]
[.72189 .22206 -5.9375 -7.21875 ]
[.72189 .22206 5.9375 7.21875 ]
[.32299 .14228 -5.9375 -7.21875 ]
[.32299 .14228 5.9375 7.21875 ]
[.09694 .24865 -5.5 -6.8125 ]
[.09694 .24865 5.5 6.8125 ]
[.05705 .14228 -4.71875 -6 ]
[.05705 .14228 4.71875 6 ]
[.92467 .13563 -11.8125 -7.96875 ]
[.92467 .13563 11.8125 7.96875 ]
[.87148 .22206 -7.5 -7.84375 ]
[.87148 .22206 7.5 7.84375 ]
[ 0 0 0 0 ]
[ 1 .39092 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.02381 .18217 m
.93796 .18217 L
s
0 0 .55 r
.05705 .31513 m
.52244 .18217 L
s
.28061 .26509 m
.28974 .24865 L
.27331 .23952 L
s
.52244 .18217 m
.97619 .05252 L
s
.96706 .06896 m
.97619 .05252 L
.95975 .04339 L
s
.05705 .31513 m
.52244 .31513 L
s
.27645 .32843 m
.28974 .31513 L
.27645 .30184 L
s
.52244 .31513 m
.96123 .0226 L
s
.60651 .27507 m
.6102 .25663 L
.59176 .25294 L
s
.95754 .04104 m
.96123 .0226 L
.94279 .01892 L
s
0 g
3 Mabswid
.52244 .00931 m
.52244 .35502 L
s
.50914 .34173 m
.52244 .35502 L
.53574 .34173 L
s
.50914 .0226 m
.52244 .00931 L
.53574 .0226 L
s
gsave
.52244 .38162 -75.2812 -11.2188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.4375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(>) show
79.875 14.562 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.688 0.000 rmoveto
80.562 14.562 moveto
(0) show
87.562 14.562 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
.05705 .18217 m
.05705 .31513 L
s
.04375 .30184 m
.05705 .31513 L
.07035 .30184 L
s
gsave
.05705 .35502 -80.0625 -11.3125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(object) show
97.125 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
.87148 .18217 m
.87148 .08244 L
s
.85818 .09574 m
.87148 .08244 L
.88478 .09574 L
s
gsave
.87148 .04255 -87.0625 -11.3125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(image) show
111.125 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 g
gsave
.72189 .22206 -66.9375 -11.2188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.4375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.32299 .14228 -66.9375 -11.2188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.4375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
gsave
.09694 .24865 -66.5 -10.8125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 21.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.625 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(h) show
70.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.05705 .14228 -65.7188 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(s) show
68.438 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
gsave
.92467 .13563 -72.8125 -11.9688 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.9375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
63.000 16.688 moveto
(\\310) show
66.188 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.688 0.000 rmoveto
66.875 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(h) show
73.875 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
79.438 16.688 moveto
(\\310) show
82.625 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.87148 .22206 -68.5 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(s) show
68.438 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
74.000 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 g
.015 w
.32299 .18217 Mdot
.72189 .18217 Mdot
0 0 m
1 0 L
1 .39092 L
0 .39092 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
    GeneratedCell->False,
    CellAutoOverwrite->False,
    ImageSize->{273.5, 106.75},
    ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
    ImageRangeCache->{{{0, 546}, {212.5, 0}} -> {-83.7279, -27.4003, 0.579625,
     0.579625}}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .83636 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.58658 0.00865801 0.245022 0.00865801 [
[.58658 .50476 -14.2812 -7.21875 ]
[.58658 .50476 14.2812 7.21875 ]
[.41342 .41818 -45.3125 -7.3125 ]
[.41342 .41818 45.3125 7.3125 ]
[.0671 .81645 -22.5625 -7.3125 ]
[.0671 .81645 22.5625 7.3125 ]
[.84632 .29697 -5.9375 -7.21875 ]
[.84632 .29697 5.9375 7.21875 ]
[.32684 .19307 -5.9375 -7.21875 ]
[.32684 .19307 5.9375 7.21875 ]
[.36147 .3316 -5.5 -6.8125 ]
[.36147 .3316 5.5 6.8125 ]
[.41342 .19307 -4.71875 -6 ]
[.41342 .19307 4.71875 6 ]
[.13636 .50909 -8.28125 -7.84375 ]
[.13636 .50909 8.28125 7.84375 ]
[.0671 .19307 -7.5 -7.84375 ]
[.0671 .19307 7.5 7.84375 ]
[ 0 0 0 0 ]
[ 1 .83636 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.02381 .24502 m
.9329 .24502 L
s
0 0 .55 r
[ .02 .02 ] 0 setdash
.41342 .41818 m
.0671 .7645 L
s
.58658 .41818 m
.0671 .7645 L
s
[ ] 0 setdash
.41342 .41818 m
.58658 .24502 L
s
.5 .35609 m
.5 .3316 L
.47551 .3316 L
s
.58658 .24502 m
.81169 .01991 L
s
.81169 .0444 m
.81169 .01991 L
.7872 .01991 L
s
.41342 .41818 m
.58658 .41818 L
s
.48268 .4355 m
.5 .41818 L
.48268 .40087 L
s
.58658 .41818 m
.97619 .15844 L
s
.6597 .39025 m
.6645 .36623 L
.64049 .36143 L
s
.97139 .18245 m
.97619 .15844 L
.95218 .15364 L
s
0 g
3 Mabswid
.58658 .01991 m
.58658 .47013 L
s
.56926 .45281 m
.58658 .47013 L
.6039 .45281 L
s
.56926 .03723 m
.58658 .01991 L
.6039 .03723 L
s
gsave
.58658 .50476 -75.2812 -11.2188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.4375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(>) show
79.875 14.562 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.688 0.000 rmoveto
80.562 14.562 moveto
(0) show
87.562 14.562 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
.41342 .24502 m
.41342 .41818 L
s
.3961 .40087 m
.41342 .41818 L
.43074 .40087 L
s
gsave
.41342 .41818 -106.312 -11.3125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(object) show
149.625 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
.0671 .24502 m
.0671 .7645 L
s
.04978 .74719 m
.0671 .7645 L
.08442 .74719 L
s
gsave
.0671 .81645 -83.5625 -11.3125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(image) show
104.125 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 g
gsave
.84632 .29697 -66.9375 -11.2188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.4375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.32684 .19307 -66.9375 -11.2188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.4375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
gsave
.36147 .3316 -66.5 -10.8125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 21.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.625 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(h) show
70.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.41342 .19307 -65.7188 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(s) show
68.438 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
gsave
.13636 .50909 -69.2812 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(h) show
70.000 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
75.562 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.0671 .19307 -68.5 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(s) show
68.438 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
74.000 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 g
.015 w
.32684 .24502 Mdot
.84632 .24502 Mdot
0 0 m
1 0 L
1 .83636 L
0 .83636 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
    GeneratedCell->False,
    CellAutoOverwrite->False,
    ImageSize->{192.625, 161.},
    ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
    ImageRangeCache->{{{0, 384.25}, {321, 0}} -> {-74.3381, -29.8227, 
    0.635465, 0.635465}}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.401219926159358*^9, 3.4012199344857063`*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
\tA diverging lens has a negative focal length.  The ray tracing is similar \
but the ray that begins parallel to the central axis will diverge away from \
the focal point, which is on the same side as the object.  Here is the ray \
tracing for a diverging lens.\
\>", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .98246 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.775689 0.0167084 0.457811 0.0167084 [
[.77569 .95906 -14.2812 -7.21875 ]
[.77569 .95906 14.2812 7.21875 ]
[.10735 .85881 -19.0625 -7.3125 ]
[.10735 .85881 19.0625 7.3125 ]
[.48926 .70128 -19.0625 -7.3125 ]
[.48926 .70128 19.0625 7.3125 ]
[.27444 .39098 -5.9375 -7.21875 ]
[.27444 .39098 5.9375 7.21875 ]
[.05723 .6249 -5.5 -6.8125 ]
[.05723 .6249 5.5 6.8125 ]
[.10735 .39098 -4.71875 -6 ]
[.10735 .39098 4.71875 6 ]
[.53938 .51772 -8.28125 -7.84375 ]
[.53938 .51772 8.28125 7.84375 ]
[.48926 .39098 -7.5 -7.84375 ]
[.48926 .39098 7.5 7.84375 ]
[ 0 0 0 0 ]
[ 1 .98246 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.02381 .45781 m
.94277 .45781 L
s
0 0 .55 r
[ .02 .02 ] 0 setdash
.77569 .79198 m
.27444 .45781 L
s
[ ] 0 setdash
.10735 .79198 m
.77569 .45781 L
s
.42658 .66973 m
.44152 .6249 L
.39669 .60995 L
s
.77569 .45781 m
.97619 .35756 L
s
.96125 .40239 m
.97619 .35756 L
.93136 .34262 L
s
.10735 .79198 m
.77569 .79198 L
s
.4081 .8254 m
.44152 .79198 L
.4081 .75856 L
s
.77569 .79198 m
.92607 .89223 L
s
.87972 .9015 m
.92607 .89223 L
.9168 .84589 L
s
0 g
3 Mabswid
.77569 .05681 m
.77569 .85881 L
s
.74227 .89223 m
.77569 .85881 L
.80911 .89223 L
s
.74227 .02339 m
.77569 .05681 L
.80911 .02339 L
s
gsave
.77569 .95906 -75.2812 -11.2188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.4375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(<) show
79.875 14.562 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.688 0.000 rmoveto
80.562 14.562 moveto
(0) show
87.562 14.562 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
.10735 .45781 m
.10735 .79198 L
s
.07393 .75856 m
.10735 .79198 L
.14077 .75856 L
s
gsave
.10735 .85881 -80.0625 -11.3125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(object) show
97.125 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
.48926 .45781 m
.48926 .60103 L
s
.45584 .56761 m
.48926 .60103 L
.52268 .56761 L
s
gsave
.48926 .70128 -80.0625 -11.3125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(image) show
97.125 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 g
gsave
.27444 .39098 -66.9375 -11.2188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22.4375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
64.938 14.562 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
70.875 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .5 0 r
gsave
.05723 .6249 -66.5 -10.8125 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 21.625 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.625 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(h) show
70.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.10735 .39098 -65.7188 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
/Times-Italic findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
(s) show
68.438 13.000 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.500 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.55 0 0 r
gsave
.53938 .51772 -69.2812 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(h) show
70.000 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
75.562 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.48926 .39098 -68.5 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding MacintoshRomanEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 16.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 16.688 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(s) show
68.438 11.062 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.625 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
(\\242) show
74.000 16.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.550 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 g
.02 w
.27444 .45781 Mdot
0 0 m
1 0 L
1 .98246 L
0 .98246 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
   GeneratedCell->False,
   CellAutoOverwrite->False,
   ImageSize->{194.75, 191.25},
   ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"],
   ImageRangeCache->{{{0, 388.5}, {381.5, 0}} -> {-46.8764, -27.4003, 
   0.312756, 0.312756}}], TraditionalForm]]]], "Text",
 CellChangeTimes->{{3.401219938434349*^9, 3.401219941472927*^9}},
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
NotebookAutoSave->True,
WindowToolbars->"EditBar",
WindowSize->{1132, 942},
WindowMargins->{{Automatic, 21}, {Automatic, 0}},
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
PageHeaders->{{
   Cell[
    TextData[{
      CounterBox["Page"]}], "PageNumber"], None, 
   Cell[
    TextData[{"Chapter K - Geometric Optics"}], "Header"]}, {
   Cell[
    TextData[{"Chapter K - Geometric Optics"}], "Header"], None, 
   Cell[
    TextData[{
      CounterBox["Page"]}], "PageNumber"]}},
PrintingOptions->{"IncludeSpecialFonts"->True,
"PrintingMargins"->{{36, 28}, {30, 42}}},
PrivateNotebookOptions->{"BackgroundPostScriptRendering"->True},
CellLabelAutoDelete->False,
CellBaseline->Baseline,
DefaultNewCellStyle->"Text",
DefaultDuplicateCellStyle->"Text",
ScriptSizeMultipliers->0.9,
FontSize->12,
Magnification->2.,
FrontEndVersion->"6.0 for Mac OS X x86 (32-bit) (June 19, 2007)",
StyleDefinitions->"Textbook Blue.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[590, 23, 73, 2, 128, "Title"],
Cell[666, 27, 83, 2, 60, "Subtitle"],
Cell[CellGroupData[{
Cell[774, 33, 38, 0, 160, "Section"],
Cell[CellGroupData[{
Cell[837, 37, 40, 0, 70, "Subsection"],
Cell[880, 39, 493, 16, 73, "Text"],
Cell[1376, 57, 346, 14, 47, "Text"],
Cell[1725, 73, 255, 6, 66, "Text"],
Cell[1983, 81, 549, 10, 144, "Text"],
Cell[2535, 93, 185, 6, 66, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2757, 104, 42, 0, 70, "Subsection"],
Cell[2802, 106, 282, 5, 92, "Text"],
Cell[CellGroupData[{
Cell[3109, 115, 36, 0, 64, "Subsubsection"],
Cell[3148, 117, 20192, 838, 465, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[23377, 960, 40, 0, 64, "Subsubsection"],
Cell[23420, 962, 20244, 490, 507, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[43713, 1458, 39, 0, 70, "Subsection"],
Cell[43755, 1460, 352, 10, 92, "Text"],
Cell[44110, 1472, 176, 8, 55, "Text"],
Cell[44289, 1482, 250, 9, 40, "Text"],
Cell[44542, 1493, 304, 10, 68, "Text"],
Cell[44849, 1505, 235, 10, 55, "Text"],
Cell[45087, 1517, 310, 5, 92, "Text"],
Cell[45400, 1524, 212, 7, 42, "Text"],
Cell[45615, 1533, 113, 3, 40, "Text"],
Cell[45731, 1538, 774, 30, 70, "Text"],
Cell[46508, 1570, 328, 11, 69, "Text"],
Cell[46839, 1583, 217, 9, 65, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[47105, 1598, 44, 0, 160, "Section"],
Cell[47152, 1600, 165, 3, 66, "Text"],
Cell[CellGroupData[{
Cell[47342, 1607, 33, 0, 70, "Subsection"],
Cell[47378, 1609, 426, 12, 94, "Text"],
Cell[47807, 1623, 35134, 1070, 469, "Text"],
Cell[82944, 2695, 305, 5, 92, "Text"],
Cell[83252, 2702, 19632, 614, 332, "Text"],
Cell[102887, 3318, 114, 5, 40, "Text"],
Cell[103004, 3325, 993, 38, 70, "Text"],
Cell[104000, 3365, 86, 2, 40, "Text"],
Cell[104089, 3369, 341, 13, 42, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[104467, 3387, 60, 0, 70, "Subsection"],
Cell[104530, 3389, 219, 4, 66, "Text"],
Cell[104752, 3395, 35115, 1039, 333, "Text"],
Cell[139870, 4436, 174, 3, 66, "Text"],
Cell[140047, 4441, 319, 9, 42, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[140403, 4455, 47, 0, 70, "Subsection"],
Cell[140453, 4457, 115, 3, 40, "Text"],
Cell[140571, 4462, 207, 7, 42, "Text"],
Cell[140781, 4471, 449, 10, 94, "Text"],
Cell[141233, 4483, 232, 9, 43, "Text"],
Cell[141468, 4494, 415, 15, 43, "Text"],
Cell[141886, 4511, 288, 12, 65, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[142211, 4528, 32, 0, 70, "Subsection"],
Cell[142246, 4530, 563, 8, 144, "Text"],
Cell[142812, 4540, 441, 7, 118, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[143302, 4553, 62, 0, 160, "Section"],
Cell[CellGroupData[{
Cell[143389, 4557, 37, 0, 70, "Subsection"],
Cell[CellGroupData[{
Cell[143451, 4561, 67, 0, 64, "Subsubsection"],
Cell[143521, 4563, 303, 5, 92, "Text"],
Cell[143827, 4570, 13792, 427, 471, "Text"],
Cell[157622, 4999, 120, 3, 40, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[157779, 5007, 42, 0, 64, "Subsubsection"],
Cell[157824, 5009, 159, 3, 66, "Text"],
Cell[157986, 5014, 13508, 431, 475, "Text"],
Cell[171497, 5447, 227, 4, 66, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[171761, 5456, 62, 0, 64, "Subsubsection"],
Cell[171826, 5458, 10085, 322, 277, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[181948, 5785, 57, 0, 64, "Subsubsection"],
Cell[182008, 5787, 14799, 475, 294, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[196844, 6267, 58, 0, 64, "Subsubsection"],
Cell[196905, 6269, 15580, 492, 292, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[212534, 6767, 49, 0, 70, "Subsection"],
Cell[212586, 6769, 1103, 28, 173, "Text"],
Cell[213692, 6799, 35309, 996, 505, "Text"],
Cell[249004, 7797, 69, 0, 40, "Text"],
Cell[249076, 7799, 32764, 1069, 318, "Text"],
Cell[281843, 8870, 42, 0, 40, "Text"],
Cell[281888, 8872, 859, 31, 66, "Text"],
Cell[282750, 8905, 805, 32, 62, "Text"],
Cell[283558, 8939, 1019, 36, 98, "Text"],
Cell[284580, 8977, 666, 28, 66, "Text"],
Cell[285249, 9007, 407, 11, 95, "Text"],
Cell[285659, 9020, 806, 21, 147, "Text"],
Cell[286468, 9043, 38539, 1114, 479, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[325044, 10162, 48, 0, 70, "Subsection"],
Cell[325095, 10164, 358, 6, 118, "Text"],
Cell[325456, 10172, 14044, 474, 320, "Text"],
Cell[339503, 10648, 166, 3, 66, "Text"],
Cell[339672, 10653, 17355, 566, 394, "Text"],
Cell[357030, 11221, 123, 3, 40, "Text"],
Cell[357156, 11226, 37900, 1102, 432, "Text"],
Cell[395059, 12330, 96, 2, 40, "Text"],
Cell[395158, 12334, 666, 28, 66, "Text"],
Cell[395827, 12364, 582, 24, 69, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[396446, 12393, 100, 1, 70, "Subsection"],
Cell[396549, 12396, 546, 17, 66, "Text"],
Cell[397098, 12415, 595, 22, 64, "Text"],
Cell[397696, 12439, 718, 26, 62, "Text"],
Cell[398417, 12467, 222, 4, 66, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[398688, 12477, 41, 0, 160, "Section"],
Cell[CellGroupData[{
Cell[398754, 12481, 56, 0, 70, "Subsection"],
Cell[398813, 12483, 586, 19, 95, "Text"],
Cell[399402, 12504, 496, 10, 118, "Text"],
Cell[399901, 12516, 553, 17, 95, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[400491, 12538, 41, 0, 70, "Subsection"],
Cell[400535, 12540, 37134, 1125, 357, "Text"],
Cell[437672, 13667, 808, 16, 92, "Text"],
Cell[438483, 13685, 926, 31, 64, "Text"],
Cell[439412, 13718, 200, 4, 40, "Text"],
Cell[439615, 13724, 1442, 48, 70, "Text"],
Cell[441060, 13774, 275, 7, 66, "Text"],
Cell[441338, 13783, 639, 16, 64, "Text"],
Cell[441980, 13801, 608, 14, 43, "Text"],
Cell[442591, 13817, 1896, 58, 68, "Text"],
Cell[444490, 13877, 256, 8, 43, "Text"],
Cell[444749, 13887, 486, 20, 59, "Text"],
Cell[445238, 13909, 432, 11, 69, "Text"],
Cell[445673, 13922, 648, 21, 68, "Text"],
Cell[446324, 13945, 1171, 45, 128, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[447532, 13995, 103, 1, 70, "Subsection"],
Cell[447638, 13998, 244, 7, 40, "Text"],
Cell[447885, 14007, 961, 33, 68, "Text"],
Cell[448849, 14042, 119, 1, 40, "Text"],
Cell[448971, 14045, 1290, 44, 72, "Text"],
Cell[450264, 14091, 37147, 1128, 318, "Text"],
Cell[487414, 15221, 339, 8, 66, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[487790, 15234, 45, 0, 70, "Subsection"],
Cell[487838, 15236, 728, 21, 120, "Text"],
Cell[488569, 15259, 48, 0, 40, "Text"],
Cell[488620, 15261, 442, 16, 68, "Text"],
Cell[489065, 15279, 33, 0, 40, "Text"],
Cell[489101, 15281, 441, 18, 68, "Text"],
Cell[489545, 15301, 143, 3, 66, "Text"],
Cell[489691, 15306, 447, 19, 47, "Text"],
Cell[490141, 15327, 1124, 31, 184, "Text"],
Cell[491268, 15360, 299, 13, 47, "Text"],
Cell[491570, 15375, 200, 4, 66, "Text"],
Cell[491773, 15381, 534, 20, 72, "Text"],
Cell[492310, 15403, 345, 13, 66, "Text"],
Cell[492658, 15418, 431, 18, 72, "Text"],
Cell[493092, 15438, 171, 3, 66, "Text"],
Cell[493266, 15443, 666, 28, 66, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[493969, 15476, 53, 0, 70, "Subsection"],
Cell[494025, 15478, 865, 18, 170, "Text"],
Cell[494893, 15498, 58086, 1769, 345, "Text"],
Cell[552982, 17269, 286, 5, 92, "Text"],
Cell[553271, 17276, 27251, 818, 406, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
