(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    287418,       5889]
NotebookOptionsPosition[    282743,       5737]
NotebookOutlinePosition[    283171,       5756]
CellTagsIndexPosition[    283128,       5753]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
Numerical and Symbolic Methods 
in One Dimensional Dynamics\
\>", "Subtitle",
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
\tNewton's second law is the basis for dynamics.  it is the rule for evolving \
a dynamical system forward in time.  Here we will discuss the problem of one \
dimensional motion.  The second law in one dimension states:\
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "(", 
     RowBox[{"x", ",", "v", ",", "t"}], ")"}], "=", 
    RowBox[{"m", " ", "a"}]}], TraditionalForm]]],
 ","
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "where ",
 StyleBox["F",
  FontSlant->"Italic"],
 " is the net force which is a function of ",
 StyleBox["x",
  FontSlant->"Italic"],
 ", the position, ",
 StyleBox["v",
  FontSlant->"Italic"],
 ", the velocity, and perhaps ",
 StyleBox["t",
  FontSlant->"Italic"],
 ", the time.  If at some instant ",
 StyleBox["t",
  FontSlant->"Italic"],
 " the position and velocity are given by ",
 StyleBox["x",
  FontSlant->"Italic"],
 " and ",
 StyleBox["v",
  FontSlant->"Italic"],
 ", then an infinitesimal time step ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DifferentialD]", "t"}], TraditionalForm]]],
 " later the values change by the rule:"
}], "Text"],

Cell[TextData[Cell[BoxData[{
 FormBox[
  RowBox[{"t", "\[Rule]", 
   RowBox[{"t", "+", 
    RowBox[{"\[DifferentialD]", "t"}]}]}], 
  TraditionalForm], "\[IndentingNewLine]", 
 FormBox[
  RowBox[{"x", "\[Rule]", 
   RowBox[{"x", "+", 
    RowBox[{"v", " ", 
     RowBox[{"\[DifferentialD]", "t"}]}]}]}], 
  TraditionalForm], "\[IndentingNewLine]", 
 FormBox[
  RowBox[{"v", "\[Rule]", 
   RowBox[{"v", "+", 
    RowBox[{
     FractionBox[
      RowBox[{"F", "(", 
       RowBox[{"x", ",", "v", ",", "t"}], ")"}], "m"], " ", 
     RowBox[{
      RowBox[{"\[DifferentialD]", "t"}], " ", "."}]}]}]}], 
  TraditionalForm]}]]], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "\tIn a numerical simulation we approximate the infinitesimal time step with \
a small time step ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]", "\[ThinSpace]", "t"}], TraditionalForm]]],
 ".  We will label an instant by ",
 StyleBox["i",
  FontSlant->"Italic"],
 " and the rules for taking a time step are"
}], "Text"],

Cell[TextData[Cell[BoxData[{
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["t", "i"], "\[Rule]", 
    SubscriptBox["t", 
     RowBox[{"i", "+", "1"}]]}], "=", 
   RowBox[{
    SubscriptBox["t", "i"], "+", 
    RowBox[{"\[CapitalDelta]", "\[ThinSpace]", "t"}]}]}], 
  TraditionalForm], "\[IndentingNewLine]", 
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["x", "i"], "\[Rule]", 
    SubscriptBox["x", 
     RowBox[{"i", "+", "1"}]]}], "=", 
   RowBox[{
    SubscriptBox["x", "i"], "+", 
    RowBox[{
     SubscriptBox["v", "i"], "\[CapitalDelta]", "\[ThinSpace]", "t"}]}]}], 
  TraditionalForm], "\[IndentingNewLine]", 
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["v", "i"], "\[Rule]", 
    SubscriptBox["v", 
     RowBox[{"i", "+", "1"}]]}], "=", 
   RowBox[{
    SubscriptBox["v", "i"], "+", 
    RowBox[{
     FractionBox[
      RowBox[{"F", "(", 
       RowBox[{
        SubscriptBox["x", "i"], ",", 
        SubscriptBox["v", "i"], ",", 
        SubscriptBox["t", "i"]}], ")"}], "m"], " ", "\[CapitalDelta]", 
     "\[ThinSpace]", 
     RowBox[{"t", " ", "."}]}]}]}], TraditionalForm]}]]], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "We may numerically evolve a problem by starting with the initial \
conditions: ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "0"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "0"], TraditionalForm]]],
 "and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", "0"], TraditionalForm]]],
 ", and by giving the mass ",
 StyleBox["m",
  FontSlant->"Italic"],
 " and the force function ",
 StyleBox["F",
  FontSlant->"Italic"],
 "."
}], "Text"],

Cell[TextData[{
 "\tThe second law is the rule that gives the differential equation for the \
motion.  A differential equation is an equation for a ",
 StyleBox["function",
  FontSlant->"Italic"],
 " that involves the function and its derivatives.  After the third semester \
of calculus students take a course in differential equations to learn how to \
solve them.  Here we will just use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to solve them."
}], "Text"],

Cell[TextData[{
 "\tThe second law gives a second order differential equation for the \
position as a function of time ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "(", "t", ")"}], TraditionalForm]]],
 ":"
}], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox[
     SuperscriptBox["\[DifferentialD]", "2"], 
     RowBox[{"\[DifferentialD]", 
      SuperscriptBox["t", "2"]}]], 
    RowBox[{"x", "(", "t", ")"}]}], "=", 
   RowBox[{
    FractionBox["1", "m"], " ", 
    RowBox[{"F", "(", 
     RowBox[{
      RowBox[{"x", "(", "t", ")"}], ",", 
      RowBox[{
       FractionBox["\[DifferentialD]", 
        RowBox[{"\[DifferentialD]", "t"}]], 
       RowBox[{"x", "(", "t", ")"}]}], ",", "t"}], ")"}]}]}], 
  TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
Note that the order of a differential equations is the highest number of \
derivatives of the unknown function.\
\>", "Text"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["(A) Free Fall",
 FontSize->16]], "Section"],

Cell["\<\
\tFor free fall we will take down as the positive direction and the force \
becomes  \
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "(", 
     RowBox[{"x", ",", "v", ",", "t"}], ")"}], "=", 
    RowBox[{"m", " ", "g"}]}], TraditionalForm]]],
 ".  "
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell["We will take the initial conditions to be ", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["t", "0"], "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "0"], "=", "0"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["v", "0"], "=", "0"}], TraditionalForm]]],
 ".  "
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "We will also take the mass to be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", " ", "kg"}], TraditionalForm]]],
 ".  We will choose the final time to be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"t", "=", 
    RowBox[{"2", " ", 
     StyleBox["s",
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 " and choose our time interval at first to be the relatively large value of \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[CapitalDelta]", "\[ThinSpace]", "t"}], "=", 
    RowBox[{"0.2", " ", 
     StyleBox["s",
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 ".  XNum is the function that interpolates the discrete points with "
}], "Text"],

Cell[TextData[{
 "To evaluate ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " input place the cursor anywhere in the cell to input and hit Shift+Enter."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "0", "]"}], "=", "0"}], ";", 
  RowBox[{
   RowBox[{"x", "[", "0", "]"}], "=", "0"}], ";", 
  RowBox[{
   RowBox[{"v", "[", "0", "]"}], "=", "0"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "final", "]"}], "=", "2"}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", "0.2"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"m", "=", "2"}], ";", 
  RowBox[{"g", "=", "9.8"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"F", "[", 
   RowBox[{"x_", ",", "v_", ",", "t_"}], "]"}], ":=", 
  RowBox[{"m", " ", "g"}]}]}], "Input"],

Cell[TextData[{
 "The following is the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " code for finding the numerical solution.  "
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"iMaximum", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"t", "[", "final", "]"}], "-", 
      RowBox[{"t", "[", "0", "]"}]}], ")"}], "/", "\[CapitalDelta]t"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Do", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"t", "[", 
       RowBox[{"i", "+", "1"}], "]"}], "=", 
      RowBox[{
       RowBox[{"t", "[", "i", "]"}], "+", "\[CapitalDelta]t"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"x", "[", 
       RowBox[{"i", "+", "1"}], "]"}], "=", 
      RowBox[{
       RowBox[{"x", "[", "i", "]"}], "+", 
       RowBox[{
        RowBox[{"v", "[", "i", "]"}], " ", "\[CapitalDelta]t"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"v", "[", 
       RowBox[{"i", "+", "1"}], "]"}], "=", 
      RowBox[{
       RowBox[{"v", "[", "i", "]"}], "+", 
       RowBox[{"\[CapitalDelta]t", " ", 
        RowBox[{
         RowBox[{"F", "[", 
          RowBox[{
           RowBox[{"x", "[", "i", "]"}], ",", 
           RowBox[{"v", "[", "i", "]"}], ",", 
           RowBox[{"t", "[", "i", "]"}]}], "]"}], "/", "m"}]}]}]}], ";"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{"i", ",", "0", ",", 
      RowBox[{"iMaximum", "-", "1"}]}], "}"}]}], "\[IndentingNewLine]", "]"}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"XNum", "=", 
   RowBox[{"Interpolation", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"t", "[", "i", "]"}], ",", 
        RowBox[{"x", "[", "i", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "0", ",", "iMaximum"}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VelNum", "=", 
   RowBox[{"Interpolation", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"t", "[", "i", "]"}], ",", 
        RowBox[{"v", "[", "i", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "0", ",", "iMaximum"}], "}"}]}], "]"}], "]"}]}], 
  ";"}]}], "Input"],

Cell[TextData[{
 "The following is the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " code for finding the symbolic solution.  The function XSym is the result \
of DSolve, the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function for solving a differential equation.  VelSym is the symbolic \
velocity function, found by differentiating the position XSym."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"XSym", "=", 
   RowBox[{
    RowBox[{"DSolve", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"F", "[", 
          RowBox[{
           RowBox[{"X", "[", "t", "]"}], ",", 
           RowBox[{
            RowBox[{"X", "'"}], "[", "t", "]"}], ",", "t"}], "]"}], 
         "\[Equal]", 
         RowBox[{"m", " ", 
          RowBox[{
           RowBox[{"X", "''"}], "[", "t", "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"X", "[", "0", "]"}], "\[Equal]", 
         RowBox[{"x", "[", "0", "]"}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"X", "'"}], "[", "0", "]"}], "\[Equal]", 
         RowBox[{"v", "[", "0", "]"}]}]}], "}"}], ",", "X", ",", "t"}], "]"}],
     "\[LeftDoubleBracket]", 
    RowBox[{"1", ",", "1", ",", "2"}], "\[RightDoubleBracket]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VelSym", "=", 
   RowBox[{"XSym", "'"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<x(t) = \>\"", ",", 
   RowBox[{"XSym", "[", "t", "]"}]}], "]"}], "\n", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<v(t) = \>\"", ",", 
   RowBox[{"VelSym", "[", "t", "]"}]}], "]"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"x(t) = \"\>", "\[InvisibleSpace]", 
   RowBox[{"4.9`", " ", 
    SuperscriptBox["t", "2"]}]}],
  SequenceForm["x(t) = ", 4.9 $CellContext`t^2],
  Editable->False]], "Print",
 CellChangeTimes->{3.4012733676722317`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"v(t) = \"\>", "\[InvisibleSpace]", 
   RowBox[{"9.8`", " ", "t"}]}],
  SequenceForm["v(t) = ", 9.8 $CellContext`t],
  Editable->False]], "Print",
 CellChangeTimes->{3.401273367695548*^9}]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
 "\tThis is the code for plotting the numerical solution, the wide yellow \
line, and the symbolic solution as a narrow blue line.  The first plot is ",
 StyleBox["x",
  FontSlant->"Italic"],
 " vs. ",
 StyleBox["t",
  FontSlant->"Italic"],
 " and the second is ",
 StyleBox["v",
  FontSlant->"Italic"],
 " vs. ",
 StyleBox["t",
  FontSlant->"Italic"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphicsColumn", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Plot", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"XNum", "[", "t", "]"}], ",", 
        RowBox[{"XSym", "[", "t", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"t", ",", "0", ",", 
        RowBox[{"t", "[", "final", "]"}]}], "}"}], ",", 
      RowBox[{"PlotStyle", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"RGBColor", "[", 
            RowBox[{"1", ",", "1", ",", "0.4`"}], "]"}], ",", 
           RowBox[{"Thickness", "[", "0.02`", "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"RGBColor", "[", 
            RowBox[{"0", ",", "0", ",", "0.6`"}], "]"}], ",", 
           RowBox[{"Thickness", "[", "0.008`", "]"}]}], "}"}]}], "}"}]}], ",",
       
      RowBox[{"AxesLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"\"\<\!\(\*
StyleBox[\"t\",
FontSlant->\"Italic\"]\)\>\"", ",", "\"\<\!\(\*
StyleBox[\"x\",
FontSlant->\"Italic\"]\)\>\""}], "}"}]}], ",", 
      RowBox[{"Background", "\[Rule]", 
       RowBox[{"GrayLevel", "[", "0.6`", "]"}]}]}], "]"}], ",", 
    RowBox[{"Plot", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"VelNum", "[", "t", "]"}], ",", 
        RowBox[{"VelSym", "[", "t", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"t", ",", "0", ",", 
        RowBox[{"t", "[", "final", "]"}]}], "}"}], ",", 
      RowBox[{"PlotStyle", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"RGBColor", "[", 
            RowBox[{"1", ",", "1", ",", "0.4`"}], "]"}], ",", 
           RowBox[{"Thickness", "[", "0.02`", "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"RGBColor", "[", 
            RowBox[{"0", ",", "0", ",", "0.6`"}], "]"}], ",", 
           RowBox[{"Thickness", "[", "0.008`", "]"}]}], "}"}]}], "}"}]}], ",",
       
      RowBox[{"AxesLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"\"\<\!\(\*
StyleBox[\"t\",
FontSlant->\"Italic\"]\)\>\"", ",", "\"\<\!\(\*
StyleBox[\"v\",
FontSlant->\"Italic\"]\)\>\""}], "}"}]}], ",", 
      RowBox[{"Background", "\[Rule]", 
       RowBox[{"GrayLevel", "[", "0.6`", "]"}]}]}], "]"}]}], "}"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.399718388690641*^9, 3.3997184059529533`*^9}, {
  3.399718438587409*^9, 3.3997184964725027`*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {InsetBox[
     GraphicsBox[{{}, {}, 
       {RGBColor[1, 1, 0.4], Thickness[0.02], LineBox[CompressedData["
1:eJwV03k4lO0XB3AKKWXJrrKUFtlTlNR5opBUryXZQtnKEkqlehPKkqWEkH2b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         "]]}, 
       {RGBColor[0, 0, 0.6], Thickness[0.008], LineBox[CompressedData["
1:eJwV0Gk8lQkDBXBLShtREoUsqeEmIdsM55FS1rKm6GbJlX1LCyPL2AoV6Vqz
RYWsWULl4srYS0xTlspIN+QqIYX7Pu+H8zu//8dzpJ18LFx4uLi4ssj8v41c
WC8Zn1103f5ss+x25NFZpS/u+07sEEY/p8T4sJ/pqmanZj4Ss8K3nq481vSI
7tgOufCL8lTMjb68aUj7qmvc/bryjthZGMUPKBdMr+jKxUsUcKTcMCJ6JpKm
wI8lI+cUR3lvmM/KGCjThDAR2c90pPtBO03QNqdUFCfOLjxNEjuP4pC5i39N
S4KpL17bknkBFscn+qaSZKEsq1P+XeoyNoX/42uusAeZPA6Fu+4GQ9i88baP
zl7wj0bk2chfAcWhL0eWth/17/u6W6zC4OOqr+3poQbf42eN7OnhsHmmkpJY
qoHhfTH6CWKRWGsXn+E/rYvk7G2MXXZR8DgeLJ0SqAcjwaI/nmVGQyaDbfNf
kj5qprsOsKWugvuByCcDhaPwpFIrYxyv4Tf3Nj1qvhFkethKO+/GQdZKc8VZ
xxTXS4T3mMtfB7NX0UyCZo7DEvn5n11vIJiLt5HvgwWuPE/f1GR1E9tb6xxc
PKygpk5JOElPhNC65+rXSk8gR+RY9DWxZIxGnNzrPk1FcHcK06Y+Gd6FdOFo
fwfYRL3nkbW7jTN2M/eSAh2xYc4vtCGTDgmJ2dSRJGdc6k++NCWVho3b1PP0
FM7BMn645jEjDUXHshx9nN2gdEh+LtIxHTrmF+ps890x9qjWT+JuBnQrA1Wo
Ol4wv/XW3Uw+C4Nz/8qJ0/xAMZYtFG/LQrCWCS1b2h/8vJ6fxl2zsdVpIYDn
gz/O1K1UNFrlQMtpbtbR4zxEfrupZEvPhYbkzejo0osI53+0+6pYPsSiiHmX
6StQOJIYW+mbjyZevj3spVD0Rfl8HmzLx8Pt6we4Y8Mgx0spVrpQgCrqjWOf
csPxfDlfqb/vHrxkzsrPz/yFDbP0A1LxhcjO0bWTXYpF9f5A+tH/ChETE32h
+f5VUH0tF/y0i8D6+7vWfetrKPsiWMdkFUHE7lCd3uM4WLBi/3A//BBST8vS
6lOvI3U4SL96pRQLDYMzp52ToLfjZP6IdRnU/6psPNWZhIlTGnz8JWUov5jp
FKR2C5JTl51L9pbDjlUoJ7MuGT5u6ru4hsoRwypK29xyG8Jnyx4UaFZC9+zh
I1c80mB7Mrds5msVkuS8bbU6c8AetFqXJ16Nj02hIvUbchFN5Xex1K/GyO7D
6XVmuXjk7CNek1yNZ6aTP9X6cyHgrRMVrFEDPe6CbRIf89Aa8ebU6pBaGK/p
aa2ULIDKQ6HVO/jr4XFjfnx2phDtSq0O3cqk9cLTTQ8WwaHiUsOVk/UQetIo
Ip9chISad74fiuphvN7W2VKrGCxGyeA90waEC82fyr76EFkDhhX7bz0B+8mn
hiW9MqznhNsfkWwEu+143nhsJVLdTlApBo2YWpVmFve4EnL9FAch70Z8arFg
ebLI3YWvnQefNuJdm/jY+NFHqCukV9cqMcD8tLRUvaEKXyynnhj3MOAZs7aK
kV0N6wcpnQEbm7Hf4JBG7OxjZJ6OMKFJN2Nby5PEDoU6jAp7ddseaMZy0703
hk518Ak52Ktzuhmth3v3c17W4arFl741Jc0wdfHnPVxVj6dLB99mmLRAexU3
RSfyCXaZT7Na4phgvpdqZ+9hQDz0iLZrLhM5Tput1xszsKkkJ25dLRN+Pa11
Rl4M/OS3VDIfZWLS2XXjgUoGXjBqA0Y0W2EcfWe4VqcJQcrhKz8+tqL8wYD0
Bbtm9Ahu2aJEtGHH2UCtgQomLvZoIfV7O4Kq/QskOtohoFT9sZ2/Aw0uHoFm
39qRn6Ac/2tHBwYeKl3JEu/AS5Pdb6iHO5DB2aKd6dEBxc4tAbvoHfCsdbAT
EOwkf2Tfq9ToxBbf7/rddl040lQg0B3UhWi+jSXVG3ohWiU0ws3diz8dwn22
0fpQM6rf1ftHHzRbCg0kL75Bs/ud7Sen+mFIOfH2YPN7zMnrCJZ4vcYdBfkc
ad6P4Fks3rR911s0cVFvjyhMQJ3jUGNZOgTv3/s1A0PY6IpU63mu8A70H6az
XSnfIHVzcrFw+j0KOidK7GfnMLnwbmRAeRSnhiwNaq0XkT/R0CFw/T/cKrY0
YRssQ0DiPntsbAwV5mfUrWq4iHi3alqO3Dhu7N09LdrKQ8wTxWPnQj+hJcN3
ZkmGjzivwqza2sKCb15/yMKlNcS7UmbcS+kJvDvtGWEkt4745d6f8VvQJAKl
132eH99AKOqtWaE1TOFVuzmvSrogcSRP2rVCdBoGmqLBhLUQcVHijMwHezY6
3gsbMV8JE5Qp6/WvTWcwbRHRJHlqC7FZcIaf+XMGc/P3aH8LbyVWNWaUyWZ+
BY1iT8g9EyU81BrerDX+hoDJzQ9YWWJEsaFTzrP5b6iQVm6pjN1OzE2Kbxqn
z8KKN87E21GCqLY8wpt86DsYonvtvQKkiNCjUdlV49+hu6lHTLtuJyE9bPTv
3ag5SIo9f2zTIE1oa0Qmi6nMY3qr45n5QRnCXrJ3kfvVPNg0UYrBV1liriNh
1Dd8AZygxNv58ruI1FXU8xZyP7C4h6X+9Jg8sXmkOo+H+QPlk1a9NUG7Cb6Q
cCstv0Xk3vL5M69tD8FVSC0bFf6J477Jof6rFQj/9LIE4eafUHx5ab7pnCLx
VDGWX8j7F2bvNVCudVMIISN/SoTAEhQa4iKtZZSIob0vXl1+soRZH/+71zP3
EeVZkBujLuOXmWHfRIQycbpMXUhiZRm6g2oBAxr7ibCo3PChohVcaI/iO8el
QpSwJDIETDgoHnHWS2lUIcyFBDTbfnLwTb3kptotVeIaH8/2IdJrFRNkzEgz
F+eWZ0if6gupciWtPjrcIv6Lgwfaif+mkxZ/VHLMm/SsX5g0d7IqMWpl6iqy
xIFhcFhNN+nzqfF052UOFn6s/kKjqxIl8WGXL5HWtFoOCyM9HnbePoF0ldMq
kXTStm72MrWkbdIO6XST1tGmlK5b4cDUK+G6aooqsXqos7WC9GIZocaVqkrg
ReOD56Rf6B5sFyN9ifkobpD0hzArqirpiYcZ5nwcDkoli2NppHtCPIZtSf9g
7BzpIr3G/wzDi3TG5/sB46QJmuXdCNIXWrGWK02VqDT73e0h6Tf0XDVV0pMH
95k0kXaI9+gwIS2nIbvvH9I7Dxg40EhTFUWFJ0nvSNw/H0o6RWr9HIf0sZy9
8Wmk/wc/+rro
         "]]}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      AxesLabel->{
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"t\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm], 
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"x\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm]},
      AxesOrigin->{0, 0},
      Background->GrayLevel[0.6],
      PlotRange->{{0, 2}, {-0.04899984730648706, 19.59999920000001}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {189., -118.66252583997979`}, 
     ImageScaled[{0.5, 0.5}], {360., 222.4922359499621}], InsetBox[
     GraphicsBox[{{}, {}, 
       {RGBColor[1, 1, 0.4], Thickness[0.02], LineBox[CompressedData["
1:eJwVx3lY0nccwHEkLV0FYodD00AtS0gQnBUdn++y58kj8/FauimWJV6rOfV5
om0tL6w8tuXTY3ggdiwrUzKP2VOmKSvLK41pB2qRGUkzU8Ej8bfv/ng/7+fF
jPohMJpMIpH24f6/T7S2p/l99E4Sqbloor9op6mnTeIQfTeQWIz6arkY+HJp
SQ09GEgGweJf5KfBt7P/lox+GNB3Xq4H5VLYf3i6MZ+eAgyKf0uQ/CoMcE55
5tEzITEmcJmfvAHKVvlnZdPPwZcrg6u95W2QZl7jfIZ+GWJHVYu85M8gNOyC
YvxTLVxBTYkHrr6HpURa+B77JpD17koPpU7CuoAxbWuOErrONB7yYM3Csa6t
IJ16BBKP5e/K2gmwrqUNmph0g7Y//l68kYzqNZ4d3dt7IfD3rd/67l+MWuJl
tmEfVOBqobqfdcEC6dfvoFYe6Yeg99d5zjXLEHm2wtJ23Quwd/CYy1pkiTyI
A/VBVWpoINcNsc/SUEeme9cDlyHIVtJZJOsVaO0futlrY6+guXWy1NC4Eumm
hwb/4WqAbkg4L/dajS6P3nlM+e0N9EWHNSRMWSOKXfnH4eFhaK9nQ7SUjnLj
6kRlTiPwVlDqQ2PYIgOqGI49+Q4Ozmf++qptDUrhKWtXt2qB/Mlic1SIPRqq
Uub0MEchnUbOzTBhoM/xquKNP+mAYZuk+F7FQKyvlyyI7nyAgJ1uqziFTLTn
IjOm2noMeCQWNeOoAzpmF+nwOvwjXHmZ7HvzK0fE/hCytN9vHOrLFYKpGUe0
gjpurpwbB7HV+nuPOpyQaVOxwrHkE3QU6DLE+etQgvud5xa+E9BzWzfQELse
VXhHld0zTEBy5+AodZsz0utsLEcKJqF8jiFAhDOqC9qz6NzuKch9ILMjqzag
k14See3IFJjeiPOUFW1EzAGfZ5ckerBO37DGP8IFCTZnnqPzDKA2MM/kc1ko
3L571uSpAfiS+C0KPQvpH+dpEtOmYdutf3+OaGcjqakwJdBpBvpmmkze5m9C
KwbrLpKVM7CcKxFkfOOKzE6kBW/9cRb0ZpIlNS4cRLomVGis5iBR/3T13XEO
SipS5Fm1zEHXfdeIikNc1Mg6bU47+hl4JkfMTTu5iOaTxE6nzENkZ2+wN8cN
qTc9eXr87jy81YYk9Be6oZul4DQsNIL7WZJjoNENRSg8aHYLRuipdnJ/JuSh
VMmFNPX1BXjIFFHL23ioUmtXTNlLQJZ9d04Ui48CaJQtD+cIkMWH9IoK+Cjb
jGyrxo65tF+WjK2c1RvHsbnqsNhUbA/NQKvNZwJa/YQLhdg2NZX+R7G13BiX
TmxNsF/MqnkC+NPidP55PkqR5hYcMhLwKKOER5LyUWVu6nExdv7dUuMy7JHU
lPA87HB9WRsdOzQu3OEv7DHRn0I+9g4Bu+qLBQJW+lRli7AXq9v/rsaOtGzW
dGDDk6arD7A3erdUPscWK2tyXmJPpCnFI9ijN4oDzAgCJJNtFFIhH3WdSBgI
xb7e1yPgYy9Jimw+gp1MVZkhbCQKupSOvd2r78le7Fv7tsXdwO6+/UIkwtbt
4uy9jy2dULslYzttduT0YUexhuZPYgtZ1lY6bPbh1w9zsc+vXaonsPUlb/IL
sf8D8OxY8Q==
         "]]}, 
       {RGBColor[0, 0, 0.6], Thickness[0.008], LineBox[CompressedData["
1:eJwVx38803kcwPFZFKfbjOJGZChlMts6ZV29P5cej/xIHn5ddKxSJpx+yOPR
uqsLw5Ufd5dHD+bn9ONKieXn6RH5tUoxpB39GGo5yfqxsPmR+d7n/ng9Xo8n
I+poUDSZRCLtxv1/3+jxJ63vord5S5sKJwcLtxl6WR8boe+AGqZ9Q7VECFyJ
uLiWHgKeOt7S05Jz4CcfrCmhH4LTP3q7HZCIYc+hmeZcehLYUQLagyXlMMT6
zSuHngZHY4KW+0saoWxlQEYm/SJ8syKk2kfSCSnGtc7n6VchfkKxxFvyDMLC
L0k1n+vgGmo5tr/8HZgSKRE77VqgtH97ahh1CtYEfhzvyJJBz/nmgx7MOTjZ
4wni6UeQ4fH127IuAqzqaMMGBr0wPhh3L05PRg0qr+7e7/oh+A/PvX57lqL2
uBKb8PcKcDNRtGVcMkHatVuplQmDEPzuJse5djkiz1WY2ax5AXYOHvMZS8yQ
B7G/IbhKCY3k+hHXCzTUnbax54HLCGTK6EySlQVa/ad67sbHV9DaMVWqa16B
1DMjw/+4q8BaF58v8bZEVyfuPqb8/gYGosMb46etEMX2+qfR0VHoanCFaDEd
ZcfWC8qcxmCMV+pLs7dBOlQxevjsWziwkPbrq85VKIkjq7PsGAfyZ5NNUaF2
aKRKlvWEMQEiGjlbZGCPvsQpitb/rAZ7m0TpTwp7xPx+2aLg7nsI3MZeySpg
oJ2XGTHVVh+BQ2JSRUcc0EnbfQ6vIz7BtZcn/G5/64hc34eaDvproOG6lDc9
64gsqBpj2bwGhOZr7z3qdkKGLUVSx+LP0J2nFglz16D4jXefm/hNQt8d9VDj
4bWowieq7J5uEhLlwxPULc5Iq7Y2G8ubgmvz9jxEOKP64J1LLu6YhqwHJbZk
xTp01jtdUjc2DeRbsV4lhesRY8j32ZV0LVimrlsVEOmCeJvSLtI5OnipY5zP
dWeiCLveOYOnOmCnx22WaplI+zhHdSxlBng1H36J7HJFYkN+UpDTLChmWwz+
zd2ALIbrL5Nls2Dqns4T/eCGjM6khHgen4Npo/RltS4sRLrBl6rM5yFB+9Sy
ScNCiYXSHPP2eZC3uUVWHHRHzcxzxrQjX4BtkGBsKHdHNN9E11TKAvDl/SE+
LDZSbuh7eqppAUbHQ+MHC9jodik4jfL1wL1AcgzSs1Gk1INmu6iHvmqnjc/4
HJScfilFeXMR7jME1OudHFQ5bltE2UVAml1vVhSTiwJplM0P5wkojgvtF+Rx
UaYR2UaJHX1lT8kJbNmcVq/BZinDDydje6iGOqy/ENDmz18swLaurQw4gv3W
PcZFjq0K8Y9ZuUAAe0aYys3noiRxdt5BPQEPRcUckpiLKrOTTwmxLzSV6pdj
jyUnReRg79WWddKxw2IjHP7G/iD4i8/F3spzrfpqkQAL36pMAfZSZdf9amy+
WauqGxv6WsofYDv7tFc+xxbKarNeYmtSZMIx7IlbRYFGBAGiqU4KqYCLes7E
D4Vhlw884XGxlyXua03APk5VGCFsJAi+koq9xXugbxd2ze4tsbew5XdeCATY
6u2sXW3Y+ZNK9glsp02OrAHs/cyRhbPYfKaVuRrb5dDrh9nY+atNtQT2VPGb
3ALs/wDCO1rP
         "]]}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      AxesLabel->{
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"t\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm], 
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"v\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm]},
      AxesOrigin->{0, 0},
      Background->GrayLevel[0.6],
      PlotRange->{{0, 2}, {0., 19.599999600000007`}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {189., -355.98757751993935`}, 
     ImageScaled[{0.5, 0.5}], {360., 222.4922359499621}]}, {}},
  ContentSelectable->True,
  PlotRangePadding->{6, 5}]], "Output",
 CellChangeTimes->{3.4012733712809668`*^9}]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function CalculateAndGraph is now defined to follow the above procedure of \
calculating the numerical and symbolic solutions, and then graphing those \
solutions."
}], "Text"],

Cell[BoxData[
 RowBox[{"CalculateAndGraph", ":=", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"iMaximum", "=", 
     FractionBox[
      RowBox[{
       RowBox[{"t", "[", "final", "]"}], "-", 
       RowBox[{"t", "[", "0", "]"}]}], "\[CapitalDelta]t"]}], ";", 
    RowBox[{"Do", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"t", "[", 
         RowBox[{"i", "+", "1"}], "]"}], "=", 
        RowBox[{
         RowBox[{"t", "[", "i", "]"}], "+", "\[CapitalDelta]t"}]}], ";", 
       RowBox[{
        RowBox[{"x", "[", 
         RowBox[{"i", "+", "1"}], "]"}], "=", 
        RowBox[{
         RowBox[{"x", "[", "i", "]"}], "+", 
         RowBox[{
          RowBox[{"v", "[", "i", "]"}], " ", "\[CapitalDelta]t"}]}]}], ";", 
       RowBox[{
        RowBox[{"v", "[", 
         RowBox[{"i", "+", "1"}], "]"}], "=", 
        RowBox[{
         RowBox[{"v", "[", "i", "]"}], "+", 
         FractionBox[
          RowBox[{"\[CapitalDelta]t", " ", 
           RowBox[{"F", "[", 
            RowBox[{
             RowBox[{"x", "[", "i", "]"}], ",", 
             RowBox[{"v", "[", "i", "]"}], ",", 
             RowBox[{"t", "[", "i", "]"}]}], "]"}]}], "m"]}]}], ";"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "0", ",", 
        RowBox[{"iMaximum", "-", "1"}]}], "}"}]}], "]"}], ";", 
    RowBox[{"XNum", "=", 
     RowBox[{"Interpolation", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"t", "[", "i", "]"}], ",", 
          RowBox[{"x", "[", "i", "]"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "0", ",", "iMaximum"}], "}"}]}], "]"}], "]"}]}], 
    ";", 
    RowBox[{"VelNum", "=", 
     RowBox[{"Interpolation", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"t", "[", "i", "]"}], ",", 
          RowBox[{"v", "[", "i", "]"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "0", ",", "iMaximum"}], "}"}]}], "]"}], "]"}]}], 
    ";", 
    RowBox[{"XSym", "=", 
     RowBox[{"Quiet", "[", 
      RowBox[{
       RowBox[{"DSolve", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"F", "[", 
             RowBox[{
              RowBox[{"X", "[", "t", "]"}], ",", 
              RowBox[{
               SuperscriptBox["X", "\[Prime]",
                MultilineFunction->None], "[", "t", "]"}], ",", "t"}], "]"}], 
            "\[Equal]", 
            RowBox[{"m", " ", 
             RowBox[{
              SuperscriptBox["X", "\[Prime]\[Prime]",
               MultilineFunction->None], "[", "t", "]"}]}]}], ",", 
           RowBox[{
            RowBox[{"X", "[", "0", "]"}], "\[Equal]", 
            RowBox[{"x", "[", "0", "]"}]}], ",", 
           RowBox[{
            RowBox[{
             SuperscriptBox["X", "\[Prime]",
              MultilineFunction->None], "[", "0", "]"}], "\[Equal]", 
            RowBox[{"v", "[", "0", "]"}]}]}], "}"}], ",", "X", ",", "t"}], 
        "]"}], "\[LeftDoubleBracket]", 
       RowBox[{"1", ",", "1", ",", "2"}], "\[RightDoubleBracket]"}], "]"}]}], 
    ";", 
    RowBox[{"VelSym", "=", 
     SuperscriptBox["XSym", "\[Prime]",
      MultilineFunction->None]}], ";", 
    RowBox[{"Print", "[", 
     RowBox[{"\"\<x(t) = \>\"", ",", 
      RowBox[{"XSym", "[", "t", "]"}]}], "]"}], ";", 
    RowBox[{"Print", "[", 
     RowBox[{"\"\<v(t) = \>\"", ",", 
      RowBox[{"VelSym", "[", "t", "]"}]}], "]"}], ";", 
    RowBox[{"GraphicsColumn", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Plot", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"XNum", "[", "t", "]"}], ",", 
           RowBox[{"XSym", "[", "t", "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", 
           RowBox[{"t", "[", "final", "]"}]}], "}"}], ",", 
         RowBox[{"PlotStyle", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"RGBColor", "[", 
               RowBox[{"1", ",", "1", ",", "0.4`"}], "]"}], ",", 
              RowBox[{"Thickness", "[", "0.02`", "]"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"RGBColor", "[", 
               RowBox[{"0", ",", "0", ",", "0.6`"}], "]"}], ",", 
              RowBox[{"Thickness", "[", "0.008`", "]"}]}], "}"}]}], "}"}]}], 
         ",", 
         RowBox[{"AxesLabel", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"\"\<\!\(\*
StyleBox[\"t\",
FontSlant->\"Italic\"]\)\>\"", ",", "\"\<\!\(\*
StyleBox[\"x\",
FontSlant->\"Italic\"]\)\>\""}], "}"}]}], ",", 
         RowBox[{"Background", "\[Rule]", 
          RowBox[{"GrayLevel", "[", "0.6`", "]"}]}]}], "]"}], ",", 
       RowBox[{"Plot", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"VelNum", "[", "t", "]"}], ",", 
           RowBox[{"VelSym", "[", "t", "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", 
           RowBox[{"t", "[", "final", "]"}]}], "}"}], ",", 
         RowBox[{"PlotStyle", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"RGBColor", "[", 
               RowBox[{"1", ",", "1", ",", "0.4`"}], "]"}], ",", 
              RowBox[{"Thickness", "[", "0.02`", "]"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"RGBColor", "[", 
               RowBox[{"0", ",", "0", ",", "0.6`"}], "]"}], ",", 
              RowBox[{"Thickness", "[", "0.008`", "]"}]}], "}"}]}], "}"}]}], 
         ",", 
         RowBox[{"AxesLabel", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"\"\<\!\(\*
StyleBox[\"t\",
FontSlant->\"Italic\"]\)\>\"", ",", "\"\<\!\(\*
StyleBox[\"v\",
FontSlant->\"Italic\"]\)\>\""}], "}"}]}], ",", 
         RowBox[{"Background", "\[Rule]", 
          RowBox[{"GrayLevel", "[", "0.6`", "]"}]}]}], "]"}]}], "}"}], 
     "]"}]}], ")"}]}]], "Input",
 CellChangeTimes->{
  3.3997185926724863`*^9, {3.4011347419479303`*^9, 3.401134757505969*^9}}],

Cell[TextData[{
 "\tNote that with the very crude value of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[CapitalDelta]", "\[ThinSpace]", "t"}], "=", 
    RowBox[{"0.2", " ", 
     StyleBox["s",
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 ", where there are only 10 intervals, the symbolic and numerical values are \
quite different.  To improve this we must choose a smaller ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]", "\[ThinSpace]", "t"}], TraditionalForm]]],
 " to make a more accurate numerical approximation.  Now we will try ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[CapitalDelta]", "\[ThinSpace]", "t"}], "=", 
    RowBox[{"0.02", " ", 
     StyleBox["s",
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 " and redo the numerical solutions and plots."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[CapitalDelta]t", "=", "0.02"}], 
  ";"}], "\[IndentingNewLine]", "CalculateAndGraph"}], "Input",
 CellChangeTimes->{3.399733141903624*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"x(t) = \"\>", "\[InvisibleSpace]", 
   RowBox[{"4.9`", " ", 
    SuperscriptBox["t", "2"]}]}],
  SequenceForm["x(t) = ", 4.9 $CellContext`t^2],
  Editable->False]], "Print",
 CellChangeTimes->{3.401273379915504*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"v(t) = \"\>", "\[InvisibleSpace]", 
   RowBox[{"9.8`", " ", "t"}]}],
  SequenceForm["v(t) = ", 9.8 $CellContext`t],
  Editable->False]], "Print",
 CellChangeTimes->{3.4012733799392843`*^9}]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[{{}, {InsetBox[
     GraphicsBox[{{}, {}, 
       {RGBColor[1, 1, 0.4], Thickness[0.02], LineBox[CompressedData["
1:eJwV0mc8lgsDBnCp9LSsUlnZyvFkRDRw3Rk5hBKJsmecjoiyUogQ0tCDyMiI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         "]]}, 
       {RGBColor[0, 0, 0.6], Thickness[0.008], LineBox[CompressedData["
1:eJwV0Gk8lQkDBXBLShtREoUsqeEmIdsM55FS1rKm6GbJlX1LCyPL2AoV6Vqz
RYWsWULl4srYS0xTlspIN+QqIYX7Pu+H8zu//8dzpJ18LFx4uLi4ssj8v41c
WC8Zn1103f5ss+x25NFZpS/u+07sEEY/p8T4sJ/pqmanZj4Ss8K3nq481vSI
7tgOufCL8lTMjb68aUj7qmvc/bryjthZGMUPKBdMr+jKxUsUcKTcMCJ6JpKm
wI8lI+cUR3lvmM/KGCjThDAR2c90pPtBO03QNqdUFCfOLjxNEjuP4pC5i39N
S4KpL17bknkBFscn+qaSZKEsq1P+XeoyNoX/42uusAeZPA6Fu+4GQ9i88baP
zl7wj0bk2chfAcWhL0eWth/17/u6W6zC4OOqr+3poQbf42eN7OnhsHmmkpJY
qoHhfTH6CWKRWGsXn+E/rYvk7G2MXXZR8DgeLJ0SqAcjwaI/nmVGQyaDbfNf
kj5qprsOsKWugvuByCcDhaPwpFIrYxyv4Tf3Nj1qvhFkethKO+/GQdZKc8VZ
xxTXS4T3mMtfB7NX0UyCZo7DEvn5n11vIJiLt5HvgwWuPE/f1GR1E9tb6xxc
PKygpk5JOElPhNC65+rXSk8gR+RY9DWxZIxGnNzrPk1FcHcK06Y+Gd6FdOFo
fwfYRL3nkbW7jTN2M/eSAh2xYc4vtCGTDgmJ2dSRJGdc6k++NCWVho3b1PP0
FM7BMn645jEjDUXHshx9nN2gdEh+LtIxHTrmF+ps890x9qjWT+JuBnQrA1Wo
Ol4wv/XW3Uw+C4Nz/8qJ0/xAMZYtFG/LQrCWCS1b2h/8vJ6fxl2zsdVpIYDn
gz/O1K1UNFrlQMtpbtbR4zxEfrupZEvPhYbkzejo0osI53+0+6pYPsSiiHmX
6StQOJIYW+mbjyZevj3spVD0Rfl8HmzLx8Pt6we4Y8Mgx0spVrpQgCrqjWOf
csPxfDlfqb/vHrxkzsrPz/yFDbP0A1LxhcjO0bWTXYpF9f5A+tH/ChETE32h
+f5VUH0tF/y0i8D6+7vWfetrKPsiWMdkFUHE7lCd3uM4WLBi/3A//BBST8vS
6lOvI3U4SL96pRQLDYMzp52ToLfjZP6IdRnU/6psPNWZhIlTGnz8JWUov5jp
FKR2C5JTl51L9pbDjlUoJ7MuGT5u6ru4hsoRwypK29xyG8Jnyx4UaFZC9+zh
I1c80mB7Mrds5msVkuS8bbU6c8AetFqXJ16Nj02hIvUbchFN5Xex1K/GyO7D
6XVmuXjk7CNek1yNZ6aTP9X6cyHgrRMVrFEDPe6CbRIf89Aa8ebU6pBaGK/p
aa2ULIDKQ6HVO/jr4XFjfnx2phDtSq0O3cqk9cLTTQ8WwaHiUsOVk/UQetIo
Ip9chISad74fiuphvN7W2VKrGCxGyeA90waEC82fyr76EFkDhhX7bz0B+8mn
hiW9MqznhNsfkWwEu+143nhsJVLdTlApBo2YWpVmFve4EnL9FAch70Z8arFg
ebLI3YWvnQefNuJdm/jY+NFHqCukV9cqMcD8tLRUvaEKXyynnhj3MOAZs7aK
kV0N6wcpnQEbm7Hf4JBG7OxjZJ6OMKFJN2Nby5PEDoU6jAp7ddseaMZy0703
hk518Ak52Ktzuhmth3v3c17W4arFl741Jc0wdfHnPVxVj6dLB99mmLRAexU3
RSfyCXaZT7Na4phgvpdqZ+9hQDz0iLZrLhM5Tput1xszsKkkJ25dLRN+Pa11
Rl4M/OS3VDIfZWLS2XXjgUoGXjBqA0Y0W2EcfWe4VqcJQcrhKz8+tqL8wYD0
Bbtm9Ahu2aJEtGHH2UCtgQomLvZoIfV7O4Kq/QskOtohoFT9sZ2/Aw0uHoFm
39qRn6Ac/2tHBwYeKl3JEu/AS5Pdb6iHO5DB2aKd6dEBxc4tAbvoHfCsdbAT
EOwkf2Tfq9ToxBbf7/rddl040lQg0B3UhWi+jSXVG3ohWiU0ws3diz8dwn22
0fpQM6rf1ftHHzRbCg0kL75Bs/ud7Sen+mFIOfH2YPN7zMnrCJZ4vcYdBfkc
ad6P4Fks3rR911s0cVFvjyhMQJ3jUGNZOgTv3/s1A0PY6IpU63mu8A70H6az
XSnfIHVzcrFw+j0KOidK7GfnMLnwbmRAeRSnhiwNaq0XkT/R0CFw/T/cKrY0
YRssQ0DiPntsbAwV5mfUrWq4iHi3alqO3Dhu7N09LdrKQ8wTxWPnQj+hJcN3
ZkmGjzivwqza2sKCb15/yMKlNcS7UmbcS+kJvDvtGWEkt4745d6f8VvQJAKl
132eH99AKOqtWaE1TOFVuzmvSrogcSRP2rVCdBoGmqLBhLUQcVHijMwHezY6
3gsbMV8JE5Qp6/WvTWcwbRHRJHlqC7FZcIaf+XMGc/P3aH8LbyVWNWaUyWZ+
BY1iT8g9EyU81BrerDX+hoDJzQ9YWWJEsaFTzrP5b6iQVm6pjN1OzE2Kbxqn
z8KKN87E21GCqLY8wpt86DsYonvtvQKkiNCjUdlV49+hu6lHTLtuJyE9bPTv
3ag5SIo9f2zTIE1oa0Qmi6nMY3qr45n5QRnCXrJ3kfvVPNg0UYrBV1liriNh
1Dd8AZygxNv58ruI1FXU8xZyP7C4h6X+9Jg8sXmkOo+H+QPlk1a9NUG7Cb6Q
cCstv0Xk3vL5M69tD8FVSC0bFf6J477Jof6rFQj/9LIE4eafUHx5ab7pnCLx
VDGWX8j7F2bvNVCudVMIISN/SoTAEhQa4iKtZZSIob0vXl1+soRZH/+71zP3
EeVZkBujLuOXmWHfRIQycbpMXUhiZRm6g2oBAxr7ibCo3PChohVcaI/iO8el
QpSwJDIETDgoHnHWS2lUIcyFBDTbfnLwTb3kptotVeIaH8/2IdJrFRNkzEgz
F+eWZ0if6gupciWtPjrcIv6Lgwfaif+mkxZ/VHLMm/SsX5g0d7IqMWpl6iqy
xIFhcFhNN+nzqfF052UOFn6s/kKjqxIl8WGXL5HWtFoOCyM9HnbePoF0ldMq
kXTStm72MrWkbdIO6XST1tGmlK5b4cDUK+G6aooqsXqos7WC9GIZocaVqkrg
ReOD56Rf6B5sFyN9ifkobpD0hzArqirpiYcZ5nwcDkoli2NppHtCPIZtSf9g
7BzpIr3G/wzDi3TG5/sB46QJmuXdCNIXWrGWK02VqDT73e0h6Tf0XDVV0pMH
95k0kXaI9+gwIS2nIbvvH9I7Dxg40EhTFUWFJ0nvSNw/H0o6RWr9HIf0sZy9
8Wmk/wc/+rro
         "]]}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      AxesLabel->{
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"t\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm], 
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"x\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm]},
      AxesOrigin->{0, 0},
      Background->GrayLevel[0.6],
      PlotRange->{{0, 2}, {-0.0004898311373797477, 19.59999920000001}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {189., -118.66252583997979`}, 
     ImageScaled[{0.5, 0.5}], {360., 222.4922359499621}], InsetBox[
     GraphicsBox[{{}, {}, 
       {RGBColor[1, 1, 0.4], Thickness[0.02], LineBox[CompressedData["
1:eJwVz388E3gcx/FZFKfbTD/2GJH5WbYM61Tr6vO99HjkR/LIj4uO6cSEy0ke
j9bdVYZ1CXfl4cH8nH7cqcSSH+lRCDtUltIu/Rir5ZbM1cKGZXbf++P9eD2e
f77p8T+GJRIJBMIevP8blDj+5N77xO0EQlv51HD5dnN/u3QlbSfUMJxaG8V8
YItFlU20CPDWc5b+Ij4DwbLhm1W0BOB/F+D1vVgE+xJm24tombCGFNodLr4C
I6xf/QtpuZCWFLY8RNwGNatCT5+lFQN1ZURjoLgfBJZNHnm0y5A6IV8SIH4O
UdEXJNpPzfAn6kw/cOU9WJsEMbscO6FyaEd2FHka3PZ+GO/Jl8JAXvtBP8Y8
HHu0BUQz90Ho9+W7mocmoDZTRs3MBmF8OKUjxUhErSr/gcGvhyD89y37g/ct
Rd0pVfbRk3JgWcm7Tl+wQjr3beT6w8MQ8f6ar0fTckScr7Oxd3sJa539DKeX
2CA/04HW8AYF3CK2KJnnKWggd+OjXk8l5ElpDAJ1BVp7TjN/9cNr6OiZrta3
r0SaWeXo394qoOpTS8UBq9HliTsPSL+9haHE6LbUGSoiOdR+HBsbg95WJiSK
aKgguYVX46oGJac6iOJkj/SobuzQqXewfyH35Ov+NSjTV9q8umccDFqrTfGR
jkjZIM1/Qp+AkxRiQY6ZE/qcIq9Y/5MG7O0zJD/InRDjm2WLvDuTELzdZxWr
jI52XaQnNVI/AJPAIOekOaNjDnHOb2I+gvjV0eAbX7kg5mSk9XCIFm7USjgz
cy5oBVlrKTVoId3WveP+gCsy76yQuFR+AmmJJodf5IZSN955YRU8Bf23NSNt
h9xRXWB8TYd+CpJloxPkrR5Ip7GzUZdMQ5XBiYNMHqglfNeS4p0zIOitciDK
16FTAUJxs3oG5uqS/avK1yP6SNDzS0IdWGevWxMa64k4m3KLab56GNTT84q8
GSjGcXDe7Kke3IQpmyU6BtI9KFSlC2Zhw81/f459yEQic25mmOsc9M11mv1T
tAGtGG25SJTOwSJLyMn51gtZnBBEbDkyD2oL4bImTxYiXOVKVLYGiNM9XX1X
y0IZ5ZJC224D9HZ5xdYd9EbtjDOWlLTPwDA7bGku80aUoAxmNmkBomVDEYEs
H6TY8Pjp8bsLoByPTB0u80E3qsF1jGsE1nmCS5jRB8VK/CgOi0aQNbpufM71
RVnCCwLFtUXoofPItf2+qH7coYK02wTZjoP58Qw22kshbe4zmKAiJXKIV8JG
Zy2I9grshEv7qo5iS+d1Ri22lyL6UBa2n2qkx+6zCbpCuItl2HZN9aFp2Grv
JE8ZtioiJGnVggl8ZvnZ7FI2yhQVlBw0mqAvp9KXIGKj+oKs43zsc3erjcux
1VmZMYXY0bqafhp2VHKM8y3sSd4fXDb2Ng6z4YtFE9gGNZzlYS9VPPyrETvW
5p5qABsed17pxXYP7K5/gc2XNuW/wtYKpHw19sT1ir0WJvx3up9EKGOjRydS
R6Kwa5894bCxl2XE3TuMfYQst0DYiBd+KRubE/Ds8W7sm3u2Jl/Hlt1+yeNh
a3awdndhl0wpfI5iu25yYT3DjmMoF05hcxlUWw32+oQ3fQXYpWutdSbs6cq3
RWXY/wFt5VuN
         "]]}, 
       {RGBColor[0, 0, 0.6], Thickness[0.008], LineBox[CompressedData["
1:eJwVx38803kcwPFZFKfbjOJGZChlMts6ZV29P5cej/xIHn5ddKxSJpx+yOPR
uqsLw5Ufd5dHD+bn9ONKieXn6RH5tUoxpB39GGo5yfqxsPmR+d7n/ng9Xo8n
I+poUDSZRCLtxv1/3+jxJ63vord5S5sKJwcLtxl6WR8boe+AGqZ9Q7VECFyJ
uLiWHgKeOt7S05Jz4CcfrCmhH4LTP3q7HZCIYc+hmeZcehLYUQLagyXlMMT6
zSuHngZHY4KW+0saoWxlQEYm/SJ8syKk2kfSCSnGtc7n6VchfkKxxFvyDMLC
L0k1n+vgGmo5tr/8HZgSKRE77VqgtH97ahh1CtYEfhzvyJJBz/nmgx7MOTjZ
4wni6UeQ4fH127IuAqzqaMMGBr0wPhh3L05PRg0qr+7e7/oh+A/PvX57lqL2
uBKb8PcKcDNRtGVcMkHatVuplQmDEPzuJse5djkiz1WY2ax5AXYOHvMZS8yQ
B7G/IbhKCY3k+hHXCzTUnbax54HLCGTK6EySlQVa/ad67sbHV9DaMVWqa16B
1DMjw/+4q8BaF58v8bZEVyfuPqb8/gYGosMb46etEMX2+qfR0VHoanCFaDEd
ZcfWC8qcxmCMV+pLs7dBOlQxevjsWziwkPbrq85VKIkjq7PsGAfyZ5NNUaF2
aKRKlvWEMQEiGjlbZGCPvsQpitb/rAZ7m0TpTwp7xPx+2aLg7nsI3MZeySpg
oJ2XGTHVVh+BQ2JSRUcc0EnbfQ6vIz7BtZcn/G5/64hc34eaDvproOG6lDc9
64gsqBpj2bwGhOZr7z3qdkKGLUVSx+LP0J2nFglz16D4jXefm/hNQt8d9VDj
4bWowieq7J5uEhLlwxPULc5Iq7Y2G8ubgmvz9jxEOKP64J1LLu6YhqwHJbZk
xTp01jtdUjc2DeRbsV4lhesRY8j32ZV0LVimrlsVEOmCeJvSLtI5OnipY5zP
dWeiCLveOYOnOmCnx22WaplI+zhHdSxlBng1H36J7HJFYkN+UpDTLChmWwz+
zd2ALIbrL5Nls2Dqns4T/eCGjM6khHgen4Npo/RltS4sRLrBl6rM5yFB+9Sy
ScNCiYXSHPP2eZC3uUVWHHRHzcxzxrQjX4BtkGBsKHdHNN9E11TKAvDl/SE+
LDZSbuh7eqppAUbHQ+MHC9jodik4jfL1wL1AcgzSs1Gk1INmu6iHvmqnjc/4
HJScfilFeXMR7jME1OudHFQ5bltE2UVAml1vVhSTiwJplM0P5wkojgvtF+Rx
UaYR2UaJHX1lT8kJbNmcVq/BZinDDydje6iGOqy/ENDmz18swLaurQw4gv3W
PcZFjq0K8Y9ZuUAAe0aYys3noiRxdt5BPQEPRcUckpiLKrOTTwmxLzSV6pdj
jyUnReRg79WWddKxw2IjHP7G/iD4i8/F3spzrfpqkQAL36pMAfZSZdf9amy+
WauqGxv6WsofYDv7tFc+xxbKarNeYmtSZMIx7IlbRYFGBAGiqU4KqYCLes7E
D4Vhlw884XGxlyXua03APk5VGCFsJAi+koq9xXugbxd2ze4tsbew5XdeCATY
6u2sXW3Y+ZNK9glsp02OrAHs/cyRhbPYfKaVuRrb5dDrh9nY+atNtQT2VPGb
3ALs/wDCO1rP
         "]]}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      AxesLabel->{
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"t\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm], 
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"v\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm]},
      AxesOrigin->{0, 0},
      Background->GrayLevel[0.6],
      PlotRange->{{0, 2}, {0., 19.5999996}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {189., -355.98757751993935`}, 
     ImageScaled[{0.5, 0.5}], {360., 222.4922359499621}]}, {}},
  ContentSelectable->True,
  PlotRangePadding->{6, 5}]], "Output",
 CellChangeTimes->{3.40127337999473*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["(B) Motion with Viscous Damping",
 FontSize->16]], "Section"],

Cell[TextData[{
 "\tViscous damping is a resistive force ",
 StyleBox["R",
  FontSlant->"Italic"],
 " that is proportional to the velocity ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "=", 
    RowBox[{
     RowBox[{"-", "b"}], " ", "v"}]}], TraditionalForm]]],
 ".  Take the only force to be this.  The force function becomes:"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "(", 
     RowBox[{"x", ",", "v", ",", "t"}], ")"}], "=", 
    RowBox[{
     RowBox[{"-", "b"}], " ", "v"}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "Below is a repeat of the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " code for free fall.  Modify it to change the force function to this and \
change the initial conditions and other parameters to"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["t", "0"], "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "0"], "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["v", "0"], "=", 
    RowBox[{"3", " ", 
     StyleBox[
      FractionBox[
       StyleBox["m",
        FontSlant->"Plain"], 
       StyleBox["s",
        FontSlant->"Plain"]],
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "=", 
    RowBox[{"2", " ", "kg"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "=", 
    RowBox[{"5", " ", 
     StyleBox[
      FractionBox[
       StyleBox["kg",
        FontSlant->"Plain"], 
       StyleBox["s",
        FontSlant->"Plain"]],
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "0", "]"}], "=", "0"}], ";", 
  RowBox[{
   RowBox[{"x", "[", "0", "]"}], "=", "0"}], ";", 
  RowBox[{
   RowBox[{"v", "[", "0", "]"}], "=", "3"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "final", "]"}], "=", "2"}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", "0.02"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"m", "=", "2"}], ";", 
  RowBox[{"g", "=", "9.8"}], ";", 
  RowBox[{"b", "=", "5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"F", "[", 
   RowBox[{"x_", ",", "v_", ",", "t_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"-", "b"}], " ", 
   "v"}]}], "\[IndentingNewLine]", "CalculateAndGraph"}], "Input",
 CellChangeTimes->{
  3.3997186413508787`*^9, {3.4012734337747383`*^9, 3.401273455091448*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"x(t) = \"\>", "\[InvisibleSpace]", 
   RowBox[{
    FractionBox["6", "5"], " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "5"}], " ", "t"}], "/", "2"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{
        RowBox[{"5", " ", "t"}], "/", "2"}]]}], ")"}]}]}],
  SequenceForm[
  "x(t) = ", Rational[6, 5] 
   E^(Rational[-5, 2] $CellContext`t) (-1 + 
    E^(Rational[5, 2] $CellContext`t))],
  Editable->False]], "Print",
 CellChangeTimes->{3.40127338541199*^9, 3.4012734585525637`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"v(t) = \"\>", "\[InvisibleSpace]", 
   RowBox[{"3", "-", 
    RowBox[{"3", " ", 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "5"}], " ", "t"}], "/", "2"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"5", " ", "t"}], "/", "2"}]]}], ")"}]}]}]}],
  SequenceForm[
  "v(t) = ", 3 - 3 
   E^(Rational[-5, 2] $CellContext`t) (-1 + 
    E^(Rational[5, 2] $CellContext`t))],
  Editable->False]], "Print",
 CellChangeTimes->{3.40127338541199*^9, 3.401273458570731*^9}]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[{{}, {InsetBox[
     GraphicsBox[{{}, {}, 
       {RGBColor[1, 1, 0.4], Thickness[0.02], LineBox[CompressedData["
1:eJwVz30w1HkABvC1Z49L3ouWoZSul2V0ZUQvnj0VveG83MSctpK3VNQmqemF
ktSmumaTC6VSohC2mrlN1G65vEuvF/ZoabOs3d9awrL33T+eeebzzzPzOEUm
BkfTaTRaAIm+N0bL2mq/RXvTaFIFrSPV23CN3T4Jcy3mxMw0psk4WHYjJ6+K
GYraZfWszO4EbGr6UJnPjEJr9hM2/d1xbIkarb7MTEKjNIu/8ukFdLqdWZPF
TEdNu2OXJe86CmYGZpxj8lFn23RweFEZ0oyrFpxlFsLr5ceD0XuqERZ+s1yp
EkA55HW43qoJJrq0CD/HGrSyfcM///kW84MUMhFPjLMriuPv9n/CoWYv5Ay/
RqqBiW+FswS2AssuA4MWZHr6zOWLu/G4Z01jy6o3OBDdEDEu+4IX8fn24QNv
0cNOG6EaeqH5ebV56d4P4Pq3tSp2fAV97L6F/fx/MdrblZ6skMFDt/1xSFkH
uKpvLndW9qMx3b351WIJVlVvmUHbKcfsS/KxYsV/EBZ5Pu86NQD5qKTr3ZIe
FF1jFRYlD6KwX1hvduELyiTJp7J3KmDmUDQklUoRJ5v2zN99COd3PYopcO5D
gvlyN/GXIYyw70vjTnzFVIF/eb6fEklLxQIbkQxzjkb23ruohKRMzGtz6kfI
urq/ZO+UmIh/m7voiBwin4AwsbUKrF+NpmKEA3hTGWgs8lPB75ZTbIWtAoX5
jGtnjqhwyGHb3O6IIQy6RXm63VHBZeB3kw/+SizqvSSYrFfB2lxpLB5XwjfO
u2B4SAXDmtzyeXkqCHL/jmSYU9jtLvz00yYKbxZcoYW6Uri/IbLg2QgFUbIw
m7GegkZuZ9GXrYZzFN8mmUPhUYjfD/y1w9DOL1HsSKJwYv3pG4K+YfjW/pbY
lEHBqXPjx9unNbg0z51yukphxfJ0PnPpCBqEh47YF1GIcGwZM2gfwcJe18eZ
ArJfn9WzL20UV+JNRbefU8gx5CQFO39HpXdYyt1GCtZdj27Rxd/RO+G9L+49
BcaxtFCv/WM4biHWqiQUaMWc8h6rcYQHrHup+UqBe608y+rFOKxSZe2vhihU
szKNLRMmwL5oaFJMfllu5LqcNNOiWHQ0mKul0OHa2n74qRaTtqsUs+hqPLwO
ZylnEo557LiPP6qxtdzD0mFqErzVynZTEzVST99M6yiZQkrz4EGWuRqlModc
s8062PCWTBRYqRFkaeZZN67D1olgUwMbNc4x6PYdxI40q0S9xWOaSSVx9LOW
Vr09ejpFdhM6XI7149OJ7apKAxOI1XsW2BsS94T6x87U6vDer2ShEXFSzvns
nZNknyP0MSUuPZ96OIW4JC+oUO++1KSILGKLK30MM+KwXRFznxB7CKa/1nv1
CpeyaVM6cB9sCLQg/rGj4WUFcUVmxh/WxGitufeKuDTeuFrvFHEV7zOxidFZ
xxnE/Q9ygxg6HQwSM7r1bj62uzOMOEi7P9aG2Ii7rXYvsaTi2z96s2NCbp8k
1jhtX2xLXBmwctcD4rpfNgzqLfdx2/ycOKtNGDCL2Hn5PLf3xIMs14d6c1i2
VnJiOTvfkkl8dbaJRkfMs5h+QO//AZdne78=
         "]]}, 
       {RGBColor[0, 0, 0.6], Thickness[0.008], LineBox[CompressedData["
1:eJwVzGs01HkAxnHXkmoY1tZMR7lM2dDa00opeTbssSE1aJddTa2iUG5NqU4W
nVSL2WRVRBfU2VqhXGpPiDTbRYMuNLWNa5MmI2b+YzAM//3Ni+d8z+fNYxsR
HxRpoKenF0imq1+k7EXTp0jPo+KDp7fEp3kaebMTelg+6HpeWie14eHby/lF
1awQHLlwgnuPHQf/VnHVRdYu6NWl1IkYv+GnXeMNuSw+DNp7w1aNC9DlctJb
wDoOztMDLXueXMQVq80nMll58FmxzJyOLUe6SbXD76yruC7Jjr12tx6hYcWV
CmUNasFvqogWYS6dHu67uBHxrUebvWw6sJQ7LHuYJcTZu65mDRFvkdzmjvzR
p1j+QGC4N7MbC2qY3fr67eDsl91amdiHO/3eonaPl3hn5eJB5bxHc8zFRWFD
HXh3xiHXLvED1MvWm5XvEyP45bxIsfFHGGjKzBct/Q/sxM5cx1wZ3Ogdd4Ir
JGDsrPlo1vsJouOubY8cezCSyQlsNZFjSY5cc2O4F5bvm9Y8dRqCfLynu/Ob
fjz26/x5xuYzrg7WtTD+eI+yLb7iIfNhMKz/GpFKpTiYfGyLRDqM7OjaqCuc
AbDtX3imnh/B2Hdl0j2pH3Err0XrZa8Af6Ww5suHMlTbC+zovQr0VAizXtgO
YmKsQ2tVq8BUTEfh8iNyhFc5ezeMKuC0YfZMVN0Q7nu/8olwVMK3xHb37QXD
OCtwKx7gKZFsvd2uL3wECWO5f3KylHAe2jpXvEmB/BPuzfwqJSzNFCbCSQVa
ungFM2+UMGosrLQvUqK+iP6UrFEi1rXu7Rx/Ci6Ucds6FoWyjRFX7o9RiH5s
8GzjKgpqOdt84JwKUXHPZo0EUKgN9jXM8xmFTNM5f+EuCqk/ZFyuGRjFL9O9
3MJkCrZdfm9KM9T4UF2wUHWKwtrVx/NYK8cgu2dUpC6gEL64XaP/agyO8So3
/+vkv0XQn5A+Do/zgtNBdyjkG/H4QZwJOGYHvrFvpmDZXVtiIJyAzYOMf0pF
FIxT0kPcEzU40LC75fNrCno3eJX9FpN4Yv712oIeCkkXKgUWzZMwDAzLtpRR
aHA6ZcKMm4Lt/u8tckYoMP2SnI8xtIibuOffo6YgWfH81eF6LX50VfXO01K4
dQkcKW8aidasTRv0VdhW6ca0nplG2KOtO/uMVUjLKE6X/D2Dxv1922JMVSiX
WRcyAmhEJBlWejBU4DIZax5P0uidb/Raw1Qh09hgkYQYogSvSWKhRj2tIH7y
laRCZ7f+rofsKRp76itOThGzq8s3xxHvCPR2nybuD9m020pLw8HWsUjPQgV+
fva5ndM08ouHd8wmLs9OO3yIWLrCpVXngTR+uIBYHhrnbkIcGh1ud5d4dZaM
OYd4/VrnCtMZGht/FTWbEs+SPPv3NnGAmMthEON54/VHxFzu0RydDwmrs94R
h8Zc0+o8eLOQa0zTSDms6jAjbkuJ7QolLpmXmsEknp20vWkfsWvqJaXO30UF
lx4jHsyp32ZBXBW4LvomseTM6CpLYrmXS8ADYpt0RonOnNX2Lq+JL5g5ML4g
5jktsJATB633PKLz+SVz1TSxv2nIgM7/A06Qf4A=
         "]]}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      AxesLabel->{
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"t\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm], 
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"x\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm]},
      AxesOrigin->{0, 0},
      Background->GrayLevel[0.6],
      PlotRange->{{0, 2}, {0., 1.1928953641918578`}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {189., -118.66252583997979`}, 
     ImageScaled[{0.5, 0.5}], {360., 222.4922359499621}], InsetBox[
     GraphicsBox[{{}, {}, 
       {RGBColor[1, 1, 0.4], Thickness[0.02], LineBox[CompressedData["
1:eJwVxw0w1HkAxnG70bo0WE3HbsmoraZi1HLkrWe79IZyolbZqJDYJB1Tuumk
KEJTmzp3UpRe5K2sl6YSaeNS3vKykaV2N2R3JFnysv73u5nnO898LA9E7gih
6+jobCf9/+4hAy3VX0LW5Ws72ymKwdPdwD7ay3LDb7HaOP4Yg2d7M+O6mOWL
7BzlHLGawfNokJZksYKRwLapjOlk8PjB45UiVjTcsuYKV4gZPJnN+Q1prATI
QvLubgll8LLne527wEqHiTbkd8G72bx4ffHyZFYuMozfmCSJ9Xh+u3OKh7+V
4uyI4V7jmFk8AypesHlRFaoN2NFCEY231Hto4GWKBELnXVp+A4XjjY7IGH0N
poWjHTN5GqalzB4arQl8R+v2TfUTKJdveNvk8g7cF7S5dQvHUROetWC3ug39
R3o8jF+MQrPM1agwQgrbwMmTV1+NgD6Rb7xgaRdS933pVLUPw57aV+5T1I0Q
ToWRInwIbxPsGmtX9kJ9l9MQl6SCxSXVRN7QRxQKRhb6lA9ANd7b075ajkCt
6LOA34fcwaf1hhcVaGreVcFgKGFofu+rUqnE9BNlrqv2E1LDyg5mc/pwdPdY
8tVNHzHGy1ceiuuHefn4MqmtDNFcSenPLwfw77Llc0KDutBbJElpsRxEi7vD
zJdcKabC2zJXnFQhMvhbqq9TO1atZ8wcfKrGx1mKgv3cVmy+ZRn6yHQI5dK6
KbF9C46bBy7+JPiK++utMrzMmmCl3mkg3TYMrVmR0przFvOMhvUlk8Oo7LL4
Q8F7Dd2qzOIl179Bv0O4Nb21FkK7p50/eYzgjsJvNDJGgvytB7Kfj42g5fEe
x0uBNdCo2MZ9177DxfqQS+2NapT5bJ6V7jaKO8bhZSXNlYjbkniztG8UKzX7
M/u+PoGlzP397UQNHkR1LS858BhODgnpLO4Ydt53l5UqyiBY1DRBax3D4Jq2
pNv1Ymjq0+RH48fR7NefGPb8ETJ0A6J3cH7A086eq9V7iHk9Zbfokh+I2OKt
FlAF0DsV7+sYNYH690/o+3UfQCcvoFhuMgnvBP4e7tl7OPZPcZpJzSR2TPLr
ZMw7qFyVpM88MoUQlysXrVS3wHQ/ZnXGcBobP9AdNy3MQbd1c2vss2m40qPY
woc3QMZRBmgRHHFD+vxxJvYW2zPNZ7TgJHcm+5v+jdOJOfHdD2ZgoQnw6Ei4
hsIB80xDTwq2Jnm0IP90eDMN19ZNUsj9vLiADxEu6NEXdBP3qqvS2cSSCY12
mJhGW/GnbJ0I9nLZS/YUhfNugT5BxGxxodcR4vZ+f0akqwhy322h86cp7GJf
uXrOWYTojNRrQVoKJ/Q+u5c7iFCYejr2BPHh+TOescR9p6MFacQWHXIvF2K/
MMHiCmIEGPnV2Ivg6mRVNGeGwrO1o1GNv4gwu/vNq0fEKZcH3/TZioDmqvu1
xFKfyq484hMSccoH4oYM/8HDxIMFmd56FAXuzUmD71wRGk8JZX7ETttq/Kg1
IjCOBVZHEK9pG46oIeYd9Ll9hriudvBsInHJduewAmJPBa/EgFj1q43nC+JY
RlZ942oROA5LbDqIq1IkisvEAatMTVTEzhEVWl/ivywMNBQxMy/GzIz4P9T4
d58=
         "]]}, 
       {RGBColor[0, 0, 0.6], Thickness[0.008], LineBox[CompressedData["
1:eJwVz31QzAkYB/AtZddl0uaS3SRpnSMut7qE+P7IXKQUm7llEoNy1eQ60011
U2NLOSFH87v0IlvJeell1baLkrZaRPRCr/Rmb/W20UZbdmvb++0fzzzzme/z
PDOP49Hf9gWb0mi0PVQZu3fwUItsOHhrpnS43WCgE2ae7Mg+1g40BSY/+GeS
TqwXZlwTswLQZ5G62+Ujndj9qqMsh3Ucbfk2goi3dOKX41NVaawoTPsKdTYS
OtHj8pdnKisJ1mKLCZtwOpFr43f2PIvEpsSgefVdc4kEhnhlCqsA8tmMhbdq
zAn+gTyRerwc89s/lsb9PYewMCQEei2txiuDu5NKYkKs2PtpqO6CHLlnoeIw
aUR040ZkTDzHjhPP4uOGZ2Bbzuw1MWkCM7InX8DVQarwfNnk8RqFnCeuLeen
UBuWY3dgtBUlKXuek1wNNN9tWVAc0QGWVlRT5v4FptpCK7sVbyHk/lHvvWsc
boYjUl5JN/iMKo9OvzG8THJtfLq6D0UFBY9mQ0fhcFmlvfOpHz719p/PpQ1D
NdXX27ZOAasnI0uS1g6iYKTyheWl/8CXvvogalPC0v7WmFKphH/bQ7euOgUu
hkpCcjkDIL7XKLaZv8ckUaj89fQgFv+4aNlRYS+iuPLyRXVDUGfF/ct4/A59
JfILLY4j8D9Y4sD52InpsNbsVX+qEMnYdiQ7sh3O2+izIZWjOHew2m8VvxVe
+Y4nSm0/YV1Mr1jv/RrR9oeXvw8cQ+7h7R5pl5uxZnS/RYevGsviEt70WzZi
4QI1Q65TY11RutmS+Q0wq84WOV0bB1FDf5CXWI9w18quebs/w//blEEe/SkK
dx3NfTz5GbLweieP9jpoVGyrgfQvsDpiF8PT10DC85pD7piAOnanaukPMpze
mSwsH5hAf2ina6VfFRx7vDtvJGtAKKdXmmVVYNOGJJLFnUR/w3ahcPw+Apc2
aU3eTILImwxFuQSaF6mKyIQp5Ab1X2GdESPDLChqH+crBKyK9SF3S7GwV5Jv
Kv+K/tcs5/C592AenxCw8XctBA1k9b22ItDuBIkU1jrqn6wW+2V3cSpLlGpd
q4Psui1HGXsLVc7nGMyT05CZ+3jrlt8E0/vUmkTLGcgOpe13X30D3Wub38Q+
mgHtpkGs3ZyHe9fBUQbpIcj0bXbZIMQhkRvTflYPGuP4iCPvGgTJeQndd2ch
uGPLS47PRPGQfbaljwGCxM4ca/ZV7GVauj/TGUALiN7yto7EeXNTu27KAr4g
Np2yXKvRq41ODJHuo+ym6KljT1PGC25DLQm2uNjvJGWa2dhPj2pIKAJ8T9jM
ULlXYfD1ahJRGRfTj+mp3ORd2LFKEsUXBbExlAX+Z2odKA8IogJTjY5g2HVX
kOCHBi6/b5zX324MoLxl05qSb2apnOtJ/PyQxNzuhiellGmRXT6r7pNAc/Xt
p0av9S37ICURIxdfeGd0Ss7ifMojRdl7zQ3UPrN1kE25MT68h0+Zxs64PF9C
gn7qsCzCaIF6pr6cBBHCu5Fo9MkVYcmUy/ZsDi0yuoPjNSsmodru4lNjtM+Y
tIIyZ4OTS7vx/tb0ldGUg5xtrVVGF7Iy11O+6mChMRjnr8RaqMtI/A93UHRB

         "]]}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      AxesLabel->{
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"t\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm], 
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"v\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm]},
      AxesOrigin->{0, 0},
      Background->GrayLevel[0.6],
      PlotRange->{{0, 2}, {0., 2.9999996938775664`}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {189., -355.98757751993935`}, 
     ImageScaled[{0.5, 0.5}], {360., 222.4922359499621}]}, {}},
  ContentSelectable->True,
  PlotRangePadding->{6, 5}]], "Output",
 CellChangeTimes->{3.40127338548843*^9, 3.40127345863558*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["(C) Free Fall with Viscous Damping",
 FontSize->16]], "Section"],

Cell["\<\
\tNow combine free fall with viscous damping.  The force function becomes:\
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "(", 
     RowBox[{"x", ",", "v", ",", "t"}], ")"}], "=", 
    RowBox[{
     RowBox[{"m", " ", "g"}], "-", 
     RowBox[{"b", " ", "v"}]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "Once again, below is a repeat of the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " code for free fall.  Modify it to change the force function to this and \
change the initial conditions and other parameters to"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["t", "0"], "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "0"], "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["v", "0"], "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "=", 
    RowBox[{"2", " ", "kg"}]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "=", 
    RowBox[{"9.80", " ", 
     StyleBox[
      FractionBox[
       StyleBox["m",
        FontSlant->"Plain"], 
       SuperscriptBox[
        StyleBox["s",
         FontSlant->"Plain"], "2"]],
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "=", 
    RowBox[{"5", " ", 
     StyleBox[
      FractionBox[
       StyleBox["kg",
        FontSlant->"Plain"], 
       StyleBox["s",
        FontSlant->"Plain"]],
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "0", "]"}], "=", "0"}], ";", 
  RowBox[{
   RowBox[{"x", "[", "0", "]"}], "=", "0"}], ";", 
  RowBox[{
   RowBox[{"v", "[", "0", "]"}], "=", "0"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "final", "]"}], "=", "2"}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", "0.02"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"m", "=", "2"}], ";", 
  RowBox[{"g", "=", "9.8"}], ";", 
  RowBox[{"b", "=", "5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"F", "[", 
   RowBox[{"x_", ",", "v_", ",", "t_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"m", " ", "g"}], "-", 
   RowBox[{
   "b", " ", "v"}]}]}], "\[IndentingNewLine]", "CalculateAndGraph"}], "Input",\

 CellChangeTimes->{
  3.3997186413508787`*^9, {3.4012734662632933`*^9, 3.4012734914977818`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"x(t) = \"\>", "\[InvisibleSpace]", 
   SuperscriptBox["\[ExponentialE]", 
    RowBox[{
     RowBox[{"-", "2.5`"}], " ", "t"}]], " ", 
   RowBox[{"(", 
    RowBox[{"1.5679999999999998`", "\[InvisibleSpace]", "-", 
     RowBox[{"1.568`", " ", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"2.5`", " ", "t"}]]}], "+", 
     RowBox[{"3.92`", " ", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"2.5`", " ", "t"}]], " ", "t"}]}], ")"}]}],
  SequenceForm[
  "x(t) = ", 
   E^((-2.5) $CellContext`t) (1.5679999999999998` - 1.568 
    E^(2.5 $CellContext`t) + 3.92 E^(2.5 $CellContext`t) $CellContext`t)],
  Editable->False]], "Print",
 CellChangeTimes->{3.4012733939514933`*^9, 3.401273497766822*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"\<\"v(t) = \"\>", "\[InvisibleSpace]", 
    RowBox[{"-", "2.5`"}], " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{"-", "2.5`"}], " ", "t"}]], " ", 
    RowBox[{"(", 
     RowBox[{"1.5679999999999998`", "\[InvisibleSpace]", "-", 
      RowBox[{"1.568`", " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"2.5`", " ", "t"}]]}], "+", 
      RowBox[{"3.92`", " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"2.5`", " ", "t"}]], " ", "t"}]}], ")"}]}], "+", 
   RowBox[{
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{"-", "2.5`"}], " ", "t"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"0.`", " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"2.5`", " ", "t"}]]}], "+", 
      RowBox[{"9.8`", " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"2.5`", " ", "t"}]], " ", "t"}]}], ")"}]}]}],
  SequenceForm[
  "v(t) = ", (-2.5) 
    E^((-2.5) $CellContext`t) (1.5679999999999998` - 1.568 
     E^(2.5 $CellContext`t) + 3.92 E^(2.5 $CellContext`t) $CellContext`t) + 
   E^((-2.5) $CellContext`t) (0. E^(2.5 $CellContext`t) + 
     9.8 E^(2.5 $CellContext`t) $CellContext`t)],
  Editable->False]], "Print",
 CellChangeTimes->{3.4012733939514933`*^9, 3.401273497770752*^9}]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[{{}, {InsetBox[
     GraphicsBox[{{}, {}, 
       {RGBColor[1, 1, 0.4], Thickness[0.02], LineBox[CompressedData["
1:eJwV0Xk4lAsbBnCRjJQt5YwMlaVtslSI0v2KTkXqCEVJCxGVKCk61sguCglF
jLJNtoYivRODU0paVbYakuyVKft87/fHc93X78/7fpYePb3nmKiIiMht6v6f
5sd6X3G/H9t8yemBfzZX9/FsUyXPTroZps62PR2NlSHXZaSkl9FtsN9CPGxT
lArZrawefF7TEWskEk0NNDRJixctpTfpzthY9q5h7BKTVI9h5AhV3UDOf1j8
VEeXnDJ3un5E0wP8DzteLe5cR/aFvuUdSfZC89lSaSlLfXKf85/qq3Rv+CjS
Sov8NpA8U6WK2nQfFE4dmr3suBGpo2ZcPKrqCz2Hl7v8Fm4i00UP52lkX4TQ
48Sn5hBjksYPydqrGYCfp3aMGBdvJis/v35RaxOEtoSAjx9vgvT8x9ncITkY
TyTjT3doEKQmV1A/2hKCwaSlOUIHgmzXDjeNpYeiLXKmxMGbIBMz/uJqHAjD
dHBJ5ayTBGkuk7/pcfplVCrVdU2YEqRI4MaHezvCMetLeHbuGMjyoed6w6qR
CIDXnvWrQJ50dCwNPxKFhsgDjBLlzeSypmGtJdnR4IwdsDRo3kR+MA4ueNAd
A5W68NWdZhvJOLb8CivNOPQsHxV54mdIbmWwWN9dr6DTvnyRwMuADKhPlX1i
E4++a12ca6v0SPuArl2xgnh8lk5YzmGtJdfrM2PtkxOgJ5Hr1sXVJmWGvBs1
DK6i1/aSZW3VSrIvp1ryZ8tVXLvYEWNkpEZmLtx9OYqeCE/zsdS6uWLkxRfX
eXsrEyFSZ/bG4/r45r1hn0XVDiRhyc7GIZftipgn8AqsSk9G8RLDf4fH1uBb
YWV1uPF1ELbth0SH1qHGWWzKuuM6PriulxIqG+LC28QLA6o3YLdjBduHvgXW
Me3lD7g3kJIu0GuZtxVaZpqC0COpEMlx0uWqbEd3WYUXIzsNQU5/Bidu7oLV
tU/uuzRvodDYnTwUuhdMC7U8pYZb6FhDNoWftwNN7OS3HtcMDM0xCqv13Y9D
D2dKSJtM8Ixibh0scMTClfFadsm3UTg/3MTwixNEw+5bPu65DX3+zNf6Rccw
8vnDSXWDLNidGVWot3ZBY8rSgpGWLPBXNA3O6z6OYFrZ8kg6C8xcr2jXgx5Y
tS0hotSThbNb7VaHD57G67DT31sbWBAJ0ulVrPWEuhizQMsnB+uHj/Dzjp9B
/TRL6+3rO7gjSLO3/HYOpzddujK98i68dW1l77zywV8Xj4xoBt/Ffe9lXibc
83AbY5T6aufi3+iUlg35vpj3K1lPNSYPrbt31/8u9gdH91zy9q48dCqLO+hW
B8DR0/qPl1E+jL7O/57xIhBFgzIPeb35+DXS2hAxHYQ9vRGb3LcW4uhId5Ok
2SWktPuZcmbuIUzi0Cyf9REwUbZnddgWIU5CrLe6NQJ9+w3EaewiTCY3M+XC
IqEy4OvEXlOMZT8I6/ftUTjtpq8h0laMUA9+ds3dWMg7F+XmbCjFrvLafu2m
BNjZ3y4a+XEfK3xMqfFSMNxqMzdLiYMeo8N50oMpuOxIO2ZtysHbFeQUx/4G
ypxOK5UncpA2UhoUrZcKaQ/jsIsG5QjbEhC1/U8a6kI+7p/jXwFu6GxpMisD
awvl5ijTKpHJ6k9Pb8zGU626wy90KuEx1+XvYmkWDpdcqAqwr8Tv0rrUVXtY
iC3v9PySX4krS2Uz93xioZfLbr1jWYXEZ80ZJ4ZzcOvdjhLda4+wcHwy+iYz
F1LCYIdtKiSuDmx26vhRiBS3fY7Mv0nMVWnu+wo21N8yD8t5kKgNqDO0iGNj
c16LU2s1iddJA6lazHt4mJfMqdDiYio+9ZjNySIMWg88smjiYu+D0cnpmWLY
5l5vPDu/Bt7tBe2tpWXQsBrqrY3mwSpJNck74yGUArcZud7moWBfiGr3u4eQ
ZWdGz63gYRtteVLYvEpM0Ky1rPg8tGVeGzzoV4lmbsXZjg11kM8NFz1qVwU/
neCZsa91uPTa3DCHUY0mGQUFLaIBNXcyF5m4cHG+yRApo09xIiJfJeQAD4r3
5TpmzXoJT82ntFUFz1HON33+ctNrBKvvLcs+9wY17jcX2w+8hYvne8Vb0h8h
0DSWYZ9qgaXu3cE7VR0QHS+QXazxCYmCgX8e5fOhLzxcbn2vDWt0Qno+Dn/F
89D1TfWrOpEsaZniYPkdqvH943lDn/FSp6rmKGMQ/X86O97p8HE5riAx3HUE
rL6qZ9JxXTgV7BSp5/sT0oy7w93d3RD79V99U/woYtw4LpnqPSjyrRz6EP8b
v4mC7uOB39B/Zl9B6H9j8F7Lu7+othes72dLnnVPoPMeL/rV0j6ERDN0PTZM
Y9L9bdpKv358/aG4NjRKiNUmEjMuVQMYFk66K3WIENuylrqWKA5BfLGE/xZ5
UeI849CyLw7DSKowYyzQEyOYA7ZSLZYjcHi5P+K5zWxigcwIjTcxQu2l8E/S
FXFiNplWpJb+A0/GouwSK+YQJ9ZXfZS0+IkvWz89TpyQIAp2HM18/Psn8qpz
3IsMJQlBv5JsT/IvbJnQNh50nktwrLeJJZqNQrM90mR2gRQRuD0s434P1fuA
RSC3bR6xtN38Q3aYABIXjLVTJKUJI4PQRPra3yixDHi/xkqGcFB5OT7rzW9c
NmVkd5vIEoJnsXzP4D/Icpl/Pq5XlkiZ7ei9R30MCrPUdyWFyhELOjhZorwx
VJ/S+dLFkCfE/YNtDL3GsfL5M43LPHlCJM+xiC8/gQs9H6JtDy8gzqQWxcrX
TOBej22Dj6QCUb06gibnMYn9fsPTvCIFQs78DDNEegpLgob+U7ZYSLStaX7j
+2gKmezyVU9+LSSKb0G923Eaui/6Wx0TFhEHi/TlGDPTSD33MY3GVCSCwm4H
t+XPwHe5j/PkG0WC3ctIk94phN3Toz2vPf8irOSkNzRMCJGs6MP+qUQnosRF
F7dRFvOJFpdYTCd444LpEcqKuH1wMWV9fnut0iT1Z7mm+WaUlcrYuz0od95n
eiRR5ttYui6cEkJ9fFprgzKd8E6JSXaaFuKm+0ixP4NOsGOCfC9QHlZaIHmV
ck+Qt0Ms5Qo9/SN3KNu5OSyroJyT7y/3krKxEfPe3BkhGhPkzixRoRNz2hrr
Sij3WVuuq6WMZjK3nvJAvk90C+ULvLLoVsqWJzO7+in3FaZZiQuFUNMSXFNQ
pRNN/ifa7SiPsbNHj1GWOHOIe4pyzb7mnX6UCRfr7BDKHSemWXGUS3dtdCuk
THtjZ1tBuX+L9s4nlOU1w9mNlNUN1LTfUz7exRH/TNlxtaJ8P2VJqa8HRylf
V5USCCn/Slcopy2hE/8DVoDnOw==
         "]]}, 
       {RGBColor[0, 0, 0.6], Thickness[0.008], LineBox[CompressedData["
1:eJwV0Xk01YkbBnB715LlIl1rsqQISZHK843CMKYRWSJLGKERMiM5ka0QLQhF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         "]]}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      AxesLabel->{
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"t\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm], 
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"x\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm]},
      AxesOrigin->{0, 0},
      Background->GrayLevel[0.6],
      PlotRange->{{0, 2}, {-0.0005021526921960523, 6.282564941972638}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {189., -118.66252583997979`}, 
     ImageScaled[{0.5, 0.5}], {360., 222.4922359499621}], InsetBox[
     GraphicsBox[{{}, {}, 
       {RGBColor[1, 1, 0.4], Thickness[0.02], LineBox[CompressedData["
1:eJwVx3881HccwPHz43Ic369j6M6ou2SbH9PkIbXp/Ql7yO8pPVKPIxUalkoe
j4vmVx0pd9nMzw5d1Eri5NdWLSy3fu1Goh+bc+oy5ORH6+To3D7+eD1ejyd7
36FtcfoUCiUUt/zAuPG+rtdxmykUsfbYadFmQ1/W4WGmH7xIY2TyvNNg/fny
yhZmBIjNBzzcvE5D0F/PmquYsTB/ys1E7VoBO2Pf3y5ipkKx54N1N5h1MOR2
ylfI5AM7RjZYNPsriK3C8s4wi+FVoEeMb819yKG1fHKaeRE4zbZLqUbPIXLX
BcnMbCskLlZ8798xBnRdDtffvhM44zmco7IZWBs+Nd5dIIW0c4ddgp3UwOvZ
COXvHsC/Kp437ZAGbFoZCj29Xujpt94dSdVCu9JX1vvVY1DVfu6e70NBdxKr
bHdNDgCxYu9utUwPqR29yYaDz+B1XTzt5EoDpK+pN7dd+w90OD3fEsk1RJ66
mPbtjXIIvxxZeOQ7KpLxPXruOg3DR/sNeD9HrUCrflBp6qZewGK+O78wwAip
3g8rnqxTgnG6XtsTkoYuTtx6SJx9BYqEOX5EHw0RdpenR0ZGQOatoN4/bowE
CW3xYodRuBrVn2hJN0FzqH7k26wxsKqTO9qWmaBUd2mrdfc4TGt9zO+Z0tFw
o7Sgjz0B4p21KfwsOlpMHBB9lq6CjzOdsoQjdOS8xWgp/tYkHNCEBqRvNEX+
NewD122mYL03yzBeYIp4dns4L7nTIKqtjWU8NkUukzvoz0JmIKl8sKSPNEOW
5AxNujADRVNxTV8EmyHDTpFkTeUswJtt7tOZZijJ49bfxkFv4YSX8NyhJjNU
H7BP3DH3FkoG5auN5WZIrWKZj5b+B2PzBb0SCoHatvsbFPu9AzhS/TjDgUBZ
W3PPt46+A/ugxN6ffAjEHgp8XpurBi9u/gJEEWjTBn4x030OOMxgwU4egbj2
vRq9/jmw7rZp/lpIIPVDofJwznuQ3Dta2VBDoHLD6NRtDvMwNBSUXddKIEtF
W42+dB6SRRx+hZRA1IyciI1HNDDNst/rOkAgSl20RGmxACE7LMV+LwmUck4i
tLizACrBfUPjKQLdds6nMZIXoeXSzRB/DYEYgSkuJ4gPABNvbtzUJ5Hc9VF/
2m8fYEJpl6umk6ipGhxGorWQdypB1m5JoiiJJ8NuSQv7fUZLSlkkys69kCO/
ugTNW5X8MDaJGsbtRESwDkKpBQq1I4nCGYTXvQUd9Cu4bmkuJDpD1beVY2tl
Br3LlmrU2hnssUtXktOxPZVD3axFHaxcOdlwHJvV0hCWjN25tMslE1sZEXLA
6oMObg72fXoSO7VcULpfqwPS2H21ELtBkJ12DLsrqrRz2aPZqVwhNrt6Pvos
dmQCl/MLtoWyvaoQ23uTS6PJkg7yeCzbIuwV8j//uI69ZTjDqgwbHnVeuYud
/barddnHpC0Fg9hjs3oR5dgT10ThVJ0OQtoyiiqwezKShiKxvwnjkpXYRil7
ug5iX5GebVw2it9eewI71rEzpAq7OfTLhGvYzhKmoBpb5eMW/Du231Nf5/PY
DhvWuD3FfvE66eGyo51tLFTYRhM/Joixy1bR1TrsrqdttAvY/wOIUEjb
         "]]}, 
       {RGBColor[0, 0, 0.6], Thickness[0.008], LineBox[CompressedData["
1:eJwVzX8003scx/HZxTab7zZddTdXTCoiXNehXHp/xE2JXL/u5STk+hGdUtq9
6V5hDiUsnQ5FuCld1RWrqHSpqVaJiHC5RX60O6LWlPk5vvfjj9d5nsdfL0Fk
gn80lUKh7MBbqlf0aEfju+hN1SnMjE895zZpu/MPDPA8oLvotLLvXBJ8e76w
pIYXCCGHb8s7C7Jge2vPzVJeFBhJKgTLcwvhp6jpe6d5QjD7NWimI+kK9Nse
dxfzMuDt6+zeIv86KDP0PZbNy4c6X7cwAa0JRPSatSd4l0DqorDoPdQDwSEX
JKqJWtBzKM+3dhgBJikK9VwphVd1NzVVCSpY7accfZQjA/tXTkU+9ZNwuG0j
FE4+g8ZdPnHMiRlYUct9o6X1ApLNPptaHNXA7WH35y9cXsLUiHekp5qEh/Gl
RiHvu6AC8ta1pGsh9RpXdtW+HhDF3n3s00pF1NlKjtHqV3BwPYs/w9FGjmTE
7YDqPpD/0HmmQqCDnmc4tD1ZNwBcZgepWaaLTE6Nz15VDoJJ1O+cMi0aGp8e
eNNtNwyedjbNg89o6NJYfTNx8i1Iw0uSX4rpiDC+/FEul8NBaanJiDMD5cbd
iikzV0DgcQWr+TkDTaFK+Z7UEdB8meL49049JLSX1S5/NAo2hdPIsksPDVTL
cjoEY+Cfrmszuo2J5uO7ii1/G4e51oKWgRtMZOVGW4ypfw8yTkh4CoOFPC8K
Ym+sUEIUoynl+3AWOmwcbjYU+hHsXcuzsy6zkPX7IGaPjwoeetXU3h9hoWVs
FV02pwLTrQfeXV+jj7SlxZJVJROwR3E8oH6nPtrrUP8vY/sn8H7Kjy/N00eV
2yLL7k99Aue2psWEe/pIPc7nKM58hlNJqROEXB/dCvD8It9jEpBlls5bBoFS
t2aer1VMQpRT5eQDSwIJ+r16yzPVEGFjfNZwC4GcnTLyefZTYLq4fap/N4FC
V76Y1eqcgplicYXnEQKpm8XDB0TTkKzkZnxzikCF2mFCf/MZoNNc2lPKCbTs
za2LVNkMsILqqH/WEkjnqChw48FZKBs1DTj0mECUq2GSYYM5aBflxZZ0Eijx
nERs8HAOeqP57r8MEeieVRadu38evvbxLZB8IBDXK9E6ndBAhG1K08I0gfrW
t3ceadBAcI02w5DKRtf/AHN52AKwzm8OKNBjo10SR67x4gI0pCYGrTJgo7TM
C6K+vxYhqfzHohweG1WNGhcT3iSoQt0ynpmwkR+X2PB0joTGtXn98tVslK1D
NerDFlpdPvkftmxWvaDCRh8aQIHtONz/iD9Pgl3l0IURbH5Nle9+bA6XEzOG
PRzoE2uoISEth6L8iC0szD3z8wIJwfvtNHPYVblpR5KwIzZTquaxFWnCUDF2
VnfrLg12cFyo2R3shg2R0gVsV2frar1F/D+ZkE5Zw0a6fS2Pb2CztpjQdbGh
XXrlCXaEfkvdkpNkNTmvsb/KF8bRsMeuFfvpkCTQBxub6dhtR/f2B2OnJW4S
M7FpieGN+7CtHXtdWdgoJqA8HTv3boJyyTd3fBd3DdubPOtLYI9vtvV+gN31
wILCxjZ3WmX7D7apx53rSw6zWmEwjr31mPtuDvZZE6aaxPbIauVysf8HGSVG
3Q==
         "]]}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      AxesLabel->{
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"t\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm], 
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"v\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm]},
      AxesOrigin->{0, 0},
      Background->GrayLevel[0.6],
      PlotRange->{{0, 2}, {0., 3.896791523026735}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {189., -355.98757751993935`}, 
     ImageScaled[{0.5, 0.5}], {360., 222.4922359499621}]}, {}},
  ContentSelectable->True,
  PlotRangePadding->{6, 5}]], "Output",
 CellChangeTimes->{3.401273394027603*^9, 3.40127349782544*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["(D) Free Fall with Quadratic Damping",
 FontSize->16]], "Section"],

Cell[TextData[{
 "\tFor free fall at sufficiently small speeds viscous damping is an accurate \
model.  As speeds increase and the flow of air becomes turbulent, the \
resistive force becomes quadratic.  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "=", 
    RowBox[{
     RowBox[{"-", "c"}], " ", 
     SuperscriptBox["v", "2"]}]}], TraditionalForm]]],
 ".  Combining quadratic friction with free fall we get:"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "(", 
     RowBox[{"x", ",", "v", ",", "t"}], ")"}], "=", 
    RowBox[{
     RowBox[{"m", " ", "g"}], "-", 
     RowBox[{"c", " ", 
      SuperscriptBox["v", "2"]}]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "Once again, below is a repeat of the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " code for free fall.  Modify it to change the force function to this and \
change the initial conditions and other parameters to"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["t", "0"], "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "0"], "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["v", "0"], "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "=", 
    RowBox[{"2", " ", "kg"}]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "=", 
    RowBox[{"9.80", " ", 
     StyleBox[
      FractionBox[
       StyleBox["m",
        FontSlant->"Plain"], 
       SuperscriptBox[
        StyleBox["s",
         FontSlant->"Plain"], "2"]],
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "=", 
    RowBox[{"0.5", " ", 
     StyleBox[
      FractionBox[
       StyleBox["kg",
        FontSlant->"Plain"], 
       StyleBox["m",
        FontSlant->"Plain"]],
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "Note that solving this differential equation causes ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " some anxiety, demonstarted by warnings in red, but it gives the correct \
result"
}], "Text",
 CellChangeTimes->{{3.399720302682769*^9, 3.3997203078263407`*^9}, {
  3.399720361834365*^9, 3.3997203970424423`*^9}, {3.399720455158101*^9, 
  3.399720472162797*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "0", "]"}], "=", "0"}], ";", 
  RowBox[{
   RowBox[{"x", "[", "0", "]"}], "=", "0"}], ";", 
  RowBox[{
   RowBox[{"v", "[", "0", "]"}], "=", "0"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "final", "]"}], "=", "2"}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", "0.02"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"m", "=", "2"}], ";", 
  RowBox[{"g", "=", "9.8"}], ";", 
  RowBox[{"c", "=", ".5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"F", "[", 
   RowBox[{"x_", ",", "v_", ",", "t_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"m", " ", "g"}], "-", 
   RowBox[{"c", " ", 
    RowBox[{
    "v", "^", 
     "2"}]}]}]}], "\[IndentingNewLine]", "CalculateAndGraph"}], "Input",
 CellChangeTimes->{
  3.3997186413508787`*^9, {3.401133696970508*^9, 3.4011337023087893`*^9}, {
   3.401133760203377*^9, 3.40113376365164*^9}, {3.401134694262236*^9, 
   3.4011347184917583`*^9}, {3.401134777605248*^9, 3.4011347781424923`*^9}, {
   3.4011348456729527`*^9, 3.4011348556073923`*^9}, {3.401273516588118*^9, 
   3.401273533707345*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"\<\"x(t) = \"\>", "\[InvisibleSpace]", 
    RowBox[{"-", "2.772588722239781`"}]}], "-", 
   RowBox[{"6.260990336999411`", " ", "t"}], "+", 
   RowBox[{"4.`", " ", 
    RowBox[{"Log", "[", 
     RowBox[{"1.`", "\[InvisibleSpace]", "+", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"3.1304951684997055`", " ", "t"}]]}], "]"}]}]}],
  SequenceForm[
  "x(t) = ", -2.772588722239781 - 6.260990336999411 $CellContext`t + 
   4. Log[1. + E^(3.1304951684997055` $CellContext`t)]],
  Editable->False]], "Print",
 CellChangeTimes->{3.401273535658963*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"\<\"v(t) = \"\>", "\[InvisibleSpace]", 
    RowBox[{"-", "6.260990336999411`"}]}], "+", 
   FractionBox[
    RowBox[{"12.521980673998822`", " ", 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"3.1304951684997055`", " ", "t"}]]}], 
    RowBox[{"1.`", "\[InvisibleSpace]", "+", 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"3.1304951684997055`", " ", "t"}]]}]]}],
  SequenceForm[
  "v(t) = ", -6.260990336999411 + 
   12.521980673998822` E^(3.1304951684997055` $CellContext`t)/(1. + 
    E^(3.1304951684997055` $CellContext`t))],
  Editable->False]], "Print",
 CellChangeTimes->{3.4012735356764317`*^9}]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[{{}, {InsetBox[
     GraphicsBox[{{}, {}, 
       {RGBColor[1, 1, 0.4], Thickness[0.02], LineBox[CompressedData["
1:eJwVx3c8lgsDh3EjpWWPqCdbR0lWSPjdkmRERkJ6IlRURinhyIiUUXIchEpG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         "]]}, 
       {RGBColor[0, 0, 0.6], Thickness[0.008], LineBox[CompressedData["
1:eJwd1Hk41NsfB/BvKs2l7BIZ0lhahFuiFJ8vKoUWTKGYlO1SsuSGFoaQPbfc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         "]]}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      AxesLabel->{
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"t\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm], 
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"x\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm]},
      AxesOrigin->{0, 0},
      Background->GrayLevel[0.6],
      PlotRange->{{0, 2}, {-0.0004900726398541472, 9.757021831009729}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {189., -118.66252583997979`}, 
     ImageScaled[{0.5, 0.5}], {360., 222.4922359499621}], InsetBox[
     GraphicsBox[{{}, {}, 
       {RGBColor[1, 1, 0.4], Thickness[0.02], LineBox[CompressedData["
1:eJwVx3s0lGkcwPFhKUs7GWHeGWSV6uRySCLC72m72CIlFdpJCK1LatGWjaII
oW13VXNcBtHpIlmZUiGVN22tdb+0rjUsw8RomDEu491n/vie7/mYBJ7cH6xK
o9E8cMrvDha2vBwNdqHRJpM6OrNd1LaxTw2wtgO6X9lbxzsLG/O5uRWsA/Dy
053uYl4quP3T9SiPFQQFNuJAPo8L3kEzNb+zYmA6xLtDzLsLfVYp2zJZSdBp
XOrrlv8UCvT2Xr7CygLWofgIj/y/IFGjYl0aqxhmrz9NCMj9AD6+hWWTX/gg
Xr/pXP61UdCiEjmuK2shoX1O9JtAAms8J4R16SSQeRGyuho5nGl0AO70O0jT
6LL8VrgITD6jX0WlCXjDCRt3OaiiJ4JtDU1OrXA5jlLPYqqj12F5Br6f20FS
Ky00fL0USdc6Ly890QXy9WZcA7YmUp0t0TZY0w2BqbYntc4tQ3aU/xOvh73Q
PVexvTCdjhqSbBvrzQYgdrTU0NpOGxlfE83em/gIdVvEEV5rGUg0M9DfYS2A
oh4xmTzFQMVjVe/pVwfhbu1z3Q0vdBDd6I54aGgIrCJ6HM2jVqCM0MchBabD
4F0d08rQ1EUyVDL044URaD4cpHKTp4tibEi+fp0QLN7Vq+kZ6KGBh2R6i8kY
pEYax7X+oYfmw9pz1v8iAi2H6qs7FXrIfOvSxZCqz3By4yX3JQf1kestk+Pl
zAnQYYbll9zTR2eMjq76xBHDzlz7mQaxPrL4fFCra88koPC0N5qWTLRi+aQG
OTcJ56N7bo8fZyK12pyy1blfYJTvPZPCZaJw26p/v3aTwLpn9M1ab5moZFdg
wQuZBCb8ZSpmYiaSitjawzemoKjaKm6DNoEee7l+lbV9GnZsyfMlrQh04fvk
fP7wNIh0rMv5uwhk0rf7Q1GyFDhGadM2/gRytE/KYtnI4PybUfdHpwnEWdk0
q9Img3GN5OhDKQSSvs8UnEqcARdF8z5TLoG4an4x+03loHmx22P0NoFW9D++
pUrKodJpJKe1nEDq8YkHHH6ahapBL4VFDYFo9/zKBDpz4Di0kxdbT6Co7LJM
nddz8E10EG1fE4FqzFM1GJHz0Momjwx2EoixO8riIn0B+PcLK5l9BOq1bG6L
rV4AwZWs2kUBgf7kgemQnwIYnQ30thECHSmzYxgtKkAiF3IoEYESkgsTe+8v
Amk2LpSLCVQqNMqhu1NgfzmD6zBFIE8GffPbOQpcI//L3icj0BV1VYNebN4m
wymlyVmpYhKbk+7p5oltJ+irY89TYCyonFeaXVG6NxI7zTma44UtOLDnuN4C
BU/Li40OYcdwM24cU1Bg8ELC+wG7NCMh9iz2gt+8TOnhhBhOJjbzV9W9HGyf
UM6qSmz9vGWU0s6OFg81FymQnFp+1A97Se/fb8qx5YqmlQHY0Fx7tx47+/qr
n5U+S1ak92B/rC5vVHrsQY6nOkXB6eeZ5wOxG+PD+3yw46Ns+49hL406+vIE
NpXCtgvCRiFeRRexD1tSmUo/8tgS+gD72TrSORhb9J2V+yvs93HF15U2tV9t
1YndHHBpXGk/c6aOCLul039HCPZNYy0phd3e45Sn9P9AVW6o
         "]]}, 
       {RGBColor[0, 0, 0.6], Thickness[0.008], LineBox[CompressedData["
1:eJwVx38003scx/FZlLh3tvmx7yZJoaRfJ45wq/enKPfKdGSxOpLTRah2u3Ir
dVRkqOE6Hbc0tJTjZ4xt9EPRrRWV68etUPnVWlrmlpZhfu1+9sfrPF8PhwO/
7Yoik0ikQDxD/aNUnQ8/R23Gt0DTLdxs7MM6OsD0BRLhuqFWdBLcRHkFUiYH
SLJjZqdEGbDjn25JITMSSH/6wR5RHoRGTjy4xEwAUk7xfHdRGfStTffJYqYC
yXrfkrZrd+C69c60i8xciLhNLA8sbIFkU+nyC8xiQPacFWFXeoC7p0g8+k0G
NSsGZ6Spn8Fcnxzmt7gJygK5tYGvNeAU9EX1WCAHK5a2QlcyCSfavCBv7Bnk
cE9I3svngCGj9RsZtUNEZbmbkwMZ1St8Wts3/gtcF68izYQxehRXaLtn5BWM
ZNeFmuUvQFrnTRZVR7pBNb/1VcDIQkTWVVJtnd7CkojGFYe3/4A89BH1wdW9
EBNHKWEHUVBrqnvb05UD0LKo/63Qhorsc9S68i+DQGLXpF82piH1xED/63UK
WPzXXfXGDhoqHm54Tsn+ADwBa+z+VTqi2JV+VSqVsMspgEJlW6LM2Lro645D
MPlfyGG50hKNo0plzNlP4FzSH3w60QolrJfLbB6rYE2m7UTSuBUaqJYLOh2G
YW/9R2R71BpNx73Kdzmlhgp+k0X1W2vkumXBXHTDCAj9P8wrd7NBfjccDtYy
voAVcu96ccEGnbDbv/R92FeQxMTuPdxpg1aN7DbvZo/C/UXtZ9ZTGcjSYtRU
PjUKzjw651kAAxk35YuXFXyDji09p2fPMNAh94Y3C3dogJfu/clLzECVvxy4
3jiugcHdHQVGbxhIq2ZRhy5/h8EQyR8XpxmoLthvXq7vGNxovuVZs4hAZ3/m
i2RDY7DSfCrJ1JNADn3+PTf5WohU5dO6dxLIe0NqLnP9ONRnHn8njiZQ2OJ2
ndHLcUhz0WecTySQ9nmW4mjyBIRccG/jCwiUZxyesMtxEuTD7OxQIYEs++tu
kOWTEEd+VO1RSiCTpGSO1+864JolpWVICUQqDxcr6FPg+5DfUvGAQPFCcRb9
0RSUPSkqCWkm0APXDFMabxqEUiqnop1ANP/4VSmUGeA1zfLPdxGod3XHy8T7
M6Da9O4gpY9ANdfAURk+C0KfRCFPQaB9Yg+a3dwsIEFRIfGJQOf4Rcm9FXOw
nbq1NFtNoCqVXT4lQA+pH5OXe30lUBCN4tk8pYeWl5yEOxoCXTQh2/Zir/wx
qsdguU47O4q9Tnxs411sD0XfY9a0HvauzjS+h82SVu3kYUvoFbkN2AoO+6D1
jB7WpF+VNWIn5GVe/nVWDym+ft/l2FWZ5xJPYqeJHEOfYA+dSwjLwiZHkhoM
5saGLb2NfdxFmvIUe5P3qmqzOT2wSk3oLdjze188qcXuig9f9wIbOprKnmJz
YpbmGnxSLhW8w5Y1KycMHr6VH2Si14OzPLKpFbst6VAfF/uSLCiwDXtB/P6H
R7Dv6UwlBqPo4Jsp2JqyRut2bEngT7G3sOu3OfUZrN66NuBv7AC3LtSB7bhh
2doubKqAX2xwuCuDrsYmsd1MO7Gv2Jtr9dhrEgcOGfw/JSRf0Q==
         "]]}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      AxesLabel->{
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"t\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm], 
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"v\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm]},
      AxesOrigin->{0, 0},
      Background->GrayLevel[0.6],
      PlotRange->{{0, 2}, {0., 6.240625817145844}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {189., -355.98757751993935`}, 
     ImageScaled[{0.5, 0.5}], {360., 222.4922359499621}]}, {}},
  ContentSelectable->True,
  PlotRangePadding->{6, 5}]], "Output",
 CellChangeTimes->{3.401273535718481*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["(E) Oscillations with a Spring and Mass",
 FontSize->16]], "Section"],

Cell[TextData[{
 "\tA spring produces a force that is proportional to the amount the spring \
is stretched.  If it is stretched from its equilibrium position by ",
 StyleBox["x",
  FontSlant->"Italic"],
 " then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "\[Proportional]", "x"}], TraditionalForm]]],
 ".  We define ",
 StyleBox["k",
  FontSlant->"Italic"],
 ", the spring constant, as the constant of proportionality and write:",
 "  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "=", 
    RowBox[{
     RowBox[{"-", "k"}], " ", "x"}]}], TraditionalForm]]],
 ".  The minus sign indicates that this is the force ",
 StyleBox["of",
  FontSlant->"Italic"],
 " the spring on the mass and not the force ",
 StyleBox["on",
  FontSlant->"Italic"],
 " the spring."
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "(", 
     RowBox[{"x", ",", "v", ",", "t"}], ")"}], "=", 
    RowBox[{
     RowBox[{"-", "k"}], " ", "x"}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "Once again, below is a repeat of the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " code for free fall.  Modify it to change the force function to this and \
change the initial conditions and other parameters to"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["t", "0"], "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "0"], "=", "2"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["v", "0"], "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "=", 
    RowBox[{"2", " ", "kg"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "=", 
    RowBox[{"90", " ", 
     StyleBox[
      FractionBox[
       StyleBox["N",
        FontSlant->"Plain"], 
       StyleBox["m",
        FontSlant->"Plain"]],
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "0", "]"}], "=", "0"}], ";", 
  RowBox[{
   RowBox[{"x", "[", "0", "]"}], "=", "2"}], ";", 
  RowBox[{
   RowBox[{"v", "[", "0", "]"}], "=", "0"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "final", "]"}], "=", "2"}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", "0.02"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"m", "=", "2"}], ";", 
  RowBox[{"g", "=", "9.8"}], ";", 
  RowBox[{"k", "=", "90"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"F", "[", 
   RowBox[{"x_", ",", "v_", ",", "t_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"-", " ", "k"}], " ", 
   "x"}]}], "\[IndentingNewLine]", "CalculateAndGraph"}], "Input",
 CellChangeTimes->{
  3.3997186413508787`*^9, {3.401273555886104*^9, 3.4012736052439528`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"x(t) = \"\>", "\[InvisibleSpace]", 
   RowBox[{"2", " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"3", " ", 
      SqrtBox["5"], " ", "t"}], "]"}]}]}],
  SequenceForm["x(t) = ", 2 Cos[3 5^Rational[1, 2] $CellContext`t]],
  Editable->False]], "Print",
 CellChangeTimes->{3.4012736060540113`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"v(t) = \"\>", "\[InvisibleSpace]", 
   RowBox[{
    RowBox[{"-", "6"}], " ", 
    SqrtBox["5"], " ", 
    RowBox[{"Sin", "[", 
     RowBox[{"3", " ", 
      SqrtBox["5"], " ", "t"}], "]"}]}]}],
  SequenceForm[
  "v(t) = ", (-6) 5^Rational[1, 2] Sin[3 5^Rational[1, 2] $CellContext`t]],
  Editable->False]], "Print",
 CellChangeTimes->{3.401273606058113*^9}]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[{{}, {InsetBox[
     GraphicsBox[{{}, {}, 
       {RGBColor[1, 1, 0.4], Thickness[0.02], LineBox[CompressedData["
1:eJwVl3k8VO8Xx8eMZWJmMKQsJZFUlpIo23kSylKUFiHZkooIJVpEKYnQIpGv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         "]]}, 
       {RGBColor[0, 0, 0.6], Thickness[0.008], LineBox[CompressedData["
1:eJwVmnk8VO8Xxy0tkogkpFRUyFKStZwWyZYtlUpKQlEkRIuIqGRLkkKbFCIi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         "]]}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      AxesLabel->{
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"t\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm], 
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"x\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm]},
      AxesOrigin->{0, 0},
      Background->GrayLevel[0.6],
      PlotRange->{{0, 2}, {-3.7649341476035234`, 4.645251372013171}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {189., -118.66252583997979`}, 
     ImageScaled[{0.5, 0.5}], {360., 222.4922359499621}], InsetBox[
     GraphicsBox[{{}, {}, 
       {RGBColor[1, 1, 0.4], Thickness[0.02], LineBox[CompressedData["
1:eJwVl3k8lN8XxysV2Yq0UL5l32Ysg7GMnJutZMlaFJJokaVQUSmylkohFUKJ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         "]]}, 
       {RGBColor[0, 0, 0.6], Thickness[0.008], LineBox[CompressedData["
1:eJwVmHk8lN8Xx0PKrkV2ZRlm5knylYjKuUWLkCUVhUpSFBWiRNlTSAuSSKTF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         "]]}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      AxesLabel->{
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"t\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm], 
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"v\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm]},
      AxesOrigin->{0, 0},
      Background->GrayLevel[0.6],
      PlotRange->{{0, 2}, {-22.797984386883716`, 28.053772026252553`}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {189., -355.98757751993935`}, 
     ImageScaled[{0.5, 0.5}], {360., 222.4922359499621}]}, {}},
  ContentSelectable->True,
  PlotRangePadding->{6, 5}]], "Output",
 CellChangeTimes->{3.401273606144853*^9}]
}, Open  ]],

Cell[TextData[{
 "If the error is too large between the symbolic solution, the blue line, and \
the numerical solution, the yellow line then ou will need a smaller value of \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]", "\[VeryThinSpace]", "t"}], TraditionalForm]]],
 ".  Try ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[CapitalDelta]", "\[VeryThinSpace]", "t"}], "=", ".002"}], 
   TraditionalForm]]],
 ".  To repaste the above input move the mouse curser below this text and \
type ",
 StyleBox["Ctrl-L",
  FontSlant->"Italic"],
 ". "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "0", "]"}], "=", "0"}], ";", 
  RowBox[{
   RowBox[{"x", "[", "0", "]"}], "=", "2"}], ";", 
  RowBox[{
   RowBox[{"v", "[", "0", "]"}], "=", "0"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "final", "]"}], "=", "2"}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", "0.002"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"m", "=", "2"}], ";", 
  RowBox[{"g", "=", "9.8"}], ";", 
  RowBox[{"k", "=", "90"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"F", "[", 
   RowBox[{"x_", ",", "v_", ",", "t_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"-", " ", "k"}], " ", 
   "x"}]}], "\[IndentingNewLine]", "CalculateAndGraph"}], "Input",
 CellChangeTimes->{
  3.3997186413508787`*^9, {3.401273555886104*^9, 3.401273626883453*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"x(t) = \"\>", "\[InvisibleSpace]", 
   RowBox[{"2", " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"3", " ", 
      SqrtBox["5"], " ", "t"}], "]"}]}]}],
  SequenceForm["x(t) = ", 2 Cos[3 5^Rational[1, 2] $CellContext`t]],
  Editable->False]], "Print",
 CellChangeTimes->{3.401273628411508*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"v(t) = \"\>", "\[InvisibleSpace]", 
   RowBox[{
    RowBox[{"-", "6"}], " ", 
    SqrtBox["5"], " ", 
    RowBox[{"Sin", "[", 
     RowBox[{"3", " ", 
      SqrtBox["5"], " ", "t"}], "]"}]}]}],
  SequenceForm[
  "v(t) = ", (-6) 5^Rational[1, 2] Sin[3 5^Rational[1, 2] $CellContext`t]],
  Editable->False]], "Print",
 CellChangeTimes->{3.401273628413186*^9}]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[{{}, {InsetBox[
     GraphicsBox[{{}, {}, 
       {RGBColor[1, 1, 0.4], Thickness[0.02], LineBox[CompressedData["
1:eJwVl3k8VV8Xxs+lJIkMmSIZSyEpY7LOj5SpEJJI5kqRqZQGQzSIqCQpUShj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         "]]}, 
       {RGBColor[0, 0, 0.6], Thickness[0.008], LineBox[CompressedData["
1:eJwVmnk8VO8Xxy0tkogkpFRUyFKStZwWyZYtlUpKQlEkRIuIqGRLkkKbFCIi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         "]]}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      AxesLabel->{
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"t\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm], 
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"x\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm]},
      AxesOrigin->{0, 0},
      Background->GrayLevel[0.6],
      PlotRange->{{0, 2}, {-2.130571171822409, 2.1759480742363406`}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {189., -118.66252583997979`}, 
     ImageScaled[{0.5, 0.5}], {360., 222.4922359499621}], InsetBox[
     GraphicsBox[{{}, {}, 
       {RGBColor[1, 1, 0.4], Thickness[0.02], LineBox[CompressedData["
1:eJwVl3k8lF8bxkVKlJB9yc7MPGOpRFrcp2gRIRQtpEIk9EPRTigUStosSUkl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         "]]}, 
       {RGBColor[0, 0, 0.6], Thickness[0.008], LineBox[CompressedData["
1:eJwVmHk8lN8Xx0PKrkV2ZRlm5knylYjKuUWLkCUVhUpSFBWiRNlTSAuSSKTF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         "]]}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      AxesLabel->{
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"t\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm], 
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"v\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm]},
      AxesOrigin->{0, 0},
      Background->GrayLevel[0.6],
      PlotRange->{{0, 2}, {-14.142491446903787`, 14.443711307793498`}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {189., -355.98757751993935`}, 
     ImageScaled[{0.5, 0.5}], {360., 222.4922359499621}]}, {}},
  ContentSelectable->True,
  PlotRangePadding->{6, 5}]], "Output",
 CellChangeTimes->{3.401273628508678*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["(F) Damped Oscillations with a Spring and Mass",
 FontSize->16]], "Section"],

Cell["\<\
\tNow combine the spring with viscous damping.  Here you should see \
oscillations that damp out with time.\
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "(", 
     RowBox[{"x", ",", "v", ",", "t"}], ")"}], "=", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "k"}], " ", "x"}], "-", 
     RowBox[{"b", " ", "v"}]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "Once again, below is a repeat of the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " code for free fall.  Modify it to change the force function to this and \
change the initial conditions and other parameters to"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["t", "0"], "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "0"], "=", "2"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["v", "0"], "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "=", 
    RowBox[{"2", " ", "kg"}]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "=", 
    RowBox[{"60", " ", 
     StyleBox[
      FractionBox[
       StyleBox["N",
        FontSlant->"Plain"], 
       StyleBox["m",
        FontSlant->"Plain"]],
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "=", 
    RowBox[{"5", " ", 
     FractionBox["kg", "s"]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "Modify the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]", "\[VeryThinSpace]", "t"}], TraditionalForm]]],
 " as in the previous part."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "0", "]"}], "=", "0"}], ";", 
  RowBox[{
   RowBox[{"x", "[", "0", "]"}], "=", "2"}], ";", 
  RowBox[{
   RowBox[{"v", "[", "0", "]"}], "=", "0"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"t", "[", "final", "]"}], "=", "2"}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", "0.002"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"m", "=", "2"}], ";", 
  RowBox[{"g", "=", "9.8"}], ";", 
  RowBox[{"k", "=", "60"}], ";", 
  RowBox[{"b", "=", "5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"F", "[", 
   RowBox[{"x_", ",", "v_", ",", "t_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "k"}], " ", "x"}], "-", 
   RowBox[{
   "b", " ", "v"}]}]}], "\[IndentingNewLine]", "CalculateAndGraph"}], "Input",\

 CellChangeTimes->{
  3.399718647839086*^9, {3.401273644219874*^9, 3.401273697382266*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"x(t) = \"\>", "\[InvisibleSpace]", 
   RowBox[{
    FractionBox["2", "91"], " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "5"}], " ", "t"}], "/", "4"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"91", " ", 
       RowBox[{"Cos", "[", 
        FractionBox[
         RowBox[{
          SqrtBox["455"], " ", "t"}], "4"], "]"}]}], "+", 
      RowBox[{
       SqrtBox["455"], " ", 
       RowBox[{"Sin", "[", 
        FractionBox[
         RowBox[{
          SqrtBox["455"], " ", "t"}], "4"], "]"}]}]}], ")"}]}]}],
  SequenceForm[
  "x(t) = ", Rational[2, 91] 
   E^(Rational[-5, 4] $CellContext`t) (
    91 Cos[Rational[1, 4] 455^Rational[1, 2] $CellContext`t] + 
    455^Rational[1, 2] 
     Sin[Rational[1, 4] 455^Rational[1, 2] $CellContext`t])],
  Editable->False]], "Print",
 CellChangeTimes->{{3.401273664605914*^9, 3.401273699035481*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"v(t) = \"\>", "\[InvisibleSpace]", 
   RowBox[{
    RowBox[{
     FractionBox["2", "91"], " ", 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "5"}], " ", "t"}], "/", "4"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FractionBox["455", "4"], " ", 
        RowBox[{"Cos", "[", 
         FractionBox[
          RowBox[{
           SqrtBox["455"], " ", "t"}], "4"], "]"}]}], "-", 
       RowBox[{
        FractionBox["91", "4"], " ", 
        SqrtBox["455"], " ", 
        RowBox[{"Sin", "[", 
         FractionBox[
          RowBox[{
           SqrtBox["455"], " ", "t"}], "4"], "]"}]}]}], ")"}]}], "-", 
    RowBox[{
     FractionBox["5", "182"], " ", 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "5"}], " ", "t"}], "/", "4"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"91", " ", 
        RowBox[{"Cos", "[", 
         FractionBox[
          RowBox[{
           SqrtBox["455"], " ", "t"}], "4"], "]"}]}], "+", 
       RowBox[{
        SqrtBox["455"], " ", 
        RowBox[{"Sin", "[", 
         FractionBox[
          RowBox[{
           SqrtBox["455"], " ", "t"}], "4"], "]"}]}]}], ")"}]}]}]}],
  SequenceForm[
  "v(t) = ", 
   Rational[2, 91] 
    E^(Rational[-5, 4] $CellContext`t) (
     Rational[455, 4] Cos[Rational[1, 4] 455^Rational[1, 2] $CellContext`t] + 
     Rational[-91, 4] 455^Rational[1, 2] 
      Sin[Rational[1, 4] 455^Rational[1, 2] $CellContext`t]) + 
   Rational[-5, 182] 
    E^(Rational[-5, 4] $CellContext`t) (
     91 Cos[Rational[1, 4] 455^Rational[1, 2] $CellContext`t] + 
     455^Rational[1, 2] 
      Sin[Rational[1, 4] 455^Rational[1, 2] $CellContext`t])],
  Editable->False]], "Print",
 CellChangeTimes->{{3.401273664605914*^9, 3.401273699037772*^9}}]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[{{}, {InsetBox[
     GraphicsBox[{{}, {}, 
       {RGBColor[1, 1, 0.4], Thickness[0.02], LineBox[CompressedData["
1:eJwVl3k8VO8Xx7NFSJHKWlmSytJXIZQjS4VkLUv2JYoiqbTILkmiJKFUVMgu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         "]]}, 
       {RGBColor[0, 0, 0.6], Thickness[0.008], LineBox[CompressedData["
1:eJwdmnk81N/3xy2J7CFbJJSUNVmicpQ1ImXPUpYSWSKVFikhpUXpIyEUspO9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         "]]}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      AxesLabel->{
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"t\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm], 
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"x\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm]},
      AxesOrigin->{0, 0},
      Background->GrayLevel[0.6],
      PlotRange->{{0, 2}, {-0.9747724701539368, 2.000028522194832}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {189., -118.66252583997979`}, 
     ImageScaled[{0.5, 0.5}], {360., 222.4922359499621}], InsetBox[
     GraphicsBox[{{}, {}, 
       {RGBColor[1, 1, 0.4], Thickness[0.02], LineBox[CompressedData["
1:eJwV13k4VF8fAPDs+0yE7CMpKlQqIvreSiVL1kiWSqKUJZQoP1TWrGmhpFCR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         "]]}, 
       {RGBColor[0, 0, 0.6], Thickness[0.008], LineBox[CompressedData["
1:eJwV13c8Vf8bAHB7uyUj+zZUNl9EQs+paCB7p0gqKhlZiYqQrAaVihJSysrK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         "]]}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->True,
      AxesLabel->{
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"t\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm], 
        FormBox[
        "\"\\!\\(\\*\\nStyleBox[\\\"v\\\",\\nFontSlant->\\\"Italic\\\"]\\)\"",
          TraditionalForm]},
      AxesOrigin->{0, 0},
      Background->GrayLevel[0.6],
      PlotRange->{{0, 2}, {-8.061328060650494, 3.9289245985188415`}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {189., -355.98757751993935`}, 
     ImageScaled[{0.5, 0.5}], {360., 222.4922359499621}]}, {}},
  ContentSelectable->True,
  PlotRangePadding->{6, 5}]], "Output",
 CellChangeTimes->{{3.401273664701907*^9, 3.4012736991347017`*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowToolbars->{},
WindowSize->{952, 691},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingCopies->55,
PrintingPageRange->{1, Automatic},
Magnification->2.,
FrontEndVersion->"6.0 for Mac OS X x86 (32-bit) (June 19, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[590, 23, 134, 5, 142, "Subtitle"],
Cell[727, 30, 243, 4, 112, "Text"],
Cell[973, 36, 244, 10, 52, "Text"],
Cell[1220, 48, 667, 27, 112, "Text"],
Cell[1890, 77, 677, 23, 135, "Text"],
Cell[2570, 102, 347, 10, 112, "Text"],
Cell[2920, 114, 1162, 40, 141, "Text"],
Cell[4085, 156, 488, 21, 88, "Text"],
Cell[4576, 179, 472, 11, 172, "Text"],
Cell[5051, 192, 217, 7, 82, "Text"],
Cell[5271, 201, 601, 20, 81, "Text"],
Cell[5875, 223, 135, 3, 82, "Text"],
Cell[CellGroupData[{
Cell[6035, 230, 67, 1, 126, "Section"],
Cell[6105, 233, 109, 3, 82, "Text"],
Cell[6217, 238, 246, 10, 52, "Text"],
Cell[6466, 250, 58, 0, 52, "Text"],
Cell[6527, 252, 392, 18, 55, "Text"],
Cell[6922, 272, 671, 22, 112, "Text"],
Cell[7596, 296, 171, 5, 82, "Text"],
Cell[7770, 303, 615, 18, 145, "Input"],
Cell[8388, 323, 149, 5, 52, "Text"],
Cell[8540, 330, 2136, 67, 355, "Input"],
Cell[10679, 399, 386, 10, 142, "Text"],
Cell[CellGroupData[{
Cell[11090, 413, 1211, 36, 205, "Input"],
Cell[CellGroupData[{
Cell[12326, 453, 266, 7, 47, "Print"],
Cell[12595, 462, 236, 6, 40, "Print"]
}, Open  ]]
}, Open  ]],
Cell[12858, 472, 385, 15, 112, "Text"],
Cell[CellGroupData[{
Cell[13268, 491, 2478, 70, 415, "Input"],
Cell[15749, 563, 13374, 239, 879, "Output"]
}, Open  ]],
Cell[29138, 805, 253, 7, 112, "Text"],
Cell[29394, 814, 6114, 169, 1031, "Input"],
Cell[35511, 985, 820, 23, 142, "Text"],
Cell[CellGroupData[{
Cell[36356, 1012, 176, 4, 85, "Input"],
Cell[CellGroupData[{
Cell[36557, 1020, 264, 7, 47, "Print"],
Cell[36824, 1029, 238, 6, 40, "Print"]
}, Open  ]],
Cell[37077, 1038, 12262, 221, 879, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[49388, 1265, 85, 1, 126, "Section"],
Cell[49476, 1268, 340, 11, 82, "Text"],
Cell[49819, 1281, 265, 11, 52, "Text"],
Cell[50087, 1294, 238, 6, 82, "Text"],
Cell[50328, 1302, 947, 43, 75, "Text"],
Cell[CellGroupData[{
Cell[51300, 1349, 777, 23, 175, "Input"],
Cell[CellGroupData[{
Cell[52102, 1376, 661, 20, 70, "Print"],
Cell[52766, 1398, 662, 20, 52, "Print"]
}, Open  ]],
Cell[53443, 1421, 8467, 158, 879, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[61959, 1585, 88, 1, 126, "Section"],
Cell[62050, 1588, 98, 2, 52, "Text"],
Cell[62151, 1592, 296, 12, 52, "Text"],
Cell[62450, 1606, 250, 6, 112, "Text"],
Cell[62703, 1614, 1062, 48, 78, "Text"],
Cell[CellGroupData[{
Cell[63790, 1666, 810, 25, 175, "Input"],
Cell[CellGroupData[{
Cell[64625, 1695, 763, 19, 52, "Print"],
Cell[65391, 1716, 1335, 35, 94, "Print"]
}, Open  ]],
Cell[66741, 1754, 11700, 210, 879, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[78490, 1970, 90, 1, 126, "Section"],
Cell[78583, 1973, 423, 11, 119, "Text"],
Cell[79009, 1986, 324, 13, 59, "Text"],
Cell[79336, 2001, 250, 6, 112, "Text"],
Cell[79589, 2009, 1064, 48, 78, "Text"],
Cell[80656, 2059, 386, 9, 82, "Text"],
Cell[CellGroupData[{
Cell[81067, 2072, 1092, 30, 175, "Input"],
Cell[CellGroupData[{
Cell[82184, 2106, 612, 15, 52, "Print"],
Cell[82799, 2123, 676, 17, 78, "Print"]
}, Open  ]],
Cell[83490, 2143, 12045, 216, 879, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[95584, 2365, 93, 1, 126, "Section"],
Cell[95680, 2368, 769, 26, 172, "Text"],
Cell[96452, 2396, 265, 11, 52, "Text"],
Cell[96720, 2409, 250, 6, 112, "Text"],
Cell[96973, 2417, 763, 35, 71, "Text"],
Cell[CellGroupData[{
Cell[97761, 2456, 783, 23, 175, "Input"],
Cell[CellGroupData[{
Cell[98569, 2483, 341, 9, 51, "Print"],
Cell[98913, 2494, 406, 12, 51, "Print"]
}, Open  ]],
Cell[99334, 2509, 61225, 1022, 879, "Output"]
}, Open  ]],
Cell[160574, 3534, 573, 18, 142, "Text"],
Cell[CellGroupData[{
Cell[161172, 3556, 782, 23, 175, "Input"],
Cell[CellGroupData[{
Cell[161979, 3583, 339, 9, 51, "Print"],
Cell[162321, 3594, 406, 12, 51, "Print"]
}, Open  ]],
Cell[162742, 3609, 60339, 1008, 879, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[223130, 4623, 100, 1, 126, "Section"],
Cell[223233, 4626, 131, 3, 82, "Text"],
Cell[223367, 4631, 318, 13, 52, "Text"],
Cell[223688, 4646, 250, 6, 112, "Text"],
Cell[223941, 4654, 894, 41, 75, "Text"],
Cell[224838, 4697, 175, 6, 52, "Text"],
Cell[CellGroupData[{
Cell[225038, 4707, 859, 27, 175, "Input"],
Cell[CellGroupData[{
Cell[225922, 4738, 949, 29, 85, "Print"],
Cell[226874, 4769, 1861, 57, 202, "Print"]
}, Open  ]],
Cell[228750, 4829, 53953, 903, 879, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
