(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    139578,       3016]
NotebookOptionsPosition[     77882,       1705]
NotebookOutlinePosition[    136379,       2915]
CellTagsIndexPosition[    136336,       2912]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Exercise"
}], "Title"],

Cell["Work and Integration", "Subtitle"],

Cell["Phys 2425 - Blinn College", "Subsubtitle"],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell["\<\
This is an exercise to demonstrate the calculation and interpretation of work \
integrals.  We will consider the case of one dimensional forces\
\>", "Text",
 FontFamily->"Times"],

Cell[TextData[{
 "If we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "(", "x", ")"}], TraditionalForm]]],
 " as a one dimension force as a function of position.  The work when moving \
from position ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "i"], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "f"], TraditionalForm]]],
 " the work is the one dimensional definite integral"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{"W", "=", 
    RowBox[{
     SubsuperscriptBox["\[Integral]", 
      SubscriptBox["x", "i"], 
      SubscriptBox["x", "f"]], " ", 
     RowBox[{
      RowBox[{"F", "(", "x", ")"}], " ", 
      RowBox[{"\[DifferentialD]", "x"}]}]}]}], TraditionalForm]],
  FontSize->12],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
This is the sort of one variable integral discussed in the first semester of \
calculus and the student will be responsible for such integrals.  To evaluate \
a definite integral one uses the fundamental theorem of calculus.\
\>", "Text"],

Cell["Initialization - Define PlotAndShade and WorkPlotAndShade", \
"Subsubsection",
 CellChangeTimes->{{3.400372788152141*^9, 3.4003727932919207`*^9}, {
  3.40040498894398*^9, 3.400404997074417*^9}, {3.400405199815077*^9, 
  3.4004052238499813`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Attributes", "[", "PlotAndShade", "]"}], "=", 
   RowBox[{"{", "HoldAll", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"PlotAndShade", "[", 
   RowBox[{"expr_", ",", 
    RowBox[{"{", 
     RowBox[{"x_", ",", "xMin_", ",", "xMax_"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"a_", ",", "b_"}], "}"}], ",", "opts___"}], "]"}], ":=", 
  "\[IndentingNewLine]", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"expr", ",", 
      RowBox[{"Piecewise", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"expr", ",", 
           RowBox[{"a", "<", "x", "<", "b"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "True"}], "}"}]}], "}"}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
    RowBox[{"FillingStyle", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Lighter", "[", 
        RowBox[{"Red", ",", ".6"}], "]"}], ",", 
       RowBox[{"Lighter", "[", 
        RowBox[{"Green", ",", ".6"}], "]"}]}], "}"}]}], ",", 
    RowBox[{"Filling", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"2", "\[Rule]", "Axis"}], "}"}]}], ",", "opts"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"WorkPlotAndShade", "[", 
   RowBox[{"expr_", ",", 
    RowBox[{"{", 
     RowBox[{"x_", ",", "xMin_", ",", "xMax_"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"a_", ",", "b_"}], "}"}], ",", "opts___"}], "]"}], ":=", 
  RowBox[{"PlotAndShade", "[", 
   RowBox[{"expr", ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"a", ",", "b"}], "}"}], ",", "opts", ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"\"\<x (m)\>\"", ",", "\"\<F(x) (N)\>\""}], "}"}]}]}], 
   "]"}]}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.400370309204721*^9, 3.400370309947556*^9}, {
  3.400370345795672*^9, 3.400370429822926*^9}, {3.400370766257679*^9, 
  3.4003707761025753`*^9}, {3.400371512561397*^9, 3.400371692338335*^9}, {
  3.4003724570127296`*^9, 3.400372474384777*^9}, {3.400372514467833*^9, 
  3.400372515944113*^9}, {3.400372632254334*^9, 3.400372655479686*^9}, {
  3.4003728401967916`*^9, 3.4003728411333103`*^9}, {3.400373004532098*^9, 
  3.40037301032368*^9}, {3.400373296418889*^9, 3.4003733186578693`*^9}, {
  3.4003736686519957`*^9, 3.4003736861696367`*^9}, {3.4003737185805473`*^9, 
  3.400373756793833*^9}}]
}, {1, 2, 3, 4, 5, 6}]],

Cell[CellGroupData[{

Cell["The Definite Integral and Area", "Section"],

Cell[TextData[{
 " The definite integral ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "a", "b"], " ", 
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], " ", 
     RowBox[{"\[DifferentialD]", "x"}]}]}], TraditionalForm]]],
 " has the graphical interpretation as being the area under the curve ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "=", 
    RowBox[{"f", "(", "x", ")"}]}], TraditionalForm]]],
 " between ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "=", "a"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "=", "b"}], TraditionalForm]]],
 ".  When the function is negative, when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is below the axis, the contribution to the area is taken to be negative.  \
We will illustrate this with ths funciton ",
 Cell[BoxData[
  FormBox[
   RowBox[{"sin", " ", "x"}], TraditionalForm]]],
 " integrated between three different sets of limits."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAndShade", "[", 
  RowBox[{
   RowBox[{"Sin", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", 
     RowBox[{"2", " ", "\[Pi]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "\[Pi]"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{3.400372858882946*^9}],

Cell[BoxData[
 GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJzteXk0Ve/bvjFD4gyESlFpMqVEg7qfChGJSAMhSUiSSpI0i5IMpZIpZSiV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   "], {{{}, {}, {}, 
     {RGBColor[0.6, 1., 0.6], EdgeForm[None], 
      GraphicsGroupBox[
       PolygonBox[{{433, 738, 739, 497, 732, 663, 720, 605, 652, 709, 556, 
        592, 639, 696, 513, 539, 575, 622, 679, 457, 523, 549, 585, 632, 689, 
        496, 512, 538, 574, 621, 678, 456, 455, 454, 453, 452, 451, 495, 511, 
        537, 573, 620, 677, 450, 494, 510, 536, 572, 619, 676, 449, 493, 509, 
        535, 571, 618, 675, 448, 522, 548, 584, 631, 688, 492, 508, 534, 570, 
        617, 674, 447, 521, 547, 583, 630, 687, 491, 555, 591, 638, 695, 507, 
        600, 647, 704, 533, 569, 616, 673, 446, 607, 654, 711, 559, 595, 642, 
        699, 520, 602, 649, 706, 546, 657, 714, 582, 724, 629, 686, 490, 731, 
        662, 719, 604, 728, 651, 708, 554, 729, 659, 716, 590, 726, 637, 694, 
        506, 730, 661, 718, 599, 727, 646, 703, 532, 656, 713, 568, 723, 615, 
        672, 445, 664, 721, 606, 653, 710, 558, 660, 717, 594, 641, 698, 519, 
        601, 648, 705, 545, 581, 628, 685, 489, 553, 589, 636, 693, 505, 598, 
        645, 702, 531, 567, 614, 671, 444, 557, 593, 640, 697, 518, 544, 580, 
        627, 684, 488, 552, 588, 635, 692, 504, 530, 566, 613, 670, 443, 517, 
        543, 579, 626, 683, 487, 503, 529, 565, 612, 669, 442, 516, 542, 578, 
        625, 682, 486, 502, 528, 564, 611, 668, 441, 485, 501, 527, 563, 610, 
        667, 440, 439, 438, 437, 436, 435, 434, 484, 500, 526, 562, 609, 
        666}},
        VertexColors->None]]}, {}, {}, {}, {}, {}, {}}, {{}, {}, 
     {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwV1GWAFVUABeClU7p7CemUlA5Jc+3WxQ5WQSlFTFABJVQQBSUEabAbu7sT
u7sT9Ts/vnfOvW/nvpk7M1tYXFI0oVRBQcE2H8mxPn6WGziLPelJacpQlnKU
pwIVGeeYX+RGzmYvelEpx/vuV7mJyexNb4rNV+ERfS7HUTnHmvtNbmYK+9CH
8ear8qg+j+OpkvXM/S63MJV9mWiuMa/o19M36xvvwmP6FZzAdONWvKOvpmp+
z/gPuZVpXGDcgY/0/ZikN+FVfRn9ci7G1Xhcv5JZelc+10/kHL017+pL5FBu
ynnIPnyvL5ejco65BvmnXCG3yYWyP9P1C2VH1vCx8VI5nE0U5Z7JpqzmNeNr
5GA2sNx4juzJzXyT/ZF7sIXdsy+yOit5wvgqOZD5+mzZjS/0kzhXb8N7+ppc
u96XH/Rq2S/9L3lL9oCLjDvxib5/nhW9Ga/rN9A/+2Vcgyf1BZxMdYrM/S1v
ze9yAANyHuZr8lT2iVOokfXN/SNvYwYHMpCaOdZ3O+XtnMdBDKJW/s53/8o7
mMnBDKY2dahLPerTgIbs8HI0ko1pQlOa0ZwWWd+a/8k7OZ9DGEJh1s9Lx115
1jiUobluc7V4Wl/EqbTMsXlPuTvPAocxLNdtrjbP5J5xGq2ynrnS3JP953CG
Zz1zdXhWv5rTaZ31zJXhXv1ijmCycXPe0G9kRNY3rstzeca4VO/Ol/qE7Lm+
Kzv0tbTJbxuX5T79kqyvd+ZT/Uim6C14U18sh7BCnyt78a2+TI7MM5tzlvVY
xfO5DjmI9Sw2vkz2YC1fGV8nR7CZktx72TbvIu8bXyuHsTHvh/F82Y91/Jhn
VI7OdWVfZDnuz/2RA5iV65Jd+Ew/iql6IW/pK5mn9+Y7fWT2Sq/PC/oSLtd3
42v9jDyDejs+0NfRNntlXJ4H9NkczTTjlrytr2IUJcYNeDHXxpm0y3mZq8D2
3DeOYXR+z1xDXtKXMpH2Wd9cRR7MnnIsY+iQY81X4qGcO8WMpSOd6EwXutKN
7vTIGo6pzMP6HMYzLudorhEv534xKe+McXs+1NezIP+r+CnPoxzD1uyb8f/B
38BP
       "]]}, 
     {Hue[0.9060679774997897, 0.6, 0.6], 
      LineBox[{433, 666, 609, 562, 526, 500, 484, 434, 435, 436, 437, 438, 
       439, 440, 667, 610, 563, 527, 501, 485, 441, 668, 611, 564, 528, 502, 
       486, 682, 625, 578, 542, 516, 442, 669, 612, 565, 529, 503, 487, 683, 
       626, 579, 543, 517, 443, 670, 613, 566, 530, 504, 692, 635, 588, 552, 
       488, 684, 627, 580, 544, 518, 697, 640, 593, 557, 444, 671, 614, 567, 
       531, 702, 645, 598, 505, 693, 636, 589, 553, 489, 685, 628, 581, 545, 
       705, 648, 601, 519, 698, 641, 594, 717, 660, 558, 710, 653, 606, 721, 
       664, 445, 672, 615, 723, 568, 713, 656, 532, 703, 646, 727, 599, 718, 
       661, 730, 506, 694, 637, 726, 590, 716, 659, 729, 554, 708, 651, 728, 
       604, 719, 662, 731, 490, 686, 629, 724, 582, 714, 657, 546, 706, 649, 
       602, 520, 699, 642, 595, 559, 711, 654, 607, 446, 673, 616, 569, 533, 
       704, 647, 600, 507, 695, 638, 591, 555, 491, 687, 630, 583, 547, 521, 
       447, 674, 617, 570, 534, 508, 492, 688, 631, 584, 548, 522, 448, 675, 
       618, 571, 535, 509, 493, 449, 676, 619, 572, 536, 510, 494, 450, 677, 
       620, 573, 537, 511, 495, 451, 452, 453, 454, 455, 456, 678, 621, 574, 
       538, 512, 496, 689, 632, 585, 549, 523, 457, 679, 622, 575, 539, 513, 
       696, 639, 592, 556, 709, 652, 605, 720, 663, 732, 497}], 
      LineBox[{690, 633, 725, 586, 715, 658, 550, 707, 650, 603, 524, 700, 
       643, 596, 560, 458, 680, 623, 576, 540, 514, 498, 459, 460, 461, 462, 
       463, 464, 465, 466, 467, 468, 469, 470, 471, 472, 473, 474, 475, 476, 
       477, 478, 479, 480, 481, 482, 681, 624, 577, 541, 515, 499, 691, 634, 
       587, 551, 525, 701, 644, 597, 561, 712, 655, 608, 722, 665, 733, 
       483}]}, 
     {RGBColor[1., 0.6, 0.6], LineBox[{736, 737, 735}]}}, {{}, {}, {}, {}}}],
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesOrigin->{0, 0},
  Method->{"AxesInFront" -> True},
  PlotRange->
   NCache[{{0, 2 Pi}, {-0.9999998592131705, 0.9999998782112116}}, {{
     0, 6.283185307179586}, {-0.9999998592131705, 0.9999998782112116}}],
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.4012326607164717`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox["\[Integral]", "0", "\[Pi]"], 
  RowBox[{
   RowBox[{"Sin", "[", "x", "]"}], " ", 
   RowBox[{"\[DifferentialD]", "x"}]}]}]], "Input"],

Cell[BoxData["2"], "Output",
 CellChangeTimes->{3.40123266519462*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAndShade", "[", 
  RowBox[{
   RowBox[{"Sin", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", 
     RowBox[{"2", " ", "\[Pi]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Pi]", ",", 
     RowBox[{"2", " ", "\[Pi]"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{3.4003728959206467`*^9}],

Cell[BoxData[
 GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJzteXlUjW37diOVaA9FIQoZGwwpQ5wXmiiRkimVCpU0IUkyp0hSCWkSDfQQ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   "], {{{}, {}, 
     {RGBColor[1., 0.6, 0.6], EdgeForm[None], 
      GraphicsGroupBox[
       PolygonBox[{{483, 714, 715, 665, 614, 701, 572, 693, 642, 539, 514, 
        458, 652, 601, 559, 526, 501, 486, 459, 460, 461, 462, 463, 464, 653, 
        602, 560, 527, 502, 487, 465, 654, 603, 561, 528, 503, 488, 666, 615, 
        573, 540, 515, 466, 655, 604, 562, 529, 504, 489, 667, 616, 574, 541, 
        516, 467, 656, 605, 563, 530, 505, 490, 668, 617, 575, 542, 517, 468, 
        657, 606, 564, 531, 506, 675, 624, 582, 549, 491, 669, 618, 576, 543, 
        685, 634, 592, 518, 679, 628, 586, 553, 469, 658, 607, 565, 532, 683, 
        632, 590, 507, 676, 625, 583, 695, 644, 550, 688, 637, 595, 698, 647, 
        492, 670, 619, 702, 577, 694, 643, 706, 544, 686, 635, 704, 593, 697, 
        646, 708, 519, 680, 629, 703, 587, 696, 645, 707, 554, 690, 639, 705, 
        597, 699, 648, 470, 659, 608, 566, 692, 641, 533, 684, 633, 591, 508, 
        677, 626, 584, 551, 689, 638, 596, 493, 671, 620, 578, 545, 687, 636, 
        594, 520, 681, 630, 588, 555, 471, 660, 609, 567, 534, 509, 678, 627, 
        585, 552, 494, 672, 621, 579, 546, 521, 472, 661, 610, 568, 535, 510, 
        495, 673, 622, 580, 547, 522, 473, 662, 611, 569, 536, 511, 496, 474, 
        663, 612, 570, 537, 512, 497, 475, 476, 477, 478, 479, 480, 481, 482, 
        664, 613, 571, 538, 513, 498, 674, 623, 581, 548, 523, 682, 631, 589, 
        556, 691, 640, 598, 700, 649, 709}},
        VertexColors->None]]}, {}, {}, {}, {}, {}, {}, {}}, {{}, {}, 
     {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwV1GWAFVUABeClU7p7CemUlO4y125d7GAVlDIwQQklVBAFJQRpsBsTu7u7
uzu+8+N759z7du6buTOzhcUlRRNLFRQUbPeRHOfjJ7mRM9mDnpSmDGUpR3kq
UJHxjvlZbuIs9qQXlXK8736Rm5nCXvSm2HwVHtbncSyVc6y5X+UWprI3fZhg
viqP6PM5jipZz9xvcivT2IdJ5hrzkn4dfbO+8S48ql/O8cwwbsVb+hqq5veM
f5fbmM4Fxh34UN+XyXoTXtaX0y/nYlyNnfoVzNK78pl+AmfrrXlbXyqHcmPO
Q/bhO32FHJ1zzDXIP+RKuV0ukv2ZoV8oO7KWj4yXyeFspij3TDZlDa8YXy0H
s5EVxnNlT27i6+yPHMlWds++yOqs4jHjK+VAFuizZTc+10/kHL0N7+hrc+16
X77Xq2W/9D/lzdkDLjLuxMf6fnlW9Ga8ql9P/+yXcQ0e1xdyEtUpMveXvCW/
y/4MyHmYr8kT2SdOpkbWN/e3vJVzOYCB1MyxvvtH3sZ5HMggauXvfPevvJ2Z
HMRgalOHutSjPg1oyE4vRyPZmCY0pRnNaZH1rfmfvIPzOZghFGb9vHTcmWeN
Qxia6zZXiyf1xZxCyxyb95S78ixwKMNy3eZq81TuGafSKuuZK83d2X8OY3jW
M1eHp/WrOI3WWc9cGe7RL+Zwphg35zX9BkZkfeO6PJNnjEv17nyhT8ye67vy
rr6ONvlt47Lcq1+S9fXOfKIfwVS9Ba/rS+QQVurzZC++0ZfLUXlmc86yHqt5
NtchB7GBJcaXyR6s40vja+UItlCSey/b5l3kPeNr5DA25f0wXiD7sZ4f8ozK
Mbmu7Issx325P3IAs3Jdsguf6kcyTS/kDX0V8/XefKuPyl7p9XlOX8ocfTe+
0k/PM6i34319PW2zV8bluV+fzVFMN27Jm/pqRlNi3IDnc22cQbucl7kK7Mh9
42jG5PfMNeQFfRmTaJ/1zVXkgewpxzCWDjnWfCUezLlTzDg60onOdKEr3ehO
j6zhmMo8pM9lAuNzjuYa8WLuF5Pzzhi35wN9Awvzv4of8zzKsWzLvhn/D2y/
wDc=
       "]]}, 
     {Hue[0.9060679774997897, 0.6, 0.6], 
      LineBox[{433, 650, 599, 557, 524, 499, 484, 434, 435, 436, 437, 438, 
       439, 440, 441, 442, 443, 444, 445, 446, 447, 448, 449, 450, 451, 452, 
       453, 454, 455, 456, 457, 651, 600, 558, 525, 500, 485}], 
      LineBox[{665, 614, 701, 572, 693, 642, 539, 514, 458, 652, 601, 559, 
       526, 501, 486, 459, 460, 461, 462, 463, 464, 653, 602, 560, 527, 502, 
       487, 465, 654, 603, 561, 528, 503, 488, 666, 615, 573, 540, 515, 466, 
       655, 604, 562, 529, 504, 489, 667, 616, 574, 541, 516, 467, 656, 605, 
       563, 530, 505, 490, 668, 617, 575, 542, 517, 468, 657, 606, 564, 531, 
       506, 675, 624, 582, 549, 491, 669, 618, 576, 543, 685, 634, 592, 518, 
       679, 628, 586, 553, 469, 658, 607, 565, 532, 683, 632, 590, 507, 676, 
       625, 583, 695, 644, 550, 688, 637, 595, 698, 647, 492, 670, 619, 702, 
       577, 694, 643, 706, 544, 686, 635, 704, 593, 697, 646, 708, 519, 680, 
       629, 703, 587, 696, 645, 707, 554, 690, 639, 705, 597, 699, 648, 470, 
       659, 608, 566, 692, 641, 533, 684, 633, 591, 508, 677, 626, 584, 551, 
       689, 638, 596, 493, 671, 620, 578, 545, 687, 636, 594, 520, 681, 630, 
       588, 555, 471, 660, 609, 567, 534, 509, 678, 627, 585, 552, 494, 672, 
       621, 579, 546, 521, 472, 661, 610, 568, 535, 510, 495, 673, 622, 580, 
       547, 522, 473, 662, 611, 569, 536, 511, 496, 474, 663, 612, 570, 537, 
       512, 497, 475, 476, 477, 478, 479, 480, 481, 482, 664, 613, 571, 538, 
       513, 498, 674, 623, 581, 548, 523, 682, 631, 589, 556, 691, 640, 598, 
       700, 649, 709, 483}]}, 
     {RGBColor[1., 0.6, 0.6], LineBox[{712, 713, 711}]}}, {{}, {}, {}, {}}}],
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesOrigin->{0, 0},
  Method->{"AxesInFront" -> True},
  PlotRange->
   NCache[{{0, 2 Pi}, {-0.9999998592131705, 0.9999998782112116}}, {{
     0, 6.283185307179586}, {-0.9999998592131705, 0.9999998782112116}}],
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.401232670007104*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox["\[Integral]", "\[Pi]", 
   RowBox[{"2", " ", "\[Pi]"}]], 
  RowBox[{
   RowBox[{"Sin", "[", "x", "]"}], " ", 
   RowBox[{"\[DifferentialD]", "x"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"-", "2"}]], "Output",
 CellChangeTimes->{3.401232673750667*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAndShade", "[", 
  RowBox[{
   RowBox[{"Sin", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", 
     RowBox[{"2", " ", "\[Pi]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Pi]", "/", "2"}], ",", 
     RowBox[{"3", " ", 
      RowBox[{"\[Pi]", "/", "2"}]}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{3.4003729202874737`*^9}],

Cell[BoxData[
 GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztens0lVv7tmMhsQ4UlaLzgaSUDuqelWOklHQSQoWEVJKkcyFJSCWnlEPZ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   "], {{{}, {}, 
     {RGBColor[1., 0.6, 0.6], EdgeForm[None], 
      GraphicsGroupBox[
       PolygonBox[{{692, 736, 734, 458, 459, 460, 461, 462, 463, 464, 668, 
        613, 568, 534, 508, 492, 465, 669, 614, 569, 535, 509, 493, 680, 625, 
        580, 546, 520, 466, 670, 615, 570, 536, 510, 494, 681, 626, 581, 547, 
        521, 467, 671, 616, 571, 537, 511, 495, 682, 627, 582, 548, 522, 468, 
        672, 617, 572, 538, 512, 688, 633, 588, 554, 496, 683, 628, 583, 549, 
        698, 643, 598, 523, 691, 636, 591, 557, 469, 673, 618, 573, 539, 696, 
        641, 596, 513, 689, 634, 589, 708, 653, 555, 701, 646, 601, 713, 658, 
        497, 684, 629, 716, 584, 706, 651, 721, 550, 699, 644, 719, 599, 711, 
        656, 723, 524}},
        VertexColors->None]]}, 
     {RGBColor[0.6, 1., 0.6], EdgeForm[None], 
      GraphicsGroupBox[
       PolygonBox[{{586, 735, 734, 457, 456, 455, 454, 453, 452, 451, 491, 
        507, 533, 567, 612, 667, 450, 490, 506, 532, 566, 611, 666, 449, 489, 
        505, 531, 565, 610, 665, 448, 519, 545, 579, 624, 679, 488, 504, 530, 
        564, 609, 664, 447, 518, 544, 578, 623, 678, 487, 553, 587, 632, 687, 
        503, 595, 640, 695, 529, 563, 608, 663, 446, 602, 647, 702, 556, 590, 
        635, 690, 517, 597, 642, 697, 543, 650, 705, 577, 715, 622, 677, 486, 
        724, 657, 712, 600, 720, 645, 700, 552, 722, 652, 707}},
        VertexColors->None]]}, {}, {}, {}, {}, {}, {}}, {{}, {}, 
     {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwV1GWAVUUABeCVLunuJaRTUjokzbVbFztYBaUMTELaAlFQQpDGALsV7O7u
7m79zo9vz5l53Hn3zp1HYXFJ0fhdCgoKbvUnOcafn+QGzmYvelCK0pShLOUo
TwXGuuZnuZFz2JueVMz1PvtFbmIS+9CLYvOVeUSfy/FUyrXmfpWbmcy+9Gac
+So8qs/jBCpnPXO/yS1MYT8mmGvES/r19Mn6xrvymD6fE5lm3JK39NVUyfcZ
/y63MpWLjNvzob4/E/XGvKwvo2/uxbgqO/QFzNC78Jl+EufqrXhbXyKHcFPu
Q/bmO325HJl7zDPIP+QKeYu8QvZjmn6x7MAaPjJeKoexiaK8M9mE1bzmpb1i
7hp9EBtYbjxH9uBmvs4eyT3Zwh7ZG1mNlew0vkoOYKE+U3blc/1kztNb846+
Js+v9+F7vWr2TP8z5yv7wCXGHflYPyDnRW/Kq/oN9MueGVfncX0Rp1CNInN/
ydvyvRxI/9yH+Ro8kb3iVKpnfXN/y9s5n4MYQI1c67N/5DYu4GAGUjP/zmf/
yu1M5xAGUYva1KEu9ahPAxrSiMY0oSnNaJ61rfefvIMLOZTBFGbt/Oi4M2eN
wxiSZzZXkyf1KzmNFrk2v1PuylngcIbmmc3V4qm8L06nZdYzV4q7s/ccwbCs
Z642T+tXcwatsp650tyjX8qRTDJulvOk38jwrG9ch2dyvpild+MLfXz2W9+N
d/W1tM53G5fhXv2yrK934hP9KCbrzXldXywHs0KfK3vyjb5Mjsh5zT3Luqzi
2TyHHMh6FhvPlt1Zy5fG18nhbKYk7z3nzm+kTX6PvGfuWjmUjfl9GC+UfVnH
DzmjclSeLXsjy3Jf3pHsz4w8m+zMp/rRTNELeUNfyTy9F9/qI7Jfej2e05dw
ub47X+ln5gzqbXlfX0eb7JdxOe7XZ3IMU41b8Ka+ipGUGNfn+TwbZ9E292Wu
PA/k3XEso/J95hrwgr6UCbTL+uYq8GD2leMYTftca74iD+XeKWYMHehIJzrT
ha50o3vWcE0lHtbnMI6xuUdzDXkx74yJ+d0Yt+MDfT2L8n8VP+ZMytFszb4Z
/w/+hcEh
       "]]}, 
     {Hue[0.9060679774997897, 0.6, 0.6], 
      LineBox[{433, 660, 605, 560, 526, 500, 484, 434, 435, 436, 437, 438, 
       439, 440, 441, 442, 443, 444, 661, 606, 561, 527, 501, 485, 676, 621, 
       576, 542, 516, 445, 662, 607, 562, 528, 694, 639, 594, 710, 655, 502, 
       686, 631, 717}], 
      LineBox[{586, 707, 652, 722, 552, 700, 645, 720, 600, 712, 657, 724, 
       486, 677, 622, 715, 577, 705, 650, 543, 697, 642, 597, 517, 690, 635, 
       590, 556, 702, 647, 602, 446, 663, 608, 563, 529, 695, 640, 595, 503, 
       687, 632, 587, 553, 487, 678, 623, 578, 544, 518, 447, 664, 609, 564, 
       530, 504, 488, 679, 624, 579, 545, 519, 448, 665, 610, 565, 531, 505, 
       489, 449, 666, 611, 566, 532, 506, 490, 450, 667, 612, 567, 533, 507, 
       491, 451, 452, 453, 454, 455, 456, 457, 734, 458, 459, 460, 461, 462, 
       463, 464, 668, 613, 568, 534, 508, 492, 465, 669, 614, 569, 535, 509, 
       493, 680, 625, 580, 546, 520, 466, 670, 615, 570, 536, 510, 494, 681, 
       626, 581, 547, 521, 467, 671, 616, 571, 537, 511, 495, 682, 627, 582, 
       548, 522, 468, 672, 617, 572, 538, 512, 688, 633, 588, 554, 496, 683, 
       628, 583, 549, 698, 643, 598, 523, 691, 636, 591, 557, 469, 673, 618, 
       573, 539, 696, 641, 596, 513, 689, 634, 589, 708, 653, 555, 701, 646, 
       601, 713, 658, 497, 684, 629, 716, 584, 706, 651, 721, 550, 699, 644, 
       719, 599, 711, 656, 723, 524, 692}], 
      LineBox[{637, 718, 592, 709, 654, 558, 703, 648, 603, 470, 674, 619, 
       574, 540, 514, 498, 471, 472, 473, 474, 475, 476, 477, 478, 479, 480, 
       481, 482, 675, 620, 575, 541, 515, 499, 685, 630, 585, 551, 525, 693, 
       638, 593, 559, 704, 649, 604, 714, 659, 725, 483}]}, 
     {RGBColor[1., 0.6, 0.6], LineBox[{730, 732, 729}], 
      LineBox[{731, 733, 728}]}}, {{}, {}, {}, {}}}],
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesOrigin->{0, 0},
  Method->{"AxesInFront" -> True},
  PlotRange->
   NCache[{{0, 2 Pi}, {-0.9999998592131705, 0.9999998782112116}}, {{
     0, 6.283185307179586}, {-0.9999998592131705, 0.9999998782112116}}],
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.4012326775314503`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox["\[Integral]", 
   RowBox[{"\[Pi]", "/", "2"}], 
   RowBox[{"3", " ", 
    RowBox[{"\[Pi]", "/", "2"}]}]], 
  RowBox[{
   RowBox[{"Sin", "[", "x", "]"}], " ", 
   RowBox[{"\[DifferentialD]", "x"}]}]}]], "Input"],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.401232681384036*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Fundamental Theorem of Calculus", "Section"],

Cell["\<\
The Fundamental Theorem of Calculus gives the rule used in evaluating \
definite integrals.  It relates integral and differential calculus.  It is \
the rule one uses to evaluate a definite integral.  \
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubsuperscriptBox["\[Integral]", "a", "b"], " ", 
     RowBox[{
      RowBox[{"g", "(", "x", ")"}], " ", 
      RowBox[{"\[DifferentialD]", "x"}]}]}], "=", 
    RowBox[{
     RowBox[{
      RowBox[{"G", "(", "x", ")"}], 
      SubsuperscriptBox[
       StyleBox["\[VerticalSeparator]",
        FontSize->16], "a", 
       AdjustmentBox["b",
        BoxMargins->{{0.0769231, -0.0769231}, {0, 0}}]]}], "=", 
     RowBox[{
      RowBox[{"G", "(", "b", ")"}], "-", 
      RowBox[{"G", "(", "a", ")"}]}]}]}], TraditionalForm]],
  FontSize->12],
 "  where  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["G", "\[Prime]"], "(", "x", ")"}], "=", 
    RowBox[{"g", "(", "x", ")"}]}], TraditionalForm]],
  FontSize->12]
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
The definite integral of a function is the difference of some antiderivative \
of the function evaluated at the endpoints of the interval.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Antiderivatives and Indefinite Integrals", "Section"],

Cell[TextData[{
 "The reverse of the differentiation procedure is called antidifferentiation. \
 If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " is a function then its antiderivative is another function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "(", "x", ")"}], TraditionalForm]]],
 "  that satisfies  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["\[DifferentialD]", 
      RowBox[{"\[DifferentialD]", "x"}]], 
     RowBox[{"G", "(", "x", ")"}]}], "=", 
    RowBox[{"g", "(", "x", ")"}]}], TraditionalForm]]],
 ".  Generally, antidifferentiation is a more complicated procedure than \
differentiation.  For the simple case of polynomials, however, it is quite \
simple."
}], "Text"],

Cell["\<\
The most general antiderivative is called the indefinite integral.  It is \
written like the difinite integral but without limits.\
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Integral]", " ", 
     RowBox[{
      RowBox[{"g", "(", "x", ")"}], " ", 
      RowBox[{"\[DifferentialD]", "x"}]}]}], "=", 
    RowBox[{
     RowBox[{"G", "(", "x", ")"}], "+", "C"}]}], TraditionalForm]],
  FontSize->12],
 "  where  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["G", "\[Prime]"], "(", "x", ")"}], "=", 
    RowBox[{"g", "(", "x", ")"}]}], TraditionalForm]],
  FontSize->12],
 " and ",
 StyleBox["C",
  FontSlant->"Italic"],
 " is an arbitrary constant"
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Integral]", 
  RowBox[{
   RowBox[{"Sin", "[", "x", "]"}], " ", 
   RowBox[{"\[DifferentialD]", "x"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{"Cos", "[", "x", "]"}]}]], "Output",
 CellChangeTimes->{3.401232689919622*^9}]
}, Open  ]],

Cell["\<\
Note that with Mathematica arbitrary constants are omitted from the output.\
\>", "Text"],

Cell[TextData[{
 "As an example we can use this antiderivative to evaluate ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "\[Pi]"], " ", 
    RowBox[{"sin", " ", "x", " ", 
     RowBox[{"\[DifferentialD]", "x"}]}]}], TraditionalForm]]],
 " using the Fundamental Theorem."
}], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "\[Pi]"], " ", 
    RowBox[{"sin", " ", "x", " ", 
     RowBox[{"\[DifferentialD]", "x"}]}]}], "=", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"-", " ", "cos"}], " ", "x"}], " ", 
     SubsuperscriptBox[
      StyleBox["\[VerticalSeparator]",
       FontSize->16], "0", 
      AdjustmentBox["\[Pi]",
       BoxMargins->{{0.0769231, -0.0769231}, {0, 0}}]]}], "=", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"cos", " ", "\[Pi]"}], " ", "-", 
        RowBox[{"cos", " ", "0"}]}], ")"}]}], "=", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "-", "1"}], ")"}]}], "=", "2"}]}]}]}], 
  TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]],

Cell[CellGroupData[{

Cell["Antiderivatives of Polynomials", "Section"],

Cell[TextData[{
 "Note that since the derivative of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "p"], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", " ", 
    SuperscriptBox["x", 
     RowBox[{"p", "-", "1"}]]}], TraditionalForm]]],
 ",  an antiderivative of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "p"], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox["1", 
     RowBox[{"p", "+", "1"}]], 
    SuperscriptBox["x", 
     RowBox[{"p", "+", "1"}]]}], TraditionalForm]]],
 " for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "\[NotEqual]", 
    RowBox[{"-", "1"}]}], TraditionalForm]]],
 ".  Since the derivative of the sum is the sum of derivatives and constants \
come out of derivatives, the same is true of antiderivative."
}], "Text"],

Cell["(Exercise 1)", "Subsubsection"],

Cell["\<\
The easiest way to input mathematical notation is using a \"Palette\".In the \
menu click Palettes\[Rule]BasicMathInput,  To start a blank cell, click \
between cells until you see a horizontal line.  Then you are ready to type \
input or use the Palette input.\
\>", "Text",
 CellChangeTimes->{{3.400405399479744*^9, 3.400405461714314*^9}}],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find an antiderivative of  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"6", " ", 
     SuperscriptBox["x", "2"]}], "-", 
    RowBox[{"4", " ", "x"}], "+", "9"}], TraditionalForm]]],
 ".  Use brackets around the  function in the integral as is proper \
mathematical notation."
}], "Text",
 CellChangeTimes->{{3.400405547163039*^9, 3.400405582930048*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Integral]", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"6", " ", 
      SuperscriptBox["x", "2"]}], "-", 
     RowBox[{"4", " ", "x"}], "+", "9"}], ")"}], 
   RowBox[{"\[DifferentialD]", "x"}]}]}]], "Input",
 CellChangeTimes->{{3.40040547715718*^9, 3.400405530795896*^9}, {
  3.400405591683942*^9, 3.400405599469542*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"9", " ", "x"}], "-", 
  RowBox[{"2", " ", 
   SuperscriptBox["x", "2"]}], "+", 
  RowBox[{"2", " ", 
   SuperscriptBox["x", "3"]}]}]], "Output",
 CellChangeTimes->{3.401232694990548*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Work Example", "Section"],

Cell["\<\
Suppose a force in one-dimension is given (in SI units) by the espression\
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "(", "x", ")"}], "=", 
    RowBox[{
     RowBox[{"12", " ", 
      SuperscriptBox["x", "3"]}], "-", 
     RowBox[{"150", " ", 
      SuperscriptBox["x", "2"]}], "+", 
     RowBox[{"450", " ", "x"}], "-", "180"}]}], TraditionalForm]]],
 "."
}], "Text",
 CellChangeTimes->{{3.4004056627002573`*^9, 3.400405692203561*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "As an example, we want to find the work done by this force between ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "=", 
    RowBox[{"1", " ", 
     StyleBox["m",
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "=", 
    RowBox[{"5", " ", 
     StyleBox["m",
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 ".  The following inputs the force."
}], "Text",
 CellChangeTimes->{{3.40040563056806*^9, 3.400405649535966*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"F", "[", "x_", "]"}], ":=", 
  RowBox[{
   RowBox[{"12", " ", 
    SuperscriptBox["x", "3"]}], "-", 
   RowBox[{"150", " ", 
    SuperscriptBox["x", "2"]}], "+", 
   RowBox[{"450", " ", "x"}], "-", "180"}]}]], "Input",
 CellChangeTimes->{{3.4003740093673677`*^9, 3.4003740415180063`*^9}, 
   3.4003742200649357`*^9, {3.400374306840117*^9, 3.400374328887824*^9}, {
   3.400374393428051*^9, 3.400374476941381*^9}, {3.400374543336144*^9, 
   3.40037460130906*^9}, {3.400374632552332*^9, 3.4003747452559357`*^9}, {
   3.4003747959601803`*^9, 3.400374885815928*^9}}],

Cell[TextData[{
 "The function ",
 StyleBox["WorkPlotAndShade",
  FontWeight->"Bold"],
 " plots the force and gives the area to represent the work as the area under \
the curve."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"WorkPlotAndShade", "[", 
  RowBox[{
   RowBox[{"F", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "5.5"}], " ", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "5"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.400372970843437*^9, 3.4003742610087557`*^9, {3.400374346575897*^9, 
   3.400374351365325*^9}, {3.400375043597837*^9, 3.400375044837475*^9}}],

Cell[BoxData[
 GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJztmHs4lFv7x2dGJTqgQTqXKDlVDm1KrTtyTAcqxw4TZsw8pJKiAykpRTsb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   "], {{{}, {}, 
     {RGBColor[1., 0.6, 0.6], EdgeForm[None], 
      GraphicsGroupBox[
       PolygonBox[{{624, 643, 641, 306, 307, 308, 309, 310, 311, 312, 313, 
        568, 497, 438, 391, 357, 337, 583, 512, 453, 406, 372, 314, 569, 498, 
        439, 392, 358, 592, 521, 462, 415, 606, 535, 476, 620, 549, 338, 584, 
        513}},
        VertexColors->None]]}, 
     {RGBColor[0.6, 1., 0.6], EdgeForm[None], 
      GraphicsGroupBox[
       PolygonBox[{{621, 642, 641, 305, 304, 303, 302, 301, 300, 299, 298, 
        297, 296, 295, 294, 293, 336, 356, 390, 437, 496, 567, 292, 335, 355, 
        389, 436, 495, 566, 291, 371, 405, 452, 511, 582, 334, 354, 388, 435, 
        494, 565, 290, 370, 404, 451, 510, 581, 333, 414, 461, 520, 591, 353, 
        387, 434, 493, 564, 289, 419, 466, 525, 596, 369, 403, 450, 509, 580, 
        332, 413, 460, 519, 590, 352, 470, 529, 600, 386, 433, 492, 563, 288, 
        479, 538, 609, 418, 545, 616, 465, 524, 595, 368, 547, 618, 473, 626, 
        532, 603, 402, 629, 541, 612, 449, 623, 508, 579, 331, 631, 548, 619, 
        475, 627, 534, 605, 412, 630, 543, 614, 459, 625, 518, 589, 351, 546, 
        617, 469, 528, 599, 385, 540, 611, 432, 491, 562, 287, 478, 537, 608, 
        417, 464, 523, 594, 367, 472, 531, 602, 401, 448, 507, 578, 330, 411, 
        458, 517, 588, 350, 384, 431, 490, 561, 286, 366, 400, 447, 506, 577, 
        329, 410, 457, 516, 587, 349, 383, 430, 489, 560, 285, 365, 399, 446, 
        505, 576, 328, 348, 382, 429, 488, 559, 284, 364, 398, 445, 504, 575, 
        327, 347, 381, 428, 487, 558, 283, 363, 397, 444, 503, 574, 326, 346, 
        380, 427, 486, 557, 282, 325, 345, 379, 426, 485, 556, 281, 280, 324, 
        344, 378, 425, 484, 555, 279, 632, 550}},
        VertexColors->None]]}, {}, {}, {}, {}, {}, {}}, {{}, {}, 
     {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwV03WQVWUAxuElFmRp6e4OAUnpDpFYumWRkNglJQWU7lBCkO5QOpRWUhol
lE6luyWe88dzf/N9c2funHnPzRQRFR4ZLSQkJKGPoCdZSH+aUoHCRCcGMQkl
FrH5gDj084UwjUs8TrGIATSjIkWIz2kW8zXNqURRrrKWUXSgDgk4wxIG0oLK
FOMa6xjNl9QNnoO/WcogWlKF4lxnPWPoSDiJ+IdlDKYVVbnLVr6jOyW4wQbG
0ol6JOYsy/mGz6nGPbbxPT34hH/ZyDg684J9zKY+j9jFdHrzmj+Yx4ecYwXf
8pbDtOYZe/gx2MmIx7U699ke7ORuigYDHw020p76igPBfs5zg231z2A/Lan/
sSnY2nm8vuMIYc5d9CX7g3fAeY7G0BPagMf8xg/Bz7o/pn34n4PMJwnnWckQ
InjOXmZRgwfsYCq9KMVNNjOBSBryhN+ZQV+ScoGfGEobPuUhO5nGV5TmFr8w
kSgakYyL/MwwvqAmZbjNr0yiK41JziVWMZy2fEZZUnCZ1YygHbUoR0pSkZo0
pCUd6clARjKRmSxkJRvZyUFOcpGbPOQlH/39H/PrRxSgIIW4whpG0p7alOcO
W5hMN5rwlN3MpB9vOMQCQm36l8bXj/U9XdiCpg==
       "]]}, 
     {Hue[0.9060679774997897, 0.6, 0.6], 
      LineBox[{270, 552, 481, 422, 375, 341, 321, 271, 272, 273, 274, 275, 
       276, 277, 553, 482, 423, 376, 342, 322, 572, 501, 442, 395, 361, 278, 
       554, 483, 424, 377, 343, 586, 515, 456, 409, 323, 573, 502, 443, 396, 
       601, 530, 471, 362, 593, 522, 463, 615, 544, 416, 607, 536, 628}], 
      LineBox[CompressedData["
1:eJwV00VsVVEUBdBfgksCgSClpRR3Kc4Ad6eQ4FYkuFvw4hNcBmiAEe4wwWWA
BkggEAhMcIcJLusMVve+5//2vXffbXrWpMyJSYlEIr8fE3IkEpWYTQnr6vK5
3Msc+tKGkpSihs9fyH3MpR9tSaamz17K/cyjP+1oZv5NnmEV4yhNLfNX8gDz
GUB7mpt/l2dZzXhSqG3+Wh5kAQPpQAvzH/Ica+KZSKWO+Rt5iIUMorNZTq7p
W+hIS+uf8jxric0pQ13zt/IwixhMF7NcXNe30olW1r/kBdbFc1sX5aE+iUy9
AHf0XQzVS/FUTyNDfyePMFxPIVvvIwtzXx8jyzFE7ypzM4Ub1qNkGjPZFs8p
izONx7HfsgKz6GzdWv6Oe5MX5QiZygzWx3uQxZjKI+uxsjyT9V6yIHf10TKd
3fowmcwzvSz19PfyKIvjHFkX4UE8P930PNzUt8e+xjmz/iMvsYEp9DYrxD19
D+nUt/4gj7Ekrk13s7zc0nfEHsW5tP4rL7ORqZSjgflHeZylZMX9xFk1/yev
sIlplKeh+Sd5gmXxjuJ6VKCRzz7LkyyPvaQHFalEZapQlWpUpwY14/zHmY5z
GmeNbH8rI/YunjHuM64d16AxTSJ954s8xQpG0pP25gmu6puZTtP4HbOv8jQr
46zE/Znl47a+M851/O/zRB8vK5Kpd5BJ8d75D7+jdkA=
       "]], 
      LineBox[{454, 613, 542, 407, 604, 533, 474, 373, 597, 526, 467, 420, 
       315, 570, 499, 440, 393, 359, 339, 316, 317, 318, 319, 571, 500, 441, 
       394, 360, 340, 585, 514, 455, 408, 374, 598, 527, 468, 421, 610, 539, 
       480, 622, 551, 633, 320}]}, 
     {RGBColor[1., 0.6, 0.6], LineBox[{638, 639, 637}], 
      LineBox[{636, 640, 477}]}}, {{}, {}, {}, {}}}],
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox["\"x (m)\"", TraditionalForm], 
    FormBox["\"F(x) (N)\"", TraditionalForm]},
  AxesOrigin->{0, 0},
  Method->{"AxesInFront" -> True},
  PlotRange->{{0, 5.5}, {-245.9999875408157, 216.11466647767242`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.401232704476215*^9}]
}, Open  ]],

Cell["To evaluate the work do the following integral.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox["\[Integral]", "1", "5"], 
  RowBox[{
   RowBox[{"F", "[", "x", "]"}], " ", 
   RowBox[{"\[DifferentialD]", "x"}]}]}]], "Input",
 CellChangeTimes->{{3.400376683440263*^9, 3.4003766886695023`*^9}}],

Cell[BoxData["352"], "Output",
 CellChangeTimes->{3.4012327113982897`*^9}]
}, Open  ]],

Cell["\<\
To illustrate the Fundamental Theorem we will find the antiderivative and \
evaluate it between the limits.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"G", "[", "x_", "]"}], "=", 
  RowBox[{"\[Integral]", 
   RowBox[{
    RowBox[{"F", "[", "x", "]"}], " ", 
    RowBox[{"\[DifferentialD]", "x"}]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "180"}], " ", "x"}], "+", 
  RowBox[{"225", " ", 
   SuperscriptBox["x", "2"]}], "-", 
  RowBox[{"50", " ", 
   SuperscriptBox["x", "3"]}], "+", 
  RowBox[{"3", " ", 
   SuperscriptBox["x", "4"]}]}]], "Output",
 CellChangeTimes->{3.4012327147873096`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"G", "[", "5", "]"}], "-", 
  RowBox[{"G", "[", "1", "]"}]}]], "Input",
 CellChangeTimes->{{3.4003766986958113`*^9, 3.4003767030455837`*^9}}],

Cell[BoxData["352"], "Output",
 CellChangeTimes->{3.401232718225821*^9}]
}, Open  ]],

Cell["(Exercise 2)", "Subsubsection"],

Cell["\<\
Now consider the example with the force (in SI units) given by \
\>", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"F", "(", "x", ")"}], "=", 
   RowBox[{
    RowBox[{"2", " ", 
     SuperscriptBox["x", "3"]}], "-", 
    RowBox[{"18", " ", 
     SuperscriptBox["x", "2"]}], "+", 
    RowBox[{"24", " ", "x"}], "+", "30"}]}], TraditionalForm]]]], "Text",
 CellChangeTimes->{{3.40040623854467*^9, 3.400406246139043*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "We want the work done by this force when moving ",
 "between ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "=", 
    RowBox[{"1", " ", 
     StyleBox["m",
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "=", 
    RowBox[{"4", " ", 
     StyleBox["m",
      FontSlant->"Plain"]}]}], TraditionalForm]]]
}], "Text"],

Cell[TextData[{
 "Input the function F[x] as in the example above.  (It is easiest to paste \
the previous  ",
 StyleBox["F[x_] :=",
  FontWeight->"Bold"],
 "  input and modify.)"
}], "Text",
 CellChangeTimes->{{3.40040578062849*^9, 3.4004057942902308`*^9}, {
  3.400405888629249*^9, 3.4004059351495667`*^9}}],

Cell[TextData[{
 "Use ",
 StyleBox["WorkPlotAndShade",
  FontWeight->"Bold"],
 " to display this function between ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "=", "0"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "=", 
    RowBox[{"5", " ", 
     StyleBox["m",
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 " and show the area representing the work."
}], "Text"],

Cell[TextData[{
 "By hand, find the antiderivative of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "(", "x", ")"}], TraditionalForm]]],
 " and use the Fundamental Theorem to evaluate the integral."
}], "Text",
 CellChangeTimes->{{3.400405817866866*^9, 3.4004058236188583`*^9}}]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowToolbars->{},
InitializationCellEvaluation->True,
WindowSize->{958, 722},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
ShowSelection->True,
DefaultNewCellStyle->"Input",
FontFamily->"Times",
Magnification->2.,
FrontEndVersion->"6.0 for Mac OS X x86 (32-bit) (June 19, 2007)",
StyleDefinitions->Notebook[{
   Cell[
    CellGroupData[{
      Cell["Style Definitions", "Title"], 
      Cell[
      "Modify the definitions below to change the default appearance of all \
cells in a given style.  Make modifications to any definition using commands \
in the Format menu.", "Text"], 
      Cell[
       CellGroupData[{
         Cell["Style Environment Names", "Section"], 
         Cell[
          StyleData[All, "Working"], PageWidth -> WindowWidth, ScriptMinSize -> 
          9], 
         Cell[
          StyleData[All, "Presentation"], PageWidth -> WindowWidth, 
          ScriptMinSize -> 9], 
         Cell[
          StyleData[All, "SlideShow"], PageWidth -> WindowWidth, 
          ScrollingOptions -> {
           "PagewiseDisplay" -> True, "VerticalScrollRange" -> Fit}, 
          ShowCellBracket -> False, ScriptMinSize -> 9], 
         Cell[
          StyleData[All, "Printout"], PageWidth -> PaperWidth, ScriptMinSize -> 
          7, FontSize -> 10, 
          PrivateFontOptions -> {"FontType" -> "Outline"}]}, Open]], 
      Cell[
       CellGroupData[{
         Cell["Notebook Options", "Section"], 
         Cell[
         "The options defined for the style below will be used at the \
Notebook level.", "Text"], 
         Cell[
          StyleData["Notebook"], PageHeaders -> {{
             Cell[
              TextData[{
                CounterBox["Page"]}], "PageNumber"], None, 
             Cell[
              TextData[{
                ValueBox["FileName"]}], "Header"]}, {
             Cell[
              TextData[{
                ValueBox["FileName"]}], "Header"], None, 
             Cell[
              TextData[{
                CounterBox["Page"]}], "PageNumber"]}}, 
          PrintingOptions -> {
           "FirstPageFace" -> Right, "FirstPageHeader" -> False}, 
          StyleMenuListing -> None, Background -> 
          RGBColor[0.900008, 0.863996, 0.809995]]}, Open]], 
      Cell[
       CellGroupData[{
         Cell["Styles for Headings", "Section"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Title"], ShowCellBracket -> False, 
             CellMargins -> {{0, 0}, {0, 0}}, PageBreakBelow -> False, 
             InputAutoReplacements -> {"TeX" -> StyleBox[
                 RowBox[{"T", 
                   AdjustmentBox[
                   "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, 
                    BoxBaselineShift -> 0.5], "X"}]], "LaTeX" -> StyleBox[
                 RowBox[{"L", 
                   StyleBox[
                    AdjustmentBox[
                    "A", BoxMargins -> {{-0.36, -0.1}, {0, 0}}, 
                    BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
                   AdjustmentBox[
                   "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, 
                    BoxBaselineShift -> 0.5], "X"}]], "mma" -> "Mathematica", 
               "Mma" -> "Mathematica", "MMA" -> "Mathematica", 
               "gridMathematica" -> FormBox[
                 RowBox[{"grid", 
                   AdjustmentBox[
                    StyleBox["Mathematica", FontSlant -> "Italic"], 
                    BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
               "webMathematica" -> FormBox[
                 RowBox[{"web", 
                   AdjustmentBox[
                    StyleBox["Mathematica", FontSlant -> "Italic"], 
                    BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
               Inherited}, TextAlignment -> Center, TextJustification -> 0, 
             LineSpacing -> {1, 13}, 
             CounterAssignments -> {{"Section", 0}, {"Equation", 0}, {
               "Figure", 0}}, FontFamily -> "Helvetica", FontSize -> 34, 
             FontWeight -> "Bold", FontColor -> GrayLevel[1], Background -> 
             RGBColor[0.0500038, 0.100008, 0.2]], 
            Cell[
             StyleData["Title", "Presentation"], 
             CellFrameMargins -> {{14, 10}, {15, 15}}, FontSize -> 51], 
            Cell[
             StyleData["Title", "Printout"], CellMargins -> {{0, 0}, {0, 0}}, 
             FontSize -> 24, Background -> GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Subtitle"], ShowCellBracket -> False, 
             CellMargins -> {{0, 0}, {0, 0}}, PageBreakBelow -> False, 
             InputAutoReplacements -> {"TeX" -> StyleBox[
                 RowBox[{"T", 
                   AdjustmentBox[
                   "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, 
                    BoxBaselineShift -> 0.5], "X"}]], "LaTeX" -> StyleBox[
                 RowBox[{"L", 
                   StyleBox[
                    AdjustmentBox[
                    "A", BoxMargins -> {{-0.36, -0.1}, {0, 0}}, 
                    BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
                   AdjustmentBox[
                   "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, 
                    BoxBaselineShift -> 0.5], "X"}]], "mma" -> "Mathematica", 
               "Mma" -> "Mathematica", "MMA" -> "Mathematica", 
               "gridMathematica" -> FormBox[
                 RowBox[{"grid", 
                   AdjustmentBox[
                    StyleBox["Mathematica", FontSlant -> "Italic"], 
                    BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
               "webMathematica" -> FormBox[
                 RowBox[{"web", 
                   AdjustmentBox[
                    StyleBox["Mathematica", FontSlant -> "Italic"], 
                    BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
               Inherited}, TextAlignment -> Center, TextJustification -> 0, 
             LineSpacing -> {1, 3}, ParagraphIndent -> -96, 
             CounterAssignments -> {{"Section", 0}, {"Equation", 0}, {
               "Figure", 0}}, FontFamily -> "Helvetica", FontSize -> 24, 
             FontColor -> GrayLevel[1], Background -> 
             RGBColor[0.250004, 0.400015, 0.6]], 
            Cell[
             StyleData["Subtitle", "Presentation"], 
             CellFrameMargins -> {{14, 10}, {14, 14}}, FontSize -> 27], 
            Cell[
             StyleData["Subtitle", "Printout"], 
             CellMargins -> {{0, 0}, {0, 0}}, ParagraphIndent -> -85, 
             FontSize -> 16]}, Open]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Subsubtitle"], ShowCellBracket -> False, 
             CellMargins -> {{0, 0}, {0, 0}}, PageBreakBelow -> False, 
             InputAutoReplacements -> {"TeX" -> StyleBox[
                 RowBox[{"T", 
                   AdjustmentBox[
                   "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, 
                    BoxBaselineShift -> 0.5], "X"}]], "LaTeX" -> StyleBox[
                 RowBox[{"L", 
                   StyleBox[
                    AdjustmentBox[
                    "A", BoxMargins -> {{-0.36, -0.1}, {0, 0}}, 
                    BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
                   AdjustmentBox[
                   "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, 
                    BoxBaselineShift -> 0.5], "X"}]], "mma" -> "Mathematica", 
               "Mma" -> "Mathematica", "MMA" -> "Mathematica", 
               "gridMathematica" -> FormBox[
                 RowBox[{"grid", 
                   AdjustmentBox[
                    StyleBox["Mathematica", FontSlant -> "Italic"], 
                    BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
               "webMathematica" -> FormBox[
                 RowBox[{"web", 
                   AdjustmentBox[
                    StyleBox["Mathematica", FontSlant -> "Italic"], 
                    BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
               Inherited}, TextAlignment -> Right, TextJustification -> 0, 
             CounterAssignments -> {{"Section", 0}, {"Equation", 0}, {
               "Figure", 0}}, FontFamily -> "Helvetica", FontSize -> 16, 
             FontSlant -> "Italic", Background -> 
             RGBColor[0.624994, 0.700008, 0.8]], 
            Cell[
             StyleData["Subsubtitle", "Presentation"], 
             CellMargins -> {{0, 0}, {0, 0}}, FontSize -> 24], 
            Cell[
             StyleData["Subsubtitle", "Printout"], 
             CellMargins -> {{0, 0}, {0, 0}}, FontSize -> 16]}, Open]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Section"], CellFrame -> {{0, 0}, {0, 1.5}}, 
             CellDingbat -> None, CellMargins -> {{12, Inherited}, {4, 24}}, 
             CellGroupingRules -> {"SectionGrouping", 30}, PageBreakBelow -> 
             False, CellFrameMargins -> {{0, 4}, {6, 2}}, CellFrameColor -> 
             RGBColor[0.0500038, 0.100008, 0.2], CellFrameLabelMargins -> 2, 
             InputAutoReplacements -> {"TeX" -> StyleBox[
                 RowBox[{"T", 
                   AdjustmentBox[
                   "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, 
                    BoxBaselineShift -> 0.5], "X"}]], "LaTeX" -> StyleBox[
                 RowBox[{"L", 
                   StyleBox[
                    AdjustmentBox[
                    "A", BoxMargins -> {{-0.36, -0.1}, {0, 0}}, 
                    BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
                   AdjustmentBox[
                   "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, 
                    BoxBaselineShift -> 0.5], "X"}]], "mma" -> "Mathematica", 
               "Mma" -> "Mathematica", "MMA" -> "Mathematica", 
               "gridMathematica" -> FormBox[
                 RowBox[{"grid", 
                   AdjustmentBox[
                    StyleBox["Mathematica", FontSlant -> "Italic"], 
                    BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
               "webMathematica" -> FormBox[
                 RowBox[{"web", 
                   AdjustmentBox[
                    StyleBox["Mathematica", FontSlant -> "Italic"], 
                    BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
               Inherited}, CounterIncrements -> "Section", 
             CounterAssignments -> {{"Subsection", 0}, {"Subsubsection", 0}}, 
             FontFamily -> "Helvetica", FontSize -> 18, FontWeight -> "Bold", 
             FontColor -> RGBColor[0.0500038, 0.100008, 0.2]], 
            Cell[
             StyleData["Section", "Presentation"], 
             CellFrame -> {{0, 0}, {0, 2}}, 
             CellMargins -> {{14, 10}, {6, 24}}, FontSize -> 27], 
            Cell[
             StyleData["Section", "Printout"], 
             CellMargins -> {{9, 0}, {2, 50}}, FontSize -> 14, FontTracking -> 
             "Plain", FontColor -> GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Subsection"], CellMargins -> {{12, 10}, {25, 0}}, 
             InputAutoReplacements -> {"TeX" -> StyleBox[
                 RowBox[{"T", 
                   AdjustmentBox[
                   "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, 
                    BoxBaselineShift -> 0.5], "X"}]], "LaTeX" -> StyleBox[
                 RowBox[{"L", 
                   StyleBox[
                    AdjustmentBox[
                    "A", BoxMargins -> {{-0.36, -0.1}, {0, 0}}, 
                    BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
                   AdjustmentBox[
                   "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, 
                    BoxBaselineShift -> 0.5], "X"}]], "mma" -> "Mathematica", 
               "Mma" -> "Mathematica", "MMA" -> "Mathematica", 
               "gridMathematica" -> FormBox[
                 RowBox[{"grid", 
                   AdjustmentBox[
                    StyleBox["Mathematica", FontSlant -> "Italic"], 
                    BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
               "webMathematica" -> FormBox[
                 RowBox[{"web", 
                   AdjustmentBox[
                    StyleBox["Mathematica", FontSlant -> "Italic"], 
                    BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
               Inherited}, LineSpacing -> {1, 3}, ParagraphSpacing -> {0, 12},
              FontFamily -> "Helvetica"], 
            Cell[
             StyleData["Subsection", "Presentation"], 
             CellMargins -> {{14, 10}, {45, 4}}, FontSize -> 18], 
            Cell[
             StyleData["Subsection", "Printout"], 
             CellMargins -> {{9, 0}, {4, 40}}, FontSize -> 12]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Subsubsection"], CellMargins -> {{12, 10}, {5, 20}}, 
             CellFrameMargins -> {{0, 0}, {0, 2}}, CellFrameColor -> 
             RGBColor[0.100008, 0.2, 0.4], 
             InputAutoReplacements -> {"TeX" -> StyleBox[
                 RowBox[{"T", 
                   AdjustmentBox[
                   "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, 
                    BoxBaselineShift -> 0.5], "X"}]], "LaTeX" -> StyleBox[
                 RowBox[{"L", 
                   StyleBox[
                    AdjustmentBox[
                    "A", BoxMargins -> {{-0.36, -0.1}, {0, 0}}, 
                    BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
                   AdjustmentBox[
                   "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, 
                    BoxBaselineShift -> 0.5], "X"}]], "mma" -> "Mathematica", 
               "Mma" -> "Mathematica", "MMA" -> "Mathematica", 
               "gridMathematica" -> FormBox[
                 RowBox[{"grid", 
                   AdjustmentBox[
                    StyleBox["Mathematica", FontSlant -> "Italic"], 
                    BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
               "webMathematica" -> FormBox[
                 RowBox[{"web", 
                   AdjustmentBox[
                    StyleBox["Mathematica", FontSlant -> "Italic"], 
                    BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
               Inherited}, LineSpacing -> {1, 0}, ParagraphSpacing -> {0, 12},
              FontFamily -> "Helvetica", FontSize -> 14, FontWeight -> "Bold",
              FontSlant -> "Italic", FontColor -> 
             RGBColor[0.100008, 0.2, 0.4]], 
            Cell[
             StyleData["Subsubsection", "Presentation"], 
             CellMargins -> {{14, 10}, {8, 20}}, 
             CellFrameMargins -> {{0, 0}, {0, 4}}, FontSize -> 22], 
            Cell[
             StyleData["Subsubsection", "Printout"], 
             CellMargins -> {{19, 0}, {4, 20}}, FontSize -> 13]}, Open]]}, 
        Open]], 
      Cell[
       CellGroupData[{
         Cell["Styles for Body Text", "Section"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Text"], CellMargins -> {{12, 10}, {5, 5}}, 
             InputAutoReplacements -> {"TeX" -> StyleBox[
                 RowBox[{"T", 
                   AdjustmentBox[
                   "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, 
                    BoxBaselineShift -> 0.5], "X"}]], "LaTeX" -> StyleBox[
                 RowBox[{"L", 
                   StyleBox[
                    AdjustmentBox[
                    "A", BoxMargins -> {{-0.36, -0.1}, {0, 0}}, 
                    BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
                   AdjustmentBox[
                   "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, 
                    BoxBaselineShift -> 0.5], "X"}]], "mma" -> "Mathematica", 
               "Mma" -> "Mathematica", "MMA" -> "Mathematica", 
               "gridMathematica" -> FormBox[
                 RowBox[{"grid", 
                   AdjustmentBox[
                    StyleBox["Mathematica", FontSlant -> "Italic"], 
                    BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
               "webMathematica" -> FormBox[
                 RowBox[{"web", 
                   AdjustmentBox[
                    StyleBox["Mathematica", FontSlant -> "Italic"], 
                    BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
               Inherited}, LineSpacing -> {1, 3}, ParagraphSpacing -> {0, 12},
              FontFamily -> "Times"], 
            Cell[
             StyleData["Text", "Presentation"], 
             CellMargins -> {{14, 10}, {8, 10}}, FontSize -> 18], 
            Cell[
             StyleData["Text", "Printout"], CellMargins -> {{9, 0}, {4, 4}}, 
             Hyphenation -> True, ParagraphSpacing -> {0, 6}, FontSize -> 
             10]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Commentary"], CellMargins -> {{52, 10}, {0, 10}}, 
             PageBreakBelow -> False, CellFrameMargins -> {{0, 0}, {0, 2}}, 
             CellFrameColor -> GrayLevel[1], LineSpacing -> {1, 0}, 
             ParagraphSpacing -> {0, 12}, FontFamily -> "Helvetica", FontSize -> 
             10, FontWeight -> "Plain", FontSlant -> "Plain"], 
            Cell[
             StyleData["Commentary", "Presentation"], 
             CellMargins -> {{72, 10}, {4, 14}}, FontSize -> 15], 
            Cell[
             StyleData["Commentary", "Printout"], 
             CellMargins -> {{9, 0}, {4, 4}}, Hyphenation -> True, 
             ParagraphSpacing -> {0, 6}, FontSize -> 10]}, Closed]]}, Open]], 
      
      Cell[
       CellGroupData[{
         Cell["Inline Formatting", "Section"], 
         Cell[
         "These styles are for modifying individual words or letters in a \
cell exclusive of the cell tag.", "Text"], 
         Cell[
          StyleData["RM"], StyleMenuListing -> None, FontWeight -> "Plain", 
          FontSlant -> "Plain"], 
         Cell[
          StyleData["BF"], StyleMenuListing -> None, FontWeight -> "Bold"], 
         Cell[
          StyleData["IT"], StyleMenuListing -> None, FontSlant -> "Italic"], 
         Cell[
          StyleData["TR"], StyleMenuListing -> None, FontFamily -> "Times", 
          FontWeight -> "Plain", FontSlant -> "Plain"], 
         Cell[
          StyleData["TI"], StyleMenuListing -> None, FontFamily -> "Times", 
          FontWeight -> "Plain", FontSlant -> "Italic"], 
         Cell[
          StyleData["TB"], StyleMenuListing -> None, FontFamily -> "Times", 
          FontWeight -> "Bold", FontSlant -> "Plain"], 
         Cell[
          StyleData["TBI"], StyleMenuListing -> None, FontFamily -> "Times", 
          FontWeight -> "Bold", FontSlant -> "Italic"], 
         Cell[
          StyleData["MR"], "TwoByteSyntaxCharacterAutoReplacement" -> True, 
          HyphenationOptions -> {"HyphenationCharacter" -> "\[Continuation]"},
           StyleMenuListing -> None, FontFamily -> "Courier", FontWeight -> 
          "Plain", FontSlant -> "Plain"], 
         Cell[
          StyleData["MO"], "TwoByteSyntaxCharacterAutoReplacement" -> True, 
          HyphenationOptions -> {"HyphenationCharacter" -> "\[Continuation]"},
           StyleMenuListing -> None, FontFamily -> "Courier", FontWeight -> 
          "Plain", FontSlant -> "Italic"], 
         Cell[
          StyleData["MB"], "TwoByteSyntaxCharacterAutoReplacement" -> True, 
          HyphenationOptions -> {"HyphenationCharacter" -> "\[Continuation]"},
           StyleMenuListing -> None, FontFamily -> "Courier", FontWeight -> 
          "Bold", FontSlant -> "Plain"], 
         Cell[
          StyleData["MBO"], "TwoByteSyntaxCharacterAutoReplacement" -> True, 
          HyphenationOptions -> {"HyphenationCharacter" -> "\[Continuation]"},
           StyleMenuListing -> None, FontFamily -> "Courier", FontWeight -> 
          "Bold", FontSlant -> "Italic"], 
         Cell[
          StyleData["SR"], StyleMenuListing -> None, FontFamily -> 
          "Helvetica", FontWeight -> "Plain", FontSlant -> "Plain"], 
         Cell[
          StyleData["SO"], StyleMenuListing -> None, FontFamily -> 
          "Helvetica", FontWeight -> "Plain", FontSlant -> "Italic"], 
         Cell[
          StyleData["SB"], StyleMenuListing -> None, FontFamily -> 
          "Helvetica", FontWeight -> "Bold", FontSlant -> "Plain"], 
         Cell[
          StyleData["SBO"], StyleMenuListing -> None, FontFamily -> 
          "Helvetica", FontWeight -> "Bold", FontSlant -> "Italic"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["SO10"], StyleMenuListing -> None, FontFamily -> 
             "Helvetica", FontSize -> 10, FontWeight -> "Plain", FontSlant -> 
             "Italic"], 
            Cell[
             StyleData["SO10", "Printout"], StyleMenuListing -> None, 
             FontFamily -> "Helvetica", FontSize -> 7, FontWeight -> "Plain", 
             FontSlant -> "Italic"]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Inert"], StyleMenuListing -> None, Background -> 
             RGBColor[0.870588, 0.905882, 0.972549]], 
            Cell[
             StyleData["Inert", "Printout"], StyleMenuListing -> None, 
             Background -> GrayLevel[1]]}, Closed]]}, Closed]], 
      Cell[
       CellGroupData[{
         Cell["Styles for Input/Output", "Section"], 
         Cell[
         "The cells in this section define styles used for input and output \
to the kernel.  Be careful when modifying, renaming, or removing these \
styles, because the front end associates special meanings with these style \
names.", "Text"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Input"], CellFrame -> 4, 
             CellMargins -> {{62, 10}, {0, 2}}, Evaluatable -> True, 
             CellGroupingRules -> "InputGrouping", CellHorizontalScrolling -> 
             True, PageBreakWithin -> False, GroupPageBreakWithin -> False, 
             CellLabelMargins -> {{21, Inherited}, {Inherited, Inherited}}, 
             CellFrameColor -> RGBColor[0.8, 0.700008, 0.6], 
             DefaultFormatType -> DefaultInputFormatType, 
             "TwoByteSyntaxCharacterAutoReplacement" -> True, 
             HyphenationOptions -> {
              "HyphenationCharacter" -> "\[Continuation]"}, LanguageCategory -> 
             "Formula", FormatType -> InputForm, ShowStringCharacters -> True,
              NumberMarks -> True, 
             LinebreakAdjustments -> {0.85, 2, 10, 0, 1}, FontWeight -> 
             "Bold", Background -> GrayLevel[1]], 
            Cell[
             StyleData["Input", "Presentation"], 
             CellFrame -> {{6, 6}, {0, 6}}, CellMargins -> {{72, 10}, {0, 2}},
              CellFrameMargins -> 12, FontSize -> 18], 
            Cell[
             StyleData["Input", "Printout"], CellMargins -> {{37, 0}, {6, 6}},
              LinebreakAdjustments -> {0.85, 2, 10, 1, 1}]}, Open]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["InlineInput"], Evaluatable -> True, CellGroupingRules -> 
             "InputGrouping", CellHorizontalScrolling -> True, 
             PageBreakWithin -> False, GroupPageBreakWithin -> False, 
             DefaultFormatType -> DefaultInputFormatType, 
             "TwoByteSyntaxCharacterAutoReplacement" -> True, 
             HyphenationOptions -> {
              "HyphenationCharacter" -> "\[Continuation]"}, 
             AutoItalicWords -> {}, FormatType -> InputForm, 
             ShowStringCharacters -> True, NumberMarks -> True, 
             CounterIncrements -> "Input", FontWeight -> "Bold"], 
            Cell[
             StyleData["InlineInput", "Presentation"], 
             CellMargins -> {{10, 10}, {4, 4}}, FontSize -> 18]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Output"], CellFrame -> {{4, 4}, {4, 0}}, 
             CellMargins -> {{62, 10}, {12, 0}}, CellEditDuplicate -> True, 
             CellGroupingRules -> "OutputGrouping", CellHorizontalScrolling -> 
             True, PageBreakWithin -> False, GroupPageBreakWithin -> False, 
             GeneratedCell -> True, CellAutoOverwrite -> True, 
             CellLabelMargins -> {{21, Inherited}, {Inherited, Inherited}}, 
             CellFrameColor -> RGBColor[0.8, 0.700008, 0.6], 
             DefaultFormatType -> DefaultOutputFormatType, 
             "TwoByteSyntaxCharacterAutoReplacement" -> True, 
             HyphenationOptions -> {
              "HyphenationCharacter" -> "\[Continuation]"}, LanguageCategory -> 
             "Formula", FormatType -> InputForm, Background -> GrayLevel[1]], 
            
            Cell[
             StyleData["Output", "Presentation"], 
             CellFrame -> {{6, 6}, {6, 0}}, 
             CellMargins -> {{72, 10}, {20, 2}}, CellFrameMargins -> 12, 
             FontSize -> 18], 
            Cell[
             StyleData["Output", "Printout"], 
             CellMargins -> {{37, 0}, {6, 2}}]}, Open]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["InputOnly"], CellFrame -> 4, 
             CellMargins -> {{62, 10}, {4, 2}}, Evaluatable -> True, 
             CellGroupingRules -> "NormalGrouping", CellHorizontalScrolling -> 
             True, PageBreakWithin -> False, GroupPageBreakWithin -> False, 
             CellLabelMargins -> {{21, Inherited}, {Inherited, Inherited}}, 
             CellFrameColor -> RGBColor[0.8, 0.700008, 0.6], 
             DefaultFormatType -> DefaultInputFormatType, 
             "TwoByteSyntaxCharacterAutoReplacement" -> True, 
             HyphenationOptions -> {
              "HyphenationCharacter" -> "\[Continuation]"}, LanguageCategory -> 
             "Formula", FormatType -> InputForm, ShowStringCharacters -> True,
              NumberMarks -> True, 
             LinebreakAdjustments -> {0.85, 2, 10, 0, 1}, FontWeight -> 
             "Bold", Background -> GrayLevel[1]], 
            Cell[
             StyleData["InputOnly", "Presentation"], CellFrame -> 6, 
             CellMargins -> {{72, 10}, {5, 2}}, CellFrameMargins -> 12, 
             FontSize -> 18], 
            Cell[
             StyleData["InputOnly", "Printout"], 
             LinebreakAdjustments -> {0.85, 2, 10, 1, 1}]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["OutputOnly"], CellFrame -> 4, 
             CellMargins -> {{62, 10}, {4, 2}}, CellEditDuplicate -> True, 
             CellGroupingRules -> "OutputGrouping", CellHorizontalScrolling -> 
             True, PageBreakWithin -> False, GroupPageBreakWithin -> False, 
             GeneratedCell -> True, CellAutoOverwrite -> True, 
             CellLabelMargins -> {{21, Inherited}, {Inherited, Inherited}}, 
             CellFrameColor -> RGBColor[0.8, 0.700008, 0.6], 
             DefaultFormatType -> DefaultOutputFormatType, 
             "TwoByteSyntaxCharacterAutoReplacement" -> True, 
             HyphenationOptions -> {
              "HyphenationCharacter" -> "\[Continuation]"}, LanguageCategory -> 
             "Formula", FormatType -> InputForm, Background -> GrayLevel[1]], 
            
            Cell[
             StyleData["OutputOnly", "Presentation"], CellFrame -> 6, 
             CellMargins -> {{72, 10}, {4, 4}}, CellFrameMargins -> 12, 
             FontSize -> 18], 
            Cell[
             StyleData["OutputOnly", "Printout"]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["SymbolicExpression"], CellFrame -> 1, 
             CellMargins -> {{62, 10}, {4, 2}}, CellEditDuplicate -> True, 
             CellGroupingRules -> "OutputGrouping", CellHorizontalScrolling -> 
             True, PageBreakWithin -> False, GroupPageBreakWithin -> False, 
             GeneratedCell -> True, CellAutoOverwrite -> True, 
             CellLabelMargins -> {{21, Inherited}, {Inherited, Inherited}}, 
             CellFrameColor -> GrayLevel[0], DefaultFormatType -> 
             DefaultOutputFormatType, "TwoByteSyntaxCharacterAutoReplacement" -> 
             True, HyphenationOptions -> {
              "HyphenationCharacter" -> "\[Continuation]"}, FormatType -> 
             InputForm, FontFamily -> "Courier", FontWeight -> "Plain", 
             FontSlant -> "Plain", FontTracking -> "Plain", 
             FontVariations -> {
              "Outline" -> False, "Shadow" -> False, "Underline" -> False}, 
             Background -> GrayLevel[1]], 
            Cell[
             StyleData["SymbolicExpression", "Presentation"], CellFrame -> 2, 
             CellMargins -> {{72, 10}, {4, 4}}, CellFrameMargins -> 12, 
             FontSize -> 18], 
            Cell[
             StyleData["SymbolicExpression", "Printout"]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Message"], 
             CellMargins -> {{52, Inherited}, {Inherited, Inherited}}, 
             CellGroupingRules -> "OutputGrouping", PageBreakWithin -> False, 
             GroupPageBreakWithin -> False, GeneratedCell -> True, 
             CellAutoOverwrite -> True, ShowCellLabel -> False, 
             CellLabelMargins -> {{21, Inherited}, {Inherited, Inherited}}, 
             DefaultFormatType -> DefaultOutputFormatType, 
             "TwoByteSyntaxCharacterAutoReplacement" -> True, 
             HyphenationOptions -> {
              "HyphenationCharacter" -> "\[Continuation]"}, FormatType -> 
             InputForm, StyleMenuListing -> None, FontColor -> 
             RGBColor[1, 0, 0]], 
            Cell[
             StyleData["Message", "Presentation"], 
             CellMargins -> {{72, 10}, {6, 6}}, FontSize -> 18], 
            Cell[
             StyleData["Message", "Printout"], 
             CellMargins -> {{30, Inherited}, {Inherited, Inherited}}, 
             FontColor -> GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Print"], 
             CellMargins -> {{52, Inherited}, {Inherited, Inherited}}, 
             CellGroupingRules -> "OutputGrouping", CellHorizontalScrolling -> 
             True, PageBreakWithin -> False, GroupPageBreakWithin -> False, 
             GeneratedCell -> True, CellAutoOverwrite -> True, ShowCellLabel -> 
             False, CellLabelMargins -> {{21, Inherited}, {
               Inherited, Inherited}}, DefaultFormatType -> 
             DefaultOutputFormatType, "TwoByteSyntaxCharacterAutoReplacement" -> 
             True, HyphenationOptions -> {
              "HyphenationCharacter" -> "\[Continuation]"}, FormatType -> 
             InputForm, StyleMenuListing -> None], 
            Cell[
             StyleData["Print", "Presentation"], 
             CellMargins -> {{72, 10}, {6, 6}}, FontSize -> 18], 
            Cell[
             StyleData["Print", "Printout"], 
             CellMargins -> {{30, Inherited}, {Inherited, Inherited}}]}, 
           Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Info"], 
             CellMargins -> {{52, Inherited}, {Inherited, Inherited}}, 
             CellGroupingRules -> "OutputGrouping", CellHorizontalScrolling -> 
             True, PageBreakWithin -> False, GroupPageBreakWithin -> False, 
             GeneratedCell -> True, CellAutoOverwrite -> True, ShowCellLabel -> 
             False, CellLabelMargins -> {{21, Inherited}, {
               Inherited, Inherited}}, DefaultFormatType -> 
             DefaultOutputFormatType, "TwoByteSyntaxCharacterAutoReplacement" -> 
             True, HyphenationOptions -> {
              "HyphenationCharacter" -> "\[Continuation]"}, FormatType -> 
             InputForm, StyleMenuListing -> None], 
            Cell[
             StyleData["Info", "Presentation"], 
             CellMargins -> {{72, 10}, {6, 6}}, FontSize -> 18], 
            Cell[
             StyleData["Info", "Printout"], 
             CellMargins -> {{30, Inherited}, {Inherited, Inherited}}]}, 
           Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Graphics"], CellFrame -> {{4, 4}, {4, 0}}, 
             CellMargins -> {{62, 10}, {0, 0}}, CellGroupingRules -> 
             "GraphicsGrouping", CellHorizontalScrolling -> True, 
             PageBreakWithin -> False, GeneratedCell -> True, 
             CellAutoOverwrite -> True, ShowCellLabel -> False, 
             CellFrameColor -> RGBColor[0.8, 0.700008, 0.6], 
             DefaultFormatType -> DefaultOutputFormatType, FormatType -> 
             InputForm, ImageMargins -> {{43, Inherited}, {Inherited, 0}}, 
             StyleMenuListing -> None, Background -> GrayLevel[1]], 
            Cell[
             StyleData["Graphics", "Presentation"], 
             CellFrame -> {{6, 6}, {0, 0}}, CellMargins -> {{72, 10}, {0, 0}},
              CellFrameMargins -> 12, FontSize -> 18], 
            Cell[
             StyleData["Graphics", "Printout"], ImageSize -> {250, 250}, 
             ImageMargins -> {{28, Inherited}, {Inherited, 0}}]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["ImportPict"], CellMargins -> {{52, 10}, {0, 0}}, 
             CellGroupingRules -> "GraphicsGrouping", CellHorizontalScrolling -> 
             True, PageBreakWithin -> False, GeneratedCell -> True, 
             DefaultFormatType -> DefaultOutputFormatType, FormatType -> 
             InputForm, StyleMenuListing -> "ImportPict", Background -> None],
             
            Cell[
             StyleData["ImportPict", "Presentation"], 
             CellMargins -> {{72, 10}, {0, 0}}, FontSize -> 18], 
            Cell[
             StyleData["ImportPict", "Printout"]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["CellLabel"], StyleMenuListing -> None, FontFamily -> 
             "Helvetica", FontSize -> 10, FontWeight -> "Plain", FontSlant -> 
             "Italic", FontColor -> RGBColor[0.2, 0.2, 0.700008]], 
            Cell[
             StyleData["CellLabel", "Presentation"], FontSize -> 15], 
            Cell[
             StyleData["CellLabel", "Printout"], FontFamily -> "Courier", 
             FontSize -> 8, FontSlant -> "Italic", FontColor -> 
             GrayLevel[0]]}, Open]]}, Open]], 
      Cell[
       CellGroupData[{
         Cell["Styles for Unique Items", "Section"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["PullQuote"], CellFrame -> {{0, 0}, {0, 3}}, 
             CellMargins -> {{0, 250}, {15, 5}}, 
             CellFrameMargins -> {{20, 4}, {8, 6}}, LineSpacing -> {1, 1}, 
             ParagraphSpacing -> {0, 12}, FontFamily -> "Helvetica", FontSize -> 
             12, FontWeight -> "Bold", FontColor -> GrayLevel[1], Background -> 
             RGBColor[0.6, 0, 0.8]], 
            Cell[
             StyleData["PullQuote", "Presentation"], 
             CellFrame -> {{0, 0}, {0, 5}}, 
             CellMargins -> {{0, 250}, {25, 8}}, 
             CellFrameMargins -> {{30, 10}, {10, 10}}, FontSize -> 18], 
            Cell[
             StyleData["PullQuote", "Printout"], 
             CellMargins -> {{9, 0}, {4, 4}}, ParagraphSpacing -> {0, 6}, 
             FontSize -> 10]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["GrayBox"], CellFrame -> {{0, 0}, {0, 3}}, 
             CellMargins -> {{62, 36}, {10, 5}}, 
             CellFrameMargins -> {{8, 4}, {8, 4}}, LineSpacing -> {1, 3}, 
             ParagraphSpacing -> {0, 12}, FontFamily -> "Helvetica", FontSize -> 
             10, FontWeight -> "Plain", FontColor -> GrayLevel[0], Background -> 
             RGBColor[0.8, 0.700008, 0.6]], 
            Cell[
             StyleData["GrayBox", "Presentation"], 
             CellFrame -> {{0, 0}, {0, 5}}, 
             CellMargins -> {{90, 55}, {16, 5}}, 
             CellFrameMargins -> {{10, 10}, {12, 6}}, FontSize -> 15], 
            Cell[
             StyleData["GrayBox", "Printout"], 
             CellMargins -> {{62, 36}, {10, 5}}, Background -> 
             GrayLevel[0.8]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Picture"], CellFrame -> 4, 
             CellMargins -> {{52, 10}, {0, 0}}, CellGroupingRules -> 
             "GraphicsGrouping", CellHorizontalScrolling -> True, 
             PageBreakWithin -> False, CellFrameColor -> 
             RGBColor[0.8, 0.700008, 0.6], DefaultFormatType -> 
             DefaultOutputFormatType, FormatType -> InputForm, 
             StyleMenuListing -> None, Background -> GrayLevel[1]], 
            Cell[
             StyleData["Picture", "Presentation"], CellFrame -> 6, 
             CellMargins -> {{72, 10}, {0, 6}}, CellFrameMargins -> 12, 
             FontSize -> 18], 
            Cell[
             StyleData["Picture", "Printout"], ImageSize -> {250, 250}, 
             ImageMargins -> {{28, Inherited}, {Inherited, 0}}]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["PictureGroup"], CellFrame -> {{4, 4}, {0, 0}}, 
             CellMargins -> {{52, Inherited}, {0, 0}}, CellGroupingRules -> 
             "GraphicsGrouping", CellHorizontalScrolling -> True, 
             PageBreakWithin -> False, ShowCellLabel -> False, CellFrameColor -> 
             RGBColor[0.8, 0.700008, 0.6], 
             ImageMargins -> {{43, Inherited}, {Inherited, 0}}, 
             StyleMenuListing -> None, Background -> GrayLevel[1]], 
            Cell[
             StyleData["PictureGroup", "Presentation"], 
             CellFrame -> {{6, 6}, {0, 0}}, CellMargins -> {{72, 10}, {0, 0}},
              CellFrameMargins -> 12, FontSize -> 18], 
            Cell[
             StyleData["PictureGroup", "Printout"], ImageSize -> {250, 250}, 
             ImageMargins -> {{28, Inherited}, {Inherited, 0}}]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["LineSpace"], CellMargins -> {{0, 0}, {0, 0}}, 
             CellElementSpacings -> {"CellMinHeight" -> 2}, FontSize -> 9], 
            Cell[
             StyleData["LineSpace", "Presentation"], 
             CellMargins -> {{0, 4}, {0, 0}}, FontSize -> 14]}, Closed]]}, 
        Closed]], 
      Cell[
       CellGroupData[{
         Cell["Hyperlink Styles", "Section"], 
         Cell[
         "The cells below define styles useful for making hypertext \
ButtonBoxes.  The \"Hyperlink\" style is for links within the same Notebook, \
or between Notebooks.", "Text"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["Hyperlink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontColor -> 
             RGBColor[0.269993, 0.308507, 0.6], 
             ButtonBoxOptions -> {
              Active -> True, ButtonFrame -> "None", 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`NotebookLocate[#2]}]& ), ButtonNote -> 
               ButtonData}], 
            Cell[
             StyleData["Hyperlink", "Presentation"], FontSize -> 16], 
            Cell[
             StyleData["Hyperlink", "Condensed"], FontSize -> 11], 
            Cell[
             StyleData["Hyperlink", "SlideShow"]], 
            Cell[
             StyleData["Hyperlink", "Printout"], FontSize -> 10, 
             FontVariations -> {"Underline" -> False}, FontColor -> 
             GrayLevel[0]]}, Closed]], 
         Cell[
         "The following styles are for linking automatically to the on-line \
help system.", "Text"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["MainBookLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontColor -> 
             RGBColor[0.269993, 0.308507, 0.6], 
             ButtonBoxOptions -> {
              Active -> True, ButtonFrame -> "None", 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["MainBook", #]}]& )}], 
            Cell[
             StyleData["MainBookLink", "Presentation"], FontSize -> 16], 
            Cell[
             StyleData["MainBookLink", "Condensed"], FontSize -> 11], 
            Cell[
             StyleData["MainBookLink", "SlideShow"]], 
            Cell[
             StyleData["MainBookLink", "Printout"], FontSize -> 10, 
             FontVariations -> {"Underline" -> False}, FontColor -> 
             GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["AddOnsLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontFamily -> "Courier", 
             FontColor -> RGBColor[0.269993, 0.308507, 0.6], 
             ButtonBoxOptions -> {
              Active -> True, ButtonFrame -> "None", 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["AddOns", #]}]& )}], 
            Cell[
             StyleData["AddOnsLink", "Presentation"], FontSize -> 16], 
            Cell[
             StyleData["AddOnsLink", "Condensed"], FontSize -> 11], 
            Cell[
             StyleData["AddOnsLink", "SlideShow"]], 
            Cell[
             StyleData["AddOnsLink", "Printout"], FontSize -> 10, 
             FontVariations -> {"Underline" -> False}, FontColor -> 
             GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["RefGuideLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontFamily -> "Courier", 
             FontColor -> RGBColor[0.269993, 0.308507, 0.6], 
             ButtonBoxOptions -> {
              Active -> True, ButtonFrame -> "None", 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["RefGuide", #]}]& )}], 
            Cell[
             StyleData["RefGuideLink", "Presentation"], FontSize -> 16], 
            Cell[
             StyleData["RefGuideLink", "Condensed"], FontSize -> 11], 
            Cell[
             StyleData["RefGuideLink", "SlideShow"]], 
            Cell[
             StyleData["RefGuideLink", "Printout"], FontSize -> 10, 
             FontVariations -> {"Underline" -> False}, FontColor -> 
             GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["RefGuideLinkText"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontColor -> 
             RGBColor[0.269993, 0.308507, 0.6], 
             ButtonBoxOptions -> {
              Active -> True, ButtonFrame -> "None", 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["RefGuide", #]}]& )}], 
            Cell[
             StyleData["RefGuideLinkText", "Presentation"], FontSize -> 16], 
            Cell[
             StyleData["RefGuideLinkText", "Condensed"], FontSize -> 11], 
            Cell[
             StyleData["RefGuideLinkText", "SlideShow"]], 
            Cell[
             StyleData["RefGuideLinkText", "Printout"], FontSize -> 10, 
             FontVariations -> {"Underline" -> False}, FontColor -> 
             GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["GettingStartedLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontColor -> 
             RGBColor[0.269993, 0.308507, 0.6], 
             ButtonBoxOptions -> {
              Active -> True, ButtonFrame -> "None", 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["GettingStarted", #]}]& )}], 
            Cell[
             StyleData["GettingStartedLink", "Presentation"], FontSize -> 16],
             
            Cell[
             StyleData["GettingStartedLink", "Condensed"], FontSize -> 11], 
            Cell[
             StyleData["GettingStartedLink", "SlideShow"]], 
            Cell[
             StyleData["GettingStartedLink", "Printout"], FontSize -> 10, 
             FontVariations -> {"Underline" -> False}, FontColor -> 
             GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["DemosLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontColor -> 
             RGBColor[0.269993, 0.308507, 0.6], 
             ButtonBoxOptions -> {
              Active -> True, ButtonFrame -> "None", 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["Demos", #]}]& )}], 
            Cell[
             StyleData["DemosLink", "SlideShow"]], 
            Cell[
             StyleData["DemosLink", "Printout"], 
             FontVariations -> {"Underline" -> False}, FontColor -> 
             GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["TourLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontColor -> 
             RGBColor[0.269993, 0.308507, 0.6], 
             ButtonBoxOptions -> {
              Active -> True, ButtonFrame -> "None", 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["Tour", #]}]& )}], 
            Cell[
             StyleData["TourLink", "SlideShow"]], 
            Cell[
             StyleData["TourLink", "Printout"], 
             FontVariations -> {"Underline" -> False}, FontColor -> 
             GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["OtherInformationLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontColor -> 
             RGBColor[0.269993, 0.308507, 0.6], 
             ButtonBoxOptions -> {
              Active -> True, ButtonFrame -> "None", 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["OtherInformation", #]}]& )}], 
            Cell[
             StyleData["OtherInformationLink", "Presentation"], FontSize -> 
             16], 
            Cell[
             StyleData["OtherInformationLink", "Condensed"], FontSize -> 11], 
            
            Cell[
             StyleData["OtherInformationLink", "SlideShow"]], 
            Cell[
             StyleData["OtherInformationLink", "Printout"], FontSize -> 10, 
             FontVariations -> {"Underline" -> False}, FontColor -> 
             GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["MasterIndexLink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontColor -> 
             RGBColor[0.269993, 0.308507, 0.6], 
             ButtonBoxOptions -> {
              Active -> True, ButtonFrame -> "None", 
               ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`HelpBrowserLookup["MasterIndex", #]}]& )}], 
            Cell[
             StyleData["MasterIndexLink", "SlideShow"]], 
            Cell[
             StyleData["MasterIndexLink", "Printout"], 
             FontVariations -> {"Underline" -> False}, FontColor -> 
             GrayLevel[0]]}, Closed]]}, Closed]], 
      Cell[
       CellGroupData[{
         Cell["Palette Styles", "Section"], 
         Cell[
         "The cells below define styles that define standard ButtonFunctions, \
for use in palette buttons.", "Text"], 
         Cell[
          StyleData["Paste"], StyleMenuListing -> None, 
          ButtonStyleMenuListing -> Automatic, 
          ButtonBoxOptions -> {ButtonFunction :> (FrontEndExecute[{
               FrontEnd`NotebookApply[
                FrontEnd`InputNotebook[], #, After]}]& )}], 
         Cell[
          StyleData["Evaluate"], StyleMenuListing -> None, 
          ButtonStyleMenuListing -> Automatic, 
          ButtonBoxOptions -> {ButtonFunction :> (FrontEndExecute[{
               FrontEnd`NotebookApply[
                FrontEnd`InputNotebook[], #, All], 
               FrontEnd`SelectionEvaluate[
                FrontEnd`InputNotebook[], All]}]& )}], 
         Cell[
          StyleData["EvaluateCell"], StyleMenuListing -> None, 
          ButtonStyleMenuListing -> Automatic, 
          ButtonBoxOptions -> {ButtonFunction :> (FrontEndExecute[{
               FrontEnd`NotebookApply[
                FrontEnd`InputNotebook[], #, All], 
               FrontEnd`SelectionMove[
                FrontEnd`InputNotebook[], All, Cell, 1], 
               FrontEnd`SelectionEvaluateCreateCell[
                FrontEnd`InputNotebook[], All]}]& )}], 
         Cell[
          StyleData["CopyEvaluate"], StyleMenuListing -> None, 
          ButtonStyleMenuListing -> Automatic, 
          ButtonBoxOptions -> {ButtonFunction :> (FrontEndExecute[{
               FrontEnd`SelectionCreateCell[
                FrontEnd`InputNotebook[], All], 
               FrontEnd`NotebookApply[
                FrontEnd`InputNotebook[], #, All], 
               FrontEnd`SelectionEvaluate[
                FrontEnd`InputNotebook[], All]}]& )}], 
         Cell[
          StyleData["CopyEvaluateCell"], StyleMenuListing -> None, 
          ButtonStyleMenuListing -> Automatic, 
          ButtonBoxOptions -> {ButtonFunction :> (FrontEndExecute[{
               FrontEnd`SelectionCreateCell[
                FrontEnd`InputNotebook[], All], 
               FrontEnd`NotebookApply[
                FrontEnd`InputNotebook[], #, All], 
               FrontEnd`SelectionEvaluateCreateCell[
                FrontEnd`InputNotebook[], All]}]& )}]}, Closed]], 
      Cell[
       CellGroupData[{
         Cell["Slide Show Styles", "Section"], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["SlideShowNavigationBar"], Editable -> False, 
             CellFrame -> True, CellMargins -> {{0, 0}, {3, 3}}, 
             CellElementSpacings -> {"CellMinHeight" -> 0.8125}, 
             CellGroupingRules -> {"SectionGrouping", 30}, CellFrameMargins -> 
             False, CellFrameColor -> GrayLevel[1], CellFrameLabelMargins -> 
             False, TextAlignment -> Center, CounterIncrements -> 
             "SlideShowNavigationBar", StyleMenuListing -> None, FontSize -> 
             10, Magnification -> 1, Background -> GrayLevel[0.8], 
             GridBoxOptions -> {
              BaselinePosition -> Center, 
               GridBoxAlignment -> {
                "Columns" -> {
                  Center, Center, Center, Center, Center, Center, Right, {
                   Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
                 "RowsIndexed" -> {}}, 
               GridBoxItemSize -> {
                "Columns" -> {3.5, 3.5, 3.5, 3.5, 13, 5, {4}}, 
                 "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
                 "RowsIndexed" -> {}}, GridBoxSpacings -> {"Columns" -> {
                   Offset[0.27999999999999997`], {
                    Offset[0.]}, 
                   Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
                 "Rows" -> {
                   Offset[0.2], {
                    Offset[0.]}, 
                   Offset[0.2]}, "RowsIndexed" -> {}}}], 
            Cell[
             StyleData["SlideShowNavigationBar", "SlideShow"], Deletable -> 
             False, ShowCellBracket -> False, 
             CellMargins -> {{-1, -1}, {-1, -1}}, PageBreakAbove -> True, 
             CellFrameMargins -> {{1, 1}, {0, 0}}], 
            Cell[
             StyleData["SlideShowNavigationBar", "Printout"], 
             CellMargins -> {{18, 4}, {4, 4}}, LineSpacing -> {1, 3}, 
             FontSize -> 10]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["SlideShowSection"], CellFrame -> {{0, 0}, {0, 0.5}}, 
             CellMargins -> {{0, 0}, {10, 0}}, 
             CellGroupingRules -> {"SectionGrouping", 40}, PageBreakBelow -> 
             False, CellFrameMargins -> {{12, 4}, {6, 12}}, 
             InputAutoReplacements -> {"TeX" -> StyleBox[
                 RowBox[{"T", 
                   AdjustmentBox[
                   "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, 
                    BoxBaselineShift -> 0.5], "X"}]], "LaTeX" -> StyleBox[
                 RowBox[{"L", 
                   StyleBox[
                    AdjustmentBox[
                    "A", BoxMargins -> {{-0.36, -0.1}, {0, 0}}, 
                    BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
                   AdjustmentBox[
                   "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, 
                    BoxBaselineShift -> 0.5], "X"}]], "mma" -> "Mathematica", 
               "Mma" -> "Mathematica", "MMA" -> "Mathematica", 
               "gridMathematica" -> FormBox[
                 RowBox[{"grid", 
                   AdjustmentBox[
                    StyleBox["Mathematica", FontSlant -> "Italic"], 
                    BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
               "webMathematica" -> FormBox[
                 RowBox[{"web", 
                   AdjustmentBox[
                    StyleBox["Mathematica", FontSlant -> "Italic"], 
                    BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
               Inherited}, CounterIncrements -> "Section", 
             CounterAssignments -> {{"Subsection", 0}, {"Subsubsection", 0}}, 
             StyleMenuListing -> None, FontFamily -> "Helvetica", FontSize -> 
             18, FontWeight -> "Plain", FontColor -> GrayLevel[1], Background -> 
             RGBColor[0.408011, 0.440726, 0.8]], 
            Cell[
             StyleData["SlideShowSection", "Presentation"], 
             CellFrameMargins -> {{18, 10}, {10, 18}}, FontSize -> 27], 
            Cell[
             StyleData["SlideShowSection", "SlideShow"], ShowCellBracket -> 
             False, PageBreakAbove -> True], 
            Cell[
             StyleData["SlideShowSection", "Printout"], 
             CellMargins -> {{18, 30}, {0, 30}}, CellFrameMargins -> 5, 
             FontSize -> 14]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["SlideHyperlink"], StyleMenuListing -> None, 
             ButtonStyleMenuListing -> Automatic, FontSize -> 26, FontColor -> 
             GrayLevel[0.400015], 
             ButtonBoxOptions -> {
              Active -> True, ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`NotebookLocate[#2]}]& ), ButtonMargins -> 0.5, 
               ButtonMinHeight -> 0.85, ButtonNote -> None}], 
            Cell[
             StyleData["SlideHyperlink", "Presentation"], 
             CellMargins -> {{10, 10}, {8, 12}}, FontSize -> 36], 
            Cell[
             StyleData["SlideHyperlink", "SlideShow"]], 
            Cell[
             StyleData["SlideHyperlink", "Printout"], FontSize -> 10, 
             FontVariations -> {"Underline" -> False}, FontColor -> 
             GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["SlideTOCLink"], 
             CellMargins -> {{24, Inherited}, {Inherited, Inherited}}, 
             StyleMenuListing -> None, ButtonStyleMenuListing -> Automatic, 
             FontFamily -> "Helvetica", 
             ButtonBoxOptions -> {
              Active -> True, ButtonFunction :> (FrontEndExecute[{
                  FrontEnd`NotebookLocate[#2]}]& ), ButtonMargins -> 1.5, 
               ButtonNote -> ButtonData}], 
            Cell[
             StyleData["SlideTOCLink", "Presentation"], 
             CellMargins -> {{35, 10}, {6, 4}}, FontSize -> 18], 
            Cell[
             StyleData["SlideTOCLink", "SlideShow"]], 
            Cell[
             StyleData["SlideTOCLink", "Printout"], 
             FontVariations -> {"Underline" -> False}, FontColor -> 
             GrayLevel[0]]}, Closed]], 
         Cell[
          CellGroupData[{
            Cell[
             StyleData["SlideTOC"], CellDingbat -> "\[Bullet]", 
             CellMargins -> {{18, Inherited}, {Inherited, Inherited}}, 
             StyleMenuListing -> None, FontFamily -> "Helvetica"], 
            Cell[
             StyleData["SlideTOC", "Presentation"], 
             CellMargins -> {{25, 10}, {6, 6}}, FontSize -> 18], 
            Cell[
             StyleData["SlideTOC", "SlideShow"], FontSize -> 14], 
            Cell[
             StyleData["SlideTOC", "Printout"], FontSize -> 10, FontColor -> 
             GrayLevel[0]]}, Closed]]}, Open]], 
      Cell[
       CellGroupData[{
         Cell["Styles for Headers and Footers", "Section"], 
         Cell[
          StyleData["Header"], CellMargins -> {{0, 0}, {4, 1}}, 
          StyleMenuListing -> None, FontFamily -> "Helvetica", FontSize -> 9, 
          FontSlant -> "Italic"], 
         Cell[
          StyleData["Footer"], CellMargins -> {{0, 0}, {0, 4}}, 
          StyleMenuListing -> None, FontFamily -> "Helvetica", FontSize -> 6],
          
         Cell[
          StyleData["PageNumber"], CellMargins -> {{0, 0}, {4, 1}}, 
          StyleMenuListing -> None, FontFamily -> "Helvetica", FontSize -> 9, 
          FontWeight -> "Bold"]}, Open]], 
      Cell[
       CellGroupData[{
         Cell["FormatType Styles", "Section"], 
         Cell[
         "The cells below define styles that are mixed in with the styles of \
most cells.  If a cell's FormatType matches the name of one of the styles \
defined below, then that style is applied between the cell's style and its \
own options.", "Text"], 
         Cell[
          StyleData["CellExpression"], CellFrame -> 1, 
          CellMargins -> {{52, 10}, {4, 2}}, ShowCellLabel -> False, 
          CellLabelMargins -> {{23, Inherited}, {Inherited, Inherited}}, 
          ShowSpecialCharacters -> False, AllowInlineCells -> False, 
          StyleMenuListing -> None, FontFamily -> "Courier", FontSize -> 10, 
          FontColor -> GrayLevel[0], Background -> GrayLevel[1]]}, Open]]}, 
     Open]]}, Visible -> False, FrontEndVersion -> 
  "6.0 for Mac OS X x86 (32-bit) (June 19, 2007)", StyleDefinitions -> 
  "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[568, 21, 90, 4, 100, "Title"],
Cell[661, 27, 40, 0, 80, "Subtitle"],
Cell[704, 29, 48, 0, 64, "Subsubtitle"],
Cell[CellGroupData[{
Cell[777, 33, 31, 0, 99, "Section"],
Cell[811, 35, 189, 4, 74, "Text"],
Cell[1003, 41, 427, 15, 81, "Text"],
Cell[1433, 58, 386, 15, 62, "Text"],
Cell[1822, 75, 248, 4, 104, "Text"],
Cell[2073, 81, 251, 4, 78, "Subsubsection"],
Cell[2327, 87, 2508, 64, 320, "Input",
 InitializationCell->True]
}, {1, 2, 3, 4, 5, 6}]],
Cell[CellGroupData[{
Cell[4884, 156, 49, 0, 99, "Section"],
Cell[4936, 158, 992, 32, 148, "Text"],
Cell[CellGroupData[{
Cell[5953, 194, 313, 9, 76, "Input"],
Cell[6269, 205, 15424, 252, 485, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[21730, 462, 178, 5, 110, "Input"],
Cell[21911, 469, 69, 1, 88, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[22017, 475, 345, 10, 76, "Input"],
Cell[22365, 487, 14780, 241, 485, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[37182, 733, 206, 6, 114, "Input"],
Cell[37391, 741, 87, 2, 88, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[37515, 748, 398, 12, 76, "Input"],
Cell[37916, 762, 14834, 245, 485, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[52787, 1012, 255, 8, 115, "Input"],
Cell[53045, 1022, 70, 1, 88, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[53164, 1029, 54, 0, 99, "Section"],
Cell[53221, 1031, 225, 4, 104, "Text"],
Cell[53449, 1037, 867, 31, 59, "Text"],
Cell[54319, 1070, 162, 3, 74, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[54518, 1078, 59, 0, 99, "Section"],
Cell[54580, 1080, 751, 22, 154, "Text"],
Cell[55334, 1104, 154, 3, 74, "Text"],
Cell[55491, 1109, 626, 25, 53, "Text"],
Cell[CellGroupData[{
Cell[56142, 1138, 142, 4, 108, "Input"],
Cell[56287, 1144, 117, 3, 88, "Output"]
}, Open  ]],
Cell[56419, 1150, 99, 2, 44, "Text"],
Cell[56521, 1154, 319, 9, 85, "Text"],
Cell[56843, 1165, 866, 29, 55, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[57746, 1199, 49, 0, 99, "Section"],
Cell[57798, 1201, 814, 30, 128, "Text"],
Cell[58615, 1233, 37, 0, 78, "Subsubsection"],
Cell[58655, 1235, 351, 6, 104, "Text"],
Cell[59009, 1243, 442, 14, 81, "Text"],
Cell[CellGroupData[{
Cell[59476, 1261, 359, 10, 108, "Input"],
Cell[59838, 1273, 221, 7, 95, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[60108, 1286, 31, 0, 99, "Section"],
Cell[60142, 1288, 97, 2, 44, "Text"],
Cell[60242, 1292, 437, 15, 51, "Text"],
Cell[60682, 1309, 494, 17, 74, "Text"],
Cell[61179, 1328, 596, 13, 83, "Input"],
Cell[61778, 1343, 190, 6, 74, "Text"],
Cell[CellGroupData[{
Cell[61993, 1353, 411, 10, 76, "Input"],
Cell[62407, 1365, 12143, 205, 458, "Output"]
}, Open  ]],
Cell[74565, 1573, 63, 0, 44, "Text"],
Cell[CellGroupData[{
Cell[74653, 1577, 240, 6, 113, "Input"],
Cell[74896, 1585, 74, 1, 88, "Output"]
}, Open  ]],
Cell[74985, 1589, 131, 3, 74, "Text"],
Cell[CellGroupData[{
Cell[75141, 1596, 195, 6, 108, "Input"],
Cell[75339, 1604, 305, 10, 95, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[75681, 1619, 175, 4, 76, "Input"],
Cell[75859, 1625, 72, 1, 88, "Output"]
}, Open  ]],
Cell[75946, 1629, 37, 0, 78, "Subsubsection"],
Cell[75986, 1631, 87, 2, 44, "Text"],
Cell[76076, 1635, 410, 12, 51, "Text"],
Cell[76489, 1649, 384, 16, 44, "Text"],
Cell[76876, 1667, 309, 8, 74, "Text"],
Cell[77188, 1677, 398, 16, 74, "Text"],
Cell[77589, 1695, 277, 7, 74, "Text"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
