(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[   1459422,      25882]
NotebookOptionsPosition[   1453392,      25694]
NotebookOutlinePosition[   1454015,      25719]
CellTagsIndexPosition[   1453972,      25716]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Keplers Laws", "Subtitle",
 TextAlignment->Center,
 TextJustification->0],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell[CellGroupData[{

Cell["Particle Dynamics", "Subsection"],

Cell[TextData[{
 "\tWe can numerically simulate the dynamics of a particle moving under some \
force.  We write the net force acting on the particle as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["F", "\[RightVector]"], "(", 
    RowBox[{
     OverscriptBox["r", "\[RightVector]"], ",", 
     OverscriptBox["v", "\[RightVector]"], ",", "t"}], ")"}], 
   TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["r", "\[RightVector]"], TraditionalForm]]],
 " is the particle's position vector, ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["v", "\[RightVector]"], TraditionalForm]]],
 " is its velocity vector and ",
 StyleBox["t",
  FontSlant->"Italic"],
 " is the time.  Newton's second law gives a differential equation that can \
be solved to describe the motion.  "
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{
      OverscriptBox["F", "\[RightVector]"], "(", 
      RowBox[{
       OverscriptBox["r", "\[RightVector]"], ",", 
       OverscriptBox["v", "\[RightVector]"], ",", "t"}], ")"}],
     TraditionalForm], "=", 
    RowBox[{"m", " ", 
     OverscriptBox["a", "\[RightVector]"]}]}], TraditionalForm]]],
 "  \[DoubleLongRightArrow]  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox[
      SuperscriptBox["\[DifferentialD]", "2"], 
      RowBox[{"\[DifferentialD]", 
       SuperscriptBox["t", "2"]}]], " ", 
     OverscriptBox["r", "\[RightVector]"]}], "=", 
    RowBox[{
     FractionBox["1", "m"], " ", 
     RowBox[{
      OverscriptBox["F", "\[RightVector]"], "(", 
      RowBox[{
       OverscriptBox["r", "\[RightVector]"], ",", 
       RowBox[{
        FractionBox["\[DifferentialD]", 
         RowBox[{"\[DifferentialD]", "t"}]], " ", 
        OverscriptBox["r", "\[RightVector]"]}], ",", "t"}], ")"}]}]}], 
   TraditionalForm]]]
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "This is a second order ordinary differential equation (ODE), where the \
order is the highest number of derivatives.  An ODE can always be reduced to \
a coupled set of first order equations.  In this case we use ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["v", "\[RightVector]"], "=", 
    RowBox[{
     RowBox[{"\[DifferentialD]", 
      OverscriptBox["r", "\[RightVector]"]}], "/", 
     RowBox[{"\[DifferentialD]", "t"}]}]}], TraditionalForm]]],
 " and write"
}], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["\[DifferentialD]", 
     RowBox[{"\[DifferentialD]", "t"}]], " ", 
    OverscriptBox["r", "\[RightVector]"]}], "=", 
   OverscriptBox["v", "\[RightVector]"]}], TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["\[DifferentialD]", 
      RowBox[{"\[DifferentialD]", "t"}]], " ", 
     OverscriptBox["v", "\[RightVector]"]}], "=", 
    RowBox[{
     FractionBox["1", "m"], " ", 
     RowBox[{
      OverscriptBox["F", "\[RightVector]"], "(", 
      RowBox[{
       OverscriptBox["r", "\[RightVector]"], ",", 
       OverscriptBox["v", "\[RightVector]"], ",", "t"}], ")"}]}]}], 
   TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
\tThe general problem of motion can be written in terms of a vector formed \
from these variables\
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["z", "\[RightVector]"], "=", 
    RowBox[{"\[LeftAngleBracket]", 
     RowBox[{
      OverscriptBox["r", "\[RightVector]"], ",", 
      OverscriptBox["v", "\[RightVector]"]}], "\[RightAngleBracket]"}]}], 
   TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "In two dimension this would represent 4 variables, ",
 StyleBox["x",
  FontSlant->"Italic"],
 ", ",
 StyleBox["y",
  FontSlant->"Italic"],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", "x"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", "y"], TraditionalForm]]],
 ".  In three dimensions it would be 6 variables.  Think of this 'vector' as \
a collection of four (or six) numbers and not as anything with a three \
dimensional geometrical interpretation.  The differential equation can then \
be rewritten as"
}], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[LeftAngleBracket]", 
    RowBox[{
     RowBox[{
      FractionBox["\[DifferentialD]", 
       RowBox[{"\[DifferentialD]", "t"}]], 
      OverscriptBox["r", "\[RightVector]"]}], ",", 
     RowBox[{
      FractionBox["\[DifferentialD]", 
       RowBox[{"\[DifferentialD]", "t"}]], 
      OverscriptBox["v", "\[RightVector]"]}]}], "\[RightAngleBracket]"}], "=",
    
   RowBox[{"\[LeftAngleBracket]", 
    RowBox[{
     OverscriptBox["v", "\[RightVector]"], ",", 
     RowBox[{
      FractionBox["1", "m"], 
      RowBox[{
       OverscriptBox["F", "\[RightVector]"], "(", 
       RowBox[{
        OverscriptBox["r", "\[RightVector]"], ",", 
        OverscriptBox["v", "\[RightVector]"], ",", "t"}], ")"}]}]}], 
    "\[RightAngleBracket]"}]}], TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell["or in the general form", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["\[DifferentialD]", 
      RowBox[{"\[DifferentialD]", "t"}]], 
     OverscriptBox["z", "\[RightVector]"]}], "=", 
    RowBox[{
     OverscriptBox["f", "\[RightVector]"], "(", 
     RowBox[{
      OverscriptBox["z", "\[RightVector]"], ",", "t"}], ")"}]}], 
   TraditionalForm]]],
 "  where  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     OverscriptBox["f", "\[RightVector]"], "(", 
     RowBox[{
      OverscriptBox["z", "\[RightVector]"], ",", "t"}], ")"}], "=", 
    RowBox[{"\[LeftAngleBracket]", 
     RowBox[{
      OverscriptBox["v", "\[RightVector]"], ",", 
      RowBox[{
       FractionBox["1", "m"], 
       RowBox[{
        OverscriptBox["F", "\[RightVector]"], "(", 
        RowBox[{
         OverscriptBox["r", "\[RightVector]"], ",", 
         OverscriptBox["v", "\[RightVector]"], ",", "t"}], ")"}]}]}], 
     "\[RightAngleBracket]"}]}], TraditionalForm]]]
}], "Text",
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]],

Cell[CellGroupData[{

Cell["Numerical Methods", "Subsection"],

Cell[TextData[{
 "\tWe can step forward the position and velocity by an infinitesimal time \
increment ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DifferentialD]", "t"}], TraditionalForm]]],
 " using the rule:"
}], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{" ", 
   RowBox[{
    OverscriptBox["r", "\[RightVector]"], " ", "\[Rule]", " ", 
    RowBox[{
     OverscriptBox["r", "\[RightVector]"], "+", 
     RowBox[{
      OverscriptBox["v", "\[RightVector]"], " ", 
      RowBox[{"\[DifferentialD]", "t"}]}]}]}]}], TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["v", "\[RightVector]"], " ", "\[Rule]", " ", 
    RowBox[{
     OverscriptBox["v", "\[RightVector]"], "+", 
     RowBox[{
      FractionBox["1", "m"], " ", 
      RowBox[{
       OverscriptBox["F", "\[RightVector]"], "(", 
       RowBox[{
        OverscriptBox["r", "\[RightVector]"], ",", 
        OverscriptBox["v", "\[RightVector]"], ",", "t"}], ")"}], " ", 
      RowBox[{"\[DifferentialD]", "t"}]}]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "In terms of the variable ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["z", "\[RightVector]"], TraditionalForm]]],
 " this is written"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["z", "\[RightVector]"], " ", "\[Rule]", " ", 
    RowBox[{
     OverscriptBox["z", "\[RightVector]"], "+", " ", 
     RowBox[{
      RowBox[{
       OverscriptBox["f", "\[RightVector]"], "(", 
       RowBox[{
        OverscriptBox["z", "\[RightVector]"], ",", "t"}], ")"}], " ", 
      RowBox[{"\[DifferentialD]", "t"}]}]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "To numerically integrate a system forward in time we approximate the \
infinitesimal ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DifferentialD]", "t"}], TraditionalForm]]],
 " with the finite ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]", "\[ThinSpace]", "t"}], TraditionalForm]]],
 ".  We start at time ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "0"], TraditionalForm]]],
 " at the \"position\", actually the position and velocity, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["z", "\[RightVector]"], "0"], TraditionalForm]]],
 ".  This is then stepped forward to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "1"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["z", "\[RightVector]"], "1"], TraditionalForm]]],
 " then to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "2"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["z", "\[RightVector]"], "2"], TraditionalForm]]],
 ", etc."
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "0"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["z", "\[RightVector]"], "0"], TraditionalForm]]],
 "  \[Rule]  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "1"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["z", "\[RightVector]"], "1"], TraditionalForm]]],
 "  \[Rule]  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "2"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["z", "\[RightVector]"], "2"], TraditionalForm]]],
 "  \[Rule]  \[Ellipsis]  \[Rule]  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "final"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["z", "\[RightVector]"], "fnal"], TraditionalForm]]]
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell["\tThe rule for stepping forward the time is", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["t", "i"], "  ", "\[Rule]", "  ", 
    SubscriptBox["t", 
     RowBox[{"i", "+", "1"}]]}], "=", 
   RowBox[{
    SubscriptBox["t", "i"], "+", 
    RowBox[{"\[CapitalDelta]", "\[ThinSpace]", 
     SubscriptBox["t", "i"]}]}]}], TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "The rule for stepping to the next ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["z", "\[RightVector]"], TraditionalForm]]],
 " variable will be written as: "
}], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     OverscriptBox["z", "\[RightVector]"], "i"], "  ", "\[Rule]", "  ", 
    SubscriptBox[
     OverscriptBox["z", "\[RightVector]"], 
     RowBox[{"i", "+", "1"}]]}], "=", 
   RowBox[{
    SubscriptBox[
     OverscriptBox["z", "\[RightVector]"], "i"], "+", 
    RowBox[{
     OverscriptBox["S", "\[RightVector]"], "(", 
     RowBox[{
      SubscriptBox[
       OverscriptBox["z", "\[RightVector]"], "i"], ",", 
      SubscriptBox["t", "i"], ",", 
      OverscriptBox["f", "\[RightVector]"], ",", 
      RowBox[{"\[CapitalDelta]", "\[ThinSpace]", 
       SubscriptBox["t", "i"]}]}], ")"}]}]}], TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "Where the numerical method used will be given by the stepping function ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["S", "\[RightVector]"], TraditionalForm]]],
 ".  To improve accuracy one must choose a sufficiently small time step ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]", "\[ThinSpace]", 
    SubscriptBox["t", "i"]}], TraditionalForm]]],
 " and an efficient numerical method.  The smaller we choose the values of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]", "\[ThinSpace]", "t"}], TraditionalForm]]],
 " the larger the computer's calculation becomes, so it is essential to \
choose an efficient numerical method.  The simplest numerical method is known \
as Euler's method, where ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["S", "\[RightVector]"], TraditionalForm]]],
 " is determined by the previous values of ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["z", "\[RightVector]"], TraditionalForm]]],
 " and ",
 StyleBox["t",
  FontSlant->"Italic"],
 "."
}], "Text"],

Cell[TextData[{
 "                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     OverscriptBox["S", "\[RightVector]"], "(", 
     RowBox[{
      SubscriptBox[
       OverscriptBox["z", "\[RightVector]"], "i"], ",", 
      SubscriptBox["t", "i"], ",", 
      OverscriptBox["f", "\[RightVector]"], ",", 
      RowBox[{"\[CapitalDelta]", "\[ThinSpace]", 
       SubscriptBox["t", "i"]}]}], ")"}], "=", 
    RowBox[{
     RowBox[{
      OverscriptBox["f", "\[RightVector]"], "(", 
      RowBox[{
       SubscriptBox[
        OverscriptBox["z", "\[RightVector]"], "i"], ",", 
       SubscriptBox["t", "i"]}], ")"}], "\[CapitalDelta]", "\[ThinSpace]", 
     SubscriptBox["t", "i"]}]}], TraditionalForm]]],
 "     (Euler's Method)"
}], "Text",
 CellChangeTimes->{{3.401286699172183*^9, 3.401286725219215*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "Euler's method is conceptually simple but it's accuracy is only of order ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]", "\[ThinSpace]", 
    SubscriptBox["t", "i"]}], TraditionalForm]]],
 ".  That means to decrease the error by a factor of 10 we must decrease ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]", "\[ThinSpace]", 
    SubscriptBox["t", "i"]}], TraditionalForm]]],
 " by a factor of 10.  A ",
 StyleBox["much",
  FontSlant->"Italic"],
 " more efficient method is called the ",
 StyleBox["fourth order Runge-Kutta method",
  FontSlant->"Italic"],
 ".  Fourth order means that decreasing ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]", "\[ThinSpace]", 
    SubscriptBox["t", "i"]}], TraditionalForm]]],
 " by a factor of 10 will decreas the error by a factor of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["10", "4"], "=", "10000"}], TraditionalForm]]],
 ".  The precise form of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["S", "\[RightVector]"], "(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["z", "\[RightVector]"], "i"], ",", 
     SubscriptBox["t", "i"], ",", 
     OverscriptBox["f", "\[RightVector]"], ",", 
     RowBox[{"\[CapitalDelta]", "\[ThinSpace]", 
      SubscriptBox["t", "i"]}]}], ")"}], TraditionalForm]]],
 " will not be given here.."
}], "Text",
 CellChangeTimes->{3.401202477064226*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Orbital Dynamics", "Subsection"],

Cell[TextData[{
 "\tThe gravitational force on mass ",
 StyleBox["m",
  FontSlant->"Italic"],
 " due to a second mass ",
 StyleBox["M",
  FontSlant->"Italic"],
 " is "
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["F", "\[RightVector]"], "=", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{"G", " ", "M", " ", "m"}], 
       SuperscriptBox["r", "2"]]}], 
     OverscriptBox["r", "^"]}]}], TraditionalForm]]],
 ","
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "where ",
 StyleBox["M",
  FontSlant->"Italic"],
 " is at the origin, ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["r", "\[RightVector]"], TraditionalForm]]],
 " is the vector from ",
 StyleBox["M",
  FontSlant->"Italic"],
 " to ",
 StyleBox["m",
  FontSlant->"Italic"],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["r", "^"], "=", 
    RowBox[{
     OverscriptBox["r", "\[RightVector]"], "/", "r"}]}], TraditionalForm]]],
 " is the unit vector in the direction of ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["r", "\[RightVector]"], TraditionalForm]]],
 ".  We will assume that ",
 StyleBox["M",
  FontSlant->"Italic"],
 " is much larger than ",
 StyleBox["m",
  FontSlant->"Italic"],
 " (",
 Cell[BoxData[
  FormBox[
   RowBox[{"M", "\[VeryThinSpace]", "\[GreaterGreater]", "m"}], 
   TraditionalForm]]],
 ") so that ",
 StyleBox["M",
  FontSlant->"Italic"],
 " stays fixed at the origin.  The gravitational field ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["g", "\[RightVector]"], TraditionalForm]]],
 " is defined as the force per mass"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["g", "\[RightVector]"], "=", 
    RowBox[{
     FractionBox[
      OverscriptBox["F", "\[RightVector]"], "m"], "=", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "G"}], " ", "M", 
       FractionBox[
        OverscriptBox["r", "^"], 
        SuperscriptBox["r", "2"]]}], "=", 
      RowBox[{
       RowBox[{"-", "G"}], " ", "M", 
       FractionBox[
        OverscriptBox["r", "\[RightVector]"], 
        SuperscriptBox["r", "3"]]}]}]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
\tThe inverse square law shown above is not the only possibility.  A more \
general form is\
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["g", "\[RightVector]"], "=", 
    RowBox[{
     FractionBox[
      OverscriptBox["F", "\[RightVector]"], "m"], "=", 
     RowBox[{
      RowBox[{"-", "G"}], " ", "M", " ", 
      RowBox[{"\[Phi]", "(", "r", ")"}], "  ", 
      OverscriptBox["r", "^"]}]}]}], TraditionalForm]]],
 ","
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "where ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "r", ")"}], TraditionalForm]]],
 " is a function of ",
 StyleBox["r",
  FontSlant->"Italic"],
 ", the distance from the origin, that decreases with distance and goes to \
zero at infinity.  Here we will only consider power law drop-off,  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", "r", ")"}], "=", 
    RowBox[{"1", "/", 
     SuperscriptBox["r", "p"]}]}], TraditionalForm]]],
 ".  We will define ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["g", "\[RightVector]"], "p"], TraditionalForm]]]
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox[
      OverscriptBox["g", "\[RightVector]"], "p"], "(", 
     OverscriptBox["r", "\[RightVector]"], ")"}], "=", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{"G", " ", "M"}], 
       SuperscriptBox["r", "P"]]}], 
     OverscriptBox["r", "^"]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "\tSince we are interested about orbits about the sun take ",
 StyleBox["M",
  FontSlant->"Italic"],
 " to be the mass of the sun.  ",
 "We will make a very useful choice of units to simplify our discussion.  \
Measure all lengths in astronomical units, AU, where ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", "AU"}], TraditionalForm]]],
 " is the radius of the earth's orbit and measure time in units of years.  In \
these units the speed of a circular orbit of radius 1 with period 1 is"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "=", "1"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"T", "=", "1"}], TraditionalForm]]],
 "  \[DoubleLongRightArrow]  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", "=", 
    RowBox[{
     FractionBox[
      RowBox[{"2", " ", "\[Pi]", " ", "r"}], "T"], "=", 
     RowBox[{"2", " ", "\[Pi]"}]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "For circular orbits, since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", "p"], "=", 
    SubscriptBox["a", "c"]}], TraditionalForm]]],
 ", we have "
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{"G", " ", "M"}], 
     SuperscriptBox["r", "p"]], "=", 
    RowBox[{
     SubscriptBox["g", "p"], "=", 
     RowBox[{
      SubscriptBox["a", "c"], "=", 
      FractionBox[
       SuperscriptBox["v", "2"], "r"]}]}]}], TraditionalForm]]],
 "  \[DoubleLongRightArrow]  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{"G", " ", "M"}], "1"], "=", 
    FractionBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"2", " ", "\[Pi]"}], ")"}], "2"], "1"]}], TraditionalForm]]],
 "  \[DoubleLongRightArrow]  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"G", " ", "M"}], "=", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"2", " ", "\[Pi]"}], ")"}], "2"]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell["Using these units we can write our gravitational field as", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox[
      OverscriptBox["g", "\[RightVector]"], "p"], "(", 
     OverscriptBox["r", "\[RightVector]"], ")"}], "=", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"2", " ", "\[Pi]"}], ")"}], "2"], 
        SuperscriptBox["r", "P"]]}], 
      OverscriptBox["r", "^"]}], "=", 
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"2", " ", "\[Pi]"}], ")"}], "2"]}], 
      FractionBox[
       OverscriptBox["r", "\[RightVector]"], 
       SuperscriptBox["r", 
        RowBox[{"P", "+", "1"}]]]}]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]],

Cell[CellGroupData[{

Cell["Numerical Methods and Orbits", "Subsection"],

Cell[TextData[{
 "\tIn the discussion of numerical methods above we defined the 'vector' ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["z", "\[RightVector]"], TraditionalForm]]],
 " by"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["z", "\[RightVector]"], "=", 
    RowBox[{"\[LeftAngleBracket]", 
     RowBox[{
      OverscriptBox["r", "\[RightVector]"], ",", 
      OverscriptBox["v", "\[RightVector]"]}], "\[RightAngleBracket]"}]}], 
   TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "Newton's second law gives dynamics to ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["z", "\[RightVector]"], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["\[DifferentialD]", 
      RowBox[{"\[DifferentialD]", "t"}]], 
     OverscriptBox["z", "\[RightVector]"]}], "=", 
    RowBox[{
     OverscriptBox["f", "\[RightVector]"], "(", 
     RowBox[{
      OverscriptBox["z", "\[RightVector]"], ",", "t"}], ")"}]}], 
   TraditionalForm]]],
 "  where  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     OverscriptBox["f", "\[RightVector]"], "(", 
     RowBox[{
      OverscriptBox["z", "\[RightVector]"], ",", "t"}], ")"}], "=", 
    RowBox[{"\[LeftAngleBracket]", 
     RowBox[{
      OverscriptBox["v", "\[RightVector]"], ",", 
      RowBox[{
       FractionBox["1", "m"], 
       RowBox[{
        OverscriptBox["F", "\[RightVector]"], "(", 
        RowBox[{
         OverscriptBox["r", "\[RightVector]"], ",", 
         OverscriptBox["v", "\[RightVector]"], ",", "t"}], ")"}]}]}], 
     "\[RightAngleBracket]"}]}], TraditionalForm]]]
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "It follows that from our orbit discussion the form of the function ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["f", "\[RightVector]"], TraditionalForm]]],
 " becomes"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     OverscriptBox["f", "\[RightVector]"], "(", 
     RowBox[{
      OverscriptBox["z", "\[RightVector]"], ",", "t"}], ")"}], "=", 
    RowBox[{"\[LeftAngleBracket]", 
     RowBox[{
      OverscriptBox["v", "\[RightVector]"], ",", 
      RowBox[{
       SubscriptBox[
        OverscriptBox["g", "\[RightVector]"], "p"], "(", 
       OverscriptBox["r", "\[RightVector]"], ")"}]}], 
     "\[RightAngleBracket]"}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Code"
}], "Subsection"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Acceleration", "[", 
   RowBox[{"r_", ",", "v_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"-", "4"}], " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   FractionBox["r", 
    SuperscriptBox[
     RowBox[{"Norm", "[", "r", "]"}], 
     RowBox[{"p", "+", "1"}]]]}]}], "\n", 
 RowBox[{
  RowBox[{"Clear", "[", "f", "]"}], ";", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"r_", ",", "v_"}], "}"}], ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"Acceleration", "[", 
      RowBox[{"r", ",", "v"}], "]"}]}], "}"}]}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"RungeKutta4thOrder", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"z_", ",", "t_"}], "}"}], ",", "f_", ",", "\[CapitalDelta]t_"}],
     "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"With", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"k1", "=", 
         RowBox[{"\[CapitalDelta]t", " ", 
          RowBox[{"f", "[", 
           RowBox[{"z", ",", "t"}], "]"}]}]}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"With", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"k2", "=", 
           RowBox[{"\[CapitalDelta]t", " ", 
            RowBox[{"f", "[", 
             RowBox[{
              RowBox[{"z", "+", 
               RowBox[{
                FractionBox["1", "2"], "k1"}]}], ",", 
              RowBox[{"t", "+", 
               RowBox[{
                FractionBox["1", "2"], "\[CapitalDelta]t"}]}]}], "]"}]}]}], 
          "}"}], ",", "\[IndentingNewLine]", 
         RowBox[{"With", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"k3", "=", 
             RowBox[{"\[CapitalDelta]t", " ", 
              RowBox[{"f", "[", 
               RowBox[{
                RowBox[{"z", "+", 
                 RowBox[{
                  FractionBox["1", "2"], "k2"}]}], ",", 
                RowBox[{"t", "+", 
                 RowBox[{
                  FractionBox["1", "2"], "\[CapitalDelta]t"}]}]}], "]"}]}]}], 
            "}"}], ",", "\[IndentingNewLine]", 
           RowBox[{"With", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"k4", "=", 
               RowBox[{"\[CapitalDelta]t", " ", 
                RowBox[{"f", "[", 
                 RowBox[{
                  RowBox[{"z", "+", "k3"}], ",", 
                  RowBox[{"t", "+", "\[CapitalDelta]t"}]}], "]"}]}]}], "}"}], 
             ",", "\[IndentingNewLine]", 
             RowBox[{"z", "+", 
              RowBox[{
               FractionBox["1", "6"], 
               RowBox[{"(", 
                RowBox[{"k1", "+", 
                 RowBox[{"2", " ", "k2"}], "+", 
                 RowBox[{"2", " ", "k3"}], "+", "k4"}], ")"}]}]}]}], "]"}]}], 
          "]"}]}], "]"}]}], "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"t", "+", "\[CapitalDelta]t"}]}], "}"}]}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"DataPointPosition", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"r_", ",", "v_"}], "}"}], ",", "t_"}], "}"}], "]"}], ":=", 
  "r"}], "\n", 
 RowBox[{
  RowBox[{"DataPointVelocity", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"r_", ",", "v_"}], "}"}], ",", "t_"}], "}"}], "]"}], ":=", 
  "v"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"DataPointTime", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"r_", ",", "v_"}], "}"}], ",", "t_"}], "}"}], "]"}], ":=", 
  "t"}], "\n", 
 RowBox[{"CalculateOrbit", ":=", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"iMaximum", "=", 
     RowBox[{"Floor", "[", 
      FractionBox["tFinal", "\[CapitalDelta]t"], "]"}]}], ";", 
    RowBox[{
     RowBox[{"r", "[", "0", "]"}], "=", 
     RowBox[{"{", 
      RowBox[{"R0", ",", "0"}], "}"}]}], ";", 
    RowBox[{
     RowBox[{"v", "[", "0", "]"}], "=", 
     RowBox[{"{", 
      RowBox[{"0", ",", "V0"}], "}"}]}], ";", 
    RowBox[{
     RowBox[{"t", "[", "0", "]"}], "=", "0"}], ";", "\[IndentingNewLine]", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"i", "=", "0"}], "}"}], ",", 
      RowBox[{
       RowBox[{"While", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"i", "\[LessEqual]", "iMaximum"}], "\[And]", 
          RowBox[{"InRegionQ", "[", 
           RowBox[{"r", "[", "i", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
         
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"r", "[", 
                RowBox[{"i", "+", "1"}], "]"}], ",", 
               RowBox[{"v", "[", 
                RowBox[{"i", "+", "1"}], "]"}]}], "}"}], ",", 
             RowBox[{"t", "[", 
              RowBox[{"i", "+", "1"}], "]"}]}], "}"}], "=", 
           RowBox[{"RungeKutta4thOrder", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"r", "[", "i", "]"}], ",", 
                 RowBox[{"v", "[", "i", "]"}]}], "}"}], ",", 
               RowBox[{"t", "[", "i", "]"}]}], "}"}], ",", "f", ",", 
             "\[CapitalDelta]t"}], "]"}]}], ";", 
          RowBox[{"i", "++"}]}]}], "]"}], ";", 
       RowBox[{"iMaximum", "=", "i"}]}]}], "]"}]}], ")"}]}], "\n", 
 RowBox[{
  RowBox[{"DisplayOrbit", "[", "stuff___", "]"}], ":=", 
  RowBox[{"Show", "[", 
   RowBox[{"Graphics", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"stuff", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"RGBColor", "[", 
          RowBox[{"1", ",", "1", ",", "1"}], "]"}], ",", 
         RowBox[{"Thickness", "[", "0.0075`", "]"}], ",", 
         RowBox[{"Line", "[", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"r", "[", "i", "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "0", ",", "iMaximum"}], "}"}]}], "]"}], 
          "]"}]}], "}"}], ",", "Sun"}], "}"}], ",", 
     RowBox[{"Background", "\[Rule]", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0.6`", ",", "0.6`", ",", "0.6`"}], "]"}]}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}]}], "]"}], 
   "]"}]}], "\n", 
 RowBox[{
  RowBox[{"Sun", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"RGBColor", "[", 
      RowBox[{"1", ",", "1", ",", "0"}], "]"}], ",", 
     RowBox[{"Disk", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", "0.03`"}], "]"}]}], "}"}]}], 
  ";"}], "\n", 
 RowBox[{"OrbitData", ":=", 
  RowBox[{"Module", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"i", "=", "1"}], ",", "rMin", ",", "rMax", ",", "Perihelion", 
      ",", "Aphelion", ",", "Period"}], "}"}], ",", 
    RowBox[{
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"i", "\[LessEqual]", "iMaximum"}], "\[And]", 
        RowBox[{
         RowBox[{
          RowBox[{"r", "[", "i", "]"}], "\[LeftDoubleBracket]", "2", 
          "\[RightDoubleBracket]"}], ">", "0"}]}], ",", 
       RowBox[{"i", "=", 
        RowBox[{"i", "+", "1"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"With", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\[Lambda]", "=", 
         FractionBox[
          RowBox[{
           RowBox[{"r", "[", 
            RowBox[{"i", "-", "1"}], "]"}], "\[LeftDoubleBracket]", "2", 
           "\[RightDoubleBracket]"}], 
          RowBox[{
           RowBox[{
            RowBox[{"r", "[", 
             RowBox[{"i", "-", "1"}], "]"}], "\[LeftDoubleBracket]", "2", 
            "\[RightDoubleBracket]"}], "-", 
           RowBox[{
            RowBox[{"r", "[", "i", "]"}], "\[LeftDoubleBracket]", "2", 
            "\[RightDoubleBracket]"}]}]]}], "}"}], ",", "\[IndentingNewLine]",
        
       RowBox[{
        RowBox[{"Period", "=", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"\[Lambda]", " ", 
             RowBox[{"t", "[", 
              RowBox[{"i", "-", "1"}], "]"}]}], "+", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Lambda]"}], ")"}], " ", 
             RowBox[{"t", "[", "i", "]"}]}]}], ")"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"R1", "=", 
         RowBox[{"Min", "[", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{
             RowBox[{"r", "[", 
              RowBox[{"i", "-", "1"}], "]"}], "\[LeftDoubleBracket]", "1", 
             "\[RightDoubleBracket]"}]}], ",", 
           RowBox[{"-", 
            RowBox[{
             RowBox[{"r", "[", "i", "]"}], "\[LeftDoubleBracket]", "1", 
             "\[RightDoubleBracket]"}]}]}], "]"}]}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"rMin", ",", "rMax"}], "}"}], "=", 
      RowBox[{"Sort", "[", 
       RowBox[{"{", 
        RowBox[{"R0", ",", "R1"}], "}"}], "]"}]}], ";", "\[IndentingNewLine]",
      
     RowBox[{"SemimajorAxis", "=", 
      FractionBox[
       RowBox[{"rMin", "+", "rMax"}], "2"]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"i", "\[GreaterEqual]", "iMaximum"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
       "Print", "[", "\"\<tFinal is too small to find data.\>\"", "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{
         "\"\<Perihelion = \!\(\*SubscriptBox[\(r\), \(min\)]\) = \>\"", ",", 
          "rMin", ",", "\"\< AU\>\""}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Print", "[", 
         RowBox[{
         "\"\<Aphelion = \!\(\*SubscriptBox[\(r\), \(max\)]\) = \>\"", ",", 
          "rMax", ",", "\"\< AU\>\""}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Print", "[", 
         RowBox[{"\"\<Semimajor Axis = a = \>\"", ",", 
          FractionBox[
           RowBox[{"rMax", "+", "rMin"}], "2"], ",", "\"\< AU\>\""}], "]"}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"Print", "[", 
         RowBox[{"\"\<Period = \>\"", ",", "Period", ",", "\"\< yr\>\""}], 
         "]"}]}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}]}], "\n", 
 RowBox[{
  RowBox[{"TriangleArea", "[", 
   RowBox[{"A_", ",", "B_"}], "]"}], ":=", 
  RowBox[{
   FractionBox["1", "2"], 
   RowBox[{"Abs", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"A", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
      " ", 
      RowBox[{"B", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
     "-", 
     RowBox[{
      RowBox[{"A", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
      " ", 
      RowBox[{"B", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}]}],
     "]"}]}]}], "\n", 
 RowBox[{
  RowBox[{"SweptArea", "[", 
   RowBox[{"ti_", ",", "tf_"}], "]"}], ":=", 
  RowBox[{"Module", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"mi", "=", "0"}], ",", 
      RowBox[{"mf", "=", "0"}], ",", "ri", ",", "rf", ",", "Area", ",", 
      "PolygonList"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"t", "[", "mi", "]"}], "<", "ti"}], "\[And]", 
        RowBox[{"mi", "\[LessEqual]", "iMaximum"}]}], ",", 
       RowBox[{"mi", "=", 
        RowBox[{"mi", "+", "1"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"With", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\[Lambda]", "=", 
         FractionBox[
          RowBox[{
           RowBox[{"t", "[", "mi", "]"}], "-", "ti"}], "\[CapitalDelta]t"]}], 
        "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"ri", "=", 
        RowBox[{
         RowBox[{"\[Lambda]", " ", 
          RowBox[{"r", "[", 
           RowBox[{"mi", "+", "1"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "\[Lambda]"}], ")"}], " ", 
          RowBox[{"r", "[", "mi", "]"}]}]}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"t", "[", "mf", "]"}], "<", "tf"}], "\[And]", 
        RowBox[{"mf", "\[LessEqual]", "iMaximum"}]}], ",", 
       RowBox[{"mf", "=", 
        RowBox[{"mf", "+", "1"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"With", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\[Lambda]", "=", 
         FractionBox[
          RowBox[{
           RowBox[{"t", "[", "mf", "]"}], "-", "tf"}], "\[CapitalDelta]t"]}], 
        "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"rf", "=", 
        RowBox[{
         RowBox[{"\[Lambda]", " ", 
          RowBox[{"r", "[", "mf", "]"}]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "\[Lambda]"}], ")"}], " ", 
          RowBox[{"r", "[", 
           RowBox[{"mf", "+", "1"}], "]"}]}]}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Area", "=", 
      RowBox[{
       RowBox[{"TriangleArea", "[", 
        RowBox[{"ri", ",", 
         RowBox[{"r", "[", 
          RowBox[{"mi", "+", "1"}], "]"}]}], "]"}], "+", 
       RowBox[{"TriangleArea", "[", 
        RowBox[{
         RowBox[{"r", "[", "mf", "]"}], ",", "rf"}], "]"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"PolygonList", "=", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Polygon", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", "0"}], "}"}], ",", "ri", ",", 
           RowBox[{"r", "[", 
            RowBox[{"mi", "+", "1"}], "]"}]}], "}"}], "]"}], ",", 
        RowBox[{"Polygon", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", "0"}], "}"}], ",", 
           RowBox[{"r", "[", "mf", "]"}], ",", "rf"}], "}"}], "]"}]}], 
       "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"Area", "=", 
         RowBox[{"Area", "+", 
          RowBox[{"TriangleArea", "[", 
           RowBox[{
            RowBox[{"r", "[", "m", "]"}], ",", 
            RowBox[{"r", "[", 
             RowBox[{"m", "+", "1"}], "]"}]}], "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"PolygonList", "=", 
         RowBox[{"Append", "[", 
          RowBox[{"PolygonList", ",", 
           RowBox[{"Polygon", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0", ",", "0"}], "}"}], ",", 
              RowBox[{"r", "[", "m", "]"}], ",", 
              RowBox[{"r", "[", 
               RowBox[{"m", "+", "1"}], "]"}]}], "}"}], "]"}]}], "]"}]}]}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"m", ",", 
         RowBox[{"mi", "+", "1"}], ",", 
         RowBox[{"mf", "-", "1"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{
      "\"\<Area between \!\(\*SubscriptBox[\(t\), \(i\)]\) = \>\"", ",", "ti",
        ",", "\"\< yr and \!\(\*SubscriptBox[\(t\), \(f\)]\) = \>\"", ",", 
       "tf", ",", "\"\< yr  is \>\"", ",", "Area", ",", 
       "\"\< \!\(\*SuperscriptBox[\(AU\), \(2\)]\)\>\""}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"RGBColor", "[", 
        RowBox[{".2", ",", ".2", ",", ".6"}], "]"}], ",", "PolygonList"}], 
      "}"}]}]}], "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rMin", "=", ".05"}], ";", 
  RowBox[{"rMax", "=", "50"}], ";", 
  RowBox[{
   RowBox[{"InRegionQ", "[", "r_List", "]"}], ":=", 
   RowBox[{"rMin", "\[LessEqual]", 
    RowBox[{"Norm", "[", "r", "]"}], "\[LessEqual]", "rMax"}]}]}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.401201516297084*^9, 3.4012015236441*^9}}],

Cell[TextData[{
 "The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " code for the functions defined above is hidden here.   "
}], "Text",
 CellChangeTimes->{{3.401202310449915*^9, 3.4012023257943287`*^9}, {
  3.401276825988501*^9, 3.401276827087308*^9}}],

Cell[TextData[{
 StyleBox["CalculateOrbit",
  FontWeight->"Bold"],
 " is the code to use one of the above iteration methods to calculate the \
orbit data.  In addition values will need to be given to the variables: ",
 StyleBox["tFinal",
  FontWeight->"Bold"],
 ", ",
 StyleBox["R0",
  FontWeight->"Bold"],
 ", ",
 StyleBox["V0",
  FontWeight->"Bold"],
 ", ",
 StyleBox["\[CapitalDelta]t",
  FontWeight->"Bold"],
 " and ",
 StyleBox["p",
  FontWeight->"Bold"],
 "."
}], "Text",
 CellChangeTimes->{3.401193273546269*^9}],

Cell[TextData[{
 StyleBox["DisplayOrbit",
  FontWeight->"Bold"],
 " is the code to display the orbit's trajectory.  Extra information (labeled \
",
 StyleBox["stuff",
  FontWeight->"Bold"],
 ") may be added to the graphic."
}], "Text"],

Cell[TextData[{
 "To evaluate a cell click the mouse anywhere inside the cell and click ",
 StyleBox["Shift-Enter",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellChangeTimes->{{3.401202310449915*^9, 3.4012023527973833`*^9}}]
}, {1, 3, 4, 5, 6}]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["(A) Inverse Square Law",
 FontSize->16]], "Section"],

Cell[TextData[{
 "Here we will simulate the Earth's orbit (",
 Cell[BoxData[
  FormBox[
   RowBox[{"R0", "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"V0", "=", 
    RowBox[{"2", " ", "\[Pi]"}]}], TraditionalForm]]],
 ") over a two year time interval (",
 Cell[BoxData[
  FormBox[
   RowBox[{"tFinal", "=", "2"}], TraditionalForm]]],
 "), using an overly large time step of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]t", "=", "0.1"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tFinal", "=", "2"}], ";", 
  RowBox[{"R0", "=", "1"}], ";", 
  RowBox[{"V0", "=", 
   RowBox[{"2", " ", "\[Pi]"}]}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", ".1"}], ";", 
  RowBox[{"p", "=", "2"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CalculateOrbit", ";", 
  RowBox[{"DisplayOrbit", "[", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Clear", "[", 
  RowBox[{
  "tFinal", ",", "R0", ",", "V0", ",", "iMaximum", ",", "t", ",", "r", ",", 
   "v"}], "]"}]}], "Input",
 CellChangeTimes->{{3.401147228550185*^9, 3.40114723385944*^9}}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 1, 1], Thickness[0.0075], 
    LineBox[{{1, 0}, {0.808871203274945, 0.5865983890012474}, {
      0.3087388983953714, 0.9482313803961031}, {-0.3093435298079089, 
      0.9448096852593972}, {-0.8064174015763488, 
      0.5750214670205713}, {-0.9865360060850484, -0.019453739538471515`}, \
{-0.7755426470602922, -0.6051591601469972}, {-0.2549585268186926, \
-0.9480710053133833}, {0.36738302708938964`, -0.909393468574681}, {
      0.840829360402113, -0.5041018930613567}, {0.9741932489593657, 
      0.10425407970754208`}, {0.7132743268301338, 0.6696327403064702}, {
      0.16302474958695234`, 0.9621184895513653}, {-0.45291336184042874`, 
      0.8603965924677726}, {-0.8803652687107446, 
      0.4032502420464755}, {-0.9387351734505553, -0.2215335068270503}, \
{-0.5999953345510562, -0.7513116698748366}, {-0.00586131147582758, \
-0.9594426335764439}, {0.5897606318621001, -0.7552119478678185}, {
      0.9301609311390704, -0.22572706193370518`}, {0.8676610310354445, 
      0.4005100093922327}, {0.42825529531482065`, 0.8515380372725203}}]}, 
   {RGBColor[1, 1, 0], DiskBox[{0, 0}, 0.03]}},
  AspectRatio->Automatic,
  Background->RGBColor[0.6, 0.6, 0.6],
  ImageSize->{295.5, Automatic}]], "Output",
 CellChangeTimes->{3.401147237903009*^9, 3.40114728159017*^9, 
  3.401193044739525*^9, 3.401194239712131*^9, 3.401194316113969*^9, 
  3.401194410920349*^9, 3.4011947454517803`*^9, 3.401194793451723*^9, 
  3.40119488322106*^9, 3.401195090057837*^9, 3.401195138747272*^9, 
  3.4011954162633038`*^9, 3.4012768407114077`*^9, 3.401281912958885*^9}]
}, Open  ]],

Cell[TextData[{
 "Now try a smaller \[CapitalDelta]t. value of 0.01.  To copy the preceding \
input move the cursor below a cell and click.  When you see the horizontal \
line and type ",
 StyleBox["Control-L",
  FontSlant->"Italic"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tFinal", "=", "2"}], ";", 
  RowBox[{"R0", "=", "1"}], ";", 
  RowBox[{"V0", "=", 
   RowBox[{"2", " ", "\[Pi]"}]}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", ".01"}], ";", 
  RowBox[{"p", "=", "2"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CalculateOrbit", ";", 
  RowBox[{"DisplayOrbit", "[", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Clear", "[", 
  RowBox[{
  "tFinal", ",", "R0", ",", "V0", ",", "iMaximum", ",", "t", ",", "r", ",", 
   "v"}], "]"}]}], "Input",
 CellChangeTimes->{{3.4011472432515078`*^9, 3.4011472444070187`*^9}}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 1, 1], Thickness[0.0075], 
    LineBox[{{1, 0}, {0.998026728035636, 0.06279051132432557}, {
      0.9921147005907914, 0.12533321713650664`}, {0.9822872497179352, 
      0.18738128995072337`}, {0.968583159846038, 0.2486898543669286}, {
      0.9510565147169145, 0.30901695348013275`}, {0.9297764839424637, 
      0.3681245037718368}, {0.9048270500251513, 0.4257792347151311}, {
      0.8763066769190387, 0.48175360938530576`}, {0.8443279214393697, 
      0.5358267224427764}, {0.8090169890542741, 0.5877851719444273}, {
      0.7705132358116404, 0.6374239015427565}, {0.7289686183667891, 
      0.6845470097490711}, {0.6845470942813933, 0.7289685230669543}, {
      0.6374239749603541, 0.7705131299448137}, {0.5877852337802476, 
      0.8090168726509415}, {0.5358267721398016, 0.8443277943405699}, {
      0.48175364632892104`, 0.8763065387612426}, {0.42577925826741597`, 
      0.9048269002297246}, {0.36812451330717183`, 0.9297763217099292}, {
      0.30901694842149585`, 0.9510563390261556}, {0.2486898342222396, 
      0.968582969458501}, {0.18738125434859354`, 0.9822870431868116}, {
      0.12533316586075632`, 0.9921144762750781}, {0.06279044434665075, 
      0.9980264841188926}, {-8.248980544467432*^-8, 
      0.9999997345135476}, {-0.06279060894490317, 
      0.9980264397386419}, {-0.1253333292914373, 
      0.9921143872957333}, {-0.18738141575547154`, 
      0.9822869091776847}, {-0.24868999263394273`, 
      0.9685827897909364}, {-0.30901710270869986`, 
      0.9510561128940469}, {-0.36812466214296874`, 
      0.9297760481565225}, {-0.4257794000916833, 
      0.9048265781802458}, {-0.48175377931744623`, 
      0.8763061670607856}, {-0.5358268941788391, 
      0.8443273717965867}, {-0.587785342447098, 
      0.8090163980795986}, {-0.6374240675104557, 
      0.7705126022204024}, {-0.6845471676423118, 
      0.7289679411734881}, {-0.7289686691393775, 
      0.6845463728331598}, {-0.7705132602785227, 
      0.6374232089668145}, {-0.8090169831956877, 
      0.5877844233392621}, {-0.8443278809562804, 
      0.5358259177586016}, {-0.8763065972633187, 
      0.481752748940239}, {-0.9048269264364974, 
      0.4257783192402705}, {-0.9297763114916155, 
      0.3681235344520513}, {-0.9510562883546296, 
      0.30901593198976396`}, {-0.9685828744571802, 
      0.24868878289967195`}, {-0.9822869001799445, 
      0.18738017124304146`}, {-0.9921142818357264, 
      0.12533205448402107`}, {-0.9980262351149137, 
      0.06278930859073809}, {-0.9999994281509065, -1.2383817963868005`*^-6}, \
{-0.9980260736014173, -0.06279178041292188}, {-0.9921139593822267, \
-0.12533451149724428`}, {-0.9822864179320945, -0.1873826036255405}, \
{-0.9685822341301199, -0.24869118090595949`}, {-0.951055492228964, \
-0.3090182859810597}, {-0.929775362408028, -0.3681258349266281}, \
{-0.9048258277889788, -0.42578055686368593`}, {-0.8763053529909879, \
-0.481754914575417}, {-0.844326495533771, -0.5358280024957234}, \
{-0.8090154616220715, -0.58778641852542}, {-0.7705116080647395, \
-0.6374251062353753}, {-0.7289668922941421, -0.684548164132776}, \
{-0.6845452726564683, -0.7289696187966893}, {-0.6374220613397514, \
-0.77051415883169}, {-0.5877832324933553, -0.8090178267429601}, \
{-0.5358246882695024, -0.8443286660023366}, {-0.48175148568349385`, \
-0.8763073207516345}, {-0.42577702734389594`, -0.904827585776491}, \
{-0.36812221924655597`, -0.9297769045802459}, {-0.3090145989562954, \
-0.9510568135922036}, {-0.24868743761698936`, -0.9685833307572096}, \
{-0.1873788193340268, -0.9822872869729795}, {-0.12533070156244386`, \
-0.9921145990671806}, {-0.06278796020897907, -0.998026483236981}, {
      2.576783376714853*^-6, -0.9999996081087565}, {
      0.0627931035543144, -0.9980261868139373}, {
      0.12533581430420604`, -0.9921140077176532}, {
      0.18738388127152633`, -0.9822864036789511}, {
      0.24869242885006118`, -0.9685821599639417}, {
      0.30901950000150774`, -0.9510553611753453}, {
      0.3681270111494829, -0.929775177802579}, {
      0.4257816917864208, -0.9048255932348153}, {
      0.4817560050851769, -0.8763050723144068}, {
      0.5358290458821261, -0.8443261727387832}, {
      0.5877874124878507, -0.8090151008444724}, {
      0.637426048884811, -0.7705112135263938}, {
      0.6845490539882599, -0.728966468258147}, {
      0.7289704547766558, -0.6845448233838348}, {
      0.7705149402404188, -0.6374215910482126}, {
      0.8090185532525094, -0.5877827453188628}, {
      0.8443293376303737, -0.5358241882309269}, {
      0.8763079378356372, -0.4817509766509817}, {
      0.9048281489448484, -0.425776513011269}, {
      0.9297774147208268, -0.3681217031080724}, {
      0.9510572718189934, -0.3090140842880097}, {
      0.9685837383756397, -0.24868692746287213`}, {
      0.9822876454445864, -0.187378316496923}, {
      0.9921149099742311, -0.1253302085998652}, {
      0.9980267482476949, -0.06278747943364542}, {0.9999998289437353, 
      3.0432984128858642`*^-6}, {0.9980263652143813, 0.06279355396620201}, {
      0.9921141454159105, 0.12533624698680237`}, {0.9822865023718937, 
      0.18738429479825763`}, {0.9685822212896101, 0.2486928219735775}, {
      0.9510553866932395, 0.3090198716306249}, {0.9297751689779056, 
      0.36812736032394816`}, {0.9048255514269271, 0.4257820176500741}, {
      0.8763049987695952, 0.4817563068581032}, {0.8443260685875014, 
      0.5358293228324101}, {0.8090149671029885, 0.5877876639035502}, {
      0.7705110511027919, 0.6374262740467569}, {0.7289662779625142, 
      0.6845492521442776}, {0.6845446059423963, 0.7289706251177744}, {
      0.637421347121103, 0.7705150818795985}, {0.5877824755211567, 
      0.8090186652063739}, {0.535823893156488, 0.8443294188043797}, {
      0.48175065689863733`, 0.8763079870130533}, {0.425776169212771, 
      0.9048281647798471}, {0.3681213359572684, 0.9297773957359219}, {
      0.30901369457062167`, 0.9510572164069787}, {0.2486865160862618, 
      0.9685836448061027}, {0.18737788451921727`, 0.9822875118749907}, {
      0.12532975725781792`, 0.9921147344654807}, {0.06278701016862615, 
      0.9980265287840085}, {-3.5288168877128934`*^-6, 
      0.9999995634565856}, {-0.06279405381994675, 
      0.9980260516101733}, {-0.12533675899195545`, 
      0.9921137816070079}, {-0.1873848164922452, 
      0.9822860863105335}, {-0.24869335060644177`, 
      0.9685817510041915}, {-0.30902040416104476`, 
      0.9510548603264221}, {-0.36812789342020574`, 
      0.9297745848259114}, {-0.42578254769614343`, 
      0.9048249079794104}, {-0.48175682996495583`, 
      0.8763042947494222}, {-0.5358298348545483, 
      0.8443253029897712}, {-0.5877881604606937, 
      0.8090141392326275}, {-0.6374267505505256, 
      0.7705101606100633}, {-0.6845497038296287, 
      0.7289653248758048}, {-0.7289710470788737, 
      0.6845435906976767}, {-0.770515469109731, 
      0.6374202705874955}, {-0.8090190126414272, 
      0.5877813390220973}, {-0.8443297213693719, 
      0.5358226984860268}, {-0.8763082396711221, 
      0.48174940633248975`}, {-0.9048283625830686, 
      0.4257748655138029}, {-0.9297775338772906, 
      0.36811998237517873`}, {-0.9510572902728519, 
      0.309012294835668}, {-0.9685836500283971, 
      0.2486850743969605}, {-0.9822874443824101, 
      0.187376405524037}, {-0.9921145905330592, 
      0.12532824603097378`}, {-0.9980263050802691, 
      0.062785472180168}, {-0.9999992570876387, -5.087736414235988*^-6}, \
{-0.9980256601601172, -0.06279562751990724}, {-0.9921133031740375, \
-0.1253383410466664}, {-0.9822855195382201, -0.18738640025014133`}, \
{-0.9685810951074584, -0.24869492924280273`}, {-0.9510541151118115, \
-0.3090219707336585}, {-0.9297737507058156, -0.3681294409273006}, \
{-0.9048239859800213, -0.4257840691358842}, {-0.8763032865122392, \
-0.48175831839577393`}, {-0.8443242107665956, -0.5358312834555613}, \
{-0.8090129658740619, -0.5877895625914645}, {-0.7705089095476226, \
-0.6374280998094193}, {-0.7289640000978351, -0.6845509941100419}, \
{-0.684542196719361, -0.7289722726226412}, {-0.6374188124153085, \
-0.7705166245570497}, {-0.5877798221131463, -0.80902009307669}, \
{-0.5358211287027848, -0.8443307223623568}, {-0.4817477898934884, \
-0.8763091573130496}, {-0.42577320894090737`, -0.90482919351711}, \
{-0.3681182924381087, -0.9297782753231908}, {-0.30901057849446445`, \
-0.9510579400454094}, {-0.24868333874312198`, -0.9685842065496312}, \
{-0.18737465772106066`, -0.9822879066873341}, {-0.12532649325503994`, \
-0.9921149582690664}, {-0.06278372156169777, -0.9980265785002259}, {
      6.829169618305375*^-6, -0.9999994370368669}, {
      0.06279735289725719, -0.9980257480575294}, {
      0.12534004370648089`, -0.9921133009877687}, {
      0.1873880737878422, -0.9822854297561714}, {
      0.24869656755508185`, -0.9685809207032341}, {
      0.3090235680581457, -0.951053859506591}, {
      0.3681309918773346, -0.9297734177267487}, {
      0.4257855687300659, -0.9048235798157778}, {
      0.4817597620820616, -0.876302811666373}, {
      0.5358326671297512, -0.8443236720094058}, {
      0.5877908826099862, -0.8090123681936385}, {
      0.6374293529964604, -0.7705082581007678}, {
      0.6845521777591006, -0.7289633001615365}, {
      0.7289733844926876, -0.6845414536434721}, {
      0.7705176628635562, -0.637418031577059}, {
      0.8090210564778613, -0.5877790088740634}, {
      0.8443316099409615, -0.5358202883685568}, {
      0.8763099685543112, -0.4817469276772422}, {
      0.9048299282831717, -0.4257723299303091}, {
      0.9297789338247071, -0.3681174015666232}, {
      0.9510585228108559, -0.30900968051703576`}, {
      0.968584714392945, -0.24868243821645641`}, {
      0.9822883406745126, -0.18737375898861514`}, {
      0.9921153196843534, -0.12532560043673444`}, {
      0.9980268688125103, -0.06278283854829053}, {0.9999996578668982, 
      7.698717633924934*^-6}, {0.9980259011465289, 0.06279820554645163}}]}, 
   {RGBColor[1, 1, 0], DiskBox[{0, 0}, 0.03]}},
  AspectRatio->Automatic,
  Background->RGBColor[0.6, 0.6, 0.6],
  ImageSize->{301.5, Automatic}]], "Output",
 CellChangeTimes->{3.4011472452123117`*^9, 3.401147288374295*^9, 
  3.401193051188056*^9, 3.4011948929590483`*^9, 3.401276846318138*^9}]
}, Open  ]],

Cell[TextData[{
 "\[LineSeparator]Try giving the earth half the speed needed for a circular \
orbit, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"V0", "=", 
    RowBox[{"0.5", "\[Times]", "2", " ", "\[Pi]"}]}], TraditionalForm]]],
 ".  To multiply in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " use a \[LineSeparator]space."
}], "Text",
 CellChangeTimes->{3.401193054161109*^9, 3.401194895653761*^9}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tFinal", "=", "2"}], ";", 
  RowBox[{"R0", "=", "1"}], ";", 
  RowBox[{"V0", "=", 
   RowBox[{"0.53", " ", "2", " ", "\[Pi]"}]}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", ".01"}], ";", 
  RowBox[{"p", "=", "2"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CalculateOrbit", ";", 
  RowBox[{"DisplayOrbit", "[", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Clear", "[", 
  RowBox[{
  "tFinal", ",", "R0", ",", "V0", ",", "iMaximum", ",", "t", ",", "r", ",", 
   "v"}], "]"}]}], "Input",
 CellChangeTimes->{{3.401147249401937*^9, 3.4011472502309713`*^9}}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 1, 1], Thickness[0.0075], 
    LineBox[{{1, 0}, {0.9980253265436033, 0.03327894765488235}, {
      0.9920922542782361, 0.06642571484573057}, {0.9821734007962779, 
      0.0993052266655356}, {0.9682223878635804, 0.13177645103352484`}, {
      0.9501726867340387, 0.16368898041866298`}, {0.9279359107987715, 
      0.1948790295717652}, {0.90139945464651, 0.22516454920684956`}, {
      0.8704233340276067, 0.2543390388785959}, {0.8348360226063638, 
      0.28216345429472933`}, {0.7944290034865237, 0.30835529819584245`}, {
      0.748949651124146, 0.3325734742251224}, {0.6980919315255034, 
      0.35439660818309315`}, {0.6414842733755294, 0.3732909818718053}, {
      0.5786739032195666, 0.38856132145142525`}, {0.5091072566276512, 
      0.3992719904439079}, {0.43210782711862666`, 0.4041142866700934}, {
      0.34685977387495415`, 0.4011690768647682}, {0.25243245040194495`, 
      0.387449991583923}, {0.14799205439248492`, 0.35794433189437913`}, {
      0.033873312556530685`, 0.3034102442304968}, {-0.08286163109489006, 
      0.2053762847140786}, {-0.16069241286410602`, 
      0.03707339285221303}, {-0.12150466377054304`, -0.15006586586521709`}, \
{-0.010045862522610596`, -0.2674296043501083}, {
      0.10746705432611654`, -0.3285069076741567}, {
      0.21548182570097296`, -0.3572050035240416}, {
      0.3119297098908549, -0.3660348841679876}, {
      0.39732757214207887`, -0.3616602716350944}, {
      0.4726046281565123, -0.34796838675100894`}, {
      0.5386397024265885, -0.3274176109523269}, {
      0.5961728640486487, -0.3016677255542819}, {
      0.6458055448340214, -0.2719008773900561}, {
      0.6880195187340249, -0.23899799933578364`}, {
      0.7231968281157894, -0.20364230290715749`}, {
      0.7516364205980954, -0.16638351725106382`}, {
      0.7735670025750718, -0.12767986728678543`}, {
      0.7891565520703447, -0.08792692312337308}, {
      0.7985190574787966, -0.04747852617922771}, {
      0.8017189505371901, -0.006662947133409425}, {0.7987735575928354, 
      0.03420366297906814}, {0.7896537502750036, 0.07480503357088347}, {
      0.7742828415219846, 0.11481249077144703`}, {0.7525336392936465, 
      0.15387136173608587`}, {0.7242234286754802, 0.1915845351469435}, {
      0.6891064959619817, 0.22749104003625928`}, {0.6468636398500948, 
      0.26103634045942836`}, {0.597087979838253, 0.29152885522563154`}, {
      0.5392664484883443, 0.3180729976592834}, {0.47275726228493464`, 
      0.33946053136650123`}, {0.3967675556923871, 0.35398383803929645`}, {
      0.3103501128699333, 0.35909285558363796`}, {0.2124955388781487, 
      0.3507135693208132}, {0.1026432685545236, 
      0.32177190792122046`}, {-0.016830090982189974`, 
      0.25881055929459934`}, {-0.127841238783489, 
      0.13649375370323275`}, {-0.15837511217762162`, -0.0511159861183802}, \
{-0.07338625832703663, -0.2081171919268856}, {
      0.0463773974380087, -0.29157452540677176`}, {
      0.16107281360463083`, -0.3289607663889725}, {
      0.26323453326417934`, -0.3392075357418691}, {
      0.352344844449778, -0.33220454748097566`}, {
      0.4292109592807564, -0.31348641954636725`}, {
      0.4947804225214093, -0.2864351685302191}, {
      0.5498832832099991, -0.2532797378262554}, {
      0.5951905930684983, -0.21558579995004296`}, {
      0.6312244571047575, -0.1745163189865579}, {
      0.6583773257381234, -0.1309809482942582}, {
      0.6769294417897321, -0.08572793971713702}, {
      0.6870619195198382, -0.039404964854009905`}, {0.6888652512879008, 
      0.007397072627657736}, {0.6823435198976259, 0.0541089525489874}, {
      0.6674145233226428, 0.1001511573158091}, {0.6439057836063315, 
      0.14490024980646454`}, {0.6115461457636886, 0.1876483845231302}, {
      0.5699524763129012, 0.22754865481257464`}, {0.5186110986156196, 
      0.26353427213773517`}, {0.45685489750403085`, 0.2941895709366279}, {
      0.383842413638414, 0.317528946605499}, {0.29856552695832106`, 
      0.33058893709201637`}, {0.199991635876507, 0.3286117176094469}, {
      0.08779169897120521, 0.30326864131363623`}, {-0.034153411800110944`, 
      0.23869151637420924`}, {-0.14042035006049292`, 
      0.1071859977007524}, {-0.15022421124700108`, -0.08077299298781343}, \
{-0.052223949857802876`, -0.22132491854323322`}, {
      0.07016589315120794, -0.288916060669285}, {
      0.18370943349637464`, -0.3118462465194985}, {
      0.2823045484685318, -0.308152745888774}, {
      0.3656959057367424, -0.2874783364026201}, {
      0.43474400933777047`, -0.25536801751042504`}, {
      0.4903877475852673, -0.21530457233165798`}, {
      0.5334131914359442, -0.16965756393456963`}, {
      0.5644159936728818, -0.12016163316762589`}, {
      0.5838096678240875, -0.06818005731321125}, {
      0.5918405885350502, -0.014864722811596454`}, {0.5885989796326063, 
      0.03873464190856041}, {0.574023040856148, 0.09158336953599677}, {
      0.5478955199833418, 0.14257768100063986`}, {0.5098327640885684, 
      0.1904453718228963}, {0.45926756482460107`, 0.23360694512152927`}, {
      0.3954319220557501, 0.2699545537722311}, {0.3173635822211681, 
      0.29646206546701276`}, {0.22402732909005948`, 0.30843114304246094`}, {
      0.11492111845891746`, 0.29790365557871135`}, {-0.007135163503789219, 
      0.2501863100109037}, {-0.12268252058646285`, 
      0.13890352713765547`}, {-0.15871623319277106`, -0.04227952552211184}, \
{-0.07480452935953127, -0.1961405730645005}, {
      0.04864210137107036, -0.27073768923177277`}, {
      0.16532769794979918`, -0.29271123578022695`}, {
      0.2653633700282114, -0.28347922299479456`}, {
      0.3476134703554693, -0.25475021509939766`}, {
      0.412728524049386, -0.21326433533922196`}, {
      0.4615785952485157, -0.16327893541502453`}, {
      0.4948830213216606, -0.10774529778382008`}, {
      0.5131297013274021, -0.048908670387268524`}, {0.5165651722452443, 
      0.011348424696730378`}, {0.505196704707451, 0.07127586973037711}, {
      0.47879004477848536`, 0.1290627952093357}, {0.43686030664488834`, 
      0.1826259549500857}, {0.37866419475631513`, 0.2293137180205812}, {
      0.3032285853295062, 0.26541003285562825`}, {0.20954891862887573`, 
      0.2851885283042404}, {0.09750441980021532, 
      0.2789315979867912}, {-0.02798805318336753, 
      0.22878075222645755`}, {-0.13815522443159112`, 
      0.10573452625812794`}, {-0.14947859863394064`, -0.0768862485098581}, \
{-0.04930865281616312, -0.21115495098597226`}, {
      0.07679846291652484, -0.2659526175235206}, {
      0.19060549079162542`, -0.270036736184927}, {
      0.28401894400111516`, -0.24387982579561296`}, {
      0.3561497071851507, -0.19901768416977933`}, {
      0.40757775002823027`, -0.14240801372967482`}, {
      0.4389907184810758, -0.07872290235889416}, {
      0.45084117612390445`, -0.011495858924222932`}, {0.44325942467002316`, 
      0.05622802064864381}, {0.41603175608175685`, 0.12142938759789958`}, {
      0.3686023848114219, 0.18065872256432575`}, {0.3001270435275694, 
      0.22940997268120517`}, {0.20973557708201568`, 0.2610267976499412}, {
      0.0976526524252047, 0.26451080726336035`}, {-0.0299170364890098, 
      0.22019024623321792`}, {-0.14005002462880797`, 
      0.09828270623527413}, {-0.14668501150421678`, -0.08167238180844383}, \
{-0.042480962286170626`, -0.20881361436239887`}, {
      0.08566000959431211, -0.2532235181243309}, {
      0.19816812029777908`, -0.24414133245876618`}, {
      0.28599864898810523`, -0.20332801696914898`}, {
      0.3477988728699247, -0.1435756905589839}, {
      0.3838241323605651, -0.07301102441474673}, {0.39441550813091864`, 
      0.002452640904307926}, {0.37959698807007153`, 0.07774810913026538}, {
      0.3389913743072872, 0.14767526899886735`}, {0.27192111885467884`, 
      0.20585254300428918`}, {0.17793525758062467`, 0.24308757730736757`}, {
      0.05897278842757778, 0.24430881525752607`}, {-0.07150517950762064, 
      0.18372269304696107`}, {-0.15750122467475797`, 
      0.036111595274824465`}, {-0.11740031670869658`, -0.13416360466759833`}, \
{0.005189996395958879, -0.2243993789874285}, {
      0.13088925813213434`, -0.236627238929417}, {
      0.2318680568625997, -0.20059587043321372`}, {
      0.3014319598014718, -0.13650308118026303`}, {
      0.3382170049930235, -0.057267735699159605`}, {0.34191265335624177`, 
      0.0276466025465429}, {0.31214973098149906`, 0.10977951177219668`}, {
      0.2484134259201572, 0.17967534713190664`}, {0.15093932901504875`, 
      0.22434370489202096`}, {0.02465860872976927, 
      0.2230892736990324}, {-0.10507662834304829`, 
      0.14406653236654338`}, {-0.15858530950776586`, -0.020080764868965983`}, \
{-0.08118936408289788, -0.16836146605100044`}, {
      0.050326997321281694`, -0.22346396317097744`}, {
      0.16899999099699792`, -0.20478294792188118`}, {
      0.25309098804712676`, -0.1420558148939021}, {
      0.29658654233816717`, -0.055955895287610205`}, {0.29756096125377757`, 
      0.03827997971440056}, {0.255100196771953, 0.12672194976397758`}, {
      0.16938025251811062`, 0.19291821559509803`}, {0.04552892707939671, 
      0.21296294287616296`}, {-0.09055005782856068, 
      0.15167318390139486`}, {-0.15761777091034845`, \
-0.0037986084595985625`}, {-0.08903389770307923, -0.1549203073735505}, {
      0.04414871151364566, -0.20942364592021775`}, {
      0.16378440408637723`, -0.18001841793269577`}, {
      0.23997612248666778`, -0.10058571382258474`}, {0.2628361004210078, 
      0.0016940481806274887`}, {0.2288350315580393, 0.10339225236031016`}, {
      0.13853132573105104`, 0.17847410034359779`}, {0.0036835317846137805`, 
      0.1912612052348869}, {-0.12534204667734167`, 
      0.10045661230325428`}, {-0.14378626750246068`, -0.066548518598507}, \
{-0.0351484568261789, -0.17910313508160028`}, {
      0.10119332634144396`, -0.18195309983417857`}, {
      0.19654097994474476`, -0.10546811769532209`}, {0.22330182017622455`, 
      0.008622274907604796}, {0.17221637190976818`, 0.11896544887512252`}, {
      0.05010279379594579, 0.17673132373856887`}, {-0.09524651956766311, 
      0.12517443776201165`}, {-0.14711326471109293`, -0.03160382571807255}, \
{-0.05189553996015038, -0.1569721526367185}}]}, 
   {RGBColor[1, 1, 0], DiskBox[{0, 0}, 0.03]}},
  AspectRatio->Automatic,
  Background->RGBColor[0.6, 0.6, 0.6],
  ImageSize->{315., Automatic}]], "Output",
 CellChangeTimes->{3.4011472513954477`*^9, 3.401147297414538*^9, 
  3.401193060985428*^9, 3.401194899400824*^9, 3.401276867267982*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tFinal", "=", "2"}], ";", 
  RowBox[{"R0", "=", "1"}], ";", 
  RowBox[{"V0", "=", 
   RowBox[{"0.5", " ", "2", " ", "\[Pi]"}]}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", ".003"}], ";", 
  RowBox[{"p", "=", "2"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CalculateOrbit", ";", 
  RowBox[{"DisplayOrbit", "[", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Clear", "[", 
  RowBox[{
  "tFinal", ",", "R0", ",", "V0", ",", "iMaximum", ",", "t", ",", "r", ",", 
   "v"}], "]"}]}], "Input",
 CellChangeTimes->{{3.401147309664631*^9, 3.401147327388998*^9}}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 1, 1], Thickness[0.0075], 
    LineBox[{{1, 0}, {0.9998223405448563, 0.009424219792007447}, {
      0.9992892832490045, 0.018845089194910757`}, {0.9984005911284263, 
      0.02825925096593939}, {0.9971558685864552, 0.03766333411803541}, {
      0.9955545604752138, 0.047053946954923105`}, {0.99359595077438, 
      0.05642766999376439}, {0.9912791608799504, 0.06578104873612561}, {
      0.9886031474934556, 0.07511058624639728}, {0.9855667000997718, 
      0.0844127354947781}, {0.9821684380192497, 0.09368389141942421}, {
      0.9784068070173149, 0.10292038265932932`}, {0.9742800754519568, 
      0.11211846290588762`}, {0.9697863299365946, 0.12127430181683466`}, {
      0.9649234704926379, 0.13038397543128172`}, {0.9596892051626358, 
      0.1394434560187645}, {0.9540810440511657, 0.1484486012885039}, {
      0.9480962927565277, 0.15739514287729295`}, {0.9417320451518193, 
      0.16627867402542174`}, {0.9349851754690167, 0.17509463633964162`}, {
      0.9278523296342247, 0.18383830553012756`}, {0.9203299157961937, 
      0.19250477599445634`}, {0.9124140939834733, 0.20108894410545514`}, {
      0.9041007648180737, 0.20958549004100963`}, {0.8953855572051496, 
      0.21798885797208853`}, {0.8862638149088737, 0.22629323439978785`}, {
      0.8767305819142173, 0.23449252440244997`}, {0.8667805864626409, 
      0.24258032551905337`}, {0.856408223636568, 0.25054989895410495`}, {
      0.8456075363527775, 0.2583941377409824}, {0.8343721946083185, 
      0.26610553144357907`}, {0.8226954728039957, 0.2736761269083585}, {
      0.8105702249496748, 0.28109748449826}, {0.7979888575323671, 
      0.28836062914347466`}, {0.7849432998020432, 0.2954559954284117}, {
      0.7714249712011656, 0.3023733657947511}, {0.757424745631863, 
      0.30910180077174226`}, {0.7429329122194047, 0.31562955993975633`}, {
      0.7279391321922541, 0.3219440120824698}, {0.7124323914578433, 
      0.32803153267532276`}, {0.6964009484100873, 0.3338773864780376}, {
      0.6798322764610255, 0.33946559252747865`}, {0.6627130007473747, 
      0.3447787682384327}, {0.6450288284274274, 0.34979794858031804`}, {
      0.626764471961446, 0.3545023753626673}, {0.6079035647703417, 
      0.3588692504716245}, {0.5884285687101286, 0.3628734453728974}, {
      0.5683206729103771, 0.36648715722369796`}, {0.5475596837460743, 
      0.3696794993662456}, {0.5261239061108616, 0.37241601059916}, {
      0.5039900168416491, 0.37465806314758215`}, {0.4811329322795003, 
      0.37636214326380163`}, {0.4575256738123252, 0.3774789702952359}, {
      0.4331392382756844, 0.37795240899983346`}, {0.40794248502151387`, 
      0.3777181146195183}, {0.3819020595260025, 0.3767018288919451}, {
      0.3549823866788803, 0.3748172150293775}, {0.3271457889771936, 
      0.37196307655850225`}, {0.2983528220864347, 0.36801974247126945`}, {
      0.26856298406975593`, 0.3628443097374719}, {0.2377360661144334, 
      0.3562642992213524}, {0.20583461167230152`, 0.3480690806060327}, {
      0.17282831491707384`, 0.33799812532825485`}, {0.13870187230009756`, 
      0.32572471685964693`}, {0.10346911683624481`, 0.310833164269272}, {
      0.06719886519825663, 0.29278691864404305`}, {0.03006314275103318, 
      0.270884835228183}, {-0.0075710239585300015`, 
      0.24420539199369934`}, {-0.04496509899499723, 
      0.2115519379767531}, {-0.08067764407469835, 
      0.1714616204642756}, {-0.11201760280884128`, 
      0.1224907699900958}, {-0.13453018454274426`, 
      0.06427184271211801}, {-0.14284266756812386`, -0.0004359771408719154}, \
{-0.13429303593737016`, -0.06509067790952885}, {-0.1116029876593285, \
-0.1231847848671023}, {-0.08015153608948702, -0.1720125984393205}, \
{-0.04437322268606965, -0.2119680108508831}, {-0.006940802883044821, \
-0.24450061202109485`}, {0.03071543005928335, -0.2710720370830522}, {
      0.06786310351577785, -0.2928763465015527}, {
      0.10413858840115084`, -0.31083273101902836`}, {
      0.13937178963753716`, -0.3256405416840562}, {
      0.17349503399412008`, -0.33783500616722345`}, {
      0.20649519676453193`, -0.3478308491813417}, {
      0.23838804010087083`, -0.3559540758731214}, {
      0.2692041795671768, -0.36246468457632675`}, {
      0.29898128932905416`, -0.36757290475745114`}, {
      0.32775973673563413`, -0.3714509081112022}, {
      0.3555801430551335, -0.37424135857272717`}, {
      0.3824820450806688, -0.3760637387751506}, {
      0.4085031937782551, -0.37701909499187275`}, {
      0.43367922389648644`, -0.3771936429395311}, {
      0.4580435392553676, -0.37666154232829996`}, {
      0.48162732186011753`, -0.375487056990514}, {
      0.5044596099898185, -0.37372625520245056`}, {
      0.5265674123524874, -0.37142836183074207`}, {
      0.5479758385851391, -0.3686368438854291}, {
      0.5687082343869687, -0.36539028980067007`}, {
      0.5887863144746059, -0.3617231275423696}, {
      0.6082302895581766, -0.35766621561841}, {
      0.6270589853838308, -0.35324733299560146`}, {
      0.6452899530141705, -0.3484915879550392}, {
      0.6629395701938701, -0.34342176145403264`}, {
      0.6800231340421172, -0.33805859719522396`}, {
      0.6965549455317388, -0.332421048039943}, {
      0.7125483863235301, -0.32652648643467125`}, {
      0.7280159885655203, -0.3203908849962292}, {
      0.7429694982674654, -0.31402897221336684`}, {
      0.7574199328377864, -0.3074543672893079}, {
      0.7713776333342318, -0.30067969741178163`}, {
      0.7848523119374978, -0.2937167001495785}, {
      0.797853095113075, -0.286576313204098}, {
      0.8103885628832083, -0.2792687533652292}, {
      0.8224667845895177, -0.2718035862137517}, {
      0.8340953514882952, -0.26418978786227054`}, {
      0.8452814064851002, -0.25643579982191556`}, {
      0.8560316712831118, -0.24854957791359175`}, {
      0.8663524711906533, -0.24053863600338432`}, {
      0.8762497578072305, -0.23241008522621392`}, {
      0.8857291297840796, -0.22417066926557752`}, {
      0.8947958518343756, -0.215826796176674}, {
      0.9034548721496597, -0.20738456717257847`}, {
      0.9117108383624761, -0.19884980273612549`}, {
      0.919568112180446, -0.19022806637195444`}, {
      0.927030782803847, -0.18152468627227264`}, {
      0.9341026792270355, -0.1727447751350889}, {
      0.9407873815135712, -0.16389324834397273`}, {
      0.9470882311255442, -0.15497484069298373`}, {
      0.9530083403792268, -0.14599412181862398`}, {
      0.9585506010916591, -0.13695551048193608`}, {
      0.9637176924760307, -0.12786328782774048`}, {
      0.968512088337644, -0.1187216097340957}, {
      0.972936063616764, -0.10953451835305289`}, {
      0.9769917003196935, -0.10030595293339628`}, {
      0.9806808928749103, -0.09103976000708788}, {
      0.9840053529469946, -0.08173970301338118}, {
      0.9869666137373233, -0.07240947142787743}, {
      0.9895660337970585, -0.06305268945803492}, {
      0.991804800374774, -0.05367292436169728}, {
      0.9936839323181106, -0.04427369444098611}, {
      0.9952042825460946, -0.03485847676032969}, {
      0.9963665401061645, -0.02543071463440698}, {
      0.9971712318274992, -0.015993824929323292`}, {
      0.9976187235799071, -0.006551205218355428}, {0.9977092211452895, 
      0.0028937591679221615`}, {0.9974427707065135, 0.012337688158420285`}, {
      0.9968192589564026, 0.021777199987211893`}, {0.9958384128274439, 
      0.031208904230387906`}, {0.9944997988407209, 0.040629394796287244`}, {
      0.9928028220704582, 0.05003524283081519}, {0.9907467247184256, 
      0.0594229894986839}, {0.9883305842902419, 0.06878913860011426}, {
      0.9855533113633371, 0.07813014898080739}, {0.9824136469339537, 
      0.08744242669079752}, {0.9789101593280586, 0.09672231684509727}, {
      0.9750412406583817, 0.10596609513579369`}, {0.9708051028069618, 
      0.1151699589413918}, {0.9661997729095324, 0.12433001797465917`}, {
      0.9612230883147921, 0.133442284404918}, {0.9558726909880285, 
      0.14250266238455697`}, {0.9501460213246717, 0.15150693690237488`}, {
      0.9440403113350839, 0.1604507618780769}, {0.9375525771572006, 
      0.16932964740265227`}, {0.9306796108484684, 0.17813894601826533`}, {
      0.923417971402797, 0.1868738379184474}, {0.9157639749319023, 
      0.19552931493449596`}, {0.9077136839433556, 0.2041001631567213}, {
      0.8992628956397981, 0.21258094401911434`}, {0.8904071291550045, 
      0.2209659736526411}, {0.881141611632674, 0.22924930028509305`}, {
      0.8714612630428481, 0.23742467943350715`}, {0.8613606796185551, 
      0.24548554659772245`}, {0.8508341157814867, 0.2534249871195774}, {
      0.8398754644100342, 0.2612357028202399}, {0.8284782352856511, 
      0.2689099749665695}, {0.8166355315340333, 0.27643962304421416`}, {
      0.8043400238557966, 0.28381595872784215`}, {0.7915839223169335, 
      0.291029734334381}, {0.7783589454421307, 0.29807108491947465`}, {
      0.7646562863238194, 0.304929463025663}, {0.7504665754264848, 
      0.3115935649068195}, {0.7357798397292631, 0.3180512468292439}, {
      0.720585457810406, 0.324289429775388}, {0.7048721104353054, 
      0.3302939905384751}, {0.6886277261665319, 0.33604963677840266`}, {
      0.6718394214716291, 0.34153976308931555`}, {0.654493434765473, 
      0.34674628447825756`}, {0.6365750537940893, 0.35164944283420385`}, {
      0.6180685357543855, 0.3562275809267194}, {0.5989570195626348, 
      0.3604568771452169}, {0.5792224297548186, 0.36431103248084445`}, {
      0.5588453716571388, 0.3677608990371072}, {0.5378050177572343, 
      0.37077403645823703`}, {0.5160789857181072, 0.37331417884359264`}, {
      0.4936432093382093, 0.3753405896310222}, {0.47047180518201137`, 
      0.37680727509767403`}, {0.44653693992449794`, 0.37766201784736025`}, {
      0.4218087072249702, 0.37784517891883285`}, {0.39625502908993515`, 
      0.37728819947555975`}, {0.36984160674983224`, 0.37591170822230896`}, {
      0.3425319627098484, 0.3736231054335944}, {0.3142876434618565, 
      0.3703134437745455}, {0.2850686995854784, 0.3658533523338078}, {
      0.25483464158262253`, 0.3600876418705428}, {0.22354621366157856`, 
      0.3528280686979682}, {0.1911685871026781, 0.34384349630280264`}, {
      0.15767705435198037`, 0.33284634348695563`}, {0.123067215044421, 
      0.3194737109893605}, {0.08737341863936161, 0.30326094431832373`}, {
      0.05070276550733907, 0.28360484297463384`}, {0.01329910852587346, 
      0.2597143160883371}, {-0.024334365576475545`, 
      0.23055197914181955`}, {-0.06120045218104606, 
      0.1947945515174938}, {-0.09538321687099072, 
      0.1509233311234825}, {-0.1234428217468928, 
      0.09777455919678643}, {-0.14025507855317004`, 
      0.036094518021208694`}, {-0.1410903154223677, -0.029513225005070448`}, \
{-0.1256550372207813, -0.09186287846938548}, {-0.0984012320481032, \
-0.14593663063439713`}, {-0.06456891039213933, -0.19066598842122537`}, \
{-0.027788516411930017`, -0.2271148567895896}, {
      0.009894619501610927, -0.2568078690094745}, {
      0.047412263607125485`, -0.28110015586068465`}, {
      0.08422450433762738, -0.3010599676578076}, {
      0.12006874583451826`, -0.31750262510786614`}, {
      0.15482837095120108`, -0.3310492554208565}, {
      0.18846428400191287`, -0.3421776006073002}, {
      0.22097864167245784`, -0.35126015282783823`}, {
      0.2523952356165339, -0.3585916016922967}, {
      0.2827486847070715, -0.3644084222146739}, {
      0.312078405276681, -0.36890292377071604`}, {
      0.3404252336945833, -0.3722334344797151}, {
      0.3678295507461498, -0.37453178012245497`}, {
      0.3943302694544963, -0.37590885124600326`}, {
      0.4199643242140989, -0.37645880304975393`}, {
      0.4447664518584776, -0.3762622648335966}, {
      0.46876914170480466`, -0.3753888225755346}, {
      0.49200268151667276`, -0.37389896127087774`}, {
      0.5144952556522305, -0.37184560084841406`}, {
      0.5362730691609016, -0.3692753228005263}, {
      0.557360482158073, -0.3662293588865103}, {
      0.5777801452469397, -0.36274439493391225`}, {
      0.5975531307042699, -0.35885322956912175`}, {
      0.6166990565709819, -0.35458531810672667`}, {
      0.6352362022721234, -0.34996722476365505`}, {
      0.6531816152897395, -0.3450230011142887}, {
      0.6705512089456424, -0.33977450476262344`}, {
      0.6873598516559994, -0.33424166922296894`}, {
      0.7036214481818636, -0.32844273372001465`}, {
      0.7193490134735653, -0.322394439861819}, {
      0.7345547397267519, -0.31611220077461577`}, {
      0.7492500572556732, -0.309610247220653}, {
      0.7634456897589779, -0.30290175437904365`}, {
      0.7771517045135575, -0.29599895230234846`}, {
      0.7903775579883161, -0.2889132225289743}, {
      0.803132137325494, -0.28165518290373454`}, {
      0.815423798094313, -0.2742347623134447}, {
      0.8272603986813265, -0.26666126676387397`}, {
      0.8386493316445153, -0.2589434379953607}, {
      0.8495975523240458, -0.25108950564652927`}, {
      0.8601116049717153, -0.24310723382070842`}, {
      0.8701976466332895, -0.2350039627814567}, {
      0.8798614689930004, -0.2267866463970211}, {
      0.889108518367186, -0.21846188586457774`}, {
      0.8979439140141544, -0.21003596017053824`}, {
      0.9063724649096305, -0.20151485368048597`}, {
      0.9143986851213386, -0.1929042811993674}, {
      0.9220268079022008, -0.18420971079773216`}, {
      0.9292607986090803, -0.1754363846617309}, {
      0.9361043665428008, -0.16658933819213362`}, {
      0.9425609757951784, -0.15767341754991213`}, {
      0.9486338551798542, -0.1486932958221904}, {
      0.9543260073157119, -0.13965348796199023`}, {
      0.9596402169244709, -0.130558364637672}, {
      0.9645790583975834, -0.12141216511287124`}, {
      0.9691449026817375, -0.11221900926470282`}, {
      0.9733399235269969, -0.10298290883675498`}, {
      0.9771661031368386, -0.09370777801367415}, {
      0.9806252372549994, -0.08439744339574444}, {
      0.9837189397200837, -0.07505565344461705}, {
      0.9864486465152443, -0.06568608746509974}, {
      0.9888156193369012, -0.05629236418255196}, {
      0.9908209487033636, -0.04687804997084427}, {
      0.9924655566213333, -0.037446666781946386`}, {
      0.9937501988255644, -0.028001699824933585`}, {
      0.9946754666043999, -0.018546605039489104`}, {
      0.9952417882214851, -0.009084816406780816}, {0.9954494299416311, 
      0.00038024686113307185`}, {0.9952984966665562, 0.009845173213493076}, {
      0.9947889321840467, 0.01930655169560084}, {0.9939205190319194, 
      0.028760964887579298`}, {0.9926928779760291, 0.03820498180859941}, {
      0.9911054670994159, 0.04763515074768586}, {0.989157580497507, 
      0.05704799198145122}, {0.9868483465720647, 0.06643999033792533}, {
      0.9841767259142724, 0.07580758756402445}, {0.9811415087649588, 
      0.08514717445211453}, {0.9777413120374479, 0.09445508267852949}, {
      0.97397457588587, 0.10372757630376117`}, {0.9698395597989357, 
      0.11296084288028865`}, {0.9653343381961443, 0.12215098410959027`}, {
      0.9604567955001291, 0.1312940059847012}, {0.9552046206552915, 
      0.1403858083486432}, {0.9495753010590227, 0.1494221737920445}, {
      0.943566115867577, 0.15839875580514523`}, {0.9371741286340323, 
      0.16731106608998184`}, {0.9303961792306543, 0.17615446092766285`}, {
      0.9232288750023356, 0.18492412648305123`}, {0.9156685810915177, 
      0.19361506291457228`}, {0.9077114098680493, 0.20222206713993196`}, {
      0.8993532093896864, 0.21073971408885067`}, {0.890589550810301, 
      0.2191623362510046}, {0.8814157146432104, 0.22748400130062732`}, {
      0.8718266757762361, 0.23569848754794728`}, {0.8618170871229986, 
      0.2437992569309492}, {0.8513812617813924, 0.2517794252177943}, {
      0.8405131535549625, 0.2596317290393179}, {0.8292063356758416, 
      0.26734848931075106`}, {0.8174539775487624, 0.27492157053022015`}, {
      0.8052488193142222, 0.2823423353562328}, {0.7925831440049012, 
      0.2896015937642232}, {0.7794487470426962, 0.29668954595951463`}, {
      0.7658369027940314, 0.3035957180759954}, {0.7517383278683065, 
      0.31030888951035807`}, {0.7371431408084157, 0.3168170105232739}, {
      0.7220408177833901, 0.32310710847155805`}, {0.706420143851862, 
      0.32916518070664413`}, {0.6902691593222219, 0.3349760717682283}, {
      0.6735751006928289, 0.34052333199654483`}, {0.6563243356164844, 
      0.34578905405456156`}, {0.6385022913025693, 0.3507536830556647}, {
      0.6200933757557636, 0.35539579498420726`}, {0.6010808912647675, 
      0.35969183680996425`}, {0.5814469396178495, 0.3636158200442595}, {
      0.561172318665659, 0.36713895734410823`}, {0.5402364101247827, 
      0.3702292289742524}, {0.5186170589956905, 0.37285086225341507`}, {
      0.49629044577932874`, 0.3749637022146236}, {0.47323095401500564`, 
      0.3765224451377615}, {0.44941103784705655`, 0.3774756977026281}, {
      0.424801097879699, 0.377764812301505}, {0.3993693793601006, 
      0.3773224321323535}, {0.373081916186089, 0.376070655978602}, {
      0.3459025598380374, 0.3739186989407481}, {0.3177931583810588, 
      0.3707598770904379}, {0.28871399479746823`, 0.36646767387556406`}, {
      0.2586246699171485, 0.3608905431525419}, {0.22748574879239217`, 
      0.35384495138676}, {0.1952617294159567, 0.34510593549074087`}, {
      0.1619263348014724, 0.33439411983465156`}, {0.12747196537116018`, 
      0.32135766068699856`}, {0.09192677164357775, 0.3055469672973877}, {
      0.055386034706232046`, 0.2863794632271204}, {0.018071051588700884`, 
      0.2630919746812984}, {-0.019558321292315634`, 
      0.23468292120804823`}, {-0.05658786902422211, 
      0.19986694601393412`}, {-0.09125347058542044, 
      0.15713639151736886`}, {-0.1203471264804271, 
      0.10522136394843287`}, {-0.13891793183666853`, 
      0.04449375317553366}, {-0.14195771836314586`, -0.02098738020239522}, \
{-0.12843665640167848`, -0.08412302138947542}, {-0.10237933845755143`, \
-0.1393947258498594}, {-0.06913012072462224, -0.18528553478771373`}, \
{-0.032561616980368, -0.22270044350866647`}, {
      0.005106696262837329, -0.25315822054175485`}, {
      0.042708584195253335`, -0.2780500766694834}, {
      0.07965253465519478, -0.2984822640256117}, {
      0.11564904909474819`, -0.315300676745933}, {
      0.15056741506272034`, -0.3291493011262105}, {
      0.1843611405837916, -0.3405226615395288}, {
      0.217028555142531, -0.34980559699228375`}, {
      0.24859155956020268`, -0.3573019737292531}, {
      0.27908394617808757`, -0.36325517811049485`}, {
      0.30854489229836474`, -0.36786279846835856`}, {
      0.33701531670682905`, -0.3712872512938119}, {
      0.3645358532157848, -0.3736635707213431}, {
      0.3911457520929231, -0.3751051963459232}, {
      0.41688231956451255`, -0.3757083319439968}, {
      0.4417806705646104, -0.37555527072209277`}, {
      0.4658736629664823, -0.374716963411888}, {
      0.4891919351248097, -0.37325502455107545`}, {
      0.5117639999814307, -0.37122331677663784`}, {
      0.5336163676981317, -0.36866921446666645`}, {
      0.5547736800614448, -0.3656346210587282}, {
      0.5752588467715658, -0.3621567951934234}, {
      0.595093177933581, -0.35826902705128544`}, {
      0.6142965096512234, -0.3540011962370428}, {
      0.6328873212042437, -0.3493802352087516}, {
      0.6508828432494175, -0.34443051678896874`}, {
      0.6682991570558647, -0.33917418020224605`}, {
      0.6851512851131399, -0.33363140698663873`}, {
      0.7014532736267038, -0.32782065576336405`}, {
      0.7172182674975403, -0.3217588630295731}, {
      0.732458578407372, -0.3154616157279084}, {
      0.7471857466213715, -0.3089433002434325}, {
      0.7614105970911786, -0.302217231610177}, {
      0.775143290401729, -0.29529576602140895`}, {
      0.7883933690616497, -0.2881903991888533}, {
      0.8011697995923607, -0.28091185265569435`}, {
      0.8134810108276369, -0.27347014981273154`}, {
      0.8253349287944167, -0.26587468307862167`}, {
      0.8367390085076967, -0.2581342734698428}, {
      0.8477002629776492, -0.25025722359312036`}, {
      0.8582252896956528, -0.24225136493417032`}, {
      0.8683202948375996, -0.2341241001851535}, {
      0.8779911153974477, -0.22588244124399584`}, {
      0.8872432394412741, -0.2175330434275863}, {
      0.8960818246518244, -0.2090822363645214}, {
      0.9045117153154822, -0.20053605196888383`}, {
      0.9125374578874952, -0.19190024984239623`}, {
      0.9201633152569458, -0.18318034040645734`}, {
      0.9273932798201786, -0.17438160602665137`}, {
      0.9342310854599886, -0.1655091203591787}, {
      0.9406802185176953, -0.15656776612035558`}, {
      0.9467439278361226, -0.14756225145610266`}, {
      0.9524252339433457, -0.13849712506755452`}, {
      0.9577269374397556, -0.1293767902310498}, {
      0.9626516266444068, -0.1202055178353697}, {
      0.9672016845506862, -0.11098745854581839`}, {
      0.9713792951359803, -0.10172665419328066`}, {
      0.975186449065162, -0.09242704847649591}, {
      0.9786249488232955, -0.08309249705724325}, {
      0.9816964133089275, -0.07372677712076015}, {
      0.9844022819156321, -0.0643335964673693}, {
      0.9867438181260707, -0.05491660219583939}, {
      0.9887221126396765, -0.045479389034349615`}, {
      0.9903380860521314, -0.03602550737097939}, {
      0.9915924911020569, -0.026558471032328543`}, {
      0.9924859144977394, -0.01708176485613155}, {
      0.9930187783342415, -0.007598852101511811}, {0.9931913411088872, 
      0.0018868182612098664`}, {0.9930036983408176, 0.011371804340514336`}, {
      0.9924557827980829, 0.02085266415369498}, {0.9915473643335383, 
      0.030325948457507614`}, {0.9902780493286255, 0.039788193540097785`}, {
      0.9886472797419259, 0.04923591393444283}, {0.9866543317571476, 
      0.05866559501272667}, {0.9842983140229303, 0.06807368541981333}, {
      0.9815781654745013, 0.077456589302279}, {0.9784926527247627, 
      0.08681065828727297}, {0.9750403670098213, 0.0961321831627663}, {
      0.9712197206712417, 0.1054173852074658}, {0.967028943154406, 
      0.1146624071147601}, {0.9624660764992432, 0.12386330345045307`}, {
      0.9575289702962345, 0.1330160305786405}, {0.9522152760769523, 
      0.14211643598379545`}, {0.9465224411044187, 0.15116024690982238`}, {
      0.9404477015242205, 0.16014305822837255`}, {0.9339880748325402, 
      0.16906031943890754`}, {0.9271403516119955, 0.17790732069164558`}, {
      0.919901086480357, 0.18667917771137577`}, {0.9122665881907549, 
      0.19537081548488042`}, {0.9042329088148136, 0.2039769505570084}, {
      0.8957958319321554, 0.21249207175986173`}, {0.8869508597408015, 
      0.22091041917557816`}, {0.8776931989930281, 0.22922596110518823`}, {
      0.8680177456500849, 0.23743236878323956`}, {0.8579190681366858, 
      0.24552298853939403`}, {0.8473913890621764, 0.25349081106288984`}, {
      0.8364285652595745, 0.2613284373722454}, {0.8250240659760638, 
      0.26902804102917033`}, {0.8131709490287752, 0.2765813260602483}, {
      0.8008618347175929, 0.2839794799599714}, {0.7880888772620278, 
      0.2912131210408949}, {0.7748437335017019, 0.29827223926699414`}, {
      0.7611175285694985, 0.30514612954962594`}, {0.7469008182128517, 
      0.31182331629536697`}, {0.7321835474019969, 0.318291467763182}, {
      0.7169550048245505, 0.3245372985043325}, {0.7012037728241425, 
      0.3305464578085888}, {0.684917672298208, 0.33630340164705896`}, {
      0.6680837020285819, 0.34179124506232367`}, {0.650687971881777, 
      0.34699159128038465`}, {0.6327156292895467, 0.351884332966229}, {
      0.6141507784137309, 0.3564474199622837}, {0.5949763914269978, 
      0.3606565864649941}, {0.5751742114257803, 0.36448502881199873`}, {
      0.5547246466693317, 0.36790302273789216`}, {0.5336066561658482, 
      0.3708774659267235}, {0.5117976271922104, 0.3733713276881239}, {
      0.48927324627992086`, 0.37534298225070595`}, {0.4660073667537017, 
      0.376745394987968}, {0.4419718784394051, 0.3775251211301808}, {
      0.4171365892682855, 0.37762106309696497`}, {0.39146913519868975`, 
      0.37696291393053294`}, {0.3649349458465968, 0.37546918807149515`}, {
      0.3374973113679278, 0.3730447033725764}, {0.3091176265473223, 
      0.36957732445138464`}, {0.2797559397866715, 0.36493369911118495`}, {
      0.24937202430388966`, 0.358953604235642}, {0.21792734730363428`, 
      0.3514423466966568}, {0.1853885995572042, 0.34216041151907356`}, {
      0.15173397969065994`, 0.3308091788377048}, {0.11696444072963781`, 
      0.31701101161986356`}, {0.08112408798100257, 0.30028137552452805`}, {
      0.04433787863407592, 0.279990189741886}, {0.006882761715289572, 
      0.25531064289037675`}, {-0.030675949427217415`, 
      0.22516111808446673`}, {-0.06722470468412363, 
      0.18817603172924058`}, {-0.10064139109082879`, 
      0.14284111025973714`}, {-0.12719239708755362`, 
      0.08817105336634917}, {-0.14159575414274556`, 
      0.02543919138731912}, {-0.1395884465152492, -0.04010635797883266}, \
{-0.12179575627015268`, -0.10130973357176294`}, {-0.09308436534130694, \
-0.15381155931892787`}, {-0.05851220631783874, -0.19704235408822612`}, \
{-0.021420270081078364`, -0.232223900825071}, {
      0.016342851757437257`, -0.26087828499993837`}, {
      0.05381835762252818, -0.2843176733160655}, {
      0.09052429013275486, -0.30356740080534467`}, {
      0.1262287602172267, -0.31940874511410905`}, {
      0.160831660785705, -0.3324374972144939}, {
      0.1943029030775214, -0.34311282039003277`}, {
      0.2266495932106589, -0.35179353543224534`}, {
      0.2578982265306092, -0.35876416242772446`}, {
      0.2880848577157595, -0.36425350885681085`}, {
      0.3172496126330202, -0.3684480205772308}, {
      0.3454336180193458, -0.37150147917529763`}, {
      0.37267730350669487`, -0.3735421384295887}, {
      0.3990194938513212, -0.3746780478943305}, {
      0.42449696012960025`, -0.37500107740322625`}, {
      0.449144237917722, -0.37458999854881414`}, {
      0.47299359951365355`, -0.3735128726780824}, {
      0.4960751130249085, -0.3718289224546093}, {
      0.5184167481049886, -0.369590014183701}, {
      0.5400445042347867, -0.36684184341066317`}, {
      0.5609825471889858, -0.36362489187806674`}, {
      0.5812533452701358, -0.35997520652340786`}, {
      0.600877800535838, -0.3559250386514149}, {
      0.6198753724793817, -0.35150337226804834`}, {
      0.6382641929892068, -0.34673636382295075`}, {
      0.6560611722363936, -0.341647710589566}, {
      0.6732820956251407, -0.3362589611407593}, {
      0.6899417122155619, -0.3305897785169482}, {
      0.7060538151708088, -0.3246581644947376}, {
      0.7216313148418566, -0.3184806516752634}, {
      0.7366863051152418, -0.31207246879839645`}, {
      0.7512301236319064, -0.3054476836604954}, {
      0.7652734064519324, -0.29861932720219037`}, {
      0.778826137698431, -0.2915995016885835}, {
      0.7918976946692039, -0.28439947538961347`}, {
      0.8044968888600714, -0.2770297657546656}, {
      0.8166320033007448, -0.2695002127411163}, {
      0.8283108265637865, -0.2618200436847201}, {
      0.8395406837700194, -0.25399793087772193`}, {
      0.8503284648798527, -0.24604204283830788`}, {
      0.8606806505293502, -0.23796009010467623`}, {
      0.8706033356423167, -0.22975936626246862`}, {
      0.8801022510249961, -0.22144678481069152`}, {
      0.8891827831279295, -0.21302891238470978`}, {
      0.8978499921398585, -0.20451199878232534`}, {
      0.9061086285610273, -0.19590200417788228`}, {
      0.9139631483876327, -0.18720462385776732`}, {
      0.9214177270252534, -0.1784253107669826}, {
      0.9284762720366899, -0.1695692961193372}, {
      0.9351424348185825, -0.16064160829216062`}, {
      0.9414196212912821, -0.15164709019940134`}, {
      0.9473110016776115, -0.142590415313813}, {
      0.9528195194382225, -0.1334761024890478}, {
      0.9579478994241396, -0.12430852971537715`}, {
      0.962698655300675, -0.11509194692802925`}, {
      0.9670740962911208, -0.10583048797442951`}, {
      0.971076333283396, -0.09652818183566553}, {
      0.9747072843380783, -0.0871889631880334}, {
      0.9779686796319299, -0.07781668238235523}, {
      0.9808620658670735, -0.06841511491172081}, {
      0.9833888101723398, -0.058987970432257605`}, {
      0.985550103519962, -0.0495389013963532}, {
      0.9873469636776752, -0.04007151135334572}, {
      0.9887802377133803, -0.030589362968975818`}, {
      0.9898506040667955, -0.021095985811791708`}, {
      0.9905585741999376, -0.011594883952159647`}, {
      0.9909044938358067, -0.002089543417511327}, {0.9908885437922723, 
      0.0074165604540787945`}, {0.9905107404158571, 0.016919955720975724`}, {
      0.9897709356178529, 0.02641716656536066}, {0.98866881651297, 
      0.035904706000756466`}, {0.9872039046584872, 0.04537906851935332}, {
      0.9853755548896183, 0.054836722638157386`}, {0.9831829537445151, 
      0.06427410330192036}, {0.9806251174699693, 0.0736876040992883}, {
      0.9777008895964252, 0.08307356924660747}, {0.9744089380683586, 
      0.09242828529130852}, {0.9707477519133717, 0.1017479724837135}, {
      0.9667156374304893, 0.1110287757624169}, {0.9623107138750695, 
      0.12026675529401558`}, {0.9575309086144408, 0.12945787650282078`}, {
      0.9523739517248042, 0.1385979995201797}, {0.9468373699960523, 
      0.1476828679760478}, {0.9409184803069099, 0.15670809704734637`}, {
      0.9346143823281462, 0.16566916066824433`}, {0.927921950506479, 
      0.17456137779662165`}, {0.9208378252761336, 0.18337989761836576`}, {
      0.9133584034387455, 0.19211968355653863`}, {0.9054798276453382, 
      0.20077549593548993`}, {0.8971979749063583, 0.20934187313027566`}, {
      0.8885084440471162, 0.2178131110087779}, {0.8794065420163308, 
      0.22618324044712088`}, {0.8698872689446948, 0.23444600266762258`}, {
      0.8599453018382898, 0.24259482211173958`}, {0.8495749767781507, 
      0.25062277651719317`}, {0.8387702694821002, 0.2585225638174089}, {
      0.8275247740679559, 0.26628646542096057`}, {0.8158316798381456, 
      0.2739063053569146}, {0.8036837458844112, 0.28137340468637906`}, {
      0.791073273287417, 0.28867853047813036`}, {0.7779920746594875, 
      0.2958118385231211}, {0.7644314407491754, 0.30276280881417234`}, {
      0.7503821037937959, 0.30952017263718723`}, {0.7358341972704381, 
      0.31607182990109733`}, {0.7207772116574624, 0.32240475506565286`}, {
      0.7051999457776438, 0.32850488969646063`}, {0.6890904532519334, 
      0.3343570192690182}, {0.6724359835511521, 0.33994463133660513`}, {
      0.6552229170948838, 0.34524975154285703`}, {0.6374366938175038, 
      0.3502527531617893}, {0.6190617346087555, 0.35493213483688263`}, {
      0.6000813550534292, 0.3592642599007421}, {0.5804776709616846, 
      0.36322304899873425`}, {0.5602314953300721, 0.36677961559235195`}, {
      0.5393222266531325, 0.36990183111348995`}, {0.5177277289950948, 
      0.3725538028466491}, {0.4954242050552179, 0.37469524270561744`}, {
      0.4723860648178967, 0.3762806984808729}, {0.44858579459135794`, 
      0.3772586101991566}, {0.42399383483170877`, 0.377570141993785}, {
      0.398578480991039, 0.3771477229208291}, {0.37230583117794086`, 
      0.3759132063790556}, {0.34513982016628497`, 0.37377552406617637`}, {
      0.3170424055625823, 0.37062766199452063`}, {0.28797401642338827`, 
      0.3663427157935093}, {0.2578944512271495, 0.3607686793757713}, {
      0.22676454670493412`, 0.3537214684742005}, {0.19454918085160872`, 
      0.34497545427085946`}, {0.16122261868478355`, 0.3342504494053304}, {
      0.12677805090244948`, 0.3211936141953299}, {0.09124480904059032, 
      0.30535413526682254`}, {0.05471998732501335, 0.2861479543068905}, {
      0.017427744519505642`, 0.2628101814877276}, {-0.020167436796603037`, 
      0.23433751548247414`}, {-0.05714380249239909, 
      0.1994437303797923}, {-0.09172543430602437, 
      0.15662372428819074`}, {-0.12068907386783162`, 
      0.10461974939691523`}, {-0.13907589127440012`, 
      0.04383335995183277}, {-0.14190069788331536`, -0.021639299039972082`}, \
{-0.1281832295415566, -0.08469424941940154}, {-0.1019763358391425, \
-0.13984482262402742`}, {-0.06862001656417993, -0.18560280594764392`}, \
{-0.031973966703325626`, -0.2228851171527298}, {
      0.005751940614281706, -0.25321346858274857`}, {
      0.04339721464169677, -0.2779791884187218}, {
      0.08037371903879101, -0.2982880463185022}, {
      0.11639403209546698`, -0.3149854524953652}, {
      0.15132880092915424`, -0.3287150214519532}, {
      0.18513247879167186`, -0.3399710114346091}, {
      0.21780408722629854`, -0.34913807409651193`}, {
      0.24936605551661, -0.3565199432700567}, {
      0.2798525932029835, -0.3623599105577405}, {
      0.30930321566190233`, -0.3668554953990447}, {
      0.33775912130757507`, -0.3701690633623164}, {
      0.3652611790129556, -0.37243561034893546`}, {
      0.39184883920585345`, -0.37376854685312855`}, {
      0.4175595803279864, -0.3742640542608654}, {
      0.44242866673111, -0.3740044084284462}, {
      0.4664890868094476, -0.37306054661076904`}, {
      0.48977159356015476`, -0.37149407291606923`}, {
      0.5123048010660065, -0.36935884200658525`}, {
      0.5341153090288238, -0.36670222230992444`}}]}, 
   {RGBColor[1, 1, 0], DiskBox[{0, 0}, 0.03]}},
  AspectRatio->Automatic,
  Background->RGBColor[0.6, 0.6, 0.6],
  ImageSize->{311., Automatic}]], "Output",
 CellChangeTimes->{3.401147328855382*^9, 3.401193069006318*^9, 
  3.401194902020866*^9, 3.401276872148888*^9}]
}, Open  ]],

Cell["\<\
Repeat this with \[CapitalDelta]t = .001 .  We will use this for the rest to \
be safe.\
\>", "Text",
 CellChangeTimes->{{3.401193534964027*^9, 3.4011935880634413`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tFinal", "=", "2"}], ";", 
  RowBox[{"R0", "=", "1"}], ";", 
  RowBox[{"V0", "=", 
   RowBox[{"0.5", " ", "2", " ", "\[Pi]"}]}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", ".001"}], ";", 
  RowBox[{"p", "=", "2"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CalculateOrbit", ";", 
  RowBox[{"DisplayOrbit", "[", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Clear", "[", 
  RowBox[{
  "tFinal", ",", "R0", ",", "V0", ",", "iMaximum", ",", "t", ",", "r", ",", 
   "v"}], "]"}]}], "Input",
 CellChangeTimes->{{3.401147309664631*^9, 3.401147327388998*^9}, 
   3.4011935273885307`*^9, 3.401193595402927*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 1, 1], Thickness[0.0075], 
    LineBox[{{1, 0}, {0.9999802607100224, 0.0031415719825091536`}, {
      0.9999210418659591, 0.006283019932872732}, {0.9998223405451535, 
      0.009424219790785737}, {0.9996841518756591, 0.012565047439607487`}, {
      0.9995064690349592, 0.015705378678151673`}, {0.9992892832481749, 
      0.01884508919242581}, {0.9990325837857579, 0.021984054527303162`}, {
      0.9987363579606688, 0.025122150058110096`}, {0.9984005911250381, 
      0.02825925096211173}, {0.9980252666663071, 0.03139523218987866}, {
      0.9976103660028485, 0.034529968436517344`}, {0.9971558685790617, 
      0.03766333411274671}, {0.9966617518599414, 0.040795203315803066`}, {
      0.9961279913251152, 0.04392544980015571}, {0.9955545604623471, 
      0.047053946948014805`}, {0.994941430760503, 0.050180567739613366`}, {
      0.9942885717019724, 0.053305184723244615`}, {0.9935959507545441, 
      0.056427669985035865`}, {0.9928635333627288, 0.05954789511843963}, {
      0.9920912829385237, 0.06266573119342249}, {0.9912791608516137, 
      0.06578104872533173}, {0.9904271264190033, 0.06889371764341952}, {
      0.9895351368940715, 0.0720036072590039}, {0.9886031474550436, 
      0.07511058623324543}, {0.9876311111928736, 0.078214522544518}, {
      0.9866189790985271, 0.0813152834553516}, {0.985566700049659, 
      0.08441273547892458}, {0.9844742207966758, 0.08750674434508209}, {
      0.983341485948175, 0.09059717496585716}, {0.9821684379557509, 
      0.09368389140047002}, {0.9809550170981576, 0.0967667568197806}, {
      0.9797011614648192, 0.09984563347016863}, {0.9784068069386764, 
      0.102920382636815}, {0.9770718871783571, 0.10599086460635708`}, {
      0.9756963335996613, 0.10905693862889036`}, {0.9742800753563468, 
      0.11211846287928733`}, {0.9728230393202024, 0.11517529441780414`}, {
      0.9713251500603975, 0.11822728914994453`}, {0.9697863298220918, 
      0.1212743017855493}, {0.9682064985042919, 0.12431618579707895`}, {
      0.9665855736369403, 0.12735279337705582`}, {0.9649234703572206, 
      0.1303839753946309}, {0.963220101385062, 0.13340958135123948`}, {
      0.9614753769978277, 0.13642945933530853`}, {0.9596892050041689, 
      0.13944345597597718`}, {0.9578614907170261, 0.14245141639579056`}, {
      0.9559921369257592, 0.14545318416232586`}, {0.9540810438673868, 
      0.1484486012387076}, {0.9521281091969135, 0.15143750793296812`}, {
      0.9501332279567256, 0.15441974284620688`}, {0.9480962925450317, 
      0.15739514281950123`}, {0.9460171926833264, 0.16036354287951884`}, {
      0.943895815382853, 0.16332477618278052`}, {0.9417320449100411, 
      0.16627867395851975`}, {0.939525762750893, 0.16922506545008356`}, {
      0.9372768475742929, 0.1721637778548167}, {0.9349851751942124, 
      0.1750946362623693}, {0.9326506185307819, 0.17801746359136505`}, {
      0.9302730475702, 0.1809320805243652}, {0.92785232932345, 
      0.18383830544106008`}, {0.9253883277837912, 0.18673595434961776`}, {
      0.9228809038829937, 0.18962484081611614`}, {0.9203299154462802, 
      0.19250477589198145`}, {0.9177352171459416, 0.19537556803935346`}, {
      0.9150966604535886, 0.19823702305429322`}, {0.9124140935910015, 
      0.20108894398774668`}, {0.9096873614795385, 0.20393113106417268`}, {
      0.9069163056880628, 0.20676338159774038`}, {0.9041007643793431, 
      0.20958548990599662`}, {0.9012405722548873, 0.2123972472208993}, {
      0.8983355604981611, 0.21519844159710813`}, {0.895385556716144, 
      0.2179888578174191}, {0.8923903848791751, 0.22076827729522386`}, {
      0.8893498652590367, 0.2235364779738692}, {0.8862638143652226, 
      0.22629323422278674`}, {0.8831320448793363, 0.22903831673025607`}, {
      0.8799543655875622, 0.23177149239265799`}, {0.8767305813111516, 
      0.2344925242000685}, {0.8734604928348587, 0.2372011711180358}, {
      0.8701438968332681, 0.23989718796537543`}, {0.8667805857949424, 
      0.24258032528781057`}, {0.8633703479443252, 0.24525032922727563`}, {
      0.8599129671613255, 0.24790694138669234`}, {0.8564082228985116, 
      0.25054989869001765`}, {0.8528558900958361, 0.253178933237353}, {
      0.8492557390928132, 0.25579377215489335`}, {0.8456075355380649, 
      0.25839413743948225`}, {0.8419110402961513, 0.26097974579752903`}, {
      0.838166009351594, 0.2635503084780286}, {0.8343721937100019, 
      0.2661055310994134}, {0.8305293392962015, 0.26864511346994924`}, {
      0.8266371868492733, 0.271168749401376}, {0.82269547181439, 
      0.27367612651547185`}, {0.8187039242313483, 0.27616692604320825`}, {
      0.8146622686196844, 0.278640822616139}, {0.8105702238602549, 
      0.28109748404965157`}, {0.8064275030731648, 0.28353657111768443`}, {
      0.8022338134919165, 0.28595773731849344`}, {0.7979888563336501, 
      0.28836062863102646`}, {0.7936923266653421, 0.29074488326143955`}, {
      0.7893439132658198, 0.29311013137926084`}, {0.7849432984834503, 
      0.29545599484268}, {0.7804901580893514, 0.29778208691240965`}, {
      0.7759841611259689, 0.30008801195353246`}, {0.7714249697508606, 
      0.3023733651247124}, {0.7668122390755177, 0.30463773205411065`}, {
      0.7621456169990511, 0.30688068850130684`}, {0.7574247440365645, 
      0.30910180000448273`}, {0.752649253142025, 0.31130062151207927`}, {
      0.7478187695254439, 0.31347669699808867`}, {0.7429329104641641, 
      0.3156295590600896}, {0.7379912851080527, 0.3177587284990769}, {
      0.7329934942783838, 0.3198637138800756}, {0.7279391302601947, 
      0.32194401107246423`}, {0.7228277765878889, 0.3239991027688595}, {
      0.7176590078238533, 0.3260284579813416}, {0.7124323893298524, 
      0.3280315315137137}, {0.7071474770309516, 0.3300077634084047}, {
      0.7018038171717196, 0.3319565783665259}, {0.6964009460644514, 
      0.3338773851394923}, {0.690938389829148, 0.33576957589050604`}, {
      0.6854156641249856, 0.337632525524083}, {0.6798322738729978, 
      0.3394655909816703}, {0.674187712969698, 0.34126811050126604`}, {
      0.6684814639913578, 0.3430394028387998}, {0.6627129978886612, 
      0.3447787664488674}, {0.6568817736714532, 0.34648547862223744`}, {
      0.6509872380832985, 0.34815879457735227`}, {0.6450288252655753, 
      0.34979794650283796`}, {0.6390059564108284, 0.3514021425478087}, {
      0.6329180394051189, 0.3529705657565038}, {0.6267644684591158, 
      0.35450237294352566`}, {0.6205446237276907, 0.3559966935056524}, {
      0.6142578709177974, 0.35745262816587925`}, {0.6079035608844416, 
      0.3588692476449915}, {0.6014810292145775, 0.3602455912555895}, {
      0.5949895957988046, 0.3615806654130692}, {0.5884285643907881, 
      0.36287344205760136`}, {0.5817972221543769, 0.3641228569806534}, {
      0.5750948391984674, 0.36532780804904547`}, {0.568320668099735, 
      0.3664871533189295}, {0.5614739434134602, 0.36759970903141476`}, {
      0.5545538811727831, 0.3686642474808344}, {0.5475596783768625, 
      0.36967949474584116`}, {0.5404905124685739, 0.37064412827263554`}, {
      0.5333455408025731, 0.37155677429864814`}, {0.5261239001047745, 
      0.372416005103919}, {0.518824705924561, 0.37322033607621796`}, {
      0.5114470520813512, 0.3739682225746266}, {0.5039900101075178, 
      0.3746580565748315}, {0.4964526286900808, 0.3752881630777497}, {
      0.4888339331141075, 0.37585679626129287`}, {0.48113292471134433`, 
      0.3763621353530602}, {0.4733485803183081, 0.37680228019950396`}, {
      0.4654798517488869, 0.3771752465046035}, {0.4575256652874699, 
      0.37747896070828346`}, {0.44948492120976796`, 0.3777112544716816}, {
      0.44135649333982985`, 0.3778698587328629}, {0.4331392286533469, 
      0.37795239729264546`}, {0.4248319469392101, 0.3779563798857872}, {
      0.4164334405334969, 0.3778791946878172}, {0.40794247414268314`, 
      0.377718100202208}, {0.39935778477597755`, 0.3774702164662803}, {
      0.3906780818103563, 0.3771325155071233}, {0.3819020472162482, 
      0.3767018109707673}, {0.37302833597702495`, 0.3761747468387512}, {
      0.3640555767416558, 0.375547785135908}, {0.35498237275729533`, 
      0.3748171925214946}, {0.3458073031374397, 0.3739790256424936}, {
      0.3365289245319172, 0.37302911511279613`}, {0.3271457732777524, 
      0.3719630479647592}, {0.31765636812532094`, 0.3707761484000115}, {
      0.30805921365277505`, 0.36946345664400504`}, {0.29835280450417195`, 
      0.3680197056832703}, {0.288535630613967, 0.36643929563517175`}, {
      0.27860618361363876`, 0.3647162654666576}, {0.26856296465656787`, 
      0.3628442617404711}, {0.25840449394661785`, 0.3608165040238961}, {
      0.2481293223163254, 0.35862574654564544`}, {0.23773604527492642`, 
      0.35626423563022375`}, {0.22722332003806525`, 0.3537236623752472}, {
      0.2165898861643194, 0.3509951099650559}, {0.2058345905641691, 
      0.3480689949329005}, {0.194956417821804, 0.34493500159366414`}, {
      0.18395452698820314`, 0.34158200876958755`}, {0.17282829627676863`, 
      0.3379980078236892}, {0.16157737743518577`, 0.33417001090168297`}, {
      0.15020176199801763`, 0.3300839481674408}, {0.13870186216801283`, 
      0.3257245527069144}, {0.12707860976113794`, 0.32107523168344826`}, {
      0.11533357752034386`, 0.3161179222738781}, {0.10346912820593054`, 
      0.31083293093186365`}, {0.09148859826864454, 0.30519875466275476`}, {
      0.07939652468255265, 0.29919188333048785`}, {0.06719892575124904, 
      0.29278658267036}, {0.05490364951044523, 0.28595465883276455`}, {
      0.042520806847560544`, 0.2786652072051746}, {0.030063310749925215`, 
      0.27088435136232025`}, {0.0175475482350283, 0.26257498288584685`}, {
      0.004994217441815156, 0.25369652036637125`}, {-0.0075706312652977885`, 
      0.24420471743490407`}, {-0.020114305378234346`, 
      0.23405156695726462`}, {-0.032595883047928674`, 
      0.22318537394715532`}, {-0.044964282388017064`, 
      0.21155110622513623`}, {-0.05715595578660602, 
      0.19909118235500722`}, {-0.0690922161288215, 
      0.18574692259737416`}, {-0.08067628154423397, 
      0.1714609676609323}, {-0.09179027122845257, 
      0.15618104801252214`}, {-0.10229262565664003`, 
      0.1398655274282709}, {-0.11201677761727077`, 
      0.12249107537920675`}, {-0.12077233598380867`, 
      0.10406252387750511`}, {-0.12835042060140484`, 
      0.0846242893545386}, {-0.13453478340752764`, 
      0.06427161369818919}, {-0.13911949138000523`, 
      0.043158502947027416`}, {-0.14193187734268722`, 
      0.021498321800961128`}, {-0.1428565337317014, -0.0004463479544767994}, \
{-0.14185381838966815`, -0.022385024167755497`}, {-0.13896667319496217`, \
-0.044027828766027274`}, {-0.13431324867767722`, -0.06511383024655144}, \
{-0.12806804111866685`, -0.08543195411925854}, {-0.12043782337658793`, \
-0.10483065310571002`}, {-0.1116388459549219, -0.12321693701487677`}, \
{-0.1018794129073021, -0.14054822410962003`}, {-0.0913490199576953, \
-0.15682104112406203`}, {-0.08021322185983283, -0.17205963985692083`}, \
{-0.0686125805162426, -0.18630622289113305`}, {-0.056664066182436926`, \
-0.19961336166086063`}, {-0.04446366781357318, -0.21203853276137516`}, \
{-0.03208940250441879, -0.22364041159629533`}, {-0.019604262618639793`, \
-0.23447649984214716`}, {-0.007058875630218151, -0.24460170675267054`}, {
      0.005506205607517894, -0.25406758285349956`}, {
      0.018058586850738297`, -0.26292198326463756`}, {
      0.030572458255215967`, -0.27120900346929316`}, {
      0.04302731547039734, -0.2789690802345226}, {
      0.055406927637692546`, -0.28623918635250917`}, {
      0.06769850770485555, -0.2930530729098124}, {
      0.07989204646287815, -0.29944152980318756`}, {
      0.09197977807858775, -0.30543264656300095`}, {
      0.1039557507940762, -0.311052062986335}, {
      0.11581548157157796`, -0.3163232038855998}, {
      0.12755567771966458`, -0.3212674953030258}, {
      0.13917401200412483`, -0.32590456142548097`}, {
      0.1506689405309634, -0.3302524025605367}, {
      0.16203955490460367`, -0.3343275551721744}, {
      0.17328546191849273`, -0.3381452353006988}, {
      0.1844066854202136, -0.3417194668230826}, {
      0.19540358608550606`, -0.3450631960233248}, {
      0.20627679569729046`, -0.3481883938867722}, {
      0.21702716320630216`, -0.3511061474393007}, {
      0.2276557103883003, -0.3538267413416854}, {
      0.23816359533963466`, -0.3563597308336852}, {
      0.24855208239219384`, -0.35871400700865863`}, {
      0.25882251729911704`, -0.36089785529205237`}, {
      0.26897630675873774`, -0.3629190078979713}, {
      0.27901490151744096`, -0.36478469094809257`}, {
      0.2889397824313834, -0.3665016668565019}, {
      0.2987524489793338, -0.3680762725122317}, {
      0.308454409809733, -0.36951445372775177`}, {
      0.31804717497876833`, -0.3708217963656757}, {
      0.3275322495962154, -0.3720035545067428}, {
      0.336911128644725, -0.37306467597897003`}, {
      0.34618529277826265`, -0.3740098255300461}, {
      0.35535620493825004`, -0.3748434058919132}, {
      0.36442530765297376`, -0.37556957695747967`}, {
      0.37339402090810636`, -0.37619227326400184`}, {
      0.3822637404946037, -0.3767152199554161}, {
      0.39103583675550657`, -0.3771419473763876}, {
      0.3997116536658506, -0.37747580443371526`}, {
      0.4082925081904421, -0.377719970845693}, {
      0.4167796898730584, -0.37787746838680053`}, {
      0.42517446061798775`, -0.377951171223454}, {
      0.43347805463098404`, -0.3779438154262866}, {
      0.44169167849187896`, -0.37785800773537415`}, {
      0.4498165113354364, -0.37769623364682087`}, {
      0.4578537051206884, -0.3774608648820406}, {
      0.4658043849720728, -0.37715416629480025`}, {
      0.4736696495782929, -0.37677830226552755`}, {
      0.48145057163701693`, -0.37633534262744595`}, {
      0.4891481983353962, -0.37582726816470274`}, {
      0.49676355185795384`, -0.375255975718744}, {
      0.5042976299147349, -0.3746232829356969}, {
      0.5117514062837396, -0.3739309326844054}, {
      0.5191258313626265, -0.37318059717197405`}, {
      0.5264218327254894, -0.37237388178118297`}, {
      0.5336403156812078, -0.37151232865189665`}, {
      0.540782163830463, -0.3705974200265824}, {
      0.5478482396190132, -0.3696305813782499}, {
      0.5548393848852529, -0.36861318433749735`}, {
      0.5617564214004406, -0.36754654943389}, {
      0.568600151400294, -0.366431948665573}, {
      0.5753713581069103, -0.36527060790983323`}, {
      0.5820708062401951, -0.36406370918624414`}, {
      0.5886992425181714, -0.3628123927830541}, {
      0.5952573961457008, -0.36151775925659535`}, {
      0.6017459792912853, -0.36018087131268867`}, {
      0.6081656875517303, -0.3588027555782914}, {
      0.6145172004045475, -0.35738440427097534`}, {
      0.6208011816480553, -0.35592677677321855`}, {
      0.6270182798292031, -0.35443080111794717`}, {
      0.6331691286591992, -0.3528973753912634}, {
      0.6392543474170711, -0.3513273690578397}, {
      0.6452745413413223, -0.3497216242140415}, {
      0.6512303020098815, -0.3480809567734621}, {
      0.6571222077085629, -0.34640615758920046`}, {
      0.6629508237882794, -0.344697993516897}, {
      0.6687167030112603, -0.3429572084222478}, {
      0.6744203858865403, -0.3411845241364476}, {
      0.6800624009949927, -0.33938064136276713`}, {
      0.6856432653041852, -0.33754624053724147`}, {
      0.6911634844733396, -0.33568198264623844`}, {
      0.6966235531486787, -0.3337885100034823}, {
      0.702023955249441, -0.3318664469889318}, {
      0.7073651642448446, -0.32991640075174783`}, {
      0.7126476434222773, -0.32793896187943467`}, {
      0.7178718461469842, -0.32593470503509925`}, {
      0.7230382161135247, -0.3239041895646449}, {
      0.7281471875892591, -0.32184796007559546`}, {
      0.733199185650124, -0.3197665469891363}, {
      0.7381946264089478, -0.3176604670668561}, {
      0.7431339172365521, -0.3155302239135779}, {
      0.7480174569758756, -0.3133763084575815}, {
      0.7528456361493558, -0.31119919940943574`}, {
      0.757618837159791, -0.3089993637005842}, {
      0.7623374344849021, -0.3067772569027579}, {
      0.7670017948658072, -0.30453332362922236`}, {
      0.7716122774896119, -0.3022679979188043}, {
      0.7761692341663153, -0.2999817036035891}, {
      0.7806730095002236, -0.2976748546611239}, {
      0.7851239410560555, -0.29534785555191484`}, {
      0.789522359519921, -0.29300110154295805`}, {
      0.7938685888553434, -0.29063497901800356`}, {
      0.7981629464544951, -0.2882498657752087}, {
      0.8024057432848054, -0.285846131312802}, {
      0.8065972840310971, -0.2834241371033418}, {
      0.8107378672334025, -0.2809842368571224}, {
      0.814827785420602, -0.2785267767752487}, {
      0.8188673252400253, -0.2760520957928712}, {
      0.8228567675831495, -0.2735605258130485}, {
      0.8267963877075241, -0.27105239193167524`}, {
      0.8306864553550463, -0.2685280126538934}, {
      0.8345272348667081, -0.26598770010238076`}, {
      0.8383189852939303, -0.2634317602178888}, {
      0.8420619605065953, -0.26086049295238434`}, {
      0.8457564092978853, -0.25827419245512956`}, {
      0.8494025754860309, -0.25567314725201756`}, {
      0.8530006980130682, -0.25305764041846524`}, {
      0.8565510110407015, -0.2504279497461487}, {
      0.8600537440433637, -0.24778434790385265`}, {
      0.8635091218985628, -0.24512710259269155`}, {
      0.8669173649746025, -0.24245647669594686`}, {
      0.8702786892157564, -0.23977272842375322`}, {
      0.8735933062249779, -0.23707611145285465`}, {
      0.8768614233442217, -0.23436687506164092`}, {
      0.8800832437324501, -0.23164526426066445`}, {
      0.8832589664413967, -0.22891151991882805`}, {
      0.8863887864891544, -0.2261658788854248}, {
      0.8894728949316555, -0.22340857410820325`}, {
      0.8925114789321056, -0.22063983474762197`}, {
      0.8955047218284347, -0.21785988628745104`}, {
      0.8984528031988227, -0.2150689506418697}, {
      0.9013558989253581, -0.21226724625920326`}, {
      0.9042141812558822, -0.2094549882224352}, {
      0.907027818864075, -0.20663238834662498`}, {
      0.9097969769078312, -0.20379965527335556`}, {
      0.9125218170859759, -0.20095699456232918`}, {
      0.9152024976933681, -0.19810460878022534`}, {
      0.9178391736744378, -0.1952426975869289}, {
      0.9204319966751987, -0.19237145781923223`}, {
      0.922981115093782, -0.18949108357211097`}, {
      0.925486674129529, -0.18660176627766772`}, {
      0.9279488158306848, -0.1837036947818353}, {
      0.9303676791407285, -0.18079705541892607`}, {
      0.9327433999433786, -0.17788203208411127`}, {
      0.935076111106308, -0.17495880630390986`}, {
      0.9373659425236021, -0.1720275573047641}, {
      0.9396130211569944, -0.16908846207977485`}, {
      0.9418174710759097, -0.16614169545366753`}, {
      0.9439794134963462, -0.16318743014605644`}, {
      0.9460989668186254, -0.16022583683307232`}, {
      0.9481762466640394, -0.15725708420741574`}, {
      0.9502113659104213, -0.1542813390368963}, {
      0.9522044347266658, -0.15129876622151553`}, {
      0.954155560606226, -0.14830952884914858`}, {
      0.9560648483996096, -0.14531378824987856`}, {
      0.9579324003458994, -0.1423117040490346}, {
      0.9597583161033203, -0.13930343421898325`}, {
      0.9615426927788735, -0.13628913512972082`}, {
      0.9632856249570618, -0.13326896159831253`}, {
      0.9649872047277238, -0.1302430669372229}, {
      0.9666475217129972, -0.1272116030015799}, {
      0.9682666630934311, -0.12417472023541415`}, {
      0.9698447136332647, -0.1211325677169129}, {
      0.9713817557048888, -0.11808529320272729`}, {
      0.9728778693125093, -0.11503304317136984`}, {
      0.9743331321150267, -0.11197596286573817`}, {
      0.9757476194481487, -0.10891419633479961`}, {
      0.9771214043457495, -0.10584788647447023`}, {
      0.9784545575604912, -0.10277717506772094`}, {
      0.9797471475837207, -0.09970220282394193}, {
      0.9809992406646558, -0.09662310941759623}, {
      0.9822109008288719, -0.09354003352619177}, {
      0.983382189896103, -0.09045311286760077}, {
      0.9845131674973686, -0.08736248423675444}, {
      0.9856038910914359, -0.08426828354173985}, {
      0.9866544159806306, -0.08117064583932557}, {
      0.9876647953260047, -0.07806970536994157}, {
      0.9886350801618714, -0.07496559559213838}, {
      0.9895653194097174, -0.07185844921654978}, {
      0.9904555598914999, -0.0687483982393828}, {
      0.9913058463423388, -0.065635573975458}, {
      0.9921162214226101, -0.06252010709082285}, {
      0.9928867257294499, -0.059402127634960035`}, {
      0.9936173978076749, -0.05628176507261229}, {
      0.9943082741601271, -0.053159148315245056`}, {
      0.9949593892574494, -0.05003440575216743}, {
      0.9955707755472973, -0.046907665281331806`}, {
      0.996142463462993, -0.04377905433983202}, {
      0.9966744814316274, -0.04064869993411964}, {
      0.9971668558816152, -0.037516728669957505`}, {
      0.9976196112497071, -0.0343832667821295}, {
      0.9980327699874634, -0.031248440163925154`}, {
      0.9984063525671945, -0.028112374396417367`}, {
      0.99874037748737, -0.024975194777551515`}, {
      0.9990348612775011, -0.021837026351063698`}, {
      0.9992898185024982, -0.018697993935245915`}, {
      0.9995052617665079, -0.015558222151575674`}, {
      0.99968120171623, -0.012417835453227402`}, {
      0.9998176470437178, -0.009276958153482909}, {
      0.999914604488663, -0.006135714454058013}, {
      0.9999720788401661, -0.0029942284733623674`}, {0.9999900729379949, 
      0.0001473757252905706}, {0.9999685876733293, 0.0032889741055065294`}, {
      0.999907621988996, 0.006430442629649515}, {0.9998071728791903, 
      0.009571657230680168}, {0.9996672353886874, 0.012712493783976552`}, {
      0.9994878026115399, 0.015852828079120494`}, {0.9992688656892641, 
      0.018992535791632615`}, {0.9990104138085106, 0.022131492454639055`}, {
      0.9987124341982204, 0.02526957343045288}, {0.9983749121262631, 
      0.028406653882053042`}, {0.9979978308955558, 0.031542608744443625`}, {
      0.9975811718396612, 0.034677312695876004`}, {0.9971249143178597, 
      0.037810640128916384`}, {0.996629035709695, 0.040942465121340955`}, {
      0.996093511408989, 0.04407266140684082}, {0.9955183148173219, 
      0.047201102345518456`}, {0.9949034173369734, 0.050327660894157446`}, {
      0.9942487883633219, 0.053452209576246704`}, {0.9935543952766949, 
      0.05657462045174039}, {0.9928202034336681, 0.0596947650865342}, {
      0.992046176157805, 0.0628125145216385}, {0.9912322747298337, 
      0.06592773924202829}, {0.990378458377253, 0.06904030914514989}, {
      0.9894846842633622, 0.07215009350906336}, {0.9885509074757073, 
      0.07525696096019972}, {0.9875770810139364, 0.07836077944071126}, {
      0.9865631557770567, 0.08146141617539285}, {0.9855090805500849, 
      0.08455873763815161}, {0.9844148019900824, 0.08765260951800188}, {
      0.9832802646115664, 0.09074289668456156}, {0.9821054107712875, 
      0.0938294631530256}, {0.9808901806523646, 0.0969121720485915}, {
      0.9796345122477648, 0.09999088557031138}, {0.9783383413431208, 
      0.10306546495434385`}, {0.9770016014988714, 0.10613577043657874`}, {
      0.9756242240317149, 0.10920166121460677`}, {0.9742061379953636, 
      0.11226299540900517`}, {0.9727472701605854, 0.1153196300239097}, {
      0.9712475449945196, 0.11837142090684245`}, {0.9697068846392541, 
      0.12141822270776381`}, {0.968125208889648, 0.12445988883731597`}, {
      0.9665024351703855, 0.12749627142422443`}, {0.9648384785122451, 
      0.13052722127182256`}, {0.963133251527569, 0.1335525878136633}, {
      0.9613866643849135, 0.13657221906818073`}, {0.9595986247828654, 
      0.13958596159236314`}, {0.9577690379230046, 0.14259366043439747`}, {
      0.9558978064819946, 0.1455951590852441}, {0.9539848305827809, 
      0.14859029942909868`}, {0.9520300077648773, 0.1515789216926973}, {
      0.9500332329537187, 0.15456086439341818`}, {0.947994398429058, 
      0.15753596428613267`}, {0.9459133937923857, 0.1605040563087555}, {
      0.9437901059333466, 0.16346497352644296`}, {0.9416244189951302, 
      0.16641854707438541`}, {0.9394162143388095, 0.16936460609913842`}, {
      0.9371653705066012, 0.17230297769843445`}, {0.9348717631840195, 
      0.17523348685941528`}, {0.9325352651608965, 0.17815595639522197`}, {
      0.9301557462912379, 0.1810702068798774}, {0.9277330734518848, 
      0.18397605658139346`}, {0.9252671104999495, 0.18687332139303175`}, {
      0.9227577182289911, 0.18976181476264428`}, {0.9202047543238995, 
      0.1926413476200171}, {0.9176080733144488, 0.19551172830213664`}, {
      0.9149675265274871, 0.19837276247629482`}, {0.9122829620377213, 
      0.20122425306094574`}, {0.9095542246170594, 0.2040660001442226}, {
      0.9067811556824686, 0.20689780090001936`}, {0.903963593242307, 
      0.20971944950153776`}, {0.9011013718410841, 0.2125307370321954}, {
      0.8981943225026046, 0.2153314513937861}, {0.8952422726714487, 
      0.21812137721177846`}, {0.8922450461527391, 0.22090029573763378`}, {
      0.8892024630501425, 0.22366798474801824`}, {0.8861143397020543, 
      0.226424218440779}, {0.8829804886159114, 0.22916876732754718`}, {
      0.8798007184005738, 0.2319013981228245}, {0.8765748336967184, 
      0.23462187362940326`}, {0.8733026351051818, 0.23732995261996181`}, {
      0.869983919113189, 0.2400253897146707}, {0.8666184780184011, 
      0.24270793525463535`}, {0.8632060998507143, 0.24537733517099372`}, {
      0.8597465682917375, 0.24803333084947704`}, {0.856239662591875, 
      0.2506756589902333}, {0.8526851574849383, 0.2533040514627018}, {
      0.8490828231002048, 0.2559182351553167}, {0.8454324248718429, 
      0.2585179318198063}, {0.8417337234456163, 0.26110285790984183`}, {
      0.8379864745827778, 0.2636727244137774}, {0.834190429061061, 
      0.26622723668120846`}, {0.8303453325726722, 0.26876609424306136`}, {
      0.8264509256191834, 0.27128899062491246`}, {0.8225069434032226, 
      0.2737956131532166}, {0.8185131157168527, 0.27628564275410933`}, {
      0.8144691668265287, 0.27875875374442743`}, {0.8103748153545153, 
      0.281214613614573}, {0.8062297741566457, 0.2836528828028253}, {
      0.8020337501962954, 0.2860732144606815}, {0.7977864444144426, 
      0.2884752542087849}, {0.7934875515956782, 0.29085863988297217`}, {
      0.7891367602300283, 0.29322300126994505`}, {0.784733752370442, 
      0.29556795983204254`}, {0.7802782034857948, 0.2978931284205589}, {
      0.7757697823092525, 0.300198110977019}, {0.7712081506818326, 
      0.3024825022217886}, {0.7665929633909959, 0.3047458873293575}, {
      0.7619238680040964, 0.3069878415895949}, {0.7572005046965069, 
      0.30920793005423164`}, {0.7524225060742367, 0.31140570716777816`}, {
      0.7475894969908479, 0.31358071638203744`}, {0.7427010943584713, 
      0.31573248975331847`}, {0.7377569069527167, 0.3178605475213991}, {
      0.732756535211264, 0.3199643976692247}, {0.7276995710259182, 
      0.3220435354622648}, {0.7225855975278995, 0.3240974429663764}, {
      0.7174141888661393, 0.3261255885429492}, {0.7121849099783395, 
      0.3281274263200232}, {0.7068973163545518, 0.330102395637982}, {
      0.7015509537930236, 0.33204992046833}, {0.696145358148053, 
      0.3339694088039568}, {0.6906800550695893, 0.33586025201918396`}, {
      0.6851545597343085, 0.33772182419776564`}, {0.679568376567893, 
      0.3395534814268886}, {0.6739209989582359, 0.34135456105507367`}, {
      0.66821190895929, 0.34312438091172975`}, {0.6624405769852807, 
      0.3448622384859481}, {0.6566064614949992, 0.34656741006194347`}, {
      0.6507090086658949, 0.3482391498083569}, {0.6447476520576891, 
      0.3498766888184244}, {0.6387218122652385, 0.351479234097785}, {
      0.6326308965603813, 0.35304596749645617`}, {0.6264742985225162, 
      0.35457604458123015`}, {0.6202513976576743, 0.3560685934444519}, {
      0.6139615590058669, 0.3575227134448166}, {0.6076041327365159, 
      0.3589374738754729}, {0.6011784537318056, 0.3603119125543341}, {
      0.5946838411578308, 0.3616450343310796}, {0.5881195980234668, 
      0.3629358095048693}, {0.5814850107269395, 0.36418317214628876`}, {
      0.5747793485901446, 0.3653860183164911}, {0.5680018633808462, 
      0.366543204175895}, {0.5611517888229822, 0.3676535439741301}, {
      0.5542283400954191, 0.3687158079121886}, {0.5472307133196385, 
      0.36972871986693295`}, {0.5401580850369982, 0.37069095496721816`}, {
      0.5330096116764031, 0.37160113700990544`}, {0.5257844290134482, 
      0.37245783570295515`}, {0.5184816516223626, 0.3732595637215861}, {
      0.5111003723223969, 0.3740047735621566}, {0.5036396616206663, 
      0.3746918541769467}, {0.4960985671538943, 0.37531912737138134`}, {
      0.48847611313201517`, 0.37588484394341365`}, {0.48077129978719074`, 
      0.3763871795427604}, {0.47298310283250333`, 0.3768242302254229}, {
      0.4651104729354178, 0.3771940076764068}, {0.4571523352120807, 
      0.37749443407074124`}, {0.4491075887496756, 0.37772333653974743`}, {
      0.4409751061654081, 0.37787844120598385`}, {0.43275373321229327`, 
      0.37795736674633956`}, {0.42444228844380394`, 0.37795761743830897`}, {
      0.41603956295166955`, 0.3778765756394892}, {0.4075443201937529, 
      0.37771149364472045`}, {0.39895529593205975`, 0.3774594848589547}, {
      0.39027119830464474`, 0.3771175142167822}, {0.38149070805958596`, 
      0.3766823877714624}, {0.3726124789844464, 0.3761507413671501}, {
      0.3636351385708962, 0.375519028297636}, {0.35455728896164096`, 
      0.37478350584314957`}, {0.3453775082357416, 0.37394022056339854`}, {
      0.336094352099132, 0.37298499220980863`}, {0.3267063560600243, 
      0.37191339610260854`}, {0.31721203818440163`, 0.3707207437986666}, {
      0.3076099025455256, 0.36940206185347274`}, {0.2978984435040362, 
      0.36795206845496375`}, {0.2880761509826977, 0.3663651476775495}, {
      0.27814151693325145`, 0.3646353210711919}, {0.268093043233565, 
      0.3627562162621274}, {0.2579292513030541, 0.36072103219816865`}, {
      0.2476486937853921, 0.3585225006217567}, {0.2372499687225526, 
      0.35615284329732966`}, {0.2267317367367518, 0.3536037244553558}, {
      0.21609274185126362`, 0.35086619784283807`}, {0.20533183672299204`, 
      0.34793064768861987`}, {0.19444801323622873`, 0.3447867228010758}, {
      0.18344043962732312`, 0.341423262915872}, {0.1723085055856911, 
      0.33782821630336435`}, {0.16105187712261101`, 0.3339885475311217}, {
      0.14967056343470825`, 0.32989013416137425`}, {0.1381649985383846, 
      0.32551765105357516`}, {0.12653614114598072`, 0.32085444085154746`}, {
      0.11478559713401143`, 0.31588236918359713`}, {0.10291577006880873`, 
      0.31058166312534224`}, {0.0909300466685431, 0.3049307316200648}, {
      0.07883302587080235, 0.29890596689924254`}, {0.06663080243537611, 
      0.2924815266177107}, {0.05433131885055352, 0.28562909759787175`}, {
      0.04194480284323202, 0.2783176440437972}, {0.029484312122642624`, 
      0.2705131462560767}, {0.016966413168241632`, 0.2621783408711141}, {
      0.004412026828658534, 0.2532724813713506}, {-0.00815252012378467, 
      0.2437511493686046}, {-0.02069419244335089, 
      0.2335661647675291}, {-0.03317164433860645, 
      0.2226656687805257}, {-0.04553327199847016, 
      0.2109944908213253}, {-0.057714890367716626`, 
      0.19849496151472373`}, {-0.06963704368083258, 
      0.18510840097636622`}, {-0.08120204142916675, 
      0.1707775909326251}, {-0.09229096112123493, 
      0.155450616536131}, {-0.10276110494560153`, 
      0.13908650152386193`}, {-0.11244475558617796`, 
      0.1216629836625582}, {-0.12115051346837591`, 
      0.10318646430649993`}, {-0.12866886524349294`, 
      0.0837034704864043}, {-0.1347836016638939, 
      0.06331182241990696}, {-0.13928979256694654`, 
      0.042168324879997165`}, {-0.14201690176607626`, 
      0.020488927185587896`}, {-0.14285268127637946`, \
-0.0014619555856199562`}, {-0.14176125765619546`, -0.023393213007224682`}, \
{-0.13878931191299046`, -0.04501571239875599}, {-0.13405807650625562`, \
-0.06607044191613064}, {-0.1277440794143925, -0.08634895123655745}, \
{-0.12005500258448092`, -0.10570248722088292`}, {-0.11120706406722414`, \
-0.12404060392070934`}, {-0.10140789064548351`, -0.14132278342352247`}, \
{-0.0908459490589818, -0.1575470749674001}, {-0.07968564006540269, \
-0.17273875859714605`}, {-0.06806639332688708, -0.18694066588287994`}, \
{-0.05610414831733771, -0.20020570067408922`}, {-0.04389399783273605, \
-0.2125914655056584}, {-0.031513202507310406`, -0.2241566260071125}, \
{-0.019024128258379583`, -0.2349585901816035}, {-0.006476890110805902, \
-0.2450521258069512}, {0.006088375086141903, -0.254488618295919}, {
      0.018639609291897893`, -0.263315749612506}, {
      0.03115127374992093, -0.27157744370458764`}, {
      0.043603082516002506`, -0.2793139731017283}, {
      0.05597898072518977, -0.2865621567183691}, {
      0.06826632325617699, -0.29335560351057355`}, {
      0.0804552154939663, -0.2997249733371629}, {
      0.09253798425001268, -0.3056982375039906}, {
      0.10450875276493311`, -0.3113009287651284}, {
      0.11636309878717528`, -0.31655637526051383`}, {
      0.12809777894001134`, -0.3214859158492403}, {
      0.13971050602272436`, -0.3261090961391546}, {
      0.15119976864721013`, -0.3304438456122692}, {
      0.1625646848033432, -0.33450663686524906`}, {
      0.17380488268163996`, -0.3383126282992331}, {
      0.18492040345160102`, -0.34187579171801513`}, {
      0.19591162177452284`, -0.3452090263028598}, {
      0.20677918068190693`, -0.3483242603742766}, {
      0.2175239381237684, -0.3512325422568434}, {
      0.22814692302374145`, -0.353944121452133}, {
      0.2386492991001794, -0.3564685212089547}, {
      0.24903233504813393`, -0.35881460346662736`}, {
      0.25929737994466084`, -0.3609906270398715}, {
      0.26944584295377844`, -0.3630042998151878}, {
      0.2794791765788733, -0.36486282563906886`}, {
      0.28939886284823085`, -0.3665729464981291}, {
      0.2992064019305766, -0.3681409805198305}, {
      0.308903302767476, -0.36957285625931885`}, {
      0.31849107538243465`, -0.37087414368222565`}, {
      0.3279712245859352, -0.37205008220438146`}, {
      0.3373452448441182, -0.373105606106481}, {
      0.3466146161184768, -0.3740453676041489}, {
      0.35578080051648153`, -0.3748737578209305}, {
      0.36484523961982623`, -0.3755949258829049}, {
      0.3738093523790706, -0.37621279632836846`}, {
      0.3826745334817123, -0.376731085003914}, {
      0.3914421521158559, -0.37715331359883253`}, {
      0.4001135510642138, -0.37748282295274227`}, {
      0.4086900460736397, -0.3777227852563971}, {
      0.4171729254541229, -0.3778762152524781}, {
      0.4255634498684684, -0.3779459805315981}, {
      0.4338628522799967, -0.3779348110085461}, {
      0.4420723380307247, -0.3778453076547966}, {
      0.450193085026795, -0.3776799505553535}, {
      0.458226244011548, -0.3774411063509581}, {
      0.46617293890968703`, -0.37713103512045687`}, {
      0.4740342672285669, -0.3767518967525909}, {
      0.4818113005048169, -0.3763057568515552}, {
      0.4895050847863558, -0.37579459221630435`}, {
      0.4971166411414181, -0.3752202959296876}, {
      0.5046469661875387, -0.37458468209002355`}, {
      0.5120970326345659, -0.3738894902146236}, {
      0.5194677898367311, -0.37313638934199966`}, {
      0.5267601643496139, -0.372326981857009}, {
      0.5339750604885302, -0.37146280706096246`}, {
      0.5411133608854631, -0.37054534450672433`}, {
      0.5481759270421516, -0.36957601711703647`}, {
      0.5551635998773762, -0.3685561941026818}, {
      0.5620772002668482, -0.3674871936956482}, {
      0.5689175295744082, -0.3663702857111387}, {
      0.5756853701735075, -0.365206693951089}, {
      0.5823814859581623, -0.36399759846078006`}, {
      0.5890066228427624, -0.36274413764916386`}, {
      0.5955615092502728, -0.36144741028264005`}, {
      0.6020468565885017, -0.36010847736122403`}, {
      0.6084633597142202, -0.35872836388532214`}, {
      0.6148116973850187, -0.35730806052067127`}, {
      0.6210925326988587, -0.3558485251684007}, {
      0.6273065135213495, -0.3543506844466283}, {
      0.6334542729008339, -0.3528154350895052}, {
      0.6395364294714112, -0.35124364526916857`}, {
      0.6455535878440646, -0.34963615584564767`}, {
      0.6515063389860895, -0.34799378154938804`}, {
      0.6573952605890439, -0.3463173121007115}, {
      0.6632209174254612, -0.3446075132702115}, {
      0.6689838616945794, -0.3428651278837915}, {
      0.6746846333573537, -0.34109087677578537`}, {
      0.6803237604610253, -0.33928545969335444`}, {
      0.6859017594535244, -0.3374495561551273}, {
      0.6914191354879913, -0.3355838262668425}, {
      0.6968763827176955, -0.3336889114965613}, {
      0.7022739845816365, -0.3317654354118411}, {
      0.7076124140811054, -0.3298140043810977}, {
      0.7128921340474849, -0.3278352082412339}, {
      0.7181135974015596, -0.32582962093347284`}, {
      0.7232772474046073, -0.32379780110920603`}, {
      0.7283835179015311, -0.32174029270754795`}, {
      0.7334328335562926, -0.3196576255061776}, {
      0.7384256100798956, -0.31755031564694713`}, {
      0.7433622544511663, -0.315418866137642}, {
      0.7482431651305684, -0.31326376733118944`}, {
      0.7530687322672838, -0.3110854973835324}, {
      0.757839337899785, -0.3088845226913072}, {
      0.7625553561501174, -0.306661298310396}, {
      0.7672171534121021, -0.304416268356359}, {
      0.7718250885336652, -0.30214986638768826`}, {
      0.7763795129934906, -0.299862515772772}, {
      0.7808807710721895, -0.2975546300414022}, {
      0.7853292000181692, -0.2952266132216109}, {
      0.7897251302083839, -0.29287886016257425`}, {
      0.794068885304136, -0.29051175684427966`}, {
      0.7983607824020988, -0.2881256806746128}, {
      0.8026011321807182, -0.28572100077448226`}, {
      0.80679023904215, -0.2832980782515654}, {
      0.810928401249884, -0.2808572664632263}, {
      0.8150159110621956, -0.27839891126912536`}, {
      0.8190530548615678, -0.27592335127401213`}, {
      0.8230401132802149, -0.2734309180611652}, {
      0.8269773613218394, -0.2709219364169185}, {
      0.830865068479745, -0.26839672454668884`}, {
      0.8347034988514255, -0.26585559428289873`}, {
      0.8384929112497472, -0.263298851285165}, {
      0.8422335593108342, -0.26072679523310743`}, {
      0.8459256915987641, -0.2581397200121104}, {
      0.8495695517071785, -0.2555379138923545}, {
      0.853165378357907, -0.2529216597014191}, {
      0.8567134054966999, -0.2502912349907403}, {
      0.8602138623861644, -0.24764691219619459`}, {
      0.8636669736959891, -0.2449889587930665}, {
      0.8670729595905475, -0.24231763744564297`}, {
      0.8704320358139584, -0.2396332061516671}, {
      0.8737444137726857, -0.2369359183818721}, {
      0.8770103006157529, -0.23422602321480462`}, {
      0.8802298993126461, -0.2315037654671378}, {
      0.8834034087289769, -0.22876938581966355`}, {
      0.8865310236999739, -0.22602312093914545`}, {
      0.8896129351018675, -0.22326520359620436`}, {
      0.8926493299212324, -0.22049586277940117`}, {
      0.8956403913223504, -0.2177153238056733}, {
      0.8985862987126493, -0.2149238084272742}, {
      0.9014872278062772, -0.2121215349353586}, {
      0.9043433506858669, -0.20930871826034905`}, {
      0.9071548358625418, -0.20648557006921409`}, {
      0.9099218483342149, -0.20365229885978156`}, {
      0.9126445496422311, -0.2008091100522059}, {
      0.9153230979263978, -0.19795620607770228`}, {
      0.917957647978451, -0.19509378646465625`}, {
      0.9205483512940009, -0.1922220479222118}, {
      0.9230953561229978, -0.18934118442143752`}, {
      0.9255988075187607, -0.18645138727416533`}, {
      0.9280588473856077, -0.1835528452095924}, {
      0.9304756145251254, -0.1806457444487335}, {
      0.9328492446811139, -0.1777302687768068}, {
      0.9351798705832439, -0.17480659961363326`}, {
      0.9374676219894587, -0.17187491608212557`}, {
      0.9397126257271537, -0.16893539507494076`}, {
      0.9419150057331668, -0.16598821131936647`}, {
      0.944074883092607, -0.16303353744050844`}, {
      0.9461923760765545, -0.16007154402284446`}, {
      0.9482676001786567, -0.15710239967020692`}, {
      0.9503006681506512, -0.1541262710642539}, {
      0.9522916900368379, -0.15114332302148642`}, {
      0.95424077320753, -0.14815371854886736`}, {
      0.9561480223915049, -0.14515761889809506`}, {
      0.9580135397074802, -0.1421551836185833}, {
      0.9598374246946371, -0.13914657060919666`}, {
      0.9616197743422134, -0.13613193616878919`}, {
      0.9633606831181861, -0.13311143504559175`}, {
      0.9650602429970662, -0.1300852204854928}, {
      0.9667185434868216, -0.12705344427925483`}, {
      0.9683356716549515, -0.12401625680870779`}, {
      0.9699117121537256, -0.12097380709195901`}, {
      0.9714467472446102, -0.11792624282765823`}, {
      0.9729408568218938, -0.11487371043835454`}, {
      0.9743941184355314, -0.11181635511298121`}, {
      0.9758066073132219, -0.10875432084850302`}, {
      0.9771783963817329, -0.1056877504907595}, {
      0.9785095562874884, -0.1026167857745368}, {
      0.9798001554164322, -0.09954156736289932}, {
      0.9810502599131813, -0.09646223488581185}, {
      0.9822599336994807, -0.09337892697808159}, {
      0.9834292384919732, -0.09029178131664889}, {
      0.984558233819294, -0.08720093465725444}, {
      0.9856469770385035, -0.0841065228705101}, {
      0.9866955233508674, -0.08100868097739951}, {
      0.9877039258169946, -0.07790754318423429}, {
      0.9886722353713434, -0.07480324291709062}, {
      0.9896005008361041, -0.07169591285575046}, {
      0.9904887689344682, -0.0685856849671712}, {
      0.9913370843032923, -0.06547269053850675}, {
      0.992145489505163, -0.06235706020970265}, {
      0.9929140250398741, -0.05923892400568717}, {
      0.9936427293553197, -0.05611841136818011}, {
      0.9943316388578118, -0.0529956511871402}, {
      0.9949807879218294, -0.04987077183187188}, {
      0.9955902088992035, -0.04674390118181173}, {
      0.9961599321277453, -0.04361516665701435}, {
      0.9966899859393226, -0.04048469524835724}, {
      0.9971803966673879, -0.03735261354748393}, {
      0.9976311886539659, -0.03421904777650423}, {
      0.9980423842561019, -0.031084123817470097`}, {
      0.9984140038517761, -0.02794796724164568}, {
      0.9987460658452884, -0.02481070333858946}, {
      0.9990385866721149, -0.021672457145066465`}, {
      0.9992915808032415, -0.01853335347380826}, {
      0.9995050607489755, -0.0153935169421382}, {
      0.9996790370622376, -0.012253072000479344`}, {
      0.999813518341338, -0.00911214296076224}, {
      0.9999085112322362, -0.0059708540247497294`}, {
      0.9999640204302872, -0.002829329312295751}, {0.9999800486814744, 
      0.00031230711044485417`}, {0.999956596783131, 0.0034539312028386863`}, {
      0.999893663584149, 0.006595418921611399}, {0.9997912459846768, 
      0.009736646192683746}, {0.9996493389353054, 0.012877488882989226`}, {
      0.9994679354357415, 0.016017822772256465`}, {0.9992470265329684, 
      0.01915752352473941}, {0.9989866013188918, 0.022296466660878422`}, {
      0.9986866469274709, 0.02543452752887516}, {0.9983471485313311, 
      0.02857158127616419}, {0.997968089337858, 0.031707502820764004`}, {
      0.9975494505847687, 0.03484216682249009}, {0.99709121153516, 
      0.03797544765401247}, {0.9965933494720266, 0.041107219371740014`}, {
      0.9960558396922505, 0.044237355686513546`}, {0.9954786555000545, 
      0.04736572993408966}, {0.9948617681999172, 0.05049221504539678}, {
      0.9942051470889448, 0.05361668351654482}, {0.9935087594486951, 
      0.05673900737856958}, {0.9927725705364482, 0.05985905816689249}, {
      0.9919965435759194, 0.06297670689047621}, {0.9911806397474073, 
      0.06609182400065612}, {0.9903248181773717, 0.06920427935962732}, {
      0.9894290359274346, 0.07231394220856645}, {0.9884932479827971, 
      0.07542068113536719}, {0.9875174072400659, 0.07852436404196768}, {
      0.9865014644944793, 0.08162485811124785}, {0.9854453684265282, 
      0.08472202977347397}, {0.9843490655879595, 0.08781574467226716}, {
      0.9832125003871566, 0.09090586763007222}, {0.9820356150738845, 
      0.09399226261310221}, {0.9808183497233924, 0.0970747926957339}, {
      0.9795606422198615, 0.10015332002432813`}, {0.9782624282391873, 
      0.10322770578044901`}, {0.9769236412310871, 0.10629781014345432`}, {
      0.9755442124005184, 0.10936349225242936`}, {0.9741240706883975, 
      0.11242461016743535`}, {0.9726631427516054, 0.1154810208300427}, {
      0.9711613529422668, 0.11853258002311834`}, {0.9696186232862896, 
      0.12157914232983562`}, {0.9680348734611487, 0.12462056109187412`}, {
      0.9664100207729015, 0.12765668836677557`}, {0.9647439801324169, 
      0.13068737488442103`}, {0.9630366640308035, 0.13371247000259334`}, {
      0.9612879825140186, 0.13673182166158757`}, {0.9594978431566421, 
      0.1397452763378307}, {0.9576661510347951, 0.1427526789964709}, {
      0.9557928086981861, 0.14575387304289458`}, {0.9538777161412633, 
      0.1487487002731287}, {0.9519207707734547, 0.15173700082308364`}, {
      0.9499218673884727, 0.15471861311659046`}, {0.9478808981326624, 
      0.15769337381218476`}, {0.9457977524723704, 0.16066111774858724`}, {
      0.9436723171603102, 0.16362167788882956`}, {0.9415044762009004, 
      0.16657488526297154`}, {0.9392941108145488, 0.16952056890935407`}, {
      0.9370410994008573, 0.1724585558143297}, {0.934745317500719, 
      0.17538867085041043`}, {0.9324066377572796, 0.1783107367127701}, {
      0.930024929875734, 0.1812245738540356}, {0.9276000605819261, 
      0.1841300004172993}, {0.925131893579721, 0.18702683216728117`}, {
      0.9226202895071164, 0.18991488241956705`}, {0.920065105891059, 
      0.19279396196784576`}, {0.9174661971009305, 0.19566387900906446`}, {
      0.9148234143006653, 0.1985244390664187}, {0.9121366053994641, 
      0.20137544491008916`}, {0.909405615001061, 0.20421669647563376`}, {
      0.9066302843515053, 0.2070479907799397}, {0.9038104512854135, 
      0.2098691218346353}, {0.9009459501706492, 0.2126798805568574}, {
      0.8980366118513832, 0.21548005467726553`}, {0.8950822635894873, 
      0.21826942864518784`}, {0.8920827290042125, 0.22104778353078033`}, {
      0.8890378280100996, 0.2238148969240737}, {0.8859473767530697, 
      0.2265705428307769}, {0.8828111875446393, 0.22931449156470082`}, {
      0.8796290687942028, 0.23204650963665738`}, {0.8764008249393223, 
      0.23476635963968437`}, {0.8731262563739648, 0.23747380013043748`}, {
      0.8698051593746207, 0.2401685855065838}, {0.8664373260242388, 
      0.242850465880023}, {0.8630225441339083, 0.24551918694575348`}, {
      0.859560597162216, 0.2481744898461919}, {0.8560512641322049, 
      0.25081611103074414`}, {0.8524943195458572, 0.25344378211041646`}, {
      0.8488895332960212, 0.25605722970724387`}, {0.8452366705756992, 
      0.2586561752983015}, {0.8415354917846103, 0.2612403350540527}, {
      0.837785752432939, 0.26380941967077437`}, {0.833987203042175, 
      0.26636313419678603`}, {0.8301395890429503, 0.26890117785219536`}, {
      0.8262426506697708, 0.27142324384185623`}, {0.8222961228525398, 
      0.27392901916122}, {0.818299735104766, 0.27641818439474186`}, {
      0.8142532114083413, 0.2788904135064873}, {0.8101562700947756, 
      0.28134537362256135`}, {0.8060086237227653, 0.28378272480496486`}, {
      0.8018099789519699, 0.2862021198164575}, {0.7975600364128693, 
      0.28860320387598426`}, {0.7932584905725636, 0.2909856144041962}, {
      0.7889050295963773, 0.2933489807585692}, {0.7844993352051222, 
      0.2956929239575952}, {0.7800410825278679, 0.29801705639348963`}, {
      0.775529939950064, 0.3003209815328247}, {0.7709655689568526, 
      0.30260429360446434`}, {0.7663476239714015, 0.3048665772741367}, {
      0.7616757521880863, 0.3071074073049413}, {0.7569495934003397, 
      0.30932634820304317`}, {0.752168779822984, 0.3115229538477606}, {
      0.7473329359088499, 0.31369676710520245`}, {0.7424416781594866, 
      0.3158473194245587}, {0.7374946149297549, 0.31797413041608874`}, {
      0.7324913462260905, 0.3200767074097921}, {0.7274314634982193, 
      0.3221545449936782}, {0.7223145494240973, 0.3242071245304824}, {
      0.7171401776878427, 0.32623391365159743`}, {0.7119079127504204, 
      0.3282343657269071}, {0.7066173096128339, 0.3302079193091215}, {
      0.7012679135715697, 0.33215399755111497`}, {0.695859259966039, 
      0.33407200759466654`}, {0.6903908739177502, 0.3359613399288906}, {
      0.684862270060944, 0.33782136771652393`}, {0.6792729522644176, 
      0.3396514460861066}, {0.6736224133442602, 0.3414509113879515}, {
      0.6679101347672188, 0.34321908041164645`}, {0.662135586344414, 
      0.3449552495626659}, {0.6562982259151215, 0.3466586939954904}, {
      0.6503974990203393, 0.34832866670043855`}, {0.6444328385658622, 
      0.3499643975412025}, {0.6384036644745931, 0.35156509223985016`}, {
      0.6323093833278236, 0.3531299313058071}, {0.6261493879952363, 
      0.354658068905058}, {0.619923057253388, 0.35614863166551214`}, {
      0.6136297553924595, 0.3576007174141529}, {0.6072688318110793, 
      0.3590133938412391}, {0.6008396205990625, 0.3603856970864393}, {
      0.5943414401079419, 0.36171663024135864`}, {0.5877735925092187, 
      0.36300516176245545`}, {0.5811353633403158, 0.364250223787839}, {
      0.574426021038286, 0.36545071035088406`}, {0.5676448164614099, 
      0.3666054754829879}, {0.560790982398916, 0.3677133311971258}, {
      0.553863733069175, 0.36877304534312344`}, {0.5468622636068541, 
      0.369783339324752}, {0.5397857495396872, 0.3707428856678567}, {
      0.5326333462557055, 0.3716503054277406}, {0.5254041884620041, 
      0.37250416542293385`}, {0.5180973896363887, 0.373302975281269}, {
      0.5107120414735623, 0.3740451842828454}, {0.5032472133278849, 
      0.37472917798298117`}, {0.4957019516551779, 0.3753532745966038}, {
      0.4880752794565571, 0.37591572112369803`}, {0.48036619572788614`, 
      0.37641468919339305`}, {0.47257367491915153`, 0.37684827060199777`}, {
      0.46469666640889923`, 0.3772144725177609}, {0.45673409399985615`, 
      0.3775112123223006}, {0.4486848554430206, 0.37773631205548264`}, {
      0.44054782199887277`, 0.37788749242697944`}, {0.4323218380459672, 
      0.37796236635376584`}, {0.4240057207490741, 0.3779584319783399}, {
      0.4155982598012847, 0.3778730651174361}, {0.4070982172571578, 
      0.37770351108534234`}, {0.39850432747714015`, 0.3774468758295561}, {
      0.3898152972072348, 0.37710011630931767`}, {0.3810298058223393, 
      0.37666003003941856`}, {0.3721465057669716, 0.37612324371247347`}, {
      0.36316402323341224`, 0.37548620080240125`}, {0.35408095912483356`, 
      0.3747451480400139}, {0.3448958903600111, 0.37389612063814803`}, {
      0.3356073715870351, 0.37293492612846324`}, {0.3262139373864453, 
      0.3718571266545933}, {0.3167141050598778, 0.3706580195464662}, {
      0.3071063781192197, 0.3693326159779402}, {0.29738925061414684`, 
      0.3678756174840319}, {0.28756121246367344`, 0.36628139008447136`}, {
      0.2776207559910908, 0.36454393572658}, {0.2675663839028241, 
      0.3626568607219482}, {0.25739661900204913`, 0.36061334080743185`}, {
      0.24711001598959764`, 0.35840608241089345`}, {0.23670517578052674`, 
      0.3560272796451296}, {0.2261807628582606, 0.35346856648880004`}, {
      0.21553552630389694`, 0.3507209635401822}, {0.20476832528178027`, 
      0.3477748186476287}, {0.1938781599410124, 0.3446197406293791}, {
      0.18286420891542177`, 0.3412445251950228}, {0.17172587488345467`, 
      0.3376370720724235}, {0.1604628399995891, 0.3337842922296228}, {
      0.14907513344956405`, 0.32967200396565244`}, {0.13756321393774668`, 
      0.3252848165351891}, {0.12592807061800163`, 0.32060599988273447`}, {
      0.11417134686981358`, 0.3156173390137418}, {0.10229549245018259`, 
      0.31029897155643876`}, {0.09030395098286922, 0.3046292072214297}, {
      0.07820139155865174, 0.2985843282273743}, {0.06599399550785885, 
      0.2921383704539661}, {0.05368981227809752, 0.2852628862963071}, {
      0.041299201920952315`, 0.2779266922118168}, {0.028835386064273806`, 
      0.2700956071978826}, {0.016315134470903173`, 0.26173219354736554`}, {
      0.00375962027468112, 0.2527955191245844}, {-0.008804516634398962, 
      0.24324097241318549`}, {-0.021343852774903523`, 
      0.233020179554917}, {-0.03381656220605954, 
      0.2220810989660402}, {-0.04617045016163268, 
      0.21036840683683894`}, {-0.05834061017936035, 
      0.19782433882610115`}, {-0.07024671726193471, 
      0.1843902209660373}, {-0.08179005458790782, 
      0.1700090025955375}, {-0.09285052525483778, 
      0.15462918055488115`}, {-0.10328415191986084`, 
      0.13821053797036773`}, {-0.11292193091168092`, 
      0.12073203540818898`}, {-0.12157134578334346`, 
      0.10220186280176456`}, {-0.12902220195677563`, 
      0.08266894211707308}, {-0.1350583793481593, 
      0.06223400430057568}, {-0.13947613137059603`, 
      0.04105699277533279}, {-0.14210737261865328`, 
      0.01935672741005856}, {-0.142843444234417, -0.002600377501571497}, \
{-0.14165271381135341`, -0.024522581413480054`}, {-0.13858603316659235`, \
-0.04612166007478949}, {-0.13376803958455458`, -0.06714080617644226}, \
{-0.12737747616255082`, -0.08737453413941348}, {-0.11962298587306489`, \
-0.10667721889957449`}, {-0.11072070678490187`, -0.12496124467600278`}, \
{-0.10087747980992859`, -0.14218837782709476`}, {-0.09028060599134527, \
-0.15835834535129112`}, {-0.07909319034700475, -0.17349755536034414`}, \
{-0.06745339714813087, -0.187649525575524}, {-0.05547601708467898, \
-0.20086751889152799`}, {-0.04325514597368696, -0.21320926981822513`}, \
{-0.030867203855677636`, -0.22473342713344116`}, {-0.018373861051751907`, \
-0.23549729029375144`}, {-0.005824663846303112, -0.2455554665519606}, {
      0.006740712473562078, -0.25495915535745306`}, {
      0.019290582823480194`, -0.2637558443666975}, {
      0.03179970914144521, -0.271989265445676}, {
      0.04424804773391367, -0.27969950745682154`}, {
      0.05661973894312557, -0.2869232173809808}, {
      0.06890229507633398, -0.29369384545754484`}, {
      0.08108594862843128, -0.3000419063884331}, {
      0.0931631291765446, -0.30599523954340846`}, {
      0.10512804315435377`, -0.3115792582386096}, {
      0.11697633573816889`, -0.3168171827583085}, {
      0.12870481825303104`, -0.3217302546982125}, {
      0.14031124790275704`, -0.32633793200335853`}, {
      0.1517941493513098, -0.33065806514216095`}, {
      0.16315266984889298`, -0.3347070554585023}, {
      0.17438646131036073`, -0.3384999970465145}, {
      0.1854955841066938, -0.3420508036099236}, {
      0.19648042839752325`, -0.34537232177259547`}, {
      0.207341649674659, -0.34847643224644703`}, {
      0.21808011585162007`, -0.3513741401673822}, {
      0.22869686376037707`, -0.3540756557983771}, {
      0.23919306333381085`, -0.35659046668297356`}, {
      0.2495699880841402, -0.35892740221919334`}, {
      0.2598289907520358, -0.3610946915171508}, {
      0.26997148321257103`, -0.3631000153053848}, {
      0.2799989198936935, -0.3649505525618919}, {
      0.28991278409925053`, -0.3666530224660457}, {
      0.2997145767385875, -0.36821372219662735`}, {
      0.30940580705372905`, -0.3696385610384297}, {
      0.31898798500736564`, -0.37093309120460777`}, {
      0.3284626150536375, -0.3721025357333651}, {
      0.3378311910616767, -0.3731518137749518}, {
      0.3470951922011227, -0.374085563547614}, {
      0.3562560796310462, -0.37490816320843534`}, {
      0.36531529386022155`, -0.3756237498563803}, {
      0.37427425266855585`, -0.37623623685977287`}, {
      0.3831343494975638, -0.37674932967846986`}, {
      0.3918969522327635, -0.37716654033172264`}, {
      0.40056340231332027`, -0.37749120064581043`}, {
      0.4091350141146299, -0.377726474400677}, {
      0.41761307455818075`, -0.37787536848173986`}, {
      0.42599884291026385`, -0.37794074313154274`}, {
      0.434293550737153, -0.37792532138578616`}, {
      0.4424984019894581, -0.37783169776932474`}, {
      0.45061457319262394`, -0.37766234631981666`}, {
      0.4586432137241386, -0.3774196279997141}, {
      0.46658544616104825`, -0.377105797551088}, {
      0.47444236668393014`, -0.37672300984227997`}, {
      0.482215045525639, -0.3762733257504931}, {
      0.48990452745497126`, -0.37575871762008345`}, {
      0.49751183228694185`, -0.37518107433244785`}, {
      0.5050379554126825, -0.37454220601994875`}, {
      0.5124838683430867, -0.373843848453234}, {
      0.5198505192612735, -0.3730876671285524}, {
      0.5271388335797482, -0.37227526107919706`}, {
      0.5343497144988225, -0.3714081664329935}, {
      0.5414840435634382, -0.3704878597357643}, {
      0.5485426812160352, -0.36951576105891387`}, {
      0.5555264673435263, -0.36849323690767105`}, {
      0.5624362218167954, -0.36742160294507803`}, {
      0.569272745021448, -0.36630212654550776`}, {
      0.5760368183787927, -0.365136029190313}, {
      0.5827292048562602, -0.36392448871714217`}, {
      0.5893506494666461, -0.3626686414334916}, {
      0.5959018797557253, -0.3613695841041905}, {
      0.6023836062779185, -0.3600283758217185}, {
      0.6087965230598017, -0.3586460397675363}, {
      0.6151413080513447, -0.35722356487195495`}, {
      0.6214186235648466, -0.3557619073794714}, {
      0.6276291167015973, -0.3542619923259558}, {
      0.6337734197663523, -0.35272471493358026`}, {
      0.6398521506697523, -0.3511509419289266}, {
      0.6458659133188557, -0.34954151278929674`}, {
      0.6518152979959816, -0.34789724092187346`}, {
      0.6577008817260871, -0.34621891478003064`}, {
      0.6635232286329183, -0.3445072989207779}, {
      0.6692828902841916, -0.34276313500703187`}, {
      0.6749804060260702, -0.3409871427581409}, {
      0.6806163033072121, -0.3391800208518448}, {
      0.6861910979926656, -0.3373424477806253}, {
      0.6917052946678968, -0.33547508266519627`}, {
      0.6971593869332294, -0.33357856602769215`}, {
      0.7025538576889809, -0.33165352052693575`}, {
      0.7078891794115723, -0.32970055165800627`}, {
      0.7131658144208909, -0.32772024841817693`}, {
      0.7183842151391777, -0.3257131839411538}, {
      0.7235448243417072, -0.32367991610142016`}, {
      0.7286480753995229, -0.32162098809037126`}, {
      0.7336943925144858, -0.3195369289658152}, {
      0.7386841909468862, -0.3174282541763147}, {
      0.7436178772358647, -0.3152954660617495}, {
      0.7484958494128786, -0.3131390543313922}, {
      0.753318497208447, -0.3109594965207097}, {
      0.7580862022523979, -0.3087572584280258}, {
      0.7627993382678359, -0.30653279453211263`}, {
      0.7674582712590419, -0.3042865483917106}, {
      0.7720633596935085, -0.30201895302791876`}, {
      0.7766149546783094, -0.299730431290339}, {
      0.7811134001309944, -0.29742139620780583`}, {
      0.7855590329451939, -0.29509225132448413`}, {
      0.7899521831511132, -0.2927433910220713}, {
      0.7942931740710868, -0.29037520082879803`}, {
      0.7985823224703616, -0.2879880577158812}, {
      0.8028199387032681, -0.2855823303820457}, {
      0.807006326854934, -0.28315837952669687`}, {
      0.8111417848786925, -0.2807165581122924}, {
      0.8152266047293251, -0.27825721161643224`}, {
      0.819261072492282, -0.2757806782741561}, {
      0.8232454685090109, -0.27328728931091156`}, {
      0.827180067498525, -0.2707773691666307}, {
      0.8310651386753332, -0.26825123571132864`}, {
      0.8349009458638532, -0.265709200452617}, {
      0.8386877476094232, -0.26315156873550244`}, {
      0.8424257972860212, -0.26057863993482233`}, {
      0.8461153432008017, -0.2579907076406505}, {
      0.8497566286955516, -0.2553880598369892}, {
      0.8533498922451634, -0.2527709790740464}, {
      0.8568953675532246, -0.2501397426343833}, {
      0.8603932836448117, -0.24749462269320166`}, {
      0.8638438649565806, -0.24483588647302684`}, {
      0.867247331424237, -0.24216379639303104`}, {
      0.8706038985674704, -0.239478610213227}, {
      0.8739137775724295, -0.2367805811737531}, {
      0.8771771753718172, -0.23406995812945866`}, {
      0.8803942947226777, -0.23134698567998863`}, {
      0.883565334281947, -0.22861190429555758`}, {
      0.8866904886798345, -0.22586495043859306`}, {
      0.8897699485911026, -0.22310635668142056`}, {
      0.8928039008043068, -0.22033635182015385`}, {
      0.8957925282890581, -0.21755516098494698`}, {
      0.8987360102613654, -0.21476300574675694`}, {
      0.9016345222471166, -0.21196010422075906`}, {
      0.904488236143751, -0.20914667116655078`}, {
      0.907297320280177, -0.20632291808527348`}, {
      0.9100619394749856, -0.20348905331377581`}, {
      0.9127822550930086, -0.200645282115937}, {
      0.9154584251002682, -0.19779180677126265`}, {
      0.9180906041173643, -0.19492882666086156`}, {
      0.9206789434713432, -0.19205653835090633`}, {
      0.9232235912460889, -0.18917513567367691`}, {
      0.9257246923312796, -0.18628480980628162`}, {
      0.928182388469947, -0.18338574934714588`}, {
      0.9305968183046768, -0.18047814039035595`}, {
      0.9329681174224872, -0.1775621665979402}, {
      0.9352964183984209, -0.1746380092701679}, {
      0.9375818508378834, -0.17170584741394193`}, {
      0.9398245414177606, -0.16876585780935832`}, {
      0.942024613926349, -0.1658182150745033}, {
      0.9441821893021256, -0.16286309172855512`}, {
      0.9462973856713908, -0.15990065825325533`}, {
      0.9483703183848085, -0.15693108315281218`}, {
      0.950401100052874, -0.15395453301229525`}, {
      0.9523898405803339, -0.15097117255457956`}, {
      0.9543366471995843, -0.14798116469589379`}, {
      0.956241624503071, -0.1449846706000262}, {
      0.9581048744747164, -0.14198184973123948`}, {
      0.9599264965203939, -0.1389728599059433}, {
      0.9617065874974736, -0.1359578573431728}, {
      0.9634452417434594, -0.13293699671391823`}, {
      0.9651425511037365, -0.12991043118934983`}, {
      0.9667986049584522, -0.12687831248798104`}, {
      0.9684134902485451, -0.12384079092181051`}, {
      0.9699872915009429, -0.12079801544148276`}, {
      0.9715200908529458, -0.11775013368050584`}, {
      0.973011968075812, -0.11469729199856288`}, {
      0.974463000597562, -0.11163963552395334`}, {
      0.9758732635250156, -0.10857730819519851`}, {
      0.9772428296650791, -0.10551045280184469`}, {
      0.9785717695452948, -0.10243921102449666`}, {
      0.9798601514336673, -0.0993637234741125}, {
      0.9811080413577801, -0.09628412973059056}, {
      0.9823155031232146, -0.09320056838067778}, {
      0.9834825983312843, -0.09011317705522824}, {
      0.9846093863960949, -0.08702209246583958}, {
      0.9856959245609427, -0.0839274504408945}, {
      0.9867422679140609, -0.08082938596103338}, {
      0.9877484694037246, -0.07772803319408383}, {
      0.9887145798527234, -0.07462352552947188}, {
      0.9896406479722123, -0.07151599561213917}, {
      0.9905267203749493, -0.06840557537598974}, {
      0.9913728415879265, -0.06529239607688939}, {
      0.9921790540644054, -0.062176588325240416`}, {
      0.9929453981953623, -0.059058282118153387`}, {
      0.9936719123203512, -0.05593760687123771}, {
      0.9943586327377916, -0.052814691450032016`}, {
      0.995005593714687, -0.04968966420109497}, {
      0.9956128274957814, -0.046562652982776724`}, {
      0.9961803643121574, -0.043433785195690994`}, {
      0.9967082323892837, -0.04030318781290711}, {
      0.997196457954515, -0.03717098740988132}, {
      0.9976450652440498, -0.03403731019414624}, {
      0.9980540765093513, -0.030902282034776966`}, {
      0.9984235120230331, -0.02776602849165221}, {
      0.9987533900842157, -0.02462867484452863}, {
      0.9990437270233553, -0.021490346121946145`}, {
      0.9992945372065496, -0.018351167129981954`}, {
      0.9995058330393211, -0.015211262480870812`}, {
      0.9996776249698821, -0.012070756621508852`}, {
      0.9998099214918829, -0.008929773861858257}, {
      0.9999027291466438, -0.005788438403269838}, {
      0.9999560525248741, -0.002646874366740575}, {0.9999698942678773, 
      0.0004947941788769452}, {0.9999442550682451, 0.003636443188696306}, {
      0.9998791336700388, 0.006777948613633388}, {0.999774526868459, 
      0.009919186372240004}, {0.9996304295090047, 0.013060032322518211`}, {
      0.9994468344861184, 0.016200362233698418`}, {0.999223732741319, 
      0.019340051757964393`}, {0.9989611132608206, 0.022478976402108213`}, {
      0.9986589630726341, 0.02561701149909807}, {0.9983172672431531, 
      0.028754032179541842`}, {0.9979360088732186, 0.031889913343029125`}, {
      0.9975151690936631, 0.03502452962933432}, {0.9970547270603282, 
      0.03815775538946324}, {0.9965546599485555, 0.041289464656525485`}, {
      0.9960149429471467, 0.044419531116414585`}, {0.9954355492517867, 
      0.04754782807827786}, {0.9948164500579302, 0.05067422844475744}, {
      0.9941576145531428, 0.05379860468198394}, {0.9934590099088951, 
      0.05692082878930364}, {0.992720601271803, 0.06004077226872005}, {
      0.9919423517543091, 0.06315830609403018}, {0.9911242224248005, 
      0.06627330067963554}, {0.9902661722971541, 0.06938562584900748}, {
      0.9893681583197064, 0.07249515080278633}, {0.9884301353636373, 
      0.07560174408649271}, {0.9874520562107642, 0.07870527355782982}, {
      0.9864338715407343, 0.08180560635355416}, {0.9853755299176116, 
      0.0849026088558923}, {0.9842769777758467, 0.08799614665848025}, {
      0.9831381594056212, 0.09108608453180171}, {0.9819590169375587, 
      0.09417228638810077}, {0.98073949032679, 0.09725461524574391}, {
      0.9794795173363646, 0.10033293319300567`}, {0.9781790335199966, 
      0.10340710135125122`}, {0.9768379722041327, 0.10647697983748901`}, {
      0.9754562644693331, 0.10954242772626491`}, {0.9740338391309501, 
      0.11260330301086946`}, {0.9725706227190937, 0.11565946256382809`}, {
      0.97106653945787, 0.11871076209664369`}, {0.9695215112438786, 
      0.12175705611875999`}, {0.9679354576239542, 0.12479819789571284`}, {
      0.9663082957721378, 0.12783403940643562`}, {0.9646399404658621, 
      0.13086443129968406`}, {0.9629303040613336, 0.13388922284954402`}, {
      0.9611792964680957, 0.13690826190998523`}, {0.9593868251227549, 
      0.13992139486842198`}, {0.9575527949618508, 0.14292846659824096`}, {
      0.9556771083938528, 0.14592932041025475`}, {0.953759665270262, 
      0.1489237980030377}, {0.9518003628557989, 0.1519117394121001}, {
      0.9497990957976549, 0.1548929829578537}, {0.9477557560937859, 
      0.15786736519232134`}, {0.9456702330602258, 0.16083472084454029`}, {
      0.9435424132973945, 0.1637948827646077}, {0.9413721806553772, 
      0.16674768186631447`}, {0.9391594161981486, 0.16969294706831134`}, {
      0.936903998166716, 0.1726305052337495}, {0.9346058019411535, 
      0.17556018110833474`}, {0.9322647000014981, 0.17848179725673236`}, {
      0.9298805618874797, 0.18139517399725746`}, {0.9274532541570526, 
      0.18430012933478201`}, {0.9249826403436973, 0.18719647889178775`}, {
      0.9224685809124599, 0.19008403583749078`}, {0.9199109332146941, 
      0.19296261081496044`}, {0.917309551441471, 0.19583201186615184`}, {
      0.9146642865756187, 0.19869204435476795`}, {0.9119749863423549, 
      0.2015425108868634}, {0.9092414951584713, 0.20438321122909822`}, {
      0.90646365408003, 0.20721394222454587`}, {0.9036413007485282, 
      0.21003449770595514`}, {0.9007742693354869, 0.21284466840636176`}, {
      0.8978623904854188, 0.2156442418669398}, {0.8949054912571263, 
      0.21843300234197882`}, {0.8919033950632805, 0.22121073070086666`}, {
      0.8888559216082311, 0.2239772043269528}, {0.8857628868239915, 
      0.22673219701316089`}, {0.8826241028043464, 0.22947547885421263`}, {
      0.8794393777370232, 0.2322068161353196}, {0.8762085158338669, 
      0.2349259712171911}, {0.8729313172589596, 0.23763270241720033`}, {
      0.8696075780546165, 0.240326763886542}, {0.866237090065196, 
      0.2430079054832077}, {0.8628196408586511, 0.24567587264059518`}, {
      0.8593550136457535, 0.24833040623155972`}, {0.8558429871969135, 
      0.2509712424277053}, {0.852283335756521, 0.2535981125537032}, {
      0.8486758289547244, 0.25621074293641466`}, {0.845020231716567, 
      0.2588088547485831}, {0.841316304168393, 0.2613921638468481}, {
      0.837563801541433, 0.26396038060382127`}, {0.8337624740724771, 
      0.26651320973395015`}, {0.8299120669015385, 0.26905035011288136`}, {
      0.826012319966406, 0.27157149459001817`}, {0.822062967893983, 
      0.27407632979395247`}, {0.8180637398883036, 0.27656453593043223`}, {
      0.8140143596151136, 0.27903578657250744`}, {0.8099145450829001, 
      0.2814897484424771}, {0.8057640085202489, 0.2839260811852395}, {
      0.8015624562494029, 0.2863444371326246}, {0.7973095885558927, 
      0.28874446105826346`}, {0.7930050995541028, 0.29112578992252425`}, {
      0.7886486770486337, 0.29348805260701705`}, {0.7842400023913147, 
      0.2958308696381397}, {0.779778750333715, 0.29815385289910723`}, {
      0.7752645888749983, 0.30045660532987206`}, {0.7706971791049572, 
      0.30273872061430884`}, {0.7660761750420599, 0.30499978285399765`}, {
      0.7614012234663355, 0.3072393662279}, {0.7566719637469158, 
      0.3094570346371776}, {0.7518880276640486, 0.3116523413343578}, {
      0.7470490392253889, 0.3138248285359986}, {0.7421546144763677, 
      0.3159740270179531}, {0.7372043613044322, 0.31809945569227605`}, {
      0.7321978792369457, 0.32020062116475173`}, {0.7271347592325242, 
      0.322277017271957}, {0.7220145834655876, 0.3243281245967021}, {
      0.716836925103887, 0.3263534099606135}, {0.7116013480787736, 
      0.32835232589254154`}, {0.7063074068479582, 0.3303243100713855}, {
      0.7009546461505111, 0.33226878474183347`}, {0.6955426007538426, 
      0.3341851561014087}, {0.6900707951924014, 0.33607281365710445`}, {
      0.6845387434978192, 0.33793112954976656`}, {0.6789459489202284, 
      0.3397594578442525}, {0.673291903640477, 0.3415571337832542}, {
      0.6675760884729571, 0.3433234730025191}, {0.661797972558768, 
      0.34505777070503596`}, {0.655957013048929, 0.34675930079157435`}, {
      0.6500526547773625, 0.34842731494476886`}, {0.6440843299233702, 
      0.3500610416637283}, {0.6380514576633288, 0.3516596852459184}, {
      0.6319534438113443, 0.3532224247128158}, {0.6257896804486109, 
      0.35474841267555757`}, {0.6195595455412417, 0.3562367741365124}, {
      0.6132624025463531, 0.357686605222374}, {0.6068976000062151, 
      0.3590969718440239}, {0.6004644711303081, 0.3604669082780211}, {
      0.5939623333651688, 0.36179541566415246`}, {0.5873904879519531, 
      0.3630814604130147}, {0.5807482194717046, 0.36432397251708687`}, {
      0.5740347953783833, 0.36552184375819713`}, {0.567249465519797, 
      0.36667392580367225`}, {0.5603914616466726, 0.3677790281827855}, {
      0.5534599969102264, 0.3688359161343765}, {0.5464542653487301, 
      0.36984330831570056`}, {0.5393734413637363, 0.37079987436166384`}, {
      0.5322166791868219, 0.37170423228260635`}, {0.5249831123379383, 
      0.3725549456876971}, {0.5176718530767304, 0.3733505208197891}, {
      0.5102819918485041, 0.37408940338623614`}, {0.5028125967268984, 
      0.3747699751686801}, {0.4952627128557604, 0.37539055039315994`}, {
      0.4876313618932418, 0.3759493718400503}, {0.4799175414617445, 
      0.37644460667128776`}, {0.4721202246080624, 0.3768743419500566}, {
      0.46423835927891155`, 0.3772365798255557}, {0.4562708678180334, 
      0.37752923235262076`}, {0.4482166464922289, 0.3777501159127872}, {
      0.44007456505505965`, 0.37789694519981143`}, {0.43184346635857923`, 
      0.3779673267286637}, {0.4235221660253818, 0.37795875182250877`}, {
      0.4151094521955233, 0.37786858902713644`}, {0.4066040853655617, 
      0.37769407589660753`}, {0.3980047983401446, 0.37743231008746303`}, {
      0.38931029632035613`, 0.3770802396915931}, {0.3805192571575238, 
      0.3766346527296686}, {0.3716303318065371, 0.376092165717759}, {
      0.3626421450191041, 0.3754492112092446}, {0.35355329632499277`, 
      0.3747020242021962}, {0.344362361358422, 0.37384662728883267`}, {
      0.3350678935977037, 0.37287881440824233`}, {0.32566842659938294`, 
      0.3717941330459862}, {0.3161624768239555, 0.3705878647041794}, {
      0.30654854716935764`, 0.36925500344280865`}, {0.2968251313515562, 
      0.36779023226697416`}, {0.2869907192996288, 0.36618789710497834`}, {
      0.27704380376686033`, 0.36444197808818846`}, {0.2669828884010038, 
      0.36254605780478644`}, {0.25680649756775775`, 0.36049328615523246`}, {
      0.24651318828392563`, 0.3582763413868002}, {0.2361015646934846, 
      0.35588738682714177`}, {0.22557029561446035`, 0.35331802277175456`}, {
      0.21491813580161853`, 0.3505592329067338}, {0.20414395171529132`, 
      0.3476013245657233}, {0.19324675276749217`, 0.3444338620282202}, {
      0.18222572924219343`, 0.3410455919655365}, {0.17108029836921765`, 
      0.33742436003182363`}, {0.15981016038595386`, 0.3335570174830588}, {
      0.14841536686768328`, 0.3294293165914346}, {0.1368964041708241, 
      0.325025793514318}, {0.12525429554599904`, 0.3203296371893871}, {
      0.11349072638038428`, 0.31532254278252386`}, {0.1016081981730246, 
      0.30998454824703064`}, {0.08961021829740871, 0.30429385271548554`}, {
      0.07750153444221969, 0.29822661582195586`}, {0.06528842493918219, 
      0.2917567377692078}, {0.05297905909551118, 0.28485562120575586`}, {
      0.0405839452627232, 0.27749191805203777`}, {0.028116488794605387`, 
      0.2696312677493923}, {0.015593687319325466`, 0.2612360386458895}, {
      0.003036996776870927, 0.25226509232406563`}, {-0.00952659194504495, 
      0.24267360297055662`}, {-0.022063220193476195`, 
      0.23241298226486698`}, {-0.034530523926726356`, 
      0.22143098719931936`}, {-0.046875646317748924`, 
      0.20967212670496552`}, {-0.059032873142456324`, 
      0.1970785358520558}, {-0.07092090697662079, 
      0.18359155495998464`}, {-0.08243988440863168, 
      0.1691543311367274}, {-0.09346839927324727, 
      0.15371583510122716`}, {-0.10386105259246853`, 
      0.13723671590386985`}, {-0.11344741968846316`, 
      0.11969732041561101`}, {-0.12203376437457217`, 
      0.10110785659032342`}, {-0.12940917372768782`, 
      0.08151993530183246}, {-0.13535768391677722`, 
      0.06103753668158164}, {-0.13967693359131755`, 
      0.03982408151113801}, {-0.14220162873694484`, 
      0.018101534937959747`}, {-0.14282714374224378`, -0.003861544436569008}, \
{-0.14152656272167308`, -0.025772810094802465`}, {-0.138355330534266, \
-0.04734513339996614}, {-0.13344179362662134`, -0.06832421267740782}, \
{-0.12696707203784663`, -0.08850787356863307}, {-0.11914080252085898`, \
-0.10775395005579949`}, {-0.11017898181143485`, -0.12597793410534355`}, \
{-0.10028754831046739`, -0.14314408654948996`}, {-0.08965249731752763, \
-0.15925395765545733`}, {-0.07843549620336503, -0.17433517534560622`}, \
{-0.06677331233641555, -0.18843199454772264`}, {-0.05477947207471913, \
-0.20159805951654652`}, {-0.04254697609285978, -0.2138912398965471}, \
{-0.030151322228591272`, -0.225370158729427}, {-0.01765341829646639, \
-0.23609199099771053`}, {-0.005102187457454183, -0.24611116383414655`}, {
      0.007463200535492879, -0.2554786696592526}, {
      0.020011469009362612`, -0.2642417806627805}, {
      0.032517709151862875`, -0.27244401619827635`}, {
      0.04496214251703004, -0.2801252622159205}, {
      0.05732912318705378, -0.2873219759245827}, {
      0.0696063358490072, -0.2940674324889356}, {
      0.08178415218499402, -0.300391986560314}, {
      0.09385511430643978, -0.3063233320754858}, {
      0.10581351975094432`, -0.3118867507177136}, {
      0.11765508753550089`, -0.3171053439159365}, {
      0.12937668888897524`, -0.3220002460889633}, {
      0.14097612964080491`, -0.3265908185856855}, {
      0.15245197393135845`, -0.33089482480802035`}, {
      0.16380340104681107`, -0.3349285875826283}, {
      0.1750300888726072, -0.3387071301369845}, {
      0.18613211879448205`, -0.342244302144485}, {
      0.19710989792883052`, -0.3455528923032046}, {
      0.20796409539485658`, -0.3486447288497539}, {
      0.21869558999707914`, -0.3515307693128229}, {
      0.22930542720600866`, -0.3542211806989516}, {
      0.2397947837366229, -0.35672541118719764`}, {
      0.2501649383517152, -0.35905225429641024`}, {
      0.26041724777826336`, -0.3612099063825317}, {
      0.27055312683372423`, -0.3632060182256039}, {
      0.28057403202657727`, -0.36504774137765117`}, {
      0.2904814480301087, -0.3667417698633346}, {
      0.3002768765370804, -0.3682943777547961}, {
      0.30996182709084935`, -0.36971145307979936`}, {
      0.319537809559866, -0.37099852846738685`}, {
      0.32900632798056373`, -0.3721608088870499}, {
      0.3383688755410678, -0.37320319679511665`}, {
      0.3476269305169645, -0.37413031496500615`}, {
      0.35678195300222704`, -0.37494652724554717`}, {
      0.36583538230461043`, -0.3756559574631456}, {
      0.3747886348964555, -0.37626250665869626`}, {
      0.3836431028297275, -0.3767698688283271}, {
      0.3924001525389453, -0.37718154531794074`}, {
      0.40106112396797244`, -0.3775008580047348}, {
      0.4096273299669042, -0.3777309613841382}, {
      0.41810005591383875`, -0.37787485366763524`}, {
      0.4264805595234794, -0.377935386985533}, {
      0.43477007081050706`, -0.37791527677866366`}, {
      0.4429697921806909, -0.37781711045413297`}, {
      0.45108089862693296`, -0.377643355372374}, {
      0.4591045380110007, -0.3773963662258222}, {
      0.46704183141470096`, -0.3770783918633709}, {
      0.4748938735467835, -0.3766915816093061}, {
      0.4826617331940017, -0.37623799112056755`}, {
      0.49034645370657204`, -0.37571958782186676`}, {
      0.49794905350980767`, -0.37513825595434713`}, {
      0.5054705266350054, -0.37449580127004184`}, {
      0.512911843263769, -0.3737939554013212}, {
      0.5202739502808927, -0.37303437993178123`}, {
      0.5275577718317227, -0.37221867019257204`}, {
      0.534764209880598, -0.3713483588059641}, {
      0.541894144767545, -0.37042491899597585`}, {
      0.5489484357608917, -0.3694497676841102}, {
      0.5559279216038858, -0.36842426838665027`}, {
      0.5628334210537566, -0.36734973392852466`}, {
      0.5696657334119584, -0.3662274289874536}, {
      0.5764256390445959, -0.3650585724809147}, {
      0.5831138998922424, -0.363844339807406}, {
      0.5897312599685527, -0.3625858649525235}, {
      0.5962784458472237, -0.3612842424695017}, {
      0.6027561671369905, -0.3599405293430736}, {
      0.6091651169444559, -0.35855574674479246`}, {
      0.6155059723246443, -0.35713088168730395`}, {
      0.6217793947192509, -0.35566688858446444`}, {
      0.6279860303826195, -0.3541646907236615}, {
      0.6341265107955395, -0.3526251816561992}, {
      0.6402014530669956, -0.35104922651116005`}, {
      0.6462114603240394, -0.34943766323774683`}, {
      0.6521571220899841, -0.34779130378072914`}, {
      0.6580390146511449, -0.34611093519327635`}, {
      0.6638577014123676, -0.3443973206911438}, {
      0.669613733241601, -0.34265120065188914`}, {
      0.6753076488037808, -0.3408732935625305}, {
      0.6809399748842985, -0.3390642969188151}, {
      0.686511226702335, -0.33722488807904133`}, {
      0.6920219082143407, -0.3353557250751721}, {
      0.6974725124079447, -0.3334574473837874}, {
      0.7028635215865744, -0.33153067665924824`}, {
      0.708195407645066, -0.32957601743128223`}, {
      0.7134686323365428, -0.327594057769054}, {
      0.718683647530832, -0.3255853699136431}, {
      0.7238408954646905, -0.3235505108807273}, {
      0.7289408089841003, -0.3214900230351488}, {
      0.7339838117788905, -0.31940443463893475`}, {
      0.7389703186099378, -0.31729426037423947`}, {
      0.7439007355291881, -0.3151600018425836}, {
      0.7487754600927382, -0.31300214804167886`}, {
      0.7535948815672078, -0.3108211758210448}, {
      0.7583593811296268, -0.3086175503175503}, {
      0.7630693320610548, -0.3063917253719422}, {
      0.7677250999341444, -0.304144143927359}, {
      0.7723270427948513, -0.3018752384107661}, {
      0.7768755113384898, -0.2995854310981944}, {
      0.7813708490803234, -0.2972751344646108}, {
      0.785813392520877, -0.2949447515191995}, {
      0.7902034713061467, -0.2925946761267893}, {
      0.7945414083828806, -0.2902252933161173}, {
      0.7988275201490971, -0.28783697957558174`}, {
      0.8030621165999998, -0.28543010313709766`}, {
      0.8072455014694446, -0.28300502424863544`}, {
      0.8113779723671076, -0.28056209543598964`}, {
      0.815459820911498, -0.27810166175429435`}, {
      0.8194913328589544, -0.2756240610297735}, {
      0.8234727882287572, -0.2731296240921877}, {
      0.8274044614244884, -0.27061867499841363`}, {
      0.8312866213517591, -0.26809153124756874`}, {
      0.8351195315324278, -0.2655485039880724}, {
      0.8389034502154221, -0.26298989821701313`}, {
      0.842638630484276, -0.26041601297217215`}, {
      0.846325320361489, -0.2578271415170359}, {
      0.8499637629098106, -0.25522357151911224`}, {
      0.8535541963305483, -0.25260558522184884`}, {
      0.8570968540589957, -0.24997345961043738`}, {
      0.8605919648570718, -0.24732746657177276`}, {
      0.8640397529032603, -0.24466787304882262`}, {
      0.8674404378799345, -0.24199494118964998`}, {
      0.8707942350581496, -0.23930892849131996`}, {
      0.8741013553799819, -0.23661008793890975`}, {
      0.8773620055384909, -0.23389866813983065`}, {
      0.8805763880553775, -0.23117491345366065`}, {
      0.8837447013564103, -0.22843906411767675`}, {
      0.886867139844686, -0.22569135636826693`}, {
      0.8899438939717921, -0.22293202255839317`}, {
      0.892975150306933, -0.22016129127126924`}, {
      0.895961091604082, -0.2173793874304086}, {
      0.8989018968672171, -0.2145865324061916}, {
      0.9017977414136975, -0.21178294411909313`}, {
      0.9046487969358356, -0.20896883713970643`}, {
      0.9074552315607163, -0.2061444227856921}, {
      0.9102172099083157, -0.20330990921577557`}, {
      0.9129348931479662, -0.20046550152091117`}, {
      0.9156084390532168, -0.1976114018127251}, {
      0.9182380020551328, -0.19474780930934535`}, {
      0.9208237332940795, -0.19187492041872128`}, {
      0.9233657806700308, -0.18899292881953172`}, {
      0.9258642888914456, -0.1861020255397757}, {
      0.9283193995227483, -0.18320239903313626`}, {
      0.9307312510304542, -0.1802942352532038}, {
      0.9330999788279731, -0.17737771772564187`}, {
      0.9354257153191293, -0.17445302761837497`}, {
      0.9377085899404294, -0.1715203438098743}, {
      0.9399487292021123, -0.16857984295561482`}, {
      0.9421462567280114, -0.1656316995527735}, {
      0.9443012932942609, -0.16267608600323627`}, {
      0.9464139568668737, -0.15971317267497817`}, {
      0.9484843626382208, -0.15674312796187878`}, {
      0.9505126230624381, -0.15376611834203274`}, {
      0.9524988478897879, -0.1507823084346125}, {
      0.9544431441999991, -0.1477918610553387}, {
      0.9563456164346116, -0.14479493727061116`}, {
      0.9582063664283478, -0.1417916964503513}, {
      0.9600254934395338, -0.13878229631960579`}, {
      0.9618030941795925, -0.13576689300895808`}, {
      0.963539262841629, -0.13274564110379414`}, {
      0.9652340911281287, -0.1297186936924659}, {
      0.9668876682777882, -0.12668620241339512`}, {
      0.9685000810914964, -0.12364831750115872`}, {
      0.9700714139574862, -0.12060518783159485`}, {
      0.9716017488756703, -0.11755696096596821`}, {
      0.9730911654811827, -0.11450378319423128`}, {
      0.9745397410671375, -0.11144579957741733`}, {
      0.9759475506066235, -0.10838315398919966`}, {
      0.9773146667739482, -0.10531598915665044`}, {
      0.9786411599651448, -0.10224444670023157`}, {
      0.9799270983177581, -0.09916866717304888}, {
      0.9811725477299205, -0.09608879009940002}, {
      0.9823775718787308, -0.09300495401264554}, {
      0.9835422322379496, -0.08991729649243173}, {
      0.984666588095021, -0.08682595420129302}, {
      0.9857506965674326, -0.08373106292066089}, {
      0.9867946126184253, -0.08063275758630556}, {
      0.9877983890720601, -0.07753117232323595}, {
      0.9887620766276561, -0.07442644048008279}, {
      0.989685723873605, -0.07131869466298907}, {
      0.9905693773005723, -0.06820806676903146}, {
      0.9914130813140953, -0.0650946880191958}, {
      0.992216878246583, -0.06197868899092892}, {
      0.9929808083687273, -0.05886019965028914}, {
      0.9937049099003328, -0.055739349383716594`}, {
      0.994389219020571, -0.05261626702944468}, {
      0.9950337698776661, -0.04949108090857306}, {
      0.9956385945980184, -0.04636391885582262}, {
      0.9962037232947704, -0.043234908249992064`}, {
      0.9967291840758217, -0.040104176044135714`}, {
      0.9972150030512976, -0.036971848795481696`}, {
      0.9976612043404754, -0.03383805269510928}, {
      0.9980678100781731, -0.030702913597404114`}, {
      0.9984348404206046, -0.027566557049309458`}, {
      0.9987623135507047, -0.02442910831939174}, {
      0.9990502456829274, -0.021290692426738117`}, {
      0.9992986510675205, -0.018151434169703818`}, {
      0.9995075419942783, -0.015011458154526765`}, {
      0.9996769287957765, -0.011870888823826791`}, {
      0.9998068198500892, -0.008729850485006721}, {
      0.9998972215829911, -0.005588467338572393}, {
      0.9999481384696456, -0.0024468635063886546`}, {0.9999595730357799, 
      0.0006948369401117219}, {0.9999315258583484, 0.0038365099517493222`}, {
      0.9998639955656838, 0.006978031473432705}, {0.9997569788371368, 
      0.010119277415989517`}, {0.9996104704022035, 0.013260123627977244`}, {
      0.9994244630391413, 0.016400445867456762`}, {0.9991989475730704, 
      0.019540119773711746`}, {0.9989339128735613, 0.02267902083889696}, {
      0.9986293458517068, 0.025817024379598427`}, {0.9982852314566759, 
      0.028954005508288253`}, {0.9979015526717476, 0.032089839104656875`}, {
      0.9974782905098232, 0.03522439978680531}, {0.9970154240084126, 
      0.03835756188227984}, {0.9965129302240936, 0.0414891993989313}, {
      0.995970784226439, 0.04461918599558118}, {0.9953889590914086, 
      0.04774739495247613}, {0.9947674258942015, 0.05087369914151268}, {
      0.9941061537015659, 0.053997970996213236`}, {0.9934051095635585, 
      0.05712008248143463}, {0.9926642585047523, 0.06023990506278967}, {
      0.991883563514883, 0.06335730967576228}, {0.9910629855389329, 
      0.06647216669449604}, {0.9902024834666412, 0.06958434590023596}, {
      0.9893020141214384, 0.07269371644940242}, {0.9883615322487947, 
      0.07580014684127637}, {0.9873809905039773, 0.07890350488527378}, {
      0.9863603394392064, 0.08200365766778742}, {0.9852995274902048, 
      0.08510047151857296}, {0.9841985009621291, 0.08819381197665652}, {
      0.9830572040148771, 0.09128354375573933}, {0.9818755786477579, 
      0.09436953070907546}, {0.980653564683519, 0.0974516357937972}, {
      0.9793910997517167, 0.10052972103466246`}, {0.9780881192714203, 
      0.10360364748719744`}, {0.9767445564332391, 0.1066732752002076}, {
      0.9753603421806603, 0.10973846317762838`}, {0.9739354051906841, 
      0.11279906933968714`}, {0.9724696718537456, 0.11585495048334622`}, {
      0.970963066252907, 0.11890596224199643`}, {0.9694155101423099, 
      0.12195195904436916`}, {0.9678269229248696, 0.1249927940726345}, {
      0.9661972216291985, 0.1280283192196513}, {0.9645263208857423, 
      0.13105838504533418`}, {0.9628141329021122, 0.1340828407321016}, {
      0.9610605674375968, 0.13710153403936712`}, {0.9592655317768366, 
      0.14011431125703527`}, {0.95742893070264, 0.1431210171579622}, {
      0.9555506664679255, 0.14612149494933888`}, {0.9536306387667681, 
      0.14911558622295448`}, {0.9516687447045291, 0.15210313090429461`}, {
      0.9496648787670496, 0.15508396720042852`}, {0.9476189327888842, 
      0.1580579315466368}, {0.9455307959205523, 0.16102485855172996`}, {
      0.9434003545947842, 0.1639845809420056}, {0.9412274924917348, 
      0.16693692950379063`}, {0.9390120905031424, 0.16988173302451215`}, {
      0.9367540266954036, 0.1728188182322389}, {0.9344531762715378, 
      0.17574800973363264`}, {0.9321094115320119, 0.1786691299502461}, {
      0.9297226018343969, 0.18158199905310193`}, {0.9272926135518244, 
      0.18448643489548447`}, {0.9248193100302103, 0.18738225294387226`}, {
      0.9223025515442158, 0.19026926620693782`}, {0.9197421952519068, 
      0.19314728516253654`}, {0.9171380951480801, 0.19601611768260419`}, {
      0.9144901020162169, 0.19887556895587838`}, {0.9117980633790267, 
      0.20172544140835622`}, {0.9090618234475399, 0.20456553462139593`}, {
      0.9062812230687112, 0.2073956452473665}, {0.9034560996714867, 
      0.210215566922745}, {0.9005862872112942, 0.21302509017855645`}, {
      0.8976716161129081, 0.21582400234804677`}, {0.8947119132116422, 
      0.21861208747147398`}, {0.8917070016928205, 0.22138912619789725`}, {
      0.8886567010294744, 0.2241548956838384}, {0.8855608269182137, 
      0.22690916948868417`}, {0.8824191912132151, 0.22965171746669105`}, {
      0.8792316018582705, 0.2323823056554484}, {0.8759978628168373, 
      0.2351006961606482}, {0.8727177740000264, 0.23780664703700255`}, {
      0.8693911311924647, 0.24049991216514227`}, {0.8660177259759668, 
      0.2431802411243215}, {0.8625973456509436, 0.24584737906074466`}, {
      0.8591297731554789, 0.2485010665513231}, {0.8556147869819977, 
      0.2511410394626581}, {0.8520521610914495, 0.253767028805038}, {
      0.8484416648249261, 0.25637876058122483`}, {0.8447830628126297, 
      0.25897595562979486`}, {0.8410761148801065, 0.26155832946278545`}, {
      0.8373205759516537, 0.2641255920973868}, {0.8335161959508082, 
      0.26667744788140363`}, {0.8296627196978191, 0.26921359531219785`}, {
      0.8257598868040041, 0.2717337268488056}, {0.8218074315628839, 
      0.2742375287169081}, {0.8178050828379885, 0.276724680706316}, {
      0.8137525639472203, 0.27919485596060883`}, {0.809649592543659, 
      0.2816477207585513}, {0.8054958804926858, 0.28408293428688713`}, {
      0.801291133745302, 0.286500148404087}, {0.7970350522075097, 
      0.28889900739460567`}, {0.7927273296056211, 0.29127914771317426`}, {
      0.7883676533473544, 0.2936401977186294}, {0.7839557043785701, 
      0.2959817773967494}, {0.779491157035498, 0.29830349807153683`}, {
      0.7749736788922961, 0.3006049621043545}, {0.7704029306037796, 
      0.30288576258028466`}, {0.7657785657431522, 0.30514548298104344`}, {
      0.7611002306345633, 0.30738369684374217`}, {0.7563675641803126, 
      0.30959996740474294`}, {0.7515801976825125, 0.3117938472278085}, {
      0.7467377546590171, 0.313964877815697}, {0.741839850653416, 
      0.3161125892042977}, {0.7368860930388864, 0.318236499538346}, {
      0.7318760808156914, 0.3203361146276935}, {0.7268094044021022, 
      0.322410927483043}, {0.7216856454185182, 0.32446041782998464`}, {
      0.7165043764645521, 0.3264840516000948}, {0.7112651608888382, 
      0.3284812803977732}, {0.7059675525513174, 0.33045154094140594`}, {
      0.700611095577747, 0.3323942544773449}, {0.6951953241061752, 
      0.33430882616508917`}, {0.6897197620251152, 0.3361946444319427}, {
      0.6841839227031502, 0.33805108029529923`}, {0.6785873087096954, 
      0.33987748665057527`}, {0.6729294115266378, 0.34167319752266867`}, {
      0.6672097112505738, 0.34343752727866717`}, {0.6614276762853639, 
      0.3451697697993631}, {0.6555827630247193, 0.3468691976069504}, {
      0.6496744155245427, 0.34853506094608316`}, {0.6437020651647444, 
      0.3501665868152611}, {0.6376651303002623, 0.3517629779452757}, {
      0.6315630159010248, 0.35332341172119874`}, {0.6253951131806038, 
      0.35484703904411824`}, {0.6191607992133255, 0.35633298312852957`}, {
      0.6128594365396234, 0.35778033823096034`}, {0.606490372759447, 
      0.3591881683050525}, {0.6000529401135692, 0.3605555055779342}, {
      0.5935464550526777, 0.3618813490422871}, {0.5869702177941815, 
      0.36316466285804855`}, {0.5803235118667261, 0.3644043746571756}, {
      0.5736056036424765, 0.36559937374433593`}, {0.5668157418573144, 
      0.3667485091857747}, {0.5599531571191978, 0.3678505877779281}, {
      0.5530170614050464, 0.3689043718866085}, {0.5460066475466615, 
      0.3699085771467635}, {0.5389210887063548, 0.37086187001190696`}, {
      0.5317595378431583, 0.3717628651413174}, {0.5245211271707209, 
      0.3726101226119951}, {0.5172049676082701, 0.3734021449411455}, {
      0.509810148226344, 0.37413737390360063`}, {0.5023357356893725, 
      0.3748141871270892}, {0.494780773697639, 0.375430894446595}, {
      0.48714428243167457`, 0.3759857339971905}, {0.4794252580027566, 
      0.3764768680226656}, {0.47162267191390705`, 0.3769023783749723}, {
      0.463735470536641, 0.3772602616769368}, {0.45576257460971914`, 
      0.37754842411782275`}, {0.44770287876734205`, 0.3777646758481189}, {
      0.4395552511056176, 0.3779067249363305}, {0.4313185327977776, 
      0.37797217084651913`}, {0.42299153777056175`, 0.37795849739080883`}, {
      0.4145730524564825, 0.37786306510597967`}, {0.40606183563940107`, 
      0.37768310299753505`}, {0.3974566184140644, 0.37741569958815946`}, {
      0.38875610428407453`, 0.3770577932001779}, {0.37995896942730306`, 
      0.3766061613933695}, {0.37106386316317036`, 0.376057409470137}, {
      0.36206940866265913`, 0.3754079579494331}, {0.3529742039496353, 
      0.3746540288988153}, {0.34377682325127384`, 0.3737916310003276}, {
      0.3344758187664497, 0.37281654321035756`}, {0.32506972293425385`, 
      0.37172429685590386`}, {0.31555705130081374`, 0.37051015598950326`}, {
      0.30593630610194494`, 0.36916909580203633`}, {0.29620598070257154`, 
      0.36769577886634475`}, {0.2863645650622654, 0.3660845289545767}, {
      0.27641055243081164`, 0.36432930213789794`}, {0.26634244751986225`, 
      0.3624236548380686}, {0.2561587764482912, 0.3603607084557321}, {
      0.24585809882209655`, 0.35813311014938043`}, {0.2354390223874636, 
      0.3557329892811024}, {0.22490022079154517`, 0.3531519089796263}, {
      0.2142404551042155, 0.35038081219713235`}, {0.2034585999013638, 
      0.3474099615532644}, {0.19255367489365244`, 0.344228872167428}, {
      0.18152488331357775`, 0.34082623657906724`}, {0.1703716585602988, 
      0.3371898407462764}, {0.15909372096160312`, 0.3333064699983726}, {
      0.1476911469655036, 0.32916180370277426`}, {0.13616445364583044`, 
      0.32474029729909565`}, {0.12451470212941662`, 0.3200250502677458}, {
      0.11274362446857657`, 0.31499765855896783`}, {0.10085377964401478`, 
      0.30963805004665385`}, {0.08884874585566878, 0.3039243017446581}, {
      0.07673335812290111, 0.29783243791687747`}, {0.0645140025672469, 
      0.2913362089563371}, {0.052198981700477654`, 0.28440685220078016`}, {
      0.03979896870302374, 0.27701283799132115`}, {0.027327573151920928`, 
      0.2691196077133672}, {0.014802045981952186`, 0.26068931594719574`}, {
      0.002244157528181117, 0.25168059716621016`}, {-0.010318711122523545`, 
      0.242048390036963}, {-0.022852217587815624`, 
      0.23174387121122464`}, {-0.03531339980792854, 
      0.22071457806894912`}, {-0.04764866545638148, 
      0.20890483916169816`}, {-0.05979140351389535, 
      0.19625668497958865`}, {-0.07165923802346316, 
      0.1827114811648205}, {-0.0831510358404374, 
      0.16821260684135772`}, {-0.09414394440632078, 
      0.15270957472763425`}, {-0.10449100023021737`, 
      0.13616401440597192`}, {-0.11402022506377918`, 
      0.11855783270300982`}, {-0.12253656674199102`, 
      0.0999034963974944}, {-0.1298283695651242, 
      0.0802556097391327}, {-0.1356799121413795, 
      0.05972174667745657}, {-0.13989044348211904`, 
      0.038469141830307266`}, {-0.14229782434700886`, 
      0.016723166889795256`}, {-0.1428019217187347, -0.00524535404161881}, \
{-0.14138101394312377`, -0.02714352162191071}, {-0.13809554951804195`, \
-0.0486855162477392}, {-0.13307786718702375`, -0.06961985997195391}, \
{-0.1265116018101662, -0.08974804165138106}, {-0.1186073958523702, \
-0.1089316812619391}, {-0.10958102864403134`, -0.12708964648909477`}, \
{-0.09963741107218288, -0.1441888914369809}, {-0.08896108935164154, \
-0.16023292447325663`}, {-0.0777121513658263, -0.17525067631837427`}, \
{-0.06602583801172653, -0.1892871835884851}, {-0.05401429853423456, \
-0.20239648959711704`}, {-0.041769343245685084`, -0.21463659922341968`}, \
{-0.029365468945789116`, -0.22606609897807767`}, {-0.016862756481444648`, \
-0.23674202233698838`}, {-0.004309453588148547, -0.24671859615470676`}, {
      0.00825581776101871, -0.25604658454593815`}, {
      0.020802223348722074`, -0.2647730230824422}, {
      0.033305211025111574`, -0.2729411983128279}, {
      0.045745289668539656`, -0.28059077423933254`}, {
      0.05810704489478269, -0.2877580007035429}, {
      0.07037834812886323, -0.29447596169048107`}, {
      0.0825497218412889, -0.3007748371561244}, {
      0.09461383006375561, -0.30668216234390855`}, {
      0.10656506904057077`, -0.31222307532927757`}, {
      0.11839923779368251`, -0.3174205478878387}, {
      0.13011327245683868`, -0.3222955975310671}, {
      0.141705031543241, -0.32686748024348306`}, {
      0.15317312196158175`, -0.3311538644558614}, {
      0.16451675770177357`, -0.3351709873459408}, {
      0.17573564477801654`, -0.33893379483353725`}, {
      0.18682988733198824`, -0.3424560667374124}, {
      0.19779991083616552`, -0.3457505285560605}, {
      0.20864639915567165`, -0.34882895126855473`}, {
      0.2193702428735659, -0.35170224045332005`}, {
      0.2299724967961823, -0.35438051591013087`}, {
      0.24045434496101398`, -0.3568731828547807}, {
      0.25081707179242724`, -0.3591889956432412}, {
      0.2610620383078969, -0.36133611487633366`}, {
      0.2711906624833161, -0.3633221586387606}, {
      0.2812044030510607, -0.36515424853842415`}, {
      0.2911047461373416, -0.3668390511332475}, {
      0.3008931942525805, -0.3683828152627736}, {
      0.3105712572353168, -0.369791405739991}, {
      0.32014044482058823`, -0.37107033380438875`}, {
      0.32960226056107184`, -0.37222478468941633`}, {
      0.33895819687609113`, -0.3732596426155769}, {
      0.34820973104192443`, -0.37417951348363715`}, {
      0.3573583219683153, -0.3749887455102527}, {
      0.3664054076319843, -0.3756914480201324}, {
      0.37535240305930984`, -0.37629150858418037`}, {
      0.38420069876802276`, -0.3767926086714283}, {
      0.39295165959241307`, -0.3771982379636027}, {
      0.401606623828725, -0.37751170746452545`}, {
      0.4101669026475578, -0.37773616152192085`}, {
      0.41863377972855226`, -0.3778745888663405}, {
      0.4270085110797179, -0.37792983276059094`}, {
      0.43529232500968357`, -0.3779046003430654}}]}, 
   {RGBColor[1, 1, 0], DiskBox[{0, 0}, 0.03]}},
  AspectRatio->Automatic,
  Background->RGBColor[0.6, 0.6, 0.6],
  ImageSize->{324., Automatic}]], "Output",
 CellChangeTimes->{3.401193596371539*^9, 3.401194904993032*^9, 
  3.401276875876768*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["(B) Other Power Laws",
 FontSize->16]], "Section",
 CellChangeTimes->{3.401193334041584*^9}],

Cell[TextData[{
 "Hooke, a contemporary of Newton, suggested that gravity satisfied an \
inverse law ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "\[Proportional]", 
    RowBox[{"1", "/", "r"}]}], TraditionalForm]]],
 ".  We can test this using ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "=", "1"}], TraditionalForm]]],
 ".  Using ",
 Cell[BoxData[
  FormBox[
   RowBox[{"V0", "=", 
    RowBox[{"2", " ", "\[Pi]"}]}], TraditionalForm]]],
 " show that there is a circular orbit."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tFinal", "=", "2"}], ";", 
  RowBox[{"R0", "=", "1"}], ";", 
  RowBox[{"V0", "=", 
   RowBox[{"2", " ", "\[Pi]"}]}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", ".001"}], ";", 
  RowBox[{"p", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CalculateOrbit", ";", 
  RowBox[{"DisplayOrbit", "[", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Clear", "[", 
  RowBox[{
  "tFinal", ",", "R0", ",", "V0", ",", "iMaximum", ",", "t", ",", "r", ",", 
   "v"}], "]"}]}], "Input",
 CellChangeTimes->{{3.401193360004507*^9, 3.401193381354719*^9}, 
   3.401193500226789*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 1, 1], Thickness[0.0075], 
    LineBox[{{1, 0}, {0.9999802608561372, 0.006283143965477344}, {
      0.999921044203816, 0.01256603988318939}, {0.9998223523808084, 
      0.018848439715163347`}, {0.9996841892832987, 0.025130095443011036`}, {
      0.9995065603657294, 0.031410759077720236`}, {0.9992894726405859, 
      0.03769018266944487}, {0.9990329346781199, 0.04396811831729363}, {
      0.998736956606011, 0.05024431817911668}, {0.9984015501089666, 
      0.05651853448129006}, {0.9980267284282609, 0.06279051952849733}, {
      0.997612506361212, 0.06906002571350818}, {0.997158900260598, 
      0.07532680552695355}, {0.9966659280340109, 0.08159061156709684}, {
      0.9961336091431502, 0.08785119654960093}, {0.9955619646030542, 
      0.09410831331729055}, {0.9949510169812705, 0.10036171484990963`}, {
      0.9943007903969652, 0.10661115427387319`}, {0.9936113105199703, 
      0.11285638487201351`}, {0.992882604569771, 0.11909716009332012`}, {
      0.9921147013144302, 0.12533323356267323`}, {0.9913076310694539, 
      0.1315643590905701}, {0.9904614256965931, 0.1377902906828443}, {
      0.9895761186025872, 0.14401078255037705`}, {0.9886517447378442, 
      0.15022558911880063`}, {0.9876883405950617, 0.15643446503819317`}, {
      0.9866859442077855, 0.16263716519276467`}, {0.9856445951489087, 
      0.16883344471053371`}, {0.984564334529109, 0.1750230589729947}, {
      0.9834452049952259, 0.18120576362477486`}, {0.9822872507285773, 
      0.18738131458328103`}, {0.9810905174432147, 0.1935494680483356}, {
      0.9798550523841193, 0.1997099805118013}, {0.978580904325336, 
      0.2058626087671946}, {0.9772681235680485, 0.21200710991928692`}, {
      0.9759167619385932, 0.2181432413936939}, {0.9745268727864135, 
      0.22427076094645162`}, {0.9730985109819531, 0.23038942667358017`}, {
      0.9716317329144905, 0.23649899702063346`}, {0.970126596489912, 
      0.24259923079223542`}, {0.9685831611284266, 0.24868988716160204`}, {
      0.9670014877622195, 0.2547707256800487}, {0.9653816388330471, 
      0.26084150628648284`}, {0.9637236782897712, 0.26690198931688086`}, {
      0.9620276715858355, 0.27295193551374997`}, {0.9602936856766804, 
      0.2789911060355734}, {0.9585217890171006, 0.2850192624662398}, {
      0.9567120515585422, 0.29103616682445504`}, {0.9548645447463414, 
      0.2970415815731377}, {0.9529793415169037, 0.3030352696287964}, {
      0.9510565162948245, 0.3090169943708895}, {0.9490961449899511, 
      0.31498651965116653`}, {0.9470983049943861, 0.32094360980299097`}, {
      0.9450630751794316, 0.32688802965064373`}, {0.9429905358924758, 
      0.3328195445186077}, {0.9408807689538211, 0.33873792024083227`}, {
      0.9387338576534536, 0.3446429231699778}, {0.9365498867477554, 
      0.35053432018663966`}, {0.9343289424561583, 0.3564118787085513}, {
      0.93207111245774, 0.36227536669976623`}, {0.9297764858877628, 
      0.3681245526798185}, {0.9274451533341548, 0.37395920573286096`}, {
      0.9250772068339331, 0.3797790955167817}, {0.9226727398695712, 
      0.38558399227229717`}, {0.9202318473653076, 0.3913736668320231}, {
      0.9177546256833988, 0.39714789062952144`}, {0.9152411726203153, 
      0.40290643570832363`}, {0.9126915874028803, 0.40864907473093015`}, {
      0.9101059706843524, 0.4143755809877854}, {0.9074844245404525, 
      0.42008572840622765`}, {0.9048270524653336, 0.42577929155941424`}, {
      0.9021339593674949, 0.4314560456752208}, {0.8994052515656407, 
      0.4371157666451149}, {0.8966410367844826, 0.4427582310330037}, {
      0.8938414241504872, 0.4483832160840546}, {0.8910065241875675, 
      0.45399049973348937`}, {0.88813644881272, 0.4595798606153506}, {
      0.8852313113316062, 0.4651510780712413}, {0.8822912264340792, 
      0.47070393215903567`}, {0.8793163101896567, 0.4762382036615624}, {
      0.8763066800429381, 0.48175367409525865`}, {0.8732624548089682, 
      0.4872501257187958}, {0.8701837546685469, 0.49272734154167513`}, {
      0.867070701163484, 0.4981851053327945}, {0.8639234171918014, 
      0.5036232016289844}, {0.8607420270028815, 0.5090414157435149}, {
      0.8575266561925614, 0.51443953377457}, {0.854277431698175, 
      0.519817342613693}, {0.850994481793542, 0.5251746299541991}, {
      0.8476779360839032, 0.5305111842995572}, {0.8443279255008044, 
      0.5358267949717396}, {0.8409445822969273, 0.5411212521195384}, {
      0.8375280400408682, 0.5463943467268513}, {0.8340784336118655, 
      0.551645870620932}, {0.8305958991944743, 0.5568756164806091}, {
      0.8270805742731903, 0.562083377844471}, {0.8235325976270221, 
      0.5672689491190163}, {0.8199521093240129, 0.5724321255867698}, {
      0.8163392507157098, 0.5775727034143656}, {0.8126941644315845, 
      0.5826904796605931}, {0.8090169943734022, 0.587785252284409}, {
      0.8053078857095405, 0.5928568201529139}, {0.8015669848692583, 
      0.5979049830492922}, {0.7977944395369153, 0.6029295416807166}, {
      0.7939903986461417, 0.6079302976862156}, {0.7901550123739584, 
      0.612907053644505}, {0.7862884321348479, 0.617859613081781}, {
      0.7823908105747777, 0.622787780479477}, {0.7784623015651729, 
      0.6276913612819828}, {0.7745030601968428, 0.6325701619043244}, {
      0.770513242773857, 0.6374239897398071}, {0.766493006807376, 
      0.642252653167619}, {0.7624425110094317, 0.6470559615603961}, {
      0.7583619152866629, 0.6518337252917474}, {0.7542513807340014, 
      0.6565857557437418}, {0.7501110696283132, 0.6613118653143537}, {
      0.7459411454219913, 0.6660118674248697}, {0.7417417727365033, 
      0.6706855765272542}, {0.7375131173558924, 0.6753328081114744}, {
      0.7332553462202324, 0.6799533787127848}, {0.7289686274190371, 
      0.6845471059189697}, {0.7246531301846248, 0.6891138083775447}, {
      0.7203090248854371, 0.6936533058029161}, {0.7159364830193129, 
      0.6981654189834984}, {0.7115356772067181, 0.7026499697887892}, {
      0.7071067811839306, 0.7071067811764016}, {0.702649969796182, 
      0.7115356771990532}, {0.6981654189907547, 0.7159364830115125}, {
      0.6936533058100356, 0.7203090248775014}, {0.6891138083845272, 
      0.724653130176554}, {0.6845471059258148, 0.7289686274108313}, {
      0.6799533787194922, 0.733255346211892}, {0.6753328081180437, 
      0.7375131173474178}, {0.6706855765336851, 0.7417417727278945}, {
      0.6660118674311618, 0.7459411454132486}, {0.6613118653205066, 
      0.7501110696194369}, {0.6565857557497551, 0.7542513807249915}, {
      0.6518337252976208, 0.7583619152775196}, {0.6470559615661292, 
      0.7624425110001553}, {0.6422526531732115, 0.7664930067979665}, {
      0.6374239897452584, 0.7705132427643148}, {0.6325701619096342, 
      0.7745030601871677}, {0.6276913612871508, 0.7784623015553653}, {
      0.6227877804845027, 0.7823908105648376}, {0.6178596130866638, 
      0.7862884321247755}, {0.6129070536492447, 0.7901550123637536}, {
      0.6079302976908117, 0.7939903986358048}, {0.6029295416851687, 
      0.7977944395264461}, {0.5979049830535998, 0.801566984858657}, {
      0.5928568201570767, 0.8053078856988071}, {0.5877852522884266, 
      0.8090169943625367}, {0.5826904796644649, 0.812694164420587}, {
      0.5775727034180912, 0.8163392507045801}, {0.5724321255903487, 
      0.8199521093127512}, {0.567268949122448, 0.8235325976156284}, {
      0.5620833778477552, 0.8270805742616643}, {0.5568756164837452, 
      0.8305958991828161}, {0.5516458706239195, 0.834078433600075}, {
      0.5463943467296898, 0.8375280400289454}, {0.5411212521222275, 
      0.840944582284872}, {0.5358267949742787, 0.8443279254886167}, {
      0.530511184301946, 0.8476779360715828}, {0.525174629956437, 
      0.8509944817810888}, {0.5198173426157796, 0.8542774316855889}, {
      0.5144395337765049, 0.8575266561798421}, {0.5090414157452975, 
      0.860742026990029}, {0.5036232016306144, 0.8639234171788155}, {
      0.4981851053342711, 0.8670707011503643}, {0.49272734154299813`, 
      0.8701837546552934}, {0.4872501257199647, 0.8732624547955806}, {
      0.4817536740962729, 0.8763066800294161}, {0.47623820366242153`, 
      0.8793163101760001}, {0.4707039321597393, 0.8822912264202878}, {
      0.46515107807178896`, 0.8852313113176796}, {0.45957986061574185`, 
      0.888136448798658}, {0.4539904997337237, 0.8910065241733698}, {
      0.4483832160841316, 0.8938414241361533}, {0.4427582310329229, 
      0.8966410367700123}, {0.43711576664487584`, 0.8994052515510335}, {
      0.43145604567482304`, 0.9021339593527505}, {0.4257792915588574, 
      0.9048270524504517}, {0.4200857284055113, 0.9074844245254328}, {
      0.414375580986909, 0.9101059706691943}, {0.4086490747298934, 
      0.9126915873875834}, {0.4029064357071261, 0.9152411726048792}, {
      0.3971478906281627, 0.9177546256678231}, {0.3913736668305028, 
      0.9202318473495916}, {0.38558399227061485`, 0.9226727398537147}, {
      0.37977909551493694`, 0.9250772068179356}, {0.37395920573085345`, 
      0.9274451533180156}, {0.3681245526776478, 0.9297764858714815}, {
      0.36227536669743204`, 0.932071112441316}, {0.35641187870605323`, 
      0.9343289424395911}, {0.3505343201839774, 0.9365498867310444}, {
      0.344642923167151, 0.9387338576365983}, {0.33873792023784055`, 
      0.9408807689368209}, {0.33281954451545076`, 0.9429905358753302}, {
      0.3268880296473213, 0.9450630751621398}, {0.32094360979950265`, 
      0.9470983049769476}, {0.31498651964751206`, 0.9490961449723652}, {
      0.3090169943670686, 0.9510565162770904}, {0.30303526962480876`, 
      0.952979341499021}, {0.2970415815689831, 0.9548645447283094}, {
      0.29103616682013317`, 0.9567120515403602}, {0.2850192624617504, 
      0.9585217889987678}, {0.2789911060309163, 0.9602936856581962}, {
      0.2729519355089249, 0.9620276715671991}, {0.26690198931188763`, 
      0.963723678270982}, {0.26084150628132124`, 0.9653816388141042}, {
      0.2547707256747186, 0.9670014877431223}, {0.24868988715610316`, 
      0.9685831611091743}, {0.24259923078656764`, 0.9701265964705038}, {
      0.23649899701479662`, 0.9716317328949255}, {0.23038942666757414`, 
      0.9730985109622308}, {0.2242707609402763, 0.9745268727665328}, {
      0.2181432413873491, 0.9759167619185535}, {0.2120071099127726, 
      0.9772681235478488}, {0.20586260876051063`, 0.9785809043049755}, {
      0.19970998050494765`, 0.9798550523635972}, {0.19354946804131218`, 
      0.9810905174225302}, {0.18738131457608784`, 0.9822872507077294}, {
      0.18120576361741186`, 0.9834452049742138}, {0.17502305896546186`, 
      0.9845643345079318}, {0.16883344470283104`, 0.9856445951275656}, {
      0.16263716518489216`, 0.9866859441862756}, {0.15643446503015088`, 
      0.9876883405733841}, {0.1502255891105886, 0.9886517447159979}, {
      0.1440107825419953, 0.9895761185805713}, {0.13779029067429296`, 
      0.9904614256744066}, {0.13156435908184927`, 0.991307631047096}, {
      0.125333233553783, 0.9921147012918999}, {0.11909716008426066`, 
      0.9928826045470673}, {0.11285638486278493`, 0.9936113104970924}, {
      0.10661115426447566`, 0.994300790373912}, {0.10036171484034334`, 
      0.994951016958041}, {0.0941083133075557, 0.9955619645796474}, {
      0.08785119653969772, 0.9961336091195652}, {0.0815906115570255, 
      0.9966659280102467}, {0.07532680551671435, 0.9971589002366535}, {
      0.06906002570310138, 0.9976125063370864}, {0.06279051951792321, 
      0.9980267284039531}, {0.05651853447054893, 0.9984015500844756}, {
      0.05024431816820886, 0.9987369565813359}, {0.04396811830621945, 
      0.9990329346532596}, {0.0376901826582047, 0.9992894726155394}, {
      0.03141075906631446, 0.9995065603404957}, {0.02513009543144004, 
      0.9996841892578768}, {0.018848439703427557`, 0.9998223523551972}, {
      0.012566039871289241`, 0.9999210441780144}, {0.006283143953413292, 
      0.9999802608301442}, {-1.2227483782434323`*^-11, 
      0.9999999999738146}, {-0.006283143977867772, 
      0.999980260829758}, {-0.012566039895742257`, 
      0.9999210441772419}, {-0.01884843972787813, 
      0.9998223523540385}, {-0.025130095455887198`, 
      0.9996841892563318}, {-0.03141075909075722, 
      0.9995065603385644}, {-0.03769018268264209, 
      0.9992894726132219}, {-0.04396811833065049, 
      0.9990329346505559}, {-0.05024431819263255, 
      0.9987369565782458}, {-0.0565185344949643, 
      0.9984015500809992}, {-0.06279051954232928, 
      0.9980267284000903}, {-0.06906002572749716, 
      0.9976125063328373}, {-0.07532680554109884, 
      0.997158900232018}, {-0.08159061158139773, 
      0.9966659280052247}, {-0.08785119656405668, 
      0.9961336091141567}, {-0.09410831333190041, 
      0.9955619645738524}, {-0.1003617148646728, 
      0.9949510169518594}, {-0.10661115428878887`, 
      0.9943007903673436}, {-0.11285638488708087`, 
      0.9936113104901374}, {-0.11909716010853834`, 
      0.9928826045397257}, {-0.1253332335780414, 
      0.9921147012841716}, {-0.13156435910608738`, 
      0.9913076310389808}, {-0.1377902906985098, 
      0.9904614256659046}, {-0.14401078256618982`, 
      0.9895761185716822}, {-0.15022558913475972`, 
      0.9886517447067218}, {-0.15643446505429762`, 
      0.9876883405637209}, {-0.16263716520901347`, 
      0.9866859441762253}, {-0.16883344472692588`, 
      0.9856445951171281}, {-0.1750230589895292, 
      0.984564334497107}, {-0.18120576364145063`, 
      0.9834452049630017}, {-0.187381314600097, 
      0.9822872506961298}, {-0.19354946806529066`, 
      0.981090517410543}, {-0.19970998052889438`, 
      0.9798550523512224}, {-0.20586260878442456`, 
      0.978580904292213}, {-0.21200710993665264`, 
      0.9772681235346985}, {-0.21814324141119418`, 
      0.9759167619050154}, {-0.2242707609640853, 
      0.974526872752607}, {-0.23038942669134602`, 
      0.9730985109479169}, {-0.23649899703853022`, 
      0.9716317328802236}, {-0.24259923081026188`, 
      0.9701265964554138}, {-0.24868988717975693`, 
      0.968583161093696}, {-0.25477072569833076`, 
      0.9670014877272558}, {-0.26084150630489067`, 
      0.9653816387978493}, {-0.2669019893354132, 
      0.9637236782543387}, {-0.27295193553240543`, 
      0.9620276715501672}, {-0.2789911060543507, 
      0.9602936856407757}, {-0.2850192624851374, 
      0.9585217889809585}, {-0.29103616684347167`, 
      0.9567120515221621}, {-0.2970415815922719, 
      0.9548645447097224}, {-0.3030352696480467, 
      0.952979341480045}, {-0.3090169943902545, 
      0.9510565162577255}, {-0.3149865196706447, 
      0.9490961449526111}, {-0.32094360982258086`, 
      0.9470983049568042}, {-0.3268880296703438, 
      0.9450630751416073}, {-0.3328195445384164, 
      0.9429905358544083}, {-0.33873792026074806`, 
      0.9408807689155096}, {-0.3446429231899991, 
      0.9387338576148976}, {-0.3505343202067649, 
      0.9365498867089541}, {-0.35641187872877883`, 
      0.9343289424171112}, {-0.36227536672009447`, 
      0.9320711124184464}, {-0.3681245527002458, 
      0.9297764858482221}, {-0.3739592057533857, 
      0.9274451532943664}, {-0.37977909553740213`, 
      0.9250772067938965}, {-0.38558399229301166`, 
      0.9226727398292858}, {-0.39137366685283, 
      0.9202318473247728}, {-0.3971478906504189, 
      0.9177546256426142}, {-0.40290643572930995`, 
      0.9152411725792804}, {-0.4086490747520036, 
      0.9126915873615945}, {-0.4143755810089442, 
      0.9101059706428153}, {-0.4200857284274701, 
      0.9074844244986636}, {-0.42577929158073846`, 
      0.9048270524232924}, {-0.43145604569662493`, 
      0.9021339593252011}, {-0.4371157666665972, 
      0.8994052515230939}, {-0.4427582310545623, 
      0.8966410367416826}, {-0.44838321610568765`, 
      0.8938414241074336}, {-0.4539904997551949, 
      0.89100652414426}, {-0.4595798606371268, 
      0.8881364487691582}, {-0.46515107809308626`, 
      0.8852313112877896}, {-0.47070393218094747`, 
      0.8822912263900078}, {-0.4762382036835391, 
      0.8793163101453302}, {-0.48175367411729836`, 
      0.8763066799983562}, {-0.48725012574089654`, 
      0.8732624547641308}, {-0.49272734156383496`, 
      0.8701837546234537}, {-0.49818510535501137`, 
      0.8670707011181348}, {-0.5036232016512565, 
      0.8639234171461961}, {-0.50904141576584, 
      0.86074202695702}, {-0.5144395337969462, 
      0.8575266561464435}, {-0.5198173426361181, 
      0.8542774316518008}, {-0.5251746299766712, 
      0.8509944817469114}, {-0.5305111843220743, 
      0.8476779360370161}, {-0.5358267949942995, 
      0.8443279254536609}, {-0.5411212521421392, 
      0.8409445822495273}, {-0.5463943467494908, 
      0.8375280399932118}, {-0.5516458706436083, 
      0.8340784335639527}, {-0.5568756165033201, 
      0.8305958991463053}, {-0.5620833778672145, 
      0.8270805742247651}, {-0.5672689491417902, 
      0.823532597578341}, {-0.572432125609572, 
      0.8199521092750758}, {-0.577572703437194, 
      0.816339250666517}, {-0.5826904796834456, 
      0.8126941643821363}, {-0.5877852523072834, 
      0.8090169943236987}, {-0.592856820175808, 
      0.8053078856595819}, {-0.5979049830722039, 
      0.801566984819045}, {-0.6029295417036437, 
      0.7977944394864477}, {-0.607930297709156, 
      0.79399039859542}, {-0.6129070536674565, 
      0.7901550123229828}, {-0.6178596131047414, 
      0.786288432083619}, {-0.6227877805024442, 
      0.7823908105232957}, {-0.6276913613049545, 
      0.7784623015134384}, {-0.6325701619272985, 
      0.7745030601448561}, {-0.6374239897627815, 
      0.7705132427216188}, {-0.6422526531905914, 
      0.7664930067548866}, {-0.6470559615833643, 
      0.7624425109566918}, {-0.6518337253147093, 
      0.7583619152336729}, {-0.6565857557666951, 
      0.754251380680762}, {-0.6613118653372962, 
      0.750111069574825}, {-0.6660118674477993, 
      0.7459411453682548}, {-0.6706855765501686, 
      0.7417417726825193}, {-0.6753328081343714, 
      0.7375131173016615}, {-0.6799533787356621, 
      0.7332553461657553}, {-0.6845471059418251, 
      0.7289686273643146}, {-0.689113808400376, 
      0.7246531301296577}, {-0.693653305825721, 
      0.7203090248302262}, {-0.6981654190062748, 
      0.7159364829638589}, {-0.7026499698115349, 
      0.7115356771510218}, {-0.7071067811991144, 
      0.7071067811279931}, {-0.7115356772217308, 
      0.7026499697400042}, {-0.7159364830341528, 
      0.6981654189343374}, {-0.7203090249001022, 
      0.6936533057533797}, {-0.724653130199113, 
      0.6891138083276337}, {-0.7289686274333464, 
      0.6845471058686848}, {-0.7332553462343611, 
      0.6799533786621267}, {-0.7375131173698385, 
      0.6753328080604437}, {-0.7417417727502648, 
      0.6706855764758516}, {-0.7459411454355661, 
      0.666011867373096}, {-0.7501110696416994, 
      0.6613118652622096}, {-0.754251380747197, 
      0.6565857556912281}, {-0.7583619152996659, 
      0.6518337252388651}, {-0.7624425110222401, 
      0.6470559615071458}, {-0.7664930068199878, 
      0.6422526531140018}, {-0.7705132427862703, 
      0.6374239896858237}, {-0.7745030602090555, 
      0.6325701618499757}, {-0.7784623015771831, 
      0.6276913612272698}, {-0.7823908105865832, 
      0.6227877804244005}, {-0.786288432146447, 
      0.617859613026342}, {-0.7901550123853488, 
      0.6129070535887045}, {-0.7939903986573216, 
      0.6079302976300546}, {-0.7977944395478824, 
      0.602929541624196}, {-0.8015669848800107, 
      0.5979049829924131}, {-0.805307885720076, 
      0.5928568200956774}, {-0.809016994383719, 
      0.5877852522268162}, {-0.8126941644416804, 
      0.582690479602645}, {-0.8163392507255828, 
      0.5775727033560634}, {-0.8199521093336609, 
      0.5724321255281146}, {-0.8235325976364433, 
      0.5672689490600092}, {-0.8270805742823824, 
      0.5620833777851134}, {-0.8305958992034352, 
      0.556875616420902}, {-0.8340784336205933, 
      0.5516458705608766}, {-0.8375280400493608, 
      0.5463943466664489}, {-0.8409445823051825, 
      0.5411212520587905}, {-0.8443279255088204, 
      0.5358267949106472}, {-0.8476779360916779, 
      0.5305111842381219}, {-0.8509944818010733, 
      0.5251746298924221}, {-0.8542774317054609, 
      0.5198173425515757}, {-0.8575266561995997, 
      0.5144395337121139}, {-0.8607420270096702, 
      0.5090414156807214}, {-0.8639234171983385, 
      0.5036232015658548}, {-0.8670707011697674, 
      0.4981851052693302}, {-0.8701837546745746, 
      0.4927273414778777}, {-0.8732624548147383, 
      0.48725012565466674`}, {-0.8763066800484484, 
      0.4817536740307995}, {-0.8793163101949052, 
      0.47623820359677466`}, {-0.8822912264390639, 
      0.47070393209392103`}, {-0.8852313113363249, 
      0.4651510780058013}, {-0.8881364488171709, 
      0.4595798605495869}, {-0.8910065241917484, 
      0.45399049966740357`}, {-0.893841424154396, 
      0.44838321601764847`}, {-0.8966410367881175, 
      0.44275823096627887`}, {-0.8994052515689995, 
      0.4371157665780731}, {-0.9021339593705757, 
      0.43145604560786377`}, {-0.9048270524681344, 
      0.4257792914917438}, {-0.9074844245429715, 
      0.4200857283382456}, {-0.9101059706865875, 
      0.4143755809194935}, {-0.9126915874048295, 
      0.4086490746623303}, {-0.9152411726219767, 
      0.4029064356394177}, {-0.9177546256847703, 
      0.3971478905603113}, {-0.9202318473663872, 
      0.39137366676251073`}, {-0.922672739870357, 
      0.3855839922024844}, {-0.9250772068344233, 
      0.37977909544667054`}, {-0.9274451533343473, 
      0.37395920566245344`}, {-0.9297764858876558, 
      0.3681245526091166}, {-0.9320711124573314, 
      0.362275366628772}, {-0.9343289424554463, 
      0.3564118786372668}, {-0.9365498867467381, 
      0.350534320115067}, {-0.9387338576521291, 
      0.3446429230981191}, {-0.9408807689521875, 
      0.33873792016868964`}, {-0.9429905358905313, 
      0.33281954444618334`}, {-0.9450630751771742, 
      0.32688802957793983`}, {-0.947098304991814, 
      0.32094360973000974`}, {-0.9490961449870625, 
      0.3149865195779102}, {-0.9510565162916175, 
      0.3090169942973603}, {-0.9529793415133766, 
      0.3030352695549966}, {-0.9548645447424925, 
      0.2970415814990697}, {-0.9567120515543696, 
      0.2910361667501211}, {-0.9585217890126024, 
      0.28501926239164227`}, {-0.960293685671855, 
      0.2789911059607147}, {-0.9620276715806811, 
      0.27295193543863244`}, {-0.9637236782842862, 
      0.266901989241507}, {-0.9653816388272295, 
      0.26084150621085506`}, {-0.9670014877560679, 
      0.2547707256041695}, {-0.9685831611219391, 
      0.2486898870854739}, {-0.970126596483087, 
      0.24259923071586087`}, {-0.9716317329073263, 
      0.23649899694401502`}, {-0.9730985109744481, 
      0.23038942659672043`}, {-0.974526872778566, 
      0.22427076086935316`}, {-0.9759167619304017, 
      0.21814324131635932`}, {-0.9772681235595112, 
      0.21200710984171886`}, {-0.9785809043164515, 
      0.2058626086893957}, {-0.979855052374886, 
      0.19970998043377428`}, {-0.9810905174336312, 
      0.19354946797008313`}, {-0.9822872507186419, 
      0.18738131450480588`}, {-0.9834452049849371, 
      0.18120576354607976`}, {-0.9845643345184654, 
      0.1750230588940824}, {-0.9856445951379088, 
      0.168833444631407}, {-0.9866859441964279, 
      0.16263716511342635`}, {-0.9876883405833449, 
      0.1564344649586461}, {-0.9886517447257668, 
      0.15022558903904767`}, {-0.9895761185901477, 
      0.14401078247042107`}, {-0.9904614256837903, 
      0.13779029060268816`}, {-0.9913076310562863, 
      0.13156435901021674`}, {-0.9921147013008966, 
      0.12533323348212558`}, {-0.99288260455587, 
      0.11909716001258117`}, {-0.9936113105057006, 
      0.11285638479108621`}, {-0.9943007903823255, 
      0.10661115419276056`}, {-0.9949510169662596, 
      0.10036171476861469`}, {-0.9955619645876709, 
      0.09410831323581632}, {-0.9961336091273932, 
      0.08785119646795046}, {-0.9966659280178791, 
      0.08159061148527322}, {-0.9971589002440903, 
      0.07532680544495987}, {-0.9976125063443272, 
      0.06906002563134755}, {-0.9980267284109978, 
      0.06279051944617288}, {-0.9984015500913243, 
      0.05651853439880494}, {-0.9987369565879883, 
      0.05024431809647406}, {-0.9990329346597159, 
      0.043968118234496686`}, {-0.9992894726217995, 
      0.03769018258649681}, {-0.9995065603465596, 
      0.03141075899462428}, {-0.9996841892637447, 
      0.025130095359770423`}, {-0.9998223523608691, 
      0.01884843963178133}, {-0.9999210441834905, 
      0.012566039799669245`}, {-0.9999802608354246, 
      0.006283143881822355}, {-0.9999999999788994, -8.378653066493102*^-11}, \
{-0.9999802608346475, -0.006283144049392103}, {-0.9999210441819364, \
-0.012566039967229055`}, {-0.9998223523585381, -0.018848439799324577`}, \
{-0.9996841892606368, -0.02513009552729048}, {-0.9995065603426753, \
-0.03141075916211453}, {-0.9992894726171389, -0.03769018275395063}, \
{-0.9990329346542792, -0.04396811840190745}, {-0.9987369565817761, \
-0.050244318263835165`}, {-0.9984015500843367, -0.05651853456610978}, \
{-0.9980267284032356, -0.06279051961341485}, {-0.9976125063357907, \
-0.06906002579852005}, {-0.9971589002347802, -0.0753268056120563}, \
{-0.9966659280077961, -0.081590611652287}, {-0.9961336091165378, \
-0.087851196634875}, {-0.9955619645760438, -0.09410831340264506}, \
{-0.9949510169538618, -0.10036171493534105`}, {-0.9943007903691576, \
-0.10661115435937799`}, {-0.9936113104917637, -0.11285638495758817`}, \
{-0.9928826045411648, -0.1190971601789611}, {-0.9921147012854243, \
-0.12533323364837695`}, {-0.9913076310400479, -0.13156435917633302`}, \
{-0.9904614256667869, -0.13779029076866287`}, {-0.9895761185723805, \
-0.14401078263624767`}, {-0.9886517447072369, -0.15022558920471973`}, \
{-0.9876883405640536, -0.15643446512415712`}, {-0.9866859441763766, \
-0.16263716527876987`}, {-0.9856445951170987, -0.16883344479657658`}, \
{-0.9845643344968978, -0.1750230590590716}, {-0.9834452049626137, \
-0.18120576371088215`}, {-0.982287250695564, -0.18738131466941507`}, \
{-0.9810905174098005, -0.19354946813449275`}, {-0.979855052350304, \
-0.19970998059797793`}, {-0.9785809042911199, -0.20586260885338703`}, \
{-0.9772681235334317, -0.21200711000549152`}, {-0.975916761903576, \
-0.218143241479907}, {-0.9745268727509959, -0.22427076103266955`}, \
{-0.9730985109461354, -0.23038942675979926`}, {-0.9716317328782729, \
-0.23649899710685002`}, {-0.9701265964532948, -0.2425992308784458}, \
{-0.9685831610914102, -0.24868988724780253`}, {-0.9670014877248042, \
-0.25477072576623566`}, {-0.9653816387952332, -0.2608415063726525}, \
{-0.9637236782515594, -0.26690198940302956`}, {-0.962027671547226, \
-0.272951935599874}, {-0.9602936856376738, -0.27899110612166916`}, \
{-0.9585217889776975, -0.28501926255230353`}, {-0.9567120515187431, \
-0.29103616691048306`}, {-0.9548645447061468, -0.2970415816591263}, \
{-0.9529793414763141, -0.3030352697147419}, {-0.9510565162538407, \
-0.30901699445678815`}, {-0.9490961449485739, -0.3149865197370147}, \
{-0.9470983049526162, -0.3209436098887849}, {-0.9450630751372696, \
-0.32688802973637976`}, {-0.9429905358499227, -0.33281954460428215`}, \
{-0.9408807689108775, -0.33873792032644146`}, {-0.9387338576101204, \
-0.34464292325551804`}, {-0.9365498867040335, -0.35053432027210724`}, \
{-0.9343289424120486, -0.35641187879394254`}, {-0.9320711124132434, \
-0.3622753667850775}, {-0.9297764858428803, -0.3681245527650461}, \
{-0.9274451532888874, -0.37395920581800124`}, {-0.925077206788282, \
-0.379779095601831}, {-0.9226727398235373, -0.3855839923572518}, \
{-0.920231847318892, -0.3913736669168795}, {-0.9177546256366027, \
-0.3971478907142759}, {-0.9152411725731399, -0.40290643579297253`}, \
{-0.9126915873553266, -0.40864907481546986`}, {-0.9101059706364218, \
-0.4143755810722123}, {-0.9074844244921464, -0.42008572849053816`}, \
{-0.9048270524166531, -0.4257792916436047}, {-0.9021339593184414, \
-0.43145604575928753`}, {-0.8994052515162156, -0.4371157667290544}, \
{-0.8966410367346874, -0.4427582311168124}, {-0.8938414241003233, \
-0.4483832161677289}, {-0.8910065241370364, -0.45399049981702566`}, \
{-0.8881364487618231, -0.4595798606987454}, {-0.8852313112803449, \
-0.465151078154491}, {-0.8822912263824553, -0.4707039322421368}, \
{-0.8793163101376716, -0.4762382037445114}, {-0.8763066799905935, \
-0.4817536741780521}, {-0.8732624547562658, -0.4872501258014301}, \
{-0.8701837546154884, -0.4927273416241469}, {-0.867070701110071, \
-0.49818510541510014`}, {-0.8639234171380358, -0.5036232017111206}, \
{-0.8607420269487649, -0.5090414158254781}, {-0.8575266561380956, \
-0.5144395338563569}, {-0.854277431643362, -0.5198173426953}, \
{-0.8509944817383835, -0.5251746300356228}, {-0.8476779360284012, \
-0.5305111843807944}, {-0.8443279254449608, -0.5358267950527867}, \
{-0.840944582240744, -0.5411212522003923}, {-0.8375280399843473, \
-0.5463943468075085}, {-0.834078433555009, -0.5516458707013893}, \
{-0.8305958991372842, -0.5568756165608632}, {-0.8270805742156686, \
-0.5620833779245186}, {-0.8235325975691711, -0.5672689491988538}, \
{-0.8199521092658345, -0.5724321256663943}, {-0.8163392506572063, \
-0.5775727034937738}, {-0.8126941643727581, -0.5826904797397816}, \
{-0.8090169943142551, -0.5877852523633748}, {-0.8053078856500748, \
-0.5928568202316536}, {-0.8015669848094764, -0.5979049831278028}, \
{-0.7977944394768196, -0.6029295417589949}, {-0.7939903985857345, \
-0.6079302977642586}, {-0.7901550123132419, -0.6129070537223094}, \
{-0.7862884320738246, -0.6178596131593437}, {-0.7823908105134498, \
-0.6227877805567951}, {-0.7784623015035431, -0.6276913613590531}, \
{-0.7745030601349133, -0.632570161981144}, {-0.7705132427116305, \
-0.637423989816373}, {-0.7664930067448548, -0.6422526532439283}, \
{-0.7624425109466185, -0.6470559616364457}, {-0.7583619152235602, \
-0.6518337253675345}, {-0.7542513806706119, -0.6565857558192634}, \
{-0.7501110695646395, -0.661311865389607}, {-0.7459411453580359, \
-0.6660118674998518}, {-0.7417417726722689, -0.6706855766019622}, \
{-0.7375131172913817, -0.6753328081859055}, {-0.733255346155448, \
-0.6799533787869361}, {-0.7289686273539818, -0.6845471059928384}, \
{-0.7246531301193014, -0.6891138084511281}, {-0.7203090248198484, \
-0.6936533058762115}, {-0.7159364829534616, -0.6981654190565032}, \
{-0.711535677140607, -0.7026499698615007}, {-0.7071067811175625, \
-0.7071067812488171}, {-0.70264996972956, -0.7115356772711701}, \
{-0.6981654189238815, -0.7159364830833281}, {-0.6936533057429142, \
-0.7203090249490133}, {-0.6891138083171604, -0.7246531302477596}, \
{-0.6845471058582057, -0.7289686274817282}, {-0.6799533786516436, \
-0.7332553462824776}, {-0.6753328080499587, -0.7375131174176895}, \
{-0.6706855764653665, -0.7417417727978501}, {-0.6660118673626128, \
-0.7459411454828856}, {-0.6613118652517301, -0.7501110696887527}, \
{-0.6565857556807543, -0.7542513807939841}, {-0.6518337252283987, \
-0.7583619153461866}, {-0.6470559614966889, -0.7624425110684943}, \
{-0.6422526531035561, -0.7664930068659752}, {-0.637423989675391, \
-0.770513242831991}, {-0.632570161839558, -0.7745030602545093}, \
{-0.6276913612168689, -0.7784623016223701}, {-0.6227877804140183, \
-0.7823908106315033}, {-0.6178596130159801, -0.7862884321911}, \
{-0.612907053578365, -0.7901550124297348}, {-0.6079302976197392, \
-0.7939903987014407}, {-0.6029295416139064, -0.7977944395917348}, \
{-0.5979049829821512, -0.8015669849235962}, {-0.5928568200854449, \
-0.8053078857633951}, {-0.5877852522166149, -0.8090169944267716}, \
{-0.5826904795924767, -0.8126941644844666}, {-0.5775727033459297, \
-0.8163392507681028}, {-0.5724321255180173, -0.8199521093759149}, \
{-0.56726894904995, -0.8235325976784315}, {-0.5620833777750939, \
-0.8270805743241049}, {-0.5568756164109239, -0.8305958992448924}, \
{-0.5516458705509415, -0.8340784336617854}, {-0.5463943466565586, \
-0.837528040090288}, {-0.5411212520489465, -0.8409445823458453}, \
{-0.5358267949008512, -0.844327925549219}, {-0.5305111842283754, \
-0.8476779361318125}, {-0.5251746298827268, -0.8509944818409444}, \
{-0.5198173425419331, -0.8542774317450688}, {-0.5144395337025256, \
-0.8575266562389448}, {-0.5090414156711889, -0.8607420270487529}, \
{-0.5036232015563797, -0.8639234172371593}, {-0.49818510525991405`, \
-0.8670707012083266}, {-0.49272734146852204`, -0.8701837547128728}, \
{-0.487250125645373, -0.8732624548527758}, {-0.48175367402156916`, \
-0.8763066800862256}, {-0.47623820358760915`, -0.8793163102324227}, \
{-0.4707039320848218, -0.8822912264763222}, {-0.4651510779967698, \
-0.8852313113733247}, {-0.45957986054062455`, -0.8881364488539125}, \
{-0.45399049965851174`, -0.8910065242282326}, {-0.44838321600882847`, \
-0.8938414241906233}, {-0.4427582309575321, -0.8966410368240882}, \
{-0.4371157665694009, -0.8994052516047144}, {-0.4314560455992674, \
-0.9021339594060354}, {-0.42577929148322463`, -0.9048270525033395}, \
{-0.4200857283298049, -0.9074844245779226}, {-0.4143755809111325, \
-0.9101059707212852}, {-0.4086490746540503, -0.9126915874392745}, \
{-0.4029064356312199, -0.9152411726561697}, {-0.39714789055219696`, \
-0.9177546257187119}, {-0.391373666754481, -0.9202318474000781}, \
{-0.3855839921945405, -0.922672739903798}, {-0.37977909543881355`, \
-0.9250772068676149}, {-0.37395920565468455`, -0.9274451533672903}, \
{-0.3681245526014369, -0.9297764859203509}, {-0.36227536662118265`, \
-0.9320711124897795}, {-0.35641187862976886`, -0.934328942487648}, \
{-0.35053432010766156`, -0.9365498867786942}, {-0.3446429230908072, \
-0.9387338576838403}, {-0.3387379201614723, -0.9408807689836547}, \
{-0.3328195444390616, -0.9429905359217552}, {-0.3268880295709146, \
-0.9450630752081556}, {-0.32094360972308206`, -0.9470983050225537}, \
{-0.31498651957108104`, -0.9490961450175612}, {-0.30901699429063056`, \
-0.951056516321876}, {-0.30303526954836724`, -0.9529793415433958}, \
{-0.29704158149254156`, -0.9548645447722731}, {-0.29103616674369515`, \
-0.9567120515839125}, {-0.2850192623853193, -0.9585217890419084}, \
{-0.2789911059544956, -0.960293685700925}, {-0.272951935432518, \
-0.9620276716095159}, {-0.266901989235498, -0.9637236783128865}, \
{-0.26084150620495233`, -0.9653816388555964}, {-0.2547707255983738, \
-0.967001487784202}, {-0.24868988707978595`, -0.9685831611498414}, \
{-0.2425992307102814, -0.9701265965107584}, {-0.23649899693854473`, \
-0.9716317329347676}, {-0.23038942659136002`, -0.9730985110016602}, \
{-0.2242707608641033, -0.9745268728055497}, {-0.21814324131122068`, \
-0.9759167619571578}, {-0.21200710983669208`, -0.9772681235860408}, \
{-0.20586260868448136`, -0.9785809043427552}, {-0.19970998042897298`, \
-0.9798550524009648}, {-0.19354946796539546`, -0.981090517459486}, \
{-0.18738131450023238`, -0.9822872507442735}, {-0.181205763541621, \
-0.9834452050103465}, {-0.17502305888973888`, -0.9845643345436533}, \
{-0.16883344462717922`, -0.9856445951628763}, {-0.1626371651093148, \
-0.9866859442211757}, {-0.15643446495465127`, -0.9876883406078739}, \
{-0.15022558903516997`, -0.9886517447500779}, {-0.14401078246666094`, \
-0.9895761186142418}, {-0.13779029059904604`, -0.9904614257076682}, \
{-0.13156435900669303`, -0.9913076310799489}, {-0.12533323347872066`, \
-0.9921147013243449}, {-0.11909716000929538`, -0.9928826045791046}, \
{-0.1128563847879199, -0.9936113105287225}, {-0.10661115418971404`, \
-0.9943007904051355}, {-0.10036171476568827`, -0.9949510169888587}, \
{-0.0941083132330103, -0.9955619646100599}, {-0.08785119646526512, \
-0.9961336091495729}, {-0.08159061148270881, -0.9966659280398504}, \
{-0.07532680544251663, -0.997158900265854}, {-0.06906002562902572, \
-0.9976125063658842}, {-0.06279051944397265, -0.9980267284323489}, \
{-0.056518534396726515`, -0.9984015501124702}, {-0.05024431809451761, \
-0.99873695660893}, {-0.043968118232662376`, -0.9990329346804542}, \
{-0.03769018258478478, -0.9992894726423351}, {-0.03141075899303466, \
-0.9995065603668934}, {-0.02513009535830332, -0.9996841892838775}, \
{-0.018848439630436843`, -0.9998223523808016}, {-0.012566039798447455`, \
-0.9999210442032235}, {-0.006283143880723332, -0.9999802608549588}, {
      8.476273415797264*^-11, -0.9999999999982357}, {
      0.0062831440502454485`, -0.9999802608537867}, {
      0.012566039967959517`, -0.9999210442008791}, {
      0.01884843979993215, -0.9998223523772851}, {
      0.025130095527775162`, -0.999684189279189}, {
      0.03141075916247634, -0.9995065603610332}, {
      0.0376901827541896, -0.9992894726353033}, {
      0.04396811840202364, -0.999032934672251}, {
      0.05024431826382861, -0.9987369565995557}, {
      0.05651853456598055, -0.9984015501019251}, {
      0.06279051961316302, -0.9980267284206332}, {
      0.06906002579814573, -0.9976125063529984}, {
      0.07532680561155959, -0.9971589002517984}, {
      0.08159061165166802, -0.9966659280246256}, {
      0.0878511966341339, -0.9961336091331794}, {
      0.09410831340178197, -0.995561964592498}, {
      0.10036171493435612`, -0.9949510169701292}, {
      0.1066111543582714, -0.994300790385239}, {
      0.11285638495636009`, -0.9936113105076595}, {
      0.11909716017761172`, -0.9928826045568758}, {
      0.1253332336469065, -0.992114701300951}, {
      0.13156435917474169`, -0.9913076310553909}, {
      0.13779029076695085`, -0.9904614256819467}, {
      0.14401078263441522`, -0.9895761185873577}, {
      0.15022558920276707`, -0.9886517447220321}, {
      0.15643446512208453`, -0.9876883405786673}, {
      0.16263716527657757`, -0.9866859441908092}, {
      0.16883344479426482`, -0.9856445951313509}, {
      0.17502305905664067`, -0.9845643345109701}, {
      0.18120576370833236`, -0.9834452049765064}, {
      0.18738131466674668`, -0.9822872507092777}, {
      0.19354946813170607`, -0.9810905174233355}, {
      0.19970998059507325`, -0.979855052363661}, {
      0.2058626088503647, -0.9785809043042992}, {
      0.21200711000235184`, -0.9772681235464338}, {
      0.21814324147665026`, -0.9759167619164012}, {
      0.22427076102929613`, -0.9745268727636447}, {
      0.23038942675630947`, -0.9730985109586082}, {
      0.2364989971032442, -0.97163173289057}, {
      0.2425992308747243, -0.9701265964654168}, {
      0.2486898872439657, -0.9685831611033572}, {
      0.2547707257622839, -0.9670014877365767}, {
      0.2608415063685861, -0.9653816388068315}, {
      0.26690198939884896`, -0.9637236782629837}, {
      0.27295193559557956`, -0.9620276715584767}, {
      0.27899110611726124`, -0.9602936856487511}, {
      0.2850192625477825, -0.9585217889886016}, {
      0.2910361669058493, -0.9567120515294744}, {
      0.2970415816543802, -0.9548645447167056}, {
      0.30303526970988387`, -0.9529793414867006}, {
      0.3090169944518186, -0.9510565162640551}, {
      0.31498651973193403`, -0.9490961449586164}, {
      0.3209436098835935, -0.9470983049624869}, {
      0.326888029731078, -0.9450630751469689}, {
      0.3328195445988704, -0.9429905358594505}, {
      0.3387379203209202, -0.9408807689202341}, {
      0.34464292324988766`, -0.938733857619306}, {
      0.35053432026636816`, -0.9365498867130481}, {
      0.35641187878809516`, -0.9343289424208924}, {
      0.3622753667791222, -0.9320711124219165}, {
      0.36812455275898326`, -0.929776485851383}, {
      0.37395920581183134`, -0.9274451532972195}, {
      0.37977909559555434`, -0.9250772067964437}, {
      0.3855839923508689, -0.9226727398315286}, {
      0.39137366691039066`, -0.920231847326713}, {
      0.39714789070768153`, -0.9177546256442535}, {
      0.402906435786273, -0.9152411725806203}, {
      0.40864907480866564`, -0.9126915873626368}, {
      0.41437558106530376`, -0.9101059706435617}, {
      0.42008572848352566`, -0.9074844244991158}, {
      0.42577929163648864`, -0.9048270524234522}, {
      0.43145604575206836`, -0.9021339593250701}, {
      0.43711576672173247`, -0.8994052515226739}, {
      0.44275823110938806`, -0.8966410367409752}, {
      0.4483832161602026, -0.8938414241064404}, {
      0.4539904998093977, -0.8910065241429828}, {
      0.4595798606910162, -0.8881364487675987}, {
      0.4651510781466609, -0.8852313112859496}, {
      0.47070393223420615`, -0.8822912263878888}, {
      0.47623820373648057`, -0.879316310142934}, {
      0.4817536741699214, -0.8763066799956846}, {
      0.48725012579319993`, -0.8732624547611854}, {
      0.4927273416158175, -0.8701837546202361}, {
      0.49818510540667194`, -0.8670707011146469}, {
      0.5036232017025939, -0.8639234171424396}, {
      0.5090414158168531, -0.8607420269529964}, {
      0.514439533847634, -0.8575266561421546}, {
      0.5198173426864795, -0.8542774316472483}, {
      0.5251746300267051, -0.8509944817420969}, {
      0.5305111843717796, -0.8476779360319414}, {
      0.5358267950436751, -0.8443279254483276}, {
      0.5411212521911842, -0.8409445822439371}, {
      0.5463943467982042, -0.8375280399873664}, {
      0.551645870691989, -0.8340784335578537}, {
      0.5568756165513672, -0.8305958991399544}, {
      0.5620833779149271, -0.8270805742181639}, {
      0.5672689491891673, -0.8235325975714912}, {
      0.5724321256566127, -0.8199521092679791}, {
      0.5775727034838973, -0.816339250659175}, {
      0.5826904797298106, -0.8126941643745507}, {
      0.5877852523533094, -0.809016994315871}, {
      0.5928568202214941, -0.8053078856515139}, {
      0.5979049831175492, -0.8015669848107382}, {
      0.6029295417486474, -0.7977944394779037}, {
      0.6079302977538174, -0.7939903985866406}, {
      0.6129070537117746, -0.7901550123139696}, {
      0.6178596131487155, -0.7862884320743735}, {
      0.6227877805460736, -0.7823908105138195}, {
      0.6276913613482384, -0.7784623015037331}, {
      0.6325701619702362, -0.7745030601349233}, {
      0.6374239898053722, -0.77051324271146}, {
      0.6422526532328344, -0.7664930067445035}, {
      0.6470559616252589, -0.7624425109460858}, {
      0.6518337253562548, -0.7583619152228458}, {
      0.6565857558078909, -0.7542513806697153}, {
      0.6613118653781417, -0.7501110695635601}, {
      0.6660118674882937, -0.7459411453567735}, {
      0.6706855765903112, -0.7417417726708229}, {
      0.6753328081741617, -0.7375131172897516}, {
      0.6799533787750996, -0.7332553461536333}, {
      0.6845471059809092, -0.7289686273519822}, {
      0.689113808439106, -0.7246531301171164}, {
      0.6936533058640965, -0.7203090248174774}, {
      0.6981654190442951, -0.7159364829509042}, {
      0.7026499698491995, -0.7115356771378626}, {
      0.7071067812364227, -0.7071067811146308}, {
      0.7115356772586824, -0.7026499697264403}, {
      0.715936483070747, -0.6981654189205735}, {
      0.7203090249363385, -0.6936533057394173}, {
      0.7246531302349911, -0.6891138083134741}, {
      0.7289686274688658, -0.6845471058543295}, {
      0.733255346269521, -0.6799533786475771}, {
      0.7375131174046388, -0.6753328080457013}, {
      0.7417417727847049, -0.6706855764609178}, {
      0.7459411454696456, -0.6660118673579721}, {
      0.7501110696754179, -0.6613118652468972}, {
      0.754251380780554, -0.6565857556757285}, {
      0.7583619153326608, -0.6518337252231796}, {
      0.7624425110548727, -0.647055961491276}, {
      0.7664930068522576, -0.6422526530979489}, {
      0.770513242818177, -0.637423989669589}, {
      0.7745030602405987, -0.6325701618335607}, {
      0.7784623016083624, -0.6276913612106758}, {
      0.7823908106173982, -0.6227877804076289}, {
      0.7862884321768971, -0.6178596130093941}, {
      0.7901550124154338, -0.6129070535715816}, {
      0.7939903986870411, -0.6079302976127581}, {
      0.7977944395772362, -0.6029295416067272}, {
      0.8015669849089982, -0.5979049829747733}, {
      0.8053078857486972, -0.5928568200778679}, {
      0.8090169944119733, -0.5877852522088381}, {
      0.8126941644695675, -0.5826904795844997}, {
      0.8163392507531024, -0.5775727033377522}, {
      0.8199521093608128, -0.5724321255096388}, {
      0.823532597663227, -0.56726894904137}, {
      0.8270805743087976, -0.5620833777663121}, {
      0.8305958992294817, -0.5568756164019398}, {
      0.8340784336462708, -0.5516458705417547}, {
      0.837528040074669, -0.5463943466471687}, {
      0.8409445823301211, -0.5411212520393529}, {
      0.8443279255333891, -0.5358267948910537}, {
      0.8476779361158764, -0.5305111842183735}, {
      0.8509944818249013, -0.5251746298725201}, {
      0.8542774317289181, -0.5198173425315212}, {
      0.8575266562226859, -0.5144395336919082}, {
      0.8607420270323851, -0.5090414156603655}, {
      0.8639234172206819, -0.5036232015453501}, {
      0.8670707011917389, -0.49818510524867776`}, {
      0.870183754696174, -0.49272734145707875`}, {
      0.8732624548359653, -0.4872501256337224}, {
      0.8763066800693027, -0.4817536740097109}, {
      0.8793163102153866, -0.476238203575543}, {
      0.8822912264591721, -0.47070393207254735`}, {
      0.8852313113560598, -0.4651510779842868}, {
      0.8881364488365321, -0.4595798605279327}, {
      0.8910065242107357, -0.4539904996456108}, {
      0.8938414241730092, -0.4483832159957182}, {
      0.8966410368063561, -0.4427582309442122}, {
      0.8994052515868635, -0.43711576655587114`}, {
      0.9021339593880647, -0.43145604558552764`}, {
      0.9048270524852482, -0.42577929146927457`}, {
      0.9074844245597098, -0.42008572831564434`}, {
      0.9101059707029501, -0.4143755808967613}, {
      0.912691587420816, -0.4086490746394682}, {
      0.915241172637587, -0.4029064356164268}, {
      0.9177546257000041, -0.39714789053719274`}, {
      0.9202318473812443, -0.3913736667392655}, {
      0.9226727398848372, -0.3855839921791136}, {
      0.9250772068485262, -0.3797790954231752}, {
      0.9274451533480728, -0.3739592056388346}, {
      0.9297764859010035, -0.3681245525853753}, {
      0.9320711124703012, -0.36227536660490933`}, {
      0.9343289424680378, -0.3564118786132838}, {
      0.9365498867589511, -0.3505343200909647}, {
      0.9387338576639633, -0.3446429230738985}, {
      0.9408807689636427, -0.3387379201443518}, {
      0.9429905359016072, -0.33281954442172923`}, {
      0.9450630751878706, -0.32688802955337054`}, {
      0.9470983050021305, -0.32094360970532626`}, {
      0.9490961449969989, -0.3149865195531136}, {
      0.9510565163011735, -0.30901699427245155`}, {
      0.9529793415225519, -0.3030352695299768}, {
      0.9548645447512868, -0.2970415814739398}, {
      0.9567120515627827, -0.29103616672488214`}, {
      0.9585217890206341, -0.28501926236629527`}, {
      0.9602936856795049, -0.27899110593526066`}, {
      0.962027671587949, -0.27295193541307233`}, {
      0.9637236782911717, -0.26690198921584185`}, {
      0.9653816388337324, -0.2608415061850859}, {
      0.9670014877621879, -0.25477072557829733`}, {
      0.9685831611276758, -0.2486898870594997}, {
      0.9701265964884402, -0.24259923068978567`}, {
      0.9716317329122958, -0.23649899691783985`}, {
      0.9730985109790335, -0.23038942657044628`}, {
      0.9745268727827671, -0.22427076084298103`}, {
      0.975916761934218, -0.2181432412898902}, {
      0.9772681235629425, -0.2120071098151538}, {
      0.9785809043194974, -0.2058626086627357}, {
      0.9798550523775462, -0.19970998040702034`}, {
      0.9810905174359054, -0.19354946794323627`}, {
      0.9822872507205298, -0.1873813144778671}, {
      0.9834452049864385, -0.1812057635190501}, {
      0.9845643345195798, -0.17502305886696287`}, {
      0.985644595138636, -0.16883344460419863`}, {
      0.9866859441967675, -0.16263716508613016`}, {
      0.9876883405832965, -0.15643446493126312`}, {
      0.9886517447253301, -0.1502255890115789}, {
      0.9895761185893225, -0.14401078244286752`}, {
      0.9904614256825761, -0.1377902905750509}, {
      0.9913076310546829, -0.13156435898249683`}, {
      0.9921147012989036, -0.12533323345432404`}, {
      0.9928826045534869, -0.11909715998469902`}, {
      0.9936113105029272, -0.11285638476312453`}, {
      0.9943007903791614, -0.10661115416472038`}, {
      0.9949510169627045, -0.10036171474049706`}, {
      0.9955619645837244, -0.09410831320762231}, {
      0.996133609123055, -0.08785119643968113}, {
      0.9966659280131488, -0.08159061145692964}, {
      0.9971589002389675, -0.07532680541654312}, {
      0.9976125063388116, -0.0690600256028587}, {
      0.998026728405089, -0.06279051941761302}, {
      0.9984015500850218, -0.05651853437017516}, {
      0.998736956581292, -0.05024431806777545}, {
      0.9990329346526252, -0.04396811820573035}, {
      0.9992894726143141, -0.037690182557663855`}, {
      0.9995065603386791, -0.03141075896572582}, {
      0.9996841892554686, -0.025130095330807577`}, {
      0.9998223523521972, -0.018848439602755222`}, {
      0.9999210441744223, -0.012566039770581006`}, {
      0.9999802608259598, -0.006283143852673126}, {0.9999999999690377, 
      1.1299560419286703`*^-10}, {0.9999802608243883, 0.006283144078659868}, {
      0.9999210441712794, 0.012566039996554349`}, {0.9998223523474828, 
      0.018848439828706234`}, {0.999684189249183, 0.025130095556727322`}, {
      0.9995065603308224, 0.031410759191605374`}, {0.9992894726048865, 
      0.037690182783494286`}, {0.9990329346416271, 0.04396811843150273}, {
      0.9987369565687237, 0.05024431829348085}, {0.9984015500708838, 
      0.056518534595804644`}, {0.9980267283893817, 0.06279051964315767}, {
      0.9976125063215354, 0.0690600258283096}, {0.9971589002201231, 
      0.07532680564189133}, {0.9966659279927368, 0.08159061168216626}, {
      0.9961336091010761, 0.08785119666479725}, {0.9955619645601792, 
      0.094108313432609}, {0.9949510169375938, 0.10036171496534542`}, {
      0.994300790352486, 0.1066111543894215}, {0.9936113104746879, 
      0.1128563849876695}, {0.9928826045236845, 0.11909716020907897`}, {
      0.9921147012675391, 0.12533323367853005`}, {0.9913076310217573, 
      0.13156435920652002`}, {0.9904614256480906, 0.1377902907988824}, {
      0.9895761185532781, 0.1440107826664984}, {0.9886517446877281, 
      0.15022558923500026`}, {0.9876883405441379, 0.15643446515446613`}, {
      0.9866859441560536, 0.16263716530910596`}, {0.9856445950963681, 
      0.16883344482693835`}, {0.9845643344757594, 0.17502305908945764`}, {
      0.9834452049410669, 0.18120576374129108`}, {0.9822872506736084, 
      0.18738131469984545`}, {0.9810905173874358, 0.19354946816494314`}, {
      0.97985505232753, 0.1997099806284469}, {0.9785809042679361, 
      0.20586260888387312`}, {0.9772681235098377, 0.21200711003599326`}, {
      0.9759167618795714, 0.2181432415104229}, {0.9745268727265806, 
      0.22427076106319813`}, {0.973098510921309, 0.230389426790339}, {
      0.971631732853035, 0.23649899713739939`}, {0.9701265964276451, 
      0.24259923090900326`}, {0.9685831610653483, 0.24868988727836658`}, {
      0.9670014876983298, 0.2547707257968047}, {0.965381638768346, 
      0.260841506403225}, {0.9637236782242591, 0.26690198943360394`}, {
      0.9620276715195123, 0.27295193563044867`}, {0.9602936856095464, 
      0.2789911061522425}, {0.9585217889491561, 0.2850192625828739}, {
      0.9567120514897874, 0.2910361669410489}, {0.9548645446767766, 
      0.2970415816896859}, {0.9529793414465292, 0.30303526974529366`}, {
      0.9510565162236407, 0.30901699448733044`}, {0.9490961449179586, 
      0.3149865197675458}, {0.9470983049215852, 0.32094360991930315`}, {
      0.9450630751058229, 0.3268880297668834}, {0.9429905358180598, 
      0.3328195446347695}, {0.9408807688785984, 0.33873792035691075`}, {
      0.9387338575774248, 0.3446429232859676}, {0.9365498866709212, 
      0.3505343203025353}, {0.9343289423785194, 0.35641187882434733`}, {
      0.9320711123792972, 0.36227536681545724`}, {0.9297764858085169, 
      0.368124552795399}, {0.9274451532541066, 0.37395920584832554`}, {
      0.9250772067530836, 0.3797790956321248}, {0.9226727397879212, 
      0.3855839923875134}, {0.920231847282858, 0.39137366694710696`}, {
      0.9177546256001508, 0.3971478907444674}, {0.9152411725362698, 
      0.40290643582312613`}, {0.9126915873180383, 0.40864907484558377`}, {
      0.9101059705987152, 0.41437558110228456`}, {0.9074844244540213, 
      0.4200857285205669}, {0.9048270523781096, 0.425779291673588}, {
      0.9021339592794794, 0.43145604578922353`}, {0.899405251476835, 
      0.43711576675894115`}, {0.8966410366948883, 0.4427582311466479}, {
      0.8938414240601055, 0.44838321619751126`}, {0.8910065240963999, 
      0.4539904998467528}, {0.888136448720768, 0.4595798607284154}, {
      0.8852313112388712, 0.46515107818410184`}, {0.882291226340563, 
      0.47070393227168644`}, {0.8793163100953608, 0.47623820377399784`}, {
      0.8763066799478643, 0.4817536742074733}, {0.8732624547131183, 
      0.48725012583078403`}, {0.8701837545719224, 0.49272734165343146`}, {
      0.8670707010660869, 0.4981851054443133}, {0.8639234170936335, 
      0.5036232017402603}, {0.8607420269039447, 0.5090414158545422}, {
      0.8575266560928577, 0.5144395338853432}, {0.8542774315977065, 
      0.5198173427242065}, {0.8509944816923106, 0.5251746300644474}, {
      0.8476779359819111, 0.5305111844095349}, {0.8443279253980537, 
      0.535826795081441}, {0.84094458219342, 0.5411212522289582}, {
      0.8375280399366068, 0.5463943468359839}, {0.8340784335068523, 
      0.5516458707297719}, {0.8305958990887116, 0.5568756165891509}, {
      0.8270805741666803, 0.5620833779527091}, {0.8235325975197675, 
      0.5672689492269452}, {0.8199521092160159, 0.5724321256943841}, {
      0.8163392506069731, 0.5775727035216598}, {0.8126941643221106, 
      0.5826904797675617}, {0.8090169942631937, 0.5877852523910466}, {
      0.8053078855986, 0.592856820259215}, {0.8015669847575886, 
      0.5979049831552514}, {0.7977944394245192, 0.6029295417863285}, {
      0.793990398533022, 0.6079302977914749}, {0.7901550122601179, 
      0.6129070537494061}, {0.7862884320202895, 0.6178596131863185}, {
      0.7823908104595043, 0.6227877805836458}, {0.7784623014491876, 
      0.6276913613857773}, {0.7745030600801485, 0.6325701620077393}, {
      0.7705132426564568, 0.6374239898428371}, {0.766493006689273, 
      0.6422526532702588}, {0.7624425108906292, 0.6470559616626403}, {
      0.758361915167164, 0.6518337253935909}, {0.7542513806138095, 
      0.6565857558451792}, {0.7501110695074314, 0.6613118654153798}, {
      0.7459411453004231, 0.6660118675254793}, {0.7417417726142521, 
      0.670685576627442}, {0.7375131172329616, 0.6753328082112353}, {
      0.7332553460966255, 0.6799533788121136}, {0.7289686272947576, 
      0.6845471060178612}, {0.7246531300596765, 0.6891138084759937}, {
      0.7203090247598235, 0.6936533059009174}, {0.7159364828930377, 
      0.6981654190810471}, {0.7115356770797849, 0.7026499698858802}, {
      0.7071067810563434, 0.7071067812730298}, {0.7026499696679446, 
      0.7115356772952136}, {0.6981654188618708, 0.7159364831072}, {
      0.6936533056805091, 0.7203090249727111}, {0.6891138082543621, 
      0.7246531302712809}, {0.6845471057950151, 0.7289686275050705}, {
      0.6799533785880619, 0.7332553463056385}, {0.6753328079859869, 
      0.7375131174406666}, {0.6706855764010059, 0.741741772820641}, {
      0.6660118672978644, 0.7459411455054878}, {0.6613118651865952, 
      0.7501110697111639}, {0.656585755615234, 0.7542513808162017}, {
      0.6518337251624943, 0.7583619153682081}, {0.6470559614304016, 
      0.7624425110903174}, {0.6422526530368872, 0.7664930068875975}, {
      0.6374239896083419, 0.7705132428534099}, {0.6325701617721299, 
      0.7745030602757225}, {0.6276913611490631, 0.778462301643375}, {
      0.6227877803458363, 0.7823908106522977}, {0.6178596129474233, 
      0.7862884322116813}, {0.6129070535094348, 0.7901550124501007}, {
      0.607930297550437, 0.7939903987215886}, {0.6029295415442338, 
      0.7977944396116623}, {0.5979049829121097, 0.8015669849433009}, {
      0.5928568200150359, 0.8053078857828745}, {0.58778525214584, 
      0.8090169944460233}, {0.5826904795213375, 0.8126941645034883}, {
      0.5775727032744278, 0.816339250786892}, {0.5724321254461543, 
      0.8199521093944692}, {0.5672689489777276, 0.8235325976967484}, {
      0.5620833777025138, 0.8270805743421821}, {0.5568756163379879, 
      0.8305958992627275}, {0.5516458704776513, 0.834078433679376}, {
      0.546394346582916, 0.8375280401076317}, {0.5411212519749532, 
      0.8409445823629397}, {0.5358267948265091, 0.8443279255660616}, {
      0.5305111841536864, 0.8476779361484011}, {0.5251746298076926, 
      0.8509944818572766}, {0.5198173424665558, 0.8542774317611421}, {
      0.5144395336268069, 0.857526656254757}, {0.5090414155951309, 
      0.8607420270643017}, {0.5036232014799845, 0.8639234172524423}, {
      0.4981851051831835, 0.8670707012233414}, {0.4927273413914582, 
      0.870183754727617}, {0.4872501255679779, 0.8732624548672472}, {
      0.48175367394384494`, 0.876306680100422}, {0.476238203509558, 
      0.8793163102463417}, {0.4707039320064457, 0.8822912264899615}, {
      0.46515107791807103`, 0.885231311386682}, {0.45957986046160526`, 
      0.8881364488669857}, {0.45399049957917414`, 0.8910065242410193}, {
      0.44838321592917485`, 0.8938414242031213}, {0.4427582308775647, 
      0.8966410368362954}, {0.43711576648912204`, 0.8994052516166284}, {
      0.43145604551867944`, 0.9021339594176542}, {0.4257792914023299, 
      0.9048270525146608}, {0.42008572824860574`, 0.9074844245889441}, {
      0.4143755808296314, 0.9101059707320048}, {0.4086490745722496, 
      0.91269158744969}, {0.4029064355491221, 0.915241172666279}, {
      0.39714789046980453`, 0.9177546257285131}, {0.39137366667179646`, 
      0.920231847409569}, {0.3855839921115664, 0.9226727399129764}, {
      0.3797790953555525, 0.9250772068764788}, {0.3739592055711391, 
      0.9274451533758375}, {0.36812455251760967`, 0.9297764859285795}, {
      0.36227536653707626`, 0.9320711124976874}, {0.356411878545386, 
      0.9343289424952332}, {0.35053432002300483`, 0.9365498867859547}, {
      0.34464292300587934`, 0.9387338576907742}, {0.33873792007627607`, 
      0.94088076899026}, {0.3328195443535997, 0.9429905359280298}, {
      0.32688802948518986`, 0.9450630752140978}, {0.3209436096370973, 
      0.9470983050281615}, {0.314986519484839, 0.9490961450228328}, {
      0.3090169942041342, 0.9510565163268095}, {0.3030352694616194, 
      0.9529793415479894}, {0.29704158140554515`, 0.9548645447765248}, {
      0.29103616665645304`, 0.9567120515878206}, {0.2850192622978345, 
      0.9585217890454711}, {0.278991105866771, 0.9602936857041404}, {
      0.27295193534455664`, 0.9620276716123825}, {0.2669019891473029, 
      0.9637236783154025}, {0.2608415061165265, 0.96538163885776}, {
      0.25477072550972035`, 0.9670014877860116}, {0.24868988699090794`, 
      0.9685831611512953}, {0.24259923062118194`, 0.9701265965118548}, {
      0.23649899684922696`, 0.9716317329355051}, {0.23038942650182706`, 
      0.9730985110020371}, {0.2242707607743583, 0.9745268728055645}, {
      0.21814324122126683`, 0.975916761956809}, {0.21200710974653258`, 
      0.9772681235853267}, {0.20586260859411948`, 0.9785809043416744}, {
      0.19970998033841195`, 0.9798550523995159}, {0.19354946787463856`, 
      0.9810905174576674}, {0.18738131440928288`, 0.9822872507420839}, {
      0.1812057634504822, 0.9834452050077844}, {0.17502305879841412`, 
      0.9845643345407175}, {0.16883344453567187`, 0.9856445951595652}, {
      0.1626371650176282, 0.986685944217488}, {0.15643446486278878`, 
      0.9876883406038084}, {0.150225588943135, 0.9886517447456333}, {
      0.1440107823744569, 0.9895761186094169}, {0.1377902905066764, 
      0.9904614257024617}, {0.1315643589141612, 0.9913076310743596}, {
      0.12533323338603014`, 0.9921147013183717}, {0.11909715991644965`, 
      0.9928826045727464}, {0.11285638469492246`, 0.9936113105219782}, {
      0.1066111540965684, 0.9943007903980039}, {0.10036171467239798`, 
      0.9949510169813388}, {0.0941083131395789, 0.9955619646021506}, {
      0.08785119637169617, 0.9961336091412734}, {0.08159061138900588, 
      0.9966659280311595}, {0.07532680534868333, 0.9971589002567708}, {
      0.06906002553506564, 0.9976125063564079}, {0.06279051934988944, 
      0.9980267284224785}, {0.05651853430252379, 0.998401550102205}, {
      0.05024431800019903, 0.9987369565982691}, {0.0439681181382316, 
      0.9990329346693967}, {0.037690182490245486`, 0.9992894726308804}, {
      0.031410758898390526`, 0.9995065603550407}, {0.025130095263558052`, 
      0.999684189271626}, {0.018848439535594154`, 0.9998223523681508}, {
      0.012566039703511066`, 0.9999210441901726}, {0.006283143785696977, 
      0.9999802608415074}, {-1.7987531169411097`*^-10, 
      0.9999999999843832}, {-0.006283144145440494, 
      0.9999802608395323}, {-0.012566040063233265`, 
      0.9999210441862224}, {-0.018848439895280823`, 
      0.9998223523622257}, {-0.025130095623194983`, 
      0.9996841892637264}, {-0.031410759257963515`, 
      0.9995065603451669}, {-0.03769018284974032, 
      0.999289472619033}, {-0.043968118497634094`, 
      0.9990329346555763}, {-0.05024431835949499, 
      0.9987369565824764}, {-0.05651853466169903, 
      0.9984015500844408}, {-0.06279051970892978, 
      0.9980267284027436}, {-0.06906002589395692, 
      0.9976125063347033}, {-0.07532680570741136, 
      0.9971589002330978}, {-0.08159061174755652, 
      0.9966659280055192}, {-0.08785119673005529, 
      0.9961336091136671}, {-0.09410831349773235, 
      0.9955619645725797}, {-0.10036171503033166`, 
      0.9949510169498048}, {-0.1066111544542682, 
      0.9943007903645085}, {-0.11285638505237429`, 
      0.9936113104865228}, {-0.11909716027363944`, 
      0.9928826045353331}, {-0.12533323374294383`, 
      0.9921147012790023}, {-0.13156435927078475`, 
      0.9913076310330362}, {-0.13779029086299577`, 
      0.9904614256591863}, {-0.1440107827304581, 
      0.9895761185641917}, {-0.15022558929880397`, 
      0.9886517446984606}, {-0.15643446521811158`, 
      0.9876883405546906}, {-0.16263716537259088`, 
      0.9866859441664276}, {-0.1688334448902605, 
      0.9856445951065648}, {-0.17502305915261482`, 
      0.9845643344857797}, {-0.18120576380428105`, 
      0.9834452049509123}, {-0.18738131476266603`, 
      0.98228725068328}, {-0.19354946822759217`, 
      0.9810905173969349}, {-0.19970998069092222`, 
      0.9798550523368578}, {-0.20586260894617264`, 
      0.9785809042770939}, {-0.21200711009811488`, 
      0.977268123518827}, {-0.2181432415723645, 
      0.9759167618883935}, {-0.22427076112495767`, 
      0.9745268727352367}, {-0.2303894268519144, 
      0.9730985109298006}, {-0.23649899719878867`, 
      0.9716317328613634}, {-0.24259923097020442`, 
      0.9701265964358118}, {-0.2486898873393776, 
      0.9685831610733546}, {-0.25477072585762367`, 
      0.9670014877061773}, {-0.26084150646384996`, 
      0.9653816387760361}, {-0.266901989494033, 
      0.9637236782317932}, {-0.2729519356906799, 
      0.9620276715268918}, {-0.278991106212274, 
      0.960293685616773}, {-0.2850192626427039, 
      0.9585217889562311}, {-0.29103616700067547`, 
      0.9567120514967125}, {-0.29704158174910733`, 
      0.9548645446835532}, {-0.3030352698045081, 
      0.9529793414531589}, {-0.3090169945463362, 
      0.9510565162301251}, {-0.3149865198263411, 
      0.9490961449242993}, {-0.32094360997788635`, 
      0.9470983049277838}, {-0.3268880298252528, 
      0.9450630751118809}, {-0.33281954469292335`, 
      0.942990535823979}, {-0.3387379204148475, 
      0.9408807688843802}, {-0.34464292334368557`, 
      0.9387338575830709}, {-0.3505343203600329, 
      0.9365498866764332}, {-0.356411878881623, 
      0.9343289423838991}, {-0.36227536687250944`, 
      0.9320711123845461}, {-0.36812455285222623`, 
      0.9297764858136368}, {-0.37395920590492626`, 
      0.9274451532590992}, {-0.3797790956884976, 
      0.9250772067579505}, {-0.38558399244365676`, 
      0.9226727397926641}, {-0.3913736670030195, 
      0.9202318472874786}, {-0.3971478908001477, 
      0.9177546256046508}, {-0.40290643587857283`, 
      0.9152411725406511}, {-0.4086490749007955, 
      0.9126915873223026}, {-0.41437558115726003`, 
      0.9101059706028641}, {-0.4200857285753048, 
      0.9074844244580567}, {-0.42577929172808704`, 
      0.9048270523820331}, {-0.43145604584348246`, 
      0.9021339592832928}, {-0.4371157668129587, 
      0.8994052514805401}, {-0.44275823120042296`, 
      0.8966410366984867}, {-0.44838321625104255`, 
      0.8938414240635991}, {-0.45399049990003926`, 
      0.8910065240997905}, {-0.45957986078145585`, 
      0.8881364487240574}, {-0.4651510782368952, 
      0.8852313112420611}, {-0.47070393232423163`, 
      0.8822912263436552}, {-0.4762382038262938, 
      0.8793163100983572}, {-0.481753674259519, 
      0.8763066799507665}, {-0.4872501258825785, 
      0.8732624547159281}, {-0.4927273417049736, 
      0.8701837545746418}, {-0.49818510549560224`, 
      0.8670707010687175}, {-0.5036232017912952, 
      0.8639234170961774}, {-0.509041415905322, 
      0.8607420269064034}, {-0.5144395339358672, 
      0.8575266560952332}, {-0.5198173427744737, 
      0.8542774316000005}, {-0.525174630114457, 
      0.850994481694525}, {-0.530511184459286, 
      0.8476779359840475}, {-0.5358267951309329, 
      0.844327925400114}, {-0.5411212522781902, 
      0.8409445821954062}, {-0.5463943468849551, 
      0.8375280399385205}, {-0.5516458707784817, 
      0.8340784335086953}, {-0.5568756166375987, 
      0.8305958990904856}, {-0.5620833780008941, 
      0.8270805741683873}, {-0.5672689492748667, 
      0.8235325975214092}, {-0.5724321257420416, 
      0.8199521092175941}, {-0.5775727035690527, 
      0.8163392506084896}, {-0.5826904798146894, 
      0.8126941643235672}, {-0.5877852524379086, 
      0.8090169942645923}, {-0.5928568203058108, 
      0.8053078855999423}, {-0.5979049832015805, 
      0.8015669847588762}, {-0.6029295418323903, 
      0.797794439425754}, {-0.607930297837269, 
      0.7939903985342057}, {-0.6129070537949323, 
      0.7901550122612523}, {-0.6178596132315762, 
      0.7862884320213764}, {-0.6227877806286346, 
      0.7823908104605454}, {-0.627691361430497, 
      0.7784623014501846}, {-0.6325701620521895, 
      0.7745030600811031}, {-0.6374239898870175, 
      0.770513242657371}, {-0.6422526533141691, 
      0.7664930066901483}, {-0.6470559617062802, 
      0.7624425108914673}, {-0.6518337254369602, 
      0.7583619151679667}, {-0.6565857558882776, 
      0.7542513806145784}, {-0.6613118654582072, 
      0.7501110695081684}, {-0.6660118675680354, 
      0.7459411453011298}, {-0.6706855766697267, 
      0.74174177261493}, {-0.6753328082532485, 
      0.7375131172336125}, {-0.679953378853855, 
      0.733255346097251}, {-0.6845471060593308, 
      0.7289686272953594}, {-0.6891138085171914, 
      0.7246531300602561}, {-0.6936533059418433, 
      0.7203090247603826}, {-0.6981654191217009, 
      0.7159364828935779}, {-0.702649969926262, 
      0.7115356770803078}, {-0.7071067813131396, 
      0.7071067810568505}, {-0.7115356773350514, 
      0.7026499696684376}, {-0.715936483146766, 
      0.6981654188623513}, {-0.7203090250120051, 
      0.6936533056809787}, {-0.7246531303103031, 
      0.6891138082548223}, {-0.7289686275438211, 
      0.6845471057954674}, {-0.7332553463441175, 
      0.6799533785885079}, {-0.7375131174788742, 
      0.6753328079864281}, {-0.7417417728585772, 
      0.6706855764014438}, {-0.7459411455431528, 
      0.6660118672983004}, {-0.750111069748558, 
      0.6613118651870309}, {-0.754251380853325, 
      0.6565857556156708}, {-0.7583619154050608, 
      0.6518337251629336}, {-0.7624425111268998, 
      0.6470559614308449}, {-0.7664930069239099, 
      0.6422526530373358}, {-0.7705132428894526, 
      0.6374239896087973}, {-0.7745030603114956, 
      0.6325701617725935}, {-0.778462301678879, 
      0.6276913611495363}, {-0.7823908106875327, 
      0.6227877803463204}, {-0.7862884322466478, 
      0.6178596129479198}, {-0.7901550124847989, 
      0.6129070535099448}, {-0.793990398756019, 
      0.607930297550962}, {-0.7977944396458252, 
      0.6029295415447751}, {-0.8015669849771968, 
      0.5979049829126685}, {-0.8053078858165037, 
      0.5928568200156136}, {-0.8090169944793862, 
      0.5877852521464377}, {-0.8126941645365854, 
      0.5826904795219565}, {-0.8163392508197236, 
      0.5775727032750695}, {-0.8199521094270359, 
      0.5724321254468198}, {-0.8235325977290506, 
      0.5672689489784181}, {-0.8270805743742203, 
      0.5620833777032305}, {-0.8305958992945023, 
      0.5568756163387318}, {-0.8340784337108877, 
      0.5516458704784237}, {-0.837528040138881, 
      0.546394346583718}, {-0.840944582393927, 
      0.541121251975786}, {-0.8443279255967875, 
      0.5358267948273737}, {-0.8476779361788661, 
      0.5305111841545839}, {-0.8509944818874813, 
      0.5251746298086241}, {-0.8542774317910872, 
      0.5198173424675223}, {-0.8575266562844429, 
      0.5144395336278096}, {-0.860742027093729, 
      0.5090414155961707}, {-0.8639234172816116, 
      0.5036232014810623}, {-0.8670707012522534, 
      0.49818510518430037`}, {-0.8701837547562723, 
      0.4927273413926151}, {-0.8732624548956464, 
      0.4872501255691758}, {-0.8763066801285656, 
      0.4817536739450847}, {-0.8793163102742305, 
      0.47623820351084056`}, {-0.8822912265175962, 
      0.47070393200777205`}, {-0.8852313114140631, 
      0.46515107791944194`}, {-0.8881364488941138, 
      0.45957986046302157`}, {-0.8910065242678951, 
      0.45399049958063675`}, {-0.8938414242297454, 
      0.4483832159306846}, {-0.8966410368626686, 
      0.44275823087912236`}, {-0.8994052516427514, 
      0.43711576649072836`}, {-0.9021339594435276, 
      0.4314560455203353}, {-0.9048270525402853, 
      0.425779291404036}, {-0.9074844246143204, 
      0.4200857282503628}, {-0.9101059707571337, 
      0.4143755808314401}, {-0.9126915874745722, 
      0.4086490745741107}, {-0.9152411726909152, 
      0.4029064355510363}, {-0.9177546257529039, 
      0.39714789047177246`}, {-0.9202318474337151, 
      0.3913736666738188}, {-0.9226727399368786, 
      0.38558399211364375`}, {-0.9250772069001378, 
      0.37977909535768545`}, {-0.9274451533992543, 
      0.37395920557332823`}, {-0.9297764859517545, 
      0.3681245525198556}, {-0.9320711125206215, 
      0.36227536653937953`}, {-0.9343289425179271, 
      0.35641187854774714`}, {-0.9365498868084091, 
      0.35053432002542445`}, {-0.9387338577129898, 
      0.3446429230083579}, {-0.9408807690122374, 
      0.3387379200788141}, {-0.94299053594977, 
      0.33281954435619765`}, {-0.9450630752356013, 
      0.32688802948784823`}, {-0.9470983050494292, 
      0.32094360963981644`}, {-0.9490961450438653, 
      0.3149865194876194}, {-0.9510565163476076, 
      0.3090169942069762}, {-0.9529793415685537, 
      0.3030352694645234}, {-0.9548645447968562, 
      0.29704158140851156`}, {-0.9567120516079198, 
      0.29103616665948223`}, {-0.9585217890653389, 
      0.2850192623009268}, {-0.9602936857237775, 
      0.27899110586992676`}, {-0.9620276716317894, 
      0.2729519353477762}, {-0.9637236783345801, 
      0.2669019891505865}, {-0.9653816388767089, 
      0.26084150611987444`}, {-0.9670014878047326, 
      0.25477072551313285`}, {-0.9685831611697892, 
      0.24868988699438524`}, {-0.9701265965301222, 
      0.2425992306247243}, {-0.9716317329535467, 
      0.2364989968528346}, {-0.9730985110198537, 
      0.23038942650550018`}, {-0.9745268728231566, 
      0.2242707607780971}, {-0.9759167619741774, 
      0.21814324122507148`}, {-0.9772681236024722, 
      0.21200710975040324`}, {-0.9785809043585976, 
      0.20586260859805627`}, {-0.9798550524162174, 
      0.199709980342415}, {-0.9810905174741479, 
      0.19354946787870797`}, {-0.982287250758344, 
      0.18738131441341876`}, {-0.9834452050238249, 
      0.18120576345468464`}, {-0.9845643345565389, 
      0.1750230588026832}, {-0.9856445951751682, 
      0.16883344454000762`}, {-0.9866859442328734, 
      0.16263716502203068`}, {-0.9876883406189766, 
      0.15643446486725804`}, {-0.988651744760585, 
      0.15022558894767105`}, {-0.9895761186241527, 
      0.14401078237905973`}, {-0.9904614257169823, 
      0.13779029051134598`}, {-0.9913076310886656, 
      0.13156435891889753`}, {-0.9921147013324635, 
      0.1253332333908332}, {-0.9928826045866247, 
      0.11909715992131936`}, {-0.9936113105356436, 
      0.11285638469985879`}, {-0.994300790411457, 
      0.10661115410157128`}, {-0.9949510169945801, 
      0.10036171467746732`}, {-0.9955619646151807, 
      0.09410831314471463}, {-0.9961336091540928, 
      0.08785119637689817}, {-0.9966659280437687, 
      0.08159061139427405}, {-0.9971589002691704, 
      0.07532680535401753}, {-0.9976125063685983, 
      0.06906002554046575}, {-0.9980267284344604, 
      0.0627905193553553}, {-0.9984015501139787, 
      0.05651853430805526}, {-0.9987369566098352, 
      0.05024431800579593}, {-0.9990329346807557, 
      0.04396811814389376}, {-0.9992894726420327, 
      0.03769018249597272}, {-0.9995065603659868, 
      0.03141075890418264}, {-0.9996841892823664, 
      0.025130095269414857`}, {-0.9998223523786859, 
      0.018848439541515435`}, {-0.9999210442005029, 
      0.012566039709496605`}, {-0.9999802608516332, 
      0.006283143791746547}, {-0.9999999999943048, \
-1.7376194320212202`*^-10}, {-0.9999802608492502, -0.006283144139263569}, \
{-0.999921044195737, -0.012566040056993032`}, {-0.9998223523715374, \
-0.01884843988897754}, {-0.9996841892728354, -0.025130095616828906`}, \
{-0.9995065603540737, -0.031410759251534914`}, {-0.9992894726277378, \
-0.03769018284324948}, {-0.9990329346640795, -0.04396811849108129}, \
{-0.9987369565907783, -0.050244318352880506`}, {-0.9984015500925417, \
-0.05651853465502316}, {-0.998026728410644, -0.06279051970219282}, \
{-0.9976125063424033, -0.06906002588715918}, {-0.9971589002405976, \
-0.07532680570055315}, {-0.9966659280128192, -0.08159061174063816}, \
{-0.9961336091207673, -0.08785119672307706}, {-0.9955619645794805, \
-0.09410831349069461}, {-0.9949510169565065, -0.10036171502323471`}, \
{-0.994300790371011, -0.10661115444711237`}, {-0.9936113104928267, \
-0.11285638504515991`}, {-0.9928826045414383, -0.11909716026636684`}, \
{-0.992114701284909, -0.12533323373561334`}, {-0.9913076310387445, \
-0.1315643592633967}, {-0.9904614256646964, -0.13779029085555053`}, \
{-0.9895761185695038, -0.144010782722956}, {-0.9886517447035748, \
-0.1502255892912454}, {-0.9876883405596069, -0.15643446521049684`}, \
{-0.9866859441711461, -0.16263716536492032`}, {-0.9856445951110855, \
-0.16883344488253446`}, {-0.9845643344901028, -0.17502305914483365`}, \
{-0.9834452049550377, -0.18120576379644512`}, {-0.982287250687208, \
-0.18738131475477568`}, {-0.9810905174006653, -0.19354946821964775`}, \
{-0.9798550523403908, -0.19970998068292406`}, {-0.9785809042804295, \
-0.20586260893812108`}, {-0.9772681235219651, -0.21200711009001025`}, \
{-0.9759167618913341, -0.21814324156420717`}, {-0.9745268727379799, \
-0.22427076111674796`}, {-0.9730985109323461, -0.23038942684365268`}, \
{-0.9716317328637113, -0.23649899719047526`}, {-0.9701265964379621, \
-0.24259923096183963`}, {-0.9685831610753072, -0.24868988733096178`}, \
{-0.967001487707932, -0.25477072584915716`}, {-0.9653816387775929, \
-0.26084150645533305`}, {-0.9637236782331519, -0.266901989485466}, \
{-0.9620276715280524, -0.27295193568206316`}, {-0.9602936856177351, \
-0.2789911062036079}, {-0.9585217889569947, -0.2850192626339887}, \
{-0.9567120514972774, -0.2910361669919115}, {-0.9548645446839193, \
-0.2970415817402948}, {-0.9529793414533261, -0.3030352697956474}, \
{-0.951056516230093, -0.30901699453742754`}, {-0.9490961449240677, \
-0.31498651981738485`}, {-0.9470983049273526, -0.32094360996888266`}, \
{-0.9450630751112499, -0.326888029816202}, {-0.9429905358231477, \
-0.3328195446838258}, {-0.9408807688833486, -0.3387379204057034}, \
{-0.9387338575818387, -0.34464292333449514`}, {-0.9365498866750002, \
-0.35053432035079646`}, {-0.9343289423822648, -0.35641187887234077`}, \
{-0.9320711123827105, -0.3622753668631816}, {-0.9297764858115993, \
-0.368124552842853}, {-0.9274451532568595, -0.3739592058955079}, \
{-0.9250772067555084, -0.3797790956790343}, {-0.9226727397900193, \
-0.3855839924341487}, {-0.9202318472846308, -0.3913736669934669}, \
{-0.9177546256015996, -0.39714789079055063`}, {-0.915241172537396, \
-0.40290643586893155`}, {-0.9126915873188434, -0.40864907489111013`}, \
{-0.9101059705992004, -0.41437558114753076`}, {-0.907484424454188, \
-0.4200857285655318}, {-0.9048270523779591, -0.4257792917182704}, \
{-0.9021339592790132, -0.4314560458336223}, {-0.8994052514760543, \
-0.43711576680305514`}, {-0.8966410366937945, -0.4427582311904761}, \
{-0.8938414240586999, -0.4483832162410525}, {-0.891006524094684, \
-0.45399049989000606`}, {-0.888136448718743, -0.4595798607713796}, \
{-0.8852313112365384, -0.4651510782267759}, {-0.8822912263379238, \
-0.47070393231406943`}, {-0.8793163100924166, -0.47623820381608867`}, \
{-0.8763066799446163, -0.48175367424927096`}, {-0.8732624547095679, \
-0.48725012587228755`}, {-0.870183754568071, -0.49272734169463983`}, \
{-0.8670707010619356, -0.4981851054852256}, {-0.8639234170891839, \
-0.5036232017808756}, {-0.8607420268991979, -0.5090414158948595}, \
{-0.857526656087815, -0.5144395339253616}, {-0.8542774315923692, \
-0.519817342763925}, {-0.8509944816866801, -0.5251746301038652}, \
{-0.8476779359759886, -0.5305111844486511}, {-0.8443279253918405, \
-0.5358267951202547}, {-0.8409445821869176, -0.5411212522674684}, \
{-0.8375280399298163, -0.5463943468741899}, {-0.8340784334997748, \
-0.5516458707676727}, {-0.8305958990813486, -0.5568756166267458}, \
{-0.8270805741590331, -0.5620833779899973}, {-0.8235325975118373, \
-0.5672689492639257}, {-0.8199521092078039, -0.5724321257310562}, \
{-0.8163392505984806, -0.5775727035580226}, {-0.8126941643133389, \
-0.5826904798036144}, {-0.809016994254144, -0.5877852524267886}, \
{-0.8053078855892735, -0.5928568202946454}, {-0.8015669847479865, \
-0.5979049831903694}, {-0.7977944394146427, -0.6029295418211333}, \
{-0.7939903985228723, -0.6079302978259659}, {-0.7901550122496962, \
-0.6129070537835825}, {-0.7862884320095971, -0.6178596132201796}, \
{-0.7823908104485423, -0.6227877806171908}, {-0.7784623014379572, \
-0.6276913614190055}, {-0.7745030600686509, -0.6325701620406501}, \
{-0.7705132426446932, -0.6374239898754297}, {-0.7664930066772446, \
-0.6422526533025326}, {-0.7624425108783371, -0.6470559616945945}, \
{-0.7583619151546095, -0.6518337254252248}, {-0.7542513806009937, \
-0.6565857558764921}, {-0.7501110694943556, -0.6613118654463712}, \
{-0.7459411452870882, -0.6660118675561484}, {-0.7417417726006593, \
-0.6706855766577882}, {-0.7375131172191121, -0.6753328082412579}, \
{-0.7332553460825203, -0.6799533788418118}, {-0.728968627280398, \
-0.6845471060472345}, {-0.7246531300450635, -0.6891138085050414}, \
{-0.7203090247449583, -0.693653305929639}, {-0.7159364828779213, \
-0.6981654191094419}, {-0.7115356770644186, -0.7026499699139476}, \
{-0.7071067810407281, -0.7071067813007691}, {-0.7026499696520814, \
-0.7115356773226241}, {-0.6981654188457609, -0.7159364831342813}, \
{-0.6936533056641536, -0.7203090249994625}, {-0.689113808237762, \
-0.7246531302977017}, {-0.6845471057781716, -0.7289686275311603}, \
{-0.679953378570976, -0.7332553463313967}, {-0.6753328079686597, \
-0.7375131174660925}, {-0.6706855763834384, -0.741741772845734}, \
{-0.6660118672800577, -0.7459411455302473}, {-0.6613118651685503, \
-0.7501110697355893}, {-0.656585755596952, -0.7542513808402924}, \
{-0.6518337251439762, -0.7583619153919636}, {-0.6470559614116485, \
-0.7624425111137371}, {-0.6422526530179, -0.7664930069106808}, \
{-0.6374239895891217, -0.7705132428761563}, {-0.6325701617526778, \
-0.7745030602981313}, {-0.6276913611293801, -0.7784623016654457}, \
{-0.6227877803259233, -0.7823908106740296}, {-0.6178596129272814, \
-0.786288432233074}, {-0.6129070534890649, -0.7901550124711535}, \
{-0.6079302975298403, -0.793990398742301}, {-0.6029295415234112, \
-0.7977944396320338}, {-0.5979049828910621, -0.8015669849633309}, \
{-0.5928568199937644, -0.8053078858025623}, {-0.5877852521243455, \
-0.8090169944653683}, {-0.5826904794996209, -0.8126941645224901}, \
{-0.5775727032524903, -0.8163392508055499}, {-0.5724321254239969, \
-0.8199521094127827}, {-0.5672689489553512, -0.8235325977147169}, \
{-0.5620833776799194, -0.8270805743598051}, {-0.5568756163151765, \
-0.8305958992800043}, {-0.5516458704546239, -0.8340784336963061}, \
{-0.5463943465596734, -0.8375280401242146}, {-0.5411212519514965, \
-0.8409445823791747}, {-0.5358267948028392, -0.8443279255819482}, \
{-0.5305111841298044, -0.8476779361639387}, {-0.5251746297835994, \
-0.8509944818724645}, {-0.5198173424422523, -0.8542774317759799}, \
{-0.5144395336022942, -0.8575266562692441}, {-0.5090414155704098, \
-0.8607420270784374}, {-0.503623201455056, -0.863923417266226}, \
{-0.49818510515804854`, -0.8670707012367727}, {-0.4927273413661178, \
-0.8701837547406952}, {-0.487250125542433, -0.8732624548799717}, \
{-0.4817536739180965, -0.8763066801127922}, {-0.4762382034836069, \
-0.8793163102583571}, {-0.47070393198029303`, -0.8822912265016215}, \
{-0.4651510778917176, -0.8852313113979859}, {-0.4595798604350521, \
-0.888136448877933}, {-0.45399049955242227`, -0.8910065242516092}, \
{-0.4483832159022252, -0.8938414242133533}, {-0.4427582308504182, \
-0.8966410368461689}, {-0.43711576646177963`, -0.8994052516261429}, \
{-0.43145604549114214`, -0.9021339594268089}, {-0.42577929137459863`, \
-0.9048270525234551}, {-0.4200857282206815, -0.9074844245973774}, \
{-0.41437558080151515`, -0.9101059707400766}, {-0.4086490745439423, \
-0.9126915874573996}, {-0.40290643552062466`, -0.9152411726736258}, \
{-0.397147890441118, -0.9177546257354964}, {-0.3913736666429218, \
-0.9202318474161881}, {-0.38558399208250455`, -0.9226727399192308}, \
{-0.37977909532630444`, -0.9250772068823678}, {-0.37395920554170586`, \
-0.9274451533813606}, {-0.3681245524879922, -0.9297764859337359}, \
{-0.36227536650727554`, -0.9320711125024763}, {-0.356411878515403, \
-0.9343289424996541}, {-0.3505343199928407, -0.9365498867900071}, \
{-0.344642922975535, -0.9387338576944573}, {-0.33873792004575254`, \
-0.940880768993573}, {-0.33281954432289795`, -0.9429905359309723}, \
{-0.32688802945431095`, -0.9450630752166689}, {-0.3209436096060422, \
-0.9470983050303605}, {-0.3149865194536088, -0.9490961450246591}, \
{-0.3090169941727298, -0.9510565163282625}, {-0.30303526943004194`, \
-0.9529793415490682}, {-0.29704158137379566`, -0.9548645447772289}, \
{-0.2910361666245326, -0.9567120515881493}, {-0.2850192622657441, \
-0.9585217890454237}, {-0.27899110583451175`, -0.9602936857037162}, \
{-0.2729519353121296, -0.9620276716115806}, {-0.26690198911470914`, \
-0.9637236783142225}, {-0.2608415060837671, -0.965381638856201}, \
{-0.25477072547679636`, -0.967001487784073}, {-0.24868988695782046`, \
-0.9685831611489764}, {-0.2425992305879321, -0.9701265965091549}, \
{-0.23649899681581585`, -0.9716317329324233}, {-0.2303894264682558, \
-0.9730985109985729}, {-0.22427076074062804`, -0.9745268728017171}, \
{-0.21814324118737868`, -0.9759167619525777}, {-0.2120071097124877, \
-0.9772681235807108}, {-0.205862608559919, -0.9785809043366732}, \
{-0.19970998030405704`, -0.9798550523941286}, {-0.19354946784013036`, \
-0.9810905174518934}, {-0.18738131437462258`, -0.9822872507359224}, \
{-0.18120576341567096`, -0.9834452050012347}, {-0.17502305876345317`, \
-0.9845643345337787}, {-0.16883344450056237`, -0.9856445951522367}, \
{-0.16263716498237138`, -0.9866859442097692}, {-0.15643446482738588`, \
-0.9876883405956984}, {-0.15022558890758725`, -0.9886517447371315}, \
{-0.14401078233876555`, -0.9895761186005224}, {-0.13779029047084265`, \
-0.9904614256931739}, {-0.13156435887818635`, -0.9913076310646777}, \
{-0.12533323334991542`, -0.9921147013082948}, {-0.11909715988019635`, \
-0.992882604562274}, {-0.11285638465853189`, -0.9936113105111094}, \
{-0.10661115406004186`, -0.9943007903867381}, {-0.10036171463573679`, \
-0.9949510169696751}, {-0.09410831310278439, -0.9955619645900884}, \
{-0.08785119633476968, -0.9961336091288119}, {-0.08159061135194876, \
-0.996665928018298}, {-0.07532680531149694, -0.9971589002435085}, \
{-0.06906002549775135, -0.997612506342744}, {-0.06279051931244863, \
-0.9980267284084124}, {-0.056518534264957856`, -0.9984015500877359}, \
{-0.05024431796250938, -0.9987369565833962}, {-0.04396811810041966, \
-0.9990329346541194}, {-0.03769018245231268, -0.9992894726151978}, \
{-0.03141075886033832, -0.9995065603389522}, {-0.025130095225387894`, \
-0.9996841892551309}, {-0.01884843949730752, -0.9998223523512484}, \
{-0.01256603966510944, -0.9999210441728622}, {-0.006283143747181858, \
-0.9999802608237882}, {2.185024123441126*^-10, -0.9999999999662544}, {
      0.006283144184178051, -0.9999802608209933}, {
      0.012566040102079741`, -0.9999210441672726}, {
      0.018848439934234663`, -0.9998223523428642}, {
      0.025130095662254624`, -0.9996841892439524}, {
      0.031410759297127375`, -0.9995065603249799}, {
      0.037690182889006815`, -0.9992894725984321}, {
      0.04396811853700161, -0.999032934634561}, {
      0.05024431839896191, -0.9987369565610459}, {
      0.056518534701263716`, -0.9984015500625945}, {
      0.06279051974859058, -0.9980267283804809}, {
      0.06906002593371217, -0.9976125063120234}, {
      0.07532680574725939, -0.99715890021}, {
      0.08159061178749563, -0.9966659279820028}, {
      0.08785119677008377, -0.9961336090897315}, {
      0.0941083135378485, -0.9955619645482242}, {
      0.10036171507053372`, -0.9949510169250289}, {
      0.10661115449455444`, -0.9943007903393114}, {
      0.11285638509274293`, -0.993611310460904}, {
      0.1190971603140887, -0.9928826045092918}, {
      0.12533323378347191`, -0.992114701252538}, {
      0.13156435931138985`, -0.9913076310061483}, {
      0.13779029090367606`, -0.990461425631874}, {
      0.14401078277121174`, -0.9895761185364546}, {
      0.15022558933962915`, -0.988651744670298}, {
      0.1564344652590064, -0.987688340526102}, {
      0.16263716541355344`, -0.9866859441374124}, {
      0.1688334449312889, -0.9856445950771223}, {
      0.17502305919370714`, -0.9845643344559095}, {
      0.18120576384543538`, -0.9834452049206137}, {
      0.18738131480388043`, -0.9822872506525526}, {
      0.19354946826886466`, -0.9810905173657781}, {
      0.19970998073225085`, -0.9798550523052711}, {
      0.2058626089875554, -0.9785809042450769}, {
      0.21200711013954976`, -0.9772681234863791}, {
      0.2181432416138495, -0.9759167618555142}, {
      0.22427076116649072`, -0.9745268727019256}, {
      0.2303894268934935, -0.9730985108960571}, {
      0.23649899724041173`, -0.9716317328271872}, {
      0.24259923101186937`, -0.9701265964012024}, {
      0.24868988738108233`, -0.9685831610383115}, {
      0.25477072589936606`, -0.9670014876707002}, {
      0.2608415065056279, -0.9653816387401245}, {
      0.2669019895358443, -0.9637236781954467}, {
      0.2729519357325225, -0.9620276714901101}, {
      0.27899110625414575`, -0.9602936855795556}, {
      0.28501926268460254`, -0.9585217889185778}, {
      0.2910361670425988, -0.9567120514586228}, {
      0.29704158179105317`, -0.9548645446450269}, {
      0.3030352698464742, -0.9529793414141957}, {
      0.3090169945883203, -0.9510565161907247}, {
      0.31498651986834103`, -0.9490961448844614}, {
      0.32094361001989974`, -0.9470983048875081}, {
      0.3268880298672774, -0.9450630750711672}, {
      0.3328195447349569, -0.9429905357828269}, {
      0.33873792045688766`, -0.9408807688427896}, {
      0.34464292338573005`, -0.9387338575410417}, {
      0.3505343204020794, -0.9365498866339651}, {
      0.3564118789236691, -0.9343289423409918}, {
      0.36227536691455275`, -0.9320711123411997}, {
      0.3681245528942644, -0.929776485769851}, {
      0.37395920594695686`, -0.9274451532148738}, {
      0.3797790957305182, -0.9250772067132854}, {
      0.38558399248566505`, -0.9226727397475593}, {
      0.391373667045013, -0.9202318472419341}, {
      0.3971478908421239, -0.9177546255586665}, {
      0.4029064359205293, -0.9152411724942269}, {
      0.40864907494272984`, -0.9126915872754385}, {
      0.41437558119916973`, -0.9101059705555602}, {
      0.42008572861718735`, -0.9074844244103129}, {
      0.42577929176993995`, -0.9048270523338494}, {
      0.4314560458853032, -0.9021339592346694}, {
      0.43711576685474474`, -0.8994052514314771}, {
      0.44275823124217173`, -0.8966410366489841}, {
      0.4483832162927515, -0.8938414240136571}, {
      0.45399049994170587`, -0.8910065240494092}, {
      0.4595798608230775, -0.8881364486732368}, {
      0.4651510782784693, -0.8852313111908017}, {
      0.4707039323657556, -0.882291226291957}, {
      0.476238203867765, -0.8793163100462205}, {
      0.4817536743009348, -0.8763066798981916}, {
      0.4872501259239363, -0.8732624546629152}, {
      0.4927273417462708, -0.8701837545211911}, {
      0.4981851055368362, -0.8670707010148294}, {
      0.5036232018324631, -0.8639234170418522}, {
      0.5090414159464214, -0.8607420268516416}, {
      0.5144395339768952, -0.857526656040035}, {
      0.5198173428154278, -0.8542774315443664}, {
      0.5251746301553345, -0.8509944816384554}, {
      0.5305111845000842, -0.8476779359275429}, {
      0.535826795171649, -0.8443279253431748}, {
      0.5411212523188215, -0.8409445821380329}, {
      0.5463943469254989, -0.8375280398807137}, {
      0.5516458708189352, -0.8340784334504554}, {
      0.5568756166779592, -0.8305958990318133}, {
      0.5620833780411589, -0.8270805741092833}, {
      0.567268949315033, -0.8235325974618739}, {
      0.5724321257821067, -0.8199521091576283}, {
      0.5775727036090137, -0.8163392505480939}, {
      0.5826904798545436, -0.8126941642627423}, {
      0.5877852524776532, -0.8090169942033388}, {
      0.5928568203454431, -0.805307885538261}, {
      0.5979049832410976, -0.801566984696768}, {
      0.6029295418717896, -0.7977944393632196}, {
      0.6079302978765476, -0.793990398471246}, {
      0.6129070538340873, -0.790155012197868}, {
      0.6178596132706049, -0.7862884319575685}, {
      0.6227877806675342, -0.7823908103963146}, {
      0.6276913614692645, -0.7784623013855321}, {
      0.6325701620908222, -0.7745030600160298}, {
      0.6374239899255125, -0.7705132425918777}, {
      0.6422526533525237, -0.7664930066242361}, {
      0.6470559617444915, -0.7624425108251374}, {
      0.6518337254750254, -0.75836191510122}, {
      0.656585755926194, -0.754251380547416}, {
      0.6613118654959718, -0.7501110694405914}, {
      0.6660118676056453, -0.7459411452331393}, {
      0.6706855767071792, -0.7417417725465273}, {
      0.6753328082905407, -0.7375131171647987}, {
      0.6799533788909841, -0.7332553460280273}, {
      0.6845471060962939, -0.7289686272257272}, {
      0.6891138085539857, -0.7246531299902166}, {
      0.693653305978466, -0.7203090246899372}, {
      0.6981654191581492, -0.7159364828227278}, {
      0.702649969962533, -0.7115356770090545}, {
      0.7071067813492306, -0.7071067809851954}, {
      0.7115356773709594, -0.702649969596382}, {
      0.715936483182488, -0.6981654187898968}, {
      0.7203090250475386, -0.6936533056081267}, {
      0.7246531303456452, -0.6891138081815742}, {
      0.7289686275789689, -0.6845471057218249}, {
      0.7332553463790682, -0.6799533785144725}, {
      0.737513117513625, -0.6753328079120013}, {
      0.7417417728931255, -0.6706855763266273}, {
      0.7459411455774957, -0.6660118672230959}, {
      0.7501110697826926, -0.66131186511144}, {
      0.7542513808872486, -0.6565857555396953}, {
      0.7583619154387706, -0.6518337250865752}, {
      0.762442511160393, -0.6470559613541054}, {
      0.7664930069571837, -0.642252652960217}, {
      0.7705132429225042, -0.6374239895313011}, {
      0.7745030603443224, -0.6325701616947217}, {
      0.778462301711478, -0.6276913610712908}, {
      0.7823908107199014, -0.6227877802677031}, {
      0.7862884322787833, -0.6178596128689325}, {
      0.7901550125166986, -0.6129070534305896}, {
      0.7939903987876801, -0.6079302974712408}, {
      0.797794439677245, -0.60292954146469}, {
      0.8015669850083725, -0.5979049828322216}, {
      0.8053078858474327, -0.592856819934807}, {
      0.8090169945100658, -0.5877852520652734}, {
      0.8126941645670129, -0.5826904794404367}, {
      0.8163392508498964, -0.5775727031931963}, {
      0.8199521094569513, -0.5724321253645954}, {
      0.823532597758706, -0.5672689488958448}, {
      0.827080574403613, -0.5620833776203106}, {
      0.8305958993236296, -0.5568756162554676}, {
      0.8340784337397472, -0.5516458703948174}, {
      0.8375280401674701, -0.5463943464997719}, {
      0.840944582422243, -0.5411212518915025}, {
      0.844327925624828, -0.5358267947427553}, {
      0.8476779362066285, -0.5305111840696329}, {
      0.8509944819149629, -0.5251746297233431}, {
      0.8542774318182855, -0.5198173423819137}, {
      0.8575266563113555, -0.5144395335418759}, {
      0.8607420271203534, -0.5090414155099143}, {
      0.8639234173079453, -0.5036232013944859}, {
      0.867070701278294, -0.49818510509740654`}, {
      0.8701837547820173, -0.49272734130540646`}, {
      0.8732624549210932, -0.487250125481655}, {
      0.876306680153712, -0.4817536738572545}, {
      0.8793163102990741, -0.4762382034227035}, {
      0.8822912265421344, -0.4707039319193309}, {
      0.8852313114382938, -0.4651510778306995}, {
      0.8881364489180347, -0.4595798603739806}, {
      0.8910065242915037, -0.4539904994913}, {
      0.8938414242530396, -0.4483832158410549}, {
      0.8966410368856459, -0.4427582307892026}, {
      0.8994052516654096, -0.43711576640052147`}, {
      0.9021339594658644, -0.43145604542984406`}, {
      0.9048270525622986, -0.4257792913132634}, {
      0.907484424636008, -0.42008572815931183`}, {
      0.9101059707784933, -0.41437558074011377`}, {
      0.9126915874956016, -0.408649074482512}, {
      0.9152411727116123, -0.40290643545916816`}, {
      0.9177546257732666, -0.397147890379638}, {
      0.9202318474537413, -0.39137366658142103`}, {
      0.9226727399565663, -0.3855839920209858}, {
      0.9250772069194849, -0.3797790952647705}, {
      0.9274451534182586, -0.37395920548015943`}, {
      0.9297764859704142, -0.36812455242643605`}, {
      0.9320711125389344, -0.36227536644571245`}, {
      0.9343289425358913, -0.35641187845383576`}, {
      0.9365498868260227, -0.350534319931272}, {
      0.9387338577302509, -0.34464292291396764`}, {
      0.9408807690291441, -0.3387379199841893}, {
      0.9429905359663204, -0.3328195442613416}, {
      0.9450630752517937, -0.32688802939276423`}, {
      0.9470983050652616, -0.3209436095445079}, {
      0.949096145059336, -0.3149865193920897}, {
      0.9510565163627148, -0.30901699411122874`}, {
      0.9529793415832957, -0.30303526936856157`}, {
      0.9548645448112313, -0.29704158131233876`}, {
      0.9567120516219262, -0.29103616656310194`}, {
      0.958521789078975, -0.2850192622043425}, {
      0.9602936857370417, -0.2789911057731419}, {
      0.9620276716446801, -0.27295193525079425`}, {
      0.9637236783470957, -0.26690198905341106`}, {
      0.965381638888848, -0.26084150602250905`}, {
      0.9670014878164935, -0.25477072541558105`}, {
      0.9685831611811704, -0.2486898868966506}, {
      0.9701265965411223, -0.24259923052681043`}, {
      0.9716317329641642, -0.23649899675474512`}, {
      0.9730985110300872, -0.23038942640723872`}, {
      0.9745268728330048, -0.2242707606796673}, {
      0.9759167619836389, -0.21814324112647698`}, {
      0.9772681236115456, -0.21200710965164773`}, {
      0.9785809043672817, -0.20586260849914345`}, {
      0.979855052424511, -0.1997099802433486}, {
      0.9810905174820497, -0.19354946777949172`}, {
      0.9822872507658529, -0.1873813143140564}, {
      0.9834452050309396, -0.18120576335517993`}, {
      0.9845643345632582, -0.1750230587030399}, {
      0.9856445951814912, -0.16883344444022952`}, {
      0.9866859442387989, -0.16263716492212157`}, {
      0.9876883406245036, -0.15643446476722173`}, {
      0.9886517447657125, -0.15022558884751136`}, {
      0.9895761186288796, -0.1440107822787805}, {
      0.9904614257213077, -0.13779029041095106`}, {
      0.9913076310925887, -0.1315643588183908}, {
      0.9921147013359833, -0.12533323329021848`}, {
      0.9928826045897404, -0.11909715982060057`}, {
      0.9936113105383544, -0.1128563845990398}, {
      0.9943007904137621, -0.10661115400065599`}, {
      0.9949510169964787, -0.10036171457645965`}, {
      0.995561964616672, -0.0941083130436185}, {
      0.9961336091551762, -0.08785119627571751}, {
      0.9966659280444438, -0.0815906112930128}, {
      0.9971589002694362, -0.07532680525267965}, {
      0.9976125063684543, -0.06906002543905519}, {
      0.9980267284339062, -0.06279051925387602}, {
      0.9984015501130137, -0.056518534206511234`}, {
      0.9987369566084588, -0.05024431790419114}, {
      0.9990329346789675, -0.0439681180422322}, {
      0.9992894726398323, -0.03769018239425838}, {
      0.9995065603633737, -0.031410758802419524`}, {
      0.9996841892793402, -0.02513009516760695}, {
      0.9998223523752464, -0.018848439439666752`}, {
      0.9999210441966498, -0.01256603960761116}, {
      0.9999802608473664, -0.006283143689828356}, {0.999999999989624, 
      2.7570885852146176`*^-10}, {0.9999802608441551, 
      0.006283144241235183}}]}, 
   {RGBColor[1, 1, 0], DiskBox[{0, 0}, 0.03]}},
  AspectRatio->Automatic,
  Background->RGBColor[0.6, 0.6, 0.6],
  ImageSize->{293., Automatic}]], "Output",
 CellChangeTimes->{3.401193391435135*^9, 3.401193501314971*^9, 
  3.401194908766206*^9, 3.401276879728649*^9}]
}, Open  ]],

Cell[TextData[{
 "Try ",
 Cell[BoxData[
  FormBox[
   RowBox[{"V0", "=", 
    RowBox[{"0.7", "\[Times]", "2", " ", "\[Pi]"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"V0", "=", 
    RowBox[{"1.3", "\[Times]", "2", " ", "\[Pi]"}]}], TraditionalForm]]],
 " and show that the orbits do not close.  For the second case increase the \
",
 Cell[BoxData[
  FormBox["tFinal", TraditionalForm]]],
 " to 5 to see the pattern more clearly."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tFinal", "=", "2"}], ";", 
  RowBox[{"R0", "=", "1"}], ";", 
  RowBox[{"V0", "=", 
   RowBox[{".7", "\[Times]", "2", " ", "\[Pi]"}]}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", ".001"}], ";", 
  RowBox[{"p", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CalculateOrbit", ";", 
  RowBox[{"DisplayOrbit", "[", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Clear", "[", 
  RowBox[{
  "tFinal", ",", "R0", ",", "V0", ",", "iMaximum", ",", "t", ",", "r", ",", 
   "v"}], "]"}]}], "Input",
 CellChangeTimes->{{3.401193360004507*^9, 3.401193404706635*^9}, 
   3.4011936086606703`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 1, 1], Thickness[0.0075], 
    LineBox[{{1, 0}, {0.999980260789899, 0.004398200775688474}, {
      0.9999210431440207, 0.008796227912697236}, {0.9998223470156984, 
      0.01319390775822481}, {0.9996841723273524, 0.017591066631223833`}, {
      0.999506518970789, 0.02198753080827218}, {0.9992893868076181, 
      0.026383126509436992`}, {0.999032775669791, 0.030777679884129227`}, {
      0.9987366853602577, 0.035171016996946365`}, {0.9984011156537462, 
      0.03956296381350089}, {0.9980260662976613, 0.0439533461862322}, {
      0.9976115370131059, 0.04834198984019952}, {0.9971575274960242, 
      0.05272872035885352}, {0.9966640374184683, 0.0571133631697842}, {
      0.9961310664299882, 0.061495743530442654`}, {0.9955586141591459, 
      0.06587568651383442}, {0.9949466802151566, 0.07025301699418195}, {
      0.9942952641896547, 0.07462755963255381}, {0.9936043656585887, 
      0.07899913886245835}, {0.992873984184244, 0.08336757887539926}, {
      0.9921041193173974, 0.08773270360639078}, {0.9912947705996011, 
      0.09209433671943014}, {0.9904459375656011, 0.09645230159292477}, {
      0.989557619745889, 0.10080642130507195`}, {0.9886298166693906, 
      0.1051565186191885}, {0.9876625278662922, 0.10950241596898805`}, {
      0.986655752871005, 0.11384393544380353`}, {0.9856094912252731, 
      0.1181808987737525}, {0.9845237424814223, 0.12251312731484285`}, {
      0.9833985062057563, 0.1268404420340165}, {0.9822337819820992, 
      0.13116266349412875`}, {0.9810295694154878, 0.13547961183886056`}, {
      0.9797858681360162, 0.1397911067775619}, {0.9785026778028347, 
      0.14409696757002324`}, {0.9771799981083055, 0.148397013011173}, {
      0.9758178287823179, 0.1526910614156984}, {0.9744161695967667, 
      0.1569789306025877}, {0.972975020370194, 0.1612604378795906}, {
      0.9714943809726009, 0.16553540002759548`}, {0.969974251330429, 
      0.1698036332849199}, {0.9684146314317171, 0.1740649533315129}, {
      0.9668155213314348, 0.17831917527306615`}, {0.9651769211569972, 
      0.18256611362503167`}, {0.9634988311139643, 0.18680558229654376`}, {
      0.9617812514919284, 0.19103739457424276`}, {0.9600241826705934, 
      0.19526136310599804`}, {0.9582276251260498, 0.19947729988452811`}, {
      0.9563915794372488, 0.20368501623091523`}, {0.954516046292682, 
      0.20788432277801233`}, {0.9526010264972674, 0.21207502945373963`}, {
      0.9506465209794491, 0.2162569454642689}, {0.9486525307985142, 
      0.22042987927709273`}, {0.9466190571521307, 0.22459363860397663`}, {
      0.9445461013841134, 0.22874803038379166`}, {0.9424336649924208, 
      0.23289286076522503`}, {0.9402817496373884, 0.23702793508936684`}, {
      0.9380903571502061, 0.24115305787217015`}, {0.935859489541641, 
      0.24526803278678252`}, {0.9335891490110158, 0.24937266264574667`}, {
      0.9312793379554444, 0.25346674938306796`}, {0.9289300589793346, 
      0.2575500940361465}, {0.926541314904161, 0.26162249672757215`}, {
      0.9241131087785164, 0.2656837566467795}, {0.9216454438884467, 
      0.2697336720315617}, {0.9191383237680782, 0.2737720401494399}, {
      0.916591752210542, 0.2777986572788879}, {0.9140057332792036, 
      0.28181331869040815`}, {0.9113802713192062, 0.2858158186274588}, {
      0.9087153709693324, 0.289805950287229}, {0.9060110371741958, 
      0.29378350580126095`}, {0.9032672751967673, 0.2977482762159167}, {
      0.9004840906312459, 0.3017000514726885}, {0.8976614894162828, 
      0.30563862038835043`}, {0.8947994778485662, 0.30956377063495005`}, {
      0.8918980625967775, 0.31347528871963865`}, {0.8889572507159264, 
      0.3173729599643386}, {0.8859770496620754, 0.3212565684852463}, {
      0.8829574673074635, 0.3251258971721697}, {0.8798985119560391, 
      0.3289807276676993}, {0.8768001923594112, 0.33282084034621107`}, {
      0.8736625177332321, 0.3366460142927012}, {0.8704854977740195, 
      0.340456027281451}, {0.8672691426764315, 0.3442506557545219}, {
      0.8640134631510048, 0.3480296748000794}, {0.8607184704423685, 
      0.3517928581305462}, {0.8573841763479453, 0.35553997806058363`}, {
      0.8540105932371526, 0.3592708054849014}, {0.8505977340711166, 
      0.36298510985589616`}, {0.847145612422913, 0.36668265916111814`}, {
      0.8436542424983456, 0.37036321990056725`}, {0.8401236391572807, 
      0.3740265570638186}, {0.836553817935547, 0.3776724341069785}, {
      0.832944795067419, 0.38130061292947154`}, {0.8292965875086984, 
      0.38491085385066076`}, {0.8256092129604063, 0.3885029155863016}, {
      0.8218826898931069, 0.39207655522483187`}, {0.8181170375718732, 
      0.3956315282034993}, {0.8143122760819169, 0.3991675882843294}, {
      0.8104684263548958, 0.4026844875299363}, {0.8065855101959175, 
      0.40618197627917857`}, {0.8026635503112581, 0.4096598031226646}, {
      0.7987025703368132, 0.41311771487810994`}, {0.7947025948673013, 
      0.4165554565655515}, {0.7906636494862378, 0.41997277138242206`}, {
      0.7865857607967011, 0.42336940067849094`}, {0.7824689564529093, 
      0.42674508393067484`}, {0.7783132651926304, 0.4300995587177257}, {
      0.7741187168704464, 0.43343256069480063`}, {0.7698853424918927, 
      0.4367438235679214}, {0.765613174248497, 0.4400330790683302}, {
      0.7613022455537394, 0.4433000569267497}, {0.7569525910799578, 
      0.4465444848475551}, {0.752564246796223, 0.4497660884828684}, {
      0.7481372500072073, 0.45296459140658296`}, {0.7436716393930737, 
      0.4561397150883294}, {0.7391674550504095, 0.45929117886739423`}, {
      0.7346247385342326, 0.46241869992660184`}, {0.7300435329010964, 
      0.4655219932661734}, {0.7254238827533217, 0.46860077167757513`}, {
      0.7207658342843837, 0.47165474571737126`}, {0.7160694353254824, 
      0.4746836236810953}, {0.7113347353933275, 0.47768711157715715`}, {
      0.706561785739166, 0.480664913100802}, {0.7017506393990848, 
      0.4836167296081398}, {0.6969013512456189, 0.4865422600902641}, {
      0.6920139780406978, 0.48944120114748085`}, {0.6870885784899613, 
      0.49231324696366846`}, {0.6821252132984806, 0.49515808928079186`}, {
      0.6771239452279155, 0.49797541737359513`}, {0.6720848391551442, 
      0.5007649180244975}, {0.6670079621324001, 0.50352627549872}, {
      0.6618933834489517, 0.506259171519671}, {0.6567411746943601, 
      0.5089632852446212}, {0.651551409823354, 0.5116382932406979}, {
      0.6463241652223558, 0.5142838694612343}, {0.6410595197776998, 
      0.5168996852225072}, {0.635757554945579, 0.5194854091809001}, {
      0.6304183548237597, 0.5220407073105318}, {0.6250420062251043, 
      0.5245652428813895}, {0.6196285987529394, 0.527058676438012}, {
      0.6141782248783124, 0.5295206657787648}, {0.6086909800191744, 
      0.5319508659357582}, {0.6031669626215321, 0.5343489291554561}, {
      0.597606274242608, 0.5367145048800274}, {0.5920090196360516, 
      0.5390472397294973}, {0.5863753068392412, 0.5413467774847542}, {
      0.5807052472627198, 0.5436127590714732}, {0.5749989557818045, 
      0.5458448225450194}, {0.5692565508304118, 0.5480426030763977}, {
      0.5634781544971402, 0.5502057329393187}, {0.55766389262365, 
      0.5523338414984527}, {0.5518138949053817, 0.554426555198948}, {
      0.5459282949946532, 0.5564834975572948}, {0.5400072306061755, 
      0.5585042891536146}, {0.5340508436250245, 0.560488547625466}, {
      0.5280592802171096, 0.5624358876632535}, {0.5220326909421729, 
      0.564345921007338}, {0.5159712308693588, 0.5662182564469457}, {
      0.5098750596953858, 0.5680524998209778}, {0.5037443418653551, 
      0.5698482540208318}, {0.497579246696228, 0.5716051189953428}, {
      0.491379948503001, 0.5733226917579639}, {0.4851466267276076, 
      0.5750005663963063}, {0.47887946607057136`, 0.5766383340841655}, {
      0.4725786566254336, 0.5782355830961659}, {0.4662443940159762, 
      0.5797918988251584}, {0.4598768795362565, 0.5813068638025152}, {
      0.4534763202934675, 0.5827800577214676}, {0.4470429293536348, 
      0.5842110574636398}, {0.440576925890155, 0.5855994371289373}, {
      0.4340785353351784, 0.5869447680689537}, {0.42754798953383216`, 
      0.5882466189240659}, {0.4209855269012768, 0.5895045556643932}, {
      0.41439139258258134`, 0.5907181416348014}, {0.40776583861539867`, 
      0.591886937604141}, {0.4011091240954141, 0.593010501818911}, {
      0.394421515344535, 0.5940883900615492}, {0.38770328608178095`, 
      0.5951201557135541}, {0.380954717596826, 0.5961053498236502}, {
      0.3741760989261366, 0.5970435211812131}, {0.3673677270316401, 
      0.5979342163951801}, {0.36052990698184667`, 0.5987769799786724}, {
      0.3536629521353412, 0.5995713544395664}, {0.34676718432654735`, 
      0.6003168803772521}, {0.33984293405365584`, 0.6010130965858257}, {
      0.33289054066859686`, 0.6016595401639662}, {0.3259103525689233, 
      0.6022557466317513}, {0.31890272739145764`, 0.6028012500546726}, {
      0.3118680322075411, 0.6032955831751127}, {0.3048066437197085, 
      0.6037382775515538}, {0.2977189484595959, 0.6041288637057859}, {
      0.290605342986872, 0.6044668712783882}, {0.28346623408896604`, 
      0.60475182919276}, {0.2763020389813465, 0.6049832658279757}, {
      0.26911318550808655`, 0.6051607092007399}, {0.26190011234243143`, 
      0.6052836871567226}, {0.25466326918706156`, 0.6053517275715443}, {
      0.24740311697372538`, 0.6053643585616884}, {0.24012012806189167`, 
      0.6053211087056075}, {0.2328147864360488, 0.60522150727529}, {
      0.22548758790125442`, 0.6050650844785449}, {0.2181390402765143, 
      0.6048513717122572}, {0.21076966358554394`, 0.6045799018268576}, {
      0.20337999024444087`, 0.6042502094022381}, {0.1959705652457686, 
      0.6038618310353351}, {0.18854194633852744`, 0.6034143056395874}, {
      0.18109470420345897`, 0.6029071747564597}, {0.1736294226231045, 
      0.6023399828792053}, {0.16614669864600956`, 0.601712277789023}, {
      0.15864714274443947`, 0.6010236109037378}, {0.15113137896494228`, 
      0.6002735376391144}, {0.14360004507106958`, 0.5994616177828834}, {
      0.13605379267753748`, 0.5985874158815304}, {0.12849328737508403`, 
      0.5976505016398679}, {0.12091920884525423`, 0.5966504503333733}, {
      0.11333225096431845`, 0.5955868432332401}, {0.1057331218955072, 
      0.5944592680440486}, {0.09812254416872274, 0.5932673193539189}, {
      0.09050125474686849, 0.5920105990969625}, {0.0828700050779179, 
      0.5906887170278022}, {0.07522956113182934, 0.5893012912078736}, {
      0.0675807034213993, 0.5878479485031717}, {0.05992422700613612, 
      0.586328325093045}, {0.05226094147822858, 0.5847420669895796}, {
      0.044591670929679646`, 0.5830888305670552}, {0.03691725389967586, 
      0.5813682831008857}, {0.029238543301266344`, 0.5795801033153936}, {
      0.021556406326434123`, 0.5777239819396929}, {0.013871724328655502`, 
      0.5757996222708889}, {0.006185392682061766, 
      0.5738067407437244}, {-0.0015016793836588648`, 
      0.5717450675057328}, {-0.009188568973451487, 
      0.5696143469968789}, {-0.01687433974297388, 
      0.5674143385325947}, {-0.024558042141320104`, 
      0.5651448168890438}, {-0.03223871368109508, 
      0.5628055728893662}, {-0.03991537923649194, 
      0.5603964139895888}, {-0.04758705136996212, 
      0.5579171648628102}, {-0.05525273068799932, 
      0.555367667980197}, {-0.06291140622648272, 
      0.5527477841872647}, {-0.07056205586594227, 
      0.5500573932738491}, {-0.07820364677701878, 
      0.5472963945361154}, {-0.08583513589629554, 
      0.5444647073288978}, {-0.09345547043257478, 
      0.5415622716066097}, {-0.10106358840356307`, 
      0.5385890484509265}, {-0.10865841920281427`, 
      0.5355450205834009}, {-0.11623888419665787`, 
      0.5324301928611477}, {-0.12380389735071433`, 
      0.5292445927537088}, {-0.1313523658854691, 
      0.5259882707992045}, {-0.13888319096024246`, 
      0.5226613010378699}, {-0.146395268384755, 
      0.5192637814210875}, {-0.15388748935735005`, 
      0.5157958341940464}, {-0.16135874122879262`, 
      0.5122576062501851}, {-0.16880790829042427`, 
      0.508649269455624}, {-0.1762338725853127, 
      0.5049710209418408}, {-0.18363551474089662`, 
      0.5012230833649111}, {-0.19101171482149062`, 
      0.49740570512971577`}, {-0.19836135319888354`, 
      0.49351916057760203`}, {-0.20568331143913712`, 
      0.4895637501360914}, {-0.21297647320357246`, 
      0.485539800429338}, {-0.22023972516181864`, 
      0.4814476643481661}, {-0.22747195791469446`, 
      0.47728772107865}, {-0.2346720669246004, 
      0.47306037608834417`}, {-0.24183895345101447`, 
      0.4687660610694264}, {-0.24897152548861395`, 
      0.46440523383817606`}, {-0.25606869870548693`, 
      0.45997837819038184`}, {-0.2631293973788511, 
      0.4554860037124467}, {-0.2701525553256668, 
      0.4509286455481374}, {-0.27713711682551406`, 
      0.4463068641211114}, {-0.2840820375331023, 
      0.441621244813538}, {-0.2909862853777938, 
      0.4368723976013176}, {-0.29784884144755314`, 
      0.4320609566465897}, {-0.3046687008547771, 
      0.4271875798484031}, {-0.31144487358152084`, 
      0.4222529483526015}, {-0.31817638530171044`, 
      0.4172577660221552}, {-0.32486227817802016`, 
      0.4122027588693356}, {-0.33150161163119674`, 
      0.4070886744512926}, {-0.33809346307972815`, 
      0.4019162812307452}, {-0.34463692864788176`, 
      0.3966863679036395}, {-0.35113112384027695`, 
      0.3913997426957569}, {-0.35757518418130485`, 
      0.38605723263037545`}, {-0.36396826581786595`, 
      0.3806596827691915}, {-0.37030954608406147`, 
      0.37520795542880125`}, {-0.37659822402664495`, 
      0.36970292937511867`}, {-0.38283352089021644`, 
      0.3641454989981691}, {-0.3890146805613205, 
      0.35853657346974593`}, {-0.3951409699707898, 
      0.3528770758864496}, {-0.4012116794538572, 
      0.34716794240064724`}, {-0.40722612306774003`, 
      0.3414101213418927}, {-0.41318363886657633`, 
      0.33560457233133667`}, {-0.41908358913376925`, 
      0.3297522653916301}, {-0.42492536057196345`, 
      0.32385418005478556`}, {-0.4307083644510416, 
      0.31791130447041005`}, {-0.43643203671468583`, 
      0.31192463451665875`}, {-0.4420958380461969, 
      0.30589517291618473`}, {-0.4476992538944049, 
      0.29982392835927535`}, {-0.4532417944606354, 
      0.2937119146362723}, {-0.45872299464781585`, 
      0.2875601497812701}, {-0.46414241397291756`, 
      0.2813696552289798}, {-0.4694996364440282, 
      0.2751414549865293}, {-0.4747942704034389, 
      0.2688765748218529}, {-0.4800259483382068, 
      0.2625760414701987}, {-0.48519432665972156`, 
      0.2562408818601577}, {-0.4902990854538601, 
      0.2498721223604894}, {-0.49533992820335676`, 
      0.24347078804889172`}, {-0.5003165814840538, 
      0.23703790200373406`}, {-0.5052287946367179, 
      0.2305744846196455}, {-0.510076339416125, 
      0.22408155294772497`}, {-0.5148590096191207, 
      0.21756012006101766`}, {-0.5195766206933584, 
      0.2110111944457822}, {-0.5242290093284071, 
      0.20443577941895805`}, {-0.5288160330309003, 
      0.197834872572131}, {-0.5333375696853722, 
      0.19120946524218843`}, {-0.5377935171023918, 
      0.18456054200875469`}, {-0.5421837925555707, 
      0.17788908021840133`}, {-0.5465083323089726, 
      0.1711960495355369}, {-0.5507670911364068, 
      0.1644824115197965}, {-0.5549600418340358, 
      0.15774911922967363`}, {-0.5590871747276692, 
      0.15099711685206454`}, {-0.5631484971760602, 
      0.1442273393573293}, {-0.5671440330714599, 
      0.13744071217941348`}, {-0.5710738223386221, 
      0.13063815092052095`}, {-0.5749379204333883, 
      0.12382056107977918`}, {-0.5787363978419178, 
      0.11698883780529647`}, {-0.5824693395815657, 
      0.11014386566897297`}, {-0.5861368447043444, 
      0.10328651846339564`}, {-0.589739025803843, 
      0.09641765902012034}, {-0.5932760085264142, 
      0.08953813904862229}, {-0.596747931087378, 
      0.0826487989951783}, {-0.6001549437929293, 
      0.07575046792093125}, {-0.6034972085683785, 
      0.06884396339837773}, {-0.6067748984932965, 
      0.0619300914255141}, {-0.6099881973440804, 
      0.0550096463568745}, {-0.6131372991444018, 
      0.0480834108506947}, {-0.6162224077239501, 
      0.041152155831440315`}, {-0.6192437362858306, 
      0.03421664046694374}, {-0.6222015069829367, 
      0.02727761215940375}, {-0.6250959505035631, 
      0.020335806549512267`}, {-0.6279273056664908, 
      0.013391947532986026`}, {-0.6306958190257332, 
      0.0064467472887954295`}, {-0.6334017444850935, -0.0004990936816010542}, \
{-0.6360453429226502, -0.007444886504691205}, {-0.6386268818252557, \
-0.014389953875667966`}, {-0.6411466349330974, -0.021333629990149535`}, \
{-0.6436048818943498, -0.02827526046747722}, {-0.646001907929913, \
-0.035214202266187024`}, {-0.6483380035082117, -0.04214982359222999}, \
{-0.6506134640300077, -0.049081503800495044`}, {-0.6528285895231554, \
-0.05600863329016654}, {-0.6549836843472131, -0.0629306133944273}, \
{-0.6570790569078075, -0.06984685626499641}, {-0.6591150193806294, \
-0.0767567847519696}, {-0.6610918874449292, -0.0836598322794091}, \
{-0.6630099800263669, -0.09055544271710858}, {-0.6648696190490603, \
-0.09744307024893885}, {-0.6666711291966659, -0.10432217923815905`}, \
{-0.6684148376823204, -0.11119224409005912`}, {-0.6701010740272604, \
-0.11805274911227946`}, {-0.6717301698479361, -0.12490318837313531`}, \
{-0.6733024586514262, -0.13174306555825502`}, {-0.6748182756389597, \
-0.13857189382582374`}, {-0.6762779575173462, -0.14538919566070707`}, \
{-0.6776818423181142, -0.1521945027277124}, {-0.6790302692241555, \
-0.15898735572423017`}, {-0.6803235784036724, -0.16576730423248173`}, \
{-0.681562110851225, -0.17253390657158565`}, {-0.6827462082356757, \
-0.17928672964964062`}, {-0.6838762127548286, -0.18602534881600902`}, \
{-0.6849524669965636, -0.19274934771397267`}, {-0.6859753138062658, \
-0.1994583181339201}, {-0.6869450961603529, -0.206151859867212}, \
{-0.687862157045708, -0.2128295805608615}, {-0.6887268393448228, \
-0.2194910955731543}, {-0.6895394857264664, -0.22613602783032388`}, \
{-0.6903004385416899, -0.23276400768438749`}, {-0.6910100397249882, \
-0.2393746727722391}, {-0.6916686307004386, -0.24596766787608737`}, \
{-0.6922765522926406, -0.25254264478531807`}, {-0.6928341446422881, \
-0.25909926215985274`}, {-0.6933417471262046, -0.26563718539506853`}, \
{-0.6937996982816792, -0.2721560864883364}, {-0.694208335734945, \
-0.2786556439072294}, {-0.6945679961336427, -0.28513554245944617`}, \
{-0.6948790150831211, -0.29159547316448864`}, {-0.6951417270864257, \
-0.2980351331271287}, {-0.6953564654878343, -0.3044542254126927}, \
{-0.6955235624197991, -0.3108524589241879}, {-0.6956433487531636, \
-0.3172295482812914}, {-0.6957161540505211, -0.3235852137012174}, \
{-0.6957423065225901, -0.3299191808814746}, {-0.6957221329874834, \
-0.3362311808845228}, {-0.6956559588327539, -0.34252095002433375`}, \
{-0.6955441079800992, -0.3487882297548587}, {-0.6953869028526183, \
-0.35503276656040195`}, {-0.6951846643445092, -0.3612543118478975}, \
{-0.694937711793107, -0.36745262184108307`}, {-0.6946463629531607, \
-0.37362745747656323`}, {-0.6943109339732527, -0.37977858430175326`}, \
{-0.6939317393742682, -0.38590577237469037`}, {-0.6935090920298241, \
-0.39200879616570006`}, {-0.6930433031485718, -0.3980874344609023}, \
{-0.6925346822582886, -0.40414147026754094`}, {-0.691983537191681, \
-0.4101706907211196}, {-0.6913901740738176, -0.41617488699432414`}, \
{-0.6907548973111218, -0.4221538542077126}, {-0.69007800958185, \
-0.428107391342152}, {-0.6893598118279866, -0.4340353011529798}, \
{-0.6886006032484911, -0.4399373900858686}, {-0.6878006812938323, \
-0.4458134681943702}, {-0.6869603416617476, -0.4516633490591165}, \
{-0.6860798782941713, -0.45748684970865267`}, {-0.6851595833752732, \
-0.4632837905418784}, {-0.6841997473305537, -0.46905399525207275`}, \
{-0.683200658826944, -0.47479729075247734`}, {-0.6821626047738617, \
-0.48051350710341306`}, {-0.6810858703251721, -0.4862024774409046}, \
{-0.6799707388820115, -0.49186403790678795`}, {-0.6788174920964278, \
-0.4974980275802745}, {-0.6776264098757951, -0.5031042884109471}, \
{-0.6763977703879633, -0.5086826651531624}, {-0.6751318500671032, \
-0.5142330053018334}, {-0.6738289236202105, -0.5197551590295675}, \
{-0.6724892640342326, -0.525248979125135}, {-0.6711131425837846, \
-0.5307143209332426}, {-0.6697008288394218, -0.5361510422955863}, \
{-0.6682525906764378, -0.5415590034931608}, {-0.6667686942841582, \
-0.546938067189799}, {-0.6652494041757014, -0.5522880983769188}, \
{-0.66369498319818, -0.5576089643194523}, {-0.6621056925433149, \
-0.5629005345029335}, {-0.66048179175844, -0.5681626805817226}, \
{-0.6588235387578717, -0.5733952763283413}, {-0.6571311898346208, \
-0.5785981975838987}, {-0.6554049996724263, -0.5837713222095835}, \
{-0.6536452213580889, -0.5889145300392007}, {-0.6518521063940862, \
-0.5940277028327318}, {-0.6500259047114488, -0.5991107242308954}, \
{-0.648166864682882, -0.6041634797106887}, {-0.6462752331361128, \
-0.6091858565418871}, {-0.6443512553674493, -0.614177743744483}, \
{-0.6423951751555346, -0.6191390320470429}, {-0.6404072347752812, \
-0.6240696138459629}, {-0.6383876750119722, -0.6289693831656034}, \
{-0.6363367351755163, -0.6338382356192831}, {-0.6342546531148416, \
-0.6386760683711152}, {-0.6321416652324201, -0.6434827800986653}, \
{-0.6299980064989072, -0.6482582709564141}, {-0.6278239104678891, \
-0.6530024425400069}, {-0.6256196092907241, -0.6577151978512733}, \
{-0.6233853337314718, -0.6623964412639974}, {-0.6211213131818976, \
-0.6670460784904259}, {-0.618827775676546, -0.6716640165484933}, \
{-0.6165049479078731, -0.6762501637297522}, {-0.6141530552414307, \
-0.6808044295679897}, {-0.6117723217310952, -0.6853267248085163}, \
{-0.6093629701343329, -0.689816961378112}, {-0.6069252219274968, \
-0.6942750523556144}, {-0.6044592973211468, -0.6987009119431348}, \
{-0.6019654152753892, -0.7030944554378877}, {-0.5994437935152279, \
-0.7074555992046206}, {-0.5968946485459243, -0.7117842606486295}, \
{-0.5943181956683591, -0.716080358189348}, {-0.5917146489943921, \
-0.7203438112344962}, {-0.5890842214622165, -0.7245745401547775}, \
{-0.586427124851702, -0.7287724662591103}, {-0.583743569799724, \
-0.7329375117703825}, {-0.5810337658154754, -0.7370695998017176}, \
{-0.5782979212957572, -0.7411686543332404}, {-0.575536243540245, \
-0.7452346001893311}, {-0.5727489387667293, -0.7492673630163562}, \
{-0.569936212126325, -0.7532668692608671}, {-0.5670982677186506, \
-0.7572330461482534}, {-0.5642353086069724, -0.7611658216618432}, \
{-0.5613475368333123, -0.7650651245224384}, {-0.5584351534335186, \
-0.7689308841682757}, {-0.5554983584522961, -0.7727630307354048}, \
{-0.5525373509581958, -0.7765614950384726}, {-0.5495523290585612, \
-0.780326208551906}, {-0.5465434899144318, -0.7840571033914832}, \
{-0.5435110297554007, -0.7877541122962849}, {-0.540455143894427, \
-0.7914171686110184}, {-0.5373760267426015, -0.7950462062687037}, \
{-0.5342738718238654, -0.7986411597737159}, {-0.5311488717896802, \
-0.8022019641851744}, {-0.5280012184336499, -0.8057285551006724}, \
{-0.524831102706095, -0.8092208686403382}, {-0.5216387147285766, \
-0.8126788414312213}, {-0.518424243808372, -0.8161024105919967}, \
{-0.5151878784529015, -0.8194915137179796}, {-0.5119298063841052, \
-0.8228460888664436}, {-0.5086502145527723, -0.8261660745422373}, \
{-0.50534928915282, -0.8294514096836906}, {-0.5020272156355258, \
-0.8327020336488058}, {-0.4986841787237107, -0.8359178862017277}, \
{-0.4953203624258749, -0.839098907499485}, {-0.49193595005028695`, \
-0.8422450380789991}, {-0.4885311242190259, -0.845356218844353}, \
{-0.48510606688197827`, -0.8484323910543158}, {-0.48166095933078945`, \
-0.8514734963101173}, {-0.478195982212771, -0.8544794765434663}, \
{-0.47471131554476437`, -0.8574502740048094}, {-0.47120713872696235`, \
-0.8603858312518232}, {-0.46768363055668843`, -0.8632860911381368}, \
{-0.46414096924213577`, -0.8661509968022789}, {-0.46057933241606663`, \
-0.8689804916568448}, {-0.4569988971494729, -0.8717745193778794}, \
{-0.4533998399651996, -0.8745330238944716}, {-0.449782336851532, \
-0.8772559493785556}, {-0.44614656327574764`, -0.8799432402349147}, \
{-0.44249269419763454`, -0.8825948410913852}, {-0.4388209040829772, \
-0.8852106967892541}, {-0.4351313669170111, -0.8877907523738485}, \
{-0.43142425621784763`, -0.8903349530853125}, {-0.42769974504987057`, \
-0.892843244349568}, {-0.42395800603710554`, -0.8953155717694556}, \
{-0.4201992113765639, -0.8977518811160535}, {-0.41642353285156297`, \
-0.9001521183201686}, {-0.4126311418450232, -0.9025162294639996}, \
{-0.408822209352745, -0.9048441607729666}, {-0.4049969059966658, \
-0.907135858607706}, {-0.4011554020380997, -0.9093912694562258}, \
{-0.39729786739096057`, -0.9116103399262213}, {-0.39342447163497146`, \
-0.913793016737545}, {-0.3895353840288604, -0.9159392467148313}, \
{-0.38563077352354586`, -0.9180489767802715}, {-0.38171080877531266`, \
-0.9201221539465372}, {-0.37777565815898034`, -0.9221587253098494}, \
{-0.3738254897810659, -0.9241586380431924}, {-0.36986047149294266`, \
-0.9261218393896676}, {-0.36588077090399695`, -0.9280482766559883}, \
{-0.36188655539478476`, -0.9299378972061106}, {-0.35787799213018984`, \
-0.9317906484550004}, {-0.3538552480725856, -0.9336064778625335}, \
{-0.34981848999500254`, -0.9353853329275276}, {-0.3457678844943028, \
-0.9371271611819041}, {-0.3417035980043647, -0.9388319101849782}, \
{-0.3376257968092781, -0.9404995275178756}, {-0.33353464705655406`, \
-0.9421299607780742}, {-0.3294303147703489, -0.9437231575740704}, \
{-0.32531296586470665`, -0.9452790655201666}, {-0.32118276615682095`, \
-0.9467976322313805}, {-0.31703988138031836`, -0.948278805318474}, \
{-0.312884477198566, -0.9497225323831011}, {-0.3087167192180048, \
-0.9511287610130732}, {-0.3045367730015113, -0.9524974387777408}, \
{-0.30034480408178926`, -0.9538285132234916}, {-0.2961409779747946, \
-0.9551219318693628}, {-0.29192546019319443`, -0.9563776422027679}, \
{-0.28769841625986364`, -0.9575955916753364}, {-0.2834600117214206, \
-0.9587757276988667}, {-0.27921041216180464`, -0.9599179976413906}, \
{-0.27494978321589736`, -0.9610223488233501}, {-0.2706782905831904, \
-0.9620887285138846}, {-0.2663961000415017, -0.9631170839272292}, \
{-0.26210337746074336`, -0.9641073622192236}, {-0.25780028881674216`, \
-0.9650595104839316}, {-0.2534870002051169, -0.96597347575037}, \
{-0.2491636778552135, -0.966849204979348}, {-0.24483048814410152`, \
-0.9676866450604168}, {-0.2404875976106336, -0.9684857428089287}, \
{-0.2361351729695713, -0.9692464449632074}, {-0.23177338112577917`, \
-0.9699686981818276}, {-0.22740238918849012`, -0.9706524490410066}, \
{-0.22302236448564453`, -0.9712976440321058}, {-0.21863347457830554`, \
-0.9719042295592452}, {-0.21423588727515366`, -0.972472151937028}, \
{-0.209829770647063, -0.9730013573883799}, {-0.20541529304176204`, \
-0.9734917920425005}, {-0.2009926230985815, -0.9739434019329292}, \
{-0.19656192976329254`, -0.9743561329957259}, {-0.19212338230303733`, \
-0.9747299310677677}, {-0.18767715032135573`, -0.9750647418851625}, \
{-0.18322340377331037`, -0.9753605110817797}, {-0.17876231298071293`, \
-0.9756171841879017}, {-0.1742940486474552, -0.975834706628994}, \
{-0.16981878187494712`, -0.976013023724598}, {-0.16533668417766523`, \
-0.9761520806873466}, {-0.16084792749881457`, -0.9762518226221047}, \
{-0.15635268422610687`, -0.9763121945252357}, {-0.15185112720765814`, \
-0.9763331412839958}, {-0.14734342976800888`, -0.9763146076760594}, \
{-0.14282976572427006`, -0.9762565383691743}, {-0.13831030940239797`, \
-0.9761588779209521}, {-0.13378523565360112`, -0.9760215707787941}, \
{-0.12925471987088255`, -0.9758445612799551}, {-0.12471893800572068`, \
-0.9756277936517476}, {-0.12017806658489218`, -0.9753712120118891}, \
{-0.11563228272744001`, -0.9750747603689952}, {-0.11108176416179019`, \
-0.9747383826232195}, {-0.10652668924302051`, -0.9743620225670462}, \
{-0.10196723697028477`, -0.9739456238862346}, {-0.09740358700439597, \
-0.9734891301609223}, {-0.09283591968557196, -0.972992484866887}, \
{-0.08826441605134705, -0.9724556313769724}, {-0.08368925785465317, \
-0.9718785129626805}, {-0.07911062758207428, -0.971261072795934}, \
{-0.07452870847227748, -0.9706032539510134}, {-0.06994368453462461, \
-0.9699049994066715}, {-0.06535574056796808, -0.9691662520484295}, \
{-0.06076506217963451, -0.9683869546710595}, {-0.05617183580460004, \
-0.9675670499812571}, {-0.05157624872486119, -0.9667064806005092}, \
{-0.04697848908900486, -0.9658051890681607}, {-0.04237874593198153, \
-0.9648631178446851}, {-0.03777720919508547, -0.9638802093151657}, \
{-0.03317406974614581, -0.962856405792989}, {-0.028569519399932537`, \
-0.96179164952376}, {-0.02396375093878122, -0.9606858826894403}, \
{-0.019356958133440596`, -0.9595390474127198}, {-0.014749335764146929`, \
-0.9583510857616222}, {-0.010141079641929195`, -0.9571219397543562}, \
{-0.005532386630149181, -0.9558515513644138}, {-0.000923454666280521, \
-0.9545398625259255}, {0.0036855172160692243`, -0.9531868151392776}, {
      0.008294328864889327, -0.9517923510769993}, {
      0.012902778987245402`, -0.9503564121899264}, {
      0.017510665126503525`, -0.9488789403136486}, {
      0.022117783639294814`, -0.9473598772752502}, {
      0.02672392967221636, -0.9457991649003489}, {
      0.03132889713826433, -0.944196745020444}, {
      0.03593247869299514, -0.9425525594805808}, {
      0.04053446571041046, -0.9408665501473401}, {
      0.0451346482585621, -0.9391386589171624}, {
      0.04973281507487243, -0.9373688277250163}, {
      0.05432875354116634, -0.9355569985534193}, {
      0.058922249658410725`, -0.9337031134418234}, {
      0.06351308802115721, -0.9318071144963732}, {
      0.06810105179168428, -0.9298689439000488}, {
      0.07268592267383468, -0.9278885439232044}, {
      0.07726748088654417, -0.9258658569345131}, {
      0.0818455051370577, -0.9238008254123293}, {
      0.08641977259382905, -0.9216933919564833}, {
      0.09099005885910028, -0.919543499300516}, {
      0.09555613794115703, -0.9173510903243718}, {
      0.10011778222625616`, -0.9151161080675576}, {
      0.10467476245022198`, -0.9128384957427864}, {
      0.10922684766970765`, -0.9105181967501156}, {
      0.11377380523311831`, -0.9081551546915968}, {
      0.11831540075119248`, -0.9057493133864515}, {
      0.12285139806723883`, -0.9033006168867873}, {
      0.12738155922702482`, -0.9008090094938718}, {
      0.13190564444831476`, -0.8982744357749789}, {
      0.13642341209005404`, -0.8956968405808259}, {
      0.1409346186211971, -0.893076169063618}, {
      0.14543901858917688`, -0.8904123666957175}, {
      0.149936364588013, -0.8877053792889579}, {
      0.15442640722605708`, -0.8849551530146191}, {
      0.1589088950933731, -0.8821616344240868}, {
      0.16338357472875129`, -0.8793247704702123}, {
      0.16785019058635395`, -0.8764445085293968}, {
      0.1723084850019926, -0.8735207964244195}, {
      0.1767581981590352, -0.870553582448032}, {
      0.18119906805394306`, -0.8675428153873423}, {
      0.18563083046143747`, -0.8644884445490101}, {
      0.1900532188992961, -0.861390419785279}, {
      0.19446596459277976`, -0.8582486915208697}, {
      0.19886879643869038`, -0.8550632107807589}, {
      0.20326144096906198`, -0.8518339292188706}, {
      0.2076436223144861, -0.8485607991477069}, {
      0.21201506216707425`, -0.8452437735689453}, {
      0.2163754797430602, -0.8418828062050311}, {
      0.22072459174504563`, -0.838477851531794}, {
      0.22506211232389292`, -0.8350288648121191}, {
      0.22938775304026998`, -0.8315358021307032}, {
      0.2337012228258522, -0.8279986204299276}, {
      0.23800222794418757`, -0.8244172775468798}, {
      0.2422904719512317, -0.8207917322515589}, {
      0.24656565565556035`, -0.817121944286297}, {
      0.2508274770782674, -0.8134078744064335}, {
      0.2550756314125582, -0.809649484422277}, {
      0.25930981098304734`, -0.8058467372423936}, {
      0.26352970520477337`, -0.8019995969182578}, {
      0.26773500054194105`, -0.7981080286903066}, {
      0.2719253804664058, -0.7941719990354353}, {
      0.27610052541591346`, -0.7901914757159778}, {
      0.28026011275211227`, -0.7861664278302111}, {
      0.2844038167183525, -0.7820968258644293}, {
      0.28853130839729346`, -0.7779826417466302}, {
      0.29264225566833607`, -0.7738238489018583}, {
      0.2967363231649035, -0.7696204223092526}, {
      0.30081317223159154`, -0.765372338560845}, {
      0.3048724608812138, -0.7610795759221572}, {
      0.3089138437517673, -0.7567421143946468}, {
      0.3129369720633471, -0.7523599357800504}, {
      0.31694149357503903`, -0.7479330237466785}, {
      0.32092705254182335`, -0.7434613638977104}, {
      0.3248932896715224, -0.7389449438415466}, {
      0.3288398420818296, -0.7343837532642693}, {
      0.33276634325745724`, -0.729777784004269}, {
      0.33667242300744543`, -0.7251270301290923}, {
      0.3405577074226746, -0.7204314880145688}, {
      0.3444218188336288, -0.7156911564262738}, {
      0.34826437576845864`, -0.7109060366033877}, {
      0.3520849929113956, -0.7060761323450107}, {
      0.35588328106157396`, -0.7012014500989935}, {
      0.35965884709231766`, -0.6962819990533463}, {
      0.3634112939109544, -0.6913177912302864}, {
      0.3671402204192226, -0.6863088415829885}, {
      0.3708452214743399, -0.681255168095101}, {
      0.3745258878508057, -0.6761567918830895}, {
      0.3781818062030156, -0.6710137373014753}, {
      0.38181255902876804`, -0.6658260320510289}, {
      0.3854177246337484, -0.6605937072899863}, {
      0.38899687709708114`, -0.6553167977483506}, {
      0.39254958623804403`, -0.649995341845343}, {
      0.39607541758404413`, -0.6446293818100688}, {
      0.39957393233996, -0.6392189638054595}, {
      0.4030446873589604, -0.6337641380555569}, {
      0.4064872351149136, -0.628264958976198}, {
      0.4099011236765101, -0.6227214853091658}, {
      0.4132858966832235, -0.6171337802598615}, {
      0.4166410933232437, -0.6115019116385612}, {
      0.4199662483135208, -0.60582595200531}, {
      0.4232608918820658, -0.6001059788185116}, {
      0.4265245497526594, -0.5943420745872636}, {
      0.4297567431321288, -0.5885343270274902}, {
      0.4329569887003575, -0.5826828292219179}, {
      0.43612479860320197`, -0.5767876797839402}, {
      0.4392596804484958, -0.570848983025409}, {
      0.4423611373053287, -0.5648668491283907}, {
      0.4454286677067973, -0.5588413943209181}, {
      0.44846176565643026`, -0.552772741056764}, {
      0.45145992063850093`, -0.5466610181992589}, {
      0.4544226176324461, -0.5405063612091675}, {
      0.45734933713161996`, -0.5343089123366317}, {
      0.46023955516661924`, -0.5280688208171836}, {
      0.46309274333342526`, -0.5217862430718211}, {
      0.4659083688266158, -0.5154613429111309}, {
      0.46868589447790887`, -0.509094291743437}, {
      0.4714247788003094, -0.5026852687869406}, {
      0.4741244760381374, -0.4962344612858067}, {
      0.476784436223225, -0.4897420647301433}, {
      0.47940410523757876`, -0.4832082830798059}, {
      0.4819829248828101, -0.47663332899194655`}, {
      0.4845203329566464, -0.4700174240522138}, {
      0.4870157633368423, -0.463360799009495}, {
      0.48946864607281704`, -0.4566636940140761}, {
      0.4918784074853529, -0.4499263588590783}, {
      0.49424447027469426`, -0.44314905322501313`}, {
      0.4965662536373941, -0.43633204692727817`}, {
      0.4988431733922592, -0.4294756201663969}, {
      0.5010746421157516, -0.4225800637807851}, {
      0.5032600692872059, -0.4156456795018041}, {
      0.5053988614442271, -0.4086727802108387}, {
      0.5074904223486338, -0.4016616901981139}, {
      0.5095341531633161, -0.3946127454229384}, {
      0.5115294526403723, -0.38752629377503844`}, {
      0.5134757173208939, -0.38040269533661647`}, {
      0.5153723417467589, -0.37324232264474283`}, {
      0.517218718684795, -0.36604556095365776`}, {
      0.5190142393636644, -0.35881280849653224`}, {
      0.5207582937238174, -0.35154447674620415`}, {
      0.5224502706808498, -0.3442409906743747}, {
      0.5240895584025893, -0.33690278900871756`}, {
      0.5256755446002226, -0.3295303244873196}, {
      0.5272076168337573, -0.3221240641098377}, {
      0.5286851628320953, -0.3146844893847229}, {
      0.5301075708279739, -0.30721209657182735`}, {
      0.5314742299080046, -0.2997073969196753}, {
      0.5327845303780167, -0.2921709168966434}, {
      0.5340378641438778, -0.28460319841526316`}, {
      0.5352336251079374, -0.2770047990488214}, {
      0.536371209581196, -0.26937629223940235`}, {
      0.5374500167112694, -0.2617182674964824}, {
      0.5384694489261707, -0.25403133058515587`}, {
      0.5394289123938895, -0.2463161037030402}, {
      0.5403278174976944, -0.23857322564488087`}, {
      0.5411655793270358, -0.2308033519538487}, {
      0.5419416181838647, -0.2230071550584988}, {
      0.5426553601041271, -0.21518532439433824`}, {
      0.5433062373941265, -0.20733856650893162`}, {
      0.5438936891813813, -0.19946760514945777`}, {
      0.5444171619795318, -0.19157318133162018`}, {
      0.5448761102667817, -0.1836560533888066}, {
      0.5452699970772757, -0.17571699700039053`}, {
      0.5455982946047426, -0.16775680519806982`}, {
      0.5458604848176469, -0.1597762883491459}, {
      0.5460560600850096, -0.15177627411566047`}, {
      0.5461845238119736, -0.14375760738832644`}, {
      0.5462453910841012, -0.13572115019421613`}, {
      0.5462381893193058, -0.12766778157720315`}, {
      0.5461624589262282, -0.11959839745019427`}, {
      0.5460177539677842, -0.11151391041823551`}, {
      0.5458036428285208, -0.103415249571632}, {
      0.5455197088843324, -0.09530336024828386}, {
      0.545165551173009, -0.08717920376451165}, {
      0.5447407850640018, -0.07904375711372386}, {
      0.5442450429257223, -0.07089801263236545}, {
      0.5436779747886119, -0.06274297763268157}, {
      0.5430392490021576, -0.054579674001933046`}, {
      0.542328552883964, -0.04640913776781001}, {
      0.5415455933589413, -0.038232418629907415`}, {
      0.54069009758662, -0.03005057945725002}, {
      0.5397618135745667, -0.021864695751984763`}, {
      0.5387605107758437, -0.01367585507949413}, {
      0.5376859806684419, -0.005485156465325216}, {0.536538037314599, 
      0.002706290240525813}, {0.5353165178979283, 0.010897365031285544`}, {
      0.5340212832362881, 0.01908693844219224}, {0.5326522182683541, 
      0.02727387227941015}, {0.5312092325118954, 0.03545702037792482}, {
      0.5296922604918035, 0.043635229387122894`}, {0.5281012621359911, 
      0.05180733958260986}, {0.5264362231373545, 0.05997218570267222}, {
      0.524697155280083, 0.0681285978076481}, {0.5228840967287007, 
      0.0762754021603332}, {0.5209971122783436, 0.08441142212541912}, {
      0.5190362935648986, 0.09253547908583909}, {0.5170017592337716, 
      0.1006463933737832}, {0.5148936550661974, 0.1087429852140432}, {
      0.5127121540621631, 0.1168240756772557}, {0.5104574564791817, 
      0.12488848764053428`}, {0.5081297898263271, 0.13293504675291565`}, {
      0.5057294088131182, 0.14096258240299364`}, {0.5032565952530276, 
      0.14896992868607845`}, {0.5007116579215728, 0.15695592536819658`}, {
      0.4980949323691433, 0.1649194188442413}, {0.4954067806889011, 
      0.17285926308759286`}, {0.49264759124028434`, 0.180774320588553}, {
      0.48981777832883044`, 0.1886634632789792}, {0.4869177818432158, 
      0.1965255734405605}, {0.4839480668505889, 0.20435954459424754`}, {
      0.48090912315144185`, 0.21216428236843513`}, {0.4778014647954309, 
      0.21993870534359464`}, {0.4746256295597076, 0.2276817458711652}, {
      0.4713821783914671, 0.23539235086463617`}, {0.46807169481655125`, 
      0.24306948256088798`}, {0.46469478431606426`, 0.2507121192500016}, {
      0.46125207367306575`, 0.2583192559718998}, {0.45774421029149914`, 
      0.2658899051783419}, {0.4541718614895923, 0.273423097358958}, {
      0.450535713770034, 0.2809178816301797}, {0.446836472069278, 
      0.28837332628609363`}, {0.443074858988366, 0.29578851931041894`}, {
      0.43925161400767865`, 0.3031625688489845}, {0.43536749268803565`, 
      0.310494603642252}, {0.431423265860557, 0.317783773417603}, {
      0.4274197188076799, 0.32502924924127413`}, {0.4233576504376934, 
      0.33223022382998646`}, {0.41923787245510963`, 0.33938591182247196`}, {
      0.4150612085291359, 0.3464955500112494}, {0.41082849346244593`, 
      0.35355839753514434`}, {0.40654057236237495`, 0.3605737360331833}, {
      0.40219829981658073`, 0.3675408697606162}, {0.39780253907512103`, 
      0.37445912566793865`}, {0.39335416124080264`, 0.38132785344389347`}, {
      0.3888540444695541, 0.3881464255235252}, {0.3843030731824678, 
      0.3949142370624515}, {0.3797021372910461, 0.4016307058785906}, {
      0.3750521314370752, 0.40829527236265073`}, {0.3703539542484337, 
      0.4149073993587453}, {0.365608507612029, 0.4214665720165449}, {
      0.3608166959649396, 0.4279722976164131}, {0.35597942560472523`, 
      0.43442410536900394`}, {0.35109760401975604`, 0.4408215461908172}, {
      0.3461721392402997, 0.4471641924572184}, {0.3412039392109973, 
      0.4534516377344355}, {0.3361939111852545, 0.4596834964920385}, {
      0.3311429611419733, 0.46585940379739793`}, {0.32605199322495126`, 
      0.471979014993601}, {0.3209219092051844, 0.47804200536228003`}, {
      0.3157536079662195, 0.48404806977278053`}, {0.31054798501262043`, 
      0.48999692231906217`}, {0.3053059320015331, 0.49588829594568873`}, {
      0.30002833629726144`, 0.501721942064222}, {0.2947160805486988, 
      0.5074976301612903}, {0.28937004228939606`, 0.5132151473995561}, {
      0.28399109355998997`, 0.5188742982127563}, {0.2785801005526633, 
      0.5244749038959411}, {0.2731379232772603, 0.5300168021919811}, {
      0.2676654152486383, 0.5354998468753638}, {0.2621634231947989, 
      0.5409239073342427}, {0.2566327867853077, 0.5462888681516527}, {
      0.2510743383794836, 0.5515946286867479}, {0.24548890279381363`, 
      0.5568411026568673}, {0.2398772970880274, 0.5620282177211801}, {
      0.2342403303692498, 0.5671559150666117}, {0.22857880361363517`, 
      0.5722241489966979}, {0.2228935095048775, 0.5772328865239694}, {
      0.2171852322889828, 0.5821821069664164}, {0.211454747644686, 
      0.5870718015485402}, {0.20570282256889294`, 0.5919019730074517}, {
      0.19993021527652854`, 0.5966726352044326}, {0.19413767511417523`, 
      0.6013838127423351}, {0.1883259424868907, 0.6060355405891573}, {
      0.18249574879760055`, 0.6106278637080896}, {0.17664781639847008`, 
      0.615160836694295}, {0.17078285855366868`, 0.6196345234186509}, {
      0.16490157941295183`, 0.6240489966786475}, {0.1590046739954977, 
      0.6284043378566069}, {0.15309282818344822`, 0.6327006365853591}, {
      0.1471667187246187, 0.6369379904214854}, {0.1412270132438545, 
      0.6411165045262102}, {0.13527437026252834`, 0.645236291354006}, {
      0.12930943922568758`, 0.6492974703489458}, {0.1233328605363766, 
      0.6533001676488251}, {0.11734526559667556`, 0.6572445157970516}, {
      0.11134727685501317`, 0.6611306534622846}, {0.10533950785932743`, 
      0.6649587251657945}, {0.0993225633156648, 0.6687288810164898}, {
      0.09329703915182432, 0.6724412764535551}, {0.08726352258566956, 
      0.676096071996626}, {0.08122259219774743, 0.6796934330034167}, {
      0.07517481800786809, 0.6832335294347082}, {0.06912076155531657, 
      0.6867165356265963}, {0.06306097598238114, 0.6901426300698876}, {
      0.0569960061208987, 0.6935119951965297}, {0.050926388581532235`, 
      0.6968248171729523}, {0.044852651845508955`, 0.7000812857001926}, {
      0.03877531635856215, 0.7032815938206732}, {0.03269489462683266, 
      0.7064259377314984}, {0.02661189131449912, 0.7095145166041289}, {
      0.020526803342918527`, 0.712547532410296}, {0.014440119991070838`, 
      0.7155251897540127}, {0.00835232299711294, 0.7184476957095371}, {
      0.002263886660858614, 0.7213152596651445}, {-0.003824722052988101, 
      0.7241280931725624}, {-0.009913043411008364, 
      0.7268864098019235}, {-0.016000624806805003`, 
      0.7295904250020921}, {-0.022087020656442743`, 
      0.7322403559662197}, {-0.02817179229372571, 
      0.7348364215023859}, {-0.03425450786543705, 
      0.737378841909183}, {-0.04033474222665714, 
      0.7398678388561037}, {-0.04641207683626887, 
      0.742303635268593}, {-0.052486099652751035`, 
      0.7446864552176258}, {-0.0585564050303534, 
      0.7470165238136763}, {-0.06462259361574038, 
      0.7492940671049464}, {-0.07068427224518362, 
      0.7515193119797198}, {-0.0767410538423776, 
      0.7536924860727164}, {-0.08279255731694649, 
      0.7558138176753191}, {-0.0888384074637048, 
      0.7578835356495488}, {-0.09487823486272955, 
      0.7599018693456697}, {-0.10091167578029585`, 
      0.7618690485233031}, {-0.10693837207072411`, 
      0.7637853032759361}, {-0.11295797107918164`, 
      0.7656508639587122}, {-0.11897012554547799`, 
      0.7674659611193936}, {-0.12497449350888894`, 
      0.7692308254323869}, {-0.13097073821404043`, 
      0.770945687635729}, {-0.13695852801788047`, 
      0.7726107784709307}, {-0.14293753629776332`, 
      0.7742263286255785}, {-0.14890744136066744`, 
      0.7757925686785982}, {-0.1548679263535659, 
      0.7773097290480881}, {-0.16081867917496465`, 
      0.7787780399416282}, {-0.16675939238762258`, 
      0.7801977313089791}, {-0.17268976313246331`, 
      0.7815690327970836}, {-0.17860949304368828`, 
      0.7828921737072883}, {-0.18451828816509705`, 
      0.784167382954704}, {-0.19041585886761972`, 
      0.7853948890296276}, {-0.19630191976806421`, 
      0.7865749199609476}, {-0.20217618964907966`, 
      0.7877077032814628}, {-0.2080383913803353, 
      0.7887934659950394}, {-0.21388825184091317`, 
      0.7898324345455416}, {-0.2197255018429114, 
      0.7908248347874657}, {-0.2255498760562536, 
      0.7917708919582152}, {-0.23136111293469888`, 
      0.7926708306519539}, {-0.23715895464304587`, 
      0.7935248747949771}, {-0.24294314698552305`, 
      0.7943332476225429}, {-0.24871343933535717`, 
      0.7950961716571056}, {-0.2544695845655103, 
      0.7958138686879002}, {-0.26021133898057563`, 
      0.7964865597518224}, {-0.26593846224982154`, 
      0.7971144651155548}, {-0.27165071734137186`, 
      0.797697804258891}, {-0.2773478704575121, 
      0.7982367958592091}, {-0.28302969097110736`, 
      0.7987316577770502}, {-0.28869595136312126`, 
      0.7991826070427575}, {-0.2943464271612208, 
      0.7995898598441336}, {-0.29998089687945534`, 
      0.7999536315150758}, {-0.30559914195899485`, 
      0.8002741365251499}, {-0.31120094670991394`, 
      0.8005515884700632}, {-0.3167860982540072, 
      0.8007862000630032}, {-0.3223543864686217, 
      0.800978183126803}, {-0.3279056039314915, 
      0.8011277485869037}, {-0.3334395458665602, 
      0.8012351064650769}, {-0.33895601009077625`, 
      0.8013004658738794}, {-0.3444547969618459, 
      0.801324035011808}, {-0.34993570932692974`, 
      0.8013060211591261}, {-0.355398552472267, 
      0.8012466306743323}, {-0.36084313407371293`, 
      0.8011460689912477}, {-0.36626926414817484`, 
      0.8010045406166918}, {-0.37167675500593067`, 
      0.800822249128725}, {-0.37706542120381675`, 
      0.8005993971754325}, {-0.3824350794992688, 
      0.8003361864742264}, {-0.387785548805202, 
      0.8000328178116448}, {-0.3931166501457151, 
      0.799689491043625}, {-0.39842820661260486`, 
      0.7993064050962322}, {-0.4037200433226753, 
      0.7988837579668228}, {-0.40899198737582837`, 
      0.7984217467256228}, {-0.4142438678139215, 
      0.7979205675177055}, {-0.419475515580378, 
      0.7973804155653479}, {-0.42468676348053663`, 
      0.7968014851707523}, {-0.42987744614272644`, 
      0.7961839697191139}, {-0.4350473999800533, 
      0.7955280616820211}, {-0.44019646315288513`, 
      0.7948339526211733}, {-0.445324475532022, 
      0.7941018331924012}, {-0.4504312786625383, 
      0.7933318931499769}, {-0.45551671572828467`, 
      0.7925243213511993}, {-0.46058063151703565`, 
      0.7916793057612447}, {-0.465622872386272, 
      0.7907970334582675}, {-0.4706432862295844, 
      0.7898776906387424}, {-0.47564172244368685`, 
      0.7889214626230348}, {-0.4806180318960273, 
      0.7879285338611899}, {-0.4855720668929843, 
      0.7868990879389306}, {-0.49050368114863785`, 
      0.7858333075838532}, {-0.4954127297541026, 
      0.7847313746718131}, {-0.5002990691474133, 
      0.7835934702334908}, {-0.50516255708395, 
      0.7824197744611296}, {-0.5100030526073931, 
      0.7812104667154375}, {-0.5148204160211978, 
      0.7799657255326442}, {-0.5196145088605764, 
      0.778685728631708}, {-0.5243851938649793, 
      0.7773706529216629}, {-0.5291323349510636, 
      0.7760206745091006}, {-0.5338557971861404, 
      0.7746359687057814}, {-0.5385554467620898, 
      0.7732167100363662}, {-0.5432311509697352, 
      0.7717630722462644}, {-0.547882778173667, 
      0.7702752283095933}, {-0.5525101977875063, 
      0.7687533504372414}, {-0.5571132802495995, 
      0.7671976100850323}, {-0.5616918969991358, 
      0.7656081779619833}, {-0.5662459204526774, 
      0.7639852240386548}, {-0.5707752239810954, 
      0.7623289175555852}, {-0.5752796818869018, 
      0.7606394270318083}, {-0.5797591693819704, 
      0.758916920273448}, {-0.5842135625656372, 
      0.7571615643823876}, {-0.5886427384031739, 
      0.755373525765009}, {-0.5930465747046262, 
      0.7535529701410002}, {-0.5974249501040082, 
      0.7517000625522252}, {-0.6017777440388477, 
      0.7498149673716564}, {-0.6061048367300728, 
      0.747897848312364}, {-0.6104061091622344, 
      0.7459488684365615}, {-0.6146814430640566, 
      0.7439681901647035}, {-0.6189307208893084, 
      0.7419559752846346}, {-0.6231538257979905, 
      0.7399123849607863}, {-0.6273506416378292, 
      0.7378375797434197}, {-0.631521052926073, 
      0.7357317195779136}, {-0.6356649448315838, 
      0.7335949638140938}, {-0.6397822031572176, 
      0.7314274712156037}, {-0.6438727143224879, 
      0.7292293999693138}, {-0.6479363653465067, 
      0.7270009076947698}, {-0.6519730438311964, 
      0.7247421514536757}, {-0.6559826379447677, 
      0.7224532877594134}, {-0.6599650364054578, 
      0.7201344725865955}, {-0.6639201284655224, 
      0.7177858613806511}, {-0.6678478038954782, 
      0.7154076090674452}, {-0.6717479529685888, 
      0.7129998700629268}, {-0.67562046644559, 
      0.7105627982828102}, {-0.679465235559649, 
      0.7080965471522852}, {-0.6832821520015526, 
      0.7056012696157561}, {-0.6870711079051207, 
      0.7030771181466108}, {-0.6908319958328383, 
      0.700524244757018}, {-0.6945647087617035, 
      0.6979428010077532}, {-0.6982691400692865, 
      0.6953329380180517}, {-0.701945183519994, 
      0.692694806475491}, {-0.7055927332515364, 
      0.6900285566459001}, {-0.7092116837615927, 
      0.6873343383832974}, {-0.7128019298946696, 
      0.6846123011398559}, {-0.7163633668291498, 
      0.6818625939758975}, {-0.7198958900645269, 
      0.6790853655699147}, {-0.7233993954088221, 
      0.6762807642286217}, {-0.7268737789661797, 
      0.6734489378970341}, {-0.7303189371246366, 
      0.670590034168578}, {-0.7337347665440644, 
      0.6677042002952291}, {-0.7371211641442782, 
      0.6647915831976823}, {-0.7404780270933101, 
      0.6618523294755513}, {-0.7438052527958442, 
      0.6588865854176015}, {-0.747102738881809, 
      0.6558944970120133}, {-0.7503703831951252, 
      0.6528762099566796}, {-0.7536080837826044, 
      0.6498318696695368}, {-0.7568157388829971, 
      0.6467616212989298}, {-0.7599932469161867, 
      0.6436656097340135}, {-0.7631405064725257, 
      0.6405439796151904}, {-0.7662574163023126, 
      0.6373968753445847}, {-0.7693438753054064, 
      0.6342244410965563}, {-0.7723997825209759, 
      0.6310268208282538}, {-0.7754250371173809, 
      0.6278041582902074}, {-0.778419538382184, 
      0.6245565970369652}, {-0.7813831857122895, 
      0.621284280437771}, {-0.7843158786042068, 
      0.6179873516872875}, {-0.7872175166444376, 
      0.614665953816365}, {-0.7900879994999833, 
      0.6113202297028566}, {-0.7929272269089706, 
      0.6079503220824822}, {-0.7957350986713934, 
      0.6045563735597427}, {-0.7985115146399697, 
      0.6011385266188845}, {-0.8012563747111107, 
      0.5976969236349179}, {-0.8039695788159991, 
      0.5942317068846896}, {-0.8066510269117783, 
      0.5907430185580111}, {-0.8093006189728464, 
      0.5872310007688449}, {-0.8119182549822571, 
      0.5836957955665498}, {-0.8145038349232226, 
      0.5801375449471883}, {-0.8170572587707197, 
      0.5765563908648955}, {-0.8195784264831952, 
      0.5729524752433145}, {-0.822067237994372, 
      0.5693259399870964}, {-0.8245235932051505, 
      0.5656769269934706}, {-0.8269473919756081, 
      0.5620055781638849}, {-0.8293385341170927, 
      0.5583120354157175}, {-0.8316969193844104, 
      0.5545964406940648}, {-0.8340224474681052, 
      0.5508589359836044}, {-0.836315017986831, 
      0.5470996633205379}, {-0.8385745304798135, 
      0.5433187648046144}, {-0.8408008843994011, 
      0.5395163826112365}, {-0.842993979103706, 
      0.5356926590036523}, {-0.8451537138493307, 
      0.5318477363452349}, {-0.8472799877841833, 
      0.5279817571118519}, {-0.8493726999403783, 
      0.5240948639043275}, {-0.8514317492272226, 
      0.5201871994609986}, {-0.8534570344242867, 
      0.5162589066703697}, {-0.8554484541745601, 
      0.5123101285838664}, {-0.8574059069776909, 
      0.5083410084286921}, {-0.8593292911833088, 
      0.5043516896207894}, {-0.8612185049844304, 
      0.5003423157779091}, {-0.8630734464109484, 
      0.49631303073279054`}, {-0.8648940133232029, 
      0.49226397854645404`}, {-0.8666801034056351, 
      0.4881953035216103}, {-0.8684316141605233, 
      0.4841071502161883}, {-0.8701484429018017, 
      0.4799996634569854}, {-0.871830486748961, 
      0.47587298835344205`}, {-0.8734776426210319, 
      0.4717272703115443}, {-0.8750898072306514, 
      0.4675626550478579}, {-0.8766668770782118, 
      0.4633792886036956}, {-0.8782087484460931, 
      0.45917731735942274`}, {-0.8797153173929795, 
      0.4549568880489028}, {-0.8811864797482598, 
      0.4507181477740867}, {-0.8826221311065129, 
      0.4464612440197502}, {-0.8840221668220785, 
      0.44218632466838087`}, {-0.8853864820037144, 
      0.43789353801522013`}, {-0.8867149715093409, 
      0.43358303278346233`}, {-0.8880075299408728, 
      0.4292549581396156}, {-0.8892640516391411, 
      0.42490946370902755`}, {-0.8904844306789038, 
      0.42054669959157925`}, {-0.8916685608639489, 
      0.41616681637755243`}, {-0.8928163357222892, 
      0.41176996516367237`}, {-0.8939276485014525, 
      0.4073562975693315}, {-0.8950023921638646, 
      0.40292596575299694`}, {-0.896040459382332, 
      0.3984791224288065}, {-0.8970417425356219, 
      0.3940159208833571}, {-0.8980061337041423, 
      0.3895365149926896}, {-0.8989335246657256, 
      0.3850410592394749}, {-0.8998238068915156, 
      0.3805297087304048}, {-0.9006768715419615, 
      0.3760026192137928}, {-0.9014926094629202, 
      0.3714599470973886}, {-0.9022709111818701, 
      0.3669018494664118}, {-0.9030116669042384, 
      0.36232848410180807`}, {-0.9037147665098443, 
      0.3577400094987339}, {-0.9043800995494614, 
      0.3531365848852738}, {-0.9050075552415021, 
      0.34851837024139476`}, {-0.9055970224688263, 
      0.34388552631814384`}, {-0.9061483897756788, 
      0.33923821465709253`}, {-0.9066615453647576, 
      0.3345765976100341}, {-0.9071363770944176, 
      0.3299008383589384}, {-0.9075727724760119, 
      0.3252111009361698}, {-0.9079706186713762, 
      0.3205075502449732}, {-0.9083298024904592, 
      0.3157903520802338}, {-0.9086502103891034, 
      0.31105967314951616`}, {-0.9089317284669806, 
      0.3063156810943874}, {-0.9091742424656862, 
      0.3015585445120315}, {-0.9093776377669988, 
      0.2967884329771591}, {-0.9095417993913066, 
      0.29200551706421946`}, {-0.9096666119962098, 
      0.28720996836992}, {-0.9097519598753006, 
      0.2824019595360599}, {-0.9097977269571286, 
      0.27758166427268316`}, {-0.909803796804357, 
      0.2727492573815578}, {-0.9097700526131146, 
      0.26790491477998735`}, {-0.909696377212551, 
      0.2630488135249606}, {-0.9095826530646006, 
      0.25818113183764646`}, {-0.9094287622639625, 
      0.2533020491282399}, {-0.9092345865383035, 
      0.24841174602116628`}, {-0.9090000072486903, 
      0.24351040438064983`}, {-0.9087249053902607, 
      0.23859820733665352`}, {-0.9084091615931387, 
      0.23367533931119708`}, {-0.9080526561236035, 
      0.22874198604505994`}, {-0.9076552688855213, 
      0.223798334624876}, {-0.9072168794220461, 
      0.2188445735106275}, {-0.9067373669176013, 
      0.21388089256354464`}, {-0.90621661020015, 
      0.2089074830744187}, {-0.9056544877437639, 
      0.20392453779233527`}, {-0.9050508776715013, 
      0.19893225095383543`}, {-0.904405657758604, 
      0.19393081831251194`}, {-0.9037187054360253, 
      0.18892043716904783`}, {-0.9029898977942977, 
      0.18390130640170518`}, {-0.902219111587756, 
      0.17887362649727118`}, {-0.9014062232391232, 
      0.1738375995824694}, {-0.9005511088444759, 
      0.1687934294558437}, {-0.8996536441785988, 
      0.1637413216201226}, {-0.8987137047007449, 
      0.15868148331507148`}, {-0.8977311655608118, 
      0.15361412355084078`}, {-0.8967059016059521, 
      0.14853945314181743`}, {-0.8956377873876298, 
      0.14345768474098772`}, {-0.8945266971691401, 
      0.138369032874819}, {-0.8933725049336078, 
      0.1332737139786683}, {-0.8921750843924803, 
      0.1281719464327252}, {-0.8909343089945344, 
      0.123063950598497}, {-0.8896500519354118, 
      0.11794994885584376`}, {-0.8883221861677038, 
      0.11283016564057087`}, {-0.886950584411603, 
      0.10770482748258665`}, {-0.8855351191661429, 
      0.1025741630446327}, {-0.8840756627210438, 
      0.09743840316159422}, {-0.8825720871691878, 
      0.09229778088039785}, {-0.8810242644197442, 
      0.08715253150050419}, {-0.8794320662119667, 
      0.0820028926150021}, {-0.877795364129687, 
      0.07684910415231185}, {-0.8761140296165284, 
      0.07169140841850381}, {-0.8743879339918641, 
      0.06653005014023956}, {-0.8726169484675453, 
      0.061365276508341546`}, {-0.8708009441654273, 
      0.05619733722199779}, {-0.868939792135719, 
      0.05102648453360752}, {-0.867033363376186, 
      0.04585297329427345}, {-0.8650815288522348, 
      0.040677060999946044`}, {-0.8630841595179101, 
      0.03549900783822502}, {-0.8610411263378349, 
      0.03031907673582274}, {-0.8589523003101267, 
      0.025137533406693762`}, {-0.8568175524903224, 
      0.019954646400834684`}, {-0.8546367540163468, 
      0.014770687153757579`}, {-0.8524097761345595, 
      0.009585930036640102}, {-0.8501364902269183, 
      0.004400652407154716}, {-0.8478167678392945, -0.000784865339021102}, \
{-0.8454504807109811, -0.005970339716012388}, {-0.8430375008054332, \
-0.011155484094871356`}, {-0.8405777003422822, -0.01634000865001932}, \
{-0.8380709518306665, -0.02152362030500674}, {-0.8355171281039225, \
-0.026706022677593123`}, {-0.8329161023556819, -0.03188691602414934}, \
{-0.8302677481774211, -0.03706599718338585}, {-0.8275719395975114, \
-0.0422429595194115}, {-0.8248285511218201, -0.047417492864128316`}, \
{-0.8220374577759115, -0.05258928345896919}, {-0.8191985351489036, \
-0.05775801389598643}, {-0.8163116594390304, -0.06292336305830036}, \
{-0.8133767075009691, -0.06808500605991878}, {-0.810393556894987, \
-0.07324261418493928}, {-0.8073620859379673, -0.0783958548261483}, \
{-0.8042821737563745, -0.08354439142303213}, {-0.8011537003412202, \
-0.08868788339921715}, {-0.7979765466050951, -0.0938259860993583}, \
{-0.7947505944413288, -0.09895835072549664}, {-0.7914757267853478, \
-0.10408462427290933`}, {-0.7881518276782984, -0.10920444946547733`}, \
{-0.7847787823330037, -0.11431746469059823`}, {-0.7813564772023298, \
-0.11942330393367485`}, {-0.7778848000500312, -0.1245215967122121}, \
{-0.7743636400241528, -0.12961196800955768`}, {-0.7707928877330644, \
-0.13469403820832548`}, {-0.7671724353242075, -0.1397674230235428}, \
{-0.7635021765656325, -0.14483173343556666`}, {-0.7597820069304099, \
-0.14988657562281724`}, {-0.7560118236839984, -0.15493155089438054`}, \
{-0.7521915259746543, -0.15996625562253586`}, {-0.7483210149269682, \
-0.16499028117526773`}, {-0.7444001937386188, -0.17000321384882627`}, \
{-0.7404289677804299, -0.17500463480040424`}, {-0.7364072446998239, \
-0.17999411998100362`}, {-0.7323349345277613, -0.18497124006856952`}, \
{-0.7282119497892598, -0.18993556040147413`}, {-0.7240382056175866, \
-0.19488664091243887`}, {-0.7198136198722175, -0.19982403606298826`}, \
{-0.7155381132606601, -0.20474729477853526`}, {-0.7112116094642342, \
-0.2096559603842031}, {-0.7068340352679071, -0.21454957054149582`}, \
{-0.7024053206942796, -0.2194276571859358}, {-0.6979253991418197, \
-0.22428974646579322`}, {-0.6933942075274383, -0.2291353586820405}, \
{-0.6888116864335042, -0.2339640082296706}, {-0.6841777802593924, \
-0.23877520354052773`}, {-0.6794924373776586, -0.24356844702780503`}, \
{-0.6747556102949337, -0.2483432350323736}, {-0.669967255817627, \
-0.253099057771115}, {-0.6651273352225274, -0.25783539928743854`}, \
{-0.6602358144323893, -0.2625517374041733}, {-0.6552926641965833, \
-0.2672475436790356}, {-0.6502978602768937, -0.2719222833628802}, \
{-0.6452513836385374, -0.27657541536095614`}, {-0.6401532206464737, \
-0.28120639219739524`}, {-0.6350033632670733, -0.28581465998317523`}, \
{-0.6298018092752038, -0.29039965838780724`}, {-0.6245485624667881, \
-0.29496082061501017`}, {-0.6192436328768804, -0.2994975733826448}, \
{-0.6138870370032997, -0.3040093369071921}, {-0.6084787980358504, \
-0.3084955248930717}, {-0.6030189460911519, -0.3129555445271078}, \
{-0.5975075184530871, -0.3173887964784622}, {-0.5919445598188691, \
-0.3217946749043648}, {-0.5863301225507136, -0.3261725674619849}, \
{-0.5806642669330917, -0.3305218553267973}, {-0.5749470614355222, \
-0.3348419132178093}, {-0.5691785829808479, -0.3391321094300269}, \
{-0.5633589172189248, -0.34339180587454776`}, {-0.5574881588056343, \
-0.34762035812668196`}, {-0.5515664116871094, -0.35181711548251055`}, \
{-0.5455937893890476, -0.3559814210243025}, {-0.5395704153109592, \
-0.3601126116952208}, {-0.5334964230251792, -0.3642100183837564}, \
{-0.5273719565804459, -0.3682729660183392}, {-0.5211971708098228, \
-0.37230077367257985`}, {-0.5149722316427159, -0.37629275468160517`}, \
{-0.5086973164207084, -0.38024821676995424`}, {-0.5023726142169039, \
-0.3841664621915072}, {-0.49599832615844275`, -0.38804678788192054`}, \
{-0.4895746657518171, -0.3918884856240463}, {-0.48310185921058224`, \
-0.3956908422268098}, {-0.4765801457850231, -0.39945313971802054`}, \
{-0.47000977809329886`, -0.40317465555158705`}, {-0.4633910224535521, \
-0.406854662829598}, {-0.4567241592164286, -0.41049243053972734`}, \
{-0.4500094830974147, -0.4140872238084074}, {-0.44324730350835817`, \
-0.4176383041702014}, {-0.43643794488749715`, -0.4211449298537919}, \
{-0.4295817470272779, -0.4246063560849801}, {-0.42267906539920214`, \
-0.4280218354070705}, {-0.4157302714748981, -0.4313906180189893}, \
{-0.4087357530425697, -0.43471195213145464`}, {-0.40169591451793196`, \
-0.43798508434148536`}, {-0.3946111772487007, -0.4412092600254975}, \
{-0.3874819798116608, -0.44438372375119706`}, {-0.38030877830129745`, \
-0.4475077197084339}, {-0.3730920466089342, -0.45058049215913126`}, \
{-0.36583227669128426`, -0.45360128590635423`}, {-0.35852997882728543`, \
-0.4565693467825216}, {-0.3511856818620546, -0.4594839221567052}, \
{-0.34379993343676857`, -0.4623442614608948}, {-0.33637330020324874`, \
-0.46514961673503563`}, {-0.32890636802200507`, -0.4678992431905731}, \
{-0.32139974214247363`, -0.4705923997921596}, {-0.31385404736416794`, \
-0.47322834985709755`}, {-0.3062699281774543, -0.47580636167200546`}, \
{-0.29864804888265656`, -0.47832570912610695`}, {-0.29098909368619763`, \
-0.4807856723604486}, {-0.28329376677249335`, -0.4831855384322593}, \
{-0.2755627923503287, -0.4855246019935651}, {-0.26779691467246947`, \
-0.48780216598307685`}, {-0.25999689802729137`, -0.4900175423302658}, \
{-0.2521635267012476, -0.492170052670443}, {-0.24429760491104177`, \
-0.49425902906955765`}, {-0.23639995670442762`, -0.4962838147573281}, \
{-0.2284714258286211, -0.49824376486722277`}, {-0.22051287556538304`, \
-0.5001382471817103}, {-0.21252518853191177`, -0.5019666428811053}, \
{-0.20450926644677558`, -0.5037283472942464}, {-0.19646602986021477`, \
-0.5054227706491603}, {-0.18839641784825048`, -0.5070493388217827}, \
{-0.18030138767015394`, -0.5086074940807386}, {-0.17218191438895478`, \
-0.5100966958261165}, {-0.16403899045479792`, -0.5115164213201158}, \
{-0.15587362525109846`, -0.5128661664073972}, {-0.14768684460358839`, \
-0.5141454462229312}, {-0.13947969025250004`, -0.515353795885109}, \
{-0.13125321928828623`, -0.516490771171868}, {-0.1230085035514362, \
-0.5175559491775785}, {-0.11474662899710776`, -0.5185489289484482}, \
{-0.10646869502545972`, -0.5194693320942226}, {-0.09817581377873145, \
-0.5203168033739962}, {-0.08986910940627982, -0.5210910112539967}, \
{-0.08154971729894342, -0.521791648435269}, {-0.07321878329426176, \
-0.5224184323492612}, {-0.06487746285422857, -0.5229711056194049}, \
{-0.056526920217405185`, -0.5234494364868859}, {-0.048168327527358404`, \
-0.5238532191989173}, {-0.039802863939517485`, -0.5241822743579525}, \
{-0.03143171470866535, -0.5244364492304203}, {-0.023056070259388516`, \
-0.5246156180137083}, {-0.014677125241907747`, -0.5247196820602839}, \
{-0.006296077575795908, -0.5247485700580108}, {
      0.0020858725158395854`, -0.524702238165891}, {
      0.010467523479076075`, -0.5245806701046458}, {
      0.018847673406121625`, -0.5243838772017353}, {
      0.0272251210137867, -0.5241118983906037}, {
      0.035598666620770304`, -0.5237648001641293}, {
      0.04396711312105496, -0.52334267648245}, {
      0.05232926695072615, -0.5228456486355222}, {
      0.060683939045568736`, -0.5222738650609613}, {
      0.06902994578684499, -0.5216275011178935}, {
      0.0773661099327255, -0.5209067588177257}, {
      0.08569126153292497, -0.5201118665129122}, {
      0.09400423882418919, -0.5192430785449574}, {
      0.10230388910438531`, -0.5183006748530509}, {
      0.11058906958306597`, -0.5172849605448682}, {
      0.11885864820650499`, -0.51619626543121}, {
      0.1271115044553402, -0.5150349435262657}, {
      0.13534653011310296`, -0.5138013725153987}, {
      0.14356263000406616`, -0.512495953192444}, {
      0.15175872269899834`, -0.511119108868589}, {
      0.1599337411875729, -0.5096712847549757}, {
      0.16808663351634381`, -0.5081529473212169}, {
      0.17621636339136443`, -0.5065645836320563}, {
      0.1843219107446897, -0.5049067006644309}, {
      0.1924022722641666, -0.5031798246072052}, {
      0.2004564618860772, -0.5013845001458471}, {
      0.20848351125035824`, -0.4995212897343031}, {
      0.21648247011827282`, -0.4975907728563056}, {
      0.22445240675255884`, -0.4955935452783111}, {
      0.23239240826022062`, -0.49353021829622074`}, {
      0.240301580898265, -0.49140141797798076`}, {
      0.2481790503428113, -0.4892077844040968}, {
      0.25602396192212384`, -0.486949970908023}, {
      0.263835480814228, -0.48462864331831057`}, {
      0.27161279220987194`, -0.4822444792043139}, {
      0.2793551014416922, -0.4797981671271644}, {
      0.28706163408052343`, -0.4772904058976274}, {
      0.2947316359998712, -0.4747219038423616}, {
      0.30236437340963007`, -0.4720933780800002}, {
      0.30995913286019056`, -0.469405553808371}, {
      0.317515221218124, -0.46665916360407367`}, {
      0.32503196561467707`, -0.4638549467355258}, {
      0.33250871336833954`, -0.4609936484904914}, {
      0.3399448318827713, -0.4580760195190028}, {
      0.3473397085213944, -0.45510281519248796`}, {
      0.3546927504599625, -0.45207479497981984`}, {
      0.36200338451842556`, -0.44899272184090794`}, {
      0.3692710569734021, -0.44585736163836326`}, {
      0.3764952333525642, -0.4426694825676781}, {
      0.38367539821222424`, -0.4394298546062794}, {
      0.39081105489939527`, -0.43613924898173323`}, {
      0.39790172529957124`, -0.43279843765930087`}, {
      0.4049469495714473, -0.4294081928489776}, {
      0.41194628586976895`, -0.4259692865320736}, {
      0.41889931005746556`, -0.42248249000733684`}, {
      0.4258056154081861, -0.4189485734565552}, {
      0.4326648123003183, -0.41536830552952314`}, {
      0.43947652790352965`, -0.41174245294820677`}, {
      0.44624040585882985`, -0.40807178012989354`}, {
      0.4529561059531085, -0.40435704882907386`}, {
      0.45962330378906024`, -0.4005990177977611}, {
      0.466241690451365, -0.396798442463924}, {
      0.47281097216994616`, -0.3929560746276747}, {
      0.4793308699810865, -0.3890726621748286}, {
      0.4858011193871358, -0.38514894880743045`}, {
      0.4922214700155038, -0.38118567379081825`}, {
      0.4985916852775851, -0.3771835717167835}, {
      0.5049115420282249, -0.37314337228236905`}, {
      0.5111808302262904, -0.3690658000838362}, {
      0.5173993525968741, -0.364951574425324}, {
      0.5235669242956165, -0.36080140914171704`}, {
      0.5296833725755962, -0.356616012435234}, {
      0.5357485364572027, -0.35239608672524725`}, {
      0.5417622664013687, -0.34814232851084387`}, {
      0.5477244239865076, -0.3438554282456391}, {
      0.5536348815894678, -0.33953607022435756`}, {
      0.5594935220707882, -0.3351849324807007}, {
      0.5653002384645046, -0.3308026866960254}, {
      0.5710549336727356, -0.32638999811836555`}, {
      0.5767575201652447, -0.3219475254913354}, {
      0.5824079196841537, -0.31747592099246286`}, {
      0.5880060629539584, -0.31297583018051056`}, {
      0.593551889396974, -0.30844789195135075`}, {
      0.59904534685432, -0.3038927385019736}, {
      0.6044863913125316, -0.2993109953022167}, {
      0.6098749866358685, -0.29470328107381655`}, {
      0.6152111043043753, -0.2900702077763941}, {
      0.6204947231577302, -0.2854123805999984}, {
      0.6257258291449057, -0.2807303979638444}, {
      0.6309044150796511, -0.2760248515208927}, {
      0.636030480401794, -0.271296326167932}, {
      0.6411040309443461, -0.2665454000608372}, {
      0.646125078706389, -0.26177264463468686`}, {
      0.6510936416317061, -0.25697862462843735`}, {
      0.6560097433931159, -0.2521638981138624}, {
      0.6608734131824573, -0.24732901652847822`}, {
      0.6656846855061683, -0.24247452471218578`}, {
      0.6704435999863924, -0.2376009609473742}, {
      0.675150201167543, -0.23270885700223895`}, {
      0.6798045383282512, -0.22779873817708038`}, {
      0.6844066652986138, -0.22287112335335832`}, {
      0.6889566402826613, -0.217926525045289}, {
      0.6934545256859547, -0.21296544945378018`}, {
      0.6979003879482222, -0.20798839652251072`}, {
      0.7022942973809437, -0.2029958599959697}, {
      0.7066363280097854, -0.19798832747927986`}, {
      0.7109265574217889, -0.1929662804996385}, {
      0.7151650666172168, -0.1879301945692179}, {
      0.7193519398659551, -0.18288053924937536`}, {
      0.7234872645683719, -0.17781777821603098`}, {
      0.7275711311205322, -0.17274236932607884`}, {
      0.731603632783669, -0.16765476468470486`}, {
      0.7355848655578086, -0.16255541071349142`}, {
      0.7395149280594506, -0.1574447482191954}, {
      0.7433939214032034, -0.15232321246309374`}, {
      0.7472219490872739, -0.1471912332307953}, {
      0.7509991168827157, -0.14204923490242544`}, {
      0.7547255327263359, -0.13689763652309409`}, {
      0.7584013066171659, -0.13173685187356435`}, {
      0.7620265505163993, -0.12656728954104338`}, {
      0.7656013782507054, -0.1213893529900227}, {
      0.7691259054188226, -0.11620344063309934`}, {
      0.772600249301344, -0.11100994590171412`}, {
      0.7760245287736023, -0.10580925731674742`}, {
      0.7793988642215691, -0.10060175855891706`}, {
      0.7827233774606803, -0.09538782853892652}, {
      0.7859981916575033, -0.09016784146731568}, {
      0.7892234312541633, -0.08494216692396978}, {
      0.7923992218954479, -0.0797111699272452}, {
      0.7955256903585092, -0.07447521100267446}, {
      0.7986029644850869, -0.06923464625121521}, {
      0.8016311731161766, -0.06398982741701123}, {
      0.8046104460290671, -0.058741101954635846`}, {
      0.8075409138766768, -0.05348881309579102}, {
      0.8104227081291177, -0.04823329991543737}, {
      0.8132559610174167, -0.042974897397333}, {
      0.8160408054793298, -0.03771393649896083}, {
      0.8187773751071808, -0.03245074421582646}, {
      0.8214658040976637, -0.02718564364511018}, {
      0.824106227203544, -0.021918954048658745`}, {
      0.8266987796872014, -0.0166509909153041}, {
      0.8292435972759529, -0.011382066022497868`}, {
      0.8317408161191003, -0.006112487497251888}, {
      0.8341905727466473, -0.0008425598763764797}, {0.8365930040296307, 
      0.00442741583399058}, {0.8389482471420148, 0.009697142099570148}, {
      0.8412564395240979, 0.0149663247998475}, {0.843517718847381, 
      0.020234673170701396`}, {0.8457322229808515, 0.025501899747758933`}, {
      0.847900089958635, 0.030767720310477857`}, {0.8500214579489704, 
      0.03603185382695701}, {0.8520964652244652, 0.041294022399474904`}, {
      0.854125250133587, 0.04655395121075549}, {0.8561079510733517, 
      0.05181136847095968}, {0.8580447064631687, 0.05706600536540031}, {
      0.8599356547198024, 0.06231759600297788}, {0.8617809342334161, 
      0.06756587736533351}, {0.8635806833446583, 0.07281058925671535}, {
      0.8653350403227602, 0.0780514742545541}, {0.8670441433446067, 
      0.0832882776607426}, {0.8687081304747512, 0.0885207474536146}, {
      0.8703271396463409, 0.09374863424061686}, {0.8719013086429217, 
      0.09897169121166864}, {0.8734307750810935, 0.1041896740932026}, {
      0.874915676393987, 0.10940234110288019`}, {0.8763561498155342, 
      0.11460945290497516`}, {0.877752332365505, 0.11981077256641784`}, {
      0.8791043608352845, 0.12500606551349308`}, {0.8804123717743665, 
      0.13019509948918462`}, {0.8816765014775365, 0.135377644511158}, {
      0.8828968859727234, 0.14055347283037456`}, {0.8840736610094951, 
      0.1457223588903286}, {0.8852069620481773, 0.15088407928689979`}, {
      0.8862969242495731, 0.15603841272881255`}, {0.8873436824652641, 
      0.16118513999869477`}, {0.8883473712284721, 0.16632404391472716`}, {
      0.8893081247454626, 0.17145490929287546`}, {0.8902260768874719, 
      0.1765775229096968}, {0.8911013611831392, 0.1816916734657123}, {
      0.8919341108114271, 0.18679715154933732`}, {0.8927244585950138, 
      0.1918937496013612}, {0.8934725369941396, 0.1969812618799682}, {
      0.8941784781008948, 0.202059484426291}, {0.8948424136339307, 
      0.20712821503048895`}, {0.895464474933582, 0.21218725319834253`}, {
      0.8960447929573855, 0.21723640011835574`}, {0.8965834982759803, 
      0.22227545862935844`}, {0.8970807210693796, 0.22730423318860032`}, {
      0.8975365911235987, 0.23232252984032845`}, {0.8979512378276282, 
      0.2373301561848404}, {0.8983247901707405, 0.24232692134800504`}, {
      0.8986573767401191, 0.24731263595124267`}, {0.8989491257187987, 
      0.2522871120819574}, {0.899200164883906, 0.25725016326441313`}, {
      0.8994106216051916, 0.2622016044310461}, {0.8995806228438428, 
      0.26714125189420607`}, {0.899710295151568, 0.2720689233183185}, {
      0.8997997646699436, 0.2769844376924605}, {0.899849157130015, 
      0.2818876153033431}, {0.8998585978521428, 0.2867782777086923}, {
      0.899828211746087, 0.29165624771102194`}, {0.8997581233113199, 
      0.29652134933179125`}, {0.8996484566375625, 0.30137340778593985`}, {
      0.8994993354055348, 0.30621224945679343`}, {0.8993108828879148, 
      0.311037701871333}, {0.8990832219504984, 0.31584959367582144`}, {
      0.8988164750535548, 0.32064775461177986`}, {0.8985107642533708, 
      0.3254320154923081}, {0.8981662112039775, 0.3302022081787422}, {
      0.8977829371590548, 0.3349581655576425}, {0.897361062974007, 
      0.3396997215181065}, {0.896900709108206, 0.34442671092939964`}, {
      0.8964019956273951, 0.3491389696188983}, {0.89586504220625, 
      0.353836334350339}, {0.8952899681310914, 0.35851864280236745`}, {
      0.8946768923027459, 0.36318573354738165`}, {0.8940259332395495, 
      0.36783744603066404`}, {0.8933372090804907, 0.3724736205497957}, {
      0.8926108375884898, 0.37709409823434825`}, {0.8918469361538082, 
      0.3816987210258469}, {0.8910456217975871, 0.3862873316580002}, {
      0.8902070111755108, 0.39085977363719027`}, {0.8893312205815909, 
      0.3954158912232188}, {0.888418365952069, 0.3999555294103036}, {
      0.8874685628694353, 0.4044785339083202}, {0.886481926566559, 
      0.4089847511242843}, {0.885458571930929, 0.41347402814406886`}, {
      0.8843986135090022, 0.4179462127143528}, {0.8833021655106561, 
      0.42240115322479466`}, {0.8821693418137446, 0.42683869869042784`}, {
      0.881000255968754, 0.4312586987342723}, {0.8797950212035572, 
      0.4356610035701582}, {0.8785537504282651, 0.44004546398575706`}, {
      0.8772765562401719, 0.44441193132581647`}, {0.8759635509287934, 
      0.4487602574755931}, {0.8746148464809976, 0.4530902948444808}, {
      0.8732305545862239, 0.4574018963498291}, {0.871810786641791, 
      0.4616949154009478}, {0.8703556537582928, 0.4659692058832946}, {
      0.8688652667650792, 0.4702246221428405}, {0.8673397362158222, 
      0.47446101897061005`}, {0.8657791723941654, 0.4786782515873925}, {
      0.8641836853194569, 0.4828761756286198}, {0.862553384752564, 
      0.48705464712940755`}, {0.8608883802017687, 0.49121352250975653`}, {
      0.8591887809287442, 0.49535265855990956`}, {0.8574546959546113, 
      0.4994719124258614}, {0.8556862340660741, 0.5035711415950178}, {
      0.853883503821635, 0.5076502038820006}, {0.8520466135578877, 
      0.5117089574145949}, {0.8501756713958891, 0.5157472606198362}, {
      0.8482707852476097, 0.5197649722102333}, {0.8463320628224605, 
      0.5237619511701238}, {0.8443596116338999, 0.5277380567421605}, {
      0.8423535390061166, 0.5316931484139235}, {0.8403139520807918, 
      0.5356270859046575}, {0.8382409578239387, 0.539539729152129}, {
      0.8361346630328209, 0.5434309382996022}, {0.8339951743429492, 
      0.5473005736829302}, {0.8318225982351574, 0.5511484958177587}, {
      0.829617041042757, 0.5549745653868399}, {0.8273786089587726, 
      0.5587786432274531}, {0.8251074080432566, 0.5625605903189306}, {
      0.8228035442306854, 0.5663202677702851}, {0.8204671233374374, 
      0.5700575368079372}, {0.8180982510693529, 0.57377225876354}, {
      0.8156970330293765, 0.5774642950618979}, {0.8132635747252847, 
      0.5811335072089786}, {0.8107979815774972, 0.5847797567800148}, {
      0.8083003589269743, 0.5884029054076937}, {0.8057708120432011, 
      0.5920028147704322}, {0.8032094461322596, 0.5955793465807357}, {
      0.8006163663449891, 0.5991323625736386}, {0.7979916777852377, 
      0.6026617244952224}, {0.7953354855182044, 0.6061672940912135}, {
      0.7926478945788742, 0.6096489330956546}, {0.7899290099805473, 
      0.6131065032196499}, {0.7871789367234637, 0.6165398661401815}, {
      0.7843977798035248, 0.6199488834889961}, {0.7815856442211131, 
      0.6233334168415586}, {0.778742634990013, 0.6266933277060726}, {
      0.7758688571464318, 0.630028477512564}, {0.7729644157581265, 
      0.6333387276020288}, {0.7700294159336328, 0.6366239392156408}, {
      0.7670639628316044, 0.6398839734840189}, {0.7640681616702587, 
      0.6431186914165532}, {0.7610421177369354, 0.6463279538907857}, {
      0.7579859363977668, 0.6495116216418481}, {0.7548997231074639, 
      0.6526695552519515}, {0.7517835834192195, 0.6558016151399294}, {
      0.7486376229947307, 0.6589076615508311}, {0.7454619476143434, 
      0.6619875545455646}, {0.7422566631873205, 0.6650411539905883}, {
      0.7390218757622373, 0.66806831954765}, {0.7357576915375048, 
      0.6710689106635717}, {0.7324642168720252, 0.6740427865600794}, {
      0.7291415582959819, 0.6769898062236778}, {0.7257898225217653, 
      0.6799098283955668}, {0.7224091164550387, 0.6828027115616012}, {
      0.7189995472059469, 0.685668313942291}, {0.7155612221004694, 
      0.6885064934828423}, {0.7120942486919217, 0.6913171078432384}, {
      0.7085987347726093, 0.694100014388359}, {0.705074788385634, 
      0.696855070178138}, {0.7015225178368593, 0.6995821319577594}, {
      0.6979420317070358, 0.7022810561478897}, {0.6943334388640916, 
      0.7049516988349473}, {0.6906968484755888, 0.7075939157614081}, {
      0.6870323700213526, 0.7102075623161463}, {0.6833401133062746, 
      0.712792493524812}, {0.6796201884732938, 0.7153485640402424}, {
      0.67587270601656, 0.7178756281329095}, {0.6720977767947842, 
      0.7203735396814018}, {0.6682955120447768, 0.7228421521629415}, {
      0.6644660233951811, 0.7252813186439356}, {0.6606094228804048, 
      0.7276908917705637}, {0.6567258229547527, 0.730070723759399}, {
      0.6528153365067663, 0.732420666388066}, {0.6488780768737746, 
      0.7347405709859336}, {0.6449141578566598, 0.7370302884248436}, {
      0.6409236937348426, 0.7392896691098769}, {0.6369067992814925, 
      0.741518562970156}, {0.6328635897789676, 0.7437168194496849}, {
      0.6287941810344878, 0.7458842874982291}, {0.624698689396048, 
      0.7480208155622331}, {0.6205772317685754, 0.7501262515757791}, {
      0.6164299256303357, 0.7522004429515867}, {0.6122568890495953, 
      0.7542432365720556}, {0.6080582407015425, 0.7562544787803507}, {
      0.6038340998854752, 0.7582340153715333}, {0.5995845865422603, 
      0.7601816915837384}, {0.5953098212720696, 0.7620973520893993}, {
      0.5910099253523989, 0.7639808409865233}, {0.5866850207563773, 
      0.7658320017900173}, {0.5823352301713705, 0.7676506774230683}, {
      0.5779606770178863, 0.7694367102085781}, {0.5735614854687885, 
      0.7711899418606568}, {0.5691377804688241, 0.772910213476176}, {
      0.5646896877544734, 0.7745973655263838}, {0.5602173338741261, 
      0.7762512378485869}, {0.5557208462085944, 0.7778716696378986}, {
      0.5512003529919667, 0.7794584994390598}, {0.546655983332811, 
      0.7810115651383328}, {0.5420878672357349, 0.7825307039554735}, {
      0.5374961356233099, 0.7840157524357843}, {0.5328809203583673, 
      0.7854665464422513}, {0.5282423542666738, 0.7868829211477716}, {
      0.5235805711599941, 0.7882647110274718}, {0.5188957058595507, 
      0.7896117498511243}, {0.5141878942198856, 0.7909238706756654}, {
      0.5094572731531363, 0.7922009058378194}, {0.504703980653731, 
      0.7934426869468344}, {0.4999281558235153, 0.7946490448773342}, {
      0.49512993889731594`, 0.7958198097622934}, {0.49030947126895336`, 
      0.796954810986139}, {0.4854668955177106, 0.7980538771779871}, {
      0.4806023554352688, 0.7991168362050198}, {0.4757159960531185, 
      0.8001435151660078}, {0.47080796367045663`, 0.8011337403849882}, {
      0.4658784058825794, 0.8020873374051026}, {0.4609274716097802, 
      0.8030041309826043}, {0.45595531112676413`, 0.8038839450810411}, {
      0.4509620760925884, 0.8047266028656243}, {0.44594791958114016`, 
      0.8055319266977893}, {0.44091299611216134`, 0.8062997381299605}, {
      0.435857461682833, 0.8070298579005267}, {0.4307814737999292, 
      0.8077221059290387}, {0.4256851915125516, 0.8083763013116384}, {
      0.4205687754454571, 0.808992262316731}, {0.41543238783298975`, 
      0.8095698063809106}, {0.41027619255362796`, 0.8101087501051505}, {
      0.40510035516516074`, 0.8106089092512722}, {0.399905042940503, 
      0.8110700987387033}, {0.39469042490416395`, 0.8114921326415391}, {
      0.38945667186938016`, 0.8118748241859205}, {0.38420395647592587`, 
      0.8122179857477441}, {0.37893245322861374`, 0.8125214288507167}, {
      0.37364233853649864`, 0.8127849641647726}, {0.3683337907527975, 
      0.8130084015048679}, {0.36300699021553845`, 0.8131915498301683}, {
      0.35766211928895286`, 0.81333421724365}, {0.35229936240562304`, 
      0.8134362109921291}, {0.3469189061093995, 0.8134973374667408}, {
      0.34152093909910186`, 0.8135174022038857}, {0.336105652273016, 
      0.8134962098866657}, {0.33067323877420246`, 0.8134335643468297}, {
      0.32522389403662916`, 0.8133292685672507}, {0.3197578158321428, 
      0.8131831246849587}, {0.3142752043182925, 0.8129949339947511}, {
      0.3087762620870202, 0.8127644969534079}, {0.3032611942142315, 
      0.8124916131845353}, {0.2977302083102609, 0.8121760814840655}, {
      0.29218351457124575`, 0.8118176998264399}, {0.2866213258314223, 
      0.8114162653715052}, {0.2810438576163584, 0.8109715744721521}, {
      0.275451328197136, 0.8104834226827264}, {0.26984395864549754`, 
      0.8099516047682476}, {0.2642219728899693, 0.809375914714465}, {
      0.25858559777297563`, 0.8087561457387894}, {0.25293506310895647`, 
      0.8080920903021329}, {0.24727060174350154`, 0.8073835401216981}, {
      0.24159244961351373`, 0.8066302861847519}, {0.23590084580841353`, 
      0.8058321187634261}, {0.23019603263239702`, 0.8049888274305863}, {
      0.22447825566775845`, 0.8041002010768112}, {0.21874776383928857`, 
      0.8031660279285283}, {0.2130048094797594, 0.8021860955673515}, {
      0.207249648396505, 0.8011601909506695}, {0.20148253993910814`, 
      0.8000881004335327}, {0.19570374706820098`, 0.7989696097918936}, {
      0.18991353642538802`, 0.7978045042472495}, {0.18411217840429828`, 
      0.796592568492747}, {0.1782999472227728, 0.7953335867208019}, {
      0.17247712099619303`, 0.7940273426522957}, {0.16664398181195386`, 
      0.7926736195674082}, {0.16080081580508448`, 0.7912722003381506}, {
      0.15494791323501897`, 0.7898228674626626}, {0.14908556856351687`, 
      0.7883254031013426}, {0.1432140805337329, 0.7867795891148786}, {
      0.13733375225043312`, 0.7851852071042534}, {0.13144489126135336`, 
      0.7835420384527969}, {0.12554780963969409`, 0.7818498643703611}, {
      0.11964282406774343`, 0.7801084659396992}, {0.11373025592161862`, 
      0.7783176241651271}, {0.10781043135711356`, 0.7764771200235522}, {
      0.10188368139663789`, 0.7745867345179559}, {0.09595034201723061, 
      0.7726462487334173}, {0.09001075423962825, 0.7706554438957701}, {
      0.08406526421836516, 0.7686141014329866}, {0.07811422333288018, 
      0.7665220030393837}, {0.07215798827960059, 0.7643789307427518}, {
      0.06619692116497114, 0.7621846669745065}, {0.06023138959939205, 
      0.7599389946429683}, {0.054261766792025934`, 0.7576416972098765}, {
      0.04828843164642971, 0.7552925587702467}, {0.04231176885696304, 
      0.7528913641356841}, {0.036332169005920206`, 0.7504378989212674}, {
      0.030350028661327515`, 0.7479319496361193}, {0.024365750475343177`, 
      0.7453733037777841}, {0.018379743283191082`, 0.7427617499305337}, {
      0.012392422202554236`, 0.7400970778677263}, {0.0064042087333475, 
      0.7373790786583444}, {0.00041553085778298776`, 
      0.7346075447778403}, {-0.005573176859365293, 
      0.7317822702234197}, {-0.011561473170397967`, 
      0.7289030506338959}, {-0.017548910044335786`, 
      0.7259696834142485}, {-0.023535032566689634`, 
      0.7229819678650217}, {-0.02951937883947874, 
      0.7199397053167002}, {-0.035501479881629196`, 
      0.7168426992691989}, {-0.04148085952989389, 
      0.7136907555366064}, {-0.04745703434044407, 
      0.7104836823973213}, {-0.053429513491292636`, 
      0.7072212907497191}, {-0.059397798685719094`, 
      0.7039033942734914}, {-0.0653613840568768, 
      0.700529809596794}, {-0.07131975607377394, 
      0.697100356469343}, {-0.07727239344883098, 
      0.6936148579415936}, {-0.08321876704722908, 
      0.6900731405501381}, {-0.08915833979827617, 
      0.6864750345094532}, {-0.09509056660902981, 
      0.6828203739101263}, {-0.10101489428042898`, 
      0.6791089969236862}, {-0.10693076142620031`, 
      0.6753407460141593}, {-0.11283759839481786`, 
      0.6715154681564663}, {-0.11873482719480977`, 
      0.6676330150617711}, {-0.12462186142371946`, 
      0.6636932434098834}, {-0.13049810620104368`, 
      0.6596960150888134}, {-0.13636295810548485`, 
      0.655641197441563}, {-0.14221580511687046`, 
      0.6515286635202353}, {-0.14805602656310754`, 
      0.6473582923475262}, {-0.1538829930725561, 
      0.6431299691856562}, {-0.15969606653222101`, 
      0.6388435858127853}, {-0.16549460005217798`, 
      0.6344990408069386}, {-0.17127793793666451`, 
      0.6300962398374579}, {-0.17704541566228338`, 
      0.6256350959639749}, {-0.18279635986378076`, 
      0.6211155299428854}, {-0.18853008832787754`, 
      0.616537470541283}, {-0.19424590999564675`, 
      0.6119008548582906}, {-0.19994312497394504`, 
      0.6072056286537043}, {-0.20562102455642015`, 
      0.6024517466838417}, {-0.21127889125462992`, 
      0.5976391730444567}, {-0.21691599883982093`, 
      0.5927678815205598}, {-0.22253161239592673`, 
      0.5878378559429505}, {-0.22812498838435658`, 
      0.582849090551236}, {-0.23369537472115476`, 
      0.5778015903630817}, {-0.239242010867119, 
      0.5726953715493971}, {-0.24476412793147273`, 
      0.5675304618151324}, {-0.25026094878969163`, 
      0.5623069007853143}, {-0.25573168821608655`, 
      0.5570247403959138}, {-0.2611755530317469, 
      0.5516840452890968}, {-0.26659174226844656`, 
      0.5462848932123622}, {-0.2719794473491102, 
      0.5408273754210282}, {-0.2773378522854315, 
      0.5353115970834813}, {-0.2826661338932245, 
      0.5297376776885523}, {-0.287963462026077, 
      0.5241057514543369}, {-0.29322899982785816`, 
      0.5184159677377254}, {-0.29846190400461214`, 
      0.5126684914438561}, {-0.30366132511634664`, 
      0.506863503434655}, {-0.3088264078891967, 
      0.5010012009355711}, {-0.3139562915484123, 
      0.49508179793956447`}, {-0.3190501101725819, 
      0.48910552560735204`}, {-0.32410699306946206`, 
      0.48307263266286377`}, {-0.32912606517373894`, 
      0.4769833857828102}, {-0.3341064474669957, 
      0.4708380699792132}, {-0.3390472574201038, 
      0.464636988973703}, {-0.3439476094581969, 
      0.45838046556233764`}, {-0.34880661544831904`, 
      0.45206884196965824`}, {-0.3536233852097674, 
      0.4457024801906511}, {-0.3583970270470763, 
      0.4392817623192519}, {-0.36312664830550584`, 
      0.43280709086199204`}, {-0.36781135594881303`, 
      0.42627888903535927`}, {-0.3724502571589956, 
      0.41969760104542125`}, {-0.3770424599575998, 
      0.4130636923482407}, {-0.3815870738480883, 
      0.40637764988959996`}, {-0.38608321047865996`, 
      0.39963998232254755`}, {-0.39052998432480707`, 
      0.39285122020127944`}, {-0.39492651339078877`, 
      0.38601191614987823`}, {-0.39927191992908634`, 
      0.3791226450044511}, {-0.40356533117679483`, 
      0.37218400392723205`}, {-0.4078058801077917, 
      0.365196612491251}, {-0.41199270619940864`, 
      0.3581611127342146}, {-0.4161249562122206, 
      0.35107816918029866`}, {-0.42020178498145283`, 
      0.3439484688286163}, {-0.4242223562183981, 
      0.3367727211071975}, {-0.42818584332012966`, 
      0.3295516577914019}, {-0.43209143018569246`, 
      0.32228603288577784`}, {-0.43593831203685884`, 
      0.3149766224684863}, {-0.439725696241443, 
      0.3076242244975186}, {-0.44345280313708485`, 
      0.3002296585780616}, {-0.44711886685333835`, 
      0.29279376569049304`}, {-0.45072313612983045`, 
      0.28531740787862914`}, {-0.45426487512820246`, 
      0.27780146789799437`}, {-0.45774336423549694`, 
      0.2702468488240355}, {-0.46115790085661923`, 
      0.26265447362036165`}, {-0.46450780019348076`, 
      0.2550252846672571}, {-0.46779239600842076`, 
      0.24736024325088093`}, {-0.47101104136950744`, 
      0.2396603290137391}, {-0.4741631093753372, 
      0.23192653936718688`}, {-0.47724799385698286`, 
      0.22415988886689228`}, {-0.4802651100547875, 
      0.21636140855236258`}, {-0.4832138952677615, 
      0.20853214525180577`}, {-0.4860938094734157, 
      0.20067316085376366`}, {-0.48890433591595234`, 
      0.192785531547114}, {-0.49164498166083764`, 
      0.18487034703119262`}, {-0.49431527811389664`, 
      0.17692870969793242`}, {-0.4969147815031984, 
      0.16896173378805324`}, {-0.49944307332213905`, 
      0.16097054452346263`}, {-0.5018997607322819, 
      0.15295627721814328`}, {-0.5042844769246722, 
      0.14492007636990523`}, {-0.5065968814385133, 
      0.13686309473547048`}, {-0.5088366604362675, 
      0.1287864923914334}, {-0.5110035269344222, 
      0.1206914357837}, {-0.5130972209893537, 
      0.11257909676805622`}, {-0.5151175098379043, 
      0.10445065164454356`}, {-0.5170641879924796, 
      0.09630728018833609}, {-0.518937077290666, 
      0.08815016467981064}, {-0.5207360268995523, 
      0.07998048893648521}, {-0.5224609132751277, 
      0.07179943734946832}, {-0.5241116400773089, 
      0.06360819392701478}, {-0.525688138041326, 
      0.05540794134772183}, {-0.5271903648063613, 
      0.04719986002582483}, {-0.5286183047025028, 
      0.0389851271909635}, {-0.5299719684972176, 
      0.030764915984690513`}, {-0.531251393102699, 
      0.022540394575883475`}, {-0.5324566412455661, 
      0.01431272529710153}, {-0.5335878011005188, 
      0.00608306380379901}, {-0.5346449858896526, -0.002147441741827341}, \
{-0.5356283334492364, -0.010377651460723008`}, {-0.5365380057658374, \
-0.018606434519104667`}, {-0.5373741884837423, -0.026832669861496197`}, \
{-0.5381370903856842, -0.035055246913487485`}, {-0.5388269428489255, \
-0.04327306625318555}, {-0.5394439992787753, -0.0514850402504816}, \
{-0.5399885345216413, -0.05969009367341044}, {-0.5404608442597183, \
-0.06788716426102931}, {-0.540861244389412, -0.07607520326239087}, \
{-0.5411900703855794, -0.08425317594132853}, {-0.5414476766536412, \
-0.09242006204691113}, {-0.541634435871583, -0.10057485624955712`}, \
{-0.5417507383238201, -0.1087165685429254}, {-0.5417969912288473, \
-0.11684422461182042`}, {-0.5417736180625312, -0.12495686616646215`}, \
{-0.5416810578788426, -0.1330535512435772}, {-0.5415197646297475, \
-0.1411333544748657}, {-0.5412902064859046, -0.14919536732348768`}, \
{-0.540992865159732, -0.1572386982892955}, {-0.5406282352323245, \
-0.16526247308361186`}, {-0.5401968234856162, -0.17326583477441948`}, \
{-0.5396991482410933, -0.1812479439028854}, {-0.5391357387062772, \
-0.18920797857219399`}, {-0.5385071343301021, -0.1971451345097041}, \
{-0.5378138841682294, -0.20505862510348202`}, {-0.5370565462592464, \
-0.21294768141428874`}, {-0.5362356870126136, -0.22081155216412296`}, \
{-0.5353518806091372, -0.22864950370243486`}, {-0.5344057084146597, \
-0.23646081995113588`}, {-0.5333977584075815, -0.24424480232953247`}, \
{-0.5323286246207467, -0.25200076966031076`}, {-0.531198906598151, \
-0.25972805805769206`}, {-0.5300092088668572, -0.2674260207988685}, \
{-0.5287601404244339, -0.27509402817981354`}, {-0.5274523142421688, \
-0.28273146735654275`}, {-0.526086346784244, -0.2903377421728801}, \
{-0.5246628575430037, -0.29791227297575895`}, {-0.5231824685903869, \
-0.3054544964190609}, {-0.521645804145552, -0.3129638652569671}, \
{-0.5200534901586651, -0.32043984812776394`}, {-0.5184061539107858, \
-0.3278819293290149}, {-0.5167044236297403, -0.33528960858497525`}, \
{-0.5149489281218353, -0.34266240080709265`}, {-0.5131402964192324, \
-0.34999983584840105`}, {-0.5112791574427719, -0.3573014582525802}, \
{-0.5093661396800069, -0.3645668269984169}, {-0.507401870878185, \
-0.37179551524036897`}, {-0.5053869777518913, -0.37898711004589614`}, \
{-0.5033220857050491, -0.38614121213018804`}, {-0.5012078185669568, \
-0.39325743558888354`}, {-0.49904479834202686`, -0.4003354076293419}, \
{-0.49683364497287785`, -0.40737476830099256`}, {-0.4945749761164251, \
-0.4143751702252574}, {-0.49226940693260335`, -0.4213362783255071}, \
{-0.4899175498853519, -0.4282577695574828}, {-0.4875200145554868, \
-0.4351393326405837}, {-0.4850774074650827, -0.441980667790392}, \
{-0.48259033191298534`, -0.44878148645277977`}, {-0.480059387821076, \
-0.4555415110399131}, {-0.47748517159091036`, -0.4622604746684465}, \
{-0.47486827597035636`, -0.4689381209001723}, {-0.47220928992985944`, \
-0.4755742034853688}, {-0.4695087985479673, -0.4821684861090673}, \
{-0.46676738290575154`, -0.48872074214043726`}, {-0.46398561998976956`, \
-0.495230754385468}, {-0.4611640826032153, -0.5016983148431071}, \
{-0.45830333928491573`, -0.508123224464997}, {-0.45540395423583624`, \
-0.5145052929189339}, {-0.4524664872527657, -0.5208443383561573}, \
{-0.44949149366886093`, -0.5271401871825645}, {-0.4464795243007374, \
-0.5333926738339273}, {-0.4434311254018023, -0.5396016405551785}, \
{-0.4403468386215344, -0.5457669371838201}, {-0.43722720097042367`, \
-0.5518884209374962}, {-0.4340727447902929, -0.5579659562057608}, \
{-0.43088399772973224`, -0.5639994143460619}, {-0.4276614827243861, \
-0.5699886734839539}, {-0.4244057179818412, -0.5759336183175411}, \
{-0.421117216970873, -0.5818341399261482}, {-0.4177964884148166, \
-0.5876901355832052}}]}, 
   {RGBColor[1, 1, 0], DiskBox[{0, 0}, 0.03]}},
  AspectRatio->Automatic,
  Background->RGBColor[0.6, 0.6, 0.6],
  ImageSize->{325., Automatic}]], "Output",
 CellChangeTimes->{3.401193407875846*^9, 3.4011936102014503`*^9, 
  3.401194912568746*^9, 3.4012768833829527`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tFinal", "=", "5"}], ";", 
  RowBox[{"R0", "=", "1"}], ";", 
  RowBox[{"V0", "=", 
   RowBox[{"1.3", "\[Times]", "2", " ", "\[Pi]"}]}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", ".001"}], ";", 
  RowBox[{"p", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CalculateOrbit", ";", 
  RowBox[{"DisplayOrbit", "[", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Clear", "[", 
  RowBox[{
  "tFinal", ",", "R0", ",", "V0", ",", "iMaximum", ",", "t", ",", "r", ",", 
   "v"}], "]"}]}], "Input",
 CellChangeTimes->{{3.401193360004507*^9, 3.401193452940729*^9}, 
   3.401193616562953*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 1, 1], Thickness[0.0075], 
    LineBox[{{1, 0}, {0.9999802609457525, 0.008168087155486545}, {
      0.9999210456375505, 0.016335851862053115`}, {0.9998223596382548, 
      0.02450297173619905}, {0.999684212216754, 0.032669124525217155`}, {
      0.9995066163441467, 0.04083398817247763}, {0.9992895886884042, 
      0.04899724088257688}, {0.9990331496075161, 0.05715856118630637}, {
      0.9987373231411278, 0.06531762800539713}, {0.9984021370006785, 
      0.07347412071699569}, {0.9980276225580483, 0.08162771921782766}, {
      0.997613814832729, 0.08977810398800572}, {0.9971607524775283, 
      0.09792495615443905}, {0.996668477762825, 0.10606795755380215`}, {
      0.9961370365593892, 0.11420679079502125`}, {0.9955664783197875, 
      0.12234113932123752`}, {0.9949568560583882, 0.13047068747120688`}, {
      0.9943082263299915, 0.13859512054009687`}, {0.9936206492071017, 
      0.14671412483964227`}, {0.9928941882558678, 0.15482738775762164`}, {
      0.9921289105107142, 0.16293459781661815`}, {0.9913248864476871, 
      0.17103544473202892`}, {0.9904821899565448, 0.17912961946928843`}, {
      0.989600898311616, 0.1872168143002719}, {0.9886810921414573, 
      0.19529672285884672`}, {0.987722855397338, 0.20336904019554006`}, {
      0.986726275320583, 0.21143346283129283`}, {0.9856914424088051, 
      0.2194896888102709}, {0.9846184503810579, 0.22753741775170575`}, {
      0.983507396141943, 0.23557635090073825`}, {0.9823583797447053, 
      0.24360619117824034`}, {0.981171504353349, 0.2516266432295905}, {
      0.9799468762038109, 0.2596374134723808}, {0.9786846045642257, 
      0.267638210143034}, {0.9773848016943177, 0.27562874334231124`}, {
      0.9760475828039582, 0.28360872507969087`}, {0.9746730660109215, 
      0.2915778693166025}, {0.97326137229788, 0.2995358920084995}, {
      0.9718126254686724, 0.30748251114575575`}, {0.9703269521038852, 
      0.3154174467933738}, {0.9688044815157828, 0.3233404211294925}, {
      0.9672453457026252, 0.33125115848268377`}, {0.9656496793024102, 
      0.3391493853680292}, {0.9640176195460782, 0.3470348305219695}, {
      0.9623493062102174, 0.35490722493591964`}, {0.9606448815693059, 
      0.36276630188864495`}, {0.9589044903475292, 0.37061179697739455`}, {
      0.9571282796702095, 0.378443448147789}, {0.9553163990148835, 
      0.3862609957224618}, {0.9534690001620664, 0.39406418242845337`}, {
      0.9515862371457363, 0.40185275342336024`}, {0.9496682662035775, 
      0.4096264563202403}, {0.9477152457270149, 0.41738504121127823`}, {
      0.9457273362110782, 0.4251282606902159}, {0.943704700204126, 
      0.4328558698735527}, {0.941647502257468, 0.4405676264205227}, {
      0.9395559088749156, 0.4482632905518574}, {0.9374300884622949, 
      0.4559426250673402}, {0.9352702112769542, 0.46360539536216555`}, {
      0.933076449377297, 0.4712513694421105}, {0.9308489765723719, 
      0.47888031793753155`}, {0.928587968371549, 0.4864920141161988}, {
      0.9262936019343124, 0.49408623389498035`}, {0.9239660560201972, 
      0.5016627558503908}, {0.9216055109388993, 0.5092213612280183}, {
      0.9192121485005846, 0.5167618339508464}, {0.9167861519664244, 
      0.5242839606264846}, {0.914327705999382, 0.5317875305533266}, {
      0.9118369966152752, 0.5392723357256511}, {0.9093142111341393, 
      0.5467381708376851}, {0.9067595381319131, 0.5541848332866451}, {
      0.9041731673924692, 0.5616121231747785}, {0.901555289860012, 
      0.5690198433104203}, {0.8989060975918621, 0.576407799208088}, {
      0.8962257837116478, 0.5837757990876321}, {0.8935145423629219, 
      0.591123653872464}, {0.8907725686632222, 0.5984511771868811}, {
      0.8880000586585924, 0.6057581853525097}, {0.8851972092785811, 
      0.6130444973838872}, {0.8823642182917317, 0.6203099349832044}, {
      0.8795012842615809, 0.6275543225342293}, {0.8766086065031772, 
      0.6347774870954338}, {0.8736863850401341, 0.6419792583923454}, {
      0.8707348205622293, 0.6491594688091445}, {0.8677541143835618, 
      0.6563179533795308}, {0.864744468401277, 0.6634545497768776}, {
      0.8617060850548713, 0.6705690983036993}, {0.8586391672860822, 
      0.6776614418804513}, {0.8555439184993764, 0.6847314260336846}, {
      0.8524205425230402, 0.6917788988835767}, {0.8492692435708811, 
      0.6988037111308605}, {0.846090226204546, 0.7058057160431727}, {
      0.8428836952964623, 0.7127847694408418}, {0.8396498559934065, 
      0.7197407296821384}, {0.8363889136807036, 0.7266734576480073}, {
      0.8331010739470638, 0.7335828167263027}, {0.8297865425500565, 
      0.740468672795547}, {0.8264455253822255, 0.7473308942082335}, {
      0.8230782284378478, 0.7541693517736928}, {0.8196848577803362, 
      0.7609839187405423}, {0.816265619510287, 0.7677744707787391}, {
      0.8128207197341736, 0.7745408859612554}, {0.809350364533685, 
      0.7812830447453945}, {0.8058547599357091, 0.7880008299537669}, {
      0.8023341118829588, 0.7946941267549443}, {0.7987886262052403, 
      0.8013628226438101}, {0.7952185085913611, 0.8080068074216229}, {
      0.7916239645616752, 0.8146259731758122}, {0.788005199441262, 
      0.821220214259521}, {0.7843624183337375, 0.8277894272709142}, {
      0.7806958260956918, 0.8343335110322677}, {0.7770056273117508, 
      0.8408523665688543}, {0.7732920262702572, 0.8473458970876429}, {
      0.7695552269395656, 0.8538140079558248}, {0.7657954329449479, 
      0.8602566066791839}, {0.7620128475461031, 0.8666736028803231}, {
      0.7582076736152661, 0.873064908276763}, {0.7543801136159098, 
      0.8794304366589253}, {0.7505303695820355, 0.885770103868016}, {
      0.7466586430980434, 0.8920838277738192}, {0.7427651352791794, 
      0.8983715282524162}, {0.7388500467525495, 0.9046331271638411}, {
      0.734913577638696, 0.910868548329686}, {0.7309559275337291, 
      0.9170777175106655}, {0.7269772954920053, 0.9232605623841543}, {
      0.7229778800093479, 0.9294170125217072}, {0.7189578790067991, 
      0.9355469993665715}, {0.714917489814899, 0.9416504562112046}, {
      0.7108569091584831, 0.9477273181748037}, {0.7067763331419894, 
      0.9537775221808594}, {0.7026759572352699, 0.959801006934741}, {
      0.698555976259896, 0.9657977129013232}, {0.6944165843759521, 
      0.9717675822826629}, {0.6902579750693087, 0.9777105589957333}, {
      0.6860803411393664, 0.9836265886502246}, {0.6818838746872637, 
      0.9895156185264177}, {0.6776687671045408, 0.9953775975531395}, {
      0.6734352090622503, 1.0012124762858055`}, {0.6691833905005073, 
      1.0070202068845573`}, {0.664913500618472, 1.0128007430925015`}, {
      0.660625727864754, 1.0185540402140554`}, {0.6563202599282338, 
      1.0242800550934064`}, {0.6519972837292901, 1.02997874609309}, {
      0.6476569854114274, 1.0356500730726916`}, {0.6432995503332948, 
      1.0412939973676787`}, {0.6389251630610883, 1.0469104817683657`}, {
      0.6345340073613287, 1.0524994904990186`}, {0.6301262661940076, 
      1.058060989197103}, {0.6257021217060924, 1.0635949448926774`}, {
      0.6212617552253853, 1.0691013259879407`}, {0.6168053472547252, 
      1.0745801022369317`}, {0.6123330774665277, 1.0800312447253881`}, {
      0.6078451246976538, 1.0854547258507679`}, {0.6033416669446018, 
      1.0908505193024327`}, {0.5988228813590117, 1.0962186000420022`}, {
      0.5942889442434796, 1.101558944283875}, {0.5897400310476691, 
      1.1068715294759246`}, {0.5851763163647186, 1.1121563342803693`}, {
      0.5805979739279313, 1.1174133385548186`}, {0.576005176607747, 
      1.122642523333499}, {0.5713980964089829, 1.1278438708086604`}, {
      0.5667769044683418, 1.1330173643121653`}, {0.5621417710521773, 
      1.13816298829726}, {0.5574928655545108, 1.1432807283205324`}, {
      0.552830356495294, 1.148370571024056}, {0.5481544115189099, 
      1.1534325041177196`}, {0.5434651973929061, 1.158466516361747}, {
      0.5387628800069545, 1.1634725975494038`}, {0.5340476243720302, 
      1.1684507384898948`}, {0.5293195946198052, 1.173400930991451}, {
      0.5245789540022491, 1.178323167844607}, {0.5198258648914327, 
      1.1832174428056683`}, {0.5150604887795274, 1.1880837505803712`}, {
      0.510282986278995, 1.1929220868077315`}, {0.5054935171229635, 
      1.1977324480440852`}, {0.5006922401657812, 1.2025148317473202`}, {
      0.4958793133837462, 1.2072692362612973`}, {0.49105489387600426`, 
      1.2119956608004625`}, {0.4862191378656107, 1.2166941054346485`}, {
      0.4813722007007509, 1.2213645710740655`}, {0.4765142368561145, 
      1.2260070594544812`}, {0.47164539993441895`, 1.2306215731225887`}, {
      0.4667658426680767, 1.2352081154215617`}, {0.46187571692100193`, 
      1.239766690476797}, {0.45697517369055246`, 1.244297303181842}, {
      0.4520643631096016, 1.2487999591845078`}, {0.44714343444873617`, 
      1.2532746648731665`}, {0.4422125361185764, 1.2577214273632311`}, {
      0.43727181567221296`, 1.2621402544838176`}, {0.43232141980775735`, 
      1.2665311547645886`}, {0.42736149437100196`, 1.2708941374227756`}, {
      0.42239218435818504`, 1.2752292123503814`}, {0.41741363391885755`, 
      1.2795363901015582`}, {0.4124259863588476, 1.2838156818801632`}, {
      0.40742938414331875`, 1.2880670995274888`}, {0.4024239688999193, 
      1.2922906555101654`}, {0.3974098814220182, 1.2964863629082377`}, {
      0.3923872616720243, 1.3006542354034112`}, {0.3873562487847863, 
      1.3047942872674674`}, {0.38231698107106926`, 1.308906533350848}, {
      0.3772695960211048, 1.312990989071405}, {0.37221423030821227`, 
      1.3170476704033158`}, {0.367151019792487, 1.3210765938661613`}, {
      0.36208009952455356`, 1.325077776514167}, {0.35700160374938067`, 
      1.329051235925604}, {0.35191566591015483`, 1.3329969901923475`}, {
      0.3468224186522104, 1.336915057909595}, {0.34172199382701307`, 
      1.340805458165738}, {0.3366145224961941, 1.3446682105323893`}, {
      0.33150013493563324`, 1.3485033350545612`}, {0.32637896063958716`, 
      1.3523108522409963`}, {0.3212511283248617, 1.3560907830546463`}, {
      0.31611676593502513`, 1.3598431489032983`}, {0.31097600064466047`, 
      1.3635679716303484`}, {0.3058289588636541, 1.367265273505718}, {
      0.30067576624151915`, 1.3709350772169142`}, {0.2955165476717517, 
      1.3745774058602303`}, {0.2903514272962163, 1.3781922829320858`}, {
      0.28518052850956127`, 1.3817797323205054`}, {0.28000397396365917`, 
      1.385339778296732}, {0.274821885572073, 1.3888724455069765`}, {
      0.2696343845145444, 1.3923777589642992`}, {0.26444159124150346`, 
      1.3958557440406223`}, {0.2592436254785974, 1.3993064264588735`}, {
      0.2540406062312374, 1.4027298322852557`}, {0.24883265178916117`, 
      1.4061259879216448`}, {0.24361987973101004`, 1.4094949200981113`}, {
      0.23840240692891934`, 1.4128366558655656`}, {0.2331803495531199, 
      1.416151222588525}, {0.22795382307654996`, 1.419438647938001}, {
      0.22272294227947573`, 1.4226989598845035`}, {0.21748782125411917`, 
      1.4259321866911645`}, {0.21224857340929212`, 1.4291383569069758`}, {
      0.20700531147503515`, 1.4323174993601397`}, {0.20175814750725993`, 
      1.4354696431515341`}, {0.1965071928923943, 1.4385948176482868`}, {
      0.19125255835202842`, 1.4416930524774594`}, {0.1859943539475613, 
      1.4447643775198398`}, {0.1807326890848464, 1.4478088229038402`}, {
      0.17546767251883538`, 1.4508264189994997`}, {0.17019941235821903`, 
      1.4538171964125919`}, {0.16492801607006433`, 1.456781185978832}, {
      0.15965359048444683`, 1.4597184187581869`}, {0.15437624179907736`, 
      1.4626289260292822`}, {0.14909607558392224`, 1.4655127392839087`}, {
      0.14381319678581617`, 1.468369890221624}, {0.138527709733067, 
      1.4712004107444494`}, {0.13323971814005167`, 1.4740043329516601`}, {
      0.12794932511180243`, 1.4767816891346683`}, {0.12265663314858269`, 
      1.4795325117719962`}, {0.11736174415045192`, 1.4822568335243391`}, {
      0.11206475942181883`, 1.4849546872297164`}, {0.10676577967598216`, 
      1.4876261058987095`}, {0.10146490503965855`, 1.490271122709786}, {
      0.09616223505749688, 1.492889771004707}, {0.09085786869657854, 
      1.495482084284019}, {0.08555190435090299, 1.498048096202626}, {
      0.08024443984585823, 1.500587840565443}, {0.07493557244267561, 
      1.5031013513231302`}, {0.06962539884286845, 1.5055886625679031`}, {
      0.06431401519265414, 1.508049808529421}, {0.05900151708735923, 
      1.5104848235707509`}, {0.053687999575807, 1.5128937421844069`}, {
      0.048373557164687324`, 1.5152765989884618`}, {0.04305828382290823, 
      1.5176334287227327`}, {0.037742272985928936`, 1.5199642662450363`}, {
      0.03242561756007396, 1.5222691465275162`}, {0.027108409926828003`, 
      1.5245481046530376`}, {0.021790741947111274`, 1.5268011758116513`}, {
      0.016472704965535004`, 1.5290283952971242`}, {0.011154389814636807`, 
      1.5312297985035361`}, {0.005835886819095687, 1.5334054209219412`}, {
      0.0005172857999263987, 1.5355552981370935`}, {-0.004801323921347049, 
      1.537679465824235}, {-0.01011985351853897, 
      1.539777959745947}, {-0.015438214656670878`, 
      1.5418508157490602`}, {-0.02075631948786185, 
      1.5438980697616278`}, {-0.026074080647240552`, 
      1.545919757789954}, {-0.031391411248879106`, 
      1.5479159159156837`}, {-0.03670822488174896, 
      1.5498865802929485`}, {-0.042024435605698914`, 
      1.5518317871455685`}, {-0.047339957947455366`, 
      1.5537515727643099`}, {-0.052654706896645066`, 
      1.555645973504197}, {-0.057968597901840294`, 
      1.5575150257818784`}, {-0.06328154686662671, 
      1.5593587660730446`}, {-0.0685934701456939, 
      1.5611772309099}, {-0.0739042845409488, 
      1.5629704568786829`}, {-0.07921390729765185, 
      1.5647384806172389`}, {-0.08452225610057625, 
      1.5664813388126417`}, {-0.08982924907019019, 
      1.5681990681988636`}, {-0.09513480475886205, 
      1.5698917055544932`}, {-0.1004388421470888, 
      1.5715592877005011`}, {-0.10574128063974751`, 
      1.5732018514980513`}, {-0.11104204006237002`, 
      1.5748194338463586`}, {-0.11634104065744076`, 
      1.5764120716805914`}, {-0.12163820308071788`, 
      1.5779798019698168`}, {-0.12693344839757742`, 
      1.5795226617149918`}, {-0.13222669807938084`, 
      1.5810406879469954`}, {-0.13751787399986568`, 
      1.5825339177247029`}, {-0.1428068984315594, 
      1.584002388133103}, {-0.14809369404221642`, 
      1.5854461362814531`}, {-0.15337818389127833`, 
      1.5868651993014777`}, {-0.15866029142635713`, 
      1.5882596143456038`}, {-0.1639399404797416, 
      1.589629418585236}, {-0.1692170552649269, 
      1.5909746492090695`}, {-0.1744915603731667, 
      1.592295343421441}, {-0.17976338077004891`, 
      1.5935915384407162`}, {-0.18503244179209366`, 
      1.594863271497713}, {-0.19029866914337462`, 
      1.5961105798341617`}, {-0.19556198889216284`, 
      1.5973335007011997`}, {-0.20082232746759343`, 
      1.5985320713578997`}, {-0.2060796116563548, 
      1.5997063290698335`}, {-0.21133376859940053`, 
      1.6008563111076688`}, {-0.2165847257886837, 
      1.6019820547457977`}, {-0.22183241106391371`, 
      1.6030835972609991`}, {-0.22707675260933533`, 
      1.6041609759311324`}, {-0.2323176789505301, 
      1.6052142280338613`}, {-0.23755511895123985`, 
      1.6062433908454106`}, {-0.24278900181021226`, 
      1.6072485016393512`}, {-0.24801925705806846`, 
      1.6082295976854164`}, {-0.2532458145541924, 
      1.609186716248346}, {-0.25846860448364223`, 
      1.6101198945867607`}, {-0.26368755735408295`, 
      1.611029169952064}, {-0.26890260399274113`, 
      1.611914579587372}, {-0.2741136755433806, 
      1.6127761607264706`}, {-0.27932070346329996`, 
      1.6136139505928002`}, {-0.28452361952035093`, 
      1.6144279863984663`}, {-0.2897223557899782, 
      1.6152183053432763`}, {-0.2949168446522798, 
      1.6159849446138024`}, {-0.30010701878908913`, 
      1.6167279413824693`}, {-0.30529281118107693`, 
      1.617447332806667}, {-0.31047415510487464`, 
      1.6181431560278872`}, {-0.3156509841302177, 
      1.6188154481708856`}, {-0.32082323211710984`, 
      1.6194642463428661`}, {-0.32599083321300726`, 
      1.6200895876326884`}, {-0.3311537218500231, 
      1.6206915091101}, {-0.3363118327421521, 
      1.6212700478249884`}, {-0.341465100882515, 
      1.6218252408066576`}, {-0.346613461540623, 
      1.6223571250631252`}, {-0.35175685025966136`, 
      1.622865737580441}, {-0.35689520285379334`, 
      1.623351115322027}, {-0.3620284554054829, 
      1.6238132952280382`}, {-0.3671565442628369, 
      1.6242523142147431`}, {-0.3722794060369665, 
      1.6246682091739257`}, {-0.3773969775993677, 
      1.6250610169723054`}, {-0.3825091960793204, 
      1.6254307744509777`}, {-0.38761599886130654`, 
      1.625777518424874}, {-0.3927173235824469, 
      1.626101285682239}, {-0.3978131081299561, 
      1.6264021129841284`}, {-0.4029032906386162, 
      1.6266800370639234`}, {-0.4079878094882685, 
      1.6269350946268637`}, {-0.41306660330132344`, 
      1.627167322349598}, {-0.41813961094028834`, 
      1.6273767568797517`}, {-0.423206771505313, 
      1.6275634348355128`}, {-0.4282680243317532, 
      1.6277273928052332`}, {-0.4333233089877515, 
      1.6278686673470464`}, {-0.4383725652718356, 
      1.6279872949885037`}, {-0.44341573321053396`, 
      1.6280833122262237`}, {-0.44845275305600857`, 
      1.6281567555255596`}, {-0.45348356528370476`, 
      1.6282076613202807`}, {-0.45850811059001784`, 
      1.6282360660122701`}, {-0.4635263298899766, 
      1.628242005971237}, {-0.46853816431494333`, 
      1.6282255175344444`}, {-0.47354355521033037`, 
      1.6281866370064504`}, {-0.478542444133333, 
      1.6281254006588641`}, {-0.4835347728506787, 
      1.6280418447301173`}, {-0.4885204833363921, 
      1.6279360054252474`}, {-0.4934995177695764, 
      1.6278079189156964`}, {-0.49847181853221023`, 
      1.6276576213391218`}, {-0.5034373282069604, 
      1.627485148799222}, {-0.50839598957501, 
      1.6272905373655735`}, {-0.5133477456139022, 
      1.6270738230734827`}, {-0.5182925394953993, 
      1.6268350419238489`}, {-0.5232303145833568, 
      1.6265742298830403`}, {-0.5281610144316123, 
      1.6262914228827832`}, {-0.5330845827818902, 
      1.6259866568200614`}, {-0.53800096356172, 
      1.6256599675570298`}, {-0.5429101008823702, 
      1.6253113909209387`}, {-0.5478119390367961, 
      1.6249409627040687`}, {-0.5527064224976026, 
      1.62454871866368}, {-0.5575934959150204, 
      1.6241346945219692`}, {-0.5624731041148974, 
      1.6236989259660408`}, {-0.5673451920967033, 
      1.623241448647887}, {-0.5722097050315487, 
      1.62276229818438}, {-0.5770665882602171, 
      1.6222615101572735`}, {-0.5819157872912116, 
      1.621739120113217}, {-0.5867572477988148, 
      1.6211951635637774`}, {-0.5915909156211612, 
      1.6206296759854735`}, {-0.5964167367583243, 
      1.620042692819819}, {-0.6012346573704161, 
      1.6194342494733764`}, {-0.6060446237756996, 
      1.6188043813178195`}, {-0.6108465824487141, 
      1.6181531236900069`}, {-0.6156404800184143, 
      1.6174805118920645`}, {-0.6204262632663204, 
      1.6167865811914774`}, {-0.6252038791246826, 
      1.6160713668211906`}, {-0.6299732746746562, 
      1.6153349039797193`}, {-0.6347343971444912, 
      1.6145772278312693`}, {-0.6394871939077318, 
      1.6137983735058643`}, {-0.6442316124814297, 
      1.6129983760994835`}, {-0.6489676005243687, 
      1.6121772706742072`}, {-0.6536951058353012, 
      1.6113350922583713`}, {-0.6584140763511965, 
      1.6104718758467307`}, {-0.6631244601455012, 
      1.6095876564006295`}, {-0.66782620542641, 
      1.608682468848182}, {-0.6725192605351493, 
      1.6077563480844592`}, {-0.6772035739442713, 
      1.6068093289716858`}, {-0.6818790942559596, 
      1.6058414463394433`}, {-0.6865457702003454, 
      1.6048527349848816`}, {-0.6912035506338353, 
      1.6038432296729392`}, {-0.6958523845374505, 
      1.6028129651365703`}, {-0.7004922210151746, 
      1.6017619760769792`}, {-0.7051230092923151, 
      1.6006902971638632`}, {-0.7097446987138728, 
      1.5995979630356625`}, {-0.7143572387429235, 
      1.5984850082998172`}, {-0.7189605789590088, 
      1.597351467533032}, {-0.7235546690565382, 
      1.5961973752815484`}, {-0.7281394588432007, 
      1.5950227660614227`}, {-0.7327148982383869, 
      1.5938276743588131`}, {-0.7372809372716201, 
      1.592612134630272}, {-0.741837526080999, 
      1.5913761813030463`}, {-0.7463846149116482, 
      1.5901198487753838`}, {-0.7509221541141804, 
      1.5888431714168476`}, {-0.755450094143166, 
      1.5875461835686357`}, {-0.759968385555614, 
      1.5862289195439085`}, {-0.7644769790094612, 
      1.5848914136281225`}, {-0.7689758252620714, 
      1.5835337000793699`}, {-0.7734648751687433, 
      1.582155813128726}, {-0.7779440796812274, 
      1.5807577869806027`}, {-0.7824133898462523, 
      1.5793396558131063`}, {-0.7868727568040601, 
      1.5779014537784055`}, {-0.79132213178695, 
      1.5764432150031025`}, {-0.7957614661178308, 
      1.574964973588611}, {-0.8001907112087827, 
      1.5734667636115425`}, {-0.8046098185596265, 
      1.5719486191240948`}, {-0.8090187397565025, 
      1.5704105741544505`}, {-0.8134174264704568, 
      1.5688526627071793`}, {-0.8178058304560365, 
      1.5672749187636472`}, {-0.8221839035498931, 
      1.5656773762824312`}, {-0.8265515976693936, 
      1.5640600691997402`}, {-0.8309088648112397, 
      1.5624230314298417`}, {-0.8352556570500959, 
      1.560766296865495}, {-0.8395919265372238, 
      1.5590898993783882`}, {-0.8439176254991261, 
      1.5573938728195842`}, {-0.8482327062361965, 
      1.5556782510199698`}, {-0.8525371211213787, 
      1.5539430677907116`}, {-0.8568308225988321, 
      1.5521883569237174`}, {-0.8611137631826056, 
      1.5504141521921033`}, {-0.8653858954553176, 
      1.5486204873506673`}, {-0.8696471720668452, 
      1.5468073961363664`}, {-0.8738975457330188, 
      1.5449749122688021`}, {-0.8781369692343255, 
      1.5431230694507088`}, {-0.8823653954146179, 
      1.54125190136845}, {-0.8865827771798315, 
      1.5393614416925183`}, {-0.8907890674967079, 
      1.5374517240780425`}, {-0.894984219391526, 
      1.5355227821652986`}, {-0.8991681859488383, 
      1.5335746495802276`}, {-0.9033409203102162, 
      1.5316073599349582`}, {-0.9075023756729999, 
      1.529620946828335}, {-0.9116525052890558, 
      1.5276154438464522`}, {-0.915791262463541, 
      1.525590884563193}, {-0.9199186005536729, 
      1.5235473025407735`}, {-0.9240344729675062, 
      1.521484731330294}, {-0.928138833162716, 
      1.519403204472293}, {-0.9322316346453869, 
      1.5173027554973089`}, {-0.936312830968809, 
      1.5151834179264463`}, {-0.9403823757322786, 
      1.513045225271947}, {-0.9444402225799066, 
      1.5108882110377682`}, {-0.9484863251994318, 
      1.5087124087201633`}, {-0.9525206373210406, 
      1.5065178518082716`}, {-0.9565431127161927, 
      1.5043045737847098`}, {-0.9605537051964518, 
      1.5020726081261724`}, {-0.964552368612323, 
      1.4998219883040342`}, {-0.9685390568520954, 
      1.4975527477849606`}, {-0.9725137238406907, 
      1.4952649200315222`}, {-0.9764763235385165, 
      1.4929585385028152`}, {-0.9804268099403266, 
      1.490633636655087}, {-0.9843651370740848, 
      1.4882902479423674`}, {-0.988291258999836, 
      1.485928405817105}, {-0.9922051298085816, 
      1.4835481437308091`}, {-0.9961067036211599, 
      1.4811494951346975`}, {-0.9999959345871333, 
      1.4787324934803494`}, {-1.0038727768836784`, 
      1.4762971722203633`}, {-1.0077371847144831`, 
      1.4738435648090213`}, {-1.0115891123086482`, 
      1.4713717047029589`}, {-1.0154285139195935`, 
      1.4688816253618384`}, {-1.0192553438239695`, 
      1.4663733602490314`}, {-1.0230695563205738`, 
      1.4638469428323033`}, {-1.0268711057292723`, 
      1.4613024065845055`}, {-1.0306599463899258`, 
      1.4587397849842725`}, {-1.0344360326613198`, 
      1.4561591115167245`}, {-1.038199318920101, 
      1.453560419674176}, {-1.0419497595597178`, 
      1.4509437429568501`}, {-1.045687308989364, 
      1.448309114873598}, {-1.0494119216329292`, 
      1.4456565689426244`}, {-1.0531235519279527`, 
      1.442986138692219}, {-1.056822154324582, 
      1.4402978576614929`}, {-1.0605076832845355`, 
      1.4375917594011225`}, {-1.06418009328007, 
      1.4348678774740968`}, {-1.0678393387929528`, 
      1.4321262454564725`}, {-1.0714853743134374`, 
      1.4293668969381341`}, {-1.0751181543392436`, 
      1.42658986552356}, {-1.078737633374543, 
      1.423795184832595}, {-1.082343765928947, 
      1.4209828885012277`}, {-1.0859365065164996`, 
      1.4181530101823754`}, {-1.0895158096546755`, 
      1.4153055835466737`}, {-1.0930816298633803`, 
      1.4124406422832727`}, {-1.0966339216639562`, 
      1.4095582201006394`}, {-1.1001726395781912`, 
      1.4066583507273662`}, {-1.1036977381273323`, 
      1.4037410679129858`}, {-1.1072091718311026`, 
      1.4008064054287912`}, {-1.1107068952067227`, 
      1.3978543970686637`}, {-1.1141908627679353`, 
      1.394885076649906}, {-1.1176610290240345`, 
      1.3918984780140815`}, {-1.1211173484788979`, 
      1.388894635027862}, {-1.1245597756300232`, 
      1.385873581583878}, {-1.1279882649675688`, 
      1.3828353516015799`}, {-1.1314027709733974`, 
      1.3797799790281022`}, {-1.1348032481201233`, 
      1.3767074978391367`}, {-1.1381896508701645`, 
      1.3736179420398105`}, {-1.141561933674797, 
      1.3705113456655715`}, {-1.1449200509732136`, 
      1.3673877427830816`}, {-1.1482639571915858`, 
      1.3642471674911143`}, {-1.1515936067421297`, 
      1.3610896539214612`}, {-1.1549089540221757`, 
      1.357915236239845}, {-1.1582099534132404`, 
      1.3547239486468383`}, {-1.1614965592801036`, 
      1.351515825378791}, {-1.1647687259698876`, 
      1.3482909007087627`}, {-1.1680264078111406`, 
      1.345049208947465}, {-1.1712695591129239`, 
      1.3417907844442083`}, {-1.1744981341639005`, 
      1.3385156615878568`}, {-1.1777120872314302`, 
      1.3352238748077916`}, {-1.1809113725606657`, 
      1.3319154585748796`}, {-1.1840959443736525`, 
      1.3285904474024512`}, {-1.187265756868433, 
      1.3252488758472847`}, {-1.1904207642181535`, 
      1.321890778510598}, {-1.1935609205701734`, 
      1.3185161900390487`}, {-1.1966861800451802`, 
      1.315125145125741}, {-1.199796496736305, 
      1.311717678511241}, {-1.2028918247082439`, 
      1.308293824984599}, {-1.2059721179963796`, 
      1.3048536193843812`}, {-1.20903733060591, 
      1.301397096599707}, {-1.2120874165109763`, 
      1.2979242915712965`}, {-1.2151223296537979`, 
      1.2944352392925247`}, {-1.218142023943807, 
      1.290929974810484}, {-1.2211464532567895`, 
      1.2874085332270562`}, {-1.2241355714340272`, 
      1.2838709496999905`}, {-1.2271093322814441`, 
      1.280317259443992}, {-1.2300676895687557`, 
      1.2767474977318176`}, {-1.2330105970286211`, 
      1.27316169989538}, {-1.2359380083557994`, 
      1.269559901326862}, {-1.2388498772063075`, 
      1.2659421374798367`}, {-1.2417461571965835`, 
      1.2623084438704}, {-1.2446268019026518`, 
      1.2586588560783087`}, {-1.2474917648592907`, 
      1.25499340974813}, {-1.2503409995592052`, 
      1.2513121405903969`}, {-1.2531744594522018`, 
      1.2476150843827771`}, {-1.2559920979443666`, 
      1.2439022769712471`}, {-1.258793868397247, 
      1.2401737542712776`}, {-1.2615797241270368`, 
      1.2364295522690278`}, {-1.264349618403763, 
      1.232669707022549}, {-1.2671035044504788`, 
      1.2288942546629975`}, {-1.269841335442457, 
      1.2251032313958585`}, {-1.2725630645063881`, 
      1.221296673502178}, {-1.2752686447195822`, 
      1.2174746173398048`}, {-1.2779580291091728`, 
      1.213637099344644}, {-1.2806311706513245`, 
      1.209784156031918}, {-1.283288022270445, 
      1.2059158239974406`}, {-1.2859285368383995`, 
      1.202032139918898}, {-1.2885526671737284`, 
      1.198133140557143}, {-1.2911603660408688`, 
      1.1942188627574972`}, {-1.2937515861493798`, 
      1.1902893434510666`}, {-1.2963262801531703`, 
      1.1863446196560643`}, {-1.2988844006497313`, 
      1.1823847284791469`}, {-1.3014259001793713`, 
      1.17840970711676}, {-1.3039507312244547`, 
      1.1744195928564958`}, {-1.306458846208645, 
      1.17041442307846}, {-1.3089501974961506`, 
      1.1663942352566528`}, {-1.3114247373909744`, 
      1.1623590669603572`}, {-1.3138824181361677`, 
      1.1583089558555428`}, {-1.3163231919130869`, 
      1.154243939706278}, {-1.3187470108406545`, 
      1.150164056376155}, {-1.3211538269746244`, 
      1.1460693438297271`}, {-1.3235435923068495`, 
      1.1419598401339563`}, {-1.3259162587645548`, 
      1.137835583459675}, {-1.3282717782096136`, 
      1.133696612083057}, {-1.3306101024378276`, 
      1.1295429643871038`}, {-1.3329311831782116`, 
      1.1253746788631407`}, {-1.3352349720922811`, 
      1.1211917941123264`}, {-1.3375214207733457`, 
      1.1169943488471759`}, {-1.339790480745805, 
      1.112782381893094}, {-1.3420421034644499`, 
      1.1085559321899243`}, {-1.3442762403137674`, 
      1.1043150387935092`}, {-1.3464928426072502`, 
      1.1000597408772632`}, {-1.3486918615867107`, 
      1.0957900777337606`}, {-1.350873248421599, 
      1.091506088776335}, {-1.3530369542083258`, 
      1.0872078135406942`}, {-1.3551829299695894`, 
      1.0828952916865464`}, {-1.3573111266537086`, 
      1.078568562999242}, {-1.359421495133958, 
      1.0742276673914288`}, {-1.3615139862079115`, 
      1.069872644904721}, {-1.3635885505967866`, 
      1.0655035357113818`}, {-1.365645138944797, 
      1.0611203801160227`}, {-1.3676837018185084`, 
      1.056723218557314}, {-1.3697041897062001`, 
      1.0523120916097128`}, {-1.3717065530172317`, 
      1.0478870399852036`}, {-1.3736907420814142`, 
      1.0434481045350554`}, {-1.3756567071483883`, 
      1.0389953262515932`}, {-1.3776043983870059`, 
      1.0345287462699844`}, {-1.379533765884719, 
      1.030048405870041}, {-1.3814447596469737`, 
      1.025554346478038}, {-1.3833373295966083`, 
      1.0210466096685458`}, {-1.3852114255732602`, 
      1.0165252371662803`}, {-1.3870669973327763`, 
      1.0119902708479676`}, {-1.3889039945466302`, 
      1.0074417527442252`}, {-1.3907223668013458`, 
      1.0028797250414607`}, {-1.392522063597927, 
      0.9983042300837852}, {-1.3943030343512932`, 
      0.9937153103749446}, {-1.3960652283897226`, 
      0.9891130085802672}, {-1.3978085949543002`, 
      0.9844973675286284}, {-1.3995330831983746`, 
      0.9798684302144328}, {-1.4012386421870195`, 
      0.9752262397996135}, {-1.4029252208965042`, 
      0.9705708396156487}, {-1.4045927682137693`, 
      0.9659022731655961}, {-1.4062412329359115`, 
      0.9612205841261453}, {-1.4078705637696742`, 
      0.9565258163496885}, {-1.4094807093309463`, 
      0.9518180138664085}, {-1.4110716181442688`, 
      0.9470972208863864}, {-1.412643238642349, 
      0.9423634818017264}, {-1.4141955191655822`, 
      0.9376168411887007}, {-1.4157284079615822`, 
      0.9328573438099128}, {-1.41724185318472, 
      0.9280850346164796}, {-1.4187358028956698`, 
      0.9232999587502334}, {-1.420210205060966, 
      0.9185021615459432}, {-1.4216650075525656`, 
      0.9136916885335558}, {-1.4231001581474232`, 
      0.9088685854404562}, {-1.4245156045270715`, 
      0.9040328981937494}, {-1.4259112942772139`, 
      0.899184672922561}, {-1.4272871748873253`, 
      0.8943239559603593}, {-1.4286431937502626`, 
      0.8894507938472976}, {-1.4299792981618855`, 
      0.8845652333325779}, {-1.4312954353206864`, 
      0.8796673213768347}, {-1.4325915523274317`, 
      0.8747571051545415}, {-1.433867596184813, 
      0.8698346320564375}, {-1.4351235137971088`, 
      0.8648999496919766}, {-1.4363592519698571`, 
      0.8599531058917983}, {-1.43757475740954, 
      0.8549941487102208}, {-1.4387699767232782`, 
      0.850023126427756}, {-1.4399448564185375`, 
      0.8450400875536475}, {-1.4410993429028482`, 
      0.840045080828431}, {-1.442233382483535, 
      0.8350381552265179}, {-1.4433469213674595`, 
      0.8300193599588023}, {-1.4444399056607764`, 
      0.8249887444752904}, {-1.4455122813687011`, 
      0.8199463584677549}, {-1.4465639943952906`, 
      0.8148922518724121}, {-1.447594990543238, 
      0.8098264748726237}, {-1.4486052155136804`, 
      0.8047490779016225}, {-1.4495946149060213`, 
      0.799660111645263}, {-1.4505631342177656`, 
      0.7945596270447959}, {-1.4515107188443708`, 
      0.7894476752996687}, {-1.4524373140791114`, 
      0.7843243078703509}, {-1.4533428651129592`, 
      0.7791895764811839}, {-1.4542273170344784`, 
      0.7740435331232579}, {-1.4550906148297371`, 
      0.7688862300573134}, {-1.455932703382234, 
      0.7637177198166695}, {-1.4567535274728411`, 
      0.7585380552101785}, {-1.4575530317797645`, 
      0.7533472893252067}, {-1.45833116087852, 
      0.7481454755306418}, {-1.4590878592419283`, 
      0.7429326674799283}, {-1.4598230712401252`, 
      0.7377089191141287}, {-1.460536741140593, 
      0.7324742846650136}, {-1.4612288131082072`, 
      0.7272288186581786}, {-1.4618992312053036`, 
      0.7219725759161889}, {-1.4625479393917646`, 
      0.7167056115617533}, {-1.4631748815251242`, 
      0.7114279810209254}, {-1.4637800013606934`, 
      0.7061397400263341}, {-1.4643632425517061`, 
      0.7008409446204424}, {-1.4649245486494848`, 
      0.6955316511588364}, {-1.4654638631036292`, 
      0.6902119163135423}, {-1.4659811292622238`, 
      0.684881797076374}, {-1.4664762903720705`, 
      0.67954135076231}, {-1.4669492895789413`, 
      0.6741906350129009}, {-1.4674000699278549`, 
      0.6688297077997062}, {-1.467828574363377, 
      0.6634586274277628}, {-1.4682347457299434`, 
      0.658077452539084}, {-1.4686185267722085`, 
      0.6526862421161885}, {-1.4689798601354183`, 
      0.6472850554856621}, {-1.469318688365808, 
      0.6418739523217497}, {-1.469634953911027, 
      0.6364529926499801}, {-1.4699285991205884`, 
      0.6310222368508216}, {-1.4701995662463472`, 
      0.6255817456633707}, {-1.4704477974430044`, 
      0.6201315801890727}, {-1.47067323476864, 
      0.6146718018954751}, {-1.4708758201852745`, 
      0.609202472620014}, {-1.471055495559459, 
      0.6037236545738334}, {-1.4712122026628958`, 
      0.5982354103456375}, {-1.471345883173088, 
      0.5927378029055778}, {-1.4714564786740212`, 
      0.5872308956091719}, {-1.4715439306568772`, 
      0.5817147522012586}, {-1.4716081805207775`, 
      0.5761894368199854}, {-1.471649169573562, 
      0.5706550140008312}, {-1.4716668390326, 
      0.5651115486806638}, {-1.4716611300256353`, 
      0.5595591062018316}, {-1.4716319835916665`, 
      0.5539977523162909}, {-1.4715793406818625`, 
      0.5484275531897682}, {-1.471503142160514, 
      0.5428485754059582}, {-1.4714033288060222`, 
      0.5372608859707569}, {-1.4712798413119246`, 
      0.5316645523165314}, {-1.4711326202879602`, 
      0.5260596423064249}, {-1.4709616062611726`, 
      0.5204462242386982}, {-1.470766739677053, 
      0.5148243668511083}, {-1.4705479609007255`, 
      0.5091941393253221}, {-1.4703052102181713`, 
      0.5035556112913684}, {-1.4700384278374972`, 
      0.4979088528321258}, {-1.4697475538902462`, 
      0.4922539344878483}, {-1.4694325284327523`, 
      0.4865909272607276}, {-1.4690932914475394`, 
      0.48091990261949336`}, {-1.4687297828447674`, 
      0.4752409325040505}, {-1.4683419424637225`, 
      0.4695540893301545}, {-1.4679297100743574`, 
      0.4638594459941247}, {-1.4674930253788776`, 
      0.4581570758775948}, {-1.4670318280133785`, 
      0.4524470528523022}, {-1.4665460575495315`, 
      0.44672945128491526`}, {-1.4660356534963221`, 
      0.4410043460418987}, {-1.4655005553018405`, 
      0.4352718124944175}, {-1.4649407023551233`, 
      0.4295319265232793}, {-1.4643560339880517`, 
      0.42378476452391534`}, {-1.4637464894773036`, 
      0.4180304034113998}, {-1.4631120080463618`, 
      0.4122689206255082}, {-1.46245252886758, 
      0.4065003941358142}, {-1.461767991064307, 
      0.4007249024468253}, {-1.4610583337130703`, 
      0.3949425246031575}, {-1.4603234958458196`, 
      0.3891533401947483}, {-1.4595634164522335`, 
      0.3833574293621091}, {-1.4587780344820878`, 
      0.377554872801616}, {-1.4579672888476873`, 
      0.37174575177083957`}, {-1.4571311184263651`, 
      0.3659301480939135}, {-1.4562694620630456`, 
      0.3601081441669417}, {-1.4553822585728773`, 
      0.35427982296344424`}, {-1.454469446743933, 
      0.3484452680398419}, {-1.453530965339981, 
      0.3426045635409791}, {-1.4525667531033275`, 
      0.33675779420568563`}, {-1.4515767487577313`, 
      0.330905045372376}, {-1.450560891011394, 
      0.32504640298468757`}, {-1.4495191185600238`, 
      0.31918195359715623`}, {-1.448451370089977, 
      0.3133117843809303}, {-1.447357584281478, 
      0.3074359831295218}, {-1.4462376998119186`, 
      0.3015546382645953}, {-1.4450916553592377`, 
      0.2956678388417937}, {-1.4439193896053841`, 
      0.2897756745566015}, {-1.4427208412398644`, 
      0.28387823575024435`}, {-1.4414959489633734`, 
      0.2779756134156249}, {-1.4402446514915144`, 
      0.27206789920329516`}, {-1.4389668875586064`, 
      0.2661551854274644}, {-1.4376625959215819`, 
      0.2602375650720424}, {-1.436331715363976, 
      0.2543151317967175}, {-1.4349741847000097`, 
      0.24838797994307}, {-1.433589942778766, 
      0.24245620454071878`}, {-1.432178928488465, 
      0.23651990131350226`}, {-1.430741080760836, 
      0.23057916668569234`}, {-1.4292763385755893`, 
      0.22463409778824103`}, {-1.4277846409649895`, 
      0.21868479246505929`}, {-1.4262659270185327`, 
      0.21273134927932733`}, {-1.4247201358877293`, 
      0.20677386751983595`}, {-1.4231472067909918`, 
      0.20081244720735814`}, {-1.4215470790186333`, 
      0.19484718910105028`}, {-1.4199196919379746`, 
      0.18887819470488232`}, {-1.418264984998566, 
      0.18290556627409604`}, {-1.4165828977375206`, 
      0.1769294068216908}, {-1.4148733697849658`, 
      0.17094982012493573`}, {-1.413136340869612, 
      0.1649669107319077}, {-1.411371750824441, 
      0.15898078396805404`}, {-1.4095795395925168`, 
      0.15299154594277922`}, {-1.40775964723292, 
      0.14699930355605426`}, {-1.4059120139268075`, 
      0.14100416450504807`}, {-1.4040365799836023`, 
      0.13500623729077965`}, {-1.4021332858473097`, 
      0.12900563122478978`}, {-1.4002020721029682`, 
      0.12300245643583134`}, {-1.398242879483234, 
      0.11699682387657694`}, {-1.3962556488751003`, 
      0.11098884533034248`}, {-1.394240321326757, 
      0.10497863341782554`}, {-1.3921968380545877`, 
      0.09896630160385708}, {-1.3901251404503132`, 
      0.0929519642041651}, {-1.3880251700882764`, 
      0.08693573639214881}, {-1.3858968687328752`, 
      0.0809177342056617}, {-1.383740178346145, 
      0.07489807455380201}, {-1.3815550410954918`, 
      0.06887687522370897}, {-1.3793413993615793`, 
      0.06285425488736301}, {-1.3770991957463714`, 
      0.05683033310838829}, {-1.3748283730813329`, 
      0.050805230348855576`}, {-1.3725288744357902`, 
      0.0447790679760838}, {-1.3702006431254556`, 
      0.03875196826943812}, {-1.367843622721115, 
      0.03272405442712255}, {-1.365457757057484, 
      0.02669545057296507}, {-1.3630429902422325`, 
      0.020666281763193083`}, {-1.3605992666651827`, 
      0.014636673993196868`}, {-1.358126531007679, 
      0.008606754204278828}, {-1.3556247282521368`, 
      0.0025766502903860934`}, {-1.3530938036917675`, \
-0.0034535088951759777`}, {-1.3505337029404876`, -0.00948359353305195}, \
{-1.3479443719430089`, -0.015513472831168726`}, {-1.3453257569851154`, \
-0.021543015018094955`}, {-1.3426778047041292`, -0.027572087336430205`}, \
{-1.3400004620995656`, -0.03360055603622682}, {-1.3372936765439818`, \
-0.03962828636844733}, {-1.3345573957940209`, -0.045655142578460636`}, \
{-1.331791568001653, -0.05168098789957996}, {-1.3289961417256166`, \
-0.05770568454664584}, {-1.3261710659430623`, -0.0637290937096576}, \
{-1.3233162900613997`, -0.06975107554745659}, {-1.3204317639303527`, \
-0.07577148918146484}, {-1.3175174378542227`, -0.08179019268948262}, \
{-1.3145732626043636`, -0.08780704309954887}, {-1.3115991894318697`, \
-0.09382189638386801}, {-1.3085951700804797`, -0.09983460745280742}, \
{-1.305561156799698, -0.10584503014896934`}, {-1.3024971023581366`, \
-0.11185301724134152`}, {-1.2994029600570782`, -0.11785842041953082`}, \
{-1.296278683744264, -0.12386109028808417`}, {-1.2931242278279074`, \
-0.12986087636090138`}, {-1.2899395472909359`, -0.13585762705574425`}, \
{-1.2867245977054629`, -0.14185118968884702`}, {-1.2834793352474914`, \
-0.14784141046963273`}, {-1.2802037167118514`, -0.15382813449554045`}, \
{-1.2768976995273733`, -0.15981120574696867`}, {-1.273561241772298, \
-0.16579046708233985`}, {-1.2701943021899262`, -0.17176576023329154`}, \
{-1.266796840204509, -0.17773692579999945`}, {-1.2633688159373784`, \
-0.18370380324663818`}, {-1.259910190223324, -0.1896662308969852}, \
{-1.256420924627213, -0.19562404593017382`}, {-1.2529009814608567`, \
-0.20157708437660146`}, {-1.2493503238001236`, -0.20752518111399879`}, \
{-1.2457689155023035`, -0.21346816986366646`}, {-1.2421567212237181`, \
-0.21940588318688536`}, {-1.2385137064375842`, -0.2253381524815073}, \
{-1.234839837452127, -0.23126480797873225`}, {-1.2311350814289457`, \
-0.23718567874007934`}, {-1.2273994064016318`, -0.24310059265455813`}, \
{-1.2236327812946401`, -0.24900937643604748`}, {-1.2198351759424133`, \
-0.25491185562088886`}, {-1.2160065611087598`, -0.2608078545657016}, \
{-1.2121469085064853`, -0.26669719644542744`}, {-1.2082561908172778`, \
-0.27257970325161185`}, {-1.2043343817118457`, -0.2784551957909298}, \
{-1.2003814558703094`, -0.28432349368396403`}, {-1.196397389002844, \
-0.29018441536424316`}, {-1.1923821578705749`, -0.29603777807754844`}, \
{-1.1883357403067225`, -0.3018833978814969}, {-1.1842581152379985`, \
-0.30772108964540895`}, {-1.1801492627062484`, -0.31355066705046986`}, \
{-1.1760091638903425`, -0.3193719425901923}, {-1.1718378011283108`, \
-0.3251847275711898}, {-1.167635157939722, -0.3309888321142694}, \
{-1.1634012190483027`, -0.33678406515585213`}, {-1.1591359704047972`, \
-0.3425702344497311}, {-1.154839399210061, -0.3483471465691756}, \
{-1.1505114939383903`, -0.35411460690939073`}, {-1.1461522443610799`, \
-0.35987241969034217`}, {-1.1417616415702097`, -0.3656203879599547}, \
{-1.1373396780026537`, -0.371358313597695}, {-1.1328863474643094`, \
-0.3770859973185475}, {-1.128401645154542, -0.38280323867739346`}, \
{-1.123885567690841, -0.38850983607380296`}, {-1.1193381131336824`, \
-0.3942055867572492}, {-1.1147592810115927`, -0.3998902868327556}, \
{-1.1101490723464094`, -0.40556373126698547`}, {-1.1055074896787314`, \
-0.4112257138947837}, {-1.1008345370935553`, -0.4168760274261815}, \
{-1.0961302202460896`, -0.42251446345387345`}, {-1.0913945463877406`, \
-0.4281408124611771}, {-1.0866275243922627`, -0.4337548638304856}, \
{-1.0818291647820684`, -0.4393564058522234}, {-1.076999479754686, \
-0.4449452257343143}, {-1.0721384832093608`, -0.45052110961217334`}, \
{-1.0672461907737891`, -0.4560838425592319}, {-1.062322619830978, \
-0.4616332085980056}, {-1.0573677895462201`, -0.4671689907117159}, \
{-1.052381720894174, -0.472690970856475}, {-1.0473644366860426`, \
-0.4781989299740437}, {-1.0423159615968365`, -0.4836926480051724}, \
{-1.037236322192712, -0.4891719039035351}, {-1.0321255469583734`, \
-0.49463647565026486`}, {-1.0269836663245298`, -0.5000861402691021}, \
{-1.0218107126953881`, -0.5055206738421636}, {-1.016606720476179, \
-0.5109398515263419}, {-1.0113717261006943`, -0.5163434475703442}, \
{-1.006105768058828, -0.52173123533238}, {-1.0008088869241052`, \
-0.5271029872985047}, {-0.995481125381186, -0.5324584751016302}, \
{-0.9901225282533268, -0.537797469541208}, {-0.9847331425297877, \
-0.5431197406035942}, {-0.9793130173931687, -0.5484250574831048}, \
{-0.9738622042466583, -0.5537131886037662}, {-0.9683807567411795, \
-0.5589839016417716}, {-0.9628687308024155, -0.5642369635486467}, \
{-0.9573261846576978, -0.5694721405751336}, {-0.9517531788627399, \
-0.5746891982957977}, {-0.9461497763281976, -0.5798879016343635}, \
{-0.9405160423460379, -0.585068014889786}, {-0.9348520446156975, \
-0.5902293017630597}, {-0.9291578532700112, -0.5953715253847732}, \
{-0.9234335409008914, -0.600494448343411}, {-0.9176791825847371, \
-0.605597832714406}, {-0.9118948559075536, -0.610681440089948}, \
{-0.9060806409897599, -0.6157450316095475}, {-0.900236620510665, \
-0.62078836799136}, {-0.8943628797325889, -0.625811209564269}, \
{-0.8884595065246086, -0.6308133163007321}, {-0.8825265913859052, \
-0.6357944478503859}, {-0.8765642274686898, -0.640754363574414}, \
{-0.8705725106006863, -0.6456928225806736}, {-0.8645515393071462, \
-0.6506095837595809}, {-0.8585014148323732, -0.6555044058207508}, \
{-0.8524222411607334, -0.6603770473303904}, {-0.8463141250371272, \
-0.6652272667494404}, {-0.8401771759868991, -0.6700548224724604}, \
{-0.8340115063351597, -0.6748594728672525}, {-0.8278172312254978, \
-0.6796409763152174}, {-0.8215944686380553, -0.6843990912524358}, \
{-0.8153433394069425, -0.6891335762114672}, {-0.809063967236967, \
-0.6938441898638571}, {-0.8027564787196533, -0.698530691063344}, \
{-0.7964210033485261, -0.7031928388897545}, {-0.7900576735336343, \
-0.707830392693577}, {-0.7836666246152898, -0.7124431121411997}, \
{-0.7772479948769961, -0.7170307572608021}, {-0.7708019255575435, \
-0.7215930884888837}, {-0.7643285608622442, -0.7261298667174169}, \
{-0.7578280479732853, -0.7306408533416073}, {-0.7513005370591733, \
-0.7351258103082453}, {-0.7447461812832479, -0.7395845001646297}, \
{-0.7381651368112405, -0.7440166861080472}, {-0.7315575628178551, \
-0.7484221320357852}, {-0.7249236214923467, -0.7528006025956605}, \
{-0.7182634780430774, -0.7571518632370392}, {-0.711577300701026, \
-0.7614756802623283}, {-0.70486526072223, -0.7657718208789127}, \
{-0.6981275323891394, -0.7700400532515165}, {-0.6913642930108617, \
-0.7742801465549598}, {-0.6845757229222771, -0.778491871027288}, \
{-0.6777620054820064, -0.7826749980232444}, {-0.6709233270692115, \
-0.7868293000680586}, {-0.6640598770792115, -0.7909545509115234}, \
{-0.6571718479178967, -0.7950505255823274}, {-0.6502594349949249, \
-0.7991170004426154}, {-0.6433228367156842, -0.803153753242745}, \
{-0.6363622544720079, -0.8071605631762051}, {-0.629377892631627, \
-0.8111372109346665}, {-0.62236995852635, -0.8150834787631276}, \
{-0.6153386624389555, -0.8189991505151241}, {-0.6082842175887889, \
-0.8228840117079649}, {-0.6012068401160536, -0.8267378495779608}, \
{-0.5941067490647872, -0.8305604531356076}, {-0.586984166364517, \
-0.834351613220688}, {-0.5798393168105873, -0.8381111225572552}, \
{-0.5726724280431549, -0.8418387758084593}, {-0.5654837305248485, \
-0.8455343696311788}, {-0.5582734575170899, -0.849197702730418}, \
{-0.5510418450550761, -0.852828575913432}, {-0.5437891319214225, \
-0.8564267921435397}, {-0.5365155596184688, -0.8599921565935843}, \
{-0.5292213723392506, -0.8635244766990027}, {-0.5219068169371421, \
-0.867023562210463}, {-0.5145721428941735, -0.8704892252460306}, \
{-0.5072176022880326, -0.8739212803428225}, {-0.499843449757758, \
-0.8773195445081096}, {-0.49244994246813467`, -0.8806838372698271}, \
{-0.4850373400728022, -0.8840139807264539}, {-0.47760590467609026`, \
-0.8873097995962193}, {-0.47015590079359454`, -0.8905711212656}, \
{-0.4626875953115094, -0.8937977758370671}, {-0.45520125744473416`, \
-0.8969895961760438}, {-0.4476971586937726, -0.9001464179570364}, \
{-0.44017557280044417`, -0.9032680797089009}, {-0.4326367757024303, \
-0.9063544228592066}, {-0.4250810454866771, -0.9094052917776623}, \
{-0.4175086623416792, -0.912420533818566}, {-0.40991990850867105`, \
-0.9153999993622463}, {-0.40231506823175117`, -0.9183435418554579}, \
{-0.39469442770696866`, -0.9212510178507002}, {-0.3870582750304009, \
-0.9241222870444237}, {-0.37940690014525313`, -0.9269572123140947}, \
{-0.37174059478801186`, -0.9297556597540859}, {-0.3640596524336849, \
-0.9325174987103643}, {-0.3563643682401625, -0.9352426018139461}, \
{-0.34865503899173417`, -0.9379308450130938}, {-0.340931963041798, \
-0.9405821076042257}, {-0.3331954402547991, -0.9431962722615165}, \
{-0.3254457719474355, -0.9457732250651619}, {-0.3176832608291706, \
-0.9483128555282869}, {-0.30990821094209126`, -0.9508150566224759}, \
{-0.30212092760015313`, -0.9532797248019046}, {-0.29432171732785334`, \
-0.9557067600260564}, {-0.28651088779837314`, -0.9580960657810051}, \
{-0.2786887477712326, -0.9604475490992503}, {-0.2708556070295004, \
-0.962761120578089}, {-0.2630117763166023, -0.9650366943965144}, \
{-0.2551575672727718, -0.9672741883306284}, {-0.24729329237118747`, \
-0.9694735237675595}, {-0.23941926485384124`, -0.971634625717879}, \
{-0.23153579866718213`, -0.9737574228265081}, {-0.2236432083975804, \
-0.9758418473821131}, {-0.2157418092066569, -0.9778878353249834}, \
{-0.2078319167665224, -0.9798953262533932}, {-0.19991384719497182`, \
-0.9818642634284461}, {-0.19198791699067774`, -0.9837945937774045}, \
{-0.18405444296842818`, -0.9856862678955063}, {-0.17611374219445225`, \
-0.9875392400462754}, {-0.16816613192187807`, -0.9893534681603312}, \
{-0.16021192952636634`, -0.9911289138327058}, {-0.15225145244196267`, \
-0.992865542318677}, {-0.14428501809721123`, -0.9945633225281294}, \
{-0.13631294385157186`, -0.996222227018456}, {-0.12833554693218224`, \
-0.9978422319860117}, {-0.12035314437100558`, -0.9994233172561376}, \
{-0.11236605294240427`, -1.0009654662717686`}, {-0.1043745891011786, \
-1.0024686660806452`}, {-0.09637906892110916, -1.0039329073211463`}, \
{-0.08837980803404058, -1.0053581842067651`}, {-0.08037712156954332, \
-1.006744494509248}, {-0.0723713240951894, -1.0080918395404197`}, \
{-0.06436272955747674, -1.0094002241327187`}, {-0.056351651223436175`, \
-1.0106696566184672`}, {-0.048338401622953636`, -1.0119001488079014`}, \
{-0.04032329249183933, -1.0130917159659887`}, {-0.032306634715674395`, \
-1.0142443767880602`}, {-0.024288738274464438`, -1.0153581533742864`}, \
{-0.016269912188128205`, -1.0164330712030243`}, {-0.00825046446284838, \
-1.0174691591030682`}, {-0.00023070203831038194`, -1.0184664492248334`}, {
      0.007789069264146265, -1.0194249770105035`}, {
      0.015808544792440645`, -1.0203447811631754`}, {
      0.02382742111370514, -1.0212259036150328`}, {
      0.03184539606326125, -1.0220683894945815`}, {
      0.039862168792597, -1.0228722870929796`}, {
      0.047877439816327666`, -1.0236376478294975`}, {
      0.05589091105812275, -1.0243645262161405`}, {
      0.06390228589558355, -1.025052979821469}, {
      0.0719112692040568, -1.0257030692336506`}, {
      0.07991756739937123, -1.0263148580227792`}, {
      0.08792088847948512, -1.0268884127024953`}, {
      0.09592094206503421, -1.0274238026909428`}, {
      0.10391743943877035`, -1.027921100271098}, {
      0.11191009358388304`, -1.0283803805505032`}, {
      0.11989861922119648`, -1.0288017214204437`}, {
      0.1278827328452366, -1.0291852035145987`}, {
      0.13586215275916347`, -1.0295309101672034`}, {
      0.1438365991085653, -1.0298389273707558`}, {
      0.15180579391411222`, -1.030109343733302}, {
      0.15976946110306814`, -1.0303422504353343`}, {
      0.1677273265396609, -1.030537741186336}, {
      0.17567911805431163`, -1.030695912181003}, {
      0.18362456547172498`, -1.0308168620551814`}, {
      0.1915634006378437, -1.0309006918415449`}, {
      0.1994953574456711, -1.0309475049250516`}, {
      0.20742017185996664`, -1.0309574069982064`}, {
      0.21533758194082014`, -1.0309305060161627`}, {
      0.22324732786611173`, -1.0308669121516914`}, {
      0.23114915195286484`, -1.0307667377500502`}, {
      0.2390427986775006, -1.0306300972837787`}, {
      0.24692801469500325`, -1.03045710730745}, {
      0.25480454885700593`, -1.0302478864124065`}, {
      0.26267215222880796`, -1.0300025551815069`}, {
      0.270530578105335, -1.0297212361439108`}, {
      0.2783795820260536, -1.0294040537299274`}, {
      0.2862189217888538, -1.0290511342259532`}, {
      0.2940483574629115, -1.0286626057295227`}, {
      0.3018676514005464, -1.0282385981044981`}, {
      0.30967656824808776`, -1.0277792429364172`}, {
      0.31747487495576465`, -1.0272846734880263`}, {
      0.3252623407866345, -1.026755024655017}, {
      0.33303873732456685`, -1.0261904329219882`}, {
      0.3408038384812973, -1.025591036318655}, {
      0.3485574205025693, -1.0249569743763207`}, {
      0.35629926197337985`, -1.0242883880846356`}, {
      0.3640291438223461, -1.0235854198486551`}, {
      0.37174684932521074`, -1.0228482134462187`}, {
      0.37945216410750404`, -1.0220769139856631`}, {
      0.38714487614637927`, -1.021271667863889}, {
      0.3948247757716408, -1.0204326227247915`}, {
      0.4024916556659824, -1.0195599274180742`}, {
      0.41014531086445405`, -1.0186537319584557`}, {
      0.4177855387531766, -1.0177141874852842`}, {
      0.4254121390673213, -1.0167414462225717`}, {
      0.43302491388837433`, -1.0157356614394595`}, {
      0.4406236676407041, -1.014696987411125}, {
      0.4482082070874502, -1.0136255793801416`}, {
      0.45577834132575273`, -1.0125215935182998`}, {
      0.4633338817813409, -1.0113851868888983`}, {
      0.47087464220249864`, -1.0102165174095155`}, {
      0.4784004386534271, -1.0090157438152656`}, {
      0.485911089507021, -1.0077830256225504`}, {
      0.4934064154370788, -1.0065185230933087`}, {
      0.5008862394099632, -1.0052223971997736`}, {
      0.5083503866757318, -1.0038948095897398`}, {
      0.5157986847587545, -1.002535922552347}, {
      0.5232309634478359, -1.0011458989843833`}, {
      0.5306470547858618, -0.9997249023571138}, {
      0.5380467930589841, -0.998273096683633}, {
      0.5454300147853643, -0.9967906464867503}, {
      0.5527965587034918, -0.9952777167674056}, {
      0.5601462657600925, -0.9937344729736184}, {
      0.5674789790976469, -0.9921610809699727}, {
      0.5747945440415313, -0.9905577070076371}, {
      0.5820928080868004, -0.9889245176949215}, {
      0.5893736208846261, -0.9872616799683697}, {
      0.596636834228408, -0.9855693610643883}, {
      0.603882302039572, -0.9838477284914111}, {
      0.6111098803530716, -0.9820969500025969}, {
      0.618319427302605, -0.9803171935690608}, {
      0.6255108031055665, -0.9785086273536363}, {
      0.632683870047742, -0.9766714196851664}, {
      0.6398384924677657, -0.9748057390333211}, {
      0.6469745367413509, -0.9729117539839395}, {
      0.6540918712653069, -0.970989633214892}, {
      0.6611903664413579, -0.9690395454724612}, {
      0.6682698946597736, -0.9670616595482369}, {
      0.6753303302828263, -0.9650561442565218}, {
      0.6823715496280857, -0.9630231684122441}, {
      0.6893934309515637, -0.9609629008093733}, {
      0.6963958544307208, -0.9588755101998334}, {
      0.703378702147345, -0.9567611652729112}, {
      0.7103418580703147, -0.9546200346351535}, {
      0.7172852080382566, -0.9524522867907487}, {
      0.7242086397421083, -0.9502580901223874}, {
      0.7311120427075964, -0.9480376128725988}, {
      0.7379953082776399, -0.9457910231255545}, {
      0.744858329594688, -0.9435184887893374}, {
      0.7517010015830023, -0.9412201775786678}, {
      0.7585232209308924, -0.9388962569980828}, {
      0.7653248860729128, -0.9365468943255619}, {
      0.7721058971720309, -0.9341722565965951}, {
      0.7788661561017732, -0.9317725105886842}, {
      0.7856055664283583, -0.9293478228062754}, {
      0.7923240333928242, -0.9268983594661142}, {
      0.7990214638931572, -0.9244242864830178}, {
      0.8056977664664302, -0.9219257694560582}, {
      0.8123528512709562, -0.9194029736551518}, {
      0.8189866300684652, -0.916856064008046}, {
      0.8255990162063089, -0.9142852050876997}, {
      0.8321899245997022, -0.9116905611000503}, {
      0.8387592717140044, -0.90907229587216}, {
      0.8453069755470486, -0.9064305728407368}, {
      0.8518329556115223, -0.9037655550410233}, {
      0.858337132917407, -0.9010774050960465}, {
      0.8648194299544797, -0.8983662852062234}, {
      0.8712797706748827, -0.8956323571393152}, {
      0.8777180804757657, -0.8928757822207244}, {
      0.8841342861820044, -0.8900967213241286}, {
      0.8905283160290006, -0.887295334862445}, {
      0.8969000996455674, -0.8844717827791186}, {
      0.9032495680369035, -0.8816262245397293}, {
      0.9095766535676602, -0.8787588191239104}, {
      0.9158812899451048, -0.8758697250175744}, {
      0.9221634122023837, -0.8729591002054374}, {
      0.9284229566818887, -0.8700271021638388}, {
      0.9346598610187279, -0.8670738878538488}, {
      0.940874064124307, -0.8640996137146584}, {
      0.9470655061700208, -0.8611044356572465}, {
      0.9532341285710595, -0.8580885090583167}, {
      0.9593798739703304, -0.8550519887545006}, {
      0.9655026862224994, -0.8519950290368201}, {
      0.9716025103781518, -0.8489177836454036}, {
      0.9776792926680775, -0.8458204057644507}, {
      0.9837329804876791, -0.8427030480174403}, {
      0.9897635223815072, -0.8395658624625761}, {
      0.9957708680279236, -0.8364090005884647}, {
      1.001754968223893, -0.8332326133100201}, {
      1.0077157748699073`, -0.8300368509645915}, {
      1.0136532409550385`, -0.8268218633083064}, {
      1.0195673205421283`, -0.8235877995126267}, {
      1.0254579687531098`, -0.820334808161111}, {
      1.0313251417544642`, -0.8170630372463787}, {
      1.037168796742814, -0.813772634167272}, {
      1.0429888919306531`, -0.8104637457262092}, {
      1.0487853865322128`, -0.8071365181267265}, {
      1.054558240749467, -0.8037910969712024}, {
      1.0603074157582746`, -0.800427627258761}, {
      1.0660328736946623`, -0.7970462533833487}, {
      1.0717345776412435`, -0.793647119131981}, {
      1.0774124916137797`, -0.7902303676831552}, {
      1.0830665805478785`, -0.7867961416054228}, {
      1.088696810285833, -0.7833445828561199}, {
      1.0943031475635996`, -0.7798758327802502}, {
      1.099885559997916, -0.7763900321095168}, {
      1.1054440160735581`, -0.7728873209614985}, {
      1.110978485130736, -0.7693678388389671}, {
      1.1164889373526294`, -0.7658317246293423}, {
      1.1219753437530617`, -0.7622791166042798}, {
      1.1274376761643121`, -0.7587101524193884}, {
      1.132875907225066, -0.7551249691140741}, {
      1.1382900103685034`, -0.7515237031115063}, {
      1.1436799598105234`, -0.747906490218703}, {
      1.1490457305381065`, -0.7442734656267322}, {
      1.154387298297812, -0.740624763911025}, {
      1.1597046395844117`, -0.7369605190317983}, {
      1.1649977316296578`, -0.7332808643345825}, {
      1.1702665523911853`, -0.7295859325508536}, {
      1.175511080541548, -0.7258758557987629}, {
      1.1807312954573872`, -0.7221507655839651}, {
      1.185927177208732, -0.7184107928005388}, {
      1.1910987065484322`, -0.7146560677319987}, {
      1.196245864901719, -0.7108867200523958}, {
      1.2013686343558978`, -0.7071028788275017}, {
      1.2064669976501694`, -0.7033046725160766}, {
      1.2115409381655773`, -0.6994922289712173}, {
      1.216590439915083, -0.695665675441781}, {
      1.2216154875337686`, -0.6918251385738858}, {
      1.2266160662691614`, -0.6879707444124818}, {
      1.2315921619716854`, -0.6841026184029932}, {
      1.2365437610852343`, -0.6802208853930275}, {
      1.2414708506378673`, -0.6763256696341494}, {
      1.2463734182326258`, -0.6724170947837189}, {
      1.2512514520384699`, -0.6684952839067887}, {
      1.256104940781334, -0.664560359478061}, {
      1.2609338737353013`, -0.6606124433839009}, {
      1.2657382407138935`, -0.6566516569244033}, {
      1.2705180320614782`, -0.6526781208155132}, {
      1.2752732386447896`, -0.6486919551911964}, {
      1.280003851844564, -0.6446932796056574}, {
      1.2847098635472871`, -0.640682213035606}, {
      1.289391266137054, -0.6366588738825673}, {
      1.2940480524875382`, -0.632623379975235}, {
      1.2986802159540711`, -0.6285758485718668}, {
      1.3032877503658296`, -0.6245163963627185}, {
      1.3078706500181296`, -0.6204451394725168}, {
      1.3124289096648285`, -0.6163621934629682}, {
      1.3169625245108287`, -0.6122676733353026}, {
      1.3214714902046885`, -0.608161693532851}, {
      1.3259558028313334`, -0.6040443679436537}, {
      1.3304154589048707`, -0.5999158099031004}, {
      1.3348504553615033`, -0.5957761321965983}, {
      1.3392607895525435`, -0.5916254470622679}, {
      1.3436464592375257`, -0.5874638661936655}, {
      1.348007462577415, -0.5832915007425302}, {
      1.3523437981279134`, -0.579108461321555}, {
      1.35665546483286, -0.5749148580071802}, {
      1.3609424620177257`, -0.5707108003424081}, {
      1.365204789383201, -0.5664963973396374}, {
      1.3694424469988742`, -0.5622717574835177}, {
      1.373655435297003, -0.5580369887338202}, {
      1.377843755066373, -0.5537921985283271}, {
      1.3820074074462458`, -0.5495374937857346}, {
      1.3861463939203964`, -0.5452729809085729}, {
      1.3902607163112335`, -0.5409987657861383}, {
      1.3943503767740089`, -0.5367149537974384}, {
      1.3984153777911086`, -0.5324216498141499}, {
      1.402455722166429, -0.5281189582035863}, {
      1.4064714130198352`, -0.5238069828316767}, {
      1.410462453781699, -0.5194858270659529}, {
      1.4144288481875198`, -0.5151555937785455}, {
      1.4183706002726229`, -0.510816385349187}, {
      1.422287714366937, -0.5064683036682229}, {
      1.426180195089849, -0.5021114501396272}, {
      1.4300480473451342`, -0.49774592568402437`}, {
      1.4338912763159641`, -0.4933718307417156}, {
      1.437709887459986, -0.4889892652757089}, {
      1.4415038865044778`, -0.4845983287747525}, {
      1.4452732794415748`, -0.4801991202563709}, {
      1.4490180725235682`, -0.4757917382699026}, {
      1.4527382722582745`, -0.47137628089953965`}, {
      1.4564338854044736`, -0.46695284576736745`}, {
      1.4601049189674173`, -0.46252153003640534`}, {
      1.4637513801944044`, -0.4580824304136461}, {
      1.467373276570424, -0.4536356431530953}, {
      1.4709706158138633`, -0.4491812640588086}, {
      1.4745434058722833`, -0.44471938848792764`}, {
      1.4780916549182552`, -0.4402501113537133}, {
      1.4816153713452647`, -0.4357735271285759}, {
      1.4851145637636753`, -0.43128972984710273`}, {
      1.4885892409967554`, -0.42679881310908097`}, {
      1.4920394120767662`, -0.4223008700825172}, {
      1.495465086241109, -0.4177959935066519}, {
      1.4988662729285331`, -0.41328427569496917`}, {
      1.5022429817754017`, -0.4087658085382009}, {
      1.505595222612015, -0.4042406835073255}, {
      1.5089230054589915`, -0.3997089916565605}, {
      1.5122263405237044`, -0.3951708236263484}, {
      1.5155052381967749`, -0.3906262696463365}, {
      1.5187597090486178`, -0.38607541953834895`}, {
      1.5219897638260442`, -0.3815183627193519}, {
      1.525195413448915, -0.376955188204411}, {
      1.5283766690068472`, -0.3723859846096408}, {
      1.531533541755974, -0.36781084015514603`}, {
      1.5346660431157522`, -0.36322984266795477`}, {
      1.5377741846658242`, -0.3586430795849423}, {
      1.5408579781429264`, -0.35405063795574665`}, {
      1.5439174354378478`, -0.3494526044456747}, {
      1.5469525685924372`, -0.344849065338599}, {
      1.5499633897966574`, -0.34024010653984493`}, {
      1.5529499113856864`, -0.3356258135790684}, {
      1.5559121458370653`, -0.3310062716131231}, {
      1.5588501057678914`, -0.32638156542891816`}, {
      1.5617638039320565`, -0.32175177944626493`}, {
      1.5646532532175292`, -0.317116997720714}, {
      1.5675184666436823`, -0.3124773039463809}, {
      1.570359457358661, -0.3078327814587614}, {
      1.5731762386367956`, -0.30318351323753595`}, {
      1.5759688238760556`, -0.2985295819093628}, {
      1.578737226595545, -0.29387106975066013`}, {
      1.581481460433038, -0.2892080586903769}, {
      1.5842015391425555`, -0.28454063031275234`}, {
      1.5868974765919808`, -0.27986886586006354`}, {
      1.589569286760715, -0.275192846235362}, {
      1.5922169837373703`, -0.2705126520051982}, {
      1.5948405817175009`, -0.2658283634023342}, {
      1.5974400950013723`, -0.2611400603284447}, {
      1.6000155379917673`, -0.2564478223568061}, {
      1.6025669251918273`, -0.251751728734973}, {
      1.6050942712029315`, -0.24705185838744353`}, {
      1.6075975907226088`, -0.24234828991831195`}, {
      1.6100768985424871`, -0.23764110161390892`}, {
      1.612532209546276, -0.23293037144543}, {
      1.614963538707782, -0.22821617707155148`}, {
      1.6173709010889599`, -0.22349859584103404`}, {
      1.6197543118379942`, -0.21877770479531403`}, {
      1.6221137861874158`, -0.21405358067108216`}, {
      1.6244493394522475`, -0.20932629990285004`}, {
      1.6267609870281843`, -0.2045959386255041}, {
      1.6290487443898027`, -0.19986257267684696`}, {
      1.6313126270888014`, -0.19512627760012644`}, {
      1.6335526507522717`, -0.19038712864655205`}, {
      1.6357688310809986`, -0.18564520077779884`}, {
      1.6379611838477905`, -0.18090056866849888`}, {
      1.6401297248958384`, -0.1761533067087201}, {
      1.642274470137103, -0.17140348900643262`}, {
      1.6443954355507309`, -0.16665118938996265`}, {
      1.6464926371814972`, -0.16189648141043375`}, {
      1.6485660911382773`, -0.15713943834419566`}, {
      1.650615813592544, -0.15238013319524063`}, {
      1.652641820776892, -0.14761863869760722`}, {
      1.654644128983589, -0.14285502731777178`}, {
      1.6566227545631511`, -0.13808937125702725`}, {
      1.658577713922946, -0.13332174245384984`}, {
      1.660509023525819, -0.12855221258625307`}, {
      1.662416699888746, -0.12378085307412959`}, {
      1.6643007595815087`, -0.11900773508158063`}, {
      1.666161219225396, -0.11423292951923317`}, {
      1.6679980954919276`, -0.10945650704654478`}, {
      1.6698114051016018`, -0.1046785380740963}, {
      1.6716011648226665`, -0.09989909276587233}, {
      1.6733673914699125`, -0.09511824104152954}, {
      1.6751101019034897`, -0.09033605257865285}, {
      1.6768293130277447`, -0.08555259681499962}, {
      1.6785250417900814`, -0.08076794295073175}, {
      1.6801973051798424`, -0.07598215995063588}, {
      1.681846120227212, -0.07119531654633157}, {
      1.683471504002139, -0.0664074812384677}, {
      1.685073473613283, -0.06161872229890697}, {
      1.6866520462069776`, -0.056829107772898735`}, {
      1.688207238966216, -0.05203870548124}, {
      1.6897390691096554`, -0.04724758302242484}, {
      1.691247553890642, -0.04245580777478221}, {
      1.6927327105962542`, -0.03766344689860214}, {
      1.6941945565463652`, -0.03287056733825054}, {
      1.6956331090927246`, -0.028077235824272538`}, {
      1.6970483856180585`, -0.023283518875484407`}, {
      1.6984404035351877`, -0.01848948280105426}, {
      1.6998091802861635`, -0.01369519370257146}, {
      1.7011547333414225`, -0.008900717476104875}, {
      1.702477080198958, -0.00410611981424999}, {1.7037762383835082`, 
      0.0006885337918349941}, {1.7050522254457634`, 0.005483178050404062}, {
      1.706305058961588, 0.0102777478671092}, {1.7075347565312602`, 
      0.015072178342997063`}, {1.7087413357787282`, 0.0198664047725164}, {
      1.7099248143508818`, 0.02466036264153621}, {1.711085209916841, 
      0.02945398762537455}, {1.7122225401672595`, 0.034247215586837874`}, {
      1.7133368228136445`, 0.0390399825742709}, {1.714428075587691, 
      0.04383222481961692}, {1.7154963162406325`, 0.048623878736488435`}, {
      1.716541562542605, 0.053414880918248094`}, {1.717563832282028, 
      0.05820516813609985}, {1.7185631432649975`, 0.06299467733719025}, {
      1.7195395133146958`, 0.06778334564271978}, {1.7204929602708139`, 
      0.07257111034606425}, {1.721423501988989, 0.07735790891090608}, {
      1.7223311563402546`, 0.08214367896937534}, {1.723215941210506, 
      0.08692835832020075}, {1.7240778744999776`, 0.09171188492687021}, {
      1.7249169741227355`, 0.09649419691580105}, {1.7257332580061808`, 
      0.10127523257451977`}, {1.7265267440905685`, 0.1060549303498513}, {
      1.7272974503285372`, 0.11083322884611764`}, {1.728045394684653, 
      0.11561006682334579`}, {1.728770595134965, 0.12038538319548502`}, {
      1.7294730696665739`, 0.1251591170286332}, {1.7301528362772123`, 
      0.12993120753927226`}, {1.7308099129748373`, 0.13470159409251278`}, {
      1.7314443177772358`, 0.13947021620034736`}, {1.7320560687116402`, 
      0.14423701351991292`}, {1.7326451838143568`, 0.14900192585176184`}, {
      1.7332116811304061`, 0.1537648931381418}, {1.733755578713173, 
      0.15852585546128417`}, {1.7342768946240703`, 0.1632847530417011}, {
      1.7347756469322118`, 0.16804152623649102`}, {1.7352518537140966`, 
      0.1727961155376526}, {1.7357055330533062`, 0.1775484615704069}, {
      1.7361367030402095`, 0.18229850509152812`}, {1.7365453817716803`, 
      0.18704618698768208`}, {1.7369315873508255`, 0.1917914482737732}, {
      1.7372953378867229`, 0.19653423009129933`}, {1.7376366514941695`, 
      0.20127447370671453`}, {1.7379555462934406`, 0.20601212050979986`}, {
      1.7382520404100588`, 0.21074711201204183`}, {1.738526151974572, 
      0.2154793898450187}, {1.7387778991223437`, 0.22020889575879438`}, {
      1.7390072999933501`, 0.22493557162031994`}, {1.7392143727319893`, 
      0.22965935941184262`}, {1.7393991354868992`, 0.23438020122932232`}, {
      1.7395616064107848`, 0.23909803928085535`}, {1.7397018036602552`, 
      0.24381281588510556`}, {1.7398197453956699`, 0.24852447346974274`}, {
      1.7399154497809943`, 0.253232954569888}, {1.7399889349836648`, 
      0.25793820182656624`}, {1.7400402191744613`, 0.2626401579851659}, {
      1.7400693205273916`, 0.26733876589390515`}, {1.7400762572195818`, 
      0.27203396850230555`}, {1.7400610474311775`, 0.27672570885967174`}, {
      1.7400237093452529`, 0.28141393011357846`}, {1.7399642611477286`, 
      0.28609857550836393`}, {1.7398827210272987`, 0.2907795883836299}, {
      1.7397791071753648`, 0.29545691217274794`}, {1.7396534377859805`, 
      0.3001304904013727}, {1.7395057310558029`, 0.3048002666859607}, {
      1.7393360051840532`, 0.3094661847322964}, {1.7391442783724849`, 
      0.31412818833402345`}, {1.7389305688253607`, 0.3187862213711828}, {
      1.738694894749437, 0.32344022780875664`}, {1.738437274353957, 
      0.3280901516952185}, {1.7381577258506518`, 0.3327359371610888}, {
      1.737856267453749, 0.33737752841749685`}, {1.7375329173799898`, 
      0.3420148697547484}, {1.7371876938486532`, 0.34664790554089864`}, {
      1.7368206150815884`, 0.3512765802203316}, {1.7364316993032551`, 
      0.35590083831234415`}, {1.7360209647407714`, 0.36052062440973637`}, {
      1.7355884296239685`, 0.365135883177407}, {1.735134112185455, 
      0.36974655935095413`}, {1.7346580306606865`, 0.3743525977352817}, {
      1.7341602032880434`, 0.3789539432032107}, {1.7336406483089173`, 
      0.383550540694096}, {1.7330993839678033`, 0.38814233521244806`}, {
      1.7325364285124005`, 0.3927292718265597}, {1.7319518001937197`, 
      0.39731129566713813`}, {1.731345517266198, 0.4018883519259415}, {
      1.730717597987821, 0.4064603858544207}, {1.7300680606202525`, 
      0.4110273427623656}, {1.7293969234289708`, 0.41558916801655643`}, {
      1.7287042046834122`, 0.4201458070394196}, {1.7279899226571227`, 
      0.4246972053076881}, {1.727254095627915, 0.4292433083510665}, {
      1.726496741878034, 0.4337840617509004}, {1.7257178796943284`, 
      0.43831941113885065`}, {1.724917527368431, 0.4428493021955713}, {
      1.7240957031969428`, 0.44737368064939237`}, {1.723252425481628, 
      0.4518924922750067}, {1.7223877125296123`, 0.45640568289216105`}, {
      1.7215015826535913`, 0.460913198364351}, {1.7205940541720433`, 
      0.4654149845975205}, {1.71966514540945, 0.4699109875387648}, {
      1.7187148746965246`, 0.47440115317503784`}, {1.7177432603704461`, 
      0.4788854275318631}, {1.7167503207751, 0.4833637566720486}, {
      1.715736074261327, 0.48783608669440565`}, {1.714700539187178, 
      0.49230236373247094`}, {1.7136437339181747`, 0.49676253395323283`}, {
      1.7125656768275797`, 0.5012165435558609}, {1.7114663862966704`, 
      0.5056643387704387}, {1.7103458807150218`, 0.5101058658567013}, {
      1.7092041784807954`, 0.5145410711027744}, {1.7080412980010344`, 
      0.5189699008239183}, {1.7068572576919663`, 0.5233923013612746}, {
      1.7056520759793117`, 0.5278082190806161}, {1.7044257712986006`, 
      0.5322176003711003}, {1.7031783620954952`, 0.5366203916440253}, {
      1.7019098668261188`, 0.5410165393315898}, {1.700620303957392, 
      0.5454059898856554}, {1.699309691967376, 0.5497886897765122}, {
      1.6979780493456222`, 0.5541645854916473}, {1.6966253945935286`, 
      0.558533623534516}, {1.6952517462247028`, 0.5628957504233162}, {
      1.6938571227653327`, 0.5672509126897654}, {1.692441542754563, 
      0.5715990568778803}, {1.6910050247448782`, 0.5759401295427592}, {
      1.6895475873024945`, 0.5802740772493672}, {1.688069249007756, 
      0.5846008465713238}, {1.6865700284555392`, 0.588920384089693}, {
      1.685049944255664, 0.593232636391776}, {1.6835090150333112`, 
      0.5975375500699063}, {1.6819472594294482`, 0.601835071720247}, {
      1.68036469610126, 0.6061251479415909}, {1.6787613437225877`, 
      0.6104077253341627}, {1.6771372209843745`, 0.6146827504984227}, {
      1.675492346595118, 0.6189501700338742}, {1.6738267392813295`, 
      0.6232099305378718}, {1.6721404177880013`, 0.627461978604432}, {
      1.6704334008790793`, 0.6317062608230464}, {1.668705707337944, 
      0.6359427237774966}, {1.6669573559678987`, 0.6401713140446711}, {
      1.6651883655926634`, 0.6443919781933836}, {1.6633987550568776`, 
      0.6486046627831942}, {1.6615885432266093`, 0.6528093143632318}, {
      1.6597577489898707`, 0.6570058794710182}, {1.657906391257143, 
      0.6611943046312942}, {1.6560344889619059`, 0.6653745363548479}, {
      1.654142061061177, 0.669546521137343}, {1.6522291265360567`, 
      0.6737102054581511}, {1.650295704392281, 0.6778655357791833}, {
      1.6483418136607824`, 0.682012458543725}, {1.6463674733982567`, 
      0.6861509201752718}, {1.6443727026877397`, 0.6902808670763656}, {
      1.6423575206391885`, 0.6944022456274344}, {1.6403219463900731`, 
      0.6985150021856315}, {1.6382659991059736`, 0.7026190830836769}, {
      1.636189697981186, 0.7067144346287004}, {1.6340930622393364`, 
      0.7108010031010848}, {1.6319761111340008`, 0.7148787347533115}, {
      1.6298388639493353`, 0.7189475758088066}, {1.6276813400007122`, 
      0.7230074724607887}, {1.6255035586353652`, 0.7270583708711168}, {
      1.6233055392330416`, 0.7311002171691411}, {1.621087301206663, 
      0.7351329574505526}, {1.6188488640029945`, 0.7391565377762359}, {
      1.616590247103322, 0.7431709041711212}, {1.6143114700241359`, 
      0.7471760026230384}, {1.6120125523178253`, 0.7511717790815717}, {
      1.6096935135733796`, 0.7551581794569153}, {1.6073543734170985`, 
      0.7591351496187295}, {1.6049951515133098`, 0.7631026353949982}, {
      1.602615867565097, 0.7670605825708869}, {1.6002165413150342`, 
      0.7710089368876013}, {1.5977971925459304`, 0.774947644041247}, {
      1.595357841081582, 0.77887664968169}, {1.5928985067875343`, 
      0.7827958994114171}, {1.5904192095718515`, 0.7867053387843982}, {
      1.587919969385896, 0.7906049133049479}, {1.585400806225117, 
      0.7944945684265886}, {1.5828617401298466`, 0.7983742495509141}, {
      1.5803027911861063`, 0.8022439020264531}, {1.577723979526423, 
      0.8061034711475343}, {1.5751253253306532`, 0.809952902153151}, {
      1.5725068488268164`, 0.8137921402258265}, {1.569868570291939, 
      0.8176211304904804}, {1.5672105100529077`, 0.8214398180132947}, {
      1.5645326884873307`, 0.8252481478005808}, {1.5618351260244108`, 
      0.8290460647976465}, {1.5591178431458266`, 0.8328335138876638}, {
      1.5563808603866243`, 0.8366104398905366}, {1.5536241983361192`, 
      0.8403767875617691}, {1.5508478776388075`, 0.8441325015913346}, {
      1.548051918995288, 0.8478775266025436}, {1.5452363431631932`, 
      0.8516118071509141}, {1.5424011709581325`, 0.8553352877230403}, {
      1.5395464232546436`, 0.8590479127354627}, {1.5366721209871563`, 
      0.8627496265335376}, {1.5337782851509656`, 0.866440373390308}, {
      1.530864936803216, 0.8701200975053734}, {1.5279320970638963`, 
      0.873788743003761}, {1.5249797871168458`, 0.8774462539347957}, {
      1.5220080282107704`, 0.8810925742709722}, {1.5190168416602712`, 
      0.8847276479068255}, {1.5160062488468833`, 0.8883514186578023}, {
      1.512976271220126, 0.8919638302591331}, {1.5099269302985647`, 
      0.8955648263647031}, {1.506858247670884, 0.8991543505459252}, {
      1.503770244996973, 0.9027323462906108}, {1.5006629440090213`, 
      0.906298757001843}, {1.4975363665126282`, 0.9098535259968483}, {
      1.4943905343879227`, 0.9133965965058696}, {1.4912254695906961`, 
      0.9169279116710383}, {1.4880411941535472`, 0.9204474145452476}, {
      1.4848377301870386`, 0.9239550480910254}, {1.481615099880867, 
      0.9274507551794071}, {1.4783733255050457`, 0.9309344785888093}, {
      1.4751124294110982`, 0.9344061610039028}, {1.4718324340332674`, 
      0.9378657450144868}, {1.4685333618897352`, 0.9413131731143622}, {
      1.4652152355838572`, 0.9447483877002061}, {1.4618780778054095`, 
      0.9481713310704455}, {1.4585219113318486`, 0.9515819454241323}, {
      1.4551467590295866`, 0.9549801728598178}, {1.451752643855278, 
      0.9583659553744275}, {1.448339588857121, 0.9617392348621363}, {
      1.4449076171761746`, 0.9650999531132444}, {1.4414567520476858`, 
      0.9684480518130526}, {1.437987016802435, 0.9717834725407387}, {
      1.4344984348680934`, 0.975106156768234}, {1.4309910297705968`, 
      0.9784160458590998}, {1.4274648251355317`, 0.9817130810674053}, {
      1.4239198446895385`, 0.9849972035366045}, {1.4203561122617285`, 
      0.9882683542984148}, {1.4167736517851168`, 0.9915264742716955}, {
      1.4131724872980695`, 0.9947715042613269}, {1.4095526429457677`, 
      0.9980033849570897}, {1.405914142981686, 1.0012220569325458`}, {
      1.4022570117690876`, 1.0044274606439187`}, {1.3985812737825354`, 
      1.007619536428975}, {1.3948869536094186`, 1.0107982245059068`}, {
      1.3911740759514966`, 1.0139634649722147`}, {1.3874426656264582`, 
      1.0171151978035915`}, {1.3836927475694996`, 1.0202533628528065`}, {
      1.379924346834916, 1.0233778998485914`}, {1.3761374885977142`, 
      1.0264887483945262`}, {1.3723321981552392`, 1.029585847967927}, {
      1.3685085009288198`, 1.032669137918734}, {1.364666422465432, 
      1.0357385574684015`}, {1.3608059884393788`, 1.038794045708788}, {
      1.3569272246539905`, 1.041835541601049}, {1.3530301570433403`, 
      1.0448629839745287`}, {1.3491148116739804`, 1.0478763115256557`}, {
      1.3451812147466964`, 1.0508754628168384`}, {1.3412293925982792`, 
      1.0538603762753627`}, {1.3372593717033179`, 1.0568309901922905`}, {
      1.3332711786760105`, 1.059787242721361}, {1.3292648402719944`, 
      1.0627290718778923`}, {1.3252403833901971`, 1.065656415537686}, {
      1.3211978350747058`, 1.0685692114359326`}, {1.3171372225166582`, 
      1.0714673971661204`}, {1.3130585730561524`, 1.0743509101789446`}, {
      1.3089619141841784`, 1.07721968778122}, {1.30484727354457, 
      1.0800736671347948`}, {1.3007146789359774`, 1.0829127852554685`}, {
      1.2965641583138612`, 1.0857369790119098`}, {1.292395739792508, 
      1.0885461851245792`}, {1.2882094516470681`, 1.0913403401646533`}, {
      1.2840053223156136`, 1.0941193805529517`}, {1.2797833804012204`, 
      1.096883242558867}, {1.2755436546740717`, 1.0996318622992987`}, {
      1.2712861740735846`, 1.102365175737588}, {1.2670109677105585`, 
      1.1050831186824588`}, {1.2627180648693486`, 1.1077856267869595`}, {
      1.258407495010061, 1.1104726355474095`}, {1.2540792877707725`, 
      1.1131440803023496`}, {1.2497334729697736`, 1.1157998962314954`}, {
      1.2453700806078367`, 1.1184400183546956`}, {1.2409891408705076`, 
      1.121064381530893}, {1.2365906841304224`, 1.123672920457091}, {
      1.2321747409496493`, 1.1262655696673245`}, {1.2277413420820555`, 
      1.1288422635316342`}, {1.2232905184757001`, 1.1314029362550466`}, {
      1.2188223012752517`, 1.1339475218765587`}, {1.2143367218244332`, 
      1.1364759542681275`}, {1.2098338116684928`, 1.1389881671336655`}, {
      1.205313602556701, 1.1414840940080402`}, {1.200776126444876, 
      1.143963668256081}, {1.1962214154979338`, 1.1464268230715904`}, {
      1.1916495020924687`, 1.1488734914763623`}, {1.1870604188193596`, 
      1.1513036063192057`}, {1.1824541984864059`, 1.153717100274975}, {
      1.17783087412099, 1.156113905843608}, {1.1731904789727703`, 
      1.1584939553491689`}, {1.168533046516403, 1.1608571809389}, {
      1.1638586104542918`, 1.1632035145822797`}, {1.1591672047193693`, 
      1.1655328880700893`}, {1.1544588634779065`, 1.1678452330134856`}, {
      1.1497336211323543`, 1.1701404808430838`}, {1.1449915123242147`, 
      1.1724185628080468`}, {1.1402325719369426`, 1.1746794099751847`}, {
      1.1354568350988798`, 1.1769229532280618`}, {1.1306643371862204`, 
      1.1791491232661135`}, {1.1258551138260073`, 1.1813578506037725`}, {
      1.1210292008991625`, 1.1835490655696046`}, {1.1161866345435483`, 
      1.185722698305454}, {1.111327451157063, 1.1878786787656002`}, {
      1.1064516874007688`, 1.1900169367159237`}, {1.101559380202054, 
      1.1921374017330846`}, {1.0966505667578288`, 1.1942400032037108`}, {
      1.091725284537756, 1.1963246703235988`}, {1.0867835712875156`, 
      1.1983913320969255`}, {1.0818254650321049`, 1.2004399173354734`}, {
      1.0768510040791743`, 1.2024703546578674`}, {1.071860227022399, 
      1.2044825724888257`}, {1.066853172744885, 1.206476499058423}, {
      1.0618298804226147`, 1.2084520624013673`}, {1.056790389527927, 
      1.2104091903562926`}, {1.0517347398330346`, 1.2123478105650645`}, {
      1.0466629714135816`, 1.214267850472101}, {1.0415751246522351`, 
      1.216169237323709}, {1.036471240242319, 1.2180518981674369`}, {
      1.0313513591914838`, 1.2199157598514425`}, {1.0262155228254164`, 
      1.2217607490238789`}, {1.0210637727915906`, 1.2235867921322965`}, {
      1.0158961510630553`, 1.2253938154230626`}, {1.0107126999422653`, 
      1.2271817449408002`}, {1.0055134620649504`, 1.2289505065278445`}, {
      1.0002984804040274`, 1.2307000258237184`}, {0.9950677982735532, 
      1.2324302282646282`}, {0.9898214593327188, 1.2341410390829792`}, {
      0.9845595075898869, 1.2358323833069118`}, {0.9792819874066707, 
      1.2375041857598592`}, {0.9739889435020572, 1.2391563710601268`}, {
      0.9686804209565727, 1.2407888636204931`}, {0.9633564652164925, 
      1.2424015876478356`}, {0.9580171220980958, 1.2439944671427776`}, {
      0.952662437791963, 1.2455674258993616`}, {0.9472924588673208, 
      1.2471203875047459`}, {0.9419072322764304, 1.248653275338927}, {
      0.9365068053590223, 1.250166012574489}, {0.9310912258467783, 
      1.2516585221763779`}, {0.9256605418678584, 1.2531307269017051`}, {
      0.9202148019514762, 1.2545825492995784`}, {0.9147540550325207, 
      1.256013911710961}, {0.9092783504562265, 1.2574247362685615`}, {
      0.9037877379828921, 1.2588149448967534`}, {0.8982822677926467, 
      1.2601844593115252`}, {0.8927619904902662, 1.2615332010204636`}, {
      0.8872269571100382, 1.262861091322768}, {0.8816772191206776, 
      1.2641680513092992`}, {0.8761128284302917, 1.265454001862662}, {
      0.8705338373913958, 1.2667188636573234`}, {0.8649402988059806, 
      1.2679625571597652`}, {0.8593322659306298, 1.2691850026286757`}, {
      0.8537097924816911, 1.2703861201151774`}, {0.8480729326404973, 
      1.2715658294630932`}, {0.8424217410586414, 1.2727240503092536`}, {
      0.8367562728633041, 1.2738607020838433`}, {0.8310765836626346, 
      1.2749757040107885`}, {0.8253827295511846, 1.2760689751081886`}, {
      0.8196747671153974, 1.2771404341887893`}, {0.8139527534391497, 
      1.2781899998605015`}, {0.8082167461093498, 1.2792175905269654`}, {
      0.8024668032215893, 1.2802231243881603`}, {0.7967029833858512, 
      1.2812065194410633`}, {0.7909253457322737, 1.2821676934803556`}, {
      0.7851339499169693, 1.28310656409918}, {0.7793288561279007, 
      1.2840230486899473`}, {0.7735101250908137, 1.284917064445198}, {
      0.7676778180752265, 1.2857885283585133`}, {0.7618319969004764, 
      1.2866373572254854`}, {0.7559727239418247, 1.2874634676447394`}, {
      0.7501000621366184, 1.2882667760190147`}, {0.7442140749905111, 
      1.2890471985563043`}, {0.7383148265837411, 1.2898046512710537`}, {
      0.7324023815774696, 1.2905390499854208`}, {0.7264768052201764, 
      1.2912503103305981`}, {0.7205381633541155, 1.291938347748199}, {
      0.7145865224218295, 1.292603077491709}, {0.7086219494727246, 
      1.2932444146280042`}, {0.7026445121697038, 1.2938622740389367`}, {
      0.696654278795861, 1.2944565704229904`}, {0.6906513182612348, 
      1.2950272182970082`}, {0.6846357001096229, 1.2955741319979912`}, {
      0.6786074945254564, 1.2960972256849725`}, {0.6725667723407344, 
      1.2965964133409666`}, {0.6665136050420197, 1.2970716087749965`}, {
      0.660448064777495, 1.2975227256241997`}, {0.6543702243640792, 
      1.2979496773560153`}, {0.6482801572946055, 1.2983523772704535`}, {
      0.642177937745059, 1.298730738502449}, {0.6360636405818759, 
      1.2990846740243007`}, {0.6299373413693033, 1.2994140966482004`}, {
      0.6237991163768196, 1.2997189190288485`}, {0.617649042586615, 
      1.2999990536661639`}, {0.6114871977011338, 1.3002544129080855`}, {
      0.6053136601506756, 1.3004849089534711`}, {0.5991285091010576, 
      1.300690453855091}, {0.5929318244613374, 1.3008709595227228`}, {
      0.5867236868915944, 1.3010263377263473`}, {0.5805041778107717, 
      1.3011565000994472`}, {0.5742733794045776, 1.3012613581424113`}, {
      0.5680313746334456, 1.3013408232260473`}, {0.5617782472405533, 
      1.3013948065952028`}, {0.5555140817599, 1.3014232193724997`}, {
      0.5492389635244422, 1.3014259725621815`}, {0.5429529786742863, 
      1.3014029770540778`}, {0.5366562141649387, 1.3013541436276872`}, {
      0.530348757775613, 1.301279382956381}, {0.5240306981175922, 
      1.30117860561173}, {0.5177021246426472, 1.3010517220679587`}, {
      0.5113631276515098, 1.3008986427065246`}, {0.5050137983023999, 
      1.3007192778208307`}, {0.49865422861960595`, 1.30051353762107}, {
      0.49228451150211844`, 1.3002813322392048`}, {0.4859047407323149, 
      1.300022571734086}, {0.4795150109846959, 1.2997371660967114`}, {
      0.47311541783467137`, 1.2994250252556272`}, {0.46670605776739554`, 
      1.2990860590824769`}, {0.46028702818665035`, 1.298720177397697}, {
      0.4538584274237754, 1.2983272899763656`}, {0.44742035474664416`, 
      1.2979073065542035`}, {0.44097291036868436`, 1.2974601368337333`}, {
      0.43451619545794207`, 1.2969856904905972`}, {0.4280503121461875, 
      1.2964838771800378`}, {0.4215753635380615, 1.2959546065435437`}, {
      0.41509145372026146`, 1.2953977882156635`}, {0.4085986877707642, 
      1.294813331830991}, {0.40209717176808557`, 1.2942011470313233`}, {
      0.3955870128005741, 1.293561143472998}, {0.38906831897573707`, 
      1.2928932308344083`}, {0.38254119942959747`, 1.2921973188237015`}, {
      0.37600576433607985`, 1.2914733171866648`}, {0.36946212491642316`, 
      1.2907211357147983`}, {0.3629103934486179, 1.2899406842535819`}, {
      0.3563506832768666, 1.2891318727109358`}, {0.3497831088210638, 
      1.2882946110658813`}, {0.343207785586295, 1.2874288093774013`}, {
      0.3366248301723507, 1.2865343777935074`}, {0.3300343602832535, 
      1.2856112265605129`}, {0.32343649473679614`, 1.2846592660325187`}, {
      0.31683135347408664`, 1.2836784066811129`}, {0.31021905756909907`, 
      1.2826685591052878`}, {0.303599729238226, 1.2816296340415778`}, {
      0.2969734918498298, 1.2805615423744219`}, {0.2903404699337904, 
      1.2794641951467525`}, {0.2837007891910451, 1.278337503570816}, {
      0.2770545765031177, 1.2771813790392268`}, {0.270401959941634, 
      1.275995733136258}, {0.2637430687778189, 1.2747804776493725`}, {
      0.2570780334919721, 1.2735355245809994`}, {0.250406985782919, 
      1.2722607861605548`}, {0.2437300585774314, 1.2709561748567146`}, {
      0.23704738603961586`, 1.2696216033899401`}, {0.23035910358026368`, 
      1.26825698474526}, {0.2236653478661593, 1.266862232185313}, {
      0.21696625682934234`, 1.265437259263652}, {0.21026196967631847`, 
      1.2639819798383163`}, {0.20355262689721457`, 1.2624963080856717`}, {
      0.1968383702748733, 1.2609801585145237`}, {0.19011934289388185`, 
      1.2594334459805054`}, {0.18339568914952983`, 1.2578560857007455`}, {
      0.17666755475669094`, 1.2562479932688153`}, {0.16993508675862298`, 
      1.2546090846699631`}, {0.16319843353568048`, 1.2529392762966332`}, {
      0.1564577448139341, 1.251238484964277}, {0.1497131716736912, 
      1.2495066279274565`}, {0.14296486655791094`, 1.2477436228962422`}, {
      0.13621298328050815`, 1.2459493880529116`}, {0.12945767703453923`, 
      1.2441238420689467`}, {0.12269910440026353`, 1.242266904122336}, {
      0.11593742335307376`, 1.2403784939151816`}, {0.10917279327128794`, 
      1.2384585316916148`}, {0.1024053749437964, 1.2365069382560219`}, {
      0.09563533057755613, 1.2345236349915827`}, {0.08886282380492534, 
      1.232508543879123}, {0.08208801969083042, 1.2304615875162843`}, {
      0.07531108473975767, 1.2283826891370109`}, {0.06853218690256185, 
      1.2262717726313574`}, {0.061751495583083335`, 1.2241287625656176`}, {
      0.054969181644565636`, 1.2219535842027758`}, {0.04818541741586491, 
      1.219746163523282}, {0.04140037669744268, 1.2175064272461535`}, {
      0.03461423476713303, 1.2152343028504013`}, {0.027827168385675276`, 
      1.2129297185967833`}, {0.021039355802002967`, 1.210592603549886}, {
      0.014250976758279868`, 1.2082228876005334`}, {0.007462212494673436, 
      1.205820501488524}, {0.0006732457538561028, 
      1.2033853768256952`}, {-0.006115739214775447, 
      1.2009174461193177`}, {-0.01290455665117302, 
      1.198416642795815}, {-0.01969301928101185, 
      1.1958829012248107`}, {-0.02648093831237805, 
      1.1933161567435029`}, {-0.03326812343277907, 
      1.1907163456813614`}, {-0.040054382806487814`, 
      1.1880834053851508`}, {-0.04683952307223113, 
      1.185417274244273}, {-0.053623349341233806`, 
      1.182717891716432}, {-0.060405665195628906`, 
      1.179985198353616}, {-0.06718627268724589, 
      1.1772191358283937`}, {-0.07396497233678777, 
      1.1744196469605248`}, {-0.08074156313340876, 
      1.1715866757438802`}, {-0.08751584253470417, 
      1.1687201673736671`}, {-0.09428760646712411, 
      1.1658200682739588`}, {-0.10105664932682304`, 
      1.1628863261255205`}, {-0.10782276398095682`, 
      1.1599188898939319`}, {-0.11458574176943967`, 
      1.156917709857998}, {-0.12134537250717285`, 
      1.1538827376384442`}, {-0.1281014444867575, 
      1.150813926226891}, {-0.13485374448170376`, 
      1.1477112300151016`}, {-0.14160205775014853`, 
      1.1445746048244962`}, {-0.1483461680390945, 
      1.1414040079359271`}, {-0.15508585758918256`, 
      1.1381993981197065`}, {-0.16182090714001013`, 
      1.1349607356658804`}, {-0.16855109593600823`, 
      1.1316879824147408`}, {-0.17527620173288921`, 
      1.1283811017875662`}, {-0.18199600080467826`, 
      1.125040058817585}, {-0.18871026795134052`, 
      1.1216648201811488`}, {-0.19541877650701686`, 
      1.1182553542291078`}, {-0.20212129834888026`, 
      1.1148116310183793`}, {-0.20881760390662535`, 
      1.1113336223436965`}, {-0.21550746217260322`, 
      1.107821301769527}, {-0.22219064071261385`, 
      1.104274644662151}, {-0.228866905677368, 
      1.1006936282218849`}, {-0.23553602181463074`, 
      1.0970782315154393`}, {-0.24219775248205802`, 
      1.093428435508398}, {-0.24885185966073847`, 
      1.089744223097804}, {-0.25549810396945116`, 
      1.086025579144838}, {-0.26213624467965135`, 
      1.0822724905075767`}, {-0.26876603973119445`, 
      1.0784849460738128`}, {-0.2753872457488097, 
      1.074662936793925}, {-0.2819996180593336, 
      1.070806455713777}, {-0.2886029107097137, 
      1.0669154980076327`}, {-0.2951968764857922, 
      1.0629900610110687`}, {-0.3017812669318794, 
      1.0590301442538672`}, {-0.30835583237112646`, 
      1.0550357494928688`}, {-0.31492032192670566`, 
      1.05100688074477}, {-0.32147448354380725`, 
      1.046943544318842}, {-0.3280180640124608, 
      1.0428457488495557`}, {-0.33455080899118883`, 
      1.0387135053290872`}, {-0.3410724630314996, 
      1.0345468271396874`}, {-0.3475827696032259, 
      1.0303457300858918`}, {-0.35408147112071675`, 
      1.0261102324265505`}, {-0.3605683089698859, 
      1.0218403549066537`}, {-0.36704302353612434`, 
      1.0175361207889326`}, {-0.37350535423307973`, 
      1.0131975558852102`}, {-0.3799550395323075, 
      1.0088246885874805`}, {-0.3863918169937965, 
      1.00441754989869}, {-0.3928154232973716, 
      0.9999761734631972}, {-0.3992255942749762, 
      0.9955005955968877}, {-0.4056220649438343, 
      0.9909908553169162}, {-0.4120045695404937, 
      0.9864469943710514}, {-0.41837284155574994`, 
      0.9818690572665985}, {-0.4247266137704495, 
      0.9772570912988711}, {-0.43106561829217055`, 
      0.9726111465791878}, {-0.4373895865927785, 
      0.9679312760623654}, {-0.44369824954685333`, 
      0.9632175355736815}, {-0.4499913374709834, 
      0.9584699838352803}, {-0.4562685801639211, 
      0.9536886824919923}, {-0.46252970694759366`, 
      0.9488736961365427}, {-0.4687744467089629, 
      0.9440250923341171}, {-0.4750025279427243, 
      0.93914294164626}, {-0.4812136787948378, 
      0.9342273176540761}, {-0.4874076271068795, 
      0.9292782969807065}, {-0.4935841004612037, 
      0.9242959593130531}, {-0.4997428262269028, 
      0.9192803874227204}, {-0.5058835316065523, 
      0.91423166718615}, {-0.512005943683727, 
      0.9091498876039171}, {-0.5181097894712727, 
      0.9040351408191637}, {-0.5241947959603187, 
      0.8988875221351388}, {-0.5302606901700113, 
      0.8937071300318198}, {-0.5363071991979532, 
      0.8884940661815873}, {-0.5423340502713269, 
      0.8832484354639274}, {-0.5483409707986835, 
      0.8779703459791335}, {-0.5543276884223745, 
      0.872659909060984}, {-0.5602939310716042, 
      0.8673172392883683}, {-0.5662394270160804, 
      0.8619424544958371}, {-0.5721639049202369, 
      0.856535675783052}, {-0.5780670938980037, 
      0.8510970275231118}, {-0.5839487235680977, 
      0.84562663736973}, {-0.5898085241098068, 
      0.8401246362632425}, {-0.5956462263192381, 
      0.834591158435424}, {-0.6014615616660017, 
      0.8290263414130896}, {-0.6072542623502977, 
      0.8234303260204645}, {-0.6130240613603775, 
      0.8178032563802994}, {-0.6187706925303448, 
      0.8121452799137152}, {-0.6244938905982649, 
      0.806456547338758}, {-0.6301933912645464, 
      0.8007372126676482}, {-0.6358689312505623, 
      0.7949874332027081}, {-0.6415202483574729, 
      0.7892073695309539}, {-0.647147081525216, 
      0.7833971855173372}, {-0.6527491708916262, 
      0.7775570482966255}, {-0.6583262578516448, 
      0.7716871282639095}, {-0.6638780851165837, 
      0.765787599063727}, {-0.6694043967734032, 
      0.7598586375777955}, {-0.674904938343963, 
      0.753900423911346}, {-0.6803794568442092, 
      0.7479131413780516}, {-0.6858277008432538, 
      0.741896976483546}, {-0.6912494205223083, 
      0.7358521189075291}, {-0.6966443677334276, 
      0.7297787614844573}, {-0.7020122960580256, 
      0.7236771001828181}, {-0.7073529608651177, 
      0.7175473340829899}, {-0.7126661193692507, 
      0.7113896653536893}, {-0.7179515306880773, 
      0.7052042992270093}, {-0.7232089558995314, 
      0.6989914439720543}, {-0.7284381580985656, 
      0.6927513108671781}, {-0.7336389024534051, 
      0.6864841141708342}, {-0.7388109562612788, 
      0.6801900710910458}, {-0.7439540890035838, 
      0.67386940175351}, {-0.7490680724004444, 
      0.6675223291683459}, {-0.7541526804646206, 
      0.661149079195503}, {-0.7592076895547297, 
      0.6547498805088446}, {-0.7642328784277369, 
      0.6483249645589231}, {-0.7692280282906769, 
      0.6418745655344669}, {-0.7741929228515669, 
      0.6353989203225984}, {-0.7791273483694717, 
      0.6288982684678047}, {-0.7840310937036834, 
      0.6223728521296837}, {-0.7889039503619785, 
      0.6158229160394905}, {-0.7937457125479149, 
      0.6092487074555089}, {-0.7985561772071348, 
      0.6026504761172763}, {-0.8033351440726361, 
      0.5960284741986892}, {-0.8080824157089819, 
      0.5893829562600187}, {-0.8127977975554125, 
      0.5827141791988679}, {-0.8174810979678289, 
      0.5760224022001023}, {-0.8221321282596182, 
      0.5693078866847858}, {-0.8267507027412905, 
      0.5625708962581581}, {-0.8313366387588982, 
      0.5558116966566865}, {-0.8358897567312132, 
      0.5490305556942302}, {-0.8404098801856338, 
      0.5422277432073531}, {-0.8448968357927976, 
      0.5354035309998231}, {-0.8493504533998784, 
      0.5285581927863372}, {-0.8537705660625438, 
      0.5216920041355119}, {-0.8581570100755547, 
      0.5148052424121785}, {-0.8625096250019869, 
      0.5078981867190252}, {-0.8668282537010579, 
      0.5009711178376266}, {-0.8711127423545427, 
      0.49402431816890363`}, {-0.8753629404917654, 
      0.4870580716730551}, {-0.8795787010131523, 
      0.48007266380900526`}, {-0.8837598802123354, 
      0.47306838147340946`}, {-0.887906337796799, 
      0.46604551293926205`}, {-0.8920179369070568, 
      0.45900434779414984`}, {-0.8960945441343575, 
      0.4519451768781955}, {-0.9001360295369106, 
      0.4448682922217343}, {-0.9041422666546305, 
      0.43777398698276837`}, {-0.9081131325223959, 
      0.4306625553842427}, {-0.9120485076818258, 
      0.4235342926511862}, {-0.9159482761915707, 
      0.4163894949477617}, {-0.9198123256361239, 
      0.40922845931426816`}, {-0.9236405471331557, 
      0.4020514836041388}, {-0.9274328353393764, 
      0.3948588664209766}, {-0.9311890884549352, 
      0.38765090705567135`}, {-0.9349092082263637, 
      0.380427905423638}, {-0.9385930999480727, 
      0.37319016200221994`}, {-0.9422406724624155, 
      0.365937977768296}, {-0.9458518381583282, 
      0.3586716541361327}, {-0.949426512968562, 
      0.35139149289552063`}, {-0.9529646163655227, 
      0.34409779615023384`}, {-0.956466071355733, 
      0.3367908662568504}, {-0.9599308044729359, 
      0.32947100576397126`}, {-0.9633587457698569, 
      0.3221385173518735}, {-0.9667498288086463, 
      0.314793703772634}, {-0.9701039906500212, 
      0.3074368677907577}, {-0.9734211718411286, 
      0.3000683121243439}, {-0.976701316402155, 
      0.2926883393868239}, {-0.9799443718117024, 
      0.2852972520293004}, {-0.9831502889909595, 
      0.2778953522835205}, {-0.98631902228669, 
      0.270482942105511}, {-0.9894505294530662, 
      0.2630603231199042}, {-0.9925447716323744, 
      0.25562779656498236`}, {-0.9956017133346198, 
      0.24818566323846594`}, {-0.9986213224160586, 
      0.24073422344407133`}, {-1.0016035700566872`, 
      0.23327377693886156`}, {-1.0045484307367167`, 
      0.22580462288141281`}, {-1.0074558822120636`, 
      0.21832705978081823`}, {-1.0103259054888856`, 
      0.21084138544654926`}, {-1.0131584847971944`, 
      0.2033478969391937}, {-1.015953607563577, 
      0.19584689052208856`}, {-1.0187112643830543`, 
      0.1883386616138643}, {-1.0214314489901095`, 
      0.18082350474191625`}, {-1.0241141582289213`, 
      0.17330171349681733`}, {-1.0267593920228275`, 
      0.1657735804876859}, {-1.029367153343056, 
      0.1582393972985201}, {-1.0319374481767525`, 
      0.1506994544455104}, {-1.0344702854943388`, 
      0.14315404133533954`}, {-1.0369656772162312`, 
      0.1356034462244791}, {-1.0394236381789526`, 
      0.12804795617948986`}, {-1.0418441861006695`, 
      0.1204878570383329}, {-1.0442273415461865`, 
      0.11292343337269645`}, {-1.0465731278914265`, 
      0.10535496845134333`}, {-1.0488815712874324`, 
      0.09778274420448205}, {-1.0511527006239167`, 
      0.090207041189164}, {-1.0533865474923934`, 
      0.08262813855570837}, {-1.0555831461489191`, 
      0.0750463140151548}, {-1.0577425334764765`, 
      0.06746184380774367}, {-1.0598647489470268`, 
      0.059875002672422475`}, {-1.0619498345832623`, 
      0.052286063817375876`}, {-1.0639978349200876`, 
      0.04469529889157653}, {-1.0660087969658563`, 
      0.03710297795735256}, {-1.0679827701633924`, 
      0.029509369463967032`}, {-1.0699198063508237`, 
      0.021914740222203802`}, {-1.0718199597222529`, 
      0.014319355379953578`}, {-1.073683286788293, 
      0.006723478398793152}, {-1.0755098463364945`, -0.0008726289684498146}, \
{-1.0772996993916855`, -0.008468706699154773}, {-1.0790529091762542`, \
-0.016064496521397957`}, {-1.0807695410703941`, -0.023659741933483254`}, \
{-1.0824496625723365`, -0.03125418822252438}, {-1.0840933432585929`, \
-0.038847582482088944`}, {-1.0857006547442298`, -0.046439673628915545`}, \
{-1.0872716706431977`, -0.05403021241871549}, {-1.0888064665287338`, \
-0.06161895146107119}, {-1.0903051198938594`, -0.0692056452334437}, \
{-1.0917677101119927`, -0.0767900500943022}, {-1.0931943183976942`, \
-0.08437192429538876}, {-1.0945850277675644`, -0.09195102799313178}, \
{-1.0959399230013096`, -0.09952712325922218}, {-1.0972590906029966`, \
-0.10709997409036631`}, {-1.0985426187625078`, -0.11466934641723006`}, \
{-1.0997905973172164`, -0.12223500811258887`}, {-1.101003117713895, \
-0.12979672899869843`}, {-1.1021802729708732`, -0.13735428085390108`}, \
{-1.1033221576404564`, -0.1449074374184834}, {-1.1044288677716212`, \
-0.15245597439979974`}, {-1.105500500872998, -0.15999966947667804`}, \
{-1.1065371558761552`, -0.16753830230312258`}, {-1.1075389330991947`, \
-0.17507165451132992`}, {-1.1085059342106693`, -0.18259950971403366`}, \
{-1.1094382621938346`, -0.1901216535061936}, {-1.1103360213112428`, \
-0.19763787346604503`}, {-1.1111993170696888`, -0.20514795915552428`}, \
{-1.1120282561855168`, -0.21265170212008588`}, {-1.1128229465502961`, \
-0.2201488958879274}, {-1.1135834971968732`, -0.2276393359686373}, \
{-1.1143100182658068`, -0.2351228198512819}, {-1.1150026209721933`, \
-0.24259914700194649`}, {-1.1156614175728887`, -0.2500681188607466}, \
{-1.116286521334132, -0.2575295388383243}, {-1.1168780464995747`, \
-0.2649832123118453}, {-1.117436108258723, -0.2724289466205114}, \
{-1.117960822715795, -0.27986655106060393`}, {-1.1184523068589982`, \
-0.28729583688007254`}, {-1.1189106785302285`, -0.29471661727268444`}, \
{-1.1193360563951966`, -0.3021287073717483}, {-1.1197285599139821`, \
-0.30953192424342746`}, {-1.1200883093120186`, -0.3169260868796567}, \
{-1.1204154255515106`, -0.32431101619067604`}, {-1.120710030303285, \
-0.3316865349971963}, {-1.120972245919078, -0.33905246802220934`}, \
{-1.1212021954042566`, -0.3464086418824566}, {-1.1214000023909771`, \
-0.3537548850795694}, {-1.121565791111781, -0.36109102799089365`}, \
{-1.1216996863736242`, -0.3684169028600119}, {-1.1218018135323464`, \
-0.3757323437869757}, {-1.121872298467572, -0.3830371867182598}, \
{-1.1219112675580458`, -0.3903312694364513}, {-1.121918847657404, \
-0.39761443154968457`}, {-1.121895166070373, -0.40488651448083457`}, \
{-1.1218403505294017`, -0.4121473614564796}, {-1.121754529171718, \
-0.41939681749564456`}, {-1.1216378305168138`, -0.42663472939833563`}, \
{-1.1214903834443521`, -0.4338609457338778}, {-1.121312317172495, \
-0.44107531682906476`}, {-1.1211037612366497`, -0.4482776947561321}, \
{-1.1208648454686303`, -0.4554679333205637}, {-1.1205956999762312`, \
-0.4626458880487407}, {-1.1202964551232097`, -0.46981141617544336`}, \
{-1.1199672415096746`, -0.47696437663121444`}, {-1.119608189952876, \
-0.48410463002959336`}, {-1.119219431468395, -0.4912320386542305}, \
{-1.118801097251727, -0.4983464664458894}, {-1.1183533186602574`, \
-0.5054477789893461}, {-1.1178762271956229`, -0.5125358435001932}, \
{-1.1173699544864568`, -0.5196105288115568}, {-1.1168346322715133`, \
-0.5266717053607344}, {-1.1162703923831658`, -0.5337192451757605}, \
{-1.1156773667312765`, -0.5407530218619081}, {-1.1150556872874318`, \
-0.5477729105881324}, {-1.1144054860695392`, -0.5547787880734639}, \
{-1.1137268951267814`, -0.5617705325733582}, {-1.1130200465249238`, \
-0.5687480238660072}, {-1.1122850723319695`, -0.5757111432386204}, \
{-1.111522104604158, -0.5826597734736801}, {-1.110731275372303, \
-0.5895937988351782}, {-1.1099127166284648`, -0.5965131050548381}, \
{-1.109066560312951, -0.6034175793183306}, {-1.108192938301644, \
-0.6103071102514845}, {-1.1072919823936465`, -0.6171815879065019}, \
{-1.1063638242992446`, -0.624040903748179}, {-1.1054085956281798`, \
-0.63088495064014}, {-1.104426427878228, -0.6377136228310876}, \
{-1.1034174524240794`, -0.6445268159410736}, {-1.1023818005065142`, \
-0.6513244269477964}, {-1.1013196032218715`, -0.6581063541729264}, \
{-1.1002309915118034`, -0.6648724972684661}, {-1.0991160961533133`, \
-0.6716227572031462}, {-1.0979750477490697`, -0.6783570362488638}, \
{-1.096807976717995, -0.6850752379671635}, {-1.0956150132861209`, \
-0.6917772671957675}, {-1.0943962874777085`, -0.6984630300351556}, \
{-1.0931519291066267`, -0.7051324338351999}, {-1.0918820677679861`, \
-0.7117853871818552}, {-1.0905868328300214`, -0.7184217998839112}, \
{-1.0892663534262208`, -0.7250415829598044}, {-1.0879207584476949`, \
-0.7316446486244972}, {-1.0865501765357832`, -0.7382309102764231}, \
{-1.0851547360748923`, -0.744800282484502}, {-1.0837345651855617`, \
-0.7513526809752273}, {-1.0822897917177532`, -0.7578880226198271}, \
{-1.0808205432443598`, -0.7644062254215009}, {-1.079326947054929, \
-0.770907208502735}, {-1.077809130149597, -0.7773908920926965}, \
{-1.0762672192332297`, -0.7838571975147097}, {-1.0747013407097663`, \
-0.7903060471738141}, {-1.073111620676761, -0.796737364544408}, \
{-1.0714981849201193`, -0.8031510741579769}, {-1.0698611589090243`, \
-0.8095471015909098}, {-1.0682006677910514`, -0.8159253734524031}, \
{-1.0665168363874633`, -0.8222858173724542}, {-1.0648097891886856`, \
-0.8286283619899458}, {-1.063079650349957, -0.8349529369408215}, \
{-1.0613265436871517`, -0.8412594728463535}, {-1.0595505926727677`, \
-0.8475479013015043}, {-1.0577519204320822`, -0.8538181548633815}, \
{-1.0559306497394652`, -0.8600701670397883}, {-1.0540869030148514`, \
-0.8663038722778682}, {-1.0522208023203665`, -0.8725192059528465}, \
{-1.050332469357102, -0.8787161043568676}, {-1.0484220254620382`, \
-0.8848945046879301}, {-1.0464895916051122`, -0.8910543450389176}, \
{-1.0445352883864234`, -0.8971955643867288}, {-1.0425592360335785`, \
-0.9033181025815042}, {-1.0405615543991702`, -0.9094219003359519}, \
{-1.038542362958387, -0.9155068992147706}, {-1.0365017808067505`, \
-0.921573041624172}, {-1.0344399266579787`, -0.9276202708015012}, \
{-1.0323569188419703`, -0.9336485308049562}, {-1.0302528753029099`, \
-0.9396577665034054}, {-1.0281279135974875`, -0.9456479235663037}, \
{-1.0259821508932336`, -0.9516189484537076}, {-1.0238157039669642`, \
-0.9575707884063885}, {-1.021628689203334, -0.9635033914360438}, \
{-1.0194212225934955`, -0.9694167063156062}, {-1.0171934197338617`, \
-0.9753106825696515}, {-1.0149453958249672`, -0.9811852704649031}, \
{-1.0126772656704301`, -0.9870404210008344}, {-1.010389143676007, \
-0.992876085900368}, {-1.0080811438487431`, -0.9986922176006716}, \
{-1.0057533797962108`, -1.0044887692440503`}, {-1.003405964725839, \
-1.010265694668935}, {-1.0010390114443277`, -1.0160229484009662`}, \
{-0.9986526323571459, -1.0217604856441727`}, {-0.9962469394681118, \
-1.0274782622722454`}, {-0.9938220443790539, -1.0331762348199054`}, \
{-0.9913780582895468, -1.0388543604743654`}, {-0.9889150919967259, \
-1.0445125970668845`}, {-0.9864332558951723, -1.0501509030644156`}, \
{-0.9839326599768721, -1.0557692375613457`}, {-0.981413413831243, \
-1.0613675602713268`}, {-0.9788756266452305, -1.0669458315191986`}, \
{-0.9763194072034682, -1.072504012233001}, {-0.9737448638885035, \
-1.078042063936077}, {-0.9711521046810846, -1.0835599487392649`}, \
{-0.9685412371605089, -1.0890576293331786`}, {-0.9659123685050294, \
-1.094535068980577}, {-0.9632656054923187, -1.0999922315088193`}, \
{-0.9606010544999882, -1.1054290813024081`}, {-0.9579188215061619, \
-1.1108455832956183`}, {-0.9552190120901015, -1.1162417029652112`}, \
{-0.9525017314328832, -1.121617406323234}, {-0.9497670843181233, \
-1.1269726599099026`}, {-0.947015175132752, -1.132307430786568}, \
{-0.9442461078678338, -1.1376216865287652`}, {-0.9414599861194322, \
-1.142915395219345}, {-0.9386569130895186, -1.1481885254416861`}, \
{-0.9358369915869239, -1.1534410462729885`}, {-0.9330003240283301, \
-1.1586729272776453`}, {-0.9301470124393026, -1.163884138500696}, \
{-0.9272771584553606, -1.1690746504613567`}, {-0.9243908633230847, \
-1.1742444341466283`}, {-0.9214882279012607, -1.1793934610049814`}, \
{-0.9185693526620585, -1.1845217029401187`}, {-0.9156343376922446, \
-1.1896291323048118`}, {-0.9126832826944268, -1.1947157218948132`}, \
{-0.9097162869883317, -1.1997814449428428`}, {-0.9067334495121117, \
-1.2048262751126475`}, {-0.9037348688236814, -1.2098501864931333`}, \
{-0.9007206431020834, -1.21485315359257}, {-0.8976908701488808, \
-1.219835151332867}, {-0.8946456473895765, -1.2247961550439186`}, \
{-0.8915850718750581, -1.2297361404580214`}, {-0.8885092402830673, \
-1.2346550837043584`}, {-0.8854182489196937, -1.239552961303553}, \
{-0.8823121937208901, -1.24442975016229}, {-0.8791911702540116, \
-1.2492854275680045`}, {-0.8760552737193747, -1.254119971183636}, \
{-0.8729045989518374, -1.2589333590424485`}, {-0.8697392404223981, \
-1.2637255695429161`}, {-0.8665592922398149, -1.2684965814436717`}, \
{-0.8633648481522409, -1.2732463738585198`}, {-0.8601560015488788, \
-1.277974926251513}, {-0.856932845461651, -1.2826822184320883`}, \
{-0.8536954725668859, -1.2873682305502676`}, {-0.8504439751870198, \
-1.2920329430919169`}, {-0.8471784452923123, -1.2966763368740664`}, \
{-0.8438989745025778, -1.3012983930402904`}, {-0.8406056540889276, \
-1.3058990930561454`}, {-0.8372985749755273, -1.3104784187046659`}, \
{-0.833977827741364, -1.3150363520819193`}, {-0.8306435026220271, \
-1.3195728755926155`}, {-0.8272956895114988, -1.3240879719457743`}, \
{-0.8239344779639549, -1.3285816241504478`}, {-0.8205599571955768, \
-1.3330538155114966`}, {-0.8171722160863715, -1.3375045296254227`}, \
{-0.8137713431820008, -1.341933750376253}, {-0.8103574266956196, \
-1.3463414619314786`}, {-0.8069305545097208, -1.3507276487380444`}, \
{-0.8034908141779891, -1.3550922955183917`}, {-0.8000382929271601, \
-1.3594353872665514`}, {-0.7965730776588879, -1.363756909244288}, \
{-0.7930952549516166, -1.3680568469772934`}, {-0.7896049110624593, \
-1.3723351862514297`}, {-0.7861021319290814, -1.3765919131090223`}, \
{-0.7825870031715889, -1.380827013845199}, {-0.7790596100944215, \
-1.3850404750042788`}, {-0.7755200376882494, -1.3892322833762065`}, \
{-0.7719683706318742, -1.3934024259930338`}, {-0.7684046932941331, \
-1.397550890125447}, {-0.7648290897358059, -1.4016776632793402`}, \
{-0.7612416437115251, -1.4057827331924324`}, {-0.7576424386716883, \
-1.4098660878309297`}, {-0.7540315577643727, -1.4139277153862309`}, \
{-0.750409083837251, -1.4179676042716765`}, {-0.7467750994395096, \
-1.4219857431193403`}, {-0.7431296868237671, -1.4259821207768637`}, \
{-0.7394729279479947, -1.4299567263043307`}, {-0.7358049044774363, \
-1.4339095489711848`}, {-0.7321256977865298, -1.4378405782531867`}, \
{-0.7284353889608282, -1.4417498038294114`}, {-0.7247340587989203, \
-1.4456372155792856`}, {-0.721021787814352, -1.4495028035796642`}, \
{-0.7172986562375462, -1.4533465581019456`}, {-0.7135647440177221, \
-1.4571684696092244`}, {-0.709820130824814, -1.4609685287534837`}, \
{-0.7060648960513883, -1.4647467263728222`}, {-0.7022991188145596, \
-1.4685030534887207`}, {-0.6985228779579047, -1.4722375013033424`}, \
{-0.6947362520533752, -1.4759500611968719`}, {-0.6909393194032085, \
-1.479640724724887}, {-0.6871321580418358, -1.4833094836157679`}, \
{-0.6833148457377886, -1.486956329768139}, {-0.6794874599956021, \
-1.4905812552483462`}, {-0.6756500780577169, -1.4941842522879676`}, \
{-0.6718027769063768, -1.4977653132813573`}, {-0.6679456332655246, \
-1.5013244307832225`}, {-0.6640787236026945, -1.5048615975062325`}, \
{-0.6602021241309014, -1.508376806318661}, {-0.6563159108105268, \
-1.5118700502420583`}, {-0.6524201593512015, -1.5153413224489571`}, \
{-0.6485149452136845, -1.5187906162606066`}, {-0.6446003436117385, \
-1.5222179251447394`}, {-0.6406764295140011, -1.5256232427133676`}, \
{-0.6367432776458529, -1.5290065627206086`}, {-0.6328009624912814, \
-1.5323678790605404`}, {-0.6288495582947399, -1.5357071857650857`}, \
{-0.6248891390630038, -1.539024477001926}, {-0.6209197785670215, \
-1.5423197470724417`}, {-0.6169415503437613, -1.5455929904096828`}, \
{-0.6129545276980538, -1.5488442015763648`}, {-0.6089587837044304, \
-1.552073375262894}, {-0.6049543912089559, -1.5552805062854171`}, \
{-0.6009414228310581, -1.5584655895839015`}, {-0.5969199509653516, \
-1.561628620220237}, {-0.5928900477834568, -1.5647695933763675`}, \
{-0.588851785235815, -1.567888504352446}, {-0.5848052350534977, \
-1.5709853485650154`}, {-0.5807504687500109, -1.5740601215452152`}, \
{-0.5766875576230958, -1.5771128189370118`}, {-0.5726165727565223, \
-1.5801434364954532`}, {-0.5685375850218798, -1.5831519700849488`}, \
{-0.5644506650803605, -1.586138415677572}, {-0.5603558833845396, \
-1.5891027693513864`}, {-0.5562533101801492, -1.592045027288796}, \
{-0.5521430155078471, -1.5949651857749172`}, {-0.5480250692049804, \
-1.5978632411959737`}, {-0.5438995409073442, -1.600739190037715}, \
{-0.5397665000509343, -1.603593028883855}, {-0.5356260158736954, \
-1.6064247544145336`}, {-0.5314781574172631, -1.6092343634048}, \
{-0.527322993528701, -1.6120218527231167`}, {-0.5231605928622326, \
-1.6147872193298842`}, {-0.5189910238809672, -1.6175304602759877`}, \
{-0.5148143548586208, -1.620251572701363}, {-0.510630653881232, \
-1.622950553833584}, {-0.506439988848871, -1.6256274009864682`}, \
{-0.5022424274773452, -1.6282821115587043`}, {-0.4980380372998975, \
-1.6309146830324974`}, {-0.49382688566890043`, -1.6335251129722352`}, \
{-0.48960903975754405`, -1.6361133990231715`}, {-0.4853845665615189, \
-1.63867953891013}, {-0.48115353290069324`, -1.641223530436226}, \
{-0.4769160054207848, -1.6437453714816062`}, {-0.47267205059502754`, \
-1.6462450600022074`}, {-0.4684217347258323, -1.6487225940285328`}, \
{-0.46416512394644266`, -1.6511779716644461`}, {-0.4599022842225849, \
-1.653611191085983}, {-0.45563328135411296`, -1.6560222505401796`}, \
{-0.4513581809766478, -1.658411148343919}, {-0.4470770485632114, \
-1.6607778828827942`}, {-0.4427899494258553, -1.6631224526099868`}, \
{-0.4384969487172842, -1.6654448560451647`}, {-0.4341981114324736, \
-1.6677450917733927`}, {-0.42989350241028296`, -1.6700231584440626`}, \
{-0.4255831863350627, -1.6722790547698367`}, {-0.4212672277382566, \
-1.674512779525609}, {-0.41694569099999856`, -1.6767243315474796`}, \
{-0.41261864035070445`, -1.6789137097317477`}, {-0.4082861398726584, \
-1.6810809130339168`}, {-0.40394825350159425`, -1.683225940467717}, \
{-0.3996050450282718, -1.6853487911041407`}, {-0.39525657810004783`, \
-1.6874494640704942`}, {-0.39090291622244194`, -1.6895279585494631`}, \
{-0.3865441227606976, -1.6915842737781928`}, {-0.3821802609413381, \
-1.6936184090473823`}, {-0.3778113938537173, -1.6956303637003924`}, \
{-0.3734375844515657, -1.6976201371323685`}, {-0.36905889555453125`, \
-1.6995877287893761`}, {-0.3646753898497155, -1.701533138167551}, \
{-0.3602871298932049, -1.703456364812261}, {-0.35589417811159707`, \
-1.7053574083172838`}, {-0.3514965968035225, -1.7072362683239957`}, \
{-0.34709444814116114`, -1.7090929445205743`}, {-0.34268779417175466`, \
-1.7109274366412137`}, {-0.3382766968191137, -1.7127397444653532`}, \
{-0.33386121788512074`, -1.714529867816917}, {-0.3294414190512281, \
-1.7162978065635686`}, {-0.3250173618799515, -1.7180435606159759`}, \
{-0.3205891078163591, -1.7197671299270887`}, {-0.31615671818955576`, \
-1.721468514491429}, {-0.3117202542141634, -1.7231477143443925`}, \
{-0.3072797769917963, -1.7248047295615627`}, {-0.30283534751253255`, \
-1.7264395602580358`}, {-0.2983870266563807, -1.7280522065877584`}, \
{-0.29393487519474265`, -1.729642668742875}, {-0.2894789537918716, \
-1.731210946953089}, {-0.28501932300632643`, -1.7327570414850328`}, \
{-0.2805560432924216, -1.7342809526416507`}, {-0.27608917500167296`, \
-1.7357826807615921`}, {-0.2716187783842394, -1.7372622262186157`}, \
{-0.26714491359036074`, -1.7387195894210044`}, {-0.2626676406717912, \
-1.7401547708109915`}, {-0.25818701958322937`, -1.7415677708641966`}, \
{-0.2537031101837439, -1.742958590089073}, {-0.24921597223819564`, \
-1.744327229026365}, {-0.24472566541865567`, -1.7456736882485753`}, \
{-0.24023224930581993`, -1.7469979683594434`}, {-0.2357357833904197, \
-1.7483000699934332`}, {-0.23123632707462874`, -1.7495799938152319`}, \
{-0.2267339396734667, -1.750837740519257}, {-0.2222286804161989, \
-1.7520733108291753`}, {-0.2177206084477326, -1.7532867054974302`}, \
{-0.2132097828300099, -1.7544779253047793`}, {-0.20869626254339702`, \
-1.7556469710598417`}, {-0.20418010648807022`, -1.756793843598654}, \
{-0.19966137348539867`, -1.7579185437842377`}, {-0.19514012227932356`, \
-1.7590210725061735`}, {-0.19061641153773434`, -1.7601014306801865`}, \
{-0.18609029985384154`, -1.7611596192477408`}, {-0.18156184574754664`, \
-1.762195639175642}, {-0.17703110766680866`, -1.76320949145565}, \
{-0.1724981439890078, -1.7642011771041004`}, {-0.16796301302230612`, \
-1.765170697161535}, {-0.1634257730070051, -1.7661180526923406`}, \
{-0.15888648211690048`, -1.767043244784397}, {-0.1543451984606343, \
-1.7679462745487342`}, {-0.1498019800830438, -1.7688271431191982`}, \
{-0.145256884966508, -1.7696858516521246`}, {-0.14070997103229127`, \
-1.7705224013260217`}, {-0.13616129614188446`, -1.7713367933412614`}, \
{-0.13161091809834322`, -1.7721290289197795`}, {-0.127058894647624, \
-1.772899109304784}, {-0.12250528347991738`, -1.7736470357604703`}, \
{-0.11795014223097904`, -1.7743728095717481`}, {-0.11339352848345831`, \
-1.7750764320439731`}, {-0.10883549976822439`, -1.7757579045026883`}, \
{-0.10427611356569023`, -1.7764172282933743`}, {-0.09971542730713426, \
-1.7770544047812065`}, {-0.0951534983760198, -1.7776694353508207`}, \
{-0.09059038410931247, -1.7782623214060866`}, {-0.08602614179879538, \
-1.7788330643698902`}, {-0.08146082869238239, -1.7793816656839228`}, \
{-0.07689450199542931, -1.7799081268084787`}, {-0.07232721887204323, \
-1.7804124492222608`}, {-0.06775903644638984, -1.7808946344221934`}, \
{-0.06319001180399911, -1.781354683923244}, {-0.058620201993069006`, \
-1.781792599258251}, {-0.0540496640257676, -1.7822083819777612`}, \
{-0.049478454879533416`, -1.7826020336498736`}, {-0.04490663149837422, \
-1.782973555860091}, {-0.04033425079416417, -1.78332295021118}, \
{-0.0357613696479395, -1.7836502183230378`}, {-0.031188044911192706`, \
-1.7839553618325668`}, {-0.026614333407165278`, -1.784238382393557}, \
{-0.022040291932139133`, -1.7844992816765761`}, {-0.017465977256726675`, \
-1.7847380613688657`}, {-0.012891446127159605`, -1.7849547231742473`}, \
{-0.00831675526657652, -1.7851492688130335`}, {-0.003741961376309352, \
-1.785321700021948}, {0.000832878862831319, -1.7854720185540527`}, {
      0.005407708789271994, -1.7856002261786825`}, {
      0.009982471759393003, -1.7857063246813862`}, {
      0.014557111146246209`, -1.7857903158638768`}, {
      0.019131570338273615`, -1.7858522015439873`}, {
      0.023705792738026817`, -1.7858919835556348`}, {
      0.028279721760887284`, -1.7859096637487923`}, {
      0.03285330083378736, -1.7859052439894663`}, {
      0.037426473393931965`, -1.7858787261596833`}, {
      0.04199918288752098, -1.7858301121574836`}, {
      0.046571372768472195`, -1.7857594038969202`}, {
      0.05114298649714479, -1.7856666033080681`}, {
      0.05571396753906336, -1.7855517123370388`}, {
      0.060284259363642324`, -1.7854147329460026`}, {
      0.06485380544291078, -1.7852556671132187`}, {
      0.06942254925023765, -1.7850745168330717`}, {
      0.07399043425905714, -1.784871284116116}, {
      0.07855740394159445, -1.7846459709891274`}, {
      0.08312340176759163, -1.7843985794951618`}, {
      0.08768837120303358, -1.7841291116936218`}, {
      0.09225225570887419, -1.7838375696603292`}, {
      0.09681499873976243, -1.783523955487607}, {
      0.1013765437427685, -1.783188271284366}, {
      0.10593683415610983`, -1.7828305191762017`}, {
      0.11049581340787702`, -1.7824507013054953`}, {
      0.11505342491475962`, -1.7820488198315254`}, {
      0.1196096120807716, -1.7816248769305845`}, {
      0.12416431829597667`, -1.7811788747961037`}, {
      0.12871748693521315`, -1.7807108156387854`}, {
      0.13326906135681862`, -1.7802207016867426`}, {
      0.13781898490135397`, -1.7797085351856456`}, {
      0.1423672008903271, -1.7791743183988773`}, {
      0.14691365262491604`, -1.7786180536076936`}, {
      0.15145828338469158`, -1.7780397431113946`}, {
      0.15600103642633914`, -1.7774393892274996`}, {
      0.16054185498238008`, -1.776816994291933}, {
      0.16508068225989217`, -1.7761725606592154`}, {
      0.16961746143922946`, -1.775506090702663}, {
      0.1741521356727411, -1.7748175868145952`}, {
      0.17868464808348952`, -1.7741070514065491`}, {
      0.18321494176396752`, -1.7733744869095018`}, {
      0.18774295977481442`, -1.7726198957741004`}, {
      0.19226864514353123`, -1.7718432804709}, {
      0.1967919408631947, -1.7710446434906086`}, {
      0.20131278989117032`, -1.770223987344342}, {
      0.20583113514782403`, -1.7693813145638828`}, {
      0.21034691951523285`, -1.7685166277019508`}, {
      0.21486008583589417`, -1.7676299293324795`}, {
      0.2193705769114336, -1.7667212220509008`}, {
      0.22387833550131178`, -1.7657905084744379`}, {
      0.22838330432152942`, -1.7648377912424063`}, {
      0.23288542604333112`, -1.763863073016522}, {
      0.2373846432919075, -1.7628663564812193`}, {
      0.241880898645096, -1.7618476443439752`}, {
      0.24637413463207974`, -1.7608069393356436`}, {
      0.2508642937320852, -1.7597442442107956`}, {
      0.25535131837307756`, -1.7586595617480703`}, {
      0.259835150930455, -1.7575528947505328`}, {
      0.2643157337257407, -1.7564242460460406`}, {
      0.26879300902527303`, -1.755273618487619}, {
      0.2732669190388942, -1.7541010149538439`}, {
      0.2777374059186366, -1.7529064383492348`}, {
      0.28220441175740724`, -1.7516898916046548`}, {
      0.2866678785876704, -1.7504513776777209`}, {
      0.2911277483801279, -1.749190899553221}, {
      0.29558396304239737`, -1.747908460243542}, {
      0.3000364644176884, -1.7466040627891044`}, {
      0.30448519428347637`, -1.7452777102588084`}, {
      0.30893009435017404`, -1.7439294057504866`}, {
      0.3133711062598011, -1.7425591523913673`}, {
      0.3178081715846507, -1.741166953338546}, {
      0.32224123182595454`, -1.7397528117794676`}, {
      0.3266702284125447, -1.7383167309324155`}, {
      0.33109510269951337`, -1.7368587140470122`}, {
      0.33551579596687, -1.7353787644047287`}, {
      0.3399322494181958, -1.7338768853194024`}, {
      0.34434440417929574`, -1.732353080137766}, {
      0.348752201296848, -1.7308073522399854`}, {
      0.3531555817370503, -1.7292397050402075`}, {
      0.35755448638426396`, -1.7276501419871173`}, {
      0.3619488560396548, -1.7260386665645058`}, {
      0.36633863141983164`, -1.7244052822918472`}, {
      0.37072375315548145`, -1.7227499927248868`}, {
      0.37510416179000194`, -1.7210728014562384`}, {
      0.3794797977781312, -1.7193737121159924`}, {
      0.383850601484574, -1.717652728372334}, {
      0.3882165131826256, -1.7159098539321722`}, {
      0.39257747305279206`, -1.7141450925417794`}, {
      0.3969334211814075, -1.7123584479874407`}, {
      0.40128429755924855`, -1.7105499240961157`}, {
      0.4056300420801451, -1.7087195247361089`}, {
      0.4099705945395884, -1.7068672538177534`}, {
      0.41430589463333545`, -1.704993115294104}, {
      0.4186358819560102, -1.7030971131616424`}, {
      0.4229604959997017, -1.7011792514609925`}, {
      0.42727967615255846`, -1.6992395342776487`}, {
      0.4315933616973797, -1.6972779657427148`}, {
      0.4359014918102031, -1.6952945500336545`}, {
      0.440204005558889, -1.6932892913750535`}, {
      0.44450084190170125`, -1.691262194039394}, {
      0.4487919396858844, -1.6892132623478409`}, {
      0.4530772376462372, -1.6871425006710399`}, {
      0.45735667440368294`, -1.6850499134299284`}, {
      0.4616301884638357, -1.682935505096558}, {
      0.46589771821556303`, -1.6807992801949299`}, {
      0.47015920192954525`, -1.6786412433018427`}, {
      0.47441457775683055`, -1.6764613990477537`}, {
      0.4786637837273868, -1.6742597521176517`}, {
      0.48290675774864933`, -1.672036307251944}, {
      0.4871434376040647, -1.669791069247356}, {
      0.4913737609516312, -1.6675240429578448`}, {
      0.49559766532243504`, -1.6652352332955245`}, {
      0.49981508811918285`, -1.6629246452316069`}, {
      0.5040259666147302, -1.6605922837973557`}, {
      0.5082302379506061, -1.6582381540850537`}, {
      0.5124278391355338, -1.6558622612489842`}, {
      0.5166187070439475, -1.6534646105064275`}, {
      0.5208027784145051, -1.6510452071386708`}, {
      0.5249799898485965, -1.6486040564920328`}, {
      0.5291502778088489, -1.6461411639789034`}, {
      0.5333135786176268, -1.6436565350787982`}, {
      0.537469828455529, -1.6411501753394269`}, {
      0.5416189633598809, -1.6386220903777784`}, {
      0.5457609192232228, -1.6360722858812204`}, {
      0.5498956317917942, -1.6335007676086144`}, {
      0.5540230366640144, -1.6309075413914471`}, {
      0.5581430692889577, -1.628292613134977}, {
      0.5622556649648262, -1.6256559888193973`}, {
      0.5663607588374167, -1.6229976745010148`}, {
      0.5704582858985849, -1.6203176763134457`}, {
      0.5745481809847048, -1.617616000468828}, {
      0.5786303787751238, -1.6148926532590502`}, {
      0.5827048137906136, -1.612147641056997}, {
      0.5867714203918176, -1.6093809703178132`}, {
      0.5908301327776937, -1.6065926475801835`}, {
      0.5948808849839524, -1.6037826794676313`}, {
      0.5989236108814917, -1.6009510726898346`}, {
      0.6029582441748274, -1.59809783404396}, {
      0.6069847184005188, -1.5952229704160146`}, {
      0.6110029669255914, -1.5923264887822173`}, {
      0.6150129229459543, -1.5894083962103882`}, {
      0.6190145194848141, -1.5864686998613566`}, {
      0.6230076893910845, -1.5835074069903885`}, {
      0.626992365337792, -1.5805245249486333`}, {
      0.6309684798204771, -1.5775200611845905`}, {
      0.6349359651555919, -1.5744940232455957`}, {
      0.6388947534788938, -1.571446418779327}, {
      0.6428447767438343, -1.5683772555353306`}, {
      0.6467859667199449, -1.5652865413665693`}, {
      0.650718254991218, -1.5621742842309894`}, {
      0.6546415729544851, -1.5590404921931096`}, {
      0.6585558518177898, -1.5558851734256316`}, {
      0.6624610225987577, -1.5527083362110714`}, {
      0.6663570161229628, -1.5495099889434132`}, {
      0.6702437630222888, -1.5462901401297846`}, {
      0.6741211937332883, -1.5430487983921548`}, {
      0.6779892384955366, -1.5397859724690548`}, {
      0.6818478273499838, -1.5365016712173214`}, {
      0.6856968901373015, -1.5331959036138634`}, {
      0.6895363564962268, -1.5298686787574516`}, {
      0.6933661558619026, -1.5265200058705324`}, {
      0.6971862174642152, -1.5231498943010662`}, {
      0.7009964703261267, -1.519758353524389}, {
      0.7047968432620062, -1.5163453931450994`}, {
      0.7085872648759567, -1.5129110228989697`}, {
      0.7123676635601386, -1.5094552526548832`}, {
      0.7161379674930906, -1.505978092416797}, {
      0.7198981046380474, -1.5024795523257286`}, {
      0.7236480027412545, -1.4989596426617724`}, {
      0.7273875893302801, -1.495418373846139}, {
      0.7311167917123245, -1.4918557564432235`}, {
      0.7348355369725259, -1.4882718011626996`}, {
      0.7385437519722652, -1.4846665188616428`}, {
      0.7422413633474666, -1.4810399205466795`}, {
      0.7459282975068974, -1.477392017376166}, {
      0.7496044806304639, -1.4737228206623945`}, {
      0.7532698386675071, -1.4700323418738286`}, {
      0.7569242973350941, -1.4663205926373681`}, {
      0.7605677821163098, -1.4625875847406438`}, {
      0.7642002182585452, -1.4588333301343406`}, {
      0.7678215307717848, -1.4550578409345523`}, {
      0.7714316444268924, -1.4512611294251667`}, {
      0.7750304837538949, -1.4474432080602817`}, {
      0.7786179730402663, -1.4436040894666526`}, {
      0.7821940363292086, -1.4397437864461715`}, {
      0.7857585974179337, -1.4358623119783782`}, {
      0.7893115798559436, -1.431959679223005}, {
      0.7928529069433093, -1.4280359015225523`}, {
      0.7963825017289512, -1.4240909924049008`}, {
      0.7999002870089171, -1.4201249655859536`}, {
      0.8034061853246618, -1.4161378349723157`}, {
      0.8069001189613257, -1.4121296146640065`}, {
      0.8103820099460145, -1.4081003189572072`}, {
      0.8138517800460788, -1.4040499623470448`}, {
      0.817309350767395, -1.3999785595304108`}, {
      0.8207546433526465, -1.395886125408817}, {
      0.8241875787796069, -1.3917726750912873`}, {
      0.8276080777594238, -1.3876382238972875`}, {
      0.8310160607349045, -1.3834827873596924`}, {
      0.8344114478788041, -1.3793063812277904`}, {
      0.8377941590921146, -1.3751090214703279`}, {
      0.8411641140023579, -1.3708907242785917`}, {
      0.8445212319618798, -1.3666515060695315`}, {
      0.8478654320461484, -1.3623913834889212`}, {
      0.8511966330520547, -1.358110373414563}, {
      0.8545147534962174, -1.3538084929595293`}, {
      0.8578197116132911, -1.349485759475449}, {
      0.8611114253542784, -1.3451421905558336`}, {
      0.8643898123848476, -1.3407778040394467`}, {
      0.8676547900836534, -1.3363926180137158`}, {
      0.8709062755406645, -1.3319866508181886`}, {
      0.874144185555495, -1.3275599210480318`}, {
      0.8773684366357432, -1.323112447557575}, {
      0.8805789449953352, -1.3186442494639004`}, {
      0.8837756265528759, -1.3141553461504758`}, {
      0.8869583969300069, -1.3096457572708362`}, {
      0.890127171449771, -1.3051155027523098`}, {
      0.8932818651349858, -1.3005646027997928`}, {
      0.8964223927066234, -1.2959930778995705`}, {
      0.8995486685822013, -1.2914009488231872`}, {
      0.9026606068741801, -1.2867882366313648`}, {
      0.9057581213883722, -1.2821549626779705`}, {
      0.9088411256223593, -1.277501148614035}, {
      0.9119095327639214, -1.2728268163918202`}, {
      0.914963255689476, -1.2681319882689381`}, {
      0.9180022069625291, -1.2634166868125225`}, {
      0.9210262988321382, -1.2586809349034505`}, {
      0.9240354432313873, -1.253924755740619}, {
      0.9270295517758761, -1.2491481728452736`}, {
      0.9300085357622213, -1.244351210065391}, {
      0.9329723061665733, -1.2395338915801168`}, {
      0.9359207736431472, -1.234696241904258}, {
      0.9388538485227689, -1.2298382858928303`}, {
      0.9417714408114378, -1.224960048745664}, {
      0.9446734601889062, -1.2200615560120645`}, {
      0.9475598160072749, -1.2151428335955317`}, {
      0.9504304172896082, -1.210203907758537}, {
      0.9532851727285666, -1.2052448051273592`}, {
      0.9561239906850595, -1.2002655526969808`}, {
      0.9589467791869173, -1.1952661778360418`}, {
      0.9617534459275848, -1.1902467082918573`}, {
      0.9645438982648362, -1.1852071721954942`}, {
      0.9673180432195123, -1.180147598066911}, {
      0.9700757874742806, -1.1750680148201589`}, {
      0.9728170373724199, -1.1699684517686482`}, {
      0.9755416989166296, -1.1648489386304761`}, {
      0.9782496777678641, -1.1597095055338213`}, {
      0.9809408792441946, -1.154550183022402}, {
      0.9836152083196975, -1.1493710020610004`}, {
      0.9862725696233716, -1.1441719940410535`}, {
      0.9889128674380848, -1.1389531907863117`}, {
      0.9915360056995504, -1.1337146245585636`}, {
      0.994141887995335, -1.128456328063431}, {
      0.996730417563899, -1.1231783344562312`}, {
      0.9993014972936696, -1.11788067734791}, {
      1.0018550297221485`, -1.112563390811045}, {
      1.0043909170350553`, -1.10722650938592}, {
      1.0069090610655071`, -1.101870068086669}, {
      1.0094093632932355`, -1.0964941024074966`}, {
      1.0118917248438426`, -1.0910986483289662`}, {
      1.0143560464880976`, -1.0856837423243662`}, {
      1.0168022286412735`, -1.0802494213661478`}, {
      1.0192301713625265`, -1.0747957229324379`}, {
      1.02163977435432, -1.0693226850136284`}, {
      1.024030936961892, -1.0638303461190406`}, {
      1.0264035581727706`, -1.0583187452836664`}, {
      1.0287575366163348`, -1.0527879220749874`}, {
      1.0310927705634272`, -1.047237916599871}, {
      1.0334091579260143`, -1.0416687695115459`}, {
      1.0357065962569008`, -1.0360805220166553`}, {
      1.0379849827494956`, -1.030473215882392}, {
      1.040244214237634, -1.024846893443711}, {
      1.0424841871954553`, -1.0192015976106257`}, {
      1.0447047977373387`, -1.0135373718755831`}, {
      1.0469059416178985`, -1.0078542603209226`}, {
      1.0490875142320408`, -1.0021523076264165`}, {
      1.051249410615083, -0.9964315590768935}, {
      1.053391525442937, -0.9906920605699463}, {
      1.05551375303236, -0.9849338586237227}, {
      1.057615987341272, -0.9791570003848017}, {
      1.0596981219691446`, -0.9733615336361539}, {
      1.0617600501574591`, -0.9675475068051882}, {
      1.0638016647902413`, -0.9617149689718846}, {
      1.0658228583946692`, -0.955863969877012}, {
      1.0678235231417592`, -0.9499945599304348}, {
      1.0698035508471317`, -0.9441067902195048}, {
      1.0717628329718585`, -0.9382007125175421}, {
      1.0737012606233922`, -0.9322763792924033}, {
      1.0756187245565834`, -0.9263338437151382}, {
      1.0775151151747842`, -0.9203731596687349}, {
      1.079390322531042, -0.9143943817569544}, {
      1.0812442363293864`, -0.9083975653132536}, {
      1.0830767459262094`, -0.9023827664097981}, {
      1.084887740331743, -0.896350041866565}, {
      1.086677108211636, -0.890299449260535}, {
      1.0884447378886328`, -0.8842310469349748}, {
      1.0901905173443556`, -0.8781448940088097}, {
      1.0919143342211932`, -0.8720410503860863}, {
      1.0936160758242999`, -0.8659195767655254}, {
      1.095295629123705, -0.859780534650166}, {
      1.0969528807565372`, -0.8536239863570984}, {
      1.0985877170293656`, -0.8474499950272885}, {
      1.100200023920661, -0.8412586246354926}, {
      1.1017896870833792`, -0.8350499400002616}, {
      1.10335659184767, -0.8288240067940352}, {
      1.1049006232237142`, -0.8225808915533266}, {
      1.1064216659046917`, -0.8163206616889959}, {
      1.1079196042698838`, -0.8100433854966133}, {
      1.1093943223879124`, -0.8037491321669107}, {
      1.1108457040201192`, -0.7974379717963225}, {
      1.1122736326240887`, -0.7911099753976135}, {
      1.113677991357317, -0.7847652149105954}, {
      1.1150586630810295`, -0.7784037632129291}, {
      1.116415530364154, -0.772025694131015}, {
      1.1177484754874467`, -0.7656310824509668}, {
      1.1190573804477795`, -0.7592200039296727}, {
      1.1203421269625895`, -0.7527925353059391}, {
      1.1216025964744951`, -0.7463487543117182}, {
      1.122838670156082, -0.7398887396834178}, {
      1.1240502289148613`, -0.7334125711732932}, {
      1.125237153398406, -0.7269203295609177}, {
      1.126399323999667, -0.7204120966647346}, {
      1.1275366208624733`, -0.7138879553536851}, {
      1.1286489238872202`, -0.7073479895589138}, {
      1.129736112736749, -0.7007922842855492}, {
      1.1307980668424222`, -0.694220925624558}, {
      1.1318346654103981`, -0.6876340007646714}, {
      1.1328457874281077`, -0.6810315980043823}, {
      1.1338313116709395`, -0.6744138067640105}, {
      1.1347911167091351`, -0.6677807175978357}, {
      1.1357250809148993`, -0.6611324222062952}, {
      1.13663308246973, -0.6544690134482437}, {
      1.1375149993719704`, -0.6477905853532754}, {
      1.1383707094445898`, -0.6410972331341033}, {
      1.139200090343194, -0.6343890531989951}, {
      1.1400030195642727`, -0.6276661431642631}, {
      1.1407793744536858`, -0.6209286018668044}, {
      1.1415290322153946`, -0.6141765293766906}, {
      1.1422518699204405`, -0.6074100270098016}, {
      1.1429477645161772`, -0.6006291973405029}, {
      1.143616592835759, -0.5938341442143621}, {
      1.1442582316078902`, -0.5870249727609018}, {
      1.1448725574668424`, -0.580201789406385}, {
      1.1454594469627382`, -0.5733647018866301}, {
      1.1460187765721128`, -0.5665138192598516}, {
      1.1465504227087526`, -0.5596492519195225}, {
      1.147054261734817, -0.5527711116072557}, {
      1.1475301699722487`, -0.5458795114256975}, {
      1.1479780237144763`, -0.5389745658514327}, {
      1.148397699238413, -0.5320563907478938}, {
      1.1487890728167556`, -0.5251251033782717}, {
      1.14915202073059, -0.5181808224184214}, {
      1.1494864192823073`, -0.5112236679697604}, {
      1.1497921448088313`, -0.5042537615721512}, {
      1.1500690736951675`, -0.49727122621676695`}, {
      1.1503170823882738`, -0.4902761863589305}, {
      1.1505360474112587`, -0.4832687679309245}, {
      1.1507258453779108`, -0.4762490983547653}, {
      1.1508863530075646`, -0.469217306554935}, {
      1.151017447140306, -0.4621735229710654}, {
      1.1511190047525226`, -0.4551178795705678}, {
      1.1511909029728016`, -0.44805050986120193`}, {
      1.1512330190981808`, -0.4409715489035771}, {
      1.1512452306107557`, -0.433881133323579}, {
      1.1512274151946469`, -0.426779401324715}, {
      1.1511794507533315`, -0.4196664927003701}, {
      1.1511012154273437`, -0.41254254884596675`}, {
      1.1509925876123448`, -0.4054077127710203}, {
      1.1508534459775712`, -0.3982621291110819}, {
      1.1506836694846583`, -0.3911059441395617}, {
      1.1504831374068487`, -0.3839393057794225}, {
      1.150251729348585, -0.3767623636147366}, {
      1.14998932526549, -0.36957526890209597`}, {
      1.1496958054847408`, -0.3623781745818676}, {
      1.1493710507258343`, -0.3551712352892839}, {
      1.1490149421217524`, -0.3479546073653593}, {
      1.1486273612405247`, -0.3407284488676232}, {
      1.1482081901071945`, -0.333492919580659}, {
      1.1477573112261885`, -0.3262481810264395}, {
      1.1472746076040923`, -0.318994396474448}, {
      1.1467599627728344`, -0.31173173095157475`}, {
      1.1462132608132798`, -0.30446035125177745`}, {
      1.1456343863792346`, -0.29718042594549554`}, {
      1.1450232247218621`, -0.28989212538880615`}, {
      1.1443796617145132`, -0.28259562173231073`}, {
      1.1437035838779697`, -0.27529108892974075`}, {
      1.1429948784061024`, -0.26797870274627017`}, {
      1.1422534331919432`, -0.2606586407665229}, {
      1.1414791368541721`, -0.25333108240226265`}, {
      1.1406718787640182`, -0.24599620889975282`}, {
      1.139831549072573, -0.23865420334677376`}, {
      1.1389580387385168`, -0.23130525067928415`}, {
      1.1380512395562565`, -0.22394953768771378`}, {
      1.1371110441844718`, -0.21658725302287404`}, {
      1.1361373461750686`, -0.20921858720147304`}, {
      1.1351300400025386`, -0.20184373261122127`}, {
      1.1340890210937196`, -0.19446288351551427`}, {
      1.133014185857956, -0.18707623605767829`}, {
      1.1319054317176547`, -0.17968398826476464`}, {
      1.130762657139232, -0.1722863400508787}, {
      1.1295857616644487`, -0.16488349322002885`}, {
      1.1283746459421282`, -0.15747565146848116`}, {
      1.1271292117602505`, -0.15006302038660488`}, {
      1.1258493620784193`, -0.14264580746019404`}, {
      1.1245350010606954`, -0.13522422207125048`}, {
      1.123186034108787, -0.1277984754982132}, {
      1.1218023678955942`, -0.12036878091561905`}, {
      1.1203839103990978`, -0.11293535339317984`}, {
      1.1189305709365835`, -0.10549840989426063`}, {
      1.1174422601991951`, -0.09805816927374411}, {
      1.1159188902868058`, -0.09061485227526611}, {
      1.1143603747431994`, -0.08316868152780688}, {
      1.11276662859155, -0.0757198815416232}, {
      1.1111375683701907`, -0.06826867870350625}, {
      1.1094731121686576`, -0.06081530127135021}, {
      1.1077731796640014`, -0.05335997936801659}, {
      1.1060376921573496`, -0.045902944974479706`}, {
      1.10426657261071, -0.03844443192223832}, {
      1.102459745683998, -0.030984675884979076`}, {
      1.1006171377722789`, -0.02352391436947717}, {
      1.0987386770432044`, -0.016062386705720175`}, {
      1.0968242934746315`, -0.008600334036240822}, {
      1.0948739188924081`, -0.001137999304645098}, {1.0928874870083058`, 
      0.006324372756678021}, {1.0908649334580856`, 0.0137865356396784}, {
      1.0888061958396758`, 0.021248241074611166`}, {1.0867112137514472`, 
      0.028709239044860356`}, {1.0845799288305633`, 0.0361692778026321}, {
      1.082412284791386, 0.043628103885529256`}, {1.0802082274639186`, 
      0.051085462134019086`}, {1.0779677048322642`, 0.05854109570980499}, {
      1.075690667073076, 0.06599474611511295}, {1.0733770665939801`, 
      0.07344615321290296}, {1.0710268580719473`, 0.08089505524801487}, {
      1.0686399984915886`, 0.0883411888692579}, {1.0662164471833537`, 
      0.09578428915245242}, {1.0637561658616055`, 0.10322408962443165`}, {
      1.0612591186625473`, 0.11066032228801087`}, {1.0587252721819753`, 
      0.11809271764793067`}, {1.0561545955128324`, 0.12552100473778022`}, {
      1.053547060282535, 0.1329449111479058}, {1.0509026406900475`, 
      0.14036416305430907`}, {1.0482213135426746`, 0.14777848524853904`}, {
      1.045503058292546, 0.15518760116858027`}, {1.0427478570727633`, 
      0.16259123293073996`}, {1.0399556947331814`, 0.16998910136253473`}, {
      1.0371265588757939`, 0.17738092603657776`}, {1.034260439889695, 
      0.18476642530546578`}, {1.0313573309855866`, 0.1921453163376644}, {
      1.0284172282297992`, 0.19951731515438942`}, {1.025440130577801, 
      0.20688213666748081`}, {1.0224260399071585`, 0.21423949471826503`}, {
      1.019374961049922, 0.22158910211740035`}, {1.0162869018244043`, 
      0.22893067068569858`}, {1.0131618730663188`, 0.2362639112959162}, {
      1.009999888659251, 0.24358853391550572`}, {1.0068009655644263`, 
      0.2509042476503185}, {1.0035651238497483`, 0.2582107607892473}, {
      1.0002923867180713`, 0.26550778084979787`}, {0.9969827805346806, 
      0.2727950146245756}, {0.9936363348539464, 0.2800721682286739}, {
      0.9902530824451218, 0.28733894714794844`}, {0.9868330593172538, 
      0.2945950562881614}, {0.9833763047431777, 0.301840200024978}, {
      0.9798828612825636, 0.30907408225479643`}, {0.9763527748039867, 
      0.3162964064463915}, {0.9727860945059911, 0.323506875693351}, {
      0.9691828729371185, 0.33070519276728255`}, {0.9655431660148737, 
      0.3378910601717675}, {0.9618670330435992, 0.3450641801970378}, {
      0.9581545367312309, 0.35222425497534954`}, {0.9544057432049088, 
      0.3593709865370277}, {0.9506207220254177, 0.36650407686715275`}, {
      0.9467995462004313, 0.3736232279628613}, {0.9429422921965358, 
      0.3807281418912308}, {0.9390490399500107, 0.3878185208477167}, {
      0.9351198728763422, 0.3948940672151108}, {0.9311548778784497, 
      0.4019544836229871}, {0.9271541453536022, 0.4089994730076019}, {
      0.9231177691990075, 0.41602873867221285`}, {0.919045846816054, 
      0.42304198434778106`}, {0.9149384791131884, 0.43003891425402047`}, {
      0.9107957705074122, 0.43701923316075586`}, {0.9066178289243827, 
      0.44398264644955243`}, {0.9024047657971038, 0.4509288601755772}, {
      0.8981566960631956, 0.4578575811296532}, {0.8938737381607288, 
      0.46476851690046556`}, {0.8895560140226169, 0.47166137593687957`}, {
      0.8852036490695552, 0.47853586761032835`}, {0.8808167722015011, 
      0.4853917022772289}, {0.8763955157876887, 0.49222859134138397`}, {
      0.8719400156551741, 0.49904624731632735`}, {0.8674504110759087, 
      0.5058443838875689}, {0.8629268447523382, 0.5126227159746973}, {
      0.8583694628015285, 0.5193809597932957}, {0.8537784147378197, 
      0.5261188329166273}, {0.8491538534540108, 0.5328360543370481}, {
      0.8444959352010815, 0.5395323445271011}, {0.8398048195664551, 
      0.5462074255002501}, {0.8350806694508129, 0.5528610208712094}, {
      0.8303236510434671, 0.5594928559158242}, {0.8255339337963039, 
      0.566102657630462}, {0.8207116903963112, 0.5726901547908682}, {
      0.8158570967367009, 0.5792550780104474}, {0.8109703318866454, 
      0.5857971597979253}, {0.8060515780596419, 0.5923161346143514}, {
      0.8011010205805261, 0.5988117389294005}, {0.7961188478511521, 
      0.6052837112769347}, {0.791105251314762, 0.6117317923097828}, {
      0.7860604254190673, 0.6181557248537022}, {0.7809845675780664, 
      0.6245552539604815}, {0.7758778781326229, 0.6309301269601496}, {
      0.770740560309832, 0.6372800935122526}, {0.765572820181204, 
      0.6436049056561645}, {0.7603748666196923, 0.649904317860397}, {
      0.7551469112555975, 0.6561780870708748}, {0.7498891684313786, 
      0.6624259727581441}, {0.7446018551554053, 0.6686477369634843}, {
      0.7392851910546829, 0.6748431443438917}, {0.7339393983265863, 
      0.6810119622159068}, {0.7285647016896379, 0.6871539605982588}, {
      0.7231613283333665, 0.6932689122533009}, {0.7177295078672837, 
      0.6993565927272101}, {0.7122694722690163, 0.705416780388931}, {
      0.7067814558316338, 0.7114492564678394}, {0.7012656951102105, 
      0.7174538050901064}, {0.6957224288676614, 0.7234302133137428}, {
      0.6901518980198931, 0.7293782711623082}, {0.6845543455803108, 
      0.7352977716572665}, {0.678930016603723, 0.7411885108489739}, {
      0.6732791581296848, 0.7470502878462876}, {0.6676020191253225, 
      0.7528829048447817}, {0.6618988504276822, 0.7586861671535612}, {
      0.6561699046856436, 0.7644598832206652}, {0.6504154363014432, 
      0.7702038646570525}, {0.6446357013718468, 0.7759179262591641}, {
      0.6388309576290172, 0.7816018860300578}, {0.6330014643811169, 
      0.7872555651991128}, {0.627147482452689, 0.792878788240304}, {
      0.6212692741248581, 0.7984713828890445}, {0.6153671030753936, 
      0.8040331801576001}, {0.609441234318676, 0.8095640143490769}, {
      0.6034919341456084, 0.8150637230699891}, {0.5975194700635119, 
      0.8205321472414103}, {0.5915241107360476, 0.8259691311087178}, {
      0.585506125923203, 0.8313745222499372}, {0.5794657864213822, 
      0.8367481715826977}, {0.5734033640036398, 0.8420899333698091}, {
      0.5673191313600932, 0.8473996652234735}, {0.5612133620385541, 
      0.852677228108144}, {0.5550863303854129, 0.8579224863420455}, {
      0.548938311486812, 0.863135307597375}, {0.5427695811101442, 
      0.8683155628991954}, {0.5365804156459073, 0.8734631266230432}, {
      0.5303710920499517, 0.8785778764912667}, {0.524141887786149, 
      0.8836596935681172}, {0.517893080769516, 0.8887084622536103}, {
      0.5116249493098223, 0.8937240702761824}, {0.5053377720557116, 
      0.8987064086841616}, {0.49903182793936507`, 0.9036553718360789}, {
      0.49270739612173264`, 0.9085708573898401}, {0.4863647559383604, 
      0.913452766290786}, {0.48000418684583745`, 0.9183010027586637}, {
      0.4736259683688871, 0.9231154742735356}, {0.46723038004812517`, 
      0.9278960915606517}, {0.4608177013885075, 0.9326427685743118}, {
      0.4543882118084869, 0.9373554224807451}, {0.44794219058990015`, 
      0.9420339736400332}, {0.4414799168286032, 0.9466783455871055}, {
      0.43500166938587176`, 0.951288465011835}, {0.42850772684058486`, 
      0.9558642617382609}, {0.42199836744220576`, 0.9604056687029691}, {
      0.4154738690645753, 0.9649126219326566}, {0.4089345091605304, 
      0.9693850605209102}, {0.402380564717361, 0.9738229266042279}, {
      0.39581231221311497`, 0.9782261653373108}, {0.3892300275737631, 
      0.9825947248676564}, {0.38263398613123223`, 0.9869285563094798}, {
      0.3760244625823148, 0.9912276137169932}, {0.3694017309484628, 
      0.9954918540570712}, {0.3627660645364712, 0.9997212371813317}, {
      0.3561177359000576, 1.0039157257976592`}, {0.3494570168023412, 
      1.0080752854411998`}, {0.34278417817922596`, 1.0121998844448554`}, {
      0.3360994901036891, 1.0162894939093046`}, {0.329403221750978, 
      1.0203440876725778`}, {0.3226956413647161, 1.024363642279214}, {
      0.3159770162239175, 1.0283481369490255`}, {0.30924761261090994`, 
      1.0322975535454968`}, {0.30250769578016545`, 1.0362118765438446`}, {
      0.2957575299280351, 1.0400910929987637`}, {0.28899737816338633`, 
      1.043935192511884}, {0.2822275024791386, 1.0477441671989642`}, {
      0.27544816372469344`, 1.0515180116568454`}, {0.2686596215792542, 
      1.0552567229301908`}, {0.2618621345260297, 1.0589603004780308`}, {
      0.2550559598273158, 1.0626287461401414`}, {0.2482413535004493, 
      1.0662620641032738`}, {0.2414185702946251, 1.069860260867262}, {
      0.23458786366857115`, 1.0734233452110238`}, {0.22774948576907159`, 
      1.0769513281584817`}, {0.22090368741033026`, 1.080444222944422}, {
      0.21405071805416523`, 1.08390204498031}, {0.20719082579102532`, 
      1.0873248118200844`}, {0.20032425732181844`, 1.0907125431259461`}, {
      0.1934512579405419, 1.0940652606341628`}, {0.18657207151770422`, 
      1.0973829881209038`}, {0.1796869404845273, 1.100665751368126}, {
      0.17279610581791835`, 1.103913578129523}, {0.1658998070262, 
      1.1071264980965594`}, {0.15899828213558687`, 1.1103045428645983`}, {
      0.15209176767739707`, 1.113447745899145}, {0.1451804986759864, 
      1.1165561425022141`}, {0.13826470863739304`, 1.1196297697788398`}, {
      0.13134462953868056`, 1.1226686666037384`}, {0.12442049181796641`, 
      1.1256728735881387`}, {0.11749252436512349`, 1.1286424330467906`}, {
      0.11056095451314189`, 1.1315773889651655`}, {0.10362600803013813`, 
      1.1344777869668583`}, {0.09668790911199864, 1.137343674281203}, {
      0.0897468803756448, 1.1401750997111113`}, {0.08280314285290641, 
      1.1429721136011441`}, {0.07585691598499039, 1.1457347678058265`}, {
      0.06890841761753175, 1.1484631156582126`}, {0.06195786399621372, 
      1.1511572119387117`}, {0.05500546976294395, 1.153817112844181}, {
      0.048051447952573725`, 1.1564428759572944`}, {0.04109600999014711, 
      1.159034560216193}, {0.03413936568866721, 1.161592225884425}, {
      0.02718172324736644, 1.1641159345211793`}, {0.0202232892504681, 
      1.1666057489518233`}, {0.0132642686664264, 1.1690617332387436`}, {
      0.006304864847632308, 1.1714839526524998`}, {-0.0006547204694274025, 
      1.1738724736432948`}, {-0.007614287163570377, 
      1.1762273638127645`}, {-0.014573636727890542`, 
      1.1785486918860921`}, {-0.02153257226831385, 
      1.1808365276844506`}, {-0.02849089850175004, 
      1.1830909420977762`}, {-0.03544842175385241, 
      1.185312007057876}, {-0.04240494995639744, 
      1.1874997955118716`}, {-0.049360292644295936`, 
      1.189654381395983}, {-0.05631426095224739, 
      1.191775839609652}, {-0.06326666761104875, 
      1.1938642459900095`}, {-0.07021732694356905, 
      1.1959196772866856`}, {-0.07716605486040079, 
      1.1979422111369662`}, {-0.08411266885519919, 
      1.1999319260412946`}, {-0.09105698799971988, 
      1.2018889013391214`}, {-0.09799883293856572, 
      1.2038132171850995`}, {-0.10493802588365303`, 
      1.2057049545256284`}, {-0.11187439060840762`, 
      1.207564195075745}, {-0.11880775244170055`, 
      1.2093910212963606`}, {-0.12573793826153346`, 
      1.2111855163718466`}, {-0.13266477648848324`, 
      1.2129477641879642`}, {-0.1395880970789155, 
      1.2146778493101402`}, {-0.14650773151797614`, 
      1.2163758569620864`}, {-0.15342351281237007`, 
      1.2180418730047617`}, {-0.1603352754829364, 
      1.2196759839156768`}, {-0.16724285555702828`, 
      1.2212782767685373`}, {-0.17414609056070637`, 
      1.222848839213226}, {-0.18104481951075432`, 
      1.2243877594561219`}, {-0.18793888290652427`, 
      1.2258951262407527`}, {-0.1948281227216206, 
      1.227371028828781}, {-0.20171238239542952`, 
      1.228815556981319}, {-0.20859150682450267`, 
      1.230228800940574}, {-0.21546534235380138`, 
      1.2316108514118158`}, {-0.22233373676780985`, 
      1.2329617995456703`}, {-0.22919653928152353`, 
      1.2342817369207315`}, {-0.2360536005313203, 
      1.2355707555264919`}, {-0.2429047725657208, 
      1.2368289477465875`}, {-0.24974990883604467`, 
      1.2380564063423554`}, {-0.2565888641869692, 
      1.2392532244366998`}, {-0.26342149484699656`, 
      1.2404194954982641`}, {-0.2702476584188356, 
      1.2415553133259067`}, {-0.27706721386970434`, 
      1.242660772033476}, {-0.2838800215215591, 
      1.243735966034882}, {-0.2906859430412552, 
      1.2447809900294615`}, {-0.2974848414306453, 
      1.2457959389876347`}, {-0.3042765810166206, 
      1.246780908136846}, {-0.3110610274410997, 
      1.2477359929477907`}, {-0.3178380476509706, 
      1.248661289120921}, {-0.32460750988799014`, 
      1.2495568925732288`}, {-0.331369283678646, 
      1.2504228994253008`}, {-0.3381232398239856, 
      1.2512594059886457`}, {-0.3448692503894165, 
      1.2520665087532852`}, {-0.35160718869448215`, 
      1.2528443043756092`}, {-0.35833692930261773`, 
      1.253592889666491}, {-0.3650583480108896, 
      1.2543123615796565`}, {-0.3717713218397225, 
      1.2550028172003078`}, {-0.37847572902261795`, 
      1.2556643537339938`}, {-0.3851714489958679, 
      1.2562970684957269`}, {-0.3918583623882668, 
      1.2569010588993421`}, {-0.39853635101082546`, 
      1.2574764224470936`}, {-0.4052052978464901, 
      1.2580232567194858`}, {-0.4118650870398698, 
      1.258541659365337}, {-0.41851560388697506`, 
      1.2590317280920693`}, {-0.42515673482497085`, 
      1.2594935606562239`}, {-0.43178836742194626`, 
      1.2599272548541969`}, {-0.4384103903667045, 
      1.2603329085131922`}, {-0.44502269345857454`, 
      1.260710619482389}, {-0.45162516759724836`, 
      1.2610604856243193`}, {-0.4582177047726448, 
      1.2613826048064534`}, {-0.46480019805480327`, 
      1.261677074892988}, {-0.4713725415838093, 
      1.2619439937368362`}, {-0.4779346305597538, 
      1.262183459171813}, {-0.4844863612327284, 
      1.262395569005017}, {-0.49102763089285856`, 
      1.2625804210094}, {-0.4975583378603765, 
      1.2627381129165263`}, {-0.5040783814757358, 
      1.2628687424095149`}, {-0.510587662089769, 
      1.262972407116163}, {-0.5170860810538901, 
      1.2630492046022477`}, {-0.5235735407103436, 
      1.2630992323650017`}, {-0.5300499443825014, 
      1.2631225878267611`}, {-0.5365151963652092, 
      1.2631193683287827`}, {-0.5429692019151827, 
      1.2630896711252242`}, {-0.5494118672414572, 
      1.2630335933772905`}, {-0.5558430994958885, 
      1.2629512321475367`}, {-0.5622628067637099, 
      1.2628426843943297`}, {-0.5686708980541431, 
      1.2627080469664622`}, {-0.5750672832910659, 
      1.2625474165979194`}, {-0.5814518733037378, 
      1.2623608899027916`}, {-0.5878245798175827, 
      1.2621485633703347`}, {-0.594185315445032, 
      1.2619105333601708`}, {-0.6005339936764259, 
      1.2616468960976315`}, {-0.6068705288709773, 
      1.2613577476692355`}, {-0.613194836247795, 
      1.2610431840183032`}, {-0.6195068318769708, 
      1.2607033009407023`}, {-0.6258064326707271, 
      1.2603381940807226`}, {-0.6320935563746296, 
      1.2599479589270783`}, {-0.6383681215588621, 
      1.2595326908090345`}, {-0.6446300476095665, 
      1.259092484892656}, {-0.6508792547202464, 
      1.2586274361771743`}, {-0.6571156638832362, 
      1.2581376394914745`}, {-0.6633391968812353, 
      1.2576231894906937`}, {-0.6695497762789081, 
      1.2570841806529345`}, {-0.6757473254145492, 
      1.2565207072760873`}, {-0.6819317683918156, 
      1.2559328634747615`}, {-0.6881030300715246, 
      1.2553207431773212`}, {-0.6942610360635184, 
      1.2546844401230264`}, {-0.7004057127185953, 
      1.2540240478592735`}, {-0.7065369871205076, 
      1.2533396597389366`}, {-0.7126547870780269, 
      1.2526313689178064`}, {-0.7187590411170752, 
      1.2518992683521224`}, {-0.7248496784729244, 
      1.2511434507962005`}, {-0.7309266290824612, 
      1.2503640088001506`}, {-0.7369898235765208, 
      1.2495610347076844`}, {-0.7430391932722856, 
      1.248734620654009}, {-0.7490746701657522, 
      1.2478848585638085`}, {-0.7550961869242646, 
      1.247011840149307}, {-0.761103676879114, 
      1.2461156569084155`}, {-0.7670970740182055, 
      1.2451964001229567`}, {-0.7730763129787911, 
      1.2442541608569708`}, {-0.7790413290402687, 
      1.2432890299550956`}, {-0.7849920581170478, 
      1.2423010980410234`}, {-0.7909284367514803, 
      1.24129045551603}, {-0.7968504021068579, 
      1.2402571925575758`}, {-0.8027578919604739, 
      1.2392013991179778`}, {-0.8086508446967514, 
      1.2381231649231483`}, {-0.8145291993004357, 
      1.237022579471403}, {-0.8203928953498513, 
      1.2358997320323326`}, {-0.8262418730102241, 
      1.2347547116457396`}, {-0.832076073027067, 
      1.2335876071206369`}, {-0.8378954367196297, 
      1.2323985070343084`}, {-0.843699905974412, 
      1.2311874997314287`}, {-0.8494894232387401, 
      1.2299546733232414`}, {-0.855263931514406, 
      1.228700115686794}, {-0.8610233743513694, 
      1.2274239144642296`}, {-0.8667676958415215, 
      1.2261261570621318`}, {-0.8724968406125109, 
      1.2248069306509235`}, {-0.878210753821631, 
      1.2234663221643174`}, {-0.8839093811497685, 
      1.2221044182988166`}, {-0.8895926687954117, 
      1.2207213055132657`}, {-0.8952605634687203, 
      1.2193170700284501`}, {-0.9009130123856549, 
      1.2178917978267407`}, {-0.9065499632621651, 
      1.2164455746517875`}, {-0.9121713643084377, 
      1.2149784860082555`}, {-0.917777164223203, 
      1.213490617161606}, {-0.9233673121880996, 
      1.2119820531379197`}, {-0.9289417578620972, 
      1.210452878723763}, {-0.9345004513759761, 
      1.2089031784660926`}, {-0.9400433433268651, 
      1.2073330366722026`}, {-0.9455703847728342, 
      1.2057425374097077`}, {-0.9510815272275446, 
      1.2041317645065661`}, {-0.9565767226549543, 
      1.2025008015511385`}, {-0.9620559234640788, 
      1.2008497318922828`}, {-0.9675190825038061, 
      1.199178638639484}, {-0.9729661530577672, 
      1.1974876046630185`}, {-0.9783970888392592, 
      1.195776712594151}, {-0.9838118439862231, 
      1.1940460448253645`}, {-0.9892103730562737, 
      1.1922956835106215`}, {-0.9945926310217825, 
      1.1905257105656566`}, {-0.9999585732650131, 
      1.1887362076682972`}, {-1.0053081555733074`, 
      1.1869272562588162`}, {-1.0106413341343237`, 
      1.1850989375403107`}, {-1.015958065531325, 
      1.1832513324791099`}, {-1.0212583067385188`, 
      1.181384521805209}, {-1.0265420151164457`, 
      1.1794985860127305`}, {-1.0318091484074172`, 
      1.1775936053604095`}, {-1.0370596647310037`, 
      1.175669659872105}, {-1.04229352257957, 
      1.1737268293373355`}, {-1.0475106808138592`, 
      1.1717651933118374`}, {-1.0527110986586248`, 
      1.1697848311181465`}, {-1.0578947356983084`, 
      1.1677858218462025`}, {-1.0630615518727666`, 
      1.165768244353974}, {-1.0682115074730418`, 
      1.1637321772681062`}, {-1.0733445631371803`, 
      1.161677698984587}, {-1.078460679846095, 
      1.1596048876694345`}, {-1.0835598189194742`, 
      1.157513821259404}, {-1.088641942011734, 
      1.155404577462713}, {-1.0937070111080143`, 
      1.1532772337597843`}, {-1.0987549885202206`, 
      1.151131867404008}, {-1.1037858368831064`, 
      1.1489685554225202`}, {-1.1087995191504008`, 
      1.1467873746169974`}, {-1.1137959985909764`, 
      1.1445884015644692`}, {-1.1187752387850614`, 
      1.1423717126181454`}, {-1.1237372036204902`, 
      1.1401373839082583`}, {-1.1286818572889992`, 
      1.1378854913429206`}, {-1.1336091642825592`, 
      1.1356161106089977`}, {-1.138519089389751, 
      1.1333293171729935`}, {-1.1434115976921793`, 
      1.1310251862819494`}, {-1.1482866545609272`, 
      1.128703792964358}, {-1.1531442256530484`, 
      1.1263652120310879`}, {-1.1579842769081, 
      1.1240095180763217`}, {-1.162806774544711, 
      1.1216367854785059`}, {-1.1676116850571914`, 
      1.1192470884013124`}, {-1.1723989752121766`, 
      1.1168405007946105`}, {-1.1771686120453102`, 
      1.114417096395451}, {-1.1819205628579628`, 
      1.1119769487290598`}, {-1.1866547952139874`, 
      1.1095201311098424`}, {-1.1913712769365106`, 
      1.1070467166423976`}, {-1.1960699761047595`, 
      1.104556778222541}, {-1.2007508610509237`, 
      1.1020503885383375`}, {-1.2054139003570516`, 
      1.0995276200711437`}, {-1.2100590628519825`, 
      1.0969885450966572`}, {-1.2146863176083105`, 
      1.0944332356859756`}, {-1.2192956339393843`, 
      1.0918617637066623`}, {-1.2238869813963393`, 
      1.0892742008238214`}, {-1.2284603297651624`, 
      1.086670618501179}, {-1.23301564906379, 
      1.0840510880021716`}, {-1.2375529095392384`, 
      1.081415680391043}, {-1.2420720816647646`, 
      1.0787644665339453`}, {-1.246573136137061, 
      1.0760975171000486`}, {-1.2510560438734797`, 
      1.0734149025626556`}, {-1.2555207760092875`, 
      1.0707166932003225`}, {-1.2599673038949526`, 
      1.0680029590979863`}, {-1.2643955990934608`, 
      1.0652737701480959`}, {-1.2688056333776616`, 
      1.0625291960517507`}, {-1.2731973787276438`, 
      1.059769306319842}, {-1.2775708073281409`, 
      1.056994170274201}, {-1.2819258915659641`, 
      1.0542038570487509`}, {-1.2862626040274658`, 
      1.051398435590663}, {-1.2905809174960292`, 
      1.0485779746615176`}, {-1.294880804949588, 
      1.045742542838469}, {-1.2991622395581714`, 
      1.0428922085154144`}, {-1.3034251946814794`, 
      1.040027039904166}, {-1.3076696438664819`, 
      1.0371471050356276`}, {-1.311895560845047, 
      1.0342524717609736`}, {-1.316102919531595, 
      1.0313432077528322`}, {-1.3202916940207774`, 
      1.0284193805064712`}, {-1.324461858585184, 
      1.0254810573409865`}, {-1.3286133876730735`, 
      1.0225283054004939`}, {-1.3327462559061307`, 
      1.0195611916553233`}, {-1.336860438077248, 
      1.0165797829032157`}, {-1.3409559091483318`, 
      1.013584145770522}, {-1.3450326442481342`, 
      1.010574346713404}, {-1.3490906186701068`, 
      1.0075504520190393`}, {-1.3531298078702814`, 
      1.004512527806825}, {-1.3571501874651708`, 
      1.0014606400295865`}, {-1.3611517332296967`, 
      0.998394854474786}, {-1.365134421095138, 
      0.9953152367657335}, {-1.3690982271471026`, 
      0.9922218523627989}, {-1.373043127623523, 
      0.9891147665646262}, {-1.376969098912673, 
      0.9859940445093479}, {-1.3808761175512063`, 
      0.9828597511758023}, {-1.3847641602222185`, 
      0.9797119513847502}, {-1.3886332037533293`, 
      0.976550709800094}, {-1.3924832251147865`, 
      0.9733760909300967}, {-1.3963142014175915`, 
      0.9701881591286029}, {-1.4001261099116455`, 
      0.9669869785962595}, {-1.4039189279839162`, 
      0.9637726133817377}, {-1.4076926331566246`, 
      0.9605451273829557}, {-1.4114472030854535`, 
      0.9573045843483016}, {-1.4151826155577745`, 
      0.9540510478778569}, {-1.418898848490895, 
      0.950784581424621}, {-1.422595879930326, 
      0.9475052482957348}, {-1.426273688048067, 
      0.944213111653706}, {-1.429932251140913, 
      0.9409082345176338}, {-1.4335715476287767`, 
      0.9375906797644336}, {-1.4371915560530337`, 
      0.9342605101300623}, {-1.4407922550748826`, 
      0.9309177882107438}, {-1.444373623473725, 
      0.9275625764641939}, {-1.4479356401455639`, 
      0.9241949372108457}, {-1.451478284101419, 
      0.9208149326350741}, {-1.4550015344657607`, 
      0.9174226247864213}, {-1.4585053704749606`, 
      0.9140180755808214}, {-1.4619897714757606`, 
      0.9106013468018247}, {-1.4654547169237573`, 
      0.9071725001018222}, {-1.4689001863819058`, 
      0.9037315970032691}, {-1.472326159519038, 
      0.900278698899909}, {-1.4757326161083988`, 
      0.8968138670579966}, {-1.479119536026198, 
      0.8933371626175207}, {-1.482486899250179, 
      0.8898486465934267}, {-1.4858346858582032`, 
      0.8863483798768381}, {-1.4891628760268507`, 
      0.8828364232362784}, {-1.4924714500300356`, 
      0.8793128373188916}, {-1.4957603882376385`, 
      0.8757776826516629}, {-1.4990296711141526`, 
      0.8722310196426378}, {-1.502279279217347, 
      0.8686729085821422}, {-1.5055091931969427`, 
      0.8651034096440002}, {-1.5087193937933057`, 
      0.8615225828867528}, {-1.5119098618361533`, 
      0.8579304882548743}, {-1.515080578243276, 
      0.8543271855799899}, {-1.5182315240192727`, 
      0.8507127345820911}, {-1.5213626802543017`, 
      0.8470871948707512}, {-1.5244740281228437`, 
      0.84345062594634}, {-1.5275655488824813`, 
      0.8398030872012376}, {-1.5306372238726904`, 
      0.8361446379210479}, {-1.5336890345136458`, 
      0.8324753372858105}, {-1.5367209623050406`, 
      0.8287952443712132}, {-1.5397329888249192`, 
      0.825104418149803}, {-1.5427250957285223`, 
      0.8214029174921956}, {-1.5456972647471472`, 
      0.8176908011682864}, {-1.5486494776870185`, 
      0.8139681278484581}, {-1.551581716428174, 
      0.8102349561047896}, {-1.5544939629233616`, 
      0.8064913444122632}, {-1.55738619919695, 
      0.8027373511499716}, {-1.5602584073438512`, 
      0.7989730346023233}, {-1.5631105695284555`, 
      0.7951984529602484}, {-1.5659426679835788`, 
      0.7914136643224031}, {-1.5687546850094218`, 
      0.7876187266963733}, {-1.5715466029725418`, 
      0.7838136979998782}, {-1.574318404304835, 
      0.7799986360619723}, {-1.5770700715025325`, 
      0.7761735986242475}, {-1.579801587125206, 
      0.7723386433420343}, {-1.5825129337947865`, 
      0.768493827785602}, {-1.5852040941945937`, 
      0.764639209441359}, {-1.5878750510683766`, 
      0.7607748457130513}, {-1.5905257872193652`, 
      0.7569007939229621}, {-1.5931562855093346`, 
      0.7530171113131087}, {-1.5957665288576783`, 
      0.7491238550464407}, {-1.5983565002404931`, 
      0.7452210822080367}, {-1.6009261826896752`, 
      0.7413088498063003}, {-1.6034755592920262`, 
      0.7373872147741559}, {-1.6060046131883705`, 
      0.7334562339702445}, {-1.6085133275726824`, 
      0.7295159641801177}, {-1.6110016856912242`, 
      0.7255664621174323}, {-1.613469670841694, 
      0.7216077844251441}, {-1.6159172663723844`, 
      0.7176399876767015}, {-1.6183444556813507`, 
      0.713663128377238}, {-1.6207512222155895`, 
      0.7096772629647652}, {-1.623137549470227, 
      0.7056824478113647}, {-1.625503420987717, 
      0.7016787392243803}, {-1.6278488203570496`, 
      0.6976661934476092}, {-1.6301737312129678`, 
      0.6936448666624934}, {-1.6324781372351957`, 
      0.6896148149893109}, {-1.634762022147675, 
      0.6855760944883662}, {-1.637025369717811, 
      0.681528761161181}, {-1.6392681637557283`, 
      0.6774728709516845}, {-1.6414903881135354`, 
      0.6734084797474037}, {-1.6436920266845993`, 
      0.6693356433806533}, {-1.6458730634028285`, 
      0.665254417629726}, {-1.6480334822419647`, 
      0.6611648582200821}, {-1.6501732672148846`, 
      0.6570670208255395}, {-1.6522924023729098`, 
      0.6529609610694637}, {-1.654390871805126, 
      0.6488467345259575}, {-1.6564686596377105`, 
      0.6447243967210514}, {-1.6585257500332689`, 
      0.6405940031338931}, {-1.66056212719018, 
      0.6364556091979381}, {-1.66257777534195, 
      0.6323092703021397}, {-1.6645726787565738`, 
      0.62815504179214}, {-1.6665468217359072`, 
      0.6239929789714602}, {-1.6685001886150448`, 
      0.619823137102692}, {-1.6704327637617078`, 
      0.6156455714086885}, {-1.6723445315756402`, 
      0.6114603370737565}, {-1.674235476488012, 
      0.6072674892448477}, {-1.6761055829608325`, 
      0.6030670830327522}, {-1.6779548354863691`, 
      0.5988591735132902}, {-1.6797832185865778`, 
      0.5946438157285062}, {-1.6815907168125372`, 
      0.5904210646878625}, {-1.6833773147438953`, 
      0.5861909753694339}, {-1.6851429969883196`, 
      0.5819536027211024}, {-1.6868877481809592`, 
      0.5777090016617533}, {-1.6886115529839119`, 
      0.5734572270824712}, {-1.6903143960856997`, 
      0.569198333847737}, {-1.6919962622007536`, 
      0.5649323767966262}, {-1.6936571360689041`, 
      0.5606594107440073}, {-1.6952970024548801`, 
      0.5563794904817407}, {-1.6969158461478158`, 
      0.5520926707798802}, {-1.6985136519607646`, 
      0.547799006387873}, {-1.7000904047302217`, 
      0.543498552035763}, {-1.701646089315652, 
      0.5391913624353932}, {-1.703180690599028, 
      0.5348774922816107}, {-1.7046941934843736`, 
      0.5305569962534715}, {-1.706186582897315, 
      0.5262299290154474}, {-1.707657843784641, 
      0.5218963452186335}, {-1.7091079611138682`, 
      0.5175562995019571}, {-1.7105369198728153`, 
      0.5132098464933884}, {-1.711944705069184, 
      0.5088570408111512}, {-1.7133313017301468`, 
      0.5044979370649366}, {-1.7146966949019427`, 
      0.5001325898571167}, {-1.7160408696494798`, 
      0.4957610537839607}, {-1.7173638110559455`, 
      0.4913833834368518}, {-1.718665504222423, 
      0.4869996334035065}, {-1.719945934267516, 
      0.4826098582691946}, {-1.7212050863269797`, 
      0.47821411261796143`}, {-1.7224429455533588`, 
      0.47381245103385145`}, {-1.7236594971156338`, 
      0.4694049281021338}, {-1.7248547261988734`, 
      0.46499159841052967`}, {-1.7260286180038937`, 
      0.4605725165504412}, {-1.7271811577469254`, 
      0.45614773711818246`}, {-1.7283123306592871`, 
      0.4517173147162127}, {-1.7294221219870665`, 
      0.44728130395437077`}, {-1.7305105169908073`, 
      0.44283975945111254`}, {-1.731577500945205, 
      0.4383927358347495}, {-1.732623059138808, 
      0.4339402877446903}, {-1.7336471768737267`, 
      0.4294824698326837}, {-1.7346498394653487`, 
      0.4250193367640645}, {-1.7356310322420614`, 
      0.42055094321900144`}, {-1.7365907405449825`, 
      0.41607734389374706`}, {-1.7375289497276958`, 
      0.41159859350189065`}, {-1.7384456451559946`, 
      0.40711474677561316`}, {-1.7393408122076333`, 
      0.4026258584669449}, {-1.7402144362720833`, 
      0.39813198334902544`}, {-1.741066502750299, 
      0.39363317621736654`}, {-1.741896997054488, 
      0.38912949189111745`}, {-1.74270590460789, 
      0.3846209852143329}, {-1.7434932108445624`, 
      0.3801077110572442}, {-1.7442589012091716`, 
      0.37558972431753274`}, {-1.7450029611567937`, 
      0.37106707992160654`}, {-1.7457253761527194`, 
      0.3665398328258798}, {-1.7464261316722687`, 
      0.3620080380180555}, {-1.74710521320061, 
      0.3574717505184106}, {-1.747762606232588, 
      0.35293102538108506`}, {-1.748398296272558, 
      0.3483859176953733}, {-1.7490122688342278`, 
      0.34383648258701927`}, {-1.7496045094405053`, 
      0.3392827752195147}, {-1.750175003623355, 
      0.3347248507954009}, {-1.7507237369236603`, 
      0.3301627645575732}, {-1.7512506948910933`, 
      0.32559657179059}, {-1.7517558630839918`, 
      0.321026327821984}, {-1.7522392270692435`, 
      0.31645208802357816`}, {-1.752700772422178, 
      0.31187390781280455`}, {-1.7531404847264638`, 
      0.30729184265402704`}, {-1.7535583495740161`, 
      0.30270594805986784`}, {-1.7539543525649084`, 
      0.29811627959253767`}, {-1.7543284793072935`, 
      0.29352289286516997`}, {-1.7546807154173312`, 
      0.2889258435431588}, {-1.755011046519123, 
      0.2843251873455009}, {-1.7553194582446556`, 
      0.27972098004614143`}, {-1.7556059362337493`, 
      0.2751132774753245}, {-1.7558704661340163`, 
      0.270502135520947}, {-1.7561130336008255`, 
      0.26588761012991724`}, {-1.756333624297275, 
      0.2612697573095176}, {-1.7565322238941714`, 
      0.25664863312877156`}, {-1.7567088180700183`, 
      0.2520242937198151}, {-1.7568633925110106`, 
      0.24739679527927275`}, {-1.7569959329110383`, 
      0.2427661940696377}, {-1.7571064249716966`, 
      0.23813254642065707`}, {-1.7571948544023044`, 
      0.2334959087307213}, {-1.7572612069199312`, 
      0.2288563374682586}, {-1.7573054682494302`, 
      0.22421388917313395`}, {-1.7573276241234816`, 
      0.21956862045805306`}, {-1.757327660282642, 
      0.21492058800997116`}, {-1.7573055624754026`, 
      0.21026984859150671`}, {-1.7572613164582558`, 
      0.20561645904236026`}, {-1.7571949079957694`, 
      0.20096047628073815`}, {-1.7571063228606691`, 
      0.1963019573047817}, {-1.7569955468339298`, 
      0.19164095919400126`}, {-1.756862565704874, 
      0.18697753911071574`}, {-1.7567073652712801`, 
      0.18231175430149743`}, {-1.7565299313394989`, 
      0.17764366209862226`}, {-1.7563302497245767`, 
      0.17297331992152543`}, {-1.75610830625039, 
      0.16830078527826264`}, {-1.7558640867497863`, 
      0.16362611576697694`}, {-1.7555975770647358`, 
      0.1589493690773712}, {-1.7553087630464894`, 
      0.1542706029921863}, {-1.7549976305557478`, 
      0.1495898753886852}, {-1.7546641654628383`, 
      0.1449072442401427}, {-1.754308353647902, 
      0.14022276761734132`}, {-1.7539301810010877`, 
      0.13553650369007306`}, {-1.7535296334227575`, 
      0.13084851072864728`}, {-1.7531066968237, 
      0.12615884710540462`}, {-1.7526613571253538`, 
      0.12146757129623714`}, {-1.7521936002600402`, 
      0.11677474188211484`}, {-1.7517034121712052`, 
      0.11208041755061832`}, {-1.7511907788136707`, 
      0.10738465709747792`}, {-1.750655686153897, 
      0.10268751942811938`}, {-1.7500981201702537`, 
      0.09798906355921588}, {-1.7495180668533004`, 
      0.0932893486202468}, {-1.748915512206078, 
      0.08858843385506311}, {-1.748290442244411, 
      0.0838863786234594}, {-1.7476428429972177`, 
      0.07918324240275285}, {-1.7469727005068327`, 
      0.0744790847893689}, {-1.746280000829339, 
      0.06977396550043403}, {-1.7455647300349106`, 
      0.06506794437537537}, {-1.7448268742081658`, 
      0.06036108137752745}, {-1.7440664194485318`, 
      0.055653436595746085`}, {-1.7432833518706188`, 
      0.05094507024602943}, {-1.742477657604607, 
      0.046236042673146314`}, {-1.7416493227966428`, 
      0.04152641435227192}, {-1.740798333609247, 
      0.03681624589063082}, {-1.739924676221735, 
      0.0321055980291475}, {-1.7390283368306476`, 
      0.027394531644104474`}, {-1.7381093016501934`, 
      0.022683107748807867`}, {-1.737167556912703, 
      0.01797138749526079}, {-1.7362030888690958`, 
      0.013259432175844386`}, {-1.7352158837893579`, 
      0.008547303225006686}, {-1.7342059279630317`, 
      0.0038350622209593335`}, {-1.73317320769972, -0.0008772291126177565}, \
{-1.7321177093295992`, -0.005589508904863749}, {-1.7310394192039484`, \
-0.01030171513601617}, {-1.7299383236956885`, -0.015013785635682054`}, \
{-1.7288144091999353`, -0.01972565808110091}, {-1.7276676621345668`, \
-0.024437269995399417`}, {-1.7264980689408005`, -0.0291485587458378}, \
{-1.7253056160837879`, -0.0338594615420478}, {-1.7240902900532187`, \
-0.03856991543426224}, {-1.7228520773639417`, -0.04327985731153599}, \
{-1.7215909645565979`, -0.04798922389995844}, {-1.7203069381982672`, \
-0.052697951760857266`}, {-1.7189999848831312`, -0.05740597728899352}, \
{-1.7176700912331482`, -0.062113236710747935`}, {-1.7163172438987435`, \
-0.06681966608229842}, {-1.714941429559515, -0.07152520128778861}, \
{-1.7135426349249523`, -0.07622977803748755}, {-1.712120846735173, \
-0.08093333186594029}, {-1.7106760517616708`, -0.08563579813010941}, \
{-1.7092082368080832`, -0.09033711200750748}, {-1.7077173887109713`, \
-0.09503720849432022}, {-1.7062034943406177`, -0.09973602240352053}, \
{-1.7046665406018386`, -0.1044334883629731}, {-1.703106514434814, \
-0.10912954081352975`}, {-1.7015234028159334`, -0.11382411400711528`}, \
{-1.6999171927586574`, -0.11851714200480387`}, {-1.6982878713143976`, \
-0.12320855867488596`}, {-1.696635425573413, -0.12789829769092556`}, \
{-1.6949598426657235`, -0.13258629252980783`}, {-1.6932611097620407`, \
-0.13727247646977717`}, {-1.6915392140747172`, -0.14195678258846528`}, \
{-1.6897941428587138`, -0.14663914376090978`}, {-1.6880258834125845`, \
-0.15131949265756262`}, {-1.68623442307948, -0.15599776174228894`}, \
{-1.6844197492481698`, -0.16067388327035578`}, {-1.6825818493540852`, \
-0.16534778928641092`}, {-1.6807207108803774`, -0.17001941162245157`}, \
{-1.6788363213589996`, -0.17468868189578324`}, {-1.676928668371806, \
-0.17935553150696817`}, {-1.674997739551672, -0.18401989163776397`}, \
{-1.6730435225836333`, -0.18868169324905176`}, {-1.671066005206048, \
-0.19334086707875425`}, {-1.669065175211777, -0.1979973436397436}, \
{-1.6670410204493877`, -0.20265105321773882`}, {-1.6649935288243765`, \
-0.207301925869193}, {-1.6629226883004156`, -0.2119498914191701}, \
{-1.6608284869006205`, -0.21659487945921133`}, {-1.6587109127088393`, \
-0.22123681934519127`}, {-1.6565699538709662`, -0.2258756401951633}, \
{-1.654405598596276, -0.23051127088719475`}, {-1.6522178351587835`, \
-0.23514364005719152`}, {-1.6500066518986247`, -0.23977267609671218`}, \
{-1.6477720372234634`, -0.24439830715077152`}, {-1.6455139796099216`, \
-0.24902046111563358`}, {-1.6432324676050334`, -0.25363906563659416`}, \
{-1.6409274898277249`, -0.2582540481057527}, {-1.6385990349703188`, \
-0.2628653356597736}, {-1.6362470918000647`, -0.267472855177637}, \
{-1.6338716491606946`, -0.272076533278379}, {-1.6314726959740062`, \
-0.2766762963188211}, {-1.629050221241471, -0.2812720703912895}, \
{-1.6266042140458703`, -0.285863781321323}, {-1.6241346635529583`, \
-0.2904513546653714}, {-1.6216415590131539`, -0.2950347157084825}, \
{-1.6191248897632582`, -0.29961378946197886`}, {-1.6165846452282036`, \
-0.3041885006611242}, {-1.6140208149228288`, -0.3087587737627787}, \
{-1.6114333884536858`, -0.3133245329430448}, {-1.608822355520874, \
-0.3178857020949014}, {-1.6061877059199068`, -0.3224422048258286}, \
{-1.6035294295436067`, -0.3269939644554213}, {-1.6008475163840328`, \
-0.3315409040129931}, {-1.5981419565344392`, -0.3360829462351694}, \
{-1.595412740191265, -0.34062001356347044`}, {-1.5926598576561566`, \
-0.3451520281418841}, {-1.5898832993380227`, -0.3496789118144286}, \
{-1.5870830557551232`, -0.3542005861227049}, {-1.5842591175371903`, \
-0.35871697230343913`}, {-1.5814114754275852`, -0.3632279912860155}, \
{-1.578540120285488, -0.3677335636899986}, {-1.5756450430881241`, \
-0.37223360982264625`}, {-1.5727262349330247`, -0.37672804967641277`}, \
{-1.5697836870403237`, -0.38121680292644217`}, {-1.5668173907550922`, \
-0.38569978892805235`}, {-1.563827337549709, -0.3901769267142093}, \
{-1.560813519026268, -0.3946481349929924}, {-1.5577759269190257`, \
-0.3991133321450502}, {-1.5547145530968856`, -0.4035724362210472}, \
{-1.5516293895659217`, -0.4080253649391018}, {-1.5485204284719432`, \
-0.41247203568221497`}, {-1.5453876621030977`, -0.4169123654956907}, \
{-1.5422310828925165`, -0.4213462710845476}, {-1.5390506834209998`, \
-0.425773668810922}, {-1.5358464564197454`, -0.43019447469146316`}, \
{-1.5326183947731187`, -0.4346086043947199}, {-1.5293664915214666`, \
-0.4390159732385196}, {-1.5260907398639743`, -0.4434164961873396}, \
{-1.5227911331615667`, -0.44781008784967047`}, {-1.519467664939855, \
-0.4521966624753728}, {-1.5161203288921286`, -0.45657613395302593`}, \
{-1.5127491188823918`, -0.4609484158072703}, {-1.5093540289484504`, \
-0.4653134211961431}, {-1.5059350533050422`, -0.4696710629084071}, \
{-1.5024921863470173`, -0.4740212533608736}, {-1.4990254226525672`, \
-0.47836390459571904`}, {-1.4955347569865027`, -0.48269892827779637`}, \
{-1.4920201843035819`, -0.4870262356919405}, {-1.4884816997518888`, \
-0.49134573774026874`}, {-1.4849192986762636`, -0.4956573449394767}, \
{-1.4813329766217849`, -0.499960967418129}, {-1.4777227293373045`, \
-0.5042565149139462}, {-1.4740885527790362`, -0.5085438967710876}, \
{-1.4704304431141977`, -0.5128230219374307}, {-1.4667483967247088`, \
-0.5170937989618469}, {-1.463042410210944, -0.5213561359914752}, \
{-1.459312480395543, -0.5256099407689923}, {-1.4555586043272764`, \
-0.5298551206298814}, {-1.4517807792849706`, -0.5340915824996991}, \
{-1.4479790027814916`, -0.5383192328913411}, {-1.444153272567787, \
-0.5425379779023065}, {-1.4403035866369898`, -0.5467477232119626}, \
{-1.4364299432285812`, -0.5509483740788089}, {-1.4325323408326174`, \
-0.5551398353377422}, {-1.4286107781940174`, -0.5593220113973224}, \
{-1.424665254316915, -0.56349480623704}, {-1.4206957684690749`, \
-0.5676581234045852}, {-1.4167023201863738`, -0.5718118660131207}, \
{-1.412684909277347, -0.5759559367385562}, {-1.4086435358278033`, \
-0.5800902378168274}, {-1.4045782002055052`, -0.5842146710411799}, \
{-1.4004889030649195`, -0.5883291377594562}, {-1.3963756453520362`, \
-0.5924335388713903}, {-1.3922384283092584`, -0.5965277748259074}, \
{-1.388077253480362, -0.6006117456184302}, {-1.3838921227155294`, \
-0.604685350788194}, {-1.3796830381764538`, -0.6087484894155683}, \
{-1.375450002341519, -0.6128010601193892}, {-1.3711930180110532`, \
-0.6168429610543004}, {-1.3669120883126582`, -0.6208740899081049}, \
{-1.362607216706616, -0.6248943438991285}, {-1.3582784069913723`, \
-0.6289036197735947}, {-1.3539256633090972`, -0.6329018138030132}, \
{-1.3495489901513282`, -0.6368888217815822}, {-1.3451483923646907`, \
-0.6408645390236049}, {-1.340723875156701, -0.644828860360922}, \
{-1.336275444101651, -0.6487816801403615}, {-1.3318031051465773`, \
-0.6527228922212046}, {-1.3273068646173125`, -0.6566523899726715}, \
{-1.3227867292246231`, -0.660570066271426}, {-1.318242706070433, \
-0.6644758134991012}, {-1.3136748026541338`, -0.6683695235398467}, \
{-1.3090830268789841`, -0.6722510877778985}, {-1.3044673870585963`, \
-0.6761203970951737}, {-1.299827891923515, -0.6799773418688888}, \
{-1.2951645506278855`, -0.683821811969206}, {-1.2904773727562144`, \
-0.6876536967569061}, {-1.2857663683302236`, -0.6914728850810906}, \
{-1.2810315478157985`, -0.6952792652769139}, {-1.2762729221300302`, \
-0.6990727251633466}, {-1.2714905026483547`, -0.7028531520409728}, \
{-1.2666843012117892`, -0.7066204326898199}, {-1.2618543301342655`, \
-0.7103744533672262}, {-1.2570006022100635`, -0.7141150998057435}, \
{-1.2521231307213438`, -0.7178422572110802}, {-1.2472219294457823`, \
-0.7215558102600835}, {-1.2422970126643058`, -0.7252556430987642}, \
{-1.237348395168932, -0.728941639340364}, {-1.2323760922707114`, \
-0.7326136820634691}, {-1.227380119807777, -0.7362716538101698}, \
{-1.222360494153497, -0.7399154365842691}, {-1.2173172322247368`, \
-0.7435449118495411}, {-1.212250351490228, -0.7471599605280428}, \
{-1.2071598699790462`, -0.7507604629984788}, {-1.202045806289198, \
-0.7543462990946225}, {-1.1969081795963206`, -0.7579173481037956}, \
{-1.1917470096624911`, -0.7614734887654072}, {-1.18656231684515, \
-0.7650145992695551}, {-1.181354122106137, -0.7685405572556914}, \
{-1.1761224470208418`, -0.7720512398113545}, {-1.1708673137874706`, \
-0.7755465234709696}, {-1.1655887452364275`, -0.77902628421472}, \
{-1.1602867648398147`, -0.7824903974674915}, {-1.1549613967210486`, \
-0.7859387380978927}, {-1.1496126656645962`, -0.7893711804173527}, \
{-1.1442405971258305`, -0.7927875981792994}, {-1.1388452172410048`, \
-0.7961878645784214}, {-1.1334265528373506`, -0.7995718522500137}, \
{-1.1279846314432946`, -0.8029394332694137}, {-1.1225194812988002`, \
-0.8062904791515265}, {-1.11703113136583, -0.809624860850444}, \
{-1.1115196113389345`, -0.8129424487591612}, {-1.105984951655963, \
-0.8162431127093907}, {-1.1004271835089006`, -0.8195267219714798}, \
{-1.0948463388548306`, -0.8227931452544333}, {-1.0892424504270226`, \
-0.8260422507060433}, {-1.0836155517461472`, -0.8292739059131315}, \
{-1.0779656771316182`, -0.8324879779019052}, {-1.0722928617130614`, \
-0.8356843331384309}, {-1.0665971414419113`, -0.8388628375292296}, \
{-1.0608785531031368`, -0.842023356421996}, {-1.0551371343270937`, \
-0.8451657546064448}, {-1.049372923601507, -0.8482898963152898}, \
{-1.0435859602835817`, -0.851395645225356}, {-1.037776284612242, \
-0.8544828644588314}, {-1.0319439377205, -0.8575514165846598}, \
{-1.026088961647953, -0.8606011636200808}, {-1.0202113993534099`, \
-0.8636319670323176}, {-1.014311294727647, -0.8666436877404199}, \
{-1.0083886926062908`, -0.8696361861172633}, {-1.002443638782832, \
-0.8726093219917105}, {-0.9964761800217642, -0.8755629546509376}, \
{-0.9904863640718534, -0.8784969428429307}, {-0.9844742396795331, \
-0.8814111447791558}, {-0.9784398566024266, -0.8843054181374073}, \
{-0.9723832656229958, -0.8871796200648382}, {-0.9663045185623153, \
-0.8900336071811784}, {-0.9602036682939706, -0.8928672355821435}, \
{-0.9540807687580816, -0.8956803608430395}, {-0.9479358749754473, \
-0.8984728380225684}, {-0.9417690430618139, -0.9012445216668392}, \
{-0.9355803302422624, -0.9039952658135884}, {-0.9293697948657165, \
-0.9067249239966149}, {-0.923137496419568, -0.9094333492504358}, \
{-0.9168834955444194, -0.912120394115166}, {-0.9106078540489407, \
-0.9147859106416266}, {-0.9043106349248408, -0.9174297503966896}, \
{-0.897991902361949, -0.9200517644688608}, {-0.8916517217634077, \
-0.9226518034741089}, {-0.8852901597609707, -0.9252297175619433}, \
{-0.878907284230409, -0.9277853564217489}, {-0.8725031643070176, \
-0.9303185692893804}, {-0.8660778704012244, -0.9328292049540234}, \
{-0.8596314742142962, -0.9353171117653273}, {-0.8531640487541399, \
-0.9377821376408143}, {-0.8466756683511961, -0.9402241300735725}, \
{-0.8401664086744223, -0.942642936140236}, {-0.8336363467473609, \
-0.9450384025092591}, {-0.8270855609642912, -0.947410375449491}, \
{-0.8205141311064595, -0.9497587008390543}, {-0.8139221383583847, \
-0.9520832241745353}, {-0.8073096653242349, -0.9543837905804907}, \
{-0.8006767960442724, -0.9566602448192773}, {-0.7940236160113606, \
-0.9589124313012091}, {-0.7873502121875299, -0.9611401940950495}, \
{-0.7806566730205974, -0.9633433769388426}, {-0.773943088460836, \
-0.9655218232510916}, {-0.7672095499776872, -0.9676753761422873}, \
{-0.7604561505765125, -0.9698038784267954}, {-0.7536829848153784, \
-0.971907172635107}, {-0.7468901488218692, -0.9739851010264579}, \
{-0.7400777403099206, -0.9760375056018239}, {-0.7332458585966699, \
-0.9780642281172962}, {-0.726394604619315, -0.9800651100978441}, \
{-0.719524080951976, -0.9820399928514704}, {-0.7126343918225522, \
-0.9839887174837643}, {-0.705725643129569, -0.9859111249128594}, \
{-0.6987979424590052, -0.9878070558848003}, {-0.6918513991010945, \
-0.9896763509893252}, {-0.6848861240670935, -0.9915188506760685}, \
{-0.6779022301060073, -0.9933343952711903}, {-0.6708998317212648, \
-0.995122824994437}, {-0.663879045187336, -0.9968839799766392}, \
{-0.6568399885662806, -0.9986177002776511}, {-0.6497827817242207, \
-1.0003238259047371`}, {-0.642707546347727, -1.0020021968314117`}, \
{-0.6356144059601092, -1.003652653016734}, {-0.6285034859376012, \
-1.0052750344250663`}, {-0.6213749135254301, -1.0068691810462977`}, \
{-0.6142288178537592, -1.008434932916539}, {-0.6070653299534937, \
-1.0099721301392912`}, {-0.5998845827719389, -1.0114806129070957`}, \
{-0.5926867111882984, -1.0129602215236644`}, {-0.5854718520290022, \
-1.0144107964264988`}, {-0.5782401440828513, -1.0158321782099986`}, \
{-0.5709917281159681, -1.0172242076490636`}, {-0.5637267468865392, \
-1.0185867257231922`}, {-0.5564453451593384, -1.0199195736410795`}, \
{-0.5491476697200165, -1.021222592865718}, {-0.5418338693891449, \
-1.0224956251400006`}, {-0.5345040950360003, -1.0237385125128327`}, \
{-0.5271584995920747, -1.02495109736575}, {-0.5197972380642989, \
-1.0261332224400477`}, {-0.5124204675479633, -1.027284730864418}, \
{-0.5050283472393233, -1.028405466183102}, {-0.49762103844787203`, \
-1.0294952723845503`}, {-0.4901987046082679, -1.0305539939305983`}, \
{-0.48276151129190015`, -1.031581475786151}, {-0.47530962621807654`, \
-1.0325775634493801`}, {-0.467843219264819, -1.03354210298243}, \
{-0.46036246247924917`, -1.034474941042633}, {-0.4528675300875494, \
-1.03537592491423}, {-0.44535859850448145`, -1.0362449025405962`}, \
{-0.4378358463424473, -1.0370817225569673`}, {-0.43029945442007417`, \
-1.0378862343236637`}, {-0.4227496057703076, -1.038658287959809}, \
{-0.4151864856479954, -1.0393977343775382`}, {-0.4076102815369441, \
-1.0401044253166911`}, {-0.4000211831564324, -1.0407782133799843`}, \
{-0.3924193824671618, -1.0414189520686585`}, {-0.38480507367662836`, \
-1.0420264958185916`}, {-0.3771784532438974, -1.0426007000368736`}, \
{-0.36953971988376283`, -1.043141421138832}, {-0.3618890745702735, \
-1.0436485165855043`}, {-0.35422672053960913`, -1.0441218449215444`}, \
{-0.346552863292287, -1.0445612658135555`}, {-0.33886771059468185`, \
-1.044966640088839}, {-0.33117147247984086`, -1.0453378297745508`}, \
{-0.3234643612475764, -1.0456746981372482`}, {-0.3157465914638175, \
-1.0459771097228228`}, {-0.30801837995920367`, -1.0462449303967998`}, \
{-0.3002799458269021, -1.0464780273849952`}, {-0.2925315104196317, \
-1.0466762693145146`}, {-0.2847732973458761, -1.0468395262550791`}, \
{-0.2770055324652682, -1.046967669760663}, {-0.26922844388313033`, \
-1.0470605729114277`}, {-0.2614422619441516, -1.0471181103559344`}, \
{-0.2536472192251876, -1.0471401583536177`}, {-0.24584355052716525`, \
-1.047126594817503}, {-0.23803149286607692`, -1.0470772993571467`}, \
{-0.23021128546304906`, -1.0469921533217825`}, {-0.22238316973346942`, \
-1.04687103984365}, {-0.21454738927515832`, -1.0467138438814874`}, \
{-0.20670418985556988`, -1.046520452264164}, {-0.19885381939800914`, \
-1.0462907537344317`}, {-0.190996527966852, -1.046024638992772}, \
{-0.18313256775175474`, -1.045722000741314}, {-0.17526219305084165`, \
-1.0453827337277974`}, {-0.16738566025285825`, -1.0450067347895604`}, \
{-0.15950322781827994`, -1.044593902897519}, {-0.15161515625936517`, \
-1.0441441392001183`}, {-0.14372170811914387`, -1.0436573470672226`}, \
{-0.13582314794933212`, -1.0431334321339218`}, {-0.1279197422871649, \
-1.0425723023442213`}, {-0.12001175963113968`, -1.0419738679945885`}, \
{-0.1120994704156642, -1.041338041777327}, {-0.10418314698460308`, \
-1.0406647388237464`}, {-0.09626306356371817, -1.0399538767470977`}, \
{-0.08833949623199894, -1.0392053756852453`}, {-0.08041272289188019, \
-1.0384191583430409`}, {-0.0724830232383448, -1.0375951500343692`}, \
{-0.0645506787269109, -1.0367332787238333`}, {-0.056615972540503545`, \
-1.0358334750680454`}, {-0.04867918955521182, -1.0348956724564917`}, \
{-0.04074061630493399, -1.0339198070519366`}, {-0.03280054094491382, \
-1.0329058178303339`}, {-0.02485925321417254, -1.0318536466202097`}, \
{-0.016917044396842253`, -1.0307632381414855`}, {-0.008974207282407373, \
-1.029634540043704}, {-0.0010310361248622429`, -1.0284675029436259`}, {
      0.00691217339920599, -1.0272620804621635`}, {
      0.014855124233598822`, -1.0260182292606153`}, {
      0.022797517986540537`, -1.024735909076168}, {
      0.03073905497575222, -1.0234150827566335`}, {
      0.03867943427455153, -1.0220557162943833`}, {
      0.04661835375896289, -1.020657778859451}, {
      0.05455551015582148, -1.019221242831766}, {
      0.06249059909185326, -1.0177460838324852`}, {
      0.07042331514371176, -1.0162322807543942`}, {
      0.07835335188895146, -1.0146798157913386`}, {
      0.08628040195791592, -1.0130886744666596`}, {
      0.09420415708651814, -1.0114588456605988`}, {
      0.10212430816988871`, -1.0097903216366424`}, {
      0.11004054531686669`, -1.008083098066774}, {
      0.11795255790530666`, -1.0063371740556082`}, {
      0.12586003463817425`, -1.0045525521633725`}, {
      0.1337626636004012, -1.0027292384277122`}, {
      0.1416601323164703, -1.0008672423842908`}, {
      0.14955212780869853`, -0.9989665770861569}, {
      0.15743833665618698`, -0.9970272591218555}, {
      0.16531844505440338`, -0.9950493086322564}, {
      0.17319213887536378`, -0.9930327493260767}, {
      0.1810591037283777, -0.9909776084940762}, {
      0.18891902502132052`, -0.9888839170219029}, {
      0.1967715880223961, -0.9867517094015683}, {
      0.20461647792235174`, -0.9845810237415346}, {
      0.21245337989710614`, -0.9823719017753945}, {
      0.2202819791707515, -0.9801243888691288}, {
      0.22810196107888886`, -0.9778385340269249}, {
      0.2359130111322557, -0.9755143898955434}, {
      0.24371481508060455`, -0.9731520127672195}, {
      0.25150705897678993`, -0.9707514625810895}, {
      0.25928942924102116`, -0.9683128029231309}, {
      0.2670616127252378, -0.9658361010246099}, {
      0.27482329677756406`, -0.9633214277590278}, {
      0.2825741693067983, -0.9607688576375628}, {
      0.29031391884689334`, -0.9581784688030023}, {
      0.2980422346213833, -0.9555503430221645}, {
      0.3057588066077123, -0.9528845656768082}, {
      0.3134633256014203, -0.9501812257530327}, {
      0.32115548328014143`, -0.9474404158291686}, {
      0.3288349722673701, -0.9446622320621668}, {
      0.3365014861959506, -0.9418467741724877}, {
      0.34415471977124557`, -0.9389941454275013}, {
      0.351794368833939, -0.9361044526234048}, {
      0.3594201304224305, -0.9331778060656702}, {
      0.3670317028347769, -0.9302143195480315}, {
      0.374628785690138, -0.9272141103300268}, {
      0.38221107998968484`, -0.9241772991131108}, {
      0.38977828817692695`, -0.921104010015352}, {
      0.39733011419741876`, -0.9179943705447351}, {
      0.40486626355780336`, -0.914848511571086}, {
      0.4123864433841542, -0.9116665672966423}, {
      0.41989036247957523`, -0.9084486752252892}, {
      0.4273777313810213, -0.9051949761304866}, {
      0.43484826241530145`, -0.9019056140219113}, {
      0.4423016697542281, -0.8985807361108394}, {
      0.4497376694688773, -0.895220492774299}, {
      0.45715597958292514`, -0.8918250375180186}, {
      0.4645563201250273, -0.8883945269382028}, {
      0.4719384131802088, -0.8849291206821664}, {
      0.47930198294023396`, -0.8814289814078573}, {
      0.48664675575292576`, -0.8778942747423009}, {
      0.4939724601704065, -0.8743251692390017}, {
      0.5012788269962326, -0.870721836334335}, {
      0.5085655893313968, -0.8670844503029635}, {
      0.5158324826191741, -0.8634131882123177}, {
      0.5230792446887872, -0.8597082298761732}, {
      0.5303056157978701, -0.8559697578073658}, {
      0.5375113386737093, -0.8521979571696806}, {
      0.5446961585532433, -0.8483930157289536}, {
      0.5518598232218028, -0.8445551238034265}, {
      0.5590020830505755, -0.8406844742133915}, {
      0.5661226910327802, -0.8367812622301695}, {
      0.573221402818538, -0.8328456855244573}, {
      0.5802979767484271, -0.8288779441140888}, {
      0.5873521738857133, -0.8248782403112468}, {
      0.5943837580472434, -0.8208467786691684}, {
      0.6013924958329986, -0.8167837659283836}, {
      0.6083781566542972, -0.8126894109625277}, {
      0.615340512760646, -0.8085639247237681}, {
      0.6222793392652344, -0.8044075201878871}, {
      0.62919441416907, -0.8002204122990578}, {
      0.6360855183837564, -0.7960028179143573}, {
      0.6429524357529122, -0.7917549557480533}, {
      0.6497949530722347, -0.7874770463157054}, {
      0.6566128601082113, -0.7831693118781194}, {
      0.6634059496154835, -0.778831976385194}, {
      0.6701740173528691, -0.7744652654196958}, {
      0.6769168620980508, -0.7700694061410038}, {
      0.6836342856609386, -0.7656446272288561}, {
      0.6903260928957158, -0.7611911588271386}, {
      0.6969920917115796, -0.7567092324877506}, {
      0.7036320930821883, -0.7521990811145802}, {
      0.7102459110538267, -0.7476609389076279}, {
      0.7168333627523051, -0.7430950413073075}, {
      0.7233942683886051, -0.7385016249389615}, {
      0.7299284512632894, -0.7338809275576196}, {
      0.7364357377696913, -0.729233187993033}, {
      0.7429159573959007, -0.7245586460950159}, {
      0.7493689427255668, -0.7198575426791206}, {
      0.7557945294375347, -0.7151301194726791}, {
      0.762192556304336, -0.7103766190612348}, {
      0.7685628651895544, -0.7055972848353937}, {
      0.7749053010440867, -0.7007923609381204}, {
      0.7812197119013211, -0.6959620922125025}, {
      0.7875059488712545, -0.6911067241500101}, {
      0.7937638661335722, -0.6862265028392699}, {
      0.7999933209297129, -0.6813216749153792}, {
      0.8061941735539413, -0.6763924875097785}, {
      0.8123662873434546, -0.6714391882007044}, {
      0.818509528667545, -0.6664620249642399}, {
      0.8246237669158438, -0.6614612461259834}, {
      0.8307088744856718, -0.6564371003133499}, {
      0.8367647267685208, -0.6513898364085231}, {
      0.8427912021356919, -0.6463197035020731}, {
      0.8487881819231156, -0.6412269508472532}, {
      0.8547555504153788, -0.6361118278149901}, {
      0.8606931948289858, -0.630974583849579}, {
      0.8666010052948768, -0.6258154684250965}, {
      0.8724788748402318, -0.6206347310025394}, {
      0.8783266993695831, -0.6154326209877025}, {
      0.8841443776452645, -0.6102093876898005}, {
      0.8899318112672209, -0.6049652802808451}, {
      0.8956889046522036, -0.5997005477557817}, {
      0.9014155650123787, -0.5944154388933948}, {
      0.907111702333371, -0.5891102022179837}, {
      0.9127772293517699, -0.583785085961818}, {
      0.9184120615321214, -0.5784403380283718}, {
      0.9240161170434317, -0.5730762059563435}, {
      0.9295893167352046, -0.5676929368844609}, {
      0.9351315841130391, -0.5622907775170753}, {
      0.9406428453138086, -0.5568699740905443}, {
      0.9461230290804467, -0.551430772340403}, {
      0.9515720667363617, -0.5459734174693257}, {
      0.9569898921595034, -0.5404981541158737}, {
      0.9623764417561024, -0.5350052263240301}, {
      0.9677316544341074, -0.529494877513519}, {
      0.9730554715763381, -0.5239673504509045}, {
      0.9783478370133785, -0.5184228872214688}, {
      0.983608696996228, -0.5128617292018633}, {
      0.9888380001687336, -0.5072841170335298}, {
      0.9940356975398214, -0.5016902905968859}, {
      0.999201742455547, -0.4960804889862703}, {
      1.004336090570985, -0.4904549504856415}, {
      1.0094386998219749`, -0.48481391254502404`}, {
      1.0145095303967417`, -0.47915761175769533`}, {
      1.0195485447074109`, -0.4734862838381071}, {
      1.0245557073614318`, -0.46780016360053317`}, {
      1.0295309851329286`, -0.46209948493843644`}, {
      1.0344743469339954`, -0.4563844808045472}, {
      1.0393857637859492`, -0.4506553831916442}, {
      1.0442652087905582`, -0.4449124231140306}, {
      1.0491126571012597`, -0.43915583058969515`}, {
      1.053928085894382, -0.4333858346231507}, {
      1.0587114743403847`, -0.42760266318894}, {
      1.0634628035751306`, -0.42180654321579963`}, {
      1.0681820566712026`, -0.4159977005714728}, {
      1.0728692186092785`, -0.4101763600481606}, {
      1.0775242762495758`, -0.40434274534860276`}, {
      1.082147218303378, -0.3984970790727766}, {
      1.0867380353046538`, -0.3926395827052054}, {
      1.0912967195817818`, -0.3867704766028646}, {
      1.095823265229388, -0.38088997998367674`}, {
      1.10031766808031, -0.37499831091558344`}, {
      1.1047799256776936`, -0.369095686306185}, {
      1.1092100372472355`, -0.36318232189293626`}, {
      1.1136080036695768`, -0.35725843223388865`}, {
      1.1179738274528586`, -0.35132423069896757`}, {
      1.1223075127054465`, -0.3453799294617743}, {
      1.1266090651088327`, -0.33942573949190225`}, {
      1.1308784918907218`, -0.3334618705477561}, {
      1.1351158017983078`, -0.32748853116986437`}, {
      1.13932100507175, -0.32150592867467354`}, {
      1.1434941134178522`, -0.315514269148814}, {
      1.147635139983953, -0.3095137574438269}, {
      1.151744099332031, -0.3035045971713416}, {
      1.1558210074130308`, -0.29748699069869294`}, {
      1.1598658815414173`, -0.2914611391449685}, {
      1.163878740369957, -0.2854272423774751}, {
      1.1678596038647375`, -0.2793854990086141}, {
      1.1718084932804242`, -0.2733361063931564}, {
      1.1757254311357603`, -0.26727926062590585`}, {
      1.1796104411893134`, -0.2612151565397415}, {
      1.1834635484154703`, -0.2551439877040294}, {
      1.1872847789806862`, -0.24906594642339358`}, {
      1.1910741602199866`, -0.2429812237368363}, {
      1.1948317206137284`, -0.23689000941719912`}, {
      1.1985574897646203`, -0.23079249197095406`}, {
      1.202251498375004, -0.22468885863831656`}, {
      1.2059137782243996`, -0.21857929539367046`}, {
      1.2095443621473165`, -0.2124639869462962}, {
      1.2131432840113283`, -0.2063431167413932}, {
      1.2167105786954169`, -0.20021686696138769`}, {
      1.2202462820685847`, -0.19408541852751734`}, {
      1.2237504309687346`, -0.18794895110168408`}, {
      1.2272230631818202`, -0.18180764308856667`}, {
      1.2306642174212656`, -0.175661671637985}, {
      1.2340739333076558`, -0.16951121264750763`}, {
      1.2374522513486956`, -0.163356440765295}, {
      1.2407992129194405`, -0.1571975293931702}, {
      1.2441148602427954`, -0.1510346506899095}, {
      1.2473992363702833`, -0.14486797557474557`}, {
      1.2506523851630829`, -0.1386976737310753}, {
      1.2538743512733335`, -0.13252391361036558`}, {
      1.2570651801257078`, -0.12634686243624949`}, {
      1.2602249178992504`, -0.12016668620880598`}, {
      1.2633536115094819`, -0.11398354970901646`}, {
      1.2664513085907663`, -0.10779761650339117`}, {
      1.2695180574789424`, -0.10160904894875913`}, {
      1.2725539071942156`, -0.09541800819721499}, {
      1.2755589074243103`, -0.08922465420121663}, {
      1.2785331085078804`, -0.0830291457188273}, {
      1.2814765614181767`, -0.07683164031909634}, {
      1.2843893177469703`, -0.07063229438757251}, {
      1.287271429688728, -0.06443126313194428}, {
      1.2901229500250402`, -0.05822870058780134}, {
      1.2929439321092981`, -0.052024759624512064`}, {
      1.2957344298516191`, -0.045819591951211346`}, {
      1.2984944977040171`, -0.039613348122893736`}, {
      1.3012241906458168`, -0.03340617754660672}, {
      1.3039235641693099`, -0.02719822848773923}, {
      1.306592674265649, -0.020989648076400472`}, {
      1.309231577410981, -0.01478058231388443}, {
      1.3118403305528135`, -0.008571176079215384}, {
      1.3144189910966133`, -0.0023615731357700172`}, {1.3169676168926372`, 
      0.0038480838620282938`}, {1.319486266222989, 0.010057653361947208`}, {
      1.3219749977889017`, 0.01626699490711932}, {1.3244338706982441`, 
      0.022475969129156257`}, {1.3268629444532467`, 0.028684437741206846`}, {
      1.3292622789384463`, 0.03489226353096318}, {1.3316319344088456`, 
      0.041099310353618274`}, {1.3339719714782878`, 0.04730544312477884}, {
      1.3362824511080387`, 0.053510527813336775`}, {1.3385634345955801`, 
      0.05971443143430261}, {1.3408149835636067`, 0.06591702204160434}, {
      1.3430371599492268`, 0.07211816872085465}, {1.3452300259933636`, 
      0.07831774158208984}, {1.3473936442303545`, 0.08451561175248326}, {
      1.3495280774777454`, 0.09071165136903617}, {1.3516333888262781`, 
      0.09690573357124904}, {1.353709641630069, 0.10309773249377578`}, {
      1.3557568994969733`, 0.10928752325906359`}, {1.3577752262791374`, 
      0.11547498196998107`}, {1.359764686063732, 0.12165998570243708`}, {
      1.3617253431638663`, 0.12784241249799252`}, {1.3636572621096792`, 
      0.13402214135646764`}, {1.365560507639607, 0.14019905222854687`}, {
      1.3674351446918225`, 0.14637302600838342`}, {1.3692812383958457`, 
      0.15254394452620595`}, {1.3710988540643207`, 0.15871169054092873`}, {
      1.37288805718496, 0.164876147732768}, {1.3746489134126494`, 
      0.17103720069586578`}, {1.3763814885617154`, 0.1771947349309232}, {
      1.3780858485983496`, 0.18334863683784516`}, {1.3797620596331894`, 
      0.18949879370839792`}, {1.3814101879140512`, 0.19564509371888097`}, {
      1.383030299818816, 0.20178742592281526`}, {1.384622461848462, 
      0.2079256802436488}, {1.3861867406202448`, 0.21405974746748144`}, {
      1.3877232028610216`, 0.22018951923580982`}, {1.3892319154007162`, 
      0.2263148880382942}, {1.390712945165925, 0.23243574720554822`}, {
      1.3921663591736586`, 0.2385519909019528}, {1.3935922245252204`, 
      0.24466351411849552`}, {1.3949906084002168`, 0.25077021266563626`}, {
      1.3963615780506984`, 0.2568719831662007}, {1.3977052007954305`, 
      0.26296872304830204`}, {1.3990215440142892`, 0.2690603305382927}, {
      1.4003106751427827`, 0.27514670465374613`}, {1.4015726616666948`, 
      0.2812277451964701}, {1.402807571116848, 0.2873033527455523}, {
      1.4040154710639858`, 0.293373428650439}, {1.4051964291137717`, 
      0.29943787502404745`}, {1.4063505129019005`, 0.3054965947359126}, {
      1.4074777900893243`, 0.3115494914053695}, {1.408578328357588, 
      0.31759646939477126`}, {1.4096521954042727`, 0.3236374338027435}, {
      1.4106994589385473`, 0.3296722904574763}, {1.4117201866768234`, 
      0.3357009459100532}, {1.4127144463385146`, 0.3417233074278188}, {
      1.4136823056418957`, 0.34773928298778506`}, {1.4146238323000624`, 
      0.35374878127007625`}, {1.4155390940169885`, 0.35975171165141423`}, {
      1.416428158483678, 0.3657479841986429}, {1.4172910933744132`, 
      0.37173750966229363`}, {1.4181279663430941`, 0.377720199470191}, {
      1.418938845019669, 0.3836959657210997}, {1.4197237970066545`, 
      0.38966472117841294`}, {1.4204828898757436`, 0.39562637926388206`}, {
      1.4212161911644987`, 0.4015808540513883}, {1.4219237683731305`, 
      0.40752806026075666`}, {1.4226056889613587`, 0.41346791325161175`}, {
      1.4232620203453552`, 0.4194003290172766}, {1.4238928298947666`, 
      0.42532522417871377`}, {1.4244981849298153`, 0.43124251597850943`}, {
      1.4250781527184777`, 0.4371521222749002}, {1.4256328004737384`, 
      0.44305396153584314`}, {1.4261621953509183`, 0.44894795283312905`}, {
      1.426666404445075, 0.45483401583653865`}, {1.4271454947884759`, 
      0.46071207080804233`}, {1.4275995333481406`, 0.4665820385960429}, {
      1.428028587023452, 0.4724438406296621}, {1.4284327226438356`, 
      0.4782973989130701}, {1.4288120069665045`, 0.4841426360198586}, {
      1.4291665066742698`, 0.489979475087457}, {1.4294962883734146`, 
      0.4958078398115923}, {1.4298014185916301`, 0.5016276544407922}, {
      1.430081963776014, 0.5074388437709306}, {1.4303379902911282`, 
      0.5132413331398179}, {1.4305695644171161`, 0.5190350484218328}, {
      1.430776752347878, 0.524819916022598}, {1.4309596201893018`, 
      0.5305958628736986}, {1.4311182339575514`, 0.5363628164274432}, {
      1.4312526595774089`, 0.5421207046516678}, {1.4313629628806694`, 
      0.5478694560245825}, {1.4314492096045897`, 0.5536089995296599}, {
      1.4315114653903878`, 0.5593392646505669}, {1.4315497957817926`, 
      0.5650601813661377}, {1.4315642662236447`, 0.5707716801453889}, {
      1.4315549420605427`, 0.5764736919425774}, {1.43152188853554, 
      0.5821661481922984}, {1.4314651707888872`, 0.5878489808046263}, {
      1.4313848538568192`, 0.5935221221602961}, {1.4312810026703884`, 
      0.599185505105926}, {1.4311536820543416`, 0.6048390629492815}, {
      1.4310029567260392`, 0.6104827294545788}, {1.4308288912944174`, 
      0.6161164388378308}, {1.4306315502589912`, 0.6217401257622315}, {
      1.4304109980088984`, 0.627353725333581}, {1.4301672988219825`, 
      0.6329571730957516}, {1.4299005168639156`, 0.6385504050261914}, {
      1.4296107161873584`, 0.6441333575314693}, {1.4292979607311578`, 
      0.6497059674428575}, {1.4289623143195818`, 0.6552681720119544}, {
      1.4286038406615895`, 0.6608199089063452}, {1.428222603350136, 
      0.6663611162053015}, {1.4278186658615126`, 0.6718917323955185}, {
      1.4273920915547205`, 0.6774116963668911}, {1.4269429436708763`, 
      0.682920947408327}, {1.4264712853326516`, 0.6884194252035972}, {
      1.4259771795437435`, 0.6939070698272244}, {1.425460689188376, 
      0.6993838217404077}, {1.424921877030832, 0.7048496217869842}, {
      1.424360805715015, 0.7103044111894272}, {1.4237775377640396`, 
      0.7157481315448805}, {1.423172135579852, 0.7211807248212283}, {
      1.4225446614428763`, 0.7266021333532012}, {1.4218951775116901`, 
      0.7320122998385175}, {1.4212237458227264`, 0.7374111673340591}, {
      1.42053042829, 0.7427986792520834}, {1.4198152867048632`, 
      0.7481747793564685}, {1.4190783827357831`, 0.7535394117589933}, {
      1.4183197779281462`, 0.7588925209156518}, {1.4175395337040853`, 
      0.7642340516230013}, {1.4167377113623307`, 0.7695639490145431}, {
      1.415914372078086, 0.7748821585571378}, {1.4150695769029233`, 
      0.780188626047453}, {1.4142033867647044`, 0.785483297608443}, {
      1.4133158624675202`, 0.7907661196858624}, {1.412407064691654, 
      0.7960370390448104}, {1.4114770539935646`, 0.8012960027663082}, {
      1.4105258908058893`, 0.8065429582439072}, {1.4095536354374676`, 
      0.8117778531803288}, {1.4085603480733844`, 0.8170006355841357}, {
      1.4075460887750315`, 0.8222112537664334}, {1.4065109174801893`, 
      0.8274096563376027}, {1.4054548940031248`, 0.832595792204062}, {
      1.4043780780347097`, 0.8377696105650606}, {1.4032805291425539`, 
      0.8429310609095008}, {1.402162306771158, 0.8480800930127897}, {
      1.4010234702420816`, 0.853216656933721}, {1.399864078754128, 
      0.8583407030113843}, {1.3986841913835462`, 0.8634521818621049}, {
      1.3974838670842458`, 0.8685510443764107}, {1.3962631646880312`, 
      0.8736372417160279}, {1.3950221429048477`, 0.8787107253109044}, {
      1.3937608603230442`, 0.8837714468562615}, {1.3924793754096492`, 
      0.8888193583096721}, {1.3911777465106612`, 0.8938544118881667}, {
      1.389856031851354, 0.8988765600653664}, {1.3885142895365936`, 
      0.9038857555686421}, {1.3871525775511706`, 0.9088819513763008}, {
      1.3857709537601433`, 0.9138651007147973}, {1.3843694759091947`, 
      0.9188351570559726}, {1.382948201625002, 0.9237920741143175}, {
      1.3815071884156183`, 0.928735805844262}, {1.3800464936708647`, 
      0.9336663064374895}, {1.3785661746627365`, 0.9385835303202765}, {
      1.3770662885458178`, 0.9434874321508567}, {1.3755468923577108`, 
      0.9483779668168095}, {1.3740080430194723`, 0.953255089432473}, {
      1.3724497973360632`, 0.9581187553363809}, {1.3708722119968082`, 
      0.9629689200887237}, {1.3692753435758644`, 0.9678055394688324}, {
      1.3676592485327015`, 0.9726285694726866}, {1.3660239832125907`, 
      0.9774379663104452}, {1.364369603847103, 0.9822336864040001}, {
      1.3626961665546182`, 0.9870156863845524}, {1.3610037273408417`, 
      0.9917839230902112}, {1.3592923420993304`, 0.9965383535636151}, {
      1.3575620666120278`, 1.0012789350495745`}, {1.3558129565498083`, 
      1.006005624992737}, {1.3540450674730276`, 1.0107183810352731`}, {
      1.352258454832084, 1.015417161014585}, {1.3504531739679861`, 
      1.0201019229610344`}, {1.348629280112929, 1.024772625095693}, {
      1.3467868283908775`, 1.0294292258281128`}, {1.344925873818158, 
      1.0340716837541175`}, {1.3430464713040566`, 1.0386999576536133`}, {
      1.341148675651425, 1.0433140064884212`}, {1.3392325415572939`, 
      1.0479137894001276`}, {1.3372981236134915`, 1.0524992657079564`}, {
      1.3353454763072719`, 1.0570703949066584`}, {1.3333746540219464`, 
      1.061627136664423}, {1.3313857110375247`, 1.0661694508208057`}, {
      1.3293787015313598`, 1.0706972973846771`}, {1.3273536795788012`, 
      1.0752106365321892`}, {1.3253106991538528`, 1.0797094286047613`}, {
      1.3232498141298372`, 1.0841936341070835`}, {1.321171078280066, 
      1.0886632137051382`}, {1.319074545278516, 1.0931181282242408`}, {
      1.31696026870051, 1.0975583386470968`}, {1.3148283020234048`, 
      1.1019838061118772`}, {1.312678698627283, 1.1063944919103117`}, {
      1.3105115117956507`, 1.110790357485799}, {1.308326794716141, 
      1.1151713644315326`}, {1.3061246004812215`, 1.1195374744886473`}, {
      1.3039049820889075`, 1.1238886495443778`}, {1.3016679924434797`, 
      1.1282248516302378`}, {1.2994136843562074`, 1.132546042920213}, {
      1.297142110546075, 1.136852185728972}, {1.2948533236405144`, 
      1.1411432425100916`}, {1.2925473761761421`, 1.1454191758543004`}, {
      1.2902243205994994`, 1.1496799484877356`}, {1.2878842092677978`, 
      1.1539255232702168`}, {1.285527094449669, 1.1581558631935354`}, {
      1.283153028325919, 1.1623709313797588`}, {1.2807620629902845`, 
      1.1665706910795495`}, {1.2783542504501968`, 1.1707551056705}, {
      1.275929642627547, 1.1749241386554816`}, {1.2734882913594558`, 
      1.179077753661009}, {1.2710302483990474`, 1.183215914435618}, {
      1.2685555654162275`, 1.1873385848482587`}, {1.2660642939984639`, 
      1.1914457288867029`}, {1.263556485651572, 1.1955373106559646`}, {
      1.2610321918005034`, 1.1996132943767357`}, {1.258491463790138, 
      1.2036736443838345`}, {1.2559343528860796`, 1.207718325124669}, {
      1.2533609102754548`, 1.211747301157712}, {1.2507711870677163`, 
      1.2157605371509916`}, {1.248165234295448, 1.2197579978805937`}, {
      1.2455431029151747`, 1.2237396482291771`}, {1.2429048438081742`, 
      1.227705453184504}, {1.2402505077812933`, 1.2316553778379802`}, {
      1.2375801455677664`, 1.2355893873832104`}, {1.2348938078280376`, 
      1.2395074471145657`}, {1.2321915451505858`, 1.243409522425762}, {
      1.229473408052753, 1.2472955788084528`}, {1.2267394469815753`, 
      1.2511655818508332`}, {1.223989712314617, 1.2550194972362563`}, {
      1.221224254360808, 1.258857290741861}, {1.2184431233612836`, 
      1.2626789282372126`}, {1.2156463694902275`, 1.266484375682955}, {
      1.2128340428557183`, 1.2702735991294727`}, {1.2100061935005768`, 
      1.2740465647155679`}, {1.207162871403219, 1.2778032386671452`}, {
      1.2043041264785095`, 1.2815435872959107`}, {1.2014300085786187`, 
      1.2852675769980797`}, {1.198540567493883, 1.2889751742530977`}, {
      1.195635852953667, 1.2926663456223706`}, {1.1927159146272293`, 
      1.2963410577480072`}, {1.1897808021245895`, 1.2999992773515712`}, {
      1.1868305649974, 1.3036409712328443`}, {1.1838652527398188`, 
      1.3072661062686008`}, {1.1808849147893852`, 1.310874649411391}, {
      1.1778896005278991`, 1.3144665676883358`}, {1.1748793592823021`, 
      1.318041828199932}, {1.1718542403255607`, 1.3216003981188666`}, {
      1.1688142928775538`, 1.3251422446888423`}, {1.1657595661059612`, 
      1.328667335223412}, {1.1626901091271558`, 1.3321756371048235`}, {
      1.159605971007097, 1.3356671177828747`}, {1.1565072007622286`, 
      1.3391417447737772`}, {1.153393847360378, 1.3425994856590302`}, {
      1.1502659597216578`, 1.3460403080843037`}, {1.1471235867193708`, 
      1.3494641797583318`}, {1.1439667771809172`, 1.3528710684518135`}, {
      1.1407955798887048`, 1.3562609419963254`}, {1.1376100435810608`, 
      1.359633768283241}, {1.1344102169531467`, 1.3629895152626599`}, {
      1.1311961486578765`, 1.3663281509423477`}, {1.1279678873068366`, 
      1.369649643386682}, {1.1247254814712082`, 1.372953960715609}, {
      1.1214689796826933`, 1.3762410711036077`}, {1.1181984304344423`, 
      1.379510942778665}, {1.1149138821819846`, 1.382763544021256}, {
      1.111615383344162, 1.385998843163336}, {1.1083029823040647`, 
      1.3892168085873389`}, {1.1049767274099693`, 1.3924174087251844`}, {
      1.1016366669762812`, 1.3956006120572944`}, {1.098282849284477, 
      1.3987663871116163`}, {1.0949153225840518`, 1.4019147024626555`}, {
      1.0915341350934689`, 1.4050455267305155`}, {1.0881393350011113`, 
      1.4081588285799458`}, {1.0847309704662362`, 1.411254576719398}, {
      1.0813090896199335`, 1.4143327399000902`}, {1.0778737405660848`, 
      1.4173932869150787`}, {1.0744249713823273`, 1.420436186598337}, {
      1.0709628301210197`, 1.4234614078238437`}, {1.0674873648102103`, 
      1.4264689195046767`}, {1.0639986234546095`, 1.4294586905921165`}, {
      1.0604966540365632`, 1.4324306900747554`}, {1.056981504517031, 
      1.4353848869776153`}, {1.0534532228365656`, 1.4383212503612721`}, {
      1.049911856916297, 1.441239749320988}, {1.0463574546589178`, 
      1.4441403529858512`}, {1.0427900639496726`, 1.4470230305179217`}, {
      1.0392097326573504`, 1.449887751111386}, {1.0356165086352798`, 
      1.4527344839917171`}, {1.0320104397223266`, 1.4555631984148436`}, {
      1.0283915737438962`, 1.4583738636663235`}, {1.0247599585129374`, 
      1.4611664490605267`}, {1.0211156418309504`, 1.4639409239398242`}, {
      1.0174586714889973`, 1.4666972576737831`}, {1.0137890952687165`, 
      1.4694354196583699`}, {1.0101069609433404`, 1.4721553793151594`}, {
      1.006412316278715, 1.4748571060905509`}, {1.002705209034325, 
      1.477540569454991}, {0.9989856869643194, 1.4802057389022032`}, {
      0.9952537978185445, 1.4828525839484237`}, {0.991509589343576, 
      1.485481074131644}, {0.9877531092837576, 1.4880911790108606`}, {
      0.9839844053822415, 1.4906828681653301`}, {0.9802035253820334, 
      1.4932561111938312`}, {0.9764105170270402, 1.4958108777139338`}, {
      0.972605428063122, 1.4983471373612738`}, {0.9687883062391476, 
      1.5008648597888337`}, {0.9649591993080532, 1.503364014666231}, {
      0.9611181550279054, 1.505844571679012}, {0.9572652211629679, 
      1.5083065005279515`}, {0.9534004454847713, 1.510749770928359}, {
      0.9495238757731882, 1.5131743526093924`}, {0.9456355598175104, 
      1.5155802153133753`}, {0.9417355454175312, 1.5179673287951228`}, {
      0.9378238803846322, 1.520335662821272}, {0.9339006125428717, 
      1.5226851871696199`}, {0.9299657897300806, 1.525015871628465}, {
      0.9260194597989593, 1.5273276859959577`}, {0.9220616706181806, 
      1.5296206000794557`}, {0.9180924700734963, 1.5318945836948838`}, {
      0.9141119060688478, 1.5341496066661027`}, {0.9101200265274818, 
      1.536385638824281}, {0.9061168793930688, 1.5386026500072747`}, {
      0.9021025126308282, 1.5408006100590121`}, {0.8980769742286555, 
      1.542979488828885}, {0.8940403121982562, 1.5451392561711448`}, {
      0.8899925745762822, 1.5472798819443059`}, {0.8859338094254748, 
      1.549401336010554}, {0.8818640648358106, 1.5515035882351607`}, {
      0.8777833889256532, 1.5535866084859027`}, {0.8736918298429096, 
      1.5556503666324895`}, {0.8695894357661906, 1.557694832545994}, {
      0.8654762549059772, 1.559719976098291}, {0.861352335505791, 
      1.5617257671615}, {0.8572177258433704, 1.563712175607435}, {
      0.853072474231851, 1.56567917130706}, {0.8489166290209517, 
      1.5676267241299486`}, {0.8447502385981658, 1.5695548039437526`}, {
      0.8405733513899574, 1.5714633806136724`}, {0.8363860158629628, 
      1.5733524240019372`}, {0.8321882805251977, 1.5752219039672877`}, {
      0.8279801939272692, 1.5770717903644664`}, {0.8237618046635937, 
      1.578902053043714}, {0.8195331613736206, 1.5807126618502696`}, {
      0.8152943127430606, 1.5825035866238788`}, {0.8110453075051207, 
      1.5842747971983069`}, {0.8067861944417442, 1.5860262634008568`}, {
      0.8025170223848569, 1.5877579550518952`}, {0.7982378402176191, 
      1.589469841964382}, {0.7939486968756834, 1.5911618939434071`}, {
      0.7896496413484578, 1.5928340807857329`}, {0.7853407226803768, 
      1.5944863722793419`}, {0.781021989972176, 1.5961187382029913`}, {
      0.7766934923821752, 1.597731148325773}, {0.7723552791275663, 
      1.599323572406679}, {0.7680073994857078, 1.6008959801941733`}, {
      0.7636499027954265, 1.60244834142577}, {0.7592828384583244, 
      1.6039806258276166`}, {0.7549062559400931, 1.6054928031140843`}, {
      0.7505202047718341, 1.6069848429873632`}, {0.7461247345513866, 
      1.6084567151370643`}, {0.7417198949446612, 1.6099083892398278`}, {
      0.7373057356869808, 1.6113398349589367`}, {0.7328823065844285, 
      1.6127510219439367`}, {0.7284496575152025, 1.614141919830263}, {
      0.7240078384309776, 1.6155124982388722`}, {0.719556899358274, 
      1.6168627267758822`}, {0.715096890399834, 1.6181925750322155`}, {
      0.7106278617360048, 1.6195020125832515`}, {0.7061498636261295, 
      1.620791008988484}, {0.7016629464099453, 1.622059533791185}, {
      0.6971671605089892, 1.6233075565180748`}, {0.692662556428011, 
      1.6245350466789992`}, {0.6881491847563947, 1.6257419737666126`}, {
      0.6836270961695872, 1.6269283072560679`}, {0.679096341430534, 
      1.6280940166047129`}, {0.6745569713911249, 1.629239071251793}, {
      0.6700090369936458, 1.6303634406181617`}, {0.665452589272239, 
      1.6314670941059963`}, {0.660887679354373, 1.632550001098521}, {
      0.656314358462318, 1.6336121309597376`}, {0.6517326779146326, 
      1.634653453034162}, {0.6471426891276569, 1.6356739366465682`}, {
      0.6425444436170149, 1.636673551101739}, {0.6379379929991251, 
      1.6376522656842236`}, {0.6333233889927209, 1.6386100496581038`}, {
      0.628700683420378, 1.6395468722667648`}, {0.6240699282100524, 
      1.6404627027326755`}, {0.6194311753966262, 1.641357510257175}, {
      0.6147844771234631, 1.6422312640202668`}, {0.6101298856439724, 
      1.6430839331804206`}, {0.6054674533231834, 1.643915486874381}, {
      0.6007972326393277, 1.6447258942169847`}, {0.5961192761854318, 
      1.6455151243009845`}, {0.5914336366709192, 1.6462831461968817`}, {
      0.5867403669232218, 1.647029928952766}, {0.5820395198894014, 
      1.6477554415941633`}, {0.5773311486377802, 1.6484596531238922`}, {
      0.5726153063595826, 1.6491425325219269`}, {0.5678920463705853, 
      1.64980404874527}, {0.5631614221127791, 1.6504441707278334`}, {
      0.5584234871560397, 1.6510628673803258`}, {0.5536782951998096, 
      1.651660107590151}, {0.5489259000747897, 1.6522358602213127`}, {
      0.5441663557446422, 1.6527900941143303`}, {0.5393997163077034, 
      1.6533227780861603`}, {0.5346260359987076, 1.6538338809301294`}, {
      0.5298453691905213, 1.654323371415875}, {0.5250577703958892, 
      1.6547912182892952`}, {0.5202632942691905, 1.655237390272508}, {
      0.5154619956082064, 1.6556618560638197`}, {0.510653929355899, 
      1.6560645843377024`}, {0.5058391506022014, 1.6564455437447816`}, {
      0.5010177145858193, 1.6568047029118331`}, {0.49618967669604425`, 
      1.6571420304417894`}, {0.49135509247457804`, 1.6574574949137564`}, {
      0.4865140176173696, 1.65775106488304}, {0.48166650797646315`, 
      1.6580227088811827`}, {0.47681261956185855`, 1.6582723954160103`}, {
      0.4719524085433838, 1.6585000929716893`}, {0.46708593125257936`, 
      1.658705770008795}, {0.4622132441845954, 1.6588893949643897`}, {
      0.4573344040001009, 1.659050936252112}, {0.4524494675272053, 
      1.6591903622622775`}, {0.44755849176339313`, 1.6593076413619896`}, {
      0.4426615338774712, 1.6594027418952626`}, {0.43775865121152846`, 
      1.6594756321831554`}, {0.43284990128290907`, 1.6595262805239173`}, {
      0.42793534178619846`, 1.6595546551931457`}, {0.42301503059522255`, 
      1.6595607244439547`}, {0.4180890257650603, 1.6595444565071569`}, {
      0.4131573855340696, 1.659505819591457}, {0.40822016832592695`, 
      1.6594447818836573`}, {0.40327743275168043`, 1.659361311548877}, {
      0.3983292376118167, 1.6592553767307827`}, {0.39337564189834173`, 
      1.6591269455518336`}, {0.38841670479687557`, 1.6589759861135378`}, {
      0.3834524856887613, 1.6588024664967242`}, {0.378483044153188, 
      1.6586063547618262`}, {0.3735084399693284, 1.6583876189491793`}, {
      0.36852873311849066`, 1.658146227079334}, {0.36354398378628466`, 
      1.6578821471533807`}, {0.3585542523648036, 1.6575953471532912`}, {
      0.3535595994548196, 1.6572857950422728`}, {0.3485600858679947, 
      1.6569534587651384`}, {0.3435557726291069, 1.656598306248691}, {
      0.33854672097829125`, 1.6562203054021238`}, {0.333532992373296, 
      1.655819424117435}, {0.32851464849175455`, 1.6553956302698596`}, {
      0.32349175123347285`, 1.6549488917183157`}, {0.318464362722732, 
      1.654479176305868}, {0.31343254531060705`, 1.653986451860207}, {
      0.30839636157730155`, 1.653470686194145}, {0.30335587433449795`, 
      1.6529318471061298`}, {0.29831114662772423`, 1.6523699023807739`}, {
      0.2932622417387366, 1.651784819789403}, {0.28820922318791864`, 
      1.6511765670906207`}, {0.28315215473669686`, 1.6505451120308925`}, {
      0.27809110038997276`, 1.6498904223451465`}, {0.27302612439857166`, 
      1.649212465757394}, {0.26795729126170814`, 1.6485112099813684`}, {
      0.26288466572946856`, 1.6477866227211826`}, {0.2578083128053105, 
      1.6470386716720076`}, {0.25272829774857936`, 1.6462673245207682`}, {
      0.247644686077042, 1.6454725489468613`}, {0.24255754356943804`, 
      1.644654312622893}, {0.23746693626804846`, 1.6438125832154362`}, {
      0.23237293048128163`, 1.6429473283858107`}, {0.22727559278627726`, 
      1.642058515790883}, {0.22217499003152796`, 1.6411461130838882`}, {
      0.2170711893395187, 1.640210087915275}, {0.2119642581093843, 
      1.6392504079335712`}, {0.20685426401958507`, 1.6382670407862732`}, {
      0.20174127503060044`, 1.637259954120758}, {0.19662535938764114`, 
      1.636229115585219}, {0.1915065856233796, 1.6351744928296252`}, {
      0.18638502256069894`, 1.6340960535067046`}, {0.18126073931546044`, 
      1.6329937652729527`}, {0.17613380529928988`, 1.6318675957896651`}, {
      0.17100429022238248`, 1.630717512723996}, {0.16587226409632683`, 
      1.629543483750042}, {0.1607377972369477, 1.6283454765499534`}, {
      0.15560096026716805`, 1.62712345881507}, {0.15046182411988998`, 
      1.625877398247085}, {0.14532046004089502`, 1.6246072625592367`}, {
      0.14017693959176375`, 1.6233130194775274`}, {0.1350313346528147, 
      1.6219946367419709`}, {0.12988371742606283`, 1.6206520821078687`}, {
      0.12473416043819752`, 1.6192853233471154`}, {0.11958273654358001`, 
      1.6178943282495337`}, {0.11442951892726078`, 1.6164790646242402`}, {
      0.10927458110801644`, 1.6150395003010405`}, {0.10411799694140662`, 
      1.6135756031318569`}, {0.09895984062285058, 1.6120873409921872`}, {
      0.09380018669072379, 1.6105746817825954`}, {0.08863911002947451, 
      1.6090375934302354`}, {0.0834766858727604, 1.6074760438904072`}, {
      0.07831298980660517, 1.605890001148148}, {0.07314809777257536, 
      1.604279433219857}, {0.06798208607097728, 1.6026443081549546`}, {
      0.06281503136407414, 1.6009845940375769`}, {0.0576470106793234, 
      1.5993002589883076`}, {0.05247810141263434, 1.5975912711659448`}, {
      0.047308381331645936`, 1.5958575987693053`}, {0.04213792857902497, 
      1.594099210039067}, {0.03696682167578449, 1.592316073259649}, {
      0.03179513952462252, 1.5905081567611317`}, {0.026622961413281094`, 
      1.5886754289212146`}, {0.02145036701792559, 1.5868178581672163`}, {
      0.016277436406544325`, 1.584935412978114}, {0.011104250042368457`, 
      1.583028061886624}, {0.005930888787312095, 1.581095773481326}, {
      0.0007574339054326586, 1.5791385164088294`}, {-0.0044160329335885965`, 
      1.5771562593759818`}, {-0.009589429650945898, 
      1.5751489711521236`}, {-0.014762673755188194`, 
      1.5731166205713862`}, {-0.019935682338678345`, 
      1.5710591765350366`}, {-0.02510837207403234, 
      1.568976608013867}, {-0.030280659210538616`, 
      1.5668688840506333`}, {-0.035452459570557544`, 
      1.5647359737625395`}, {-0.0406236885459012, 
      1.5625778463437712`}, {-0.04579426109419343, 
      1.5603944710680797`}, {-0.05096409173521048, 
      1.5581858172914138`}, {-0.056133094547202105`, 
      1.5559518544546043`}, {-0.061301183163193426`, 
      1.5536925520860991`}, {-0.06646827076726759, 
      1.5514078798047517`}, {-0.07163427009082945, 
      1.549097807322662}, {-0.07679909340885031, 
      1.5467623044480716`}, {-0.08196265253609397, 
      1.5444013410883144`}, {-0.08712485882332427, 
      1.5420148872528219`}, {-0.09228562315349424, 
      1.5396029130561866`}, {-0.0974448559379171, 
      1.5371653887212822`}, {-0.1026024671124193, 
      1.5347022845824418`}, {-0.10775836613347585`, 
      1.5322135710886968`}, {-0.11291246197432817`, 
      1.5296992188070744`}, {-0.11806466312108459`, 
      1.5271591984259576`}, {-0.12321487756880398`, 
      1.5245934807585064`}, {-0.1283630128175626, 
      1.5220020367461433`}, {-0.13350897586850452`, 
      1.519384837462101}, {-0.13865267321987584`, 
      1.5167418541150375`}, {-0.1437940108630433, 
      1.5140730580527144`}, {-0.14893289427849712`, 
      1.5113784207657452`}, {-0.15406922843183896`, 
      1.5086579138914098`}, {-0.15920291776975495`, 
      1.5059115092175375`}, {-0.16433386621597437`, 
      1.5031391786864625`}, {-0.1694619771672144, 
      1.5003408943990486`}, {-0.17458715348911116`, 
      1.4975166286187862`}, {-0.17970929751213774`, 
      1.4946663537759641`}, {-0.18482831102750943`, 
      1.491790042471914}, {-0.18994409528307685`, 
      1.4888876674833313`}, {-0.19505655097920718`, 
      1.4859592017666732`}, {-0.20016557826465436`, 
      1.483004618462633}, {-0.2052710767324185, 
      1.4800238909006949`}, {-0.21037294541559518`, 
      1.4770169926037675`}, {-0.2154710827832151, 
      1.4739838972929}, {-0.22056538673607504`, 
      1.4709245788920795`}, {-0.22565575460256015`, 
      1.4678390115331126`}, {-0.23074208313445888`, 
      1.4647271695605917`}, {-0.23582426850277055`, 
      1.4615890275369479`}, {-0.24090220629350695`, 
      1.4584245602475898`}, {-0.245975791503488, 
      1.4552337427061326`}, {-0.2510449185361328, 
      1.4520165501597155`}, {-0.2561094811972466, 
      1.4487729580944113`}, {-0.26116937269080415`, 
      1.4455029422407275`}, {-0.26622448561473117`, 
      1.4422064785792017`}, {-0.2712747119566836, 
      1.4388835433460914`}, {-0.2763199430898268, 
      1.4355341130391606`}, {-0.28136006976861433`, 
      1.4321581644235637`}, {-0.2863949821245683, 
      1.428755674537829}, {-0.2914245696620619, 
      1.4253266206999406`}, {-0.2964487212541049, 
      1.4218709805135252`}, {-0.3014673251381331, 
      1.4183887318741382`}, {-0.30648026891180413`, 
      1.4148798529756574`}, {-0.31148743952879837`, 
      1.4113443223167808`}, {-0.31648872329462824`, 
      1.4077821187076316`}, {-0.3214840058624561, 
      1.4041932212764734`}, {-0.3264731722289222, 
      1.4005776094765332`}, {-0.33145610672998355`, 
      1.396935263092938}, {-0.336432693036766, 
      1.3932661622497626`}, {-0.3414028141514296, 
      1.3895702874171934`}, {-0.34636635240304986`, 
      1.385847619418806}, {-0.3513231894435153, 
      1.3820981394389633`}, {-0.3562732062434435, 
      1.378321829030328}, {-0.3612162830881168, 
      1.3745186701214998`}, {-0.3661522995734395, 
      1.3706886450247704`}, {-0.3710811346019176, 
      1.3668317364440052`}, {-0.37600266637866375`, 
      1.362947927482647}, {-0.38091677240742783`, 
      1.3590372016518468`}, {-0.38582332948665615`, 
      1.3550995428787231`}, {-0.3907222137055803, 
      1.351134935514749}, {-0.3956133004403376, 
      1.3471433643442707`}, {-0.4004964643501254, 
      1.3431248145931582`}, {-0.40537157937339113`, 
      1.3390792719375892`}, {-0.41023851872405936`, 
      1.3350067225129685`}, {-0.41509715488779947`, 
      1.3309071529229841`}, {-0.4199473596183344, 
      1.3267805502488013`}, {-0.4247890039337939, 
      1.3226269020583958`}, {-0.42962195811311377`, 
      1.3184461964160294`}, {-0.43444609169248394`, 
      1.3142384218918668`}, {-0.43926127346184746`, 
      1.3100035675717367`}, {-0.44406737146145264`, 
      1.3057416230670393`}, {-0.4488642529784615, 
      1.3014525785247995`}, {-0.4536517845436163, 
      1.2971364246378694`}, {-0.45842983192796766`, 
      1.29279315265528}, {-0.46319826013966575`, 
      1.288422754392745}, {-0.4679569334208187, 
      1.2840252222433153`}, {-0.47270571524441984`, 
      1.2796005491881899`}, {-0.4774444683113479, 
      1.2751487288076806`}, {-0.4821730545474417, 
      1.2706697552923336`}, {-0.4868913351006535, 
      1.2661636234542093`}, {-0.4915991703382841, 
      1.2616303287383215`}, {-0.49629641984430173`, 
      1.2570698672342369`}, {-0.5009829424167496, 
      1.2524822356878362`}, {-0.5056585960652443, 
      1.2478674315132396`}, {-0.5103232380085692, 
      1.243225452804894}, {-0.5149767246723649, 
      1.2385562983498284`}, {-0.5196189116869233, 
      1.2338599676400732`}, {-0.5242496538850853, 
      1.2291364608852493`}, {-0.5288688053002485, 
      1.2243857790253245`}, {-0.533476219164488, 
      1.219607923743539}, {-0.5380717479067932, 
      1.2148028974795038`}, {-0.5426552431514271, 
      1.2099707034424674`}, {-0.5472265557164085, 
      1.2051113456247575`}, {-0.5517855356121251, 
      1.2002248288153945`}, {-0.5563320320400789, 
      1.195311158613879}, {-0.5608658933917698, 
      1.190370341444154}, {-0.5653869672477216, 
      1.1854023845687434`}, {-0.569895100376654, 
      1.1804072961030638`}, {-0.5743901387348065, 
      1.1753850850299155`}, {-0.5788719274654177, 
      1.1703357612141476`}, {-0.5833403108983664, 
      1.1652593354175027`}, {-0.5877951325499772, 
      1.160155819313637}, {-0.5922362351229977, 
      1.1550252255033189`}, {-0.5966634605067509, 
      1.1498675675298058`}, {-0.6010766497774684, 
      1.1446828598943977`}, {-0.6054756431988105, 
      1.139471118072169}, {-0.6098602802225768, 
      1.1342323585278784`}, {-0.6142303994896142, 
      1.1289665987320547`}, {-0.6185858388309279, 
      1.1236738571772618`}, {-0.6229264352689992, 
      1.1183541533945383`}, {-0.6272520250193183, 
      1.1130075079700141`}, {-0.6315624434921363, 
      1.1076339425617028`}, {-0.6358575252944422, 
      1.1022334799164673`}, {-0.6401371042321721, 
      1.09680614388716}, {-0.6444010133126558, 
      1.0913519594499357`}, {-0.6486490847473068, 
      1.085870952721736}, {-0.6528811499545626, 
      1.0803631509779432`}, {-0.6570970395630815, 
      1.0748285826702038`}, {-0.6612965834152016, 
      1.0692672774444183`}, {-0.6654796105706693, 
      1.063679266158897}, {-0.669645949310644, 
      1.0580645809026794`}, {-0.6737954271419851, 
      1.0524232550140145`}, {-0.6779278708018279, 
      1.0467553230990008`}, {-0.6820431062624562, 
      1.0410608210503831`}, {-0.6861409587364776, 
      1.0353397860665032`}, {-0.6902212526823089, 
      1.0295922566704026`}, {-0.6942838118099783, 
      1.0238182727290739`}, {-0.698328459087252, 
      1.018017875472857}, {-0.702355016746092, 
      1.0121911075149783`}, {-0.7063633062894523, 
      1.006338012871228}, {-0.7103531484984211, 
      1.0004586369797726`}, {-0.714324363439716, 
      0.9945530267210977}, {-0.7182767704735395, 
      0.9886212304380791}, {-0.7222101882618037, 
      0.9826632979561748}, {-0.7261244347767282, 
      0.976679280603735}, {-0.7300193273098229, 
      0.9706692312324262}, {-0.7338946824812604, 
      0.9646332042377618}, {-0.7377503162496468, 
      0.9585712555797358}, {-0.7415860439221973, 
      0.9524834428035535}, {-0.7454016801653257, 
      0.9463698250604529}, {-0.7491970390156546, 
      0.9402304631286112}, {-0.7529719338914537, 
      0.9340654194341302}, {-0.7567261776045138, 
      0.9278747580720925}, {-0.7604595823724649, 
      0.9216585448276838}, {-0.7641719598315451, 
      0.9154168471973724}, {-0.7678631210498275, 
      0.9091497344101385}, {-0.771532876540914, 
      0.9028572774487468}, {-0.7751810362781021, 
      0.8965395490710523}, {-0.7788074097090326, 
      0.8901966238313327}, {-0.7824118057708253, 
      0.8838285781016376}, {-0.78599403290571, 
      0.8774354900931456}, {-0.7895538990771611, 
      0.8710174398775202}, {-0.7930912117865402, 
      0.8645745094082552}, {-0.7966057780902562, 
      0.8581067825419981}, {-0.8000974046174489, 
      0.8516143450598433}, {-0.8035658975882023, 
      0.8450972846885824}, {-0.8070110628322945, 
      0.8385556911219016}, {-0.8104327058084915, 
      0.8319896560415159}, {-0.8138306316243893, 
      0.825399273138225}, {-0.8172046450568127, 
      0.8187846381328839}, {-0.8205545505727743, 
      0.8121458487972707}, {-0.8238801523510013, 
      0.8054830049748425}, {-0.827181254304035, 
      0.798796208601365}, {-0.8304576601009074, 
      0.7920855637254023}, {-0.8337091731904014, 
      0.785351176528653}, {-0.8369355968248984, 
      0.7785931553461198}, {-0.8401367340848177, 
      0.771811610686096}, {-0.8433123879036526, 
      0.7650066552499564}, {-0.8464623610936066, 
      0.7581784039517354}, {-0.8495864563718334, 
      0.7513269739374789}, {-0.8526844763872837, 
      0.7444524846043522}, {-0.8557562237481623, 
      0.7375550576194887}, {-0.8588015010499973, 
      0.7306348169385636}, {-0.8618201109043244, 
      0.7236918888240746}, {-0.8648118559679868, 
      0.7167264018633134}, {-0.8677765389730536, 
      0.7097384869860094}, {-0.8707139627573561, 
      0.7027282774816296}, {-0.8736239302956436, 
      0.6956959090163156}, {-0.8765062447313575, 
      0.6886415196494392}, {-0.8793607094090253, 
      0.6815652498497586}, {-0.8821871279072703, 
      0.6744672425111565}, {-0.884985304072438, 
      0.6673476429679404}, {-0.8877550420528372, 
      0.6602065990096858}, {-0.8904961463335906, 
      0.6530442608956037}, {-0.8932084217720951, 
      0.6458607813684113}, {-0.8958916736340851, 
      0.6386563156676864}, {-0.8985457076302964, 
      0.6314310215426864}, {-0.901170329953725, 
      0.6241850592646093}, {-0.9037653473174745, 
      0.6169185916382788}, {-0.9063305669931867, 
      0.6096317840132303}, {-0.9088657968500486, 
      0.6023248042941792}, {-0.9113708453943671, 
      0.5949978229508498}, {-0.9138455218097037, 
      0.5876510130271433}, {-0.9162896359975606, 
      0.580284550149625}, {-0.9187029986186075, 
      0.5728986125353089}, {-0.9210854211344387, 
      0.5654933809987194}, {-0.9234367158498504, 
      0.5580690389582081}, {-0.9257566959556248, 
      0.5506257724415062}, {-0.9280451755718084, 
      0.54316377009049}, {-0.930301969791472, 
      0.5356832231651399}, {-0.9325268947249363, 
      0.5281843255466726}, {-0.9347197675444504, 
      0.5206672737398237}, {-0.936880406529304, 
      0.5131322668742637}, {-0.9390086311113596, 
      0.5055795067051247}, {-0.941104261920985, 
      0.49800919761261925`}, {-0.9431671208333692, 
      0.49042154660073123`}, {-0.9451970310152014, 
      0.48281676329496015`}, {-0.9471938169716932, 
      0.4751950599390993}, {-0.9491573045939243, 
      0.4675566513910294}, {-0.9510873212064878, 
      0.4599017551175105}, {-0.952983695615415, 
      0.45223059118795284`}, {-0.954846258156355, 
      0.444543382267151}, {-0.9566748407429858, 
      0.43684035360696427`}, {-0.9584692769156302, 
      0.42912173303692674`}, {-0.960229401890054, 
      0.4213877509537721}, {-0.9619550526064171, 
      0.41363864030985825`}, {-0.9636460677783517, 
      0.4058746366004772}, {-0.9653022879421395, 
      0.39809597785003725`}, {-0.9669235555059591, 
      0.3903029045971045}, {-0.9685097147991745, 
      0.38249565987829154`}, {-0.9700606121216336, 
      0.37467448921098245`}, {-0.9715760957929479, 
      0.3668396405748838}, {-0.9730560162017193, 
      0.358991364392392}, {-0.9745002258546851, 
      0.35112991350776873`}, {-0.9759085794257458, 
      0.34325554316511697`}, {-0.9772809338048447, 
      0.33536851098515047`}, {-0.9786171481466639, 
      0.32746907694075217`}, {-0.9799170839191035, 
      0.31955750333131605`}, {-0.9811806049515093, 
      0.31163405475586947`}, {-0.9824075774826136, 
      0.3036989980849739}, {-0.9835978702081538, 
      0.29575260243140233`}, {-0.9847513543281322, 
      0.28779513911959376`}, {-0.9858679035936829, 
      0.2798268816538858}, {-0.9869473943535069, 
      0.2718481056855278}, {-0.9879897055998401, 
      0.26385908897847815`}, {-0.9889947190139182, 
      0.2558601113739906}, {-0.9899623190108989, 
      0.24785145475399592`}, {-0.9908923927842075, 
      0.23983340300328582`}, {-0.9917848303492665, 
      0.2318062419705087}, {-0.9926395245865737, 
      0.2237702594279866}, {-0.9934563712840908, 
      0.21572574503036504`}, {-0.9942352691789063, 
      0.20767299027210845`}, {-0.9949761199981354, 
      0.19961228844385523`}, {-0.9956788284990217, 
      0.19154393458764785`}, {-0.9963433025082027, 
      0.1834682254510548}, {-0.9969694529601056, 
      0.17538545944020228`}, {-0.9975571939344356, 
      0.16729593657173555`}, {-0.9981064426927235, 
      0.15919995842373003`}, {-0.9986171197138977, 
      0.15109782808557476`}, {-0.9990891487288461, 
      0.14298985010685158`}, {-0.9995224567539363, 
      0.13487633044523437`}, {-0.9999169741234587, 
      0.12675757641343466`}, {-1.0002726345209643`, 
      0.11863389662522088`}, {-1.0005893750094625`, 
      0.11050560094053938`}, {-1.0008671360604526`, 
      0.10237300040976717`}, {-1.0011058615817552`, 
      0.09423640721712725}, {-1.0013054989441195`, 
      0.08609613462329835}, {-1.0014659990065762`, 
      0.07795249690725237}, {-1.001587316140511, 
      0.06980580930735368}, {-1.0016694082524338`, 
      0.06165638796175553}, {-1.0017122368054174`, 
      0.05350454984812973}, {-1.0017157668391867`, 
      0.045350612722767046`}, {-1.0016799669888325`, 
      0.03719489505908614}, {-1.0016048095021328`, 
      0.02903771598559029}, {-1.001490270255461, 
      0.020879395223311545`}, {-1.001336328768263, 
      0.012720253022782934`}, {-1.0011429682160893`, 
      0.004560610100580013}, {-1.0009101754421628`, -0.0035992124245263456`}, \
{-1.0006379409674744`, -0.011758893095763365`}, {-1.00032625899939, \
-0.019918110182852923`}, {-0.9999751274387598, -0.028076541746767728`}, \
{-0.9995845478855224, -0.0362338657047031}, {-0.999154525642794, \
-0.04438975989522011}, {-0.9986850697194383, -0.0525439021435154}, \
{-0.9981761928311131, -0.06069597032677288}, {-0.9976279113997896, \
-0.06884564243955232}, {-0.9970402455517435, -0.07699259665916965}, \
{-0.9964132191140205, -0.08513651141102396}, {-0.9957468596093747, \
-0.09327706543382605}, {-0.995041198249687, -0.10141393784468342`}, \
{-0.9942962699278658, -0.10954680820399706`}, {-0.9935121132082398, \
-0.11767535658012537`}, {-0.992688770315448, -0.12579926361377067`}, \
{-0.9918262871218394, -0.13391821058204478`}, {-0.9909247131333929, \
-0.14203187946216966`}, {-0.9899841014741699, -0.1501399529947705}, \
{-0.9890045088693148, -0.15824211474671837`}, {-0.9879859956266185, \
-0.1663380491734807}, {-0.9869286256166635, -0.1744274416809384}, \
{-0.9858324662515675, -0.18250997868662902`}, {-0.984697588462347, \
-0.19058534768037608`}, {-0.9835240666749221, -0.19865323728426584`}, \
{-0.9823119787847844, -0.2067133373119334}, {-0.9810614061303526, \
-0.21476533882712082`}, {-0.9797724334650415, -0.22280893420147144`}, \
{-0.9784451489280689, -0.23084381717152494`}, {-0.9770796440140297, \
-0.23886968289487925`}, {-0.9756760135412649, -0.24688622800548632`}, \
{-0.9742343556190546, -0.2548931506680498}, {-0.9727547716136663, \
-0.26289015063149407`}, {-0.9712373661132893, -0.2708769292814752}, \
{-0.9696822468918873, -0.2788531896919057}, {-0.9680895248720031, \
-0.2868186366754653}, {-0.9664593140865478, -0.2947729768330737}, \
{-0.9647917316396106, -0.3027159186022991}, {-0.9630868976663225, \
-0.31064717230468103`}, {-0.9613449352918112, -0.3185664501919444}, \
{-0.9595659705892824, -0.3264734664910858}, {-0.9577501325372644, \
-0.3343679374483117}, {-0.9558975529760528, -0.3422495813718125}, \
{-0.9540083665633939, -0.3501181186733544}, {-0.9520827107294422, \
-0.35797327190867595`}, {-0.950120725631032, -0.3658147658166746}, \
{-0.9481225541053008, -0.37364232735737213`}, {-0.9460883416227012, \
-0.3814556857486473}, {-0.9440182362394416, -0.38925457250172724`}, \
{-0.9419123885493929, -0.39703872145542907`}, {-0.9397709516354987, \
-0.40480786880914527`}}]}, 
   {RGBColor[1, 1, 0], DiskBox[{0, 0}, 0.03]}},
  AspectRatio->Automatic,
  Background->RGBColor[0.6, 0.6, 0.6],
  ImageMargins->{{0.5, 0.}, {1., 32.}},
  ImageSize->{331.5, Automatic}]], "Output",
 CellChangeTimes->{{3.401193439426774*^9, 3.401193454265441*^9}, 
   3.401193618597225*^9, 3.401194916518215*^9, 3.4012768874760227`*^9}]
}, Open  ]],

Cell[TextData[{
 "Now consider an inverse-cubed law using ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "=", "3"}], TraditionalForm]]],
 ".  Using ",
 Cell[BoxData[
  FormBox[
   RowBox[{"V0", "=", 
    RowBox[{"2", " ", "\[Pi]"}]}], TraditionalForm]]],
 " show that there is a circular orbit.  Choose ",
 Cell[BoxData[
  FormBox[
   RowBox[{"tFinal", "=", "2"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tFinal", "=", "2"}], ";", 
  RowBox[{"R0", "=", "1"}], ";", 
  RowBox[{"V0", "=", 
   RowBox[{"2", " ", "\[Pi]"}]}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", ".001"}], ";", 
  RowBox[{"p", "=", "3"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CalculateOrbit", ";", 
  RowBox[{"DisplayOrbit", "[", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Clear", "[", 
  RowBox[{
  "tFinal", ",", "R0", ",", "V0", ",", "iMaximum", ",", "t", ",", "r", ",", 
   "v"}], "]"}]}], "Input",
 CellChangeTimes->{{3.401193360004507*^9, 3.401193414015883*^9}, 
   3.40119347898011*^9, 3.401193626282501*^9, 3.4012012039931593`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 1, 1], Thickness[0.0075], 
    LineBox[{{1, 0}, {0.9999802608561359, 0.006283143965477338}, {
      0.9999210442038147, 0.012566039883189386`}, {0.9998223523808084, 
      0.01884843971516336}, {0.9996841892833013, 0.02513009544301109}, {
      0.9995065603657357, 0.03141075907772037}, {0.9992894726405973, 
      0.03769018266944512}, {0.9990329346781377, 0.043968118317294046`}, {
      0.9987369566060365, 0.050244318179117314`}, {0.998401550109001, 
      0.056518534481290975`}, {0.9980267284283055, 0.0627905195284986}, {
      0.9976125063612682, 0.06906002571350989}, {0.997158900260667, 
      0.07532680552695578}, {0.996665928034094, 0.08159061156709968}, {
      0.9961336091432487, 0.0878511965496045}, {0.9955619646031694, 
      0.09410831331729495}, {0.9949510169814036, 0.10036171484991499`}, {
      0.9943007903971175, 0.10661115427387963`}, {0.9936113105201432, 
      0.11285638487202117`}, {0.9928826045699656, 0.11909716009332916`}, {
      0.9921147013146479, 0.12533323356268378`}, {0.9913076310696958, 
      0.13156435909058234`}, {0.9904614256968606, 0.1377902906828584}, {
      0.9895761186028815, 0.14401078255039318`}, {0.9886517447381667, 
      0.150225589118819}, {0.9876883405954137, 0.15643446503821395`}, {
      0.9866859442081681, 0.16263716519278806`}, {0.9856445951493232, 
      0.16883344471055994`}, {0.9845643345295567, 0.17502305897302398`}, {
      0.9834452049957082, 0.18120576362480742`}, {0.9822872507290953, 
      0.1873813145833171}, {0.9810905174437698, 0.19354946804837542`}, {
      0.9798550523847127, 0.19970998051184516`}, {0.978580904325969, 
      0.20586260876724274`}, {0.9772681235687224, 0.21200710991933963`}, {
      0.9759167619393093, 0.21814324139375144`}, {0.9745268727871731, 
      0.2242707609465143}, {0.9730985109827575, 0.23038942667364826`}, {
      0.9716317329153408, 0.23649899702070726`}, {0.9701265964908096, 
      0.24259923079231527`}, {0.9685831611293727, 0.24868988716168824`}, {
      0.9670014877632155, 0.25477072568014164`}, {0.9653816388340941, 
      0.26084150628658276`}, {0.9637236782908707, 0.26690198931698816`}, {
      0.9620276715869885, 0.272951935513865}, {0.9602936856778883, 
      0.2789911060356966}, {0.9585217890183646, 0.2850192624663715}, {
      0.9567120515598636, 0.2910361668245956}, {0.9548645447477214, 
      0.2970415815732875}, {0.9529793415183436, 0.3030352696289559}, {
      0.9510565162963256, 0.30901699437105906`}, {0.9490961449915147, 
      0.31498651965134666`}, {0.9470983049960134, 0.32094360980318204`}, {
      0.945063075181124, 0.3268880296508462}, {0.9429905358942345, 
      0.332819544518822}, {0.9408807689556473, 0.33873792024105887`}, {
      0.9387338576553486, 0.34464292317021716`}, {0.9365498867497204, 
      0.3505343201868923}, {0.9343289424581945, 0.3564118787088177}, {
      0.9320711124598487, 0.3622753667000469}, {0.9297764858899454, 
      0.3681245526801139}, {0.9274451533364124, 0.37395920573317165`}, {
      0.9250772068362672, 0.37977909551710815`}, {0.9226727398719827, 
      0.38558399227264}, {0.920231847367798, 0.39137366683238284`}, {
      0.9177546256859693, 0.3971478906298986}, {0.9152411726229671, 
      0.40290643570871876`}, {0.9126915874056146, 0.4086490747313439}, {
      0.9101059706871706, 0.4143755809882183}, {0.9074844245433558, 
      0.4200857284066804}, {0.9048270524683231, 0.42577929155988736`}, {
      0.902133959370572, 0.4314560456757149}, {0.8994052515688065, 
      0.4371157666456307}, {0.8966410367877384, 0.4427582310335418}, {
      0.8938414241538343, 0.4483832160846157}, {0.8910065241910071, 
      0.453990499734074}, {0.8881364488162533, 0.45957986061595957`}, {
      0.8852313113352344, 0.4651510780718752}, {0.8822912264378036, 
      0.47070393215969525`}, {0.8793163101934784, 0.47623820366224834`}, {
      0.8763066800468584, 0.4817536740959717}, {0.8732624548129885, 
      0.48725012571953663`}, {0.8701837546726682, 0.4927273415424445}, {
      0.8670707011677076, 0.4981851053335931}, {0.8639234171961285, 
      0.5036232016298132}, {0.8607420270073133, 0.5090414157443744}, {
      0.8575266561970991, 0.5144395337754611}, {0.8542774317028199, 
      0.5198173426146164}, {0.8509944817982952, 0.5251746299551557}, {
      0.8476779360887658, 0.5305111843005479}, {0.8443279255057777, 
      0.5358267949727651}, {0.8409445823020124, 0.5411212521205997}, {
      0.8375280400460665, 0.5463943467279491}, {0.834078433617178, 
      0.5516458706220672}, {0.8305958991999023, 0.5568756164817826}, {
      0.8270805742787349, 0.5620833778456837}, {0.8235325976326846, 
      0.567268949120269}, {0.8199521093297942, 0.5724321255880636}, {
      0.8163392507216113, 0.5775727034157012}, {0.8126941644376073, 
      0.5826904796619715}, {0.8090169943795475, 0.5877852522858312}, {
      0.8053078857158094, 0.5928568201543809}, {0.8015669848756519, 
      0.5979049830508049}, {0.7977944395434349, 0.602929541682276}, {
      0.7939903986527884, 0.6079302976878227}, {0.7901550123807333, 
      0.6129070536461607}, {0.7862884321417521, 0.6178596130834864}, {
      0.7823908105818123, 0.6227877804812332}, {0.7784623015723391, 
      0.6276913612837908}, {0.7745030602041416, 0.6325701619061852}, {
      0.7705132427812896, 0.6374239897417219}, {0.7664930068149435, 
      0.6422526531695888}, {0.7624425110171352, 0.647055961562422}, {
      0.7583619152945035, 0.6518337252938305}, {0.7542513807419803, 
      0.6565857557458833}, {0.7501110696364314, 0.6613118653165547}, {
      0.7459411454302498, 0.6660118674271314}, {0.7417417727449033, 
      0.6706855765295776}, {0.7375131173644348, 0.6753328081138608}, {
      0.7332553462289183, 0.6799533787152352}, {0.7289686274278676, 
      0.6845471059214854}, {0.724653130193601, 0.6891138083801268}, {
      0.72030902489456, 0.693653305805566}, {0.7159364830285834, 
      0.6981654189862172}, {0.7115356772161372, 0.7026499697915782}, {
      0.7071067811934995, 0.7071067811792621}, {0.7026499698059017, 
      0.7115356772019864}, {0.698165419000626, 0.7159364830145197}, {
      0.6936533058200596, 0.7203090248805839}, {0.6891138083947048, 
      0.7246531301797131}, {0.6845471059361471, 0.7289686274140684}, {
      0.6799533787299802, 0.7332553462152084}, {0.6753328081286883, 
      0.7375131173508147}, {0.6706855765444872, 0.7417417727313735}, {
      0.6660118674421225, 0.745941145416811}, {0.6613118653316267, 
      0.7501110696230839}, {0.6565857557610355, 0.7542513807287248}, {
      0.6518337253090625, 0.7583619152813404}, {0.6470559615777329, 
      0.7624425110040651}, {0.6422526531849783, 0.7664930068019667}, {
      0.6374239897571892, 0.7705132427684068}, {0.6325701619217298, 
      0.7745030601913532}, {0.6276913612994119, 0.7784623015596456}, {
      0.6227877804969304, 0.7823908105692141}, {0.617859613099259, 
      0.7862884321292498}, {0.612907053662008, 0.7901550123683273}, {
      0.6079302977037442, 0.7939903986404793}, {0.602929541698271, 
      0.7977944395312231}, {0.5979049830668728, 0.8015669848635378}, {
      0.5928568201705211, 0.8053078857037934}, {0.5877852523020431, 
      0.8090169943676302}, {0.5826904796782544, 0.8126941644257891}, {
      0.5775727034320544, 0.8163392507098927}, {0.5724321256044863, 
      0.8199521093181756}, {0.5672689491367607, 0.8235325976211664}, {
      0.5620833778622438, 0.8270805742673175}, {0.5568756164984104, 
      0.8305958991885861}, {0.551645870638762, 0.8340784336059636}, {
      0.5463943467447103, 0.8375280400349541}, {0.5411212521374267, 
      0.8409445822910026}, {0.5358267949896571, 0.8443279254948708}, {
      0.5305111843175043, 0.8476779360779623}, {0.5251746299721759, 
      0.8509944817875953}, {0.5198173426316997, 0.8542774316922241}, {
      0.5144395337926067, 0.8575266561866078}, {0.5090414157615817, 
      0.8607420269969269}, {0.5036232016470814, 0.8639234171858474}, {
      0.49818510535092164`, 0.8670707011575322}, {0.49272734155983267`, 
      0.8701837546625989}, {0.48725012573698373`, 0.8732624548030256}, {
      0.481753674113477, 0.8763066800370023}, {0.4762382036798112, 
      0.8793163101837294}, {0.47070393217731493`, 0.8822912264281619}, {
      0.46515107808955103`, 0.8852313113257005}, {0.45957986063369083`, 
      0.8881364488068276}, {0.45399049975186, 0.8910065241816899}, {
      0.4483832161024557, 0.8938414241446259}, {0.44275823105143514`, 
      0.8966410367786392}, {0.4371157666635766, 0.8994052515598167}, {
      0.4314560456937127, 0.9021339593616918}, {0.4257792915779363, 
      0.9048270524595531}, {0.42008572842477976`, 0.9074844245346961}, {
      0.4143755810063674, 0.9101059706786216}, {0.408649074749542, 
      0.9126915873971767}, {0.4029064357269651, 0.9152411726146406}, {
      0.39714789064819245`, 0.9177546256777543}, {0.3913736668507235, 
      0.9202318473596949}, {0.38558399229102674`, 0.9226727398639918}, {
      0.37977909553554023`, 0.9250772068283886}, {0.3739592057516483, 
      0.9274451533286465}, {0.3681245526986344, 0.9297764858822924}, {
      0.36227536671861055`, 0.932071112452309}, {0.35641187872742375`, 
      0.9343289424507681}, {0.35053432020554004`, 0.9365498867424077}, {
      0.3446429231889058, 0.9387338576481498}, {0.33873792025978766`, 
      0.9408807689485627}, {0.3328195445375902, 0.9429905358872642}, {
      0.3268880296696531, 0.9450630751742684}, {0.32094360982202685`, 
      0.9470983049892726}, {0.31498651967022867`, 0.9490961449848889}, {
      0.30901699438997754`, 0.951056516289815}, {0.30303526964791, 
      0.9529793415119485}, {0.29704158159227656`, 0.954864544741442}, {
      0.29103616684361877`, 0.9567120515537}, {0.285019262485428, 
      0.958521789012317}, {0.2789911060547858, 0.9602936856719568}, {
      0.2729519355329861, 0.9620276715811734}, {0.26690198933614034`, 
      0.9637236782851721}, {0.2608415063057653, 0.9653816388285124}, {
      0.2547707256993538, 0.9670014877577506}, {0.24868988718092921`, 
      0.9685831611240249}, {0.2425992308115843, 0.9701265964855789}, {
      0.2364989970400036, 0.9716317329102274}, {0.23038942669297113`, 
      0.9730985109777616}, {0.22427076096586293`, 0.9745268727822948}, {
      0.21814324141312508`, 0.9759167619345487}, {0.2120071099387375, 
      0.9772681235640795}, {0.20586260878666407`, 0.9785809043214441}, {
      0.1997099805312892, 0.9798550523803058}, {0.19354946806784137`, 
      0.9810905174394811}, {0.18738131460280422`, 0.9822872507249248}, {
      0.18120576364431493`, 0.9834452049916561}, {0.1750230589925511, 
      0.9845643345256231}, {0.1688334447301059, 0.9856445951455081}, {
      0.16263716521235205`, 0.9866859442044715}, {0.15643446505779524`, 
      0.9876883405918356}, {0.1502255891384168, 0.9886517447347075}, {
      0.14401078257000674`, 0.9895761185995411}, {0.1377902907024869, 
      0.990461425693639}, {0.13156435911022504`, 0.9913076310665931}, {
      0.1253332335823399, 0.9921147013116642}, {0.11909716011299791`, 
      0.9928826045671009}, {0.11285638489170179`, 0.9936113105173975}, {
      0.10661115429357133`, 0.9943007903944908}, {0.10036171486961701`, 
      0.994951016978896}, {0.09410831333700652, 0.9955619646007808}, {
      0.08785119656932482, 0.9961336091409793}, {0.08159061158682798, 
      0.9966659280319436}, {0.07532680554669129, 0.9971589002586357}, {
      0.06906002573325183, 0.997612506359356}, {0.0627905195482462, 
      0.9980267284265124}, {0.056518534501043466`, 0.9984015501073269}, {
      0.05024431819887392, 0.9987369566044814}, {0.04396811833705399, 
      0.9990329346767016}, {0.03769018268920764, 0.9992894726392801}, {
      0.03141075909748468, 0.9995065603645374}, {0.02513009546277643, 
      0.9996841892822219}, {0.018848439734928953`, 0.999822352379848}, {
      0.01256603990295446, 0.999921044202973}, {0.006283143985241123, 
      0.9999802608554129}, {1.9761723507594198`*^-11, 
      0.9999999999993956}, {-0.00628314394571845, 
      0.9999802608556535}, {-0.012566039863434113`, 
      0.9999210442034543}, {-0.01884843969541248, 
      0.9998223523805699}, {-0.02513009542326538, 
      0.9996841892831846}, {-0.0314107590579806, 
      0.9995065603657407}, {-0.03769018264971208, 
      0.9992894726407239}, {-0.043968118297568505`, 
      0.9990329346783858}, {-0.05024431815940005, 
      0.9987369566064059}, {-0.05651853446158276, 
      0.9984015501094917}, {-0.0627905195088002, 
      0.9980267284289174}, {-0.06906002569382208, 
      0.997612506362001}, {-0.07532680550727933, 
      0.9971589002615208}, {-0.08159061154743537, 
      0.9966659280350686}, {-0.08785119652995309, 
      0.9961336091443439}, {-0.09410831329765722, 
      0.9955619646043851}, {-0.10036171483029169`, 
      0.9949510169827397}, {-0.10661115425427152`, 
      0.9943007903985738}, {-0.11285638485242902`, 
      0.9936113105217195}, {-0.11909716007375373`, 
      0.9928826045716618}, {-0.12533323354312584`, 
      0.9921147013164637}, {-0.13156435907104264`, 
      0.9913076310716311}, {-0.1377902906633377, 
      0.9904614256989153}, {-0.14401078253089225`, 
      0.9895761186050553}, {-0.15022558909933856`, 
      0.9886517447404595}, {-0.15643446501875477`, 
      0.9876883405978251}, {-0.1626371651733509, 
      0.9866859442106981}, {-0.16883344469114556`, 
      0.9856445951519716}, {-0.17502305895363313`, 
      0.9845643345323232}, {-0.18120576360544083`, 
      0.9834452049985926}, {-0.18738131456397553`, 
      0.9822872507320974}, {-0.1935494680290596, 
      0.9810905174468895}, {-0.1997099804925558, 
      0.9798550523879496}, {-0.20586260874798062`, 
      0.9785809043293229}, {-0.2120071099001055, 
      0.9772681235721931}, {-0.218143241374546, 
      0.9759167619428966}, {-0.2242707609273383, 
      0.9745268727908766}, {-0.23038942665450243`, 
      0.973098510986577}, {-0.23649899700159233`, 
      0.9716317329192761}, {-0.24259923077323198`, 
      0.9701265964948604}, {-0.24868988714263732`, 
      0.9685831611335388}, {-0.2547707256611238, 
      0.9670014877674965}, {-0.2608415062675987, 
      0.9653816388384898}, {-0.26690198929803866`, 
      0.9637236782953807}, {-0.2729519354949507, 
      0.9620276715916127}, {-0.27899110601681826`, 
      0.9602936856826264}, {-0.2850192624475298, 
      0.9585217890232163}, {-0.2910361668057913, 
      0.9567120515648286}, {-0.2970415815545213, 
      0.9548645447527995}, {-0.3030352696102285, 
      0.9529793415235343}, {-0.3090169943523712, 
      0.9510565163016286}, {-0.31498651963269897`, 
      0.9490961449969297}, {-0.32094360978457526`, 
      0.9470983050015402}, {-0.32688802963228103`, 
      0.9450630751867621}, {-0.33281954450029916`, 
      0.9429905358999837}, {-0.33873792022257904`, 
      0.9408807689615072}, {-0.344642923151781, 
      0.9387338576613188}, {-0.3505343201685005, 
      0.9365498867558006}, {-0.3564118786904709, 
      0.9343289424643845}, {-0.36227536668174587`, 
      0.932071112466148}, {-0.3681245526618593, 
      0.9297764858963536}, {-0.37395920571496416`, 
      0.9274451533429292}, {-0.37977909549894845`, 
      0.9250772068428921}, {-0.3855839922545287, 
      0.9226727398787156}, {-0.3913736668143207, 
      0.9202318473746381}, {-0.3971478906118862, 
      0.9177546256929165}, {-0.40290643569075685`, 
      0.915241172630021}, {-0.4086490747134331, 
      0.9126915874127748}, {-0.41437558097035926`, 
      0.9101059706944367}, {-0.42008572838887376`, 
      0.9074844245507273}, {-0.4257792915421338, 
      0.9048270524757998}, {-0.43145604565801504`, 
      0.9021339593781533}, {-0.4371157666279852, 
      0.8994052515764922}, {-0.4427582310159513, 
      0.8966410367955281}, {-0.4483832160670808, 
      0.8938414241617274}, {-0.4539904997165954, 
      0.8910065241990033}, {-0.4595798605985378, 
      0.888136448824352}, {-0.46515107805451095`, 
      0.8852313113434352}, {-0.47070393214238915`, 
      0.8822912264461061}, {-0.47623820364500097`, 
      0.8793163102018823}, {-0.4817536740787837, 
      0.876306680055363}, {-0.48725012570240867`, 
      0.8732624548215935}, {-0.49272734152537717`, 
      0.8701837546813731}, {-0.498185105316587, 
      0.8670707011765119}, {-0.5036232016128689, 
      0.8639234172050319}, {-0.5090414157274926, 
      0.8607420270163151}, {-0.5144395337586424, 
      0.857526656206199}, {-0.5198173425978614, 
      0.8542774317120173}, {-0.5251746299384649, 
      0.8509944818075897}, {-0.5305111842839219, 
      0.847677936098157}, {-0.5358267949562044, 
      0.844327925515265}, {-0.541121252104105, 
      0.8409445823115954}, {-0.546394346711521, 
      0.8375280400557447}, {-0.5516458706057062, 
      0.8340784336269509}, {-0.5568756164654894, 
      0.8305958992097693}, {-0.5620833778294588, 
      0.8270805742886955}, {-0.5672689491041129, 
      0.8235325976427383}, {-0.5724321255719768, 
      0.8199521093399407}, {-0.5775727033996844, 
      0.8163392507318499}, {-0.5826904796460252, 
      0.8126941644479375}, {-0.5877852522699559, 
      0.8090169943899688}, {-0.5928568201385771, 
      0.8053078857263212}, {-0.5979049830350732, 
      0.8015669848862538}, {-0.6029295416666169, 
      0.7977944395541262}, {-0.6079302976722368, 
      0.7939903986635687}, {-0.6129070536306486, 
      0.7901550123916019}, {-0.6178596130680485, 
      0.7862884321527086}, {-0.6227877804658701, 
      0.7823908105928561}, {-0.627691361268503, 
      0.7784623015834696}, {-0.6325701618909733, 
      0.7745030602153583}, {-0.6374239897265862, 
      0.770513242792592}, {-0.64225265315453, 
      0.7664930068263309}, {-0.6470559615474404, 
      0.7624425110286072}, {-0.6518337252789267, 
      0.7583619153060593}, {-0.6565857557310577, 
      0.7542513807536195}, {-0.6613118653018079, 
      0.7501110696481534}, {-0.6660118674124638, 
      0.7459411454420541}, {-0.6706855765149897, 
      0.7417417727567892}, {-0.675332808099353, 
      0.7375131173764018}, {-0.6799533787008082, 
      0.7332553462409657}, {-0.6845471059071394, 
      0.7289686274399949}, {-0.6891138083658624, 
      0.7246531302058076}, {-0.6936533057913836, 
      0.7203090249068452}, {-0.6981654189721174, 
      0.7159364830409468}, {-0.7026499697775613, 
      0.7115356772285781}, {-0.7071067811653285, 
      0.7071067812060173}, {-0.7115356771881367, 
      0.7026499698184958}, {-0.7159364830007543, 
      0.6981654190132959}, {-0.7203090248669032, 
      0.6936533058328046}, {-0.7246531301661175, 
      0.6891138084075243}, {-0.7289686274005585, 
      0.6845471059490406}, {-0.7332553462017843, 
      0.6799533787429468}, {-0.737513117337477, 
      0.6753328081417276}, {-0.7417417727181226, 
      0.6706855765575984}, {-0.7459411454036472, 
      0.6660118674553049}, {-0.7501110696100077, 
      0.6613118653448798}, {-0.7542513807157365, 
      0.6565857557743588}, {-0.7583619152684404, 
      0.6518337253224553}, {-0.7624425109912538, 
      0.6470559615911946}, {-0.7664930067892446, 
      0.6422526531985081}, {-0.7705132427557742, 
      0.6374239897707865}, {-0.7745030601788103, 
      0.6325701619353941}, {-0.778462301547193, 
      0.6276913613131425}, {-0.7823908105568521, 
      0.6227877805107266}, {-0.7862884321169787, 
      0.6178596131131201}, {-0.7901550123561475, 
      0.6129070536759335}, {-0.7939903986283912, 
      0.6079302977177332}, {-0.7977944395192269, 
      0.6029295417123229}, {-0.8015669848516339, 
      0.597904983080987}, {-0.8053078856919822, 
      0.592856820184697}, {-0.8090169943559119, 
      0.58778525231628}, {-0.8126941644141641, 
      0.5826904796925516}, {-0.8163392506983611, 
      0.5775727034464113}, {-0.8199521093067379, 
      0.5724321256189023}, {-0.8235325976098229, 
      0.5672689491512352}, {-0.8270805742560685, 
      0.562083377876776}, {-0.8305958991774319, 
      0.5568756165129995}, {-0.8340784335949045, 
      0.5516458706534074}, {-0.8375280400239904, 
      0.5463943467594113}, {-0.8409445822801346, 
      0.5411212521521827}, {-0.8443279254840987, 
      0.5358267950044674}, {-0.8476779360672864, 
      0.5305111843323682}, {-0.8509944817770159, 
      0.5251746299870926}, {-0.8542774316817414, 
      0.5198173426466686}, {-0.8575266561762221, 
      0.5144395338076271}, {-0.8607420269866385, 
      0.5090414157766529}, {-0.8639234171756565, 
      0.5036232016622028}, {-0.8670707011474389, 
      0.49818510536609245`}, {-0.8701837546526036, 
      0.4927273415750522}, {-0.8732624547931285, 
      0.48725012575225135`}, {-0.8763066800272036, 
      0.481753674128792}, {-0.8793163101740293, 
      0.47623820369517283`}, {-0.8822912264185607, 
      0.4707039321927226}, {-0.8852313113161984, 
      0.465151078105004}, {-0.8881364487974247, 
      0.4595798606491884}, {-0.8910065241723865, 
      0.45399049976740147`}, {-0.8938414241354221, 
      0.44838321611804033`}, {-0.8966410367695352, 
      0.44275823106706225`}, {-0.8994052515508127, 
      0.4371157666792455}, {-0.9021339593527882, 
      0.4314560457094227}, {-0.9048270524507498, 
      0.42577929159368666`}, {-0.9074844245259934, 
      0.4200857284405698}, {-0.9101059706700196, 
      0.4143755810221964}, {-0.9126915873886756, 
      0.4086490747654092}, {-0.9152411726062405, 
      0.40290643574286983`}, {-0.9177546256694554, 
      0.397147890664134}, {-0.9202318473514972, 
      0.39137366686670116`}, {-0.9226727398558956, 
      0.38558399230703977`}, {-0.925077206820394, 
      0.37977909555158795`}, {-0.9274451533207536, 
      0.37395920576773}, {-0.9297764858745013, 
      0.36812455271474936`}, {-0.9320711124446198, 
      0.362275366734758}, {-0.934328942443181, 
      0.3564118787436031}, {-0.9365498867349227, 
      0.3505343202217505}, {-0.938733857640767, 
      0.3446429232051467}, {-0.9408807689412821, 
      0.33873792027605826`}, {-0.9429905358800862, 
      0.3328195445538898}, {-0.9450630751671928, 
      0.3268880296859809}, {-0.9470983049822996, 
      0.32094360983838227`}, {-0.9490961449780185, 
      0.3149865196866109}, {-0.9510565162830473, 
      0.30901699440638586`}, {-0.9529793415052835, 
      0.30303526966434374`}, {-0.9548645447348798, 
      0.297041581608735}, {-0.9567120515472407, 
      0.29103616686010125`}, {-0.9585217890059606, 
      0.28501926250193377`}, {-0.9602936856657035, 
      0.2789911060713141}, {-0.9620276715750231, 
      0.27295193554953623`}, {-0.9637236782791249, 
      0.26690198935271164`}, {-0.9653816388225681, 
      0.26084150632235703`}, {-0.9670014877519095, 
      0.2547707257159652}, {-0.9685831611182869, 
      0.24868988719755966`}, {-0.970126596479944, 
      0.24259923082823304`}, {-0.9716317329046956, 
      0.2364989970566699}, {-0.9730985109723329, 
      0.2303894267096543}, {-0.9745268727769693, 
      0.22427076098256224`}, {-0.9759167619293263, 
      0.21814324142983982`}, {-0.9772681235589602, 
      0.21200710995546695`}, {-0.9785809043164277, 
      0.20586260880340754`}, {-0.9798550523753925, 
      0.19970998054804595`}, {-0.9810905174346708, 
      0.19354946808461074`}, {-0.9822872507202175, 
      0.18738131461958546`}, {-0.9834452049870517, 
      0.18120576366110736`}, {-0.9845643345211216, 
      0.175023059009354}, {-0.9856445951411094, 
      0.16883344474691855`}, {-0.9866859442001756, 
      0.1626371652291738}, {-0.9876883405876424, 
      0.15643446507462533`}, {-0.9886517447306169, 
      0.15022558915525455`}, {-0.9895761185955531, 
      0.14401078258685143`}, {-0.9904614256897535, 
      0.13779029071933785`}, {-0.99130763106281, 
      0.13156435912708153`}, {-0.9921147013079833, 
      0.12533323359920126`}, {-0.9928826045635222, 
      0.11909716012986345`}, {-0.9936113105139208, 
      0.11285638490857079`}, {-0.9943007903911162, 
      0.10661115431044312`}, {-0.9949510169756233, 
      0.10036171488649089`}, {-0.9955619645976098, 
      0.09410831335388178}, {-0.9961336091379099, 
      0.08785119658620079}, {-0.9966659280289759, 
      0.08159061160370398}, {-0.9971589002557694, 
      0.07532680556356662}, {-0.9976125063565909, 
      0.06906002575012582}, {-0.9980267284238484, 
      0.06279051956511816}, {-0.998401550104764, 
      0.056518534517912715`}, {-0.9987369566020192, 
      0.050244318215739776`}, {-0.9990329346743402, 
      0.04396811835391577}, {-0.9992894726370192, 
      0.03769018270606467}, {-0.9995065603623768, 
      0.03141075911433629}, {-0.9996841892801615, 
      0.025130095479621934`}, {-0.9998223523778874, 
      0.018848439751767684`}, {-0.9999210442011123, 
      0.012566039919785752`}, {-0.9999802608536518, 
      0.006283144002064306}, {-0.9999999999977339, 
      3.657613534963078*^-11}, {-0.9999802608540911, \
-0.0062831439289134745`}, {-0.9999210442019908, -0.012566039846639235`}, \
{-0.9998223523792051, -0.01884843967862836}, {-0.9996841892819184, \
-0.025130095406492678`}, {-0.9995065603645729, -0.031410759041219975`}, \
{-0.9992894726396542, -0.03769018263296417}, {-0.999032934677414, \
-0.04396811828083397}, {-0.9987369566055319, -0.05024431814267954}, \
{-0.9984015501087151, -0.056518534444876915`}, {-0.998026728428238, \
-0.06279051949210968}, {-0.9976125063614187, -0.0690600256771475}, \
{-0.9971589002610352, -0.07532680549062135}, {-0.9966659280346793, \
-0.08159061153079461}, {-0.9961336091440508, -0.08785119651333019}, \
{-0.9955619646041879, -0.09410831328105282}, {-0.9949510169826381, \
-0.10036171481370641`}, {-0.9943007903985676, -0.10661115423770601`}, \
{-0.9936113105218084, -0.1128563848358839}, {-0.9928826045718455, \
-0.1190971600572296}, {-0.9921147013167421, -0.12533323352662332`}, \
{-0.9913076310720037, -0.13156435905456235`}, {-0.9904614256993818, \
-0.13779029064688025`}, {-0.9895761186056156, -0.14401078251445826`}, \
{-0.9886517447411131, -0.15022558908292866`}, {-0.9876883405985718, \
-0.15643446500236954`}, {-0.9866859442115377, -0.16263716515699095`}, \
{-0.9856445951529036, -0.16883344467481148`}, {-0.9845643345333474, \
-0.1750230589373255}, {-0.9834452049997084, -0.18120576358916027`}, \
{-0.9822872507333047, -0.1873813145477226}, {-0.9810905174481878, \
-0.1935494680128349}, {-0.9798550523893387, -0.19970998047635993`}, \
{-0.9785809043308025, -0.20586260873181414`}, {-0.9772681235737627, \
-0.21200710988396898`}, {-0.9759167619445559, -0.21814324135844004`}, \
{-0.9745268727926253, -0.22427076091126347`}, {-0.9730985109884147, \
-0.2303894266384593}, {-0.9716317329212024, -0.23649899698558147`}, \
{-0.970126596496875, -0.24259923075725393`}, {-0.9685831611356414, \
-0.24868988712669263`}, {-0.9670014877696866, -0.254770725645213}, \
{-0.9653816388407671, -0.2608415062517224}, {-0.9637236782977447, \
-0.2669019892821974}, {-0.9620276715940631, -0.27295193547914504`}, \
{-0.9602936856851627, -0.2789911060010487}, {-0.9585217890258382, \
-0.28501926243179687`}, {-0.9567120515675357, -0.2910361667900956}, \
{-0.9548645447555912, -0.2970415815388633}, {-0.9529793415264104, \
-0.3030352695946087}, {-0.9510565163045887, -0.30901699433679014`}, \
{-0.9490961449999733, -0.3149865196171572}, {-0.9470983050046669, \
-0.3209436097690732}, {-0.9450630751899713, -0.32688802961681923`}, \
{-0.9429905359032751, -0.33281954448487816`}, {-0.9408807689648804, \
-0.33873792020719934`}, {-0.9387338576647734, -0.3446429231364431}, \
{-0.9365498867593361, -0.3505343201532049}, {-0.9343289424680005, \
-0.3564118786752181}, {-0.9320711124698441, -0.3622753666665363}, \
{-0.9297764859001293, -0.36812455264669347`}, {-0.927445153346784, \
-0.37395920569984253`}, {-0.9250772068468256, -0.3797790954838715}, \
{-0.9226727398827272, -0.38558399223949696`}, {-0.9202318473787277, \
-0.39137366679933455`}, {-0.9177546256970833, -0.39714789059694616`}, \
{-0.9152411726342646, -0.4029064356758633}, {-0.9126915874170948, \
-0.4086490746985866}, {-0.9101059706988325, -0.4143755809555602}, \
{-0.9074844245551985, -0.4200857283741226}, {-0.9048270524803458, \
-0.425779291527431}, {-0.9021339593827737, -0.431456045643361}, \
{-0.8994052515811864, -0.4371157666133803}, {-0.8966410368002957, \
-0.44275823100139605`}, {-0.893841424166568, -0.4483832160525756}, \
{-0.8910065242039162, -0.45399049970214067`}, {-0.888136448829337, \
-0.45957986058413397`}, {-0.8852313113484915, -0.4651510780401584}, \
{-0.8822912264512334, -0.4707039321280883}, {-0.8793163102070799, \
-0.47623820363075225`}, {-0.8763066800606306, -0.4817536740645875}, \
{-0.8732624548269304, -0.48725012568826537`}, {-0.8701837546867789, \
-0.49272734151128716`}, {-0.867070701181986, -0.49818510530255067`}, \
{-0.8639234172105738, -0.5036232015988866}, {-0.8607420270219244, \
-0.5090414157135648}, {-0.857526656211875, -0.5144395337447693}, \
{-0.8542774317177596, -0.5198173425840433}, {-0.8509944818133977, \
-0.5251746299247023}, {-0.8476779361040301, -0.5305111842702152}, \
{-0.8443279255212027, -0.5358267949425539}, {-0.8409445823175972, \
-0.541121252090511}, {-0.8375280400618099, -0.5463943466979838}, \
{-0.8340784336330791, -0.5516458705922263}, {-0.8305958992159599, \
-0.556875616452067}, {-0.8270805742949481, -0.5620833778160942}, \
{-0.8235325976490523, -0.5672689490908065}, {-0.8199521093463153, \
-0.5724321255587289}, {-0.8163392507382847, -0.5775727033864954}, \
{-0.8126941644544321, -0.5826904796328952}, {-0.8090169943965224, \
-0.5877852522568854}, {-0.8053078857329333, -0.5928568201255663}, \
{-0.8015669848929239, -0.5979049830221224}, {-0.7977944395608538, \
-0.6029295416537264}, {-0.7939903986703531, -0.6079302976594069}, \
{-0.7901550123984427, -0.6129070536178794}, {-0.7862884321596051, \
-0.6178596130553404}, {-0.7823908105998076, -0.6227877804532235}, \
{-0.7784623015904758, -0.627691361255918}, {-0.7745030602224184, \
-0.63257016187845}, {-0.7705132427997056, -0.6374239897141252}, \
{-0.7664930068334973, -0.6422526531421313}, {-0.7624425110358258, \
-0.6470559615351044}, {-0.7583619153133297, -0.6518337252666536}, \
{-0.754251380760941, -0.6565857557188477}, {-0.7501110696555253, \
-0.6613118652896611}, {-0.7459411454494759, -0.6660118674003805}, \
{-0.7417417727642602, -0.6706855765029702}, {-0.7375131173839216, \
-0.6753328080873976}, {-0.7332553462485336, -0.6799533786889169}, \
{-0.7289686274476104, -0.6845471058953125}, {-0.72465313021347, \
-0.6891138083541001}, {-0.7203090249145541, -0.693653305779686}, \
{-0.7159364830487014, -0.6981654189604847}, {-0.7115356772363779, \
-0.7026499697659938}, {-0.7071067812138616, -0.7071067811538263}, \
{-0.7026499698263841, -0.7115356771766999}, {-0.6981654190212275, \
-0.715936482989383}, {-0.693653305840779, -0.7203090248555976}, \
{-0.6891138084155408, -0.7246531301548778}, {-0.6845471059570986, \
-0.7289686273893846}, {-0.6799533787510458, -0.7332553461906767}, \
{-0.6753328081498668, -0.7375131173264357}, {-0.6706855765657774, \
-0.7417417727071477}, {-0.6660118674635231, -0.7459411453927388}, \
{-0.6613118653531366, -0.750111069599166}, {-0.6565857557826534, \
-0.7542513807049616}, {-0.6518337253307872, -0.7583619152577324}, \
{-0.6470559615995632, -0.7624425109806128}, {-0.6422526532069128, \
-0.7664930067786706}, {-0.6374239897792268, -0.7705132427452673}, \
{-0.6325701619438692, -0.7745030601683708}, {-0.6276913613216519, \
-0.7784623015368207}, {-0.6227877805192696, -0.7823908105465472}, \
{-0.6178596131216962, -0.7862884321067414}, {-0.6129070536845421, \
-0.7901550123459776}, {-0.6079302977263737, -0.7939903986182888}, \
{-0.6029295417209948, -0.7977944395091922}, {-0.5979049830896896, \
-0.8015669848416669}, {-0.5928568201934297, -0.8053078856820829}, \
{-0.5877852523250421, -0.8090169943460804}, {-0.5826904797013425, \
-0.8126941644044005}, {-0.5775727034552304, -0.8163392506886653}, \
{-0.572432125627749, -0.81995210929711}, {-0.5672689491601088, \
-0.8235325976002629}, {-0.5620833778856761, -0.8270805742465763}, \
{-0.5568756165219254, -0.8305958991680077}, {-0.5516458706623585, \
-0.8340784335855481}, {-0.546394346768387, -0.8375280400147019}, \
{-0.5411212521611823, -0.840944582270914}, {-0.5358267950134905, \
-0.844327925474946}, {-0.5305111843414141, -0.8476779360582014}, \
{-0.5251746299961607, -0.8509944817679987}, {-0.5198173426557583, \
-0.854277431672792}, {-0.5144395338167379, -0.8575266561673405}, \
{-0.5090414157857841, -0.8607420269778245}, {-0.5036232016713538, \
-0.8639234171669101}, {-0.49818510537526267`, -0.8670707011387601}, \
{-0.4927273415842411, -0.8701837546439922}, {-0.4872501257614582, \
-0.8732624547845845}, {-0.4817536741380163, -0.876306680018727}, \
{-0.476238203704414, -0.8793163101656201}, {-0.47070393220198, \
-0.8822912264102186}, {-0.46515107811427703`, -0.8852313113079234}, \
{-0.4595798606584765, -0.8881364487892168}, {-0.4539904997767041, \
-0.8910065241642454}, {-0.4483832161273569, -0.8938414241273478}, \
{-0.4427582310763921, -0.8966410367615276}, {-0.4371157666885881, \
-0.8994052515428717}, {-0.43145604571877744`, -0.9021339593449135}, \
{-0.425779291603053, -0.9048270524429414}, {-0.4200857284499471, \
-0.9074844245182512}, {-0.4143755810315842, -0.9101059706623434}, \
{-0.40864907477480683`, -0.9126915873810653}, {-0.4029064357522768, \
-0.9152411725986959}, {-0.3971478906735497, -0.9177546256619764}, \
{-0.391373666876125, -0.9202318473440837}, {-0.3855839923164712, \
-0.9226727398485474}, {-0.37977909556102646`, -0.9250772068131109}, \
{-0.373959205777175, -0.9274451533135355}, {-0.3681245527242003, \
-0.929776485867348}, {-0.36227536674421434`, -0.932071112437531}, \
{-0.3564118787530642, -0.9343289424361566}, {-0.35053432023121583`, \
-0.9365498867279625}, {-0.34464292321461576`, -0.9387338576338708}, \
{-0.33873792028553046`, -0.9408807689344498}, {-0.3328195445633646, \
-0.9429905358733175}, {-0.32688802969545777`, -0.9450630751604875}, \
{-0.32094360984786063`, -0.9470983049756575}, {-0.31498651969609026`, \
-0.9490961449714393}, {-0.30901699441586566`, -0.9510565162765309}, \
{-0.30303526967382344`, -0.9529793414988296}, {-0.29704158161821403`, \
-0.9548645447284881}, {-0.2910361668695791, -0.956712051540911}, \
{-0.28501926251140997`, -0.9585217889996926}, {-0.2789911060807881, \
-0.960293685659497}, {-0.2729519355590075, -0.9620276715688779}, \
{-0.2669019893621797, -0.9637236782730406}, {-0.2608415063318213, \
-0.9653816388165446}, {-0.2547707257254252, -0.9670014877459464}, \
{-0.24868988720701485`, -0.968583161112384}, {-0.24259923083768292`, \
-0.9701265964741009}, {-0.236498997066114, -0.9716317328989121}, \
{-0.2303894267190921, -0.9730985109666087}, {-0.22427076099199325`, \
-0.974526872771304}, {-0.21814324143926356`, -0.9759167619237198}, \
{-0.21200710996488292`, -0.9772681235534121}, {-0.20586260881281523`, \
-0.9785809043109378}, {-0.1997099805574449, -0.9798550523699603}, \
{-0.19354946809400048`, -0.981090517429296}, {-0.1873813146289655, \
-0.9822872507148998}, {-0.18120576367047722`, -0.9834452049817907}, \
{-0.17502305901871323`, -0.984564334515917}, {-0.1688334447562667, \
-0.9856445951359609}, {-0.16263716523851035`, -0.9866859441950829}, \
{-0.15643446508394987`, -0.9876883405826052}, {-0.1502255891645666, \
-0.9886517447256348}, {-0.14401078259615058`, -0.9895761185906257}, \
{-0.13779029072862362`, -0.9904614256848804}, {-0.1315643591363535, \
-0.9913076310579908}, {-0.12533323360845897`, -0.9921147013032178}, \
{-0.11909716013910646`, -0.99288260455881}, {-0.11285638491779867`, \
-0.9936113105092615}, {-0.10661115431965545`, -0.9943007903865094}, \
{-0.10036171489568724`, -0.9949510169710686}, {-0.09410831336306175, \
-0.9955619645931069}, {-0.08785119659536393, -0.9961336091334583}, \
{-0.08159061161284989, -0.9966659280245752}, {-0.0753268055726949, \
-0.9971589002514192}, {-0.06906002575923606, -0.9976125063522909}, \
{-0.06279051957420996, -0.9980267284195982}, {-0.05651853452698567, \
-0.998401550100563}, {-0.050244318224793506`, -0.9987369565978672}, \
{-0.043968118362949886`, -0.9990329346702367}, {-0.037690182715078784`, \
-0.9992894726329639}, {-0.03141075912333003, -0.9995065603583692}, \
{-0.02513009548859493, -0.999684189276201}, {-0.018848439760719562`, \
-0.9998223523739738}, {-0.012566039928716147`, -0.9999210441972449}, \
{-0.006283144010972859, -0.9999802608498304}, {-4.546248967562949*^-11, \
-0.999999999993958}, {0.006283143920049669, -0.9999802608503603}, {
      0.012566039837798325`, -0.9999210441983046}, {
      0.018848439669810688`, -0.9998223523755632}, {
      0.025130095397698577`, -0.99968418927832}, {
      0.031410759032449775`, -0.9995065603610178}, {
      0.03769018262421821, -0.9992894726361419}, {
      0.04396811827211256, -0.9990329346739439}, {
      0.05024431813398301, -0.9987369566021036}, {
      0.056518534436205574`, -0.9984015501053282}, {
      0.06279051948346383, -0.9980267284248919}, {
      0.06906002566852745, -0.9976125063581129}, {
      0.07532680548202739, -0.9971589002577692}, {
      0.08159061152222705, -0.9966659280314528}, {
      0.0878511965047893, -0.9961336091408632}, {
      0.0941083132725389, -0.9955619646010387}, {
      0.10036171480521971`, -0.9949510169795269}, {
      0.10661115422924682`, -0.9943007903954938}, {
      0.11285638482745247`, -0.9936113105187715}, {
      0.11909716004882621`, -0.992882604568845}, {
      0.12533323351824824`, -0.9921147013137775}, {
      0.13156435904621583`, -0.9913076310690746}, {
      0.13779029063856252`, -0.9904614256964877}, {
      0.14401078250616955`, -0.9895761186027557}, {
      0.1502255890746692, -0.9886517447382871}, {
      0.1564344649941396, -0.9876883405957791}, {
      0.16263716514879073`, -0.9866859442087778}, {
      0.1688334446666412, -0.985644595150176}, {
      0.1750230589291854, -0.9845643345306515}, {
      0.18120576358105053`, -0.9834452049970439}, {
      0.18738131453964343`, -0.9822872507306709}, {
      0.1935494680047865, -0.9810905174455842}, {
      0.1997099804683425, -0.9798550523867647}, {
      0.20586260872382786`, -0.9785809043282576}, {
      0.21200710987601404`, -0.9772681235712465}, {
      0.2181432413505166, -0.9759167619420678}, {
      0.2242707609033717, -0.9745268727901647}, {
      0.23038942663059936`, -0.9730985109859811}, {
      0.2364989969777535, -0.9716317329187952}, {
      0.2425992307494581, -0.9701265964944937}, {
      0.24868988711892911`, -0.9685831611332854}, {
      0.25477072563748193`, -0.9670014877673554}, {
      0.2608415062440239, -0.9653816388384601}, {
      0.26690198927453157`, -0.9637236782954615}, {
      0.27295193547151203`, -0.9620276715918029}, {
      0.2789911059934486, -0.9602936856829251}, {
      0.28501926242422987`, -0.9585217890236225}, {
      0.2910361667825617, -0.9567120515653414}, {
      0.2970415815313627, -0.9548645447534179}, {
      0.3030352695871414, -0.9529793415242573}, {
      0.30901699432935625`, -0.9510565163024554}, {
      0.3149865196097568, -0.9490961449978592}, {
      0.3209436097617065, -0.9470983050025714}, {
      0.3268880296094862, -0.945063075187894}, {
      0.33281954447757883`, -0.9429905359012153}, {
      0.3387379201999338, -0.9408807689628375}, {
      0.34464292312921146`, -0.9387338576627468}, {
      0.3505343201460071, -0.9365498867573253}, {
      0.35641187866805424`, -0.9343289424660048}, {
      0.3622753666594064, -0.932071112467863}, {
      0.3681245526395976, -0.9297764858981622}, {
      0.3739592056927807, -0.9274451533448304}, {
      0.3797790954768437, -0.9250772068448848}, {
      0.38558399223250317`, -0.9226727398807988}, {
      0.3913736667923748, -0.920231847376811}, {
      0.3971478905900205, -0.9177546256951779}, {
      0.4029064356689717, -0.9152411726323697}, {
      0.40864907469172895`, -0.9126915874152098}, {
      0.4143755809487366, -0.910105970696957}, {
      0.420085728367333, -0.9074844245533319}, {
      0.4257792915206753, -0.9048270524784875}, {
      0.4314560456366392, -0.9021339593809231}, {
      0.4371157666066924, -0.899405251579343}, {
      0.44275823099474193`, -0.8966410367984589}, {
      0.4483832160459552, -0.8938414241647371}, {
      0.45399049969555394`, -0.8910065242020908}, {
      0.4595798605775809, -0.8881364488275164}, {
      0.46515107803363887`, -0.8852313113466752}, {
      0.4707039321216022, -0.8822912264494207}, {
      0.4762382036242995, -0.8793163102052703}, {
      0.48175367405816805`, -0.8763066800588236}, {
      0.4872501256818791, -0.8732624548251253}, {
      0.4927273415049339, -0.8701837546849751}, {
      0.49818510529623034`, -0.867070701180183}, {
      0.5036232015925991, -0.863923417208771}, {
      0.5090414157073099, -0.8607420270201211}, {
      0.5144395337385469, -0.8575266562100708}, {
      0.5198173425778534, -0.8542774317159539}, {
      0.5251746299185447, -0.85099448181159}, {
      0.5305111842640896, -0.8476779361022199}, {
      0.5358267949364603, -0.8443279255193893}, {
      0.5411212520844493, -0.84094458231578}, {
      0.5463943466919537, -0.8375280400599884}, {
      0.5516458705862276, -0.8340784336312527}, {
      0.5568756164460996, -0.830595899214128}, {
      0.5620833778101579, -0.8270805742931101}, {
      0.5672689490849011, -0.8235325976472077}, {
      0.5724321255528543, -0.8199521093444636}, {
      0.5775727033806513, -0.8163392507364253}, {
      0.5826904796270814, -0.8126941644525644}, {
      0.5877852522511017, -0.8090169943946459}, {
      0.5928568201198124, -0.8053078857310475}, {
      0.5979049830163982, -0.8015669848910281}, {
      0.6029295416480315, -0.7977944395589475}, {
      0.6079302976537411, -0.7939903986684357}, {
      0.6129070536122425, -0.7901550123965135}, {
      0.6178596130497321, -0.7862884321576639}, {
      0.6227877804476434, -0.7823908105978538}, {
      0.6276913612503658, -0.7784623015885087}, {
      0.6325701618729257, -0.7745030602204376}, {
      0.6374239897086282, -0.7705132427977104}, {
      0.6422526531366616, -0.7664930068314872}, {
      0.6470559615296615, -0.7624425110338005}, {
      0.6518337252612373, -0.7583619153112884}, {
      0.6565857557134577, -0.7542513807588832}, {
      0.6613118652842971, -0.7501110696534506}, {
      0.6660118673950421, -0.7459411454473838}, {
      0.6706855764976571, -0.7417417727621501}, {
      0.6753328080821094, -0.7375131173817929}, {
      0.6799533786836534, -0.733255346246386}, {
      0.6845471058900733, -0.7289686274454432}, {
      0.6891138083488848, -0.7246531302112829}, {
      0.6936533057744942, -0.7203090249123464}, {
      0.6981654189553161, -0.7159364830464726}, {
      0.702649969760848, -0.7115356772341276}, {
      0.707106781148703, -0.7071067812115892}, {
      0.7115356771715986, -0.7026499698240891}, {
      0.7159364829843035, -0.6981654190189095}, {
      0.7203090248505394, -0.6936533058384374}, {
      0.7246531301498405, -0.6891138084131753}, {
      0.7289686273843679, -0.6845471059547085}, {
      0.73325534618568, -0.6799533787486307}, {
      0.7375131173214587, -0.6753328081474264}, {
      0.74174177270219, -0.6706855765633111}, {
      0.7459411453878, -0.6660118674610304}, {
      0.7501110695942456, -0.661311865350617}, {
      0.754251380700059, -0.6565857557801066}, {
      0.7583619152528474, -0.6518337253282126}, {
      0.7624425109757448, -0.6470559615969604}, {
      0.7664930067738192, -0.6422526532042814}, {
      0.770513242740432, -0.6374239897765662}, {
      0.7745030601635511, -0.6325701619411791}, {
      0.7784623015320162, -0.6276913613189319}, {
      0.7823908105417575, -0.6227877805165192}, {
      0.7862884321019659, -0.6178596131189149}, {
      0.7901550123412159, -0.6129070536817295}, {
      0.7939903986135404, -0.6079302977235295}, {
      0.7977944395044565, -0.6029295417181184}, {
      0.8015669848369434, -0.5979049830867808}, {
      0.8053078856773711, -0.5928568201904878}, {
      0.8090169943413799, -0.5877852523220669}, {
      0.8126941643997105, -0.5826904796983337}, {
      0.8163392506839856, -0.5775727034521875}, {
      0.8199521092924399, -0.5724321256246716}, {
      0.8235325975956017, -0.5672689491569965}, {
      0.8270805742419237, -0.5620833778825285}, {
      0.830595899163363, -0.5568756165187422}, {
      0.8340784335809108, -0.5516458706591393}, {
      0.8375280400100714, -0.5463943467651314}, {
      0.8409445822662897, -0.54112125215789}, {
      0.8443279254703273, -0.5358267950101611}, {
      0.8476779360535879, -0.5305111843380472}, {
      0.8509944817633898, -0.5251746299927561}, {
      0.854277431668187, -0.5198173426523156}, {
      0.8575266561627388, -0.5144395338132567}, {
      0.8607420269732257, -0.5090414157822641}, {
      0.8639234171623135, -0.5036232016677947}, {
      0.8670707011341651, -0.4981851053716642}, {
      0.8701837546393981, -0.4927273415806029}, {
      0.8732624547799908, -0.48725012575778}, {
      0.876306680014133, -0.4817536741342977}, {
      0.8793163101610252, -0.4762382037006547}, {
      0.8822912264056223, -0.4707039321981798}, {
      0.8852313113033249, -0.4651510781104356}, {
      0.8881364487846155, -0.45957986065459355`}, {
      0.8910065241596407, -0.45399049977277933`}, {
      0.893841424122739, -0.44838321612339005`}, {
      0.8966410367569141, -0.442758231072383}, {
      0.8994052515382528, -0.43711576668453644`}, {
      0.9021339593402886, -0.431456045714683}, {
      0.9048270524383099, -0.42577929159891553`}, {
      0.9074844245136123, -0.42008572844576636`}, {
      0.9101059706576966, -0.4143755810273599}, {
      0.9126915873764098, -0.4086490747705388}, {
      0.9152411725940309, -0.4029064357479648}, {
      0.9177546256573014, -0.39714789066919354`}, {
      0.9202318473393979, -0.3913736668717245}, {
      0.9226727398438501, -0.3855839923120261}, {
      0.9250772068084014, -0.37977909555653655`}, {
      0.9274451533088132, -0.37395920577264014`}, {
      0.9297764858626121, -0.3681245527196203}, {
      0.9320711124327808, -0.36227536673958904`}, {
      0.9343289424313915, -0.3564118787483934}, {
      0.9365498867231817, -0.35053432022649944`}, {
      0.9387338576290737, -0.34464292320985357`}, {
      0.9408807689296356, -0.3387379202807223}, {
      0.9429905358684854, -0.33281954455851037`}, {
      0.9450630751556367, -0.32688802969055736`}, {
      0.9470983049707874, -0.32094360984291387`}, {
      0.9490961449665493, -0.31498651969109703`}, {
      0.95105651627162, -0.30901699441082586`}, {
      0.9529793414938972, -0.30303526966873695`}, {
      0.9548645447235334, -0.2970415816130808}, {
      0.9567120515359333, -0.29103616686439904`}, {
      0.9585217889946912, -0.285019262506183}, {
      0.960293685654471, -0.2789911060755141}, {
      0.9620276715638266, -0.2729519355536864}, {
      0.9637236782679633, -0.2669019893568114}, {
      0.9653816388114405, -0.26084150632640585`}, {
      0.9670014877408147, -0.2547707257199625}, {
      0.968583161107224, -0.2486898872015049}, {
      0.9701265964689119, -0.24259923083212565`}, {
      0.9716317328936932, -0.23649899706050936`}, {
      0.9730985109613591, -0.2303894267134401}, {
      0.974526872766023, -0.2242707609862939}, {
      0.9759167619184066, -0.21814324143351682`}, {
      0.9772681235480659, -0.21200710995908884`}, {
      0.9785809043055578, -0.2058626088069738}, {
      0.9798550523645457, -0.19970998055155614`}, {
      0.9810905174238461, -0.19354946808806442`}, {
      0.9822872507094138, -0.1873813146229822}, {
      0.9834452049762678, -0.18120576366444668`}, {
      0.9845643345103563, -0.1750230590126355}, {
      0.9856445951303617, -0.16883344475014186`}, {
      0.9866859441894444, -0.16263716523233848`}, {
      0.9876883405769266, -0.15643446507773104`}, {
      0.9886517447199151, -0.1502255891583009}, {
      0.9895761185848643, -0.14401078258983804`}, {
      0.9904614256790765, -0.1377902907222644}, {
      0.9913076310521436, -0.13156435912994768`}, {
      0.9921147012973264, -0.12533323360200666`}, {
      0.9928826045528736, -0.1190971601326078}, {
      0.9936113105032794, -0.11285638491125378`}, {
      0.9943007903804807, -0.10661115431306446`}, {
      0.9949510169649924, -0.10036171488905031`}, {
      0.9955619645869825, -0.09410831335637902}, {
      0.996133609127285, -0.08785119658863559}, {
      0.9966659280183521, -0.0815906116060761}, {
      0.9971589002451454, -0.07532680556587584}, {
      0.9976125063459657, -0.06906002575237193}, {
      0.9980267284132206, -0.06279051956730095}, {
      0.9984015500941323, -0.056518534520031985`}, {
      0.9987369565913826, -0.050244318217795354`}, {
      0.9990329346636972, -0.04396811835590749}, {
      0.9992894726263688, -0.037690182707992376`}, {
      0.9995065603517177, -0.03141075911619985}, {
      0.9996841892694922, -0.025130095481421227`}, {
      0.999822352367207, -0.018848439753502592`}, {
      0.9999210441904193, -0.012566039921456171`}, {
      0.999980260842945, -0.00628314400367015}, {
      0.9999999999870121, -3.811732960518066*^-11}, {0.9999802608433529, 
      0.00628314392743699}, {0.9999210441912351, 0.012566039845227507`}, {
      0.9998223523684306, 0.018848439677281423`}, {0.9996841892711237, 
      0.025130095405210554`}, {0.9995065603537567, 0.03141075904000267}, {
      0.9992894726288154, 0.03769018263181169}, {0.9990329346665512, 
      0.04396811827974628}, {0.9987369565946437, 0.05024431814165661}, {
      0.9984015500978003, 0.05651853444391871}, {0.9980267284172953, 
      0.06279051949121613}, {0.9976125063504468, 0.06906002567631854}, {
      0.9971589002500327, 0.07532680548985687}, {0.9966659280236452, 
      0.08159061153009453}, {0.9961336091329835, 0.0878511965126944}, {
      0.9955619645930862, 0.09410831328048118}, {0.9949510169715008, 
      0.10036171481319878`}, {0.9943007903873933, 0.10661115423726222`}, {
      0.9936113105105957, 0.11285638483550378`}, {0.9928826045605932, 
      0.119097160056913}, {0.9921147013054488, 0.12533323352637002`}, {
      0.9913076310606682, 0.13156435905437214`}, {0.9904614256880028, 
      0.1377902906467529}, {0.9895761185941916, 0.14401078251439356`}, {
      0.9886517447296429, 0.15022558908292633`}, {0.9876883405870541, 
      0.1564344650024293}, {0.9866859441999711, 0.16263716515711252`}, {
      0.9856445951412869, 0.16883344467499456`}, {0.9845643345216791, 
      0.17502305893756978`}, {0.9834452049879875, 0.1812057635894654}, {
      0.9822872507215297, 0.1873813145480883}, {0.9810905174363573, 
      0.19354946801326078`}, {0.9798550523774515, 0.19970998047684563`}, {
      0.9785809043188572, 0.20586260873235926`}, {0.9772681235617582, 
      0.21200710988457314`}, {0.9759167619324907, 0.21814324135910282`}, {
      0.9745268727804981, 0.22427076091198445`}, {0.9730985109762242, 
      0.23038942663923803`}, {0.9716317329089473, 0.2364989969864175}, {
      0.9701265964845539, 0.24259923075814677`}, {0.9685831611232529, 
      0.2486898871276418}, {0.9670014877572296, 0.25477072564621805`}, {
      0.9653816388282401, 0.2608415062527828}, {0.9637236782851466, 
      0.26690198928331255`}, {0.9620276715813925, 0.27295193548031443`}, {
      0.9602936856724182, 0.2789911060022718}, {0.9585217890130185, 
      0.2850192624330731}, {0.9567120515546396, 0.2910361667914243}, {
      0.9548645447426174, 0.29704158154024396`}, {0.9529793415133575, 
      0.30303526959604066`}, {0.9510565162914555, 0.3090169943382728}, {
      0.9490961449867586, 0.3149865196186899}, {0.9470983049913694, 
      0.32094360977065534`}, {0.9450630751765898, 0.3268880296184501}, {
      0.9429905358898082, 0.33281954448655704`}, {0.9408807689513268, 
      0.3387379202089255}, {0.9387338576511318, 0.34464292313821593`}, {
      0.9365498867456054, 0.35053432015502356`}, {0.9343289424541792, 
      0.3564118786770819}, {0.932071112455931, 0.36227536666844445`}, {
      0.9297764858861232, 0.3681245526486452}, {0.9274451533326836, 
      0.37395920570183705`}, {0.9250772068326296, 0.379779095485908}, {
      0.9226727398684345, 0.3855839922415746}, {0.9202318473643368, 
      0.39137366680145247`}, {0.9177546256825933, 0.39714789059910355`}, {
      0.9152411726196741, 0.40290643567805934`}, {0.9126915874024025, 
      0.4086490747008203}, {0.9101059706840373, 0.4143755809578308}, {
      0.9074844245402992, 0.4200857283764291}, {0.9048270524653411, 
      0.42577929152977245`}, {0.9021339593676625, 0.43145604564573653`}, {
      0.8994052515659675, 0.43711576661578894`}, {0.8966410367849679, 
      0.4427582310038368}, {0.89384142415113, 0.44838321605504755`}, {
      0.891006524188367, 0.4539904997046428}, {0.8881364488136753, 
      0.4595798605866652}, {0.8852313113327162, 0.4651510780427178}, {
      0.8822912264353432, 0.4707039321306748}, {0.8793163101910738, 
      0.4762382036333648}, {0.8763066800445074, 0.481753674067225}, {
      0.8732624548106889, 0.48725012569092674`}, {0.870183754670418, 
      0.4927273415139713}, {0.8670707011655046, 0.49818510530525645`}, {
      0.8639234171939707, 0.5036232016016129}, {0.8607420270051985, 
      0.5090414157163105}, {0.8575266561950252, 0.5144395337475333}, {
      0.8542774317007849, 0.5198173425868244}, {0.8509944817962969, 
      0.5251746299274993}, {0.8476779360868022, 0.5305111842730268}, {
      0.8443279255038466, 0.535826794945379}, {0.8409445823001118, 
      0.5411212520933484}, {0.8375280400441942, 0.5463943467008323}, {
      0.8340784336153321, 0.5516458705950845}, {0.8305958991980804, 
      0.5568756164549338}, {0.8270805742769352, 0.5620833778189683}, {
      0.8235325976309048, 0.5672689490936865}, {0.8199521093280324, 
      0.5724321255616136}, {0.8163392507198652, 0.5775727033893834}, {
      0.812694164435875, 0.5826904796357852}, {0.8090169943778269, 
      0.5877852522597761}, {0.8053078857140984, 0.5928568201284563}, {
      0.8015669848739485, 0.5979049830250104}, {0.797794439541737, 
      0.602929541656611}, {0.7939903986510939, 0.6079302976622866}, {
      0.7901550123790401, 0.6129070536207529}, {0.7862884321400583, 
      0.6178596130582062}, {0.7823908105801157, 0.6227877804560801}, {
      0.7784623015706378, 0.6276913612587639}, {0.7745030602024335, 
      0.632570161881284}, {0.7705132427795728, 0.6374239897169455}, {
      0.7664930068132157, 0.6422526531449365}, {0.7624425110153947, 
      0.647055961537893}, {0.7583619152927482, 0.6518337252694242}, {
      0.7542513807402081, 0.6565857557215986}, {0.7501110696346404, 
      0.6613118652923909}, {0.745941145428438, 0.6660118674030877}, {
      0.7417417727430687, 0.6706855765056532}, {0.7375131173625755, 
      0.6753328080900547}, {0.7332553462270323, 0.6799533786915466}, {
      0.7289686274259529, 0.6845471058979132}, {0.7246531301916558, 
      0.6891138083566701}, {0.7203090248925823, 0.6936533057822237}, {
      0.7159364830265713, 0.6981654189629884}, {0.7115356772140888, 
      0.702649969768462}, {0.7071067811914128, 0.7071067811562571}, {
      0.7026499698037748, 0.7115356771790918}, {0.6981654189984572, 
      0.7159364829917344}, {0.6936533058178469, 0.7203090248579067}, {
      0.6891138083924463, 0.7246531301571429}, {0.6845471059338409, 
      0.7289686273916041}, {0.6799533787276245, 0.7332553461928487}, {
      0.6753328081262813, 0.7375131173285585}, {0.670685576542027, 
      0.7417417727092195}, {0.6660118674396072, 0.7459411453947579}, {
      0.6613118653290546, 0.7501110696011305}, {0.6565857557584049, 
      0.7542513807068698}, {0.6518337253063715, 0.7583619152595825}, {
      0.6470559615749799, 0.762442510982403}, {0.6422526531821613, 
      0.7664930067803991}, {0.6374239897543066, 0.7705132427469323}, {
      0.6325701619187798, 0.7745030601699704}, {0.6276913612963928, 
      0.778462301538353}, {0.6227877804938404, 0.7823908105480104}, {
      0.6178596130960965, 0.7862884321081335}, {0.6129070536587714, 
      0.7901550123472969}, {0.6079302977004316, 0.7939903986195334}, {
      0.6029295416948808, 0.79779443951036}, {0.5979049830634035, 
      0.8015669848427561}, {0.5928568201669709, 0.8053078856830916}, {
      0.5877852522984105, 0.8090169943470067}, {0.5826904796745377, 
      0.8126941644052423}, {0.5775727034282521, 0.8163392506894209}, {
      0.5724321256005969, 0.8199521092977773}, {0.5672689491327826, 
      0.8235325976008399}, {0.5620833778581754, 0.8270805742470612}, {
      0.5568756164942502, 0.8305958991683983}, {0.5516458706345084, 
      0.8340784335858424}, {0.5463943467403619, 0.837528040014898}, {
      0.5411212521329819, 0.8409445822710098}, {0.5358267949851145, 
      0.8443279254749395}, {0.5305111843128625, 0.8476779360580907}, {
      0.5251746299674334, 0.8509944817677817}, {0.519817342626855, 
      0.8542774316724666}, {0.5144395337876585, 0.8575266561669046}, {
      0.5090414157565286, 0.8607420269772761}, {0.5036232016419221, 
      0.8639234171662472}, {0.49818510534565474`, 0.8670707011379806}, {
      0.4927273415544568, 0.8701837546430939}, {0.4872501257314976, 
      0.8732624547835655}, {0.48175367410787934`, 0.876306680017585}, {
      0.4762382036741007, 0.8793163101643531}, {0.47070393217149037`, 
      0.8822912264088246}, {0.46515107808361117`, 0.8852313113064002}, {
      0.4595798606276344, 0.8881364487875623}, {0.45399049974568584`, 
      0.8910065241624574}, {0.4483832160961626, 0.8938414241254242}, {
      0.44275823104502193`, 0.8966410367594663}, {0.43711576665704216`, 
      0.8994052515406704}, {0.43145604568705587`, 0.9021339593425702}, {
      0.425779291571156, 0.904827052440454}, {0.4200857284178749, 
      0.9074844245156175}, {0.4143755809993369, 0.9101059706595613}, {
      0.40864907474238477`, 0.9126915873781325}, {0.40290643571968016`, 
      0.9152411725956102}, {0.39714789064077877`, 0.9177546256587357}, {
      0.3913736668431801, 0.9202318473406856}, {0.38558399228335266`, 
      0.9226727398449899}, {0.37977909552773453`, 0.9250772068093917}, {
      0.37395920574371005`, 0.9274451533096524}, {0.3681245526905627, 
      0.9297764858632989}, {0.3622753667104045, 0.9320711124333136}, {
      0.35641187871908253`, 0.9343289424317688}, {0.35053432019706277`, 
      0.9365498867234019}, {0.3446429231802917, 0.9387338576291353}, {
      0.33873792025103594`, 0.9408807689295371}, {0.33281954452870005`, 
      0.9429905358682253}, {0.32688802966062375`, 0.9450630751552137}, {
      0.32094360981285763`, 0.9470983049701999}, {0.31498651966091884`, 
      0.9490961449657956}, {0.30901699438052643`, 0.9510565162706989}, {
      0.303035269638317, 0.952979341492807}, {0.297041581582541, 
      0.9548645447222728}, {0.2910361668337401, 0.9567120515345006}, {
      0.28501926247540565`, 0.9585217889930849}, {0.27899110604461913`, 
      0.9602936856526897}, {0.27295193552267455`, 0.9620276715618687}, {
      0.2669019893256835, 0.9637236782658274}, {0.2608415062951626, 
      0.965381638809125}, {0.25477072568860476`, 0.9670014877383182}, {
      0.24868988717003346`, 0.9685831611045449}, {0.24259923080054135`, 
      0.9701265964660488}, {0.23649899702881308`, 0.9716317328906446}, {
      0.23038942668163268`, 0.9730985109581236}, {0.2242707609543762, 
      0.9745268727625991}, {0.21814324140148975`, 0.9759167619147927}, {
      0.21200710992695326`, 0.9772681235442606}, {0.20586260877473067`, 
      0.9785809043015596}, {0.19970998051920638`, 0.9798550523603533}, {
      0.19354946805560896`, 0.9810905174194579}, {0.18738131459042198`, 
      0.9822872507048283}, {0.1812057636317827, 0.9834452049714835}, {
      0.17502305897986875`, 0.9845643345053718}, {0.16883344471727332`, 
      0.9856445951251755}, {0.1626371651993692, 0.9866859441840551}, {
      0.156434465044662, 0.9876883405713327}, {0.15022558912513315`, 
      0.9886517447141153}, {0.14401078255657268`, 0.989576118578857}, {
      0.13779029068890247`, 0.9904614256728603}, {0.13156435909649028`, 
      0.9913076310457171}, {0.12533323356845488`, 0.9921147012906881}, {
      0.11909716009896273`, 0.9928826045460221}, {0.11285638487751658`, 
      0.9936113104962132}, {0.10661115427923624`, 0.9943007903731984}, {
      0.10036171485513223`, 0.9949510169574928}, {0.09410831332237225, 
      0.995561964579264}, {0.08785119655454131, 0.9961336091193461}, {
      0.08159061157189551, 0.9966659280101915}, {0.07532680553161014, 
      0.9971589002367618}, {0.06906002571802233, 0.9976125063373575}, {
      0.06279051953286871, 0.9980267284043867}, {0.05651853448551835, 
      0.9984015500850711}, {0.05024431818320158, 0.9987369565820928}, {
      0.043968118321234866`, 0.9990329346541775}, {0.03769018267324219, 
      0.9992894726166177}, {0.031410759081373404`, 0.9995065603417339}, {
      0.02513009544651984, 0.9996841892592744}, {0.0188484397185276, 
      0.9998223523567537}, {0.012566039886408925`, 0.9999210441797293}, {
      0.006283143968552009, 0.999980260832017}, {2.929670395168671*^-12, 
      0.9999999999758447}, {-0.006283143962692779, 
      0.9999802608319448}, {-0.012566039880550025`, 
      0.9999210441795849}, {-0.01884843971266925, 
      0.9998223523565372}, {-0.025130095440662262`, 
      0.9996841892589855}, {-0.03141075907551681, 
      0.9995065603413726}, {-0.0376901826673868, 
      0.9992894726161841}, {-0.0439681183153809, 
      0.9990329346536714}, {-0.050244318177349256`, 
      0.9987369565815141}, {-0.05651853447966788, 
      0.9984015500844197}, {-0.0627905195270203, 
      0.9980267284036624}, {-0.0690600257121762, 
      0.9976125063365603}, {-0.0753268055257665, 
      0.9971589002358914}, {-0.08159061156605456, 
      0.9966659280092478}, {-0.08785119654870324, 
      0.9961336091183289}, {-0.09410831331653727, 
      0.9955619645781731}, {-0.10036171484930052`, 
      0.9949510169563279}, {-0.106611154273408, 
      0.9943007903719594}, {-0.11285638487169199`, 
      0.9936113104948997}, {-0.11909716009314199`, 
      0.9928826045446338}, {-0.12533323356263815`, 
      0.992114701289225}, {-0.13156435909067776`, 
      0.9913076310441787}, {-0.13779029068309434`, 
      0.9904614256712465}, {-0.1440107825507691, 
      0.9895761185771674}, {-0.15022558911933429`, 
      0.9886517447123496}, {-0.156434465038868, 
      0.9876883405694905}, {-0.16263716519358018`, 
      0.9866859441821361}, {-0.16883344471148948`, 
      0.9856445951231794}, {-0.1750230589740902, 
      0.984564334503298}, {-0.18120576362600957`, 
      0.9834452049693317}, {-0.1873813145846544, 
      0.9822872507025981}, {-0.19354946804984705`, 
      0.981090517417149}, {-0.19970998051345026`, 
      0.9798550523579653}, {-0.20586260876898044`, 
      0.9785809042990921}, {-0.21200710992120902`, 
      0.9772681235417129}, {-0.21814324139575159`, 
      0.9759167619121644}, {-0.2242707609486442, 
      0.9745268727598897}, {-0.23038942667590692`, 
      0.9730985109553327}, {-0.23649899702309363`, 
      0.9716317328877717}, {-0.24259923079482829`, 
      0.9701265964630933}, {-0.2486898871643268, 
      0.9685831611015062}, {-0.2547707256829046, 
      0.9670014877351959}, {-0.26084150628946895`, 
      0.9653816388059184}, {-0.26690198931999637`, 
      0.963723678262536}, {-0.272951935516994, 
      0.962027671558492}, {-0.2789911060389451, 
      0.960293685649227}, {-0.2850192624697382, 
      0.9585217889895357}, {-0.2910361668280792, 
      0.9567120515308641}, {-0.29704158157688665`, 
      0.9548645447185485}, {-0.30303526963266914`, 
      0.9529793414889943}, {-0.309016994374885, 
      0.9510565162667971}, {-0.3149865196552838, 
      0.9490961449618042}, {-0.3209436098072289, 
      0.9470983049661179}, {-0.32688802965500124`, 
      0.9450630751510406}, {-0.3328195445230837, 
      0.9429905358639604}, {-0.33873792024542565`, 
      0.9408807689251797}, {-0.34464292317468737`, 
      0.9387338576246846}, {-0.35053432019146424`, 
      0.9365498867188573}, {-0.35641187871348967`, 
      0.9343289424271295}, {-0.3622753667048172, 
      0.9320711124285789}, {-0.3681245526849808, 
      0.929776485858468}, {-0.37395920573813335`, 
      0.9274451533047245}, {-0.3797790955221628, 
      0.925077206804366}, {-0.3855839922777858, 
      0.9226727398398656}, {-0.3913736668376179, 
      0.9202318473354621}, {-0.39714789063522093`, 
      0.917754625653412}, {-0.4029064357141265, 
      0.9152411725901857}, {-0.408649074736835, 
      0.9126915873726063}, {-0.4143755809937908, 
      0.9101059706539325}, {-0.42008572841233216`, 
      0.9074844245098853}, {-0.4257792915656164, 
      0.9048270524346176}, {-0.4314560456815191, 
      0.9021339593366287}, {-0.4371157666515078, 
      0.8994052515346228}, {-0.44275823103948975`, 
      0.8966410367533117}, {-0.4483832160906322, 
      0.8938414241191619}, {-0.45399049974015687`, 
      0.8910065241560865}, {-0.4595798606221065, 
      0.8881364487810818}, {-0.46515107807808387`, 
      0.8852313112998093}, {-0.47070393216596335`, 
      0.8822912264021223}, {-0.4762382036685735, 
      0.8793163101575385}, {-0.48175367410235154`, 
      0.8763066800106574}, {-0.4872501257259687, 
      0.8732624547765236}, {-0.4927273415489264, 
      0.8701837546359371}, {-0.4981851053401223, 
      0.8670707011307077}, {-0.5036232016363871, 
      0.8639234171588575}, {-0.5090414157509906, 
      0.8607420269697684}, {-0.5144395337821169, 
      0.857526656159278}, {-0.5198173426213093, 
      0.8542774316647203}, {-0.5251746299618829, 
      0.8509944817599145}, {-0.5305111843073067, 
      0.8476779360501018}, {-0.5358267949795528, 
      0.8443279254668279}, {-0.5411212521274136, 
      0.8409445822627745}, {-0.5463943467347864, 
      0.837528040006538}, {-0.5516458706289251, 
      0.8340784335773568}, {-0.5568756164886585, 
      0.8305958991597858}, {-0.5620833778525746, 
      0.827080574238321}, {-0.5672689491271719, 
      0.8235325975919711}, {-0.5724321255949755, 
      0.8199521092887788}, {-0.5775727034226193, 
      0.8163392506802917}, {-0.5826904796688929, 
      0.8126941643959814}, {-0.5877852522927528, 
      0.8090169943376131}, {-0.5928568201612996, 
      0.8053078856735643}, {-0.5979049830577177, 
      0.801566984833094}, {-0.6029295416891797, 
      0.797794439500562}, {-0.6079302976947142, 
      0.7939903986095984}, {-0.6129070536530369, 
      0.7901550123372241}, {-0.617859613090344, 
      0.7862884320979218}, {-0.622787780488069, 
      0.7823908105376587}, {-0.6276913612906015, 
      0.7784623015278603}, {-0.6325701619129677, 
      0.7745030601593357}, {-0.6374239897484727, 
      0.7705132427361546}, {-0.6422526531763048, 
      0.7664930067694774}, {-0.6470559615690996, 
      0.7624425109713362}, {-0.6518337253004665, 
      0.7583619152483696}, {-0.656585755752474, 
      0.7542513806955097}, {-0.6613118653230968, 
      0.7501110695896223}, {-0.6660118674336214, 
      0.7459411453831004}, {-0.6706855765360121, 
      0.7417417726974117}, {-0.6753328081202362, 
      0.7375131173165994}, {-0.679953378721548, 
      0.7332553461807373}, {-0.684547105927732, 
      0.7289686273793394}, {-0.6891138083863037, 
      0.724653130144724}, {-0.6936533058116693, 
      0.7203090248453324}, {-0.6981654189922435, 
      0.7159364829790038}, {-0.7026499697975238, 
      0.7115356771662039}, {-0.7071067811851232, 
      0.7071067811432109}, {-0.7115356772077592, 
      0.7026499697552563}, {-0.7159364830202005, 
      0.6981654189496224}, {-0.7203090248861689, 
      0.6936533057686962}, {-0.7246531301851985, 
      0.6891138083429803}, {-0.7289686274194505, 
      0.6845471058840601}, {-0.7332553462204832, 
      0.6799533786775291}, {-0.7375131173559784, 
      0.6753328080758719}, {-0.7417417727364221, 
      0.6706855764913042}, {-0.7459411454217404, 
      0.6660118673885715}, {-0.7501110696278903, 
      0.6613118652777066}, {-0.7542513807334041, 
      0.656585755706745}, {-0.7583619152858888, 
      0.6518337252544004}, {-0.7624425110084784, 
      0.6470559615226982}, {-0.7664930068062409, 
      0.6422526531295697}, {-0.7705132427725379, 
      0.6374239897014057}, {-0.7745030601953369, 
      0.6325701618655704}, {-0.7784623015634778, 
      0.6276913612428755}, {-0.7823908105728908, 
      0.6227877804400159}, {-0.7862884321327668, 
      0.6178596130419656}, {-0.7901550123716803, 
      0.6129070536043348}, {-0.7939903986436642, 
      0.6079302976456902}, {-0.7977944395342356, 
      0.6029295416398354}, {-0.8015669848663737, 
      0.5979049830080548}, {-0.8053078857064484, 
      0.5928568201113199}, {-0.8090169943701, 
      0.5877852522424579}, {-0.8126941644280694, 
      0.5826904796182845}, {-0.816339250711979, 
      0.5775727033716992}, {-0.8199521093200637, 
      0.5724321255437452}, {-0.8235325976228518, 
      0.5672689490756331}, {-0.827080574268796, 
      0.562083377800729}, {-0.8305958991898532, 
      0.556875616436508}, {-0.8340784336070148, 
      0.5516458705764713}, {-0.8375280400357851, 
      0.5463943466820309}, {-0.8409445822916088, 
      0.5411212520743581}, {-0.8443279254952478, 
      0.5358267949261991}, {-0.8476779360781056, 
      0.5305111842536565}, {-0.8509944817875004, 
      0.525174629907938}, {-0.8542774316918864, 
      0.5198173425670713}, {-0.8575266561860228, 
      0.5144395337275878}, {-0.8607420269960899, 
      0.509041415696172}, {-0.863923417184754, 
      0.5036232015812808}, {-0.8670707011561776, 
      0.49818510528473}, {-0.8701837546609785, 
      0.49272734149324987`}, {-0.8732624548011348, 
      0.48725012567000975`}, {-0.8763066800348366, 
      0.48175367404611186`}, {-0.879316310181284, 
      0.47623820361205493`}, {-0.8822912264254321, 
      0.47070393210916767`}, {-0.8852313113226816, 
      0.46515107802101285`}, {-0.8881364488035149, 
      0.4595798605647619}, {-0.8910065241780787, 
      0.45399049968254057`}, {-0.8938414241407113, 
      0.448383216032746}, {-0.8966410367744165, 
      0.4427582309813355}, {-0.8994052515552812, 
      0.4371157665930873}, {-0.9021339593568388, 
      0.43145604562283413`}, {-0.9048270524543778, 
      0.4257792915066689}, {-0.9074844245291938, 
      0.42008572835312397`}, {-0.9101059706727873, 
      0.4143755809343237}, {-0.9126915873910056, 
      0.4086490746771109}, {-0.9152411726081279, 
      0.4029064356541473}, {-0.9177546256708952, 
      0.3971478905749885}, {-0.9202318473524844, 
      0.3913736667771341}, {-0.9226727398564252, 
      0.3855839922170526}, {-0.9250772068204609, 
      0.3797790954611821}, {-0.9274451533203529, 
      0.373959205676907}, {-0.929776485873628, 
      0.36812455262351074`}, {-0.9320711124432688, 
      0.3622753666431054}, {-0.9343289424413473, 
      0.35641187865153806`}, {-0.9365498867326013, 
      0.3505343201292748}, {-0.9387338576379529, 
      0.34464292311226213`}, {-0.9408807689379703, 
      0.33873792018276655`}, {-0.9429905358762716, 
      0.33281954446019274`}, {-0.9450630751628704, 
      0.3268880295918804}, {-0.9470983049774644, 
      0.3209436097438802}, {-0.9490961449726655, 
      0.31498651959170926`}, {-0.9510565162771714, 
      0.30901699431108665`}, {-0.9529793414988796, 
      0.303035269568649}, {-0.954864544727943, 
      0.29704158151264687`}, {-0.9567120515397659, 
      0.2910361667636218}, {-0.9585217889979428, 
      0.28501926240506525`}, {-0.9602936856571376, 
      0.27899110597405874`}, {-0.9620276715659041, 
      0.27295193545189633`}, {-0.9637236782694477, 
      0.2669019892546895}, {-0.9653816388123279, 
      0.260841506223955}, {-0.9670014877411011, 
      0.2547707256171857}, {-0.9685831611069053, 
      0.2486898870984052}, {-0.9701265964679844, 
      0.2425992307287061}, {-0.9716317328921529, 
      0.23649899695677307`}, {-0.973098510959202, 
      0.23038942660939016`}, {-0.9745268727632451, 
      0.22427076088193346`}, {-0.9759167619150041, 
      0.21814324132884907`}, {-0.9772681235440349, 
      0.21200710985411703`}, {-0.9785809043008944, 
      0.2058626087017012}, {-0.9798550523592461, 
      0.19970998044598606`}, {-0.9810905174179063, 
      0.19354946798220019`}, {-0.9822872507028301, 
      0.18738131451682716`}, {-0.9834452049690363, 
      0.18120576355800427`}, {-0.9845643345024733, 
      0.1750230589059092}, {-0.9856445951218233, 
      0.1688334446431351}, {-0.9866859441802468, 
      0.1626371651250548}, {-0.987688340567066, 
      0.156434464970174}, {-0.9886517447093879, 
      0.15022558905047406`}, {-0.9895761185736666, 
      0.14401078248174506`}, {-0.9904614256672046, 
      0.13779029061390893`}, {-0.9913076310395939, 
      0.13156435902133343`}, {-0.9921147012840951, 
      0.12533323349313735`}, {-0.9928826045389569, 
      0.11909716002348719`}, {-0.9936113104886739, 
      0.1128563848018857}, {-0.9943007903651827, 
      0.10661115420345274`}, {-0.9949510169489983, 
      0.10036171477919881`}, {-0.9955619645702887, 
      0.09410831324629167}, {-0.9961336091098877, 
      0.08785119647831632}, {-0.9966659280002478, 
      0.0815906114955289}, {-0.9971589002263307, 
      0.07532680545510471}, {-0.9976125063264369, 
      0.06906002564138092}, {-0.9980267283929745, 
      0.06279051945609415}, {-0.9984015500731652, 
      0.05651853440861352}, {-0.9987369565696911, 
      0.05024431810616938}, {-0.9990329346412778, 
      0.0439681182440782}, {-0.9992894726032181, 
      0.037690182595963996`}, {-0.9995065603278322, 
      0.031410759003976635`}, {-0.9996841892448687, 
      0.02513009536900747}, {-0.9998223523418419, 
      0.01884843964090262}, {-0.9999210441643095, 
      0.012566039808674353`}, {-0.9999802608160872, 
      0.006283143890710882}, {-0.9999999999594028, -7.501495605244379*^-11}, \
{-0.999980260814989, -0.006283144040737828}, {-0.9999210441621134, \
-0.012566039958692397`}, {-0.9998223523385478, -0.018848439790905832`}, \
{-0.9996841892404765, -0.02513009551898991}, {-0.9995065603223421, \
-0.03141075915393238}, {-0.99928947259663, -0.037690182745887105`}, \
{-0.9990329346335921, -0.04396811839396274}, {-0.9987369565609078, \
-0.05024431825600941}, {-0.9984015500632845, -0.05651853455840311}, \
{-0.9980267283819965, -0.06279051960582736}, {-0.9976125063143619, \
-0.06906002579105182}, {-0.9971589002131588, -0.07532680560470735}, \
{-0.9966659279859793, -0.08159061164505735}, {-0.9961336090945228, \
-0.08785119662776464}, {-0.9955619645538276, -0.09410831339565391}, \
{-0.9949510169314414, -0.10036171492846904`}, {-0.9943007903465302, \
-0.10661115435262501`}, {-0.9936113104689263, -0.11285638495095407`}, \
{-0.9928826045181145, -0.1190971601724457}, {-0.9921147012621581, \
-0.12533323364198007`}, {-0.9913076310165627, -0.1315643591700544}, \
{-0.9904614256430797, -0.1377902907625022}, {-0.9895761185484484, \
-0.1440107826302047}, {-0.9886517446830768, -0.15022558919879409`}, \
{-0.9876883405396625, -0.15643446511834846`}, {-0.9866859441517514, \
-0.16263716527307778`}, {-0.9856445950922365, -0.1688334447910006}, \
{-0.9845643344717955, -0.17502305905361126`}, {-0.9834452049372682, \
-0.18120576370553695`}, {-0.9822872506699721, -0.18738131466418448`}, \
{-0.9810905173839591, -0.19354946812937618`}, {-0.9798550523242102, \
-0.1997099805929748}, {-0.9785809042647705, -0.2058626088484967}, \
{-0.9772681235068236, -0.2120071100007133}, {-0.975916761876706, \
-0.21814324147524014`}, {-0.974526872723861, -0.22427076102811333`}, \
{-0.9730985109187325, -0.23038942675535287`}, {-0.9716317328505987, \
-0.23649899710251265`}, {-0.9701265964253463, -0.24259923087421656`}, \
{-0.9685831610631841, -0.24868988724368055`}, {-0.9670014876962975, \
-0.25477072576222}, {-0.9653816387664427, -0.26084150636874215`}, \
{-0.9637236782224817, -0.26690198939922355`}, {-0.9620276715178582, \
-0.27295193559617126`}, {-0.9602936856080125, -0.2789911061180686}, \
{-0.9585217889477395, -0.28501926254880405`}, {-0.9567120514884854, \
-0.2910361669070835}, {-0.9548645446755861, -0.29704158165582545`}, \
{-0.9529793414454474, -0.3030352697115385}, {-0.9510565162226646, \
-0.30901699445368097`}, {-0.9490961449170853, -0.31498651973400243`}, \
{-0.9470983049208117, -0.3209436098858662}, {-0.9450630751051462, \
-0.3268880297335532}, {-0.942990535817477, -0.3328195446015463}, \
{-0.9408807688781066, -0.3387379203237948}, {-0.9387338575770209, \
-0.3446429232529591}, {-0.9365498866706022, -0.3505343202696345}, \
{-0.9343289423782823, -0.3564118787915544}, {-0.932071112379139, \
-0.3622753667827723}, {-0.9297764858084346, -0.36812455276282213`}, \
{-0.9274451532540972, -0.37395920581585684`}, {-0.925077206753144, \
-0.37977909559976436`}, {-0.9226727397880483, -0.3855839923552612}, \
{-0.920231847283049, -0.391373666914963}, {-0.9177546256004024, \
-0.3971478907124316}, {-0.9152411725365791, -0.40290643579119856`}, \
{-0.9126915873184023, -0.4086490748137643}, {-0.9101059705991307, \
-0.4143755810705731}, {-0.9074844244544852, -0.4200857284889633}, \
{-0.9048270523786187, -0.4257792916420921}, {-0.9021339592800308, \
-0.4314560457578351}, {-0.8994052514774256, -0.4371157667276599}, \
{-0.8966410366955148, -0.44275823111547363`}, {-0.8938414240607649, \
-0.44838321616644367`}, {-0.8910065240970892, -0.4539904998157916}, \
{-0.888136448721484, -0.4595798606975602}, {-0.8852313112396107, \
-0.46515107815335227`}, {-0.882291226341323, -0.4707039322410421}, \
{-0.8793163100961381, -0.47623820374345827`}, {-0.8763066799486557, \
-0.481753674177038}, {-0.8732624547139207, -0.48725012580045257`}, \
{-0.8701837545727328, -0.49272734162320325`}, {-0.867070701066902, \
-0.4981851054141878}, {-0.8639234170944503, -0.5036232017102369}, \
{-0.86074202690476, -0.5090414158246203}, {-0.8575266560936683, \
-0.5144395338555221}, {-0.8542774315985092, -0.5198173426944855}, \
{-0.8509944816931024, -0.5251746300348258}, {-0.8476779359826887, \
-0.5305111843800119}, {-0.844327925398814, -0.5358267950520158}, \
{-0.8409445821941599, -0.54112125219963}, {-0.837528039937323, \
-0.5463943468067518}, {-0.8340784335075417, -0.5516458707006351}, \
{-0.830595899089371, -0.5568756165601084}, {-0.8270805741673068, \
-0.5620833779237601}, {-0.8235325975203578, -0.5672689491980885}, \
{-0.8199521092165668, -0.5724321256656187}, {-0.8163392506074814, \
-0.5775727034929847}, {-0.8126941643225732, -0.5826904797389758}, \
{-0.8090169942636075, -0.5877852523625489}, {-0.8053078855989618, \
-0.5928568202308042}, {-0.8015669847578952, -0.5979049831269264}, \
{-0.7977944394247676, -0.602929541758088}, {-0.793990398533209, \
-0.6079302977633176}, {-0.7901550122602402, -0.6129070537213309}, \
{-0.7862884320203442, -0.617859613158324}, {-0.782390810459488, \
-0.6227877805557307}, {-0.7784623014490972, -0.6276913613579402}, \
{-0.7745030600799809, -0.6325701619799788}, {-0.7705132426562091, \
-0.6374239898151517}, {-0.7664930066889419, -0.642252653242647}, \
{-0.7624425108902116, -0.6470559616351006}, {-0.7583619151666569, \
-0.6518337253661216}, {-0.7542513806132097, -0.6565857558177788}, \
{-0.750111069506736, -0.6613118653880466}, {-0.7459411452996288, \
-0.6660118674982117}, {-0.7417417726133559, -0.6706855766002383}, \
{-0.7375131172319604, -0.6753328081840938}, {-0.7332553460955162, \
-0.6799533787850325}, {-0.7289686272935373, -0.6845471059908386}, \
{-0.724653130058342, -0.6891138084490278}, {-0.7203090247583718, \
-0.6936533058740064}, {-0.7159364828914657, -0.698165419054189}, \
{-0.7115356770780896, -0.7026499698590731}, {-0.7071067810545217, \
-0.7071067812462717}, {-0.7026499696659936, -0.7115356772685024}, \
{-0.6981654188597876, -0.7159364830805338}, {-0.6936533056782906, \
-0.7203090249460877}, {-0.6891138082520054, -0.7246531302446982}, \
{-0.6845471057925173, -0.7289686274785264}, {-0.67995337858542, \
-0.7332553462791308}, {-0.6753328079831981, -0.7375131174141931}, \
{-0.670685576398067, -0.7417417727941994}, {-0.6660118672947727, \
-0.7459411454790758}, {-0.6613118651833477, -0.750111069684779}, \
{-0.6565857556118279, -0.7542513807898417}, {-0.6518337251589266, \
-0.7583619153418705}, {-0.6470559614266694, -0.7624425110639997}, \
{-0.6422526530329877, -0.7664930068612973}, {-0.6374239896042724, \
-0.7705132428271246}, {-0.6325701617678875, -0.7745030602494496}, \
{-0.627691361144645, -0.7784623016171119}, {-0.6227877803412396, \
-0.7823908106260417}, {-0.6178596129426454, -0.7862884321854299}, \
{-0.6129070535044727, -0.790155012423851}, {-0.6079302975452882, \
-0.7939903986953379}, {-0.6029295415388954, -0.7977944395854079}, \
{-0.5979049829065791, -0.80156698491704}, {-0.5928568200093105, \
-0.8053078857566043}, {-0.5877852521399171, -0.8090169944197408}, \
{-0.5826904795152144, -0.8126941644771907}, {-0.577572703268102, \
-0.8163392507605762}, {-0.5724321254396231, -0.8199521093681323}, \
{-0.5672689489709885, -0.8235325976703873}, {-0.5620833776955642, \
-0.8270805743157938}, {-0.5568756163308252, -0.8305958992363089}, \
{-0.551645870470273, -0.8340784336529239}, {-0.5463943465753196, \
-0.837528040081143}, {-0.5411212519671362, -0.8409445823364111}, \
{-0.5358267948184692, -0.84432792553949}, {-0.5305111841454211, \
-0.8476779361217832}, {-0.5251746297991995, -0.850994481830609}, \
{-0.5198173424578325, -0.8542774317344216}, {-0.5144395336178511, \
-0.8575266562279801}, {-0.5090414155859402, -0.8607420270374649}, \
{-0.5036232014705565, -0.8639234172255422}, {-0.498185105173516, \
-0.8670707011963746}, {-0.492727341381549, -0.8701837547005801}, \
{-0.4872501255578248, -0.8732624548401364}, {-0.4817536739334457, \
-0.8763066800732339}, {-0.4762382034989104, -0.8793163102190725}, \
{-0.47070393199554766`, -0.8822912264626077}, {-0.46515107790692034`, \
-0.8852313113592398}, {-0.4595798604501999, -0.8881364488394514}, \
{-0.4539904995675121, -0.8910065242133891}, {-0.44838321591725405`, \
-0.8938414241753915}, {-0.44275823086538313`, -0.8966410368084621}, \
{-0.4371157664766777, -0.8994052515886878}, {-0.43145604550597044`, \
-0.9021339593896022}, {-0.42577929138935433`, -0.9048270524864935}, \
{-0.4200857282353617, -0.9074844245606577}, {-0.4143755808161171, \
-0.9101059707035953}, {-0.40864907455846317`, -0.9126915874211534}, \
{-0.4029064355350618, -0.9152411726376112}, {-0.3971478904554686, \
-0.9177546256997099}, {-0.3913736666571832, -0.9202318473806262}, \
{-0.3855839920966741, -0.92267273988389}, {-0.3797790953403795, \
-0.9250772068472446}, {-0.37395920555568385`, -0.9274451533464514}, \
{-0.36812455250187054`, -0.929776485899037}, {-0.36227536652105174`, \
-0.9320711124679844}, {-0.3564118785290746, -0.9343289424653652}, \
{-0.35053432000640505`, -0.9365498867559175}, {-0.3446429229889898, \
-0.9387338576605634}, {-0.33873792005909537`, -0.940880768959871}, \
{-0.3328195443361265, -0.9429905358974585}, {-0.32688802946742285`, \
-0.9450630751833394}, {-0.3209436096190351, -0.9470983049972116}, \
{-0.3149865194664805, -0.9490961449916868}, {-0.30901699418547807`, \
-0.951056516295463}, {-0.30303526944266457`, -0.9529793415164376}, \
{-0.29704158138629044`, -0.9548645447447633}, {-0.29103616663689746`, \
-0.9567120515558447}, {-0.28501926227797697`, -0.9585217890132762}, \
{-0.2789911058466106, -0.9602936856717218}, {-0.27295193532409234`, \
-0.9620276715797352}, {-0.26690198912653385`, -0.9637236782825219}, \
{-0.2608415060954518, -0.9653816388246413}, {-0.2547707254883392, \
-0.96700148775265}, {-0.2486898869692195, -0.968583161117686}, \
{-0.2425992305991855, -0.970126596477993}, {-0.23649899682692185`, \
-0.9716317329013858}, {-0.23038942647921265`, -0.9730985109676555}, \
{-0.22427076075143398`, -0.9745268727709155}, {-0.21814324119803202`, \
-0.9759167619218877}, {-0.2120071097229868, -0.9772681235501282}, \
{-0.20586260857026226`, -0.9785809043061936}, {-0.19970998031424286`, \
-0.9798550523637477}, {-0.19354946785015725`, -0.9810905174216068}, \
{-0.18738131438448904`, -0.9822872507057258}, {-0.18120576342537556`, \
-0.9834452049711238}, {-0.17502305877299446`, -0.984564334503749}, \
{-0.16883344450993898`, -0.9856445951222839}, {-0.16263716499158198`, \
-0.9866859441798886}, {-0.15643446483642917`, -0.9876883405658857}, \
{-0.15022558891646198`, -0.9886517447073822}, {-0.1440107823474705, \
-0.9895761185708319}, {-0.13779029047937666`, -0.9904614256635378}, \
{-0.13156435888654827`, -0.9913076310350915}, {-0.12533323335810417`, \
-0.9921147012787539}, {-0.1190971598882109, -0.9928826045327738}, \
{-0.11285638466637121`, -0.9936113104816453}, {-0.10661115406770501`, \
-0.9943007903573056}, {-0.10036171464322283`, -0.9949510169402694}, \
{-0.09410831311009243, -0.9955619645607049}, {-0.08785119634189888, \
-0.9961336090994459}, {-0.08159061135889832, -0.9966659279889449}, \
{-0.07532680531826612, -0.9971589002141636}, {-0.06906002550433944, \
-0.9976125063134026}, {-0.06279051931885496, -0.9980267283790697}, \
{-0.05651853427118181, -0.9984015500583872}, {-0.05024431796855037, \
-0.9987369565540367}, {-0.04396811810627714, -0.9990329346247443}, \
{-0.03769018245798617, -0.9992894725858024}, {-0.03141075886582736, \
-0.9995065603095316}, {-0.02513009523069208, -0.9996841892256801}, \
{-0.018848439502426498`, -0.9998223523217628}, {-0.01256603967004289, \
-0.9999210441433368}, {-0.006283143751929508, -0.9999802607942182}, {
      2.1394078522640747`*^-10, -0.9999999999366349}, {
      0.006283144179802629, -0.9999802607913194}, {
      0.012566040097890656`, -0.9999210441375393}, {
      0.018848439930232008`, -0.9998223523130668}, {
      0.025130095658438437`, -0.999684189214086}, {
      0.03141075929349765, -0.9995065602950395}, {
      0.03769018288556351, -0.9992894725684128}, {
      0.043968118533744625`, -0.9990329346044577}, {
      0.0502443183958911, -0.9987369565308537}, {
      0.0565185346983789, -0.9984015500323083}, {
      0.06279051974589152, -0.9980267283500959}, {
      0.06906002593119857, -0.9976125062815345}, {
      0.07532680574493093, -0.9971589001794022}, {
      0.08159061178535193, -0.9966659279512912}, {
      0.08785119676812439, -0.9961336090589009}, {
      0.09410831353607298, -0.9955619645172697}, {
      0.10036171506894154`, -0.9949510168939454}, {
      0.10661115449314501`, -0.9943007903080938}, {
      0.11285638509151565`, -0.9936113104295473}, {
      0.11909716031304289`, -0.992882604477791}, {
      0.12533323378260686`, -0.9921147012208879}, {
      0.1315643593107048, -0.9913076309743438}, {
      0.1377902909031702, -0.9904614255999102}, {
      0.1440107827708842, -0.9895761185043263}, {
      0.150225589339479, -0.9886517446380002}, {
      0.15643446525903268`, -0.9876883404936295}, {
      0.16263716541375517`, -0.9866859441047602}, {
      0.16883344493166502`, -0.9856445950442853}, {
      0.17502305919425654`, -0.9845643344228825}, {
      0.1812057638461569, -0.9834452048873916}, {
      0.18738131480477282`, -0.9822872506191304}, {
      0.1935494682699267, -0.9810905173321505}, {
      0.1997099807334812, -0.9798550522714331}, {
      0.2058626089889527, -0.9785809042110233}, {
      0.2120071101411126, -0.9772681234521048}, {
      0.21814324161557638`, -0.9759167618210141}, {
      0.22427076116838016`, -0.9745268726671945}, {
      0.2303894268955439, -0.97309851086109}, {
      0.23649899724262147`, -0.9716317327919789}, {
      0.24259923101423678`, -0.9701265963657477}, {
      0.2486898873836057, -0.9685831610026056}, {
      0.25477072590204364`, -0.9670014876347376}, {
      0.26084150650845783`, -0.9653816387039003}, {
      0.2669019895388247, -0.9637236781589557}, {
      0.2729519357356514, -0.9620276714533472}, {
      0.2789911062574212, -0.9602936855425157}, {
      0.28501926268802247`, -0.9585217888812557}, {
      0.2910361670461612, -0.9567120514210136}, {
      0.29704158179475587`, -0.9548645446071253}, {
      0.303035269850315, -0.9529793413759967}, {
      0.3090169945922969, -0.9510565161522232}, {
      0.3149865198724512, -0.9490961448456522}, {
      0.32094361002414107`, -0.9470983048483862}, {
      0.3268880298716475, -0.9450630750317275}, {
      0.33281954473945335`, -0.9429905357430644}, {
      0.3387379204615079, -0.9408807688026992}, {
      0.3446429233904715, -0.9387338575006183}, {
      0.3505343204069394, -0.9365498865932038}, {
      0.356411878928645, -0.9343289422998875}, {
      0.36227536691964185`, -0.9320711122997473}, {
      0.3681245528994639, -0.9297764857280455}, {
      0.37395920595226395`, -0.9274451531727103}, {
      0.37977909573593, -0.925077206670759}, {
      0.38558399249117853`, -0.9226727397046649}, {
      0.39137366705062515`, -0.9202318471986668}, {
      0.39714789084783175`, -0.9177546255150213}, {
      0.4029064359263298, -0.9152411724501989}, {
      0.4086490749486197, -0.9126915872310227}, {
      0.4143755812051457, -0.9101059705107517}, {
      0.4200857286232462, -0.9074844243651068}, {
      0.4257792917760784, -0.9048270522882409}, {
      0.4314560458915178, -0.9021339591886535}, {
      0.4371157668610321, -0.8994052513850489}, {
      0.4427582312485283, -0.8966410366021389}, {
      0.4483832162991738, -0.89384142396639}, {
      0.4539904999481903, -0.8910065240017154}, {
      0.4595798608296204, -0.8881364486251117}, {
      0.46515107828506697`, -0.8852313111422402}, {
      0.47070393237240427`, -0.8822912262429545}, {
      0.47623820387446086`, -0.8793163099967722}, {
      0.48175367430767396`, -0.8763066798482928}, {
      0.4872501259307148, -0.8732624546125612}, {
      0.4927273417530847, -0.8701837544703773}, {
      0.4981851055436814, -0.8670707009635511}, {
      0.5036232018393355, -0.8639234169901047}, {
      0.5090414159533169, -0.8607420267994202}, {
      0.5144395339838095, -0.8575266559873352}, {
      0.5198173428223567, -0.8542774314911837}, {
      0.5251746301622736, -0.8509944815847852}, {
      0.5305111845070292, -0.8476779358733807}, {
      0.5358267951785954, -0.8443279252885161}, {
      0.5411212523257649, -0.8409445820828731}, {
      0.5463943469324348, -0.8375280398250484}, {
      0.551645870825859, -0.8340784333942803}, {
      0.5568756166848662, -0.830595898975124}, {
      0.5620833780480444, -0.8270805740520752}, {
      0.5672689493218923, -0.8235325974041429}, {
      0.5724321257889349, -0.8199521090993699}, {
      0.577572703615806, -0.8163392504893038}, {
      0.5826904798612951, -0.8126941642034163}, {
      0.5877852524843589, -0.8090169941434727}, {
      0.5928568203520979, -0.8053078854778506}, {
      0.5979049832476964, -0.8015669846358091}, {
      0.6029295418783271, -0.7977944393017081}, {
      0.6079302978830188, -0.7939903984091778}, {
      0.6129070538404868, -0.7901550121352391}, {
      0.6178596132769276, -0.7862884318943747}, {
      0.6227877806737746, -0.7823908103325521}, {
      0.6276913614754174, -0.7784623013211966}, {
      0.632570162096882, -0.7745030599511176}, {
      0.6374239899314738, -0.7705132425263849}, {
      0.6422526533583807, -0.7664930065581588}, {
      0.6470559617502387, -0.7624425107584716}, {
      0.651833725480657, -0.758361915033962}, {
      0.6565857559317042, -0.7542513804795622}, {
      0.661311865501355, -0.750111069372138}, {
      0.6660118676108958, -0.7459411451640826}, {
      0.6706855767122909, -0.7417417724768637}, {
      0.6753328082955077, -0.7375131170945245}, {
      0.6799533788958004, -0.733255345957139}, {
      0.6845471061009535, -0.7289686271542213}, {
      0.6891138085584825, -0.7246531299180896}, {
      0.6936533059827937, -0.7203090246171856}, {
      0.6981654191623017, -0.7159364827493483}, {
      0.7026499699665041, -0.7115356769350437}, {
      0.7071067813530137, -0.70710678091055}, {
      0.7115356773745484, -0.7026499695210987}, {
      0.7159364831858765, -0.6981654187139723}, {
      0.72030902505072, -0.6936533055315579}, {
      0.724653130348613, -0.6891138081043581}, {
      0.7289686275817165, -0.6845471056439583}, {
      0.7332553463815892, -0.6799533784359523}, {
      0.7375131175159126, -0.6753328078328247}, {
      0.741741772895173, -0.6706855762467911}, {
      0.7459411455792962, -0.6660118671425974}, {
      0.7501110697842394, -0.6613118650302763}, {
      0.7542513808885347, -0.6565857554578636}, {
      0.7583619154397893, -0.6518337250040728}, {
      0.7624425111611371, -0.6470559612709295}, {
      0.7664930069576463, -0.642252652876365}, {
      0.7705132429226783, -0.6374239894467704}, {
      0.7745030603442008, -0.6325701616095097}, {
      0.7784623017110537, -0.627691360985395}, {
      0.7823908107191669, -0.6227877801811211}, {
      0.7862884322777316, -0.617859612781662}, {
      0.7901550125153223, -0.6129070533426283}, {
      0.7939903987859718, -0.6079302973825864}, {
      0.7977944396751974, -0.6029295413753403}, {
      0.8015669850059781, -0.5979049827421743}, {
      0.805307885844684, -0.59285681984406}, {
      0.8090169945069554, -0.5877852519738249}, {
      0.8126941645635332, -0.5826904793482846}, {
      0.8163392508460396, -0.5775727031003386}, {
      0.8199521094527099, -0.5724321252710304}, {
      0.8235325977540722, -0.5672689488015705}, {
      0.8270805743985791, -0.5620833775253253}, {
      0.8305958993181878, -0.5568756161597697}, {
      0.8340784337338896, -0.5516458702984053}, {
      0.8375280401611888, -0.546394346402644}, {
      0.8409445824155302, -0.5411212517936572}, {
      0.8443279256176756, -0.5358267946441913}, {
      0.8476779361990285, -0.5305111839703488}, {
      0.8509944819069074, -0.5251746296233375}, {
      0.8542774318097663, -0.5198173422811854}, {
      0.8575266563023645, -0.5144395334404237}, {
      0.8607420271108823, -0.5090414154077372}, {
      0.8639234172979859, -0.5036232012915828}, {
      0.8670707012678381, -0.49818510499377644`}, {
      0.8701837547710567, -0.4927273412010485}, {
      0.8732624549096196, -0.4872501253765683}, {
      0.8763066801417171, -0.48175367375143824`}, {
      0.8793163102865493, -0.47623820331615707`}, {
      0.8822912265290714, -0.47070393181205367`}, {
      0.885231311424684, -0.4651510777226908}, {
      0.8881364489038697, -0.45957986026524006`}, {
      0.891006524276775, -0.45399049938182723`}, {
      0.8938414242377385, -0.4483832157308495}, {
      0.8966410368697638, -0.4427582306782643}, {
      0.8994052516489379, -0.43711576628885}, {
      0.9021339594487944, -0.4314560453174394}, {
      0.9048270525446215, -0.4257792912001255}, {
      0.9074844246177152, -0.42008572804544064`}, {
      0.910105970759576, -0.4143755806255094}, {
      0.9126915874760511, -0.40864907436717457`}, {
      0.9152411726914196, -0.402906435343098}, {
      0.9177546257524228, -0.3971478902628354}, {
      0.9202318474322376, -0.39137366646388644`}, {
      0.9226727399343937, -0.3855839919027197}, {
      0.9250772068966344, -0.3797790951457734}, {
      0.9274451533947212, -0.373959205360432}, {
      0.929776485946181, -0.368124552305979}, {
      0.9320711125139963, -0.3622753663245266}, {
      0.9343289425102393, -0.35641187833192195`}, {
      0.9365498867996477, -0.3505343198086312}, {
      0.9387338577031438, -0.3446429227906009}, {
      0.9408807690012957, -0.33873791986009766`}, {
      0.9429905359377215, -0.33281954413652637`}, {
      0.945063075222435, -0.3268880292672267}, {
      0.947098305035134, -0.3209436094182494}, {
      0.9490961450284303, -0.3149865192651117}, {
      0.9510565163310217, -0.3090169939835327}, {
      0.952979341550806, -0.3030352692401492}, {
      0.9548645447779357, -0.2970415811832117}, {
      0.9567120515878154, -0.291036166433262}, {
      0.9585217890440396, -0.2850192620737916}, {
      0.9602936857012723, -0.27899110564188195`}, {
      0.9620276716080673, -0.2729519351188273}, {
      0.9637236783096302, -0.26690198892073924`}, {
      0.9653816388505203, -0.2608415058891346}, {
      0.9670014877772943, -0.25477072528150624`}, {
      0.9685831611410902, -0.2486898867618779}, {
      0.9701265965001518, -0.2425992303913423}, {
      0.9716317329222938, -0.23649899661858412`}, {
      0.9730985109873074, -0.23038942627038753`}, {
      0.9745268727893062, -0.2242707605421287}, {
      0.9759167619390119, -0.2181432409882538}, {
      0.9772681235659807, -0.21200710951274293`}, {
      0.9785809043207695, -0.20586260835956005`}, {
      0.9798550523770418, -0.19970998010308968`}, {
      0.9810905174336141, -0.1935494676385605}, {
      0.9822872507164413, -0.1873813141724562}, {
      0.9834452049805424, -0.18120576321291415`}, {
      0.984564334511866, -0.17502305856011202`}, {
      0.9856445951290942, -0.16883344429664313`}, {
      0.9866859441853877, -0.16263716477788034`}, {
      0.9876883405700686, -0.1564344646223294}, {
      0.9886517447102441, -0.15022558870197184`}, {
      0.9895761185723683, -0.14401078213259774`}, {
      0.9904614256637438, -0.13779029026412912`}, {
      0.9913076310339627, -0.1315643586709338}, {
      0.9921147012762856, -0.12533323314213068`}, {
      0.9928826045289615, -0.1190971596718863}, {
      0.9936113104764847, -0.1128563844497035}, {
      0.994300790350792, -0.1066111538507022}, {
      0.9949510169323986, -0.10036171442589299`}, {
      0.9955619645514724, -0.09410831289244369}, {
      0.9961336090888475, -0.08785119612393938}, {
      0.9966659279769763, -0.08159061114063626}, {
      0.9971589002008205, -0.07532680509970972}, {
      0.9976125062986809, -0.06906002528549698}, {
      0.9980267283629655, -0.06279051909973475}, {
      0.9984015500408961, -0.056518534051792205`}, {
      0.9987369565351549, -0.05024431774889977}, {
      0.9990329346044677, -0.04396811788637398}, {
      0.9992894725641273, -0.037690182237838925`}, {
      0.9995065602864539, -0.03141075864544454}, {
      0.9996841892011962, -0.025130095010082246`}, {
      0.9998223522958687, -0.01884843928159823}, {
      0.9999210441160289, -0.012566039449004817`}, {
      0.9999802607654928, -0.006283143530690299}, {0.9999999999064885, 
      4.353724264785641*^-10}, {0.9999802607597483, 0.00628314440141796}}]}, 
   {RGBColor[1, 1, 0], DiskBox[{0, 0}, 0.03]}},
  AspectRatio->Automatic,
  Background->RGBColor[0.6, 0.6, 0.6],
  ImageSize->{321., Automatic}]], "Output",
 CellChangeTimes->{3.401193627808757*^9, 3.401194919597694*^9, 
  3.401195146446289*^9, 3.401195427103249*^9, 3.401201207333535*^9, 
  3.4012768909416122`*^9}]
}, Open  ]],

Cell[TextData[{
 "Now try ",
 Cell[BoxData[
  FormBox[
   RowBox[{"V0", "=", 
    RowBox[{"0.99", "\[Times]", "2", " ", "\[Pi]"}]}], TraditionalForm]]],
 ".  Prepare for a disaster.  You will need to try smaller tFinal to observe \
what is happening."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"tFinal", "=", "2"}], ";", 
    RowBox[{"R0", "=", "1"}], ";", 
    RowBox[{"V0", "=", 
     RowBox[{".99", "\[Times]", "2", " ", "\[Pi]"}]}], ";", 
    RowBox[{"\[CapitalDelta]t", "=", ".001"}], ";", 
    RowBox[{"p", "=", "3"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"CalculateOrbit", ";", 
    RowBox[{"DisplayOrbit", "[", "]"}]}], "\[IndentingNewLine]", 
   RowBox[{"Clear", "[", 
    RowBox[{
    "tFinal", ",", "R0", ",", "V0", ",", "iMaximum", ",", "t", ",", "r", ",", 
     "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.401193360004507*^9, 3.401193414015883*^9}, 
   3.40119347898011*^9, 3.401193626282501*^9, {3.4012012039931593`*^9, 
   3.401201220802925*^9}, 3.401276893514756*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 1, 1], Thickness[0.0075], 
    LineBox[{{1, 0}, {0.9999802608509668, 0.006220312525806487}, {
      0.9999210441211108, 0.012440379483746572`}, {0.9998223519621382, 
      0.018659955313333452`}, {0.9996841879601776, 0.02487879446883947}, {
      0.9995065571357071, 0.03109665142667559}, {0.9992894659434539, 
      0.03731328069277075}, {0.9990329222722625, 0.04352843680995101}, {
      0.998736935444936, 0.04974187436531859}, {0.9984015162180475, 
      0.05595334799763054}, {0.9980266767817222, 0.0621626124046772}, {
      0.9976124307593912, 0.06836942235066032}, {0.9971587932075159, 
      0.07457353267357078}, {0.996665780615283, 0.08077469829256592}, {
      0.9961334109042714, 0.08697267421534642}, {0.9955617034280895, 
      0.09316721554553264}, {0.9949506789719827, 0.09935807749004051}, {
      0.994300359752414, 0.10554501536645672`}, {0.9936107694166127, 
      0.1117277846104134}, {0.9928819330420967, 0.11790614078296205`}, {
      0.9921138771361637, 0.12407983957794687`}, {0.9913066296353547, 
      0.1302486368293772}, {0.9904602199048874, 0.13641228851879933`}, {
      0.9895746787380614, 0.1425705507826673}, {0.9886500383556333, 
      0.148723179919713}, {0.9876863324051636, 0.15486993239831529`}, {
      0.9866835959603338, 0.16101056486386803`}, {0.9856418655202348, 
      0.16714483414614734`}, {0.9845611790086257, 0.1732724972666777}, {
      0.9834415757731637, 0.1793933114460969}, {0.9822830965846049, 
      0.18550703411152003`}, {0.9810857836359754, 0.19161342290390201`}, {
      0.9798496805417141, 0.1977122356853993}, {0.9785748323367847, 
      0.20380323054673002`}, {0.9772612854757604, 0.2098861658145329}, {
      0.9759090878318778, 0.21596080005872484`}, {0.9745182886960619, 
      0.2220268920998572}, {0.9730889387759225, 0.22808420101647048`}, {
      0.9716210901947201, 0.2341324861524476}, {0.9701147964903032, 
      0.24017150712436575`}, {0.9685701126140163, 0.24620102382884645`}, {
      0.966987094929578, 0.25222079644990425`}, {0.9653658012119302, 
      0.25823058546629357`}, {0.9637062906460573, 0.264230151658854}, {
      0.9620086238257767, 0.2702192561178537}, {0.9602728627524991, 
      0.27619766025033105`}, {0.9584990708339599, 0.2821651257874346}, {
      0.9566873128829204, 0.28812141479176095`}, {0.9548376551158402, 
      0.2940662896646909}, {0.9529501651515192, 0.2999995131537232}, {
      0.9510249120097112, 0.3059208483598069}, {0.9490619661097065, 
      0.311830058744671}, {0.9470613992688858, 0.3177269081381527}, {
      0.945023284701244, 0.32361116074552254`}, {0.9429476970158845, 
      0.32948258115480833`}, {0.940834712215484, 0.335340934344116}, {
      0.9386844076947264, 0.34118598568894865`}, {0.9364968622387086, 
      0.3470175009695231}, {0.9342721560213154, 0.35283524637808394`}, {
      0.9320103706035646, 0.3586389885262152}, {0.9297115889319225, 
      0.36442849445214937`}, {0.9273758953365895, 0.3702035316280744}, {
      0.9250033755297554, 0.37596386796743736`}, {0.9225941166038256, 
      0.3817092718322459}, {0.920148207029616, 0.38743951204036686`}, {
      0.9176657366545189, 0.39315435787282216`}, {0.9151467967006386, 
      0.398853579081082}, {0.9125914797628966, 0.4045369458943549}, {
      0.9099998798071066, 0.41020422902687526`}, {0.9073720921680202, 
      0.41585519968518736`}, {0.9047082135473412, 0.4214896295754271}, {
      0.9020083420117107, 0.4271072909106001}, {0.8992725769906607, 
      0.43270795641785675`}, {0.8965010192745388, 0.4382913993457643}, {
      0.8936937710124019, 0.4438573934715757}, {0.8908509357098788, 
      0.44940571310849503`}, {0.8879726182270036, 0.4549361331129395}, {
      0.8850589247760177, 0.46044842889179843`}, {0.8821099629191419, 
      0.46594237640968844`}, {0.8791258415663176, 0.47141775219620524`}, {
      0.8761066709729172, 0.4768743333531719}, {0.8730525627374246, 
      0.4823118975618836}, {0.8699636297990844, 0.4877302230903484}, {
      0.8668399864355202, 0.49312908880052475`}, {0.863681748260323, 
      0.4985082741555546}, {0.8604890322206078, 0.5038675592269933}, {
      0.8572619565945403, 0.5092067247020353}, {0.8540006409888317, 
      0.5145255518907356}, {0.8507052063362033, 0.5198238227332278}, {
      0.8473757748928199, 0.5251013198069379}, {0.8440124702356923, 
      0.5303578263337931}, {0.8406154172600481, 0.535593126187428}, {
      0.8371847421766722, 0.540807003900385}, {0.8337205725092157, 
      0.5459992446713114}, {0.8302230370914736, 0.5511696343721514}, {
      0.8266922660646311, 0.5563179595553343}, {0.823128390874479, 
      0.5614440074609576}, {0.8195315442685971, 0.5665475660239662}, {
      0.8159018602935069, 0.5716284238813262}, {0.8122394742917922, 
      0.5766863703791945}, {0.8085445228991885, 0.5817211955800833}, {
      0.804817144041641, 0.5867326902700204}, {0.8010574769323299, 
      0.5917206459657037}, {0.7972656620686658, 0.5966848549216515}, {
      0.7934418412292519, 0.6016251101373469}, {0.7895861574708148, 
      0.6065412053643784}, {0.785698755125104, 0.6114329351135741}, {
      0.7817797797957585, 0.6163000946621313}, {0.7778293783551425, 
      0.6211424800607405}, {0.7738476989411481, 0.625959888140704}, {
      0.7698348909539667, 0.6307521165210497}, {0.765791105052828, 
      0.6355189636156382}, {0.7617164931527065, 0.6402602286402657}, {
      0.7576112084209966, 0.6449757116197598}, {0.7534754052741542, 
      0.6496652133950706}, {0.7493092393743073, 0.6543285356303556}, {
      0.7451128676258325, 0.658965480820059}, {0.7408864481719011, 
      0.6635758522959843}, {0.7366301403909906, 0.6681594542343616}, {
      0.7323441048933652, 0.6727160916629089}, {0.7280285035175228, 
      0.6772455704678861}, {0.7236834993266092, 0.6817476974011442}, {
      0.7193092566048002, 0.6862222800871668}, {0.7149059408536498, 
      0.6906691270301064}, {0.7104737187884068, 0.6950880476208133}, {
      0.7060127583342974, 0.6994788521438582}, {0.7015232286227752, 
      0.7038413517845481}, {0.6970052999877374, 0.7081753586359356}, {
      0.6924591439617094, 0.7124806857058215}, {0.6878849332719943, 
      0.7167571469237496}, {0.6832828418367903, 0.7210045571479958}, {
      0.6786530447612745, 0.7252227321725492}, {0.6739957183336528, 
      0.7294114887340862}, {0.6693110400211768, 0.7335706445189379}, {
      0.6645991884661272, 0.7377000181700492}, {0.6598603434817627, 
      0.741799429293931}, {0.6550946860482365, 0.7458686984676053}, {
      0.6503023983084776, 0.7499076472455419}, {0.6454836635640394, 
      0.7539160981665881}, {0.6406386662709139, 0.7578938747608898}, {
      0.6357675920353117, 0.7618408015568053}, {0.6308706276094084, 
      0.7657567040878118}, {0.6259479608870562, 0.7696414088994021}, {
      0.6209997808994621, 0.7734947435559746}, {0.6160262778108311, 
      0.7773165366477143}, {0.6110276429139758, 0.7811066177974662}, {
      0.6060040686258905, 0.7848648176675993}, {0.6009557484832918, 
      0.788590967966863}, {0.595882877138124, 0.792284901457234}, {
      0.5907856503530302, 0.795946451960755}, {0.5856642649967883, 
      0.7995754543663645}, {0.5805189190397125, 0.8031717446367177}, {
      0.5753498115490202, 0.8067351598149981}, {0.5701571426841626, 
      0.8102655380317202}, {0.5649411136921221, 0.8137627185115227}, {
      0.5597019269026731, 0.8172265415799531}, {0.5544397857236084, 
      0.8206568486702416}, {0.5491548946359301, 0.8240534823300665}, {
      0.5438474591890051, 0.8274162862283095}, {0.538517685995685, 
      0.8307451051618014}, {0.5331657827273906, 0.8340397850620577}, {
      0.5277919581091614, 0.8373001730020041}, {0.5223964219146683, 
      0.8405261172026925}, {0.5169793849611913, 0.8437174670400064}, {
      0.5115410591045613, 0.8468740730513555}, {0.5060816572340653, 
      0.849995786942361}, {0.5006013932673169, 0.853082461593529}, {
      0.4951004821450893, 0.8561339510669143}, {0.4895791398261129, 
      0.859150110612773}, {0.48403758328183655`, 0.8621307966762043}, {
      0.47847603049115234`, 0.8650758669037807}, {0.47289470043508386`, 
      0.8679851801501682}, {0.4672938130914383, 0.8708585964847342}, {
      0.4616735894294215, 0.8736959771981442}, {0.4560342514042165, 
      0.8764971848089477}, {0.4503760219515256, 0.8792620830701514}, {
      0.444699124982075, 0.8819905369757816}, {0.4390037853760828, 
      0.8846824127674338}, {0.43329022897769015`, 0.8873375779408114}, {
      0.42755868258935475`, 0.889955901252251}, {0.4218093739662077, 
      0.8925372527252365}, {0.4160425318103726, 0.8950815036569002}, {
      0.41025838576524737`, 0.8975885266245112}, {0.4044571664097489, 
      0.9000581954919519}, {0.3986391052525196, 0.9024903854161809}, {
      0.3928044347260968, 0.9048849728536834}, {0.3869533881810438, 
      0.9072418355669085}, {0.3810861998800438, 0.909560852630693}, {
      0.37520310499195514`, 0.911841904438672}, {0.36930433958582926`, 
      0.9140848727096761}, {0.3633901406248897, 0.9162896404941151}, {
      0.3574607459604737, 0.9184560921803473}, {0.35151639432593496`, 
      0.9205841135010355}, {0.34555732533050804`, 0.9226735915394888}, {
      0.3395837794531346, 0.9247244147359897}, {0.3335959980362508, 
      0.9267364728941075}, {0.32759422327953575`, 0.9287096571869972}, {
      0.3215786982336219, 0.9306438601636825}, {0.3155496667937658, 
      0.9325389757553258}, {0.3095073736934803, 0.934394899281482}, {
      0.3034520644981277, 0.9362115274563372}, {0.29738398559847357`, 
      0.9379887583949328}, {0.29130338420420127`, 0.9397264916193736}, {
      0.2852105083373873, 0.9414246280650205}, {0.2791056068259373, 
      0.9430830700866673}, {0.2729889292969821, 0.9447017214647017}, {
      0.26686072617023426`, 0.9462804874112503}, {0.2607212486513053, 
      0.9478192745763069}, {0.25457074872498253`, 0.949317991053845}, {
      0.24840947914846606`, 0.9507765463879131}, {0.2422376934445661, 
      0.9521948515787139}, {0.23605564589485972`, 0.9535728190886659}, {
      0.2298635915328076, 0.9549103628484484}, {0.2236617861368304, 
      0.9562073982630291}, {0.21745048622334467`, 0.9574638422176739}, {
      0.21122994903975822`, 0.9586796130839393}, {0.20500043255742492`, 
      0.9598546307256477}, {0.19876219546455867`, 0.9609888165048432}, {
      0.19251549715910665`, 0.962082093287731}, {0.18626059774158152`, 
      0.9631343854505965}, {0.17999775800785267`, 0.9641456188857078}, {
      0.17372723944189628`, 0.9651157210071983}, {0.16744930420850418`, 
      0.9660446207569306}, {0.16116421514595133`, 0.9669322486103413}, {
      0.1548722357586219, 0.967778536582267}, {0.1485736302095938, 
      0.9685834182327497}, {0.14226866331318147`, 0.9693468286728242}, {
      0.13595760052743713`, 0.9700687045702837}, {0.12964070794660998`, 
      0.9707489841554271}, {0.12331825229356351`, 0.9713876072267845}, {
      0.11699050091215081`, 0.9719845151568235}, {0.11065772175954769`, 
      0.972539650897634}, {0.10432018339854349`, 0.9730529589865924}, {
      0.09797815498978958, 0.9735243855520049}, {0.09163190628400543, 
      0.9739538783187291}, {0.085281707614142, 0.9743413866137746}, {
      0.07892782988750253, 0.974686861371881}, {0.0725705445778206, 
      0.9749902551410752}, {0.06621012371729516, 0.9752515220882056}, {
      0.05984683988858276, 0.9754706180044544}, {0.053480966216746556`, 
      0.9756475003108271}, {0.047112776361162165`, 0.9757821280636191}, {
      0.04074254450738026, 0.9758744619598596}, {0.03437054535894573, 
      0.9759244643427323}, {0.027997054129173318`, 0.975932099206972}, {
      0.021622346532879713`, 0.9758973322042381}, {0.01524669877807188, 
      0.9758201306484641}, {0.008870387557591602, 0.9757004635211823}, {
      0.0024936900407160796`, 0.9755383014768255}, {-0.0038831161352854606`, 
      0.975333616848002}, {-0.010259752873639316`, 
      0.9750863836507477}, {-0.016635941626599066`, 
      0.974796577589752}, {-0.02301140340402167, 
      0.9744641760635584}, {-0.029385858781988024`, 
      0.9740891581697397}, {-0.035759027911468025`, 
      0.9736715047100477}, {-0.04213063052703032, 
      0.9732111981955358}, {-0.048500385955596755`, 
      0.9727082228516563}, {-0.05486801312524175, 
      0.9721625646233302}, {-0.06123323057403655, 
      0.9715742111799911}, {-0.0675957564589386, 
      0.970943151920601}, {-0.07395530856472608, 
      0.9702693779786392}, {-0.08031160431297764, 
      0.9695528822270637}, {-0.08666436077109758, 
      0.9687936592832443}, {-0.09301329466138651, 
      0.9679917055138679}, {-0.09935812237015751, 
      0.9671470190398148}, {-0.10569855995689806`, 
      0.9662595997410071}, {-0.11203432316347765`, 
      0.9653294492612271}, {-0.1183651274234014, 
      0.9643565710129078}, {-0.12469068787110955`, 
      0.9633409701818925}, {-0.13101071935132313`, 
      0.9622826537321648}, {-0.13732493642843574`, 
      0.9611816304105497}, {-0.1436330533959517, 
      0.9600379107513827}, {-0.1499347842859705, 
      0.958851507081149}, {-0.1562298428787179, 
      0.9576224335230922}, {-0.1625179427121234, 
      0.9563507060017906}, {-0.16879879709144457`, 
      0.955036342247703}, {-0.17507211909893824`, 
      0.9536793618016818}, {-0.18133762160357833`, 
      0.9522797860194547}, {-0.1875950172708209, 
      0.9508376380760736}, {-0.19384401857241623`, 
      0.9493529429703301}, {-0.20008433779626808`, 
      0.9478257275291397}, {-0.2063156870563402, 
      0.9462560204118904}, {-0.21253777830261034`, 
      0.9446438521147593}, {-0.2187503233310716, 
      0.9429892549749945}, {-0.22495303379378162`, 
      0.9412922631751627}, {-0.23114562120895904`, 
      0.9395529127473622}, {-0.23732779697112816`, 
      0.9377712415774012}, {-0.2434992723613112, 
      0.9359472894089403}, {-0.24965975855726869`, 
      0.9340810978476001}, {-0.2558089666437878, 
      0.9321727103650318}, {-0.26194660762301897`, 
      0.9302221723029527}, {-0.2680723924248606, 
      0.9282295308771442}, {-0.2741860319173922, 
      0.9261948351814135}, {-0.28028723691735613`, 
      0.9241181361915173}, {-0.2863757182006876, 
      0.9219994867690484}, {-0.2924511865130933, 
      0.9198389416652843}, {-0.29851335258067924`, 
      0.9176365575249971}, {-0.3045619271206266, 
      0.9153923928902248}, {-0.310596620851917, 
      0.9131065082040041}, {-0.31661714450610695`, 
      0.9107789658140629}, {-0.32262320883815054`, 
      0.9084098299764739}, {-0.32861452463727236`, 
      0.9059991668592673}, {-0.33459080273788877`, 
      0.9035470445460034}, {-0.340551754030579, 
      0.9010535330393047}, {-0.3464970894731056, 
      0.8985187042643457}, {-0.3524265201014841, 
      0.8959426320723021}, {-0.3583397570411029, 
      0.8933253922437576}, {-0.36423651151789194`, 
      0.8906670624920684}, {-0.3701164948695421, 
      0.8879677224666845}, {-0.37597941855677364`, 
      0.8852274537564285}, {-0.3818249941746549, 
      0.8824463398927306}, {-0.3876529334639711, 
      0.8796244663528191}, {-0.3934629483226431, 
      0.8767619205628678}, {-0.3992547508171962, 
      0.8738587919010974}, {-0.40502805319427926`, 
      0.8709151717008321}, {-0.41078256789223433`, 
      0.8679311532535113}, {-0.4165180075527164, 
      0.8649068318116546}, {-0.4222340850323636, 
      0.8618423045917804}, {-0.42793051341451815`, 
      0.858737670777278}, {-0.43360700602099783`, 
      0.8555930315212328}, {-0.43926327642391794`, 
      0.8524084899492028}, {-0.4448990384575643, 
      0.8491841511619482}, {-0.4505140062303168, 
      0.8459201222381117}, {-0.45610789413662406`, 
      0.84261651223685}, {-0.4616804168690289, 
      0.8392734322004161}, {-0.4672312894302448, 
      0.8358909951566917}, {-0.47276022714528343`, 
      0.8324693161216695}, {-0.4782669456736337, 
      0.8290085121018842}, {-0.48375116102149124`, 
      0.8255087020967928}, {-0.4892125895540399, 
      0.8219700071011035}, {-0.49465094800778414`, 
      0.8183925501070517}, {-0.500065953502933, 
      0.8147764561066243}, {-0.5054573235558353, 
      0.8111218520937304}, {-0.5108247760914664, 
      0.8074288670663188}, {-0.5161680294559667, 
      0.8036976320284412}, {-0.5214868024292317, 
      0.7999282799922617}, {-0.5267808142375531, 
      0.7961209459800102}, {-0.5320497845663124, 
      0.7922757670258814}, {-0.537293433572726, 
      0.7883928821778778}, {-0.5425114818986415, 
      0.7844724324995954}, {-0.547703650683387, 
      0.7805145610719536}, {-0.5528696615766708, 
      0.7765194129948669}, {-0.5580092367515345, 
      0.7724871353888595}, {-0.5631220989173564, 
      0.7684178773966205}, {-0.5682079713329091, 
      0.7643117901845011}, {-0.5732665778194665, 
      0.7601690269439517}, {-0.578297642773965, 
      0.7559897428928993}, {-0.5833008911822154, 
      0.7517740952770646}, {-0.5882760486321679, 
      0.7475222433712175}, {-0.5932228413272286, 
      0.7432343484803718}, {-0.5981409960996281, 
      0.7389105739409173}, {-0.603030240423843, 
      0.7345510851216892}, {-0.6078903024300687, 
      0.7301560494249747}, {-0.612720910917745, 
      0.7257256362874557}, {-0.617521795369134, 
      0.721260017181087}, {-0.6222926859629494, 
      0.7167593656139103}, {-0.6270333135880395, 
      0.7122238571308016}, {-0.6317434098571212, 
      0.7076536693141543}, {-0.6364227071205669, 
      0.7030489817844939}, {-0.6410709384802435, 
      0.6984099762010274}, {-0.6456878378034039, 
      0.6937368362621232}, {-0.6502731397366303, 
      0.6890297477057244}, {-0.6548265797198304, 
      0.6842888983096922}, {-0.6593478940002857, 
      0.6795144778920796}, {-0.6638368196467518, 
      0.6747066783113363}, {-0.6682930945636112, 
      0.6698656934664416}, {-0.6727164575050784, 
      0.6649917192969667}, {-0.6771066480894569, 
      0.6600849537830646}, {-0.6814634068134492, 
      0.6551455969453889}, {-0.6857864750665181, 
      0.6501738508449371}, {-0.6900755951453006, 
      0.6451699195828223}, {-0.694330510268074, 
      0.6401340092999696}, {-0.6985509645892742, 
      0.635066328176737}, {-0.7027367032140659, 
      0.6299670864324621}, {-0.7068874722129644, 
      0.6248364963249308}, {-0.7110030186365109, 
      0.6196747721497698}, {-0.7150830905299981, 
      0.6144821302397624}, {-0.7191274369482484, 
      0.609258788964084}, {-0.7231358079704447, 
      0.6040049687274606}, {-0.7271079547150118, 
      0.5987208919692463}, {-0.7310436293545501, 
      0.5934067831624216}, {-0.7349425851308218, 
      0.5880628688125099}, {-0.7388045763697871, 
      0.5826893774564128}, {-0.7426293584966939, 
      0.5772865396611628}, {-0.7464166880512171, 
      0.5718545880225934}, {-0.7501663227026513, 
      0.5663937571639246}, {-0.7538780212651532, 
      0.5609042837342653}, {-0.7575515437130367, 
      0.5553864064070291}, {-0.7611866511961178, 
      0.5498403658782652}, {-0.7647831060551121, 
      0.5442664048649026}, {-0.7683406718370823, 
      0.5386647681029062}, {-0.771859113310937, 
      0.533035702345346}, {-0.7753381964829805, 
      0.5273794563603769}, {-0.7787776886125131, 
      0.5216962809291285}, {-0.782177358227482, 
      0.515986428843506}, {-0.7855369751401834, 
      0.5102501549038986}, {-0.7888563104630139, 
      0.5044877159167969}, {-0.792135136624273, 
      0.4986993706923177}, {-0.7953732273840161, 
      0.4928853800416352}, {-0.798570357849956, 
      0.48704600677431836`}, {-0.8017263044934162, 
      0.48118151569557327`}, {-0.8048408451653327, 
      0.47529217360338977`}, {-0.807913759112306, 
      0.4693782492855918}, {-0.8109448269927022, 
      0.4634400135167904}, {-0.813933830892804, 
      0.4574777390552385}, {-0.8168805543430102, 
      0.4514917006395871}, {-0.8197847823340848, 
      0.44548217498554166`}, {-0.8226463013334546, 
      0.43944944078241777`}, {-0.8254648993015555, 
      0.4333937786895959}, {-0.8282403657082276, 
      0.4273154713328735}, {-0.8309724915491578, 
      0.4212148033007146}, {-0.8336610693623719, 
      0.41509206114039504`}, {-0.8363058932447734, 
      0.4089475333540434}, {-0.8389067588687306, 
      0.40278151039457605`}, {-0.8414634634987108, 
      0.3965942846615262}, {-0.8439758060079626, 
      0.390386150496765}, {-0.8464435868952446, 
      0.38415740418011524`}, {-0.8488666083016011, 
      0.37790834392485534`}, {-0.8512446740271847, 
      0.37163926987311374`}, {-0.8535775895481253, 
      0.3653504840911524}, {-0.8558651620334449, 
      0.3590422905645385}, {-0.8581072003620185, 
      0.35271499519320365`}, {-0.8603035151395814, 
      0.34636890578638924`}, {-0.8624539187157806, 
      0.3400043320574777}, {-0.8645582252012723, 
      0.3336215856187081}, {-0.8666162504848642, 
      0.32722097997577543`}, {-0.868627812250702, 
      0.32080283052231284`}, {-0.8705927299955002, 
      0.3143674545342553}, {-0.8725108250458179, 
      0.3079151711640844}, {-0.8743819205753769, 
      0.30144630143495293`}, {-0.8762058416224242, 
      0.29496116823468843`}, {-0.8779824151071375, 
      0.2884600963096744}, {-0.8797114698490727, 
      0.2819434122586089}, {-0.8813928365846551, 
      0.2754114445261389}, {-0.8830263479847116, 
      0.26886452339636957`}, {-0.8846118386720454, 
      0.26230298098624766`}, {-0.8861491452390517, 
      0.25572715123881795`}, {-0.8876381062653748, 
      0.24913736991635138`}, {-0.8890785623356052, 
      0.24253397459334458`}, {-0.8904703560570184, 
      0.23591730464938898`}, {-0.8918133320773519, 
      0.22928770126190892`}, {-0.893107337102623, 
      0.22264550739876773`}, {-0.8943522199149855, 
      0.21599106781074054`}, {-0.8955478313906244, 
      0.20932472902385285`}, {-0.8966940245176902, 
      0.20264683933158395`}, {-0.8977906544142696, 
      0.19595774878693378`}, {-0.8988375783463953, 
      0.18925780919435256`}, {-0.899834655746091, 
      0.1825473741015318}, {-0.9007817482294547, 
      0.17582679879105576`}, {-0.9016787196147765, 
      0.16909644027191226`}, {-0.9025254359406937, 
      0.1623566572708616}, {-0.903321765484379, 
      0.1556078102236628}, {-0.9040675787797656, 
      0.14885026126615553`}, {-0.9047627486358045, 
      0.14208437422519724`}, {-0.9054071501547566, 
      0.1353105146094538}, {-0.9060006607505173, 
      0.12852904960004288`}, {-0.9065431601669743, 
      0.12174034804102865`}, {-0.9070345304963967, 
      0.11494478042976702`}, {-0.9074746561978559, 
      0.10814271890709984`}, {-0.9078634241156779, 
      0.10133453724739715`}, {-0.9082007234979249, 
      0.09452061084844632}, {-0.9084864460149078, 
      0.08770131672118674}, {-0.9087204857777269, 
      0.080877033479289}, {-0.9089027393568417, 
      0.07404814132857736}, {-0.9090331058006687, 
      0.0672150220562942}, {-0.9091114866542057, 
      0.06037805902020546}, {-0.9091377859776842, 
      0.053537637137545516`}, {-0.9091119103652463, 
      0.04669414287380065}, {-0.9090337689636477, 
      0.039847964231329615`}, {-0.9089032734909858, 
      0.032999490737820124`}, {-0.9087203382554502, 
      0.026149113434580103`}, {-0.9084848801740985, 
      0.01929722486466231}, {-0.908196818791653, 
      0.012444219060821193`}, {-0.9078560762993203, 
      0.005590491533300639}, {-0.907462577553631, -0.0012635607425486419`}, \
{-0.9070162500953007, -0.008117539338823376}, {-0.90651702416811, \
-0.014971044387794908`}, {-0.9059648327378029, -0.021823674594810643`}, \
{-0.9053596115110044, -0.028675027251371}, {-0.9047012989541539, \
-0.03552469824838323}, {-0.9039898363124554, -0.04237228208959341}, \
{-0.9032251676288441, -0.049217371905197954`}, {-0.9024072397629658, \
-0.05605955946563591}, {-0.9015360024101718, -0.06289843519556348}, \
{-0.9006114081205259, -0.06973358818801209}, {-0.899633412317823, \
-0.07656460621873128}, {-0.8986019733186198, -0.08339107576071789}, \
{-0.8975170523512747, -0.0902125819989329}, {-0.8963786135749974, \
-0.09702870884520723}, {-0.8951866240989066, -0.10383903895333797`}, \
{-0.8939410540010949, -0.11064315373437636`}, {-0.8926418763477001, \
-0.11744063337210903`}, {-0.8912890672119812, -0.12423105683873378`}, \
{-0.8898826056933996, -0.13101400191073145`}, {-0.8884224739367025, \
-0.13778904518493518`}, {-0.8869086571510091, -0.14455576209479856`}, \
{-0.8853411436288973, -0.15131372692686418`}, {-0.8837199247654908, \
-0.15806251283743386`}, {-0.8820449950775443, -0.16480169186944213`}, \
{-0.8803163522225276, -0.17153083496953442`}, {-0.8785339970177045, \
-0.1782495120053514}, {-0.8766979334592089, -0.184957291783021}, \
{-0.8748081687411134, -0.19165374206485944`}, {-0.8728647132744922, \
-0.19833842958728298`}, {-0.8708675807064746, -0.20501092007893174`}, \
{-0.8688167879392897, -0.21167077827900715`}, {-0.8667123551493, \
-0.21831756795582452`}, {-0.864554305806023, -0.22495085192558228`}, \
{-0.8623426666911391, -0.23157019207134943`}, {-0.8600774679174854, \
-0.23817514936227258`}, {-0.8577587429480324, -0.24476528387300442`}, \
{-0.8553865286148444, -0.2513401548033549}, {-0.85296086513802, \
-0.25789932049816683`}, {-0.850481796144613, -0.2644423384674172}, \
{-0.8479493686875309, -0.2709687654065465}, {-0.8453636332644114, \
-0.27747815721701646`}, {-0.8427246438364726, -0.2839700690270989}, \
{-0.8400324578473383, -0.2904440552128968}, {-0.8372871362418348, \
-0.29689966941959883`}, {-0.8344887434847591, -0.30333646458297003`}, \
{-0.8316373475796155, -0.3097539929510787}, {-0.8287330200873203, \
-0.3161518061062627}, {-0.8257758361448722, -0.32252945498733565`}, \
{-0.8227658744839866, -0.3288864899120351}, {-0.8197032174496933, \
-0.33522246059971444`}, {-0.8165879510188941, -0.3415369161942797}, \
{-0.8134201648188802, -0.34782940528737366`}, {-0.8101999521458068, \
-0.35409947594180796`}, {-0.8069274099831226, -0.3603466757152457}, \
{-0.8036026390199547, -0.36657055168413594`}, {-0.8002257436694433, \
-0.3727706504679013}, {-0.7967968320870286, -0.3789465182533812}, \
{-0.7933160161886836, -0.38509770081953165`}, {-0.7897834116690957, \
-0.3912237435623838}, {-0.78619913801979, -0.3973241915202626}, \
{-0.7825633185471969, -0.4033985893992674}, {-0.7788760803906587, \
-0.4094464815990165}, {-0.7751375545403754, -0.41546741223865624`}, \
{-0.7713478758552859, -0.42146092518313855`}, {-0.7675071830808835, \
-0.42742656406976576`}, {-0.7636156188669637, -0.43336387233500706`}, \
{-0.7596733297853013, -0.4392723932415867}, {-0.7556804663472553, \
-0.4451516699058461}, {-0.7516371830212989, -0.45100124532538205`}, \
{-0.747543638250473, -0.4568206624069621}, {-0.7433999944697599, \
-0.46260946399471914`}, {-0.7392064181233763, -0.468367192898627}, \
{-0.734963079681981, -0.47409339192325883`}, {-0.730670153659798, \
-0.47978760389682984`}, {-0.7263278186316494, -0.48544937170052616`}, \
{-0.7219362572498979, -0.4910782382981219}, {-0.7174956562612951, \
-0.4966737467658859}, {-0.713006206523734, -0.5022354403227798}, \
{-0.7084681030229025, -0.5077628623609497}, {-0.7038815448888349, \
-0.5132555564765127}, {-0.69924673541236, -0.5187130665006401}, \
{-0.6945638820614425, -0.5241349365309399}, {-0.6898331964974137, \
-0.5295207109631388}, {-0.6850548945910914, -0.534869934523067}, \
{-0.6802291964387835, -0.5401821522989464}, {-0.6753563263781749, \
-0.5454569097739855}, {-0.6704365130040917, -0.5506937528592808}, \
{-0.6654699891841437, -0.5558922279270281}, {-0.6604569920742385, \
-0.5610518818440445}, {-0.6553977631339669, -0.5661722620056042}, \
{-0.6502925481418551, -0.5712529163695874}, {-0.6451415972104808, \
-0.5762933934909472}, {-0.6399451648014505, -0.581293242556494}, \
{-0.6347035097402345, -0.5862520134199997}, {-0.6294168952308564, \
-0.5911692566376247}, {-0.6240855888704341, -0.5960445235036672}, \
{-0.6187098626635681, -0.600877366086639}, {-0.6132899930365749, \
-0.6056673372656678}, {-0.6078262608515619, -0.6104139907672284}, \
{-0.6023189514203385, -0.6151168812022052}, {-0.5967683545181632, \
-0.6197755641032866}, {-0.5911747643973202, -0.6243895959626938}, \
{-0.5855384798005229, -0.6289585342702458}, {-0.5798598039741414, \
-0.6334819375517619}, {-0.5741390446812495, -0.6379593654078033}, \
{-0.5683765142144872, -0.6423903785527576}, {-0.562572529408735, \
-0.6467745388542643}, {-0.5567274116535975, -0.6511114093729861}, \
{-0.5508414869056903, -0.6554005544027266}, {-0.5449150857007273, \
-0.6596415395108963}, {-0.5389485431654047, -0.663833931579328}, \
{-0.5329421990290768, -0.6679772988454443}, {-0.52689639763522, \
-0.6720712109437785}, {-0.5208114879526806, -0.6761152389478502}, \
{-0.5146878235867025, -0.6801089554123977}, {-0.5085257627897294, \
-0.6840519344159691}, {-0.5023256684719789, -0.6879437516038732}, \
{-0.49608790821178267`, -0.691783984231492}, {-0.48981285426568827`, \
-0.695572211207957}, {-0.48350088357831905`, -0.69930801314019}, \
{-0.4771523777919867, -0.7029909723773111}, {-0.4707677232560518, \
-0.7066206730554145}, {-0.4643473110360281, -0.7101967011427142}, \
{-0.45789153692242524`, -0.7137186444850613}, {-0.45140080143932537`, \
-0.7171860928518342}, {-0.44487550985268887`, -0.7205986379822036}, \
{-0.4383160721783834, -0.7239558736317727}, {-0.43172290318993245`, \
-0.7272573956195965}, {-0.425096422425977, -0.7305028018755786}, \
{-0.4184370541974462, -0.7336916924882503}, {-0.41174522759443094`, \
-0.73682366975293}, {-0.40502137649275544`, -0.7398983382202677}, \
{-0.3982659395602415, -0.7429153047451736}, {-0.39147936026265956`, \
-0.7458741785361334}, {-0.38466208686936154`, -0.7487745712049109}, \
{-0.37781457245858935`, -0.7516160968166408}, {-0.3709372749224536, \
-0.7543983719403109}, {-0.3640306569715769, -0.7571210156996366}, \
{-0.3570951861393955, -0.7597836498243288}, {-0.35013133478611397`, \
-0.7623858987017553}, {-0.3431395801023061, -0.7649273894289993}, \
{-0.33612040411215677`, -0.7674077518653131}, {-0.32907429367633806`, \
-0.7698266186849713}, {-0.32200174049451363`, -0.7721836254305222}, \
{-0.3149032411074651, -0.7744784105664404}, {-0.30777929689883365`, \
-0.7767106155331801}, {-0.30063041409647107`, -0.7788798848016316}, \
{-0.2934571037733929, -0.7809858659279809}, {-0.2862598818483277, \
-0.7830282096089743}, {-0.27903926908585497`, -0.7850065697375885}, \
{-0.271795791096126, -0.7869206034591067}, {-0.26452997833415953`, \
-0.7887699712276031}, {-0.257242366098706, -0.7905543368628344}, \
{-0.24993349453067304`, -0.792273367607542}, {-0.24260390861110528`, \
-0.7939267341851625}, {-0.2352541581587107, -0.7955141108579497}, \
{-0.22788479782692672`, -0.7970351754855078}, {-0.22049638710051833`, \
-0.7984896095837368}, {-0.21308949029170057`, -0.7998770983841899}, \
{-0.20566467653577808`, -0.8011973308938442}, {-0.19822251978629363`, \
-0.8024499999552859}, {-0.19076359880967808`, -0.8036348023073084}, \
{-0.1832884971793937, -0.804751438645926}, {-0.17579780326956265`, \
-0.8057996136858017}, {-0.16829211024807297`, -0.8067790362220906}, \
{-0.16077201606915337`, -0.8076894191926991}, {-0.15323812346540863`, \
-0.8085304797409594}, {-0.14569103993930738`, -0.8093019392787194}, \
{-0.13813137775411338`, -0.8100035235498495}, {-0.13055975392425195`, \
-0.8106349626941644}, {-0.12297679020510258`, -0.8111959913117613}, \
{-0.1153831130822089, -0.8116863485277739}, {-0.10777935375989724`, \
-0.8121057780575412}, {-0.10016614814929434`, -0.8124540282721929}, \
{-0.09254413685573533, -0.8127308522646485}, {-0.08491396516555251, \
-0.812936007916032}, {-0.07727628303223556, -0.8130692579625005}, \
{-0.0696317450619536, -0.8131303700624861}, {-0.06198101049842952, \
-0.8131191168643512}, {-0.05432474320715677, -0.8130352760744561}, \
{-0.04666361165894871, -0.8128786305256379}, {-0.038998288912810514`, \
-0.8126489682461001}, {-0.031329452598123586`, -0.8123460825287124}, \
{-0.023657784896132086`, -0.8119697720007187}, {-0.015983972520721337`, \
-0.8115198406938527}, {-0.008308706698477537, -0.8109960981148602}, \
{-0.0006326831480181934, -0.8103983593164258}, {
      0.007043397941417491, -0.8097264449685043}, {
      0.01471883193212914, -0.808980181430053}, {
      0.022392909760877638`, -0.8081594008211652}, {
      0.030064917962580616`, -0.8072639410956024}, {
      0.03773413869479226, -0.8062936461137232}, {
      0.045399849762978776`, -0.8052483657158077}, {
      0.05306132464660118, -0.804127955795774}, {
      0.06071783252601691, -0.8029322783752865}, {
      0.06836863831021216, -0.8016612016782517}, {
      0.07601300266537686, -0.8003146002057013}, {
      0.0836501820443344, -0.7988923548110574}, {
      0.0912794287168382, -0.7973943527757796}, {
      0.09889999080074761, -0.7958204878853904}, {
      0.10651111229409564`, -0.7941706605058745}, {
      0.11411203310806096`, -0.7924447776604516}, {
      0.12170198910085728`, -0.7906427531067175}, {
      0.12928021211255267`, -0.7887645074141505}, {
      0.13684593000083228`, -0.7868099680419803}, {
      0.14439836667771733`, -0.7847790694174157}, {
      0.15193674214725392`, -0.7826717530142255}, {
      0.1594602725441852, -0.7804879674316719}, {
      0.16696817017362062`, -0.778227668473789}, {
      0.17445964355171564`, -0.7758908192290043}, {
      0.1819338974473767, -0.7734773901500979}, {
      0.18939013292500478`, -0.7709873591344949}, {
      0.1968275473882922, -0.768420711604886}, {
      0.20424533462508715`, -0.7657774405901719}, {
      0.21164268485334048`, -0.7630575468067256}, {
      0.2190187847681494, -0.7602610387399673}, {
      0.22637281758991315`, -0.757387932726248}, {
      0.2337039631136156, -0.7544382530350329}, {
      0.24101139775925023`, -0.7514120319513815}, {
      0.24829429462340263`, -0.7483093098587172}, {
      0.2555518235320062, -0.7451301353218795}, {
      0.2627831510942868, -0.7418745651704532}, {
      0.26998744075791237`, -0.738542664582367}, {
      0.27716385286536316`, -0.7351345071677546}, {
      0.2843115447115393, -0.7316501750530723}, {
      0.2914296706026218, -0.7280897589654626}, {
      0.2985173819162037, -0.7244533583173598}, {
      0.3055738271627079, -0.7207410812913269}, {
      0.3125981520481088, -0.716953044925116}, {
      0.3195894995379748, -0.7130893751969453}, {
      0.3265470099228486, -0.7091502071109812}, {
      0.3334698208849832, -0.7051356847830195}, {
      0.340357067566451, -0.7010459615263536}, {
      0.34720788263864344`, -0.696881199937822}, {
      0.35402139637317953`, -0.6926415719840248}, {
      0.36079673671424145`, -0.6883272590876973}, {
      0.367533029352355, -0.6839384522142341}, {
      0.37422939779963393`, -0.6794753519583481}, {
      0.3808849634665064, -0.6749381686308569}, {
      0.3874988457399427, -0.6703271223455837}, {
      0.39407016206320267`, -0.6656424431063611}, {
      0.400598028017123, -0.6608843708941257}, {
      0.4070815574029626, -0.6560531557540907}, {
      0.4135198623268262, -0.6511490578829853}, {
      0.41991205328568615`, -0.6461723477163447}, {
      0.42625723925502146`, -0.6411233060158404}, {
      0.4325545277780949, -0.6360022239566354}, {
      0.43880302505688784`, -0.6308094032147489}, {
      0.445001836044714, -0.6255451560544183}, {
      0.45115006454053147`, -0.6202098054154418}, {
      0.45724681328497524`, -0.6148036850004862}, {
      0.46329118405812986`, -0.6093271393623443}, {
      0.46928227777906406`, -0.6037805239911255}, {
      0.4752191946071487, -0.5981642054013631}, {
      0.48110103404517873`, -0.5924785612190206}, {
      0.486926895044322, -0.5867239802683799}, {
      0.4926958761109155, -0.5809008626587921}, {
      0.4984070754151315, -0.5750096198712744}, {
      0.5040595909015365, -0.5690506748449317}, {
      0.5096525204015635, -0.5630244620631847}, {
      0.5151849617479219, -0.5569314276397846}, {
      0.520656012890967, -0.5507720294045926}, {
      0.5260647720170506, -0.5445467369891038}, {
      0.5314103376688788, -0.5382560319116947}, {
      0.5366918088678962, -0.5319004076625699}, {
      0.5419082852387234, -0.5254803697883874}, {
      0.5470588671356679, -0.5189964359765384}, {
      0.5521426557713356, -0.5124491361390568}, {
      0.5571587533473633, -0.505839012496135}, {
      0.5621062631872978, -0.49916661965922116`}, {
      0.5669842898716457, -0.4924325247136702}, {
      0.5717919393751172, -0.4856373073009249}, {
      0.5765283192060892, -0.4787815597001975}, {
      0.581192538548311, -0.4718658869096258}, {
      0.5857837084048788, -0.464890906726875}, {
      0.5903009417445014, -0.4578572498291556}, {
      0.5947433536500842, -0.45076555985262834`}, {
      0.5991100614696555, -0.44361649347116544`}, {
      0.6034001849696592, -0.436410720474437}, {
      0.607612846490641, -0.42914892384529046`}, {
      0.6117471711053518, -0.4218317998363903}, {
      0.6158022867792947, -0.41446005804608504`}, {
      0.6197773245337398, -0.40703442149346686`}, {
      0.6236714186112338, -0.39955562669258854`}, {
      0.6274837066436292, -0.3920244237258021}, {
      0.6312133298226588, -0.3844415763161827}, {
      0.6348594330730817, -0.3768078618989992}, {
      0.6384211652284261, -0.369124071692194}, {
      0.6418976792093553, -0.3613910107658322}, {
      0.6452881322046824, -0.35360949811047926`}, {
      0.6485916858550601, -0.3457803667044673}, {
      0.6518075064393711, -0.3379044635800065}, {
      0.6549347650638456, -0.32998264988809894`}, {
      0.6579726378539319, -0.32201580096221133`}, {
      0.6609203061489453, -0.31400480638066014`}, {
      0.6637769566995229, -0.3059505700276643}, {
      0.6665417818679088, -0.2978540101530172}, {
      0.6692139798310968, -0.28971605943033074`}, {
      0.6717927547868552, -0.2815376650138006}, {
      0.6742773171626616, -0.27331978859344386`}, {
      0.6766668838275716, -0.2650634064487557}, {
      0.678960678307048, -0.2567695095007339}, {
      0.6811579310007766, -0.24843910336221603`}, {
      0.6832578794034929, -0.2400732083864746}, {
      0.6852597683288467, -0.23167285971401377`}, {
      0.6871628501363284, -0.2232391073175097}, {
      0.6889663849612834, -0.21477301604483606`}, {
      0.6906696409480388, -0.20627566566011393`}, {
      0.6922718944861677, -0.19774815088272482`}, {
      0.6937724304499147, -0.18919158142422388`}, {
      0.6951705424408092, -0.18060708202308887`}, {
      0.6964655330334883, -0.1719957924772394}, {
      0.6976567140247538, -0.16335886767425892`}, {
      0.6987434066858884, -0.15469747761925134`}, {
      0.6997249420182522, -0.14601280746026193`}, {
      0.7006006610121844, -0.1373060575111908}, {
      0.701369914909232, -0.12857844327212606`}, {
      0.702032065467727, -0.11983119544702198`}, {
      0.7025864852317364, -0.11106555995864542`}, {
      0.7030325578034036, -0.1022827979607131}, {
      0.7033696781187043, -0.09348418584713972}, {
      0.7035972527266372, -0.08467101525831577}, {
      0.7037147000718681, -0.07584459308433213}, {
      0.7037214507808494, -0.06700624146506624}, {
      0.7036169479514315, -0.0581572977870438}, {
      0.7034006474459865, -0.04929911467698712}, {
      0.703072018188061, -0.04043305999196014}, {
      0.7026305424625747, -0.03156051680601786}, {
      0.7020757162195831, -0.022682883393266063`}, {
      0.7014070493816171, -0.01380157320723531}, {
      0.7006240661546179, -0.0049180148564711805`}, {0.6997263053424779, 
      0.003966347923759363}, {0.6987133206652038, 0.012850056303747609`}, {
      0.697584681080712, 0.02173163639571303}, {0.6963399711102701, 
      0.030609599294369853`}, {0.6949787911675922, 0.03948244112179893}, {
      0.6935007578916002, 0.04834864307673567}, {0.6919055044828575, 
      0.05720667148838709}, {0.6901926810436841, 0.06605497787489338}, {
      0.6883619549219576, 0.07489199900655175}, {0.6864130110586052, 
      0.08371615697392241}, {0.6843455523387922, 0.09252585926093958}, {
      0.6821592999468069, 0.10131949882315205`}, {0.6798539937246436, 
      0.1100954541712212}, {0.6774293925342837, 0.11885208945980608`}, {
      0.6748852746236722, 0.1275877545819685}, {0.6722214379963859, 
      0.13630078526923317`}, {0.6694377007849881, 0.14498950319744086`}, {
      0.6665339016280625, 0.15365221609853516`}, {0.6635099000509177, 
      0.16228721787842632`}, {0.6603655768499517, 0.17089278874107827`}, {
      0.6571008344806625, 0.17946719531896801`}, {0.6537155974492921, 
      0.188008690810069}, {0.6502098127080849, 0.19651551512151375`}, {
      0.6465834500541427, 0.20498589502009354`}, {0.6428365025318551, 
      0.21341804428975564`}, {0.63896898683888, 0.22181016389626262`}, {
      0.6349809437356504, 0.2301604421591806}, {0.6308724384583764, 
      0.23846705493136652`}, {0.626643561135512, 0.2467281657861283}, {
      0.6222944272076532, 0.2549419262122343}, {0.6178251778508279, 
      0.26310647581695257`}, {0.6132359804031404, 0.2712199425373026}, {
      0.6085270287947249, 0.27928044285970716`}, {0.6036985439809623, 
      0.28728608204823414`}, {0.5987507743789114, 0.295234954381622}, {
      0.5936839963068989, 0.3031251433992866}, {0.5884985144272136, 
      0.3109547221565096}, {0.5831946621918424, 0.31872175348901394`}, {
      0.5777728022911844, 0.3264242902871335}, {0.5722333271056741, 
      0.33406037577978964`}, {0.56657665916024, 0.34162804382849027`}, {
      0.5608032515815232, 0.3491253192315703}, {0.5549135885577725, 
      0.35655021803889764`}, {0.5489081858013306, 0.36390074787727145`}, {
      0.5427875910136196, 0.37117490828674404`}, {0.53655238435253, 
      0.37837069106810134`}, {0.5302031789021111, 0.3854860806417416}, {
      0.5237406211444561, 0.3925190544181949}, {0.5171653914336699, 
      0.39946758318053116`}, {0.5104782044718009, 0.40632963147890794`}, {
      0.5036798097866122, 0.41310315803751385`}, {0.496770992211062, 
      0.4197861161741668}, {0.4897525723643571, 0.42637645423283155`}, {
      0.4826254071344339, 0.4328721160293247}, {0.4753903901617188, 
      0.4392710413104793}, {0.4680484523240083, 0.44557116622704623`}, {
      0.46060056222230517`, 0.45177042382061344`}, {0.453047726667437, 
      0.4578667445248279}, {0.4453909911672774, 0.4638580566812109}, {
      0.4376314404143804, 0.4697422870698601}, {0.42977019877383105`, 
      0.47551736145533746`}, {0.42180843077110597`, 0.4811812051480451}, {
      0.4137473415797288, 0.486731743581397}, {0.4055881775084961, 
      0.4921669029050983}, {0.397332226488039, 0.49748461059484717`}, {
      0.38898081855647715`, 0.5026827960787806}, {0.3805353263439089, 
      0.5077593913809885}, {0.37199716555547374`, 0.5127123317824239}, {
      0.3633677954527092, 0.5175395564995453}, {0.3546487193329148, 
      0.522239009381025}, {0.34584148500622286`, 0.5268086396228681}, {
      0.3369476852700637, 0.531246402502286}, {0.3279689583807009, 
      0.5355502601306764}, {0.3189069885214975, 0.5397181822260606}, {
      0.3097635062675626, 0.5437481469053398}, {0.3005402890464117, 
      0.5476381414967294}, {0.29123916159426144`, 0.551386163372738}, {
      0.2818619964075624, 0.5549902208040607}, {0.27241071418936086`, 
      0.5584483338347582}, {0.26288728429006164`, 0.5617585351791003}, {
      0.25329372514214915`, 0.5649188711404514}, {0.24363210468840676`, 
      0.5679274025525816}, {0.23390454080315648`, 0.5707822057437902}, {
      0.22411320170602264`, 0.573481373524229}, {0.2142603063677052, 
      0.5760230161968174}, {0.20434812490722792`, 0.5784052625921434}, {
      0.19437897898010734`, 0.5806262611277465}, {0.18435524215686763`, 
      0.5826841808921807}, {0.17427934029130482`, 0.5845772127542572}, {
      0.16415375187788175`, 0.5863035704978693}, {0.15398100839761295`, 
      0.5878614919828001}, {0.1437636946517743, 0.5892492403319195}, {
      0.13350444908274886`, 0.5904651051451708}, {0.12320596408129447`, 
      0.5915074037407551}, {0.11287098627949363`, 0.592374482423914}, {
      0.1025023168286188, 0.593064717783717}, {0.09210281166111942, 
      0.5935765180182533}, {0.0816753817359081, 0.5939083242886306}, {
      0.07122299326609437, 0.5940586121021797}, {0.0607486679282841, 
      0.5940258927252593}, {0.05025548305253195, 0.5938087146260569}, {
      0.03974657179200139, 0.593405664947772}, {0.029225123271354556`, 
      0.5928153710125698}, {0.018694382712859454`, 0.5920365018566818}, {
      0.008157651539167265, 0.5910677697970311}, {-0.002381712548323936, 
      0.5899079320297467}, {-0.01292029551664001, 
      0.5885557922609284}, {-0.02345462697509562, 
      0.5870102023700142}, {-0.033981180177052234`, 
      0.5852700641060928}, {-0.04449637205080921, 
      0.5833343308174928}, {-0.054996563260910056`, 
      0.5812020092149704}, {-0.06547805830118918, 
      0.5788721611688061}, {-0.07593710562092912, 
      0.5763439055401025}, {-0.08636989778554419, 
      0.5736164200465673}, {-0.09677257167325369, 
      0.5706889431630444}, {-0.10714120870925666`, 
      0.5675607760570412}, {-0.11747183513896989`, 
      0.5642312845594804}, {-0.12776042234194293`, 
      0.5606999011708844}, {-0.1380028871881163, 
      0.5569661271031795}, {-0.1481950924381439, 
      0.5530295343572803}, {-0.1583328471895569, 
      0.5488897678365934}, {-0.16841190737060302`, 
      0.5445465474965483}, {-0.17842797628365592`, 
      0.5399996705302367}, {-0.18837670520014857`, 
      0.5352490135902067}, {-0.198253694009048, 
      0.5302945350464288}, {-0.20805449192095293`, 
      0.5251362772804089}, {-0.2177745982299612, 
      0.5197743690153909}, {-0.227409463135522, 
      0.5142090276825433}, {-0.23695448862655705`, 
      0.5084405618229866}, {-0.24640502943020642`, 
      0.5024693735254658}, {-0.2557563940276271, 
      0.496295960899425}, {-0.2650038457393479, 
      0.48992092058318615`}, {-0.2741426038827604, 
      0.4833449502868779}, {-0.28316784500440517`, 
      0.47656885136969906`}, {-0.29207470418979103`, 
      0.46959353145103655`}, {-0.3008582764535699, 
      0.46242000705488984`}, {-0.30951361821297224`, 
      0.4550494062869804}, {-0.31803574884749464`, 
      0.4474829715438475}, {-0.3264196523479195, 
      0.43972206225314964`}, {-0.3346602790578361, 
      0.43176815764430404`}, {-0.3427525475109249, 
      0.4236228595485035}, {-0.35069134636735955`, 
      0.41528789522705306`}, {-0.35847153645277735`, 
      0.4067651202268651}, {-0.366087952903366, 
      0.3980565212618407}, {-0.37353540742071306`, 
      0.38916421911875027`}, {-0.3808086906401662, 
      0.38009047158610243`}, {-0.38790257461655364`, 
      0.37083767640436055`}, {-0.3948118154312191, 
      0.36140837423572847`}, {-0.40153115592443067`, 
      0.3518052516515814}, {-0.4080553285573298, 
      0.34203114413546404`}, {-0.414379058407695, 
      0.3320890390994161}, {-0.42049706630390504`, 
      0.3219820789112122}, {-0.4264040721015949, 
      0.3117135639299234}, {-0.4320947981076117, 
      0.30128695554701457`}, {-0.437563972655988, 
      0.29070587922999097`}, {-0.4428063338407635, 
      0.2799741275653922}, {-0.44781663341059896`, 
      0.2690956632977085}, {-0.452589640830239, 
      0.25807462236055473`}, {-0.45712014751399377`, 
      0.2469153168961883}, {-0.4614029712365218, 
      0.23562223825919199`}, {-0.46543296072630763`, 
      0.22420005999986414`}, {-0.46920500044733887`, 
      0.2126536408225655}, {-0.47271401557459547`, 
      0.20098802751396122`}, {-0.47595497716907176`, 
      0.18920845783577156`}, {-0.47892290755815575`, 
      0.17732036337629994`}, {-0.48161288592729123`, 
      0.165329372354646}, {-0.4840200541289468, 
      0.15324131237112937`}, {-0.4861396227150089, 
      0.1410622130970485}, {-0.48796687719880494`, 
      0.1287983088964769}, {-0.4894971845530452, 
      0.11645604137235349`}, {-0.4907259999500484, 
      0.10404206182865655`}, {-0.491648873750686, 
      0.09156323363995775}, {-0.49226145874853855`, 
      0.07902663451913515}, {-0.49255951767581113`, 
      0.06643955867347921}, {-0.4925389309775923, 
      0.05380951883885303}, {-0.492195704861072, 
      0.04114424818096601}, {-0.4915259796263461, 
      0.028451702052187257`}, {-0.49052603828543745`, 
      0.015740059591660083`}, {-0.48919231547614483`, 
      0.0030177251557802175`}, {-0.48752140667729743`, \
-0.009706670434633217}, {-0.4855100777319357, -0.022424268844915607`}, \
{-0.4831552746848629, -0.03512598339013412}, {-0.48045413394090747`, \
-0.04780249842776378}, {-0.4774039927501072, -0.06044426899645261}, \
{-0.4740024000258653, -0.07304152071880052}, {-0.4702471275019357, \
-0.0855842499870566}, {-0.46613618123386463`, -0.09806222445166657}, \
{-0.46166781345024727`, -0.11046498383367878`}, {-0.4568405347588441, \
-0.1227818410831452}, {-0.4516531267122428, -0.13500188390683474`}, \
{-0.4461046547373365, -0.14711397668981274`}, {-0.44019448143242323`, \
-0.1591067628367349}, {-0.43392228023519547`, -0.17096866756005655`}, \
{-0.42728804946429333`, -0.1826879011437742}, {-0.42029212673641914`, \
-0.1942524627127939}, {-0.41293520376025794`, -0.20565014453956637`}, \
{-0.4052183415076074, -0.2168685369212407}, {-0.3971429857611827, \
-0.22789503366226982`}, {-0.38871098303752194`, -0.23871683819915532`}, \
{-0.37992459688226277`, -0.24932097040584533`}, {-0.3707865245337846, \
-0.2596942741202014}, {-0.3612999139498007, -0.26982342543392823`}, \
{-0.35146838118992885`, -0.27969494179041676`}, {-0.3412960281455557, \
-0.28929519193708536`}, {-0.33078746060642805`, -0.29861040678101824`}, \
{-0.31994780665133327`, -0.30762669119899466`}, {-0.30878273534796125`, \
-0.3163300368553757}, {-0.29729847574455165`, -0.3247063360837687}, \
{-0.28550183613320496`, -0.3327413968909202}, {-0.27340022356175286`, \
-0.34042095914389536`}, {-0.26100166356782584`, -0.3477307120042822}, \
{-0.24831482010519249`, -0.3546563126759093}, {-0.23534901562856092`, \
-0.3611834065353821}, {-0.22211425129879064`, -0.36729764871761594`}, \
{-0.208621227265843, -0.3729847272314739}, {-0.19488136298176278`, \
-0.3782303876835856}, {-0.18090681749050117`, -0.3830204596914267}, \
{-0.166710509635425, -0.387340885069762}, {-0.15230613811886862`, \
-0.39117774787758075`}, {-0.1377082013410316, -0.3945173064156661}, \
{-0.1229320169378622, -0.39734602726791973`}, {-0.10799374092924403`, \
-0.3996506214824791}, {-0.09291038637976971, -0.40141808299149745`}, \
{-0.07769984146458389, -0.40263572937116177`}, {-0.06238088682214894, \
-0.4032912450460732}, {-0.046973212064264774`, -0.4033727270444552}, \
{-0.03149743130119054, -0.4028687334127393}, {-0.015975097526197243`, \
-0.40176833439984855`}, {-0.00042871568924624827`, -0.40006116652288093`}, {
      0.015118245726343732`, -0.3977374896268213}, {
      0.0306413448274993, -0.3947882470512804}, {
      0.04611515835702697, -0.39120512901698157`}, {
      0.06151327590303032, -0.3869806393436682}, {
      0.07680829653543925, -0.3821081656091572}, {
      0.09197182815626244, -0.37658205285626634`}, {
      0.10697448987527712`, -0.37039768095012426`}, {
      0.12178591774991608`, -0.3635515456827375}, {
      0.1363747742572309, -0.3560413437144149}, {
      0.1507087618971343, -0.3478660614324921}, {
      0.1647546413597893, -0.3390260677964637}, {
      0.17847825472615972`, -0.32952321122480327`}, {
      0.1918445542095039, -0.31936092056205806`}, {
      0.20481763698712147`, -0.30854431014483885`}, {
      0.21736078671609133`, -0.297080288961606}, {
      0.229436522374194, -0.28497767387315776`}, {
      0.24100665511781955`, -0.2722473068278459}, {
      0.25203235390252643`, -0.25890217596709963`}, {
      0.2624742206691274, -0.24495754047205606`}, {
      0.2722923759587906, -0.23043105895010987`}, {
      0.2814465558846856, -0.2153429211000095}, {
      0.2898962214551457, -0.19971598232464183`}, {
      0.2976006813140761, -0.18357590088059744`}, {
      0.3045192290382489, -0.1669512770615794}, {
      0.31061129620792993`, -0.14987379380711266`}, {
      0.3158366225465921, -0.13237835800702755`}, {
      0.3201554445067579, -0.11450324163379952`}, {
      0.32352870376154547`, -0.0962902216767513}, {
      0.3259182771443212, -0.07778471767179569}, {
      0.3272872296607382, -0.05903592541494292}, {
      0.3276000922767727, -0.04009694521399018}, {
      0.3268231662611369, -0.021024902767043965`}, {
      0.3249248559280932, -0.0018810604547694135`}, {0.3218760316840402, 
      0.017269083508975887`}, {0.31765042532430277`, 0.03635571101117178}, {
      0.312225059550431, 0.055304618440077}, {0.30558071367690753`, 
      0.07403717342961864}, {0.29770242746201026`, 0.09247029838483295}, {
      0.2885800449214942, 0.11051648585610599`}, {0.2782087998544914, 
      0.12808385153959004`}, {0.2665899446148329, 0.14507623147826065`}, {
      0.2537314233810784, 0.161393330931614}, {0.23964859079442863`, 
      0.1769309333813471}, {0.22436497632046987`, 0.19158117925505788`}, {
      0.20791309401804836`, 0.2052329251894603}, {0.19033529652965964`, 
      0.21777219602766126`}, {0.17168467099786355`, 0.22908274325928676`}, {
      0.15202597320718603`, 0.2390467252731082}, {0.1314365944850739, 
      0.24754552660129145`}, {0.11000755368926685`, 0.25446073528921803`}, {
      0.0878445038664124, 0.25967529961405844`}, {0.06506873977200142, 
      0.2630748875769091}, {0.041818188255181875`, 0.26454947486968944`}, {
      0.01824835836599871, 0.26399518930964105`}, {-0.005466778263865536, 
      0.26131644195140125`}, {-0.029134013922671613`, 
      0.2564283770977872}, {-0.05254028475083308, 
      0.2492596750471085}, {-0.07545226206695535, 
      0.23975574237064878`}, {-0.09761622688282424, 
      0.22788232443664938`}, {-0.11875831610868967`, 
      0.21362957327070398`}, {-0.13858524905262215`, 
      0.19701659995625423`}, {-0.15678566689917794`, 
      0.17809653365881106`}, {-0.17303224654299829`, 
      0.15696209767374514`}, {-0.18698478409809866`, 
      0.1337516948400155}, {-0.19829448320507917`, 
      0.10865596777423064`}, {-0.2066097293662359, 
      0.08192476033438567}, {-0.2115836840667082, 
      0.05387435102754685}, {-0.21288409085940824`, 
      0.02489475067972956}, {-0.2102057482262403, -0.004543252530727942}, \
{-0.20328616723585874`, -0.03388176845673946}, {-0.1919249888716154, \
-0.06247087549274436}, {-0.1760077740728414, -0.08956604278424529}, \
{-0.15553477789796422`, -0.11432924193037877`}, {-0.13065524245388363`, \
-0.13583559739401524`}, {-0.10170753317170061`, -0.1530881038752921}, \
{-0.06926500486424486, -0.16504382778307797`}, {-0.034186664537136945`, \
-0.1706561492976457}, {0.0023297490040064955`, -0.16893901396410027`}, {
      0.038697115752864185`, -0.15906081490392834`}, {
      0.072899669891783, -0.14047724622420474`}, {
      0.10248078782434368`, -0.11311376713863257`}, {
      0.12458427512388298`, -0.07760802055305914}, {
      0.13609252773404265`, -0.03561802601949393}, {0.1339306357890458, 
      0.009812481920281904}, {0.11564204410304955`, 0.053876634180653665`}, {
      0.08038602797095548, 0.08974347282769791}, {0.030537584629747094`, 
      0.10872309397687983`}, {-0.026003514050549445`, 
      0.1015412725611254}, {-0.07317456965702043, 
      0.06211579346771318}, {-0.08605986859986589, -0.003730783290772803}, \
{-0.04076103441800005, -0.06248959340697579}, {
      0.03905918860745593, -0.043971776278526414`}, {0.024099829265506377`, 
      0.02558522501812875}}]}, 
   {RGBColor[1, 1, 0], DiskBox[{0, 0}, 0.03]}},
  AspectRatio->Automatic,
  Background->RGBColor[0.6, 0.6, 0.6],
  ImageSize->{325., Automatic}]], "Output",
 CellChangeTimes->{3.401201222399558*^9, 3.401276900187669*^9}]
}, Open  ]],

Cell[TextData[{
 "Now try ",
 Cell[BoxData[
  FormBox[
   RowBox[{"V0", "=", 
    RowBox[{"1.01", "\[Times]", "2", " ", "\[Pi]"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tFinal", "=", "4"}], ";", 
  RowBox[{"R0", "=", "1"}], ";", 
  RowBox[{"V0", "=", 
   RowBox[{"1.01", "\[Times]", "2", " ", "\[Pi]"}]}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", ".001"}], ";", 
  RowBox[{"p", "=", "3"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CalculateOrbit", ";", 
  RowBox[{"DisplayOrbit", "[", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Clear", "[", 
  RowBox[{
  "tFinal", ",", "R0", ",", "V0", ",", "iMaximum", ",", "t", ",", "r", ",", 
   "v"}], "]"}]}], "Input",
 CellChangeTimes->{{3.401193360004507*^9, 3.401193414015883*^9}, 
   3.40119347898011*^9, 3.401193626282501*^9, {3.4012012039931593`*^9, 
   3.401201247843038*^9}}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 1, 1], Thickness[0.0075], 
    LineBox[{{1, 0}, {0.999980260861357, 0.006345975405148677}, {
      0.9999210442873496, 0.0126917002826508}, {0.9998223528036829, 
      0.019036924117135694`}, {0.9996841906197027, 0.025381396417783572`}, {
      0.9995065636281499, 0.03172486673059886}, {0.9992894794048184, 
      0.03806708465068089}, {0.999032947208114, 0.0444077998344913}, {
      0.998736977978516, 0.050746762012117}, {0.9984015843379415, 
      0.05708372099952815}, {0.9980267805890103, 0.06341842671083006}, {
      0.9976125827142143, 0.06975062917050828}, {0.9971590083749874, 
      0.07608007852566606}, {0.996666076910679, 0.08240652505825317}, {
      0.9961338093374297, 0.08872971919728541}, {0.9955622283469494, 
      0.09504941153105384}, {0.9949513583051988, 0.10136535281932296`}, {
      0.9943012252509728, 0.10767729400551687`}, {0.9936118568943876, 
      0.11398498622889276`}, {0.992883282615271, 0.1202881808367007}, {
      0.9921155334614554, 0.12658662939632895`}, {0.991308642146974, 
      0.132880083707434}, {0.9904626430501615, 0.13916829581405446`}, {
      0.9895775722116575, 0.14545101801670784`}, {0.9886534673323145, 
      0.1517280028844697}, {0.9876903677710093, 0.15799900326703395`}, {
      0.9866883145423586, 0.16426377230675374`}, {0.9856473503143398, 
      0.17052206345066193`}, {0.9845675194058155, 0.17677363046247058`}, {
      0.9834488677839632, 0.1830182274345482}, {0.9822914430616098, 
      0.18925560879987438`}, {0.9810952944944714, 0.19548552934397065`}, {
      0.9798604729782989, 0.20170774421680696`}, {0.9785870310459289, 
      0.20792200894468282`}, {0.9772750228642403, 0.2141280794420824}, {
      0.9759245042310177, 0.2203257120235026}, {0.9745355325717211, 
      0.2265146634152536}, {0.9731081669361616, 0.23269469076723057`}, {
      0.9716424679950854, 0.23886555166465645`}, {0.9701384980366635, 
      0.24502700413979414`}, {0.9685963209628907, 0.25117880668362813`}, {
      0.9670160022858907, 0.25732071825751407`}, {0.9653976091241305, 
      0.2634524983047961}, {0.9637412101985434, 0.2695739067623906}, {
      0.9620468758285596, 0.27568470407233614`}, {0.9603146779280473, 
      0.28178465119330803`}, {0.9585446900011623, 0.2878735096120978}, {
      0.956736987138108, 0.29395104135505573`}, {0.9548916460108049, 
      0.3000170089994965}, {0.9530087448684714, 0.3060711756850666}, {
      0.951088363533115, 0.31211330512507296`}, {0.9491305833949343, 
      0.3181431616177725}, {0.9471354874076338, 0.3241605100576207}, {
      0.9451031600836494, 0.3301651159464797}, {0.9430336874892874, 
      0.33615674540478424`}, {0.9409271572397754, 0.3421351651826654}, {
      0.9387836584942268, 0.34810014267103073`}, {0.9366032819505192, 
      0.35405144591260057`}, {0.934386119840086, 0.3599888436128995}, {
      0.9321322659226234, 0.3659121051512024}, {0.9298418154807117, 
      0.3718210005914343}, {0.9275148653143517, 0.37771530069302317`}, {
      0.9251515137354179, 0.38359477692170524`}, {0.922751860562027, 
      0.38945920146028173`}, {0.9203160071128227, 0.39530834721932684`}, {
      0.9178440562011793, 0.4011419878478457}, {0.9153361121293211, 
      0.40695989774388197`}, {0.9127922806823612, 0.41276185206507415`}, {
      0.9102126691222581, 0.4185476267391603}, {0.9075973861816917, 
      0.42431699847442994`}, {0.9049465420578591, 0.430069744770123}, {
      0.9022602484061897, 0.4358056439267747}, {0.8995386183339816, 
      0.44152447505650594`}, {0.8967817663939592, 0.4472260180932585}, {
      0.8939898085777517, 0.45291005380297444`}, {0.891162862309294, 
      0.45857636379371874`}, {0.888301046438151, 0.4642247305257451}, {
      0.8854044812327642, 0.4698549373215035}, {0.8824732883736223, 
      0.47546676837559}, {0.8795075909463564, 0.4810600087646366}, {
      0.8765075134347601, 0.48663444445714227`}, {0.8734731817137343, 
      0.4921898623232428}, {0.8704047230421591, 0.49772605014442056`}, {
      0.8673022660556913, 0.5032427966231522}, {0.8641659407594896, 
      0.5087398913924941}, {0.8609958785208675, 0.5142171250256057}, {
      0.8577922120618742, 0.5196742890452088}, {0.8545550754518048, 
      0.5251111759329838}, {0.8512846040996398, 0.5305275791389004}, {
      0.8479809347464148, 0.5359232930904838}, {0.8446442054575207, 
      0.5412981132020157}, {0.8412745556149359, 0.5466518358836685}, {
      0.8378721259093893, 0.5519842585505726}, {0.8344370583324572, 
      0.5572951796318175}, {0.8309694961685924, 0.5625843985793837}, {
      0.8274695839870878, 0.5678517158770071}, {0.8239374676339742, 
      0.5730969330489746}, {0.8203732942238531, 0.5783198526688496}, {
      0.8167772121316653, 0.5835202783681283}, {0.8131493709843964, 
      0.5886980148448251}, {0.8094899216527189, 0.5938528678719874}, {
      0.8057990162425719, 0.598984644306139}, {0.8020768080866795, 
      0.6040931520956507}, {0.7983234517360086, 0.6091782002890401}, {
      0.7945391029511657, 0.6142395990431972}, {0.7907239186937345, 
      0.6192771596315372}, {0.7868780571175545, 0.6242906944520792}, {
      0.7830016775599417, 0.6292800170354513}, {0.779094940532851, 
      0.6342449420528206}, {0.7751580077139828, 0.6391852853237474}, {
      0.7711910419378326, 0.6441008638239651}, {0.7671942071866853, 
      0.6489914956930833}, {0.763167668581555, 0.6538570002422145}, {
      0.7591115923730709, 0.658697197961524}, {0.755026145932309, 
      0.6635119105277031}, {0.7509114977415722, 0.6683009608113637}, {
      0.7467678173851178, 0.6730641728843557}, {0.7425952755398341, 
      0.6778013720270056}, {0.7383940439658664, 0.6825123847352764}, {
      0.7341642954971934, 0.6871970387278481}, {0.7299062040321541, 
      0.691855162953119}, {0.725619944523927, 0.696486587596127}, {
      0.7213056929709614, 0.7010911440853907}, {0.7169636264073618, 
      0.7056686650996696}, {0.7125939228932265, 0.710218984574644}, {
      0.7081967615049405, 0.7147419377095131}, {0.7037723223254237, 
      0.7192373609735114}, {0.699320786434336, 0.7237050921123445}, {
      0.694842335898238, 0.7281449701545406}, {0.6903371537607101, 
      0.7325568354177218}, {0.6858054240324305, 0.7369405295147906}, {
      0.6812473316812108, 0.741295895360035}, {0.6766630626219926, 
      0.7456227771751492}, {0.6720528037068048, 0.749921020495171}, {
      0.6674167427146812, 0.7541904721743353}, {0.6627550683415417, 
      0.7584309803918442}, {0.6580679701900349, 0.7626423946575513}, {
      0.6533556387593454, 0.7668245658175626}, {0.6486182654349654, 
      0.7709773460597522}, {0.6438560424784306, 0.7751005889191926}, {
      0.6390691630170238, 0.7791941492835004}, {0.6342578210334427, 
      0.7832578833980959}, {0.6294222113554382, 0.7872916488713774}, {
      0.624562529645418, 0.7912953046798092}, {0.6196789723900213, 
      0.7952687111729244}, {0.6147717368896624, 0.79921173007824}, {
      0.609841021248045, 0.803124224506087}, {0.6048870243616478, 
      0.8070060589543533}, {0.5999099459091826, 0.8108570993131395}, {
      0.5949099863410244, 0.8146772128693288}, {0.5898873468686164, 
      0.8184662683110687}, {0.5848422294538482, 0.8222241357321662}, {
      0.5797748367984099, 0.8259506866363958}, {0.574685372333122, 
      0.8296457939417194}, {0.5695740402072423, 0.8333093319844197}, {
      0.56444104527775, 0.8369411765231447}, {0.559286593098608, 
      0.8405412047428656}, {0.5541108899100047, 0.8441092952587461}, {
      0.5489141426275757, 0.8476453281199242}, {0.5436965588316054, 
      0.8511491848132058}, {0.5384583467562111, 0.8546207482666701}, {
      0.533199715278508, 0.8580599028531877}, {0.5279208739077584, 
      0.8614665343938489}, {0.5226220327745041, 0.8648405301613058}, {
      0.5173034026196829, 0.8681817788830244}, {0.5119651947837308, 
      0.8714901707444493}, {0.5066076211956709, 0.87476559739208}, {
      0.5012308943621876, 0.8780079519364583}, {0.49583522735668956`, 
      0.8812171289550683}, {0.49042083380836093`, 0.8843930244951472}, {
      0.4849879278912012, 0.8875355360764081}, {0.4795367243130553, 
      0.8906445626936744}, {0.47406743830463416`, 0.8937200048194258}, {
      0.46858028560852727`, 0.8967617644062564}, {0.4630754824682065, 
      0.8997697448892438}, {0.45755324561702376`, 0.9027438511882305}, {
      0.4520137922672016, 0.9056839897100173}, {0.44645734009881916`, 
      0.9085900683504676}, {0.44088410724879246`, 0.911461996496525}, {
      0.4352943122998513, 0.9142996850281417}, {0.4296881742695129, 
      0.9171030463201195}, {0.4240659125990524, 0.9198719942438632}, {
      0.41842774714247255`, 0.9226064441690456}, {0.41277389815547155`, 
      0.9253063129651851}, {0.4071045862844113, 0.9279715190031361}, {
      0.4014200325552857, 0.9306019821564917}, {0.3957204583626904, 
      0.9331976238028985}, {0.39000608545879445`, 0.9357583668252851}, {
      0.38427713594231483`, 0.9382841356130028}, {0.37853383224749426`, 
      0.94077485606288}, {0.37277639713308314`, 0.9432304555801888}, {
      0.36700505367132674`, 0.9456508630795262}, {0.3612200252369577, 
      0.9480360089856075}, {0.3554215354961951, 0.9503858252339744}, {
      0.3496098083957507, 0.9527002452716165}, {0.3437850681518427, 
      0.9549792040575061}, {0.33794753923921844`, 0.957222638063049}, {
      0.3320974463801862, 0.9594304852724473}, {0.32623501453365666`, 
      0.9616026851829789}, {0.32036046888419584`, 0.96373917880519}, {
      0.31447403483108854`, 0.9658399086630041}, {0.30857593797741456`, 
      0.9679048187937448}, {0.3026664041191373, 0.9699338547480751}, {
      0.29674565923420626`, 0.9719269635898516}, {0.2908139294716732, 
      0.9738840938958948}, {0.2848714411408239, 0.9758051957556754}, {
      0.2789184207003248, 0.9776902207709169}, {0.2729550947473864, 
      0.9795391220551155}, {0.2669816900069433, 0.9813518542329752}, {
      0.2609984333208522, 0.9831283734397616}, {0.25500555163710803`, 
      0.9848686373205722}, {0.24900327199907887`, 0.9865726050295248}, {
      0.24299182153476082`, 0.9882402372288632}, {0.23697142744605304`, 
      0.9898714960879819}, {0.23094231699805384`, 0.991466345282368}, {
      0.22490471750837837`, 0.9930247499924638}, {0.2188588563364985, 
      0.9945466769024467}, {0.21280496087310577`, 0.996032094198929}, {
      0.20674325852949796`, 0.997480971569578}, {0.20067397672698964`, 
      0.9988932802016556}, {0.19459734288634786`, 1.000268992780478}, {
      0.18851358441725322`, 1.0016080834877963`}, {0.18242292870778698`, 
      1.0029105280000987`}, {0.1763256031139451, 1.004176303486833}, {
      0.17022183494917956`, 1.0054053886085512`}, {0.16411185147396765`, 
      1.0065977635149765`}, {0.15799587988540992`, 1.0077534098429919`}, {
      0.15187414730685728`, 1.0088723107145525`}, {0.14574688077756803`, 
      1.00995445073452}, {0.13961430724239518`, 1.010999815988421}, {
      0.1334766535415049, 1.0120083940401297`}, {0.12733414640012647`, 
      1.0129801739294733`}, {0.12118701241833453`, 1.0139151461697642`}, {
      0.11503547806086403`, 1.014813302745255}, {0.10887976964695856`, 
      1.0156746371085208`}, {0.10272011334025259`, 1.0164991441777653`}, {
      0.09655673513868829, 1.017286820334055}, {0.09038986086446728, 
      1.0180376634184787`}, {0.08421971615403809, 1.0187516727292343`}, {
      0.07804652644811978, 1.0194288490186434`}, {0.07187051698176226, 
      1.0200691944900937`}, {0.06569191277444389, 1.0206727127949085`}, {
      0.05951093862020689, 1.0212394090291468`}, {0.05332781907783111, 
      1.0217692897303308`}, {0.04714277846104662, 1.022262362874104}, {
      0.04095604082878579, 1.0227186378708188`}, {0.0347678299754752, 
      1.0231381255620546`}, {0.028578369421368027`, 1.023520838217067}, {
      0.022387882402917378`, 1.0238667895291682`}, {0.01619659186319107, 
      1.0241759946120388`}, {0.010004720442328351`, 1.0244484699959724`}, {
      0.0038124904680390745`, 1.024684233624053}, {-0.002379876053854188, 
      1.0248833048482646`}, {-0.008572157448830622, 
      1.0250457044255363`}, {-0.01476413238304174, 
      1.0251714545137187`}, {-0.020955579872637575`, 
      1.025260578667498}, {-0.02714627929304804, 
      1.0253131018342427`}, {-0.03333601038821897, 
      1.0253290503497878`}, {-0.039524553279802414`, 
      1.025308451934153}, {-0.04571168847630068, 
      1.0252513356871997`}, {-0.051897196882163744`, 
      1.0251577320842227`}, {-0.058080859806839524`, 
      1.0250276729714813`}, {-0.06426245897377664, 
      1.0248611915616679`}, {-0.07044177652937915, 
      1.0246583224293144`}, {-0.07661859505191292, 
      1.0244191015061386`}, {-0.08279269756036314, 
      1.0241435660763292`}, {-0.08896386752324266, 
      1.023831754771772}, {-0.09513188886735052, 
      1.0234837075672152`}, {-0.10129654598648066`, 
      1.0230994657753776`}, {-0.10745762375007993`, 
      1.0226790720419967`}, {-0.11361490751185542`, 
      1.0222225703408199`}, {-0.11976818311833048`, 
      1.0217300059685377`}, {-0.12591723691734916`, 
      1.0212014255396609`}, {-0.13206185576652854`, 
      1.020636876981341}, {-0.1382018270416589, 
      1.0200364095281347`}, {-0.14433693864505087`, 
      1.019400073716713}, {-0.1504669790138298, 
      1.0187279213805167`}, {-0.15659173712817642`, 
      1.0180200056443565`}, {-0.16271100251951381`, 
      1.01727638091896}, {-0.1688245652786402, 
      1.0164971028954655`}, {-0.17493221606380738`, 
      1.0156822285398635`}, {-0.18103374610874406`, 
      1.014831816087386}, {-0.18712894723062437`, 
      1.0139459250368443`}, {-0.1932176118379807, 
      1.0130246161449166`}, {-0.19929953293856079`, 
      1.0120679514203834`}, {-0.2053745041471288, 
      1.0110759941183156`}, {-0.21144231969320995`, 
      1.0100488087342119`}, {-0.2175027744287784, 
      1.0089864609980872`}, {-0.22355566383588824`, 
      1.007889017868514}, {-0.22960078403424708`, 
      1.0067565475266158`}, {-0.2356379317887323, 
      1.0055891193700126`}, {-0.24166690451684913`, 
      1.0043868040067219`}, {-0.24768750029613096`, 
      1.0031496732490117`}, {-0.25369951787148104`, 
      1.00187780010721}, {-0.25970275666245574`, 
      1.0005712587834674`}, {-0.2656970167704886, 
      0.9992301246654788}, {-0.27168209898605583`, 
      0.9978544743201578}, {-0.27765780479578167`, 
      0.9964443854872708}, {-0.28362393638948474`, 
      0.9949999370730269}, {-0.2895802966671643, 
      0.9935212091436275}, {-0.2955266892459264, 
      0.992008282918773}, {-0.30146291846684997`, 
      0.9904612407651294}, {-0.30738878940179204`, 
      0.9888801661897554}, {-0.3133041078601326, 
      0.9872651438334884}, {-0.3192086803954581, 
      0.985616259464293}, {-0.32510231431218445`, 
      0.98393359997057}, {-0.3309848176721181, 
      0.9822172533544277}, {-0.3368559993009561, 
      0.9804673087249163}, {-0.3427156687947239, 
      0.9786838562912251}, {-0.34856363652615197`, 
      0.976866987355844}, {-0.35439971365098993`, 
      0.9750167943076887}, {-0.36022371211425863`, 
      0.9731333706151917}, {-0.36603544465643983`, 
      0.9712168108193582}, {-0.3718347248196035, 
      0.9692672105267888}, {-0.3776213669534722, 
      0.9672846664026682}, {-0.3833951862214231, 
      0.9652692761637217}, {-0.3891559986064264, 
      0.9632211385711396}, {-0.3949036209169215, 
      0.9611403534234695}, {-0.4006378707926292, 
      0.9590270215494784}, {-0.406358566710301, 
      0.9568812448009837}, {-0.4120655279894051, 
      0.9547031260456549}, {-0.4177585747977483, 
      0.9524927691597859}, {-0.42343752815703484`, 
      0.9502502790210382}, {-0.42910220994836107`, 
      0.947975761501157}, {-0.4347524429176466, 
      0.9456693234586586}, {-0.4403880506810011, 
      0.9433310727314917}, {-0.44600885773002774`, 
      0.9409611181296721}, {-0.45161468943706157`, 
      0.9385595694278912}, {-0.4572053720603448, 
      0.9361265373581003}, {-0.4627807327491369, 
      0.9336621336020687}, {-0.4683405995487611, 
      0.9311664707839196}, {-0.47388480140558614`, 
      0.9286396624626405}, {-0.4794131681719435, 
      0.9260818231245722}, {-0.4849255306109807, 
      0.9234930681758748}, {-0.4904217204014495, 
      0.9208735139349719}, {-0.4959015701424298, 
      0.9182232776249732}, {-0.5013649133579892, 
      0.9155424773660777}, {-0.5068115845017774, 
      0.9128312321679553}, {-0.5122414189615567, 
      0.910089661922109}, {-0.517654253063667, 
      0.9073178873942201}, {-0.5230499240774266, 
      0.9045160302164718}, {-0.5284282702194688, 
      0.9016842128798573}, {-0.5337891306580129, 
      0.8988225587264702}, {-0.5391323455170712, 
      0.8959311919417771}, {-0.5444577558805911, 
      0.893010237546875}, {-0.5497652037965322, 
      0.8900598213907333}, {-0.5550545322808795, 
      0.8870800701424191}, {-0.5603255853215918, 
      0.88407111128331}, {-0.5655782078824849, 
      0.8810330730992914}, {-0.5708122459070508, 
      0.8779660846729412}, {-0.5760275463222125, 
      0.8748702758757012}, {-0.5812239570420139, 
      0.8717457773600361}, {-0.5864013269712453, 
      0.868592720551581}, {-0.5915595060090048, 
      0.8654112376412775}, {-0.5966983450521954, 
      0.8622014615774989}, {-0.6018176959989577, 
      0.8589635260581654}, {-0.6069174117520378, 
      0.8556975655228498}, {-0.6119973462220921, 
      0.8524037151448735}, {-0.6170573543309273, 
      0.8490821108233945}, {-0.622097292014677, 
      0.8457328891754872}, {-0.6271170162269136, 
      0.8423561875282143}, {-0.6321163849416971, 
      0.8389521439106923}, {-0.6370952571565597, 
      0.8355208970461498}, {-0.6420534928954269, 
      0.8320625863439808}, {-0.6469909532114754, 
      0.8285773518917912}, {-0.6519075001899268, 
      0.8250653344474418}, {-0.6568029969507787, 
      0.821526675431086}, {-0.6616773076514721, 
      0.8179615169172041}, {-0.6665302974894961, 
      0.8143700016266332}, {-0.671361832704929, 
      0.810752272918596}, {-0.6761717805829173, 
      0.8071084747827253}, {-0.6809600094560913, 
      0.8034387518310883}, {-0.6857263887069183, 
      0.799743249290208}, {-0.6904707887699938, 
      0.7960221129930852}, {-0.6951930811342694, 
      0.7922754893712188}, {-0.6998931383452194, 
      0.7885035254466272}, {-0.7045708340069449, 
      0.7847063688238697}, {-0.7092260427842161, 
      0.7808841676820695}, {-0.7138586404044528, 
      0.7770370707669378}, {-0.7184685036596434, 
      0.7731652273828002}, {-0.7230555104082028, 
      0.7692687873846257}, {-0.7276195395767681, 
      0.7653479011700585}, {-0.7321604711619342, 
      0.7614027196714535}, {-0.7366781862319282, 
      0.7574333943479158}, {-0.7411725669282226, 
      0.7534400771773446}, {-0.7456434964670889, 
      0.7494229206484818}, {-0.7500908591410894, 
      0.745382077752966}, {-0.7545145403205105, 
      0.741317701977393}, {-0.7589144264547344, 
      0.7372299472953814}, {-0.7632904050735526, 
      0.7331189681596461}, {-0.7676423647884183, 
      0.7289849194940781}, {-0.7719701952936401, 
      0.7248279566858329}, {-0.7762737873675168, 
      0.7206482355774259}, {-0.7805530328734128, 
      0.716445912458837}, {-0.7848078247607745, 
      0.7122211440596247}, {-0.7890380570660885, 
      0.7079740875410478}, {-0.7932436249137816, 
      0.7037049004881994}, {-0.7974244245170622, 
      0.6994137409021489}, {-0.801580353178704, 
      0.6951007671920965}, {-0.8057113092917723, 
      0.6907661381675374}, {-0.809817192340292, 
      0.686410013030439}, {-0.81389790289986, 
      0.6820325513674288}, {-0.8179533426381984, 
      0.6776339131419953}, {-0.821983414315653, 
      0.6732142586867017}, {-0.8259880217856338, 
      0.6687737486954127}, {-0.8299670699950004, 
      0.6643125442155353}, {-0.8339204649843902, 
      0.6598308066402727}, {-0.8378481138884922, 
      0.6553286977008937}, {-0.841749924936264, 
      0.6508063794590166}, {-0.845625807451094, 
      0.6462640142989077}, {-0.8494756718509091, 
      0.6417017649197961}, {-0.8532994296482271, 
      0.6371197943282048}, {-0.8570969934501544, 
      0.6325182658302968}, {-0.8608682769583302, 
      0.6278973430242397}, {-0.8646131949688161, 
      0.6232571897925863}, {-0.868331663371932, 
      0.6185979702946726}, {-0.8720235991520398, 
      0.6139198489590348}, {-0.8756889203872719, 
      0.6092229904758436}, {-0.8793275462492086, 
      0.6045075597893572}, {-0.882939397002502, 
      0.5997737220903937}, {-0.8865243940044479, 
      0.5950216428088224}, {-0.8900824597045057, 
      0.5902514876060754}, {-0.8936135176437662, 
      0.5854634223676781}, {-0.8971174924543686, 
      0.5806576131958019}, {-0.9005943098588656, 
      0.5758342264018353}, {-0.9040438966695384, 
      0.5709934284989786}, {-0.9074661807876605, 
      0.5661353861948576}, {-0.9108610912027112, 
      0.561260266384161}, {-0.9142285579915399, 
      0.5563682361412989}, {-0.9175685123174796, 
      0.5514594627130835}, {-0.9208808864294121, 
      0.5465341135114332}, {-0.9241656136607832, 
      0.5415923561060995}, {-0.9274226284285697, 
      0.5366343582174167}, {-0.9306518662321979, 
      0.5316602877090766}, {-0.9338532636524133, 
      0.5266703125809256}, {-0.9370267583501035, 
      0.521664600961787}, {-0.9401722890650729, 
      0.516643321102308}, {-0.9432897956147699, 
      0.5116066413678311}, {-0.9463792188929679, 
      0.5065547302312906}, {-0.9494405008683995, 
      0.5014877562661356}, {-0.9524735845833435, 
      0.4964058881392774}, {-0.9554784141521677, 
      0.4913092946040637}, {-0.9584549347598246, 
      0.4861981444932792}, {-0.9614030926603021, 
      0.48107260671217245`}, {-0.9643228351750297, 
      0.4759328502315098}, {-0.9672141106912396, 
      0.47077904408065635`}, {-0.9700768686602835, 
      0.4656113573406846}, {-0.9729110595959053, 
      0.4604299591375105}, {-0.9757166350724703, 
      0.4552350186350575}, {-0.9784935477231513, 
      0.4500267050284489}, {-0.9812417512380709, 
      0.44480518753722853`}, {-0.9839612003624018, 
      0.43957063539861035`}, {-0.9866518508944242, 
      0.4343232178607568}, {-0.9893136596835419, 
      0.4290631041760862}, {-0.9919465846282558, 
      0.42379046359461}, {-0.9945505846740967, 
      0.41850546535729943`}, {-0.9971256198115169, 
      0.4132082786894821}, {-0.9996716510737409, 
      0.4078990727942687}, {-1.002188640534576, 
      0.4025780168460099}, {-1.0046765513061824`, 
      0.3972452799837844}, {-1.007135347536805, 
      0.39190103130491705`}, {-1.0095649944084641`, 
      0.3865454398585284}, {-1.011965458134608, 
      0.3811786746391154}, {-1.014336705957727, 
      0.37580090458016363`}, {-1.0166787061469291`, 
      0.3704122985477908}, {-1.0189914279954768`, 
      0.3650130253344224}, {-1.0212748418182882`, 
      0.3596032536524996}, {-1.0235289189493988`, 
      0.3541831521282187}, {-1.0257536317393878`, 
      0.34875288929530357`}, {-1.0279489535527668`, 
      0.3433126335888105}, {-1.0301148587653333`, 
      0.33786255333896587`}, {-1.0322513227614871`, 
      0.3324028167650368}, {-1.0343583219315124`, 
      0.3269335919692348}, {-1.0364358336688237`, 
      0.32145504693065313`}, {-1.0384838363671773`, 
      0.315967349499237}, {-1.0405023094178483`, 
      0.3104706673897881}, {-1.0424912332067733`, 
      0.3049651681760028}, {-1.0444505891116593`, 
      0.29945101928454376`}, {-1.04638035949906, 
      0.2939283879891466}, {-1.0482805277214164`, 
      0.28839744140476015`}, {-1.0501510781140686`, 
      0.2828583464817213}, {-1.0519919959922315`, 
      0.2773112699999648}, {-1.0538032676479403`, 
      0.27175637856326734`}, {-1.0555848803469645`, 
      0.26619383859352685`}, {-1.0573368223256892`, 
      0.2606238163250765}, {-1.0590590827879665`, 
      0.255046477799034}, {-1.0607516519019358`, 
      0.24946198885768647`}, {-1.062414520796814, 
      0.24387051513891017`}, {-1.064047681559656, 
      0.2382722220706261}, {-1.0656511272320837`, 
      0.23266727486529123`}, {-1.0672248518069887`, 
      0.22705583851442546`}, {-1.068768850225204, 
      0.22143807778317443`}, {-1.0702831183721475`, 
      0.21581415720490832`}, {-1.0717676530744378`, 
      0.21018424107585695`}, {-1.0732224520964813`, 
      0.2045484934497808}, {-1.0746475141370326`, 
      0.1989070781326785}, {-1.0760428388257275`, 
      0.1932601586775308}, {-1.0774084267195885`, 
      0.18760789837908087`}, {-1.0787442792995046`, 
      0.1819504602686514}, {-1.0800503989666843`, 
      0.17628800710899814`}, {-1.0813267890390839`, 
      0.17062070138920055`}, {-1.082573453747809, 
      0.16494870531958897`}, {-1.083790398233491, 
      0.15927218082670896`}, {-1.0849776285426411`, 
      0.15359128954832263`}, {-1.0861351516239757`, 
      0.14790619282844694`}, {-1.0872629753247218`, 
      0.14221705171242932`}, {-1.0883611083868965`, 
      0.1365240269420604}, {-1.089429560443563, 
      0.1308272789507241}, {-1.0904683420150652`, 
      0.12512696785858515`}, {-1.0914774645052372`, 
      0.1194232534678139}, {-1.0924569401975928`, 
      0.11371629525784868`}, {-1.0934067822514912`, 
      0.10800625238069568`}, {-1.0943270046982823`, 
      0.10229328365626641`}, {-1.095217622437429, 
      0.09657754756775287}, {-1.0960786512326106`, 
      0.09085920225704024}, {-1.0969101077078038`, 
      0.08513840552015742}, {-1.097712009343344, 
      0.0794153148027653}, {-1.0984843744719666`, 
      0.07369008719568283}, {-1.0992272222748287`, 
      0.06796287943045087}, {-1.0999405727775111`, 
      0.06223384787493398}, {-1.1006244468460018`, 
      0.05650314852896005}, {-1.1012788661826602`, 
      0.05077093701999787}, {-1.1019038533221637`, 
      0.045037368598872606`}, {-1.102499431627435, 
      0.039302598135519326`}, {-1.1030656252855537`, 
      0.033566780114774375`}, {-1.1036024593036486`, 
      0.02783006863220487}, {-1.104109959504774, 
      0.022092617389976112`}, {-1.1045881525237693`, 
      0.016354579692757058`}, {-1.1050370658031012`, 
      0.010616108443663808`}, {-1.1054567275886915`, 
      0.004877356140241128}, {-1.1058471669257275`, -0.0008615251295180162}, \
{-1.1062084136544585`, -0.0066003836911148715`}, {-1.1065404984059761`, \
-0.012339068287449295`}, {-1.1068434525979796`, -0.018077428082686132`}, \
{-1.1071173084305281`, -0.023815312666083565`}, {-1.1073620988817776`, \
-0.029552572055783467`}, {-1.107577857703704, -0.03528905670256378}, \
{-1.1077646194178137`, -0.041024617493552865`}, {-1.1079224193108397`, \
-0.04675910575590598}, {-1.108051293430426, -0.05249237326044372}, \
{-1.1081512785807988`, -0.058224272225252616`}, {-1.1082224123184248`, \
-0.06395465531924782}, {-1.1082647329476591`, -0.0696833756656979}, \
{-1.1082782795163806`, -0.07541028684571185}, {-1.1082630918116154`, \
-0.08113524290168825}, {-1.1082192103551505`, -0.08685809834072666}, \
{-1.1081466763991354`, -0.09257870813800137}, {-1.108045531921675, \
-0.0982969277400973}, {-1.1079158196224108`, -0.10401261306830836`}, \
{-1.1077575829180923`, -0.1097256205218982}, {-1.1075708659381402`, \
-0.11543580698132332`}, {-1.1073557135201992`, -0.12114302981141872`}, \
{-1.1071121712056826`, -0.1268471468645461}, {-1.1068402852353072`, \
-0.13254801648370454`}, {-1.1065401025446209`, -0.13824549750560394`}, \
{-1.1062116707595222`, -0.1439394492637011}, {-1.1058550381917711`, \
-0.14962973159119858`}, {-1.1054702538344934`, -0.15531620482400615`}, \
{-1.1050573673576771`, -0.16099872980366559`}, {-1.1046164291036624`, \
-0.16667716788023787`}, {-1.104147490082625, -0.17235138091515378`}, \
{-1.1036506019680523`, -0.17802123128402736`}, {-1.1031258170922156`, \
-0.18368658187943251`}, {-1.1025731884416337`, -0.18934729611364295`}, \
{-1.101992769652533, -0.19500323792133534`}, {-1.1013846150063016`, \
-0.2006542717622557}, {-1.1007487794249393`, -0.20630026262384946`}, \
{-1.1000853184665014`, -0.21194107602385484`}, {-1.0993942883205392`, \
-0.21757657801285996`}, {-1.0986757458035363`, -0.2232066351768235}, \
{-1.0979297483543409`, -0.22883111463955924`}, {-1.0971563540295939`, \
-0.23444988406518447`}, {-1.0963556214991548`, -0.2400628116605321}, \
{-1.0955276100415237`, -0.24566976617752706`}, {-1.0946723795392608`, \
-0.2512706169155268}, {-1.093789990474403, -0.25686523372362585`}, \
{-1.0928805039238791`, -0.2624534870029247}, {-1.0919439815549215`, \
-0.2680352477087633}, {-1.0909804856204772`, -0.27361038735291876`}, \
{-1.0899900789546177`, -0.27917877800576785`}, {-1.0889728249679467`, \
-0.2847402922984142}, {-1.0879287876430062`, -0.2902948034247802}, \
{-1.0868580315296839`, -0.2958421851436639}, {-1.085760621740618, \
-0.3013823117807608}, {-1.0846366239466028`, -0.30691505823065096`}, \
{-1.0834861043719946`, -0.31244029995875117`}, {-1.0823091297901162`, \
-0.31795791300323245`}, {-1.0811057675186637`, -0.32346777397690313`}, \
{-1.0798760854151128`, -0.32896976006905754`}, {-1.0786201518721268`, \
-0.3344637490472901}, {-1.0773380358129647`, -0.3399496192592754}, \
{-1.076029806686892, -0.3454272496345144}, {-1.0746955344645923`, \
-0.35089651968604596`}, {-1.0733352896335804`, -0.3563573095121256}, \
{-1.0719491431936188`, -0.3618094997978693}, {-1.0705371666521353`, \
-0.36725297181686495`}, {-1.0690994320196434`, -0.3726876074327491}, \
{-1.067636011805166, -0.37811328910075126`}, {-1.0661469790116616`, \
-0.38352989986920466`}, {-1.0646324071314532`, -0.3889373233810238}, \
{-1.0630923701416624`, -0.3943354438751492}, {-1.0615269424996445`, \
-0.399724146187959}, {-1.0599361991384302`, -0.40510331575464836`}, \
{-1.058320215462169, -0.410472838610576}, {-1.0566790673415785`, \
-0.4158326013925779}, {-1.0550128311093974`, -0.4211824913402495}, \
{-1.0533215835558436`, -0.42652239629719507`}, {-1.0516054019240766`, \
-0.4318522047122453}, {-1.0498643639056653`, -0.43717180564064284`}, \
{-1.0480985476360618`, -0.44248108874519637`}, {-1.0463080316900795`, \
-0.4477799442974027}, {-1.0444928950773773`, -0.4530682631785375}, \
{-1.0426532172379508`, -0.45834593688071484`}, {-1.0407890780376274`, \
-0.46361285750791525`}, {-1.03890055776357, -0.46886891777698303`}, \
{-1.0369877371197853`, -0.47411401101859224`}, {-1.03505069722264, \
-0.4793480311781824}, {-1.0330895195963834`, -0.48457087281686323`}, \
{-1.0311042861686768`, -0.48978243111228886`}, {-1.0290950792661304`, \
-0.49498260185950205`}, {-1.027061981609848, -0.5001712814717479}, \
{-1.0250050763109784`, -0.5053483669812578}, {-1.0229244468662748`, \
-0.5105137560400033}, {-1.0208201771536636`, -0.5156673469204199}, \
{-1.0186923514278183`, -0.5208090385161025}, {-1.0165410543157452`, \
-0.5259387303424696}, {-1.0143663708123742`, -0.5310563225374}, \
{-1.0121683862761615`, -0.536161715861839}, {-1.0099471864246978`, \
-0.5412548117003767}, {-1.0077028573303288`, -0.5463355120617965}, \
{-1.005435485415782, -0.551403719579596}, {-1.0031451574498047`, \
-0.5564593375124781}, {-1.0008319605428095`, -0.5615022697448153}, \
{-0.9984959821425325, -0.5665324207870844}, {-0.9961373100296979, \
-0.571549695776274}, {-0.9937560323136945, -0.5765540004762639}, \
{-0.9913522374282622, -0.581545241278177}, {-0.988926014127188, \
-0.5865233252007027}, {-0.9864774514800126, -0.5914881598903947}, \
{-0.9840066388677484, -0.5964396536219397}, {-0.9815136659786067, \
-0.6013777152984002}, {-0.9789986228037367, -0.60630225445143}, \
{-0.9764615996329759, -0.6112131812414631}, {-0.9739026870506101, \
-0.6161104064578754}, {-0.9713219759311463, -0.6209938415191214}, \
{-0.968719557435096, -0.6258633984728428}, {-0.9660955230047701, \
-0.6307189899959526}, {-0.9634499643600861, -0.6355605293946922}, \
{-0.9607829734943862, -0.6403879306046636}, {-0.9580946426702684, \
-0.6452011081908351}, {-0.9553850644154285, -0.6499999773475222}, \
{-0.952654331518515, -0.6547844538983432}, {-0.9499025370249965, \
-0.6595544542961491}, {-0.9471297742330411, -0.6643098956229297}, \
{-0.9443361366894085, -0.6690506955896938}, {-0.9415217181853547, \
-0.673776772536325}, {-0.9386866127525496, -0.6784880454314137}, \
{-0.9358309146590079, -0.6831844338720647}, {-0.9329547184050324, \
-0.6878658580836798}, {-0.9300581187191708, -0.6925322389197174}, \
{-0.9271412105541856, -0.6971834978614279}, {-0.9242040890830381, \
-0.7018195570175658}, {-0.9212468496948847, -0.7064403391240784}, \
{-0.9182695879910878, -0.7110457675437706}, {-0.91527239978124, \
-0.715635766265947}, {-0.9122553810792019, -0.7202102599060318}, \
{-0.9092186280991548, -0.7247691737051646}, {-0.9061622372516659, \
-0.7293124335297744}, {-0.9030863051397692, -0.7338399658711313}, \
{-0.8999909285550594, -0.7383516978448758}, {-0.8968762044738007, \
-0.7428475571905252}, {-0.8937422300530503, -0.7473274722709599}, \
{-0.8905891026267956, -0.7517913720718857}, {-0.8874169197021068, \
-0.756239186201277}, {-0.8842257789553036, -0.760670844888796}, \
{-0.8810157782281373, -0.765086278985193}, {-0.8777870155239871, \
-0.7694854199616838}, {-0.8745395890040719, -0.7738681999093078}, \
{-0.8712735969836769, -0.7782345515382635}, {-0.8679891379283955, \
-0.7825844081772257}, {-0.8646863104503861, -0.7869177037726405}, \
{-0.8613652133046446, -0.7912343728880004}, {-0.8580259453852919, \
-0.7955343507031002}, {-0.8546686057218771, -0.7998175730132724}, \
{-0.8512932934756964, -0.8040839762286025}, {-0.8479001079361268, \
-0.8083334973731258}, {-0.8444891485169768, -0.8125660740840037}, \
{-0.8410605147528513, -0.8167816446106816}, {-0.8376143062955338, \
-0.8209801478140274}, {-0.8341506229103833, -0.8251615231654511}, \
{-0.830669564472748, -0.8293257107460059}, {-0.8271712309643943, \
-0.8334726512454705}, {-0.8236557224699526, -0.8376022859614135}, \
{-0.8201231391733788, -0.8417145567982387}, {-0.8165735813544321, \
-0.8458094062662134}, {-0.8130071493851692, -0.8498867774804781}, \
{-0.8094239437264543, -0.8539466141600391}, {-0.8058240649244864, \
-0.8579888606267427}, {-0.8022076136073425, -0.8620134618042328}, \
{-0.7985746904815367, -0.866020363216891}, {-0.7949253963285976, \
-0.8700095109887598}, {-0.7912598320016601, -0.8739808518424486}, \
{-0.7875780984220755, -0.8779343330980233}, {-0.7838802965760379, \
-0.8818699026718794}, {-0.7801665275112266, -0.8857875090755981}, \
{-0.7764368923334666, -0.8896871014147872}, {-0.7726914922034046, \
-0.8935686293879052}, {-0.768930428333203, -0.8974320432850693}, \
{-0.7651538019832501, -0.9012772939868487}, {-0.7613617144588875, \
-0.9051043329630415}, {-0.7575542671071543, -0.9089131122714359}, \
{-0.7537315613135495, -0.9127035845565575}, {-0.7498936984988093, \
-0.9164757030484001}, {-0.7460407801157037, -0.9202294215611422}, \
{-0.742172907645849, -0.9239646944918489}, {-0.7382901825965376, \
-0.927681476819159}, {-0.7343927064975857, -0.9313797241019574}, \
{-0.7304805808981975, -0.9350593924780337}, {-0.726553907363847, \
-0.9387204386627266}, {-0.7226127874731773, -0.9423628199475537}, \
{-0.7186573228149167, -0.9459864941988285}, {-0.7146876149848128, \
-0.9495914198562633}, {-0.7107037655825836, -0.9531775559315581}, \
{-0.7067058762088859, -0.9567448620069768}, {-0.7026940484623017, \
-0.9602932982339105}, {-0.6986683839363412, -0.9638228253314268}, \
{-0.6946289842164646, -0.9673334045848071}, {-0.6905759508771196, \
-0.9708249978440707}, {-0.6865093854787983, -0.9742975675224868}, \
{-0.6824293895651098, -0.9777510765950733}, {-0.6783360646598723, \
-0.9811854885970848}, {-0.674229512264221, -0.9846007676224866}, \
{-0.6701098338537347, -0.9879968783224184}, {-0.6659771308755794, \
-0.9913737859036454}, {-0.6618315047456699, -0.994731456126998}, \
{-0.6576730568458486, -0.9980698553057996}, {-0.6535018885210827, \
-1.001388950304284}, {-0.6493181010766778, -1.0046887085360008`}, \
{-0.6451217957755101, -1.0079690979622098`}, {-0.6409130738352765, \
-1.011230087090265}, {-0.636692036425761, -1.0144716449719875`}, \
{-0.6324587846661202, -1.0176937412020277`}, {-0.6282134196221854, \
-1.020896345916218}, {-0.6239560423037831, -1.0240794297899132`}, \
{-0.619686753662073, -1.0272429640363232`}, {-0.6154056545869031, \
-1.0303869204048342`}, {-0.6111128459041831, -1.03351127117932}, \
{-0.6068084283732755, -1.0366159891764442`}, {-0.6024925026844036, \
-1.0397010477439523`}, {-0.5981651694560781, -1.0427664207589558`}, \
{-0.5938265292325408, -1.0458120826262038`}, {-0.5894766824812258, \
-1.0488380082763498`}, {-0.5851157295902389, -1.0518441731642054`}, \
{-0.5807437708658545, -1.0548305532669884`}, {-0.5763609065300292, \
-1.0577971250825593`}, {-0.5719672367179346, -1.060743865627652}, \
{-0.567562861475506, -1.063670752436094}, {-0.5631478807570106, \
-1.0665777635570195`}, {-0.5587223944226312, -1.069464877553073}, \
{-0.5542865022360689, -1.072332073498607}, {-0.5498403038621633, \
-1.0751793309778694`}, {-0.5453838988645292, -1.078006630083185}, \
{-0.5409173867032121, -1.0808139514131283`}, {-0.5364408667323602, \
-1.0836012760706895`}, {-0.5319544381979147, -1.0863685856614325`}, \
{-0.5274582002353164, -1.0891158622916464`}, {-0.5229522518672317, \
-1.0918430885664892`}, {-0.5184366920012938, -1.0945502475881248`}, \
{-0.5139116194278629, -1.0972373229538537`}, {-0.5093771328178031, \
-1.0999042987542362`}, {-0.5048333307202768, -1.1025511595712096`}, \
{-0.5002803115605567, -1.1051778904761989`}, {-0.4957181736378549, \
-1.1077844770282201`}, {-0.4911470151231689, -1.1103709052719797`}, \
{-0.48656693405714563`, -1.1129371617359658`}, {-0.48197802834796244`, \
-1.1154832334305345`}, {-0.47738039576922503`, -1.1180091078459902`}, \
{-0.472774133957883, -1.1205147729506602`}, {-0.46815934041216223`, \
-1.1230002171889637`}, {-0.4635361124895147, -1.1254654294794757`}, \
{-0.45890454740458514`, -1.127910399212985}, {-0.45426474222719493`, \
-1.1303351162505482`}, {-0.44961679388034304`, -1.1327395709215362`}, \
{-0.444960799138224, -1.1351237540216796`}, {-0.44029685462426277`, \
-1.1374876568111052`}, {-0.4356250568091666, -1.1398312710123706`}, \
{-0.4309455020089939, -1.142154588808493}, {-0.42625828638324, \
-1.1444576028409739`}, {-0.42156350593293945`, -1.1467403062078194`}, \
{-0.41686125649878575`, -1.1490026924615568`}, {-0.41215163375926717`, \
-1.1512447556072454`}, {-0.4074347332288199, -1.1534664901004854`}, \
{-0.4027106502559975, -1.1556678908454217`}, {-0.39797948002165734`, \
-1.1578489531927443`}, {-0.39324131753716346`, -1.160009672937685}, \
{-0.38849625764260604`, -1.16215004631801}, {-0.38374439500503754`, \
-1.1642700700120108`}, {-0.3789858241167254, -1.166369741136489}, \
{-0.37422063929342086`, -1.168449057244741}, {-0.3694489346726449, \
-1.1705080163245372`}, {-0.3646708042119897, -1.172546616796099}, \
{-0.35988634168743733`, -1.1745648575100738`}, {-0.3550956406916939, \
-1.1765627377455057`}, {-0.3502987946325409, -1.1785402572078045`}, \
{-0.34549589673120173`, -1.180497416026712}, {-0.3406870400207256, \
-1.1824342147542661`}, {-0.33587231734438633`, -1.1843506543627613`}, \
{-0.3310518213540983, -1.1862467362427087`}, {-0.3262256445088476, \
-1.1881224622007929`}, {-0.32139387907314, -1.1899778344578265`}, \
{-0.31655661711546396`, -1.1918128556467036`}, {-0.31171395050677053`, \
-1.193627528810351}, {-0.3068659709189682, -1.1954218573996775`}, \
{-0.30201276982343433`, -1.1971958452715221`}, {-0.29715443848954165`, \
-1.1989494966865997`}, {-0.29229106798320104`, -1.2006828163074466`}, \
{-0.28742274916541977`, -1.2023958091963636`}, {-0.28254957269087516`, \
-1.2040884808133583`}, {-0.2776716290065042, -1.2057608370140867`}, \
{-0.2727890083501084, -1.2074128840477925`}, {-0.2679018007489741, \
-1.2090446285552467`}, {-0.26301009601850833`, -1.2106560775666864`}, \
{-0.2581139837608902, -1.212247238499751}, {-0.253213553363737, \
-1.2138181191574198`}, {-0.24830889399878633`, -1.2153687277259484`}, \
{-0.24340009462059278`, -1.216899072772804}, {-0.23848724396524018`, \
-1.2184091632446006`}, {-0.23357043054906876`, -1.2198990084650343`}, \
{-0.2286497426674175, -1.2213686181328183`}, {-0.22372526839338128`, \
-1.222818002319617}, {-0.21879709557658314`, -1.2242471714679806`}, \
{-0.21386531184196136`, -1.2256561363892804`}, {-0.20893000458857136`, \
-1.2270449082616426`}, {-0.2039912609884024, -1.2284134986278838`}, \
{-0.19904916798520894`, -1.2297619193934464`}, {-0.19410381229335683`, \
-1.2310901828243337`}, {-0.18915528039668383`, -1.232398301545046}, \
{-0.18420365854737497`, -1.2336862885365176`}, {-0.17924903276485224`, \
-1.234954157134054}, {-0.17429148883467874`, -1.2362019210252697`}, \
{-0.1693311123074772, -1.2374295942480265`}, {-0.1643679884978629, \
-1.2386371911883736`}, {-0.1594022024833908, -1.2398247265784883`}, \
{-0.1544338391035168, -1.240992215494617}, {-0.1494629829585733, \
-1.242139673355018}, {-0.1444897184087588, -1.243267115917906}, \
{-0.1395141295731414, -1.2443745592793962`}, {-0.13453630032867653`, \
-1.2454620198714512`}, {-0.12955631430923825`, -1.2465295144598287`}, \
{-0.12457425490466478`, -1.2475770601420313`}, {-0.11959020525981755`, \
-1.2486046743452563`}, {-0.11460424827365409`, -1.2496123748243493`}, \
{-0.10961646659831459`, -1.2506001796597577`}, {-0.10462694263822203`, \
-1.2515681072554867`}, {-0.09963575854919596, -1.2525161763370576`}, \
{-0.09464299623757962, -1.2534444059494667`}, {-0.08964873735938064, \
-1.2543528154551475`}, {-0.08465306331942507, -1.2552414245319345`}, \
{-0.07965605527052463, -1.2561102531710286`}, {-0.07465779411265736, \
-1.2569593216749657`}, {-0.06965836049216127, -1.2577886506555864`}, \
{-0.06465783480094137, -1.2585982610320092`}, {-0.059656297175689446`, \
-1.2593881740286048`}, {-0.05465382749711715, -1.2601584111729742`}, \
{-0.049650505389201786`, -1.2609089942939282`}, {-0.04464641021844514, \
-1.2616399455194705`}, {-0.03964162109314501, -1.2623512872747817`}, \
{-0.03463621686267955, -1.2630430422802086`}, {-0.029630276116804303`, \
-1.2637152335492539`}, {-0.024623877184961826`, -1.26436788438657}, \
{-0.019617098135603916`, -1.2650010183859555`}, {-0.01461001677552631, \
-1.2656146594283537`}, {-0.00960271064921584, -1.2662088316798563`}, \
{-0.004595257038209958, -1.2667835595897075`}, {
      0.00041226703953142936`, -1.267338867888313}, {
      0.005419784830241877, -1.2678747815852518`}, {
      0.010427219844952127`, -1.2683913259672908`}, {
      0.01543449586008172, -1.2688885265964032`}, {
      0.020441536918018546`, -1.2693664093077899`}, {
      0.0254482673276864, -1.2698250002079046`}, {
      0.030454611665100592`, -1.2702643256724822`}, {
      0.035460494773911735`, -1.2706844123445704`}, {
      0.04046584176593766, -1.271085287132566}, {
      0.045470578021683666`, -1.271466977208253}, {
      0.05047462919085103, -1.2718295100048458`}, {
      0.05547792119283395, -1.272172913215036}, {
      0.06048038021720493, -1.2724972147890419`}, {
      0.06548193272418865, -1.2728024429326636`}, {
      0.07048250544512445, -1.27308862610534}, {
      0.07548202538291748, -1.273355793018211}, {
      0.08048041981247848, -1.2736039726321833`}, {
      0.08547761628115243, -1.2738331941560006`}, {
      0.09047354260913591, -1.274043487044317}, {
      0.09546812688988356, -1.274234880995775}, {
      0.10046129749050324`, -1.2744074059510886`}, {
      0.10545298305214047`, -1.274561092091128}, {
      0.11044311249035181`, -1.2746959698350115`}, {
      0.11543161499546745`, -1.2748120698382002`}, {
      0.120418420032943, -1.2749094229905966`}, {
      0.1254034573437006, -1.2749880604146493`}, {
      0.13038665694445933`, -1.2750480134634594`}, {
      0.13536794912805514`, -1.2750893137188948`}, {
      0.14034726446375007`, -1.2751119929897057`}, {
      0.14532453379753116`, -1.2751160833096467`}, {
      0.15029968825239895`, -1.275101616935603}, {
      0.15527265922864564`, -1.2750686263457214`}, {
      0.16024337840412295`, -1.2750171442375442`}, {
      0.16521177773449977`, -1.274947203526151}, {
      0.17017778945350973`, -1.2748588373423029`}, {
      0.17514134607318868`, -1.2747520790305915`}, {
      0.18010238038410212`, -1.274626962147594}, {
      0.1850608254555627, -1.2744835204600322`}, {
      0.1900166146358379, -1.2743217879429365`}, {
      0.19496968155234776`, -1.274141798777815}, {
      0.19991996011185292`, -1.2739435873508276`}, {
      0.2048673845006331, -1.2737271882509646`}, {
      0.20981188918465568`, -1.2734926362682315`}, {
      0.21475340890973496`, -1.2732399663918368`}, {
      0.21969187870168172`, -1.2729692138083875`}, {
      0.22462723386644357`, -1.2726804139000871`}, {
      0.2295594099902356, -1.272373602242941}, {
      0.23448834293966214`, -1.2720488146049655`}, {
      0.23941396886182886`, -1.271706086944403}, {
      0.2443362241844459, -1.2713454554079413`}, {
      0.24925504561592182`, -1.2709669563289396`}, {
      0.2541703701454486, -1.2705706262256593`}, {
      0.25908213504307737`, -1.2701565017994991`}, {
      0.26399027785978557`, -1.269724619933237}, {
      0.2688947364275349, -1.2692750176892766`}, {
      0.27379544885932067`, -1.2688077323078997`}, {
      0.2786923535492122, -1.2683228012055232`}, {
      0.2835853891723848, -1.2678202619729626`}, {
      0.2884744946851429, -1.2673001523737002`}, {
      0.2933596093249349, -1.2667625103421591`}, {
      0.2982406726103588, -1.2662073739819837`}, {
      0.30311762434116063`, -1.2656347815643232`}, {
      0.3079904045982231, -1.2650447715261235`}, {
      0.312858953743547, -1.2644373824684234`}, {
      0.3177232124202239, -1.2638126531546563`}, {
      0.3225831215524005, -1.263170622508958}, {
      0.3274386223452353, -1.26251132961448}, {
      0.3322896562848468, -1.2618348137117088`}, {
      0.33713616513825384`, -1.2611411141967899`}, {
      0.341978090953308, -1.260430270619859}, {
      0.3468153760586181, -1.2597023226833783`}, {
      0.3516479630634668, -1.2589573102404785`}, {
      0.3564757948577195, -1.2581952732933062`}, {
      0.3612988146117256, -1.2574162519913779`}, {
      0.3661169657762118, -1.2566202866299399`}, {
      0.37093019208216826`, -1.2558074176483331`}, {
      0.3757384375407269, -1.254977685628365}, {
      0.3805416464430322, -1.2541311312926862`}, {
      0.38533976336010484`, -1.2532677955031737`}, {
      0.39013273314269786`, -1.2523877192593205`}, {
      0.3949205009211455, -1.2514909436966297`}, {
      0.39970301210520465`, -1.2505775100850163`}, {
      0.4044802123838895, -1.2496474598272131`}, {
      0.4092520477252988, -1.2487008344571853`}, {
      0.41401846437643597`, -1.2477376756385472`}, {
      0.4187794088630226, -1.2467580251629895`}, {
      0.4235348279893046, -1.2457619249487084`}, {
      0.4282846688378515, -1.244749417038844}, {
      0.4330288787693493, -1.2437205435999223`}, {
      0.4377674054223862, -1.2426753469203058`}, {
      0.44250019671323154`, -1.2416138694086472`}, {
      0.4472272008356084, -1.2405361535923518`}, {
      0.4519483662604595, -1.2394422421160451`}, {
      0.45666364173570617`, -1.2383321777400458`}, {
      0.46137297628600155`, -1.237206003338846}, {
      0.4660763192124768, -1.2360637618995967`}, {
      0.4707736200924809, -1.2349054965206006`}, {
      0.4754648287793148, -1.2337312504098097`}, {
      0.48014989540195824`, -1.2325410668833297`}, {
      0.4848287703647915, -1.2313349893639312`}, {
      0.48950140434731015`, -1.2301130613795657`}, {
      0.4941677483038341, -1.2288753265618892`}, {
      0.4988277534632104, -1.227621828644791}, {
      0.5034813713285105, -1.2263526114629297`}, {
      0.5081285536767206, -1.2250677189502739`}, {
      0.5127692525584276, -1.2237671951386506`}, {
      0.5174034202974979, -1.2224510841562999`}, {
      0.5220310094907508, -1.221119430226434}, {
      0.5266519730076266, -1.2197722776658053`}, {
      0.5312662639898487, -1.2184096708832777`}, {
      0.5358738358510801, -1.217031654378407}, {
      0.5404746422765743, -1.2156382727400257`}, {
      0.5450686372228208, -1.214229570644835}, {
      0.5496557749171856, -1.212805592856002}, {
      0.554236009857545, -1.2113663842217646`}, {
      0.5588092968119153, -1.209911989674042}, {
      0.5633755908180772, -1.208442454227051}, {
      0.5679348471831939, -1.2069578229759297`}, {
      0.572487021483425, -1.2054581410953658`}, {
      0.577032069563535, -1.2039434538382334`}, {
      0.5815699475364966, -1.2024138065342345`}, {
      0.5861006117830888, -1.200869244588547}, {
      0.5906240189514905, -1.1993098134804796`}, {
      0.595140125956869, -1.1977355587621328`}, {
      0.5996488899809632, -1.1961465260570654`}, {
      0.6041502684716624, -1.1945427610589683`}, {
      0.6086442191425806, -1.1929243095303446`}, {
      0.613130699972625, -1.1912912173011951`}, {
      0.6176096692055612, -1.1896435302677113`}, {
      0.6220810853495725, -1.1879812943909733`}, {
      0.6265449071768158, -1.1863045556956553`}, {
      0.6310010937229718, -1.184613360268737}, {
      0.6354496042867915, -1.1829077542582205`}, {
      0.6398903984296385, -1.1811877838718552`}, {
      0.6443234359750255, -1.1794534953758669`}, {
      0.6487486770081483, -1.177704935093695}, {
      0.6531660818754144, -1.1759421494047353`}, {
      0.6575756111839672, -1.1741651847430883`}, {
      0.6619772258012072, -1.1723740875963156`}, {
      0.6663708868543073, -1.1705689045042007`}, {
      0.6707565557297259, -1.1687496820575176`}, {
      0.6751341940727142, -1.1669164668968042`}, {
      0.6795037637868211, -1.1650693057111434`}, {
      0.6838652270333925, -1.1632082452369497`}, {
      0.6882185462310682, -1.1613333322567623`}, {
      0.6925636840552738, -1.159444613598044}, {
      0.6969006034377099, -1.1575421361319875`}, {
      0.701229267565836, -1.1556259467723264`}, {
      0.7055496398823524, -1.1536960924741537`}, {
      0.7098616840846769, -1.151752620232746}, {
      0.7141653641244188, -1.1497955770823935`}, {
      0.7184606442068491, -1.1478250100952383`}, {
      0.7227474887903669, -1.1458409663801155`}, {
      0.7270258625859626, -1.1438434930814037`}, {
      0.7312957305566775, -1.1418326373778798`}, {
      0.7355570579170599, -1.139808446481581}, {
      0.7398098101326182, -1.1377709676366725`}, {
      0.7440539529192698, -1.1357202481183215`}, {
      0.7482894522427881, -1.1336563352315774`}, {
      0.7525162743182447, -1.1315792763102583`}, {
      0.7567343856094499, -1.1294891187158436`}, {
      0.7609437528283886, -1.1273859098363725`}, {
      0.765144342934654, -1.1252696970853493`}, {
      0.7693361231348785, -1.123140527900654}, {
      0.7735190608821604, -1.1209984497434602`}, {
      0.7776931238754885, -1.1188435100971572`}, {
      0.7818582800591639, -1.1166757564662806`}, {
      0.786014497622218, -1.1144952363754475`}, {
      0.7901617449978284, -1.112301997368298}, {
      0.7942999908627318, -1.110096087006443}, {
      0.7984292041366341, -1.107877552868418}, {
      0.8025493539816173, -1.1056464425486434`}, {
      0.8066604098015446, -1.10340280365639}, {
      0.8107623412414622, -1.101146683814752}, {
      0.8148551181869982, -1.098878130659623}, {
      0.8189387107637599, -1.0965971918386825`}, {
      0.8230130893367277, -1.094303915010385}, {
      0.8270782245096465, -1.091998347842956}, {
      0.8311340871244156, -1.0896805380133947`}, {
      0.8351806482604749, -1.0873505332064817`}, {
      0.8392178792341891, -1.0850083811137932`}, {
      0.8432457515982305, -1.0826541294327219`}, {
      0.8472642371409578, -1.080287825865502}, {
      0.8512733078857941, -1.0779095181182419`}, {
      0.8552729360906022, -1.075519253899962}, {
      0.8592630942470567, -1.073117080921639}, {
      0.8632437550800158, -1.070703046895255}, {
      0.8672148915468895, -1.0682771995328537`}, {
      0.871176476837006, -1.0658395865456018`}, {
      0.8751284843709769, -1.0633902556428572`}, {
      0.8790708878000588, -1.0609292545312408`}, {
      0.8830036610055144, -1.0584566309137176`}, {
      0.886926778097971, -1.0559724324886803`}, {
      0.8908402134167771, -1.053476706949041}, {
      0.8947439415293569, -1.0509695019813274`}, {
      0.8986379372305635, -1.048450865264786}, {
      0.9025221755420301, -1.045920844470489}, {
      0.906396631711519, -1.0433794872604503`}, {
      0.9102612812122693, -1.0408268412867434`}, {
      0.9141160997423432, -1.0382629541906283`}, {
      0.9179610632239694, -1.0356878736016821`}, {
      0.9217961478028863, -1.0331016471369363`}, {
      0.9256213298476829, -1.0305043224000197`}, {
      0.9294365859491377, -1.0278959469803062`}, {
      0.9332418929195573, -1.0252765684520695`}, {
      0.9370372277921121, -1.0226462343736424`}, {
      0.9408225678201714, -1.0200049922865828`}, {
      0.9445978904766369, -1.017352889714844}, {
      0.9483631734532746, -1.014689974163952}, {
      0.9521183946600454, -1.0120162931201866`}, {
      0.9558635322244345, -1.0093318940497713`}, {
      0.9595985644907794, -1.006636824398065}, {
      0.9633234700195963, -1.0039311315887613`}, {
      0.9670382275869057, -1.0012148630230935`}, {
      0.9707428161835568, -0.9984880660790442}, {
      0.9744372150145504, -0.9957507881105612}, {
      0.9781214034983603, -0.9930030764467783}, {
      0.9817953612662549, -0.9902449783912423}, {
      0.9854590681616161, -0.9874765412211441}, {
      0.9891125042392582, -0.9846978121865575}, {
      0.9927556497647453, -0.9819088385096807}, {
      0.9963884852137076, -0.9791096673840857}, {
      1.0000109912711574`, -0.9763003459739714}, {
      1.0036231488308034`, -0.9734809214134231}, {
      1.0072249389943642`, -0.9706514408056769}, {
      1.0108163430708812`, -0.9678119512223892}, {
      1.0143973425760304`, -0.9649624997029124}, {
      1.0179679192314337`, -0.9621031332535751}, {
      1.0215280549639687`, -0.9592338988469681}, {
      1.0250777319050786`, -0.9563548434212352}, {
      1.0286169323900802`, -0.9534660138793701}, {
      1.0321456389574726`, -0.9505674570885176}, {
      1.035663834348244, -0.9476592198792806}, {
      1.0391715015051788`, -0.9447413490450322}, {
      1.0426686235721625`, -0.9418138913412332}, {
      1.0461551838934882`, -0.9388768934847542}, {
      1.0496311660131605`, -0.9359304021532039}, {
      1.0530965536741999`, -0.9329744639842613}, {
      1.0565513308179464`, -0.930009125575014}, {
      1.059995481583363, -0.9270344334813014}, {
      1.0634289903063372`, -0.9240504342170627}, {
      1.0668518415189843`, -0.92105717425369}, {
      1.0702640199489484`, -0.9180547000193875}, {
      1.0736655105187038`, -0.9150430578985337}, {
      1.0770562983448562`, -0.912022294231051}, {
      1.0804363687374425`, -0.9089924553117784}, {
      1.083805707199232, -0.9059535873898504}, {
      1.0871642994250252`, -0.9029057366680805}, {
      1.090512131300954, -0.899848949302349}, {
      1.093849188903781, -0.8967832714009973}, {
      1.0971754585001976`, -0.8937087490242257}, {
      1.1004909265461245`, -0.8906254281834964}, {
      1.1037955796860088`, -0.8875333548409422}, {
      1.1070894047521234`, -0.8844325749087791}, {
      1.1103723887638643`, -0.8813231342487244}, {
      1.11364451892705, -0.8782050786714192}, {
      1.1169057826332187`, -0.8750784539358559}, {
      1.1201561674589264`, -0.8719433057488111}, {
      1.1233956611650449`, -0.8687996797642819}, {
      1.1266242516960594`, -0.8656476215829283}, {
      1.129841927179367, -0.86248717675152}, {
      1.1330486759245737`, -0.8593183907623873}, {
      1.136244486422793, -0.8561413090528774}, {
      1.139429347345943, -0.8529559770048157}, {
      1.142603247546045, -0.8497624399439706}, {
      1.1457661760545217`, -0.8465607431395239}, {
      1.1489181220814946`, -0.8433509318035457}, {
      1.152059075015083, -0.8401330510904738}, {
      1.1551890244207021`, -0.8369071460965977}, {
      1.1583079600403616`, -0.8336732618595473}, {
      1.1614158717919645`, -0.830431443357786}, {
      1.1645127497686063`, -0.8271817355101084}, {
      1.167598584237874, -0.8239241831751429}, {
      1.1706733656411457`, -0.8206588311508582}, {
      1.1737370845928903`, -0.8173857241740747}, {
      1.1767897318799674`, -0.8141049069199805}, {
      1.179831298460928, -0.8108164240016512}, {
      1.1828617754653148`, -0.8075203199695751}, {
      1.185881154192963, -0.804216639311182}, {
      1.1888894261133025`, -0.800905426450377}, {
      1.1918865828646588`, -0.797586725747078}, {
      1.1948726162535552`, -0.7942605814967586}, {
      1.197847518254016, -0.7909270379299943}, {
      1.2008112810068687`, -0.7875861392120135}, {
      1.203763896819048, -0.7842379294422531}, {
      1.2067053581628995`, -0.7808824526539179}, {
      1.2096356576754839`, -0.7775197528135442}, {
      1.212554788157882, -0.7741498738205688}, {
      1.2154627425745002`, -0.7707728595069002}, {
      1.2183595140523762`, -0.7673887536364965}, {
      1.221245095880486, -0.763997599904945}, {
      1.2241194815090501`, -0.760599441939048}, {
      1.2269826645488424`, -0.7571943232964118}, {
      1.229834638770497, -0.7537822874650398}, {
      1.232675398103818, -0.7503633778629303}, {
      1.2355049366370892`, -0.7469376378376775}, {
      1.2383232486163829`, -0.743505110666078}, {
      1.2411303284448723`, -0.7400658395537398}, {
      1.2439261706821418`, -0.7366198676346966}, {
      1.2467107700435003`, -0.7331672379710251}, {
      1.249484121399293, -0.7297079935524675}, {
      1.252246219774216, -0.726242177296057}, {
      1.2549970603466292`, -0.7227698320457475}, {
      1.2577366384478732`, -0.719291000572048}, {
      1.260464949561584, -0.7158057255716596}, {
      1.2631819893230103`, -0.7123140496671178}, {
      1.2658877535183304`, -0.7088160154064378}, {
      1.2685822380839709`, -0.7053116652627643}, {
      1.2712654391059255`, -0.7018010416340246}, {
      1.2739373528190754`, -0.6982841868425856}, {
      1.2765979756065093`, -0.6947611431349157}, {
      1.2792473039988461`, -0.6912319526812487}, {
      1.2818853346735568`, -0.6876966575752534}, {
      1.2845120644542876`, -0.6841552998337055}, {
      1.287127490310185, -0.6806079213961638}, {
      1.289731609355221, -0.6770545641246506}, {
      1.292324418847518, -0.6734952698033352}, {
      1.2949059161886782`, -0.6699300801382213}, {
      1.29747609892311, -0.6663590367568383}, {
      1.300034964737357, -0.6627821812079359}, {
      1.3025825114594292`, -0.6591995549611824}, {
      1.3051187370581327`, -0.6556111994068675}, {
      1.3076436396424032`, -0.6520171558556073}, {
      1.3101572174606377`, -0.6484174655380537}, {
      1.3126594689000295`, -0.6448121696046077}, {
      1.3151503924859025`, -0.6412013091251357}, {
      1.3176299868810484`, -0.6375849250886894}, {
      1.3200982508850632`, -0.6339630584032291}, {
      1.3225551834336855`, -0.6303357498953516}, {
      1.3250007835981363`, -0.6267030403100198}, {
      1.3274350505844594`, -0.6230649703102975}, {
      1.3298579837328628`, -0.6194215804770863}, {
      1.3322695825170614`, -0.6157729113088669}, {
      1.334669846543621, -0.6121190032214434}, {
      1.3370587755513035`, -0.6084598965476911}, {
      1.3394363694104123`, -0.6047956315373072}, {
      1.3418026281221405`, -0.6011262483565661}, {
      1.3441575518179187`, -0.5974517870880768}, {
      1.346501140758765, -0.5937722877305438}, {
      1.3488333953346356`, -0.5900877901985325}, {
      1.351154316063777, -0.5863983343222363}, {
      1.3534639035920786`, -0.5827039598472479}, {
      1.3557621586924284`, -0.5790047064343337}, {
      1.3580490822640678`, -0.5753006136592116}, {
      1.3603246753319485`, -0.5715917210123316}, {
      1.3625889390460904`, -0.5678780678986598}, {
      1.3648418746809419`, -0.5641596936374661}, {
      1.36708348363474, -0.560436637462114}, {
      1.3693137674288725`, -0.5567089385198548}, {
      1.3715327277072402`, -0.5529766358716239}, {
      1.373740366235623, -0.5492397684918411}, {
      1.3759366849010441`, -0.5454983752682129}, {
      1.378121685711139, -0.5417524950015391}, {
      1.3802953707935215`, -0.538002166405522}, {
      1.3824577423951554`, -0.5342474281065783}, {
      1.384608802881725, -0.5304883186436544}, {
      1.3867485547370066`, -0.5267248764680447}, {
      1.388877000562244, -0.5229571399432127}, {
      1.3909941430755215`, -0.5191851473446156}, {
      1.393099985111142, -0.5154089368595314}, {
      1.395194529619004, -0.5116285465868886}, {
      1.397277779663981, -0.5078440145371003}, {
      1.399349738425302, -0.5040553786318991}, {
      1.4014104091959332`, -0.5002626767041769}, {
      1.4034597953819616`, -0.49646594649782644`}, {
      1.4054979005019803`, -0.49266522566758586`}, {
      1.4075247281864733`, -0.4888605517788865}, {
      1.4095402821772043`, -0.48505196230770303`}, {
      1.4115445663266046`, -0.481239494640407}, {
      1.4135375845971645`, -0.47742318607362255`}, {
      1.4155193410608244`, -0.47360307381408534`}, {
      1.4174898398983684`, -0.46977919497850423`}, {
      1.419449085398819, -0.46595158659342545`}, {
      1.4213970819588333`, -0.4621202855950997}, {
      1.4233338340821002`, -0.4582853288293521}, {
      1.42525934637874, -0.4544467530514545}, {
      1.4271736235647046`, -0.4506045949260008}, {
      1.4290766704611795`, -0.4467588910267848}, {
      1.4309684919939878`, -0.4429096778366806}, {
      1.4328490931929945`, -0.43905699174752605`}, {
      1.4347184791915135`, -0.4352008690600082}, {
      1.436576655225715, -0.431341345983552}, {
      1.4384236266340356`, -0.42747845863621114`}, {
      1.4402593988565886`, -0.42361224304456163`}, {
      1.4420839774345773`, -0.41974273514359794`}, {
      1.443897368009708, -0.41586997077663185`}, {
      1.4456995763236065`, -0.4119939856951934}, {
      1.447490608217234, -0.40811481555893486`}, {
      1.4492704696303071`, -0.40423249593553673`}, {
      1.4510391666007163`, -0.4003470623006167}, {
      1.4527967052639488`, -0.39645855003764063`}, {
      1.454543091852511, -0.3925669944378363}, {
      1.4562783326953541`, -0.3886724307001095}, {
      1.458002434217299, -0.3847748939309623}, {
      1.4597154029384654`, -0.3808744191444143}, {
      1.4614172454737, -0.3769710412619256}, {
      1.4631079685320088`, -0.3730647951123227}, {
      1.4647875789159883`, -0.3691557154317264}, {
      1.4664560835212603`, -0.36524383686348205`}, {
      1.4681134893359074`, -0.3613291939580927}, {
      1.4697598034399098`, -0.35741182117315357`}, {
      1.4713950330045849`, -0.35349175287328966`}, {
      1.473019185292027, -0.34956902333009526`}, {
      1.4746322676545496`, -0.34564366672207597`}, {
      1.4762342875341288`, -0.3417157171345926}, {
      1.4778252524618483`, -0.3377852085598077}, {
      1.4794051700573463`, -0.33385217489663405`}, {
      1.480974048028264, -0.3299166499506855}, {
      1.4825318941696954`, -0.32597866743422993`}, {
      1.4840787163636386`, -0.3220382609661446}, {
      1.4856145225784494`, -0.3180954640718731}, {
      1.4871393208682955`, -0.31415031018338535`}, {
      1.4886531193726134`, -0.3102028326391388}, {
      1.4901559263155661`, -0.3062530646840424}, {
      1.4916477500055028`, -0.3023010394694225}, {
      1.4931285988344198`, -0.2983467900529906}, {
      1.4945984812774238`, -0.29439034939881353`}, {
      1.496057405892196, -0.29043175037728564`}, {
      1.4975053813184587`, -0.28647102576510275`}, {
      1.498942416277442, -0.2825082082452384}, {
      1.5003685195713545`, -0.27854333040692225`}, {
      1.5017837000828531`, -0.27457642474562}, {
      1.5031879667745163`, -0.27060752366301577`}, {
      1.504581328688318, -0.26663665946699633`}, {
      1.505963794945104, -0.26266386437163725`}, {
      1.5073353747440683`, -0.25868917049719087`}, {
      1.508696077362234, -0.25471260987007655`}, {
      1.5100459121539327`, -0.2507342144228726}, {
      1.5113848885502876`, -0.24675401599431002`}, {
      1.5127130160586972`, -0.24277204632926858`}, {
      1.5140303042623213`, -0.23878833707877425`}, {
      1.5153367628195686`, -0.23480291979999887`}, {
      1.516632401463585, -0.2308158259562615}, {
      1.5179172300017452`, -0.2268270869170317}, {
      1.5191912583151443`, -0.22283673395793466`}, {
      1.5204544963580924`, -0.21884479826075798`}, {
      1.5217069541576094`, -0.2148513109134605}, {
      1.522948641812923, -0.21085630291018267`}, {
      1.5241795694949676`, -0.20685980515125899`}, {
      1.5253997474458845`, -0.20286184844323182`}, {
      1.5266091859785247`, -0.19886246349886744`}, {
      1.5278078954759522`, -0.19486168093717332`}, {
      1.52899588639095, -0.19085953128341754`}, {
      1.5301731692455276`, -0.1868560449691497}, {
      1.5313397546304288`, -0.1828512523322236}, {
      1.5324956532046434`, -0.17884518361682158`}, {
      1.5336408756949191`, -0.17483786897348053`}, {
      1.534775432895275, -0.17082933845911963`}, {
      1.5358993356665172`, -0.16681962203706965`}, {
      1.5370125949357563`, -0.16280874957710395`}, {
      1.5381152216959262`, -0.15879675085547104`}, {
      1.5392072270053045`, -0.15478365555492885`}, {
      1.5402886219870346`, -0.15076949326478045`}, {
      1.5413594178286496`, -0.14675429348091157`}, {
      1.5424196257815976`, -0.1427380856058294}, {
      1.5434692571607695`, -0.13872089894870332`}, {
      1.544508323344027, -0.13470276272540682`}, {
      1.5455368357717338`, -0.1306837060585612}, {
      1.5465548059462868`, -0.12666375797758064`}, {
      1.5475622454316509`, -0.122642947418719}, {
      1.5485591658528937`, -0.11862130322511784`}, {
      1.5495455788957233`, -0.1145988541468561}, {
      1.5505214963060263`, -0.11057562884100125`}, {
      1.5514869298894087`, -0.10655165587166189`}, {
      1.552441891510738, -0.10252696371004175`}, {
      1.5533863930936866`, -0.09850158073449519}, {
      1.5543204466202778`, -0.09447553523058415}, {
      1.555244064130432, -0.09044885539113644}, {
      1.5561572577215161`, -0.08642156931630551}, {
      1.557060039547894, -0.08239370501363157}, {
      1.5579524218204774`, -0.07836529039810411}, {
      1.558834416806281, -0.0743363532922258}, {
      1.5597060368279771`, -0.07030692142607776}, {
      1.5605672942634523`, -0.06627702243738612}, {
      1.5614182015453666`, -0.06224668387158998}, {
      1.5622587711607132`, -0.058215933181910706`}, {
      1.5630890156503812`, -0.05418479772942248}, {
      1.563908947608718, -0.0501533047831242}, {
      1.5647185796830958`, -0.046121481520012664`}, {
      1.5655179245734774`, -0.04208935502515703}, {
      1.5663069950319852`, -0.038056952291774554`}, {
      1.5670858038624713`, -0.0340243002213076}, {
      1.567854363920089, -0.029991425623501856`}, {
      1.5686126881108666`, -0.02595835521648586}, {
      1.569360789391282, -0.021925115626851673`}, {
      1.5700986807678399`, -0.01789173338973684}, {
      1.5708263752966498`, -0.013858234948907516`}, {
      1.571543886083006, -0.009824646656842797}, {
      1.572251226280969, -0.005790994774820246}, {
      1.572948409092949, -0.001757305473002584}, {1.573635447769291, 
      0.002276395169474454}, {1.5743123556078606`, 0.0063100811644131016`}, {
      1.5749791459536324`, 0.010343726614463408`}, {1.5756358321982795`, 
      0.01437730571303195}, {1.5762824277797656`, 0.018410792744189414`}, {
      1.576918946181937, 0.022444162082577074`}, {1.5775454009341179`, 
      0.026477388193312166`}, {1.5781618056107056`, 0.030510445631892164`}, {
      1.57876817383077, 0.03454330904409798}, {1.579364519257651, 
      0.03857595316589609}, {1.5799508555985615`, 0.04260835282333959}, {
      1.5805271966041885`, 0.04664048293246821}, {1.5810935560682977`, 
      0.05067231849920729}, {1.58164994782734, 0.05470383461926569}, {
      1.5821963857600585`, 0.05873500647803274}, {1.582732883787097, 
      0.0627658093504741}, {1.5832594558706121`, 0.06679621860102669}, {
      1.5837761160138837`, 0.07082620968349256}, {1.58428287826093, 
      0.07485575814093182}, {1.5847797566961226`, 0.07888483960555459}, {
      1.5852667654438035`, 0.08291342979861198}, {1.5857439186679037`, 
      0.08694150453028607}, {1.5862112305715632`, 0.09096903969957904}, {
      1.5866687153967536`, 0.09499601129420128}, {1.5871163874239, 
      0.09902239539045861}, {1.5875542609715072`, 0.10304816815313854`}, {
      1.5879823503957857`, 0.10707330583539569`}, {1.5884006700902797`, 
      0.11109778477863627`}, {1.5888092344854972`, 0.11512158141240164`}, {
      1.5892080580485402`, 0.11914467225425103`}, {1.5895971552827386`, 
      0.1231670339096434}, {1.589976540727284, 0.12718864307181837`}, {
      1.5903462289568653`, 0.1312094765216764}, {1.5907062345813072`, 
      0.13522951112765796`}, {1.5910565722452081`, 0.13924872384562206`}, {
      1.5913972566275811`, 0.14326709171872382`}, {1.591728302441496, 
      0.1472845918772912}, {1.5920497244337235`, 0.15130120153870108`}, {
      1.5923615373843791`, 0.15531689800725432`}, {1.5926637561065715`, 
      0.1593316586740502}, {1.5929563954460497`, 0.16334546101685996`}, {
      1.593239470280853, 0.1673582825999997}, {1.5935129955209628`, 
      0.1713701010742024}, {1.5937769861079554`, 0.17538089417648903`}, {
      1.5940314570146554`, 0.17939063973003935`}, {1.5942764232447932`, 
      0.18339931564406153`}, {1.5945118998326613`, 0.18740689991366125`}, {
      1.5947379018427736`, 0.19141337061971}, {1.594954444369526, 
      0.19541870592871274`}, {1.5951615425368584`, 0.19942288409267475`}, {
      1.5953592114979178`, 0.2034258834489678}, {1.595547466434724, 
      0.20742768242019574`}, {1.5957263225578353`, 0.21142825951405922`}, {
      1.595895795106017, 0.21542759332321984`}, {1.5960558993459113`, 
      0.21942566252516366`}, {1.5962066505717072`, 0.22342244588206395`}, {
      1.596348064104814, 0.22741792224064333`}, {1.596480155293535, 
      0.23141207053203536`}, {1.5966029395127426`, 0.2354048697716452}, {
      1.5967164321635556`, 0.23939629905901008`}, {1.5968206486730177`, 
      0.24338633757765865`}, {1.596915604493777, 0.2473749645949702}, {
      1.5970013151037679`, 0.25136215946203283`}, {1.5970777960058935`, 
      0.25534790161350135`}, {1.5971450627277102`, 0.2593321705674544}, {
      1.5972031308211132`, 0.263314945925251}, {1.597252015862024, 
      0.2672962073713868}, {1.5972917334500791`, 0.27127593467334915`}, {
      1.5973222992083196`, 0.2752541076814722}, {1.5973437287828836`, 
      0.27923070632879116`}, {1.5973560378426985`, 0.2832057106308961}, {
      1.5973592420791762`, 0.28717910068578517`}, {1.5973533572059084`, 
      0.2911508566737172}, {1.597338398958364, 0.2951209588570639}, {
      1.5973143830935883`, 0.2990893875801616}, {1.597281325389903, 
      0.3030561232691622}, {1.5972392416466075`, 0.30702114643188383`}, {
      1.5971881476836827`, 0.31098443765766104`}, {1.5971280593414947`, 
      0.31494597761719434`}, {1.5970589924805014`, 0.3189057470623993}, {
      1.5969809629809595`, 0.32286372682625536`}, {1.5968939867426326`, 
      0.3268198978226537}, {1.5967980796845025`, 0.3307742410462452}, {
      1.59669325774448, 0.3347267375722876}, {1.5965795368791176`, 
      0.3386773685564923}, {1.5964569330633247`, 0.3426261152348706}, {
      1.5963254622900827`, 0.34657295892357975`}, {1.5961851405701617`, 
      0.35051788101876846`}, {1.59603598393184, 0.35446086299642166`}, {
      1.5958780084206237`, 0.3584018864122053}, {1.5957112300989666`, 
      0.3623409329013106}, {1.595535665045995, 0.3662779841782977}, {
      1.5953513293572301`, 0.3702130220369392}, {1.5951582391443138`, 
      0.37414602835006294`}, {1.5949564105347351`, 0.37807698506939486`}, {
      1.5947458596715596`, 0.382005874225401}, {1.5945266027131566`, 
      0.3859326779271295}, {1.594298655832932, 0.3898573783620518}, {
      1.594062035219059, 0.3937799577959042}, {1.5938167570742128`, 
      0.39770039857252804`}, {1.5935628376153037`, 0.4016186831137106}, {
      1.5933002930732154`, 0.4055347939190247}, {1.5930291396925407`, 
      0.4094487135656688}, {1.5927493937313213`, 0.41336042470830614`}, {
      1.5924610714607874`, 0.4172699100789037}, {1.5921641891650997`, 
      0.421177152486571}, {1.5918587631410919`, 0.4250821348173986}, {
      1.5915448096980145`, 0.428984840034296}, {1.5912223451572807`, 
      0.4328852511768294}, {1.5908913858522131`, 0.4367833513610594}, {
      1.5905519481277914`, 0.44067912377937785`}, {1.5902040483404019`, 
      0.44457255170034493`}, {1.589847702857588, 0.44846361846852567`}, {
      1.5894829280578024`, 0.45235230750432637`}, {1.5891097403301604`, 
      0.45623860230383045`}, {1.5887281560741937`, 0.46012248643863435`}, {
      1.588338191699607, 0.46400394355568303`}, {1.5879398636260345`, 
      0.4678829573771052}, {1.587533188282798, 0.47175951170004843`}, {
      1.587118182108668, 0.47563359039651354`}, {1.5866948615516223`, 
      0.47950517741318954`}, {1.5862632430686099`, 0.48337425677128765`}, {
      1.5858233431253137`, 0.4872408125663754}, {1.5853751781959153`, 
      0.4911048289682105}, {1.5849187647628609`, 0.4949662902205743}, {
      1.5844541193166282`, 0.4988251806411053}, {1.583981258355495, 
      0.5026814846211323}, {1.5835001983853094`, 0.5065351866255071}, {
      1.583010955919259, 0.5103862711924377}, {1.5825135474776453`, 
      0.5142347229333205}, {1.5820079895876549`, 0.5180805265325729}, {
      1.5814942987831353`, 0.5219236667474652}, {1.5809724916043706`, 
      0.5257641284079528}, {1.580442584597858, 0.5296018964165082}, {
      1.5799045943160865`, 0.5334369557479524}, {1.579358537317316, 
      0.537269291449286}, {1.578804430165358, 0.5410988886395215}, {
      1.5782422894293577`, 0.5449257325095136}, {1.5776721316835767`, 
      0.5487498083217905}, {1.577093973507177, 0.552571101410385}, {
      1.5765078314840073`, 0.5563895971806646}, {1.575913722202389, 
      0.5602052811091628}, {1.5753116622549035`, 0.564018138743409}, {
      1.5747016682381825`, 0.5678281557017592}, {1.574083756752697, 
      0.5716353176732256}, {1.5734579444025494`, 0.5754396104173075}, {
      1.572824247795265, 0.5792410197638204}, {1.5721826835415866`, 
      0.5830395316127264}, {1.5715332682552692`, 0.5868351319339636}, {
      1.5708760185528752`, 0.5906278067672761}, {1.5702109510535727`, 
      0.5944175422220429}, {1.5695380823789327`, 0.5982043244771079}, {
      1.5688574291527293`, 0.6019881397806089}, {1.5681690080007398`, 
      0.605768974449807}, {1.5674728355505465`, 0.6095468148709156}, {
      1.56676892843134, 0.6133216474989295}, {1.5660573032737222`, 
      0.617093458857454}, {1.5653379767095124`, 0.620862235538534}, {
      1.564610965371553, 0.6246279642024823}, {1.5638762858935167`, 
      0.6283906315777089}, {1.5631339549097156`, 0.6321502244605495}, {
      1.56238398905491, 0.6359067297150942}, {1.5616264049641198`, 
      0.6396601342730163}, {1.5608612192724354`, 0.6434104251334005}, {
      1.5600884486148314`, 0.6471575893625721}, {1.5593081096259804`, 
      0.6509016140939246}, {1.5585202189400675`, 0.6546424865277488}, {
      1.5577247931906075`, 0.6583801939310612}, {1.556921849010261, 
      0.662114723637432}, {1.5561114030306535`, 0.6658460630468138}, {
      1.555293471882195, 0.66957419962537}, {1.5544680721938988`, 
      0.6732991209053028}, {1.5536352205932058`, 0.6770208144846817}, {
      1.5527949337058045`, 0.6807392680272718}, {1.5519472281554563`, 
      0.6844544692623622}, {1.5510921205638195`, 0.6881664059845941}, {
      1.5502296275502754`, 0.6918750660537892}, {1.549359765731755, 
      0.695580437394778}, {1.548482551722567, 0.699282507997228}, {
      1.5475980021342268`, 0.702981265915472}, {1.546706133575286, 
      0.7066766992683368}, {1.5458069626511641`, 0.7103687962389706}, {
      1.5449005059639798`, 0.7140575450746728}, {1.5439867801123845`, 
      0.7177429340867209}, {1.5430658016913967`, 0.7214249516501998}, {
      1.5421375872922367`, 0.7251035862038301}, {1.5412021535021625`, 
      0.7287788262497964}, {1.5402595169043072`, 0.7324506603535759}, {
      1.5393096940775177`, 0.7361190771437668}, {1.538352701596193, 
      0.7397840653119172}, {1.5373885560301248`, 0.7434456136123534}, {
      1.5364172739443385`, 0.7471037108620088}, {1.5354388718989356`, 
      0.7507583459402524}, {1.5344533664489368`, 0.7544095077887178}, {
      1.5334607741441257`, 0.7580571854111322}, {1.5324611115288946`, 
      0.7617013678731448}, {1.5314543951420903`, 0.765342044302156}, {
      1.5304406415168608`, 0.7689792038871468}, {1.529419867180504, 
      0.7726128358785072}, {1.5283920886543163`, 0.7762429295878659}, {
      1.527357322453443, 0.7798694743879195}, {1.5263155850867285`, 
      0.7834924597122617}, {1.5252668930565692`, 0.7871118750552125}, {
      1.5242112628587652`, 0.7907277099716483}, {1.5231487109823756`, 
      0.7943399540768308}, {1.5220792539095718`, 0.797948597046237}, {
      1.5210029081154939`, 0.801553628615389}, {1.5199196900681076`, 
      0.805155038579684}, {1.518829616228061, 0.8087528167942234}, {
      1.517732703048544, 0.8123469531736441}, {1.516628966975147, 
      0.8159374376919478}, {1.515518424445722, 0.8195242603823314}, {
      1.514401091890243, 0.8231074113370175}, {1.5132769857306692`, 
      0.8266868807070847}, {1.512146122380807, 0.8302626587022987}, {
      1.5110085182461754`, 0.833834735590942}, {1.5098641897238696`, 
      0.8374031016996459}, {1.5087131532024278`, 0.8409677474132206}, {
      1.5075554250616974`, 0.8445286631744864}, {1.506391021672703, 
      0.8480858394841052}, {1.5052199593975146`, 0.8516392669004116}, {
      1.5040422545891174`, 0.8551889360392443}, {1.502857923591282, 
      0.8587348375737777}, {1.5016669827384355`, 0.8622769622343537}, {
      1.5004694483555332`, 0.8658153008083136}, {1.4992653367579325`, 
      0.8693498441398299}, {1.498054664251266, 0.8728805831297384}, {
      1.4968374471313164`, 0.8764075087353708}, {1.495613701683892, 
      0.8799306119703866}, {1.4943834441847033`, 0.8834498839046062}, {
      1.49314669089924, 0.8869653156638434}, {1.491903458082649, 
      0.890476898429738}, {1.4906537619796143`, 0.8939846234395891}, {
      1.4893976188242357`, 0.8974884819861884}, {1.4881350448399102`, 
      0.9009884654176532}, {1.4868660562392133`, 0.9044845651372603}, {
      1.4855906692237812`, 0.907976772603279}, {1.4843088999841942`, 
      0.9114650793288058}, {1.4830207646998608`, 0.9149494768815977}, {
      1.481726279538902, 0.9184299568839066}, {1.4804254606580374`, 
      0.9219065110123137}, {1.4791183242024721`, 0.9253791309975639}, {
      1.4778048863057833`, 0.9288478086244004}, {1.4764851630898088`, 
      0.9323125357313999}, {1.4751591706645353`, 0.9357733042108073}, {
      1.4738269251279892`, 0.9392301060083708}, {1.4724884425661262`, 
      0.9426829331231777}, {1.4711437390527233`, 0.94613177760749}, {
      1.4697928306492705`, 0.9495766315665795}, {1.4684357334048637`, 
      0.9530174871585644}, {1.4670724633560988`, 0.9564543365942454}, {
      1.4657030365269657`, 0.9598871721369417}, {1.4643274689287442`, 
      0.9633159861023279}, {1.4629457765598997`, 0.9667407708582705}, {
      1.4615579754059798`, 0.9701615188246646}, {1.4601640814395125`, 
      0.9735782224732715}, {1.4587641106199039`, 0.9769908743275558}, {
      1.4573580788933378`, 0.980399466962523}, {1.455946002192676, 
      0.9838039930045569}, {1.4545278964373578`, 0.9872044451312579}, {
      1.4531037775333024`, 0.9906008160712811}, {1.451673661372811, 
      0.9939930986041745}, {1.450237563834469, 0.9973812855602175}, {
      1.4487955007830504`, 1.0007653698202599`}, {1.4473474880694226`, 
      1.0041453443155606`}, {1.4458935415304504`, 1.0075212020276272`}, {
      1.4444336769889023`, 1.010892935988055}, {1.4429679102533577`, 
      1.0142605392783666`}, {1.4414962571181138`, 1.0176240050298522`}, {
      1.4400187333630936`, 1.0209833264234092`}, {1.438535354753755, 
      1.0243384966893827`}, {1.4370461370409997`, 1.027689509107406}, {
      1.4355510959610849`, 1.0310363570062417`}, {1.4340502472355323`, 
      1.0343790337636216`}, {1.4325436065710415`, 1.0377175328060888`}, {
      1.4310311896594012`, 1.0410518476088393`}, {1.429513012177403, 
      1.0443819716955627`}, {1.4279890897867544`, 1.0477078986382848`}, {
      1.4264594381339946`, 1.0510296220572097`}, {1.4249240728504082`, 
      1.0543471356205616`}, {1.4233830095519424`, 1.0576604330444281`}, {
      1.4218362638391224`, 1.0609695080926027`}, {1.4202838512969693`, 
      1.0642743545764275`}, {1.4187257874949182`, 1.0675749663546368`}, {
      1.417162087986736, 1.070871337333201}, {1.4155927683104417`, 
      1.07416346146517}, {1.4140178439882254`, 1.0774513327505173`}, {
      1.4124373305263698`, 1.0807349452359838`}, {1.410851243415171, 
      1.084014293014923}, {1.4092595981288605`, 1.0872893702271456`}, {
      1.4076624101255277`, 1.090560171058764}, {1.4060596948470439`, 
      1.0938266897420386`}, {1.404451467718985, 1.0970889205552221`}, {
      1.4028377441505573`, 1.100346857822406}, {1.401218539534522, 
      1.1036004959133665`}, {1.3995938692471215`, 1.1068498292434104`}, {
      1.3979637486480057`, 1.110094852273222}, {1.3963281930801592`, 
      1.1133355595087098`}, {1.394687217869829, 1.1165719455008536`}, {
      1.3930408383264534`, 1.1198040048455513`}, {1.3913890697425906`, 
      1.123031732183467}, {1.3897319273938482`, 1.1262551221998787`}, {
      1.3880694265388143`, 1.1294741696245258`}, {1.386401582418988, 
      1.1326888692314585`}, {1.3847284102587107`, 1.1358992158388852`}, {
      1.383049925265099, 1.1391052043090222`}, {1.3813661426279764`, 
      1.1423068295479424`}, {1.3796770775198082`, 1.1455040865054247`}, {
      1.3779827450956341`, 1.1486969701748038`}, {1.376283160493004, 
      1.1518854755928198`}, {1.374578338831912, 1.155069597839469}, {
      1.372868295214733, 1.1582493320378533`}, {1.3711530447261597`, 
      1.1614246733540323`}, {1.369432602433138, 1.164595616996873}, {
      1.367706983384806, 1.1677621582179016`}, {1.3659762026124314`, 
      1.1709242923111551`}, {1.364240275129351, 1.174082014613033}, {
      1.3624992159309095`, 1.1772353205021493`}, {1.3607530399943994`, 
      1.1803842053991849`}, {1.359001762279002, 1.18352866476674}, {
      1.357245397725728, 1.1866686941091877`}, {1.3554839612573593`, 
      1.1898042889725262`}, {1.3537174677783914`, 1.1929354449442329`}, {
      1.351945932174976, 1.1960621576531179`}, {1.3501693693148646`, 
      1.199184422769178}, {1.3483877940473523`, 1.202302236003451}, {
      1.346601221203222, 1.2054155931078705`}, {1.3448096655946895`, 
      1.2085244898751202`}, {1.3430131420153497`, 1.2116289221384895`}, {
      1.3412116652401223`, 1.2147288857717287`}, {1.3394052500251978`, 
      1.2178243766889048`}, {1.3375939111079862`, 1.2209153908442572`}, {
      1.3357776632070635`, 1.2240019242320543`}, {1.3339565210221205`, 
      1.22708397288645}, {1.3321304992339114`, 1.23016153288134}, {
      1.3302996125042037`, 1.2332346003302197`}, {1.3284638754757276`, 
      1.2363031713860408`}, {1.3266233027721266`, 1.2393672422410698`}, {
      1.3247779089979081`, 1.2424268091267452`}, {1.3229277087383957`, 
      1.2454818683135365`}, {1.3210727165596805`, 1.2485324161108018`}, {
      1.3192129470085738`, 1.2515784488666473`}, {1.31734841461256, 
      1.2546199629677866`}, {1.3154791338797505`, 1.2576569548393994`}, {
      1.3136051192988372`, 1.260689420944992}, {1.3117263853390477`, 
      1.2637173577862562`}, {1.3098429464501, 1.266740761902931}, {
      1.307954817062158, 1.2697596298726617`}, {1.3060620115857875`, 
      1.2727739583108622`}, {1.3041645444119132`, 1.2757837438705748`}, {
      1.3022624299117753`, 1.2787889832423327`}, {1.3003556824368874`, 
      1.2817896731540213`}, {1.298444316318994, 1.28478581037074}, {
      1.2965283458700296`, 1.2877773916946647`}, {1.2946077853820774`, 
      1.2907644139649108`}, {1.2926826491273289`, 1.2937468740573959`}, {
      1.2907529513580436`, 1.2967247688847026`}, {1.2888187063065097`, 
      1.2996980953959425`}, {1.2868799281850052`, 1.30266685057662}, {
      1.2849366311857588`, 1.3056310314484958`}, {1.282988829480912, 
      1.3085906350694523`}, {1.2810365372224815`, 1.3115456585333574`}, {
      1.2790797685423216`, 1.31449609896993}, {1.2771185375520884`, 
      1.3174419535446047`}, {1.2751528583432021`, 1.3203832194583982`}, {
      1.2731827449868125`, 1.3233198939477746`}, {1.2712082115337626`, 
      1.326251974284512}, {1.2692292720145544`, 1.3291794577755685`}, {
      1.267245940439314, 1.3321023417629492`}, {1.2652582307977573`, 
      1.3350206236235735`}, {1.2632661570591577`, 1.3379343007691422`}, {
      1.2612697331723113`, 1.340843370646005}, {1.2592689730655051`, 
      1.343747830735029}, {1.2572638906464848`, 1.3466476785514667`}, {
      1.2552544998024224`, 1.3495429116448248`}, {1.2532408143998854`, 
      1.3524335275987323`}, {1.2512228482848056`, 1.3553195240308105`}, {
      1.2492006152824484`, 1.3582008985925422`}, {1.247174129197383, 
      1.3610776489691419`}, {1.2451434038134526`, 1.363949772879425}, {
      1.2431084528937448`, 1.3668172680756787`}, {1.2410692901805633`, 
      1.3696801323435328`}, {1.2390259293953991`, 1.3725383635018307`}, {
      1.2369783842389026`, 1.3753919594025001`}, {1.234926668390856, 
      1.3782409179304254`}, {1.2328707955101461`, 1.381085237003319}, {
      1.2308107792347378`, 1.383924914571594}, {1.2287466331816472`, 
      1.3867599486182358`}, {1.2266783709469158`, 1.3895903371586762`}, {
      1.224606006105585, 1.3924160782406654`}, {1.2225295522116713`, 
      1.395237169944146}, {1.2204490227981408`, 1.398053610381126}, {
      1.2183644313768855`, 1.4008653976955543`}, {1.216275791438699, 
      1.4036725300631934`}, {1.2141831164532526`, 1.4064750056914948`}, {
      1.2120864198690726`, 1.4092728228194735`}, {1.2099857151135174`, 
      1.412065979717584}, {1.2078810155927548`, 1.4148544746875953`}, {
      1.2057723346917404`, 1.4176383060624662`}, {1.2036596857741952`, 
      1.4204174722062226`}, {1.201543082182585, 1.4231919715138333`}, {
      1.1994225372380993`, 1.4259618024110863`}, {1.1972980642406306`, 
      1.4287269633544668`}, {1.1951696764687543`, 1.431487452831034}, {
      1.1930373871797089`, 1.4342432693582985`}, {1.1909012096093767`, 
      1.436994411484101}, {1.1887611569722645`, 1.4397408777864895`}, {
      1.186617242461485, 1.4424826668735993`}, {1.1844694792487382`, 
      1.4452197773835307`}, {1.1823178804842942`, 1.4479522079842284`}, {
      1.1801624592969748`, 1.450679957373361}, {1.1780032287941364`, 
      1.4534030242782014`}, {1.1758402020616532`, 1.4561214074555058`}, {
      1.1736733921639004`, 1.4588351056913949`}, {1.1715028121437385`, 
      1.461544117801234}, {1.1693284750224968`, 1.4642484426295148`}, {
      1.1671503937999583`, 1.4669480790497358`}, {1.1649685814543445`, 
      1.4696430259642843`}, {1.1627830509423, 1.4723332823043185`}, {
      1.1605938151988788`, 1.475018847029649}, {1.1584008871375295`, 
      1.4776997191286225`}, {1.156204279650082, 1.4803758976180033`}, {
      1.1540040056067333`, 1.4830473815428569`}, {1.1518000778560353`, 
      1.4857141699764338`}, {1.149592509224881, 1.4883762620200531`}, {
      1.1473813125184926`, 1.4910336568029858`}, {1.145166500520409, 
      1.4936863534823397`}, {1.142948085992474, 1.4963343512429443`}, {
      1.1407260816748253`, 1.498977649297235}, {1.1385005002858821`, 
      1.5016162468851388`}, {1.1362713545223353`, 1.5042501432739597`}, {
      1.1340386570591363`, 1.5068793377582645`}, {1.1318024205494874`, 
      1.509503829659769}, {1.1295626576248308`, 1.512123618327224}, {
      1.1273193808948399`, 1.514738703136303}, {1.1250726029474094`, 
      1.5173490834894883`}, {1.1228223363486467`, 1.5199547588159585`}, {
      1.1205685936428627`, 1.5225557285714766`}, {1.1183113873525634`, 
      1.525151992238277}, {1.1160507299784426`, 1.5277435493249552`}, {
      1.113786633999373, 1.5303303993663546`}, {1.1115191118723993`, 
      1.5329125419234566`}, {1.1092481760327306`, 1.5354899765832692`}, {
      1.106973838893734, 1.5380627029587168`}, {1.1046961128469273`, 
      1.5406307206885292`}, {1.1024150102619732`, 1.5431940294371322`}, {
      1.1001305434866728`, 1.545752628894538}, {1.0978427248469604`, 
      1.5483065187762355`}, {1.0955515666468971`, 1.550855698823081}, {
      1.0932570811686666`, 1.5534001688011905`}, {1.0909592806725696`, 
      1.5559399285018296`}, {1.0886581773970192`, 1.5584749777413065`}, {
      1.086353783558537, 1.5610053163608641`}, {1.0840461113517486`, 
      1.5635309442265717`}, {1.0817351729493796`, 1.566051861229218}, {
      1.0794209805022528`, 1.5685680672842044`}, {1.0771035461392842`, 
      1.571079562331438}, {1.0747828819674803`, 1.5735863463352253`}, {
      1.0724590000719354`, 1.576088419284166}, {1.0701319125158288`, 
      1.578585781191047}, {1.0678016313404233`, 1.5810784320927378`}, {
      1.0654681685650624`, 1.5835663720500837`}, {1.0631315361871694`, 
      1.5860496011478022`}, {1.0607917461822454`, 1.5885281194943779`}, {
      1.0584488105038683`, 1.5910019272219578`}, {1.0561027410836923`, 
      1.593471024486248}, {1.0537535498314465`, 1.5959354114664093`}, {
      1.0514012486349353`, 1.598395088364954}, {1.0490458493600379`, 
      1.600850055407643}, {1.046687363850708, 1.603300312843382}, {
      1.0443258039289747`, 1.60574586094412}, {1.0419611813949432`, 
      1.6081867000047463`}, {1.0395935080267953`, 1.6106228303429888`}, {
      1.0372227955807904`, 1.613054252299312}, {1.0348490557912673`, 
      1.6154809662368164`}, {1.0324723003706453`, 1.6179029725411358`}, {
      1.0300925410094262`, 1.620320271620338}, {1.0277097893761964`, 
      1.6227328639048237`}, {1.0253240571176294`, 1.625140749847226}, {
      1.022935355858488, 1.6275439299223105`}, {1.0205436972016266`, 
      1.6299424046268758`}, {1.0181490927279957`, 1.6323361744796543`}, {
      1.0157515539966433`, 1.6347252400212122`}, {1.0133510925447198`, 
      1.637109601813852}, {1.0109477198874806`, 1.639489260441513}, {
      1.0085414475182906`, 1.6418642165096728`}, {1.0061322869086282`, 
      1.6442344706452507`}, {1.00372024950809, 1.6466000234965088`}, {
      1.0013053467443942`, 1.6489608757329548`}, {0.9988875900233869, 
      1.6513170280452452`}, {0.9964669907290461, 1.6536684811450886`}, {
      0.9940435602234872, 1.6560152357651488`}, {0.9916173098469685, 
      1.6583572926589485`}, {0.9891882509178967, 1.6606946526007738`}, {
      0.9867563947328332, 1.6630273163855782`}, {0.9843217525664991, 
      1.6653552848288873`}, {0.981884335671783, 1.6676785587667036`}, {
      0.9794441552797462, 1.6699971390554116`}, {0.97700122259963, 
      1.6723110265716836`}, {0.9745555488188625, 1.6746202222123852`}, {
      0.972107145103066, 1.676924726894481}, {0.9696560225960633, 
      1.6792245415549414`}, {0.9672021924198866, 1.6815196671506494`}, {
      0.9647456656747841, 1.6838101046583063`}, {0.9622864534392287, 
      1.6860958550743401`}, {0.9598245667699253, 1.6883769194148126`}, {
      0.9573600167018199, 1.6906532987153264`}, {0.9548928142481076, 
      1.6929249940309337`}, {0.9524229704002412, 1.695192006436044}, {
      0.9499504961279405, 1.697454337024333}, {0.9474754023792011, 
      1.6997119869086512`}, {0.9449977000803038, 1.7019649572209334`}, {
      0.9425174001358239, 1.7042132491121076`}, {0.9400345134286408, 
      1.7064568637520043`}, {0.9375490508199479, 1.7086958023292675`}, {
      0.9350610231492628, 1.710930066051264}, {0.932570441234437, 
      1.7131596561439941`}, {0.9300773158716668, 1.7153845738520024`}, {
      0.9275816578355034, 1.7176048204382888`}, {0.9250834778788638, 
      1.7198203971842199`}, {0.9225827867330418, 1.72203130538944}, {
      0.920079595107719, 1.724237546371784}, {0.9175739136909759, 
      1.7264391214671886`}, {0.9150657531493034, 1.7286360320296046`}, {
      0.9125551241276146, 1.7308282794309102`}, {0.9100420372492564, 
      1.7330158650608236`}, {0.9075265031160211, 1.7351987903268162`}, {
      0.9050085323081593, 1.737377056654026}, {0.9024881353843912, 
      1.7395506654851718`}, {0.89996532288192, 1.741719618280466}, {
      0.8974401053164438, 1.74388391651753}, {0.8949124931821687, 
      1.7460435616913086`}, {0.8923824969518214, 1.7481985553139845`}, {
      0.8898501270766627, 1.7503488989148932`}, {0.8873153939865006, 
      1.7524945940404388`}, {0.8847783080897035, 1.7546356422540093`}, {
      0.882238879773214, 1.7567720451358926`}, {0.8796971194025626, 
      1.7589038042831928`}, {0.8771530373218812, 1.761030921309746}, {
      0.874606643853918, 1.7631533978460374`}, {0.8720579493000505, 
      1.7652712355391187`}, {0.869506963940301, 1.7673844360525242`}, {
      0.8669536980333501, 1.769493001066189}, {0.864398161816552, 
      1.7715969322763665`}, {0.8618403655059491, 1.7736962313955467`}, {
      0.8592803192962867, 1.7757909001523735`}, {0.8567180333610283, 
      1.7778809402915647`}, {0.8541535178523707, 1.7799663535738293`}, {
      0.8515867829012597, 1.7820471417757873`}, {0.8490178386174051, 
      1.784123306689889}, {0.8464466950892966, 1.786194850124334}, {
      0.8438733623842196, 1.788261773902992}, {0.8412978505482711, 
      1.7903240798653213`}, {0.8387201696063757, 1.7923817698662907`}, {
      0.836140329562302, 1.7944348457762993`}, {0.8335583403986786, 
      1.7964833094810972`}, {0.8309742120770109, 1.7985271628817072`}, {
      0.8283879545376976, 1.8005664078943453`}, {0.8257995777000473, 
      1.8026010464503432`}, {0.8232090914622955, 1.8046310804960695`}, {
      0.8206165057016216, 1.8066565119928517`}, {0.818021830274166, 
      1.8086773429168994`}, {0.8154250750150475, 1.8106935752592261`}, {
      0.8128262497383804, 1.8127052110255724`}, {0.8102253642372923, 
      1.814712252236329}, {0.8076224282839415, 1.8167147009264604`}, {
      0.805017451629535, 1.8187125591454276`}, {0.8024104440043462, 
      1.8207058289571134`}, {0.7998014151177331, 1.8226945124397456`}, {
      0.797190374658156, 1.824678611685821}, {0.7945773322931964, 
      1.826658128802031}, {0.7919622976695748, 1.8286330659091858`}, {
      0.7893452804131696, 1.83060342514214}, {0.7867262901290356, 
      1.8325692086497172`}, {0.7841053364014223, 1.8345304185946365`}, {
      0.7814824287937935, 1.836487057153438}, {0.7788575768488455, 
      1.8384391265164086`}, {0.7762307900885268, 1.8403866288875088`}, {
      0.7736020780140569, 1.8423295664842991`}, {0.7709714501059456, 
      1.8442679415378667`}, {0.7683389158240126, 1.8462017562927528`}, {
      0.7657044846074069, 1.8481310130068802`}, {0.7630681658746266, 
      1.85005571395148}, {0.760429969023538, 1.8519758614110202`}, {
      0.7577899034313963, 1.8538914576831338`}, {0.755147978454865, 
      1.8558025050785463`}, {0.7525042034300362, 1.857709005921005}, {
      0.7498585876724505, 1.859610962547208}, {0.7472111404771175, 
      1.8615083773067318`}, {0.7445618711185358, 1.863401252561962}, {
      0.741910788850714, 1.8652895906880227`}, {0.7392579029071911, 
      1.867173394072705}, {0.7366032225010565, 1.8690526651163988`}, {
      0.7339467568249717, 1.8709274062320211`}, {0.7312885150511907, 
      1.8727976198449483`}, {0.7286285063315809, 1.8746633083929451`}, {
      0.7259667397976444, 1.876524474326097}, {0.7233032245605387, 
      1.87838112010674}, {0.7206379697110986, 1.8802332482093935`}, {
      0.7179709843198568, 1.882080861120691}, {0.715302277437066, 
      1.8839239613393117`}, {0.7126318580927199, 1.8857625513759135`}, {
      0.7099597352965751, 1.8875966337530647`}, {0.7072859180381726, 
      1.8894262110051767`}, {0.7046104152868601, 1.891251285678437}, {
      0.7019332359918128, 1.8930718603307415`}, {0.6992543890820566, 
      1.8948879375316288`}, {0.6965738834664893, 1.896699519862213}, {
      0.6938917280339032, 1.8985066099151175`}, {0.691207931653007, 
      1.9003092102944092`}, {0.6885225031724482, 1.9021073236155324`}, {
      0.6858354514208358, 1.903900952505243}, {0.6831467852067622, 
      1.905690099601544}, {0.6804565133188263, 1.9074747675536192`}, {
      0.6777646445256557, 1.9092549590217693`}, {0.6750711875759297, 
      1.9110306766773464`}, {0.6723761511984021, 1.91280192320269}, {
      0.6696795441019239, 1.9145687012910628`}, {0.6669813749754662, 
      1.9163310136465863`}, {0.6642816524881435, 1.9180888629841775`}, {
      0.6615803852892367, 1.9198422520294847`}, {0.6588775820082161, 
      1.9215911835188246`}, {0.6561732512547649, 1.9233356601991192`}, {
      0.6534674016188023, 1.9250756848278325`}, {0.6507600416705072, 
      1.9268112601729086`}, {0.6480511799603412, 1.9285423890127082`}, {
      0.645340825019073, 1.9302690741359472`}, {0.6426289853578008, 
      1.9319913183416342`}, {0.6399156694679771, 1.9337091244390088`}, {
      0.6372008858214318, 1.9354224952474803`}, {0.6344846428703963, 
      1.937131433596566}, {0.6317669490475274, 1.9388359423258297`}, {
      0.6290478127659311, 1.940536024284822}, {0.6263272424191867, 
      1.9422316823330181`}, {0.6236052463813709, 1.9439229193397585`}, {
      0.6208818330070821, 1.945609738184188}, {0.6181570106314643, 
      1.9472921417551956`}, {0.6154307875702318, 1.9489701329513551`}, {
      0.6127031721196933, 1.9506437146808655`}, {0.6099741725567761, 
      1.952312889861491}, {0.6072437971390511, 1.953977661420502}, {
      0.6045120541047568, 1.9556380322946163`}, {0.6017789516728245, 
      1.9572940054299404`}, {0.5990444980429023, 1.9589455837819105`}, {
      0.5963087013953803, 1.9605927703152344`}, {0.593571569891415, 
      1.9622355680038333`}, {0.5908331116729547, 1.9638739798307838`}, {
      0.5880933348627637, 1.9655080087882602`}, {0.5853522475644476, 
      1.9671376578774769`}, {0.5826098578624783, 1.9687629301086313`}, {
      0.5798661738222192, 1.9703838285008466`}, {0.5771212034899498, 
      1.9720003560821144`}, {0.5743749548928914, 1.973612515889239}, {
      0.5716274360392319, 1.9752203109677797`}, {0.5688786549181517, 
      1.9768237443719956`}, {0.5661286194998482, 1.9784228191647886`}, {
      0.5633773377355616, 1.9800175384176482`}, {0.5606248175576006, 
      1.9816079052105957`}, {0.5578710668793672, 1.9831939226321285`}, {
      0.5551160935953826, 1.9847755937791647`}, {0.552359905581313, 
      1.9863529217569886`}, {0.5496025106939948, 1.9879259096791955`}, {
      0.5468439167714604, 1.9894945606676364`}, {0.544084131632964, 
      1.991058877852365}, {0.5413231630790073, 1.992618864371582}, {
      0.5385610188913654, 1.9941745233715822`}, {0.5357977068331125, 
      1.9957258580066992`}, {0.5330332346486475, 1.9972728714392534`}, {
      0.5302676100637208, 1.9988155668394971`}, {0.5275008407854594, 
      2.0003539473855625`}, {0.5247329345023937, 2.001888016263407}, {
      0.5219638988844825, 2.003417776666763}, {0.5191937415831402, 
      2.004943231797082}, {0.5164224702312626, 2.0064643848634836`}, {
      0.5136500924432528, 2.007981239082704}, {0.5108766158150473, 
      2.009493797679043}, {0.5081020479241432, 2.0110020638843116`}, {
      0.5053263963296233, 2.0125060409377817`}, {0.5025496685721834, 
      2.0140057320861335`}, {0.4997718721741582, 2.0155011405834045`}, {
      0.49699301463954776`, 2.0169922696909386`}, {0.49421310345404407`, 
      2.018479122677334}, {0.4914321460850574, 2.019961702818395}, {
      0.48865014998174305`, 2.0214400133970782`}, {0.4858671225750276, 
      2.0229140577034443`}, {0.4830830712776358, 2.024383839034608}, {
      0.4802980034841169, 2.025849360694686}, {0.4775119265708716, 
      2.02731062599475}, {0.4747248478961786, 2.0287676382527753`}, {
      0.4719367748002212, 2.030220400793591}, {0.46914771460511445`, 
      2.0316689169488322`}, {0.4663576746149314, 2.0331131900568895`}, {
      0.46356666211573033`, 2.034553223462862}, {0.4607746843755815, 
      2.035989020518506}, {0.4579817486445939, 2.0374205845821893`}, {
      0.45518786215494234`, 2.038847919018841}, {0.45239303212089416`, 
      2.0402710271999043`}, {0.4495972657388364, 2.0416899125032875`}, {
      0.4468005701873027, 2.0431045783133177`}, {0.4440029526270003, 
      2.044515028020692}, {0.4412044202008371, 2.0459212650224305`}, {
      0.4384049800339488, 2.0473232927218294`}, {0.43560463923372583`, 
      2.048721114528414}, {0.4328034048898407, 2.0501147338578902`}, {
      0.4300012840742749, 2.051504154132101}, {0.4271982838413463, 
      2.0528893787789766`}, {0.424394411227736, 2.054270411232491}, {
      0.42158967325251606`, 2.0556472549326132`}, {0.4187840769171762, 
      2.057019913325264}, {0.4159776292056514, 2.058388389862267}, {
      0.41317033708434914`, 2.0597526880013057`}, {0.41036220750217656`, 
      2.061112811205878}, {0.40755324739056786`, 2.0624687629452487`}, {
      0.40474346366351177`, 2.0638205466944064`}, {0.4019328632175787, 
      2.065168165934018}, {0.39912145293194823`, 2.0665116241503836`}, {
      0.3963092396684365, 2.0678509248353922`}, {0.3934962302715237, 
      2.0691860714864783`}, {0.3906824315683813, 2.0705170676065756`}, {
      0.38786785036889976`, 2.071843916704075}, {0.3850524934657159, 
      2.07316662229278}, {0.38223636763424035`, 2.074485187891862}, {
      0.379419479632685, 2.0757996170258193`}, {0.3766018362020908, 
      2.07710991322443}, {0.37378344406635494`, 2.078416080022712}, {
      0.3709643099322586, 2.079718120960879}, {0.3681444404894945, 
      2.0810160395842976`}, {0.36532384241069454`, 2.0823098394434445`}, {
      0.36250252235145725`, 2.083599524093864}, {0.35968048695037547`, 
      2.0848850970961252`}, {0.356857742829064, 2.086166562015782}, {
      0.3540342965921872, 2.0874439224233274`}, {0.35121015482748674`, 
      2.0887171818941552`}, {0.34838532410580897`, 2.0899863440085165`}, {
      0.345559810981133, 2.091251412351478}, {0.342733621990598, 
      2.0925123905128817`}, {0.33990676365453115`, 2.093769282087303}, {
      0.3370792424764753, 2.09502209067401}, {0.33425106494321655`, 
      2.096270819876922}, {0.3314222375248121, 2.09751547330457}, {
      0.32859276667461784`, 2.098756054570056}, {0.3257626588293164, 
      2.09999256729101}, {0.3229319204089444, 2.1012250150895544`}, {
      0.32010055781692076`, 2.10245340159226}, {0.31726857744007403`, 
      2.1036777304301078`}, {0.31443598564867037`, 2.1048980052384496`}, {
      0.31160278879644127`, 2.106114229656967}, {0.30876899322061135`, 
      2.1073264073296336`}, {0.3059346052419261, 2.108534541904675}, {
      0.30309963116467986`, 2.1097386370345292`}, {0.3002640772767433, 
      2.1109386963758086`}, {0.29742794984959153`, 2.1121347235892607`}, {
      0.29459125513833173`, 2.11332672233973}, {0.2917539993817311, 
      2.1145146962961188`}, {0.2889161888022444, 2.1156986491313488`}, {
      0.28607782960604217`, 2.1168785845223237`}, {0.2832389279830383, 
      2.118054506149891}, {0.28039949010691784`, 2.1192264176988047`}, {
      0.277559522135165, 2.1203943228576856`}, {0.27471903020909083`, 
      2.1215582253189855`}, {0.27187802045386117`, 2.12271812877895}, {
      0.2690364989785244, 2.1238740369375804`}, {0.26619447187603923`, 
      2.1250259534985965`}, {0.26335194522330274`, 2.1261738821694003`}, {
      0.260508925081178, 2.1273178266610393`}, {0.2576654174945221, 
      2.1284577906881688`}, {0.2548214284922138, 2.129593777969017}, {
      0.25197696408718145`, 2.1307257922253475`}, {0.24913203027643094`, 
      2.1318538371824234`}, {0.24628663304107337`, 2.1329779165689717`}, {
      0.24344077834635297`, 2.1340980341171463`}, {0.24059447214167493`, 
      2.1352141935624935`}, {0.23774772036063319`, 2.1363263986439165`}, {
      0.2349005289210383, 2.137434653103638}, {0.23205290372494528`, 
      2.138538960687168}, {0.22920485065868137`, 2.1396393251432655`}, {
      0.22635637559287397`, 2.1407357502239055`}, {0.22350748438247825`, 
      2.141828239684244}, {0.22065818286680522`, 2.1429167972825818`}, {
      0.21780847686954932`, 2.1440014267803322`}, {0.2149583721988164, 
      2.1450821319419853`}, {0.21210787464715133`, 2.1461589165350725`}, {
      0.20925698999156603`, 2.147231784330135}, {0.20640572399356702`, 
      2.1483007391006885`}, {0.2035540823991834, 2.149365784623188}, {
      0.20070207093899456`, 2.150426924676997}, {0.19784969532815785`, 
      2.1514841630443513`}, {0.1949969612664365, 2.152537503510327}, {
      0.19214387443822734`, 2.1535869498628064`}, {0.18929044051258845`, 
      2.154632505892446}, {0.18643666514326707`, 2.1556741753926425`}, {
      0.18358255396872722`, 2.1567119621594997`}, {0.18072811261217744`, 
      2.1577458699917966`}, {0.17787334668159857`, 2.1587759026909543`}, {
      0.17501826176977142`, 2.1598020640610045`}, {0.17216286345430448`, 
      2.160824357908555}, {0.16930715729766163`, 2.1618427880427604`}, {
      0.16645114884718984`, 2.1628573582752875`}, {0.16359484363514679`, 
      2.1638680724202843`}, {0.16073824717872856`, 2.164874934294349}, {
      0.1578813649800974, 2.165877947716498}, {0.15502420252640914`, 
      2.1668771165081337`}, {0.1521667652898411, 2.1678724444930126`}, {
      0.14930905872761954`, 2.1688639354972166`}, {0.14645108828204734`, 
      2.1698515933491196`}, {0.14359285938053154`, 2.170835421879357}, {
      0.14073437743561104`, 2.1718154249207955`}, {0.1378756478449841, 
      2.1727916063085018`}, {0.13501667599153597`, 2.173763969879712}, {
      0.13215746724336636`, 2.1747325194738023`}, {0.1292980269538171, 
      2.1756972589322565`}, {0.1264383604614996, 2.176658192098638}, {
      0.12357847309032235`, 2.177615322818559}, {0.12071837014951854`, 
      2.178568654939649}, {0.1178580569336734, 2.1795181923115274`}, {
      0.11499753872275183`, 2.180463938785773}, {0.11213682078212575`, 
      2.181405898215895}, {0.10927590836260162`, 2.1823440744573004`}, {
      0.10641480670044787`, 2.1832784713672697`}, {0.1035535210174223, 
      2.1842090928049243`}, {0.1006920565207995, 2.1851359426311983`}, {
      0.09783041840339829, 2.1860590247088103`}, {0.09496861184360904, 
      2.1869783429022336`}, {0.09210664200542104, 2.1878939010776683`}, {
      0.08924451403844989, 2.1888057031030126`}, {0.08638223307796482, 
      2.1897137528478336`}, {0.08351980424491597, 2.19061805418334}, {
      0.08065723264596172, 2.191518610982355}, {0.07779452337349603, 
      2.1924154271192844`}, {0.07493168150567561, 2.1933085064700935`}, {
      0.07206871210644723, 2.1941978529122763`}, {0.06920562022557494, 
      2.1950834703248283`}, {0.06634241089866731, 2.1959653625882196`}, {
      0.06347908914720458, 2.196843533584367}, {0.06061565997856593, 
      2.1977179871966066`}, {0.05775212838605655, 2.1985887273096676`}, {
      0.05488849934893484, 2.199455757809644}, {0.052024777832439556`, 
      2.2003190825839685`}, {0.04916096878781687, 2.201178705521385}, {
      0.046297077152347495`, 2.2020346305119225`}, {0.04343310784937377, 
      2.2028868614468684`}, {0.04056906578832669, 2.203735402218742}, {
      0.03770495586475295, 2.204580256721268}, {0.034840782960341966`, 
      2.2054214288493497`}, {0.031976551942952874`, 2.206258922499045}, {
      0.029112267666641514`, 2.2070927415675383`}, {0.02624793497168737, 
      2.207922889953115}, {0.023383558684620522`, 2.2087493715551365`}, {
      0.020519143618248564`, 2.209572190274014}, {0.017654694571683494`, 
      2.2103913500111827`}, {0.014790216330368606`, 2.211206854669078}, {
      0.011925713666105332`, 2.2120187081511076`}, {0.009061191337080086, 
      2.2128269143616284`}, {0.006196654087891078, 2.213631477205921}, {
      0.0033321066495751086`, 2.2144324005901637`}, {0.0004675537396343372, 
      2.2152296884214095`}, {-0.0023969999379369602`, 
      2.2160233446075597`}, {-0.005261549692625668, 
      2.21681337305734}, {-0.008126090847373115, 
      2.217599777680275}, {-0.0109906187385484, 
      2.218382562386667}, {-0.013855128715921719`, 
      2.2191617310875675`}, {-0.016719616142637728`, 
      2.2199372876947554`}, {-0.019584076395188922`, 
      2.220709236120713}, {-0.022448504863389042`, 
      2.221477580278601}, {-0.0253128969503465, 
      2.2222423240822353`}, {-0.02817724807243783, 
      2.2230034714460634`}, {-0.031041553659281153`, 
      2.2237610262851404`}, {-0.033905809153709686`, 
      2.2245149925151058`}, {-0.03677001001174525, 
      2.2252653740521597`}, {-0.03963415170257182, 
      2.2260121748130404`}, {-0.04249822970850908, 
      2.2267553987149995`}, {-0.04536223952498604, 
      2.2274950496757806`}, {-0.04822617666051461, 
      2.228231131613595}, {-0.051090036636663276`, 
      2.2289636484471007`}, {-0.05395381498803074, 
      2.229692604095377}, {-0.056817507262219624`, 
      2.2304180024779034`}, {-0.05968110901981016, 
      2.2311398475145374`}, {-0.06254461583433393, 
      2.2318581431254914`}, {-0.06540802329224767, 
      2.23257289323131}, {-0.06827132699290696, 
      2.233284101752848}, {-0.07113452254854016, 
      2.23399177261125}, {-0.07399760558422215, 
      2.2346959097279253`}, {-0.07686057173784822, 
      2.235396517024528}, {-0.07972341666010796, 
      2.236093598422934}, {-0.0825861360144592, 
      2.236787157845222}, {-0.08544872547710189, 
      2.237477199213647}, {-0.0883111807369521, 
      2.2381637264506242`}, {-0.09117349749561603, 
      2.238846743478703}, {-0.09403567146736397, 
      2.2395262542205496`}, {-0.0968976983791044, 
      2.240202262598922}, {-0.09975957397035802, 
      2.2408747725366522`}, {-0.10262129399323186`, 
      2.2415437879566227`}, {-0.1054828542123934, 
      2.242209312781747}, {-0.10834425040504472`, 
      2.2428713509349483`}, {-0.11120547836089668`, 
      2.2435299063391394`}, {-0.11406653388214312`, 
      2.2441849829172007`}, {-0.11692741278343508`, 
      2.2448365845919613`}, {-0.11978811089185505`, 
      2.245484715286177}, {-0.12264862404689131`, 
      2.2461293789225123`}, {-0.12550894810041216`, 
      2.2467705794235164`}, {-0.1283690789166403, 
      2.247408320711607}, {-0.1312290123721272, 
      2.2480426067090487`}, {-0.1340887443557275, 
      2.2486734413379317`}, {-0.13694827076857344`, 
      2.2493008285201537`}, {-0.13980758752404923`, 
      2.2499247721774003`}, {-0.14266669054776565`, 
      2.2505452762311235`}, {-0.14552557577753444`, 
      2.2511623446025246`}, {-0.14838423916334295`, 
      2.251775981212533}, {-0.1512426766673286, 
      2.2523861899817867`}, {-0.15410088426375354`, 
      2.252992974830615}, {-0.1569588579389792, 
      2.253596339679016}, {-0.15981659369144105`, 
      2.2541962884466415`}, {-0.16267408753162318`, 
      2.2547928250527747`}, {-0.16553133548203303`, 
      2.255385953416313}, {-0.1683883335771761, 
      2.2559756774557482`}, {-0.17124507786353085`, 
      2.2565620010891494`}, {-0.17410156439952326`, 
      2.2571449282341423`}, {-0.17695778925550187`, 
      2.257724462807892}, {-0.17981374851371254`, 
      2.258300608727085}, {-0.1826694382682733, 
      2.2588733699079087`}, {-0.18552485462514934`, 
      2.2594427502660355`}, {-0.18837999370212788`, 
      2.260008753716603}, {-0.19123485162879317`, 
      2.260571384174197}, {-0.1940894245465015, 
      2.261130645552833}, {-0.1969437086083562, 
      2.261686541765939}, {-0.19979769997918273`, 
      2.262239076726335}, {-0.2026513948355038, 
      2.26278825434622}, {-0.20550478936551436`, 
      2.2633340785371496`}, {-0.20835787976905695`, 
      2.2638765532100216`}, {-0.2112106622575967, 
      2.2644156822750565`}, {-0.2140631330541966, 
      2.2649514696417823`}, {-0.21691528839349283`, 
      2.2654839192190153`}, {-0.21976712452166988`, 
      2.2660130349148435`}, {-0.22261863769643594`, 
      2.26653882063661}, {-0.2254698241869983, 
      2.267061280290895}, {-0.2283206802740385, 
      2.2675804177835}, {-0.231171202249688, 
      2.2680962370194298`}, {-0.2340213864175034, 
      2.2686087419028764`}, {-0.23687122909244193`, 
      2.2691179363372025`}, {-0.23972072660083707`, 
      2.2696238242249245`}, {-0.24256987528037388`, 
      2.270126409467695}, {-0.24541867148006472`, 
      2.2706256959662894`}, {-0.24826711156022468`, 
      2.2711216876205857`}, {-0.25111519189244735`, 
      2.2716143883295508`}, {-0.2539629088595803, 
      2.2721038019912236`}, {-0.25681025885570086`, 
      2.272589932502699}, {-0.2596572382860919, 
      2.2730727837601115`}, {-0.2625038435672173, 
      2.2735523596586193`}, {-0.26535007112669806`, 
      2.2740286640923895`}, {-0.2681959174032879, 
      2.274501700954581}, {-0.2710413788468491, 
      2.274971474137329}, {-0.2738864519183284, 
      2.2754379875317294`}, {-0.27673113308973285`, 
      2.275901245027825}, {-0.27957541884410597`, 
      2.2763612505145874`}, {-0.28241930567550333`, 
      2.276818007879903}, {-0.2852627900889689, 
      2.277271521010557}, {-0.2881058686005108, 
      2.27772179379222}, {-0.2909485377370776, 
      2.2781688301094305`}, {-0.29379079403653413`, 
      2.278612633845581}, {-0.29663263404763796`, 
      2.279053208882904}, {-0.2994740543300153, 
      2.279490559102454}, {-0.3023150514541372, 
      2.2799246883840967`}, {-0.3051556220012958, 
      2.280355600606491}, {-0.30799576256358074`, 
      2.280783299647076}, {-0.31083546974385523`, 
      2.2812077893820564`}, {-0.31367474015573243`, 
      2.2816290736863865`}, {-0.3165135704235519, 
      2.282047156433758}, {-0.3193519571823558, 
      2.2824620414965837`}, {-0.3221898970778656, 
      2.2828737327459843`}, {-0.3250273867664582, 
      2.2832822340517738`}, {-0.32786442291514256`, 
      2.2836875492824458`}, {-0.3307010022015363, 
      2.284089682305159}, {-0.3335371213138421, 
      2.2844886369857225`}, {-0.33637277695082446`, 
      2.2848844171885836`}, {-0.33920796582178603`, 
      2.285277026776813}, {-0.3420426846465446, 
      2.2856664696120905`}, {-0.34487693015540954`, 
      2.2860527495546927`}, {-0.34771069908915864`, 
      2.286435870463478}, {-0.3505439881990147, 
      2.286815836195874}, {-0.35337679424662255`, 
      2.2871926506078637`}, {-0.35620911400402566`, 
      2.287566317553972}, {-0.35904094425364314`, 
      2.2879368408872525`}, {-0.3618722817882465, 
      2.288304224459274}, {-0.3647031234109366, 
      2.288668472120108}, {-0.3675334659351206, 
      2.2890295877183147`}, {-0.3703633061844891, 
      2.289387575100931}, {-0.3731926409929929, 
      2.289742438113456}, {-0.3760214672048201, 
      2.2900941805998403`}, {-0.3788497816743734, 
      2.290442806402471}, {-0.38167758126624696`, 
      2.2907883193621603`}, {-0.3845048628552037, 
      2.2911307233181324`}, {-0.38733162332615245`, 
      2.2914700221080104`}, {-0.39015785957412513`, 
      2.291806219567805}, {-0.3929835685042541, 
      2.2921393195319006`}, {-0.3958087470317493, 
      2.292469325833044}, {-0.3986333920818757, 
      2.292796242302332}, {-0.4014575005899307, 
      2.293120072769198}, {-0.40428106950122117`, 
      2.2934408210614006`}, {-0.4071040957710414, 
      2.293758491005012}, {-0.4099265763646502, 
      2.294073086424405}, {-0.41274850825724835`, 
      2.2943846111422417`}, {-0.4155698884339565, 
      2.2946930689794605`}, {-0.4183907138897922, 
      2.294998463755266}, {-0.42121098162964804`, 
      2.2953007992871153`}, {-0.4240306886682688, 
      2.295600079390708}, {-0.4268498320302295, 
      2.2958963078799735`}, {-0.42966840874991286`, 
      2.296189488567059}, {-0.432486415871487, 
      2.296479625262319}, {-0.4353038504488834, 
      2.2967667217743037`}, {-0.4381207095457746, 
      2.297050781909747}, {-0.44093699023555194`, 
      2.297331809473556}, {-0.4437526896013036, 
      2.297609808268798}, {-0.4465678047357924, 
      2.2978847820966917`}, {-0.44938233274143374`, 
      2.2981567347565943`}, {-0.45219627073027363`, 
      2.298425670045991}, {-0.4550096158239666, 
      2.2986915917604844`}, {-0.4578223651537538, 
      2.298954503693783}, {-0.46063451586044113`, 
      2.2992144096376896`}, {-0.4634460650943772, 
      2.299471313382093}, {-0.46625701001543157`, 
      2.299725218714954}, {-0.469067347792973, 
      2.2999761294222973`}, {-0.47187707560584746`, 
      2.300224049288199}, {-0.47468619064235656`, 
      2.3004689820947775`}, {-0.4774946901002358, 
      2.3007109316221817`}, {-0.48030257118663283`, 
      2.300949901648582}, {-0.48310983111808575`, 
      2.3011858959501588`}, {-0.4859164671205017, 
      2.301418918301092}, {-0.488722476429135, 
      2.3016489724735516`}, {-0.491527856288566, 
      2.3018760622376875`}, {-0.49433260395267903`, 
      2.3021001913616184`}, {-0.4971367166846414, 
      2.3023213636114224`}, {-0.4999401917568817, 
      2.302539582751127}, {-0.5027430264510686, 
      2.3027548525426984`}, {-0.5055452180580893, 
      2.302967176746033}, {-0.508346763878028, 
      2.303176559118947}, {-0.5111476612201452, 
      2.3033830034171645`}, {-0.5139479074028559, 
      2.3035865133943116`}, {-0.5167474997537086, 
      2.3037870928019037`}, {-0.519546435609364, 
      2.3039847453893376`}, {-0.5223447123155738, 
      2.3041794749038806`}, {-0.5251423272271598, 
      2.304371285090662}, {-0.5279392777079926, 
      2.3045601796926634`}, {-0.5307355611309706, 
      2.304746162450709}, {-0.5335311748779991, 
      2.3049292371034573`}, {-0.5363261163399689, 
      2.30510940738739}, {-0.5391203829167358, 
      2.3052866770368046`}, {-0.5419139720170997, 
      2.3054610497838044`}, {-0.5447068810587832, 
      2.305632529358289}, {-0.5474991074684112, 
      2.3058011194879464`}, {-0.5502906486814902, 
      2.305966823898243}, {-0.5530815021423869, 
      2.306129646312416}, {-0.5558716653043082, 
      2.3062895904514615`}, {-0.5586611356292798, 
      2.3064466600341302`}, {-0.561449910588126, 
      2.3066008587769145`}, {-0.564237987660449, 
      2.3067521903940422`}, {-0.567025364334608, 
      2.3069006585974674`}, {-0.5698120381076991, 
      2.307046267096861}, {-0.5725980064855343, 
      2.3071890195996025`}, {-0.5753832669826213, 
      2.3073289198107734`}, {-0.5781678171221429, 
      2.3074659714331456`}, {-0.5809516544359367, 
      2.307600178167175}, {-0.5837347764644742, 
      2.3077315437109935`}, {-0.5865171807568416, 
      2.3078600717603988`}, {-0.5892988648707179, 
      2.3079857660088483`}, {-0.5920798263723559, 
      2.308108630147449}, {-0.5948600628365615, 
      2.308228667864952}, {-0.597639571846673, 
      2.308345882847741}, {-0.600418350994542, 
      2.308460278779827}, {-0.603196397880512, 
      2.3085718593428393`}, {-0.6059737101133994, 
      2.3086806282160177`}, {-0.6087502853104725, 
      2.3087865890762047`}, {-0.611526121097432, 
      2.3088897455978374`}, {-0.6143012151083909, 
      2.3089901014529404`}, {-0.6170755649858543, 
      2.3090876603111172`}, {-0.6198491683806998, 
      2.3091824258395435`}, {-0.6226220229521574, 
      2.3092744017029587`}, {-0.6253941263677896, 
      2.3093635915636592`}, {-0.6281654763034714, 
      2.30944999908149}, {-0.630936070443371, 
      2.3095336279138374`}, {-0.6337059064799294, 
      2.3096144817156232`}, {-0.6364749821138413, 
      2.309692564139295}, {-0.6392432950540351, 
      2.3097678788348195`}, {-0.6420108430176528, 
      2.309840429449677}, {-0.6447776237300311, 
      2.3099102196288523`}, {-0.6475436349246816, 
      2.3099772530148277`}, {-0.6503088743432708, 
      2.3100415332475763`}, {-0.6530733397356011, 
      2.3101030639645557`}, {-0.6558370288595912, 
      2.3101618488006994`}, {-0.6585999394812563, 
      2.3102178913884113`}, {-0.661362069374689, 
      2.3102711953575574`}, {-0.6641234163220401, 
      2.3103217643354608`}, {-0.6668839781134986, 
      2.3103696019468924`}, {-0.669643752547273, 
      2.310414711814066}, {-0.6724027374295718, 
      2.3104570975566316`}, {-0.675160930574584, 
      2.3104967627916673`}, {-0.6779183298044605, 
      2.3105337111336737`}, {-0.6806749329492943, 
      2.3105679461945674`}, {-0.6834307378471014, 
      2.3105994715836737`}, {-0.6861857423438021, 
      2.3106282909077214`}, {-0.6889399442932018, 
      2.3106544077708344`}, {-0.6916933415569717, 
      2.3106778257745275`}, {-0.69444593200463, 
      2.3106985485176983`}, {-0.6971977135135229, 
      2.3107165795966216`}, {-0.6999486839688055, 
      2.3107319226049428`}, {-0.7026988412634232, 
      2.3107445811336724`}, {-0.7054481832980926, 
      2.3107545587711797`}, {-0.7081967079812825, 
      2.3107618591031853`}, {-0.7109444132291959, 
      2.310766485712757}, {-0.71369129696575, 
      2.3107684421803016`}, {-0.7164373571225585, 
      2.3107677320835616`}, {-0.7191825916389125, 
      2.310764358997606}, {-0.7219269984617618, 
      2.310758326494828}, {-0.7246705755456961, 
      2.310749638144935}, {-0.7274133208529272, 
      2.310738297514947}, {-0.7301552323532693, 
      2.3107243081691875`}, {-0.7328963080241213, 
      2.3107076736692798`}, {-0.7356365458504481, 
      2.31068839757414}, {-0.738375943824762, 
      2.3106664834399724`}, {-0.7411144999471043, 
      2.310641934820263}, {-0.7438522122250272, 
      2.310614755265774}, {-0.7465890786735749, 
      2.3105849483245398`}, {-0.749325097315266, 
      2.310552517541859}, {-0.7520602661800745, 
      2.3105174664602908`}, {-0.7547945833054119, 
      2.3104797986196486`}, {-0.7575280467361092, 
      2.3104395175569956`}, {-0.7602606545243981, 
      2.3103966268066385`}, {-0.7629924047298933, 
      2.310351129900123}, {-0.7657232954195744, 
      2.3103030303662275`}, {-0.7684533246677676, 
      2.31025233173096}, {-0.7711824905561279, 
      2.3101990375175494`}, {-0.7739107911736206, 
      2.3101431512464448`}, {-0.776638224616504, 
      2.3100846764353067`}, {-0.7793647889883109, 
      2.310023616599004}, {-0.7820904823998308, 
      2.309959975249608}, {-0.7848153029690925, 
      2.309893755896388}, {-0.7875392488213455, 
      2.309824962045806}, {-0.7902623180890427, 
      2.3097535972015124`}, {-0.7929845089118224, 
      2.3096796648643405`}, {-0.7957058194364905, 
      2.3096031685323024`}, {-0.7984262478170033, 
      2.309524111700583}, {-0.8011457922144491, 
      2.309442497861537}, {-0.803864450797031, 
      2.3093583305046828`}, {-0.806582221740049, 
      2.3092716131166977`}, {-0.8092991032258828, 
      2.309182349181415}, {-0.8120150934439742, 
      2.3090905421798187`}, {-0.814730190590809, 
      2.3089961955900367`}, {-0.8174443928699003, 
      2.30889931288734}, {-0.8201576984917707, 
      2.3087998975441355`}, {-0.8228701056739347, 
      2.3086979530299634`}, {-0.8255816126408818, 
      2.3085934828114913`}, {-0.8282922176240589, 
      2.3084864903525104`}, {-0.8310019188618527, 
      2.3083769791139317`}, {-0.833710714599573, 
      2.308264952553781}, {-0.8364186030894349, 
      2.308150414127195}, {-0.8391255825905423, 
      2.3080333672864177`}, {-0.8418316513688696, 
      2.3079138154807946`}, {-0.8445368076972459, 
      2.30779176215677}, {-0.847241049855337, 
      2.307667210757883}, {-0.8499443761296284, 
      2.3075401647247618`}, {-0.8526467848134085, 
      2.3074106274951216`}, {-0.8553482742067515, 
      2.30727860250376}, {-0.8580488426165006, 
      2.3071440931825524`}, {-0.8607484883562506, 
      2.3070071029604486`}, {-0.8634472097463315, 
      2.306867635263469}, {-0.8661450051137911, 
      2.3067256935147}, {-0.8688418727923787, 
      2.306581281134292}, {-0.8715378111225279, 
      2.306434401539453}, {-0.8742328184513402, 
      2.3062850581444483`}, {-0.8769268931325676, 
      2.3061332543605926`}, {-0.8796200335265965, 
      2.3059789935962494`}, {-0.8823122380004308, 
      2.3058222792568266`}, {-0.8850035049276754, 
      2.305663114744773}, {-0.8876938326885193, 
      2.3055015034595736`}, {-0.890383219669719, 
      2.305337448797748}, {-0.8930716642645825, 
      2.305170954152845}, {-0.895759164872952, 
      2.3050020229154407`}, {-0.8984457199011882, 
      2.3048306584731333`}, {-0.9011313277621531, 
      2.304656864210542}, {-0.9038159868751942, 
      2.3044806435093013`}, {-0.9064996956661276, 
      2.3043019997480596`}, {-0.9091824525672224, 
      2.304120936302474}, {-0.9118642560171832, 
      2.3039374565452087`}, {-0.9145451044611351, 
      2.3037515638459314`}, {-0.9172249963506064, 
      2.303563261571309}, {-0.919903930143513, 
      2.3033725530850058`}, {-0.922581904304142, 
      2.303179441747679}, {-0.9252589173031355, 
      2.3029839309169775`}, {-0.9279349676174746, 
      2.3027860239475366`}, {-0.9306100537304632, 
      2.3025857241909766`}, {-0.933284174131712, 
      2.302383034995899}, {-0.9359573273171224, 
      2.3021779597078837`}, {-0.9386295117888706, 
      2.3019705016694862`}, {-0.9413007260553913, 
      2.3017606642202346`}, {-0.9439709686313624, 
      2.301548450696626}, {-0.9466402380376886, 
      2.301333864432125}, {-0.9493085328014856, 
      2.3011169087571597`}, {-0.9519758514560644, 
      2.300897586999119}, {-0.9546421925409154, 
      2.3006759024823515`}, {-0.9573075546016926, 
      2.30045185852816}, {-0.9599719361901978, 
      2.3002254584548}, {-0.9626353358643653, 
      2.299996705577479}, {-0.9652977521882457, 
      2.29976560320835}, {-0.9679591837319903, 
      2.2995321546565126`}, {-0.970619629071836, 
      2.2992963632280077`}, {-0.9732790867900891, 
      2.299058232225816}, {-0.9759375554751101, 
      2.298817764949856}, {-0.978595033721298, 
      2.2985749646969804`}, {-0.981251520129075, 
      2.298329834760975}, {-0.9839070133048708, 
      2.2980823784325537`}, {-0.9865615118611076, 
      2.2978325989993595`}, {-0.9892150144161841, 
      2.2975804997459597`}, {-0.9918675195944604, 
      2.2973260839538443`}, {-0.994519026026243, 
      2.2970693549014234`}, {-0.9971695323477691, 
      2.296810315864025}, {-0.9998190372011914, 
      2.2965489701138933`}, {-1.0024675392345628`, 
      2.2962853209201852`}, {-1.0051150371018216`, 
      2.29601937154897}, {-1.0077615294627758`, 
      2.295751125263225}, {-1.0104070149830882`, 
      2.295480585322834}, {-1.0130514923342613`, 
      2.2952077549845873`}, {-1.0156949601936223`, 
      2.294932637502176}, {-1.0183374172443076`, 
      2.294655236126193}, {-1.0209788621752482`, 
      2.294375554104129}, {-1.023619293681155, 
      2.2940935946803704`}, {-1.026258710462503, 
      2.2938093610962}, {-1.028897111225517, 
      2.2935228565897914`}, {-1.0315344946821565`, 
      2.293234084396209}, {-1.034170859550101, 
      2.2929430477474066`}, {-1.0368062045527346`, 
      2.292649749872224}, {-1.0394405284191324`, 
      2.292354193996385}, {-1.0420738298840442`, 
      2.2920563833424983`}, {-1.0447061076878807`, 
      2.2917563211300522`}, {-1.047337360576699, 
      2.291454010575415}, {-1.0499675873021868`, 
      2.291149454891832}, {-1.052596786621649, 
      2.290842657289425}, {-1.0552249572979926`, 
      2.29053362097519}, {-1.0578520980997115`, 
      2.2902223491529945`}, {-1.060478207800873, 
      2.289908845023578}, {-1.0631032851811024`, 
      2.289593111784548}, {-1.065727329025569, 
      2.2892751526303794`}, {-1.068350338124972, 
      2.2889549707524144`}, {-1.0709723112755243`, 
      2.2886325693388585`}, {-1.0735932472789405`, 
      2.2883079515747804`}, {-1.0762131449424206`, 
      2.2879811206421095`}, {-1.078832003078637, 
      2.2876520797196354`}, {-1.081449820505719, 
      2.287320831983006}, {-1.0840665960472398`, 
      2.2869873806047263`}, {-1.086682328532201, 
      2.286651728754156}, {-1.089297016795019, 
      2.28631387959751}, {-1.0919106596755106`, 
      2.285973836297855}, {-1.0945232560188793`, 
      2.2856316020151097`}, {-1.0971348046757006`, 
      2.285287179906042}, {-1.0997453045019079`, 
      2.284940573124269}, {-1.102354754358779, 
      2.2845917848202557`}, {-1.1049631531129214`, 
      2.284240818141312}, {-1.1075704996362585`, 
      2.2838876762315943`}, {-1.110176792806016, 
      2.2835323622321013`}, {-1.1127820315047072`, 
      2.2831748792806756`}, {-1.1153862146201197`, 
      2.282815230512}, {-1.117989341045301, 
      2.2824534190575987`}, {-1.1205914096785452`, 
      2.2820894480458342`}, {-1.1231924194233787`, 
      2.281723320601907}, {-1.1257923691885465`, 
      2.2813550398478557`}, {-1.1283912578879984`, 
      2.280984608902554}, {-1.1309890844408754`, 
      2.2806120308817097`}, {-1.133585847771496, 
      2.2802373088978665`}, {-1.136181546809342, 
      2.2798604460603995`}, {-1.138776180489045, 
      2.279481445475516}, {-1.141369747750374, 
      2.279100310246255}, {-1.1439622475382196`, 
      2.2787170434724846`}, {-1.1465536788025825`, 
      2.2783316482509033`}, {-1.1491440404985585`, 
      2.277944127675037}, {-1.1517333315863254`, 
      2.2775544848352394`}, {-1.1543215510311298`, 
      2.2771627228186913`}, {-1.1569086978032734`, 
      2.2767688447093986`}, {-1.1594947708780998`, 
      2.276372853588193}, {-1.1620797692359808`, 
      2.27597475253273}, {-1.164663691862303, 
      2.275574544617489}, {-1.1672465377474546`, 
      2.2751722329137722`}, {-1.169828305886812, 
      2.2747678204897035`}, {-1.1724089952807268`, 
      2.274361310410229}, {-1.1749886049345122`, 
      2.273952705737115}, {-1.1775671338584297`, 
      2.2735420095289482`}, {-1.1801445810676763`, 
      2.273129224841135}, {-1.1827209455823708`, 
      2.2727143547259003`}, {-1.1852962264275413`, 
      2.272297402232288}, {-1.1878704226331114`, 
      2.271878370406159}, {-1.1904435332338876`, 
      2.2714572622901925`}, {-1.193015557269546, 
      2.2710340809238834`}, {-1.1955864937846192`, 
      2.270608829343543}, {-1.1981563418284835`, 
      2.270181510582299}, {-1.2007251004553459`, 
      2.269752127670094}, {-1.203292768724231, 
      2.2693206836336857`}, {-1.2058593456989681`, 
      2.2688871814966456`}, {-1.2084248304481788`, 
      2.26845162427936}, {-1.2109892220452632`, 
      2.268014014999028}, {-1.2135525195683876`, 
      2.2675743566696633`}, {-1.2161147221004718`, 
      2.267132652302091}, {-1.2186758287291761`, 
      2.26668890490395}, {-1.2212358385468887`, 
      2.2662431174796906`}, {-1.2237947506507127`, 
      2.265795293030576}, {-1.226352564142454, 
      2.2653454345546806`}, {-1.2289092781286075`, 
      2.2648935450468906`}, {-1.231464891720346, 
      2.264439627498903}, {-1.234019404033506, 
      2.2639836848992263`}, {-1.2365728141885763`, 
      2.26352572023318}, {-1.2391251213106849`, 
      2.263065736482894}, {-1.2416763245295863`, 
      2.2626037366273084`}, {-1.2442264229796498`, 
      2.262139723642174}, {-1.246775415799846, 
      2.2616737005000522`}, {-1.249323302133735, 
      2.2612056701703143`}, {-1.251870081129454, 
      2.2607356356191413`}, {-1.2544157519397043`, 
      2.260263599809525}, {-1.2569603137217396`, 
      2.259789565701266}, {-1.2595037656373533`, 
      2.2593135362509758`}, {-1.2620461068528663`, 
      2.2588355144120755`}, {-1.2645873365391147`, 
      2.258355503134796}, {-1.2671274538714374`, 
      2.257873505366178}, {-1.2696664580296644`, 
      2.257389524050072}, {-1.2722043481981036`, 
      2.25690356212714}, {-1.2747411235655295`, 
      2.2564156225348517`}, {-1.2772767833251708`, 
      2.255925708207489}, {-1.2798113266746982`, 
      2.2554338220761427`}, {-1.2823447528162122`, 
      2.2549399670687147`}, {-1.2848770609562312`, 
      2.2544441461099174`}, {-1.2874082503056794`, 
      2.2539463621212734`}, {-1.2899383200798749`, 
      2.2534466180211163`}, {-1.2924672694985173`, 
      2.252944916724591}, {-1.2949950977856763`, 
      2.2524412611436535`}, {-1.2975218041697794`, 
      2.251935654187071}, {-1.3000473878836, 
      2.251428098760423}, {-1.3025718481642459`, 
      2.2509185977661006`}, {-1.3050951842531464`, 
      2.2504071541033066`}, {-1.307617395396042, 
      2.2498937706680566`}, {-1.310138480842971, 
      2.2493784503531793`}, {-1.3126584398482593`, 
      2.2488611960483165`}, {-1.315177271670507, 
      2.248342010639923}, {-1.3176949755725784`, 
      2.2478208970112674`}, {-1.3202115508215888`, 
      2.2472978580424328`}, {-1.3227269966888935`, 
      2.2467728966103166`}, {-1.3252413124500761`, 
      2.2462460155886315`}, {-1.3277544973849371`, 
      2.2457172178479046`}, {-1.3302665507774818`, 
      2.2451865062554797`}, {-1.332777471915909, 
      2.244653883675516}, {-1.3352872600925991`, 
      2.2441193529689896`}, {-1.3377959146041039`, 
      2.2435829169936943`}, {-1.3403034347511331`, 
      2.243044578604241}, {-1.3428098198385445`, 
      2.2425043406520575`}, {-1.3453150691753315`, 
      2.2419622059853923`}, {-1.3478191820746124`, 
      2.241418177449312}, {-1.3503221578536182`, 
      2.2408722578857025`}, {-1.3528239958336823`, 
      2.2403244501332704`}, {-1.3553246953402283`, 
      2.239774757027543}, {-1.3578242557027589`, 
      2.2392231814008694`}, {-1.3603226762548448`, 
      2.2386697260824198`}, {-1.3628199563341132`, 
      2.2381143938981873`}, {-1.3653160952822367`, 
      2.2375571876709883`}, {-1.3678110924449223`, 
      2.2369981102204632`}, {-1.3703049471718995`, 
      2.236437164363077}, {-1.3727976588169102`, 
      2.235874352912119}, {-1.3752892267376964`, 
      2.2353096786777056`}, {-1.37777965029599, 
      2.2347431444667794`}, {-1.3802689288575012`, 
      2.2341747530831095`}, {-1.3827570617919076`, 
      2.2336045073272937`}, {-1.3852440484728432`, 
      2.2330324099967584`}, {-1.3877298882778872`, 
      2.23245846388576}, {-1.3902145805885535`, 
      2.231882671785384}, {-1.392698124790279, 
      2.231305036483548}, {-1.3951805202724135`, 
      2.230725560765001}, {-1.3976617664282078`, 
      2.230144247411325}, {-1.4001418626548037`, 
      2.229561099200934}, {-1.4026208083532228`, 
      2.2289761189090784`}, {-1.4050986029283554`, 
      2.2283893093078424`}, {-1.4075752457889503`, 
      2.2278006731661466`}, {-1.4100507363476034`, 
      2.2272102132497484`}, {-1.412525074020747, 
      2.226617932321243}, {-1.4149982582286396`, 
      2.226023833140064}, {-1.4174702883953545`, 
      2.2254279184624854`}, {-1.4199411639487698`, 
      2.22483019104162}, {-1.422410884320557, 
      2.2242306536274237`}, {-1.424879448946171, 
      2.223629308966694}, {-1.4273468572648387`, 
      2.223026159803071}, {-1.4298131087195496`, 
      2.222421208877041}, {-1.432278202757044, 
      2.2218144589259343`}, {-1.434742138827803, 
      2.221205912683927}, {-1.437204916386038, 
      2.220595572882043}, {-1.4396665348896807`, 
      2.219983442248155}, {-1.4421269938003711`, 
      2.2193695235069844`}, {-1.444586292583449, 
      2.2187538193801033`}, {-1.4470444307079418`, 
      2.218136332585935}, {-1.4495014076465553`, 
      2.2175170658397554`}, {-1.4519572228756632`, 
      2.2168960218536937`}, {-1.4544118758752957`, 
      2.2162732033367343`}, {-1.4568653661291306`, 
      2.215648612994717}, {-1.4593176931244822`, 
      2.2150222535303388`}, {-1.4617688563522908`, 
      2.2143941276431542`}, {-1.4642188553071132`, 
      2.2137642380295772`}, {-1.4666676894871118`, 
      2.2131325873828818`}, {-1.4691153583940446`, 
      2.2124991783932035`}, {-1.4715618615332553`, 
      2.211864013747541}, {-1.4740071984136627`, 
      2.2112270961297567`}, {-1.4764513685477505`, 
      2.2105884282205768`}, {-1.4788943714515579`, 
      2.2099480126975948`}, {-1.4813362066446687`, 
      2.2093058522352718`}, {-1.4837768736502015`, 
      2.2086619495049367`}, {-1.4862163719948, 
      2.208016307174789}, {-1.4886547012086224`, 
      2.207368927909899}, {-1.4910918608253318`, 
      2.2067198143722098`}, {-1.4935278503820861`, 
      2.2060689692205373`}, {-1.4959626694195278`, 
      2.205416395110573}, {-1.4983963174817745`, 
      2.204762094694885}, {-1.5008287941164087`, 
      2.204106070622917}, {-1.503260098874468, 
      2.2034483255409945`}, {-1.5056902313104352`, 
      2.2027888620923206`}, {-1.508119190982229, 
      2.2021276829169816`}, {-1.5105469774511926`, 
      2.2014647906519453`}, {-1.512973590282086, 
      2.200800187931065}, {-1.5153990290430746`, 
      2.2001338773850785`}, {-1.5178232933057203`, 
      2.199465861641611}, {-1.5202463826449715`, 
      2.198796143325176}, {-1.5226682966391532`, 
      2.1981247250571765`}, {-1.5250890348699577`, 
      2.1974516094559067`}, {-1.5275085969224351`, 
      2.1967767991365537`}, {-1.5299269823849828`, 
      2.1961002967111978`}, {-1.5323441908493365`, 
      2.1954221047888156`}, {-1.5347602219105607`, 
      2.1947422259752796`}, {-1.537175075167039, 
      2.194060662873361}, {-1.5395887502204642`, 
      2.1933774180827306`}, {-1.5420012466758293`, 
      2.1926924941999606`}, {-1.544412564141418, 
      2.192005893818526}, {-1.5468227022287944`, 
      2.1913176195288053`}, {-1.5492316605527947`, 
      2.1906276739180837`}, {-1.5516394387315169`, 
      2.1899360595705524`}, {-1.5540460363863116`, 
      2.189242779067312}, {-1.556451453141773, 
      2.1885478349863736`}, {-1.558855688625729, 
      2.187851229902659}, {-1.5612587424692321`, 
      2.187152966388004}, {-1.56366061430655, 
      2.18645304701116}, {-1.5660613037751563`, 
      2.1857514743377933`}, {-1.5684608105157214`, 
      2.185048250930489}, {-1.570859134172103, 
      2.1843433793487517`}, {-1.573256274391337, 
      2.183636862149007}, {-1.5756522308236283`, 
      2.182928701884604}, {-1.578047003122341, 
      2.182218901105815}, {-1.5804405909439907`, 
      2.181507462359839}, {-1.5828329939482335`, 
      2.1807943881908027`}, {-1.585224211797858, 
      2.180079681139761}, {-1.5876142441587766`, 
      2.179363343744701}, {-1.5900030907000149`, 
      2.178645378540542}, {-1.5923907510937039`, 
      2.1779257880591367`}, {-1.5947772250150705`, 
      2.177204574829274}, {-1.5971625121424287`, 
      2.1764817413766813`}, {-1.59954661215717, 
      2.175757290224024}, {-1.6019295247437555`, 
      2.1750312238909086`}, {-1.6043112495897056`, 
      2.1743035448938848`}, {-1.606691786385592, 
      2.173574255746446}, {-1.609071134825029, 
      2.172843358959032}, {-1.6114492946046632`, 
      2.1721108570390304`}, {-1.6138262654241664`, 
      2.171376752490777}, {-1.6162020469862253`, 
      2.170641047815561}, {-1.6185766389965335`, 
      2.1699037455116232`}, {-1.6209500411637827`, 
      2.1691648480741588`}, {-1.6233222531996532`, 
      2.1684243579953204`}, {-1.6256932748188058`, 
      2.1676822777642184`}, {-1.6280631057388726`, 
      2.166938609866923}, {-1.630431745680449, 
      2.1661933567864673`}, {-1.6327991943670839`, 
      2.1654465210028464`}, {-1.635165451525272, 
      2.164698104993022}, {-1.6375305168844447`, 
      2.163948111230922}, {-1.6398943901769614`, 
      2.1631965421874444`}, {-1.6422570711381008`, 
      2.1624434003304573`}, {-1.6446185595060527`, 
      2.161688688124802}, {-1.646978855021909, 
      2.1609324080322936`}, {-1.6493379574296556`, 
      2.1601745625117243`}, {-1.651695866476163, 
      2.1594151540188644`}, {-1.6540525819111784`, 
      2.1586541850064638`}, {-1.6564081034873177`, 
      2.157891657924255}, {-1.658762430960056, 
      2.1571275752189534`}, {-1.6611155640877195`, 
      2.156361939334261}, {-1.6634675026314776`, 
      2.1555947527108668`}, {-1.6658182463553333`, 
      2.154826017786449}, {-1.668167795026116, 
      2.1540557369956783`}, {-1.6705161484134725`, 
      2.1532839127702172`}, {-1.6728633062898588`, 
      2.152510547538725}, {-1.6752092684305315`, 
      2.1517356437268567`}, {-1.6775540346135398`, 
      2.150959203757267}, {-1.6798976046197172`, 
      2.1501812300496113`}, {-1.682239978232673, 
      2.149401725020548}, {-1.684581155238784, 
      2.148620691083741}, {-1.6869211354271865`, 
      2.14783813064986}, {-1.689259918589768, 
      2.147054046126584}, {-1.6915975045211593`, 
      2.146268439918603}, {-1.693933893018725, 
      2.1454813144276184`}, {-1.6962690838825574`, 
      2.1446926720523476`}, {-1.6986030769154663`, 
      2.1439025151885245`}, {-1.7009358719229726`, 
      2.1431108462289017`}, {-1.7032674687132991`, 
      2.1423176675632516`}, {-1.7055978670973628`, 
      2.1415229815783703`}, {-1.7079270668887667`, 
      2.140726790658078}, {-1.7102550679037918`, 
      2.139929097183222}, {-1.712581869961389, 
      2.1391299035316775`}, {-1.7149074728831717`, 
      2.138329212078351}, {-1.7172318764934067`, 
      2.1375270251951823`}, {-1.719555080619007, 
      2.1367233452511454`}, {-1.7218770850895238`, 
      2.1359181746122506`}, {-1.7241978897371382`, 
      2.1351115156415483`}, {-1.726517494396654, 
      2.1343033706991292`}, {-1.7288358989054886`, 
      2.1334937421421274`}, {-1.7311531031036667`, 
      2.1326826323247214`}, {-1.7334691068338108`, 
      2.1318700435981373`}, {-1.735783909941135, 
      2.1310559783106506`}, {-1.7380975122734352`, 
      2.130240438807588}, {-1.7404099136810836`, 
      2.12942342743133}, {-1.7427211140170191`, 
      2.1286049465213117`}, {-1.7450311131367404`, 
      2.127784998414027}, {-1.7473399108982979`, 
      2.1269635854430287`}, {-1.7496475071622863`, 
      2.1261407099389316`}, {-1.7519539017918369`, 
      2.1253163742294143`}, {-1.754259094652609, 
      2.1244905806392222`}, {-1.756563085612784, 
      2.1236633314901683`}, {-1.7588658745430559`, 
      2.122834629101136}, {-1.7611674613166248`, 
      2.1220044757880814`}, {-1.763467845809189, 
      2.121172873864035}, {-1.7657670278989375`, 
      2.1203398256391046`}, {-1.7680650074665418`, 
      2.1195053334204763`}, {-1.7703617843951494`, 
      2.118669399512418}, {-1.7726573585703753`, 
      2.1178320262162798`}, {-1.7749517298802955`, 
      2.1169932158304983`}, {-1.777244898215438, 
      2.1161529706505973`}, {-1.7795368634687772`, 
      2.11531129296919}, {-1.7818276255357244`, 
      2.1144681850759826`}, {-1.7841171843141221`, 
      2.1136236492577742`}, {-1.7864055397042358`, 
      2.112777687798461}, {-1.7886926916087462`, 
      2.1119303029790384`}, {-1.7909786399327428`, 
      2.111081497077601}, {-1.7932633845837156`, 
      2.1102312723693473`}, {-1.7955469254715484`, 
      2.109379631126581}, {-1.7978292625085108`, 
      2.1085265756187126`}, {-1.800110395609252, 
      2.107672108112263}, {-1.8023903246907915`, 
      2.106816230870865}, {-1.8046690496725144`, 
      2.105958946155264}, {-1.806946570476162, 
      2.105100256223324}, {-1.8092228870258256`, 
      2.104240163330026}, {-1.811497999247939, 
      2.1033786697274715`}, {-1.8137719070712712`, 
      2.102515777664886}, {-1.8160446104269197`, 
      2.1016514893886202`}, {-1.8183161092483024`, 
      2.100785807142152}, {-1.8205864034711512`, 
      2.099918733166089}, {-1.8228554930335048`, 
      2.0990502696981714`}, {-1.825123377875701, 
      2.0981804189732736`}, {-1.8273900579403703`, 
      2.097309183223406}, {-1.8296555331724285`, 
      2.0964365646777186`}, {-1.8319198035190694`, 
      2.0955625655625023`}, {-1.834182868929758, 
      2.094687188101192}, {-1.8364447293562236`, 
      2.093810434514367}, {-1.8387053847524526`, 
      2.0929323070197565`}, {-1.8409648350746812`, 
      2.0920528078322382`}, {-1.8432230802813891`, 
      2.0911719391638437`}, {-1.8454801203332918`, 
      2.0902897032237595`}, {-1.8477359551933341`, 
      2.089406102218329}, {-1.8499905848266833`, 
      2.0885211383510547`}, {-1.8522440092007217`, 
      2.0876348138226017`}, {-1.8544962282850401`, 
      2.0867471308307994`}, {-1.8567472420514308`, 
      2.0858580915706435`}, {-1.8589970504738809`, 
      2.0849676982342986`}, {-1.8612456535285653`, 
      2.0840759530111}, {-1.8634930511938397`, 
      2.0831828580875578`}, {-1.8657392434502342`, 
      2.0822884156473567`}, {-1.8679842302804464`, 
      2.0813926278713604`}, {-1.8702280116693337`, 
      2.0804954969376124`}, {-1.8724705876039085`, 
      2.0795970250213403`}, {-1.8747119580733291`, 
      2.0786972142949556`}, {-1.876952123068895, 
      2.0777960669280584`}, {-1.879191082584039, 
      2.0768935850874386`}, {-1.8814288366143206`, 
      2.075989770937078}, {-1.8836653851574199`, 
      2.075084626638154}, {-1.8859007282131302`, 
      2.0741781543490405`}, {-1.888134865783352, 
      2.0732703562253105`}, {-1.890367797872086, 
      2.0723612344197395`}, {-1.8925995244854266`, 
      2.071450791082307}, {-1.894830045631555, 
      2.0705390283601983`}, {-1.8970593613207332`, 
      2.0696259483978094`}, {-1.8992874715652968`, 
      2.0687115533367466`}, {-1.901514376379649, 
      2.06779584531583}, {-1.9037400757802538`, 
      2.066878826471096}, {-1.9059645697856293`, 
      2.0659604989358}, {-1.9081878584163416`, 
      2.0650408648404173`}, {-1.910409941694998, 
      2.0641199263126473`}, {-1.9126308196462405`, 
      2.0631976854774154`}, {-1.9148504922967398`, 
      2.0622741444568744`}, {-1.9170689596751882`, 
      2.0613493053704084`}, {-1.9192862218122937`, 
      2.0604231703346345`}, {-1.9215022787407734`, 
      2.059495741463405}, {-1.923717130495347, 
      2.05856702086781}, {-1.9259307771127308`, 
      2.05763701065618}, {-1.9281432186316305`, 
      2.056705712934088}, {-1.9303544550927363`, 
      2.055773129804353}, {-1.9325644865387148`, 
      2.0548392633670405`}, {-1.9347733130142042`, 
      2.0539041157194666`}, {-1.9369809345658073`, 
      2.0529676889561994`}, {-1.9391873512420852`, 
      2.0520299851690624`}, {-1.9413925630935511`, 
      2.0510910064471366`}, {-1.9435965701726643`, 
      2.050150754876762}, {-1.9457993725338234`, 
      2.0492092325415414`}, {-1.948000970233361, 
      2.0482664415223426`}, {-1.9502013633295363`, 
      2.0473223838972996`}, {-1.9524005518825303`, 
      2.0463770617418167`}, {-1.9545985359544382`, 
      2.04543047712857}, {-1.9567953156092646`, 
      2.0444826321275107`}, {-1.958990890912916, 
      2.043533528805866}, {-1.9611852619331955`, 
      2.0425831692281426`}, {-1.9633784287397973`, 
      2.0416315554561306`}, {-1.965570391404299, 
      2.040678689548903}, {-1.9677611500001566`, 
      2.03972457356282}, {-1.9699507046026985`, 
      2.0387692095515315`}, {-1.9721390552891187`, 
      2.037812599565979}, {-1.9743262021384718`, 
      2.0368547456543977`}, {-1.9765121452316659`, 
      2.035895649862321}, {-1.9786968846514574`, 
      2.03493531423258}, {-1.9808804204824448`, 
      2.0339737408053082`}, {-1.9830627528110625`, 
      2.033010931617944}, {-1.9852438817255749`, 
      2.032046888705232}, {-1.9874238073160706`, 
      2.031081614099225}, {-1.9896025296744566`, 
      2.0301151098292896`}, {-1.991780048894452, 
      2.0291473779221048`}, {-1.9939563650715824`, 
      2.0281784204016673`}, {-1.9961314783031738`, 
      2.0272082392892927`}, {-1.998305388688347, 
      2.0262368366036183`}, {-2.000478096328012, 
      2.0252642143606057`}, {-2.002649601324861, 
      2.0242903745735434`}, {-2.004819903783363, 
      2.0233153192530486`}, {-2.00698900380976, 
      2.0223390504070706`}, {-2.0091569015120583`, 
      2.0213615700408933`}, {-2.011323597000024, 
      2.0203828801571366`}, {-2.0134890903851774`, 
      2.0194029827557607`}, {-2.015653381780787, 
      2.0184218798340665`}, {-2.0178164713018636`, 
      2.0174395733867}, {-2.0199783590651554`, 
      2.0164560654056536`}, {-2.022139045189141, 
      2.015471357880269}, {-2.0242985297940246`, 
      2.014485452797241}, {-2.02645681300173, 
      2.0134983521406173`}, {-2.028613894935895, 
      2.012510057891803}, {-2.0307697757218666`, 
      2.0115205720295632`}, {-2.032924455486693, 
      2.0105298965300245`}, {-2.0350779343591214`, 
      2.0095380333666784`}, {-2.0372302124695887`, 
      2.008544984510383}, {-2.0393812899502186`, 
      2.0075507519293665`}, {-2.041531166934815, 
      2.0065553375892287`}, {-2.0436798435588566`, 
      2.0055587434529447`}, {-2.0458273199594914`, 
      2.0045609714808665`}, {-2.0479735962755314`, 
      2.003562023630726}, {-2.050118672647446, 
      2.002561901857637}, {-2.052262549217358, 
      2.0015606081140986`}, {-2.054405226129037, 
      2.0005581443499976`}, {-2.0565467035278937`, 
      1.99955451251261}, {-2.058686981560976, 
      1.9985497145466047`}, {-2.060826060376962, 
      1.9975437523940456`}, {-2.062963940126156, 
      1.9965366279943941`}, {-2.0651006209604805`, 
      1.9955283432845121`}, {-2.067236103033474, 
      1.9945189001986638`}, {-2.0693703865002835`, 
      1.993508300668519}, {-2.0715034715176595`, 
      1.9924965466231552`}, {-2.0736353582439513`, 
      1.99148363998906}, {-2.0757660468391013`, 
      1.9904695826901342`}, {-2.077895537464639, 
      1.989454376647694}, {-2.0800238302836758`, 
      1.988438023780474}, {-2.0821509254609016`, 
      1.9874205260046287`}, {-2.084276823162577, 
      1.9864018852337364`}, {-2.0864015235565283`, 
      1.985382103378801}, {-2.0885250268121442`, 
      1.9843611823482545`}, {-2.0906473331003688`, 
      1.98333912404796}, {-2.0927684425936968`, 
      1.982315930381214}, {-2.094888355466168, 
      1.9812916032487486`}, {-2.0970070718933633`, 
      1.9802661445487353`}, {-2.099124592052397, 
      1.9792395561767862`}, {-2.101240916121915, 
      1.978211840025957}, {-2.103356044282086, 
      1.97718299798675}, {-2.105469976714599, 
      1.9761530319471161`}, {-2.1075827136026573`, 
      1.9751219437924579`}, {-2.1096942551309734`, 
      1.974089735405632}, {-2.111804601485763, 
      1.973056408666951}, {-2.1139137528547423`, 
      1.9720219654541877`}, {-2.116021709427119, 
      1.9709864076425758`}, {-2.1181284713935917`, 
      1.9699497371048136`}, {-2.1202340389463408`, 
      1.9689119557110661`}, {-2.122338412279026, 
      1.9678730653289682`}, {-2.124441591586781, 
      1.9668330678236265`}, {-2.1265435770662067`, 
      1.9657919650576223`}, {-2.128644368915369, 
      1.964749758891014}, {-2.1307439673337907`, 
      1.9637064511813402`}, {-2.132842372522449, 
      1.9626620437836215`}, {-2.1349395846837695`, 
      1.9616165385503634`}, {-2.1370356040216207`, 
      1.9605699373315593`}, {-2.13913043074131, 
      1.9595222419746923`}, {-2.141224065049578, 
      1.9584734543247386`}, {-2.1433165071545943`, 
      1.9574235762241696`}, {-2.1454077572659522`, 
      1.9563726095129546`}, {-2.1474978155946634`, 
      1.955320556028563}, {-2.1495866823531538`, 
      1.9542674176059676`}, {-2.1516743577552586`, 
      1.9532131960776469`}, {-2.1537608420162164`, 
      1.9521578932735877`}, {-2.1558461353526654`, 
      1.9511015110212873`}, {-2.157930237982639, 
      1.9500440511457564`}, {-2.1600131501255584`, 
      1.9489855154695221`}, {-2.1620948720022315`, 
      1.9479259058126297`}, {-2.1641754038348453`, 
      1.9468652239926458`}, {-2.166254745846962, 
      1.945803471824661}, {-2.1683328982635137`, 
      1.944740651121292}, {-2.170409861310799, 
      1.9436767636926844`}, {-2.172485635216477, 
      1.9426118113465156`}, {-2.174560220209563, 
      1.941545795887997}, {-2.176633616520424, 
      1.9404787191198765`}, {-2.1787058243807724`, 
      1.9394105828424417`}, {-2.1807768440236637`, 
      1.9383413888535221`}, {-2.1828466756834906`, 
      1.9372711389484913`}, {-2.184915319595978, 
      1.9361998349202705`}, {-2.1869827759981786`, 
      1.9351274785593302`}, {-2.189049045128469, 
      1.9340540716536936`}, {-2.191114127226544, 
      1.9329796159889383`}, {-2.193178022533412, 
      1.9319041133481996`}, {-2.195240731291391, 
      1.930827565512173}, {-2.1973022537441036`, 
      1.9297499742591164`}, {-2.199362590136473, 
      1.9286713413648533`}, {-2.201421740714717, 
      1.9275916686027748`}, {-2.2034797057263455`, 
      1.9265109577438424`}, {-2.205536485420154, 
      1.9254292105565909`}, {-2.20759208004622, 
      1.9243464288071306`}, {-2.209646489855898, 
      1.92326261425915}, {-2.211699715101816, 
      1.9221777686739183`}, {-2.213751756037869, 
      1.9210918938102886`}, {-2.215802612919217, 
      1.9200049914246993`}, {-2.2178522860022793`, 
      1.918917063271178}, {-2.219900775544729, 
      1.9178281111013433`}, {-2.2219480818054893`, 
      1.9167381366644078`}, {-2.2239942050447303`, 
      1.91564714170718}, {-2.226039145523863, 
      1.914555127974068}, {-2.228082903505535, 
      1.913462097207081}, {-2.230125479253627, 
      1.9123680511458327`}, {-2.232166873033248, 
      1.9112729915275435`}, {-2.234207085110729, 
      1.910176920087043}, {-2.2362461157536226`, 
      1.909079838556773}, {-2.238283965230695, 
      1.90798174866679}, {-2.240320633811923, 
      1.906882652144767}, {-2.24235612176849, 
      1.9057825507159976`}, {-2.2443904293727814`, 
      1.904681446103397}, {-2.2464235568983795`, 
      1.9035793400275056`}, {-2.2484555046200603`, 
      1.9024762342064916`}, {-2.2504862728137884`, 
      1.901372130356153}, {-2.252515861756714, 
      1.9002670301899207`}, {-2.2545442717271658`, 
      1.8991609354188608`}, {-2.25657150300465, 
      1.8980538477516773`}, {-2.258597555869844, 
      1.8969457688947147`}, {-2.2606224306045926`, 
      1.8958367005519607`}, {-2.2626461274919047`, 
      1.8947266444250486`}, {-2.2646686468159474`, 
      1.89361560221326}, {-2.266689988862043, 
      1.8925035756135271`}, {-2.268710153916664, 
      1.891390566320436}, {-2.2707291422674296`, 
      1.8902765760262286`}, {-2.2727469542031016`, 
      1.8891616064208052`}, {-2.2747635900135794`, 
      1.8880456591917278`}, {-2.2767790499898966`, 
      1.886928736024222}, {-2.278793334424216, 
      1.8858108386011794`}, {-2.2808064436098268`, 
      1.8846919686031613`}, {-2.2828183778411386`, 
      1.8835721277084}, {-2.2848291374136793`, 
      1.8824513175928026`}, {-2.28683872262409, 
      1.881329539929952}, {-2.28884713377012, 
      1.8802067963911113`}, {-2.2908543711506244`, 
      1.879083088645225}, {-2.2928604350655593`, 
      1.8779584183589224`}, {-2.2948653258159775`, 
      1.8768327871965196`}, {-2.2968690437040244`, 
      1.8757061968200228`}, {-2.298871589032935, 
      1.87457864888913}, {-2.300872962107028, 
      1.8734501450612344`}, {-2.302873163231703, 
      1.8723206869914264`}, {-2.3048721927134377`, 
      1.8711902763324963`}, {-2.3068700508597804`, 
      1.8700589147349376`}, {-2.3088667379793493`, 
      1.868926603846948}, {-2.310862254381828, 
      1.8677933453144342`}, {-2.312856600377959, 
      1.866659140781012}, {-2.314849776279543, 
      1.8655239918880107`}, {-2.316841782399434, 
      1.864387900274475}, {-2.318832619051533, 
      1.8632508675771677`}, {-2.320822286550788, 
      1.8621128954305726`}, {-2.322810785213187, 
      1.8609739854668959`}, {-2.3247981153557546`, 
      1.8598341393160702`}, {-2.32678427729655, 
      1.8586933586057564`}, {-2.3287692713546613`, 
      1.857551644961346}, {-2.330753097850202, 
      1.8564090000059648`}, {-2.3327357571043064`, 
      1.8552654253604737`}, {-2.334717249439128, 
      1.854120922643473}, {-2.3366975751778334`, 
      1.8529754934713039`}, {-2.3386767346445994`, 
      1.8518291394580515`}, {-2.3406547281646093`, 
      1.8506818622155474`}, {-2.342631556064048, 
      1.849533663353372}, {-2.3446072186701, 
      1.8483845444788574`}, {-2.346581716310945, 
      1.8472345071970893`}, {-2.3485550493157517`, 
      1.8460835531109105`}, {-2.350527218014678, 
      1.844931683820923}, {-2.3524982227388644`, 
      1.8437789009254908`}, {-2.354468063820432, 
      1.8426252060207415`}, {-2.3564367415924767`, 
      1.8414706007005703`}, {-2.3584042563890675`, 
      1.8403150865566416`}, {-2.360370608545242, 
      1.8391586651783918`}, {-2.3623357983970017`, 
      1.8380013381530322`}, {-2.3642998262813104`, 
      1.8368431070655509`}, {-2.3662626925360883`, 
      1.835683973498716}, {-2.36822439750021, 
      1.8345239390330776`}, {-2.3701849415135006`, 
      1.833363005246971}, {-2.37214432491673, 
      1.8322011737165185`}, {-2.374102548051612, 
      1.8310384460156328`}, {-2.376059611260799, 
      1.8298748237160187`}, {-2.3780155148878794`, 
      1.8287103083871763`}, {-2.379970259277373, 
      1.8275449015964031`}, {-2.381923844774728, 
      1.8263786049087969`}, {-2.383876271726317, 
      1.8252114198872582`}, {-2.3858275404794336`, 
      1.824043348092493}, {-2.387777651382289, 
      1.8228743910830147`}, {-2.3897266047840073`, 
      1.8217045504151474`}, {-2.3916744010346247`, 
      1.820533827643028}, {-2.3936210404850824`, 
      1.8193622243186087`}, {-2.3955665234872257`, 
      1.81818974199166}, {-2.3975108503937985`, 
      1.8170163822097727`}, {-2.3994540215584417`, 
      1.8158421465183607`}, {-2.4013960373356884`, 
      1.8146670364606639`}, {-2.403336898080961, 
      1.8134910535777498`}, {-2.405276604150567, 
      1.812314199408517}, {-2.4072151559016963`, 
      1.8111364754896973`}, {-2.4091525536924165`, 
      1.8099578833558583`}, {-2.4110887978816713`, 
      1.8087784245394058`}, {-2.413023888829276, 
      1.8075981005705866`}, {-2.414957826895913, 
      1.8064169129774907`}, {-2.41689061244313, 
      1.8052348632860544`}, {-2.4188222458333364`, 
      1.804051953020062}, {-2.420752727429799, 
      1.8028681837011493`}, {-2.4226820575966386`, 
      1.8016835568488052`}, {-2.424610236698828, 
      1.8004980739803746`}, {-2.4265372651021866`, 
      1.7993117366110616`}, {-2.4284631431733787`, 
      1.7981245462539308`}, {-2.4303878712799087`, 
      1.7969365044199106`}, {-2.4323114497901193`, 
      1.7957476126177956`}, {-2.4342338790731866`, 
      1.794557872354249}, {-2.4361551594991178`, 
      1.7933672851338056`}, {-2.4380752914387473`, 
      1.7921758524588733`}, {-2.439994275263734, 
      1.7909835758297366`}, {-2.4419121113465567`, 
      1.7897904567445586`}, {-2.443828800060512, 
      1.788596496699384}, {-2.4457443417797107`, 
      1.7874016971881408`}, {-2.447658736879074, 
      1.7862060597026437`}, {-2.4495719857343308`, 
      1.785009585732596}, {-2.451484088722015, 
      1.7838122767655926`}, {-2.45339504621946, 
      1.782614134287122}, {-2.4553048586047974`, 
      1.781415159780569}, {-2.4572135262569534`, 
      1.7802153547272175`}, {-2.4591210495556455`, 
      1.7790147206062525`}, {-2.461027428881379, 
      1.777813258894763}, {-2.4629326646154435`, 
      1.7766109710677445`}, {-2.4648367571399104`, 
      1.775407858598101}, {-2.466739706837629, 
      1.7742039229566484`}, {-2.468641514092224, 
      1.7729991656121162`}, {-2.4705421792880924`, 
      1.7717935880311502`}, {-2.4724417028103987`, 
      1.7705871916783151`}, {-2.4743400850450734`, 
      1.769379978016097}, {-2.4762373263788096`, 
      1.768171948504906}, {-2.4781334271990594`, 
      1.7669631046030787`}, {-2.4800283878940306`, 
      1.76575344776688}, {-2.4819222088526844`, 
      1.7645429794505063`}, {-2.483814890464731, 
      1.7633317011060883`}, {-2.485706433120628, 
      1.7621196141836926`}, {-2.487596837211575, 
      1.7609067201313247`}, {-2.489486103129514, 
      1.7596930203949313`}, {-2.491374231267123, 
      1.7584785164184031`}, {-2.4932612220178147`, 
      1.757263209643577}, {-2.495147075775732, 
      1.7560471015102384`}, {-2.4970317929357466`, 
      1.754830193456124}, {-2.4989153738934546`, 
      1.7536124869169243`}, {-2.5007978190451747`, 
      1.752393983326286}, {-2.5026791287879435`, 
      1.7511746841158147`}, {-2.504559303519514, 
      1.7499545907150764`}, {-2.5064383436383517`, 
      1.7487337045516012`}, {-2.508316249543632, 
      1.747512027050885}, {-2.5101930216352373`, 
      1.7462895596363928`}, {-2.5120686603137528`, 
      1.7450663037295597`}, {-2.5139431659804647`, 
      1.743842260749795}, {-2.5158165390373575`, 
      1.7426174321144836`}, {-2.5176887798871097`, 
      1.741391819238989}, {-2.5195598889330917`, 
      1.7401654235366553`}, {-2.521429866579363, 
      1.73893824641881}, {-2.5232987132306683`, 
      1.7377102892947665`}, {-2.5251664292924354`, 
      1.736481553571826}, {-2.5270330151707725`, 
      1.7352520406552814`}, {-2.528898471272464, 
      1.7340217519484176`}, {-2.5307627980049685`, 
      1.7327906888525155`}, {-2.532625995776416, 
      1.7315588527668542`}, {-2.5344880649956045`, 
      1.7303262450887131`}, {-2.5363490060719975`, 
      1.729092867213375}, {-2.5382088194157206`, 
      1.7278587205341271`}, {-2.540067505437559, 
      1.7266238064422657`}, {-2.5419250645489546`, 
      1.725388126327096}, {-2.543781497162003, 
      1.7241516815759372`}, {-2.5456368036894514`, 
      1.7229144735741229`}, {-2.5474909845446936`, 
      1.7216765037050044`}, {-2.5493440401417695`, 
      1.7204377733499532`}, {-2.551195970895361, 
      1.719198283888363}, {-2.5530467772207897`, 
      1.7179580366976528`}, {-2.5548964595340142`, 
      1.7167170331532686`}, {-2.5567450182516263`, 
      1.7154752746286863`}, {-2.5585924537908493`, 
      1.714232762495414}, {-2.5604387665695345`, 
      1.712989498122994}, {-2.562283957006159, 
      1.7117454828790066`}, {-2.5641280255198224`, 
      1.7105007181290706`}, {-2.565970972530244, 
      1.709255205236847}, {-2.5678127984577603`, 
      1.7080089455640415`}, {-2.5696535037233224`, 
      1.706761940470406}, {-2.5714930887484924`, 
      1.7055141913137417`}, {-2.573331553955442, 
      1.7042656994499015`}, {-2.5751688997669486`, 
      1.703016466232792}, {-2.5770051266063927`, 
      1.7017664930143765`}, {-2.5788402348977564`, 
      1.7005157811446772`}, {-2.5806742250656187`, 
      1.6992643319717768`}, {-2.5825070975351543`, 
      1.6980121468418223`}, {-2.5843388527321305`, 
      1.6967592270990266`}, {-2.586169491082904, 
      1.6955055740856708`}, {-2.5879990130144193`, 
      1.694251189142107}, {-2.5898274189542043`, 
      1.6929960736067604`}, {-2.5916547093303697`, 
      1.691740228816132}, {-2.593480884571605, 
      1.6904836561048007`}, {-2.595305945107176, 
      1.6892263568054258`}, {-2.5971298913669223`, 
      1.6879683322487493`}, {-2.5989527237812546`, 
      1.6867095837635986`}, {-2.6007744427811526`, 
      1.6854501126768886`}, {-2.602595048798161, 
      1.6841899203136244`}, {-2.604414542264389, 
      1.682929007996903}, {-2.6062329236125055`, 
      1.6816673770479167`}, {-2.608050193275737, 
      1.6804050287859544`}, {-2.609866351687867, 
      1.679141964528405}, {-2.611681399283231, 
      1.6778781855907587`}, {-2.613495336496714, 
      1.6766136932866107`}, {-2.61530816376375, 
      1.6753484889276624`}, {-2.6171198815203174`, 
      1.6740825738237242`}, {-2.618930490202937, 
      1.672815949282718}, {-2.6207399902486705`, 
      1.6715486166106794`}, {-2.622548382095116, 
      1.6702805771117604`}, {-2.6243556661804073`, 
      1.669011832088231}, {-2.6261618429432096`, 
      1.6677423828404825`}, {-2.627966912822719, 
      1.6664722306670294`}, {-2.6297708762586582`, 
      1.6652013768645115`}, {-2.6315737336912752`, 
      1.6639298227276969`}, {-2.6333754855613396`, 
      1.6626575695494834`}, {-2.6351761323101415`, 
      1.6613846186209025`}, {-2.636975674379488, 
      1.6601109712311197`}, {-2.638774112211701, 
      1.6588366286674383`}, {-2.640571446249615, 
      1.6575615922153015`}, {-2.6423676769365736`, 
      1.6562858631582942`}, {-2.6441628047164287`, 
      1.655009442778146}, {-2.6459568300335365`, 
      1.6537323323547333`}, {-2.6477497533327563`, 
      1.6524545331660812`}, {-2.6495415750594464`, 
      1.6511760464883665`}, {-2.651332295659464, 
      1.64989687359592}, {-2.6531219155791606`, 
      1.6486170157612285`}, {-2.6549104352653803`, 
      1.647336474254937}, {-2.6566978551654583`, 
      1.6460552503458517`}, {-2.6584841757272173`, 
      1.6447733453009414`}, {-2.6602693973989653`, 
      1.643490760385341}, {-2.6620535206294935`, 
      1.6422074968623528`}, {-2.663836545868074, 
      1.6409235559934492`}, {-2.6656184735644577`, 
      1.639638939038275}, {-2.6673993041688706`, 
      1.6383536472546503`}, {-2.6691790381320124`, 
      1.6370676818985714`}, {-2.670957675905054, 
      1.6357810442242147`}, {-2.6727352179396364`, 
      1.634493735483938}, {-2.674511664687865, 
      1.6332057569282834`}, {-2.676287016602311, 
      1.631917109805979}, {-2.678061274136007, 
      1.630627795363942}, {-2.6798344377424446`, 
      1.6293378148472801`}, {-2.6816065078755735`, 
      1.6280471694992946`}, {-2.683377484989798, 
      1.6267558605614822`}, {-2.685147369539974, 
      1.6254638892735376`}, {-2.686916161981409, 
      1.6241712568733557`}, {-2.688683862769857, 
      1.6228779645970337`}, {-2.690450472361519, 
      1.621584013678874}, {-2.6922159912130392`, 
      1.6202894053513857`}, {-2.693980419781502, 
      1.6189941408452873`}, {-2.695743758524431, 
      1.617698221389509}, {-2.6975060078997872`, 
      1.6164016482111954`}, {-2.6992671683659646`, 
      1.6151044225357067`}, {-2.7010272403817903`, 
      1.6138065455866222`}, {-2.70278622440652, 
      1.6125080185857414`}, {-2.704544120899839, 
      1.6112088427530877`}, {-2.7063009303218553`, 
      1.6099090193069092`}, {-2.708056653133102, 
      1.608608549463682}, {-2.7098112897945317`, 
      1.6073074344381117`}, {-2.711564840767517, 
      1.606005675443137}, {-2.7133173065138463`, 
      1.6047032736899305`}, {-2.715068687495722, 
      1.6034002303879014`}, {-2.716818984175759, 
      1.6020965467446981`}, {-2.7185681970169817`, 
      1.6007922239662107`}, {-2.7203163264828225`, 
      1.5994872632565724`}, {-2.722063373037119, 
      1.5981816658181625`}, {-2.723809337144112, 
      1.596875432851608}, {-2.7255542192684437`, 
      1.5955685655557865`}, {-2.727298019875156, 
      1.5942610651278284`}, {-2.7290407394296863`, 
      1.5929529327631187`}, {-2.7307823783978677`, 
      1.5916441696552996`}, {-2.7325229372459257`, 
      1.5903347769962728`}, {-2.7342624164404756`, 
      1.5890247559762012`}, {-2.736000816448522, 
      1.587714107783512}, {-2.7377381377374546`, 
      1.5864028336048983`}, {-2.739474380775048, 
      1.5850909346253217`}, {-2.741209546029458, 
      1.583778412028014}, {-2.742943633969222, 
      1.5824652669944805`}, {-2.7446766450632527`, 
      1.581151500704501}, {-2.74640857978084, 
      1.5798371143361327`}, {-2.748139438591647, 
      1.5785221090657127`}, {-2.7498692219657084`, 
      1.5772064860678594`}, {-2.7515979303734284`, 
      1.5758902465154756`}, {-2.753325564285578, 
      1.57457339157975}, {-2.755052124173294, 
      1.5732559224301599`}, {-2.756777610508076, 
      1.5719378402344735`}, {-2.758502023761786, 
      1.5706191461587515`}, {-2.7602253644066432`, 
      1.56929984136735}, {-2.7619476329152257`, 
      1.5679799270229222`}, {-2.7636688297604657`, 
      1.5666594042864213`}, {-2.765388955415648, 
      1.5653382743171016`}, {-2.76710801035441, 
      1.564016538272522}, {-2.7688259950507366`, 
      1.562694197308547}, {-2.7705429099789605`, 
      1.5613712525793502`}, {-2.7722587556137586`, 
      1.560047705237415}, {-2.7739735324301518`, 
      1.5587235564335384`}, {-2.7756872409035016`, 
      1.557398807316832}, {-2.7773998815095085`, 
      1.5560734590347243`}, {-2.7791114547242097`, 
      1.5547475127329637`}, {-2.7808219610239777`, 
      1.5534209695556203`}, {-2.782531400885518, 
      1.5520938306450873`}, {-2.7842397747858683`, 
      1.5507660971420847`}, {-2.785947083202393, 
      1.5494377701856599`}, {-2.7876533266127868`, 
      1.548108850913191}, {-2.789358505495067, 
      1.5467793404603891`}, {-2.7910626203275757`, 
      1.5454492399612991`}, {-2.7927656715889757`, 
      1.5441185505483035`}, {-2.7944676597582503`, 
      1.5427872733521237`}, {-2.796168585314699, 
      1.5414554095018222`}, {-2.7978684487379377`, 
      1.5401229601248052`}, {-2.799567250507896, 
      1.5387899263468243`}, {-2.801264991104815, 
      1.5374563092919789`}, {-2.8029616710092453`, 
      1.5361221100827183`}, {-2.8046572907020466`, 
      1.5347873298398442`}, {-2.806351850664384, 
      1.5334519696825124`}, {-2.8080453513777273`, 
      1.532116030728235}, {-2.809737793323848, 
      1.530779514092883}, {-2.8114291769848183`, 
      1.5294424208906878`}, {-2.8131195028430094`, 
      1.528104752234244}, {-2.8148087713810885`, 
      1.5267665092345108`}, {-2.8164969830820183`, 
      1.5254276930008155`}, {-2.818184138429054, 
      1.524088304640854}, {-2.8198702379057425`, 
      1.522748345260694}, {-2.82155528199592, 
      1.5214078159647768`}, {-2.8232392711837098`, 
      1.5200667178559197`}, {-2.824922205953521, 
      1.5187250520353177`}, {-2.8266040867900464`, 
      1.5173828196025458`}, {-2.8282849141782616`, 
      1.5160400216555616`}, {-2.8299646886034213`, 
      1.514696659290707}, {-2.8316434105510595`, 
      1.51335273360271}, {-2.833321080506986, 
      1.5120082456846875`}, {-2.8349976989572863`, 
      1.5106631966281472`}, {-2.836673266388318, 
      1.5093175875229896`}, {-2.83834778328671, 
      1.5079714194575105`}, {-2.8400212501393614`, 
      1.5066246935184022`}, {-2.841693667433438, 
      1.5052774107907567`}, {-2.8433650356563724`, 
      1.5039295723580675`}, {-2.8450353552958605`, 
      1.502581179302231}, {-2.8467046268398613`, 
      1.5012322327035497`}, {-2.848372850776593, 
      1.4998827336407337`}, {-2.8500400275945346`, 
      1.498532683190903}, {-2.8517061577824205`, 
      1.4971820824295894`}, {-2.853371241829241, 
      1.4958309324307386`}, {-2.8550352802242407`, 
      1.4944792342667128`}, {-2.8566982734569146`, 
      1.493126989008292}, {-2.858360222017009, 
      1.4917741977246772`}, {-2.8600211263945186`, 
      1.4904208614834913`}, {-2.861680987079684, 
      1.4890669813507817`}, {-2.863339804562991, 
      1.4877125583910227`}, {-2.86499757933517, 
      1.4863575936671172`}, {-2.8666543118871908`, 
      1.485002088240399}, {-2.8683100027102646`, 
      1.4836460431706346`}, {-2.8699646522958404`, 
      1.4822894595160259`}, {-2.8716182611356036`, 
      1.4809323383332114`}, {-2.873270829721475, 
      1.479574680677269}, {-2.874922358545607, 
      1.4782164876017174`}, {-2.876572848100386, 
      1.4768577601585196`}, {-2.878222298878425, 
      1.4754984993980829`}, {-2.8798707113725683`, 
      1.4741387063692626`}, {-2.8815180860758844`, 
      1.4727783821193634`}, {-2.883164423481668, 
      1.4714175276941415`}, {-2.8848097240834356`, 
      1.4700561441378068`}, {-2.8864539883749267`, 
      1.4686942324930252`}, {-2.8880972168501002`, 
      1.4673317938009203`}, {-2.889739410003133, 
      1.465968829101075}, {-2.8913805683284184`, 
      1.4646053394315348`}, {-2.8930206923205657`, 
      1.4632413258288086`}, {-2.8946597824743967`, 
      1.4618767893278717`}, {-2.896297839284945, 
      1.4605117309621671`}, {-2.8979348632474555`, 
      1.4591461517636084`}, {-2.89957085485738, 
      1.4577800527625808`}, {-2.9012058146103787`, 
      1.456413434987944}, {-2.9028397430023163`, 
      1.4550462994670341`}, {-2.904472640529262, 
      1.4536786472256653`}, {-2.906104507687486, 
      1.452310479288132}, {-2.90773534497346, 
      1.4509417966772111`}, {-2.9093651528838556`, 
      1.449572600414164}, {-2.9109939319155402`, 
      1.4482028915187382`}, {-2.912621682565578, 
      1.44683267100917}, {-2.914248405331227, 
      1.4454619399021857`}, {-2.9158741007099387`, 
      1.444090699213005}, {-2.917498769199356, 
      1.4427189499553412`}, {-2.91912241129731, 
      1.4413466931414047`}, {-2.920745027501822, 
      1.4399739297819043`}, {-2.922366618311098, 
      1.4386006608860493`}, {-2.9239871842235305`, 
      1.437226887461552}, {-2.9256067257376945`, 
      1.4358526105146285`}, {-2.927225243352348, 
      1.4344778310500026`}, {-2.9288427375664288`, 
      1.4331025500709058`}, {-2.9304592088790535`, 
      1.4317267685790809`}, {-2.932074657789517, 
      1.430350487574783}, {-2.93368908479729, 
      1.428973708056782}, {-2.9353024904020164`, 
      1.4275964310223643`}, {-2.936914875103515, 
      1.4262186574673352`}, {-2.9385262394017744`, 
      1.4248403883860206`}, {-2.940136583796954, 
      1.4234616247712688`}, {-2.9417459087893825`, 
      1.4220823676144532`}, {-2.943354214879554, 
      1.4207026179054731`}, {-2.9449615025681286`, 
      1.4193223766327572`}, {-2.946567772355931, 
      1.4179416447832642`}, {-2.948173024743949, 
      1.4165604233424858`}, {-2.9497772602333296`, 
      1.4151787132944478`}, {-2.9513804793253815`, 
      1.4137965156217132`}, {-2.952982682521571, 
      1.412413831305383}, {-2.9545838703235203`, 
      1.4110306613250987`}, {-2.956184043233008, 
      1.4096470066590445`}, {-2.9577832017519663`, 
      1.408262868283949}, {-2.9593813463824796`, 
      1.4068782471750874`}, {-2.960978477626784, 
      1.4054931443062828`}, {-2.9625745959872645`, 
      1.404107560649909}, {-2.964169701966455, 
      1.4027214971768918`}, {-2.965763796067035, 
      1.4013349548567116`}, {-2.96735687879183, 
      1.3999479346574046`}, {-2.9689489506438105`, 
      1.3985604375455658`}, {-2.970540012126088, 
      1.3971724644863497`}, {-2.9721300637419152`, 
      1.3957840164434732`}, {-2.973719105994686, 
      1.3943950943792172`}, {-2.975307139387931, 
      1.3930056992544286`}, {-2.976894164425319, 
      1.3916158320285223`}, {-2.978480181610653, 
      1.3902254936594827`}, {-2.980065191447872, 
      1.3888346851038664`}, {-2.9816491944410464`, 
      1.3874434073168036`}, {-2.983232191094378, 
      1.386051661252}, {-2.9848141819122, 
      1.3846594478617391`}, {-2.986395167398973, 
      1.3832667680968842`}, {-2.9879751480592853`, 
      1.3818736229068798`}, {-2.9895541243978516`, 
      1.3804800132397534`}, {-2.991132096919511, 
      1.3790859400421187`}, {-2.9927090661292257`, 
      1.3776914042591761`}, {-2.99428503253208, 
      1.3762964068347154`}, {-2.995859996633279, 
      1.3749009487111172`}, {-2.9974339589381467`, 
      1.3735050308293555`}, {-2.9990069199521256`, 
      1.3721086541289989`}, {-3.0005788801807745`, 
      1.370711819548213}, {-3.0021498401297673`, 
      1.3693145280237624`}, {-3.003719800304892, 
      1.367916780491012}, {-3.0052887612120496`, 
      1.3665185778839293`}, {-3.0068567233572523`, 
      1.3651199211350864`}, {-3.0084236872466223`, 
      1.3637208111756618`}, {-3.009989653386391, 
      1.3623212489354424`}, {-3.011554622282896, 
      1.3609212353428248`}, {-3.0131185944425827`, 
      1.3595207713248183`}, {-3.014681570372, 
      1.3581198578070457`}, {-3.016243550577801, 
      1.3567184957137457`}, {-3.017804535566741, 
      1.3553166859677752`}, {-3.0193645258456767`, 
      1.3539144294906102`}, {-3.020923521921564, 
      1.3525117272023486`}, {-3.022481524301458, 
      1.3511085800217115`}, {-3.0240385334925093`, 
      1.3497049888660453`}, {-3.0255945500019665`, 
      1.3483009546513236`}, {-3.027149574337172, 
      1.3468964782921493`}, {-3.028703607005562, 
      1.345491560701756}, {-3.0302566485146643`, 
      1.3440862027920097`}, {-3.031808699372098, 
      1.342680405473412}, {-3.0333597600855713`, 
      1.3412741696551003`}, {-3.034909831162882, 
      1.3398674962448505`}, {-3.0364589131119137`, 
      1.338460386149079}, {-3.0380070064406377`, 
      1.337052840272844}, {-3.0395541116571088`, 
      1.3356448595198485`}, {-3.0411002292694653`, 
      1.3342364447924402`}, {-3.0426453597859284`, 
      1.3328275969916152`}, {-3.0441895037148003`, 
      1.331418317017019}, {-3.045732661564463, 
      1.3300086057669487`}, {-3.047274833843377, 
      1.3285984641383544`}, {-3.04881602106008, 
      1.3271878930268415`}, {-3.050356223723187, 
      1.3257768933266723`}, {-3.051895442341387, 
      1.3243654659307675`}, {-3.0534336774234436`, 
      1.3229536117307092`}, {-3.054970929478193, 
      1.3215413316167417`}, {-3.0565071990145425`, 
      1.3201286264777732`}, {-3.0580424865414697`, 
      1.3187154972013786`}, {-3.0595767925680217`, 
      1.3173019446738004`}, {-3.061110117603313, 
      1.315887969779951}, {-3.062642462156526, 
      1.3144735734034148`}, {-3.0641738267369076`, 
      1.313058756426449}, {-3.06570421185377, 
      1.311643519729987}, {-3.0672336180164876`, 
      1.3102278641936385`}, {-3.068762045734498, 
      1.3088117906956924`}, {-3.0702894955172995`, 
      1.3073953001131187`}, {-3.07181596787445, 
      1.3059783933215694`}, {-3.073341463315566, 
      1.3045610711953812`}, {-3.074865982350322, 
      1.3031433346075771`}, {-3.076389525488448, 
      1.301725184429868}, {-3.077912093239731, 
      1.3003066215326546`}, {-3.0794336861140104`, 
      1.2988876467850292`}, {-3.080954304621179, 
      1.2974682610547776`}, {-3.082473949271183, 
      1.2960484652083808`}, {-3.0839926205740165`, 
      1.2946282601110168`}, {-3.0855103190397255`, 
      1.2932076466265625`}, {-3.087027045178404, 
      1.2917866256175954`}, {-3.0885427995001935`, 
      1.2903651979453952`}, {-3.090057582515281, 
      1.2889433644699462`}, {-3.0915713947339, 
      1.2875211260499384`}, {-3.0930842366663267`, 
      1.2860984835427696`}, {-3.0945961088228815`, 
      1.2846754378045473`}, {-3.0961070117139258`, 
      1.2832519896900905`}, {-3.0976169458498624`, 
      1.281828140052931}, {-3.0991259117411336`, 
      1.2804038897453156`}, {-3.100633909898221, 
      1.2789792396182078`}, {-3.1021409408316427`, 
      1.2775541905212897`}, {-3.1036470050519536`, 
      1.2761287433029636`}, {-3.1051521030697447`, 
      1.2747028988103535`}, {-3.1066562353956404`, 
      1.2732766578893073`}, {-3.108159402540299, 
      1.2718500213843988`}, {-3.109661605014411, 
      1.270422990138929}, {-3.111162843328698, 
      1.2689955649949274`}, {-3.1126631179939115`, 
      1.267567746793155}, {-3.114162429520832, 
      1.266139536373105}, {-3.115660778420269, 
      1.264710934573005}, {-3.1171581652030573`, 
      1.2632819422298187`}, {-3.118654590380059, 
      1.2618525601792479`}, {-3.1201500544621603`, 
      1.2604227892557336`}, {-3.1216445579602716`, 
      1.2589926302924583`}, {-3.123138101385326, 
      1.2575620841213477`}, {-3.1246306852482784`, 
      1.2561311515730724`}, {-3.1261223100601043`, 
      1.2546998334770492`}, {-3.1276129763317995`, 
      1.2532681306614433`}, {-3.1291026845743777`, 
      1.2518360439531702`}, {-3.1305914352988706`, 
      1.250403574177897}, {-3.1320792290163273`, 
      1.2489707221600441`}, {-3.1335660662378113`, 
      1.247537488722788}, {-3.1350519474744014`, 
      1.2461038746880608`}, {-3.1365368732371905`, 
      1.2446698808765546`}, {-3.1380208440372837`, 
      1.2432355081077213`}, {-3.139503860385797, 
      1.2418007571997751`}, {-3.140985922793859, 
      1.2403656289696938`}, {-3.142467031772606, 
      1.2389301242332211`}, {-3.143947187833184, 
      1.237494243804868}, {-3.1454263914867466`, 
      1.2360579884979142`}, {-3.1469046432444543`, 
      1.2346213591244104`}, {-3.1483819436174727`, 
      1.2331843564951799`}, {-3.149858293116973, 
      1.2317469814198199`}, {-3.1513336922541293`, 
      1.2303092347067033`}, {-3.1528081415401195`, 
      1.228871117162981}, {-3.1542816414861226`, 
      1.2274326295945825`}, {-3.1557541926033186`, 
      1.2259937728062191`}, {-3.157225795402888, 
      1.2245545476013842`}, {-3.1586964503960098`, 
      1.2231149547823557`}, {-3.1601661580938614`, 
      1.2216749951501975`}, {-3.161634919007617, 
      1.2202346695047614`}, {-3.1631027336484467`, 
      1.2187939786446884`}, {-3.164569602527516, 
      1.2173529233674107`}, {-3.1660355261559854`, 
      1.2159115044691535`}, {-3.1675005050450076`, 
      1.2144697227449364`}, {-3.1689645397057284`, 
      1.213027578988575}, {-3.1704276306492845`, 
      1.2115850739926826`}, {-3.1718897783868036`, 
      1.2101422085486726`}, {-3.173350983429403, 
      1.2086989834467592`}, {-3.1748112462881886`, 
      1.2072553994759596`}, {-3.1762705674742535`, 
      1.2058114574240955`}, {-3.1777289474986787`, 
      1.2043671580777948`}, {-3.1791863868725305`, 
      1.2029225022224934`}, {-3.18064288610686, 
      1.2014774906424366`}, {-3.1820984457127026`, 
      1.2000321241206808`}, {-3.1835530662010774`, 
      1.1985864034390958`}, {-3.185006748082985, 
      1.1971403293783653`}, {-3.186459491869408, 
      1.1956939027179898`}, {-3.1879112980713096`, 
      1.194247124236287}, {-3.189362167199632, 
      1.1927999947103949`}, {-3.1908120997652967`, 
      1.1913525149162718`}, {-3.1922610962792026`, 
      1.1899046856286992`}, {-3.193709157252226, 
      1.1884565076212832`}, {-3.1951562831952187`, 
      1.1870079816664558`}, {-3.1966024746190085`, 
      1.1855591085354769`}, {-3.198047732034397, 
      1.1841098889984354`}, {-3.199492055952159, 
      1.1826603238242517`}, {-3.200935446883043, 
      1.1812104137806785`}, {-3.2023779053377672`, 
      1.179760159634303}, {-3.2038194318270232`, 
      1.1783095621505482`}, {-3.2052600268614704`, 
      1.1768586220936748`}, {-3.2066996909517385`, 
      1.1754073402267824`}, {-3.2081384246084252`, 
      1.1739557173118118`}, {-3.209576228342096, 
      1.172503754109546}, {-3.211013102663282, 
      1.171051451379612}, {-3.2124490480824814`, 
      1.1695988098804826`}, {-3.213884065110156, 
      1.1681458303694778`}, {-3.2153181542567326`, 
      1.1666925136027668`}, {-3.2167513160326004`, 
      1.1652388603353692`}, {-3.2181835509481114`, 
      1.1637848713211565`}, {-3.2196148595135794`, 
      1.162330547312854}, {-3.221045242239278, 
      1.1608758890620428`}, {-3.222474699635442, 
      1.1594208973191604`}, {-3.2239032322122645`, 
      1.1579655728335034`}, {-3.225330840479897, 
      1.1565099163532282`}, {-3.2267575249484475`, 
      1.1550539286253532`}, {-3.228183286127982, 
      1.15359761039576}, {-3.2296081245285224`, 
      1.1521409624091952`}, {-3.231032040660044, 
      1.1506839854092723`}, {-3.2324550350324777`, 
      1.1492266801384725`}, {-3.2338771081557067`, 
      1.1477690473381472`}, {-3.235298260539568, 
      1.1463110877485188`}, {-3.236718492693849, 
      1.1448528021086828`}, {-3.2381378051282894`, 
      1.143394191156609}, {-3.239556198352578, 
      1.1419352556291438`}, {-3.240973672876353, 
      1.1404759962620108`}, {-3.242390229209203, 
      1.139016413789813}, {-3.243805867860662, 
      1.1375565089460347`}, {-3.245220589340212, 
      1.1360962824630418`}, {-3.2466343941572813`, 
      1.1346357350720848`}, {-3.248047282821244, 
      1.1331748675032995`}, {-3.249459255841418, 
      1.1317136804857089`}, {-3.2508703137270665`, 
      1.1302521747472247`}, {-3.2522804569873944`, 
      1.1287903510146489`}, {-3.25368968613155, 
      1.1273282100136752`}, {-3.255098001668622, 
      1.1258657524688909`}, {-3.2565054041076413`, 
      1.1244029791037782`}, {-3.257911893957578, 
      1.1229398906407158`}, {-3.2593174717273423`, 
      1.1214764878009802`}, {-3.2607221379257822`, 
      1.1200127713047479`}, {-3.262125893061684, 
      1.1185487418710964`}, {-3.2635287376437705`, 
      1.117084400218006}, {-3.264930672180702, 
      1.1156197470623612`}, {-3.266331697181073, 
      1.1141547831199523`}, {-3.2677318131534134`, 
      1.112689509105477}, {-3.2691310206061877`, 
      1.1112239257325418`}, {-3.270529320047793, 
      1.1097580337136639`}, {-3.2719267119865596`, 
      1.1082918337602723`}, {-3.2733231969307495`, 
      1.1068253265827095`}, {-3.2747187753885556`, 
      1.105358512890233}, {-3.2761134478681013`, 
      1.1038913933910173`}, {-3.2775072148774402`, 
      1.1024239687921544`}, {-3.2789000769245544`, 
      1.1009562397996564`}, {-3.2802920345173545`, 
      1.0994882071184564`}, {-3.2816830881636787`, 
      1.09801987145241}, {-3.2830732383712915`, 
      1.0965512335042975`}, {-3.284462485647884, 
      1.0950822939758247`}, {-3.285850830501073, 
      1.0936130535676245`}, {-3.2872382734384, 
      1.0921435129792587`}, {-3.2886248149673296`, 
      1.0906736729092195`}, {-3.2900104555952505`, 
      1.0892035340549306`}, {-3.2913951958294736`, 
      1.0877330971127495`}, {-3.292779036177232, 
      1.086262362777968}, {-3.2941619771456803`, 
      1.0847913317448146`}, {-3.295544019241893, 
      1.0833200047064555`}, {-3.2969251629728644`, 
      1.0818483823549965`}, {-3.2983054088455086`, 
      1.0803764653814838`}, {-3.2996847573666575`, 
      1.0789042544759062`}, {-3.3010632090430607`, 
      1.0774317503271962`}, {-3.302440764381386, 
      1.075958953623232}, {-3.3038174238882165`, 
      1.074485865050838}, {-3.305193188070051, 
      1.0730124852957876`}, {-3.3065680574333043`, 
      1.0715388150428036`}, {-3.3079420324843047`, 
      1.0700648549755603`}, {-3.3093151137292947`, 
      1.0685906057766845`}, {-3.310687301674429, 
      1.0671160681277578`}, {-3.3120585968257763`, 
      1.0656412427093171`}, {-3.313428999689316, 
      1.0641661302008567`}, {-3.314798510770938, 
      1.0626907312808296`}, {-3.316167130576444, 
      1.0612150466266488`}, {-3.3175348596115444`, 
      1.0597390769146893`}, {-3.318901698381859, 
      1.0582628228202893`}, {-3.320267647392916, 
      1.0567862850177512`}, {-3.321632707150152, 
      1.0553094641803438`}, {-3.32299687815891, 
      1.0538323609803033`}, {-3.32436016092444, 
      1.052354976088835}, {-3.3257225559518973`, 
      1.0508773101761149`}, {-3.327084063746343, 
      1.0493993639112902`}, {-3.3284446848127427`, 
      1.0479211379624824`}, {-3.3298044196559657`, 
      1.0464426329967873`}, {-3.331163268780785, 
      1.0449638496802773`}, {-3.3325212326918754`, 
      1.0434847886780025`}, {-3.333878311893815, 
      1.042005450653992}, {-3.335234506891083, 
      1.040525836271256}, {-3.336589818188059, 
      1.0390459461917863`}, {-3.337944246289023, 
      1.0375657810765586`}, {-3.3392977916981548`, 
      1.0360853415855338`}, {-3.340650454919533, 
      1.0346046283776589`}, {-3.3420022364571342`, 
      1.033123642110869}, {-3.3433531368148333`, 
      1.0316423834420885`}, {-3.344703156496402, 
      1.0301608530272324`}, {-3.346052296005509, 
      1.028679051521208}, {-3.3474005558457183`, 
      1.0271969795779168`}, {-3.3487479365204895`, 
      1.0257146378502544`}, {-3.350094438533177, 
      1.0242320269901137`}, {-3.3514400623870286`, 
      1.0227491476483852`}, {-3.352784808585187, 
      1.0212660004749587`}, {-3.354128677630687, 
      1.019782586118725}, {-3.355471670026455, 
      1.018298905227577}, {-3.3568137862753105`, 
      1.0168149584484112`}, {-3.358155026879963, 
      1.0153307464271293`}, {-3.359495392343014, 
      1.0138462698086392`}, {-3.3608348831669534`, 
      1.0123615292368573`}, {-3.3621734998541615`, 
      1.0108765253547085`}, {-3.363511242906907, 
      1.009391258804129}, {-3.364848112827347, 
      1.0079057302260666`}, {-3.366184110117526, 
      1.006419940260483}, {-3.3675192352793757`, 
      1.0049338895463549`}, {-3.3688534888147146`, 
      1.0034475787216752`}, {-3.370186871225247, 
      1.0019610084234547`}, {-3.3715193830125623`, 
      1.000474179287723}, {-3.372851024678135, 
      0.9989870919495304}, {-3.374181796723324, 
      0.9974997470429492}, {-3.3755116996493713`, 
      0.9960121452010751}, {-3.3768407339574025`, 
      0.9945242870560284}, {-3.3781689001484256`, 
      0.9930361732389555}, {-3.379496198723331, 
      0.9915478043800307}, {-3.38082263018289, 
      0.9900591811084565}, {-3.382148195027755, 
      0.9885703040524663}, {-3.3834728937584595`, 
      0.9870811738393247}, {-3.384796726875415, 
      0.9855917910953298}, {-3.3861196948789143`, 
      0.9841021564458139}, {-3.3874417982691276`, 
      0.9826122705151451}, {-3.388763037546104, 
      0.9811221339267286}, {-3.3900834132097697`, 
      0.9796317473030085}, {-3.391402925759928, 
      0.9781411112654685}, {-3.39272157569626, 
      0.9766502264346338}, {-3.394039363518321, 
      0.9751590934300722}, {-3.395356289725543, 
      0.9736677128703956}, {-3.3966723548172326`, 
      0.9721760853732611}, {-3.397987559292571, 
      0.9706842115553729}, {-3.3993019036506134`, 
      0.9691920920324832}, {-3.400615388390288, 
      0.9676997274193938}, {-3.4019280140103962`, 
      0.9662071183299571}, {-3.4032397810096118`, 
      0.9647142653770779}, {-3.4045506898864804`, 
      0.9632211691727147}, {-3.405860741139419, 
      0.9617278303278808}, {-3.407169935266715, 
      0.9602342494526457}, {-3.4084782727665264`, 
      0.9587404271561366}, {-3.4097857541368812`, 
      0.9572463640465397}, {-3.4110923798756767`, 
      0.9557520607311016}, {-3.412398150480678, 
      0.9542575178161306}, {-3.41370306644952, 
      0.9527627359069977}, {-3.4150071282797043`, 
      0.9512677156081385}, {-3.4163103364686003`, 
      0.9497724575230543}, {-3.417612691513444, 
      0.9482769622543135}, {-3.418914193911337, 
      0.9467812304035527}, {-3.4202148441592484`, 
      0.9452852625714782}, {-3.4215146427540106`, 
      0.9437890593578675}, {-3.4228135901923222`, 
      0.9422926213615704}, {-3.424111686970746, 
      0.9407959491805105}, {-3.4254089335857074`, 
      0.9392990434116861}, {-3.4267053305334967`, 
      0.9378019046511723}, {-3.4280008783102662`, 
      0.9363045334941215}, {-3.4292955774120304`, 
      0.9348069305347652}, {-3.4305894283346663`, 
      0.9333090963664152}, {-3.431882431573912, 
      0.9318110315814653}, {-3.4331745876253668`, 
      0.9303127367713917}, {-3.43446589698449, 
      0.9288142125267552}, {-3.4357563601466015`, 
      0.9273154594372022}, {-3.43704597760688, 
      0.9258164780914657}, {-3.438334749860364, 
      0.9243172690773672}, {-3.4396226774019496`, 
      0.9228178329818176}, {-3.440909760726392, 
      0.9213181703908188}, {-3.442196000328304, 
      0.9198182818894644}, {-3.443481396702155, 
      0.9183181680619417}, {-3.444765950342271, 
      0.9168178294915328}, {-3.446049661742835, 
      0.9153172667606158}, {-3.447332531397886, 
      0.9138164804506659}, {-3.4486145598013174`, 
      0.9123154711422572}, {-3.4498957474468783`, 
      0.9108142394150636}, {-3.451176094828172, 
      0.9093127858478601}, {-3.452455602438655, 
      0.9078111110185244}, {-3.4537342707716387`, 
      0.9063092155040379}, {-3.455012100320287, 
      0.9048070998804869}, {-3.456289091577617, 
      0.9033047647230643}, {-3.457565245036497, 
      0.9018022106060707}, {-3.458840561189648, 
      0.9002994381029154}, {-3.4601150405296415`, 
      0.8987964477861179}, {-3.461388683548901, 
      0.8972932402273094}, {-3.4626614907396993`, 
      0.8957898159972337}, {-3.4639334625941607`, 
      0.8942861756657488}, {-3.4652045996042578`, 
      0.8927823198018277}, {-3.4664749022618127`, 
      0.8912782489735603}, {-3.4677443710584965`, 
      0.8897739637481543}, {-3.4690130064858287`, 
      0.8882694646919365}, {-3.470280809035176, 
      0.8867647523703538}, {-3.4715477791977536`, 
      0.8852598273479753}, {-3.4728139174646233`, 
      0.8837546901884925}, {-3.4740792243266934`, 
      0.8822493414547216}, {-3.4753437002747183`, 
      0.8807437817086037}, {-3.4766073457992985`, 
      0.879238011511207}, {-3.47787016139088, 
      0.8777320314227274}, {-3.4791321475397536`, 
      0.8762258420024901}, {-3.4803933047360545`, 
      0.874719443808951}, {-3.481653633469762, 
      0.8732128373996972}, {-3.4829131342307, 
      0.8717060233314493}, {-3.484171807508534, 
      0.8701990021600619}, {-3.485429653792775, 
      0.8686917744405249}, {-3.4866866735727737`, 
      0.8671843407269652}, {-3.4879428673377246`, 
      0.8656767015726473}, {-3.4891982355766635`, 
      0.8641688575299754}, {-3.490452778778468, 
      0.8626608091504936}, {-3.4917064974318555`, 
      0.861152556984888}, {-3.492959392025385, 
      0.8596441015829875}, {-3.494211463047455, 
      0.8581354434937652}, {-3.4954627109863043`, 
      0.8566265832653395}, {-3.4967131363300106`, 
      0.8551175214449753}, {-3.497962739566491, 
      0.8536082585790857}, {-3.4992115211835, 
      0.8520987952132325}, {-3.5004594816686323`, 
      0.8505891318921279}, {-3.5017066215093187`, 
      0.8490792691596358}, {-3.502952941192828, 
      0.8475692075587726}, {-3.5041984412062663`, 
      0.8460589476317087}, {-3.5054431220365756`, 
      0.8445484899197699}, {-3.506686984170535, 
      0.8430378349634382}, {-3.507930028094759, 
      0.8415269833023531}, {-3.509172254295698, 
      0.8400159354753133}, {-3.5104136632596368`, 
      0.8385046920202772}, {-3.5116542554726964`, 
      0.8369932534743647}, {-3.5128940314208306`, 
      0.835481620373858}, {-3.514132991589828, 
      0.8339697932542031}, {-3.5153711364653115`, 
      0.8324577726500106}, {-3.5166084665327357`, 
      0.8309455590950579}, {-3.51784498227739, 
      0.8294331531222887}, {-3.5190806841843947`, 
      0.827920555263816}, {-3.5203155727387037`, 
      0.8264077660509223}, {-3.5215496484251014`, 
      0.8248947860140606}, {-3.5227829117282052`, 
      0.8233816156828566}, {-3.5240153631324627`, 
      0.8218682555861089}, {-3.525247003122152, 
      0.8203547062517905}, {-3.5264778321813828`, 
      0.8188409682070505}, {-3.5277078507940933`, 
      0.8173270419782145}, {-3.528937059444053, 
      0.8158129280907862}, {-3.5301654586148596`, 
      0.8142986270694488}, {-3.5313930487899405`, 
      0.8127841394380655}, {-3.5326198304525516`, 
      0.8112694657196815}, {-3.5338458040857774`, 
      0.8097546064365246}, {-3.5350709701725296`, 
      0.8082395621100067}, {-3.5362953291955486`, 
      0.8067243332607247}, {-3.5375188816374012`, 
      0.8052089204084619}, {-3.5387416279804818`, 
      0.8036933240721892}, {-3.5399635687070106`, 
      0.8021775447700661}, {-3.541184704299035, 
      0.8006615830194419}, {-3.542405035238428, 
      0.7991454393368571}, {-3.543624562006889, 
      0.7976291142380442}, {-3.544843285085941, 
      0.7961126082379292}, {-3.546061204956933, 
      0.7945959218506325}, {-3.547278322101039, 
      0.7930790555894706}, {-3.5484946369992567`, 
      0.7915620099669562}, {-3.5497101501324075`, 
      0.7900447854948007}, {-3.5509248619811373`, 
      0.7885273826839142}, {-3.552138773025915, 
      0.7870098020444073}, {-3.553351883747032, 
      0.7854920440855921}, {-3.5545641946246036`, 
      0.7839741093159837}, {-3.555775706138566, 
      0.7824559982433003}, {-3.556986418768678, 
      0.7809377113744655}, {-3.558196332994521, 
      0.7794192492156092}, {-3.5594054492954963`, 
      0.7779006122720682}, {-3.5606137681508274`, 
      0.7763818010483878}, {-3.5618212900395583`, 
      0.774862816048323}, {-3.5630280154405534`, 
      0.7733436577748393}, {-3.5642339448324973`, 
      0.7718243267301141}, {-3.5654390786938945`, 
      0.7703048234155381}, {-3.566643417503069, 
      0.7687851483317156}, {-3.567846961738164, 
      0.7672653019784667}, {-3.569049711877142, 
      0.7657452848548274}, {-3.570251668397783, 
      0.7642250974590515}, {-3.5714528317776875`, 
      0.7627047402886116}, {-3.5726532024942714`, 
      0.7611842138401999}, {-3.5738527810247707`, 
      0.7596635186097295}, {-3.575051567846237, 
      0.7581426550923356}, {-3.5762495634355402`, 
      0.7566216237823767}, {-3.577446768269367, 
      0.7551004251734357}, {-3.57864318282422, 
      0.7535790597583206}, {-3.579838807576418, 
      0.7520575280290663}, {-3.5810336430020966`, 
      0.7505358304769353}, {-3.5822276895772065`, 
      0.7490139675924187}, {-3.583420947777514, 
      0.7474919398652377}, {-3.5846134180786007`, 
      0.7459697477843446}, {-3.585805100955862, 
      0.7444473918379236}, {-3.586995996884509, 
      0.7429248725133926}, {-3.588186106339567, 
      0.7414021902974034}, {-3.5893754297958744`, 
      0.7398793456758437}, {-3.590563967728084, 
      0.7383563391338376}, {-3.5917517206106613`, 
      0.7368331711557468}, {-3.592938688917886, 
      0.735309842225172}, {-3.59412487312385, 
      0.7337863528249539}, {-3.5953102737024576`, 
      0.7322627034371739}, {-3.596494891127426, 
      0.7307388945431559}, {-3.5976787258722833`, 
      0.7292149266234668}, {-3.5988617784103707`, 
      0.7276908001579178}, {-3.6000440492148402`, 
      0.7261665156255656}, {-3.6012255387586554`, 
      0.7246420735047133}, {-3.60240624751459, 
      0.7231174742729118}, {-3.603586175955229, 
      0.7215927184069607}, {-3.604765324552968, 
      0.7200678063829092}, {-3.6059436937800124`, 
      0.7185427386760576}, {-3.6071212841083775`, 
      0.7170175157609578}, {-3.6082980960098885`, 
      0.7154921381114153}, {-3.6094741299561797`, 
      0.7139666062004894}, {-3.6106493864186944`, 
      0.7124409205004945}, {-3.611823865868685, 
      0.7109150814830016}, {-3.612997568777212, 
      0.7093890896188391}, {-3.614170495615145, 
      0.7078629453780936}, {-3.6153426468531613`, 
      0.7063366492301114}, {-3.616514022961746, 
      0.7048102016434995}, {-3.6176846244111913`, 
      0.7032836030861264}, {-3.618854451671598, 
      0.7017568540251234}, {-3.6200235052128726`, 
      0.7002299549268859}, {-3.6211917855047293`, 
      0.6987029062570739}, {-3.622359293016688, 
      0.6971757084806135}, {-3.6235260282180763`, 
      0.6956483620616979}, {-3.6246919915780267`, 
      0.6941208674637883}, {-3.6258571835654774`, 
      0.6925932251496152}, {-3.6270216046491734`, 
      0.6910654355811792}, {-3.628185255297664, 
      0.6895374992197522}, {-3.629348135979304, 
      0.6880094165258785}, {-3.630510247162253, 
      0.6864811879593761}, {-3.631671589314475, 
      0.6849528139793369}, {-3.6328321629037386`, 
      0.6834242950441289}, {-3.633991968397617, 
      0.6818956316113963}, {-3.635151006263487, 
      0.6803668241380612}, {-3.636309276968529, 
      0.678837873080324}, {-3.637466780979726, 
      0.6773087788936651}, {-3.638623518763866, 
      0.675779542032846}, {-3.639779490787539, 0.6742501629519093}}]}, 
   {RGBColor[1, 1, 0], DiskBox[{0, 0}, 0.03]}},
  AspectRatio->Automatic,
  Background->RGBColor[0.6, 0.6, 0.6],
  ImageSize->{281.5, Automatic}]], "Output",
 CellChangeTimes->{{3.401201238934959*^9, 3.4012012492050753`*^9}, 
   3.4012769074887733`*^9}]
}, Open  ]],

Cell["\<\
We have just observed that inverse cube laws are unstable.  Planets either \
spiral into the sun or spiral away from it.\
\>", "Text"],

Cell[TextData[{
 "Try force laws that are close to inverse square laws.  Modify the code \
below (with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Vo", "=", 
    RowBox[{".7", "\[Times]", "2", " ", "\[Pi]"}]}], TraditionalForm]]],
 ") to show the cases ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    StyleBox["p",
     FontSlant->"Plain"], "=", "2.02"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    StyleBox["p",
     FontSlant->"Plain"], "=", "1.98"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tFinal", "=", "4"}], ";", 
  RowBox[{"R0", "=", "1"}], ";", 
  RowBox[{"V0", "=", 
   RowBox[{".7", "\[Times]", "2", " ", "\[Pi]"}]}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", ".001"}], ";", 
  RowBox[{"p", "=", "2.02"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CalculateOrbit", ";", 
  RowBox[{"DisplayOrbit", "[", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Clear", "[", 
  RowBox[{
  "tFinal", ",", "R0", ",", "V0", ",", "iMaximum", ",", "t", ",", "r", ",", 
   "v"}], "]"}]}], "Input",
 CellChangeTimes->{{3.401193360004507*^9, 3.401193414015883*^9}, 
   3.40119347898011*^9, 3.401193626282501*^9, {3.4012012039931593`*^9, 
   3.401201283166939*^9}}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 1, 1], Thickness[0.0075], 
    LineBox[{{1, 0}, {0.9999802607561169, 0.004398200775614183}, {
      0.999921042603504, 0.008796227909874086}, {0.9998223442792471, 
      0.013193907736603856`}, {0.9996841636784338, 0.017591066539975664`}, {
      0.9995064978540711, 0.021987530529661674`}, {0.9992893430169694, 
      0.026383125815959522`}, {0.9990326945355943, 0.030777678384882476`}, {
      0.9987365469358855, 0.03517101407320537}, {0.9984008939010437, 
      0.03956295854345745}, {0.998025728271285, 0.04395333725885314}, {
      0.9976110420435642, 0.048341975458151845`}, {0.9971568263712671, 
      0.05272869813043762}, {0.9966630715638714, 0.057113329989809815`}, {
      0.9961297670865774, 0.06149569544997535}, {0.9955569015599101, 
      0.06587561859873368}, {0.9949444627592908, 0.07025292317234494}, {
      0.9942924376145816, 0.07462743252977204}, {0.9936008122096015, 
      0.07899896962678742}, {0.992869571781616, 0.08336735698993464}, {
      0.9920987007208004, 0.08773241669033553}, {0.991288182569679, 
      0.092093970317333}, {0.9904380000225389, 0.09645183895195976}, {
      0.9895481349248216, 0.10080584314022299`}, {0.9886185682724928, 
      0.10515580286619508`}, {0.9876492802113906, 0.10950153752489998`}, {
      0.9866402500365551, 0.1138428658949851}, {0.9855914561915391, 
      0.11817960611116826`}, {0.9845028762677029, 0.122511575636449}, {
      0.9833744870034928, 0.12683859123407362`}, {0.9822062642837066, 
      0.131160468939243}, {0.9809981831387463, 0.135477024030552}, {
      0.9797502177438613, 0.1397880710011495}, {0.9784623414183824, 
      0.14409342352960725`}, {0.9771345266249493, 0.14839289445048595`}, {
      0.9757667449687335, 0.15268629572458717`}, {0.9743589671966589, 
      0.15697343840887837`}, {0.972911163196622, 0.16125413262607918`}, {
      0.9714233019967136, 0.1655281875338964}, {0.9698953517644462, 
      0.16979541129389472`}, {0.9683272798059874, 0.1740556110399905}, {
      0.9667190525654038, 0.1783085928465551}, {0.9650706356239166, 
      0.18255416169611421`}, {0.9633819936991739, 0.1867921214466298}, {
      0.9616530906445409, 0.19102227479834988`}, {0.9598838894484122, 
      0.19524442326021235`}, {0.9580743522335496, 0.1994583671157879}, {
      0.9562244402564486, 0.20366390538874687`}, {0.9543341139067373, 
      0.2078608358078349}, {0.9524033327066124, 0.21204895477134156`}, {
      0.9504320553103155, 0.2162280573110459}, {0.9484202395036544, 
      0.22039793705562247`}, {0.9463678422035735, 0.2245583861934908}, {
      0.9442748194577789, 0.22870919543509155`}, {0.9421411264444218, 
      0.23285015397457112`}, {0.9399667174718466, 0.23698104945085707`}, {
      0.9377515459784082, 0.2411016679081056}, {0.9354955645323648, 
      0.24521179375550214`}, {0.9331987248318513, 0.24931120972639562`}, {
      0.9308609777049401, 0.25339969683674635`}, {0.9284822731097958, 
      0.25747703434286734`}, {0.9260625601349297, 0.2615429996984376}, {
      0.9236017869995624, 0.2655973685107661}, {0.9210999010541009, 
      0.26963991449628455`}, {0.918556848780738, 0.2736704094352458}, {
      0.9159725757941829, 0.27768862312560466`}, {0.9133470268425309, 
      0.2816943233360578}, {0.9106801458082802, 0.28568727575821706`}, {
      0.9079718757095069, 0.28966724395789184`}, {0.9052221587012066, 
      0.2936339893254544}, {0.9024309360768125, 0.297587271025261}, {
      0.8995981482699021, 0.3015268459441019}, {0.8967237348561021, 
      0.3054524686386518}, {0.8938076345552036, 0.30936389128189196`}, {
      0.8908497852335002, 0.313260863608474}, {0.8878501239063613, 
      0.3171431328589946}, {0.884808586741054, 0.32101044372315}, {
      0.8817251090598276, 0.32486253828173745`}, {0.8785996253432752, 
      0.32869915594747023`}, {0.8754320692339883, 0.3325200334045723}, {
      0.8722223735405195, 0.3363249045471166}, {0.8689704702416703, 
      0.340113500416071}, {0.8656762904911227, 0.3438855491350143}, {
      0.8623397646224313, 0.34764077584448333`}, {0.8589608221543964, 
      0.3513789026349122}, {0.8555393917968375, 0.3550996484781217}, {
      0.8520754014567893, 0.35880272915731715`}, {0.8485687782451417, 
      0.3624878571955516}, {0.8450194484837464, 0.36615474178260865`}, {
      0.8414273377130167, 0.3698030887002594}, {0.8377923707000429, 
      0.37343260024584574`}, {0.8341144714472526, 0.37704297515414076`}, {
      0.8303935632016433, 0.3806339085174359}, {0.8266295684646149, 
      0.3842050917038024}, {0.8228224090024365, 0.3877562122734737}, {
      0.818972005857376, 0.39128695389329254`}, {0.8150782793595284, 
      0.3947969962491668}, {0.8111411491393791, 0.3982860149564735}, {
      0.8071605341411373, 0.40175368146835116`}, {0.8031363526368801, 
      0.4051996629818178}, {0.799068522241548, 0.4086236223416489}, {
      0.794956959928835, 0.4120252179419491}, {0.7908015820480179, 
      0.4154041036253484}, {0.7866023043417731, 0.41875992857975225`}, {
      0.7823590419650305, 0.42209233723257056`}, {0.7780717095049152, 
      0.4254009691423516}, {0.7737402210018351, 0.42868545888774107`}, {
      0.7693644899717691, 0.43194543595368634`}, {0.7649444294298171, 
      0.43518052461480183`}, {0.7604799519150774, 0.4383903438158098}, {
      0.7559709695169156, 0.44157450704896734`}, {0.7514173939026959, 
      0.44473262222838783`}, {0.7468191363470512, 0.44786429156116203`}, {
      0.7421761077627652, 0.45096911141518076`}, {0.7374882187333515, 
      0.4540466721835581}, {0.7327553795474138, 0.45709655814555084`}, {
      0.7279775002348782, 0.46011834732386603`}, {0.7231544906051917, 
      0.4631116113382455}, {0.7182862602875867, 0.4660759152552124}, {
      0.7133727187735173, 0.46901081743386047`}, {0.7084137754613762, 
      0.47191586936756436`}, {0.7034093397036105, 0.47479061552148344`}, {
      0.6983593208563563, 0.4776345931657287}, {0.6932636283317232, 
      0.4804473322040579}, {0.6881221716528619, 0.4832283549979589}, {
      0.6829348605119605, 0.4859771761859781}, {0.6777016048313163, 
      0.48869330249814436`}, {0.6724223148276449, 0.49137623256533614`}, {
      0.6670969010797916, 0.49402545672343195`}, {0.661725274600021, 
      0.4966404568120823}, {0.6563073469090709, 0.49922070596793316`}, {
      0.6508430301151656, 0.5017656684121274}, {0.6453322369971944, 
      0.5042747992319047}, {0.6397748810922727, 0.506747544156114}, {
      0.6341708767879152, 0.5091833393244489}, {0.6285201394190629, 
      0.5115816110502067}, {0.6228225853702183, 0.51394177557637}, {
      0.6170781321829588, 0.5162632388248007}, {0.6112866986691126, 
      0.5185453961383305}, {0.6054482050298959, 0.5207876320155261}, {
      0.5995625729813302, 0.5229893198379002}, {0.5936297258862717, 
      0.5251498215893328}, {0.587649588893409, 0.5272684875674604}, {
      0.5816220890835999, 0.5293446560867844}, {0.5755471556239437, 
      0.5313776531732419}, {0.5694247199300049, 0.5333667922499764}, {
      0.5632547158366299, 0.5353113738140372}, {0.557037079777822, 
      0.5372106851037318}, {0.5507717509761683, 0.5390639997563462}, {
      0.5444586716423386, 0.5408705774559405}, {0.53809778718521, 
      0.5426296635709242}, {0.5316890464331973, 0.5443404887811047}, {
      0.5252324018674105, 0.5460022686938967}, {0.5187278098672901, 
      0.547614203449376}, {0.5121752309694126, 0.5491754773138529}, {
      0.5055746301402008, 0.5506852582616348}, {0.49892597706331077`, 
      0.5521426975446436}, {0.4922292464425222, 0.5535469292495462}, {
      0.4854844183209979, 0.5548970698420554}, {0.478691478417838, 
      0.5561922176980525}, {0.47185041848290415`, 0.5574314526211792}, {
      0.46496123667095157`, 0.5586138353465492}, {0.45802393793616464`, 
      0.5597384070302226}, {0.4510385344482622, 0.5608041887240938}, {
      0.4440050460314054, 0.5618101808358389}, {0.4369235006272178, 
      0.5627553625735779}, {0.42979393478330546`, 0.5636386913749085}, {
      0.42261639416874947`, 0.5644591023199786}, {0.41539093411813316`, 
      0.5652155075282735}, {0.40811762020576037`, 0.5659067955388067}, {
      0.4007965288518238, 0.5665318306734205}, {0.39342774796238816`, 
      0.5670894523829207}, {0.38601137760516774`, 0.5675784745757945}, {
      0.37854753072319974`, 0.5679976849292876}, {0.3710363338886416, 
      0.5683458441826519}, {0.36347792809905866`, 0.5686216854124106}, {
      0.3558724696187127, 0.5688239132895364}, {0.3482201308675153, 
      0.5689512033184881}, {0.34052110136047303`, 0.569002201058113}, {
      0.33277558870062496`, 0.5689755213244903}, {0.32498381962865464`, 
      0.5688697473758708}, {0.3171460411325528, 0.56868343007996}, {
      0.30926252162091283`, 0.5684150870638897}, {0.3013335521636562, 
      0.568063201847347}, {0.29335944780421513`, 0.5676262229594565}, {
      0.2853405489474405, 0.567102563040161}, {0.2772772228277579, 
      0.5664905979270203}, {0.26916986506236307`, 0.5657886657285293}, {
      0.2610189012945287, 0.5649950658852785}, {0.2528247889323924, 
      0.5641080582205146}, {0.24458801898890342`, 0.5631258619819307}, {
      0.23630911802893217`, 0.5620466548768182}, {0.2279886502298835, 
      0.560868572103049}, {0.21962721956250802`, 0.5595897053787363}, {
      0.21122547209897088`, 0.5582081019738432}, {0.20278409845561554`, 
      0.5567217637474814}, {0.19430383637825013`, 0.5551286461951664}, {
      0.18578547347818286`, 0.5534266575108818}, {0.17722985012764098`, 
      0.5516136576694562}, {0.16863786252362045`, 0.549687457535481}, {
      0.16001046592962864`, 0.5476458180058003}, {0.1513486781051962, 
      0.5454864491934983}, {0.14265358293344138`, 0.5432070096622966}, {
      0.13392633425736505`, 0.5408051057213691}, {0.12516815993593003`, 
      0.5382782907917938}, {0.11638036613132642`, 0.5356240648571968}, {
      0.10756434183913441`, 0.5328398740126206}, {0.09872156367335543, 
      0.5299231101272703}, {0.0898536009184789, 0.5268711106385853}, {
      0.0809621208608661, 0.5236811584970388}, {0.07204889441174725, 
      0.5203504822832276}, {0.06311580203401913, 0.5168762565211625}, {
      0.05416483998477197, 0.5132556022142468}, {0.04519812688503508, 
      0.5094855876332246}, {0.03621791062757624, 0.5055632293884288}, {
      0.027226575632678232`, 0.5014854938219502}, {0.018226650460601772`, 
      0.4972492987589097}, {0.00922081578787406, 0.4928515156608426}, {
      0.00021191275255609543`, 0.48828897222830236`}, {-0.008797048328827365, 
      0.48355845550416254`}, {-0.017802878873040107`, 
      0.47865671553372635`}, {-0.026802202574586355`, 
      0.47358046964263023`}, {-0.03579144559128958, 
      0.468326407398624}, {-0.04476682638467541, 
      0.4628911963285869}, {-0.05372434523503788, 
      0.4572714884675399}, {-0.06265977345834217, 
      0.45146392782187156`}, {-0.07156864236059177, 
      0.4454651588344058}, {-0.08044623197509596, 
      0.43927183594419233`}, {-0.08928755963935397, 
      0.4328806343388337}, {-0.09808736848116065, 
      0.4262882620015947}, {-0.10684011589816886`, 
      0.41949147315923885`}, {-0.11553996213163747`, 
      0.41248708323922806`}, {-0.12418075905355225`, 
      0.4052719854462759}, {-0.13275603930680488`, 
      0.3978431690678834}, {-0.14125900596068275`, 
      0.3901977396159583}, {-0.1496825228685329, 
      0.3823329409064206}, {-0.15801910594101665`, 
      0.37424617917025876`}, {-0.16626091557666903`, 
      0.3659350492771809}, {-0.17439975052119927`, 
      0.3573973631361318}, {-0.18242704345765198`, 
      0.3486311803147735}, {-0.19033385866055147`, 
      0.33963484089181684`}, {-0.19811089207762977`, 
      0.3304070005210866}, {-0.20574847423161868`, 
      0.3209466676437055}, {-0.21323657636053348`, 
      0.31125324273417115`}, {-0.22056482023629187`, 
      0.3013265594069222}, {-0.22772249211651008`, 
      0.29116692714200143`}, {-0.234698561290751, 
      0.2807751753117123}, {-0.24148170367797558`, 
      0.27015269810521536`}, {-0.24806033091389876`, 
      0.2593014998558274}, {-0.2544226253327158, 
      0.24822424017799796`}, {-0.26055658119462316`, 
      0.2369242782198745}, {-0.2664500524362992, 
      0.2254057152361722}, {-0.27209080712404493`, 
      0.21367343458869748`}, {-0.27746658866728896`, 
      0.20173313819314082`}, {-0.28256518370326117`, 
      0.1895913783562117}, {-0.287374496392687, 
      0.17725558389299875`}, {-0.291882628673709, 
      0.16473407938711865`}, {-0.29607796581099505`, 
      0.15203609646228225`}, {-0.2999492663551146, 
      0.1391717759794205}, {-0.30348575540163975`, 
      0.12615216016355285`}, {-0.30667721981970625`, 
      0.11298917380265212`}, {-0.30951410391707745`, 
      0.09969559384820187}, {-0.3119876038351431, 
      0.08628500698259793}, {-0.3140897588350379, 
      0.07277175499755234}, {-0.31581353755681135`, 
      0.059170868142439784`}, {-0.317152917317313, 
      0.045497986941055286`}, {-0.3181029545665423, 
      0.03176927332560791}, {-0.31865984475048836`, 
      0.018001312281838412`}, {-0.31882097003055854`, 
      0.0042110055215819515`}, {-0.3185849335805751, -0.009584541018446514}, \
{-0.31795157951239333`, -0.023368133826407338`}, {-0.3169219978566937, \
-0.037122610431316136`}, {-0.31549851442931315`, -0.05083095772482135}, \
{-0.3136846658263641, -0.0644764277341439}, {-0.3114851601933783, \
-0.07804264868106581}, {-0.308905824785613, -0.09151372932983357}, \
{-0.3059535416614843, -0.10487435486336484`}, {-0.30263617311522634`, \
-0.118109872818534}, {-0.2989624786489428, -0.13120636794127935`}, \
{-0.2949420254034799, -0.14415072517326685`}, {-0.29058509401192734`, \
-0.15693068033637272`}, {-0.28590258181321626`, -0.16953485842311056`}, \
{-0.2809059052739327, -0.18195279971640377`}, {-0.2756069033244329, \
-0.19417497423983235`}, {-0.2700177431325687, -0.20619278527203183`}, \
{-0.26415082962735514`, -0.21799856284203328`}, {-0.2580187198580046, \
-0.22958554825489813`}, {-0.25163404304218867`, -0.24094787078064492`}, \
{-0.24500942693089747`, -0.2520805176779843}, {-0.23815743090375951`, \
-0.26297929872314296`}, {-0.2310904860140824, -0.2736408063794014}, \
{-0.22382084203118738`, -0.2840623726816504}, {-0.21636052138106138`, \
-0.29424202382900266`}, {-0.20872127976564328`, -0.3041784333835985}, \
{-0.2009145731456466, -0.31387087487088794`}, {-0.19295153070020812`, \
-0.32331917447072134`}, {-0.18484293332667925`, -0.3325236643835389}, \
{-0.17659919721297967`, -0.341485137354938}, {-0.16823036200035324`, \
-0.35020480274722615`}, {-0.15974608305332344`, -0.3586842444598104}, \
{-0.15115562736347687`, -0.36692538092236204`}, {-0.14246787263193966`, \
-0.37493042731605214`}, {-0.13369130909984847`, -0.3827018601187737}, \
{-0.1248340437248346, -0.39024238401984146`}, {-0.1159038063329109, \
-0.39755490120765175`}, {-0.10690795740783687`, -0.4046424829995142}, \
{-0.09785349721296338, -0.4115083437555615}, {-0.08874707597289028, \
-0.41815581699750665`}, {-0.0795950048733819, -0.4245883336372488}, \
{-0.07040326766743185, -0.4308094022091585}, {-0.061177532702852075`, \
-0.4368225909925878}, {-0.051923165212107575`, -0.4426315119070879}, \
{-0.04264523972824748, -0.44823980606138664`}, {-0.03334855251169315, \
-0.45365113083786357`}, {-0.024037633891386426`, -0.4588691483965975}, \
{-0.01471676044046824, -0.4638975154866701}, {-0.0053899669213716625`, \
-0.4687398744569529}, {0.003938942051889828, -0.47339984536380814`}, {
      0.01326638067523019, -0.4778810190787694}, {
      0.022588970971077767`, -0.4821869513051507}, {
      0.03190353218298704, -0.48632115741850945`}, {
      0.041207070564111395`, -0.4902871080518468}, {
      0.05049676956935564, -0.4940882253522759}, {
      0.05976998045597475, -0.4977278798415541}, {
      0.06902421329318054, -0.501209387818312}, {
      0.07825712837785419, -0.5045360092449874}, {
      0.0874665280506422, -0.5077109460673553}, {
      0.09665034890444889, -0.5107373409191391}, {
      0.10580665437555281`, -0.5136182761684702}, {
      0.11493362770619725`, -0.5163567732669498}, {
      0.12402956526647546`, -0.5189557923657593}, {
      0.13309287022259625`, -0.5214182321666677}, {
      0.14212204653812704`, -0.5237469299789214}, {
      0.1511156932945294, -0.5259446619558705}, {
      0.1600724993171906, -0.5280141434878219}, {
      0.16899123809318345`, -0.5299580297300008}, {
      0.17787076296712892`, -0.531778916246698}, {
      0.18671000260176973`, -0.5334793397546612}, {
      0.19550795669016896`, -0.5350617789505973}, {
      0.20426369190680907`, -0.5365286554092866}, {
      0.21297633808527028`, -0.5378823345402894}, {
      0.22164508461060123`, -0.5391251265925626}, {
      0.23026917701494956`, -0.5402592876975106}, {
      0.2388479137654876, -0.5412870209420831}, {
      0.24738064323414188`, -0.5422104774645092}, {
      0.25586676083910953`, -0.5430317575661392}, {
      0.2643057063486155, -0.5437529118336555}, {
      0.2726969613378279, -0.5443759422666234}, {
      0.2810400467903018, -0.5449028034059875}, {
      0.28933452083576433`, -0.545335403459689}, {
      0.29757997661648006`, -0.5456756054220882}, {
      0.3057760402748499, -0.5459252281843303}, {
      0.3139223690552925, -0.5460860476331995}, {
      0.32201864951383885`, -0.5461597977363636}, {
      0.3300645958292335, -0.5461481716122396}, {
      0.3380599482096859, -0.5460528225829898}, {
      0.34600447138974355`, -0.5458753652094155}, {
      0.35389795321207584`, -0.5456173763067388}, {
      0.36174020328925527`, -0.5452803959404621}, {
      0.36953105174090656`, -0.5448659284016697}, {
      0.37727034800186277`, -0.5443754431612917}, {
      0.384957959697221, -0.5438103758029854}, {
      0.3925937715804316, -0.5431721289344092}, {
      0.40017768453077934`, -0.5424620730767689}, {
      0.40770961460683136`, -0.5416815475326049}, {
      0.41518949215262724`, -0.540831861231873}, {
      0.42261726095357627`, -0.5399142935564324}, {
      0.4299928774392071, -0.5389300951431217}, {
      0.4373163099300837, -0.5378804886656481}, {
      0.44458753792635874`, -0.5367666695955601}, {
      0.45180655143558773`, -0.5355898069426125}, {
      0.4589733503375645, -0.5343510439748593}, {
      0.46608794378407326`, -0.5330514989188373}, {
      0.4731503496315764, -0.5316922656402242}, {
      0.48016059390497196`, -0.5302744143053706}, {
      0.48711871029066783`, -0.528798992024118}, {
      0.4940247396573198, -0.5272670234743255}, {
      0.5008787296026797, -0.5256795115085346}, {
      0.5076807340250898, -0.5240374377432044}, {
      0.5144308127182464, -0.5223417631309548}, {
      0.521129030987934, -0.5205934285162527}, {
      0.5277754592895092, -0.5187933551749768}, {
      0.5343701728849849, -0.5169424453382944}, {
      0.5409132515186275, -0.5150415827012772}, {
      0.5474047791100494, -0.5130916329166822}, {
      0.5538448434638332, -0.5110934440743148}, {
      0.5602335359947803, -0.5090478471663858}, {
      0.5665709514679294, -0.5069556565392689}, {
      0.57285718775254, -0.5048176703320538}, {
      0.5790923455892808, -0.5026346709022868}, {
      0.5852765283699054, -0.5004074252392773}, {
      0.5914098419287411, -0.49813668536534583`}, {
      0.5974923943453527, -0.49582318872537506`}, {
      0.6035242957577791, -0.49346765856502023`}, {
      0.6095056581857765, -0.4910708042979246}, {
      0.6154365953635315, -0.4886333218622773}, {
      0.6213172225813379, -0.48615589406704185`}, {
      0.6271476565357614, -0.4836391909281739}, {
      0.6329280151878387, -0.48108386999513963`}, {
      0.6386584176288873, -0.4784905766680355}, {
      0.6443389839535222, -0.47585994450560265`}, {
      0.6499698351394998, -0.47319259552442006`}, {
      0.6555510929340292, -0.47048914048955265`}, {
      0.6610828797462122, -0.4677501791969213}, {
      0.6665653185452887, -0.4649763007476553}, {
      0.671998532764387, -0.46216808381467744`}, {
      0.6773826462094891, -0.459326096901767}, {
      0.6827177829733396, -0.45645089859533555`}, {
      0.6880040673540441, -0.45354303780914507`}, {
      0.6932416237781088, -0.45060305402218986`}, {
      0.6984305767276969, -0.4476314775099565}, {
      0.7035710506718788, -0.44462882956927}, {
      0.708663170001673, -0.44159562273692676`}, {
      0.7137070589686823, -0.4385323610023091}, {
      0.7187028416271376, -0.43543954001417}, {
      0.7236506417791777, -0.43231764728176986`}, {
      0.7285505829231966, -0.4291671623705414}, {
      0.7334027882051019, -0.4259885570924543}, {
      0.7382073803723349, -0.42278229569124326`}, {
      0.7429644817305125, -0.41954883502266027`}, {
      0.7476742141025543, -0.4162886247299048}, {
      0.7523366987901703, -0.41300210741438187`}, {
      0.7569520565375878, -0.40968971880193195`}, {
      0.7615204074974036, -0.4063518879046729}, {
      0.7660418711984524, -0.40298903717858914`}, {
      0.7705165665155899, -0.39960158267699836`}, {
      0.7749446116412918, -0.39618993420002324`}, {
      0.7793261240589772, -0.3927544954401895}, {
      0.7836612205179673, -0.38929566412426925`}, {
      0.7879500170099978, -0.38581383215148446`}, {
      0.7921926287472041, -0.38230938572818046`}, {
      0.796389170141505, -0.3787827054990774}, {
      0.8005397547853137, -0.37523416667520315`}, {
      0.8046444954335072, -0.3716641391586081}, {
      0.8087035039865917, -0.3680729876639584}, {
      0.8127168914750004, -0.3644610718371025}, {
      0.8166847680444671, -0.36082874637070117`}, {
      0.8206072429424208, -0.35717636111700896`}, {
      0.8244844245053471, -0.35350426119789313`}, {
      0.8283164201470676, -0.3498127871121715}, {
      0.8321033363478907, -0.3461022748403493}, {
      0.8358452786445861, -0.3423730559468324}, {
      0.8395423516211437, -0.33862545767969204`}, {
      0.8431946589002722, -0.33485980306805235`}, {
      0.8468023031356029, -0.3310764110171721}, {
      0.8503653860045578, -0.3272755964012877}, {
      0.85388400820185, -0.32345767015428356`}, {
      0.8573582694335807, -0.31962293935825326`}, {
      0.8607882684119039, -0.3157717073300138}, {
      0.864174102850225, -0.311904273705632}, {
      0.8675158694589081, -0.30802093452302143`}, {
      0.8708136639414626, -0.3041219823026658}, {
      0.8740675809911833, -0.3002077061265235}, {
      0.87727771428822, -0.2962783917151654}, {
      0.8804441564970527, -0.29233432150319805`}, {
      0.8835669992643497, -0.2883757747130205}, {
      0.886646333217188, -0.284403027426964}, {
      0.8896822479616147, -0.2804163526578605}, {
      0.8926748320815308, -0.2764160204180854}, {
      0.8956241731378791, -0.2724022977871179}, {
      0.8985303576681184, -0.26837544897766263`}, {
      0.9013934711859668, -0.26433573540037225`}, {
      0.9042135981813996, -0.2602834157272123}, {
      0.9069908221208853, -0.2562187459535066}, {
      0.9097252254478464, -0.2521419794587008}, {
      0.9124168895833302, -0.24805336706588146`}, {
      0.915065894926878, -0.24395315710008494`}, {
      0.9176723208575784, -0.2398415954454319}, {
      0.9202362457352955, -0.23571892560112007`}, {
      0.9227577469020577, -0.23158538873630818`}, {
      0.9252369006835987, -0.22744122374392287`}, {
      0.9276737823910396, -0.22328666729341878`}, {
      0.9300684663227028, -0.21912195388252237`}, {
      0.9324210257660474, -0.214947315887988}, {
      0.9347315329997188, -0.21076298361539464`}, {
      0.9370000592957033, -0.20656918534801103`}, {
      0.9392266749215797, -0.20236614739475528`}, {
      0.9414114491428606, -0.19815409413727558`}, {
      0.9435544502254157, -0.19393324807617696`}, {
      0.9456557454379715, -0.1897038298764186}, {
      0.9477154010546788, -0.18546605841190592`}, {
      0.9497334823577439, -0.18122015080930062`}, {
      0.9517100536401164, -0.1769663224910712}, {
      0.9536451782082281, -0.1727047872178063}, {
      0.9555389183847778, -0.16843575712981232`}, {
      0.957391335511557, -0.16415944278801614`}, {
      0.9592024899523118, -0.15987605321419374`}, {
      0.9609724410956362, -0.15558579593054409`}, {
      0.9627012473578918, -0.1512888769986284}, {
      0.9643889661861514, -0.14698550105769306`}, {
      0.9660356540611612, -0.1426758713623953}, {
      0.967641366500318, -0.13836018981994905`}, {
      0.9692061580606582, -0.13403865702670925`}, {
      0.9707300823418564, -0.12971147230421112`}, {
      0.9722131919892268, -0.1253788337346817}, {
      0.9736555386967295, -0.12104093819603973`}, {
      0.9750571732099738, -0.11669798139640017`}, {
      0.97641814532922, -0.11235015790809873`}, {
      0.9777385039123742, -0.1079976612012519}, {
      0.9790182968779749, -0.10364068367686745`}, {
      0.9802575712081698, -0.0992794166995199}, {
      0.9814563729516786, -0.09491405062960548}, {
      0.982614747226742, -0.09054477485519058}, {
      0.9837327382240535, -0.08617177782346727}, {
      0.9848103892096729, -0.08179524707182954}, {
      0.98584774252792, -0.07741536925858343}, {
      0.9868448396042453, -0.07303233019330387}, {
      0.9878017209480792, -0.06864631486685104}, {
      0.9887184261556546, -0.06425750748105867}, {
      0.9895949939128055, -0.05986609147810649}, {
      0.9904314619977366, -0.05547224956958878}, {
      0.9912278672837659, -0.05107616376529093}, {
      0.9919842457420375, -0.04667801540168545}, {
      0.9927006324442037, -0.04227798517015897}, {
      0.9933770615650769, -0.037876253144981276`}, {
      0.994013566385248, -0.03347299881102759}, {
      0.9946101792936735, -0.029068401091264767`}, {
      0.9951669317902281, -0.024662638374012252`}, {
      0.9956838544882238, -0.02025588853998824}, {
      0.9961609771168942, -0.015848328989151506`}, {
      0.9965983285238433, -0.011440136667349077`}, {
      0.9969959366774594, -0.007031488092779971}, {
      0.9973538286692929, -0.0026225593822848886`}, {0.9976720307163973, 
      0.0017864737225281894`}, {0.9979505681636349, 0.006195435829312764}, {
      0.9981894654859443, 0.010604151869157287`}, {0.9983887462905726, 
      0.015012447070893758`}, {0.9985484333192696, 0.019420146935532005`}, {
      0.9986685484504452, 0.02382707721081025}, {0.9987491127012904, 
      0.028233063865852677`}, {0.998790146229861, 0.03263793306592476}, {
      0.9987916683371241, 0.0370415111472772}, {0.9987536974689687, 
      0.04144362459206937}, {0.9986762512181794, 0.04584410000336327}, {
      0.9985593463263738, 0.050242764080178894`}, {0.9984029986859044, 
      0.05463944359260221}, {0.9982072233417247, 0.059033965356936724`}, {
      0.9979720344932196, 0.0634261562108898}, {0.9976974454960019, 
      0.06781584298878492}, {0.9973834688636743, 0.07220285249679095}, {
      0.997030116269557, 0.07658701148815983}, {0.9966373985483826, 
      0.0809681466384635}, {0.9962053256979592, 0.08534608452082172}, {
      0.9957339068807997, 0.08972065158111164}, {0.9952231504257217, 
      0.09409167411315049}, {0.9946730638294164, 0.09845897823384248}, {
      0.9940836537579876, 0.10282238985828114`}, {0.9934549260484619, 
      0.10718173467479827`}, {0.9927868857102718, 0.11153683811995047`}, {
      0.9920795369267117, 0.11588752535343443`}, {0.991332883056367, 
      0.1202336212329221}, {0.9905469266345202, 0.12457495028880654`}, {
      0.9897216693745319, 0.12891133669884947`}, {0.9888571121692004, 
      0.13324260426272147`}, {0.9879532550920991, 0.13756857637642553`}, {
      0.9870100973988951, 0.14188907600659476`}, {0.9860276375286485, 
      0.14620392566465493`}, {0.9850058731050947, 0.15051294738084242`}, {
      0.9839448009379113, 0.154815962678068}, {0.9828444170239711, 
      0.15911279254561694`}, {0.9817047165485825, 0.1634032574126758}, {
      0.9805256938867191, 0.1676871771216759}, {0.9793073426042403, 
      0.17196437090144373`}, {0.9780496554591059, 0.17623465734014812`}, {
      0.9767526244025837, 0.18049785435803425`}, {0.9754162405804557, 
      0.18475377917993385`}, {0.9740404943342228, 0.18900224830754137`}, {
      0.97262537520231, 0.19324307749144554`}, {0.971170871921276, 
      0.19747608170290548`}, {0.9696769724270285, 0.20170107510536042`}, {
      0.9681436638560476, 0.20591787102566217`}, {0.9665709325466195, 
      0.2101262819250187}, {0.964958764040086, 0.21432611936963786`}, {
      0.9633071430821074, 0.21851719400105912`}, {0.9616160536239475, 
      0.22269931550616187`}, {0.9598854788237792, 0.22687229258683797`}, {
      0.9581154010480162, 0.23103593292931643`}, {0.956305801872674, 
      0.23519004317312764`}, {0.9544566620847625, 0.23933442887969442`}, {
      0.9525679616837146, 0.24346889450053685`}, {0.9506396798828544, 
      0.24759324334507782`}, {0.948671795110909, 0.2517072775480355}, {
      0.946664285013568, 0.2558107980363893}, {0.9446171264550935, 
      0.25990360449590466`}, {0.9425302955199876, 0.2639854953372031}, {
      0.9404037675147193, 0.26805626766136204`}, {0.9382375169695178, 
      0.2721157172250299}, {0.9360315176402353, 0.27616363840504116`}, {
      0.9337857425102849, 0.280199824162515}, {0.9315001637926607, 
      0.2842240660064225}, {0.9291747529320428, 0.2882361539566054}, {
      0.9268094806069952, 0.29223587650622906`}, {0.9244043167322619, 
      0.29622302058365413`}, {0.9219592304611671, 0.3001973715137073}, {
      0.9194741901881276, 0.30415871297833463`}, {0.9169491635512819, 
      0.3081068269766181}, {0.9143841174352452, 0.31204149378413704`}, {
      0.9117790179739972, 0.3159624919116542}, {0.9091338305539097, 
      0.3198695980631081}, {0.9064485198169231, 0.3237625870928894}, {
      0.9037230496638798, 0.32764123196238176`}, {0.9009573832580232, 
      0.33150530369574516`}, {0.8981514830286719, 0.3353545713349198}, {
      0.8953053106750778, 0.33918880189382816`}, {0.89241882717048, 
      0.34300776031175156`}, {0.8894919927663625, 0.34681120940585786`}, {
      0.8865247669969276, 0.35059890982285596`}, {0.8835171086837981, 
      0.3543706199897514}, {0.8804689759409564, 0.35812609606367796`}, {
      0.8773803261799368, 0.36186509188077837`}, {0.8742511161152812, 
      0.36558735890410715`}, {0.8710813017702727, 0.3692926461705273}, {
      0.8678708384829613, 0.3729807002365726}, {0.8646196809124962, 
      0.37665126512324526`}, {0.8613277830457793, 0.38030408225971946`}, {
      0.8579950982044579, 0.38393889042591867`}, {0.8546215790522705, 
      0.38755542569393514`}, {0.8512071776027664, 0.3911534213682583}, {
      0.8477518452274139, 0.3947326079247781}, {0.8442555326641193, 
      0.3982927129485284}, {0.840718190026175, 0.4018334610701343}, {
      0.8371397668116571, 0.40535457390092633`}, {0.8335202119132963, 
      0.4088557699666836}, {0.8298594736288422, 0.41233676463996616`}, {
      0.8261574996719468, 0.4157972700709966}, {0.8224142371835923, 
      0.4192369951170493}, {0.8186296327440862, 0.42265564527030397`}, {
      0.8148036323856553, 0.42605292258411975`}, {0.810936181605664, 
      0.42942852559768435`}, {0.8070272253804869, 0.43278214925899117`}, {
      0.8030767081800684, 0.4361134848460964}, {0.7990845739832008, 
      0.4394222198866062}, {0.7950507662935549, 0.44270803807534265`}, {
      0.7909752281564997, 0.44597061919013603`}, {0.7868579021767476, 
      0.44920963900568844`}, {0.7826987305368646, 0.4524247692054532}, {
      0.7784976550166873, 0.4556156772914719}, {0.7742546170136884, 
      0.45878202649210936`}, {0.7699695575643357, 0.46192347566762565`}, {
      0.765642417366494, 0.46503967921352096`}, {0.7612731368029153, 
      0.4681302869615886}, {0.756861655965873, 0.47119494407860857`}, {
      0.7524079146829913, 0.4742332909626122}, {0.7479118525443289, 
      0.47724496313664605`}, {0.743373408930775, 0.4802295911399617}, {
      0.7387925230438205, 0.4831868004165545}, {0.7341691339367705, 
      0.486116211200973}, {0.7295031805474665, 0.4890174384013182}, {
      0.7247946017325894, 0.491890091479349}, {0.7200433363036206, 
      0.49473377432760635`}, {0.71524932306454, 0.4975480851434694}, {
      0.710412500851343, 0.5003326163000493}, {0.7055328085734658, 
      0.5030869542138285}, {0.7006101852572095, 0.505810679208945}, {
      0.6956445700912602, 0.5085033653780235}, {0.6906359024744055, 
      0.5111645804394482}, {0.6855841220655546, 0.5137938855909694}, {
      0.6804891688361735, 0.5163908353595349}, {0.6753509831252521, 
      0.5189549774472307}, {0.6701695056969272, 0.5214858525732147}, {
      0.6649446778008916, 0.5239829943115198}, {0.6596764412357238, 
      0.5264459289246053}, {0.654364738415284, 0.5288741751925219}, {
      0.6490095124383253, 0.5312672442375636}, {0.6436107071614801, 
      0.533624639344264}, {0.6381682672757872, 0.5359458557745991}, {
      0.6326821383869368, 0.5382303805782492}, {0.627152267099416, 
      0.5404776923977705}, {0.6215786011047524, 0.5426872612685216}, {
      0.6159610892740572, 0.5448585484131854}, {0.6102996817550863, 
      0.5469910060307223}, {0.6045943300740444, 0.5490840770795867}, {
      0.5988449872423728, 0.5511371950550311}, {0.5930516078687728, 
      0.5531497837603205}, {0.5872141482767291, 0.5551212570716724}, {
      0.581332566627816, 0.5570510186967325}, {0.5754068230510772, 
      0.5589384619263931}, {0.5694368797787956, 0.5607829693797524}, {
      0.5634227012889778, 0.5625839127420099}, {0.5573642544548998, 
      0.5643406524950886}, {0.5512615087020816, 0.5660525376407642}, {
      0.5451144361730733, 0.5677189054160838}, {0.5389230119004614, 
      0.5693390810008429}, {0.532687213988524, 0.57091237721689}, {
      0.526407023803988, 0.5724380942190207}, {0.5200824261763682, 
      0.573915519177216}, {0.5137134096083931, 0.5753439259499782}, {
      0.5072999664970511, 0.5767225747485094}, {0.5008420933658234, 
      0.5780507117914737}, {0.4943397911086982, 0.5793275689500805}, {
      0.4877930652465971, 0.5805523633832214}, {0.4812019261968812, 
      0.5817242971623874}, {0.4745663895566398, 0.5828425568860939}, {
      0.46788647640050895`, 0.5839063132835324}, {0.4611622135938057, 
      0.5849147208071701}, {0.45439363412181416`, 0.5858669172140137}, {
      0.44758077743610347`, 0.5867620231352539}, {0.4407236898188127, 
      0.5875991416340066}, {0.43382242476588984`, 0.5883773577508676}, {
      0.4268770433903305, 0.5890957380369998}, {0.4198876148465242, 
      0.5897533300744742}, {0.4128542167768796, 0.590349161983593}, {
      0.40577693578197027`, 0.5908822419169265}, {0.3986558679155157, 
      0.591351557539806}, {0.39149111920558965`, 0.5917560754970254}, {
      0.38428280620353067`, 0.5920947408655165}, {0.37703105656211783`, 
      0.5923664765927783}, {0.369736009644666, 0.5925701829208606}, {
      0.3623978171667959, 0.5927047367957236}, {0.35501664387273624`, 
      0.5927689912618214}, {0.34759266824812807`, 0.5927617748417894}, {
      0.340126083271418, 0.592681890901147}, {0.3326170972060515, 
      0.5925281169979723}, {0.3250659344358097, 0.5922992042175498}, {
      0.3174728363457739, 0.5919938764920464}, {0.3098380622515469, 
      0.5916108299053312}, {0.30216189037952157`, 0.5911487319831265}, {
      0.2944446189011479, 0.5906062209687533}, {0.28668656702432915`, 
      0.5899819050848274}, {0.2788880761452588, 0.5892743617813598}, {
      0.27104951106420905`, 0.5884821369708317}, {0.26317126126898466`, 
      0.5876037442509416}, {0.25525374228997405`, 0.5866376641158668}, {
      0.2472973971309583, 0.5855823431570423}, {0.23930269778007798`, 
      0.5844361932546432}, {0.23127014680561017`, 0.5831975907611604}, {
      0.2232002790414719, 0.5818648756786844}, {0.21509366336764268`, 
      0.5804363508317695}, {0.20695090459098714`, 0.578910281038033}, {
      0.1987726454322593, 0.5772848922789607}, {0.19055956862538226`, 
      0.5755583708737433}, {0.18231239913542022`, 0.5737288626593607}, {
      0.17403190650199393`, 0.5717944721805716}, {0.16571890731523345`, 
      0.5697532618939446}, {0.15737426783171354`, 0.5676032513906164}, {
      0.14899890673817434`, 0.565342416643053}, {0.14059379807119124`, 
      0.562968689281759}, {0.13215997430132015`, 0.5604799559086154}, {
      0.12369852959060475`, 0.5578740574543378}, {0.1152106232326851, 
      0.5551487885884508}, {0.10669748328508871`, 0.5523018971911671}, {
      0.09816041040360815, 0.5493310838976583}, {0.08960078188896727, 
      0.5462340017264145}, {0.08102005595624089, 0.5430082558047225}, {
      0.0724197762377111, 0.5396514032057567}, {0.06380157653000393, 
      0.5361609529133881}, {0.05516718579643873, 0.5325343659325785}, {
      0.0465184334355225, 0.5287690555651614}, {0.03785725482641181, 
      0.5248623878729199}, {0.029185697161924137`, 0.5208116823521765}, {
      0.020505925579280666`, 0.5166142128466116}, {0.011820229598173903`, 
      0.5122672087277524}, {0.0031310298749402596`, 
      0.5077678563755132}, {-0.005559114719459978, 
      0.5031133009943499}, {-0.01424749969134114, 
      0.4983006488040091}, {-0.022931267181505594`, 
      0.49332696964751244`}, {-0.031607397885380216`, 
      0.48818930006292527`}, {-0.04027270263634277, 
      0.4828846468695997}, {-0.04892381365572398, 
      0.47740999132395423`}, {-0.05755717547683221, 
      0.471762293904426}, {-0.06616903555494053, 
      0.4659384997899839}, {-0.07475543458059891, 
      0.45993554510147416`}, {-0.08331219651999364, 
      0.45375036398002594`}, {-0.09183491841348501, 
      0.44737989658170074`}, {-0.10031895997203429`, 
      0.44082109807242004`}, {-0.10875943302110432`, 
      0.43407094871183327`}, {-0.11715119085291399`, 
      0.4271264651190327}, {-0.12548881756076816`, 
      0.41998471281669647`}, {-0.13376661744369106`, 
      0.4126428201531168}, {-0.14197860458586842`, 
      0.4050979937033705}, {-0.150118492733536, 
      0.3973475352512948}, {-0.1581796856119952, 
      0.389388860452562}, {-0.16615526784739862`, 
      0.3812195192755711}, {-0.17403799668178546`, 
      0.37283721831059635`}, {-0.18182029469543207`, 
      0.36423984502810614`}, {-0.1894942437777002, 
      0.3554254940537743}, {-0.19705158061587438`, 
      0.34639249550980633`}, {-0.2044836940004985, 
      0.33713944544909163`}, {-0.21178162427479688`, 
      0.32766523837967654`}, {-0.2189360652840451, 
      0.31796910184142596`}, {-0.2259373692071675, 
      0.3080506329538674}, {-0.23277555467606012`, 
      0.29790983680353983`}, {-0.23944031860659454`, 
      0.2875471664803078}, {-0.24592105217710392`, 
      0.27696356450489745`}, {-0.2522068613932964, 
      0.26616050531450863`}, {-0.2582865926706685, 
      0.2551400383903253}, {-0.26414886384413067`, 
      0.24390483152114664`}, {-0.26978210097716443`, 
      0.23245821360286567`}, {-0.2751745812869257, 
      0.22080421627649838`}, {-0.2803144824250295, 
      0.20894761361104697`}, {-0.28518993825447475`, 
      0.19689395894560024`}, {-0.2897891011401376, 
      0.18464961792246412`}, {-0.2941002106232671, 
      0.17222179667521967`}, {-0.2981116681804578, 
      0.1596185640884299}, {-0.3018121175771326, 
      0.14684886702557587`}, {-0.3051905301188138, 
      0.1339225374349731}, {-0.3082362938864045, 
      0.12085029029569644`}, {-0.31093930582219004`, 
      0.10764371146171674`}, {-0.313290065320876, 
      0.09431523460573804}, {-0.3152797677856612, 
      0.08087810665568708}, {-0.3169003964449451, 
      0.06734634135484327}, {-0.3181448106027967, 
      0.05373466085654957}, {-0.31900682842708555`, 
      0.04005842557840282}, {-0.3194813023728548, 
      0.026333552877693872`}, {-0.31956418540212184`, 
      0.01257642545574797}, {-0.3192525862983386, -0.0011962092623274984`}, \
{-0.3185448125835339, -0.0149673472113277}, {-0.3174403998234385, \
-0.028719841398584328`}, {-0.3159401264408425, -0.04243651919482779}, \
{-0.3140460135361184, -0.05610030100648211}, {-0.31176130961905635`, \
-0.06969431806594552}, {-0.30909046056860956`, -0.08320202709377622}, \
{-0.3060390655368831, -0.09660731968757041}, {-0.30261381988160263`, \
-0.10989462447107742`}, {-0.29882244653049467`, -0.12304900028351143`}, \
{-0.29467361743806936`, -0.13605621898911793`}, {-0.29017686698106515`, \
-0.1489028368242982}, {-0.2853424992488319, -0.16157625355622604`}, \
{-0.280181491219423, -0.17406475908527824`}, {-0.2747053937756389, \
-0.1863575674674701}, {-0.26892623241576297`, -0.19844483864851986`}, \
{-0.26285640936191107`, -0.21031768847728274`}, {-0.25650860857702684`, \
-0.221968187795612}, {-0.24989570498239846`, -0.23338935158017401`}, \
{-0.24303067893363206`, -0.24457511923851888`}, {-0.23592653677569575`, \
-0.2555203272386949}, {-0.22859623806677007`, -0.266220675282997}, \
{-0.22105262984410837`, -0.27667268722771515`}, {-0.21330838810881256`, \
-0.2868736679086094}, {-0.20537596653426732`, -0.29682165696327906`}, \
{-0.19726755225704373`, -0.30651538065353257`}, {-0.18899502848991984`, \
-0.3159542025897731}, {-0.1805699436035486, -0.3251380741510695}, \
{-0.17200348625456233`, -0.3340674852839199}, {-0.16330646609120497`, \
-0.3427434162537504}, {-0.15448929954020818`, -0.35116729081904574`}, \
{-0.14556200016767173`, -0.35934093120095356`}, {-0.13653417310928104`, \
-0.3672665151327476}, {-0.1274150130785274, -0.37494653519455345`}, \
{-0.11821330548315309`, -0.38238376056956197`}, {-0.10893743020756011`, \
-0.3895812012985043}, {-0.09959536765044488, -0.39654207505904493`}, \
{-0.09019470664080462, -0.4032697764553506}, {-0.08074265389035783, \
-0.40976784876966554`}, {-0.07124604467526198, -0.41603995810142824`}, \
{-0.061711354473971324`, -0.4220898697994568}, {-0.052144711320550395`, \
-0.4279214270781515}, {-0.04255190866331876, -0.4335385316987186}, \
{-0.03293841854707682, -0.4389451265903491}, {-0.023309404963200383`, \
-0.4441451802834178}, {-0.013669737235538774`, -0.44914267302648037`}, \
{-0.004024003331327696, -0.4539415844606174}, {
      0.005623476994691621, -0.4585458827280268}, {
      0.015268639297958032`, -0.4629595148963171}, {
      0.024907661843837896`, -0.46718639858535943`}, {
      0.03453695335682658, -0.47123041468954874`}, {
      0.04415314115545886, -0.47509540109466825`}, {
      0.053753059699610445`, -0.4787851472950715}, {
      0.06333373956870517, -0.48230338982344123`}, {
      0.07289239688242805, -0.4856538084118447}, {
      0.08242642316973879, -0.4888400228090924}, {
      0.09193337568715046, -0.4918655901854607}, {
      0.10141096818326122`, -0.4947340030616109}, {
      0.11085706210329183`, -0.49744868770399975`}, {
      0.12026965822478736`, -0.5000130029342125}, {
      0.12964688871359842`, -0.5024302393044491}, {
      0.138987009587685, -0.5047036185958599}, {
      0.14828839357511428`, -0.5068362936005609}, {
      0.15754952335178987`, -0.5088313481519751}, {
      0.16676898514389984`, -0.5106917973716508}, {
      0.17594546267975708`, -0.5124205881039281}, {
      0.1850777314755873, -0.5140205995127567}, {
      0.194164653439861, -0.5154946438176553}, {
      0.20320517178093722`, -0.5168454671482355}, {
      0.21219830620306074`, -0.5180757504989318}, {
      0.22114314837611054`, -0.5191881107675821}, {
      0.23003885766491625`, -0.5201851018633213}, {
      0.23888465710442353`, -0.5210692158708897}, {
      0.24767982960748947`, -0.5218428842599337}, {
      0.25642371439260975`, -0.5225084791292136}, {
      0.2651157036194142, -0.5230683144768252}, {
      0.27375523922030864`, -0.5235246474886213}, {
      0.2823418099171802, -0.5238796798379823}, {
      0.2908749484126204, -0.5241355589909501}, {
      0.29935422874564366`, -0.5242943795115118}, {
      0.3077792638023956, -0.5243581843625127}, {
      0.3161497029728426, -0.5243289661982923}, {
      0.32446522994491767`, -0.5242086686456897}, {
      0.3327255606280631, -0.5239991875705514}, {
      0.340930441198557, -0.5237023723273124}, {
      0.34907964625943794`, -0.5233200269896047}, {
      0.35717297710825086`, -0.5228539115601932}, {
      0.3652102601062269, -0.5223057431588395}, {
      0.3731913451428794, -0.5216771971869616}, {
      0.38111610419034947`, -0.5209699084681966}, {
      0.3889844299421694, -0.520185472364178}, {
      0.396796234531425, -0.5193254458650232}, {
      0.40455144832359813`, -0.5183913486541859}, {
      0.4122500187796521, -0.5173846641474701}, {
      0.4198919093851871, -0.5163068405061191}, {
      0.42747709864174516`, -0.5151592916240043}, {
      0.4350055791165794, -0.5139433980890248}, {
      0.4424773565474244, -0.5126605081189102}, {
      0.4498924489990151, -0.5113119384716855}, {
      0.45725088606829595`, -0.5098989753311137}, {
      0.4645527081354508, -0.508422875167479}, {
      0.47179796565805354`, -0.5068848655741168}, {
      0.47898671850580693`, -0.5052861460801271}, {
      0.4861190353334873, -0.5036278889397352}, {
      0.49319499298985964`, -0.5019112398987878}, {
      0.5002146759604598, -0.5001373189388877}, {
      0.5071781758422712, -0.49830722099968183`}, {
      0.514085590848439, -0.4964220166798302}, {
      0.5209370253412788, -0.49448275291718535`}, {
      0.5277325893919419, -0.4924904536487173}, {
      0.5344723983651947, -0.4904461204507191}, {
      0.5411565725278676, -0.48835073315982663`}, {
      0.5477852366796101, -0.486205250475382}, {
      0.5543585198046718, -0.4840106105436677}, {
      0.5608765547435083, -0.4817677315245295}, {
      0.567339477883078, -0.47947751214090134`}, {
      0.5737474288647656, -0.4771408322117349}, {
      0.5801005503089306, -0.4747585531688306}, {
      0.5863989875551391, -0.47233151855805366`}, {
      0.5926428884171904, -0.46986055452541237`}, {
      0.5988324029521049, -0.46734647028846155`}, {
      0.6049676832422872, -0.4647900585934866}, {
      0.6110488831901232, -0.46219209615891027`}, {
      0.6170761583243161, -0.45955334410535437`}, {
      0.623049665617304, -0.45687454837277636`}, {
      0.628969563313141, -0.4541564401250902}, {
      0.6348360107652593, -0.4513997361426693}, {
      0.6406491682835632, -0.44860513920311795`}, {
      0.6464091969903383, -0.4457733384506864}, {
      0.6521162586844873, -0.4429050097546945}, {
      0.6577705157136334, -0.4400008160573167}, {
      0.6633721308536565, -0.4370614077110716}, {
      0.6689212671952549, -0.43408742280634754`}, {
      0.6744180880371449, -0.4310794874892873}, {
      0.6798627567855359, -0.42803821627034233`}, {
      0.6852554368595365, -0.4249642123237989}, {
      0.6905962916021672, -0.4218580677785688}, {
      0.6958854841966747, -0.41872036400052676`}, {
      0.7011231775878585, -0.41555167186666797`}, {
      0.7063095344081355, -0.4123525520313517}, {
      0.7114447169080876, -0.409123555184885}, {
      0.7165288868912466, -0.40586522230469635`}, {
      0.7215622056528872, -0.402578084899336}, {
      0.7265448339226112, -0.39926266524553705`}, {
      0.7314769318105168, -0.39591947661855886`}, {
      0.7363586587567597, -0.39254902351603016`}, {
      0.7411901734843213, -0.3891518018755003}, {
      0.7459716339548117, -0.3857282992859011}, {
      0.7507031973271429, -0.3822789951931135}, {
      0.7553850199189165, -0.37880436109982896`}, {
      0.7600172571703806, -0.3753048607598865}, {
      0.7646000636108147, -0.3717809503672622}, {
      0.7691335928272135, -0.36823307873988137`}, {
      0.773617997435144, -0.3646616874984171}, {
      0.7780534290516582, -0.36106721124023505`}, {
      0.7824400382701504, -0.35745007770863696`}, {
      0.7867779746370535, -0.3538107079575518}, {
      0.791067386630274, -0.35014951651181786`}, {
      0.7953084216392723, -0.34646691152319364`}, {
      0.7995012259466968, -0.34276329492223256`}, {
      0.8036459447114891, -0.3390390625661494}, {
      0.8077427219533785, -0.3352946043828046}, {
      0.8117917005386899, -0.3315303045109266}, {
      0.8157930221673937, -0.3277465414366891}, {
      0.8197468273613284, -0.3239436881267563}, {
      0.8236532554535317, -0.3201221121579049}, {
      0.8275124445786185, -0.3162821758433288}, {
      0.8313245316641474, -0.3124242363557274}, {
      0.8350896524229217, -0.3085486458472778}, {
      0.8388079413461703, -0.3046557515665846}, {
      0.8424795316975618, -0.3007458959727002}, {
      0.8461045555080021, -0.296819416846305}, {
      0.8496831435711727, -0.292876647398133}, {
      0.8532154254397668, -0.2889179163747271}, {
      0.8567015294223835, -0.2849435481616036}, {
      0.8601415825810409, -0.28095386288390545`}, {
      0.8635357107292737, -0.2769491765046183}, {
      0.86688403843078, -0.27292980092042374`}, {
      0.8701866889985851, -0.26889604405525924`}, {
      0.8734437844946918, -0.2648482099516542}, {
      0.8766554457301875, -0.260786598859908}, {
      0.8798217922657806, -0.25671150732517417`}, {
      0.8829429424127408, -0.25262322827251277`}, {
      0.8860190132342168, -0.2485220510899712}, {
      0.8890501205469087, -0.24440826170975194`}, {
      0.8920363789230726, -0.24028214268752301`}, {
      0.8949779016928369, -0.23614397327992642`}, {
      0.8978748009468075, -0.23199402952033712`}, {
      0.9007271875389461, -0.22783258429292402`}, {
      0.9035351710897013, -0.22365990740506264`}, {
      0.9062988599893758, -0.21947626565814757`}, {
      0.9090183614017132, -0.21528192291685153`}, {
      0.9116937812676902, -0.21107714017687618`}, {
      0.9143252243094964, -0.20686217563123868`}, {
      0.916912794034692, -0.20263728473513642`}, {
      0.9194565927405263, -0.1984027202694314}, {
      0.9219567215184069, -0.19415873240279402`}, {
      0.9244132802585071, -0.18990556875254547`}, {
      0.9268263676544991, -0.1856434744442361}, {
      0.9291960812084044, -0.1813726921699965}, {
      0.9315225172355496, -0.17709346224569672`}, {
      0.9338057708696189, -0.1728060226669483}, {
      0.9360459360677935, -0.1685106091639821}, {
      0.9382431056159702, -0.16420745525543504`}, {
      0.9403973711340503, -0.15989679230107662`}, {
      0.9425088230812912, -0.15557884955350665`}, {
      0.9445775507617141, -0.15125385420885323`}, {
      0.94660364232956, -0.14692203145650032`}, {
      0.9485871847947874, -0.142583604527873}, {
      0.9505282640286072, -0.13823879474430772`}, {
      0.9524269647690463, -0.13388782156403403`}, {
      0.9542833706265366, -0.1295309026282938}, {
      0.9560975640895231, -0.12516825380662294`}, {
      0.9578696265300864, -0.12080008924132014`}, {
      0.9595996382095757, -0.11642662139112656`}, {
      0.9612876782842458, -0.11204806107413928`}, {
      0.9629338248108964, -0.10766461750998155`}, {
      0.9645381547525089, -0.1032764983612513}, {
      0.9661007439838765, -0.09888390977426974}, {
      0.9676216672972241, -0.09448705641915055}, {
      0.9691009984078165, -0.09008614152921013}, {
      0.9705388099595494, -0.08568136693973866}, {
      0.9719351735305223, -0.08127293312615122}, {
      0.9732901596385889, -0.07686103924153785}, {
      0.9746038377468847, -0.07244588315363076}, {
      0.975876276269327, -0.06802766148120674}, {
      0.977107542576087, -0.0636065696299421}, {
      0.9782977029990308, -0.05918280182773725}, {
      0.9794468228371284, -0.054756551159527586`}, {
      0.9805549663618272, -0.05032800960159685}, {
      0.981622196822391, -0.04589736805540889}, {
      0.9826485764512009, -0.04146481638097337}, {
      0.9836341664690161, -0.03703054342976059}, {
      0.9845790270901968, -0.03259473707718012}, {
      0.9854832175278836, -0.028157584254638053`}, {
      0.9863467959991357, -0.02371927098118671}, {
      0.987169819730025, -0.019279982394781025`}, {
      0.9879523449606866, -0.014839902783155022`}, {
      0.988694426950323, -0.010399215614331794`}, {
      0.9893961199821634, -0.005958103566780035}, {
      0.9900574773683763, -0.0015167485592299593`}, {0.9906785514549351, 
      0.002924668219838871}, {0.9912593936264356, 0.007365966283025438}, {
      0.9918000543108665, 0.011806965815138806`}, {0.9923005829843311, 
      0.016247487644621424`}, {0.9927610281757205, 0.020687353215252512`}, {
      0.9931814374713386, 0.025126384558120073`}, {0.9935618575194771, 
      0.02956440426385023}, {0.9939023340349434, 0.03400123545508283}, {
      0.9942029118035378, 0.03843670175918239}, {0.9944636346864827, 
      0.04287062728117358}, {0.9946845456248031, 0.04730283657689074}, {
      0.9948656866436584, 0.051733154626330935`}, {0.9950070988566252, 
      0.056161406807200304`}, {0.9951088224699329, 0.06058741886864347}, {
      0.9951708967866507, 0.06501101690514609}, {0.9951933602108275, 
      0.06943202733060057}, {0.9951762502515845, 0.07385027685252524}, {
      0.9951196035271614, 0.07826559244642717}, {0.9950234557689165, 
      0.0826778013302993}, {0.9948878418252805, 0.08708673093924217}, {
      0.9947127956656663, 0.09149220890020102}, {0.9944983503843341, 
      0.09589406300680892}, {0.9942445382042125, 0.10029212119432673`}, {
      0.9939513904806772, 0.10468621151467075`}, {0.9936189377052863, 
      0.10907616211151891`}, {0.9932472095094763, 0.11346180119548659`}, {
      0.9928362346682157, 0.11784295701936309`}, {0.9923860411036195, 
      0.12221945785339977`}, {0.9918966558885257, 0.126591131960641}, {
      0.9913681052500332, 0.1309578075722892}, {0.9908004145730034, 
      0.13531931286309495`}, {0.9901936084035264, 0.1396754759267637}, {
      0.9895477104523525, 0.14402612475136997`}, {0.9888627435982901, 
      0.14837108719477063`}, {0.9881387298915723, 0.1527101909600082}, {
      0.9873756905571918, 0.1570432635706958}, {0.986573645998207, 
      0.16137013234637465`}, {0.9857326157990192, 0.16569062437783566`}, {
      0.984852618728624, 0.1700045665023962}, {0.9839336727438361, 
      0.17431178527912342`}, {0.9829757949924921, 0.17861210696399502`}, {
      0.9819790018166291, 0.18290535748498907`}, {0.9809433087556441, 
      0.1871913624170937}, {0.9798687305494339, 0.19146994695722777`}, {
      0.9787552811415183, 0.19574093589906377`}, {0.9776029736821472, 
      0.2000041536077437}, {0.9764118205313949, 0.2042594239944791}, {
      0.9751818332622421, 0.20850657049102583`}, {0.9739130226636492, 
      0.21274541602402466`}, {0.9726053987436214, 0.21697578298919823`}, {
      0.9712589707322683, 0.22119749322539506`}, {0.9698737470848608, 
      0.2254103679884712}, {0.9684497354848871, 0.22961422792499986`}, {
      0.9669869428471093, 0.23380889304579966`}, {0.965485375320625, 
      0.23799418269927125`}, {0.9639450382919343, 0.24216991554453302`}, {
      0.9623659363880172, 0.2463359095243454}, {0.9607480734794204, 
      0.25049198183781407`}, {0.9590914526833615, 0.25463794891286146`}, {
      0.9573960763668483, 0.2587736263784563}, {0.9556619461498197, 
      0.262898829036591}, {0.9538890629083098, 0.2670133708339955}, {
      0.9520774267776384, 0.27111706483357734`}, {0.9502270371556326, 
      0.2752097231855765}, {0.9483378927058805, 0.27929115709842445`}, {
      0.9464099913610233, 0.28336117680929496`}, {0.944443330326087, 
      0.28741959155433633`}, {0.9424379060818595, 0.2914662095385722}, {
      0.9403937143883164, 0.29550083790545983`}, {0.9383107502880986, 
      0.2995232827060927}, {0.9361890081100478, 0.30353334886803596`}, {
      0.9340284814728026, 0.3075308401637816}, {0.9318291632884613, 
      0.31151555917881035`}, {0.9295910457663151, 0.3154873072792469}, {
      0.9273141204166566, 0.3194458845790962}, {0.9249983780546697, 
      0.3233910899070454}, {0.9226438088044051, 0.32732272077281915`}, {
      0.9202504021028479, 0.3312405733330724}, {0.9178181467040821, 
      0.3351444423568075}, {0.9153470306835583, 0.3390341211902998}, {
      0.912837041442471, 0.34290940172151674`}, {0.9102881657122522, 
      0.34677007434401513`}, {0.9077003895591866, 0.3506159279203003}, {
      0.9050736983891565, 0.3544467497446311}, {0.9024080769525238, 
      0.35826232550525416`}, {0.899703509349155, 0.36206243924604986`}, {
      0.8969599790335991, 0.36584687332757315`}, {0.8941774688204245, 
      0.3696154083874712}, {0.8913559608897239, 0.37336782330025925`}, {
      0.8884954367927972, 0.37710389513643616`}, {0.8855958774580193, 
      0.38082339912092067`}, {0.8826572631969036, 0.3845261085907882}, {
      0.8796795737103708, 0.38821179495228836`}, {0.8766627880952323, 
      0.39188022763712255`}, {0.8736068848509011, 0.39553117405795973`}, {
      0.8705118418863376, 0.39916439956316985`}, {0.8673776365272454, 
      0.40277966739075133`}, {0.8642042455235268, 0.4063767386214306}, {
      0.8609916450570112, 0.40995537213090943`}, {0.8577398107494691, 
      0.413515324541237}, {0.854448717670926, 0.4170563501712807}, {
      0.8511183403482881, 0.4205782009862708}, {0.8477486527742981, 
      0.42408062654639345`}, {0.8443396284168317, 0.42756337395440386`}, {
      0.8408912402285553, 0.43102618780223334`}, {0.8374034606569576, 
      0.43446881011656135`}, {0.833876261654775, 0.43789098030332324`}, {
      0.8303096146908276, 0.4412924350911242}, {0.8267034907612846, 
      0.4446729084735286}, {0.8230578604013782, 0.4480321316501928}, {
      0.8193726936975884, 0.4513698329668095}, {0.8156479603003172, 
      0.4546857378538296}, {0.8118836294370763, 0.4579795687639277}, {
      0.8080796699262106, 0.4612510451081757}, {0.8042360501911818, 
      0.4644998831908881}, {0.8003527382754378, 0.46772579614310156`}, {
      0.7964297018578932, 0.47092849385465074`}, {0.7924669082690489, 
      0.47410768290480015`}, {0.7884643245077801, 0.47726306649139144`}, {
      0.7844219172588215, 0.4803943443584646}, {0.7803396529109811, 
      0.4835012127223096}, {0.7762174975761162, 0.4865833641959038}, {
      0.7720554171089051, 0.4896404877116898}, {0.7678533771274494, 
      0.4926722684426464}, {0.7636113430347455, 0.4956783877216045}, {
      0.7593292800410638, 0.4986585229587574}, {0.755007153187275, 
      0.5016123475573145}, {0.750644927369168, 0.5045395308272456}, {
      0.7462425673628034, 0.5074397378970603}, {0.741800037850947, 
      0.5103126296235683}, {0.7373173034506354, 0.5131578624995596}, {
      0.7327943287419222, 0.5159750885593488}, {0.728231078297858, 
      0.5187639552821184}, {0.7236275167157622, 0.5215241054930001}, {
      0.7189836086498416, 0.5242551772618286}, {0.7142993188452201, 
      0.5269568037994997}, {0.7095746121734412, 0.5296286133518651}, {
      0.7048094536695111, 0.5322702290910901}, {0.7000038085705526, 
      0.5348812690044041}, {0.6951576423561429, 0.537461345780165}, {
      0.6902709207904125, 0.5400100666911611}, {0.685343609965986, 
      0.5425270334750703}, {0.6803756763498484, 0.5450118422119923}, {
      0.6753670868312271, 0.5474640831989707}, {0.6703178087715806, 
      0.5498833408214143}, {0.6652278100567927, 0.5522691934213295}, {
      0.6600970591516734, 0.554621213162269}, {0.6549255251568749, 
      0.5569389658909008}, {0.6497131778683329, 0.559222010995099}, {
      0.644459987839354, 0.5614699012584539}, {0.6391659264454698, 
      0.5636821827110955}, {0.6338309659521905, 0.5658583944767248}, {
      0.6284550795857925, 0.5679980686157378}, {0.6230382416072843, 
      0.5701007299643297}, {0.6175804273896999, 0.5721658959694614}, {
      0.6120816134988791, 0.5741930765195651}, {0.6065417777779004, 
      0.5761817737708645}, {0.6009608994353397, 0.5781314819691818}, {
      0.5953389591375398, 0.5800416872670968}, {0.5896759391050823, 
      0.5819118675363238}, {0.583971823213666, 0.5837414921751656}, {
      0.5782265970996023, 0.5855300219108993}, {0.5724402482701548, 
      0.587276908596948}, {0.5666127662189567, 0.588981595004684}, {
      0.5607441425467554, 0.5906435146097092}, {0.5548343710877448, 
      0.5922620913724509}, {0.5488834480417599, 0.5938367395129093}, {
      0.5428913721126244, 0.5953668632793875}, {0.5368581446529542, 
      0.5968518567110312}, {0.5307837698157398, 0.5982911033940008}, {
      0.5246682547130435, 0.5996839762110935}, {0.518511609582169, 
      0.6010298370846316}, {0.512313847959679, 0.6023280367124254}, {
      0.506074986863655, 0.6035779142966164}, {0.49979504698461813`, 
      0.6047787972652053}, {0.49347405288554896`, 0.6059300009860591}, {
      0.4871120332114718, 0.6070308284731946}, {0.4807090209090914, 
      0.6080805700851275}, {0.4742650534569988, 0.6090785032150746}, {
      0.46778017310699044`, 0.6100238919727927}, {0.46125442713707543`, 
      0.6109159868578365}, {0.4546878681167776, 0.6117540244240134}, {
      0.4480805541853727, 0.6125372269348114}, {0.4414325493437368, 
      0.6132648020095768}, {0.43474392376052023`, 0.613935942260214}, {
      0.4280147540934009, 0.6145498249181817}, {0.4212451238262148, 
      0.6151056114515587}, {0.4144351236228044, 0.615602447171956}, {
      0.40758485169847564`, 0.6160394608310491}, {0.40069441421000357`, 
      0.6164157642065139}, {0.3937639256651806, 0.6167304516771478}, {
      0.38679350935295836`, 0.6169825997869681}, {0.37978329779529446`, 
      0.6171712667980829}, {0.37273343322187885`, 0.6172954922321423}, {
      0.3656440680689832, 0.6173542964001855}, {0.3585153655037468, 
      0.6173466799207138}, {0.35134749997529047`, 0.617271623225835}, {
      0.34414065779412883`, 0.6171280860553414}, {0.3368950377414373, 
      0.616915006938607}, {0.3296108517098207, 0.6166313026642105}, {
      0.3222883253773258, 0.6162758677372233}, {0.3149276989165417, 
      0.6158475738241291}, {0.3075292277407383, 0.6153452691853838}, {
      0.3000931832891087, 0.6147677780956616}, {0.29261985385329903`, 
      0.6141139002518864}, {0.28510954544753797`, 0.6133824101691967}, {
      0.27756258272481205`, 0.6125720565650586}, {0.2699793099416746, 
      0.6116815617318081}, {0.26236009197442556`, 0.6107096208979823}, {
      0.2547053153895596, 0.6096549015788892}, {0.24701538957154323`, 
      0.6085160429169629}, {0.2392907479111616, 0.6072916550125623}, {
      0.231531849057857, 0.6059803182459965}, {0.22373917823967862`, 
      0.6045805825916978}, {0.2159132486546657, 0.6030909669256206}, {
      0.20805460293770223`, 0.6015099583271158}, {0.20016381470710548`, 
      0.5998360113767254}, {0.19224149019544648`, 0.5980675474515557}, {
      0.18428826996934597`, 0.5962029540201309}, {0.17630483074324566`, 
      0.5942405839388922}, {0.16829188729242095`, 0.5921787547528062}, {
      0.16025019447077737`, 0.5900157480028762}, {0.15218054933925815`, 
      0.5877498085437155}, {0.14408379341098473`, 0.5853791438747482}, {
      0.13596081501955368`, 0.5829019234890516}, {0.12781255181722076`, 
      0.580316278244354}, {0.11963999341001642`, 0.5776202997612532}, {
      0.11144418413715121`, 0.5748120398543294}, {0.10322622600238557`, 
      0.5718895100025017}, {0.0949872817653501, 0.5688506808657205}, {
      0.08672857820110749, 0.565693481855911}, {0.07845140953654117, 
      0.562415800770984}, {0.07015714107243157, 0.5590154835017335}, {
      0.06184721300033422, 0.5554903338225301}, {0.053523144423594425`, 
      0.5518381132779288}, {0.045186537592013985`, 0.5480565411786319}, {
      0.03683908235981386, 0.5441432947216979}, {0.028482560876601025`, 
      0.540096009251473}, {0.020118852521032574`, 0.5359122786794631}, {
      0.011749939086758702`, 0.5315896560832514}, {0.003377910229997838, 
      0.5271256545066371}, {-0.004995030812270609, 
      0.5225177479854093}, {-0.01336656122504475, 
      0.5177633728256061}, {-0.021734232331249905`, 
      0.5128599291637401}, {-0.030095463014142464`, 
      0.5078047828413121}, {-0.038447532840645275`, 
      0.5025952676289883}, {-0.04678757488164308, 
      0.4972286878390871}, {-0.05511256822718502, 
      0.49170232136851255`}, {-0.06341933019688444, 
      0.4860134232179763}, {-0.0717045082486381, 
      0.48015922953726187`}, {-0.07996457159217833, 
      0.47413696225038837`}, {-0.08819580251800134, 
      0.4679438343188009}, {-0.09639428745696868, 
      0.4615770557051174}, {-0.1045559077914505, 
      0.4550338401044526}, {-0.11267633044536936`, 
      0.44831141251484957`}, {-0.1207509982880184, 
      0.4414070177228098}, {-0.1287751203951815, 
      0.4343179297842059}, {-0.13674366222098816`, 
      0.42704146258487596`}, {-0.14465133574521202`, 
      0.4195749815687653}, {-0.1524925896734778, 
      0.4119159167244179}, {-0.1602615997821859, 
      0.40406177692269235`}, {-0.16795225951598378`, 
      0.3960101656995187}, {-0.17555817096338022`, 
      0.3877587985769996}, {-0.1830726363556445, 
      0.3793055220138282}, {-0.19048865025545333`, 
      0.370648334071418}, {-0.19779889262477268`, 
      0.3617854068748423}, {-0.20499572298604796`, 
      0.3527151109371309}, {-0.21207117591668548`, 
      0.34343604140108547`}, {-0.2190169581436827, 
      0.3339470462339305}, {-0.22582444753260855`, 
      0.3242472563861647}, {-0.23248469429228216`, 
      0.31433611789624927`}, {-0.23898842474259335`, 
      0.30421342588663375`}, {-0.2453260480168814, 
      0.29387936035348}, {-0.25148766609083173`, 
      0.28333452360182754`}, {-0.2574630875454142, 
      0.27257997911952037`}, {-0.26324184548016916`, 
      0.26161729161690594`}, {-0.26881321999311353`, 
      0.25044856788533704`}, {-0.2741662656324536, 
      0.23907649804649336`}, {-0.27928984420077935`, 
      0.22750439667760697`}, {-0.28417266325204893`, 
      0.21573624320654522`}, {-0.28880332056310737`, 
      0.20377672087776097`}, {-0.29317035478260667`, 
      0.19163125349849452`}, {-0.2972623023593445, 
      0.1793060390882028}, {-0.3010677607282225, 
      0.1668080794776538}, {-0.30457545758518634`, 
      0.15414520484278788`}, {-0.30777432591379433`, 
      0.14132609211815841`}, {-0.3106535842380614, 
      0.12836027622163593`}, {-0.3132028213731879, 
      0.11525815304213277`}, {-0.31541208473374094`, 
      0.10203097320090754`}, {-0.31727197104562704`, 
      0.0886908256990478}, {-0.3187737181032512, 
      0.07525061071193655}, {-0.3199092960273985, 
      0.06172400098667575}, {-0.32067149632421627`, 
      0.048125391538732826`}, {-0.3210540169329572, 
      0.034469837624641966`}, {-0.3210515413908633, 
      0.020772981280399533`}, {-0.3206598102470588, 
      0.007050967049092282}, {-0.3198756829302556, -0.006679652137594463}, \
{-0.3186971884206374, -0.020402017626959375`}, {-0.31712356329349833`, \
-0.034099074658287626`}, {-0.3151552759855407, -0.04775368705360311}, \
{-0.31279403647417625`, -0.06134875491262401}, {-0.3100427909416352, \
-0.0748673331033928}, {-0.30690570140187184`, -0.08829274828489173}, \
{-0.3033881106798433, -0.1016087122268688}, {-0.29949649352980245`, \
-0.11479942930422912`}, {-0.29523839504274196`, -0.12784969623274414`}, \
{-0.2906223578063242, -0.14074499236895008`}, {-0.28565783953028584`, \
-0.1534715592056844}, {-0.2803551230274927, -0.16601646803896172`}, \
{-0.27472522054136944`, -0.17836767514386231`}, {-0.26877977443486967`, \
-0.1905140641590308}, {-0.26253095620933065`, -0.2024454757249267}, \
{-0.2559913657118535, -0.21415272473621963`}, {-0.249173932228259, \
-0.22562760584289238`}, {-0.2420918189577324, -0.23686288806037528`}, \
{-0.23475833213805353`, -0.24785229952255988`}, {-0.22718683584949162`, \
-0.2585905035322403}, {-0.2193906732825701, -0.26907306713361306`}, \
{-0.2113830950198623, -0.27929642345533295`}, {-0.20317719466267772`, \
-0.28925782905623193`}, {-0.1947858519357649, -0.298955317456048}, \
{-0.18622168323079236`, -0.30838764995761586`}, {-0.17749699940432234`, \
-0.3175542647720586}, {-0.16862377052863264`, -0.3264552253512139}, \
{-0.1596135972031697, -0.33509116871771305`}, {-0.15047768796877375`, \
-0.3434632544677969}, {-0.14122684232360092`, -0.351573115009141}, \
{-0.1318714388159922, -0.35942280748877536`}, {-0.12242142768236573`, \
-0.36701476776685094`}, {-0.11288632750452067`, -0.37435176670197834`}, \
{-0.10327522537768025`, -0.38143686893393314`}, {-0.09359678010555313, \
-0.388273394279945}, {-0.08385922796934446, -0.3948648818013942}, \
{-0.07407039065201054, -0.40121505654805595`}, {-0.06423768493546186, \
-0.4073277989463668}, {-0.05436813382552694, -0.4132071167657132}, \
{-0.04446837879622964, -0.41885711957156724`}, {-0.034544692880496865`, \
-0.42428199555550244`}, {-0.02460299436821324, -0.42948599061882564`}, \
{-0.014648860904174225`, -0.4344733895779089}, {-0.004687543807709705, \
-0.4392484993545209}, {0.005276017537566539, -0.4438156340128317}, {
      0.015237181351309683`, -0.4481791015056693}, {
      0.025191589287132388`, -0.45234319199549944`}, {
      0.03513515251309166, -0.45631216761999777`}, {
      0.04506403811175538, -0.46009025357759914`}, {
      0.05497465585487954, -0.4636816304146958}, {
      0.06486364539493655, -0.4670904274029467}, {
      0.07472786390472703, -0.4703207169022226}, {
      0.08456437418690681, -0.473376509611872}, {
      0.09437043326729677, -0.4762617506201074}, {
      0.10414348147915435`, -0.4789803161682709}, {
      0.11388113204001932`, -0.4815360110534651}, {
      0.12358116111816478`, -0.4839325665994679}, {
      0.13324149838195723`, -0.48617363913195155`}, {
      0.14286021802244356`, -0.48826280889977103`}, {
      0.1524355302371321, -0.4902035793894611}, {
      0.16196577316112892`, -0.4919993769850857}, {
      0.1714494052304455, -0.4936535509302176}, {
      0.18088499796133986`, -0.4951693735530994}, {
      0.1902712291289248, -0.49655004071997033`}, {
      0.19960687632792123`, -0.49779867248514065`}, {
      0.20889081089830294`, -0.4989183139096849}, {
      0.21812199219863068`, -0.49991193602362133`}, {
      0.22729946221007336`, -0.5007824369091679}, {
      0.23642234045443195`, -0.501532642885133}, {
      0.24548981920989169`, -0.5021653097747355}, {
      0.25450115900870945`, -0.5026831242411641}, {
      0.2634556844015768, -0.503088705177009}, {
      0.27235277997397017`, -0.5033846051353331}, {
      0.2811918866003953, -0.5035733117916261}, {
      0.289972497923043, -0.5036572494272016}, {
      0.2986941570419893, -0.5036387804257852}, {
      0.30735645340468604`, -0.503520206776098}, {
      0.3159590198830974, -0.5033037715741862}, {
      0.3245015300274317, -0.5029916605200951}, {
      0.3329836954860004, -0.5025860034042345}, {
      0.34140526358129925`, -0.5020888755794541}, {
      0.3497660150329522, -0.5015022994154429}, {
      0.3580657618186799, -0.5008282457325923}, {
      0.3663043451649598, -0.5000686352129334}, {
      0.3744816336595225, -0.49922533978616856`}, {
      0.3825975214782873, -0.49830018398918563`}, {
      0.390651926719776, -0.49729494629776316`}, {
      0.39864478984045626`, -0.49621136042945796`}, {
      0.40657607218485864`, -0.4950511166169145}, {
      0.41444575460468164`, -0.4938158628510533}, {
      0.42225383616145057`, -0.492507206093784}, {
      0.4300003329076258, -0.49112671346005493`}, {
      0.4376852767413692, -0.4896759133691925}, {
      0.4453087143304709, -0.48815629666560895`}, {
      0.45287070610121605`, -0.48656931770906203`}, {
      0.4603713252882307, -0.48491639543474124`}, {
      0.4678106570415922, -0.4831989143835329}, {
      0.4751887975877176, -0.48141822570288173`}, {
      0.4825058534407621, -0.47957564811871994`}, {
      0.4897619406614598, -0.47767246887898157`}, {
      0.4969571841605315, -0.475709944669256}, {
      0.5040917170439609, -0.47368930250116353`}, {
      0.5111656799976104, -0.47161174057406074`}, {
      0.5181792207088007, -0.469478429110699}, {
      0.5251324933226313, -0.46729051116747367`}, {
      0.5320256579309506, -0.4650491034199097}, {
      0.5388588800920175, -0.4627552969240327}, {
      0.5456323303790183, -0.46041015785427863`}, {
      0.5523461839557102, -0.45801472821859085`}, {
      0.5590006201775777, -0.4555700265513525}, {
      0.5655958222169792, -0.45307704858479403`}, {
      0.5721319767108616, -0.4505367678995101}, {
      0.578609273429701, -0.44795013655470983`}, {
      0.5850279049664178, -0.4453180856988152}, {
      0.5913880664440818, -0.44264152616101016`}, {
      0.5976899552413036, -0.43992134902433205`}, {
      0.6039337707342689, -0.43715842618088396`}, {
      0.6101197140544403, -0.43435361086973306`}, {
      0.6162479878610075, -0.4315077381980466}, {
      0.6223187961272232, -0.42862162564600376`}, {
      0.6283323439398134, -0.4256960735560071}, {
      0.6342888373107018, -0.4227318656067032}, {
      0.640188483000329, -0.4197297692723084}, {
      0.646031488351895, -0.41669053626771974`}, {
      0.6518180611358901, -0.4136149029798809}, {
      0.6575484094043201, -0.41050359088585353`}, {
      0.6632227413540627, -0.4073573069580354}, {
      0.6688412651988292, -0.40417674405695114`}, {
      0.6744041890492365, -0.4009625813120276}, {
      0.6799117208005184, -0.39771548449075383`}, {
      0.6853640680274411, -0.3944361063566121}, {
      0.6907614378860054, -0.39112508701615434`}, {
      0.6961040370215478, -0.38778305425558596`}, {
      0.701392071482872, -0.38441062386720637`}, {
      0.7066257466420666, -0.3810083999660452}, {
      0.7118052671196815, -0.3775769752970207}, {
      0.7169308367149573, -0.3741169315329366}, {
      0.7220026583408176, -0.37062883956362225`}, {
      0.7270209339633521, -0.3671132597765109}, {
      0.7319858645455338, -0.36357074232894154`}, {
      0.7368976499949264, -0.3600018274124583}, {
      0.7417564891151553, -0.35640704550937347`}, {
      0.7465625795609262, -0.35278691764185083`}, {
      0.7513161177963864, -0.3491419556137557}, {
      0.7560172990566403, -0.345472662245512}, {
      0.7606663173122348, -0.3417795316021958}, {
      0.7652633652364462, -0.33806304921508895`}, {
      0.769808634175207, -0.33432369229690667`}, {
      0.7743023141195194, -0.3305619299509074}, {
      0.7787445936802144, -0.32677822337408474`}, {
      0.7831356600649182, -0.32297302605463474`}, {
      0.7874756990571002, -0.31914678396388474`}, {
      0.7917648949970809, -0.31529993574286447`}, {
      0.7960034307648853, -0.31143291288369274`}, {
      0.8001914877648353, -0.30754613990594715`}, {
      0.8043292459117773, -0.3036400345281795}, {
      0.8084168836188519, -0.29971500783473254`}, {
      0.8124545777867109, -0.2957714644380091}, {
      0.8164425037941003, -0.29180980263633965`}, {
      0.8203808354897243, -0.2878304145675882}, {
      0.8242697451853159, -0.283833686358633}, {
      0.8281094036498416, -0.2798199982708529}, {
      0.8318999801047705, -0.2757897248417459}, {
      0.8356416422203443, -0.2717432350228022}, {
      0.8393345561127875, -0.2676808923137499}, {
      0.8429788863423978, -0.2636030548932879}, {
      0.8465747959124651, -0.2595100757464146}, {
      0.8501224462689646, -0.2554023027884612}, {
      0.8536219973009778, -0.2512800789859303}, {
      0.8570736073417927, -0.24714374247424042`}, {
      0.8604774331706425, -0.24299362667247199`}, {
      0.8638336300150381, -0.23883006039520768`}, {
      0.867142351553659, -0.2346533679615569}, {
      0.870403749919763, -0.23046386930145082`}, {
      0.8736179757050809, -0.2262618800592922}, {
      0.8767851779641643, -0.2220477116950405}, {
      0.8799055042191527, -0.21782167158281113`}, {
      0.8829791004649336, -0.21358406310706426`}, {
      0.8860061111746653, -0.20933518575645665`}, {
      0.8889866793056381, -0.20507533521542723`}, {
      0.8919209463054466, -0.20080480345358515`}, {
      0.8948090521184524, -0.19652387881296646`}, {
      0.8976511351925132, -0.19223284609322375`}, {
      0.9004473324859574, -0.1879319866348109}, {
      0.9031977794747857, -0.18362157840022272`}, {
      0.9059026101600794, -0.17930189605334826`}, {
      0.9085619570755983, -0.17497321103699345`}, {
      0.9111759512955523, -0.17063579164862824`}, {
      0.9137447224425299, -0.16628990311441058`}, {
      0.9162683986955691, -0.16193580766153878`}, {
      0.9187471067983565, -0.15757376458898148`}, {
      0.9211809720675421, -0.1532040303366334}, {
      0.9235701184011561, -0.14882685855294348`}, {
      0.9259146682871167, -0.1444425001610602}, {
      0.9282147428118179, -0.14005120342353813`}, {
      0.9304704616687864, -0.13565321400564778`}, {
      0.9326819431673978, -0.13124877503732982`}, {
      0.9348493042416428, -0.1268381271738337}, {
      0.936972660458935, -0.12242150865507896`}, {
      0.9390521260289518, -0.11799915536377709`}, {
      0.9410878138125005, -0.11357130088234975`}, {
      0.9430798353304022, -0.1091381765486791}, {
      0.9450283007723876, -0.1047000115107241}, {
      0.9469333190059974, -0.10025703278003609`}, {
      0.9487949975854808, -0.09580946528420593}, {
      0.9506134427606878, -0.09135753191827399}, {
      0.9523887594859493, -0.0869014535951332}, {
      0.9541210514289394, -0.0824414492949549}, {
      0.9558104209795171, -0.0779777361136661}, {
      0.9574569692585401, -0.07351052931050586}, {
      0.9590607961266499, -0.06904004235468815}, {
      0.9606220001930222, -0.06456648697119712}, {
      0.96214067882408, -0.060090073185740805`}, {
      0.9636169281521656, -0.05561100936888781}, {
      0.9650508430841688, -0.05112950227941128}, {
      0.966442517310109, -0.04664575710686374}, {
      0.9677920433116673, -0.04215997751340569}, {
      0.969099512370667, -0.03767236567491016}, {
      0.9703650145775006, -0.03318312232136512}, {
      0.9715886388395001, -0.02869244677659461}, {
      0.9727704728892504, -0.02420053699731947}, {
      0.9739106032928423, -0.01970758961157743}, {
      0.9750091154580643, -0.015213799956522315`}, {
      0.976066093642533, -0.010719362115621304`}, {
      0.9770816209617583, -0.006224468955268798}, {
      0.9780557793971445, -0.0017293121608350477`}, {0.9789886498039249, 
      0.002765917727832838}, {0.9798803119190301, 0.007261031281440266}, {
      0.9807308443688885, 0.011755840146788103`}, {0.9815403246771579, 
      0.016250157012812762`}, {0.982308829272389, 0.020743795577107434`}, {
      0.9830364334956193, 0.025236570512908764`}, {0.9837232116078968, 
      0.02972829743653371}, {0.9843692367977344, 0.0342187928752516}, {
      0.9849745811884937, 0.038707874235576784`}, {0.9855393158456985, 
      0.04319535977196755}, {0.9860635107842782, 0.047681068555917354`}, {
      0.98654723497574, 0.052164820445424614`}, {0.9869905563552722, 
      0.056646436054827665`}, {0.9873935418287761, 0.06112573672499179}, {
      0.9877562572798292, 0.06560254449383535}, {0.9880787675765779, 
      0.07007668206718251}, {0.9883611365785618, 0.07454797278993007}, {
      0.988603427143469, 0.07901624061751633}, {0.9888057011338228, 
      0.08348131008768009}, {0.9889680194236011, 0.08794300629249795}, {
      0.9890904419047886, 0.0924011548506886}, {0.9891730274938618, 
      0.09685558188017268}, {0.9892158341382098, 0.10130611397087706`}, {
      0.9892189188224878, 0.10575257815777282`}, {0.9891823375749079, 
      0.11019480189413591`}, {0.9891061454734659, 0.1146326130250201}, {
      0.9889903966521048, 0.11906583976093167`}, {0.9888351443068177, 
      0.12349431065169567`}, {0.9886404407016893, 0.12791785456050347`}, {
      0.988406337174877, 0.1323363006381318}, {0.9881328841445344, 
      0.13674947829732306`}, {0.9878201311146758, 0.1411572171873177}, {
      0.9874681266809857, 0.14555934716852825`}, {0.9870769185365709, 
      0.14995569828734645`}, {0.9866465534776597, 0.15434610075107308`}, {
      0.9861770774092479, 0.15873038490296174`}, {0.9856685353506921, 
      0.16310838119736737`}, {0.9851209714412538, 0.16747992017498986`}, {
      0.9845344289455931, 0.1718448324382042}, {0.9839089502592158, 
      0.1762029486264679}, {0.9832445769138731, 0.18055409939179662`}, {
      0.9825413495829174, 0.1848981153742994}, {0.9817993080866135, 
      0.18923482717776446`}, {0.9810184913974094, 0.1935640653452868}, {
      0.9801989376451655, 0.19788566033492888`}, {0.9793406841223464, 
      0.20219944249540558`}, {0.9784437672891745, 0.20650524204178497`}, {
      0.9775082227787489, 0.21080288903119596`}, {0.9765340854021315, 
      0.21509221333853432`}, {0.9755213891533997, 0.21937304463215845`}, {
      0.9744701672146713, 0.22364521234956625`}, {0.9733804519610993, 
      0.22790854567304436`}, {0.972252274965843, 0.23216287350528142`}, {
      0.9710856670050124, 0.2364080244449364}, {0.9698806580625935, 
      0.24064382676215348`}, {0.9686372773353512, 0.24487010837401504`}, {
      0.9673555532377162, 0.24908669681992357`}, {0.9660355134066556, 
      0.2532934192369041}, {0.9646771847065306, 0.2574901023348186}, {
      0.9632805932339432, 0.26167657237148295`}, {0.9618457643225742, 
      0.2658526551276781}, {0.9603727225480155, 0.2700181758820464}, {
      0.9588614917325992, 0.27417295938586417`}, {0.9573120949502247, 
      0.2783168298376812}, {0.9557245545311894, 0.2824496108578185}, {
      0.954098892067022, 0.28657112546271485`}, {0.9524351284153246, 
      0.2906811960391131}, {0.9507332837046244, 0.2947796443180764}, {
      0.9489933773392392, 0.29886629134882575`}, {0.9472154280041586, 
      0.3029409574723883}, {0.9453994536699456, 0.3070034622950479}, {
      0.9435454715976606, 0.31105362466158687`}, {0.9416534983438112, 
      0.31509126262831044`}, {0.9397235497653328, 0.31911619343584263`}, {
      0.9377556410246012, 0.32312823348168435`}, {0.9357497865944833, 
      0.32712719829252285`}, {0.9337060002634278, 0.33111290249628267`}, {
      0.9316242951406006, 0.33508515979390685`}, {0.9295046836610699, 
      0.339043782930858}, {0.9273471775910426, 0.3429885836683288}, {
      0.9251517880331597, 0.3469193727541496}, {0.922918525431852, 
      0.35083595989338345`}, {0.9206473995787633, 0.35473815371859574`}, {
      0.9183384196182431, 0.35862576175978766`}, {0.9159915940529173, 
      0.3624985904139812}, {0.9136069307493379, 0.36635644491444375`}, {
      0.9111844369437203, 0.37019912929954035`}, {0.9087241192477724, 
      0.3740264463812002}, {0.9062259836546199, 0.37783819771298566`}, {
      0.9036900355448358, 0.3816341835577502}, {0.9011162796925787, 
      0.3854142028548719}, {0.8985047202718461, 0.38917805318704946`}, {
      0.8958553608628503, 0.39292553074664666`}, {0.893168204458522, 
      0.3966564303015708}, {0.8904432534711495, 0.4003705451606713}, {
      0.8876805097391607, 0.4040676671386437}, {0.8848799745340544, 
      0.4077475865204236}, {0.88204164856749, 0.4114100920250552}, {
      0.8791655319985419, 0.41505497076901965`}, {0.8762516244411281, 
      0.4186820082290058}, {0.8732999249716213, 0.42229098820410793`}, {
      0.8703104321366507, 0.42588169277743326`}, {0.8672831439611044, 
      0.4294539022771021}, {0.8642180579563421, 0.433007395236623}, {
      0.8611151711286269, 0.4365419483546249}, {0.857974479987788, 
      0.440057336453928}, {0.854795980556123, 0.4435533324399343}, {
      0.8515796683775537, 0.44702970725831787`}, {0.8483255385270434, 
      0.45048622985199654`}, {0.8450335856202909, 0.453922667117363}, {
      0.8417038038237109, 0.45733878385975524`}, {0.8383361868647164, 
      0.4607343427481451}, {0.8349307280423138, 0.46410910426902274`}, {
      0.8314874202380261, 0.4674628266794544}, {0.828006255927159, 
      0.4707952659592908}, {0.8244872271904234, 0.4741061757625021}, {
      0.8209303257259319, 0.4773953073676156}, {0.8173355428615835, 
      0.4806624096272307}, {0.8137028695678553, 0.483907228916586}, {
      0.8100322964710176, 0.4871295090811518}, {0.8063238138667906, 
      0.49032899138322195`}, {0.8025774117344633, 0.4935054144474759}, {
      0.798793079751492, 0.4966585142054842}, {0.7949708073086017, 
      0.49978802383912646`}, {0.7911105835254092, 0.5028936737228935}, {
      0.787212397266593, 0.5059751913650414}, {0.7832762371586308, 
      0.5090323013475665}, {0.7793020916071309, 0.5120647252649688}, {
      0.7752899488147815, 0.5150721816617698}, {0.7712397967999448, 
      0.518054385968751}, {0.7671516234159234, 0.521011050437877}, {
      0.763025416370927, 0.5239418840758674}, {0.758861163248771, 
      0.5268465925763798}, {0.7546588515303352, 0.5297248782507654}, {
      0.7504184686158182, 0.5325764399573579}, {0.7461400018478194, 
      0.5354009730292545}, {0.7418234385352841, 0.538198169200548}, {
      0.7374687659783496, 0.5409677165309658}, {0.7330759714941297, 
      0.5437092993288729}, {0.7286450424434778, 0.546422598072592}, {
      0.7241759662587716, 0.5491072893299944}, {0.7196687304727614, 
      0.551763045676314}, {0.7151233227485294, 0.5543895356101333}, {
      0.710539730910607, 0.5569864234674923}, {0.7059179429772986, 
      0.5595533693340661}, {0.7012579471942672, 0.5620900289553575}, {
      0.6965597320694323, 0.5645960536448497}, {0.6918232864092405, 
      0.5670710901900601}, {0.6870485993563652, 0.5695147807564392}, {
      0.6822356604289, 0.5719267627890502}, {0.6773844595611093, 
      0.5743066689119694}, {0.6724949871458037, 0.5766541268253426}, {
      0.6675672340784137, 0.5789687592000308}, {0.6626011918028323, 
      0.5812501835697774}, {0.6575968523591075, 0.583498012220827}, {
      0.6525542084330643, 0.5857118520789227}, {0.6474732534079427, 
      0.5878913045936098}, {0.6423539814181387, 0.5900359656197666}, {
      0.6371963874051446, 0.5921454252962857}, {0.6320004671757837, 
      0.594219267921825}, {0.626766217462844, 0.5962570718275437}, {
      0.6214936359882162, 0.5982584092467387}, {0.6161827215286502, 
      0.6002228461812928}, {0.6108334739842468, 0.6021499422648454}, {
      0.6054458944498086, 0.60403925062259}, {0.6000199852891789, 
      0.6058903177276052}, {0.594555750212706, 0.607702683253619}, {
      0.589053194357973, 0.6094758799241063}, {0.583512324373944, 
      0.6112094333576141}, {0.5779331485086834, 0.6129028619092086}, {
      0.5723156767008117, 0.6145556765079339}, {0.5666599206748714, 
      0.6161673804901692}, {0.560965894040783, 0.6177374694287691}, {
      0.5552336123975831, 0.6192654309578669}, {0.5494630934416419, 
      0.6207507445932203}, {0.5436543570795721, 0.622192881547973}, {
      0.5378074255460478, 0.623591304543704}, {0.5319223235267662, 
      0.6249454676166336}, {0.5259990782867946, 0.6262548159188502}, {
      0.5200377198045587, 0.6275187855144203}, {0.514038280911741, 
      0.6287368031702402}, {0.5080007974393722, 0.6299082861414845}, {
      0.5019253083704126, 0.6310326419515041}, {0.4958118559991364, 
      0.6321092681660218}, {0.48966048609764773`, 0.6331375521614716}, {
      0.483471248089875, 0.6341168708873258}, {0.47724419523340766`, 
      0.6350465906222466}, {0.470979384809559, 0.635926066723902}, {
      0.46467687832205895`, 0.636754643372279}, {0.45833674170480215`, 
      0.6375316533063254}, {0.45195904553909927`, 0.6382564175537492}, {
      0.44554386528090334`, 0.6389282451538034}, {0.43909128149850835`, 
      0.6395464328728812}, {0.4326013801212443, 0.6401102649127433}, {
      0.42607425269972093`, 0.6406190126112024}, {0.41950999667820166`, 
      0.6410719341350832}, {0.4129087156797228, 0.6414682741652817}, {
      0.4062705198046045, 0.6418072635737422}, {0.39959552594303605`, 
      0.6420881190921753}, {0.39288385810245713`, 0.642310042972341}, {
      0.38613564775049314`, 0.642472222637719}, {0.37935103417424776`, 
      0.6425738303263986}, {0.37253016485679874`, 0.6426140227250172}, {
      0.36567319587179015`, 0.6425919405935872}, {0.3587802922970639, 
      0.6425067083810532}, {0.3518516286483268, 0.6423574338314328}, {
      0.3448873893339045, 0.6421432075804003}, {0.3378877691316919, 
      0.6418631027421846}, {0.33085297368947386`, 0.6415161744866674}, {
      0.3237832200498547, 0.6411014596065794}, {0.3166787372011049, 
      0.6406179760747113}, {0.30953976665530786`, 0.6400647225910775}, {
      0.30236656305526805`, 0.6394406781199908}, {0.29515939481172326`, 
      0.6387448014170343}, {0.28791854477249296`, 0.6379760305459455}, {
      0.2806443109252858, 0.6371332823854635}, {0.27333700713598813`, 
      0.6362154521262249}, {0.26599696392435795`, 0.6352214127578409}, {
      0.2586245292791586, 0.6341500145463352}, {0.25122006951488113`, 
      0.633000084502176}, {0.24378397017232686`, 0.6317704258392004}, {
      0.23631663696544883`, 0.6304598174247943}, {0.22881849677698732`, 
      0.6290670132217726}, {0.22128999870557675`, 0.6275907417224873}, {
      0.2137316151671515, 0.6260297053757895}, {0.2061438430536364, 
      0.6243825800075804}, {0.19852720495207346`, 0.6226480142358031}, {
      0.19088225042751142`, 0.6208246288808642}, {0.1832095573731669, 
      0.6189110163726217}, {0.17550973343155843`, 0.6169057401552404}, {
      0.16778341749051434`, 0.6148073340914023}, {0.16003128125816568`, 
      0.6126143018675616}, {0.15225403092125261`, 0.6103251164021608}, {
      0.14445240889130087`, 0.6079382192589768}, {0.136627195643459, 
      0.6054520200680433}, {0.12877921165303222`, 0.6028648959569028}, {
      0.12090931943499914`, 0.6001751909952856}, {0.11301842569205604`, 
      0.597381215656687}, {0.10510748357699845`, 0.5944812463007325}, {
      0.09717749507551805, 0.5914735246806813}, {0.08922951351576619, 
      0.5883562574809252}, {0.0812646462113088, 0.5851276158899037}, {
      0.07328405724437081, 0.5817857352144684}, {0.06528897039653725, 
      0.5783287145424192}, {0.05728067223434037, 0.5747546164606748}, {
      0.04926051535741308, 0.5710614668373731}, {0.04122992181712325, 
      0.5672472546770964}, {0.033190386713816064`, 0.5633099320594153}, {
      0.02514348198097508, 0.5592474141720354}, {0.017090860364758775`, 
      0.5550575794510259}, {0.00903425960746902, 0.5507382698419178}, {
      0.0009755068435496443, 0.5462872911968923}, {-0.007083476783316089, 
      0.5417024138248391}, {-0.015140671273555319`, 
      0.5369813732127651}, {-0.023193952687267698`, 
      0.53212187093889}, {-0.03124108781410694, 
      0.5271215757997709}, {-0.03927972859007791, 
      0.5219781251759792}, {-0.047307406254507745`, 
      0.5166891266632063}, {-0.055321525241315256`, 
      0.5112521599982163}, {-0.06331935679980696, 
      0.5056647793117887}, {-0.07129803234160936, 
      0.4999245157437201}, {-0.0792545365120495, 
      0.4940288804580714}, {-0.08718569998636981, 
      0.4879753681001581}, {-0.0950881919936636, 
      0.48176146074028275`}, {-0.10295851257440634`, 
      0.47538463235288286`}, {-0.11079298458100426`, 
      0.46884235388359985`}, {-0.11858774543495978`, 
      0.462132098960737}, {-0.12633873865914697`, 
      0.4552513503116267}, {-0.13404170520938552`, 
      0.4481976069485243}, {-0.1416921746360976, 
      0.44096839219271344`}, {-0.1492854561144242, 
      0.433561262609474}, {-0.1568166293898749, 
      0.42597381793031036`}, {-0.16428053569648998`, 
      0.41820371204224743`}, {-0.17167176871571502`, 
      0.410248665126907}, {-0.1789846656568238, 
      0.40210647703429314`}, {-0.1862132985538695, 
      0.3937750419775119}, {-0.1933514658898698, 
      0.3852523646347578}, {-0.20039268467629334`, 
      0.37653657774351046`}, {-0.20733018313492912`, 
      0.36762596126862934`}, {-0.21415689414986083`, 
      0.3585189632205108}, {-0.22086544967944852`, 
      0.3492142221912102}, {-0.22744817634177816`, 
      0.3397105916649259}, {-0.23389709241171663`, 
      0.33000716614393555`}, {-0.24020390649313517`, 
      0.3201033091113718}, {-0.24636001815552555`, 
      0.3099986828275178}, {-0.25235652084945676`, 
      0.2996932799259688}, {-0.25818420743924825`, 
      0.28918745673945917`}, {-0.263833578712803, 
      0.27848196824186405`}, {-0.2692948552464602, 
      0.2675780044424302}, {-0.27455799301546857`, 
      0.2564772280104221}, {-0.27961270314649317`, 
      0.24518181284306242`}, {-0.2844484762054702, 
      0.23369448321719374`}, {-0.2890546113999652, 
      0.22201855308618468`}, {-0.2934202510476849, 
      0.2101579649994181}, {-0.2975344206196391, 
      0.1981173280339913}, {-0.3013860746054245, 
      0.18590195403943108`}, {-0.30496414836725144`, 
      0.17351789140940266`}, {-0.30825761604714425`, 
      0.1609719555134282}, {-0.3112555544673852, 
      0.14827175485110727`}, {-0.31394721281783616`, 
      0.13542571193645656`}, {-0.31632208775654147`, 
      0.12244307788644684`}, {-0.31837000336468674`, 
      0.10933393968152681`}, {-0.32008119519788225`, 
      0.0961092190926855}, {-0.3214463974688978, 
      0.08278066233468774}, {-0.3224569321901841, 
      0.06936081961274652}, {-0.3231047989071367, 
      0.05586301388272384}, {-0.3233827634757263, 
      0.042301298343510596`}, {-0.32328444419222424`, 
      0.028690402422455564`}, {-0.32280439347975, 
      0.015045666295566286`}, {-0.3219381732869483, 
      0.001382964295536946}, {-0.32068242236725053`, -0.01228138210872357}, \
{-0.3190349136892356, -0.025930700741340414`}, {-0.3169946003824318, \
-0.03954807321212458}, {-0.3145616488472173, -0.05311644582884334}, \
{-0.3117374579465102, -0.06661874503082366}, {-0.3085246635404822, \
-0.08003799525021318}, {-0.30492712800935273`, -0.09335743698340335}, \
{-0.30094991481605093`, -0.1065606428114629}, {-0.29659924857082937`, \
-0.11963162914884812`}, {-0.2918824614538828, -0.13255496162294486`}, \
{-0.28680792721072096`, -0.1453158521868812}, {-0.28138498424187874`, \
-0.15790024633366703`}, {-0.2756238495504708, -0.1702948990965268}, \
{-0.2695355254794358, -0.18248743887126892`}, {-0.2631317012611756, \
-0.19446641846353788`}, {-0.2564246514165305, -0.20622135312892875`}, \
{-0.24942713298281832`, -0.2177427457208216}, {-0.24215228343072737`, \
-0.22902209937552323`}, {-0.2346135209585145, -0.2400519184361906}, \
{-0.22682444864205364`, -0.2508256985388941}, {-0.21879876368413112`, \
-0.2613379069524544}, {-0.21055017275886254`, -0.27158395437803956`}, \
{-0.20209231419888676`, -0.2815601594774431}, {-0.193438687534022, \
-0.29126370741518787`}, {-0.18460259066825435`, -0.300692603675376}, \
{-0.1755970647830345, -0.3098456243566969}, {-0.16643484688256532`, \
-0.3187222640656859}, {-0.157128329752877, -0.3273226824265169}, \
{-0.14768952899119447`, -0.33564765011208464`}, {-0.13813005667432413`, \
-0.34369849518186874`}, {-0.12846110117247156`, -0.3514770503921263}, \
{-0.11869341257537384`, -0.35898560202737867`}, {-0.10883729317783519`, \
-0.36622684069203454`}, {-0.09890259246851381, -0.3732038143994977}, \
{-0.08889870607598618, -0.37991988420463646`}, {-0.07883457814675675, \
-0.3863786825447502}, {-0.06871870665830158, -0.3925840743843253}, \
{-0.0585591512040711, -0.3985401211996732}, {-0.04836354282460725, \
-0.4042510477904252}, {-0.03813909549787893, -0.4097212118650455}, \
{-0.027892618941251564`, -0.4149550763161233}, {-0.017630532416128858`, \
-0.41995718407724786`}, {-0.007358879263443158, -0.4247321354357968}, {
      0.002916658066738914, -0.42928456766402856`}, {
      0.013190742695563588`, -0.4336191368235909}, {
      0.02345836787973743, -0.43774050159511524`}, {
      0.033714841606644515`, -0.4416533089842416}, {
      0.04395577134246757, -0.44536218175755715`}, {
      0.054177049033684976`, -0.4488717074659811}, {
      0.06437483644300424, -0.45218642891860117`}, {
      0.07454555088387538, -0.45531083597646715`}, {
      0.08468585140302706, -0.4582493585430314}, {
      0.0947926254477944, -0.46100636063552336`}, {
      0.10486297604416958`, -0.4635861354293345}, {
      0.11489420950231467`, -0.4659929011752908}, {
      0.12488382365854978`, -0.46823079789737304`}, {
      0.1348294966563968, -0.47030388478590224`}, {
      0.14472907626396156`, -0.47221613820836594`}, {
      0.15458056972062756`, -0.4739714502668684}, {
      0.16438213410258043`, -0.4755736278376098}, {
      0.17413206719396526`, -0.4770263920338175}, {
      0.1838287988483894, -0.4783333780391566}, {
      0.19347088282392982`, -0.4794981352638447}, {
      0.2030569890737001, -0.4805241277804895}, {
      0.21258589647330667`, -0.48141473500107146`}, {
      0.22205648596611205`, -0.4821732525605325}, {
      0.23146773410706917`, -0.48280289337611143`}, {
      0.24081870698594907`, -0.48330678885492195`}, {
      0.25010855451101205`, -0.48368799022531256`}, {
      0.25933650503453515`, -0.48394946997030364`}, {
      0.26850186030207845`, -0.48409412334389024`}, {
      0.2776039907079208, -0.48412476995324444`}, {
      0.286642330839706, -0.4840441553918749}, {
      0.2956163752959899, -0.4838549529106178}, {
      0.3045256747610581, -0.4835597651149634}, {
      0.3133698323220773, -0.48316112567868275`}, {
      0.3221485000143423, -0.48266150106502287`}, {
      0.3308613755810769, -0.4820632922479047}, {
      0.33950819943493676`, -0.481368836426595}, {
      0.34808875180903354`, -0.48058040872824903`}, {
      0.3566028500859589, -0.4797002238935387}, {
      0.365050346293921, -0.47873043794131254`}, {
      0.37343112475972023`, -0.4776731498088753}, {
      0.3817450999088788, -0.4765304029650475}, {
      0.3899922142038045, -0.4753041869936662}, {
      0.39817243621140475`, -0.4739964391456336}, {
      0.406285758792081, -0.4726090458580071}, {
      0.4143321974025196, -0.47114384423896843`}, {
      0.42231178850515677`, -0.46960262351780596`}, {
      0.4302245880776322, -0.46798712645930735`}, {
      0.43807067021595775`, -0.4662990507421844}, {
      0.44585012582551836`, -0.4645400503013508}, {
      0.45356306139438823`, -0.4627117366340413}, {
      0.46120959784379173`, -0.4608156800699089}, {
      0.46878986945086326`, -0.4588534110053606}, {
      0.4763040228391662, -0.45682642110249877`}, {
      0.4837522160327177, -0.4547361644531263}, {
      0.49113461756953647`, -0.4525840587083473}, {
      0.49845140567098206`, -0.4503714861743593}, {
      0.5057027674633925, -0.4480997948750841}, {
      0.5128888982487484, -0.4457702995823246}, {
      0.5200100008213007, -0.4433842828141689}, {
      0.5270662848272921, -0.4409429958023878}, {
      0.5340579661650885, -0.4384476594295902}, {
      0.5409852664232023, -0.43589946513691297`}, {
      0.5478484123538553, -0.4332995758030316}, {
      0.5546476353798726, -0.4306491265952778}, {
      0.5613831711328444, -0.4279492257936541}, {
      0.5680552590206187, -0.42520095558852766`}, {
      0.5746641418223144, -0.4224053728527829}, {
      0.5812100653091555, -0.41956350988920066`}, {
      0.5876932778895375, -0.41667637515382366`}, {
      0.5941140302768338, -0.41374495395605376`}, {
      0.600472575178548, -0.41077020913621404`}, {
      0.6067691670055007, -0.4077530817212936}, {
      0.6130040615998258, -0.4046944915595775}, {
      0.6191775159806264, -0.4015953379348479}, {
      0.6252897881062099, -0.39845650016082557`}, {
      0.6313411366518945, -0.3952788381565036}, {
      0.6373318208024357, -0.3920631930030087}, {
      0.6432621000581861, -0.38881038748260605`}, {
      0.6491322340541525, -0.38552122660044813`}, {
      0.6549424823911685, -0.38219649808964834`}, {
      0.6606931044784466, -0.3788369729002445}, {
      0.6663843593868228, -0.3754434056725986}, {
      0.6720165057120442, -0.3720165351957635}, {
      0.6775898014474933, -0.36855708485132826`}, {
      0.6831045038657788, -0.36506576304324007`}, {
      0.6885608694086566, -0.36154326361408157`}, {
      0.6939591535847778, -0.35799026624826913`}, {
      0.6992996108747916, -0.3544074368626199}, {
      0.7045824946433594, -0.35079542798472174`}, {
      0.7098080570576614, -0.34715487911952503`}, {
      0.7149765490120049, -0.3434864171045601}, {
      0.7200882200581662, -0.33979065645417195`}, {
      0.7251433183411177, -0.336068199693148}, {
      0.7301420905398162, -0.3323196376801038}, {
      0.7350847818127457, -0.3285455499209767}, {
      0.7399716357479269, -0.3247465048729669}, {
      0.7448028943171211, -0.32092306023925243`}, {
      0.7495787978339756, -0.31707576325479253`}, {
      0.7542995849158702, -0.31320515096352436`}, {
      0.7589654924492384, -0.3093117504872454}, {
      0.7635767555581522, -0.30539607928646434`}, {
      0.7681336075759699, -0.30145864541349304`}, {
      0.7726362800198601, -0.29749994775804245`}, {
      0.7770850025680225, -0.2935204762855753}, {
      0.7814800030394423, -0.2895207122686607}, {
      0.7858215073760183, -0.2855011285115655}, {
      0.7901097396269184, -0.2814621895683095}, {
      0.7943449219350242, -0.2774043519544041}, {
      0.798527274525331, -0.27332806435248475`}, {
      0.802657015695184, -0.26923376781204106`}, {
      0.80673436180623, -0.26512189594344104`}, {
      0.810759527277979, -0.2609928751064382}, {
      0.8147327245828696, -0.2568471245933447}, {
      0.818654164242744, -0.2526850568070457}, {
      0.8225240548266384, -0.24850707743402567`}, {
      0.8263426029498045, -0.24431358561256913`}, {
      0.8301100132738802, -0.24010497409629503`}, {
      0.8338264885081326, -0.23588162941317592`}, {
      0.8374922294117025, -0.2316439320201895}, {
      0.8411074347967815, -0.2273922564537441}, {
      0.8446723015326573, -0.22312697147601485`}, {
      0.8481870245505683, -0.21884844021732233`}, {
      0.8516517968493101, -0.2145570203146814}, {
      0.8550668095015399, -0.21025306404664265`}, {
      0.8584322516607288, -0.20593691846454532`}, {
      0.8617483105687147, -0.20160892552029622`}, {
      0.8650151715638115, -0.19726942219078494`}, {
      0.8682330180894307, -0.19291874059904215`}, {
      0.871402031703178, -0.18855720813224408`}, {
      0.8745223920863862, -0.1841851475566625}, {
      0.8775942770540491, -0.17980287712965634`}, {
      0.8806178625651245, -0.1754107107087979}, {
      0.8835933227331734, -0.17100895785822287`}, {
      0.8865208298373074, -0.16659792395229117`}, {
      0.8894005543334167, -0.16217791027664208`}, {
      0.8922326648656513, -0.15774921412672452`}, {
      0.8950173282781327, -0.1533121289038806}, {
      0.8977547096268716, -0.14886694420905802`}, {
      0.9004449721918717, -0.14441394593422424`}, {
      0.9030882774893967, -0.13995341635155312`}, {
      0.9056847852843848, -0.13548563420045204`}, {
      0.9082346536029895, -0.13101087477249584`}, {
      0.9107380387452316, -0.1265294099943311}, {
      0.9131950952977466, -0.12204150850861262`}, {
      0.9156059761466114, -0.11754743575303182`}, {
      0.9179708324902389, -0.11304745403749497`}, {
      0.9202898138523233, -0.1085418226195069}, {
      0.9225630680948285, -0.10403079777781464`}, {
      0.9247907414310045, -0.09951463288436314}, {
      0.9269729784384236, -0.09499357847461394}, {
      0.9291099220720244, -0.09046788231627582}, {
      0.931201713677155, -0.0859377894764951}, {
      0.9332484930026075, -0.08140354238755146}, {
      0.9352503982136339, -0.07686538091110423}, {
      0.9372075659049369, -0.07232354240103198}, {
      0.9391201311136288, -0.0677782617649077}, {
      0.9409882273321498, -0.0632297715241499}, {
      0.9428119865211426, -0.05867830187288909}, {
      0.9445915391222749, -0.05412408073558786}, {
      0.9463270140710061, -0.049567333823451395`}, {
      0.9480185388092923, -0.04500828468966448}, {
      0.949666239298226, -0.04044715478348968}, {
      0.9512702400306052, -0.03588416350326051}, {
      0.952830664043429, -0.031319528248302364`}, {
      0.9543476329303155, -0.026753464469812936`}, {
      0.9558212668538395, -0.02218618572073308}, {
      0.9572516845577855, -0.01761790370463801}, {
      0.9586390033793154, -0.013048828323677923`}, {
      0.9599833392610466, -0.008479167725596383}, {
      0.9612848067630393, -0.00390912834985379}, {0.9625435190746905, 
      0.0006610850271172985}, {0.963759588026533, 0.005231269247499128}, {
      0.9649331241019385, 0.009801222728492796}, {0.9660642364487217, 
      0.014370745419514892`}, {0.9671530328906461, 0.01893963876016294}, {
      0.9681996199388293, 0.023507705638926447`}, {0.9692041028030469, 
      0.02807475035262107}, {0.9701665854029348, 0.03264057856652397}, {
      0.9710871703790886, 0.03720499727518897}, {0.9719659591040611, 
      0.041767814763920844`}, {0.9728030516932543, 0.0463288405708884}, {
      0.97359854701571, 0.05088788544985676}, {0.9743525427047954, 
      0.055444761333519554`}, {0.9750651351687855, 0.059999281297412416`}, {
      0.9757364196013412, 0.06455125952438949}, {0.9763664899918849, 
      0.06910051126964517}, {0.9769554391358721, 0.07364685282626382}, {
      0.9775033586449605, 0.07819010149128046}, {0.9780103389570765, 
      0.08273007553223599}, {0.97847646934638, 0.08726659415421083}, {
      0.9789018379331281, 0.09179947746732117}, {0.9792865316934378, 
      0.09632854645466266}, {0.9796306364689487, 0.10085362294068627`}, {
      0.979934236976387, 0.10537452955999194`}, {0.9801974168170317, 
      0.10989108972652537`}, {0.9804202584860817, 0.11440312760316419`}, {
      0.9806028433819287, 0.11891046807167968`}, {0.9807452518153332, 
      0.12341293670306051`}, {0.9808475630185068, 0.12791035972818557`}, {
      0.9809098551541013, 0.13240256400883274`}, {0.9809322053241059, 
      0.1368893770090113}, {0.9809146895786529, 0.14137062676660525`}, {
      0.9808573829247348, 0.14584614186531572`}, {0.9807603593348324, 
      0.15031575140689035`}, {0.9806236917554562, 0.15477928498362814`}, {
      0.980447452115602, 0.15923657265114807`}, {0.9802317113351229, 
      0.1636874449014105}, {0.9799765393330173, 0.16813173263597997`}, {
      0.9796820050356372, 0.17256926713951892`}, {0.9793481763848153, 
      0.17699988005350112`}, {0.9789751203459146, 0.18142340335013463`}, {
      0.9785629029158012, 0.18583966930648385`}, {0.9781115891307417, 
      0.19024851047878039`}, {0.9776212430742276, 0.19464975967691253`}, {
      0.9770919278847277, 0.19904324993908373`}, {0.9765237057633699, 
      0.20342881450662995`}, {0.975916637981556, 0.20780628679898638`}, {
      0.9752707848885086, 0.21217550038879396`}, {0.9745862059187533, 
      0.216536288977136}, {0.973862959599539, 0.22088848636889613`}, {
      0.9731011035581956, 0.22523192644822757`}, {0.9723006945294338, 
      0.22956644315412522`}, {0.9714617883625868, 0.23389187045609128`}, {
      0.9705844400287968, 0.23820804232988507`}, {0.9696687036281484, 
      0.2425147927333488}, {0.9687146323967499, 0.2468119555822999}, {
      0.9677222787137667, 0.2510993647264813}, {0.9666916941084059, 
      0.25537685392556114`}, {0.9656229292668574, 0.25964425682517267`}, {
      0.9645160340391906, 0.26390140693298647`}, {0.9633710574462118, 
      0.2681481375948059}, {0.9621880476862822, 0.2723842819706771}, {
      0.9609670521421005, 0.27660967301100553`}, {0.9597081173874512, 
      0.28082414343267015`}, {0.9584112891939215, 0.28502752569512674`}, {
      0.9570766125375898, 0.28921965197649213`}, {0.955704131605687, 
      0.2934003541496007}, {0.9542938898032343, 0.2975694637580247}, {
      0.9528459297596591, 0.30172681199204976`}, {0.9513602933353931, 
      0.3058722296645976}, {0.9498370216284528, 0.3100055471870867}, {
      0.9482761549810084, 0.31412659454522335`}, {0.9466777329859403, 
      0.3182352012747133}, {0.9450417944933895, 0.32233119643688657`}, {
      0.9433683776173025, 0.32641440859422644`}, {0.9416575197419741, 
      0.33048466578579366`}, {0.9399092575285923, 0.3345417955025373}, {
      0.938123626921787, 0.33858562466248393`}, {0.9363006631561863, 
      0.3426159795857956}, {0.9344404007629842, 0.34663268596968844`}, {
      0.932542873576522, 0.35063556886320224`}, {0.9306081147408876, 
      0.35462445264181264`}, {0.9286361567165361, 0.3585991609818763}, {
      0.9266270312869352, 0.3625595168349002}, {0.9245807695652394, 
      0.36650534240162563`}, {0.9224974020009956, 0.3704364591059176}, {
      0.9203769583868866, 0.3743526875684497}, {0.9182194678655136, 
      0.3782538475801758}, {0.9160249589362235, 0.38213975807557776`}, {
      0.9137934594619855, 0.3860102371056802}, {0.9115249966763197, 
      0.3898651018108216}, {0.9092195971902841, 0.3937041683931724}, {
      0.9068772869995234, 0.3975272520889891}, {0.9044980914913847, 
      0.40133416714059444`}, {0.9020820354521052, 0.4051247267680733}, {
      0.8996291430740767, 0.4088987431406726}, {0.8971394379631918, 
      0.4126560273478955}, {0.8946129431462783, 0.4163963893702784}, {
      0.8920496810786249, 0.4201196380498387}, {0.8894496736516079, 
      0.42382558106018353`}, {0.8868129422004195, 0.4275140248762661}, {
      0.8841395075119092, 0.4311847747437789}, {0.8814293898325399, 
      0.4348376346481713}, {0.8786826088764683, 0.43847240728327924`}, {
      0.8758991838337545, 0.44208889401955453`}, {0.8730791333787086, 
      0.44568689487188107`}, {0.8702224756783807, 0.4492662084669648}, {
      0.8673292284012024, 0.4528266320102842}, {0.8643994087257862, 
      0.45636796125258783`}, {0.8614330333498923, 0.45988999045592477`}, {
      0.8584301184995687, 0.4633925123591943}, {0.8553906799384754, 
      0.46687531814320027`}, {0.8523147329773991, 0.47033819739519483`}, {
      0.8492022924839694, 0.4737809380728976}, {0.8460533728925836, 
      0.47720332646797375`}, {0.842867988214552, 0.48060514716895597`}, {
      0.8396461520484714, 0.4839861830235941}, {0.836387877590839, 
      0.4873462151006157}, {0.8330931776469158, 0.49068502265088115`}, {
      0.8297620646418518, 0.4940023830679158}, {0.8263945506320834, 
      0.49729807184780134`}, {0.822990647317016, 0.5005718625484092}, {
      0.8195503660510022, 0.5038235267479562}, {0.8160737178556307, 
      0.5070528340028646}, {0.8125607134323375, 0.5102595518049072}, {
      0.8090113631753534, 0.5134434455376163}, {0.8054256771850024, 
      0.516604278431938}, {0.8018036652813653, 0.5197418115211098}, {
      0.7981453370183244, 0.5228558035947404}, {0.7944507016980044, 
      0.5259460111520698}, {0.7907197683856265, 0.5290121883543879}, {
      0.7869525459247934, 0.5320540869765865}, {0.783149042953221, 
      0.5350714563578245}, {0.7793092679189382, 0.5380640433512783}, {
      0.7754332290969713, 0.5410315922729554}, {0.771520934606535, 
      0.5439738448495439}, {0.767572392428749, 0.5468905401652733}, {
      0.7635876104249031, 0.5497814146077576}, {0.7595665963552933, 
      0.5526462018127957}, {0.7555093578986506, 0.5554846326080978}, {
      0.7514159026721885, 0.5582964349559123}, {0.747286238252294, 
      0.5610813338945195}, {0.7431203721958874, 0.5638390514785644}, {
      0.7389183120624799, 0.5665693067181956}, {0.7346800654369556, 
      0.5692718155169787}, {0.7304056399531089, 0.57194629060855}, {
      0.7260950433179677, 0.5745924414919774}, {0.7217482833369342, 
      0.5772099743657934}, {0.7173653679397768, 0.5797985920606624}, {
      0.7129463052075088, 0.5823579939706484}, {0.7084911034001883, 
      0.5848878759830416}, {0.7039997709856793, 0.5873879304067079}, {
      0.699472316669412, 0.5898578458989181}, {0.6949087494251832, 
      0.5922973073906186}, {0.6903090785270409, 0.5947059960100983}, {
      0.6856733135822969, 0.59708358900501}, {0.6810014645657134, 
      0.5994297596627006}, {0.6762935418549135, 0.6017441772288038}, {
      0.6715495562670653, 0.6040265068240486}, {0.6667695190968927, 
      0.6062764093592339}, {0.6619534421560688, 0.6084935414483205}, {
      0.6571013378140473, 0.6106775553195869}, {0.652213219040396, 
      0.6128280987247982}, {0.6472890994486901, 0.6149448148463316}, {
      0.6423289933420356, 0.6170273422022043}, {0.6373329157602886, 
      0.6190753145489454}, {0.6323008825290419, 0.6210883607822516}, {
      0.627232910310456, 0.6230661048353685}, {0.62212901665601, 
      0.6250081655751324}, {0.6169892200612552, 0.6269141566956099}, {
      0.611813540022658, 0.6287836866092685}, {0.6066019970966183, 
      0.6306163583356114}, {0.6013546129607606, 0.6324117693872044}, {
      0.5960714104775922, 0.6341695116530275}, {0.5907524137606321, 
      0.6358891712790719}, {0.585397648243119, 0.6375703285461122}, {
      0.5800071407494064, 0.6392125577445733}, {0.5745809195691665, 
      0.6408154270464119}, {0.569119014534522, 0.6423784983739327}, {
      0.563621457100236, 0.6439013272654532}, {0.5580882804270946, 
      0.6453834627377328}, {0.5525195194686215, 0.6468244471450756}, {
      0.5469152110612748, 0.6482238160350177}, {0.5412753940182777, 
      0.6495810980005049}, {0.5356001092272479, 0.6508958145284665}, {
      0.529889399751794, 0.652167479844685}, {0.5241433109372581, 
      0.6533956007548646}, {0.51836189052079, 0.6545796764817924}, {
      0.5125451887459516, 0.65571919849849}, {0.5066932584820544, 
      0.6568136503572461}, {0.5008061553484476, 0.6578625075144207}, {
      0.49488393784398305`, 0.6588652371509077}, {0.4889266674818935, 
      0.6598212979881425}, {0.4829344089303359, 0.6607301400995347}, {
      0.4769072301588597, 0.6615912047172087}, {0.4708452025910769, 
      0.6624039240339282}, {0.46474840126382166`, 0.6631677210000808}, {
      0.45861690499310487`, 0.6638820091155961}, {0.45245079654718096`, 
      0.6645461922166682}, {0.44625016282706403`, 0.6651596642571523}, {
      0.4400150950548452, 0.6657218090845018}, {0.4337456889701821, 
      0.6662320002101122}, {0.4274420450353508, 0.6666896005739348}, {
      0.42110426864926986`, 0.667093962303224}, {0.41473247037092825`, 
      0.6674444264652785}, {0.40832676615267144`, 0.6677403228140364}, {
      0.4018872775838228, 0.6679809695303859}, {0.3954141321451438, 
      0.6681656729560496}, {0.3889074634746621, 0.6682937273209023}, {
      0.38236741164542526`, 0.6683644144635841}, {0.3757941234557666, 
      0.6683770035452677}, {0.36918775273270243`, 0.6683307507564465}, {
      0.36254846064911056`, 0.6682248990166049}, {0.35587641605537806`, 
      0.6680586776666424}, {0.34917179582624003`, 0.6678313021539226}, {
      0.34243478522357207`, 0.6675419737098235}, {0.33566557827593996`, 
      0.6671898790196725}, {0.3288643781757531, 0.6667741898849552}, {
      0.3220313976949148, 0.6662940628776949}, {0.3151668596199111, 
      0.6657486389869113}, {0.3082709972073309, 0.6651370432570737}, {
      0.3013440546608651, 0.6644583844184818}, {0.2943862876308891, 
      0.6637117545095171}, {0.2873979637377945, 0.6628962284907274}, {
      0.28037936312030026`, 0.6620108638507248}, {0.2733307790100391, 
      0.6610547002039003}, {0.2662525183337902, 0.6600267588799793}, {
      0.25914490234480153`, 0.6589260425054746}, {0.2520082672847269, 
      0.6577515345771203}, {0.24484296507778683`, 0.6565021990274084}, {
      0.23764936405885062`, 0.6551769797823848}, {0.2304278497372322, 
      0.6537748003119115}, {0.2231788255980902, 0.6522945631726441}, {
      0.21590271394342783`, 0.6507351495440343}, {0.20859995677479828`, 
      0.6490954187577245}, {0.20127101671993702`, 0.6473742078207729}, {
      0.19391637800566466`, 0.6455703309332218}, {0.18653654747953247`, 
      0.6436825790006058}, {0.17913205568281776`, 0.6417097191420941}, {
      0.17170345797761805`, 0.6396504941950611}, {0.16425133573094178`, 
      0.637503622217}, {0.1567762975588497, 0.635267795985818}, {
      0.1492789806338637, 0.6329416824996996}, {0.14176005205903175`, 
      0.630523922477878}, {0.13422021031221487`, 0.6280131298638307}, {
      0.12666018676434873`, 0.625407891332611}, {0.11908074727562537`, 
      0.6227067658042372}, {0.11148269387374168`, 0.6199082839653008}, {
      0.10386686651856869`, 0.6170109478012136}, {0.0962341449578105, 
      0.6140132301418035}, {0.08858545067844252, 0.6109135742232863}, {
      0.08092174895894486, 0.6077103932699899}, {0.07324405102757796, 
      0.604402070099596}, {0.06555341633218197, 0.6009869567560893}, {
      0.057850954927218146`, 0.5974633741750712}, {0.0501378299840077, 
      0.5938296118866155}, {0.042415260430358884`, 0.5900839277614045}, {
      0.03468452372600439, 0.5862245478065135}, {0.02694695878049464, 
      0.5822496660178935}, {0.01920396902040549, 0.5781574442973546}, {
      0.011457025612915969`, 0.5739460124426758}, {0.0037076708529882284`, 
      0.5696134682203737}, {-0.004042478278468328, 
      0.5651578775316477}, {-0.011791726377951037`, 
      0.5605772746831051}, {-0.019538295696665074`, 
      0.5558696627750491}, {-0.027280322024179494`, 
      0.5510330142214043}, {-0.03501585037088865, 
      0.5460652714167592}, {-0.04274283044184879, 
      0.5409643475675374}, {-0.0504591118947712, 
      0.5357281277059683}, {-0.0581624393752744, 
      0.5303544699073359}, {-0.06585044732294904, 
      0.5248412067329313}, {-0.07352065454239333, 
      0.5191861469232468}, {-0.08117045853416018, 
      0.5133870773682168}, {-0.08879712958154994, 
      0.5074417653837523}, {-0.09639780459041904, 
      0.5013479613264278}, {-0.1039694806806924, 
      0.49510340158096316`}, {-0.11150900853011052`, 
      0.4887058119581072}, {-0.11901308547295797`, 
      0.482152911543655}, {-0.1264782483591635, 
      0.4754424170426242}, {-0.13390086618229194`, 
      0.46857204766604654`}, {-0.14127713248863105`, 
      0.46153953061139286`}, {-0.14860305758388406`, 
      0.4543426071912987}, {-0.15587446055898954`, 
      0.4469790396689624}, {-0.16308696116238725`, 
      0.4394466188622891}, {-0.17023597155272305`, 
      0.43174317258248585`}, {-0.17731668797362946`, 
      0.42386657497628816`}, {-0.18432408240092876`, 
      0.4158147568442083}, {-0.19125289422247985`, 
      0.4075857170100123}, {-0.19809762202202275`, 
      0.39917753481889146`}, {-0.20485251555085654`, 
      0.39058838384331257`}, {-0.21151156798509438`, 
      0.3818165468760722}, {-0.21806850858163548`, 
      0.3728604322893918}, {-0.22451679586291495`, 
      0.36371859183665484`}, {-0.23084961147893898`, 
      0.3543897399692577}, {-0.2370598549150416, 
      0.3448727747346228}, {-0.24314013923510533`, 
      0.3351668003122579}, {-0.24908278807249137`, 
      0.3252711512323519}, {-0.25487983410435233`, 
      0.3151854183052431}, {-0.26052301926896254`, 
      0.3049094762696182}, {-0.26600379700967836`, 
      0.29444351314193}, {-0.2713133368524491, 
      0.2837880612186777}, {-0.2764425316455829, 
      0.272944029646339}, {-0.28138200780967054`, 
      0.26191273843040974`}, {-0.28612213896091554`, 
      0.25069595370482534`}, {-0.29065306328111146`, 
      0.2392959240258294}, {-0.2949647050104429, 
      0.22771541739015766`}, {-0.2990468004332425, 
      0.21595775860657881`}, {-0.30288892870973955`, 
      0.204026866573124}, {-0.30648054787650436`, 
      0.19192729093097216`}, {-0.30981103629252593`, 
      0.17966424748172333`}, {-0.31286973974458543`, 
      0.16724365167010832`}, {-0.31564602434297523`, 
      0.1546721493521597}, {-0.31812933523531517`, 
      0.14195714399331805`}, {-0.3203092610415494, 
      0.12910681937636753`}, {-0.32217560376741283`, 
      0.11613015685057565`}, {-0.3237184537881426, 
      0.1030369461264785}, {-0.32492826931177704`, 
      0.08983778862111967}, {-0.3257959595364069, 
      0.07654409239178617}, {-0.3263129705142821, 
      0.06316805776739465}, {-0.32647137253546143`, 
      0.04972265289962182}, {-0.32626394765398026`, 
      0.03622157861301954}, {-0.3256842758107653, 
      0.02267922213498598}, {-0.3247268178708972, 
      0.009110599530285462}, {-0.3233869937994932, -0.004468713054378624}, \
{-0.3216612541608198, -0.018042658920694725`}, {-0.31954714314789573`, \
-0.03159479038190073}, {-0.31704335144081996`, -0.04510836728597832}, \
{-0.3141497573538838, -0.058566463107420516`}, {-0.3108674549626364, \
-0.07195207692613538}, {-0.3071987681964587, -0.08524824936720576}, \
{-0.3031472502294549, -0.09843818039043056}, {-0.29871766788824794`, \
-0.11150534670596791`}, {-0.29391597120211577`, -0.12443361656024474`}, \
{-0.2887492486294812, -0.13720735968815465`}, {-0.2832256688852318, \
-0.14981155036189658`}, {-0.277354410646846, -0.16223186167700246`}, \
{-0.27114558171741615`, -0.17445474949099876`}, {-0.26461012945753853`, \
-0.1864675247549275}, {-0.2577597444572672, -0.19825841333536012`}, \
{-0.25060675950029276`, -0.2098166027962496}, {-0.24316404587639376`, \
-0.22113227597797394`}, {-0.23544490903051665`, -0.23219663155879056`}, \
{-0.22746298540664953`, -0.24300189209778586`}, {-0.21923214216359835`, \
-0.25354130032767286`}, {-0.21076638122099298`, -0.26380910468347574`}, \
{-0.2020797488509077, -0.2738005352158713}, {-0.1931862517764382, \
-0.2835117711456931}, {-0.18409978048523049`, -0.2929399013716534}, \
{-0.1748340402233077, -0.30208287925172256`}, {-0.16540248991047998`, \
-0.31093947294638885`}, {-0.15581828901884345`, -0.31950921254664555`}, \
{-0.14609425228393758`, -0.3277923351187371}, {-0.13624281197567942`, \
-0.3357897286889522}, {-0.12627598734323933`, -0.34350287607200297`}, \
{-0.11620536076328628`, -0.3509337993218591}, {-0.10604206006227535`, \
-0.3580850054594076}, {-0.0957967464478117, -0.3649594340110585}, \
{-0.08547960746838873, -0.3715604067794592}, {-0.07510035442163993, \
-0.3778915801639572}, {-0.0646682236454087, -0.3839569002556689}, \
{-0.05419198115037924, -0.38976056085059835`}, {-0.04367993008496925, \
-0.39530696445428454`}, {-0.03313992056024988, -0.4006006862925888}, \
{-0.022579361402780173`, -0.405646441294812}, {-0.012005233444733695`, \
-0.4104490539764997}, {-0.0014241040022010924`, -0.41501343111908084`}, {
      0.00915785776696788, -0.4193445371208522}, {
      0.019734864895740136`, -0.42344737187776044`}, {
      0.03030149626451177, -0.4273269510419381}, {
      0.04085268008172726, -0.43098828850010956`}, {
      0.05138367735770681, -0.4344363809119526}, {
      0.06189006551306287, -0.4376761941495213}, {
      0.07236772223818716, -0.44071265148225475`}, {
      0.08281280969821299, -0.44355062335733275`}, {
      0.0932217591584577, -0.44619491863171507`}, {
      0.10359125608844709`, -0.448650277119703}, {
      0.11391822578801938`, -0.4509213633279574}, {
      0.1241998195664948, -0.45301276125832113`}, {
      0.134433401495269, -0.4549289701673098}, {
      0.14461653574524447`, -0.45667440117958075`}, {
      0.1547469745130598, -0.4582533746609353}, {
      0.16482264653393192`, -0.45967011826436116`}, {
      0.1748416461739247, -0.4609287655702039}, {
      0.18480222309044597`, -0.4620333552487299}, {
      0.19470277244661668`, -0.4629878306800769}, {
      0.20454182566272627`, -0.46379603997286684`}, {
      0.2143180416861791, -0.4644617363285841}, {
      0.22403019876004968`, -0.4649885787041967}, {
      0.2336771866695151, -0.4653801327304433}, {
      0.24325799944494755`, -0.465639871847734}, {
      0.2527717285002635, -0.46577117862574036`}, {
      0.26221755618518366`, -0.46577734623651085`}, {
      0.2715947497303115, -0.46566158005434977`}, {
      0.2809026555643492, -0.4654269993587852}, {
      0.29014069398329884`, -0.4650766391197308}, {
      0.2993083541521227, -0.46461345184645303`}, {
      0.308405189420026, -0.4640403094842066}, {
      0.3174308129312657, -0.463360005344421}, {
      0.32638489351415834`, -0.46257525605612815`}, {
      0.33526715183174616`, -0.4616887035279378}, {
      0.3440773567783739, -0.4607029169113069}, {
      0.35281532210721706`, -0.4596203945571341}, {
      0.36148090327457916`, -0.45844356595885016`}, {
      0.3700739944875395, -0.45717479367618746`}, {
      0.37859452594227094`, -0.4558163752347094}, {
      0.3870424612410649, -0.45437054499697255`}, {
      0.39541779497679086`, -0.4528394760018948}, {
      0.40372055047417865`, -0.45122528176951954`}, {
      0.41195077767794447`, -0.4495300180689074}, {
      0.4201085511783851, -0.447755684647363}, {
      0.428193968365637, -0.44590422691961984`}, {
      0.4362071477043418, -0.4439775376159704}, {
      0.444148227120973, -0.44197745838864305`}, {
      0.45201736249656604`, -0.43990578137600334`}, {
      0.45981472625805286`, -0.43776425072439284`}, {
      0.46754050606183367`, -0.4355545640676238}, {
      0.4751949035636261, -0.4332783739643213}, {
      0.482778133269015, -0.43093728929345626`}, {
      0.49029042145948454`, -0.4285328766085355}, {
      0.4977320051890521, -0.42606666145102456`}, {
      0.5051031313469395, -0.4235401296236649}, {
      0.512404055782014, -0.4209547284244218}, {
      0.5196350424850078, -0.4183118678418575}, {
      0.5267963628247865, -0.4156129217127716}, {
      0.5338882948351782, -0.412859228842988}, {
      0.5409111225491046, -0.4100520940921949}, {
      0.5478651353769651, -0.4071927894237649}, {
      0.5547506275264277, -0.4042825549204936}, {
      0.5615678974609615, -0.4013225997672042}, {
      0.5683172473946251, -0.398314103201166}, {
      0.5749989828207809, -0.3952582154312723}, {
      0.5816134120725626, -0.39215605852691776`}, {
      0.5881608459130605, -0.3890087272775046}, {
      0.5946415971533248, -0.3858172900234963}, {
      0.6010559802964067, -0.3825827894599219}, {
      0.6074043112057765, -0.37930624341321856`}, {
      0.6136869067965623, -0.3759886455922817}, {
      0.6199040847481554, -0.37263096631457393`}, {
      0.6260561632368207, -0.3692341532081237}, {
      0.63214346068704, -0.3657991318902234}, {
      0.6381662955403968, -0.36232680662361677`}, {
      0.6441249860408873, -0.3588180609509428}, {
      0.6500198500356144, -0.355273758308183}, {
      0.6558512047898888, -0.35169474261783484`}, {
      0.6616193668158227, -0.34808183886251587`}, {
      0.6673246517135591, -0.34443585363967755`}, {
      0.6729673740243375, -0.3407575756980894}, {
      0.6785478470946418, -0.3370477764567316}, {
      0.6840663829507314, -0.33330721050671347`}, {
      0.6895232921828938, -0.3295366160968146}, {
      0.6949188838388024, -0.32573671560322637`}, {
      0.7002534653254022, -0.3219082159840502}, {
      0.7055273423187801, -0.31805180921909176`}, {
      0.7107408186815126, -0.3141681727354698}, {
      0.7158941963870153, -0.3102579698195416}, {
      0.720987775450447, -0.3063218500156281}, {
      0.7260218538657499, -0.30236044951200564`}, {
      0.7309967275484345, -0.2983743915146136}, {
      0.7359126902837395, -0.29436428660891223`}, {
      0.7407700336798239, -0.2903307331103079}, {
      0.7455690471256642, -0.2862743174035489}, {
      0.7503100177533566, -0.28219561427148027`}, {
      0.7549932304045359, -0.27809518721353094`}, {
      0.7596189676006463, -0.27397358875429406`}, {
      0.7641875095168108, -0.2698313607425476}, {
      0.7686991339590661, -0.265669034641049}, {
      0.7731541163447399, -0.26148713180742666`}, {
      0.7775527296857638, -0.25728616376647784`}, {
      0.781895244574728, -0.2530666324741721}, {
      0.7861819291734933, -0.24882903057364797`}, {
      0.7904130492041895, -0.24457384164347976`}, {
      0.7945888679424402, -0.24030154043848154`}, {
      0.79870964621266, -0.23601259312330544`}, {
      0.8027756423852853, -0.2317074574990817}, {
      0.806787112375802, -0.22738658322333893`}, {
      0.8107443096454477, -0.22305041202343445`}, {
      0.8146474852034692, -0.2186993779037157}, {
      0.8184968876108246, -0.214333907346626}, {
      0.8222927629852281, -0.20995441950795984`}, {
      0.8260353550074379, -0.2055613264064657}, {
      0.8297249049286971, -0.2011550331079866}, {
      0.8333616515792421, -0.19673593790432226`}, {
      0.8369458313777961, -0.1923044324869897}, {
      0.8404776783419742, -0.18786090211605266`}, {
      0.8439574240995275, -0.18340572578418435`}, {
      0.8473852979003604, -0.17893927637612153`}, {
      0.8507615266292583, -0.17446192082366285`}, {
      0.8540863348192661, -0.16997402025635855`}, {
      0.8573599446656641, -0.16547593014803316`}, {
      0.8605825760404882, -0.16096800045927817`}, {
      0.8637544465075462, -0.15645057577604637`}, {
      0.8668757713378852, -0.1519239954444753}, {
      0.8699467635256667, -0.14738859370206203`}, {
      0.8729676338044099, -0.14284469980530803`}, {
      0.8759385906635656, -0.13829263815394807`}, {
      0.8788598403653854, -0.133732728411873}, {
      0.8817315869620534, -0.12916528562485324`}, {
      0.8845540323130495, -0.12459062033516492`}, {
      0.8873273761027155, -0.12000903869321793`}, {
      0.8900518158579973, -0.11542084256628136`}, {
      0.8927275469663368, -0.11082632964439851`}, {
      0.8953547626936914, -0.10622579354358033`}, {
      0.8979336542026575, -0.10161952390636364`}, {
      0.9004644105706785, -0.09700780649981665}, {
      0.902947218808318, -0.09239092331107233}, {
      0.9053822638775784, -0.08776915264046695}, {
      0.907769728710251, -0.08314276919235851}, {
      0.9101097942262795, -0.0785120441636976}, {
      0.9124026393521236, -0.07387724533042028}, {
      0.9146484410391088, -0.06923863713173073}, {
      0.9168473742817493, -0.06459648075233877}, {
      0.9189996121360335, -0.05995103420271551}, {
      0.92110532573766, -0.055302552397428026`}, {
      0.9231646843202144, -0.05065128723161211}, {
      0.925177855233278, -0.04599748765564014}, {
      0.9271450039604593, -0.04134139974803939}, {
      0.9290662941373401, -0.03668326678671393}, {
      0.9309418875693295, -0.03202332931852212}, {
      0.9327719442494188, -0.027361825227259474`}, {
      0.9345566223758307, -0.022698989800095357`}, {
      0.9362960783695577, -0.01803505579251046}, {
      0.9379904668917841, -0.013370253491780337`}, {
      0.9396399408611865, -0.008704810779048961}, {
      0.9412446514711101, -0.00403895319003489}, {0.9428047482066145, 
      0.0006270960255887704}, {0.9443203788613873, 0.005293115846100653}, {
      0.9457916895545218, 0.00995888742088497}, {0.9472188247471558, 
      0.014624194014795394`}, {0.9486019272589692, 0.019288820953692627`}, {
      0.9499411382845377, 0.0239525555711204}, {0.9512365974095421, 
      0.028615187156085756`}, {0.9524884426268293, 0.03327650690191038}, {
      0.9536968103523256, 0.037936307856120954`}, {0.9548618354407999, 
      0.04259438487134723}, {0.9559836512014773, 0.04725053455719756}, {
      0.9570623894134994, 0.051904555233082586`}, {0.9580981803412345, 
      0.056556246881958414`}, {0.9590911527494348, 0.061205411104961756`}, {
      0.9600414339182407, 0.06585185107691}, {0.9609491496580334, 
      0.07049537150264014}, {0.9618144243241339, 0.0751357785741612}, {
      0.962637380831351, 0.07977287992859547}, {0.9634181406683766, 
      0.08440648460688449}, {0.9641568239120302, 0.08903640301323675}, {
      0.964853549241352, 0.09366244687529403}, {0.9655084339515468, 
      0.09828442920499475}, {0.9661215939677784, 0.1029021642601126}, {
      0.9666931438588154, 0.10751546750644969`}, {0.9672231968505306, 
      0.11212415558066394`}, {0.9677118648392533, 0.11672804625371087`}, {
      0.9681592584049777, 0.12132695839488061`}, {0.9685654868244268, 
      0.12592071193641127`}, {0.9689306580839742, 0.13050912783866053`}, {
      0.9692548788924252, 0.1350920280558175}, {0.9695382546936583, 
      0.13966923550213758`}, {0.9697808896791279, 0.14424057401868348`}, {
      0.9699828868002319, 0.14880586834055556`}, {0.9701443477805437, 
      0.15336494406459583`}, {0.9702653731279108, 0.1579176276175495}, {
      0.9703460621464228, 0.16246374622466891`}, {0.9703865129482486, 
      0.16700312787874494`}, {0.9703868224653468, 0.17153560130955092`}, {
      0.9703470864610493, 0.1760609959536851}, {0.9702673995415206, 
      0.18057914192479746`}, {0.9701478551670957, 0.18508986998418728`}, {
      0.969988545663496, 0.189593011511758}, {0.9697895622329288, 
      0.19408839847731632`}, {0.9695509949650684, 0.19857586341220274`}, {
      0.9692729328479239, 0.20305523938124082`}, {0.9689554637785948, 
      0.2075263599549932}, {0.9685986745739149, 0.21198905918231176`}, {
      0.9682026509809899, 0.21644317156317078`}, {0.9677674776876273, 
      0.2208885320217708}, {0.9672932383326631, 0.22532497587990236`}, {
      0.9667800155161868, 0.22975233883055798`}, {0.9662278908096662, 
      0.2341704569117818}, {0.9656369447659755, 0.23857916648074573`}, {
      0.9650072569293277, 0.2429783041880418}, {0.9643389058451142, 
      0.24736770695217997`}, {0.9636319690696543, 0.25174721193428135`}, {
      0.9628865231798555, 0.2561166565129566}, {0.9621026437827881, 
      0.26047587825935975`}, {0.9612804055251769, 0.26482471491240694`}, {
      0.9604198821028102, 0.26916300435415147`}, {0.9595211462698707, 
      0.27349058458530456`}, {0.9585842698481903, 0.27780729370089324`}, {
      0.9576093237364297, 0.28211296986604517`}, {0.956596377919187, 
      0.28640745129189193`}, {0.9555455014760372, 0.29069057621158123`}, {
      0.9544567625905055, 0.29496218285638925`}, {0.9533302285589749, 
      0.299222109431924}, {0.952165965799534, 0.30347019409441106`}, {
      0.9509640398607654, 0.3077062749270525}, {0.9497245154304769, 
      0.31193018991645094`}, {0.94844745634438, 0.31614177692908957`}, {
      0.9471329255947173, 0.32034087368785974`}, {0.945780985338841, 
      0.32452731774862775`}, {0.9443916969077467, 0.32870094647683196`}, {
      0.9429651208145637, 0.33286159702410234`}, {0.9415013167630061, 
      0.3370091063048936}, {0.9400003436557862, 0.3411433109731238}, {
      0.9384622596029946, 0.34526404739880995`}, {0.9368871219304479, 
      0.3493711516446926}, {0.9352749871880096, 0.35346445944284066`}, {
      0.9336259111578847, 0.3575438061712284}, {0.9319399488628932, 
      0.3616090268302765}, {0.9302171545747238, 0.3656599560193488}, {
      0.9284575818221723, 0.36969642791319607`}, {0.9266612833993679, 
      0.3737182762383391}, {0.9248283113739897, 0.3777253342493824}, {
      0.9229587170954775, 0.3817174347052506}, {0.9210525512032407, 
      0.38569440984533826`}, {0.9191098636348677, 0.3896560913655661}, {
      0.9171307036343406, 0.3936023103943339}, {0.9151151197602575, 
      0.39753289746836257`}, {0.9130631598940678, 0.4014476825084161}, {
      0.9109748712483224, 0.40534649479489565`}, {0.908850300374944, 
      0.4092291629432963}, {0.9066894931735207, 0.4130955148795185}, {
      0.9044924948996281, 0.41694537781502494`}, {0.9022593501731826, 
      0.42077857822183445`}, {0.8999901029868302, 0.4245949418073436}, {
      0.8976847967143771, 0.4283942934889671}, {0.8953434741192632, 
      0.4321764573685888}, {0.892966177363087, 0.43594125670681194`}, {
      0.8905529480141828, 0.43968851389700225`}, {0.8881038270562576, 
      0.44341805043911214`}, {0.8856188548970906, 0.4471296869132774}, {
      0.883098071377303, 0.4508232429531773}, {0.8805415157791998, 
      0.454498537219147}, {0.877949226835692, 0.45815538737103406`}, {
      0.8753212427393021, 0.4617936100407874}, {0.872657601151261, 
      0.4654130208047696}, {0.8699583392106995, 0.4690134341557821}, {
      0.8672234935439422, 0.4725946634747923}, {0.864453100273909, 
      0.4761565210023524}, {0.8616471950296302, 0.47969881780969903`}, {
      0.8588058129558825, 0.48322136376952285`}, {0.8559289887229519, 
      0.4867239675263963}, {0.8530167565365299, 0.490206436466849}, {
      0.8500691501477514, 0.4936685766890788}, {0.8470862028633797, 
      0.4971101929722865}, {0.8440679475561474, 0.5005310887456224}, {
      0.841014416675261, 0.5039310660567327}, {0.8379256422570751, 
      0.5073099255398933}, {0.8348016559359474, 0.5106674663837176}, {
      0.8316424889552803, 0.5140034862984261}, {0.8284481721787593, 
      0.5173177814826638}, {0.825218736101797, 0.5206101465898539}, {
      0.8219542108631905, 0.5238803746940707}, {0.8186546262570047, 
      0.5271282572554208}, {0.8153200117446879, 0.5303535840849166}, {
      0.8119503964674336, 0.5335561433088281}, {0.8085458092587954, 
      0.5367357213324966}, {0.8051062786575687, 0.5398921028035985}, {
      0.8016318329209494, 0.5430250705748384}, {0.7981225000379816, 
      0.546134405666061}, {0.7945783077433055, 0.5492198872257607}, {
      0.7909992835312197, 0.5522812924919756}, {0.7873854546700695, 
      0.5553183967525469}, {0.7837368482169748, 0.558330973304727}, {
      0.7800534910329124, 0.5613187934141181}, {0.7763354097981664, 
      0.5642816262729223}, {0.7725826310281609, 0.5672192389574872}, {
      0.7687951810896926, 0.5701313963851226}, {0.7649730862175766, 
      0.5730178612701748}, {0.7611163725317253, 0.5758783940793334}, {
      0.7572250660546733, 0.5787127529861519}, {0.7532991927295715, 
      0.5815206938247618}, {0.7493387784386634, 0.5843019700427564}, {
      0.7453438490222679, 0.5870563326532241}, {0.7413144302982846, 
      0.5897835301859066}, {0.7372505480822443, 0.5924833086374599}, {
      0.7331522282079266, 0.5951554114207944}, {0.729019496548565, 
      0.5977995793134674}, {0.7248523790386648, 0.6004155504051054}, {
      0.7206509016964566, 0.6030030600438289}, {0.7164150906470105, 
      0.6055618407816532}, {0.7121449721460378, 0.6080916223188388}, {
      0.7078405726044064, 0.6105921314471624}, {0.7035019186133983, 
      0.6130630919920802}, {0.6991290369707382, 0.6155042247537555}, {
      0.6947219547074246, 0.6179152474469181}, {0.6902806991153922, 
      0.6202958746395256}, {0.6858052977760429, 0.6226458176901962}, {
      0.6812957785896745, 0.6249647846843776}, {0.6767521698058468, 
      0.6272524803692204}, {0.6721745000547201, 0.6295086060871223}, {
      0.667562798379404, 0.6317328597079049}, {0.6629170942693589, 
      0.63392493555959}, {0.6582374176948893, 0.6360845243577363}, {
      0.6535237991427746, 0.6382113131332997}, {0.6487762696530806, 
      0.6403049851589756}, {0.6439948608572008, 0.642365219873986}, {
      0.6391796050171756, 0.6443916928072677}, {0.6343305350663403, 
      0.6463840754990205}, {0.6294476846513558, 0.6483420354205707}, {
      0.6245310881756782, 0.650265235892506}, {0.6195807808445236, 
      0.6521533360010358}, {0.6145967987113911, 0.6540059905125291}, {
      0.6095791787262038, 0.6558228497861817}, {0.6045279587851373, 
      0.6576035596847636}, {0.5994431777822015, 0.6593477614833949}, {
      0.5943248756626482, 0.661055091776298}, {0.5891730934782813, 
      0.6627251823814715}, {0.5839878734447433, 0.6643576602432327}, {
      0.5787692590008648, 0.665952147332569}, {0.573517294870158, 
      0.6675082605452426}, {0.5682320271245457, 0.6690256115975874}, {
      0.5629135032504176, 0.6705038069199376}, {0.5575617722171111, 
      0.6719424475476241}, {0.5521768845479186, 0.6733411290094757}, {
      0.5467588923937263, 0.6746994412137579}, {0.5413078496093972, 
      0.676016968331482}, {0.5358238118330115, 0.6772932886770148}, {
      0.5303068365680896, 0.6785279745859187}, {0.5247569832689197, 
      0.6797205922899476}, {0.5191743134291269, 0.6808707017891251}, {
      0.5135588906736206, 0.6819778567208284}, {0.5079107808540656, 
      0.6830416042257997}, {0.5022300521480297, 0.6840614848110038}, {
      0.49651677516196663`, 0.6850370322092522}, {0.4907710230382008, 
      0.6859677732355077}, {0.4849928715660896, 0.6868532276397854}, {
      0.479182399297545, 0.6876929079565615}, {0.473339687667107, 
      0.6884863193506017}, {0.4674648211167692, 0.6892329594591177}, {
      0.461557887225768, 0.6899323182301581}, {0.4556189768455553, 
      0.6905838777571403}, {0.4496481842401863, 0.691187112109426}, {
      0.4436456072323658, 0.6917414871588433}, {0.4376113473554066, 
      0.6922464604020546}, {0.4315455100113681, 0.6927014807786693}, {
      0.42544820463565486`, 0.6931059884849997}, {0.419319544868369, 
      0.6934594147833542}, {0.4131596487327258, 0.6937611818067643}, {
      0.4069686388208565, 0.6940107023590381}, {0.40074664248733816`, 
      0.6942073797100325}, {0.3944937920508088, 0.6943506073860378}, {
      0.38821022500404273`, 0.6944397689551635}, {0.381896084232881, 
      0.6944742378076183}, {0.3755515182444311, 0.6944533769307755}, {
      0.36917668140497206`, 0.6943765386789125}, {0.3627717341880224, 
      0.6942430645375195}, {0.3563368434330528, 0.6940522848820676}, {
      0.3498721826153497, 0.6938035187311318}, {0.3433779321275621, 
      0.6934960734937642}, {0.3368542795734919, 0.6931292447110171}, {
      0.33030142007471647`, 0.6927023157915156}, {0.3237195565906641, 
      0.6922145577409862}, {0.3171089002527943, 0.6916652288856502}, {
      0.31046967071356957`, 0.6910535745893973}, {0.3038020965109415, 
      0.6903788269646595}, {0.2971064154491123, 0.6896402045769133}, {
      0.2903828749963734, 0.6888369121427479}, {0.2836317327008633, 
      0.6879681402214424}, {0.27685325662513577`, 0.687033064900012}, {
      0.2700477258004724, 0.6860308474716901}, {0.26321543070192666`, 
      0.6849606341078291}, {0.25635667374513765`, 0.683821555523222}, {
      0.24947176980600916`, 0.6826127266348577}, {0.24256104676440604`, 
      0.6813332462141518}, {0.23562484607308437`, 0.6799821965327097}, {
      0.22866352335313583`, 0.6785586430017101}, {0.2216774490172961, 
      0.6770616338050218}, {0.21466700892254015`, 0.675490199526202}, {
      0.20763260505346376`, 0.6738433527695564}, {0.20057465623803164`, 
      0.6721200877754837}, {0.19349359889735762`, 0.670319380030372}, {
      0.18638988783127225`, 0.6684401858713598}, {0.17926399704152776`, 
      0.6664814420863353}, {0.1721164205945896, 0.6644420655096014}, {
      0.1649476735260683, 0.6623209526137072}, {0.15775829278895578`, 
      0.6601169790980174}, {0.15054883824794515`, 0.6578289994746764}, {
      0.14331989372223436`, 0.6554558466527128}, {0.1360720680793416, 
      0.6529963315211329}, {0.12880599638259288`, 0.6504492425319635}, {
      0.1215223410950816, 0.6478133452843269}, {0.11422179334304591`, 
      0.6450873821107682}, {0.10690507424176106`, 0.642270071667205}, {
      0.09957293628720262, 0.639360108528035}, {0.092226164816901, 
      0.6363561627881239}, {0.08486557954357858, 0.6332568796735898}, {
      0.07749203616533784, 0.6300608791635327}, {0.07010642805635198, 
      0.6267667556250935}, {0.06270968804219747, 0.6233730774645027}, {
      0.055302790264161766`, 0.6198783867970711}, {0.047886752137056794`, 
      0.616281199139405}, {0.04046263640527017, 0.6125800031274832}, {
      0.03303155330198954, 0.6087732602646287}, {0.0255946628167404, 
      0.6048594047038425}, {0.0181531770765817, 0.6008368430694381}, {
      0.010708362846505767`, 0.5967039543234408}, {0.0032615441547859834`, 
      0.5924590896827839}, {-0.004185894950794669, 
      0.5881005725939634}, {-0.011632507509722223`, 
      0.5836266987724974}, {-0.019076780612261618`, 
      0.5790357363152877}, {-0.026517132213277325`, 
      0.5743259258948038}, {-0.03395190778982447, 
      0.5694954810449049}, {-0.041379376835881714`, 
      0.5645425885490989}, {-0.04879772918754731, 
      0.5594654089431029}, {-0.05620507117201735, 
      0.5542620771447367}, {-0.06359942157371991, 
      0.5489307032254479}, {-0.07097870741110286, 
      0.5434693733391429}, {-0.07834075951777904, 
      0.5378761508254943}, {-0.08568330792203632, 
      0.5321490775065124}, {-0.09300397701913948, 
      0.5262861751969188}, {-0.10030028053140498`, 
      0.5202854474507496}, {-0.10756961625174319`, 
      0.5141448815686447}, {-0.11480926056725993`, 
      0.5078624508924656}, {-0.12201636276062092`, 
      0.5014361174162149}, {-0.129187939088242, 
      0.4948638347447222}, {-0.13632086663601223`, 
      0.48814355143420823`}, {-0.1434118769552284, 
      0.4812732147516319}, {-0.15045754948376508`, 
      0.4742507748926706}, {-0.15745430476027603`, 
      0.4670741897012588}, {-0.1643983974424782, 
      0.45974142993679995`}, {-0.17128590914437178`, 
      0.45225048513845123`}, {-0.17811274111166742`, 
      0.4445993701392202}, {-0.1848746067597987, 
      0.4367861322859661}, {-0.19156702410477397`, 
      0.42880885942471086`}, {-0.19818530812384963`, 
      0.4206656887138622}, {-0.20472456309067488`, 
      0.41235481633094356`}, {-0.2111796749382535, 
      0.4038745081411108}, {-0.2175453037128804, 
      0.3952231113979689}, {-0.22381587619322563`, 
      0.3863990675488366}, {-0.22998557876103118`, 
      0.3774009262174433}, {-0.23604835062352653`, 
      0.36822736043686327`}, {-0.24199787750270083`, 
      0.358877183204031}, {-0.24782758592301696`, 
      0.34934936542414585`}, {-0.253530638247, 
      0.33964305530831085`}, {-0.25909992862731973`, 
      0.32975759928048143`}, {-0.26452808006439166`, 
      0.3196925644397927}, {-0.2698074427799493, 
      0.30944776261111734`}, {-0.2749300941391995, 
      0.2990232759997791}, {-0.27988784037666664`, 
      0.2884194844451804}, {-0.2846722204031233, 
      0.2776370942421516}, {-0.2892745119924127, 
      0.2666771684675753}, {-0.2936857406666329, 
      0.25554115871277633`}, {-0.2978966916150249, 
      0.24423093807889282`}, {-0.30189792499473844`, 
      0.23274883524264156`}, {-0.30567979496897263`, 
      0.22109766934344605`}, {-0.30923247283814415`, 
      0.20928078537990608`}, {-0.3125459746108657, 
      0.19730208973447755`}, {-0.31561019334162477`, 
      0.1851660853707973}, {-0.3184149365290519, 
      0.17287790616960452`}, {-0.3209499688204786, 
      0.1604433497884929}, {-0.32320506020315, 
      0.14786890835020533`}, {-0.3251700397783045, 
      0.13516179618694896`}, {-0.3268348551101487, 
      0.12232997379802711`}, {-0.3281896370170121, 
      0.10938216711937067`}, {-0.3292247695270452, 
      0.09632788116127217}, {-0.3299309645572538, 
      0.08317740705014559}, {-0.3302993406953342, 
      0.06994182151696691}, {-0.3303215052731203, 
      0.056632977914532576`}, {-0.32998963872453235`, 
      0.04326348792255438}, {-0.3292965800274078, 
      0.029846693217575472`}, {-0.328235911846643, 
      0.016396626545853527`}, {-0.3268020438359521, 
      0.0029279618417401877`}, {-0.3249902924281886, -0.010544046720817121`}, \
{-0.3227969553604597, -0.024003636575578018`}, {-0.3202193791502466, \
-0.03743461903461294}, {-0.3172560177707322, -0.05082047118355017}, \
{-0.31390648087320344`, -0.06414443627770147}, {-0.31017157007394885`, \
-0.07738963120384054}, {-0.3060533020606594, -0.09053915929106515}, \
{-0.3015549175727171, -0.10357622651913488`}, {-0.2966808756603385, \
-0.11648425899856935`}, {-0.2914368330148538, -0.12924701949543874`}, \
{-0.2858296085689513, -0.1418487207530996}, {-0.27986713397215296`, \
-0.1542741334263465}, {-0.27355839093332857`, -0.16650868658868437`}, \
{-0.26691333676998036`, -0.17853855899374566`}, {-0.2599428197970701, \
-0.19035075955581665`}, {-0.25265848641369365`, -0.20193319584691247`}, \
{-0.245072681895766, -0.2132747297713723}, {-0.23719834697375972`, \
-0.224365219955038}, {-0.22904891226793453`, -0.23519555075661644`}, \
{-0.22063819257526127`, -0.24575764815733253`}, {-0.2119802828617811, \
-0.2560444830976319}, {-0.20308945762341973`, -0.26605006309600826`}, \
{-0.19398007505071854`, -0.275769413198201}, {-0.18466648718235953`, \
-0.285198547461902}, {-0.17516295697199022`, -0.2943344322829676}, \
{-0.16548358293461002`, -0.30317494291706903`}, {-0.15564223179270548`, \
-0.3117188145510704}, {-0.14565247931628977`, -0.319965589238066}, \
{-0.13552755935062283`, -0.32791555993666144`}, {-0.12528032085409424`, \
-0.3355697127967048}, {-0.1149231926280181, -0.34292966871797353`}, \
{-0.10446815530974102`, -0.3499976250823683}, {-0.0939267201190228, \
-0.3567762984301287}, {-0.08330991379268099, -0.3632688687216218}, \
{-0.07262826911094546, -0.36947892570242374`}, {-0.06189182040746685, \
-0.37541041777375306`}, {-0.051110103459992215`, -0.38106760366488934`}, \
{-0.04029215917699009, -0.38645500711026126`}, {-0.029446540523832485`, \
-0.39157737465195097`}, {-0.01858132216769882, -0.39643963661841836`}, \
{-0.007704112360699036, -0.4010468712718552}, {
      0.003177933376255623, -0.40540427206899365`}, {
      0.014058097162273983`, -0.40951711794246937`}, {
      0.024930082607753888`, -0.4133907464809286}, {
      0.0357879979155458, -0.4170305298648738}, {
      0.046626338499276734`, -0.4204418534006802}, {
      0.05743996934855998, -0.4236300964862511}, {
      0.06822410733623498, -0.4266006158374527}, {
      0.0789743036313198, -0.42935873080390063`}, {
      0.08968642635305271, -0.431909710605097}, {
      0.10035664357619156`, -0.43425876332264507`}, {
      0.11098140677550496`, -0.43641102649073277`}, {
      0.1215574347779543, -0.4383715591347711}, {
      0.13208169827422897`, -0.4401453351165992}, {
      0.14255140492683782`, -0.4417372376536824}, {
      0.15296398509965542`, -0.4431520548889667}, {
      0.1633170782234472, -0.44439447639729845`}, {
      0.17360851980323858`, -0.4454690905234047}, {
      0.18383632906624636`, -0.4463803824552364}, {
      0.19399869724326557`, -0.44713273294491157`}, {
      0.20409397647172708`, -0.4477304175974964}, {
      0.21412066930495427`, -0.44817760665539025`}, {
      0.224077418809305, -0.44847836521311446`}, {
      0.2339629992287649, -0.44863665380383594`}, {
      0.2437763071950435, -0.4486563293049912}, {
      0.253516353460221, -0.44854114611592344`}, {
      0.26318225512841015`, -0.44829475756552534`}, {
      0.2727732283626637, -0.44792071751251433`}, {
      0.28228858154340625`, -0.4474224821051797}, {
      0.29172770885494503`, -0.44680341167125937`}, {
      0.3010900842770718, -0.44606677271205625`}, {
      0.31037525595936377`, -0.44521573997801484`}, {
      0.319582840956496, -0.44425339860577695`}, {
      0.32871252030365794`, -0.4431827462992471}, {
      0.33776403441200525`, -0.44200669553944516`}, {
      0.346737178764946, -0.4407280758099334}, {
      0.3556317998969533, -0.43934963582639647`}, {
      0.3644477916374908, -0.4378740457605469}, {
      0.3731850916035285, -0.43630389944994535`}, {
      0.38184367792500623`, -0.4346417165865811}, {
      0.39042356618845986`, -0.4328899448781692}, {
      0.39892480658485996`, -0.43105096217710304`}, {
      0.40734748124851944`, -0.429127078572866}, {
      0.41569170177470194`, -0.4271205384444673}, {
      0.42395760690430573`, -0.4250335224701351}, {
      0.43214536036470697`, -0.42286814959208596`}, {
      0.44025514885652156`, -0.4206264789346988}, {
      0.44828718017668545`, -0.41831051167486877`}, {
      0.4562416814688604, -0.4159221928637014}, {
      0.46411889759274616`, -0.41346341319904273`}, {
      0.4719190896044211, -0.41093601074862923`}, {
      0.47964253334034357`, -0.40834177262388927`}, {
      0.487289518098127, -0.40568243660463876`}, {
      0.49486034540765106`, -0.40295969271509463`}, {
      0.5023553278864966, -0.40017518475177954`}, {
      0.5097747881740872, -0.3973305117640205}, {
      0.5171190579392931, -0.39442722948784725`}, {
      0.524388476956603, -0.39146685173418405`}, {
      0.5315833922462899, -0.38845085173229693`}, {
      0.5387041572743108, -0.38538066342951466`}, {
      0.545751131207954, -0.38225768274828287`}, {
      0.5527246782235246, -0.37908326880164334`}, {
      0.5596251668625977, -0.37585874506825073`}, {
      0.5664529694336091, -0.3725854005280546}, {
      0.5732084614557629, -0.3692644907597787}, {
      0.5798920211424423, -0.3658972390013316}, {
      0.5865040289214959, -0.36248483717427754`}, {
      0.5930448669899478, -0.35902844687348584`}, {
      0.5995149189008451, -0.3555292003230663}, {
      0.6059145691801064, -0.3519882012996798}, {
      0.6122442029713819, -0.3484065260242965}, {
      0.6185042057070625, -0.3447852240234519}, {
      0.6246949628037057, -0.34112531896102877`}, {
      0.6308168593802566, -0.33742780944156986`}, {
      0.6368702799975516, -0.3336936697861006}, {
      0.6428556084176953, -0.32992385078141556`}, {
      0.6487732273819882, -0.3261192804037563}, {
      0.6546235184061786, -0.3222808645177823}, {
      0.6604068615918859, -0.31840948755170834`}, {
      0.6661236354531229, -0.3145060131494572}, {
      0.6717742167569141, -0.3105712848006483}, {
      0.6773589803770718, -0.306606126449217}, {
      0.6828782991602561, -0.30261134308143395`}, {
      0.6883325438034994, -0.2985877212940668}, {
      0.6937220827424311, -0.29453602984340294`}, {
      0.699047282049489, -0.2904570201758262}, {
      0.7043085053414477, -0.2863514269406163}, {
      0.7095061136956414, -0.2822199684856174}, {
      0.7146404655742958, -0.278063347336397}, {
      0.7197119167564248, -0.2738822506594963}, {
      0.7247208202767798, -0.26967735071035037`}, {
      0.7296675263713764, -0.26544930526643457`}, {
      0.7345523824291508, -0.261198758046175}, {
      0.7393757329493273, -0.2569263391141395}, {
      0.7441379195041097, -0.2526326652730069}, {
      0.7488392807063268, -0.24831834044279447`}, {
      0.7534801521816941, -0.24398395602780404`}, {
      0.7580608665453691, -0.23963009127173132`}, {
      0.7625817533825023, -0.23525731360136587`}, {
      0.7670431392325033, -0.23086617895929262`}, {
      0.77144534757676, -0.2264572321259909}, {
      0.7757886988295659, -0.22203100703171136`}, {
      0.7800735103320259, -0.2175880270584977}, {
      0.7843000963487251, -0.21312880533270484`}, {
      0.7884687680669609, -0.20865384500835335`}, {
      0.7925798335983487, -0.20416363954164574`}, {
      0.7966335979826268, -0.19965867295695877`}, {
      0.800630363193494, -0.19513942010461363`}, {
      0.8045704281463271, -0.19060634691071443`}, {
      0.8084540887076328, -0.18605991061933438`}, {
      0.8122816377061002, -0.18150056002731876`}, {
      0.8160533649451259, -0.17692873571196324`}, {
      0.8197695572166955, -0.17234487025181658`}, {
      0.8234304983165086, -0.16774938844084714`}, {
      0.8270364690602455, -0.163142707496204}, {
      0.8305877473008785, -0.15852523725979423`}, {
      0.834084607946936, -0.15389738039389023`}, {
      0.8375273229816367, -0.14925953257097224`}, {
      0.8409161614828133, -0.14461208265800446`}, {
      0.8442513896435517, -0.13995541289533459`}, {
      0.8475332707934773, -0.13528989907040076`}, {
      0.8507620654206235, -0.1306159106864222}, {
      0.8539380311938216, -0.12593381112624363`}, {
      0.857061422985556, -0.1212439578114973}, {
      0.8601324928952319, -0.11654670235724024`}, {
      0.8631514902728066, -0.11184239072221884`}, {
      0.8661186617427384, -0.10713136335490701`}, {
      0.8690342512282102, -0.10241395533545895`}, {
      0.8718984999755877, -0.09769049651371248}, {
      0.8747116465790757, -0.09296131164337379}, {
      0.8774739270055377, -0.08822672051250999}, {
      0.8801855746194452, -0.08348703807047085}, {
      0.8828468202079295, -0.07874257455135728}, {
      0.8854578920059045, -0.07399363559414944}, {
      0.8880190157212386, -0.06924052235960367}, {
      0.8905304145599479, -0.06448353164402319}, {
      0.8929923092513903, -0.05972295599000425}, {
      0.8954049180734404, -0.05495908379425532}, {
      0.897768456877624, -0.050192199412583835`}, {
      0.900083139114196, -0.04542258326214161}, {
      0.9023491758571451, -0.040650511921016694`}, {
      0.9045667758291095, -0.03587625822525656}, {
      0.9067361454261897, -0.031100091363404518`}, {
      0.9088574887426474, -0.02632227696862838}, {
      0.9109310075954751, -0.021543077208517697`}, {
      0.9129569015488292, -0.016762750872623303`}, {
      0.9149353679383149, -0.01198155345781034}, {
      0.9168666018951129, -0.007199737251493528}, {
      0.9187507963699427, -0.0024175514128211223`}, {0.9205881421568516, 
      0.0023647579481282438`}, {0.9223788279168244, 0.007146947693073805}, {
      0.9241230402012081, 0.011928777580126036`}, {0.9258209634749446, 
      0.016710010190187172`}, {0.9274727801396074, 0.02149041085510287}, {
      0.929078670556238, 0.026269747587510772`}, {0.9306388130679776, 
      0.031047791012333632`}, {0.9321533840224907, 0.03582431429986623}, {
      0.9336225577941788, 0.04059909310040713}, {0.9350465068061796, 
      0.0453719054803877}, {0.9364254015521517, 0.05014253185995264}, {
      0.9377594106178411, 0.054910754951947416`}, {0.9390487007024297, 
      0.059676359702269664`}, {0.940293436639664, 0.06443913323154286}, {
      0.9414937814187619, 0.06919886477807198}, {0.9426498962051001, 
      0.07395534564204195}, {0.9437619403606782, 0.07870836913092123}, {
      0.9448300714643627, 0.08345773050603361}, {0.9458544453319087, 
      0.08820322693026293}, {0.9468352160357605, 0.09294465741685612}, {
      0.9477725359246327, 0.0976818227792912}, {0.9486665556428707, 
      0.10241452558217794`}, {0.9495174241495926, 0.10714257009315965`}, {
      0.950325288737614, 0.1118657622357858}, {0.9510902950521558, 
      0.1165839095433258}, {0.9518125871093376, 0.12129682111349536`}, {
      0.9524923073144566, 0.12600430756406758`}, {0.9531295964800557, 
      0.13070618098934175`}, {0.9537245938437808, 0.13540225491744365`}, {
      0.95427743708603, 0.1400923442684319}, {0.9547882623473957, 
      0.14477626531318566`}, {0.9552572042459034, 0.1494538356330496}, {
      0.9556843958940476, 0.15412487408021264`}, {0.956069968915627, 
      0.1587892007387983}, {0.9564140534623827, 0.1634466368866438}, {
      0.9567167782304402, 0.16809700495774718`}, {0.956978270476558, 
      0.17274012850536125`}, {0.9571986560341859, 0.17737583216571393`}, {
      0.957378059329334, 0.1820039416223355}, {0.9575166033962563, 
      0.18662428357097346`}, {0.957614409892951, 0.19123668568507624`}, {
      0.9576715991164789, 0.1958409765818273}, {0.9576882900181052, 
      0.20043698578871244`}, {0.9576646002182639, 0.20502454371060233`}, {
      0.9576006460213509, 0.20960348159733397`}, {0.9574965424303451, 
      0.21417363151177418`}, {0.9573524031612636, 0.2187348262983494}, {
      0.9571683406574508, 0.22328689955202588`}, {0.9569444661037063, 
      0.2278296855877254}, {0.9566808894402528, 0.23236301941016105`}, {
      0.9563777193765479, 0.23688673668407895`}, {0.9560350634049425, 
      0.24140067370489157`}, {0.9556530278141873, 0.2459046673696887}, {
      0.955231717702792, 0.2503985551486124}, {0.9547712369922395, 
      0.25488217505658317`}, {0.954271688440057, 0.25935536562536327`}, {
      0.9537331736527481, 0.26381796587594586`}, {0.9531557930985881, 
      0.2682698152912563}, {0.9525396461202854, 0.27271075378915427`}, {
      0.9518848309475125, 0.27714062169572407`}, {0.9511914447093088, 
      0.28155925971884205`}, {0.9504595834463581, 0.28596650892200925`}, {
      0.949689342123144, 0.2903622106984383}, {0.9488808146399873, 
      0.29474620674538293`}, {0.9480340938449648, 0.2991183390387002}, {
      0.9471492715457173, 0.30347844980763394`}, {0.946226438521145, 
      0.30782638150980923`}, {0.9452656845329975, 0.312161976806428}, {
      0.944267098337358, 0.31648507853765473`}, {0.9432307676960262, 
      0.3207955296981833}, {0.9421567793878036, 0.3250931734129744}, {
      0.9410452192196824, 0.329377852913154}, {0.9398961720379426, 
      0.3336494115120641}, {0.9387097217391593, 0.3379076925814548}, {
      0.9374859512811241, 0.3421525395278101}, {0.9362249426936826, 
      0.3463837957687971}, {0.9349267770894925, 0.3506013047098299}, {
      0.9335915346747038, 0.35480490972073997`}, {0.932219294759566, 
      0.35899445411254227`}, {0.9308101357689634, 0.36316978111429077`}, {
      0.9293641352528834, 0.36733073385001286`}, {0.9278813698968201, 
      0.3714771553157149}, {0.9263619155321168, 0.3756088883564502}, {
      0.92480584714625, 0.3797257756434408}, {0.9232132388930596, 
      0.3838276596512454}, {0.921584164102926, 0.38791438263496386`}, {
      0.9199186952929005, 0.3919857866074713}, {0.9182169041767905, 
      0.3960417133166727}, {0.9164788616752025, 0.4000820042227703}, {
      0.9147046379255485, 0.4041065004755352}, {0.9128943022920162, 
      0.4081150428915755}, {0.9110479233755095, 0.4121074719315928}, {
      0.9091655690235614, 0.41608362767761853`}, {0.9072473063402215, 
      0.4200433498102227}, {0.9052932016959258, 0.4239864775856865}, {
      0.9033033207373471, 0.4279128498131313}, {0.9012777283972353, 
      0.4318223048315952}, {0.8992164889042465, 0.43571468048704964`}, {
      0.8971196657927683, 0.43958981410934805`}, {0.894987321912743, 
      0.4434475424890982}, {0.8928195194394944, 0.4472877018544501}, {
      0.8906163198835607, 0.45111012784779186`}, {0.8883777841005382, 
      0.4549146555023445}, {0.886103972300941, 0.45870111921864837`}, {
      0.883794944060079, 0.46246935274093237`}, {0.8814507583279605, 
      0.466219189133358}, {0.8790714734392223, 0.46995046075612995`}, {
      0.8766571471230931, 0.47366299924146427`}, {0.8742078365133932, 
      0.4773566354694067}, {0.8717235981585776, 0.48103119954349166`}, {
      0.8692044880318245, 0.4846865207662339}, {0.8666505615411764, 
      0.4883224276144441}, {0.8640618735397376, 0.49193874771435914`}, {
      0.8614384783359339, 0.49553530781657923`}, {0.8587804297038387, 
      0.4991119337708019}, {0.8560877808935723, 0.5026684505003445}, {
      0.853360584641779, 0.5062046819764461}, {0.8505988931821875, 
      0.509720451192339}, {0.8478027582562612, 0.513215580137081}, {
      0.8449722311239434, 0.5166898897691391}, {0.8421073625745042, 
      0.5201431999897143}, {0.8392082029374951, 0.5235753296157982}, {
      0.8362748020938165, 0.5269860963529525}, {0.8333072094869081, 
      0.5303753167677988}, {0.8303054741340636, 0.5337428062602119}, {
      0.8272696446378828, 0.537088379035203}, {0.8241997691978622, 
      0.5404118480744852}, {0.8210958956221363, 0.5437130251077078}, {
      0.8179580713393738, 0.5469917205833519}, {0.814786343410839, 
      0.5502477436392723}, {0.8115807585426237, 0.5534809020728786}, {
      0.8083413630980602, 0.5566910023109404}, {0.8050682031103223, 
      0.5598778493790075}, {0.8017613242952244, 0.5630412468704327}, {
      0.7984207720642253, 0.5661809969149835}, {0.7950465915376493, 
      0.5692969001470334}, {0.7916388275581305, 0.5723887556733173}, {
      0.7881975247042934, 0.57545636104024}, {0.7847227273046772, 
      0.5784995122007246}, {0.7812144794519167, 0.5815180034805862}, {
      0.7776728250171884, 0.5845116275444195}, {0.7740978076649342, 
      0.5874801753609844}, {0.7704894708678744, 0.5904234361680761}, {
      0.766847857922321, 0.5933411974368669}, {0.7631730119638036, 
      0.5962332448357011}, {0.7594649759830221, 0.5990993621933328}, {
      0.7557237928421371, 0.601939331461587}, {0.7519495052914132, 
      0.6047529326774302}, {0.7481421559862291, 0.6075399439244351}, {
      0.7443017875044673, 0.6103001412936205}, {0.7404284423643009, 
      0.6130332988436522}, {0.7365221630423916, 0.6157391885603859}, {
      0.7325829919925146, 0.6184175803157353}, {0.7286109716646285, 
      0.6210682418258471}, {0.7246061445244059, 0.623690938608564}, {
      0.7205685530732431, 0.6262854339401582}, {0.7164982398687686, 
      0.6288514888113148}, {0.7123952475458671, 0.6313888618823446}, {
      0.7082596188382411, 0.6338973094376087}, {0.7040913966005289, 
      0.6363765853391314}, {0.6998906238310024, 0.6388264409793818}, {
      0.6956573436948649, 0.641246625233201}, {0.6913915995481721, 
      0.643636884408854}, {0.6870934349624015, 0.6459969621981826}, {
      0.6827628937496936, 0.6483265996258356}, {0.6784000199887902, 
      0.650625534997553}, {0.6740048580516982, 0.6528935038474795}, {
      0.6695774526311044, 0.6551302388844806}, {0.6651178487685709, 
      0.6573354699374385}, {0.6606260918835422, 0.6595089238994978}, {
      0.6561022278031913, 0.6616503246712363}, {0.6515463027931412, 
      0.6637593931027319}, {0.6469583635890921, 0.6658358469344974}, {
      0.6423384574293903, 0.6678794007372534}, {0.6376866320885746, 
      0.6698897658505104}, {0.6330029359119373, 0.671866650319928}, {
      0.62828741785114, 0.6738097588334198}, {0.6235401275009221, 
      0.6757187926559727}, {0.6187611151369481, 0.677593449563146}, {
      0.613950431754833, 0.6794334237732176}, {0.6091081291103954, 
      0.68123840587794}, {0.6042342597611833, 0.6830080827718739}, {
      0.5993288771093228, 0.6847421375802585}, {0.5943920354457417, 
      0.6864402495853849}, {0.5894237899958199, 0.6881020941514301}, {
      0.5844241969665249, 0.6897273426477152}, {0.5793933135950879, 
      0.6913156623703447}, {0.5743311981992828, 0.6928667164621871}, {
      0.5692379102293706, 0.6943801638311529}, {0.5641135103217749, 
      0.6958556590667263}, {0.5589580603545569, 0.6972928523547068}, {
      0.5537716235047615, 0.6986913893901131}, {0.5485542643077096, 
      0.7000509112882031}, {0.5433060487183127, 0.7013710544935619}, {
      0.5380270441744925, 0.7026514506872075}, {0.5327173196627893, 
      0.7038917266916638}, {0.5273769457862469, 0.7050915043739493}, {
      0.5220059948346674, 0.7062504005464281}, {0.5166045408573301, 
      0.7073680268654681}, {0.5111726597382775, 0.7084439897278519}, {
      0.5057104292742701, 0.7094778901648822}, {0.5002179292555214, 
      0.7104693237341236}, {0.49469524154932715`, 0.7114178804087223}, {
      0.4891424501867076, 0.71232314446424}, {0.4835596414521881, 
      0.7131846943629429}, {0.4779469039768481, 0.7140021026354786}, {
      0.47230432883477486`, 0.7147749357598795}, {0.46663200964306495`, 
      0.7155027540378224}, {0.4609300426655218, 0.7161851114680793}, {
      0.45519852692020535`, 0.7168215556170893}, {0.44943756429099757`, 
      0.7174116274865803}, {0.4436472596433534, 0.7179548613781696}, {
      0.4378277209444162, 0.7184507847548701}, {0.4319790593876847, 
      0.7188989180994275}, {0.4261013895224268, 0.7192987747694124}, {
      0.42019482938804487`, 0.7196498608489912}, {0.4142595006536074, 
      0.7199516749972975}, {0.40829552876277175`, 0.720203708293325}, {
      0.40230304308433273`, 0.7204054440772613}, {0.3962821770686442, 
      0.7205563577881822}, {0.39023306841017197`, 0.7206559167980249}, {
      0.3841558592164486, 0.7207035802417566}, {0.3780506961837149, 
      0.7206987988436565}, {0.3719177307795445, 0.7206410147396269}, {
      0.36575711943276545`, 0.72052966129545}, {0.35956902373100513`, 
      0.7203641629209062}, {0.3533536106262027, 0.7201439348796704}, {
      0.34711105264844944`, 0.7198683830949035}, {0.34084152812853585`, 
      0.7195369039504544}, {0.3345452214296021, 0.7191488840875933}, {
      0.328222323188309, 0.7187037001971931}, {0.3218730305659682, 
      0.7182007188072828}, {0.3154975475100898, 0.7176392960658927}, {
      0.3090960850268317, 0.7170187775191198}, {0.30266886146485794`, 
      0.7163384978843402}, {0.2962161028111379, 0.7155977808185013}, {
      0.2897380429992485, 0.7147959386814301}, {0.2832349242307658, 
      0.713932272294098}, {0.2767069973103672, 0.7130060706917892}, {
      0.27015452199529294`, 0.7120166108721272}, {0.26357776735985194`, 
      0.7109631575379164}, {0.25697701217569113`, 0.7098449628347718}, {
      0.25035254530858436`, 0.7086612660835127}, {0.2437046661325373, 
      0.7074112935073094}, {0.23703368496204488`, 0.7060942579535865}, {
      0.23033992350338192`, 0.7047093586106948}, {0.2236237153258538, 
      0.7032557807193844}, {0.21688540635398185`, 0.7017326952791244}, {
      0.21012535538164978`, 0.7001392587493377}, {0.20334393460929068`, 
      0.6984746127456367}, {0.19654153020525114`, 0.6967378837311731}, {
      0.1897185428925289, 0.6949281827032383}, {0.18287538856214267`, 
      0.6930446048752839}, {0.17601249891445983`, 0.6910862293545594}, {
      0.16913032212987675`, 0.6890521188156067}, {0.16222932357032005`, 
      0.6869413191698843}, {0.15530998651311476`, 0.684752859231845}, {
      0.14837281291884558`, 0.6824857503818337}, {0.14141832423492376`, 
      0.6801389862262317}, {0.1344470622366612, 0.67771154225533}, {
      0.12745958990774742`, 0.6752023754994817}, {0.12045649236212445`, 
      0.6726104241841557}, {0.11343837780935757`, 0.669934607384597}, {
      0.10640587856570868`, 0.6671738246808825}, {0.09935965211323282, 
      0.6643269558142647}, {0.09230038220933676, 0.6613928603457984}, {
      0.085228780049363, 0.6583703773183668}, {0.07814558548489169, 
      0.6552583249233531}, {0.07105156830058786, 0.6520555001733461}, {
      0.0639475295525617, 0.6487606785824266}, {0.05683430297135468, 
      0.6453726138557548}, {0.04971275643281551, 0.6418900375903697}, {
      0.04258379350028524, 0.6383116589893176}, {0.03544835504167157, 
      0.6346361645914624}, {0.028307420925157048`, 0.6308622180195751}, {
      0.02116201179745462, 0.626988459749583}, {0.014013190948696122`, 
      0.6230135069041588}, {0.006862066268213445, 
      0.618935953074164}, {-0.00029020770435169655`, 
      0.6147543681718254}, {-0.007442427627997515, 
      0.610467298319923}, {-0.014593339104584421`, 
      0.6060732657817068}, {-0.02174163431337094, 
      0.6015707689367383}, {-0.02888594953073742, 
      0.5969582823083754}, {-0.036024862529812504`, 
      0.5922342566491976}, {-0.043156889854568586`, 
      0.58739711909129}, {-0.050280483962818104`, 
      0.5824452733689939}, {-0.05739403023242615, 
      0.5773771001224763}, {-0.06449584382496201, 
      0.5721909572912885}, {-0.07158416640094778, 
      0.5668851806079699}, {-0.07865716268083355, 
      0.5614580842027235}, {-0.08571291684584269, 
      0.5559079613312408}, {-0.09274942877289696, 
      0.5502330852388948}, {-0.09976461009795906, 
      0.5444317101757655}, {-0.10675628010233244`, 
      0.5385020725783008}, {-0.11372216141674735`, 
      0.5324423924348735}, {-0.12065987553845378`, 
      0.526250874854064}, {-0.1275669381570544, 
      0.5199257118561905}, {-0.13444075428546298`, 
      0.5134650844104284}, {-0.14127861319318963`, 
      0.506867164741811}, {-0.14807768314015804`, 
      0.5001301189344953}, {-0.15483500591048344`, 
      0.49325210985989587`}, {-0.16154749114711195`, 
      0.48623130046065843`}, {-0.16821191048998307`, 
      0.47906585742394275`}, {-0.1748248915224654, 
      0.47175395528011277`}, {-0.18138291153327776`, 
      0.4642937809656862}, {-0.1878822911039945, 
      0.45668353889225277`}, {-0.19431918753559715`, 
      0.44892145656601684`}, {-0.2006895881314412, 
      0.441005790805626}, {-0.20698930335851265`, 
      0.43293483460897675`}, {-0.21321395991403408`, 
      0.4247069247226945}, {-0.21935899373041043`, 
      0.41632044997091083`}, {-0.22541964295826383`, 
      0.40777386040272806`}, {-0.23139094097497206`, 
      0.3990656773202845}, {-0.23726770947477913`, 
      0.39019450425149727`}, {-0.2430445517062685, 
      0.3811590389332331}, {-0.24871584593385457`, 
      0.37195808637167904`}, {-0.2542757392120231, 
      0.36259057304686915`}, {-0.2597181415743916, 
      0.35305556232744495`}, {-0.26503672075429674`, 
      0.34335227115953443`}, {-0.2702248975695606, 
      0.3334800880898308}, {-0.2752758421213041, 
      0.3234385926772058}, {-0.2801824709750967, 
      0.3132275763391285}, {-0.2849374455122106, 
      0.30284706466836914`}, {-0.28953317165908293`, 
      0.29229734124149725`}, {-0.29396180122396814`, 
      0.2815789729230579}, {-0.2982152350907844, 
      0.2706928366475304}, {-0.30228512854077005`, 
      0.2596401476347392}, {-0.30616289899208926`, 
      0.2484224889628055}, {-0.3098397364651025, 
      0.23704184238556364`}, {-0.31330661709560864`, 
      0.2255006202382422}, {-0.31655432002874495`, 
      0.2138016982258937}, {-0.31957344803096693`, 
      0.20194844883346982`}, {-0.3223544521550056, 
      0.18994477503475476`}, {-0.3248876607811246, 
      0.1777951439100582}, {-0.3271633133354526, 
      0.16550461971048477`}, {-0.3291715989506601, 
      0.15307889583105136`}, {-0.33090270028381397`, 
      0.14052432507775847`}, {-0.3323468426390529, 
      0.12784794753737122`}, {-0.33349434845723824`, 
      0.11505751528620183`}, {-0.33433569712988975`, 
      0.10216151310932271`}, {-0.3348615899701213, 
      0.08916917434870436}, {-0.3350630200294668, 
      0.07609049096261004}, {-0.334931346288076, 
      0.06293621686439095}, {-0.3344583715697387, 
      0.04971786362192586}, {-0.3336364233470455, 
      0.036447687644447}, {-0.33245843641176615`, 
      0.02313866806588357}, {-0.33091803619884136`, 
      0.009804474656558966}, {-0.3290096213782667, -0.0035405747400539637`}, \
{-0.32672844417768576`, -0.016881567542000597`}, {-0.32407068678039486`, \
-0.030203060585367467`}, {-0.32103353206929225`, -0.043489154873332575`}, \
{-0.3176152269668087, -0.05672358038340153}, {-0.3138151366620087, \
-0.06988979007950254}, {-0.3096337881241793, -0.08297106195637288}, \
{-0.30507290147922855`, -0.09595060763964386}, {-0.30013540806895844`, \
-0.10881168579249931`}, {-0.29482545431728613`, -0.12153771835467805`}, \
{-0.2891483908810047, -0.13411240747624892`}, {-0.28311074695111743`, \
-0.14651985091850458`}, {-0.276720189976607, -0.15874465368507945`}, \
{-0.2699854714863747, -0.17077203372087682`}, {-0.26291636006730723`, \
-0.18258791967237403`}, {-0.25552356289839356`, -0.19417903893311209`}, \
{-0.24781863752644312`, -0.20553299449090767`}, {-0.23981389578584378`, \
-0.21663832943320793`}, {-0.23152230190504927`, -0.22748457833634342`}, \
{-0.22295736690309076`, -0.23806230514456816`}, {-0.21413304136219438`, \
-0.24836312751749032`}, {-0.20506360857374328`, -0.25837972797328945`}, \
{-0.19576357990409304`, -0.268105852466209}, {-0.18624759402642083`, \
-0.2775362972998387}, {-0.17653032142856814`, -0.2866668854860446}, \
{-0.16662637534877167`, -0.29549443381017076`}, {-0.15655023002468113`, \
-0.3040167119568608}, {-0.14631614687815764`, -0.31223239509095313`}, \
{-0.13593810900911155`, -0.3201410112800516}, {-0.12542976414391627`, \
-0.3277428850967404}, {-0.11480437598324654`, -0.3350390786569944}, \
{-0.10407478372382893`, -0.3420313312453485}, {-0.09325336938986183, \
-0.3487219985547151}, {-0.08235203250239216, -0.3551139924366046}, \
{-0.07138217153704018, -0.3612107219221646}, {-0.060354671569511506`, \
-0.36701603614110223`}, {-0.04927989748108549, -0.37253416963826247`}, \
{-0.03816769208917268, -0.37776969046939085`}, {-0.027027378577474146`, \
-0.3827274513504284}, {-0.015867766622761466`, -0.3874125440397164}, \
{-0.004697161647592164, -0.3918302570501794}, {
      0.006476623332509217, -0.39598603671878463`}, {
      0.017646253755240687`, -0.39988545160280503`}, {
      0.02880485789296631, -0.4035341601257969}, {
      0.03994600996472981, -0.4069378813597204}, {
      0.05106371232831338, -0.41010236880214}, {
      0.06215237705771346, -0.4130333869877875}, {
      0.07320680716976527, -0.4157366907608066}, {
      0.08422217772495647, -0.4182180070266422}, {
      0.0951940169920125, -0.4204830187997917}, {
      0.10611818783373955`, -0.4225373513645945}, {
      0.11699086944287879`, -0.4243865603701061}, {
      0.12780853953126964`, -0.4260361216861889}, {
      0.1385679570532847, -0.427491422855665}, {
      0.1492661455250856, -0.4287577559862162}, {
      0.15990037698453102`, -0.42984031193527716`}, {
      0.17046815662230505`, -0.43074417565111284`}, {
      0.180967208102782, -0.4314743225433332}, {
      0.19139545958306575`, -0.43203561576607247`}, {
      0.2017510304303074, -0.4324328043067861}, {
      0.21203221863060007`, -0.43267052178297677`}, {
      0.22223748887727662`, -0.43275328585807327`}, {
      0.23236546132211083`, -0.4326854981960933}, {
      0.24241490096958312`, -0.43247144488259365`}, {
      0.25238470769187105`, -0.43211529724673253`}, {
      0.2622739068404299, -0.4316211130260398}, {
      0.2720816404288295, -0.43099283782171904`}, {
      0.2818071588608033, -0.43023430679801195`}, {
      0.29144981317716445`, -0.42934924658435725`}, {
      0.3010090477952687, -0.4283412773438084}, {
      0.3104843937149977, -0.4272139149754574}, {
      0.31987546216573626`, -0.4259705734224817}, {
      0.32918193866948425`, -0.42461456706091516`}, {
      0.3384035774960312, -0.4231491131473761}, {
      0.3475401964870017, -0.4215773343067864}, {
      0.356591672226522, -0.41990226104362643`}, {
      0.3655579355372423, -0.41812683426250347`}, {
      0.37443896728145115`, -0.41625390778580396`}, {
      0.38323479444803266`, -0.41428625085796617`}, {
      0.3919454865070208, -0.4122265506274764}, {
      0.4005711520144968, -0.410077414599078}, {
      0.40911193545154145`, -0.4078413730499057}, {
      0.41756801428189755`, -0.40552088140433296`}, {
      0.425939596213899, -0.40311832256326846`}, {
      0.4342269166530996, -0.40063600918446635`}, {
      0.442430236332865, -0.39807618591113997`}, {
      0.4505498391109865, -0.39544103154680266`}, {
      0.4585860299211326, -0.39273266117480826`}, {
      0.4665391328686682, -0.38995312822154155`}, {
      0.4744094894610507, -0.38710442646262055`}, {
      0.48219745696365135`, -0.3841884919718275}, {
      0.48990340687245165`, -0.3812072050127881}, {
      0.49752772349563396`, -0.3781623918736792}, {
      0.5050708026366152, -0.3750558266454619}, {
      0.5125330503715747, -0.37188923294432363`}, {
      0.5199148819149935, -0.3686642855791661}, {
      0.527216720567163, -0.36538261216510237`}, {
      0.5344389967380289, -0.36204579468403103`}, {
      0.5415821470421215, -0.358655370993438}, {
      0.5486466134596809, -0.3552128362846426}, {
      0.5556328425594201, -0.351719644491753}, {
      0.5625412847786815, -0.34817720965263466`}, {
      0.5693723937570321, -0.3445869072232176}, {
      0.5761266257196169, -0.3409500753464863}, {
      0.582804438906839, -0.33726801607749873`}, {
      0.5894062930471732, -0.3335419965657829}, {
      0.5959326488701402, -0.32977325019645015`}, {
      0.602383967656669, -0.32596297769135296`}, {
      0.6087607108242703, -0.3221123481715964}, {
      0.6150633395446168, -0.31822250018269427`}, {
      0.6212923143912942, -0.3142945426836343}, {
      0.6274480950156385, -0.31032955600109335`}, {
      0.6335311398487188, -0.3063285927500129}, {
      0.6395419058276576, -0.30229267872171806`}, {
      0.6454808481446045, -0.29822281374073056`}, {
      0.6513484200167943, -0.2941199724913954}, {
      0.6571450724762281, -0.28998510531540866`}, {
      0.6628712541776146, -0.28581913898130196`}, {
      0.6685274112233036, -0.2816229774269052}, {
      0.6741139870040282, -0.2773975024757779}, {
      0.6796314220543541, -0.2731435745285674}, {
      0.6850801539218079, -0.268862033230219}, {
      0.6904606170487284, -0.26455369811393237`}, {
      0.6957732426659468, -0.2602193692227282}, {
      0.7010184586974636, -0.25585982770945787`}, {
      0.7061966896753485, -0.251475836416059}, {
      0.7113083566641373, -0.2470681404328322}, {
      0.7163538771940521, -0.24263746763848462`}, {
      0.7213336652024174, -0.23818452922165897`}, {
      0.7262481309826827, -0.2337100201846405}, {
      0.7310976811405069, -0.22921461982990757`}, {
      0.7358827185563924, -0.2246989922301675}, {
      0.7406036423543946, -0.22016378668249378`}, {
      0.7452608478764609, -0.2156096381471585}, {
      0.7498547266619868, -0.21103716767173047`}, {
      0.7543856664322023, -0.20644698280098767`}, {
      0.7588540510790264, -0.20183967797317165`}, {
      0.7632602606580566, -0.19721583490309114`}, {
      0.7676046713853776, -0.19257602295256232`}, {
      0.7718876556378962, -0.18792079948865448`}, {
      0.7761095819569317, -0.18325071023019143`}, {
      0.7802708150548048, -0.17856628958294152`}, {
      0.7843717158241886, -0.17386806096391239`}, {
      0.7884126413500007, -0.16915653711515022`}, {
      0.7923939449236276, -0.1644322204074277}, {
      0.7963159760592915, -0.1596956031341899}, {
      0.8001790805123775, -0.15494716779611312`}, {
      0.8039836002995546, -0.15018738737661735`}, {
      0.8077298737205342, -0.14541672560866056`}, {
      0.8114182353813199, -0.1406356372331294}, {
      0.8150490162188136, -0.13584456824912935`}, {
      0.8186225435266511, -0.13104395615646508`}, {
      0.8221391409821479, -0.126234230190591}, {
      0.825599128674248, -0.12141581155030055`}, {
      0.8290028231323711, -0.11658911361841315`}, {
      0.8323505373560639, -0.11175454217570684`}, {
      0.8356425808453672, -0.10691249560833607`}, {
      0.8388792596318149, -0.10206336510896405`}, {
      0.8420608763099895, -0.0972075348718307}, {
      0.8451877300695624, -0.09234538228196892}, {
      0.8482601167277523, -0.08747727809877329}, {
      0.8512783287621398, -0.08260358663411803}, {
      0.8542426553437823, -0.07772466592521332}, {
      0.8571533823705739, -0.07284086790238195}, {
      0.8600107925008034, -0.06795253855193141}, {
      0.8628151651868625, -0.06306001807429007}, {
      0.8655667767090638, -0.058163641037569505`}, {
      0.8682659002095274, -0.05326373652670916}, {
      0.870912805726102, -0.0483606282883537}, {
      0.8735077602262845, -0.04345463487160764}, {
      0.8760510276411101, -0.03854606976480676}, {
      0.8785428688989815, -0.03363524152844034}, {
      0.8809835419594125, -0.028722453924353557`}, {
      0.883373301846662, -0.023808006041354534`}, {
      0.8857124006832341, -0.018892192417345983`}, {
      0.8880010877232275, -0.013975303158096994`}, {
      0.8902396093855115, -0.00905762405276636}, {
      0.8924282092867142, -0.004139436686284789}, {0.8945671282740064, 
      0.0007789814513005549}, {0.8966566044576673, 0.005697356858419425}, {
      0.8986968732434177, 0.010615419918413593`}, {0.9006881673645121, 
      0.015532904798720961`}, {0.9026307169135752, 0.020449549352719205`}, {
      0.904524749374177, 0.025365095024142713`}, {0.9063704896521352, 
      0.03027928675398952}, {0.9081681601065387, 0.035191872889838}, {
      0.9099179805804862, 0.040102605097495775`}, {0.9116201684315318, 
      0.045011238274905996`}, {0.9132749385618345, 0.04991753046823888}, {
      0.9148825034480059, 0.054821242790098756`}, {0.9164430731706535, 
      0.05972213933977926}, {0.9179568554436149, 0.06461998712550172}, {
      0.9194240556428817, 0.069514555988574}, {0.9208448768352094, 
      0.074405618529409}, {0.9222195198064145, 0.07929295003534434}, {
      0.9235481830893533, 0.08417632841020661}, {0.9248310629915865, 
      0.08905553410556533}, {0.9260683536227253, 0.09393035005362377}, {
      0.9272602469214615, 0.0988005616016956}, {0.9284069326822803, 
      0.10366595644821779`}, {0.9295085985818582, 0.10852632458025183`}, {
      0.9305654302051453, 0.11338145821242732`}, {0.931577611071134, 
      0.11823115172728288`}, {0.9325453226583159, 0.12307520161696119`}, {
      0.9334687444298275, 0.12791340642621624`}, {0.9343480538582878, 
      0.13274556669669227`}, {0.9351834264503291, 0.13757148491243504`}, {
      0.9359750357708243, 0.14239096544659743`}, {0.9367230534668107, 
      0.14720381450930276`}, {0.9374276492911165, 0.15200984009662966`}, {
      0.9380889911256899, 0.15680885194068453`}, {0.938707245004634, 
      0.16160066146072774`}, {0.9392825751369525, 0.16638508171532118`}, {
      0.9398151439290061, 0.17116192735546581`}, {0.9403051120066862, 
      0.17593101457869867`}, {0.9407526382373057, 0.18069216108411973`}, {
      0.9411578797512133, 0.18544518602831994`}, {0.9415209919631324, 
      0.19018990998218274`}, {0.9418421285932297, 0.19492615488853196`}, {
      0.9421214416879156, 0.1996537440205998}, {0.9423590816403813, 
      0.2043725019412898}, {0.9425551972108754, 0.20908225446320977`}, {
      0.9427099355467234, 0.2137828286094508}, {0.9428234422020948, 
      0.2184740525750893}, {0.9428958611575204, 0.22315575568938917`}, {
      0.9429273348391646, 0.22782776837868202`}, {0.9429180041378551, 
      0.2324899221299046}, {0.9428680084278755, 0.23714204945477216`}, {
      0.942777485585523, 0.24178398385456765`}, {0.9426465720074355, 
      0.24641555978552732`}, {0.9424754026286931, 0.2510366126248033}, {
      0.9422641109406953, 0.255646978636985}, {0.9420128290088204, 
      0.26024649494116037`}, {0.9417216874898692, 0.26483499947850075`}, {
      0.9413908156492968, 0.26941233098035067`}, {0.9410203413782384, 
      0.2739783289368068}, {0.9406103912103293, 0.27853283356576936`}, {
      0.9401610903383266, 0.28307568578245046`}, {0.9396725626305339, 
      0.2876067271693234}, {0.939144930647034, 0.292125799946498}, {
      0.9385783156557327, 0.29663274694250746`}, {0.9379728376482186, 
      0.30112741156549183`}, {0.9373286153554408, 0.3056096377747645}, {
      0.9366457662632103, 0.31007927005274794`}, {0.9359244066275276, 
      0.3145361533772647}, {0.9351646514897405, 0.31898013319417157`}, {
      0.9343666146915358, 0.32341105539032294`}, {0.9335304088897695, 
      0.32782876626685176`}, {0.9326561455711373, 0.3322331125127552}, {
      0.9317439350666915, 0.3366239411787736}, {0.930793886566206, 
      0.34100109965154995`}, {0.929806108132394, 0.3453644356280598}, {
      0.9287807067149824, 0.3497137970902989}, {0.9277177881646458, 
      0.3540490322802183}, {0.9266174572468042, 0.3583699896748961}, {
      0.9254798176552884, 0.36267651796193445`}, {0.9243049720258758, 
      0.3669684660150725}, {0.9230930219497012, 0.3712456828700036}, {
      0.9218440679865463, 0.3755080177003879}, {0.9205582096780102, 
      0.37975531979404986`}, {0.9192355455605667, 0.38398743852935086`}, {
      0.9178761731785104, 0.38820422335172716`}, {0.9164801890967954, 
      0.3924055237503845}, {0.9150476889137712, 0.3965911892351389}, {
      0.913578767273819, 0.4007610693133955}, {0.9120735178798918, 
      0.4049150134672554}, {0.9105320335059616, 0.4090528711307426}, {
      0.9089544060093792, 0.4131744916671414}, {0.9073407263431481, 
      0.41727972434643545`}, {0.9056910845681178, 0.42136841832284083`}, {
      0.904005569865099, 0.4254404226124235}, {0.9022842705469059, 
      0.42949558607079297`}, {0.9005272740703277, 0.4335337573708646}, {
      0.8987346670480348, 0.4375547849806809}, {0.8969065352604217, 
      0.44155851714128486`}, {0.8950429636673921, 0.44554480184463635`}, {
      0.8931440364200887, 0.4495134868115639}, {0.8912098368725727, 
      0.45346441946974364`}, {0.8892404475934562, 0.4573974469316977}, {
      0.8872359503774915, 0.46131241597280354`}, {0.8851964262571211, 
      0.4652091730093069}, {0.8831219555139939, 0.46908756407633057`}, {
      0.8810126176904485, 0.47294743480587004`}, {0.8788684916009715, 
      0.4767886304047701}, {0.876689655343632, 0.4806109956326726}, {
      0.8744761863114978, 0.48441437477992916`}, {0.8722281612040373, 
      0.48819861164546996`}, {0.869945656038511, 0.4919635495146217}, {
      0.867628746161358, 0.4957090311368661}, {0.8652775062595801, 
      0.4994348987035322}, {0.8628920103721301, 0.5031409938254134}, {
      0.8604723319013072, 0.5068271575103029}, {0.8580185436241642, 
      0.5104932301404383}, {0.8555307177039322, 0.5141390514498486}, {
      0.8530089257014669, 0.5177644605015951}, {0.8504532385867207, 
      0.5213692956648983}, {0.8478637267502468, 0.5249533945921439}, {
      0.8452404600147387, 0.5285165941957576}, {0.8425835076466122, 
      0.5320587306249437}, {0.8398929383676326, 0.5355796392422766}, {
      0.8371688203665948, 0.5390791546001388}, {0.8344112213110597, 
      0.5425571104169966}, {0.8316202083591538, 0.5460133395535054}, {
      0.8287958481714365, 0.5494476739884355}, {0.8259382069228411, 
      0.5528599447944113}, {0.8230473503146968, 0.5562499821134539}, {
      0.8201233435868343, 0.5596176151323203}, {0.8171662515297852, 
      0.5629626720576284}, {0.8141761384970779, 0.56628498009076}, {
      0.8111530684176389, 0.5695843654025339}, {0.8080971048083038, 
      0.572860653107638}, {0.8050083107864471, 0.5761136672388132}, {
      0.8018867490827356, 0.5793432307207783}, {0.7987324820540136, 
      0.5825491653438878}, {0.7955455716963264, 0.5857312917375118}, {
      0.7923260796580903, 0.58888942934313}, {0.7890740672534157, 
      0.5920233963871282}, {0.7857895954755916, 0.5951330098532885}, {
      0.7824727250107401, 0.598218085454963}, {0.7791235162516474, 
      0.60127843760692}, {0.7757420293117822, 0.6043138793968533}, {
      0.7723283240395067, 0.6073242225565436}, {0.7688824600324938, 
      0.6103092774326601}, {0.7654044966523548, 0.6132688529571934}, {
      0.7618944930394903, 0.6162027566175068}, {0.7583525081281733, 
      0.6191107944259958}, {0.7547786006618736, 0.6219927708893427}, {
      0.7511728292088344, 0.6248484889773561}, {0.7475352521779125, 
      0.627677750091383}, {0.7438659278346913, 0.6304803540322798}, {
      0.7401649143178795, 0.6332560989679313}, {0.7364322696560063, 
      0.6360047814003049}, {0.7326680517844253, 0.6387261961320254}, {
      0.7288723185626395, 0.6414201362324587}, {0.7250451277919596, 
      0.6440863930032911}, {0.7211865372335106, 0.6467247559435879}, {
      0.7172966046265971, 0.6493350127143211}, {0.7133753877074451, 
      0.6519169491023478}, {0.709422944228332, 0.6544703489838269}, {
      0.7054393319771206, 0.6569949942870595}, {0.7014246087972137, 
      0.6594906649547351}, {0.6973788326079432, 0.6619571389055707}, {
      0.6933020614254125, 0.6643941919953252}, {0.6891943533838077, 
      0.6668015979771726}, {0.6850557667571961, 0.6691791284614182}, {
      0.6808863599818302, 0.6715265528745399}, {0.6766861916789758, 
      0.6738436384175369}, {0.6724553206782844, 0.6761301500235691}, {
      0.6681938060417302, 0.6783858503148668}, {0.6639017070881319, 
      0.6806104995588937}, {0.6595790834182823, 0.6828038556237438}, {
      0.6552259949407074, 0.6849656739327507}, {0.6508425018980787, 
      0.687095707418292}, {0.646428664894303, 0.689193706474766}, {
      0.6419845449223148, 0.6912594189107203}, {0.6375102033925975, 
      0.6932925899001112}, {0.6330057021624593, 0.695292961932671}, {
      0.6284711035660945, 0.6972602747633608}, {0.6239064704454553, 
      0.6991942653608869}, {0.6193118661819685, 0.701094667855254}, {
      0.614687354729125, 0.7029612134843345}, {0.610033000645976, 
      0.704793630539427}, {0.6053488691315705, 0.7065916443097783}, {
      0.6006350260603667, 0.7083549770260438}, {0.5958915380186559, 
      0.7100833478026592}, {0.5911184723420354, 0.7117764725790965}, {
      0.5863158971539703, 0.7134340640599746}, {0.5814838814054837, 
      0.7150558316539983}, {0.5766224949160197, 0.7166414814116949}, {
      0.571731808415521, 0.7181907159619175}, {0.5668118935877681, 
      0.7197032344470864}, {0.5618628231150276, 0.7211787324571344}, {
      0.5568846707240582, 0.722616901962126}, {0.5518775112335272, 
      0.7240174312435163}, {0.5468414206028892, 0.7253800048240144}, {
      0.5417764759827844, 0.7267043033960202}, {0.5366827557670136, 
      0.7279900037485945}, {0.5315603396461497, 0.7292367786929297}, {
      0.5264093086628493, 0.7304442969862821}, {0.5212297452689298, 
      0.7316122232543282}, {0.5160217333842791, 0.732740217911905}, {
      0.5107853584576696, 0.7338279370820968}, {0.5055207075295497, 
      0.7348750325136245}, {0.5002278692968908, 0.7358811514964986}, {
      0.49490693418016846`, 0.7368459367758909}, {0.48955799439256253`, 
      0.7377690264641833}, {0.48418114401146256`, 0.7386500539511469}, {
      0.47877647905236975`, 0.7394886478122081}, {0.47334409754529, 
      0.7402844317147534}, {0.46788409961371663`, 0.7410370243224261}, {
      0.4623965875563063, 0.7417460391973664}, {0.456881665931355, 
      0.7424110847003451}, {0.45133944164418693`, 0.7430317638887411}, {
      0.44577002403757315`, 0.7436076744123095}, {0.44017352498530193`, 
      0.7441384084066895}, {0.43455005898902893`, 0.7446235523845987}, {
      0.4288997432785405, 0.7450626871246571}, {0.42322269791556894`, 
      0.7454553875577883}, {0.41751904590130556`, 0.7458012226511392}, {
      0.4117889132877635, 0.7460997552894629}, {0.4060324292931491, 
      0.7463505421539042}, {0.40024972642140766`, 0.7465531335981314}, {
      0.3944409405861178, 0.7467070735217529}, {0.388606211238915, 
      0.7468118992409583}, {0.3827456815026352, 0.7468671413563225}, {
      0.3768594983093763, 0.7468723236177122}, {0.37094781254368536`, 
      0.74682696278623}, {0.36501077919108904`, 0.7467305684931356}, {
      0.3590485574921951, 0.7465826430956787}, {0.35306131110260225`, 
      0.7463826815297809}, {0.3470492082588683, 0.746130171159503}, {
      0.3410124219507978, 0.7458245916232324}, {0.33495113010032157`, 
      0.7454654146765294}, {0.328865515747256, 0.7450521040315655}, {
      0.3227557672422404, 0.7445841151930944}, {0.3166220784471687, 
      0.7440608952908903}, {0.31046464894344306`, 0.7434818829085934}, {
      0.30428368424839597`, 0.7428465079089035}, {0.29807939604024286`, 
      0.7421541912550604}, {0.29185200239194403`, 0.7414043448285564}, {
      0.2856017280143749, 0.7405963712430248}, {0.27932880450922154`, 
      0.7397296636542526}, {0.27303347063203987`, 0.7388036055662682}, {
      0.266715972565938, 0.7378175706334575}, {0.2603765642063642, 
      0.7367709224586674}, {0.2540155074575063, 0.7356630143872582}, {
      0.24763307254083433`, 0.7344931892970739}, {0.24122953831634375`, 
      0.7332607793843023}, {0.23480519261708535`, 0.7319651059452077}, {
      0.22836033259759664`, 0.7306054791537206}, {0.2218952650968806, 
      0.7291811978348842}, {0.2154103070166106, 0.7276915492341596}, {
      0.2089057857152738, 0.7261358087826093}, {0.20238203941900218`, 
      0.7245132398579845}, {0.19583941764987794`, 0.7228230935417582}, {
      0.18927828167253993`, 0.7210646083721594}, {0.18269900495996022`, 
      0.7192370100932811}, {0.17610197367930394`, 0.7173395114003519}, {
      0.16948758719883236`, 0.7153713116812797}, {0.16285625861685804`, 
      0.7133315967546018}, {0.15620841531381294`, 0.7112195386039971}, {
      0.14954449952854432`, 0.7090342951095431}, {0.14286496896001072`, 
      0.706775009775933}, {0.13617029739561026`, 0.7044408114578984}, {
      0.12946097536743686`, 0.7020308140831203}, {0.12273751083782619`, 
      0.6995441163729529}, {0.11600042991562302`, 0.6969798015613253}, {
      0.10925027760467512`, 0.6943369371122387}, {0.10248761858613524`, 
      0.6916145744363273}, {0.09571303803623384, 0.6888117486070121}, {
      0.08892714248126952, 0.6859274780768385}, {0.08213056069165274, 
      0.6829607643946641}, {0.07532394461693098, 0.6799105919244367}, {
      0.06850797036382082, 0.676775927566391}, {0.061683339219373176`, 
      0.6735557204815862}, {0.05485077872150419, 0.6702489018208082}, {
      0.04801104377923413, 0.6668543844589776}, {0.04116491784509153, 
      0.6633710627363232}, {0.03431321414225909, 0.6597978122077232}, {
      0.027456776949161595`, 0.6561334894017605}, {0.020596482944324136`, 
      0.6523769315912065}, {0.013733242614461558`, 0.6485269565768239}, {
      0.006868001728896414, 0.644582362486579}, {1.7428835429721817`*^-6, 
      0.6405419275925661}, {-0.006864512882161425, 
      0.6364044101481855}, {-0.013729704391849791`, 
      0.6321685482483719}, {-0.02059272855866687, 
      0.6278330597159552}, {-0.02745243852773115, 
      0.6233966420175426}, {-0.034307641730638966`, 
      0.6188579722126506}, {-0.04115709784788159, 
      0.6142157069401823}, {-0.047999516674897186`, 
      0.6094684824467538}, {-0.05483355588733435, 
      0.6046149146618083}, {-0.06165781870096369, 
      0.5996535993249453}, {-0.06847085142154191, 
      0.5945831121714107}, {-0.07527114087981067, 
      0.5894020091822719}, {-0.08205711174670435, 
      0.5841088269064261}, {-0.08882712372374973, 
      0.5787020828622684}, {-0.09557946860357107, 
      0.5731802760275911}, {-0.10231236719537135`, 
      0.5675418874270888}, {-0.1090239661102503, 
      0.5617853808277228}, {-0.11571233440124849`, 
      0.5559092035531492}, {-0.12237546005308313`, 
      0.5499117874294464}, {-0.1290112463166732, 
      0.5437915498754963}, {-0.1356175078837502, 
      0.5375468951525864}, {-0.1421919668971272, 
      0.5311762157891017}, {-0.1487322487925678, 
      0.5246778941975905}, {-0.15523587796867164`, 
      0.5180503045030013}, {-0.1617002732817939, 
      0.5112918146025222}, {-0.16812274336376024`, 
      0.5044007884791974}, {-0.17450048176105107`, 
      0.4973755887933661}, {-0.18083056189523267`, 
      0.49021457977795413`}, {-0.18710993184573768`, 
      0.48291613046576587`}, {-0.19333540895767531`, 
      0.47547861827915155`}, {-0.19950367427921753`, 
      0.4679004330147796}, {-0.20561126683530193`, 
      0.4601799812587013}, {-0.2116545777469577, 
      0.4523156912694537}, {-0.21762984420854742`, 
      0.44430601836958755`}, {-0.22353314333867613`, 
      0.43614945088870494`}, {-0.22936038592450855`, 
      0.4278445167038216}, {-0.23510731008381555`, 
      0.4193897904255885}, {-0.24076947487431036`, 
      0.4107839012815682}, {-0.2463422538858004, 
      0.402025541750306}, {-0.251820828857446, 
      0.3931134770022805}, {-0.2572001833700562, 
      0.38404655520587905`}, {-0.2624750966719373, 
      0.37482371875820303`}, {-0.2676401377064138, 
      0.3654440165016314}, {-0.2726896594198288, 
      0.3559066169875054}, {-0.2776177934406593, 
      0.3462108228478488}, {-0.2824184452333908, 
      0.3363560863344922}, {-0.2870852898450107, 
      0.3263420260820781}, {-0.2916117683773871, 
      0.31616844514689446`}, {-0.29599108533536794`, 
      0.3058353503669988}, {-0.30021620701805674`, 
      0.2953429730802906}, {-0.3042798611392627, 0.2846917912256741}}]}, 
   {RGBColor[1, 1, 0], DiskBox[{0, 0}, 0.03]}},
  AspectRatio->Automatic,
  Background->RGBColor[0.6, 0.6, 0.6],
  ImageSize->{316.5, Automatic}]], "Output",
 CellChangeTimes->{3.401201284528718*^9, 3.401276919331201*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tFinal", "=", "4"}], ";", 
  RowBox[{"R0", "=", "1"}], ";", 
  RowBox[{"V0", "=", 
   RowBox[{".7", "\[Times]", "2", " ", "\[Pi]"}]}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", ".001"}], ";", 
  RowBox[{"p", "=", "1.98"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CalculateOrbit", ";", 
  RowBox[{"DisplayOrbit", "[", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Clear", "[", 
  RowBox[{
  "tFinal", ",", "R0", ",", "V0", ",", "iMaximum", ",", "t", ",", "r", ",", 
   "v"}], "]"}]}], "Input",
 CellChangeTimes->{{3.401193360004507*^9, 3.401193414015883*^9}, 
   3.40119347898011*^9, 3.401193626282501*^9, {3.4012012039931593`*^9, 
   3.401201294841939*^9}}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 1, 1], Thickness[0.0075], 
    LineBox[{{1, 0}, {0.9999802607574417, 0.004398200775617096}, {
      0.999921042624701, 0.0087962279099848}, {0.999822344386562, 
      0.013193907737451764`}, {0.9996841640176247, 0.017591066543554242`}, {
      0.9995064986822451, 0.02198753054058863}, {0.9992893447344513, 
      0.026383125843158516`}, {0.9990326977178361, 0.030777678443687655`}, {
      0.9987365523654261, 0.03517101418789039}, {0.9984009025995276, 
      0.039562958750190884`}, {0.9980257415315499, 0.04395333760908256}, {
      0.9976110614618059, 0.048341976022419145`}, {0.9971568538792913, 
      0.05272869900262859}, {0.9966631094614417, 0.057113331291841124`}, {
      0.9961298180738689, 0.0614956973369227}, {0.995556968770078, 
      0.06587562126440492}, {0.9949445497911631, 0.07025292685530259}, {
      0.9942925485654852, 0.0746274375198098}, {0.9936009517083323, 
      0.07899897627186568}, {0.9928697450215614, 0.08336736570358048}, {
      0.9920989134932251, 0.08773242795951282}, {0.9912884412971814, 
      0.09209398471078875}, {0.9904383117926915, 0.09645185712905338}, {
      0.989548507524002, 0.10080586586024522`}, {0.988619010219917, 
      0.10515583099818383`}, {0.9876498007933585, 0.10950157205796102`}, {
      0.9866408593409175, 0.11384290794912548`}, {0.9855921651423977, 
      0.11817965694865115`}, {0.9845036966603522, 0.12251163667367888`}, {
      0.9833754315396154, 0.12683866405402122`}, {0.9822073466068318, 
      0.1311605553044199}, {0.9809994178699837, 0.1354771258965452}, {
      0.9797516205179182, 0.1397881905307268}, {0.9784639289198778, 
      0.14409356310740465`}, {0.9771363166250345, 0.14839305669828917`}, {
      0.9757687563620311, 0.15268648351721917`}, {0.9743612200385303, 
      0.156973654890706}, {0.972913678740776, 0.16125438122815244`}, {
      0.9714261027331673, 0.16552847199173393`}, {0.9698984614578495, 
      0.1697957356659304}, {0.9683307235343234, 0.17405597972669604`}, {
      0.9667228567590764, 0.17830901061025445`}, {0.9650748281052391, 
      0.1825546336815063}, {0.9633866037222686, 0.18679265320203622`}, {
      0.9616581489356639, 0.19102287229770565`}, {0.9598894282467156, 
      0.19524509292581788`}, {0.9580804053322935, 0.19945911584184117`}, {
      0.9562310430446765, 0.20366474056567593`}, {0.9543413034114281, 
      0.20786176534745107`}, {0.9524111476353219, 0.21204998713283468`}, {
      0.9504405360943211, 0.2162292015278439}, {0.9484294283416174, 
      0.22039920276313815`}, {0.9463777831057323, 0.22455978365778}, {
      0.9442855582906874, 0.2287107355824471}, {0.942152710976248, 
      0.23285184842207846`}, {0.9399791974182451, 0.23698291053793838`}, {
      0.9377649730489825, 0.24110370872907974`}, {0.9355099924777335, 
      0.2452140281931894}, {0.9332142094913337, 0.24931365248679696`}, {
      0.9308775770548778, 0.2534023634848279}, {0.9285000473125244, 
      0.2574799413394824}, {0.9260815715884179, 0.26154616443841877`}, {
      0.9236221003877337, 0.2656008093622231}, {0.921121583397856, 
      0.26964365084114206`}, {0.9185799694896928, 0.27367446171105947`}, {
      0.9159972067191402, 0.2776930128686932}, {0.9133732423287025, 
      0.28169907322599075`}, {0.9107080227492776, 0.2856924096637003}, {
      0.9080014936021177, 0.28967278698409304`}, {0.9052535997009744, 
      0.29363996786281304`}, {0.90246428505444, 0.2975937127998294}, {
      0.8996334928684939, 0.3015337800694644}, {0.8967611655492678, 
      0.3054599256694727}, {0.8938472447060387, 0.3093719032691427}, {
      0.8908916711544659, 0.31326946415639356`}, {0.8878943849200807, 
      0.31715235718383855`}, {0.884855325242046, 0.32102032871378566`}, {
      0.8817744305771983, 0.3248731225621447}, {0.8786516386043868, 
      0.3287104799412101}, {0.8754868862291257, 0.3325321394012878}, {
      0.8722801095885766, 0.33633783677113205`}, {0.8690312440568758, 
      0.34012730509715977`}, {0.8657402242508272, 0.34390027458140654`}, {
      0.862406984035977, 0.34765647251818876`}, {0.8590314565330913, 
      0.351395623229435}, {0.8556135741250553, 0.35511744799864875`}, {
      0.8521532684642181, 0.35882166500346313`}, {0.8486504704802023, 
      0.36250798924674754`}, {0.8451051103882037, 0.366176132486225}, {
      0.8415171176978058, 0.369825803162557}, {0.8378864212223331, 
      0.37345670632585243`}, {0.834212949088772, 0.37706854356055525`}, {
      0.8304966287482849, 0.38066101290866394`}, {0.8267373869873503, 
      0.3842338087912353}, {0.8229351499395561, 0.38778662192812274`}, {
      0.819089843098081, 0.39131913925589834`}, {0.8152013913288968, 
      0.3948310438439061}, {0.8112697188847268, 0.39832201480839236`}, {
      0.8072947494197976, 0.4017917272246576}, {0.8032764060054245, 
      0.40523985203717255`}, {0.7992146111464681, 0.4086660559675987}, {
      0.7951092867987086, 0.41207000142065303`}, {0.7909603543871802, 
      0.4154513463877537}, {0.7867677348255125, 0.41880974434838175`}, {
      0.7825313485363303, 0.4221448441690925}, {0.7782511154727606, 
      0.42545629000010726`}, {0.7739269551411033, 0.4287437211694148}, {
      0.7695587866247211, 0.4320067720743091}, {0.7651465286092088, 
      0.4352450720702881}, {0.7606900994089046, 0.43845824535723577`}, {
      0.7561894169948083, 0.4416459108628066}, {0.7516443990239768, 
      0.44480768212293104`}, {0.7470549628704675, 0.44794316715935567`}, {
      0.7424210256579067, 0.45105196835413025`}, {0.7377425042937631, 
      0.4541336823209516}, {0.7330193155054084, 0.4571878997732701}, {
      0.7282513758780551, 0.46021420538906366`}, {0.7234386018946621, 
      0.4632121776721778}, {0.7185809099779066, 0.466181388810132}, {
      0.7136782165343232, 0.46912140452828444`}, {0.7087304380007187, 
      0.47203178394024803`}, {0.7037374908929737, 0.47491207939444435`}, {
      0.6986992918573507, 0.4777618363166808}, {0.6936157577244332, 
      0.48058059304863116`}, {0.6884868055658256, 0.4833678806820972}, {
      0.6833123527537538, 0.48612322288892473`}, {0.6780923170237089, 
      0.48884613574644387`}, {0.6728266165402891, 0.4915361275582989}, {
      0.667515169966398, 0.49419269867053}, {0.6621578965359697, 
      0.4968153412827644}, {0.656754716130397, 0.4994035392543699}, {
      0.6513055493588523, 0.5019567679054207}, {0.6458103176426949, 
      0.5044744938123195}, {0.6402689433041753, 0.506956174597917}, {
      0.6346813496596526, 0.5094012587159639}, {0.6290474611175577, 
      0.5118091852297266}, {0.6233672032813422, 0.5141793835845939}, {
      0.6176405030576708, 0.5165112733744948}, {0.6118672887701256, 
      0.5188042641019458}, {0.6060474902787069, 0.5210577549315394}, {
      0.6001810391054285, 0.5232711344366796}, {0.5942678685663264, 
      0.5254437803393689}, {0.5883079139102102, 0.5275750592428415}, {
      0.5823011124645109, 0.5296643263568378}, {0.5762474037885952, 
      0.531710925215305}, {0.5701467298349375, 0.5337141873863092}, {
      0.5639990351185614, 0.5356734321739336}, {0.5578042668951877, 
      0.5375879663119398}, {0.5515623753485478, 0.5394570836489582}, {
      0.5452733137873487, 0.5412800648249718}, {0.5389370388524017, 
      0.5430561769388567}, {0.5325535107344573, 0.5447846732067313}, {
      0.5261226934033173, 0.5464647926108708}, {0.519644554848829, 
      0.5480957595389336}, {0.5131190673344007, 0.5496767834132491}, {
      0.506546207663712, 0.5512070583099082}, {0.49992595746133284`, 
      0.5526857625674026}, {0.4932583034680062, 0.5541120583845517}, {
      0.4865432378513892, 0.5554850914074596}, {0.4797807585330973, 
      0.5568039903052451}, {0.4729708695329421, 0.5580678663342855}, {
      0.46611358133130437`, 0.5592758128907231}, {0.45920891125064045`, 
      0.5604269050509822}, {0.4522568838571747, 0.5615201991000529}, {
      0.4452575313838943, 0.562554732047304}, {0.43821089417602554`, 
      0.5635295211295958}, {0.43111702116024037`, 0.5644435633014747}, {
      0.42397597033891304`, 0.5652958347122452}, {0.41678780931082526`, 
      0.5660852901697285}, {0.4095526158197973, 0.5668108625905391}, {
      0.4022704783328102, 0.5674714624367292}, {0.3949414966492749, 
      0.5680659771386772}, {0.38756578254319923`, 0.5685932705041282}, {
      0.3801434604401069, 0.5690521821133231}, {0.37267466813066963`, 
      0.5694415267001983}, {0.36515955752312707`, 0.5697600935196774}, {
      0.35759829543668964`, 0.5700066457011286}, {0.34999106443824624`, 
      0.5701799195881202}, {0.342338063724832, 0.5702786240646692}, {
      0.3346395100544534, 0.5703014398682539}, {0.32689563872801564`, 
      0.570247018889942}, {0.3191067046252537, 0.5701139834620822}, {
      0.3112729832977344, 0.5699009256341075}, {0.30339477212216726`, 
      0.5696064064371211}, {0.2954723915174458, 0.5692289551380627}, {
      0.28750618622902924`, 0.5687670684844042}, {0.2794965266844744, 
      0.5682192099404849}, {0.27144381042413473`, 0.567583808916783}, {
      0.2633484636112608, 0.5668592599936202}, {0.2552109426259611, 
      0.5660439221410303}, {0.24703173574771642`, 0.5651361179367681}, {
      0.23881136493138322`, 0.5641341327847229}, {0.23055038768187083`, 
      0.563036214136309}, {0.22224939903293442`, 0.5618405707177527}, {
      0.21390903363578823`, 0.5605453717665803}, {0.20552996796351172`, 
      0.5591487462810325}, {0.19711292263749175`, 0.5576487822866061}, {
      0.1886586648824173, 0.5560435261244343}, {0.18016801111661473`, 
      0.5543309817667991}, {0.17164182968478, 0.5525091101656914}, {
      0.1630810437404266, 0.5505758286410374}, {0.15448663428561776`, 
      0.5485290103159717}, {0.14585964337578788`, 0.5463664836073792}, {
      0.13720117749767138`, 0.5440860317808532}, {0.12851241112854442`, 
      0.5416853925802315}, {0.11979459048513498`, 0.5391622579429772}, {
      0.11104903747066372`, 0.5365142738138923}, {0.10227715382852892`, 
      0.5337390400709707}, {0.09348042551113261, 0.530834110578646}, {
      0.08466042727224757, 0.52779699338526}, {0.07581882749112943, 
      0.5246251510832852}, {0.06695739323626705, 0.5213160013526849}, {
      0.05807799557621468, 0.5178669177097911}, {0.04918261514433947, 
      0.5142752304862385}, {0.040273347963517725`, 0.5105382280648041}, {
      0.031352411535793503`, 0.5066531584014844}, {0.02242215120073859, 
      0.5026172308657838}, {0.013485046764684198`, 0.49842761843399713`}, {
      0.004543719401089087, 0.4940814602732356}, {-0.004399061179982311, 
      0.48957586475706205`}, {-0.013340369298027196`, 
      0.4849079129568521}, {-0.02227711561189738, 
      0.4800746626563605}, {-0.03120603918498146, 
      0.47507315294042274`}, {-0.04012369932194815, 
      0.46990040941221084`}, {-0.04902646719832801, 
      0.4645534500969513}, {-0.057910517310087214`, 
      0.45902929209342763`}, {-0.0667718187770712, 
      0.4533249590378569}, {-0.075606126541846, 
      0.44743748944774797`}, {-0.08440897251411567, 
      0.44136394601600104`}, {-0.093175656720618, 
      0.43510142592765155`}, {-0.10190123853126029`, 
      0.42864707227312876`}, {-0.11058052804430353`, 
      0.4219980866324939}, {-0.11920807772667014`, 
      0.41515174290461665`}, {-0.12777817441994357`, 
      0.4081054024533837}, {-0.13628483183832457`, 
      0.40085653063951443`}, {-0.14472178370163852`, 
      0.393402714801061}, {-0.15308247766433192`, 
      0.3857416837378362}, {-0.16136007022006563`, 
      0.3778713287444545}, {-0.1695474227807446, 
      0.36978972622298767`}, {-0.1776370991482608, 
      0.3614951618890048}, {-0.18562136461640308`, 
      0.35298615656357835`}, {-0.19349218695872503`, 
      0.3442614935182916}, {-0.20124123957494044`, 
      0.3353202473100428}, {-0.20885990708278426`, 
      0.32616181400721933`}, {-0.21633929365323426`, 
      0.3167859426684614}, {-0.22367023439339584`, 
      0.3071927678897251}, {-0.23084331008193945`, 
      0.2973828431848813}, {-0.23784886555537516`, 
      0.2873571749100732}, {-0.24467703202820879`, 
      0.2771172563832389}, {-0.2513177536046988, 
      0.26666510178864883`}, {-0.2577608182031279, 
      0.25600327939348605`}, {-0.26399589306398014`, 
      0.24513494354128681`}, {-0.27001256495019715`, 
      0.23406386482777275`}, {-0.27580038507018106`, 
      0.22279445781098084`}, {-0.28134891866235573`, 
      0.21133180556272915`}, {-0.2866477990744793, 
      0.19968168033573616`}, {-0.29168678605290116`, 
      0.18785055960367814`}, {-0.29645582782884383`, 
      0.1758456367336428}, {-0.3009451264537961, 
      0.16367482557509488`}, {-0.3051452056984192, 
      0.15134675829938765`}, {-0.3090469806940974, 
      0.13887077590103608`}, {-0.31264182836926957`, 
      0.12625691087734828`}, {-0.31592165762034446`, 
      0.11351586173621699`}, {-0.31887897806593135`, 
      0.10065895914099245`}, {-0.32150696616969093`, 
      0.08769812368283941}, {-0.32379952748708873`, 
      0.07464581546960304}, {-0.325751353799327, 
      0.06151497592920944}, {-0.3273579739468027, 
      0.048318962436962036`}, {-0.3286157972656826, 
      0.03507147658083776}, {-0.3295221486634947, 
      0.021786487067748557`}, {-0.3300752945395718, 
      0.008478148437658612}, {-0.33027445895806035`, -0.004839283116754972}, \
{-0.33011982970736486`, -0.018151535436266228`}, {-0.32961255412110146`, \
-0.03144440615211712}, {-0.3287547247817004, -0.04470384707958247}, \
{-0.3275493554682265, -0.057916045861324345`}, {-0.3260003479346736, \
-0.07106750356555462}, {-0.3241124503048879, -0.08414510707508391}, \
{-0.3218912080379357, -0.09713619526681613}, {-0.31934290854773323`, \
-0.11002861816890731`}, {-0.316474520649928, -0.12281078848561877`}, \
{-0.313293630056483, -0.1354717250886763}, {-0.3098083721454601, \
-0.14800108827994146`}, {-0.3060273632032985, -0.16038920682557423`}, \
{-0.3019596312741148, -0.17262709694008443`}, {-0.29761454766093726`, \
-0.18470647355473627`}, {-0.29300176001363565`, -0.19661975433532763`}, \
{-0.2881311278140817, -0.20836005701766563`}, {-0.2830126609369923, \
-0.21992119070482685`}, {-0.27765646183064757`, -0.23129764181952112`}, \
{-0.27207267173015826`, -0.24248455542956937`}, {-0.26627142119121094`, \
-0.2534777126673967}, {-0.2602627851173261, -0.26427350494872776`}, \
{-0.25405674235078807`, -0.27486890566476846`}, {-0.24766313980784346`, \
-0.28526143997948894`}, {-0.24109166106306726`, -0.2954491533124505}, \
{-0.23435179922588242`, -0.30543057903092974`}, {-0.22745283390350995`, \
-0.31520470581550686`}, {-0.2204038120081814, -0.3247709451030238}, \
{-0.21321353214107153`, -0.3341290989516893}, {-0.20589053226977103`, \
-0.34327932861651617`}, {-0.19844308040882985`, -0.35222212407025555`}, \
{-0.19087916801258012`, -0.3609582746562506}, {-0.18320650579478362`, \
-0.3694888410155838}, {-0.17543252169942697`, -0.37781512839172954`}, \
{-0.16756436076011835`, -0.3859386613816388}, {-0.15960888660106576`, \
-0.3938611601726363}, {-0.1515726843497171, -0.4015845182794429}, \
{-0.14346206474913126`, -0.4091107817747331}, {-0.13528306927646214`, \
-0.416442129989524}, {-0.1270414760921271, -0.42358085764598813`}, \
{-0.11874280666194834`, -0.4305293583745851}, {-0.11039233291154137`, \
-0.437290109559347}, {-0.10199508478829329`, -0.4438656584493507}, \
{-0.0935558581213024, -0.4502586094705427}, {-0.08507922268356874, \
-0.45647161266982417`}, {-0.07656953037350116, -0.4625073532223911}, \
{-0.06803092344444267, -0.46836854193349475`}, {-0.059467342721437194`, \
-0.4740579066668337}, {-0.05088253575391177, -0.47957818463351704`}, \
{-0.04228006486138481, -0.4849321154777874}, {-0.03366331503679751, \
-0.49012243509832365`}, {-0.025035501678671527`, -0.4951518701468456}, \
{-0.01639967812909244, -0.5000231331488151}, {-0.007758742999576445, \
-0.5047389181941937}, {0.0008845527287332407, -0.5093018971494155}, {
      0.009527598840307568, -0.5137147163449082}, {
      0.01816791926243091, -0.5179799936956127}, {
      0.026803165688112838`, -0.5221003162149781}, {
      0.035431111446127074`, -0.5260782378858199}, {
      0.04404964561762864, -0.5299162778542139}, {
      0.052656767397166925`, -0.5336169189152465}, {
      0.061250580694568485`, -0.5371826062619345}, {
      0.06982928897307099, -0.5406157464709836}, {
      0.07839119031821604, -0.5439187067012543}, {
      0.0869346727313234, -0.5470938140828563}, {
      0.09545820964084599, -0.5501433552767098}, {
      0.10396035562452094`, -0.5530695761861801}, {
      0.11243974233496534`, -0.5558746818040384}, {
      0.12089507462119935`, -0.5585608361795145}, {
      0.12932512683849645`, -0.561130162491606}, {
      0.137728739338949, -0.5635847432160921}, {
      0.14610481513518242`, -0.5659266203748811}, {
      0.15445231672974563`, -0.568157795857401}, {
      0.16277026310283677`, -0.570280231804734}, {
      0.17105772685118706`, -0.5722958510481004}, {
      0.1793138314711123, -0.5742065375941244}, {
      0.18753774877894744`, -0.5760141371500682}, {
      0.1957286964622998, -0.5777204576829081}, {
      0.20388593575578448`, -0.5793272700067519}, {
      0.21200876923514217`, -0.5808363083936647}, {
      0.22009653872387663`, -0.5822492712034898}, {
      0.2281486233067901, -0.5835678215287154}, {
      0.23616443744503188`, -0.58479358785087}, {
      0.24414342918751214`, -0.5859281647053083}, {
      0.2520850784737635, -0.5869731133516047}, {
      0.2599888955235601, -0.5879299624470825}, {
      0.2678544193088247, -0.5888002087212951}, {
      0.2756812161035677, -0.5895853176495348}, {
      0.2834688781078096, -0.5902867241236737}, {
      0.2912170221416381, -0.5909058331188558}, {
      0.29892528840574395`, -0.5914440203547455}, {
      0.3065933393049628, -0.5919026329502093}, {
      0.3142208583315292, -0.5922829900704614}, {
      0.3218075490049168, -0.5925863835658408}, {
      0.3293531338653022, -0.5928140786015115}, {
      0.3368573535178421, -0.5929673142774872}, {
      0.3443199657251035, -0.5930473042384834}, {
      0.3517407445451246, -0.5930552372731872}, {
      0.3591194795127181, -0.592992277902613}, {
      0.3664559748617558, -0.5928595669572865}, {
      0.3737500487862935, -0.5926582221430591}, {
      0.38100153273850895`, -0.592389338595412}, {
      0.3882102707615347, -0.5920539894221598}, {
      0.3953761188553698, -0.5916532262345052}, {
      0.4024989443741521, -0.5911880796664368}, {
      0.4095786254531651, -0.5906595598824956}, {
      0.4166150504640396, -0.5900686570739655}, {
      0.42360811749669514`, -0.589416341943569}, {
      0.4305577338666414, -0.5887035661787728}, {
      0.43746381564633713`, -0.5879312629138275}, {
      0.44432628721937073`, -0.5871003471806798}, {
      0.4511450808562955, -0.5862117163489171}, {
      0.4579201363110136, -0.5852662505549063}, {
      0.46465140043666164`, -0.5842648131203099}, {
      0.47133882682000877`, -0.5832082509601616}, {
      0.4779823754334277, -0.582097394980698}, {
      0.48458201230355225`, -0.5809330604671422}, {
      0.49113770919578, -0.5797160474616446}, {
      0.49764944331382416`, -0.5784471411315866}, {
      0.5041171970135607, -0.5771271121284549}, {
      0.5105409575304564, -0.5757567169374974}, {
      0.5169207167199015, -0.5743366982183699}, {
      0.5232564708098063, -0.5728677851369843}, {
      0.5295482201648537, -0.5713506936887687}, {
      0.5357959690618324, -0.5697861270135451}, {
      0.5419997254755053, -0.5681747757022343}, {
      0.5481595008744968, -0.5665173180955894}, {
      0.554275310026706, -0.5648144205751623}, {
      0.5603471708137848, -0.5630667378467011}, {
      0.5663751040542364, -0.5612749132161743}, {
      0.5723591333347181, -0.5594395788586158}, {
      0.5782992848491508, -0.5575613560799803}, {
      0.5841955872452592, -0.5556408555721932}, {
      0.5900480714781844, -0.5536786776615814}, {
      0.5958567706708314, -0.5516754125508593}, {
      0.6016217199806284, -0.5496316405548483}, {
      0.6073429564723921, -0.5475479323300992}, {
      0.6130205189970103, -0.545424849098586}, {
      0.6186544480756638, -0.5432629428656328}, {
      0.6242447857893285, -0.5410627566322352}, {
      0.629791575673306, -0.5388248246019306}, {
      0.6352948626165489, -0.5365496723823693}, {
      0.6407546927655544, -0.534237817181735}, {
      0.6461711134326136, -0.5318897680001583}, {
      0.6515441730082123, -0.5295060258162633}, {
      0.6568739208773924, -0.5270870837689848}, {
      0.6621604073398871, -0.5246334273347893}, {
      0.6674036835338587, -0.522145534500428}, {
      0.67260380136307, -0.5196238759313487}, {
      0.6777608134273329, -0.51706891513589}, {
      0.6828747729560831, -0.5144811086253749}, {
      0.6879457337449381, -0.5118609060702212}, {
      0.6929737500951008, -0.5092087504521812}, {
      0.6979588767554813, -0.506525078212819}, {
      0.7029011688674113, -0.5038103193983333}, {
      0.7078006819118335, -0.5010648978008277}, {
      0.7126574716588555, -0.4982892310961297}, {
      0.7174715941195589, -0.49548373097825527`}, {
      0.7222431054999638, -0.49264880329061433`}, {
      0.7269720621570515, -0.48978484815404877`}, {
      0.7316585205567501, -0.48689226009179204`}, {
      0.7363025372337998, -0.48397142815143757`}, {
      0.7409041687534091, -0.481022736024}, {
      0.7454634716746243, -0.47804656216015057`}, {
      0.7499805025153351, -0.47504327988370726`}, {
      0.7544553177188432, -0.4720132575024551}, {
      0.7588879736219251, -0.4689568584163728}, {
      0.763278526424321, -0.46587444122333793`}, {
      0.7676270321595888, -0.46276635982238107`}, {
      0.7719335466672598, -0.45963296351455735`}, {
      0.7761981255662412, -0.4564745971015024}, {
      0.7804208242294086, -0.4532916009817366}, {
      0.7846016977593355, -0.4500843112447809}, {
      0.7887408009651111, -0.4468530597631445}, {
      0.7928381883401969, -0.443598174282244}, {
      0.7968939140412769, -0.4403199785083112}, {
      0.8009080318680569, -0.4370187921943451}, {
      0.8048805952439727, -0.43369493122416236`}, {
      0.8088116571977643, -0.4303487076945992}, {
      0.8127012703458806, -0.42698042999591485`}, {
      0.8165494868756763, -0.4235904028904469}, {
      0.820356358529367, -0.42017892758956654`}, {
      0.8241219365887078, -0.4167463018289799}, {
      0.8278462718603651, -0.4132928199424222}, {
      0.8315294146619493, -0.40981877293378716`}, {
      0.8351714148086805, -0.40632444854773625`}, {
      0.8387723216006582, -0.40281013133882815`}, {
      0.8423321838107085, -0.3992761027392096}, {
      0.8458510496727838, -0.395722641124907}, {
      0.8493289668708893, -0.39215002188075676`}, {
      0.8527659825285137, -0.38855851746401154`}, {
      0.856162143198541, -0.38494839746665915`}, {
      0.8595174948536227, -0.3813199286764884}, {
      0.8628320828769886, -0.37767337513693666`}, {
      0.8661059520536776, -0.3740089982057525}, {
      0.8693391465621687, -0.370327056612505}, {
      0.8725317099663947, -0.36662780651497195`}, {
      0.8756836852081211, -0.3629115015544373}, {
      0.8787951145996731, -0.3591783929099273}, {
      0.8818660398169951, -0.35542872935141495`}, {
      0.8848965018930282, -0.3516627572920202}, {
      0.8878865412113897, -0.347880720839234}, {
      0.8908361975003412, -0.3440828618451927}, {
      0.8937455098270322, -0.3402694199560284}, {
      0.8966145165920058, -0.33644063266032115`}, {
      0.8994432555239541, -0.33259673533667766`}, {
      0.9022317636747123, -0.32873796130046007`}, {
      0.9049800774144786, -0.32486454184968855`}, {
      0.907688232427251, -0.3209767063101405}, {
      0.9103562637064685, -0.31707468207966866`}, {
      0.9129842055508477, -0.31315869467175933`}, {
      0.9155720915604059, -0.3092289677583529}, {
      0.9181199546326598, -0.30528572321194586`}, {
      0.9206278269589925, -0.3013291811469957}, {
      0.9230957400211787, -0.2973595599606473}, {
      0.9255237245880618, -0.29337707637280075`}, {
      0.9279118107123736, -0.28938194546553886`}, {
      0.9302600277276901, -0.28537438072193233`}, {
      0.9325684042455146, -0.281354594064241}, {
      0.9348369681524836, -0.2773227958915282}, {
      0.9370657466076868, -0.273279195116705}, {
      0.9392547660400955, -0.2692239992030215}, {
      0.9414040521460936, -0.26515741420002054`}, {
      0.9435136298871049, -0.2610796447789709}, {
      0.9455835234873111, -0.2569908942677939}, {
      0.9476137564314556, -0.25289136468550016`}, {
      0.9496043514627264, -0.24878125677614998`}, {
      0.951555330580715, -0.24466077004235298`}, {
      0.9534667150394456, -0.2405301027783203}, {
      0.9553385253454698, -0.23638945210248377`}, {
      0.9571707812560223, -0.23223901398969568`}, {
      0.9589635017772339, -0.2280789833030222}, {
      0.9607167051623973, -0.22390955382514374`}, {
      0.962430408910282, -0.2197309182893751}, {
      0.9641046297634943, -0.21554326841031787`}, {
      0.965739383706879, -0.21134679491415745`}, {
      0.9673346859659584, -0.2071416875686169}, {
      0.9688905510054077, -0.20292813521257938`}, {
      0.97040699252756, -0.198706325785391}, {
      0.971884023470941, -0.19447644635585534`}, {
      0.9733216560088288, -0.19023868315093131`}, {
      0.9747199015478359, -0.1859932215841449}, {
      0.9760787707265122, -0.18174024628372623`}, {
      0.9773982734139636, -0.1774799411204824}, {
      0.978678418708487, -0.1732124892354168}, {
      0.9799192149362167, -0.16893807306710515`}, {
      0.9811206696497805, -0.16465687437883883`}, {
      0.9822827896269648, -0.1603690742855453}, {
      0.983405580869384, -0.15607485328049583`}, {
      0.9844890486011543, -0.15177439126181025`}, {
      0.9855331972675687, -0.14746786755876856`}, {
      0.9865380305337725, -0.14315546095793907`}, {
      0.9875035512834358, -0.13883734972913225`}, {
      0.9884297616174226, -0.1345137116511901}, {
      0.9893166628524548, -0.1301847240376201}, {
      0.9901642555197689, -0.12585056376208298`}, {
      0.9909725393637644, -0.1215114072837435}, {
      0.991741513340642, -0.1171674306724932}, {
      0.9924711756170308, -0.11281880963405407`}, {
      0.993161523568603, -0.10846571953497212`}, {
      0.9938125537786743, -0.10410833542750969`}, {
      0.9944242620367898, -0.09974683207444501}, {
      0.9949966433372937, -0.09538138397378816}, {
      0.9955296918778812, -0.09101216538342169}, {
      0.9960234010581335, -0.08663935034567476}, {
      0.9964777634780322, -0.08226311271183931}, {
      0.9968927709364551, -0.07788362616663687}, {
      0.9972684144296499, -0.0735010642526444}, {
      0.9976046841496872, -0.06911560039468785}, {
      0.9979015694828907, -0.06472740792421186}, {
      0.9981590590082445, -0.06033666010363399}, {
      0.9983771404957772, -0.0559435301506921}, {
      0.9985558009049205, -0.05154819126279332}, {
      0.9986950263828445, -0.04715081664137298}, {
      0.9987948022627673, -0.04275157951627219}, {
      0.9988551130622388, -0.0383506531701424}, {
      0.9988759424813985, -0.033948210962885556`}, {
      0.9988572734012073, -0.02954442635613844}, {
      0.9987990878816524, -0.025139472937809687`}, {
      0.9987013671599259, -0.02073352444667819}, {
      0.9985640916485758, -0.016326754797061516`}, {
      0.9983872409336307, -0.01191933810356309}, {
      0.9981707937726966, -0.007511448705906873}, {
      0.9979147280930268, -0.0031032611938683983`}, {0.9976190209895658, 
      0.0013050495676889488`}, {0.9972836487229643, 0.005713308413663442}, {
      0.9969085867175685, 0.01012133985344127}, {0.9964938095593835, 
      0.01452896804549041}, {0.9960392909940082, 0.018936016771812905`}, {
      0.9955450039245467, 0.023342309412246257`}, {0.9950109204094918, 
      0.027747668918604466`}, {0.9944370116605851, 0.032151917788649294`}, {
      0.9938232480406513, 0.03655487803988203}, {0.9931695990614086, 
      0.04095637118314607}, {0.9924760333812561, 0.045356218196030446`}, {
      0.9917425188030378, 0.04975423949606418}, {0.9909690222717848, 
      0.054150254913691534`}, {0.9901555098724368, 0.05854408366501763}, {
      0.9893019468275424, 0.06293554432431415}, {0.988408297494941, 
      0.06732445479627451}, {0.9874745253654265, 0.07171063228800752}, {
      0.9865005930603932, 0.07609389328075894}, {0.9854864623294669, 
      0.0804740535013495}, {0.984432094048121, 0.08485092789331816}, {
      0.9833374482152798, 0.08922433058775914}, {0.9822024839509098, 
      0.09359407487384086}, {0.9810271594936022, 0.097959973168995}, {
      0.9798114321981461, 0.10232183698876336`}, {0.9785552585330961, 
      0.1066794769162902}, {0.9772585940783348, 0.11103270257144743`}, {
      0.975921393522634, 0.1153813225795796}, {0.9745436106612142, 
      0.11972514453985576`}, {0.9731251983933081, 0.12406397499321453`}, {
      0.9716661087197267, 0.12839761938988872`}, {0.9701662927404335, 
      0.1327258820564957}, {0.9686257006521289, 0.13704856616267871`}, {
      0.967044281745846, 0.14136547368728516`}, {0.965421984404563, 
      0.1456764053840663}, {0.963758756100834, 0.1499811607468833}, {
      0.9620545433944421, 0.15427953797440427`}, {0.9603092919300779, 
      0.15857133393427555`}, {0.9585229464350475, 0.1628563441267518}, {
      0.9566954507170139, 0.16713436264776718`}, {0.9548267476617746, 
      0.17140518215143127`}, {0.9529167792310824, 0.17566859381193145`}, {
      0.9509654864605112, 0.17992438728482416`}, {0.9489728094573723, 
      0.1841723506676963}, {0.9469386873986877, 0.1884122704601781}, {
      0.9448630585292233, 0.19264393152328774`}, {0.9427458601595887, 
      0.19686711703808785`}, {0.9405870286644102, 0.20108160846363382`}, {
      0.9383864994805807, 0.20528718549419253`}, {0.9361442071055949, 
      0.2094836260157101}, {0.933860085095976, 0.21367070606150682`}, {
      0.9315340660658001, 0.21784819976717615`}, {0.929166081685327, 
      0.22201587932466516`}, {0.926756062679743, 0.2261735149355118}, {
      0.9243039388280261, 0.23032087476321503`}, {0.9218096389619409, 
      0.23445772488471225`}, {0.9192730909651716, 0.23858382924093832`}, {
      0.9166942217726035, 0.2426989495864393}, {0.914072957369762, 
      0.24680284543801395`}, {0.9114092227924195, 0.2508952740223543}, {
      0.9087029421263815, 0.2549759902226568}, {0.9059540385074619, 
      0.25904474652417436`}, {0.9031624341216612, 0.2631012929586778}, {
      0.9003280502055574, 0.2671453770477967}, {0.8974508070469255, 
      0.27117674374520584`}, {0.8945306239855969, 0.27519513537762486`}, {
      0.8915674194145745, 0.27920029158459614`}, {0.8885611107814174, 
      0.2831919492570063}, {0.8855116145899116, 0.2871698424743142}, {
      0.8824188464020429, 0.29113370244044884`}, {0.8792827208402898, 
      0.2950832574183379}, {0.8761031515902533, 0.29901823266302774`}, {
      0.8728800514036446, 0.30293835035335354`}, {0.8696133321016482, 
      0.30684332952211807`}, {0.8663029045786842, 0.3107328859847352}, {
      0.8629486788065888, 0.31460673226629365`}, {0.8595505638392382, 
      0.318464577526995}, {0.8561084678176399, 0.3223061274859184}, {
      0.8526222979755151, 0.32613108434306337`}, {0.8490919606454008, 
      0.3299391466996196}, {0.845517361265299, 0.3337300094764126}, {
      0.8418984043859012, 0.3375033638304713}, {0.8382349936784212, 
      0.34125889706966195`}, {0.8345270319430662, 0.3449962925653323}, {
      0.8307744211181814, 0.34871522966290647`}, {0.8269770622901047, 
      0.35241538359037006`}, {0.8231348557037659, 0.35609642536458397`}, {
      0.819247700774073, 0.35975802169536103`}, {0.8153154960981257, 
      0.36339983488724015`}, {0.8113381394682976, 0.3670215227388888}, {
      0.807315527886236, 0.3706227384400634}, {0.8032475575778245, 
      0.3742031304660546}, {0.7991341240091603, 0.37776234246954193`}, {
      0.7949751219035971, 0.3813000131697804}, {0.7907704452599118, 
      0.3848157762390386}, {0.7865199873716509, 0.3883092601862059}, {
      0.7822236408477191, 0.391780088237483}, {0.7778812976342749, 
      0.3952278782140683}, {0.7734928490379996, 0.3986522424067482}, {
      0.7690581857508126, 0.4020527874472984}, {0.7645771978761066, 
      0.4054291141765991}, {0.7600497749565825, 0.4087808175093636}, {
      0.7554758060037672, 0.4121074862953779}, {0.7508551795292989, 
      0.4154087031771442}, {0.7461877835780764, 0.4186840444438188}, {
      0.7414735057633642, 0.4219330798813309}, {0.7367122333039587, 
      0.42515537261856484`}, {0.7319038530635202, 0.42835047896948564`}, {
      0.7270482515921847, 0.43151794827108253`}, {0.7221453151705732, 
      0.4346573227170025}, {0.7171949298563254, 0.43776813718674046`}, {
      0.7121969815332858, 0.44084991907024945`}, {0.7071513559634854, 
      0.4439021880878298}, {0.7020579388420608, 0.4469244561051508}, {
      0.6969166158552674, 0.4499162269432558}, {0.6917272727417485, 
      0.452876996183394}, {0.6864897953572291, 0.4558062509665209}, {
      0.6812040697428183, 0.4587034697873012}, {0.675869982197107, 
      0.4615681222824453}, {0.6704874193522629, 0.4643996690132035}, {
      0.6650562682543338, 0.46719756124183814`}, {0.6595764164479828, 
      0.4699612407018876}, {0.6540477520658889, 0.4726901393620309}, {
      0.6484701639230631, 0.4753836791833557}, {0.6428435416163407, 
      0.47804127186982726`}, {0.6371677756293268, 0.4806623186117494}, {
      0.6314427574430863, 0.4832462098220033}, {0.6256683796528876, 
      0.48579232486484303`}, {0.6198445360913237, 0.488300031777022}, {
      0.613971121958157, 0.49076868698101656`}, {0.6080480339572472, 
      0.49319763499010844`}, {0.6020751704409502, 0.495586208105081}, {
      0.5960524315623894, 0.4979337261022775}, {0.5899797194360308, 
      0.5002394959127642}, {0.5838569383070125, 0.502502811292335}, {
      0.5776839947297085, 0.5047229524820874}, {0.5714607977560336, 
      0.5068991858592952}, {0.5651872591340222, 0.5090307635782954}, {
      0.5588632935172497, 0.5111169232011049}, {0.5524888186856943, 
      0.5131568873174726}, {0.5460637557786726, 0.5151498631540711}, {
      0.5395880295405197, 0.5170950421725272}, {0.5330615685797226, 
      0.5189915996559848}, {0.5264843056422597, 0.5208386942838911}, {
      0.5198561778999378, 0.522635467694694}, {0.5131771272545715, 
      0.5243810440361384}, {0.5064471006588944, 0.5260745295028421}, {
      0.49966605045514445`, 0.5277150118608388}, {0.49283393473232434`, 
      0.5293015599587717}, {0.4859507177031943, 0.5308332232254236}, {
      0.4790163701021178, 0.5323090311532753}, {0.47203086960494756`, 
      0.5337279927677872}, {0.4649942012722101, 0.5350890960821053}, {
      0.45790635801692053`, 0.5363913075369057}, {0.4507673410984402, 
      0.537633571425097}, {0.44357716064387265`, 0.5388148093011222}, {
      0.43633583619858346`, 0.53993391937461}, {0.4290433973075227, 
      0.5409897758881539}, {0.4216998841291308, 0.5419812284790174}, {
      0.41430534808371244`, 0.5429071015245933}, {0.40685985253827806`, 
      0.543766193471479}, {0.399363473529968, 0.5445572761480689}, {
      0.3918163005303041, 0.5452790940606081}, {0.38421843725264354`, 
      0.5459303636727064}, {0.3765700025053518, 0.5465097726683654}, {
      0.36887113109336, 0.547015979198645}, {0.36112197477092933`, 
      0.5474476111121658}, {0.3533227032486093, 0.547803265169736}, {
      0.345473505257553, 0.5480815062434846}, {0.33757458967453496`, 
      0.5482808665009974}, {0.3296261867112112, 0.5483998445750742}, {
      0.32162854917136335`, 0.5484369047198677}, {0.3135819537800834, 
      0.5483904759543209}, {0.30548670258907684`, 0.5482589511939968}, {
      0.29734312446249744`, 0.5480406863725926}, {0.2891515766479694, 
      0.5477339995546504}, {0.2809124464377076, 0.5473371700412248}, {
      0.2726261529249087, 0.5468484374705427}, {0.264293148860863, 
      0.5462660009159962}, {0.25591392261851426`, 0.545588017984155}, {
      0.2474890002684901, 0.5448126039158598}, {0.23901894777392116`, 
      0.5439378306938835}, {0.23050437331067028`, 0.5429617261611145}, {
      0.22194592971990282`, 0.5418822731537347}, {0.21334431710023727`, 
      0.5406974086544402}, {0.2047002855470256, 0.539405022971387}, {
      0.19601463804661753`, 0.5380029589492497}, {0.18728823353376114`, 
      0.5364890112195506}, {0.17852199012057762`, 0.5348609254982784}, {
      0.1697168885058153, 0.533116397939749}, {0.1608739755733318, 
      0.5312530745567026}, {0.1519943681889645, 0.5292685507177622}, {
      0.14307925720511888`, 0.527160370734628}, {0.13412991168252347`, 
      0.5249260275527436}, {0.12514768333865417`, 0.5225629625606649}, {
      0.11613401123230625`, 0.5200685655349842}, {0.10709042669367388`, 
      0.5174401747394375}, {0.09801855850906367, 0.5146750771987458}, {
      0.08892013836900015, 0.5117705091698249}, {0.07979700658795166, 
      0.5087236568352544}, {0.07065111810318672, 0.5055316572463263}, {
      0.06148454875933117, 0.5021915995455982}, {0.05229950188399975, 
      0.4987005265016727}, {0.0430983151583805, 0.4950554363918937}, {
      0.03388346778481193, 0.4912532852718075}, {0.02465758795116066, 
      0.48729098967355594`}, {0.015423460589125255`, 0.4831654297788549}, {
      0.006184035420399511, 0.47887345311582746`}, {-0.0030575647191410163`, 
      0.4744118788326903}, {-0.012298035291482929`, 
      0.4697775026050898}, {-0.021533881224145252`, 
      0.46496710223770266`}, {-0.03076140785881833, 
      0.4599774440244885}, {-0.03997671171314723, 
      0.4548052899356308}, {-0.04917567109564024, 
      0.44944740570262426`}, {-0.05835393662254176, 
      0.4439005698760405}, {-0.06750692169550002, 
      0.4381615839330874}, {-0.07662979301005554, 
      0.4322272835139883}, {-0.08571746117743384, 
      0.4260945508672496}, {-0.09476457155588539, 
      0.41976032858381773`}, {-0.1037654954028931, 
      0.41322163469867634`}, {-0.11271432147594132`, 
      0.40647557923529876`}, {-0.12160484822714783`, 
      0.3995193822632004}, {-0.13043057675578176`, 
      0.3923503935312577}, {-0.13918470470233923`, 
      0.3849661137290666}, {-0.1478601212881535, 
      0.37736421741496917`}, {-0.15644940372511315`, 
      0.3695425776320494}, {-0.1649448152404692, 
      0.3614992922119317}, {-0.17333830498132924`, 
      0.3532327117402036}, {-0.18162151008151572`, 
      0.34474146912634024`}, {-0.18978576018912133`, 
      0.3360245106848457}, {-0.19782208476527877`, 
      0.3270811285927547}, {-0.2057212234721891, 
      0.3179109945416405}, {-0.21347363996999133`, 
      0.3085141943500279}, {-0.2210695394361726, 
      0.2988912632450587}, {-0.22849889010640612`, 
      0.28904322146115535`}, {-0.23575144911044507`, 
      0.27897160973938845`}, {-0.24281679283954608`, 
      0.2686785242458072}, {-0.24968435203155354`, 
      0.25816665036209413`}, {-0.25634345169523953`, 
      0.24743929473997353`}, {-0.2627833559161587, 
      0.23650041495467383`}, {-0.2689933174920951, 
      0.22535464604564928`}, {-0.27496263223778605`, 
      0.21400732319821195`}, {-0.280680697677469, 
      0.20246449980137554`}, {-0.2861370757123004, 
      0.1907329601186902}, {-0.29132155871121296`, 
      0.17882022583350582`}, {-0.2962242383326805, 
      0.16673455578075808`}, {-0.30083557624643287`, 
      0.15448493825601767`}, {-0.30514647579445103`, 
      0.14208107540005843`}, {-0.30914835351613557`, 
      0.1295333592930895}, {-0.3128332093700911, 
      0.11685283955498922`}, {-0.3161936944209881, 
      0.10405118243261384`}, {-0.3192231747301981, 
      0.09114062155708591}, {-0.3219157901979181, 
      0.07813390076590296}, {-0.32426650715521493`, 
      0.06504420959849526}, {-0.32627116359774216`, 
      0.0518851122804143}, {-0.32792650608744595`, 
      0.03867047120126513}, {-0.329230217520693, 
      0.025414366055735334`}, {-0.33018093516497504`, 
      0.01213100994749759}, {-0.33077825859378673`, -0.001165336154208928}, \
{-0.33102274739105625`, -0.014460449170420905`}, {-0.33091590874236704`, \
-0.027740228447820786`}, {-0.3304601752696627, -0.04099077768944474}, \
{-0.3296588736891808, -0.054198482989067515`}, {-0.3285161850701425, \
-0.06735008580881313}, {-0.3270370976370268, -0.08043274990245271}, \
{-0.3252273531859181, -0.09343412137317973}, {-0.32309338827252093`, \
-0.1063423812554179}, {-0.32064227137533813`, -0.11914629021635036`}, \
{-0.3178816372436738, -0.13183522517578206`}, {-0.3148196196098388, \
-0.14439920783503732`}, {-0.311464783382911, -0.15682892528054626`}, \
{-0.30782605735333307`, -0.16911574298080853`}, {-0.3039126683297297, \
-0.18125171062337314`}, {-0.29973407750791736`, -0.1932295613397387}, \
{-0.29529991974320985`, -0.20504270494048304`}, {-0.29061994626629295`, \
-0.2166852158315151}, {-0.28570397125489116`, -0.22815181630710096`}, \
{-0.2805618225520414, -0.23943785591893824`}, {-0.2752032967099737, \
-0.25053928760615596`}, {-0.2696381184384296, -0.2614526412420124}, \
{-0.26387590444895076`, -0.2721749952125558}, {-0.2579261316127578, \
-0.28270394659374964`}, {-0.2517981092892147, -0.29303758043941086`}, \
{-0.24550095563397872`, -0.3031744386352869}, {-0.2390435776598571, \
-0.3131134887168372}, {-0.23243465479798917`, -0.3228540929915188}, \
{-0.2256826256909663, -0.33239597825195705`}, {-0.2187956779415638, \
-0.3417392063153063}, {-0.21178174053957247`, -0.3508841455770544}, \
{-0.20464847869352878`, -0.3598314437249084}, {-0.19740329080278884`, \
-0.368582001720409}, {-0.19005330731733408`, -0.3771369491225603}, \
{-0.18260539124701491`, -0.38549762079890765`}, {-0.1750661400978479, \
-0.3936655350449134}, {-0.16744188902982024`, -0.40164237311187245`}, \
{-0.15973871504788095`, -0.4094299601266344}, {-0.15196244205497467`, \
-0.4170302473726917}, {-0.14411864661276444`, -0.4244452958913868}, \
{-0.13621266427183812`, -0.43167726135372486`}, {-0.12824959634851585`, \
-0.43872838014720555`}, {-0.12023431703974949`, -0.445600956617894}, \
{-0.11217148078095462`, -0.45229735140533284`}, {-0.10406552976390492`, \
-0.4588199708065974}, {-0.0959207015430453, -0.46517125710557294`}, \
{-0.08774103666876128, -0.4713536798041784}, {-0.07953038629531603, \
-0.47736972769359615`}, {-0.07129241971938291, -0.4832219017054322}, \
{-0.06303063181241714, -0.48891270848499685`}, {-0.05474835031658956, \
-0.494444654631445}, {-0.04644874297971259, -0.49982024155226057`}, \
{-0.038134824509589466`, -0.5050419608824268}, {-0.029809463332577453`, \
-0.5101122904215369}, {-0.02147538814493493, -0.515033690545012}, \
{-0.013135194248781139`, -0.5198086010484702}, {-0.004791349667290648, \
-0.5244394383870957}, {0.003553798963876203, -0.5289285932745745}, {
      0.011898020729893032`, -0.5332784286087634}, {
      0.02023919499413421, -0.5374912776937489}, {
      0.028575306177480213`, -0.5415694427303052}, {
      0.03690443875629593, -0.5455151935489815}, {
      0.04522477247487567, -0.5493307665621427}, {
      0.05353457776739512, -0.5530183639132374}, {
      0.061832211383819835`, -0.5565801528033942}, {
      0.07011611221376779, -0.5600182649771466}, {
      0.07838479730199141, -0.5633347963506581}, {
      0.08663685804891393, -0.5665318067672821}, {
      0.09487095658951025, -0.5696113198666352}, {
      0.10308582234375097`, -0.5725753230546082}, {
      0.11128024873181666`, -0.5754257675628818}, {
      0.11945309004732915`, -0.5781645685875643}, {
      0.12760325848192686`, -0.5807936054975384}, {
      0.13572972129462585`, -0.5833147221039837}, {
      0.14383149811954932`, -0.585729726983354}, {
      0.1519076584057714, -0.5880403938468317}, {
      0.15995731898319945`, -0.5902484619499532}, {
      0.16797964174861088`, -0.5923556365367221}, {
      0.1759738314661601, -0.5943635893130877}, {
      0.1839391336768768, -0.5962739589451801}, {
      0.19187483271188532`, -0.5980883515781593}, {
      0.199780249804285, -0.5998083413719655}, {
      0.20765474129484013`, -0.601435471050636}, {
      0.2154976969268351, -0.6029712524622146}, {
      0.22330853822565416`, -0.6044171671465889}, {
      0.2310867169588449, -0.6057746669088871}, {
      0.23883171367261863`, -0.6070451743963239}, {
      0.24654303630093066`, -0.608230083676622}, {
      0.25422021884346585`, -0.6093307608163548}, {
      0.2618628201090325, -0.6103485444577431}, {
      0.26947042252103787`, -0.6112847463926212}, {
      0.2770426309818824, -0.6121406521324423}, {
      0.2845790717932681, -0.6129175214733364}, {
      0.2920793916295655, -0.613616589055365}, {
      0.2995432565615309, -0.6142390649152298}, {
      0.3069703511278004, -0.6147861350318023}, {
      0.3143603774517207, -0.6152589618639288}, {
      0.3217130544012023, -0.6156586848800557}, {
      0.3290281167893994, -0.6159864210792921}, {
      0.3363053146141349, -0.6162432655035963}, {
      0.343544412334097, -0.6164302917408326}, {
      0.3507451881799381, -0.6165485524185013}, {
      0.3579074334985016, -0.6165990796879889}, {
      0.36503095212849773`, -0.6165828856992345}, {
      0.3721155598060358, -0.6165009630657418}, {
      0.3791610835985044, -0.6163542853199022}, {
      0.3861673613653706, -0.6161438073586256}, {
      0.393134241244542, -0.6158704658792994}, {
      0.4000615811630101, -0.6155351798061225}, {
      0.4069492483705578, -0.6151388507068795}, {
      0.41379711899537797`, -0.6146823632002385}, {
      0.4206050776205126, -0.6141665853536732}, {
      0.4273730168800756, -0.61359236907212}, {
      0.4341008370742801, -0.6129605504774966}, {
      0.4407884458023388, -0.6122719502792142}, {
      0.44743575761235754`, -0.6115273741358271}, {
      0.45404269366738575`, -0.6107276130079684}, {
      0.4606091814268315, -0.6098734435027268}, {
      0.4671351543424906, -0.6089656282096237}, {
      0.47362055156847754`, -0.6080049160283552}, {
      0.48006531768438215`, -0.6069920424884624}, {
      0.4864694024310116, -0.6059277300611009}, {
      0.4928327604581102, -0.6048126884630755}, {
      0.4991553510834797, -0.6036476149533121}, {
      0.5054371380629535, -0.6024331946219358}, {
      0.5116780893707049, -0.6011701006721241}, {
      0.5178781769893966, -0.5998589946949046}, {
      0.5240373767097039, -0.5985005269370643}, {
      0.5301556679387663, -0.5970953365623365}, {
      0.5362330335171475, -0.5956440519060283}, {
      0.5422694595438999, -0.5941472907232538}, {
      0.5482649352093574, -0.592605660430929}, {
      0.5542194526352896, -0.5910197583436886}, {
      0.5601330067220792, -0.5893901719038788}, {
      0.5660055950025914, -0.5877174789057779}, {
      0.5718372175024282, -0.5860022477141946}, {
      0.5776278766062708, -0.5842450374775907}, {
      0.5833775769300299, -0.5824463983358711}, {
      0.5890863251985373, -0.5806068716229826}, {
      0.594754130128525, -0.5787269900644588}, {
      0.60038100231665, -0.5768072779700454}, {
      0.605966954132336, -0.5748482514215377}, {
      0.6115119996152117, -0.57285041845596}, {
      0.61701615437694, -0.5708142792442094}, {
      0.6224794355072377, -0.5687403262652901}, {
      0.6279018614838991, -0.5666290444762556}, {
      0.6332834520866426, -0.5644809114779744}, {
      0.6386242283146096, -0.562296397676836}, {
      0.6439242123073535, -0.560075966442505}, {
      0.6491834272691626, -0.5578200742618317}, {
      0.6544018973965693, -0.5555291708890258}, {
      0.6595796478089057, -0.5532036994921934}, {
      0.6647167044817689, -0.5508440967963384}, {
      0.6698130941832704, -0.5484507932229241}, {
      0.6748688444129459, -0.5460242130260906}, {
      0.6798839833432095, -0.5435647744256181}, {
      0.6848585397632413, -0.5410728897367271}, {
      0.689792543025202, -0.5385489654968004}, {
      0.6946860229926738, -0.5359934025891141}, {
      0.6995390099912306, -0.5334065963636561}, {
      0.7043515347610461, -0.5307889367551153}, {
      0.7091236284114504, -0.5281408083981173}, {
      0.7138553223773536, -0.5254625907397817}, {
      0.7185466483774532, -0.5227546581496768}, {
      0.723197638374151, -0.5200173800272393}, {
      0.7278083245351051, -0.5172511209067319}, {
      0.7323787391963484, -0.514456240559804}, {
      0.7369089148269047, -0.5116330940957216}, {
      0.7413988839948407, -0.50878203205933}, {
      0.7458486793346918, -0.5059034005268118}, {
      0.7502583335162034, -0.5029975411992985}, {
      0.7546278792143322, -0.5000647914943964}, {
      0.758957349080454, -0.49710548463568216`}, {
      0.7632467757147273, -0.49411994974022344`}, {
      0.7674961916395634, -0.49110851190417865`}, {
      0.7717056292741564, -0.4880714922865273}, {
      0.7758751209100294, -0.48500920819098237`}, {
      0.7800046986875525, -0.4819219731461334}, {
      0.784094394573393, -0.4788100969838683}, {
      0.7881442403388583, -0.4756738859161206}, {
      0.7921542675390935, -0.47251364260998735`}, {
      0.7961245074930985, -0.46932966626126127`}, {
      0.80005499126453, -0.46612225266642054`}, {
      0.8039457496432548, -0.4628916942931173}, {
      0.8077968131276246, -0.4596382803492054}, {
      0.8116082119074394, -0.45636229685034724`}, {
      0.815379975847573, -0.4530640266862369}, {
      0.8191121344722319, -0.44974374968547776`}, {
      0.8228047169498202, -0.44640174267915017`}, {
      0.8264577520783867, -0.4430382795631048}, {
      0.8300712682716281, -0.4396536313590155}, {
      0.8336452935454259, -0.4362480662742248}, {
      0.8371798555048938, -0.4328218497604157}, {
      0.8406749813319149, -0.42937524457113946`}, {
      0.8441306977731468, -0.42590851081823144`}, {
      0.847547031128476, -0.4224219060271447}, {
      0.8509240072399019, -0.4189156851912295}, {
      0.854261651480832, -0.4153901008249885}, {
      0.8575599887457707, -0.4118454030163339}, {
      0.8608190434403856, -0.408281839477874}, {
      0.8640388394719345, -0.40469965559725557`}, {
      0.8672194002400373, -0.40109909448658687`}, {
      0.8703607486277788, -0.3974803970309665}, {
      0.873462906993127, -0.3938438019361423}, {
      0.8765258971606543, -0.390189545775324}, {
      0.879549740413547, -0.3865178630351719}, {
      0.8825344574858921, -0.3828289861609848}, {
      0.885480068555227, -0.379123145601109}, {
      0.888386593235343, -0.37540056985058834`}, {
      0.891254050569329, -0.37166148549407824`}, {
      0.8940824590228459, -0.36790611724804195`}, {
      0.8968718364776204, -0.36413468800225013`}, {
      0.8996222002251492, -0.3603474188606023}, {
      0.9023335669606033, -0.35654452918128926`}, {
      0.9050059527769231, -0.35272623661631525`}, {
      0.9076393731590953, -0.3488927571503968}, {
      0.9102338429786035, -0.34504430513925677`}, {
      0.9127893764880444, -0.3411810933473303}, {
      0.9153059873159, -0.3373033329848997}, {
      0.9177836884614605, -0.3334112337446735}, {
      0.9202224922898892, -0.32950500383782805`}, {
      0.9226224105274219, -0.32558485002952414`}, {
      0.9249834542566955, -0.3216509776739168}, {
      0.9273056339121973, -0.31770359074867166`}, {
      0.9295889592758307, -0.3137428918890025}, {
      0.9318334394725892, -0.3097690824212454}, {
      0.9340390829663349, -0.3057823623959824}, {
      0.9362058975556733, -0.3017829306207288}, {
      0.9383338903699221, -0.297770984692198}, {
      0.9404230678651656, -0.2937467210281562}, {
      0.9424734358203917, -0.2897103348988809}, {
      0.9444849993337058, -0.2856620204582349}, {
      0.9464577628186173, -0.2816019707743692}, {
      0.9483917300003937, -0.2775303778600662}, {
      0.9502869039124782, -0.27344743270273547`}, {
      0.9521432868929662, -0.2693533252940745}, {
      0.9539608805811378, -0.2652482446594048}, {
      0.9557396859140403, -0.2611323788866954}, {
      0.9574797031231196, -0.2570059151552856}, {
      0.9591809317308935, -0.2528690397643162}, {
      0.960843370547667, -0.24872193816088184`}, {
      0.9624670176682836, -0.244564794967914}, {
      0.9640518704689098, -0.24039779401180547`}, {
      0.965597925603851, -0.2362211183497868}, {
      0.9671051790023936, -0.23203495029706447`}, {
      0.9685736258656716, -0.22783947145373135`}, {
      0.9700032606635561, -0.22363486273145886`}, {
      0.9713940771315612, -0.219421304379981}, {
      0.9727460682677681, -0.2151989760133797}, {
      0.9740592263297612, -0.21096805663618093`}, {
      0.9753335428315761, -0.20672872466927136`}, {
      0.9765690085406562, -0.20248115797564453`}, {
      0.9777656134748148, -0.19822553388598593`}, {
      0.9789233468992025, -0.19396202922410616`}, {
      0.9800421973232752, -0.189690820332231}, {
      0.9811221524977637, -0.18541208309615775`}, {
      0.9821631994116391, -0.18112599297028628`}, {
      0.9831653242890759, -0.176832725002534}, {
      0.984128512586408, -0.1725324538591432}, {
      0.9850527489890778, -0.16822535384938983`}, {
      0.9859380174085752, -0.1639115989502019}, {
      0.9867843009793659, -0.1595913628306966}, {
      0.9875915820558065, -0.1552648188766443}, {
      0.9883598422090462, -0.15093214021486823`}, {
      0.9890890622239112, -0.14659349973758812`}, {
      0.9897792220957738, -0.1422490701267165}, {
      0.9904303010274011, -0.13789902387811592`}, {
      0.9910422774257853, -0.13354353332582555`}, {
      0.9916151288989515, -0.12918277066626568`}, {
      0.9921488322527443, -0.1248169079824285}, {
      0.9926433634875899, -0.12044611726806351`}, {
      0.9930986977952342, -0.11607057045186601`}, {
      0.9935148095554547, -0.11169043942167724`}, {
      0.9938916723327459, -0.10730589604870432`}, {
      0.9942292588729771, -0.10291711221176884`}, {
      0.9945275411000212, -0.09852425982159232}, {
      0.9947864901123541, -0.0941275108451271}, {
      0.9950060761796246, -0.0897270373299414}, {
      0.9951862687391915, -0.08532301142866694}, {
      0.995327036392631, -0.08091560542351796}, {
      0.99542834690221, -0.07650499175089022}, {
      0.995490167187327, -0.07209134302604885}, {
      0.9955124633209191, -0.06767483206791376}, {
      0.9954952005258354, -0.06325563192395174}, {
      0.9954383431711746, -0.058833915895183914`}, {
      0.9953418547685888, -0.05440985756131793}, {
      0.9952056979685506, -0.04998363080601383}, {
      0.9950298345565854, -0.04555540984229294}, {
      0.994814225449467, -0.04112536923809909}, {
      0.994558830691377, -0.036693683942021625`}, {
      0.994263609450028, -0.032260529309189816`}, {
      0.9939285200127501, -0.027826081127348277`}, {
      0.9935535197825405, -0.023390515643123284`}, {
      0.9931385652740766, -0.018954009588489872`}, {
      0.992683612109693, -0.014516740207449839`}, {
      0.9921886150153213, -0.010078885282930869`}, {
      0.9916535278163939, -0.00564062316391716}, {
      0.9910783034337123, -0.001202132792822103}, {0.9904628938792787, 
      0.0032364062668862777`}, {0.9898072502520933, 0.007674813802796226}, {
      0.9891113227339159, 0.012112908925388445`}, {0.9883750605849938, 
      0.016550510039586276`}, {0.9875984121397555, 0.02098743481600846}, {
      0.9867813248024718, 0.025423500161912745`}, {0.985923745042884, 
      0.02985852219181849}, {0.9850256183918014, 0.03429231619779615}, {
      0.9840868894366676, 0.03872469661941129}, {0.983107501817098, 
      0.043155477013310496`}, {0.9820873982203885, 0.04758447002243643}, {
      0.981026520376997, 0.05201148734485882}, {0.9799248090559997, 
      0.056436339702208016`}, {0.9787822040605224, 0.060858836807697554`}, {
      0.9775986442231496, 0.0652787873337216}, {0.9763740674013116, 
      0.06969599887901312}, {0.9751084104726535, 0.07411027793534819}, {
      0.9738016093303856, 0.07852142985378144}, {0.9724535988786193, 
      0.08292925881039749}, {0.9710643130276889, 0.08733356777156272}, {
      0.9696336846894638, 0.09173415845866134}, {0.9681616457726511, 
      0.0961308313122996}, {0.9666481271780935, 0.10052338545596108`}, {
      0.9650930587940634, 0.10491161865909628`}, {0.9634963694915583, 
      0.10929532729962863`}, {0.9618579871195981, 0.11367430632585906`}, {
      0.9601778385005305, 0.11804834921775072`}, {0.9584558494253453, 
      0.12241724794757466`}, {0.956691944649004, 0.12678079293989747`}, {
      0.9548860478857861, 0.13113877303089053`}, {0.9530380818046591, 
      0.13549097542694083`}, {0.9511479680246739, 0.1398371856625421}, {
      0.9492156271103931, 0.14417718755744488`}, {0.9472409785673542, 
      0.1485107631730433}, {0.945223940837576, 0.15283769276797612`}, {
      0.9431644312951113, 0.1571577547529186}, {0.9410623662416538, 
      0.16147072564454093`}, {0.9389176609022035, 0.16577638001860928`}, {
      0.9367302294207985, 0.17007449046220363`}, {0.9344999848563201, 
      0.17436482752502666`}, {0.9322268391783773, 0.1786471596697768}, {
      0.9299107032632797, 0.18292125322155822`}, {0.9275514868901061, 
      0.1871868723162993}, {0.9251490987368775, 0.19144377884815053`}, {
      0.9227034463768434, 0.1956917324158321}, {0.9202144362748917, 
      0.19993049026790016`}, {0.9176819737840903, 0.2041598072469004}, {
      0.9151059631423725, 0.208379435732376}, {0.9124863074693766, 
      0.21258912558269688`}, {0.9098229087634521, 0.21678862407567517`}, {
      0.9071156678988421, 0.22097767584793174`}, {0.9043644846230591, 
      0.22515602283297703`}, {0.9015692575544633, 0.22932340419796843`}, {
      0.8987298841800608, 0.23347955627910535`}, {0.895846260853535, 
      0.23762421251562205`}, {0.8929182827935277, 0.24175710338233675`}, {
      0.8899458440821858, 0.24587795632071485`}, {0.8869288376639923, 
      0.249986495668402}, {0.8838671553448987, 0.2540824425871822}, {
      0.8807606877917784, 0.2581655149893143}, {0.8776093245322224, 
      0.2622354274621985}, {0.8744129539546969, 0.26629189119132435`}, {
      0.8711714633090872, 0.2703346138814474}, {0.8678847387076507, 
      0.2743632996759442}, {0.864552665126403, 0.2783776490742888}, {
      0.8611751264069658, 0.28237735884759707`}, {0.857752005258901, 
      0.28636212195217914`}, {0.8542831832625631, 0.29033162744104124`}, {
      0.8507685408724982, 0.29428556037327547`}, {0.8472079574214227, 
      0.2982236017212732}, {0.8436013111248161, 0.3021454282756975}, {
      0.8399484790861618, 0.3060507125481461}, {0.8362493373028752, 
      0.3099391226714351}, {0.8325037606729578, 0.3138103222974322}, {
      0.8287116230024185, 0.31766397049236356`}, {0.824872797013506, 
      0.3214997216295183}, {0.8209871543537992, 0.32531722527927115`}, {
      0.8170545656062028, 0.329116126096341}, {0.8130749002999, 
      0.3328960637042007}, {0.8090480269223157, 0.33665667257655085`}, {
      0.8049738129321465, 0.34039758191576674`}, {0.8008521247735186, 
      0.34411841552822603`}, {0.796682827891334, 0.3478187916964196}, {
      0.7924657867478734, 0.3514983230477471}, {0.7882008648407239, 
      0.35515661641989343`}, {0.7838879247221068, 0.35879327272268063`}, {
      0.7795268280196801, 0.3624078867962849}, {0.7751174354589007, 
      0.36600004726570545`}, {0.770659606887029, 0.36956933639136824`}, {
      0.7661532012988691, 0.37311532991574337`}, {0.761598076864339, 
      0.3766375969058508}, {0.756994090957971, 0.38013569959152577`}, {
      0.7523411001904499, 0.38360919319931014`}, {0.7476389604423005, 
      0.3870576257818315}, {0.7428875268998419, 0.3904805380425282}, {
      0.7380866540935346, 0.3938774631555725}, {0.7332361959388507, 
      0.39724792658084046`}, {0.728336005779807, 0.4005914458737712}, {
      0.7233859364353068, 0.4039075304899534}, {0.718385840248446, 
      0.4071956815842718}, {0.7133355691389449, 0.4104553918044406}, {
      0.7082349746588807, 0.4136861450787457}, {0.7030839080519, 
      0.4168874163978107}, {0.6978822203161054, 0.4200586715901973}, {
      0.6926297622708183, 0.42319936709164313`}, {0.6873263846274333, 
      0.42630894970773525`}, {0.6819719380645889, 0.4293868563698092}, {
      0.6765662733078958, 0.4324325138838594}, {0.6711092412144756, 
      0.43544533867223767`}, {0.6656006928625762, 0.4384247365079114}, {
      0.6600404796465479, 0.44137010224104506`}, {0.6544284533774788, 
      0.4442808195176617}, {0.6487644663898052, 0.4471562604901343}, {
      0.6430483716542326, 0.4499957855192491}, {0.6372800228973193, 
      0.4527987428675752}, {0.6314592747280995, 0.455564468383868}, {
      0.6255859827721383, 0.45829228517822584`}, {0.6196600038134418, 
      0.46098150328771165`}, {0.6136811959446624, 0.4636314193321435}, {
      0.6076494187260716, 0.46624131615975095`}, {0.6015645333537953, 
      0.4688104624823852}, {0.5954264028378407, 0.47133811249996493`}, {
      0.5892348921904693, 0.4738235055138306}, {0.5829898686255107, 
      0.47626586552867434`}, {0.5766912017692388, 0.47866440084270356`}, {
      0.5703387638834765, 0.4810183036256915}, {0.5639324301016301, 
      0.4833267494845592}, {0.5574720786783991, 0.48558889701612995`}, {
      0.5509575912539494, 0.4878038873466883}, {0.5443888531333877, 
      0.489970843657974}, {0.5377657535824241, 0.49208887069923457`}, {
      0.5310881861401627, 0.4941570542849573}, {0.52435604895002, 
      0.4961744607778997}, {0.5175692451098253, 0.4981401365570341}, {
      0.5107276830422295, 0.5000531074700248}, {0.5038312768866099, 
      0.5019123782698538}, {0.49687994691373416`, 0.503716932035217}, {
      0.48987361996452405`, 0.5054657295743169}, {0.4828122299143396, 
      0.5071577088116844}, {0.47569571816429207`, 0.5087917841576711}, {
      0.4685240341611863, 0.5103668458602683}, {0.46129713594779015`, 
      0.5118817593389223}, {0.45401499074523316`, 0.5133353645000356}, {
      0.44667757556944676`, 0.5147264750338658}, {0.4392848778836744, 
      0.5160538776925646}, {0.43183689628920496`, 0.5173163315491273}, {
      0.4243336412566136, 0.5185125672370666}, {0.41677513589993276`, 
      0.519641286170668}, {0.40916141679632523`, 0.5207011597457327}, {
      0.40149253485398495`, 0.5216908285207817}, {0.3937685562311584, 
      0.5226089013787562}, {0.3859895633093523, 0.5234539546693335}, {
      0.37815565572397863`, 0.5242245313320689}, {0.3702669514558824, 
      0.5249191400006709}, {0.36232358798740244`, 0.5255362540888426}, {
      0.35432572352683184`, 0.5260743108582485}, {0.3462735383053719, 
      0.5265317104693166}, {0.33816723595091236`, 0.5269068150157568}, {
      0.3300070449432213, 0.5271979475438642}, {0.32179322015538997`, 
      0.5274033910578909}, {0.3135260444866529, 0.5275213875130085}, {
      0.3052058305919885, 0.5275501367976521}, {0.29683292271420453`, 
      0.5274877957073384}, {0.2884076986245186, 0.5273324769123849}, {
      0.2799305716779664, 0.527082247922332}, {0.27140199299029655`, 
      0.5267351300502883}, {0.2628224537433481, 0.5262890973808844}, {
      0.2541924876262527, 0.5257420757460387}, {0.2455126734201495, 
      0.5250919417133112}, {0.23678363773445488`, 0.5243365215922626}, {
      0.2280060579030773, 0.5234735904649405}, {0.2191806650493164, 
      0.5225008712473977}, {0.2103082473285214, 0.521416033790015}, {
      0.2013896533579083, 0.5202166940253526}, {0.1924257958432381, 
      0.518900413173313}, {0.183417655412333, 0.5174646970145541}, {
      0.17436628466564447`, 0.5159069952443709}, {0.16527281245427705`, 
      0.5142247009206637}, {0.15613844839600044`, 0.5124151500211463}, {
      0.14696448763983652`, 0.5104756211266269}, {0.13775231588977058`, 
      0.5084033352490297}, {0.1285034146979871, 0.5061954558248218}, {
      0.11921936703774877`, 0.5038490888966809}, {0.10990186316559561`, 
      0.5013612835085927}, {0.10055270678191167`, 0.49872903234211075`}, {
      0.09117382149805581, 0.4959492726242492}, {0.081767257617142, 
      0.4930188873404193}, {0.07233519923414286, 0.48993470678896306`}, {
      0.06287997165922839, 0.4866935105171783}, {0.053404049166088846`, 
      0.4832920296822668}, {0.04391006306436633, 0.47972694988435444`}, {
      0.03440081009217061, 0.47599491452261306`}, {0.024879261120910538`, 
      0.47209252872953184`}, {0.015348570160256661`, 0.4680163639424981}, {
      0.005812083645882808, 0.46376296317601184`}, {-0.0037266500133713645`, 
      0.45932884706200033`}, {-0.013263870659208022`, 
      0.45471052072974755`}, {-0.02279559641669085, 
      0.4499044816007943}, {-0.03231761358347767, 
      0.4449072281776861}, {-0.04182546643302049, 
      0.4397152699084835}, {-0.051314447000814944`, 
      0.43432513821133684`}, {-0.06077958493564909, 
      0.4287333987449384}, {-0.07021563751205785, 
      0.42293666501106425`}, {-0.07961707991585573, 
      0.4169316133744144}, {-0.08897809593168349, 
      0.4107149995822318}, {-0.09829256917990484, 
      0.4042836768613699}, {-0.10755407506981016`, 
      0.3976346156631838}, {-0.11675587365673709`, 
      0.39076492511641314`}, {-0.12589090361212857`, 
      0.3836718762346492}, {-0.13495177753734097`, 
      0.3763529269075649}, {-0.14393077887369513`, 
      0.3688057486833612}, {-0.15281986068220402`, 
      0.36102825532340327`}, {-0.16161064658584146`, 
      0.35301863307837683`}, {-0.17029443418420684`, 
      0.34477537259817387`}, {-0.17886220126390517`, 
      0.3362973023449137}, {-0.18730461513665464`, 
      0.32758362332998264`}, {-0.1956120454396688, 
      0.31863394494192004`}, {-0.20377458072773724`, 
      0.30944832157283647`}, {-0.21178204917207175`, 
      0.3000272896876084}, {-0.21962404365581478`, 
      0.2903719049135107}, {-0.22728995151860776`, 
      0.28048377865981344`}, {-0.23476898915145378`, 
      0.27036511370922794`}, {-0.24205024157723815`, 
      0.2600187381584532}, {-0.24912270707108178`, 
      0.2494481370264019}, {-0.2559753467781629, 
      0.23865748079933322`}, {-0.2625971391754467, 
      0.22765165014572147`}, {-0.2689771390994565, 
      0.21643625601403113`}, {-0.2751045409273176, 
      0.20501765432737626`}, {-0.2809687453563318, 
      0.19340295451374295`}, {-0.28655942908284215`, 
      0.1816000211618991}, {-0.2918666165396031, 
      0.16961746817328358`}, {-0.29688075271849923`, 
      0.1574646448898659}, {-0.301592775988988, 
      0.14515161381658012`}, {-0.305994189728909, 
      0.13268911972221634`}, {-0.3100771315198536, 
      0.12008855009062996`}, {-0.3138344386298713, 
      0.10736188709912398`}, {-0.31725970851639684`, 
      0.09452165151565087}, {-0.32034735313467855`, 
      0.08158083912261463}, {-0.32309264593237547`, 
      0.06855285048328749}, {-0.32549176054776024`, 
      0.055451415057721776`}, {-0.327541800403153, 
      0.04229051083946018}, {-0.32924081859063, 
      0.02908428081428671}, {-0.3305878276755964, 
      0.015846947631293056`}, {-0.33158279928597223`, 
      0.0025927279203385393`}, {-0.33222665360018355`, \
-0.010664252313348582`}, {-0.3325212390854417, -0.02391003983650135}, \
{-0.33246930305904476`, -0.03713093258934369}, {-0.3320744538409803, \
-0.050313553184838676`}, {-0.3313411154289719, -0.06344491651139399}, \
{-0.33027447575240504`, -0.07651249062964015}, {-0.32888042964661995`, \
-0.08950425035250419}, {-0.32716551773345487`, -0.10240872310120083`}, \
{-0.3251368623992879, -0.11521502682955524`}, {-0.3228021020315443, \
-0.12791289999785102`}, {-0.32016932461344766`, -0.1404927237490826}, \
{-0.3172470016903484, -0.15294553659051716`}, {-0.3140439236153776, \
-0.1652630420088548}, {-0.3105691368636243, -0.1774376095465535}, \
{-0.30683188407840245`, -0.189462269939976}, {-0.30284154738577285`, \
-0.20133070496799252`}, {-0.29860759538888676`, -0.21303723268452562`}, \
{-0.2941395341356231, -0.22457678871288972`}, {-0.28944686224421845`, \
-0.23594490426669035`}, {-0.2845390302740925, -0.24713768153471818`}, \
{-0.2794254043440094, -0.2581517670288897}, {-0.2741152339275602, \
-0.2689843234478801}, {-0.26861762369660475`, -0.2796330005574206}, \
{-0.2629415092362242, -0.2900959055337111}, {-0.2570956364190345, \
-0.3003715731610756}, {-0.25108854420129567`, -0.31045893622052534`}, \
{-0.2449285505869088, -0.3203572963535957}, {-0.23862374149683943`, \
-0.33006629563664147`}, {-0.23218196227949847`, -0.3395858890553785}, \
{-0.22561081160094132`, -0.3489163180282462}, {-0.21891763746131038`, \
-0.35805808509032594`}, {-0.21210953509474567`, -0.36701192981710595`}, \
{-0.20519334652314497`, -0.3757788060392177}, {-0.19817566154891622`, \
-0.3843598603751752}, {-0.19106281998759156`, -0.3927564120888491}, \
{-0.18386091495734497`, -0.4009699342615843}, {-0.17657579705864998`, \
-0.40900203625518367`}, {-0.1692130792932023, -0.4168544474310837}, \
{-0.16177814258656395`, -0.4245290020826041}, {-0.15427614179357846`, \
-0.43202762553083485`}, {-0.14671201207933687`, -0.4393523213302307}, \
{-0.13909047558126242`, -0.44650515952703096`}, {-0.1314160482696897, \
-0.45348826591196334`}, {-0.12369304693513168`, -0.4603038122080955}, \
{-0.1159255962402673, -0.4669540071349684}, {-0.10811763578357664`, \
-0.47344108829110576`}, {-0.10027292712954076`, -0.47976731479849843`}, \
{-0.09239506076746237, -0.48593496065457364`}, {-0.08448746296730787, \
-0.49194630873937706`}, {-0.07655340250658121, -0.4978036454281235}, \
{-0.06859599724717473, -0.5035092557618347}, {-0.06061822054546279, \
-0.5090654191314197}, {-0.05262290748266724, -0.5144744054332132}, \
{-0.044612760905785, -0.519738471656629}, {-0.03659035727217946, \
-0.5248598588671808}, {-0.028558152293347103`, -0.5298407895506421}, \
{-0.02051848637542346, -0.5346834652865493}, {-0.01247358985573004, \
-0.539390064721577}, {-0.004425588036122436, -0.5439627418155297}, {
      0.0036234939848853384`, -0.5484036243347905}, {
      0.011671726678244282`, -0.5527148125700455}, {
      0.019717271644950792`, -0.5568983782569558}, {
      0.027758377366639872`, -0.5609563636801889}, {
      0.0357933751412334, -0.5648907809428416}, {
      0.04382067519829235, -0.5687036113847939}, {
      0.051838762988431486`, -0.5723968051349342}, {
      0.05984619564095269, -0.5759722807834953}, {
      0.06784159858372527, -0.5794319251619359}, {
      0.07582366231927672, -0.582777593218914}, {
      0.08379113935104551, -0.5860111079819158}, {
      0.09174284125377913, -0.589134260595043}, {
      0.09967763588212862, -0.592148810424325}, {
      0.10759444471158887`, -0.5950564852227131}, {
      0.11549224030605455`, -0.597858981347642}, {
      0.12337004390640259`, -0.6005579640247053}, {
      0.13122692313466613`, -0.6031550676516045}, {
      0.1390619898085313, -0.6056518961370848}, {
      0.14687439786106118`, -0.6080500232700787}, {
      0.15466334136073095`, -0.6103509931147462}, {
      0.1624280526270393, -0.6125563204275183}, {
      0.1701678004371453, -0.6146674910926414}, {
      0.17788188831916252`, -0.6166859625730676}, {
      0.1855696529279233, -0.6186131643738589}, {
      0.19323046249920536`, -0.6204504985155621}, {
      0.20086371537858827`, -0.6221993400152775}, {
      0.20846883862127838`, -0.6238610373733849}, {
      0.21604528665940786`, -0.625436913064106}, {
      0.2235925400334754, -0.6269282640282858}, {
      0.231110104184752, -0.6283363621669504}, {
      0.23859750830562743`, -0.629662454834364}, {
      0.24605430424501734`, -0.6309077653294556}, {
      0.25348006546609175`, -0.6320734933846189}, {
      0.26087438605371926`, -0.6331608156510101}, {
      0.2682368797691501, -0.6341708861795774}, {
      0.2755671791495847, -0.6351048368971555}, {
      0.28286493465039086`, -0.6359637780770471}, {
      0.29012981382784686`, -0.636748798803593}, {
      0.2973615005603922, -0.6374609674303067}, {
      0.30455969430647284`, -0.6381013320312126}, {
      0.31172410939716283`, -0.6386709208450881}, {
      0.318854474361837, -0.6391707427123593}, {
      0.32595053128525886`, -0.6396017875044497}, {
      0.3330120351945294, -0.6399650265454239}, {
      0.3400387534744236, -0.6402614130258033}, {
      0.3470304653097161, -0.6404918824084691}, {
      0.3539869611531691, -0.6406573528265932}, {
      0.3609080422179242, -0.6407587254735677}, {
      0.3677935199931038, -0.6407968849849275}, {
      0.3746432157814884, -0.640772699812279}, {
      0.38145696025819614`, -0.6406870225892696}, {
      0.38823459304934305`, -0.6405406904896462}, {
      0.39497596232971705`, -0.6403345255774692}, {
      0.40168092443854697`, -0.6400693351495548}, {
      0.40834934351249474`, -0.6397459120702376}, {
      0.41498109113504383`, -0.6393650350985459}, {
      0.42157604600149834`, -0.6389274692078986}, {
      0.42813409359884746`, -0.6384339658984315}, {
      0.43465512589978805`, -0.637885263502075}, {
      0.4411390410702324, -0.6372820874805014}, {
      0.4475857431896644, -0.6366251507160713}, {
      0.4539951419837371, -0.6359151537959062}, {
      0.4603671525685371, -0.6351527852892209}, {
      0.46670169520596805`, -0.6343387220180484}, {
      0.47299869506973463`, -0.633473629321493}, {
      0.47925808202143316`, -0.6325581613136463}, {
      0.48547979039627964`, -0.6315929611353037}, {
      0.49166375879802987`, -0.6305786611996158}, {
      0.4978099299026673, -0.6295158834318113}, {
      0.5039182502704561, -0.6284052395031257}, {
      0.5099886701659762, -0.6272473310590699}, {
      0.5160211433857752, -0.6260427499421723}, {
      0.522015627093293, -0.6247920784093226}, {
      0.5279720816607251, -0.6234958893438505}, {
      0.5338904705175155, -0.6221547464624649}, {
      0.5397707600051772, -0.6207692045171808}, {
      0.5456129192381579, -0.6193398094923557}, {
      0.5514169199704796, -0.6178670987969604}, {
      0.5571827364678945, -0.6163516014522019}, {
      0.5629103453853128, -0.6147938382746155}, {
      0.568599725649267, -0.6131943220547433}, {
      0.5742508583451917, -0.6115535577315097}, {
      0.5798637266093071, -0.6098720425624061}, {
      0.5854383155249013, -0.6081502662895925}, {
      0.5909746120228233, -0.6063887113020209}, {
      0.5964726047859985, -0.6045878527936859}, {
      0.6019322841577953, -0.6027481589181015}, {
      0.6073536420540735, -0.6008700909391046}, {
      0.6127366718787566, -0.5989541033780804}, {
      0.6180813684427758, -0.5970006441577046}, {
      0.6233877278862405, -0.595010154742294}, {
      0.6286557476036975, -0.592983070274855}, {
      0.6338854261723467, -0.5909198197109171}, {
      0.6390767632830876, -0.588820825949238}, {
      0.6442297596742765, -0.5866865059594609}, {
      0.6493444170680784, -0.5845172709068074}, {
      0.6544207381093059, -0.5823135262738828}, {
      0.6594587263066388, -0.580075671979672}, {
      0.6644583859761254, -0.5778041024958005}, {
      0.6694197221868684, -0.5754992069601321}, {
      0.6743427407088066, -0.5731613692877762}, {
      0.6792274479625022, -0.5707909682795719}, {
      0.6840738509708524, -0.5683883777281177}, {
      0.6888819573126439, -0.5659539665214108}, {
      0.6936517750778758, -0.5634880987441607}, {
      0.6983833128247758, -0.560991133776838}, {
      0.7030765795384419, -0.5584634263925198}, {
      0.7077315845910408, -0.5559053268515902}, {
      0.7123483377035008, -0.553317180994352}, {
      0.7169268489086372, -0.5506993303316057}, {
      0.7214671285156519, -0.5480521121332507}, {
      0.72596918707595, -0.54537585951496}, {
      0.7304330353502225, -0.5426709015229804}, {
      0.7348586842767401, -0.5399375632171067}, {
      0.7392461449408116, -0.5371761657518808}, {
      0.7435954285453591, -0.5343870264560597}, {
      0.7479065463825632, -0.5315704589104013}, {
      0.7521795098065378, -0.5287267730238099}, {
      0.7564143302069902, -0.5258562751078862}, {
      0.7606110189838285, -0.5229592679499249}, {
      0.7647695875226772, -0.5200360508843983}, {
      0.7688900471712651, -0.5170869198629691}, {
      0.7729724092166499, -0.514112167523068}, {
      0.7770166848632464, -0.5111120832550775}, {
      0.7810228852116252, -0.5080869532681553}, {
      0.7849910212380521, -0.5050370606547359}, {
      0.7889211037747371, -0.5019626854537433}, {
      0.7928131434907661, -0.498864104712551}, {
      0.7966671508736863, -0.4957415925477192}, {
      0.8004831362117203, -0.4925954202045451}, {
      0.8042611095765828, -0.4894258561154547}, {
      0.8080010808068764, -0.4862331659572681}, {
      0.8117030594920427, -0.48301761270736815`}, {
      0.8153670549568465, -0.4797794566988001}, {
      0.8189930762463717, -0.47651895567433245`}, {
      0.8225811321115085, -0.4732363648395043}, {
      0.8261312309949111, -0.46993193691468815`}, {
      0.8296433810174085, -0.46660592218619223`}, {
      0.8331175899648482, -0.46325856855642955`}, {
      0.8365538652753567, -0.45989012159317705`}, {
      0.8399522140269992, -0.45650082457794994`}, {
      0.8433126429258218, -0.4530909185535143}, {
      0.8466351582942618, -0.44966064237056114`}, {
      0.8499197660599098, -0.44621023273356414`}, {
      0.8531664717446091, -0.44273992424584313`}, {
      0.8563752804538796, -0.4392499494538542}, {
      0.8595461968666516, -0.43574053889072795`}, {
      0.8626792252252973, -0.4322119211190753}, {
      0.8657743693259469, -0.4286643227730813}, {
      0.8688316325090777, -0.42509796859990595`}, {
      0.8718510176503649, -0.42151308150041106`}, {
      0.8748325271517823, -0.41790988256923134`}, {
      0.8777761629329428, -0.41428859113420796`}, {
      0.8806819264226678, -0.41064942479520217`}, {
      0.8835498185507765, -0.40699259946230576`}, {
      0.8863798397400838, -0.40331832939346557`}, {
      0.8891719898985999, -0.3996268272315383}, {
      0.8919262684119204, -0.3959183040407917}, {
      0.8946426741357996, -0.39219296934286757`}, {
      0.8973212053888988, -0.3884510311522223}, {
      0.8999618599457005, -0.3846926960110597}, {
      0.9025646350295821, -0.380918169023771}, {
      0.9051295273060402, -0.377127653890896}, {
      0.9076565328760604, -0.3733213529426199}, {
      0.9101456472696229, -0.36949946717181986`}, {
      0.9125968654393398, -0.3656621962666733}, {
      0.9150101817542162, -0.3618097386428433}, {
      0.9173855899935284, -0.3579422914752527}, {
      0.9197230833408151, -0.3540600507294595}, {
      0.9220226543779737, -0.35016321119264726`}, {
      0.9242842950794571, -0.3462519665042413}, {
      0.9265079968065656, -0.3423265091861639}, {
      0.9286937503018279, -0.3383870306727397}, {
      0.9308415456834669, -0.33443372134026267`}, {
      0.9329513724399452, -0.3304667705362371}, {
      0.9350232194245848, -0.32648636660830216`}, {
      0.9370570748502575, -0.322492696932853}, {
      0.9390529262841405, -0.3184859479433674}, {
      0.9410107606425339, -0.31446630515844987`}, {
      0.9429305641857347, -0.31043395320960315`}, {
      0.9448123225129644, -0.30638907586873754`}, {
      0.9466560205573455, -0.3023318560754284}, {
      0.9484616425809236, -0.29826247596393163`}, {
      0.9502291721697307, -0.2941811168899671}, {
      0.9519585922288872, -0.29008795945728016`}, {
      0.9536498849777383, -0.2859831835439901}, {
      0.9553030319450212, -0.28186696832873587`}, {
      0.9569180139640612, -0.2777394923166282}, {
      0.9584948111679918, -0.27360093336501723`}, {
      0.9600334029849975, -0.2694514687090851}, {
      0.9615337681335742, -0.26529127498727245`}, {
      0.9629958846178067, -0.2611205282665482}, {
      0.964419729722659, -0.25693940406753074`}, {
      0.9658052800092743, -0.25274807738947064`}, {
      0.9671525113102841, -0.24854672273510237`}, {
      0.9684613987251213, -0.24433551413537452`}, {
      0.9697319166153365, -0.2401146251740672}, {
      0.9709640385999151, -0.23588422901230463`}, {
      0.9721577375505915, -0.23164449841297216`}, {
      0.9733129855871597, -0.22739560576504575`}, {
      0.9744297540727769, -0.22313772310784255`}, {
      0.9755080136092585, -0.21887102215520102`}, {
      0.9765477340323633, -0.21459567431959886`}, {
      0.977548884407064, -0.21031185073621736`}, {
      0.9785114330228049, -0.2060197222869602}, {
      0.9794353473887415, -0.20171945962443527`}, {
      0.9803205942289628, -0.19741123319590778`}, {
      0.9811671394776921, -0.19309521326723292`}, {
      0.9819749482744664, -0.1887715699467764}, {
      0.9827439849592923, -0.18444047320933138`}, {
      0.9834742130677755, -0.1801020929200397}, {
      0.984165595326224, -0.17575659885832637`}, {
      0.9848180936467228, -0.171404160741855}, {
      0.9854316691221778, -0.1670449482505132}, {
      0.9860062820213291, -0.162679131050436}, {
      0.9865418917837303, -0.1583068788180758}, {
      0.9870384570146945, -0.15392836126432763`}, {
      0.9874959354802039, -0.1495437481587177}, {
      0.9879142841017826, -0.14515320935366474`}, {
      0.9882934589513318, -0.14075691480882174`}, {
      0.9886334152459246, -0.13635503461550783`}, {
      0.9889341073425616, -0.13194773902123832`}, {
      0.9891954887328834, -0.12753519845436237`}, {
      0.9894175120378421, -0.12311758354881669`}, {
      0.9896001290023277, -0.1186950651690049}, {
      0.9897432904897508, -0.11426781443481125`}, {
      0.9898469464765792, -0.10983600274675817`}, {
      0.9899110460468282, -0.1053998018113169}, {
      0.9899355373865042, -0.10095938366638071`}, {
      0.9899203677779991, -0.09651492070691028}, {
      0.9898654835944369, -0.09206658571076096}, {
      0.989770830293971, -0.08761455186470166}, {
      0.9896363524140297, -0.08315899279063549}, {
      0.9894619935655133, -0.07870008257203216}, {
      0.9892476964269384, -0.07423799578058245}, {
      0.9889934027385309, -0.06977290750308515}, {
      0.9886990532962667, -0.06530499336857716}, {
      0.9883645879458607, -0.060834429575717455`}, {
      0.9879899455767013, -0.056361392920435835`}, {
      0.9875750641157336, -0.05188606082385773}, {
      0.9871198805212881, -0.047408611360516315`}, {
      0.9866243307768561, -0.042929223286863515`}, {
      0.9860883498848124, -0.03844807607009168}, {
      0.9855118718600834, -0.03396534991727795}, {
      0.9848948297237623, -0.029481225804863453`}, {
      0.9842371554966713, -0.024995885508479976`}, {
      0.9835387801928703, -0.020509511633136655`}, {
      0.9827996338131125, -0.016022287643779803`}, {
      0.9820196453382487, -0.011534397896239086`}, {
      0.9811987427225785, -0.00704602766857361}, {
      0.9803368528871508, -0.002557363192831773}, {0.9794339017130137, 
      0.0019314083127609915`}, {0.9784898140344142, 0.006420098611172091}, {
      0.9775045136319483, 0.010908518413474493`}, {0.9764779232256645, 
      0.01539647734528713}, {0.9754099644681183, 0.01988378391272752}, {
      0.9743005579373823, 0.02437024546786074}, {0.9731496231300114, 
      0.028855668173628495`}, {0.9719570784539644, 0.03333985696824171}, {
      0.9707228412214849, 0.037822615529019506`}, {0.969446827641942, 
      0.04230374623565723}, {0.9681289528146333, 0.04678305013290554}, {
      0.9667691307215516, 0.0512603268926422}, {0.9653672742201187, 
      0.055735374775317835`}, {0.9639232950358873, 0.06020799059075631}, {
      0.9624371037552144, 0.06467796965828981}, {0.9609086098179093, 
      0.06914510576620855}, {0.9593377215098572, 0.0736091911305039}, {
      0.9577243459556246, 0.07807001635288384}, {0.9560683891110469, 
      0.08252737037803846}, {0.9543697557558039, 0.08698104045013312}, {
      0.952628349485986, 0.09143081206850581}, {0.9508440727066557, 
      0.09587646894254513}, {0.9490168266244083, 0.10031779294572406`}, {
      0.947146511239937, 0.10475456406876452`}, {0.9452330253406069, 
      0.10918656037190669`}, {0.943276266493044, 0.11361355793625648`}, {
      0.9412761310357433, 0.1180353308141836}, {0.9392325140717036, 
      0.1224516509787422}, {0.9371453094610948, 0.12686228827208482`}, {
      0.9350144098139629, 0.13126701035284005`}, {0.932839706482983, 
      0.1356655826424228}, {0.9306210895562647, 0.1400577682702457}, {
      0.9283584478502193, 0.14444332801779938`}, {0.9260516689024977, 
      0.14882202026156705`}, {0.9237006389650071, 0.15319360091474}, {
      0.9213052429970161, 0.15755782336769755`}, {0.9188653646583588, 
      0.1619144384272152}, {0.9163808863027476, 0.16626319425436284`}, {
      0.9138516889712072, 0.17060383630105427`}, {0.9112776523856404, 
      0.17493610724520783`}, {0.9086586549425391, 0.17925974692447666`}, {
      0.9059945737068529, 0.18357449226850583`}, {0.9032852844060307, 
      0.18788007722967254`}, {0.9005306614242491, 0.19217623271226378`}, {
      0.8977305777968424, 0.1964626865000447}, {0.8948849052049532, 
      0.2007391631821693}, {0.8919935139704189, 0.20500538407738386`}, {
      0.8890562730509132, 0.20926106715647122`}, {0.8860730500353619, 
      0.2135059269628834}, {0.8830437111396546, 0.21773967453150736`}, {
      0.8799681212026724, 0.2219620173055076}, {0.8768461436826551, 
      0.2261726590511872}, {0.873677640653933, 0.23037129977080692`}, {
      0.8704624728040474, 0.23455763561330048`}, {0.8672004994312875, 
      0.2387313587828215}, {0.8638915784426722, 0.2428921574450559}, {
      0.8605355663524062, 0.24703971563123114`}, {0.8571323182808435, 
      0.25117371313975184`}, {0.8536816879539891, 0.2552938254353883}, {
      0.8501835277035775, 0.2593997235459427}, {0.8466376884677623, 
      0.26349107395631466`}, {0.8430440197924584, 0.26756753849988607`}, {
      0.8394023698333765, 0.27162877424714116`}, {0.8357125853587956, 
      0.2756744333914363}, {0.8319745117531174, 0.27970416313183044`}, {
      0.8281879930212543, 0.2837176055528837}, {0.8243528717938995, 
      0.28771439750133}, {0.8204689893337355, 0.2916941704595242}, {
      0.8165361855426385, 0.29565655041556393`}, {0.8125542989699384, 
      0.2996011577299787}, {0.8085231668217978, 0.3035276069988799}, {
      0.8044426249717809, 0.307435506913458}, {0.8003125079726787, 
      0.31132446011571163`}, {0.796132649069671, 0.3151940630502878}, {
      0.7919028802149003, 0.3190439058123096}, {0.7876230320835458, 
      0.32287357199106304`}, {0.7832929340914825, 0.3266826385094093}, {
      0.7789124144146227, 0.3304706754587869}, {0.7744813000100365, 
      0.33423724592965964`}, {0.7699994166389575, 0.3379819058372652}, {
      0.7654665888917846, 0.3417042037425115}, {0.7608826402151951, 
      0.34540368066786437`}, {0.7562473929414953, 0.34907986990806356`}, {
      0.751560668320338, 0.3527322968354996}, {0.7468222865529449, 
      0.35636047870007803`}, {0.7420320668289815, 0.3599639244233905}, {
      0.737189827366237, 0.36354213438700816`}, {0.7322953854532751, 
      0.3670946002147052}, {0.7273485574952268, 0.3706208045484134}, {
      0.7223491590629089, 0.37412022081770374`}, {0.7172970049454617, 
      0.3775923130025821}, {0.712191909206711, 0.38103653538938115`}, {
      0.7070336852454706, 0.384452332319521}, {0.7018221458600148, 
      0.3878391379309058}, {0.6965571033169643, 0.39119637589171463`}, {
      0.6912383694248423, 0.39452345912633685`}, {0.6858657556125727, 
      0.3978197895331955}, {0.680439073013209, 0.40108475769419244`}, {
      0.6749581325532, 0.4043177425755012}, {0.6694227450475136, 
      0.40751811121942527`}, {0.6638327213009644, 0.4106852184270301}, {
      0.6581878722161051, 0.41381840643124784`}, {0.6524880089080696, 
      0.4169170045601462}, {0.6467329428267726, 0.41998032889004183`}, {
      0.6409224858869006, 0.4230076818881303}, {0.6350564506061506, 
      0.42599835204429504`}, {0.6291346502522045, 0.42895161349174793`}, {
      0.6231568989989528, 0.43186672561614525`}, {0.6171230120925154, 
      0.4347429326528121}, {0.6110328060276393, 0.4375794632717}, {
      0.6048860987350878, 0.4403755301496922}, {0.5986827097806746, 
      0.44313032952986275`}, {0.5924224605766357, 0.4458430407672848}, {
      0.5861051746060738, 0.448512825860977}, {0.5797306776612561, 
      0.45113882897156593`}, {0.5732987980965938, 0.4537201759242369}, {
      0.5668093670971835, 0.456255973696536}, {0.5602622189638445, 
      0.4587453098905803}, {0.553657191415643, 0.46118725218922807`}, {
      0.5469941259109588, 0.46358084779575276`}, {0.5402728679882122, 
      0.46592512285656434`}, {0.5334932676274404, 0.4682190818665159}, {
      0.5266551796339867, 0.47046170705633333`}, {0.5197584640456434, 
      0.4726519577617061}, {0.5128029865646764, 0.4747887697735797}, {
      0.5057886190162438, 0.4768710546691948}, {0.49871523983482113`, 
      0.4788976991234255}, {0.4915827345803416, 0.4808675641999789}, {
      0.4843909964858718, 0.48277948462203224`}, {0.4771399270387537, 
      0.4846322680219009}, {0.46982943659726595`, 0.486424694169352}, {
      0.4624594450449878, 0.4881555141782047}, {0.45502988248518306`, 
      0.48982344969089087`}, {0.44754068997766816`, 0.49142719204068713`}, {
      0.43999182032078205`, 0.49296540139137474`}, {0.43238323888123864`, 
      0.49443670585413596`}, {0.4247149244748155, 0.4958397005815586}, {
      0.416986870301016, 0.4971729468386898}, {0.40919908493503604`, 
      0.4984349710511643}, {0.40135159338057164`, 0.49962426383052577`}, {
      0.3934444381872205, 0.5007392789769692}, {0.38547768063645993`, 
      0.5017784324598553}, {0.3774514020004242, 0.5027401013764891}, {
      0.36936570487796033`, 0.503622622889815}, {0.36122071461270666`, 
      0.5044242931458613}, {0.35301658079822174`, 0.5051433661719724}, {
      0.344753478875484, 0.5057780527570965}, {0.33643161182839215`, 
      0.5063265193156599}, {0.3280512119832169, 0.5067868867368478}, {
      0.31961254291829105`, 0.507157229221443}, {0.3111159014905715, 
      0.5074355731087392}, {0.3025616199860681, 0.5076198956964605}, {
      0.29395006840150456`, 0.5077081240570767}, {0.2852816568649582, 
      0.5076981338544199}, {0.27655683820361404`, 0.5075877481650799}, {
      0.26777611066716356`, 0.5073747363096952}, {0.25894002081577583`, 
      0.5070568126999646}, {0.2500491665819662, 0.5066316357079941}, {
      0.24110420051607878`, 0.5060968065654669}, {0.23210583322548237`, 
      0.5054498683010933}, {0.22305483701794218`, 0.5046883047258666}, {
      0.21395204975997353`, 0.5038095394768353}, {0.20479837896128963`, 
      0.5028109351314054}, {0.19559480609672103`, 0.5016897924056224}, {
      0.18634239117719167`, 0.5004433494514591}, {0.17704227758147437`, 
      0.49906878126986925`}, {0.16769569716049687`, 0.4975631992582582}, {
      0.1583039756259122, 0.49592365091309343`}, {0.14886853823445712`, 
      0.49414711971063385`}, {0.139390915779277, 0.49223052519120647`}, {
      0.12987275089886233`, 0.49017072327511735`}, {0.12031580471348699`, 
      0.48796450684115306`}, {0.11072196379802439`, 0.48560860660171873`}, {
      0.10109324749869647`, 0.4830996923119698}, {0.09143181559963776, 
      0.48043437435383035`}, {0.08173997634307205, 0.477609205739542}, {
      0.07202019480434489, 0.474620684583348}, {0.06227510161996053, 
      0.4714652570940662}, {0.05250750206306466, 0.46813932114561946`}, {
      0.04272038545641274, 0.4646392304870308}, {0.03291693490768151, 
      0.46096129965791094`}, {0.02310053734592757, 0.4571018096799929}, {
      0.01327479383097503, 0.45305701459973063`}, {0.003443530099426823, 
      0.4488231489612604}, {-0.006389192698256706, 
      0.4443964362930084}, {-0.016219067125678017`, 
      0.43977309869474907`}, {-0.026041528521343324`, 
      0.43494936761479946`}, {-0.03585174407362345, 
      0.4299214959090482}, {-0.04564460200578888, 
      0.42468577127440865`}, {-0.05541470098333548, 
      0.4192385311487574}, {-0.06515633987358024, 
      0.4135761791671318}, {-0.07486350800671543, 
      0.4076952032595356}, {-0.08452987610805025, 
      0.4015921954687209}, {-0.09414878809285339, 
      0.3952638735563201}, {-0.103713253937757, 
      0.3887071044522191}, {-0.11321594386570873`, 
      0.38191892958457985`}, {-0.12264918410445186`, 
      0.3748965921059524}, {-0.1320049545008273, 
      0.3676375660039732}, {-0.14127488829402007`, 
      0.36013958705280263`}, {-0.15045027436925226`, 
      0.3524006855233641}, {-0.15952206232821842`, 
      0.34441922052638857`}, {-0.16848087072246787`, 
      0.33619391581219316`}, {-0.1773169987995149, 
      0.3277238967952141}, {-0.18602044210713198`, 
      0.3190087285100346}, {-0.1945809122874111, 
      0.31004845413981114`}, {-0.20298786136711364`, 
      0.30084363368881345`}, {-0.21123051081299526`, 
      0.2913953822999128}, {-0.21929788556880295`, 
      0.2817054076474077}, {-0.22717885322342177`, 
      0.27177604576819037`}, {-0.2348621683765821, 
      0.261610294633005}, {-0.24233652216961346`, 
      0.25121184470791497`}, {-0.2495905968346989, 
      0.24058510571784764`}, {-0.25661312498858646`, 
      0.22973522880313224`}, {-0.26339295325838796`, 
      0.21866812326011298`}, {-0.269919109681631, 
      0.20739046708170858`}, {-0.2761808741748526, 
      0.19590971056607978`}, {-0.2821678512203935, 
      0.18423407234331307`}, {-0.28787004378609166`, 
      0.17237252728196303`}, {-0.29327792737414876`, 
      0.16033478587866662`}, {-0.2983825230005241, 
      0.1481312649024215}, {-0.3031754678413847, 
      0.13577304925631947`}, {-0.30764908225415777`, 
      0.12327184522763054`}, {-0.31179643189198486`, 
      0.1106399255146948}, {-0.3156113836844848, 
      0.09789006663744242}, {-0.3190886545551668, 
      0.08503547954814514}, {-0.3222238518847445, 
      0.07208973445066078}, {-0.32501350490575237`, 
      0.059066681000905584`}, {-0.32745508642082016`, 
      0.045980365190630756`}, {-0.3295470244664302, 
      0.03284494430453748}, {-0.33128870378630776`, 
      0.019674601383242035`}, {-0.3326804572234266, 
      0.006483460619950088}, {-0.33372354737654364`, -0.006714494932185729}, \
{-0.33442013908647783`, -0.019905502058827636`}, {-0.33477326351053743`, \
-0.0330760913373197}, {-0.33478677470386486`, -0.04621315535305212}, \
{-0.33446529974936845`, -0.05930401037202057}, {-0.3338141835609401, \
-0.07233645085862445}, {-0.3328394295276021, -0.08529879642833088}, \
{-0.3315476371708689, -0.09817993102144035}, {-0.32994593795742333`, \
-0.1109693342696354}, {-0.32804193034894263`, -0.12365710519541685`}, \
{-0.32584361508616544`, -0.13623397853157515`}, {-0.32335933160108177`, \
-0.1486913340707003}, {-0.32059769633547625`, -0.16102119955207117`}, \
{-0.3175675436216748, -0.17321624766508403`}, {-0.31427786965737337`, \
-0.1852697877957859}, {-0.3107377799852596, -0.1971757531680483}, \
{-0.30695644077332584`, -0.2089286840360308}, {-0.30294303408600964`, \
-0.22052370757279716`}, {-0.2987067172414359, -0.2319565150743638}, \
{-0.2942565862671765, -0.2432233370621599}, {-0.2896016433965246, \
-0.25432091682277824`}, {-0.28475076848919456`, -0.2652464828746418}, \
{-0.2797126942140799, -0.27599772079912893`}, {-0.274495984796383, \
-0.28657274482074957`}, {-0.2691090181060264, -0.29697006946874865`}, \
{-0.2635599708475975, -0.3071885816022757}, {-0.25785680660295207`, \
-0.3172275130339298}, {-0.2520072664748092, -0.32708641394270815`}, \
{-0.24601886208206697`, -0.3367651272275443}, {-0.2398988706640932, \
-0.34626376391691993`}, {-0.2336543320609507, -0.3555826797184876}, \
{-0.22729204734855557`, -0.3647224527651693}, {-0.22081857892142098`, \
-0.3736838625905909}, {-0.21424025183029397`, -0.38246787034673574`}, \
{-0.20756315619714538`, -0.3910756002600439}, {-0.2007931505452156, \
-0.39950832230853794`}, {-0.19393586589683012`, -0.40776743609159327`}, \
{-0.18699671050623473`, -0.4158544558553738}, {-0.1799808751085795, \
-0.42377099663041445`}, {-0.17289333857927844`, -0.4315187614330662}, \
{-0.16573887391020836`, -0.4390995294792575}, {-0.15852205442054046`, \
-0.44651514535703324`}, {-0.15124726013040854`, -0.45376750910338914`}, \
{-0.14391868423511622`, -0.46085856713083595`}, {-0.13654033962619508`, \
-0.4677903039497372}, {-0.1291160654133866, -0.47456473463362225`}, \
{-0.12164953340857633`, -0.4811838979762555}, {-0.11414425453891136`, \
-0.48764985029114133`}, {-0.10660358516183671`, -0.4939646598062704}, \
{-0.09903073325964659, -0.5001304016091912}, {-0.09142876449541952, \
-0.5061491530998651}, {-0.08380060811594431, -0.5120229899111757}, \
{-0.07614906269049909, -0.5177539822593821}, {-0.06847680167716608, \
-0.5233441916891917}, {-0.06078637881079662, -0.528795668180461}, \
{-0.05308023330882601, -0.5341104475857938}, {-0.04536069489291466, \
-0.5392905493704775}, {-0.03762998862589753, -0.5443379746282723}, \
{-0.02989023956478872, -0.5492547043485452}, {-0.022143477231642755`, \
-0.5540426979121008}, {-0.014391639904944641`, -0.5587038917948229}, \
{-0.0066365787349103155`, -0.5632401984598886}, {
      0.0011199383133502485`, -0.5676535054208607}, {
      0.008876222684405608, -0.571945674459407}, {
      0.016630661620063693`, -0.5761185409827339}, {
      0.02438171470491719, -0.580173913507068}, {
      0.03212791056468971, -0.5841135732546777}, {
      0.039867843712083725`, -0.5879392738529887}, {
      0.047600171534771016`, -0.5916527411253428}, {
      0.05532361142015186, -0.5952556729638527}, {
      0.06303693801153369, -0.5987497392756502}, {
      0.07073898059043558, -0.6021365819945942}, {
      0.07842862057980714, -0.6054178151512157}, {
      0.08610478916305339, -0.6085950249943298}, {
      0.09376646501387714, -0.6116697701583392}, {
      0.10141267213208308`, -0.6146435818708063}, {
      0.10904247778063111`, -0.6175179641953694}, {
      0.11665499051937628`, -0.6202943943055398}, {
      0.12424935833108766`, -0.6229743227853353}, {
      0.13182476683549726`, -0.6255591739530907}, {
      0.13938043758728902`, -0.6280503462051372}, {
      0.14691562645409778`, -0.6304492123763598}, {
      0.15442962207074712`, -0.6327571201149378}, {
      0.16192174436611093`, -0.6349753922688379}, {
      0.169391343159138, -0.6371053272818713}, {
      0.17683779682072945`, -0.6391481995973487}, {
      0.18426051099830534`, -0.6411052600675694}, {
      0.19165891740004004`, -0.642977736367561}, {
      0.19903247263588353`, -0.644766833411655}, {
      0.20638065711262, -0.6464737337716358}, {
      0.21370297398034374`, -0.6480995980953349}, {
      0.22099894812785645`, -0.6496455655246712}, {
      0.2282681252246094, -0.6511127541122488}, {
      0.2355100708069282, -0.6525022612357275}, {
      0.242724369406368, -0.653815164009276}, {
      0.24991062371815112`, -0.6550525196914982}, {
      0.25706845380774046`, -0.656215366089306}, {
      0.26419749635369716`, -0.6573047219572733}, {
      0.2712974039250627, -0.6583215873920762}, {
      0.27836784429159284`, -0.6592669442216752}, {
      0.28540849976525445`, -0.6601417563889491}, {
      0.2924190665714744, -0.6609469703295356}, {
      0.29939925424870595`, -0.6616835153436748}, {
      0.3063487850749497, -0.6623523039618886}, {
      0.3132673935199337, -0.6629542323043653}, {
      0.3201548257217228, -0.6634901804339435}, {
      0.32701083898658895`, -0.6639610127026209}, {
      0.33383520131103184`, -0.6643675780915372}, {
      0.3406276909248959, -0.6647107105443987}, {
      0.34738809585458197`, -0.6649912292943336}, {
      0.3541162135054011, -0.6652099391841862}, {
      0.36081185026216817`, -0.6653676309802669}, {
      0.36747482110717417`, -0.6654650816795956}, {
      0.37410494925472293`, -0.6655030548106833}, {
      0.3807020658014554, -0.665482300727907}, {
      0.38726600939172495`, -0.6654035568995456}, {
      0.39379662589732345`, -0.665267548189547}, {
      0.4002937681108915, -0.6650749871331099}, {
      0.406757295452381, -0.664826574206162}, {
      0.41318707368796703`, -0.6645229980888294}, {
      0.4195829746608382, -0.6641649359229883}, {
      0.42594487603332026`, -0.6637530535640004}, {
      0.43227266103981615`, -0.6632880058267311}, {
      0.43856621825007025`, -0.6627704367259556}, {
      0.44482544134228824`, -0.6622009797112566}, {
      0.45105022888566754`, -0.6615802578965213}, {
      0.45724048413191376`, -0.6609088842841445}, {
      0.4633961148153406, -0.6601874619840464}, {
      0.469517032961168, -0.6594165844276121}, {
      0.47560315470165393`, -0.6585968355766622}, {
      0.4816544000997113, -0.6577287901275617}, {
      0.4876706929796781, -0.656813013710574}, {
      0.4936519607649258, -0.6558500630845672}, {
      0.4995981343220045, -0.6548404863271773}, {
      0.5055091478110386, -0.6537848230205334}, {
      0.5113849385420999, -0.6526836044326482}, {
      0.5172254468372975, -0.651537353694574}, {
      0.5230306158983375, -0.6503465859734274}, {
      0.5288003916793138, -0.6491118086413786}, {
      0.5345347227645073, -0.6478335214407046}, {
      0.540233560250976, -0.6465122166450007}, {
      0.5458968576357323, -0.6451483792166454}, {
      0.5515245707073104, -0.6437424869606093}, {
      0.5571166574415396, -0.6422950106747001}, {
      0.5626730779013414, -0.6408064142963303}, {
      0.5681937941403841, -0.6392771550458974}, {
      0.5736787701104302, -0.6377076835668575}, {
      0.5791279715722221, -0.6360984440625791}, {
      0.584541366009758, -0.6344498744300572}, {
      0.5899189225478174, -0.6327624063905661}, {
      0.5952606118725992, -0.6310364656173306}, {
      0.6005664061553452, -0.62927247186029}, {
      0.6058362789788246, -0.6274708390680303}, {
      0.611070205266562, -0.6256319755069556}, {
      0.6162681612146955, -0.6237562838777718}, {
      0.6214301242263586, -0.6218441614293486}, {
      0.6265560728484805, -0.6198960000700295}, {
      0.6316459867109093, -0.6179121864764548}, {
      0.6366998464677608, -0.6158931021999614}, {
      0.6417176337409044, -0.6138391237706219}, {
      0.6466993310654998, -0.6117506227989845}, {
      0.6516449218375009, -0.6096279660755727}, {
      0.656554390263049, -0.6074715156682022}, {
      0.6614277213096783, -0.6052816290171731}, {
      0.666264900659263, -0.6030586590283894}, {
      0.6710659146626343, -0.6008029541644633}, {
      0.6758307502958028, -0.5985148585338527}, {
      0.6805593951177212, -0.5961947119780857}, {
      0.6852518372295279, -0.5938428501571196}, {
      0.6899080652352104, -0.5914596046328838}, {
      0.6945280682036357, -0.5890453029510537}, {
      0.6991118356318905, -0.5866002687211002}, {
      0.7036593574098834, -0.5841248216946607}, {
      0.7081706237861558, -0.5816192778422749}, {
      0.7126456253348572, -0.5790839494285263}, {
      0.717084352923838, -0.576519145085633}, {
      0.7214867976838163, -0.5739251698855259}, {
      0.725852950978577, -0.5713023254104542}, {
      0.7301828043761635, -0.5686509098221564}, {
      0.7344763496210219, -0.5659712179296336}, {
      0.7387335786070636, -0.5632635412555613}, {
      0.7429544833516064, -0.5605281681013758}, {
      0.7471390559701653, -0.5577653836110673}, {
      0.7512872886520553, -0.5549754698337154}, {
      0.7553991736367779, -0.5521587057847986}, {
      0.7594747031911596, -0.5493153675063094}, {
      0.7635138695872143, -0.5464457281257054}, {
      0.7675166650807005, -0.5435500579137285}, {
      0.7714830818903471, -0.5406286243411189}, {
      0.7754131121777222, -0.5376816921342547}, {
      0.7793067480277202, -0.5347095233297428}, {
      0.7831639814296416, -0.5317123773279911}, {
      0.7869848042588458, -0.5286905109457858}, {
      0.7907692082589507, -0.5256441784679007}, {
      0.7945171850245621, -0.522573631697764}, {
      0.7982287259845092, -0.519479120007207}, {
      0.8019038223855678, -0.5163608903853175}, {
      0.8055424652766524, -0.5132191874864234}, {
      0.8091446454934587, -0.5100542536772267}, {
      0.8127103536435386, -0.5068663290831129}, {
      0.816239580091792, -0.5036556516336549}, {
      0.8197323149463566, -0.5004224571073338}, {
      0.8231885480448835, -0.4971669791754978}, {
      0.8266082689411801, -0.49388944944557794`}, {
      0.8299914668922078, -0.49059009750358157`}, {
      0.8333381308454202, -0.48726915095588186`}, {
      0.8366482494264267, -0.4839268354703229}, {
      0.839921810926971, -0.48056337481665795`}, {
      0.8431588032932098, -0.47717899090633936`}, {
      0.8463592141142798, -0.4737739038316774}, {
      0.8495230306111428, -0.4703483319043849}, {
      0.8526502396256952, -0.46690249169352493`}, {
      0.8557408276101337, -0.463436598062877}, {
      0.8587947806165639, -0.4599508642077389}, {
      0.8618120842868436, -0.45644550169117865`}, {
      0.8647927238426505, -0.45292072047975285`}, {
      0.8677366840757637, -0.44937672897870545`}, {
      0.8706439493385514, -0.4458137340666623}, {
      0.8735145035346554, -0.4422319411298356}, {
      0.8763483301098633, -0.438631554095752}, {
      0.8791454120431602, -0.43501277546651856`}, {
      0.8819057318379528, -0.43137580635163975`}, {
      0.8846292715134572, -0.4277208465003986}, {
      0.887316012596243, -0.42404809433381524`}, {
      0.8899659361119278, -0.4203577469761954}, {
      0.8925790225770117, -0.4166500002862811}, {
      0.8951552519908499, -0.41292504888801607`}, {
      0.8976946038277523, -0.40918308620093785`}, {
      0.9001970570292065, -0.4054243044702078}, {
      0.9026625899962167, -0.4016488947962914}, {
      0.9050911805817534, -0.39785704716429976`}, {
      0.9074828060833073, -0.39404895047300337`}, {
      0.9098374432355425, -0.39022479256352927`}, {
      0.9121550682030427, -0.38638476024775253`}, {
      0.9144356565731463, -0.38252903933639243`}, {
      0.9166791833488641, -0.37865781466682386`}, {
      0.9188856229418756, -0.3747712701306143}, {
      0.9210549491655979, -0.3708695887007965}, {
      0.9231871352283244, -0.3669529524588868}, {
      0.9252821537264262, -0.3630215426216591}, {
      0.9273399766376142, -0.3590755395676837}, {
      0.9293605753142566, -0.35511512286364155`}, {
      0.9313439204767467, -0.3511404712904225}, {
      0.9332899822069189, -0.34715176286901683`}, {
      0.9351987299415073, -0.3431491748862107}, {
      0.9370701324656437, -0.3391328839200926}, {
      0.9389041579063919, -0.33510306586538147`}, {
      0.9407007737263131, -0.3310598959585847}, {
      0.9424599467170605, -0.32700354880299515`}, {
      0.9441816429929982, -0.32293419839353565`}, {
      0.9458658279848433, -0.3188520181414601}, {
      0.9475124664333241, -0.31475718089891935`}, {
      0.9491215223828559, -0.31064985898340103`}, {
      0.9506929591752271, -0.306530224202051}, {
      0.9522267394432958, -0.3023984478758858}, {
      0.9537228251046915, -0.2982547008639038}, {
      0.9551811773555208, -0.2940991535871037}, {
      0.9566017566640728, -0.2899319760524186}, {
      0.9579845227645234, -0.28575333787657414`}, {
      0.9593294346506336, -0.2815634083098787}, {
      0.9606364505694404, -0.2773623562599538}, {
      0.9619055280149378, -0.27315035031541357`}, {
      0.9631366237217452, -0.2689275587695004}, {
      0.964329693658761, -0.26469414964368626`}, {
      0.9654846930227973, -0.2604502907112464}, {
      0.966601576232197, -0.25619614952081515`}, {
      0.9676802969204271, -0.25193189341993055`}, {
      0.9687208079296485, -0.24765768957857737`}, {
      0.969723061304259, -0.2433737050127355}, {
      0.9706870082844078, -0.23908010660794296`}, {
      0.9716125992994793, -0.23477706114288097`}, {
      0.9724997839615442, -0.23046473531299003`}, {
      0.9733485110587758, -0.22614329575412492`}, {
      0.9741587285488293, -0.22181290906625722`}, {
      0.9749303835521831, -0.21747374183723356`}, {
      0.9756634223454388, -0.2131259606665983}, {
      0.9763577903545799, -0.2087697321894891}, {
      0.9770134321481855, -0.20440522310061374`}, {
      0.9776302914305982, -0.20003260017831728`}, {
      0.9782083110350454, -0.19565203030874795`}, {
      0.9787474329167096, -0.19126368051013073`}, {
      0.97924759814575, -0.18686771795715754`}, {
      0.9797087469002695, -0.1824643100055029}, {
      0.980130818459229, -0.17805362421647442`}, {
      0.9805137511953058, -0.17363582838180702`}, {
      0.9808574825676948, -0.16921109054861044`}, {
      0.9811619491148513, -0.16477957904447924`}, {
      0.9814270864471745, -0.16034146250277506`}, {
      0.981652829239628, -0.1558969098880908}, {
      0.9818391112243003, -0.15144609052190627`}, {
      0.9819858651828991, -0.14698917410844567`}, {
      0.9820930229391817, -0.14252633076074672`}, {
      0.9821605153513191, -0.13805773102695185`}, {
      0.9821882723041921, -0.13358354591683186`}, {
      0.9821762227016192, -0.1291039469285527}, {
      0.9821242944585145, -0.12461910607569605`}, {
      0.9820324144929754, -0.12012919591454489`}, {
      0.9819005087182985, -0.11563438957164499`}, {
      0.9817285020349221, -0.11113486077165394`}, {
      0.9815163183222968, -0.10663078386548919`}, {
      0.9812638804306799, -0.10212233385878697`}, {
      0.9809711101728564, -0.0976096864406841}, {
      0.9806379283157833, -0.09309301801293511}, {
      0.9802642545721575, -0.08857250571937707}, {
      0.9798500075919067, -0.0840483274757551}, {
      0.9793951049536027, -0.07952066199992146}, {
      0.978899463155797, -0.07498968884242184}, {
      0.9783629976082773, -0.0704555884174822}, {
      0.9777856226232453, -0.06591854203441046}, {
      0.9771672514064158, -0.06137873192942699}, {
      0.9765077960480364, -0.056836341297938825`}, {
      0.9758071675138275, -0.05229155432727227}, {
      0.9750652756358432, -0.04774455622987942}, {
      0.9742820291032536, -0.04319553327703412}, {
      0.9734573354530465, -0.038644672833033425`}, {
      0.9725911010606517, -0.034092163389921125`}, {
      0.9716832311304862, -0.02953819460274995}, {
      0.9707336296864212, -0.024982957325399957`}, {
      0.969742199562173, -0.02042664364697062}, {
      0.9687088423916156, -0.015869446928764894`}, {
      0.967633458599019, -0.011311561841883797`}, {
      0.9665159473892119, -0.006753184405450677}, {
      0.9653562067376712, -0.0021945120254847315`}, {0.9641541333805393, 
      0.002364256465556061}, {0.9629096228045703, 0.006922920768541933}, {
      0.9616225692370068, 0.011481279076727021`}, {0.9602928656353901, 
      0.016039128034071936`}, {0.9589204036773038, 0.02059626269281081}, {
      0.9575050737500552, 0.02515247647023979}, {0.9560467649402954, 
      0.029707561104703302`}, {0.9545453650235807, 0.03426130661075373}, {
      0.9530007604538792, 0.038813501233459544`}, {0.951412836353025, 
      0.04336393140183604}, {0.949781476500123, 0.04791238168137227}, {
      0.9481065633209084, 0.05245863472562693}, {0.9463879778770647, 
      0.057002471226865116`}, {0.9446255998555049, 0.0615436698657071}, {
      0.9428193075576184, 0.06608200725975942}, {0.940968977888492, 
      0.07061725791119773}, {0.9390744863461058, 0.0751491941532698}, {
      0.937135707010514, 0.0796775860956863}, {0.9351525125330127, 
      0.08420220156886592}, {0.933124774125304, 0.08872280606700038}, {
      0.9310523615486614, 0.09323916268990379}, {0.9289351431031045, 
      0.09775103208360983}, {0.9267729856165906, 0.10225817237967903`}, {
      0.9245657544342317, 0.10676033913317722`}, {0.9223133134075457, 
      0.11125728525928508`}, {0.9200155248837505, 0.11574876096849757`}, {
      0.9176722496951119, 0.12023451370037035`}, {0.9152833471483556, 
      0.12471428805576941`}, {0.9128486750141549, 0.1291878257275784}, {
      0.9103680895167052, 0.13365486542981692`}, {0.9078414453233993, 
      0.13811514282512136`}, {0.9052685955346159, 0.14256839045053846`}, {
      0.9026493916736376, 0.1470143376415801}, {0.8999836836767107, 
      0.151452710454486}, {0.8972713198832665, 0.15588323158663983`}, {
      0.8945121470263186, 0.16030562029508152`}, {0.8917060102230562, 
      0.16471959231305763`}, {0.888852752965651, 0.16912485976454925`}, {
      0.8859522171123017, 0.17352113107671469`}, {0.8830042428785326, 
      0.17790811089018285`}, {0.8800086688287757, 0.18228549996713025`}, {
      0.8769653318682545, 0.18665299509707295`}, {0.8738740672352009, 
      0.19101028900030212`}, {0.8707347084934286, 0.19535707022888965`}, {
      0.8675470875252949, 0.19969302306518777`}, {0.8643110345250801, 
      0.20401782741774394`}, {0.861026377992818, 0.20833115871454969`}, {
      0.8576929447286119, 0.21263268779353933`}, {0.8543105598274723, 
      0.21692208079025166`}, {0.8508790466747159, 0.22119899902256454`}, {
      0.8473982269419668, 0.22546309887240948`}, {0.8438679205838024, 
      0.22971403166437}, {0.8402879458350923, 0.23395144354106406`}, {
      0.8366581192090754, 0.23817497533520807`}, {0.8329782554962318, 
      0.24238426243825506`}, {0.8292481677639989, 0.24657893466549788`}, {
      0.8254676673573933, 0.2507586161175224}, {0.8216365639005988, 
      0.25492292503789343`}, {0.8177546652995842, 0.2590714736669504}, {
      0.8138217777458213, 0.2632038680915873}, {0.8098377057211753, 
      0.2673197080908856}, {0.8058022520040434, 0.27141858697746424`}, {
      0.8017152176768257, 0.27550009143440735`}, {0.797576402134812, 
      0.27956380134762426`}, {0.7933856030965777, 0.2836092896334914}, {
      0.7891426166159861, 0.287636122061621}, {0.7848472370958973, 
      0.29164385707259505`}, {0.7804992573036955, 0.2956320455904989}, {
      0.7760984683887469, 0.29960023083008086`}, {0.7716446599019122, 
      0.30354794809836033`}, {0.7671376198172426, 0.3074747245904986}, {
      0.7625771345559947, 0.31138007917974225`}, {0.7579629890131114, 
      0.31526352220123965`}, {0.753294966586321, 0.31912455522952726`}, {
      0.7485728492080181, 0.3229626708494721}, {0.7437964173800988, 
      0.32677735242045175`}, {0.7389654502119332, 0.3305680738335444}, {
      0.7340797254616689, 0.33433429926149405`}, {0.7291390195810725, 
      0.33807548290120715`}, {0.7241431077641249, 0.34179106870852993`}, {
      0.7190917639996043, 0.34548049012504534`}, {0.7139847611278999, 
      0.3491431697966208}, {0.7088218709023187, 0.3527785192834287}, {
      0.7036028640551598, 0.3563859387611512}, {0.6983275103688495, 
      0.3599648167130733}, {0.6929955787524491, 0.363514529612755}, {
      0.6876068373238627, 0.36703444159696713`}, {0.6821610534980987, 
      0.3705239041285617}, {0.676657994081953, 0.37398225564893856`}, {
      0.6710974253755105, 0.3774088212197597}, {0.6654791132808835, 
      0.3808029121535502}, {0.6598028234186311, 0.3841638256328147}, {
      0.654068321252333, 0.38749084431728614`}, {0.6482753722218186, 
      0.39078323593891284`}, {0.6424237418855848, 0.3940402528841773}, {
      0.636513196072969, 0.3972611317633292}, {0.6305435010466792, 
      0.400445092966103}, {0.6245144236763205, 0.40359134020347853`}, {
      0.6184257316235974, 0.40669906003503253`}, {0.6122771935399154, 
      0.40976742138141525`}, {0.6060685792771497, 0.4127955750214773}, {
      0.5997996601123973, 0.4157826530735593}, {0.5934702089875816, 
      0.4187277684604468}, {0.5870800007648334, 0.42163001435748226`}, {
      0.5806288124986294, 0.42448846362331727`}, {0.5741164237257355, 
      0.4273021682127775}, {0.5675426167740637, 0.4300701585713069}, {
      0.560907177091628, 0.4327914430104492}, {0.5542098935968555, 
      0.4354650070638193}, {0.5474505590515935, 0.43808981282301424`}, {
      0.540628970458236, 0.4406647982529085}, {0.5337449294824872, 
      0.4431888764857802}, {0.5267982429033744, 0.4456609350937154}, {
      0.5197887230922295, 0.4480798353387421}, {0.5127161885224657, 
      0.4504444114001541}, {0.5055804643120962, 0.4527534695784941}, {
      0.49838138280106464`, 0.455005787475683}, {0.4911187841655921, 
      0.4572001131507992}, {0.4837925170718863, 0.45933516425103926`}, {
      0.47640243937170895`, 0.46140962711741884`}, {0.46894841884245825`, 
      0.46342215586481345`}, {0.46143033397459315`, 0.46537137143598056`}, {
      0.45384807480940653`, 0.4672558606292596}, {0.44620154383034694`, 
      0.469074175099708}, {0.4384906569112914, 0.47082483033350553`}, {
      0.4307153443253877, 0.4725063045955449}, {0.42287555181831293`, 
      0.4741170378502245}, {0.4149712417500364, 0.47565543065557486`}, {
      0.40700239430943197`, 0.47711984303097926`}, {0.39896900880635194`, 
      0.47850859329890205`}, {0.3908711050460625, 0.4798199569012058}, {
      0.3827087247912373, 0.48105216519083627`}, {0.3744819333170234, 
      0.482203404199873}, {0.36619082106502193`, 0.48327181338519587`}, {
      0.35783550540237496`, 0.48425548435329896`}, {0.34941613249250936`, 
      0.48515245956610437`}, {0.3409328792844681, 0.4859607310299867}, {
      0.33238595562814927`, 0.4866782389706253}, {0.3237756065231814, 
      0.4873028704967526}, {0.3151021145095804, 0.48783245825637905`}, {
      0.30636580220876575`, 0.4882647790896414}, {0.2975670350239506, 
      0.48859755268305954`}, {0.28870622400936885`, 0.48882844023069666`}, {
      0.27978382891824927`, 0.48895504310850774`}, {0.27080036143989616`, 
      0.48897490156904355`}, {0.26175638863667844`, 0.48888549346465404`}, {
      0.2526525365921581, 0.48868423300842123`}, {0.2434894942820008, 
      0.4883684695832533}, {0.2342680176796928, 0.4879354866109016}, {
      0.22498893410943133`, 0.4873825004941298}, {0.2156531468588477, 
      0.48670665964688303`}, {0.2062616400644467, 0.48590504362908316`}, {
      0.19681548388278783`, 0.48497466240463133`}, {0.18731583996047174`, 
      0.48391245574333874`}, {0.17776396721590534`, 0.48271529278984454`}, {
      0.1681612279455766, 0.48137997182512887`}, {0.15850909426714024`, 
      0.47990322024899995`}, {0.1488091549109674, 0.4782816948149338}, {
      0.13906312237090152`, 0.47651198215188606`}, {0.12927284042374512`, 
      0.474590599611178}, {0.11944029202542672`, 0.4725139964802898}, {
      0.10956760758980344`, 0.4702785556093645}, {0.09965707365358169, 
      0.46788059550043404`}, {0.0897111419278103, 0.46531637291380396`}, {
      0.079732438732743, 0.46258208605065154`}, {0.06972377480849157, 
      0.4596738783756698}, {0.05968815548870769, 0.45658784314847495`}, {
      0.049628791218441284`, 0.4533200287374244}, {0.0395491083902242, 
      0.4498664447943821}, {0.029452760464214515`, 0.4462230693737113}, {
      0.019343639328802256`, 0.44238585708324113`}, {0.009225886847319023, 
      0.4383507483589837}, {-0.0008960934756858038, 
      0.4341136799587775}, {-0.011017624788824443`, 
      0.429670596772574}, {-0.021133745244383467`, 
      0.4250174650484986}, {-0.031239196517311763`, 
      0.42015028713379104`}, {-0.04132841262349862, 
      0.4150651178279178}, {-0.05139550918819002, 
      0.4097580824411376}, {-0.06143427333687312, 
      0.40422539664516527`}, {-0.0714381544037006, 
      0.3984633881928262}, {-0.08140025567624347, 
      0.3924685205702219}, {-0.09131332741965811, 
      0.3862374186274039}, {-0.10116976144770443`, 
      0.3797668962113466}, {-0.11096158753178421`, 
      0.373053985797615}, {-0.12068047196144871`, 
      0.3660959700840737}, {-0.13031771858964325`, 
      0.3588904154709138}, {-0.13986427271213311`, 
      0.35143520730593825`}, {-0.1493107281417297, 
      0.3437285867223856}, {-0.15864733784259866`, 
      0.33576918883877255`}, {-0.1678640284864426, 
      0.32755608202676567`}, {-0.1769504192789969, 
      0.3190888078848062}, {-0.18589584538033774`, 
      0.3103674214833582}, {-0.19468938620433932`, 
      0.3013925313739625}, {-0.20331989882979745`, 
      0.2921653387809758}, {-0.21177605668715294`, 
      0.28268767532468814`}, {-0.22004639359977843`, 
      0.27296203856064294`}, {-0.2281193531574536, 
      0.26299162456604935`}, {-0.23598334328275633`, 
      0.2527803567640778}, {-0.24362679572038148`, 
      0.2423329101545976}, {-0.25103823003763803`, 
      0.23165473011947985`}, {-0.25820632157542156`, 
      0.22075204499551482`}, {-0.26511997263772163`, 
      0.20963187166118835`}, {-0.27176838606006243`, 
      0.1983020134669453}, {-0.27814114015978186`, 
      0.18677104995279}, {-0.2842282639507665, 
      0.17504831794123474`}, {-0.29002031140923856`, 
      0.1631438837650925}, {-0.2955084335120897, 
      0.1510685065840058}, {-0.3006844467407823, 
      0.1388335929547568}, {-0.3055408967562998, 
      0.12645114304064042`}, {-0.3100711160064611, 
      0.11393368906565393`}, {-0.3142692741263752, 
      0.10129422683044687`}, {-0.31813042013378534`, 
      0.08854614129928148}, {-0.3216505155990427, 
      0.07570312743164245}, {-0.3248264581777833, 
      0.06277910756076646}, {-0.3276560951245772, 
      0.049788146708168994`}, {-0.3301382266481133, 
      0.036744367264362435`}, {-0.3322725992124956, 
      0.023661864459994845`}, {-0.33405988912462736`, 
      0.010554623999742017`}, {-0.335501676964857, -0.00256355686298786}, \
{-0.3366004136087929, -0.01567914366348376}, {-0.33735937874600247`, \
-0.02877893348871759}, {-0.3377826329218113, -0.04185011736092919}, \
{-0.3378749642094545, -0.05488033563083887}, {-0.33764183066138787`, \
-0.0678577259672079}, {-0.33708929969257007`, -0.08077096372284587}, \
{-0.336223985518525, -0.09360929463925678}, {-0.33505298571173675`, \
-0.1063625600172835}, {-0.33358381785696484`, -0.11902121462519756`}, \
{-0.3318243571852841, -0.13157633773604863`}, {-0.3297827759539361, \
-0.14401963778152888`}, {-0.32746748521996216`, -0.1563434511801957}, \
{-0.3248870795350368, -0.16854073594476396`}, {-0.32205028497116317`, \
-0.18060506069828264`}, {-0.31896591077532904`, -0.19253058973487686`}, \
{-0.315642804848431, -0.20431206475023403`}, {-0.3120898131515133, \
-0.21594478384314697`}, {-0.30831574306168047`, -0.22742457835516658`}, \
{-0.3043293306313354, -0.2387477880735669}, {-0.3001392116475583, \
-0.24991123527593234`}, {-0.29575389634296595`, -0.26091219804496857`}, \
{-0.2911817475744645, -0.271748383231501}, {-0.2864309622609344, \
-0.2824178993935924}, {-0.28150955585394144`, -0.2929192299914849}, \
{-0.27642534960590226`, -0.3032512070725476}, {-0.2711859603965958, \
-0.31341298563820497`}, {-0.2657987928804178, -0.32340401884632886`}, \
{-0.2602710337223087, -0.3332240341679839}, {-0.2546096476989387, \
-0.34287301058676706`}, {-0.2488213754527001, -0.3523511569021907}, \
{-0.24291273269864455`, -0.36165889117545547`}, {-0.23689001069812265`, \
-0.37079682133630787`}, {-0.23075927782705175`, -0.37976572695320837`}, \
{-0.2245263820810555, -0.38856654215544706`}, {-0.21819695437387968`, \
-0.39720033968483726`}, {-0.21177641249924767`, -0.40566831604589076`}, \
{-0.2052699656395014, -0.41397177771664334`}, {-0.19868261931685194`, \
-0.4221121283772856}, {-0.19201918069475576`, -0.43009085711020884`}, \
{-0.18528426414779148`, -0.43790952752277246`}, {-0.17848229702841753`, \
-0.44556976774283164`}, {-0.17161752556814716`, -0.4530732612366483}, \
{-0.16469402085900173`, -0.4604217383990873}, {-0.15771568486863002`, \
-0.4676169688668294}, {-0.15068625644924805`, -0.47466075450659834`}, \
{-0.14360931730660906`, -0.4815549230319912}, {-0.1364882979006046, \
-0.4883013222043409}, {-0.1293264832538763, -0.4949018145750463}, \
{-0.12212701864903489`, -0.5013582727289243}, {-0.11489291519878805`, \
-0.5076725749903168}, {-0.10762705527651889`, -0.5138466015558841}, \
{-0.10033219779767845`, -0.5198822310202041}, {-0.0930109833448019, \
-0.5257813372624455}, {-0.08566593913106657, -0.5315457866644772}, \
{-0.07829948379912051, -0.5371774356327977}, {-0.07091393205345381, \
-0.5426781283986066}, {-0.06351149912589439, -0.548049695072191}, \
{-0.05609430507491139, -0.5532939499295579}, {-0.04866437892032971, \
-0.5584126899108994}, {-0.04122366261582002, -0.563407693312045}, \
{-0.033774014862150245`, -0.5682807186515187}, {-0.02631721476468614, \
-0.5730335036971952}, {-0.018854965339024858`, -0.5776677646378271}, \
{-0.011388896868951925`, -0.5821851953859133}, {-0.003920570121140687, \
-0.5865874669994835}, {0.003548520578824362, -0.5908762272114071}, {
      0.011016944404413288`, -0.5950531000557853}, {
      0.018483331215360648`, -0.5991196855818678}, {
      0.025946368869357465`, -0.6030775596467507}, {
      0.03340480065347965, -0.6069282737788605}, {
      0.040857422830651774`, -0.6106733551049253}, {
      0.04830308229651282, -0.6143143063337673}, {
      0.055740674342136996`, -0.6178526057908392}, {
      0.06316914051816418, -0.6212897074979646}, {
      0.07058746659600788, -0.6246270412932361}, {
      0.0779946806219311, -0.6278660129864799}, {
      0.08538985105990947, -0.6310080045461098}, {
      0.09277208501933519, -0.6340543743135755}, {
      0.10014052656375193`, -0.6370064572419581}, {
      0.10749435509694942`, -0.639865565155587}, {
      0.1148327838228855, -0.6426329870278411}, {
      0.12215505827604152`, -0.6453099892745682}, {
      0.12946045491895386`, -0.6478978160607973}, {
      0.13674827980379917`, -0.6503976896186443}, {
      0.14401786729504293`, -0.6528108105745116}, {
      0.1512685788502901, -0.655138358283873}, {
      0.15849980185660179`, -0.6573814911720998}, {
      0.1657109485196644, -0.6595413470799412}, {
      0.17290145480331515`, -0.6616190436124139}, {
      0.18007077941704233`, -0.6636156784899812}, {
      0.18721840284918811`, -0.6655323299010215}, {
      0.19434382644368822`, -0.6673700568546923}, {
      0.20144657151828368`, -0.6691298995333903}, {
      0.2085261785222382, -0.6708128796441001}, {
      0.21558220623168778`, -0.6724200007680006}, {
      0.222614230980839, -0.6739522487077733}, {
      0.22962184592731807`, -0.6754105918321212}, {
      0.2366046603500547, -0.6767959814170686}, {
      0.24356229897816256`, -0.6781093519836653}, {
      0.2504944013493538, -0.6793516216317669}, {
      0.2574006211964951, -0.6805236923696119}, {
      0.2642806258609811, -0.6816264504389504}, {
      0.2711340957316664, -0.6826607666355222}, {
      0.27796072370815683`, -0.6836274966247086}, {
      0.2847602146873219, -0.6845274812522183}, {
      0.2915322850719439, -0.6853615468496903}, {
      0.29827666230047356`, -0.6861305055351219}, {
      0.3049930843969113, -0.6868351555080526}, {
      0.31168129953988266`, -0.6874762813394537}, {
      0.3183410656500204, -0.6880546542562893}, {
      0.32497214999481033`, -0.6885710324207336}, {
      0.33157432881009813`, -0.6890261612040393}, {
      0.3381473869374942, -0.6894207734550672}, {
      0.34469111747695047`, -0.6897555897634977}, {
      0.3512053214538179, -0.6900313187177524}, {
      0.3576898074997278, -0.6902486571576665}, {
      0.3641443915466711, -0.6904082904219572}, {
      0.3705688965336805, -0.6905108925905413}, {
      0.3769631521255484, -0.6905571267217605}, {
      0.3833269944430414, -0.690547645084579}, {
      0.38966026580409874`, -0.6904830893858204}, {
      0.39596281447552356`, -0.690364090992517}, {
      0.40223449443470377`, -0.6901912711494462}, {
      0.4084751651409174, -0.6899652411919305}, {
      0.41468469131580094`, -0.6896866027539826}, {
      0.42086294273257746`, -0.6893559479718769}, {
      0.4270097940136622, -0.6889738596832284}, {
      0.43312512443627904`, -0.6885409116216658}, {
      0.4392088177457407, -0.6880576686071809}, {
      0.44526076197606, -0.6875246867322415}, {
      0.451280849277576, -0.6869425135437512}, {
      0.4572689757512934, -0.6863116882209426}, {
      0.4632250412896475, -0.6856327417492883}, {
      0.46914894942341984`, -0.6849061970905153}, {
      0.47504060717454333`, -0.6841325693488047}, {
      0.4808999249145464, -0.6833123659332634}, {
      0.4867268162283986, -0.682446086716747}, {
      0.4925211977835291, -0.6815342241911184}, {
      0.4982829892038023, -0.6805772636190212}, {
      0.5040121129482419, -0.6795756831822475}, {
      0.5097084941943059, -0.6785299541267806}, {
      0.515372060725524, -0.6774405409045873}, {
      0.5210027428233163, -0.6763079013122372}, {
      0.5266004731628202, -0.6751324866264248}, {
      0.5321651867125623, -0.6739147417364658}, {
      0.5376968206378152, -0.6726551052738416}, {
      0.5431953142074918, -0.6713540097388618}, {
      0.5486606087044298, -0.6700118816245146}, {
      0.5540926473389317, -0.6686291415375735}, {
      0.5594913751654261, -0.6672062043170257}, {
      0.5648567390021266, -0.6657434791498901}, {
      0.570188687353566, -0.6642413696844852}, {
      0.5754871703358918, -0.6627002741412137}, {
      0.5807521396048124, -0.6611205854209207}, {
      0.585983548286088, -0.6595026912108892}, {
      0.5911813509084654, -0.6578469740885293}, {
      0.5963455033389603, -0.6561538116228179}, {
      0.6014759627203944, -0.6544235764735468}, {
      0.6065726874110974, -0.6526566364884315}, {
      0.6116356369266907, -0.6508533547981364}, {
      0.6166647718838704, -0.6490140899092658}, {
      0.6216600539461111, -0.6471391957953734}, {
      0.626621445771217, -0.6452290219860399}, {
      0.6315489109606476, -0.6432839136540661}, {
      0.6364424140105489, -0.6413042117008291}, {
      0.6413019202644263, -0.6392902528398478}, {
      0.6461273958673935, -0.6372423696786034}, {
      0.6509188077219378, -0.6351608907986573}, {
      0.6556761234451441, -0.6330461408341106}, {
      0.660399311327322, -0.6308984405484452}, {
      0.6650883402919799, -0.62871810690979}, {
      0.6697431798570992, -0.6265054531646486}, {
      0.6743638000976543, -0.6242607889101297}, {
      0.6789501716093354, -0.6219844201647171}, {
      0.6835022654734252, -0.6196766494376156}, {
      0.6880200532227885, -0.6173377757967096}, {
      0.6925035068089302, -0.6149680949351689}, {
      0.696952598570084, -0.612567899236736}, {
      0.7013673012002904, -0.6101374778397282}, {
      0.7057475877194296, -0.6076771166997861}, {
      0.7100934314441703, -0.6051870986514025}, {
      0.714404805959803, -0.6026677034682598}, {
      0.7186816850929233, -0.6001192079224074}, {
      0.7229240428849324, -0.5975418858423083}, {
      0.7271318535663265, -0.5949360081697842}, {
      0.7313050915317437, -0.5923018430158852}, {
      0.7354437313157415, -0.5896396557157132}, {
      0.7395477475692757, -0.5869497088822246}, {
      0.7436171150368572, -0.5842322624590386}, {
      0.747651808534359, -0.5814875737722759}, {
      0.7516518029274498, -0.5787158975814531}, {
      0.755617073110632, -0.5759174861294567}, {
      0.7595475939868594, -0.5730925891916192}, {
      0.7634433404477151, -0.5702414541239222}, {
      0.7673042873541263, -0.5673643259103474}, {
      0.771130409517598, -0.5644614472093965}, {
      0.7749216816819455, -0.5615330583998046}, {
      0.7786780785055054, -0.5585793976254643}, {
      0.7823995745438094, -0.5556007008395822}, {
      0.786086144232702, -0.5525972018480894}, {
      0.7897377618718859, -0.5495691323523214}, {
      0.7933544016088784, -0.5465167219909908}, {
      0.7969360374233629, -0.543440198381468}, {
      0.8004826431119227, -0.5403397871603893}, {
      0.8039941922731382, -0.5372157120236108}, {
      0.8074706582930379, -0.534068194765523}, {
      0.8109120143308869, -0.5308974553177457}, {
      0.8143182333053007, -0.5277037117872179}, {
      0.8176892878806714, -0.5244871804936998}, {
      0.8210251504538939, -0.5212480760067021}, {
      0.8243257931413808, -0.5179866111818578}, {
      0.8275911877663544, -0.5147029971967533}, {
      0.8308213058464041, -0.5113974435862306}, {
      0.8340161185812992, -0.5080701582771788}, {
      0.8371755968410466, -0.5047213476228257}, {
      0.8402997111541834, -0.5013512164365452}, {
      0.8433884316962946, -0.49795996802519427`}, {
      0.8464417282787463, -0.49454780422199124`}, {
      0.849459570337626, -0.49111492541895063`}, {
      0.8524419269228809, -0.48766153059888534`}, {
      0.8553887666876453, -0.4841878173669902}, {
      0.8583000578777492, -0.48069398198201835`}, {
      0.8611757683214005, -0.477180219387063}, {
      0.8640158654190323, -0.4736467232399562}, {
      0.8668203161333082, -0.47009368594329637`}, {
      0.8695890869792787, -0.466521298674116}, {
      0.8723221440146806, -0.4629297514132009}, {
      0.8750194528303735, -0.4593192329740718}, {
      0.8776809785409069, -0.4556899310316393}, {
      0.8803066857752101, -0.4520420321505431}, {
      0.8828965386674003, -0.4483757218131854}, {
      0.8854505008477016, -0.4446911844474696}, {
      0.88796853543347, -0.4409886034542536}, {
      0.8904506050203173, -0.4372681612345289}, {
      0.8928966716733296, -0.4335300392163339}, {
      0.8953066969183752, -0.42977441788141263`}, {
      0.8976806417334938, -0.42600147679162753`}, {
      0.9000184665403669, -0.42221139461513607`}, {
      0.9023201311958587, -0.4184043491523406}, {
      0.9045855949836276, -0.41458051736162077`}, {
      0.9068148166057995, -0.4107400753848574}, {
      0.909007754174702, -0.40688319857275695`}, {
      0.9111643652046514, -0.4030100615099856}, {
      0.913284606603791, -0.3991208380401216}, {
      0.915368434665975, -0.395215701290435}, {
      0.9174158050626936, -0.3912948236965026}, {
      0.9194266728350367, -0.3873583770266678}, {
      0.9214009923856897, -0.3834065324063531}, {
      0.9233387174709611, -0.3794394603422336}, {
      0.9252398011928337, -0.3754573307462806}, {
      0.9271041959910401, -0.3714603129596831}, {
      0.9289318536351555, -0.3674485757766552}, {
      0.9307227252167061, -0.36342228746813815`}, {
      0.9324767611412884, -0.35938161580540534`}, {
      0.934193911120698, -0.35532672808357757`}, {
      0.9358741241650618, -0.3512577911450573}, {
      0.937517348574972, -0.34717497140289005`}, {
      0.9391235319336199, -0.34307843486406064`}, {
      0.9406926210989226, -0.3389683471527323}, {
      0.9422245621956442, -0.334844873533437}, {
      0.9437193006075044, -0.3307081789342249}, {
      0.9451767809692739, -0.32655842796978046`}, {
      0.9465969471588537, -0.32239578496451404`}, {
      0.9479797422893333, -0.3182204139756362}, {
      0.9493251087010287, -0.31403247881622337`}, {
      0.9506329879534929, -0.30983214307828233`}, {
      0.9519033208175006, -0.30561957015582214`}, {
      0.9531360472670011, -0.3013949232679414}, {
      0.9543311064710385, -0.2971583654819383}, {
      0.9554884367856366, -0.2929100597364532}, {
      0.956607975745645, -0.28865016886464984`}, {
      0.9576896600565451, -0.28437885561744547`}, {
      0.958733425586212, -0.28009628268679654`}, {
      0.959739207356632, -0.27580261272904955`}, {
      0.9607069395355713, -0.27149800838836446`}, {
      0.9616365554281949, -0.2671826323202201}, {
      0.9625279874686327, -0.2628566472150092}, {
      0.9633811672114904, -0.2585202158217323}, {
      0.9641960253233038, -0.25417350097179936`}, {
      0.9649724915739339, -0.2498166656029471}, {
      0.9657104948278995, -0.24544987278328195`}, {
      0.9664099630356473, -0.24107328573545683`}, {
      0.9670708232247556, -0.23668706786099145`}, {
      0.9676930014910701, -0.23229138276474495`}, {
      0.9682764229897703, -0.2278863942795506}, {
      0.9688210119263636, -0.22347226649102167`}, {
      0.9693266915476051, -0.21904916376253852`}, {
      0.9697933841323424, -0.21461725076042626`}, {
      0.9702210109822819, -0.21017669247933313`}, {
      0.9706094924126755, -0.2057276542678197}, {
      0.970958747742926, -0.201270301854169}, {
      0.9712686952871088, -0.19680480137242798`}, {
      0.9715392523444082, -0.19233131938869102`}, {
      0.9717703351894672, -0.18785002292763622`}, {
      0.9719618590626482, -0.18336107949932523`}, {
      0.972113738160203, -0.17886465712627814`}, {
      0.9722258856243511, -0.1743609243708345}, {
      0.9722982135332643, -0.16985005036281237`}, {
      0.9723306328909553, -0.16533220482747682`}, {
      0.9723230536170703, -0.1608075581138305}, {
      0.9722753845365819, -0.156276281223238}, {
      0.9721875333693835, -0.15173854583839716`}, {
      0.9720594067197811, -0.14719452435266964`}, {
      0.9718909100658831, -0.1426443898997843}, {
      0.971681947748886, -0.1380883163839267}, {
      0.9714324229622542, -0.1335264785102281}, {
      0.9711422377407934, -0.12895905181566858`}, {
      0.9708112929496169, -0.12438621270040816`}, {
      0.9704394882730014, -0.1198081384595608}, {
      0.9700267222031349, -0.11522500731542634`}, {
      0.9695728920287515, -0.1106369984501957}, {
      0.969077893823656, -0.10604429203914513`}, {
      0.9685416224351343, -0.10144706928433579`}, {
      0.9679639714722517, -0.09684551244883509}, {
      0.9673448332940353, -0.09223980489147682}, {
      0.9666840989975433, -0.08763013110217757}, {
      0.9659816584058171, -0.08301667673782717}, {
      0.9652374000557189, -0.07839962865877168}, {
      0.9644512111856525, -0.07377917496590748}, {
      0.9636229777231675, -0.06915550503840613}, {
      0.9627525842724476, -0.06452880957208952}, {
      0.9618399141016818, -0.05989928061847583}, {
      0.9608848491303192, -0.05526711162451729}, {
      0.9598872699162083, -0.05063249747305111}, {
      0.9588470556426195, -0.0459956345239858}, {
      0.9577640841051533, -0.041356720656245516`}, {
      0.9566382316985333, -0.03671595531049591}, {
      0.9554693734032863, -0.032073539532675374`}, {
      0.9542573827723089, -0.027429676018356518`}, {
      0.9530021319173237, -0.022784569157963244`}, {
      0.9517034914952252, -0.018138425082869607`}, {
      0.9503613306943165, -0.013491451712407395`}, {
      0.9489755172204419, -0.008843858801810153}, {
      0.9475459172830129, -0.0041958579911221885`}, {0.946072395580933, 
      0.00045233714489801777`}, {0.9445548152884246, 0.005100511045849746}, {
      0.9429930380407567, 0.009748446113706101}, {0.9413869239198828, 
      0.01439592265974208}, {0.9397363314399858, 0.019042718850341513`}, {
      0.9380411175329397, 0.023688610651648748`}, {0.9363011375336884, 
      0.028333371773029886`}, {0.9345162451655474, 0.03297677360930727}, {
      0.9326862925254348, 0.03761858518172986}, {0.9308111300690336, 
      0.042258573077640825`}, {0.9288906065958951, 0.04689650138880263}, {
      0.9269245692344867, 0.05153213164833852}, {0.924912863427193, 
      0.05616522276624801}, {0.9228553329152753, 0.0607955309634527}, {
      0.9207518197238003, 0.06542280970432723}, {0.9186021641465436, 
      0.07004680962766885}, {0.9164062047308795, 0.07466727847605743}, {
      0.9141637782626654, 0.07928396102355637}, {0.9118747197511314, 
      0.0838965990017029}, {0.9095388624137882, 0.08850493102373515}, {
      0.9071560376613619, 0.09310869250700082}, {0.9047260750827725, 
      0.09770761559349136}, {0.9022488024301667, 0.1023014290684429}, {
      0.8997240456040217, 0.10688985827694379`}, {0.8971516286383351, 
      0.11147262503848625`}, {0.8945313736859168, 0.1160494475593977}, {
      0.8918631010038028, 0.12062004034308507`}, {0.8891466289388078, 
      0.1251841140980232}, {0.8863817739132387, 0.12974137564341612`}, {
      0.8835683504107894, 0.13429152781245715`}, {0.8807061709626409, 
      0.13883426935311236`}, {0.8777950461337907, 0.1433692948263476}, {
      0.8748347845096376, 0.14789629450171846`}, {0.8718251926828511, 
      0.15241495425023782`}, {0.8687660752405523, 0.15692495543443474`}, {
      0.8656572347518409, 0.16142597479551338`}, {0.8624984717556996, 
      0.165917684337519}, {0.8592895847493127, 0.1703997512084141}, {
      0.8560303701768361, 0.1748718375779647}, {0.8527206224186595, 
      0.17933360051233255`}, {0.8493601337812031, 0.1837846918452666}, {
      0.8459486944872949, 0.18822475804578206`}, {0.8424860926671762, 
      0.19265344008221214`}, {0.8389721143501874, 0.1970703732825132}, {
      0.8354065434571878, 0.20147518719070004`}, {0.8317891617937683, 
      0.20586750541928317`}, {0.8281197490443173, 0.210246945497576}, {
      0.8243980827670068, 0.2146131187157344}, {0.8206239383897663, 
      0.2189656299643869}, {0.8167970892073199, 0.2233040775697079}, {
      0.812917306379363, 0.22762805312378195`}, {0.8089843589299646, 
      0.23193714131010043`}, {0.8049980137482797, 0.23623091972402746`}, {
      0.8009580355906698, 0.24050895868806504`}, {0.7968641870843268, 
      0.24477082106174197`}, {0.7927162287325098, 0.24901606204594404`}, {
      0.7885139189215049, 0.2532442289814971}, {0.7842570139294278, 
      0.2574548611418075}, {0.7799452679369965, 0.2616474895193565}, {
      0.7755784330404075, 0.2658216366058402}, {0.7711562592664601, 
      0.2699768161657353}, {0.7666784945900789, 0.2741125330030678}, {
      0.7621448849543964, 0.2782282827211483}, {0.7575551742935687, 
      0.28232355147503424`}, {0.7529091045585034, 0.28639781571646716`}, {
      0.7482064157456951, 0.2904505419310253}, {0.7434468459293743, 
      0.29448118636722287`}, {0.7386301312971864, 0.2984891947572771}, {
      0.7337560061896348, 0.30247400202925456`}, {0.7288242031435338, 
      0.30643503201029804`}, {0.7238344529397337, 0.31037169712062435`}, {
      0.7187864846553972, 0.3142833980579732}, {0.7136800257211232, 
      0.31816952347217503`}, {0.7085148019832335, 0.3220294496294961}, {
      0.7032905377715568, 0.3258625400664044}, {0.698006955973068, 
      0.3296681452323911}, {0.6926637781117599, 0.33344560212146657`}, {
      0.6872607244351518, 0.3371942338919408}, {0.6817975140078637, 
      0.34091334947408114`}, {0.6762738648127125, 0.34460224316523014`}, {
      0.6706894938598156, 0.34826019421195137`}, {0.6650441173042192, 
      0.3518864663787571}, {0.6593374505726005, 0.35548030750295834`}, {
      0.6535692084996305, 0.3590409490351637}, {0.6477391054746193, 
      0.36256760556493844`}, {0.6418468555991085, 0.36605947433112157`}, {
      0.6358921728561165, 0.36951573471628407`}, {0.6298747712917903, 
      0.37293554772479715`}, {0.6237943652102624, 0.3763180554439651}, {
      0.6176506693825703, 0.3796623804876632}, {0.6114433992705434, 
      0.38296762542190765`}, {0.6051722712666294, 0.38623287217177094`}, {
      0.5988370029506901, 0.38945718140904334`}, {0.5924373133648658, 
      0.39263959192002895`}, {0.585972923307682, 0.3957791199528532}, {
      0.5794435556486447, 0.39887475854364934`}, {0.5728489356646564, 
      0.40192547682098073`}, {0.5661887913996708, 0.40493021928784984`}, {
      0.5594628540490978, 0.4078879050806379}, {0.5526708583705706, 
      0.41079742720431517`}, {0.5458125431227928, 0.4136576517432617}, {
      0.5388876515342966, 0.4164674170470374}, {0.5318959318040645, 
      0.4192255328904473}, {0.5248371376360956, 0.421930779607254}, {
      0.5177110288101359, 0.4245819071969028}, {0.5105173717909359, 
      0.42717763440364154`}, {0.503255940378559, 0.4297166477674405}, {
      0.49592651640242563`, 0.43219760064614654`}, {0.48852889046195874`, 
      0.434619112208341}, {0.4810628627168824, 0.436979766396417}, {
      0.47352824373042535`, 0.439278110859443}, {0.4659248553688949, 
      0.4415126558554451}, {0.45825253176131203`, 0.4436818731228147}, {
      0.4505111203230378, 0.4457841947206387}, {0.44270048284757624`, 
      0.44781801183785}, {0.4348204966710074, 0.4497816735712201}, {
      0.4268710559137901, 0.4516734856723511}, {0.418852072804975, 
      0.4534917092639868}, {0.4107634790941865, 0.45523455952614544`}, {
      0.40260522755706835`, 0.4569002043527855}, {0.3943772936002404, 
      0.45848676297995744`}, {0.3860796769721848, 0.45999230458666424`}, {
      0.3777124035868706, 0.46141484686996403`}, {0.3692755274673296, 
      0.4627523545961968}, {0.36076913281682516`, 0.4640027381306126}, {
      0.3521933362256926, 0.4651638519481244}, {0.34354828902239143`, 
      0.46623349312840895`}, {0.33483417977777896`, 0.4672093998391448}, {
      0.32605123697210203`, 0.46808924981180605`}, {0.31719973183469824`, 
      0.46887065881514145`}, {0.30827998136690127`, 0.4695511791322575}, {
      0.2992923515591512, 0.4701282980481112}, {0.29023726081381657`, 
      0.4705994363552013}, {0.2811151835857315, 0.4709619468863462}, {
      0.27192665425293483`, 0.4712131130846538}, {0.26267227123055814`, 
      0.4713501476221429}, {0.25335270134123455`, 0.4713701910799712}, {
      0.24396868445577996`, 0.47127031070488223`}, {0.2345210384182161, 
      0.47104749925830847`}, {0.2250106642694439, 0.4706986739765783}, {
      0.2154385517840165, 0.47022067566288345`}, {0.20580578533447696`, 
      0.46961026793408295`}, {0.19611355009759432`, 0.46886413664806414`}, {
      0.1863631386165136, 0.46797888954026495`}, {0.17655595773230195`, 
      0.46695105610109217`}, {0.16669353589757654`, 0.4657770877293618}, {
      0.15677753088379495`, 0.4644533582005444}, {0.14680973789232238`, 
      0.46297616449252404`}, {0.13679209807749904`, 0.46134172801577683`}, {
      0.1267267074875505, 0.45954619629933197`}, {0.11661582642623629`, 
      0.45758564518858597`}, {0.10646188923453621`, 0.45545608161597273`}, {
      0.09626751448733849, 0.4531534470106151}, {0.08603551559492179, 
      0.4506736214183491}, {0.07576891179290983, 0.4480124284088548}, {
      0.06547093949721314, 0.4451656408519634}, {0.055145063992143736`, 
      0.44212898765043185`}, {0.044794991410281024`, 0.4388981615214483}, {
      0.034424680951666925`, 0.4354688279236914}, {0.02403835727741075, 
      0.43183663523071164`}, {0.013640522998693557`, 0.427997226254499}, {
      0.0032359711664046355`, 0.42394625122506846`}, {-0.007170202350828891, 
      0.4196793823324041}, {-0.017572586731651702`, 
      0.41519232993579036`}, {-0.02796544446233444, 
      0.4104808605420003}, {-0.038342700167221876`, 
      0.4055408166475435}, {-0.04869793020785071, 
      0.40036813853069525`}, {-0.05902435327423231, 
      0.3949588880657945}, {-0.06931482221740298, 
      0.3893092746147485}, {-0.07956181739809653, 
      0.3834156830282585}, {-0.08975744185159451, 
      0.3772747037614343}, {-0.09989341859259025, 
      0.37088316507471314`}, {-0.1099610904052167, 
      0.3642381672509345}, {-0.11995142248099853`, 
      0.3573371187127873}, {-0.12985500827998175`, 
      0.350177773871572}, {-0.13966207899608868`, 
      0.342758272478481}, {-0.1493625170051328, 
      0.3350771801838967}, {-0.15894587366111984`, 
      0.3271335299394114}, {-0.16840139178166516`, 
      0.3189268638026967}, {-0.17771803312487497`, 
      0.3104572746287956}, {-0.18688451110638232`, 
      0.3017254470551971}, {-0.1958893289352649, 
      0.2927326971150205}, {-0.20472082326067487`, 
      0.2834810097461203}, {-0.2133672133172286, 
      0.2739730734076736}, {-0.22181665543741216`, 
      0.2642123109738645}, {-0.23005730266530255`, 
      0.254202906050796}, {-0.23807736906069754`, 
      0.24394982386175942`}, {-0.2458651981303085, 
      0.2334588258711138}, {-0.2534093346681035, 
      0.22273647737118513`}, {-0.26069859913623966`, 
      0.2117901473416805}, {-0.26772216357807355`, 
      0.20062800000766917`}, {-0.2744696279326682, 
      0.18925897766916544`}, {-0.28093109552319373`, 
      0.17769277454994106`}, {-0.28709724642632684`, 
      0.16593980161075428`}, {-0.29295940740187304`, 
      0.15401114248631642`}, {-0.29850961707555035`, 
      0.14191850092812605`}, {-0.3037406851254468, 
      0.12967414035775082`}, {-0.3086462443241178, 
      0.11729081634758892`}, {-0.31322079443124495`, 
      0.10478170303894832`}, {-0.31745973711148007`, 
      0.09216031467144808}, {-0.32135940126164486`, 
      0.07944042352555739}, {-0.32491705836218365`, 
      0.06663597566566878}, {-0.32813092770982816`, 
      0.053761005910826956`}, {-0.331000171631432, 
      0.04082955345297105}, {-0.3335248810126238, 
      0.027855579489693853`}, {-0.33570605168988726`, 
      0.014852888143838961`}, {-0.3375455524428616, 
      0.0018350518115348383`}, {-0.3390460854788567, -0.011184658079164042`}, \
{-0.3402111404196818, -0.02419333409668506}, {-0.34104494287996595`, \
-0.037178489010581954`}, {-0.3415523987663808, -0.05012810475633457}, \
{-0.34173903543062767`, -0.06303067393432878}, {-0.34161094077931636`, \
-0.0758752337957673}, {-0.3411747013857044, -0.08865139283018458}, \
{-0.34043734056714053`, -0.10134935021039838`}, {-0.3394062572937557, \
-0.11395990846864835`}, {-0.33808916668418404`, -0.12647447987125837`}, \
{-0.3364940427282538, -0.13888508702855462`}, {-0.3346290637594426, \
-0.15118435832312918`}, {-0.33250256108551957`, -0.16336551876479077`}, \
{-0.33012297107744853`, -0.17542237688715998`}, {-0.32749879091676887`, \
-0.1873493082916315}, {-0.32463853811197096`, -0.19914123642224346`}, \
{-0.32155071381583644`, -0.21079361112272782`}, {-0.31824376990869174`, \
-0.22230238548736198`}, {-0.31472607975692407`, -0.2336639914726481}, \
{-0.31100591251143156`, -0.2448753146894495}, {-0.3070914107761565, \
-0.25593366874682555`}, {-0.3029905714515195, -0.26683676947090024`}, \
{-0.29871122954037227`, -0.27758270927583173`}, {-0.2942610446939128, \
-0.28816993192018425`}, {-0.28964749027076697`, -0.29859720784134086`}, \
{-0.28487784468309785`, -0.3088636102234194}, {-0.27995918480818804`, \
-0.3189684919206572}, {-0.27489838125158805`, -0.3289114633284629}, \
{-0.26970209525785804`, -0.33869237126822577`}, {-0.2643767770764933, \
-0.34831127892937}, {-0.25892866560324496`, -0.35776844689283255`}, \
{-0.2533637891302629, -0.36706431524386923`}, {-0.2476879670519132, \
-0.3761994867685855}, {-0.24190681238645306`, -0.385174711217556}, \
{-0.23602573498675308`, -0.3939908706110655}, {-0.23004994532575565`, \
-0.4026489655535965}, {-0.22398445875422998`, -0.41115010251995104`}, \
{-0.21783410013954155`, -0.4194954820715907}, {-0.21160350880454587`, \
-0.4276863879591887}, {-0.2052971436953205, -0.43572417706581656`}, \
{-0.19891928871526346`, -0.44361027014446425`}, {-0.1924740581711243, \
-0.4513461433035569}, {-0.1859654022838227, -0.4589333201946557}, \
{-0.17939711272348363`, -0.4663733648574941}, {-0.17277282813401956`, \
-0.4736678751788052}, {-0.16609603961786173`, -0.4808184769229612}, \
{-0.15937009615613237`, -0.48782681829419056`}, {-0.1525982099437037, \
-0.4946945649920162}, {-0.1457834616222523, -0.501423395723508}, \
{-0.13892880539763666`, -0.508014998137932}, {-0.13203707403073983`, \
-0.514471065151372}, {-0.12511098369337142`, -0.5207932916308717}, \
{-0.11815313868294974`, -0.5269833714095773}, {-0.11116603599151938`, \
-0.5330429946062321}, {-0.10415206972623667`, -0.5389738452241849}, \
{-0.09711353537980125, -0.5447775990068008}, {-0.09005263395045525, \
-0.5504559215278116}, {-0.08297147591213579, -0.5560104664967084}, \
{-0.07587208503617265, -0.5614428742607527}, {-0.06875640206659199, \
-0.5667547704865762}, {-0.06162628825163489, -0.5719477650056408}, \
{-0.05448352873454312, -0.5770234508090555}, {-0.04732983580701683, \
-0.5819834031783849}, {-0.04016685202902282, -0.5868291789401503}, \
{-0.03299615321883823, -0.5915623158327089}, {-0.02581925131736256, \
-0.5961843319751203}, {-0.018637597130829495`, -0.6006967254284562}, \
{-0.011452582956106449`, -0.6051009738407992}, {-0.004265545092790466, \
-0.6093985341679068}, {0.002922233753699901, -0.6135908424621868}, {
      0.01010952217387116, -0.6176793137232539}, {
      0.017295137856628895`, -0.6216653418039096}, {
      0.024477945488439675`, -0.6255502993659138}, {
      0.031656854745226295`, -0.6293355378804015}, {
      0.03883081837313755, -0.6330223876682487}, {
      0.0459988303544379, -0.6366121579760939}, {
      0.053159924154872785`, -0.6401061370841077}, {
      0.06031317104898037, -0.6435055924419405}, {
      0.06745767851993828, -0.6468117708296027}, {
      0.07459258873065389, -0.6500258985403181}, {
      0.08171707706292683, -0.6531491815826596}, {
      0.08883035072163283, -0.6561828058995226}, {
      0.09593164740099691, -0.6591279376017125}, {
      0.10302023401014193`, -0.6619857232141291}, {
      0.11009540545521379`, -0.6647572899327197}, {
      0.1171564834754977, -0.6674437458905416}, {
      0.1242028155310505, -0.6700461804314346}, {
      0.13123377373948075`, -0.6725656643899451}, {
      0.13824875385961283`, -0.6750032503762767}, {
      0.1452471743198717, -0.6773599730651606}, {
      0.1522284752893223, -0.6796368494876502}, {
      0.15919211778939116`, -0.6818348793249419}, {
      0.16613758284438834`, -0.6839550452034177}, {
      0.17306437066903407`, -0.6859983129901869}, {
      0.17997199989127827`, -0.6879656320884815}, {
      0.18686000680878062`, -0.6898579357323266}, {
      0.19372794467749546`, -0.6916761412799729}, {
      0.2005753830308789, -0.6934211505056351}, {
      0.2074019070283059, -0.6950938498891306}, {
      0.21420711683135138`, -0.696695110903062}, {
      0.22099062700665395`, -0.6982257902972313}, {
      0.22775206595414152`, -0.6996867303800088}, {
      0.234491075359456, -0.7010787592964209}, {
      0.2412073096694706, -0.702402691302749}, {
      0.2479004355898447, -0.7036593270374629}, {
      0.25457013160361364`, -0.7048494537883393}, {
      0.2612160875098565, -0.7059738457556383}, {
      0.26783800398153235`, -0.7070332643112346}, {
      0.2744355921416185, -0.708028458253618}, {
      0.28100857315672484`, -0.7089601640586977}, {
      0.28755667784739897`, -0.7098291061263573}, {
      0.29407964631437344`, -0.7106359970227277}, {
      0.30057722758004257`, -0.7113815377181509}, {
      0.30704917924448966`, -0.7120664178208256}, {
      0.313495267155418, -0.7126913158061318}, {
      0.31991526509137014`, -0.7132568992416435}, {
      0.326308954457648, -0.7137638250078452}, {
      0.33267612399437496`, -0.7142127395145758}, {
      0.33901656949616704`, -0.7146042789132304}, {
      0.3453300935429057, -0.7149390693047561}, {
      0.3516165052411282, -0.715217726943483}, {
      0.35787561997557377`, -0.7154408584368348}, {
      0.364107259170447, -0.715609060940971}, {
      0.3703112500599781, -0.71572292235241}, {
      0.3764874254678807, -0.7157830214956923}, {
      0.38263562359532616`, -0.7157899283071393}, {
      0.38875568781707004`, -0.7157442040147693}, {
      0.39484746648538466`, -0.7156464013144328}, {
      0.4009108127414663, -0.7154970645422287}, {
      0.40694558433400085`, -0.7152967298432685}, {
      0.41295164344458757`, -0.7150459253368516}, {
      0.41892885651973183`, -0.7147451712781187}, {
      0.42487709410913355`, -0.7143949802162496}, {
      0.4307962307100079, -0.7139958571492718}, {
      0.43668614461718885`, -0.7135482996755469}, {
      0.4425467177787755, -0.7130527981420003}, {
      0.44837783565709327`, -0.7125098357891626}, {
      0.45417938709475136`, -0.7119198888930862}, {
      0.4599512641855882, -0.7112834269042052}, {
      0.4656933621503049, -0.710600912583201}, {
      0.4714055792165971, -0.7098728021339407}, {
      0.4770878165036021, -0.7090995453335492}, {
      0.48273997791048756`, -0.708281585659681}, {
      0.4883619700090151, -0.7074193604150505}, {
      0.4939537019399195, -0.7065133008492849}, {
      0.4995150853129511, -0.7055638322781582}, {
      0.5050460341104351, -0.7045713742002673}, {
      0.5105464645942094, -0.7035363404112062}, {
      0.5160162952158052, -0.7024591391152989}, {
      0.5214554465297453, -0.7013401730349449}, {
      0.5268638411098353, -0.7001798395176341}, {
      0.5322414034683318, -0.6989785306406848}, {
      0.5375880599778744, -0.6977366333137592}, {
      0.542903738796075, -0.6964545293792072}, {
      0.5481883697926605, -0.6951325957102906}, {
      0.5534418844790701, -0.6937712043073394}, {
      0.5586642159404134, -0.692370722391886}, {
      0.5638552987696979, -0.6909315124988292}, {
      0.5690150690042387, -0.6894539325666722}, {
      0.574143464064168, -0.6879383360258832}, {
      0.5792404226929635, -0.6863850718854215}, {
      0.584305884899919, -0.6847944848174753}, {
      0.5893397919044844, -0.6831669152404527}, {
      0.5943420860824039, -0.6815026994002693}, {
      0.5993127109135844, -0.679802169449973}, {
      0.6042516109316298, -0.6780656535277467}, {
      0.6091587316749781, -0.6762934758333283}, {
      0.6140340196395817, -0.6744859567028857}, {
      0.6188774222330728, -0.672643412682386}, {
      0.62368888773036, -0.6707661565994941}, {
      0.6284683652306009, -0.6688544976340373}, {
      0.6332158046155015, -0.6669087413870708}, {
      0.6379311565088928, -0.6649291899485791}, {
      0.642614372237536, -0.6629161419638446}, {
      0.6472654037931146, -0.6608698926985189}, {
      0.6518842037953648, -0.6587907341024258}, {
      0.6564707254563077, -0.6566789548721299}, {
      0.6610249225455385, -0.6545348405122984}, {
      0.6655467493565366, -0.6523586733958882}, {
      0.6700361606739588, -0.6501507328231855}, {
      0.674493111741879, -0.6479112950797271}, {
      0.6789175582329409, -0.6456406334931306}, {
      0.6833094562183891, -0.6433390184888605}, {
      0.6876687621389489, -0.6410067176449569}, {
      0.6919954327765211, -0.6386439957457513}, {
      0.6962894252266646, -0.6362511148345966}, {
      0.7005506968718372, -0.6338283342656337}, {
      0.7047792053553666, -0.6313759107546192}, {
      0.7089749085561261, -0.6288940984288391}, {
      0.713137764563889, -0.626383148876129}, {
      0.717267731655337, -0.6238433111930237}, {
      0.7213647682706985, -0.6212748320320599}, {
      0.7254288329909951, -0.6186779556482501}, {
      0.7294598845158731, -0.6160529239447509}, {
      0.7334578816419983, -0.6133999765177446}, {
      0.7374227832419961, -0.6107193507005549}, {
      0.741354548243913, -0.6080112816070152}, {
      0.7452531356111851, -0.6052760021741087}, {
      0.7491185043230907, -0.6025137432038992}, {
      0.752950613355672, -0.59972473340477}, {
      0.7567494216631083, -0.596909199431989}, {
      0.7605148881595226, -0.5940673659276173}, {
      0.7642469717012074, -0.5911994555597782}, {
      0.767945631069253, -0.5883056890613019}, {
      0.771610824952564, -0.5853862852677647}, {
      0.7752425119312486, -0.5824414611549362}, {
      0.7788406504603679, -0.5794714318756504}, {
      0.7824051988540305, -0.5764764107961168}, {
      0.78593611526982, -0.573456609531685}, {
      0.7894333576935428, -0.570412237982079}, {
      0.7928968839242833, -0.5673435043661134}, {
      0.7963266515597548, -0.564250615255907}, {
      0.7997226179819358, -0.5611337756106068}, {
      0.803084740342978, -0.5579931888096357}, {
      0.8064129755513788, -0.554829056685477}, {
      0.8097072802584037, -0.5516415795560092}, {
      0.8129676108447522, -0.5484309562564028}, {
      0.8161939234074556, -0.5451973841705916}, {
      0.8193861737469966, -0.5419410592623312}, {
      0.8225443173546435, -0.5386621761058559}, {
      0.8256683093999879, -0.5353609279161465}, {
      0.828758104718679, -0.5320375065788194}, {
      0.8318136578003444, -0.5286921026796487}, {
      0.8348349227766915, -0.5253249055337332}, {
      0.8378218534097785, -0.5219361032143176}, {
      0.8407744030804509, -0.51852588258128}, {
      0.8436925247769321, -0.5150944293092952}, {
      0.8465761710835645, -0.5116419279156847}, {
      0.8494252941696919, -0.5081685617879639}, {
      0.8522398457786763, -0.5046745132110946}, {
      0.8550197772170433, -0.5011599633944562}, {
      0.8577650393437496, -0.49762509249854137`}, {
      0.8604755825595652, -0.49407007966138855`}, {
      0.8631513567965655, -0.4904951030247598}, {
      0.8657923115077256, -0.48690033976007274`}, {
      0.8683983956566135, -0.48328596609409713`}, {
      0.8709695577071733, -0.4796521573344236}, {
      0.8735057456135961, -0.47599908789471507`}, {
      0.8760069068102698, -0.47232693131974807`}, {
      0.8784729882018063, -0.46863586031025445`}, {
      0.8809039361531373, -0.4649260467475706}, {
      0.8832996964796762, -0.46119766171810384`}, {
      0.8856602144375398, -0.4574508755376238}, {
      0.8879854347138257, -0.4536858577753877}, {
      0.8902753014169404, -0.44990277727810746`}, {
      0.8925297580669737, -0.446101802193767}, {
      0.894748747586114, -0.4422830999952982}, {
      0.8969322122891021, -0.4384468375041234}, {
      0.899080093873717, -0.43459318091357246`}, {
      0.9011923334112905, -0.4307222958121827}, {
      0.9032688713372465, -0.42683434720688923`}, {
      0.9053096474416613, -0.4229294995461144}, {
      0.9073146008598392, -0.41900791674276344`}, {
      0.9092836700629019, -0.4150697621971346}, {
      0.9112167928483863, -0.4111151988197516}, {
      0.913113906330847, -0.40714438905412637`}, {
      0.9149749469324604, -0.4031574948994594}, {
      0.9167998503736264, -0.399154677933286}, {
      0.9185885516635647, -0.3951360993340762}, {
      0.9203409850909003, -0.39110191990379556`}, {
      0.922057084214238, -0.3870523000904357}, {
      0.9237367818527189, -0.38298740001052106`}, {
      0.9253800100765585, -0.37890737947160097`}, {
      0.9269867001975609, -0.3748123979947337}, {
      0.9285567827596078, -0.3707026148369715}, {
      0.9300901875291171, -0.36657818901385325`}, {
      0.9315868434854689, -0.3624392793219139}, {
      0.933046678811397, -0.3582860443612174}, {
      0.9344696208833401, -0.3541186425579224}, {
      0.9358555962617516, -0.3499372321868874}, {
      0.9372045306813651, -0.3457419713943244}, {
      0.938516349041411, -0.3415330182205089}, {
      0.9397909753957828, -0.33731053062255456`}, {
      0.9410283329431498, -0.33307466649726003`}, {
      0.9422283440170124, -0.3288255837040376}, {
      0.9433909300756989, -0.32456344008793103`}, {
      0.9445160116922988, -0.3202883935027315}, {
      0.9456035085445321, -0.3160006018342002}, {
      0.9466533394045495, -0.3117002230234059}, {
      0.9476654221286629, -0.3073874150901872}, {
      0.9486396736470016, -0.3030623361567472}, {
      0.9495760099530933, -0.2987251444713906}, {
      0.9504743460933653, -0.2943759984324114}, {
      0.951334596156565, -0.2900150566121415}, {
      0.9521566732630956, -0.2856424777811685}, {
      0.9529404895542656, -0.2812584209327333}, {
      0.9536859561814488, -0.276863045307316}, {
      0.9543929832951521, -0.27245651041742125`}, {
      0.9550614800339893, -0.26803897607257193`}, {
      0.9556913545135574, -0.2636106024045215}, {
      0.9562825138152135, -0.25917154989269614`}, {
      0.9568348639747498, -0.2547219793898762}, {
      0.9573483099709643, -0.250262052148128}, {
      0.9578227557141238, -0.24579192984499765`}, {
      0.9582581040343179, -0.24131177460997622`}, {
      0.9586542566697015, -0.23682174905124967`}, {
      0.9590111142546227, -0.23232201628274357`}, {
      0.9593285763076338, -0.2278127399514751}, {
      0.959606541219384, -0.22329408426522412`}, {
      0.9598449062403908, -0.21876621402053578`}, {
      0.9600435674686868, -0.21422929463106705`}, {
      0.9602024198373423, -0.20968349215629006`}, {
      0.9603213571018585, -0.20512897333056546`}, {
      0.9604002718274308, -0.200565905592599}, {
      0.9604390553760797, -0.19599445711529526`}, {
      0.960437597893647, -0.19141479683602244`}, {
      0.9603957882966555, -0.18682709448730267`}, {
      0.9603135142590292, -0.18223152062794237`}, {
      0.9601906621986732, -0.1776282466746181}, {
      0.9600271172639104, -0.1730174449339328}, {
      0.9598227633197735, -0.16839928863495893`}, {
      0.9595774829341501, -0.16377395196228395`}, {
      0.9592911573637795, -0.15914161008957547`}, {
      0.9589636665400981, -0.15450243921368284`}, {
      0.9585948890549344, -0.14985661658929253`}, {
      0.9581847021460479, -0.1452043205641556}, {
      0.9577329816825142, -0.14054573061490555`}, {
      0.9572396021499522, -0.1358810273834854}, {
      0.9567044366355929, -0.13121039271420368`}, {
      0.9561273568131886, -0.12653400969143905`}, {
      0.9555082329277601, -0.12185206267801445`}, {
      0.9548469337801819, -0.11716473735426143`}, {
      0.9541433267116038, -0.11247222075779678`}, {
      0.9533972775877074, -0.10777470132403354`}, {
      0.9526086507827973, -0.10307236892744942`}, {
      0.9517773091637259, -0.09836541492363621}, {
      0.9509031140736512, -0.09365403219215448}, {
      0.9499859253156266, -0.08893841518021865}, {
      0.9490256011360232, -0.08421875994723796}, {
      0.9480219982077831, -0.07949526421024004}, {
      0.9469749716135055, -0.07476812739020436}, {
      0.9458843748283635, -0.0700375506593334}, {
      0.9447500597028534, -0.06530373698929083}, {
      0.9435718764453765, -0.06056689120043626}, {
      0.9423496736046542, -0.05582722001208745}, {
      0.9410832980519765, -0.051084932093841524`}, {
      0.9397725949632875, -0.046340238117987965`}, {
      0.9384174078011049, -0.041593350813046964`}, {
      0.9370175782962811, -0.036844485018467855`}, {
      0.9355729464296015, -0.0320938577405235}, {
      0.9340833504132282, -0.027341688209437456`}, {
      0.9325486266719866, -0.022588197937782106`}, {
      0.9309686098245019, -0.017833610780187057`}, {
      0.9293431326641864, -0.013078152994398368`}, {
      0.9276720261400825, -0.008322053303730504}, {
      0.925955119337564, -0.0035655429609542603`}, {0.9241922394589035, 
      0.0011911441863346906`}, {0.9223832118037067, 0.00594777162882054}, {
      0.9205278597492235, 0.010704100126997728`}, {0.9186260047305378, 
      0.0154598876410778}, {0.9166774662206472, 0.0202148892592219}, {
      0.9146820617104353, 0.024968857124046413`}, {0.9126396066885488, 
      0.029721540357347438`}, {0.9105499146211848, 0.03447268498298808}, {
      0.9084127969317998, 0.03922203384789054}, {0.9062280629807488, 
      0.04396932654107301}, {0.9039955200448678, 0.04871429931066954}, {
      0.9017149732970094, 0.05345668497886865}, {0.8993862257855453, 
      0.05819621285470446}, {0.8970090784138491, 0.06293260864463177}, {
      0.8945833299197757, 0.06766559436081415}, {0.8921087768551509, 
      0.07239488822705172}, {0.8895852135652899, 0.07712020458227249}, {
      0.8870124321685636, 0.08184125378150899}, {0.8843902225360317, 
      0.08655774209427847}, {0.8817183722711628, 0.09126937160028283}, {
      0.8789966666896668, 0.09597584008234085}, {0.8762248887994621, 
      0.10067684091646262`}, {0.8734028192808043, 0.10537206295897265`}, {
      0.870530236466604, 0.11006119043058493`}, {0.867606916322964, 
      0.11474390279732964`}, {0.8646326324299681, 0.11941987464822776`}, {
      0.8616071559627543, 0.12408877556960594`}, {0.8585302556729102, 
      0.12875027001594033`}, {0.8554016978702296, 0.13340401717711373`}, {
      0.8522212464048706, 0.13804967084196665`}, {0.8489886626499611, 
      0.142686879258018}, {0.8457037054846973, 0.14731528498722735`}, {
      0.8423661312779879, 0.15193452475766497`}, {0.8389756938726959, 
      0.1565442293109523}, {0.835532144570537, 0.161144023245329}, {
      0.8320352321176947, 0.16573352485419893`}, {0.828484702691218, 
      0.1703123459600005}, {0.8248802998862714, 0.17488009174324248`}, {
      0.8212217647043114, 0.17943636056653958`}, {0.8175088355422682, 
      0.18398074379347618`}, {0.8137412481828175, 0.18851282560212054`}, {
      0.8099187357858316, 0.19303218279300502`}, {0.8060410288811063, 
      0.19753838459138062`}, {0.8021078553624639, 0.20203099244354794`}, {
      0.7981189404833433, 0.20650955980705824`}, {0.794074006853991, 
      0.2109736319345714}, {0.7899727744403765, 0.2154227456511492}, {
      0.785814960564964, 0.21985642912475428`}, {0.7816002799094798, 
      0.22427420162971654`}, {0.7773284445198242, 0.2286755733029197}, {
      0.772999163813287, 0.23306004489245208`}, {0.7686121445882348, 
      0.23742710749845525`}, {0.7641670910364505, 0.24177624230589528`}, {
      0.7596637047583176, 0.24610692030897047`}, {0.7551016847810511, 
      0.2504186020268588}, {0.7504807275801957, 0.25471073721049814`}, {
      0.7458005271046202, 0.25898276454008}, {0.7410607748052571, 
      0.26323411131292646`}, {0.7362611596678493, 0.2674641931214076}, {
      0.731401368249985, 0.2716724135205446}, {0.7264810847227186, 
      0.27585816368493016`}, {0.7214999909170966, 0.28002082205458545`}, {
      0.7164577663759286, 0.2841597539693583}, {0.7113540884111624, 
      0.28827431129145376`}, {0.7061886321672513, 0.2923638320156739}, {
      0.7009610706909225, 0.29642763986692805`}, {0.6956710750077866, 
      0.30046504388456025`}, {0.6903183142062524, 0.304475337993025}, {
      0.6849024555292477, 0.30845780055842587`}, {0.6794231644742742, 
      0.31241169393041585`}, {0.6738801049023659, 0.3163362639689418}, {
      0.6682729391565515, 0.3202307395552979}, {0.662601328190467, 
      0.32409433208693633`}, {0.6568649317078055, 0.3279262349554668}, {
      0.6510634083133355, 0.33172562300725744`}, {0.6451964156762721, 
      0.3354916519860342}, {0.639263610706833, 0.33922345795685666`}, {
      0.6332646497468721, 0.34292015671083176`}, {0.6271991887755388, 
      0.3465808431499099}, {0.6210668836309792, 0.35020459065109}, {
      0.6148673902491597, 0.3537904504093449}, {0.6086003649209704, 
      0.35733745075856166`}, {0.6022654645688403, 0.3608445964697765}, {
      0.5958623470441826, 0.3643108680259707}, {0.5893906714470747, 
      0.3677352208726793}, {0.5828500984696755, 0.3711165846436549}, {
      0.5762402907649822, 0.3744538623608166}, {0.569560913342638, 
      0.37774592960770953`}, {0.5628116339936196, 0.3809916336756915}, {
      0.5559921237457541, 0.38418979268206443`}, {0.5491020573521523, 
      0.38733919465936617`}, {0.5421411138147808, 0.3904385966150449}, {
      0.5351089769455513, 0.3934867235607463}, {0.5280053359674641, 
      0.3964822675104597}, {0.5208298861585143, 0.3994238864467864}, {
      0.5135823295412534, 0.402310203254625}, {0.506262375621094, 
      0.40513980462159854`}, {0.4988697421766536, 0.40791123990459466`}, {
      0.4914041561056531, 0.4106230199618428}, {0.48386535433012495`, 
      0.41327361595001666`}, {0.47625308476493317`, 0.4158614580859281}, {
      0.46856710735387613`, 0.4183849343724709}, {0.46080719517792496`, 
      0.4208423892885808}, {0.4529731356404512, 0.4232321224431062}, {
      0.4450647317346153, 0.42555238719262967`}, {0.4370818033984236, 
      0.4278013892234539}, {0.4290241889633193, 0.42997728509816113`}, {
      0.42089174670254575`, 0.43207818076738286`}, {0.41268435648591995`, 
      0.43410213004767595`}, {0.40440192154806665`, 0.43604713306669773`}, {
      0.3960443703776035, 0.4379111346772109}, {0.3876116587352238, 
      0.439692022841832}, {0.3791037718091007, 0.4413876269908735}, {
      0.3705207265165347, 0.4429957163561195}, {0.36186257396128096`, 
      0.4445139982839324}, {0.3531294020565249, 0.4459401165317129}, {
      0.34432133832402223`, 0.4472716495524389}, {0.33543855288047686`, 
      0.4485061087727996}, {0.32648126162279717`, 0.4496409368713259}, {
      0.3174497296244417, 0.45067350606390927`}, {0.30834427475563214`, 
      0.451601116405207}, {0.2991652715407711, 0.45242099411566433`}, {
      0.2899131552669414, 0.4531302899452575}, {0.2805884263578771, 
      0.45372607758658484`}, {0.2711916550282676, 0.45420535215162355`}, {
      0.2617234862336706, 0.45456502872833987`}, {0.25218464493165477`, 
      0.4548019410354037}, {0.24257594167003954`, 0.45491284019553446`}, {
      0.23289827851823075`, 0.45489439365050255`}, {0.22315265535763526`, 
      0.4547431842435515}, {0.21334017654694248`, 0.4544557094979983}, {
      0.2034620579776494, 0.4540283811240325}, {0.19351963453453364`, 
      0.45345752478927526`}, {0.18351436797479814`, 0.45273938019249105`}, {
      0.17344785523826478`, 0.45187010148397033`}, {0.16332183719921742`, 
      0.45084575808052424`}, {0.1531382078682213, 0.4496623359277467}, {
      0.14289902404939028`, 0.44831573926719437`}, {0.1326065154550541, 
      0.44680179297138795`}, {0.12226309527549609`, 0.44511624551501705`}, {
      0.11187137119628471`, 0.44325477265638863`}, {0.10143415684960014`, 
      0.44121298190892816`}, {0.0909544836787395, 0.438986417888342}, {
      0.0804356131865538, 0.4365705686267675}, {0.06988104952879802, 
      0.4339608729507372}, {0.059294552402143444`, 0.43115272902489415`}, {
      0.04868015016379121, 0.4281415041679173}, {0.03804215310513275, 
      0.4249225460507923}, {0.027385166785643157`, 0.42149119539011254`}, {
      0.01671410531510921, 0.41784280025017395`}, {0.006034204452370212, 
      0.41397273206684765`}, {-0.004648965632978539, 
      0.40987640350313714`}, {-0.015329488112897477`, 
      0.4055492882404633}, {-0.026001087969145707`, 
      0.400986942800538}, {-0.03665712140552706, 
      0.39618503047961556`}, {-0.047290566459168154`, 
      0.39113934745935947`}, {-0.057894015093055945`, 
      0.3858458511359289}, {-0.06845966707877983, 
      0.38030069068060574`}, {-0.07897932600375952, 
      0.3745002398108188}, {-0.08944439776010128, 
      0.3684411317093558}, {-0.09984589189132591, 
      0.36212029598159784`}, {-0.11017442618707254`, 
      0.3555349974856879}, {-0.12042023492285661`, 
      0.3486828768088278}, {-0.13057318114026123`, 
      0.34156199209491006`}, {-0.14062277335070858`, 
      0.33417086185532857`}, {-0.1505581870213233, 
      0.3265085083174577}, {-0.16036829116260184`, 
      0.31857450078581007`}, {-0.1700416802831036, 
      0.31036899841170695`}, {-0.17956671190501705`, 
      0.3018927916913767}, {-0.18893154974560672`, 
      0.29314734194320574`}, {-0.19812421256329, 
      0.2841348179562895}, {-0.20713262854437997`, 
      0.2748581289586509}, {-0.2159446949693128, 
      0.2653209530287812}, {-0.2245483427485152, 
      0.25552776007264366`}, {-0.23293160526215728`, 
      0.2454838285136544}, {-0.2410826907801664, 
      0.2351952548983442}, {-0.24899005758529888`, 
      0.22466895570724804`}, {-0.25664249077973117`, 
      0.2139126607795145}, {-0.2640291796318704, 
      0.2029348979095551}, {-0.27113979422215606`, 
      0.19174496835175134`}, {-0.27796456008123677`, 
      0.18035291316991967`}, {-0.28449432948668807`, 
      0.16876947058527236`}, {-0.29072064809945397`, 
      0.157006024701882}, {-0.2966358156805129, 
      0.14507454621294946`}, {-0.30223293973167314`, 
      0.13298752590473645`}, {-0.30750598104925125`, 
      0.1207579019681766}, {-0.31244979036066606`, 
      0.1083989822919859}, {-0.3170601354245665, 
      0.09592436303797304}, {-0.32133371820618045`, 
      0.08334784488353336}, {-0.3252681819811953, 
      0.07068334835461501}, {-0.3288621084632762, 
      0.057944829663900944`}, {-0.33211500528214594`, 
      0.04514619841519476}, {-0.3350272843517418, 
      0.03230123843999443}, {-0.33760023185351756`, 
      0.01942353290190721}, {-0.3398359707125208, 
      0.006526394646288486}, {-0.3417374165595871, -0.0063771974054382015`}, \
{-0.3433082282501615, -0.019274655206312215`}, {-0.344552754049239, \
-0.0321538314604556}, {-0.3454759745948952, -0.045003062193816126`}, \
{-0.3460834437235038, -0.057811201962416424`}, {-0.3463812281827508, \
-0.07056765180018093}, {-0.3463758471793396, -0.08326238014667303}, \
{-0.3460742126125033, -0.0958859371105808}, {-0.34548357073766334`, \
-0.10842946251656108`}, {-0.3446114458920016, -0.12088468825128955`}, \
{-0.34346558679996364`, -0.1332439354704356}, {-0.3420539158656888, \
-0.1455001072536918}, {-0.3403844817541969, -0.15764667730234813`}, \
{-0.33846541546619263`, -0.16967767526591074`}, {-0.33630489002417563`, \
-0.1815876692637324}, {-0.3339110838111019, -0.19337174613730057`}, \
{-0.3312921475374859, -0.20502548993131606`}, {-0.3284561747584521, \
-0.21654495905933843`}, {-0.3254111758183668, -0.22792666256462596`}, \
{-0.32216505506656684`, -0.2391675358405971}, {-0.3187255911624341, \
-0.25026491612950263`}, {-0.3151004202706346, -0.26121651807353596`}, \
{-0.3112970219366921, -0.27202040955055784`}, {-0.30732270742816326`, \
-0.28267498798745433`}, {-0.3031846103265297, -0.29317895730826216`}, \
{-0.2988896791586031, -0.30353130564177444`}, {-0.2944446718629213, \
-0.31373128388444516`}, {-0.28985615189556097`, -0.32377838518898305`}, \
{-0.2851304857903674, -0.333672325426928}, {-0.2802738420002619, \
-0.34341302465454293`}, {-0.27529219085858275`, -0.353000589595294}, \
{-0.2701913055119761, -0.36243529713877726`}, {-0.26497676368887996`, \
-0.37171757884491624`}, {-0.2596539501799166, -0.3808480064333379}, \
{-0.2542280599183436, -0.38982727823077895`}, {-0.24870410155998954`, \
-0.3986562065439378}, {-0.24308690147273398`, -0.40733570592114643`}, \
{-0.2373811080555203, -0.41586678226338336`}, {-0.23159119631609898`, \
-0.42425052274329744`}, {-0.2257214726451707, -0.43248808648989834`}, \
{-0.21977607973234903`, -0.44058069599624194`}, {-0.21375900157641134`, \
-0.4485296292076728}, {-0.207674068548684, -0.45633621224886567`}, \
{-0.20152496247414944`, -0.46400181274894053`}, {-0.19531522170001006`, \
-0.4715278337252264}, {-0.18904824612603766`, -0.4789157079877485}, \
{-0.1827273021751204, -0.48616689302815586`}, {-0.17635552768603463`, \
-0.4932828663585386}, {-0.16993593671365742`, -0.5002651212673718}, \
{-0.16347142422463645`, -0.507115162961628}, {-0.15696477067898518`, \
-0.5138345050659012}, {-0.15041864649020745`, -0.5204246664511595}, \
{-0.14383561635840927`, -0.5268871683674717}, {-0.137218143472458, \
-0.5332235318567337}, {-0.1305685935786263, -0.5394352754230267}, \
{-0.12388923891433708`, -0.5455239129397883}, {-0.11718226200662786`, \
-0.5514909517744371}, {-0.11044975933580013`, -0.5573378911124913}, \
{-0.10369374486542986`, -0.5630662204645291}, {-0.09691615344050612, \
-0.5686774183405798}, {-0.09011884405595083, -0.5741729510776943}, \
{-0.08330360299816726, -0.5795542718075325}, {-0.07647214686258093, \
-0.5848228195518208}, {-0.06962612545038296, -0.5899800184344807}, \
{-0.06276712454787392, -0.595027277000111}, {-0.05589666859194271, \
-0.5999659876293298}, {-0.04901622322530835, -0.6047975260422375}, \
{-0.042127197745208345`, -0.609523250881973}, {-0.03523094744924168, \
-0.6141445033709866}, {-0.028328775882072825`, -0.618662607033255}, \
{-0.021421936986678318`, -0.6230788674762258}, {-0.014511637163775092`, \
-0.627394572226791}, {-0.007599037243011707, -0.631610990616067}, \
{-0.0006852543694334046, -0.6357293737081943}, {
      0.0062286361913481865`, -0.6397509542687755}, {
      0.013141599325937675`, -0.6436769467689418}, {
      0.020052638420641603`, -0.6475085474213792}, {
      0.026960793780506045`, -0.651246934244961}, {
      0.0338651411168561, -0.6548932671549227}, {
      0.04076479010039018, -0.6584486880757795}, {
      0.047658882976986006`, -0.6619143210744333}, {
      0.054546593243479276`, -0.6652912725111367}, {
      0.06142712438077934, -0.6685806312061922}, {
      0.06829970864178878, -0.6717834686204466}, {
      0.07516360589169437, -0.6749008390478222}, {
      0.08201810249829607, -0.6779337798182767}, {
      0.08886251027013692, -0.6808833115097368}, {
      0.09569616544029116, -0.6837504381676771}, {
      0.10251842769375936`, -0.6865361475311444}, {
      0.10932867923650777`, -0.689241411264135}, {
      0.11612632390427526`, -0.6918671851913358}, {
      0.12291078630935395`, -0.6944144095373379}, {
      0.12968151102362946`, -0.6968840091685099}, {
      0.13643796179624382`, -0.6992768938368047}, {
      0.14317962080431781`, -0.7015939584248393}, {
      0.14990598793524104`, -0.7038360831916564}, {
      0.1566165800991056, -0.7060041340186352}, {
      0.16331093056992485`, -0.7080989626550743}, {
      0.1699885883543418, -0.71012140696302}, {
      0.1766491175865904, -0.7120722911609567}, {
      0.1832920969485317, -0.7139524260660234}, {
      0.18991711911364043`, -0.715762609334451}, {
      0.1965237902138704, -0.7175036256999568}, {
      0.2031117293283773, -0.7191762472098604}, {
      0.20968056799312423`, -0.7207812334587137}, {
      0.216229949730442, -0.7223193318192681}, {
      0.2227595295976582, -0.7237912776706202}, {
      0.22926897375395197`, -0.7251977946234041}, {
      0.23575795904462904`, -0.7265395947419149}, {
      0.24222617260205087`, -0.7278173787630678}, {
      0.24867331146248622`, -0.7290318363121128}, {
      0.2550990821981882, -0.7301836461150382}, {
      0.261503200564032, -0.731273476207614}, {
      0.26788539115807947`, -0.7323019841410321}, {
      0.2742453870954669, -0.733269817184117}, {
      0.2805829296950385, -0.7341776125220859}, {
      0.2868977681781765, -0.7350259974518493}, {
      0.2931896593793042, -0.7358155895738487}, {
      0.29945836746756044`, -0.736546996980438}, {
      0.3057036636791699, -0.7372208184408168}, {
      0.3119253260600527, -0.7378376435825356}, {
      0.3181231392182399, -0.7383980530695935}, {
      0.3242968940856799, -0.7389026187771542}, {
      0.33044638768903983`, -0.7393519039629122}, {
      0.3365714229291249, -0.7397464634351416}, {
      0.3426718083685548, -0.740086843717467}, {
      0.3487473580273528, -0.740373583210393}, {
      0.35479789118611865`, -0.740607212349637}, {
      0.3608232321964721, -0.7407882537613082}, {
      0.3668232102984659, -0.740917222413979}, {
      0.3727976594446827, -0.7409946257676954}, {
      0.3787464181307418, -0.741020963919976}, {
      0.38466932923195385`, -0.7409967297488476}, {
      0.3905662398458739, -0.7409224090529681}, {
      0.39643700114051317`, -0.7407984806888889}, {
      0.4022814682079817, -0.7406254167055042}, {
      0.408099499923343, -0.740403682475744}, {
      0.4138909588084714, -0.740133736825557}, {
      0.4196557109007131, -0.7398160321602396}, {
      0.4253936256261585, -0.7394510145881587}, {
      0.4311045756773441, -0.7390391240419226}, {
      0.4367884368952078, -0.7385807943970503}, {
      0.4424450881551306, -0.7380764535881885}, {
      0.4480744112569044, -0.7375265237229292}, {
      0.45367629081847166`, -0.7369314211932753}, {
      0.4592506141732906, -0.7362915567848062}, {
      0.4647972712711846, -0.7356073357835895}, {
      0.4703161545825409, -0.7348791580808894}, {
      0.4758071590057297, -0.734107418275718}, {
      0.48127018177761943`, -0.733292505775277}, {
      0.48670512238706976`, -0.7324348048933359}, {
      0.492111882491288, -0.7315346949465921}, {
      0.49749036583494094`, -0.7305925503490581}, {
      0.5028404781719162, -0.7296087407045199}, {
      0.5081621271896342, -0.7285836308971085}, {
      0.5134552224358128, -0.7275175811800301}, {
      0.5187196752475945, -0.7264109472624926}, {
      0.5239553986829462, -0.7252640803948738}, {
      0.5291623074542458, -0.7240773274521681}, {
      0.5343403178639771, -0.722851031015753}, {
      0.5394893477424514, -0.721585529453513}, {
      0.5446093163874823, -0.7202811569983603}, {
      0.5497001445059421, -0.7189382438251878}, {
      0.5547617541571283, -0.7175571161262919}, {
      0.5597940686978767, -0.7161380961853}, {
      0.5647970127293543, -0.7146815024496384}, {
      0.569770512045472, -0.7131876496015739}, {
      0.574714493582858, -0.7116568486278638}, {
      0.5796288853723347, -0.7100894068880451}, {
      0.584513616491845, -0.7084856281813967}, {
      0.5893686170207748, -0.7068458128126052}, {
      0.5941938179956223, -0.7051702576561648}, {
      0.5989891513669645, -0.7034592562195419}, {
      0.6037545499576747, -0.7017130987051323}, {
      0.6084899474223465, -0.699932072071042}, {
      0.6131952782078801, -0.6981164600907168}, {
      0.6178704775151902, -0.6962665434114507}, {
      0.6225154812619946, -0.6943825996117978}, {
      0.627130226046646, -0.6924649032579161}, {
      0.6317146491129689, -0.6905137259588657}, {
      0.6362686883160662, -0.6885293364208901}, {
      0.640792282089061, -0.6865120005007017}, {
      0.6452853694107409, -0.684461981257798}, {
      0.6497478897740715, -0.6823795390058303}, {
      0.6541797831555503, -0.6802649313630482}, {
      0.6585809899853687, -0.6781184133018427}, {
      0.6629514511183566, -0.6759402371974089}, {
      0.6672911078056782, -0.6737306528755501}, {
      0.6715999016672562, -0.6714899076596447}, {
      0.6758777746648961, -0.6692182464167952}, {
      0.6801246690760862, -0.6669159116031801}, {
      0.6843405274684505, -0.6645831433086278}, {
      0.6885252926748308, -0.6622201793004318}, {
      0.6926789077689741, -0.6598272550664256}, {
      0.6968013160418073, -0.6574046038573363}, {
      0.7008924609782743, -0.6549524567284335}, {
      0.7049522862347194, -0.6524710425804914}, {
      0.7089807356167941, -0.6499605882000827}, {
      0.712977753057871, -0.6474213182992185}, {
      0.7169432825979456, -0.6448534555543518}, {
      0.7208772683630083, -0.6422572206447619}, {
      0.7247796545448705, -0.6396328322903321}, {
      0.7286503853814285, -0.6369805072887395}, {
      0.7324894051373472, -0.6343004605520698}, {
      0.7362966580851518, -0.6315929051428724}, {
      0.7400720884867096, -0.6288580523096701}, {
      0.7438156405750892, -0.626096111521938}, {
      0.7475272585367826, -0.6233072905045643}, {
      0.7512068864942772, -0.620491795271808}, {
      0.7548544684889642, -0.6176498301607654}, {
      0.7584699484643705, -0.6147815978643593}, {
      0.7620532702497036, -0.6118872994638633}, {
      0.765604377543695, -0.6089671344609741}, {
      0.7691232138987331, -0.6060213008094425}, {
      0.7726097227052723, -0.6030499949462781}, {
      0.7760638471765098, -0.6000534118225365}, {
      0.7794855303333174, -0.5970317449337023}, {
      0.7828747149894195, -0.5939851863496787}, {
      0.7862313437368071, -0.5909139267443951}, {
      0.789555358931378, -0.5878181554250438}, {
      0.7928467026787939, -0.5846980603609557}, {
      0.796105316820546, -0.5815538282121271}, {
      0.7993311429202196, -0.5783856443574065}, {
      0.8025241222499487, -0.5751936929223529}, {
      0.8056841957770544, -0.5719781568067749}, {
      0.8088113041508559, -0.5687392177119606}, {
      0.8119053876896491, -0.5654770561676086}, {
      0.8149663863678438, -0.5621918515584692}, {
      0.8179942398032516, -0.5588837821507056}, {
      0.8209888872445186, -0.5555530251179837}, {
      0.8239502675586946, -0.5521997565673015}, {
      0.8268783192189328, -0.5488241515645649}, {
      0.8297729802923127, -0.5454263841599204}, {
      0.8326341884277797, -0.5420066274128535}, {
      0.8354618808441953, -0.5385650534170606}, {
      0.8382559943184915, -0.535101833325104}, {
      0.8410164651739238, -0.5316171373728579}, {
      0.8437432292684154, -0.5281111349037545}, {
      0.8464362219829886, -0.5245839943928381}, {
      0.8490953782102769, -0.5210358834706349}, {
      0.8517206323431115, -0.5174669689468485}, {
      0.8543119182631781, -0.5138774168338869}, {
      0.8568691693297386, -0.5102673923702308}, {
      0.8593923183684107, -0.5066370600436508}, {
      0.8618812976600025, -0.5029865836142807}, {
      0.8643360389293965, -0.4993161261375553}, {
      0.8667564733344763, -0.49562584998702086`}, {
      0.8691425314550945, -0.49191591687702513`}, {
      0.8714941432820742, -0.48818648788529556`}, {
      0.873811238206241, -0.4844377234754126}, {
      0.8760937450074796, -0.48066978351918627`}, {
      0.8783415918438131, -0.4768828273189433}, {
      0.8805547062404973, -0.47307701362973287`}, {
      0.8827330150791278, -0.4692525006814575}, {
      0.8848764445867563, -0.4654094462009384}, {
      0.8869849203250086, -0.4615480074339207}, {
      0.8890583671792045, -0.4576683411670282}, {
      0.8910967093474719, -0.4537706037496733}, {
      0.8930998703298533, -0.4498549511159311}, {
      0.8950677729173999, -0.4459215388063843}, {
      0.8970003391812497, -0.44197052198994713`}, {
      0.8988974904616845, -0.43800205548567517`}, {
      0.9007591473571652, -0.43401629378456924`}, {
      0.9025852297133373, -0.43001339107138087`}, {
      0.9043756566120064, -0.42599350124642676`}, {
      0.9061303463600782, -0.42195677794742004`}, {
      0.9078492164784604, -0.4179033745713263}, {
      0.9095321836909217, -0.4138334442962519}, {
      0.9111791639129055, -0.4097471401033721}, {
      0.9127900722402941, -0.40564461479890784`}, {
      0.9143648229381196, -0.40152602103615814`}, {
      0.9159033294292197, -0.39739151133759637`}, {
      0.9174055042828325, -0.39324123811703904`}, {
      0.9188712592031287, -0.38907535370189433`}, {
      0.9203005050176769, -0.3848940103554996}, {
      0.9216931516658392, -0.3806973602995551}, {
      0.9230491081870933, -0.3764855557366634}, {
      0.9243682827092777, -0.37225874887298244`}, {
      0.9256505824367577, -0.36801709194100046`}, {
      0.9268959136385065, -0.3637607372224428}, {
      0.9281041816361016, -0.359489837071318}, {
      0.9292752907916298, -0.35520454393711304`}, {
      0.9304091444954996, -0.3509050103881466}, {
      0.9315056451541572, -0.34659138913508947`}, {
      0.9325646941777029, -0.3422638330546617}, {
      0.9335861919674044, -0.3379224952135157}, {
      0.9345700379031042, -0.33356752889231556`}, {
      0.9355161303305172, -0.3291990876100216}, {
      0.9364243665484163, -0.32481732514839107`}, {
      0.9372946427957023, -0.32042239557670454`}, {
      0.9381268542383544, -0.3160144532767286}, {
      0.9389208949562591, -0.3115936529679257}, {
      0.9396766579299133, -0.3071601497329213}, {
      0.9403940350269997, -0.3027140990432403}, {
      0.9410729169888297, -0.29825565678532295`}, {
      0.9417131934166523, -0.2937849792868321}, {
      0.942314752757825, -0.2893022233432641}, {
      0.9428774822918432, -0.2848075462448741}, {
      0.9434012681162259, -0.2803011058039289}, {
      0.943885995132254, -0.2757830603822997}, {
      0.9443315470305579, -0.27125356891940683`}, {
      0.9447378062765519, -0.26671279096053047`}, {
      0.9451046540957123, -0.2621608866855}, {
      0.945431970458695, -0.257598016937776}, {
      0.9457196340662907, -0.2530243432539385}, {
      0.9459675223342144, -0.24844002789359648`}, {
      0.9461755113777264, -0.2438452338697325}, {
      0.9463434759960797, -0.23924012497949823`}, {
      0.9464712896567948, -0.23462486583547568`}, {
      0.9465588244797548, -0.22999962189742035`}, {
      0.9466059512211201, -0.22536455950450232`}, {
      0.9466125392570598, -0.2207198459080621}, {
      0.9465784565672951, -0.216065649304898}, {
      0.9465035697184542, -0.21140213887110287`}, {
      0.9463877438472338, -0.20672948479646813`}, {
      0.9462308426433657, -0.2020478583194733}, {
      0.9460327283323848, -0.19735743176288045`}, {
      0.9457932616581967, -0.19265837856995283`}, {
      0.9455123018654404, -0.18795087334131766`}, {
      0.9451897066816467, -0.183235091872494}, {
      0.9448253322991853, -0.17851121119210656`}, {
      0.9444190333570024, -0.17377940960080748`}, {
      0.9439706629221429, -0.16903986671092852`}, {
      0.9434800724710574, -0.16429276348688654`}, {
      0.9429471118706892, -0.15953828228636632`}, {
      0.9423716293593417, -0.15477660690230485`}, {
      0.9417534715273213, -0.1500079226057026}, {
      0.9410924832973558, -0.14523241618928748`}, {
      0.9403885079047865, -0.1404502760120583}, {
      0.9396413868775287, -0.13566169204473522`}, {
      0.9388509600158048, -0.13086685591614544`}, {
      0.938017065371642, -0.12606596096057368`}, {
      0.9371395392281383, -0.12125920226610722`}, {
      0.9362182160784925, -0.1164467767240067}, {
      0.9352529286047984, -0.11162888307913481`}, {
      0.9342435076566011, -0.1068057219814757}, {
      0.9331897822292169, -0.10197749603877937`}, {
      0.932091579441813, -0.09714440987036611}, {
      0.9309487245152495, -0.09230667016212729}, {
      0.9297610407496827, -0.08746448572275986}, {
      0.928528349501929, -0.08261806754127342}, {
      0.9272504701625912, -0.07776762884580951}, {
      0.9259272201329471, -0.07291338516381454}, {
      0.9245584148016017, -0.0680555543836087}, {
      0.9231438675209045, -0.06319435681739502}, {
      0.9216833895831336, -0.05833001526575382}, {
      0.9201767901964486, -0.053462755083669544`}, {
      0.9186238764606155, -0.048592804248138395`}, {
      0.9170244533425055, -0.04372039342740698}, {
      0.9153783236513722, -0.03884575605189362}, {
      0.9136852880139109, -0.03396912838684598}, {
      0.9119451448491047, -0.029090749606790305`}, {
      0.9101576903428618, -0.024210861871829574`}, {
      0.9083227184224506, -0.019329710405849736`}, {
      0.9064400207307396, -0.014447543576695311`}, {
      0.9045093866002467, -0.009564612978377716}, {
      0.9025306030270088, -0.004681173515381872}, {0.9005034546442787, 
      0.00020251651086103124`}, {0.8984277236960593, 0.00508619531326441}, {
      0.896303190010485, 0.009969597528068096}, {0.8941296309730634, 
      0.014852454121435831`}, {0.8919068214997875, 0.01973449229359052}, {
      0.8896345340101341, 0.024615435380406583`}, {0.887312538399962, 
      0.029495002752375807`}, {0.8849406020143262, 0.034372909710860176`}, {
      0.8825184896202265, 0.03924886738154201}, {0.8800459633793077, 
      0.04412258260497853}, {0.8775227828205342, 0.048993757824164604`}, {
      0.8749487048128579, 0.05386209096900389}, {0.8723234835379067, 
      0.05872727533758513}, {0.8696468704627164, 0.06358899947415628}, {
      0.8669186143125356, 0.06844694704368556}, {0.8641384610437323, 
      0.07330079670289427}, {0.8613061538168347, 0.07815022196764176}, {
      0.8584214329697408, 0.08299489107653919}, {0.8554840359911331, 
      0.08783446685066318}, {0.8524936974941391, 0.09266860654923663}, {
      0.8494501491902801, 0.09749696172113832}, {0.8463531198637534, 
      0.1023191780520981}, {0.8432023353460976, 0.10713489520742903`}, {
      0.8399975184912934, 0.11194374667014224`}, {0.8367383891513573, 
      0.11674535957428449`}, {0.8334246641524864, 0.12153935453333226`}, {
      0.8300560572718214, 0.12632534546347024`}, {0.8266322792148959, 
      0.1311029394015752}, {0.823153037593846, 0.13587173631771957`}, {
      0.8196180369064598, 0.1406313289220022}, {0.8160269785161528, 
      0.14538130246550607`}, {0.812379560632957, 0.15012123453517545`}, {
      0.8086754782956248, 0.1548506948423968}, {0.8049144233549479, 
      0.15956924500505976`}, {0.8010960844584044, 0.1642764383228657}, {
      0.797220147036252, 0.1689718195456427}, {0.7932862932891938, 
      0.17365492463441634`}, {0.7892942021777528, 0.17832528051497643`}, {
      0.7852435494134998, 0.18298240482366931`}, {0.7811340074522888, 
      0.1876258056451358}, {0.7769652454896658, 0.1922549812417033}, {
      0.7727369294586279, 0.19686941977412975`}, {0.7684487220299203, 
      0.2014685990133864}, {0.7641002826150739, 0.2060519860431525}, {
      0.7596912673723983, 0.21061903695268472`}, {0.7552213292161606, 
      0.2151691965197098}, {0.7506901178291948, 0.21970189788297625`}, {
      0.7460972796792059, 0.22421656220408692`}, {0.7414424580390481, 
      0.22871259831822024`}, {0.7367252930112762, 0.23318940237333294`}, {
      0.7319454215572926, 0.2376463574574219}, {0.7271024775314285, 
      0.24208283321340743`}, {0.7221960917203278, 0.24649818544118343`}, {
      0.7172258918880221, 0.25089175568636385`}, {0.7121915028271146, 
      0.25526287081523724`}, {0.7070925464165188, 0.2596108425754239}, {
      0.7019286416862268, 0.2639349671417113}, {0.6966994048896191, 
      0.26823452464652614`}, {0.6914044495838558, 0.27250877869448104`}, {
      0.686043386718934, 0.2767569758604147}, {0.6806158247360302, 
      0.28097834517032566`}, {0.675121369675794, 0.2851720975645779}, {
      0.6695596252973001, 0.28933742534273704`}, {0.663930193208421, 
      0.29347350158937524`}, {0.6582326730084284, 0.29757947958016107`}, {
      0.652466662443693, 0.3016544921675303}, {0.6466317575774094, 
      0.30569765114521263`}, {0.6407275529743376, 0.309708046590867}, {
      0.6347536419016222, 0.31368474618605896`}, {0.628709616546824, 
      0.31762679451279235`}, {0.6225950682543755, 0.3215332123257866}, {
      0.6164095877817595, 0.3254029957996732}, {0.6101527655767978, 
      0.32923511575026565`}, {0.6038241920775341, 0.33302851682903917`}, {
      0.5974234580362996, 0.33678211668994185`}, {0.5909501548696591, 
      0.3404948051276435}, {0.5844038750360553, 0.34416544318631725`}, {
      0.5777842124430947, 0.34779286223803796`}, {0.5710907628865537, 
      0.35137586302987645`}, {0.5643231245233297, 0.35491321469876363`}, {
      0.5574808983807161, 0.3584036537532005}, {0.5505636889045463, 
      0.36184588302089454`}, {0.5435711045489278, 0.36523857056141473`}, {
      0.5365027584104778, 0.36858034854297406`}, {0.5293582689101742, 
      0.3718698120824732}, {0.5221372605261493, 0.37510551804797165`}, {
      0.5148393645809853, 0.37828598382279455`}, {0.5074642200873172, 
      0.381409686030538}, {0.5000114746558055, 0.3844750592202995}, {
      0.4924807854698274, 0.3874804945115407}, {0.48487182033152143`, 
      0.3904243381980856}, {0.47718425878414616`, 0.3933048903108704}, {
      0.4694177933160393, 0.3961204031391959}, {0.4615721306518242, 
      0.39886907971039}, {0.4536469931368853, 0.40154907222797237`}, {
      0.44564212022153116`, 0.4041584804686246}, {0.43755727005168665`, 
      0.4066953501385157}, {0.42939222117339776`, 0.40915767118981505`}, {
      0.4211467743589009, 0.41154337609854896`}, {0.4128207545624985, 
      0.4138503381053277}, {0.4044140130149975, 0.41607636942089143`}, {
      0.39592642946600365`, 0.41821921939890516`}, {0.38735791458392427`, 
      0.42027657267897583`}, {0.3787084125241123, 0.4222460473034834}, {
      0.3699779036761826, 0.424125192812513}, {0.36116640760214813`, 
      0.42591148832196135`}, {0.35227398617765115`, 0.42760234059077407`}, {
      0.34330074694920226`, 0.42919508208426244`}, {0.3342468467209807, 
      0.4306869690415612}, {0.3251124953853856, 0.4320751795565315}, {
      0.3158979600121534, 0.43335681168280393`}, {0.30660356921145887`, 
      0.4345288815752028}, {0.2972297177869851, 0.43558832168151596`}, {
      0.2877768716954673, 0.43653197900048296`}, {0.27824557332966543`, 
      0.4373566134239923}, {0.26863644714208557`, 0.4380588961838136}, {
      0.25895020562701976`, 0.4386354084257685}, {0.24918765567858472`, 
      0.4390826399370739}, {0.23934970534237365`, 0.4393969880556968}, {
      0.22943737097805736`, 0.4395747567939482}, {0.2194517848497329, 
      0.43961215621223854`}, {0.2093942031599719, 0.4395053020829214}, {
      0.19926601454230583`, 0.43925021588847873`}, {0.18906874902523768`, 
      0.4388428252029543}, {0.1788040874787128, 0.4382789645105152}, {
      0.16847387155123505`, 0.4375543765203113}}]}, 
   {RGBColor[1, 1, 0], DiskBox[{0, 0}, 0.03]}},
  AspectRatio->Automatic,
  Background->RGBColor[0.6, 0.6, 0.6],
  ImageSize->{304.5, Automatic}]], "Output",
 CellChangeTimes->{3.4012012962196293`*^9, 3.4012769255407543`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["(C) Kepler's Laws",
 FontSize->16]], "Section",
 CellChangeTimes->{3.401193338856146*^9}],

Cell[TextData[{
 "We will now use the inverse square law (",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "=", "2"}], TraditionalForm]]],
 ") and plot additional orbit data.  The perihelion and aphelion are the \
smallels and largest distance between the sun an planet.  The semimajor axis \
is half the largest distance between two points on the orbit.  The period is \
the time for a complete orbit.  The function ",
 StyleBox["OrbitData",
  FontWeight->"Bold"],
 " prints this data.  The function ",
 StyleBox["SweptArea",
  FontWeight->"Bold"],
 " gives the area swept out between two times."
}], "Text"],

Cell[CellGroupData[{

Cell["Examples of Unbound Orbits", "Subsection"],

Cell["Here is an example of an elliptical orbit.  ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"tFinal", "=", ".5"}], ";", 
  RowBox[{"R0", "=", "1"}], ";", 
  RowBox[{"V0", "=", 
   RowBox[{"0.5", " ", "2", " ", "\[Pi]"}]}], ";", 
  RowBox[{"\[CapitalDelta]t", "=", ".001"}], ";", 
  RowBox[{"p", "=", "2"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CalculateOrbit", ";", 
  RowBox[{"DisplayOrbit", "[", 
   RowBox[{
    RowBox[{"SweptArea", "[", 
     RowBox[{"0", ",", ".1"}], "]"}], ",", 
    RowBox[{"SweptArea", "[", 
     RowBox[{".15", ",", ".25"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"OrbitData", ";", 
  RowBox[{"Clear", "[", 
   RowBox[{
   "tFinal", ",", "R0", ",", "V0", ",", "iMaximum", ",", "t", ",", "r", ",", 
    "v"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.40114821266616*^9, 3.401148214128132*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Area between \\!\\(\\*SubscriptBox[\\(t\\), \\(i\\)]\\) = \
\"\>", "\[InvisibleSpace]", "0", 
   "\[InvisibleSpace]", "\<\" yr and \\!\\(\\*SubscriptBox[\\(t\\), \
\\(f\\)]\\) = \"\>", "\[InvisibleSpace]", "0.1`", 
   "\[InvisibleSpace]", "\<\" yr  is \"\>", "\[InvisibleSpace]", 
   "0.15864918582255277`", 
   "\[InvisibleSpace]", "\<\" \\!\\(\\*SuperscriptBox[\\(AU\\), \\(2\\)]\\)\"\
\>"}],
  SequenceForm[
  "Area between \!\(\*SubscriptBox[\(t\), \(i\)]\) = ", 0, 
   " yr and \!\(\*SubscriptBox[\(t\), \(f\)]\) = ", 0.1, " yr  is ", 
   0.15864918582255277`, " \!\(\*SuperscriptBox[\(AU\), \(2\)]\)"],
  Editable->False]], "Print",
 CellChangeTimes->{3.401148216534042*^9, 3.4012013319301767`*^9, 
  3.4012769628723707`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Area between \\!\\(\\*SubscriptBox[\\(t\\), \\(i\\)]\\) = \
\"\>", "\[InvisibleSpace]", "0.15`", 
   "\[InvisibleSpace]", "\<\" yr and \\!\\(\\*SubscriptBox[\\(t\\), \
\\(f\\)]\\) = \"\>", "\[InvisibleSpace]", "0.25`", 
   "\[InvisibleSpace]", "\<\" yr  is \"\>", "\[InvisibleSpace]", 
   "0.15857508821438918`", 
   "\[InvisibleSpace]", "\<\" \\!\\(\\*SuperscriptBox[\\(AU\\), \\(2\\)]\\)\"\
\>"}],
  SequenceForm[
  "Area between \!\(\*SubscriptBox[\(t\), \(i\)]\) = ", 0.15, 
   " yr and \!\(\*SubscriptBox[\(t\), \(f\)]\) = ", 0.25, " yr  is ", 
   0.15857508821438918`, " \!\(\*SuperscriptBox[\(AU\), \(2\)]\)"],
  Editable->False]], "Print",
 CellChangeTimes->{3.401148216534042*^9, 3.4012013319301767`*^9, 
  3.4012769629130793`*^9}]
}, Open  ]],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0.2, 0.2, 0.6], 
    PolygonBox[{{0, 0}, {1, 0}, {0.9999802607100224, 
      0.0031415719825091536`}}], 
    PolygonBox[{{0, 0}, {0.7936923266653421, 0.29074488326143955`}, {
      0.7893439132658201, 0.2931101313792607}}], 
    PolygonBox[{{0, 0}, {0.9999802607100224, 0.0031415719825091536`}, {
      0.9999210418659591, 0.006283019932872732}}], 
    PolygonBox[{{0, 0}, {0.9999210418659591, 0.006283019932872732}, {
      0.9998223405451535, 0.009424219790785737}}], 
    PolygonBox[{{0, 0}, {0.9998223405451535, 0.009424219790785737}, {
      0.9996841518756591, 0.012565047439607487`}}], 
    PolygonBox[{{0, 0}, {0.9996841518756591, 0.012565047439607487`}, {
      0.9995064690349592, 0.015705378678151673`}}], 
    PolygonBox[{{0, 0}, {0.9995064690349592, 0.015705378678151673`}, {
      0.9992892832481749, 0.01884508919242581}}], 
    PolygonBox[{{0, 0}, {0.9992892832481749, 0.01884508919242581}, {
      0.9990325837857579, 0.021984054527303162`}}], 
    PolygonBox[{{0, 0}, {0.9990325837857579, 0.021984054527303162`}, {
      0.9987363579606688, 0.025122150058110096`}}], 
    PolygonBox[{{0, 0}, {0.9987363579606688, 0.025122150058110096`}, {
      0.9984005911250381, 0.02825925096211173}}], 
    PolygonBox[{{0, 0}, {0.9984005911250381, 0.02825925096211173}, {
      0.9980252666663071, 0.03139523218987866}}], 
    PolygonBox[{{0, 0}, {0.9980252666663071, 0.03139523218987866}, {
      0.9976103660028485, 0.034529968436517344`}}], 
    PolygonBox[{{0, 0}, {0.9976103660028485, 0.034529968436517344`}, {
      0.9971558685790617, 0.03766333411274671}}], 
    PolygonBox[{{0, 0}, {0.9971558685790617, 0.03766333411274671}, {
      0.9966617518599414, 0.040795203315803066`}}], 
    PolygonBox[{{0, 0}, {0.9966617518599414, 0.040795203315803066`}, {
      0.9961279913251152, 0.04392544980015571}}], 
    PolygonBox[{{0, 0}, {0.9961279913251152, 0.04392544980015571}, {
      0.9955545604623471, 0.047053946948014805`}}], 
    PolygonBox[{{0, 0}, {0.9955545604623471, 0.047053946948014805`}, {
      0.994941430760503, 0.050180567739613366`}}], 
    PolygonBox[{{0, 0}, {0.994941430760503, 0.050180567739613366`}, {
      0.9942885717019724, 0.053305184723244615`}}], 
    PolygonBox[{{0, 0}, {0.9942885717019724, 0.053305184723244615`}, {
      0.9935959507545441, 0.056427669985035865`}}], 
    PolygonBox[{{0, 0}, {0.9935959507545441, 0.056427669985035865`}, {
      0.9928635333627288, 0.05954789511843963}}], 
    PolygonBox[{{0, 0}, {0.9928635333627288, 0.05954789511843963}, {
      0.9920912829385237, 0.06266573119342249}}], 
    PolygonBox[{{0, 0}, {0.9920912829385237, 0.06266573119342249}, {
      0.9912791608516137, 0.06578104872533173}}], 
    PolygonBox[{{0, 0}, {0.9912791608516137, 0.06578104872533173}, {
      0.9904271264190033, 0.06889371764341952}}], 
    PolygonBox[{{0, 0}, {0.9904271264190033, 0.06889371764341952}, {
      0.9895351368940715, 0.0720036072590039}}], 
    PolygonBox[{{0, 0}, {0.9895351368940715, 0.0720036072590039}, {
      0.9886031474550436, 0.07511058623324543}}], 
    PolygonBox[{{0, 0}, {0.9886031474550436, 0.07511058623324543}, {
      0.9876311111928736, 0.078214522544518}}], 
    PolygonBox[{{0, 0}, {0.9876311111928736, 0.078214522544518}, {
      0.9866189790985271, 0.0813152834553516}}], 
    PolygonBox[{{0, 0}, {0.9866189790985271, 0.0813152834553516}, {
      0.985566700049659, 0.08441273547892458}}], 
    PolygonBox[{{0, 0}, {0.985566700049659, 0.08441273547892458}, {
      0.9844742207966758, 0.08750674434508209}}], 
    PolygonBox[{{0, 0}, {0.9844742207966758, 0.08750674434508209}, {
      0.983341485948175, 0.09059717496585716}}], 
    PolygonBox[{{0, 0}, {0.983341485948175, 0.09059717496585716}, {
      0.9821684379557509, 0.09368389140047002}}], 
    PolygonBox[{{0, 0}, {0.9821684379557509, 0.09368389140047002}, {
      0.9809550170981576, 0.0967667568197806}}], 
    PolygonBox[{{0, 0}, {0.9809550170981576, 0.0967667568197806}, {
      0.9797011614648192, 0.09984563347016863}}], 
    PolygonBox[{{0, 0}, {0.9797011614648192, 0.09984563347016863}, {
      0.9784068069386764, 0.102920382636815}}], 
    PolygonBox[{{0, 0}, {0.9784068069386764, 0.102920382636815}, {
      0.9770718871783571, 0.10599086460635708`}}], 
    PolygonBox[{{0, 0}, {0.9770718871783571, 0.10599086460635708`}, {
      0.9756963335996613, 0.10905693862889036`}}], 
    PolygonBox[{{0, 0}, {0.9756963335996613, 0.10905693862889036`}, {
      0.9742800753563468, 0.11211846287928733`}}], 
    PolygonBox[{{0, 0}, {0.9742800753563468, 0.11211846287928733`}, {
      0.9728230393202024, 0.11517529441780414`}}], 
    PolygonBox[{{0, 0}, {0.9728230393202024, 0.11517529441780414`}, {
      0.9713251500603975, 0.11822728914994453`}}], 
    PolygonBox[{{0, 0}, {0.9713251500603975, 0.11822728914994453`}, {
      0.9697863298220918, 0.1212743017855493}}], 
    PolygonBox[{{0, 0}, {0.9697863298220918, 0.1212743017855493}, {
      0.9682064985042919, 0.12431618579707895`}}], 
    PolygonBox[{{0, 0}, {0.9682064985042919, 0.12431618579707895`}, {
      0.9665855736369403, 0.12735279337705582`}}], 
    PolygonBox[{{0, 0}, {0.9665855736369403, 0.12735279337705582`}, {
      0.9649234703572206, 0.1303839753946309}}], 
    PolygonBox[{{0, 0}, {0.9649234703572206, 0.1303839753946309}, {
      0.963220101385062, 0.13340958135123948`}}], 
    PolygonBox[{{0, 0}, {0.963220101385062, 0.13340958135123948`}, {
      0.9614753769978277, 0.13642945933530853`}}], 
    PolygonBox[{{0, 0}, {0.9614753769978277, 0.13642945933530853`}, {
      0.9596892050041689, 0.13944345597597718`}}], 
    PolygonBox[{{0, 0}, {0.9596892050041689, 0.13944345597597718`}, {
      0.9578614907170261, 0.14245141639579056`}}], 
    PolygonBox[{{0, 0}, {0.9578614907170261, 0.14245141639579056`}, {
      0.9559921369257592, 0.14545318416232586`}}], 
    PolygonBox[{{0, 0}, {0.9559921369257592, 0.14545318416232586`}, {
      0.9540810438673868, 0.1484486012387076}}], 
    PolygonBox[{{0, 0}, {0.9540810438673868, 0.1484486012387076}, {
      0.9521281091969135, 0.15143750793296812`}}], 
    PolygonBox[{{0, 0}, {0.9521281091969135, 0.15143750793296812`}, {
      0.9501332279567256, 0.15441974284620688`}}], 
    PolygonBox[{{0, 0}, {0.9501332279567256, 0.15441974284620688`}, {
      0.9480962925450317, 0.15739514281950123`}}], 
    PolygonBox[{{0, 0}, {0.9480962925450317, 0.15739514281950123`}, {
      0.9460171926833264, 0.16036354287951884`}}], 
    PolygonBox[{{0, 0}, {0.9460171926833264, 0.16036354287951884`}, {
      0.943895815382853, 0.16332477618278052`}}], 
    PolygonBox[{{0, 0}, {0.943895815382853, 0.16332477618278052`}, {
      0.9417320449100411, 0.16627867395851975`}}], 
    PolygonBox[{{0, 0}, {0.9417320449100411, 0.16627867395851975`}, {
      0.939525762750893, 0.16922506545008356`}}], 
    PolygonBox[{{0, 0}, {0.939525762750893, 0.16922506545008356`}, {
      0.9372768475742929, 0.1721637778548167}}], 
    PolygonBox[{{0, 0}, {0.9372768475742929, 0.1721637778548167}, {
      0.9349851751942124, 0.1750946362623693}}], 
    PolygonBox[{{0, 0}, {0.9349851751942124, 0.1750946362623693}, {
      0.9326506185307819, 0.17801746359136505`}}], 
    PolygonBox[{{0, 0}, {0.9326506185307819, 0.17801746359136505`}, {
      0.9302730475702, 0.1809320805243652}}], 
    PolygonBox[{{0, 0}, {0.9302730475702, 0.1809320805243652}, {
      0.92785232932345, 0.18383830544106008`}}], 
    PolygonBox[{{0, 0}, {0.92785232932345, 0.18383830544106008`}, {
      0.9253883277837912, 0.18673595434961776`}}], 
    PolygonBox[{{0, 0}, {0.9253883277837912, 0.18673595434961776`}, {
      0.9228809038829937, 0.18962484081611614`}}], 
    PolygonBox[{{0, 0}, {0.9228809038829937, 0.18962484081611614`}, {
      0.9203299154462802, 0.19250477589198145`}}], 
    PolygonBox[{{0, 0}, {0.9203299154462802, 0.19250477589198145`}, {
      0.9177352171459416, 0.19537556803935346`}}], 
    PolygonBox[{{0, 0}, {0.9177352171459416, 0.19537556803935346`}, {
      0.9150966604535886, 0.19823702305429322`}}], 
    PolygonBox[{{0, 0}, {0.9150966604535886, 0.19823702305429322`}, {
      0.9124140935910015, 0.20108894398774668`}}], 
    PolygonBox[{{0, 0}, {0.9124140935910015, 0.20108894398774668`}, {
      0.9096873614795385, 0.20393113106417268`}}], 
    PolygonBox[{{0, 0}, {0.9096873614795385, 0.20393113106417268`}, {
      0.9069163056880628, 0.20676338159774038`}}], 
    PolygonBox[{{0, 0}, {0.9069163056880628, 0.20676338159774038`}, {
      0.9041007643793431, 0.20958548990599662`}}], 
    PolygonBox[{{0, 0}, {0.9041007643793431, 0.20958548990599662`}, {
      0.9012405722548873, 0.2123972472208993}}], 
    PolygonBox[{{0, 0}, {0.9012405722548873, 0.2123972472208993}, {
      0.8983355604981611, 0.21519844159710813`}}], 
    PolygonBox[{{0, 0}, {0.8983355604981611, 0.21519844159710813`}, {
      0.895385556716144, 0.2179888578174191}}], 
    PolygonBox[{{0, 0}, {0.895385556716144, 0.2179888578174191}, {
      0.8923903848791751, 0.22076827729522386`}}], 
    PolygonBox[{{0, 0}, {0.8923903848791751, 0.22076827729522386`}, {
      0.8893498652590367, 0.2235364779738692}}], 
    PolygonBox[{{0, 0}, {0.8893498652590367, 0.2235364779738692}, {
      0.8862638143652226, 0.22629323422278674`}}], 
    PolygonBox[{{0, 0}, {0.8862638143652226, 0.22629323422278674`}, {
      0.8831320448793363, 0.22903831673025607`}}], 
    PolygonBox[{{0, 0}, {0.8831320448793363, 0.22903831673025607`}, {
      0.8799543655875622, 0.23177149239265799`}}], 
    PolygonBox[{{0, 0}, {0.8799543655875622, 0.23177149239265799`}, {
      0.8767305813111516, 0.2344925242000685}}], 
    PolygonBox[{{0, 0}, {0.8767305813111516, 0.2344925242000685}, {
      0.8734604928348587, 0.2372011711180358}}], 
    PolygonBox[{{0, 0}, {0.8734604928348587, 0.2372011711180358}, {
      0.8701438968332681, 0.23989718796537543`}}], 
    PolygonBox[{{0, 0}, {0.8701438968332681, 0.23989718796537543`}, {
      0.8667805857949424, 0.24258032528781057`}}], 
    PolygonBox[{{0, 0}, {0.8667805857949424, 0.24258032528781057`}, {
      0.8633703479443252, 0.24525032922727563`}}], 
    PolygonBox[{{0, 0}, {0.8633703479443252, 0.24525032922727563`}, {
      0.8599129671613255, 0.24790694138669234`}}], 
    PolygonBox[{{0, 0}, {0.8599129671613255, 0.24790694138669234`}, {
      0.8564082228985116, 0.25054989869001765`}}], 
    PolygonBox[{{0, 0}, {0.8564082228985116, 0.25054989869001765`}, {
      0.8528558900958361, 0.253178933237353}}], 
    PolygonBox[{{0, 0}, {0.8528558900958361, 0.253178933237353}, {
      0.8492557390928132, 0.25579377215489335`}}], 
    PolygonBox[{{0, 0}, {0.8492557390928132, 0.25579377215489335`}, {
      0.8456075355380649, 0.25839413743948225`}}], 
    PolygonBox[{{0, 0}, {0.8456075355380649, 0.25839413743948225`}, {
      0.8419110402961513, 0.26097974579752903`}}], 
    PolygonBox[{{0, 0}, {0.8419110402961513, 0.26097974579752903`}, {
      0.838166009351594, 0.2635503084780286}}], 
    PolygonBox[{{0, 0}, {0.838166009351594, 0.2635503084780286}, {
      0.8343721937100019, 0.2661055310994134}}], 
    PolygonBox[{{0, 0}, {0.8343721937100019, 0.2661055310994134}, {
      0.8305293392962015, 0.26864511346994924`}}], 
    PolygonBox[{{0, 0}, {0.8305293392962015, 0.26864511346994924`}, {
      0.8266371868492733, 0.271168749401376}}], 
    PolygonBox[{{0, 0}, {0.8266371868492733, 0.271168749401376}, {
      0.82269547181439, 0.27367612651547185`}}], 
    PolygonBox[{{0, 0}, {0.82269547181439, 0.27367612651547185`}, {
      0.8187039242313483, 0.27616692604320825`}}], 
    PolygonBox[{{0, 0}, {0.8187039242313483, 0.27616692604320825`}, {
      0.8146622686196844, 0.278640822616139}}], 
    PolygonBox[{{0, 0}, {0.8146622686196844, 0.278640822616139}, {
      0.8105702238602549, 0.28109748404965157`}}], 
    PolygonBox[{{0, 0}, {0.8105702238602549, 0.28109748404965157`}, {
      0.8064275030731648, 0.28353657111768443`}}], 
    PolygonBox[{{0, 0}, {0.8064275030731648, 0.28353657111768443`}, {
      0.8022338134919165, 0.28595773731849344`}}], 
    PolygonBox[{{0, 0}, {0.8022338134919165, 0.28595773731849344`}, {
      0.7979888563336501, 0.28836062863102646`}}], 
    PolygonBox[{{0, 0}, {0.7979888563336501, 0.28836062863102646`}, {
      0.7936923266653421, 0.29074488326143955`}}]}, 
   {RGBColor[0.2, 0.2, 0.6], 
    PolygonBox[{{0, 0}, {0.5039900101075169, 0.3746580565748316}, {
      0.4964526286900808, 0.3752881630777497}}], 
    PolygonBox[{{0, 0}, {0.21702716320630216`, -0.3511061474393007}, {
      0.22765571038829852`, -0.35382674134168496`}}], 
    PolygonBox[{{0, 0}, {0.4964526286900808, 0.3752881630777497}, {
      0.4888339331141075, 0.37585679626129287`}}], 
    PolygonBox[{{0, 0}, {0.4888339331141075, 0.37585679626129287`}, {
      0.48113292471134433`, 0.3763621353530602}}], 
    PolygonBox[{{0, 0}, {0.48113292471134433`, 0.3763621353530602}, {
      0.4733485803183081, 0.37680228019950396`}}], 
    PolygonBox[{{0, 0}, {0.4733485803183081, 0.37680228019950396`}, {
      0.4654798517488869, 0.3771752465046035}}], 
    PolygonBox[{{0, 0}, {0.4654798517488869, 0.3771752465046035}, {
      0.4575256652874699, 0.37747896070828346`}}], 
    PolygonBox[{{0, 0}, {0.4575256652874699, 0.37747896070828346`}, {
      0.44948492120976796`, 0.3777112544716816}}], 
    PolygonBox[{{0, 0}, {0.44948492120976796`, 0.3777112544716816}, {
      0.44135649333982985`, 0.3778698587328629}}], 
    PolygonBox[{{0, 0}, {0.44135649333982985`, 0.3778698587328629}, {
      0.4331392286533469, 0.37795239729264546`}}], 
    PolygonBox[{{0, 0}, {0.4331392286533469, 0.37795239729264546`}, {
      0.4248319469392101, 0.3779563798857872}}], 
    PolygonBox[{{0, 0}, {0.4248319469392101, 0.3779563798857872}, {
      0.4164334405334969, 0.3778791946878172}}], 
    PolygonBox[{{0, 0}, {0.4164334405334969, 0.3778791946878172}, {
      0.40794247414268314`, 0.377718100202208}}], 
    PolygonBox[{{0, 0}, {0.40794247414268314`, 0.377718100202208}, {
      0.39935778477597755`, 0.3774702164662803}}], 
    PolygonBox[{{0, 0}, {0.39935778477597755`, 0.3774702164662803}, {
      0.3906780818103563, 0.3771325155071233}}], 
    PolygonBox[{{0, 0}, {0.3906780818103563, 0.3771325155071233}, {
      0.3819020472162482, 0.3767018109707673}}], 
    PolygonBox[{{0, 0}, {0.3819020472162482, 0.3767018109707673}, {
      0.37302833597702495`, 0.3761747468387512}}], 
    PolygonBox[{{0, 0}, {0.37302833597702495`, 0.3761747468387512}, {
      0.3640555767416558, 0.375547785135908}}], 
    PolygonBox[{{0, 0}, {0.3640555767416558, 0.375547785135908}, {
      0.35498237275729533`, 0.3748171925214946}}], 
    PolygonBox[{{0, 0}, {0.35498237275729533`, 0.3748171925214946}, {
      0.3458073031374397, 0.3739790256424936}}], 
    PolygonBox[{{0, 0}, {0.3458073031374397, 0.3739790256424936}, {
      0.3365289245319172, 0.37302911511279613`}}], 
    PolygonBox[{{0, 0}, {0.3365289245319172, 0.37302911511279613`}, {
      0.3271457732777524, 0.3719630479647592}}], 
    PolygonBox[{{0, 0}, {0.3271457732777524, 0.3719630479647592}, {
      0.31765636812532094`, 0.3707761484000115}}], 
    PolygonBox[{{0, 0}, {0.31765636812532094`, 0.3707761484000115}, {
      0.30805921365277505`, 0.36946345664400504`}}], 
    PolygonBox[{{0, 0}, {0.30805921365277505`, 0.36946345664400504`}, {
      0.29835280450417195`, 0.3680197056832703}}], 
    PolygonBox[{{0, 0}, {0.29835280450417195`, 0.3680197056832703}, {
      0.288535630613967, 0.36643929563517175`}}], 
    PolygonBox[{{0, 0}, {0.288535630613967, 0.36643929563517175`}, {
      0.27860618361363876`, 0.3647162654666576}}], 
    PolygonBox[{{0, 0}, {0.27860618361363876`, 0.3647162654666576}, {
      0.26856296465656787`, 0.3628442617404711}}], 
    PolygonBox[{{0, 0}, {0.26856296465656787`, 0.3628442617404711}, {
      0.25840449394661785`, 0.3608165040238961}}], 
    PolygonBox[{{0, 0}, {0.25840449394661785`, 0.3608165040238961}, {
      0.2481293223163254, 0.35862574654564544`}}], 
    PolygonBox[{{0, 0}, {0.2481293223163254, 0.35862574654564544`}, {
      0.23773604527492642`, 0.35626423563022375`}}], 
    PolygonBox[{{0, 0}, {0.23773604527492642`, 0.35626423563022375`}, {
      0.22722332003806525`, 0.3537236623752472}}], 
    PolygonBox[{{0, 0}, {0.22722332003806525`, 0.3537236623752472}, {
      0.2165898861643194, 0.3509951099650559}}], 
    PolygonBox[{{0, 0}, {0.2165898861643194, 0.3509951099650559}, {
      0.2058345905641691, 0.3480689949329005}}], 
    PolygonBox[{{0, 0}, {0.2058345905641691, 0.3480689949329005}, {
      0.194956417821804, 0.34493500159366414`}}], 
    PolygonBox[{{0, 0}, {0.194956417821804, 0.34493500159366414`}, {
      0.18395452698820314`, 0.34158200876958755`}}], 
    PolygonBox[{{0, 0}, {0.18395452698820314`, 0.34158200876958755`}, {
      0.17282829627676863`, 0.3379980078236892}}], 
    PolygonBox[{{0, 0}, {0.17282829627676863`, 0.3379980078236892}, {
      0.16157737743518577`, 0.33417001090168297`}}], 
    PolygonBox[{{0, 0}, {0.16157737743518577`, 0.33417001090168297`}, {
      0.15020176199801763`, 0.3300839481674408}}], 
    PolygonBox[{{0, 0}, {0.15020176199801763`, 0.3300839481674408}, {
      0.13870186216801283`, 0.3257245527069144}}], 
    PolygonBox[{{0, 0}, {0.13870186216801283`, 0.3257245527069144}, {
      0.12707860976113794`, 0.32107523168344826`}}], 
    PolygonBox[{{0, 0}, {0.12707860976113794`, 0.32107523168344826`}, {
      0.11533357752034386`, 0.3161179222738781}}], 
    PolygonBox[{{0, 0}, {0.11533357752034386`, 0.3161179222738781}, {
      0.10346912820593054`, 0.31083293093186365`}}], 
    PolygonBox[{{0, 0}, {0.10346912820593054`, 0.31083293093186365`}, {
      0.09148859826864454, 0.30519875466275476`}}], 
    PolygonBox[{{0, 0}, {0.09148859826864454, 0.30519875466275476`}, {
      0.07939652468255265, 0.29919188333048785`}}], 
    PolygonBox[{{0, 0}, {0.07939652468255265, 0.29919188333048785`}, {
      0.06719892575124904, 0.29278658267036}}], 
    PolygonBox[{{0, 0}, {0.06719892575124904, 0.29278658267036}, {
      0.05490364951044523, 0.28595465883276455`}}], 
    PolygonBox[{{0, 0}, {0.05490364951044523, 0.28595465883276455`}, {
      0.042520806847560544`, 0.2786652072051746}}], 
    PolygonBox[{{0, 0}, {0.042520806847560544`, 0.2786652072051746}, {
      0.030063310749925215`, 0.27088435136232025`}}], 
    PolygonBox[{{0, 0}, {0.030063310749925215`, 0.27088435136232025`}, {
      0.0175475482350283, 0.26257498288584685`}}], 
    PolygonBox[{{0, 0}, {0.0175475482350283, 0.26257498288584685`}, {
      0.004994217441815156, 0.25369652036637125`}}], 
    PolygonBox[{{0, 0}, {0.004994217441815156, 
      0.25369652036637125`}, {-0.0075706312652977885`, 
      0.24420471743490407`}}], 
    PolygonBox[{{0, 0}, {-0.0075706312652977885`, 
      0.24420471743490407`}, {-0.020114305378234346`, 0.23405156695726462`}}],
     PolygonBox[{{0, 0}, {-0.020114305378234346`, 
      0.23405156695726462`}, {-0.032595883047928674`, 0.22318537394715532`}}],
     PolygonBox[{{0, 0}, {-0.032595883047928674`, 
      0.22318537394715532`}, {-0.044964282388017064`, 0.21155110622513623`}}],
     PolygonBox[{{0, 0}, {-0.044964282388017064`, 
      0.21155110622513623`}, {-0.05715595578660602, 0.19909118235500722`}}], 
    PolygonBox[{{0, 0}, {-0.05715595578660602, 
      0.19909118235500722`}, {-0.0690922161288215, 0.18574692259737416`}}], 
    PolygonBox[{{0, 0}, {-0.0690922161288215, 
      0.18574692259737416`}, {-0.08067628154423397, 0.1714609676609323}}], 
    PolygonBox[{{0, 0}, {-0.08067628154423397, 
      0.1714609676609323}, {-0.09179027122845257, 0.15618104801252214`}}], 
    PolygonBox[{{0, 0}, {-0.09179027122845257, 
      0.15618104801252214`}, {-0.10229262565664003`, 0.1398655274282709}}], 
    PolygonBox[{{0, 0}, {-0.10229262565664003`, 
      0.1398655274282709}, {-0.11201677761727077`, 0.12249107537920675`}}], 
    PolygonBox[{{0, 0}, {-0.11201677761727077`, 
      0.12249107537920675`}, {-0.12077233598380867`, 0.10406252387750511`}}], 
    PolygonBox[{{0, 0}, {-0.12077233598380867`, 
      0.10406252387750511`}, {-0.12835042060140484`, 0.0846242893545386}}], 
    PolygonBox[{{0, 0}, {-0.12835042060140484`, 
      0.0846242893545386}, {-0.13453478340752764`, 0.06427161369818919}}], 
    PolygonBox[{{0, 0}, {-0.13453478340752764`, 
      0.06427161369818919}, {-0.13911949138000523`, 0.043158502947027416`}}], 
    PolygonBox[{{0, 0}, {-0.13911949138000523`, 
      0.043158502947027416`}, {-0.14193187734268722`, 
      0.021498321800961128`}}], 
    PolygonBox[{{0, 0}, {-0.14193187734268722`, 
      0.021498321800961128`}, {-0.1428565337317014, -0.0004463479544767994}}],
     PolygonBox[{{0, 
      0}, {-0.1428565337317014, -0.0004463479544767994}, \
{-0.14185381838966815`, -0.022385024167755497`}}], 
    PolygonBox[{{0, 
      0}, {-0.14185381838966815`, -0.022385024167755497`}, \
{-0.13896667319496217`, -0.044027828766027274`}}], 
    PolygonBox[{{0, 
      0}, {-0.13896667319496217`, -0.044027828766027274`}, \
{-0.13431324867767722`, -0.06511383024655144}}], 
    PolygonBox[{{0, 
      0}, {-0.13431324867767722`, -0.06511383024655144}, \
{-0.12806804111866685`, -0.08543195411925854}}], 
    PolygonBox[{{0, 
      0}, {-0.12806804111866685`, -0.08543195411925854}, \
{-0.12043782337658793`, -0.10483065310571002`}}], 
    PolygonBox[{{0, 
      0}, {-0.12043782337658793`, -0.10483065310571002`}, \
{-0.1116388459549219, -0.12321693701487677`}}], 
    PolygonBox[{{0, 
      0}, {-0.1116388459549219, -0.12321693701487677`}, {-0.1018794129073021, \
-0.14054822410962003`}}], 
    PolygonBox[{{0, 
      0}, {-0.1018794129073021, -0.14054822410962003`}, {-0.0913490199576953, \
-0.15682104112406203`}}], 
    PolygonBox[{{0, 
      0}, {-0.0913490199576953, -0.15682104112406203`}, \
{-0.08021322185983283, -0.17205963985692083`}}], 
    PolygonBox[{{0, 
      0}, {-0.08021322185983283, -0.17205963985692083`}, \
{-0.0686125805162426, -0.18630622289113305`}}], 
    PolygonBox[{{0, 
      0}, {-0.0686125805162426, -0.18630622289113305`}, \
{-0.056664066182436926`, -0.19961336166086063`}}], 
    PolygonBox[{{0, 
      0}, {-0.056664066182436926`, -0.19961336166086063`}, \
{-0.04446366781357318, -0.21203853276137516`}}], 
    PolygonBox[{{0, 
      0}, {-0.04446366781357318, -0.21203853276137516`}, \
{-0.03208940250441879, -0.22364041159629533`}}], 
    PolygonBox[{{0, 
      0}, {-0.03208940250441879, -0.22364041159629533`}, \
{-0.019604262618639793`, -0.23447649984214716`}}], 
    PolygonBox[{{0, 
      0}, {-0.019604262618639793`, -0.23447649984214716`}, \
{-0.007058875630218151, -0.24460170675267054`}}], 
    PolygonBox[{{0, 0}, {-0.007058875630218151, -0.24460170675267054`}, {
      0.005506205607517894, -0.25406758285349956`}}], 
    PolygonBox[{{0, 0}, {0.005506205607517894, -0.25406758285349956`}, {
      0.018058586850738297`, -0.26292198326463756`}}], 
    PolygonBox[{{0, 0}, {0.018058586850738297`, -0.26292198326463756`}, {
      0.030572458255215967`, -0.27120900346929316`}}], 
    PolygonBox[{{0, 0}, {0.030572458255215967`, -0.27120900346929316`}, {
      0.04302731547039734, -0.2789690802345226}}], 
    PolygonBox[{{0, 0}, {0.04302731547039734, -0.2789690802345226}, {
      0.055406927637692546`, -0.28623918635250917`}}], 
    PolygonBox[{{0, 0}, {0.055406927637692546`, -0.28623918635250917`}, {
      0.06769850770485555, -0.2930530729098124}}], 
    PolygonBox[{{0, 0}, {0.06769850770485555, -0.2930530729098124}, {
      0.07989204646287815, -0.29944152980318756`}}], 
    PolygonBox[{{0, 0}, {0.07989204646287815, -0.29944152980318756`}, {
      0.09197977807858775, -0.30543264656300095`}}], 
    PolygonBox[{{0, 0}, {0.09197977807858775, -0.30543264656300095`}, {
      0.1039557507940762, -0.311052062986335}}], 
    PolygonBox[{{0, 0}, {0.1039557507940762, -0.311052062986335}, {
      0.11581548157157796`, -0.3163232038855998}}], 
    PolygonBox[{{0, 0}, {0.11581548157157796`, -0.3163232038855998}, {
      0.12755567771966458`, -0.3212674953030258}}], 
    PolygonBox[{{0, 0}, {0.12755567771966458`, -0.3212674953030258}, {
      0.13917401200412483`, -0.32590456142548097`}}], 
    PolygonBox[{{0, 0}, {0.13917401200412483`, -0.32590456142548097`}, {
      0.1506689405309634, -0.3302524025605367}}], 
    PolygonBox[{{0, 0}, {0.1506689405309634, -0.3302524025605367}, {
      0.16203955490460367`, -0.3343275551721744}}], 
    PolygonBox[{{0, 0}, {0.16203955490460367`, -0.3343275551721744}, {
      0.17328546191849273`, -0.3381452353006988}}], 
    PolygonBox[{{0, 0}, {0.17328546191849273`, -0.3381452353006988}, {
      0.1844066854202136, -0.3417194668230826}}], 
    PolygonBox[{{0, 0}, {0.1844066854202136, -0.3417194668230826}, {
      0.19540358608550606`, -0.3450631960233248}}], 
    PolygonBox[{{0, 0}, {0.19540358608550606`, -0.3450631960233248}, {
      0.20627679569729046`, -0.3481883938867722}}], 
    PolygonBox[{{0, 0}, {0.20627679569729046`, -0.3481883938867722}, {
      0.21702716320630216`, -0.3511061474393007}}]}, 
   {RGBColor[1, 1, 1], Thickness[0.0075], 
    LineBox[{{1, 0}, {0.9999802607100224, 0.0031415719825091536`}, {
      0.9999210418659591, 0.006283019932872732}, {0.9998223405451535, 
      0.009424219790785737}, {0.9996841518756591, 0.012565047439607487`}, {
      0.9995064690349592, 0.015705378678151673`}, {0.9992892832481749, 
      0.01884508919242581}, {0.9990325837857579, 0.021984054527303162`}, {
      0.9987363579606688, 0.025122150058110096`}, {0.9984005911250381, 
      0.02825925096211173}, {0.9980252666663071, 0.03139523218987866}, {
      0.9976103660028485, 0.034529968436517344`}, {0.9971558685790617, 
      0.03766333411274671}, {0.9966617518599414, 0.040795203315803066`}, {
      0.9961279913251152, 0.04392544980015571}, {0.9955545604623471, 
      0.047053946948014805`}, {0.994941430760503, 0.050180567739613366`}, {
      0.9942885717019724, 0.053305184723244615`}, {0.9935959507545441, 
      0.056427669985035865`}, {0.9928635333627288, 0.05954789511843963}, {
      0.9920912829385237, 0.06266573119342249}, {0.9912791608516137, 
      0.06578104872533173}, {0.9904271264190033, 0.06889371764341952}, {
      0.9895351368940715, 0.0720036072590039}, {0.9886031474550436, 
      0.07511058623324543}, {0.9876311111928736, 0.078214522544518}, {
      0.9866189790985271, 0.0813152834553516}, {0.985566700049659, 
      0.08441273547892458}, {0.9844742207966758, 0.08750674434508209}, {
      0.983341485948175, 0.09059717496585716}, {0.9821684379557509, 
      0.09368389140047002}, {0.9809550170981576, 0.0967667568197806}, {
      0.9797011614648192, 0.09984563347016863}, {0.9784068069386764, 
      0.102920382636815}, {0.9770718871783571, 0.10599086460635708`}, {
      0.9756963335996613, 0.10905693862889036`}, {0.9742800753563468, 
      0.11211846287928733`}, {0.9728230393202024, 0.11517529441780414`}, {
      0.9713251500603975, 0.11822728914994453`}, {0.9697863298220918, 
      0.1212743017855493}, {0.9682064985042919, 0.12431618579707895`}, {
      0.9665855736369403, 0.12735279337705582`}, {0.9649234703572206, 
      0.1303839753946309}, {0.963220101385062, 0.13340958135123948`}, {
      0.9614753769978277, 0.13642945933530853`}, {0.9596892050041689, 
      0.13944345597597718`}, {0.9578614907170261, 0.14245141639579056`}, {
      0.9559921369257592, 0.14545318416232586`}, {0.9540810438673868, 
      0.1484486012387076}, {0.9521281091969135, 0.15143750793296812`}, {
      0.9501332279567256, 0.15441974284620688`}, {0.9480962925450317, 
      0.15739514281950123`}, {0.9460171926833264, 0.16036354287951884`}, {
      0.943895815382853, 0.16332477618278052`}, {0.9417320449100411, 
      0.16627867395851975`}, {0.939525762750893, 0.16922506545008356`}, {
      0.9372768475742929, 0.1721637778548167}, {0.9349851751942124, 
      0.1750946362623693}, {0.9326506185307819, 0.17801746359136505`}, {
      0.9302730475702, 0.1809320805243652}, {0.92785232932345, 
      0.18383830544106008`}, {0.9253883277837912, 0.18673595434961776`}, {
      0.9228809038829937, 0.18962484081611614`}, {0.9203299154462802, 
      0.19250477589198145`}, {0.9177352171459416, 0.19537556803935346`}, {
      0.9150966604535886, 0.19823702305429322`}, {0.9124140935910015, 
      0.20108894398774668`}, {0.9096873614795385, 0.20393113106417268`}, {
      0.9069163056880628, 0.20676338159774038`}, {0.9041007643793431, 
      0.20958548990599662`}, {0.9012405722548873, 0.2123972472208993}, {
      0.8983355604981611, 0.21519844159710813`}, {0.895385556716144, 
      0.2179888578174191}, {0.8923903848791751, 0.22076827729522386`}, {
      0.8893498652590367, 0.2235364779738692}, {0.8862638143652226, 
      0.22629323422278674`}, {0.8831320448793363, 0.22903831673025607`}, {
      0.8799543655875622, 0.23177149239265799`}, {0.8767305813111516, 
      0.2344925242000685}, {0.8734604928348587, 0.2372011711180358}, {
      0.8701438968332681, 0.23989718796537543`}, {0.8667805857949424, 
      0.24258032528781057`}, {0.8633703479443252, 0.24525032922727563`}, {
      0.8599129671613255, 0.24790694138669234`}, {0.8564082228985116, 
      0.25054989869001765`}, {0.8528558900958361, 0.253178933237353}, {
      0.8492557390928132, 0.25579377215489335`}, {0.8456075355380649, 
      0.25839413743948225`}, {0.8419110402961513, 0.26097974579752903`}, {
      0.838166009351594, 0.2635503084780286}, {0.8343721937100019, 
      0.2661055310994134}, {0.8305293392962015, 0.26864511346994924`}, {
      0.8266371868492733, 0.271168749401376}, {0.82269547181439, 
      0.27367612651547185`}, {0.8187039242313483, 0.27616692604320825`}, {
      0.8146622686196844, 0.278640822616139}, {0.8105702238602549, 
      0.28109748404965157`}, {0.8064275030731648, 0.28353657111768443`}, {
      0.8022338134919165, 0.28595773731849344`}, {0.7979888563336501, 
      0.28836062863102646`}, {0.7936923266653421, 0.29074488326143955`}, {
      0.7893439132658198, 0.29311013137926084`}, {0.7849432984834503, 
      0.29545599484268}, {0.7804901580893514, 0.29778208691240965`}, {
      0.7759841611259689, 0.30008801195353246`}, {0.7714249697508606, 
      0.3023733651247124}, {0.7668122390755177, 0.30463773205411065`}, {
      0.7621456169990511, 0.30688068850130684`}, {0.7574247440365645, 
      0.30910180000448273`}, {0.752649253142025, 0.31130062151207927`}, {
      0.7478187695254439, 0.31347669699808867`}, {0.7429329104641641, 
      0.3156295590600896}, {0.7379912851080527, 0.3177587284990769}, {
      0.7329934942783838, 0.3198637138800756}, {0.7279391302601947, 
      0.32194401107246423`}, {0.7228277765878889, 0.3239991027688595}, {
      0.7176590078238533, 0.3260284579813416}, {0.7124323893298524, 
      0.3280315315137137}, {0.7071474770309516, 0.3300077634084047}, {
      0.7018038171717196, 0.3319565783665259}, {0.6964009460644514, 
      0.3338773851394923}, {0.690938389829148, 0.33576957589050604`}, {
      0.6854156641249856, 0.337632525524083}, {0.6798322738729978, 
      0.3394655909816703}, {0.674187712969698, 0.34126811050126604`}, {
      0.6684814639913578, 0.3430394028387998}, {0.6627129978886612, 
      0.3447787664488674}, {0.6568817736714532, 0.34648547862223744`}, {
      0.6509872380832985, 0.34815879457735227`}, {0.6450288252655753, 
      0.34979794650283796`}, {0.6390059564108284, 0.3514021425478087}, {
      0.6329180394051189, 0.3529705657565038}, {0.6267644684591158, 
      0.35450237294352566`}, {0.6205446237276907, 0.3559966935056524}, {
      0.6142578709177974, 0.35745262816587925`}, {0.6079035608844416, 
      0.3588692476449915}, {0.6014810292145775, 0.3602455912555895}, {
      0.5949895957988046, 0.3615806654130692}, {0.5884285643907881, 
      0.36287344205760136`}, {0.5817972221543769, 0.3641228569806534}, {
      0.5750948391984674, 0.36532780804904547`}, {0.568320668099735, 
      0.3664871533189295}, {0.5614739434134602, 0.36759970903141476`}, {
      0.5545538811727831, 0.3686642474808344}, {0.5475596783768625, 
      0.36967949474584116`}, {0.5404905124685739, 0.37064412827263554`}, {
      0.5333455408025731, 0.37155677429864814`}, {0.5261239001047745, 
      0.372416005103919}, {0.518824705924561, 0.37322033607621796`}, {
      0.5114470520813512, 0.3739682225746266}, {0.5039900101075178, 
      0.3746580565748315}, {0.4964526286900808, 0.3752881630777497}, {
      0.4888339331141075, 0.37585679626129287`}, {0.48113292471134433`, 
      0.3763621353530602}, {0.4733485803183081, 0.37680228019950396`}, {
      0.4654798517488869, 0.3771752465046035}, {0.4575256652874699, 
      0.37747896070828346`}, {0.44948492120976796`, 0.3777112544716816}, {
      0.44135649333982985`, 0.3778698587328629}, {0.4331392286533469, 
      0.37795239729264546`}, {0.4248319469392101, 0.3779563798857872}, {
      0.4164334405334969, 0.3778791946878172}, {0.40794247414268314`, 
      0.377718100202208}, {0.39935778477597755`, 0.3774702164662803}, {
      0.3906780818103563, 0.3771325155071233}, {0.3819020472162482, 
      0.3767018109707673}, {0.37302833597702495`, 0.3761747468387512}, {
      0.3640555767416558, 0.375547785135908}, {0.35498237275729533`, 
      0.3748171925214946}, {0.3458073031374397, 0.3739790256424936}, {
      0.3365289245319172, 0.37302911511279613`}, {0.3271457732777524, 
      0.3719630479647592}, {0.31765636812532094`, 0.3707761484000115}, {
      0.30805921365277505`, 0.36946345664400504`}, {0.29835280450417195`, 
      0.3680197056832703}, {0.288535630613967, 0.36643929563517175`}, {
      0.27860618361363876`, 0.3647162654666576}, {0.26856296465656787`, 
      0.3628442617404711}, {0.25840449394661785`, 0.3608165040238961}, {
      0.2481293223163254, 0.35862574654564544`}, {0.23773604527492642`, 
      0.35626423563022375`}, {0.22722332003806525`, 0.3537236623752472}, {
      0.2165898861643194, 0.3509951099650559}, {0.2058345905641691, 
      0.3480689949329005}, {0.194956417821804, 0.34493500159366414`}, {
      0.18395452698820314`, 0.34158200876958755`}, {0.17282829627676863`, 
      0.3379980078236892}, {0.16157737743518577`, 0.33417001090168297`}, {
      0.15020176199801763`, 0.3300839481674408}, {0.13870186216801283`, 
      0.3257245527069144}, {0.12707860976113794`, 0.32107523168344826`}, {
      0.11533357752034386`, 0.3161179222738781}, {0.10346912820593054`, 
      0.31083293093186365`}, {0.09148859826864454, 0.30519875466275476`}, {
      0.07939652468255265, 0.29919188333048785`}, {0.06719892575124904, 
      0.29278658267036}, {0.05490364951044523, 0.28595465883276455`}, {
      0.042520806847560544`, 0.2786652072051746}, {0.030063310749925215`, 
      0.27088435136232025`}, {0.0175475482350283, 0.26257498288584685`}, {
      0.004994217441815156, 0.25369652036637125`}, {-0.0075706312652977885`, 
      0.24420471743490407`}, {-0.020114305378234346`, 
      0.23405156695726462`}, {-0.032595883047928674`, 
      0.22318537394715532`}, {-0.044964282388017064`, 
      0.21155110622513623`}, {-0.05715595578660602, 
      0.19909118235500722`}, {-0.0690922161288215, 
      0.18574692259737416`}, {-0.08067628154423397, 
      0.1714609676609323}, {-0.09179027122845257, 
      0.15618104801252214`}, {-0.10229262565664003`, 
      0.1398655274282709}, {-0.11201677761727077`, 
      0.12249107537920675`}, {-0.12077233598380867`, 
      0.10406252387750511`}, {-0.12835042060140484`, 
      0.0846242893545386}, {-0.13453478340752764`, 
      0.06427161369818919}, {-0.13911949138000523`, 
      0.043158502947027416`}, {-0.14193187734268722`, 
      0.021498321800961128`}, {-0.1428565337317014, -0.0004463479544767994}, \
{-0.14185381838966815`, -0.022385024167755497`}, {-0.13896667319496217`, \
-0.044027828766027274`}, {-0.13431324867767722`, -0.06511383024655144}, \
{-0.12806804111866685`, -0.08543195411925854}, {-0.12043782337658793`, \
-0.10483065310571002`}, {-0.1116388459549219, -0.12321693701487677`}, \
{-0.1018794129073021, -0.14054822410962003`}, {-0.0913490199576953, \
-0.15682104112406203`}, {-0.08021322185983283, -0.17205963985692083`}, \
{-0.0686125805162426, -0.18630622289113305`}, {-0.056664066182436926`, \
-0.19961336166086063`}, {-0.04446366781357318, -0.21203853276137516`}, \
{-0.03208940250441879, -0.22364041159629533`}, {-0.019604262618639793`, \
-0.23447649984214716`}, {-0.007058875630218151, -0.24460170675267054`}, {
      0.005506205607517894, -0.25406758285349956`}, {
      0.018058586850738297`, -0.26292198326463756`}, {
      0.030572458255215967`, -0.27120900346929316`}, {
      0.04302731547039734, -0.2789690802345226}, {
      0.055406927637692546`, -0.28623918635250917`}, {
      0.06769850770485555, -0.2930530729098124}, {
      0.07989204646287815, -0.29944152980318756`}, {
      0.09197977807858775, -0.30543264656300095`}, {
      0.1039557507940762, -0.311052062986335}, {
      0.11581548157157796`, -0.3163232038855998}, {
      0.12755567771966458`, -0.3212674953030258}, {
      0.13917401200412483`, -0.32590456142548097`}, {
      0.1506689405309634, -0.3302524025605367}, {
      0.16203955490460367`, -0.3343275551721744}, {
      0.17328546191849273`, -0.3381452353006988}, {
      0.1844066854202136, -0.3417194668230826}, {
      0.19540358608550606`, -0.3450631960233248}, {
      0.20627679569729046`, -0.3481883938867722}, {
      0.21702716320630216`, -0.3511061474393007}, {
      0.2276557103883003, -0.3538267413416854}, {
      0.23816359533963466`, -0.3563597308336852}, {
      0.24855208239219384`, -0.35871400700865863`}, {
      0.25882251729911704`, -0.36089785529205237`}, {
      0.26897630675873774`, -0.3629190078979713}, {
      0.27901490151744096`, -0.36478469094809257`}, {
      0.2889397824313834, -0.3665016668565019}, {
      0.2987524489793338, -0.3680762725122317}, {
      0.308454409809733, -0.36951445372775177`}, {
      0.31804717497876833`, -0.3708217963656757}, {
      0.3275322495962154, -0.3720035545067428}, {
      0.336911128644725, -0.37306467597897003`}, {
      0.34618529277826265`, -0.3740098255300461}, {
      0.35535620493825004`, -0.3748434058919132}, {
      0.36442530765297376`, -0.37556957695747967`}, {
      0.37339402090810636`, -0.37619227326400184`}, {
      0.3822637404946037, -0.3767152199554161}, {
      0.39103583675550657`, -0.3771419473763876}, {
      0.3997116536658506, -0.37747580443371526`}, {
      0.4082925081904421, -0.377719970845693}, {
      0.4167796898730584, -0.37787746838680053`}, {
      0.42517446061798775`, -0.377951171223454}, {
      0.43347805463098404`, -0.3779438154262866}, {
      0.44169167849187896`, -0.37785800773537415`}, {
      0.4498165113354364, -0.37769623364682087`}, {
      0.4578537051206884, -0.3774608648820406}, {
      0.4658043849720728, -0.37715416629480025`}, {
      0.4736696495782929, -0.37677830226552755`}, {
      0.48145057163701693`, -0.37633534262744595`}, {
      0.4891481983353962, -0.37582726816470274`}, {
      0.49676355185795384`, -0.375255975718744}, {
      0.5042976299147349, -0.3746232829356969}, {
      0.5117514062837396, -0.3739309326844054}, {
      0.5191258313626265, -0.37318059717197405`}, {
      0.5264218327254894, -0.37237388178118297`}, {
      0.5336403156812078, -0.37151232865189665`}, {
      0.540782163830463, -0.3705974200265824}, {
      0.5478482396190132, -0.3696305813782499}, {
      0.5548393848852529, -0.36861318433749735`}, {
      0.5617564214004406, -0.36754654943389}, {
      0.568600151400294, -0.366431948665573}, {
      0.5753713581069103, -0.36527060790983323`}, {
      0.5820708062401951, -0.36406370918624414`}, {
      0.5886992425181714, -0.3628123927830541}, {
      0.5952573961457008, -0.36151775925659535`}, {
      0.6017459792912853, -0.36018087131268867`}, {
      0.6081656875517303, -0.3588027555782914}, {
      0.6145172004045475, -0.35738440427097534`}, {
      0.6208011816480553, -0.35592677677321855`}, {
      0.6270182798292031, -0.35443080111794717`}, {
      0.6331691286591992, -0.3528973753912634}, {
      0.6392543474170711, -0.3513273690578397}, {
      0.6452745413413223, -0.3497216242140415}, {
      0.6512303020098815, -0.3480809567734621}, {
      0.6571222077085629, -0.34640615758920046`}, {
      0.6629508237882794, -0.344697993516897}, {
      0.6687167030112603, -0.3429572084222478}, {
      0.6744203858865403, -0.3411845241364476}, {
      0.6800624009949927, -0.33938064136276713`}, {
      0.6856432653041852, -0.33754624053724147`}, {
      0.6911634844733396, -0.33568198264623844`}, {
      0.6966235531486787, -0.3337885100034823}, {
      0.702023955249441, -0.3318664469889318}, {
      0.7073651642448446, -0.32991640075174783`}, {
      0.7126476434222773, -0.32793896187943467`}, {
      0.7178718461469842, -0.32593470503509925`}, {
      0.7230382161135247, -0.3239041895646449}, {
      0.7281471875892591, -0.32184796007559546`}, {
      0.733199185650124, -0.3197665469891363}, {
      0.7381946264089478, -0.3176604670668561}, {
      0.7431339172365521, -0.3155302239135779}, {
      0.7480174569758756, -0.3133763084575815}, {
      0.7528456361493558, -0.31119919940943574`}, {
      0.757618837159791, -0.3089993637005842}, {
      0.7623374344849021, -0.3067772569027579}, {
      0.7670017948658072, -0.30453332362922236`}, {
      0.7716122774896119, -0.3022679979188043}, {
      0.7761692341663153, -0.2999817036035891}, {
      0.7806730095002236, -0.2976748546611239}, {
      0.7851239410560555, -0.29534785555191484`}, {
      0.789522359519921, -0.29300110154295805`}, {
      0.7938685888553434, -0.29063497901800356`}, {
      0.7981629464544951, -0.2882498657752087}, {
      0.8024057432848054, -0.285846131312802}, {
      0.8065972840310971, -0.2834241371033418}, {
      0.8107378672334025, -0.2809842368571224}, {
      0.814827785420602, -0.2785267767752487}, {
      0.8188673252400253, -0.2760520957928712}, {
      0.8228567675831495, -0.2735605258130485}, {
      0.8267963877075241, -0.27105239193167524`}, {
      0.8306864553550463, -0.2685280126538934}, {
      0.8345272348667081, -0.26598770010238076`}, {
      0.8383189852939303, -0.2634317602178888}, {
      0.8420619605065953, -0.26086049295238434`}, {
      0.8457564092978853, -0.25827419245512956`}, {
      0.8494025754860309, -0.25567314725201756`}, {
      0.8530006980130682, -0.25305764041846524`}, {
      0.8565510110407015, -0.2504279497461487}, {
      0.8600537440433637, -0.24778434790385265`}, {
      0.8635091218985628, -0.24512710259269155`}, {
      0.8669173649746025, -0.24245647669594686`}, {
      0.8702786892157564, -0.23977272842375322`}, {
      0.8735933062249779, -0.23707611145285465`}, {
      0.8768614233442217, -0.23436687506164092`}, {
      0.8800832437324501, -0.23164526426066445`}, {
      0.8832589664413967, -0.22891151991882805`}, {
      0.8863887864891544, -0.2261658788854248}, {
      0.8894728949316555, -0.22340857410820325`}, {
      0.8925114789321056, -0.22063983474762197`}, {
      0.8955047218284347, -0.21785988628745104`}, {
      0.8984528031988227, -0.2150689506418697}, {
      0.9013558989253581, -0.21226724625920326`}, {
      0.9042141812558822, -0.2094549882224352}, {
      0.907027818864075, -0.20663238834662498`}, {
      0.9097969769078312, -0.20379965527335556`}, {
      0.9125218170859759, -0.20095699456232918`}, {
      0.9152024976933681, -0.19810460878022534`}, {
      0.9178391736744378, -0.1952426975869289}, {
      0.9204319966751987, -0.19237145781923223`}, {
      0.922981115093782, -0.18949108357211097`}, {
      0.925486674129529, -0.18660176627766772`}, {
      0.9279488158306848, -0.1837036947818353}, {
      0.9303676791407285, -0.18079705541892607`}, {
      0.9327433999433786, -0.17788203208411127`}, {
      0.935076111106308, -0.17495880630390986`}, {
      0.9373659425236021, -0.1720275573047641}, {
      0.9396130211569944, -0.16908846207977485`}, {
      0.9418174710759097, -0.16614169545366753`}, {
      0.9439794134963462, -0.16318743014605644`}, {
      0.9460989668186254, -0.16022583683307232`}, {
      0.9481762466640394, -0.15725708420741574`}, {
      0.9502113659104213, -0.1542813390368963}, {
      0.9522044347266658, -0.15129876622151553`}, {
      0.954155560606226, -0.14830952884914858`}, {
      0.9560648483996096, -0.14531378824987856`}, {
      0.9579324003458994, -0.1423117040490346}, {
      0.9597583161033203, -0.13930343421898325`}, {
      0.9615426927788735, -0.13628913512972082`}, {
      0.9632856249570618, -0.13326896159831253`}, {
      0.9649872047277238, -0.1302430669372229}, {
      0.9666475217129972, -0.1272116030015799}, {
      0.9682666630934311, -0.12417472023541415`}, {
      0.9698447136332647, -0.1211325677169129}, {
      0.9713817557048888, -0.11808529320272729`}, {
      0.9728778693125093, -0.11503304317136984`}, {
      0.9743331321150267, -0.11197596286573817`}, {
      0.9757476194481487, -0.10891419633479961`}, {
      0.9771214043457495, -0.10584788647447023`}, {
      0.9784545575604912, -0.10277717506772094`}, {
      0.9797471475837207, -0.09970220282394193}, {
      0.9809992406646558, -0.09662310941759623}, {
      0.9822109008288719, -0.09354003352619177}, {
      0.983382189896103, -0.09045311286760077}, {
      0.9845131674973686, -0.08736248423675444}, {
      0.9856038910914359, -0.08426828354173985}, {
      0.9866544159806306, -0.08117064583932557}, {
      0.9876647953260047, -0.07806970536994157}, {
      0.9886350801618714, -0.07496559559213838}, {
      0.9895653194097174, -0.07185844921654978}, {
      0.9904555598914999, -0.0687483982393828}, {
      0.9913058463423388, -0.065635573975458}, {
      0.9921162214226101, -0.06252010709082285}, {
      0.9928867257294499, -0.059402127634960035`}, {
      0.9936173978076749, -0.05628176507261229}, {
      0.9943082741601271, -0.053159148315245056`}, {
      0.9949593892574494, -0.05003440575216743}, {
      0.9955707755472973, -0.046907665281331806`}, {
      0.996142463462993, -0.04377905433983202}, {
      0.9966744814316274, -0.04064869993411964}, {
      0.9971668558816152, -0.037516728669957505`}, {
      0.9976196112497071, -0.0343832667821295}, {
      0.9980327699874634, -0.031248440163925154`}, {
      0.9984063525671945, -0.028112374396417367`}, {
      0.99874037748737, -0.024975194777551515`}, {
      0.9990348612775011, -0.021837026351063698`}, {
      0.9992898185024982, -0.018697993935245915`}, {
      0.9995052617665079, -0.015558222151575674`}, {
      0.99968120171623, -0.012417835453227402`}, {
      0.9998176470437178, -0.009276958153482909}, {
      0.999914604488663, -0.006135714454058013}, {
      0.9999720788401661, -0.0029942284733623674`}, {0.9999900729379949, 
      0.0001473757252905706}, {0.9999685876733293, 0.0032889741055065294`}, {
      0.999907621988996, 0.006430442629649515}, {0.9998071728791903, 
      0.009571657230680168}, {0.9996672353886874, 0.012712493783976552`}, {
      0.9994878026115399, 0.015852828079120494`}, {0.9992688656892641, 
      0.018992535791632615`}, {0.9990104138085106, 0.022131492454639055`}, {
      0.9987124341982204, 0.02526957343045288}, {0.9983749121262631, 
      0.028406653882053042`}, {0.9979978308955558, 0.031542608744443625`}, {
      0.9975811718396612, 0.034677312695876004`}, {0.9971249143178597, 
      0.037810640128916384`}, {0.996629035709695, 0.040942465121340955`}, {
      0.996093511408989, 0.04407266140684082}, {0.9955183148173219, 
      0.047201102345518456`}, {0.9949034173369734, 0.050327660894157446`}, {
      0.9942487883633219, 0.053452209576246704`}, {0.9935543952766949, 
      0.05657462045174039}, {0.9928202034336681, 0.0596947650865342}, {
      0.992046176157805, 0.0628125145216385}, {0.9912322747298337, 
      0.06592773924202829}, {0.990378458377253, 0.06904030914514989}, {
      0.9894846842633622, 0.07215009350906336}, {0.9885509074757073, 
      0.07525696096019972}, {0.9875770810139364, 0.07836077944071126}, {
      0.9865631557770567, 0.08146141617539285}, {0.9855090805500849, 
      0.08455873763815161}, {0.9844148019900824, 0.08765260951800188}, {
      0.9832802646115664, 0.09074289668456156}, {0.9821054107712875, 
      0.0938294631530256}, {0.9808901806523646, 0.0969121720485915}, {
      0.9796345122477648, 0.09999088557031138}, {0.9783383413431208, 
      0.10306546495434385`}, {0.9770016014988714, 0.10613577043657874`}, {
      0.9756242240317149, 0.10920166121460677`}, {0.9742061379953636, 
      0.11226299540900517`}, {0.9727472701605854, 0.1153196300239097}, {
      0.9712475449945196, 0.11837142090684245`}, {0.9697068846392541, 
      0.12141822270776381`}, {0.968125208889648, 0.12445988883731597`}, {
      0.9665024351703855, 0.12749627142422443`}, {0.9648384785122451, 
      0.13052722127182256`}, {0.963133251527569, 0.1335525878136633}, {
      0.9613866643849135, 0.13657221906818073`}, {0.9595986247828654, 
      0.13958596159236314`}, {0.9577690379230046, 0.14259366043439747`}, {
      0.9558978064819946, 0.1455951590852441}, {0.9539848305827809, 
      0.14859029942909868`}, {0.9520300077648773, 0.1515789216926973}, {
      0.9500332329537187, 0.15456086439341818`}, {0.947994398429058, 
      0.15753596428613267`}, {0.9459133937923857, 0.1605040563087555}, {
      0.9437901059333466, 0.16346497352644296`}, {0.9416244189951302, 
      0.16641854707438541`}, {0.9394162143388095, 0.16936460609913842`}, {
      0.9371653705066012, 0.17230297769843445`}, {0.9348717631840195, 
      0.17523348685941528`}, {0.9325352651608965, 0.17815595639522197`}, {
      0.9301557462912379, 0.1810702068798774}, {0.9277330734518848, 
      0.18397605658139346`}, {0.9252671104999495, 0.18687332139303175`}, {
      0.9227577182289911, 0.18976181476264428`}, {0.9202047543238995, 
      0.1926413476200171}, {0.9176080733144488, 0.19551172830213664`}, {
      0.9149675265274871, 0.19837276247629482`}, {0.9122829620377213, 
      0.20122425306094574`}, {0.9095542246170594, 0.2040660001442226}, {
      0.9067811556824686, 0.20689780090001936`}, {0.903963593242307, 
      0.20971944950153776`}}]}, 
   {RGBColor[1, 1, 0], DiskBox[{0, 0}, 0.03]}},
  AspectRatio->Automatic,
  Background->RGBColor[0.6, 0.6, 0.6],
  ImageSize->{329.80000000000024`, Automatic}]], "Output",
 CellChangeTimes->{3.401148216547473*^9, 3.4012013320408382`*^9, 
  3.401276962983686*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Perihelion = \\!\\(\\*SubscriptBox[\\(r\\), \\(min\\)]\\) = \"\
\>", "\[InvisibleSpace]", "0.14193187734268722`", 
   "\[InvisibleSpace]", "\<\" AU\"\>"}],
  SequenceForm[
  "Perihelion = \!\(\*SubscriptBox[\(r\), \(min\)]\) = ", 
   0.14193187734268722`, " AU"],
  Editable->False]], "Print",
 CellChangeTimes->{3.401148216534042*^9, 3.4012013319301767`*^9, 
  3.4012769630259237`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Aphelion = \\!\\(\\*SubscriptBox[\\(r\\), \\(max\\)]\\) = \
\"\>", "\[InvisibleSpace]", "1", "\[InvisibleSpace]", "\<\" AU\"\>"}],
  SequenceForm[
  "Aphelion = \!\(\*SubscriptBox[\(r\), \(max\)]\) = ", 1, " AU"],
  Editable->False]], "Print",
 CellChangeTimes->{3.401148216534042*^9, 3.4012013319301767`*^9, 
  3.401276963081995*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Semimajor Axis = a = \"\>", "\[InvisibleSpace]", 
   "0.5709659386713436`", "\[InvisibleSpace]", "\<\" AU\"\>"}],
  SequenceForm["Semimajor Axis = a = ", 0.5709659386713436, " AU"],
  Editable->False]], "Print",
 CellChangeTimes->{3.401148216534042*^9, 3.4012013319301767`*^9, 
  3.401276963133977*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Period = \"\>", "\[InvisibleSpace]", "0.43004067939590385`", 
   "\[InvisibleSpace]", "\<\" yr\"\>"}],
  SequenceForm["Period = ", 0.43004067939590385`, " yr"],
  Editable->False]], "Print",
 CellChangeTimes->{3.401148216534042*^9, 3.4012013319301767`*^9, 
  3.401276963180874*^9}]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
 "Now try ",
 Cell[BoxData[
  FormBox[
   RowBox[{"V0", "=", 
    RowBox[{"1.35", "\[Times]", "2", " ", "\[Pi]"}]}], TraditionalForm]]],
 ".  This will simulate a comet's orbit.  Keep increasing the ",
 StyleBox["tFinal",
  FontWeight->"Bold"],
 " until you see a closed orbit.  Try a ",
 StyleBox["SweptArea",
  FontWeight->"Bold"],
 " from 0 to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", "year"}], TraditionalForm]]],
 " and a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", "year"}], TraditionalForm]]],
 " interval when the comet is at its maximum distance from the sun, around \
half a period."
}], "Text"],

Cell[TextData[{
 "Now try ",
 Cell[BoxData[
  FormBox[
   RowBox[{"V0", "=", 
    RowBox[{
     RowBox[{"Sqrt", "[", "2", "]"}], " ", "2", " ", "\[Pi]"}]}], 
   TraditionalForm]]],
 ".  This will give a parabolic orbit."
}], "Text"],

Cell[TextData[{
 "Now try ",
 Cell[BoxData[
  FormBox[
   RowBox[{"V0", "=", 
    RowBox[{"1.5", "\[Times]", "2", " ", "\[Pi]"}]}], TraditionalForm]]],
 ".  This will give a hyperbolic orbit."
}], "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowToolbars->{},
InitializationCellEvaluation->True,
WindowSize->{974, 724},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
DockedCells->(FrontEndExecute[{
   FrontEnd`NotebookApply[
    FrontEnd`InputNotebook[], #, Placeholder]}]& ),
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
ShowSelection->True,
Magnification->2.,
FrontEndVersion->"6.0 for Mac OS X x86 (32-bit) (June 19, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[590, 23, 79, 2, 92, "Subtitle"],
Cell[CellGroupData[{
Cell[694, 29, 31, 0, 134, "Section"],
Cell[CellGroupData[{
Cell[750, 33, 39, 0, 68, "Subsection"],
Cell[792, 35, 814, 24, 181, "Text"],
Cell[1609, 61, 1094, 36, 81, "Text"],
Cell[2706, 99, 508, 13, 114, "Text"],
Cell[3217, 114, 313, 9, 72, "Text"],
Cell[3533, 125, 536, 19, 72, "Text"],
Cell[4072, 146, 121, 3, 82, "Text"],
Cell[4196, 151, 363, 13, 54, "Text"],
Cell[4562, 166, 578, 19, 147, "Text"],
Cell[5143, 187, 879, 26, 73, "Text"],
Cell[6025, 215, 38, 0, 52, "Text"],
Cell[6066, 217, 1026, 34, 73, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7129, 256, 39, 0, 68, "Subsection"],
Cell[7171, 258, 221, 7, 82, "Text"],
Cell[7395, 267, 380, 11, 54, "Text"],
Cell[7778, 280, 564, 18, 72, "Text"],
Cell[8345, 300, 164, 6, 54, "Text"],
Cell[8512, 308, 482, 16, 60, "Text"],
Cell[8997, 326, 1040, 38, 154, "Text"],
Cell[10040, 366, 914, 38, 57, "Text"],
Cell[10957, 406, 59, 0, 52, "Text"],
Cell[11019, 408, 379, 12, 55, "Text"],
Cell[11401, 422, 187, 6, 54, "Text"],
Cell[11591, 430, 748, 22, 64, "Text"],
Cell[12342, 454, 1021, 28, 218, "Text"],
Cell[13366, 484, 866, 26, 64, "Text"],
Cell[14235, 512, 1408, 41, 220, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15680, 558, 38, 0, 68, "Subsection"],
Cell[15721, 560, 179, 8, 52, "Text"],
Cell[15903, 570, 350, 14, 74, "Text"],
Cell[16256, 586, 1094, 44, 148, "Text"],
Cell[17353, 632, 599, 22, 81, "Text"],
Cell[17955, 656, 115, 3, 82, "Text"],
Cell[18073, 661, 413, 15, 79, "Text"],
Cell[18489, 678, 625, 21, 120, "Text"],
Cell[19117, 701, 436, 17, 73, "Text"],
Cell[19556, 720, 522, 12, 172, "Text"],
Cell[20081, 734, 463, 19, 71, "Text"],
Cell[20547, 755, 195, 8, 57, "Text"],
Cell[20745, 765, 863, 34, 79, "Text"],
Cell[21611, 801, 73, 0, 52, "Text"],
Cell[21687, 803, 792, 29, 82, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[22516, 837, 50, 0, 68, "Subsection"],
Cell[22569, 839, 197, 6, 54, "Text"],
Cell[22769, 847, 363, 13, 54, "Text"],
Cell[23135, 862, 162, 6, 54, "Text"],
Cell[23300, 870, 1026, 34, 73, "Text"],
Cell[24329, 906, 198, 6, 60, "Text"],
Cell[24530, 914, 564, 19, 66, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[25131, 938, 91, 4, 68, "Subsection"],
Cell[25225, 944, 15857, 446, 2514, "Input",
 InitializationCell->True],
Cell[41085, 1392, 262, 7, 52, "Text"],
Cell[41350, 1401, 519, 21, 112, "Text"],
Cell[41872, 1424, 235, 8, 82, "Text"],
Cell[42110, 1434, 224, 6, 52, "Text"]
}, {1, 3, 4, 5, 6}]]
}, Open  ]],
Cell[CellGroupData[{
Cell[42392, 1446, 76, 1, 126, "Section"],
Cell[42471, 1449, 515, 19, 82, "Text"],
Cell[CellGroupData[{
Cell[43011, 1472, 567, 14, 115, "Input"],
Cell[43581, 1488, 1591, 26, 602, "Output"]
}, Open  ]],
Cell[45187, 1517, 251, 7, 82, "Text"],
Cell[CellGroupData[{
Cell[45463, 1528, 573, 14, 115, "Input"],
Cell[46039, 1544, 10167, 175, 632, "Output"]
}, Open  ]],
Cell[56221, 1722, 407, 12, 142, "Text"],
Cell[CellGroupData[{
Cell[56653, 1738, 584, 14, 115, "Input"],
Cell[57240, 1754, 10475, 171, 456, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[67752, 1930, 582, 14, 115, "Input"],
Cell[68337, 1946, 33439, 555, 449, "Output"]
}, Open  ]],
Cell[101791, 2504, 179, 4, 52, "Text"],
Cell[CellGroupData[{
Cell[101995, 2512, 632, 15, 115, "Input"],
Cell[102630, 2529, 99569, 1645, 467, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[202248, 4180, 116, 2, 126, "Section"],
Cell[202367, 4184, 492, 17, 112, "Text"],
Cell[CellGroupData[{
Cell[202884, 4205, 596, 15, 115, "Input"],
Cell[203483, 4222, 97666, 1676, 615, "Output"]
}, Open  ]],
Cell[301164, 5901, 470, 16, 82, "Text"],
Cell[CellGroupData[{
Cell[301659, 5921, 616, 15, 115, "Input"],
Cell[302278, 5938, 97836, 1627, 640, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[400151, 7570, 615, 15, 115, "Input"],
Cell[400769, 7587, 245518, 4167, 749, "Output"]
}, Open  ]],
Cell[646302, 11757, 399, 15, 82, "Text"],
Cell[CellGroupData[{
Cell[646726, 11776, 641, 15, 115, "Input"],
Cell[647370, 11793, 97621, 1676, 671, "Output"]
}, Open  ]],
Cell[745006, 13472, 263, 8, 82, "Text"],
Cell[CellGroupData[{
Cell[745294, 13484, 782, 18, 145, "Input"],
Cell[746079, 13504, 55402, 924, 640, "Output"]
}, Open  ]],
Cell[801496, 14431, 169, 7, 52, "Text"],
Cell[CellGroupData[{
Cell[801690, 14442, 689, 16, 115, "Input"],
Cell[802382, 14460, 197971, 3525, 422, "Output"]
}, Open  ]],
Cell[1000368, 17988, 144, 3, 82, "Text"],
Cell[1000515, 17993, 513, 20, 82, "Text"],
Cell[CellGroupData[{
Cell[1001053, 18017, 690, 16, 115, "Input"],
Cell[1001746, 18035, 196805, 3224, 643, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1198588, 21264, 690, 16, 115, "Input"],
Cell[1199281, 21282, 198120, 3386, 623, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1397450, 24674, 113, 2, 126, "Section"],
Cell[1397566, 24678, 605, 15, 202, "Text"],
Cell[CellGroupData[{
Cell[1398196, 24697, 48, 0, 68, "Subsection"],
Cell[1398247, 24699, 60, 0, 52, "Text"],
Cell[CellGroupData[{
Cell[1398332, 24703, 778, 20, 175, "Input"],
Cell[CellGroupData[{
Cell[1399135, 24727, 780, 16, 78, "Print"],
Cell[1399918, 24745, 789, 16, 78, "Print"]
}, Open  ]],
Cell[1400722, 24764, 49992, 837, 475, "Output"],
Cell[CellGroupData[{
Cell[1450739, 25605, 435, 10, 42, "Print"],
Cell[1451177, 25617, 383, 8, 40, "Print"],
Cell[1451563, 25627, 351, 7, 40, "Print"],
Cell[1451917, 25636, 330, 7, 40, "Print"]
}, Open  ]]
}, Open  ]],
Cell[1452274, 25647, 636, 22, 142, "Text"],
Cell[1452913, 25671, 232, 9, 52, "Text"],
Cell[1453148, 25682, 204, 7, 52, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
