(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    185494,       3845]
NotebookOptionsPosition[    181867,       3728]
NotebookOutlinePosition[    182439,       3751]
CellTagsIndexPosition[    182396,       3748]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
Motion of Charged Particles
in Electric and Magnetic Fields\
\>", "Subtitle",
 TextAlignment->Center,
 TextJustification->0],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell[CellGroupData[{

Cell["Particle Dynamics", "Subsection"],

Cell[TextData[{
 "\tWe can numerically simulate the dynamics of a particle moving under some \
force.  We write the net force acting on the particle as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["F", "\[RightVector]"], "(", 
    RowBox[{
     OverscriptBox["r", "\[RightVector]"], ",", 
     OverscriptBox["v", "\[RightVector]"], ",", "t"}], ")"}], 
   TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["r", "\[RightVector]"], TraditionalForm]]],
 " is the particle's position vector, ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["v", "\[RightVector]"], TraditionalForm]]],
 " is its velocity vector and ",
 StyleBox["t",
  FontSlant->"Italic"],
 " is the time.  Newton's second law gives a differential equation that can \
be solved to describe the motion.  "
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{
      OverscriptBox["F", "\[RightVector]"], "(", 
      RowBox[{
       OverscriptBox["r", "\[RightVector]"], ",", 
       OverscriptBox["v", "\[RightVector]"], ",", "t"}], ")"}],
     TraditionalForm], "=", 
    RowBox[{"m", " ", 
     OverscriptBox["a", "\[RightVector]"]}]}], TraditionalForm]]],
 "  \[DoubleLongRightArrow]  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox[
      SuperscriptBox["\[DifferentialD]", "2"], 
      RowBox[{"\[DifferentialD]", 
       SuperscriptBox["t", "2"]}]], " ", 
     OverscriptBox["r", "\[RightVector]"]}], "=", 
    RowBox[{
     FractionBox["1", "m"], " ", 
     RowBox[{
      OverscriptBox["F", "\[RightVector]"], "(", 
      RowBox[{
       OverscriptBox["r", "\[RightVector]"], ",", 
       RowBox[{
        FractionBox["\[DifferentialD]", 
         RowBox[{"\[DifferentialD]", "t"}]], " ", 
        OverscriptBox["r", "\[RightVector]"]}], ",", "t"}], ")"}]}]}], 
   TraditionalForm]]]
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "This is a second order ordinary differential equation (ODE), where the \
order is the highest number of derivatives.  An ODE can always be reduced to \
a coupled set of first order equations.  In this case we use ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["v", "\[RightVector]"], "=", 
    RowBox[{
     RowBox[{"\[DifferentialD]", 
      OverscriptBox["r", "\[RightVector]"]}], "/", 
     RowBox[{"\[DifferentialD]", "t"}]}]}], TraditionalForm]]],
 " and write"
}], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["\[DifferentialD]", 
     RowBox[{"\[DifferentialD]", "t"}]], " ", 
    OverscriptBox["r", "\[RightVector]"]}], "=", 
   OverscriptBox["v", "\[RightVector]"]}], TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["\[DifferentialD]", 
      RowBox[{"\[DifferentialD]", "t"}]], " ", 
     OverscriptBox["v", "\[RightVector]"]}], "=", 
    RowBox[{
     FractionBox["1", "m"], " ", 
     RowBox[{
      OverscriptBox["F", "\[RightVector]"], "(", 
      RowBox[{
       OverscriptBox["r", "\[RightVector]"], ",", 
       OverscriptBox["v", "\[RightVector]"], ",", "t"}], ")"}]}]}], 
   TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
\tThe general problem of motion can be written in terms of a vector formed \
from these variables\
\>", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["z", "\[RightVector]"], "=", 
    RowBox[{"\[LeftAngleBracket]", 
     RowBox[{
      OverscriptBox["r", "\[RightVector]"], ",", 
      OverscriptBox["v", "\[RightVector]"]}], "\[RightAngleBracket]"}]}], 
   TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "In two dimension this would represent 4 variables, ",
 StyleBox["x",
  FontSlant->"Italic"],
 ", ",
 StyleBox["y",
  FontSlant->"Italic"],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", "x"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", "y"], TraditionalForm]]],
 ".  In three dimensions it would be 6 variables.  Think of this 'vector' as \
a collection of four (or six) numbers and not as anything with a three \
dimensional geometrical interpretation.  The differential equation can then \
be rewritten as"
}], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[LeftAngleBracket]", 
    RowBox[{
     RowBox[{
      FractionBox["\[DifferentialD]", 
       RowBox[{"\[DifferentialD]", "t"}]], 
      OverscriptBox["r", "\[RightVector]"]}], ",", 
     RowBox[{
      FractionBox["\[DifferentialD]", 
       RowBox[{"\[DifferentialD]", "t"}]], 
      OverscriptBox["v", "\[RightVector]"]}]}], "\[RightAngleBracket]"}], "=",
    
   RowBox[{"\[LeftAngleBracket]", 
    RowBox[{
     OverscriptBox["v", "\[RightVector]"], ",", 
     RowBox[{
      FractionBox["1", "m"], 
      RowBox[{
       OverscriptBox["F", "\[RightVector]"], "(", 
       RowBox[{
        OverscriptBox["r", "\[RightVector]"], ",", 
        OverscriptBox["v", "\[RightVector]"], ",", "t"}], ")"}]}]}], 
    "\[RightAngleBracket]"}]}], TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell["or in the general form", "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["\[DifferentialD]", 
      RowBox[{"\[DifferentialD]", "t"}]], 
     OverscriptBox["z", "\[RightVector]"]}], "=", 
    RowBox[{
     OverscriptBox["f", "\[RightVector]"], "(", 
     RowBox[{
      OverscriptBox["z", "\[RightVector]"], ",", "t"}], ")"}]}], 
   TraditionalForm]]],
 "  where  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     OverscriptBox["f", "\[RightVector]"], "(", 
     RowBox[{
      OverscriptBox["z", "\[RightVector]"], ",", "t"}], ")"}], "=", 
    RowBox[{"\[LeftAngleBracket]", 
     RowBox[{
      OverscriptBox["v", "\[RightVector]"], ",", 
      RowBox[{
       FractionBox["1", "m"], 
       RowBox[{
        OverscriptBox["F", "\[RightVector]"], "(", 
        RowBox[{
         OverscriptBox["r", "\[RightVector]"], ",", 
         OverscriptBox["v", "\[RightVector]"], ",", "t"}], ")"}]}]}], 
     "\[RightAngleBracket]"}]}], TraditionalForm]]]
}], "Text",
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]],

Cell[CellGroupData[{

Cell["Numerical Methods", "Subsection"],

Cell[TextData[{
 "\tWe can step forward the position and velocity by an infinitesimal time \
increment ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DifferentialD]", "t"}], TraditionalForm]]],
 " using the rule:"
}], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{" ", 
   RowBox[{
    OverscriptBox["r", "\[RightVector]"], " ", "\[Rule]", " ", 
    RowBox[{
     OverscriptBox["r", "\[RightVector]"], "+", 
     RowBox[{
      OverscriptBox["v", "\[RightVector]"], " ", 
      RowBox[{"\[DifferentialD]", "t"}]}]}]}]}], TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["v", "\[RightVector]"], " ", "\[Rule]", " ", 
    RowBox[{
     OverscriptBox["v", "\[RightVector]"], "+", 
     RowBox[{
      FractionBox["1", "m"], " ", 
      RowBox[{
       OverscriptBox["F", "\[RightVector]"], "(", 
       RowBox[{
        OverscriptBox["r", "\[RightVector]"], ",", 
        OverscriptBox["v", "\[RightVector]"], ",", "t"}], ")"}], " ", 
      RowBox[{"\[DifferentialD]", "t"}]}]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "In terms of the variable ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["z", "\[RightVector]"], TraditionalForm]]],
 " this is written"
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["z", "\[RightVector]"], " ", "\[Rule]", " ", 
    RowBox[{
     OverscriptBox["z", "\[RightVector]"], "+", " ", 
     RowBox[{
      RowBox[{
       OverscriptBox["f", "\[RightVector]"], "(", 
       RowBox[{
        OverscriptBox["z", "\[RightVector]"], ",", "t"}], ")"}], " ", 
      RowBox[{"\[DifferentialD]", "t"}]}]}]}], TraditionalForm]]],
 "."
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "To numerically integrate a system forward in time we approximate the \
infinitesimal ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DifferentialD]", "t"}], TraditionalForm]]],
 " with the finite ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]", "\[ThinSpace]", "t"}], TraditionalForm]]],
 ".  We start at time ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "0"], TraditionalForm]]],
 " at the \"position\", actually the position and velocity, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["z", "\[RightVector]"], "0"], TraditionalForm]]],
 ".  This is then stepped forward to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "1"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["z", "\[RightVector]"], "1"], TraditionalForm]]],
 " then to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "2"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["z", "\[RightVector]"], "2"], TraditionalForm]]],
 ", etc."
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "0"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["z", "\[RightVector]"], "0"], TraditionalForm]]],
 "  \[Rule]  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "1"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["z", "\[RightVector]"], "1"], TraditionalForm]]],
 "  \[Rule]  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "2"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["z", "\[RightVector]"], "2"], TraditionalForm]]],
 "  \[Rule]  \[Ellipsis]  \[Rule]  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "final"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["z", "\[RightVector]"], "final"], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.4007943238455963`*^9, 3.400794323845633*^9}},
 TextAlignment->Center,
 TextJustification->0],

Cell["\tThe rule for stepping forward the time is", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["t", "i"], "  ", "\[Rule]", "  ", 
    SubscriptBox["t", 
     RowBox[{"i", "+", "1"}]]}], "=", 
   RowBox[{
    SubscriptBox["t", "i"], "+", 
    RowBox[{"\[CapitalDelta]", "\[ThinSpace]", 
     SubscriptBox["t", "i"]}]}]}], TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "The rule for stepping to the next ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["z", "\[RightVector]"], TraditionalForm]]],
 " variable will be written as: "
}], "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     OverscriptBox["z", "\[RightVector]"], "i"], "  ", "\[Rule]", "  ", 
    SubscriptBox[
     OverscriptBox["z", "\[RightVector]"], 
     RowBox[{"i", "+", "1"}]]}], "=", 
   RowBox[{
    SubscriptBox[
     OverscriptBox["z", "\[RightVector]"], "i"], "+", 
    RowBox[{
     OverscriptBox["S", "\[RightVector]"], "(", 
     RowBox[{
      SubscriptBox[
       OverscriptBox["z", "\[RightVector]"], "i"], ",", 
      SubscriptBox["t", "i"], ",", 
      OverscriptBox["f", "\[RightVector]"], ",", 
      RowBox[{"\[CapitalDelta]", "\[ThinSpace]", 
       SubscriptBox["t", "i"]}]}], ")"}]}]}], TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "Where the numerical method used will be given by the stepping function ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["S", "\[RightVector]"], TraditionalForm]]],
 ".  To improve accuracy one must choose a sufficiently small time step ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]", "\[ThinSpace]", 
    SubscriptBox["t", "i"]}], TraditionalForm]]],
 " and an efficient numerical method.  The smaller we choose the values of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]", "\[ThinSpace]", "t"}], TraditionalForm]]],
 " the larger the computer's calculation becomes, so it is essential to \
choose an efficient numerical method.  The simplest numerical method is known \
as Euler's method, where ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["S", "\[RightVector]"], TraditionalForm]]],
 " is determined by the previous values of ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["z", "\[RightVector]"], TraditionalForm]]],
 " and ",
 StyleBox["t",
  FontSlant->"Italic"],
 "."
}], "Text"],

Cell[TextData[{
 "                                                               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     OverscriptBox["S", "\[RightVector]"], "(", 
     RowBox[{
      SubscriptBox[
       OverscriptBox["z", "\[RightVector]"], "i"], ",", 
      SubscriptBox["t", "i"], ",", 
      OverscriptBox["f", "\[RightVector]"], ",", 
      RowBox[{"\[CapitalDelta]", "\[ThinSpace]", 
       SubscriptBox["t", "i"]}]}], ")"}], "=", 
    RowBox[{
     RowBox[{
      OverscriptBox["f", "\[RightVector]"], "(", 
      RowBox[{
       SubscriptBox[
        OverscriptBox["z", "\[RightVector]"], "i"], ",", 
       SubscriptBox["t", "i"]}], ")"}], "\[CapitalDelta]", "\[ThinSpace]", 
     SubscriptBox["t", "i"]}]}], TraditionalForm]]],
 "                                   (Euler's Method)"
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "Euler's method is conceptually simple but it's accuracy is only of order ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]", "\[ThinSpace]", 
    SubscriptBox["t", "i"]}], TraditionalForm]]],
 ".  That means to decrease the error by a factor of 10 we must decrease ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]", "\[ThinSpace]", 
    SubscriptBox["t", "i"]}], TraditionalForm]]],
 " by a factor of 10.  A ",
 StyleBox["much",
  FontSlant->"Italic"],
 " more efficient method is called the ",
 StyleBox["fourth order Runge-Kutta method",
  FontSlant->"Italic"],
 ".  Fourth order means that decreasing ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalDelta]", "\[ThinSpace]", 
    SubscriptBox["t", "i"]}], TraditionalForm]]],
 " by a factor of 10 will decreas the error by a factor of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["10", "4"], "=", "10000"}], TraditionalForm]]],
 ".  The precise form of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["S", "\[RightVector]"], "(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["z", "\[RightVector]"], "i"], ",", 
     SubscriptBox["t", "i"], ",", 
     OverscriptBox["f", "\[RightVector]"], ",", 
     RowBox[{"\[CapitalDelta]", "\[ThinSpace]", 
      SubscriptBox["t", "i"]}]}], ")"}], TraditionalForm]]],
 " will not be given here but the interested reader can find it in the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " code."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Lorentz Force Law", "Subsection"],

Cell["\<\
\tWe are studying the motion of charged particles in electric and magnetic \
fields where the force is given by the Lorentz force law:\
\>", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   OverscriptBox["F", "\[RightVector]"], "=", 
   RowBox[{"Q", " ", 
    RowBox[{"(", 
     RowBox[{
      OverscriptBox["E", "\[RightVector]"], "+", 
      RowBox[{
       OverscriptBox["v", "\[RightVector]"], "\[Cross]", 
       OverscriptBox["B", "\[RightVector]"]}]}], ")"}]}]}], 
  TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "General ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Code"
}], "Subsection"],

Cell[TextData[{
 "The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " code for the functions defined above is here.  Evaluate it.  To evaluate a \
cell click the mouse anywhere inside the cell and click ",
 StyleBox["Shift-Enter",
  FontSlant->"Italic"],
 ".  The function ",
 StyleBox["RungeKutta4thOrder",
  FontWeight->"Bold"],
 " represents the ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["F", "\[RightVector]"], TraditionalForm]]],
 " function.  The function ",
 StyleBox["IterateData",
  FontWeight->"Bold"],
 " creates a list of data elements of the form  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       SubscriptBox[
        OverscriptBox["r", "\[RightVector]"], "i"], ",", 
       SubscriptBox[
        OverscriptBox["v", "\[RightVector]"], "i"]}], "}"}], ",", 
     SubscriptBox["t", "i"]}], "}"}], TraditionalForm]]],
 ", starting with the initial value, and the functions ",
 StyleBox["DataPointPosition",
  FontWeight->"Bold"],
 ", ",
 StyleBox["DataPointVelocity",
  FontWeight->"Bold"],
 " and ",
 StyleBox["DataPointTime",
  FontWeight->"Bold"],
 " pull out the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["r", "\[RightVector]"], "i"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["v", "\[RightVector]"], "i"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "i"], TraditionalForm]]],
 " from each data element."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Off", "[", 
   RowBox[{"General", "::", "spell1"}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"RungeKutta4thOrder", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"z_", ",", "t_"}], "}"}], ",", "f_", ",", "\[CapitalDelta]t_"}],
     "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"With", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"k1", "=", 
         RowBox[{"\[CapitalDelta]t", " ", 
          RowBox[{"f", "[", 
           RowBox[{"z", ",", "t"}], "]"}]}]}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"With", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"k2", "=", 
           RowBox[{"\[CapitalDelta]t", " ", 
            RowBox[{"f", "[", 
             RowBox[{
              RowBox[{"z", "+", 
               RowBox[{
                FractionBox["1", "2"], "k1"}]}], ",", 
              RowBox[{"t", "+", 
               RowBox[{
                FractionBox["1", "2"], "\[CapitalDelta]t"}]}]}], "]"}]}]}], 
          "}"}], ",", "\[IndentingNewLine]", 
         RowBox[{"With", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"k3", "=", 
             RowBox[{"\[CapitalDelta]t", " ", 
              RowBox[{"f", "[", 
               RowBox[{
                RowBox[{"z", "+", 
                 RowBox[{
                  FractionBox["1", "2"], "k2"}]}], ",", 
                RowBox[{"t", "+", 
                 RowBox[{
                  FractionBox["1", "2"], "\[CapitalDelta]t"}]}]}], "]"}]}]}], 
            "}"}], ",", "\[IndentingNewLine]", 
           RowBox[{"With", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"k4", "=", 
               RowBox[{"\[CapitalDelta]t", " ", 
                RowBox[{"f", "[", 
                 RowBox[{
                  RowBox[{"z", "+", "k3"}], ",", 
                  RowBox[{"t", "+", "\[CapitalDelta]t"}]}], "]"}]}]}], "}"}], 
             ",", "\[IndentingNewLine]", 
             RowBox[{"z", "+", 
              RowBox[{
               FractionBox["1", "6"], 
               RowBox[{"(", 
                RowBox[{"k1", "+", 
                 RowBox[{"2", " ", "k2"}], "+", 
                 RowBox[{"2", " ", "k3"}], "+", "k4"}], ")"}]}]}]}], "]"}]}], 
          "]"}]}], "]"}]}], "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"t", "+", "\[CapitalDelta]t"}]}], "}"}]}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"DataPointPosition", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"r_", ",", "v_"}], "}"}], ",", "t_"}], "}"}], "]"}], ":=", 
  "r"}], "\n", 
 RowBox[{
  RowBox[{"DataPointVelocity", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"r_", ",", "v_"}], "}"}], ",", "t_"}], "}"}], "]"}], ":=", 
  "v"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"DataPointTime", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"r_", ",", "v_"}], "}"}], ",", "t_"}], "}"}], "]"}], ":=", 
   "t"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"IterateData", "[", "path_List", "]"}], ":=", "\[IndentingNewLine]",
   
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"InRegionQ", "[", 
      RowBox[{"DataPointPosition", "[", 
       RowBox[{"path", "\[LeftDoubleBracket]", 
        RowBox[{"-", "1"}], "\[RightDoubleBracket]"}], "]"}], "]"}], "\[And]",
      
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Length", "[", "path", "]"}], "\[LessEqual]", "iMax"}], 
      ")"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"IterateData", "[", "\[IndentingNewLine]", 
     RowBox[{"Append", "[", 
      RowBox[{"path", ",", "\[IndentingNewLine]", 
       RowBox[{"RungeKutta4thOrder", "[", 
        RowBox[{
         RowBox[{"path", "\[LeftDoubleBracket]", 
          RowBox[{"-", "1"}], "\[RightDoubleBracket]"}], ",", "RKFunction", 
         ",", 
         FractionBox["\[CapitalDelta]s", 
          RowBox[{"Norm", "[", 
           RowBox[{"DataPointVelocity", "[", 
            RowBox[{"path", "\[LeftDoubleBracket]", 
             RowBox[{"-", "1"}], "\[RightDoubleBracket]"}], "]"}], "]"}]]}], 
        "]"}]}], "\[IndentingNewLine]", "]"}], "]"}], ",", 
    "\[IndentingNewLine]", "path"}], "]"}]}], "\n", 
 RowBox[{
  RowBox[{"RKFunction", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"r_", ",", "v_"}], "}"}], ",", "t_"}], "]"}], ":=", 
  RowBox[{"{", 
   RowBox[{"v", ",", 
    RowBox[{"Acceleration", "[", 
     RowBox[{"r", ",", "v"}], "]"}]}], "}"}]}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Two Dimensional Case", "Subsection"],

Cell[TextData[{
 "\tHere we will study motion in the ",
 StyleBox["xy",
  FontSlant->"Italic"],
 " plane with a magnetic field perpendicular to the plane and an electric \
field in the plane in the ",
 StyleBox["x",
  FontSlant->"Italic"],
 " direction."
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["v", "\[RightVector]"], "=", 
    RowBox[{
     RowBox[{
      SubscriptBox["v", "x"], " ", 
      SubscriptBox[
       OverscriptBox["e", "^"], "x"]}], "+", 
     RowBox[{
      SubscriptBox["v", "y"], " ", 
      SubscriptBox[
       OverscriptBox["e", "^"], "y"]}]}]}], TraditionalForm]]],
 ",  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["B", "\[RightVector]"], "=", 
    RowBox[{"B", " ", 
     SubscriptBox[
      OverscriptBox["e", "^"], "z"]}]}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["E", "\[RightVector]"], "=", 
    RowBox[{"E", " ", 
     SubscriptBox[
      OverscriptBox["e", "^"], "x"]}]}], TraditionalForm]]]
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell["With this the two dimensional force and acceleration become", "Text"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   OverscriptBox["a", "\[RightVector]"], "=", 
   RowBox[{
    FractionBox[
     OverscriptBox["F", "\[RightVector]"], "m"], "=", 
    RowBox[{
     RowBox[{
      FractionBox["1", "m"], 
      RowBox[{"\[LeftAngleBracket]", 
       RowBox[{
        SubscriptBox["F", "x"], ",", 
        SubscriptBox["F", "y"]}], "\[RightAngleBracket]"}]}], "=", 
     RowBox[{
      FractionBox["Q", "m"], " ", 
      RowBox[{"\[LeftAngleBracket]", 
       RowBox[{
        RowBox[{
         SubscriptBox["E", "x"], "+", 
         RowBox[{
          SubscriptBox["v", "y"], " ", "B"}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          SubscriptBox["v", "x"]}], "B"}]}], "\[RightAngleBracket]"}]}]}]}]}],
   TraditionalForm]]]], "Text",
 TextAlignment->Center,
 TextJustification->0]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Code for 2D Case"
}], "Subsection"],

Cell[TextData[{
 "The ugliness that follows is the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " code to initiate the above calculation, display the trajectory graphically \
and print the output data.  The function ",
 StyleBox["CalculateAndDraw2D",
  FontWeight->"Bold"],
 " does all this.  ",
 "The function ",
 StyleBox["Acceleration2D",
  FontWeight->"Bold"],
 " represents the 2D acceleration expression above."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Acceleration2D", "[", 
   RowBox[{"r_", ",", 
    RowBox[{"{", 
     RowBox[{"vx_", ",", "vy_"}], "}"}]}], "]"}], ":=", 
  FractionBox[
   RowBox[{"Q", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Ex", "+", 
       RowBox[{"vy", " ", "Bz"}]}], ",", 
      RowBox[{
       RowBox[{"-", "vx"}], " ", "Bz"}]}], "}"}]}], "m"]}], "\n", 
 RowBox[{"CalculateAndDraw2D", ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"DataList", "=", 
      RowBox[{"IterateData", "[", 
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"0", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "v0"}], "}"}]}], "}"}], ",", "0"}], "}"}], 
        "}"}], "]"}]}], "}"}], ",", 
    RowBox[{
     RowBox[{"InRegionQ", "=", "InRegionQ2D"}], ";", 
     RowBox[{"Acceleration", "=", "Acceleration2D"}], ";", 
     RowBox[{"With", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"x", "=", 
          RowBox[{"100", " ", 
           RowBox[{
            RowBox[{"DataPointPosition", "[", 
             RowBox[{"DataList", "\[LeftDoubleBracket]", 
              RowBox[{"-", "1"}], "\[RightDoubleBracket]"}], "]"}], 
            "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}]}], ",", 
         RowBox[{"y", "=", 
          RowBox[{"100", " ", 
           RowBox[{
            RowBox[{"DataPointPosition", "[", 
             RowBox[{"DataList", "\[LeftDoubleBracket]", 
              RowBox[{"-", "1"}], "\[RightDoubleBracket]"}], "]"}], 
            "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}]}], ",", 
         RowBox[{"t", "=", 
          RowBox[{"DataPointTime", "[", 
           RowBox[{"DataList", "\[LeftDoubleBracket]", 
            RowBox[{"-", "1"}], "\[RightDoubleBracket]"}], "]"}]}], ",", 
         RowBox[{"v", "=", 
          RowBox[{"Norm", "[", 
           RowBox[{"DataPointVelocity", "[", 
            RowBox[{"DataList", "\[LeftDoubleBracket]", 
             RowBox[{"-", "1"}], "\[RightDoubleBracket]"}], "]"}], "]"}]}]}], 
        "}"}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{
         "\"\< Exit Data:\>\"", ",", "\"\< x\[ThinSpace]=\[ThinSpace]\>\"", 
          ",", "x", ",", "\"\<\[ThinSpace]cm, y\[ThinSpace]=\[ThinSpace]\>\"",
           ",", "y", ",", 
          "\"\<\[ThinSpace]cm, t\[ThinSpace]=\[ThinSpace]\>\"", ",", "t", 
          ",", "\"\<\[ThinSpace]sec\>\"", ",", 
          "\"\<, v\[ThinSpace]=\[ThinSpace]\>\"", ",", "v", ",", 
          "\"\<\[ThinSpace]m/s\>\""}], "]"}], ";", 
        RowBox[{"Graphics", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"Draw2DGrid", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Thickness", "[", "0.006`", "]"}], ",", 
              RowBox[{"RGBColor", "[", 
               RowBox[{"0", ",", "0", ",", "0.4`"}], "]"}], ",", 
              RowBox[{"Line", "[", 
               RowBox[{"DataPointPosition", "/@", "DataList"}], "]"}]}], 
             "}"}]}], "}"}], ",", 
          RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
          RowBox[{"ImageSize", "\[Rule]", 
           RowBox[{"{", 
            RowBox[{"600", ",", "450"}], "}"}]}]}], "]"}]}]}], "]"}]}]}], 
   "]"}]}], "\n", 
 RowBox[{
  RowBox[{"xMax", "=", 
   FractionBox["8", "100"]}], ";", 
  RowBox[{"yMax", "=", 
   FractionBox["12", "100"]}], ";", 
  RowBox[{"TickSize", "=", "0.001`"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"InRegionQ2D", "[", 
   RowBox[{"{", 
    RowBox[{"x_", ",", "y_"}], "}"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "xMax"}], "\[LessEqual]", "x", "\[LessEqual]", "xMax"}], "&&",
    
   RowBox[{"0", "\[LessEqual]", "y", "\[LessEqual]", "yMax"}]}]}], "\n", 
 RowBox[{
  RowBox[{"Draw2DGrid", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"RGBColor", "[", 
        RowBox[{"1", ",", "0.9`", ",", "0.7`"}], "]"}], ",", 
       RowBox[{"Rectangle", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "xMax"}], ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"xMax", ",", "yMax"}], "}"}]}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Thickness", "[", "0.002`", "]"}], ",", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "yMax"}], "}"}]}], "}"}], "]"}], ",", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Line", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"x", ",", 
                RowBox[{
                 RowBox[{"-", "0.6`"}], " ", "TickSize"}]}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"x", ",", 
                RowBox[{"0.6`", " ", "TickSize"}]}], "}"}]}], "}"}], "]"}], 
           ",", 
           RowBox[{"Line", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"x", ",", 
                RowBox[{"yMax", "-", 
                 RowBox[{"0.6`", " ", "TickSize"}]}]}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"x", ",", 
                RowBox[{"yMax", "+", 
                 RowBox[{"0.6`", " ", "TickSize"}]}]}], "}"}]}], "}"}], 
            "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"x", ",", 
           RowBox[{"-", "xMax"}], ",", "xMax", ",", 
           FractionBox["1", "500"]}], "}"}]}], "]"}], ",", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Line", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{
                RowBox[{
                 RowBox[{"-", "xMax"}], "-", 
                 RowBox[{"0.6`", " ", "TickSize"}]}], ",", "y"}], "}"}], ",", 
              
              RowBox[{"{", 
               RowBox[{
                RowBox[{
                 RowBox[{"-", "xMax"}], "+", 
                 RowBox[{"0.6`", " ", "TickSize"}]}], ",", "y"}], "}"}]}], 
             "}"}], "]"}], ",", 
           RowBox[{"Line", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{
                RowBox[{"xMax", "-", 
                 RowBox[{"0.6`", " ", "TickSize"}]}], ",", "y"}], "}"}], ",", 
              
              RowBox[{"{", 
               RowBox[{
                RowBox[{"xMax", "+", 
                 RowBox[{"0.6`", " ", "TickSize"}]}], ",", "y"}], "}"}]}], 
             "}"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"y", ",", "0", ",", "yMax", ",", 
           FractionBox["1", "500"]}], "}"}]}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Thickness", "[", "0.005`", "]"}], ",", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "xMax"}], ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"xMax", ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"xMax", ",", "yMax"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "xMax"}], ",", "yMax"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "xMax"}], ",", "0"}], "}"}]}], "}"}], "]"}], ",", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Line", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"x", ",", 
                RowBox[{"-", "TickSize"}]}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"x", ",", "TickSize"}], "}"}]}], "}"}], "]"}], ",", 
           RowBox[{"Line", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"x", ",", 
                RowBox[{"yMax", "-", "TickSize"}]}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"x", ",", 
                RowBox[{"yMax", "+", "TickSize"}]}], "}"}]}], "}"}], "]"}], 
           ",", 
           RowBox[{"Text", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"ToString", "[", 
               RowBox[{"100", " ", "x"}], "]"}], "<>", 
              "\"\<\[ThinSpace]cm\>\""}], ",", 
             RowBox[{"{", 
              RowBox[{"x", ",", 
               RowBox[{"-", "0.005`"}]}], "}"}]}], "]"}], ",", 
           RowBox[{"Text", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"ToString", "[", 
               RowBox[{"100", " ", "x"}], "]"}], "<>", 
              "\"\<\[ThinSpace]cm\>\""}], ",", 
             RowBox[{"{", 
              RowBox[{"x", ",", 
               RowBox[{"yMax", "+", "0.005`"}]}], "}"}]}], "]"}], ",", 
           RowBox[{"BaseStyle", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{"FontSize", "\[Rule]", "12"}], "}"}]}], ",", 
           RowBox[{"BaseStyle", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{"FontSize", "\[Rule]", "12"}], "}"}]}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"x", ",", 
           RowBox[{"-", "xMax"}], ",", "xMax", ",", 
           FractionBox["1", "100"]}], "}"}]}], "]"}], ",", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Line", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{
                RowBox[{
                 RowBox[{"-", "xMax"}], "-", "TickSize"}], ",", "y"}], "}"}], 
              ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{
                 RowBox[{"-", "xMax"}], "+", "TickSize"}], ",", "y"}], 
               "}"}]}], "}"}], "]"}], ",", 
           RowBox[{"Line", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{
                RowBox[{"xMax", "-", "TickSize"}], ",", "y"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"xMax", "+", "TickSize"}], ",", "y"}], "}"}]}], "}"}],
             "]"}], ",", 
           RowBox[{"Text", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"ToString", "[", 
               RowBox[{"100", " ", "y"}], "]"}], "<>", 
              "\"\<\[ThinSpace]cm\>\""}], ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "xMax"}], "-", "0.007`"}], ",", "y"}], "}"}]}], 
            "]"}], ",", 
           RowBox[{"Text", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"ToString", "[", 
               RowBox[{"100", " ", "y"}], "]"}], "<>", 
              "\"\<\[ThinSpace]cm\>\""}], ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"xMax", "+", "0.007`"}], ",", "y"}], "}"}]}], "]"}], 
           ",", 
           RowBox[{"BaseStyle", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{"FontSize", "\[Rule]", "12"}], "}"}]}], ",", 
           RowBox[{"BaseStyle", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{"FontSize", "\[Rule]", "12"}], "}"}]}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"y", ",", "0", ",", "yMax", ",", 
           FractionBox["1", "100"]}], "}"}]}], "]"}]}], "}"}]}], "}"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.396707201694784*^9, 3.396707212515293*^9}, {
   3.396707312309967*^9, 3.396707319333552*^9}, 3.39670741721657*^9, {
   3.40079484760828*^9, 3.4007948486292477`*^9}, {3.400795162547501*^9, 
   3.40079516568725*^9}, {3.4007952441437683`*^9, 3.400795253583199*^9}, {
   3.400795300213623*^9, 3.400795319168231*^9}, {3.4007953542670727`*^9, 
   3.400795357968852*^9}, 3.4008608360660257`*^9}],

Cell[TextData[{
 "The following assigns values to several masses, the electron, proton and \
alpha particle, and it gives the value of the elementary charge ",
 StyleBox["e",
  FontSlant->"Italic"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"mElectron", "=", 
   RowBox[{"9.109", "\[Times]", 
    SuperscriptBox["10", 
     RowBox[{"-", "31"}]]}]}], ";", 
  RowBox[{"mProton", "=", 
   RowBox[{"1.673", "\[Times]", 
    SuperscriptBox["10", 
     RowBox[{"-", "27"}]]}]}], ";", 
  RowBox[{"mAlpha", "=", 
   RowBox[{"6.645", "\[Times]", 
    SuperscriptBox["10", 
     RowBox[{"-", "27"}]]}]}], ";", 
  RowBox[{"e", "=", 
   RowBox[{"1.602", "\[Times]", 
    SuperscriptBox["10", 
     RowBox[{"-", "19"}]]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.396707201694784*^9, 3.396707212515293*^9}, {
   3.396707312309967*^9, 3.396707319333552*^9}, 3.39670741721657*^9, {
   3.40079484760828*^9, 3.4007948486292477`*^9}, {3.400795162547501*^9, 
   3.40079516568725*^9}, {3.4007952441437683`*^9, 3.400795253583199*^9}, {
   3.400795300213623*^9, 3.400795319168231*^9}, {3.4007953542670727`*^9, 
   3.400795357968852*^9}, 3.4008608360660257`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["(A) Magnetic Field",
 FontSize->16]], "Section"],

Cell[TextData[{
 "Consider an electron shot with a speed of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"6", "\[Times]", 
    SuperscriptBox["10", "5"], " ", 
    StyleBox[
     RowBox[{"m", "/", "s"}],
     FontSlant->"Plain"]}], TraditionalForm]]],
 " into just a magnetic field (zero electric field)..  Calculate the value of \
 ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "z"], TraditionalForm]]],
 " needed to deflect the electron to the right in a circle of 3 cm radius.  \
Modify the code to reflect this and evaluate it.  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "z"], TraditionalForm]]],
 " is out of the screen by convention; a negative value is into the screen."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"m", "=", "mElectron"}], ";", 
  RowBox[{"Q", "=", 
   RowBox[{"-", "e"}]}], ";", 
  RowBox[{"v0", "=", 
   RowBox[{"6", "\[Times]", 
    SuperscriptBox["10", "5"]}]}], ";", 
  RowBox[{"Ex", "=", "0"}], ";", 
  RowBox[{"Bz", "=", ".0005"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[CapitalDelta]s", "=", ".0005"}], ";", 
  RowBox[{"iMax", "=", "10000"}], ";", "CalculateAndDraw2D"}]}], "Input",
 CellChangeTimes->{3.40079487738813*^9, 3.400795260579022*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Exit Data:\"\>", 
   "\[InvisibleSpace]", "\<\" x\[ThinSpace]=\[ThinSpace]\"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"-", "1.3646139468243015`"}], 
   "\[InvisibleSpace]", "\<\"\[ThinSpace]cm, y\[ThinSpace]=\[ThinSpace]\"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"-", "0.006421349283720023`"}], 
   "\[InvisibleSpace]", "\<\"\[ThinSpace]cm, t\[ThinSpace]=\[ThinSpace]\"\>", 
   "\[InvisibleSpace]", "3.5833334141937266`*^-8", 
   "\[InvisibleSpace]", "\<\"\[ThinSpace]sec\"\>", 
   "\[InvisibleSpace]", "\<\", v\[ThinSpace]=\[ThinSpace]\"\>", 
   "\[InvisibleSpace]", "599999.9722764362`", 
   "\[InvisibleSpace]", "\<\"\[ThinSpace]m/s\"\>"}],
  SequenceForm[
  " Exit Data:", " x\[ThinSpace]=\[ThinSpace]", -1.3646139468243015`, 
   "\[ThinSpace]cm, y\[ThinSpace]=\[ThinSpace]", -0.006421349283720023, 
   "\[ThinSpace]cm, t\[ThinSpace]=\[ThinSpace]", 3.5833334141937266`*^-8, 
   "\[ThinSpace]sec", ", v\[ThinSpace]=\[ThinSpace]", 599999.9722764362, 
   "\[ThinSpace]m/s"],
  Editable->False]], "Print",
 CellChangeTimes->{3.400795362196336*^9, 3.400936092248106*^9}],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[1, 0.9, 0.7], 
     RectangleBox[NCache[{Rational[-2, 25], 0}, {-0.08, 0}], 
      NCache[{Rational[2, 25], Rational[3, 25]}, {0.08, 0.12}]]}, 
    {Thickness[0.002], 
     LineBox[NCache[{{0, 0}, {0, Rational[3, 25]}}, {{0, 0}, {0, 0.12}}]], {{
       LineBox[NCache[{{Rational[-2, 25], -0.0006}, {
          Rational[-2, 25], 0.0006}}, {{-0.08, -0.0006}, {-0.08, 0.0006}}]], 
       LineBox[NCache[{{Rational[-2, 25], 0.11939999999999999`}, {
          Rational[-2, 25], 0.1206}}, {{-0.08, 0.11939999999999999`}, {-0.08, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[-39, 500], -0.0006}, {
          Rational[-39, 500], 0.0006}}, {{-0.078, -0.0006}, {-0.078, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-39, 500], 0.11939999999999999`}, {
          Rational[-39, 500], 0.1206}}, {{-0.078, 
         0.11939999999999999`}, {-0.078, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-19, 250], -0.0006}, {
          Rational[-19, 250], 0.0006}}, {{-0.076, -0.0006}, {-0.076, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-19, 250], 0.11939999999999999`}, {
          Rational[-19, 250], 0.1206}}, {{-0.076, 
         0.11939999999999999`}, {-0.076, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-37, 500], -0.0006}, {
          Rational[-37, 500], 0.0006}}, {{-0.074, -0.0006}, {-0.074, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-37, 500], 0.11939999999999999`}, {
          Rational[-37, 500], 0.1206}}, {{-0.074, 
         0.11939999999999999`}, {-0.074, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-9, 125], -0.0006}, {
          Rational[-9, 125], 0.0006}}, {{-0.072, -0.0006}, {-0.072, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-9, 125], 0.11939999999999999`}, {
          Rational[-9, 125], 0.1206}}, {{-0.072, 
         0.11939999999999999`}, {-0.072, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-7, 100], -0.0006}, {
          Rational[-7, 100], 0.0006}}, {{-0.07, -0.0006}, {-0.07, 0.0006}}]], 
       LineBox[NCache[{{Rational[-7, 100], 0.11939999999999999`}, {
          Rational[-7, 100], 0.1206}}, {{-0.07, 0.11939999999999999`}, {-0.07,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[-17, 250], -0.0006}, {
          Rational[-17, 250], 0.0006}}, {{-0.068, -0.0006}, {-0.068, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-17, 250], 0.11939999999999999`}, {
          Rational[-17, 250], 0.1206}}, {{-0.068, 
         0.11939999999999999`}, {-0.068, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-33, 500], -0.0006}, {
          Rational[-33, 500], 0.0006}}, {{-0.066, -0.0006}, {-0.066, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-33, 500], 0.11939999999999999`}, {
          Rational[-33, 500], 0.1206}}, {{-0.066, 
         0.11939999999999999`}, {-0.066, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-8, 125], -0.0006}, {
          Rational[-8, 125], 0.0006}}, {{-0.064, -0.0006}, {-0.064, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-8, 125], 0.11939999999999999`}, {
          Rational[-8, 125], 0.1206}}, {{-0.064, 
         0.11939999999999999`}, {-0.064, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-31, 500], -0.0006}, {
          Rational[-31, 500], 0.0006}}, {{-0.062, -0.0006}, {-0.062, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-31, 500], 0.11939999999999999`}, {
          Rational[-31, 500], 0.1206}}, {{-0.062, 
         0.11939999999999999`}, {-0.062, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-3, 50], -0.0006}, {
          Rational[-3, 50], 0.0006}}, {{-0.06, -0.0006}, {-0.06, 0.0006}}]], 
       LineBox[NCache[{{Rational[-3, 50], 0.11939999999999999`}, {
          Rational[-3, 50], 0.1206}}, {{-0.06, 0.11939999999999999`}, {-0.06, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[-29, 500], -0.0006}, {
          Rational[-29, 500], 0.0006}}, {{-0.058, -0.0006}, {-0.058, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-29, 500], 0.11939999999999999`}, {
          Rational[-29, 500], 0.1206}}, {{-0.058, 
         0.11939999999999999`}, {-0.058, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-7, 125], -0.0006}, {
          Rational[-7, 125], 0.0006}}, {{-0.056, -0.0006}, {-0.056, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-7, 125], 0.11939999999999999`}, {
          Rational[-7, 125], 0.1206}}, {{-0.056, 
         0.11939999999999999`}, {-0.056, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-27, 500], -0.0006}, {
          Rational[-27, 500], 0.0006}}, {{-0.054, -0.0006}, {-0.054, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-27, 500], 0.11939999999999999`}, {
          Rational[-27, 500], 0.1206}}, {{-0.054, 
         0.11939999999999999`}, {-0.054, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-13, 250], -0.0006}, {
          Rational[-13, 250], 0.0006}}, {{-0.052, -0.0006}, {-0.052, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-13, 250], 0.11939999999999999`}, {
          Rational[-13, 250], 0.1206}}, {{-0.052, 
         0.11939999999999999`}, {-0.052, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-1, 20], -0.0006}, {
          Rational[-1, 20], 0.0006}}, {{-0.05, -0.0006}, {-0.05, 0.0006}}]], 
       LineBox[NCache[{{Rational[-1, 20], 0.11939999999999999`}, {
          Rational[-1, 20], 0.1206}}, {{-0.05, 0.11939999999999999`}, {-0.05, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[-6, 125], -0.0006}, {
          Rational[-6, 125], 0.0006}}, {{-0.048, -0.0006}, {-0.048, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-6, 125], 0.11939999999999999`}, {
          Rational[-6, 125], 0.1206}}, {{-0.048, 
         0.11939999999999999`}, {-0.048, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-23, 500], -0.0006}, {
          Rational[-23, 500], 0.0006}}, {{-0.046, -0.0006}, {-0.046, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-23, 500], 0.11939999999999999`}, {
          Rational[-23, 500], 0.1206}}, {{-0.046, 
         0.11939999999999999`}, {-0.046, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-11, 250], -0.0006}, {
          Rational[-11, 250], 0.0006}}, {{-0.044, -0.0006}, {-0.044, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-11, 250], 0.11939999999999999`}, {
          Rational[-11, 250], 0.1206}}, {{-0.044, 
         0.11939999999999999`}, {-0.044, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-21, 500], -0.0006}, {
          Rational[-21, 500], 0.0006}}, {{-0.042, -0.0006}, {-0.042, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-21, 500], 0.11939999999999999`}, {
          Rational[-21, 500], 0.1206}}, {{-0.042, 
         0.11939999999999999`}, {-0.042, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-1, 25], -0.0006}, {
          Rational[-1, 25], 0.0006}}, {{-0.04, -0.0006}, {-0.04, 0.0006}}]], 
       LineBox[
        NCache[{{Rational[-1, 25], 0.11939999999999999`}, {
          Rational[-1, 25], 0.1206}}, {{-0.04, 0.11939999999999999`}, {-0.04, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[-19, 500], -0.0006}, {
          Rational[-19, 500], 0.0006}}, {{-0.038, -0.0006}, {-0.038, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-19, 500], 0.11939999999999999`}, {
          Rational[-19, 500], 0.1206}}, {{-0.038, 
         0.11939999999999999`}, {-0.038, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-9, 250], -0.0006}, {
          Rational[-9, 250], 0.0006}}, {{-0.036, -0.0006}, {-0.036, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-9, 250], 0.11939999999999999`}, {
          Rational[-9, 250], 0.1206}}, {{-0.036, 
         0.11939999999999999`}, {-0.036, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-17, 500], -0.0006}, {
          Rational[-17, 500], 0.0006}}, {{-0.034, -0.0006}, {-0.034, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-17, 500], 0.11939999999999999`}, {
          Rational[-17, 500], 0.1206}}, {{-0.034, 
         0.11939999999999999`}, {-0.034, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-4, 125], -0.0006}, {
          Rational[-4, 125], 0.0006}}, {{-0.032, -0.0006}, {-0.032, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-4, 125], 0.11939999999999999`}, {
          Rational[-4, 125], 0.1206}}, {{-0.032, 
         0.11939999999999999`}, {-0.032, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-3, 100], -0.0006}, {
          Rational[-3, 100], 0.0006}}, {{-0.03, -0.0006}, {-0.03, 0.0006}}]], 
       LineBox[NCache[{{Rational[-3, 100], 0.11939999999999999`}, {
          Rational[-3, 100], 0.1206}}, {{-0.03, 0.11939999999999999`}, {-0.03,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[-7, 250], -0.0006}, {
          Rational[-7, 250], 0.0006}}, {{-0.028, -0.0006}, {-0.028, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-7, 250], 0.11939999999999999`}, {
          Rational[-7, 250], 0.1206}}, {{-0.028, 
         0.11939999999999999`}, {-0.028, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-13, 500], -0.0006}, {
          Rational[-13, 500], 0.0006}}, {{-0.026, -0.0006}, {-0.026, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-13, 500], 0.11939999999999999`}, {
          Rational[-13, 500], 0.1206}}, {{-0.026, 
         0.11939999999999999`}, {-0.026, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-3, 125], -0.0006}, {
          Rational[-3, 125], 0.0006}}, {{-0.024, -0.0006}, {-0.024, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-3, 125], 0.11939999999999999`}, {
          Rational[-3, 125], 0.1206}}, {{-0.024, 
         0.11939999999999999`}, {-0.024, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-11, 500], -0.0006}, {
          Rational[-11, 500], 0.0006}}, {{-0.022, -0.0006}, {-0.022, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-11, 500], 0.11939999999999999`}, {
          Rational[-11, 500], 0.1206}}, {{-0.022, 
         0.11939999999999999`}, {-0.022, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-1, 50], -0.0006}, {
          Rational[-1, 50], 0.0006}}, {{-0.02, -0.0006}, {-0.02, 0.0006}}]], 
       LineBox[NCache[{{Rational[-1, 50], 0.11939999999999999`}, {
          Rational[-1, 50], 0.1206}}, {{-0.02, 0.11939999999999999`}, {-0.02, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[-9, 500], -0.0006}, {
          Rational[-9, 500], 0.0006}}, {{-0.018, -0.0006}, {-0.018, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-9, 500], 0.11939999999999999`}, {
          Rational[-9, 500], 0.1206}}, {{-0.018, 
         0.11939999999999999`}, {-0.018, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-2, 125], -0.0006}, {
          Rational[-2, 125], 0.0006}}, {{-0.016, -0.0006}, {-0.016, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-2, 125], 0.11939999999999999`}, {
          Rational[-2, 125], 0.1206}}, {{-0.016, 
         0.11939999999999999`}, {-0.016, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-7, 500], -0.0006}, {
          Rational[-7, 500], 0.0006}}, {{-0.014, -0.0006}, {-0.014, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-7, 500], 0.11939999999999999`}, {
          Rational[-7, 500], 0.1206}}, {{-0.014, 
         0.11939999999999999`}, {-0.014, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-3, 250], -0.0006}, {
          Rational[-3, 250], 0.0006}}, {{-0.012, -0.0006}, {-0.012, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-3, 250], 0.11939999999999999`}, {
          Rational[-3, 250], 0.1206}}, {{-0.012, 
         0.11939999999999999`}, {-0.012, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-1, 100], -0.0006}, {
          Rational[-1, 100], 0.0006}}, {{-0.01, -0.0006}, {-0.01, 0.0006}}]], 
       LineBox[NCache[{{Rational[-1, 100], 0.11939999999999999`}, {
          Rational[-1, 100], 0.1206}}, {{-0.01, 0.11939999999999999`}, {-0.01,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[-1, 125], -0.0006}, {
          Rational[-1, 125], 0.0006}}, {{-0.008, -0.0006}, {-0.008, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-1, 125], 0.11939999999999999`}, {
          Rational[-1, 125], 0.1206}}, {{-0.008, 
         0.11939999999999999`}, {-0.008, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-3, 500], -0.0006}, {
          Rational[-3, 500], 0.0006}}, {{-0.006, -0.0006}, {-0.006, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-3, 500], 0.11939999999999999`}, {
          Rational[-3, 500], 0.1206}}, {{-0.006, 
         0.11939999999999999`}, {-0.006, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-1, 250], -0.0006}, {
          Rational[-1, 250], 0.0006}}, {{-0.004, -0.0006}, {-0.004, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-1, 250], 0.11939999999999999`}, {
          Rational[-1, 250], 0.1206}}, {{-0.004, 
         0.11939999999999999`}, {-0.004, 0.1206}}]]}, {
       LineBox[NCache[{{Rational[-1, 500], -0.0006}, {
          Rational[-1, 500], 0.0006}}, {{-0.002, -0.0006}, {-0.002, 
         0.0006}}]], 
       LineBox[NCache[{{Rational[-1, 500], 0.11939999999999999`}, {
          Rational[-1, 500], 0.1206}}, {{-0.002, 
         0.11939999999999999`}, {-0.002, 0.1206}}]]}, {
       LineBox[{{0, -0.0006}, {0, 0.0006}}], 
       LineBox[{{0, 0.11939999999999999`}, {0, 0.1206}}]}, {
       LineBox[NCache[{{Rational[1, 500], -0.0006}, {
          Rational[1, 500], 0.0006}}, {{0.002, -0.0006}, {0.002, 0.0006}}]], 
       LineBox[NCache[{{Rational[1, 500], 0.11939999999999999`}, {
          Rational[1, 500], 0.1206}}, {{0.002, 0.11939999999999999`}, {0.002, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[1, 250], -0.0006}, {
          Rational[1, 250], 0.0006}}, {{0.004, -0.0006}, {0.004, 0.0006}}]], 
       LineBox[NCache[{{Rational[1, 250], 0.11939999999999999`}, {
          Rational[1, 250], 0.1206}}, {{0.004, 0.11939999999999999`}, {0.004, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[3, 500], -0.0006}, {
          Rational[3, 500], 0.0006}}, {{0.006, -0.0006}, {0.006, 0.0006}}]], 
       LineBox[NCache[{{Rational[3, 500], 0.11939999999999999`}, {
          Rational[3, 500], 0.1206}}, {{0.006, 0.11939999999999999`}, {0.006, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[1, 125], -0.0006}, {
          Rational[1, 125], 0.0006}}, {{0.008, -0.0006}, {0.008, 0.0006}}]], 
       LineBox[NCache[{{Rational[1, 125], 0.11939999999999999`}, {
          Rational[1, 125], 0.1206}}, {{0.008, 0.11939999999999999`}, {0.008, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[1, 100], -0.0006}, {
          Rational[1, 100], 0.0006}}, {{0.01, -0.0006}, {0.01, 0.0006}}]], 
       LineBox[NCache[{{Rational[1, 100], 0.11939999999999999`}, {
          Rational[1, 100], 0.1206}}, {{0.01, 0.11939999999999999`}, {0.01, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[3, 250], -0.0006}, {
          Rational[3, 250], 0.0006}}, {{0.012, -0.0006}, {0.012, 0.0006}}]], 
       LineBox[NCache[{{Rational[3, 250], 0.11939999999999999`}, {
          Rational[3, 250], 0.1206}}, {{0.012, 0.11939999999999999`}, {0.012, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[7, 500], -0.0006}, {
          Rational[7, 500], 0.0006}}, {{0.014, -0.0006}, {0.014, 0.0006}}]], 
       LineBox[NCache[{{Rational[7, 500], 0.11939999999999999`}, {
          Rational[7, 500], 0.1206}}, {{0.014, 0.11939999999999999`}, {0.014, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[2, 125], -0.0006}, {
          Rational[2, 125], 0.0006}}, {{0.016, -0.0006}, {0.016, 0.0006}}]], 
       LineBox[NCache[{{Rational[2, 125], 0.11939999999999999`}, {
          Rational[2, 125], 0.1206}}, {{0.016, 0.11939999999999999`}, {0.016, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[9, 500], -0.0006}, {
          Rational[9, 500], 0.0006}}, {{0.018, -0.0006}, {0.018, 0.0006}}]], 
       LineBox[NCache[{{Rational[9, 500], 0.11939999999999999`}, {
          Rational[9, 500], 0.1206}}, {{0.018, 0.11939999999999999`}, {0.018, 
         0.1206}}]]}, {
       LineBox[
        NCache[{{Rational[1, 50], -0.0006}, {Rational[1, 50], 0.0006}}, {{
         0.02, -0.0006}, {0.02, 0.0006}}]], 
       LineBox[NCache[{{Rational[1, 50], 0.11939999999999999`}, {
          Rational[1, 50], 0.1206}}, {{0.02, 0.11939999999999999`}, {0.02, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[11, 500], -0.0006}, {
          Rational[11, 500], 0.0006}}, {{0.022, -0.0006}, {0.022, 0.0006}}]], 
       LineBox[NCache[{{Rational[11, 500], 0.11939999999999999`}, {
          Rational[11, 500], 0.1206}}, {{0.022, 0.11939999999999999`}, {0.022,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[3, 125], -0.0006}, {
          Rational[3, 125], 0.0006}}, {{0.024, -0.0006}, {0.024, 0.0006}}]], 
       LineBox[NCache[{{Rational[3, 125], 0.11939999999999999`}, {
          Rational[3, 125], 0.1206}}, {{0.024, 0.11939999999999999`}, {0.024, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[13, 500], -0.0006}, {
          Rational[13, 500], 0.0006}}, {{0.026, -0.0006}, {0.026, 0.0006}}]], 
       LineBox[NCache[{{Rational[13, 500], 0.11939999999999999`}, {
          Rational[13, 500], 0.1206}}, {{0.026, 0.11939999999999999`}, {0.026,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[7, 250], -0.0006}, {
          Rational[7, 250], 0.0006}}, {{0.028, -0.0006}, {0.028, 0.0006}}]], 
       LineBox[NCache[{{Rational[7, 250], 0.11939999999999999`}, {
          Rational[7, 250], 0.1206}}, {{0.028, 0.11939999999999999`}, {0.028, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[3, 100], -0.0006}, {
          Rational[3, 100], 0.0006}}, {{0.03, -0.0006}, {0.03, 0.0006}}]], 
       LineBox[NCache[{{Rational[3, 100], 0.11939999999999999`}, {
          Rational[3, 100], 0.1206}}, {{0.03, 0.11939999999999999`}, {0.03, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[4, 125], -0.0006}, {
          Rational[4, 125], 0.0006}}, {{0.032, -0.0006}, {0.032, 0.0006}}]], 
       LineBox[NCache[{{Rational[4, 125], 0.11939999999999999`}, {
          Rational[4, 125], 0.1206}}, {{0.032, 0.11939999999999999`}, {0.032, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[17, 500], -0.0006}, {
          Rational[17, 500], 0.0006}}, {{0.034, -0.0006}, {0.034, 0.0006}}]], 
       LineBox[
        NCache[{{Rational[17, 500], 0.11939999999999999`}, {
          Rational[17, 500], 0.1206}}, {{0.034, 0.11939999999999999`}, {0.034,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[9, 250], -0.0006}, {
          Rational[9, 250], 0.0006}}, {{0.036, -0.0006}, {0.036, 0.0006}}]], 
       LineBox[NCache[{{Rational[9, 250], 0.11939999999999999`}, {
          Rational[9, 250], 0.1206}}, {{0.036, 0.11939999999999999`}, {0.036, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[19, 500], -0.0006}, {
          Rational[19, 500], 0.0006}}, {{0.038, -0.0006}, {0.038, 0.0006}}]], 
       LineBox[NCache[{{Rational[19, 500], 0.11939999999999999`}, {
          Rational[19, 500], 0.1206}}, {{0.038, 0.11939999999999999`}, {0.038,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[1, 25], -0.0006}, {
          Rational[1, 25], 0.0006}}, {{0.04, -0.0006}, {0.04, 0.0006}}]], 
       LineBox[NCache[{{Rational[1, 25], 0.11939999999999999`}, {
          Rational[1, 25], 0.1206}}, {{0.04, 0.11939999999999999`}, {0.04, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[21, 500], -0.0006}, {
          Rational[21, 500], 0.0006}}, {{0.042, -0.0006}, {0.042, 0.0006}}]], 
       LineBox[NCache[{{Rational[21, 500], 0.11939999999999999`}, {
          Rational[21, 500], 0.1206}}, {{0.042, 0.11939999999999999`}, {0.042,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[11, 250], -0.0006}, {
          Rational[11, 250], 0.0006}}, {{0.044, -0.0006}, {0.044, 0.0006}}]], 
       LineBox[NCache[{{Rational[11, 250], 0.11939999999999999`}, {
          Rational[11, 250], 0.1206}}, {{0.044, 0.11939999999999999`}, {0.044,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[23, 500], -0.0006}, {
          Rational[23, 500], 0.0006}}, {{0.046, -0.0006}, {0.046, 0.0006}}]], 
       LineBox[NCache[{{Rational[23, 500], 0.11939999999999999`}, {
          Rational[23, 500], 0.1206}}, {{0.046, 0.11939999999999999`}, {0.046,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[6, 125], -0.0006}, {
          Rational[6, 125], 0.0006}}, {{0.048, -0.0006}, {0.048, 0.0006}}]], 
       LineBox[NCache[{{Rational[6, 125], 0.11939999999999999`}, {
          Rational[6, 125], 0.1206}}, {{0.048, 0.11939999999999999`}, {0.048, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[1, 20], -0.0006}, {
          Rational[1, 20], 0.0006}}, {{0.05, -0.0006}, {0.05, 0.0006}}]], 
       LineBox[NCache[{{Rational[1, 20], 0.11939999999999999`}, {
          Rational[1, 20], 0.1206}}, {{0.05, 0.11939999999999999`}, {0.05, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[13, 250], -0.0006}, {
          Rational[13, 250], 0.0006}}, {{0.052, -0.0006}, {0.052, 0.0006}}]], 
       LineBox[NCache[{{Rational[13, 250], 0.11939999999999999`}, {
          Rational[13, 250], 0.1206}}, {{0.052, 0.11939999999999999`}, {0.052,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[27, 500], -0.0006}, {
          Rational[27, 500], 0.0006}}, {{0.054, -0.0006}, {0.054, 0.0006}}]], 
       LineBox[NCache[{{Rational[27, 500], 0.11939999999999999`}, {
          Rational[27, 500], 0.1206}}, {{0.054, 0.11939999999999999`}, {0.054,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[7, 125], -0.0006}, {
          Rational[7, 125], 0.0006}}, {{0.056, -0.0006}, {0.056, 0.0006}}]], 
       LineBox[NCache[{{Rational[7, 125], 0.11939999999999999`}, {
          Rational[7, 125], 0.1206}}, {{0.056, 0.11939999999999999`}, {0.056, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[29, 500], -0.0006}, {
          Rational[29, 500], 0.0006}}, {{0.058, -0.0006}, {0.058, 0.0006}}]], 
       LineBox[NCache[{{Rational[29, 500], 0.11939999999999999`}, {
          Rational[29, 500], 0.1206}}, {{0.058, 0.11939999999999999`}, {0.058,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[3, 50], -0.0006}, {
          Rational[3, 50], 0.0006}}, {{0.06, -0.0006}, {0.06, 0.0006}}]], 
       LineBox[NCache[{{Rational[3, 50], 0.11939999999999999`}, {
          Rational[3, 50], 0.1206}}, {{0.06, 0.11939999999999999`}, {0.06, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[31, 500], -0.0006}, {
          Rational[31, 500], 0.0006}}, {{0.062, -0.0006}, {0.062, 0.0006}}]], 
       LineBox[NCache[{{Rational[31, 500], 0.11939999999999999`}, {
          Rational[31, 500], 0.1206}}, {{0.062, 0.11939999999999999`}, {0.062,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[8, 125], -0.0006}, {
          Rational[8, 125], 0.0006}}, {{0.064, -0.0006}, {0.064, 0.0006}}]], 
       LineBox[
        NCache[{{Rational[8, 125], 0.11939999999999999`}, {
          Rational[8, 125], 0.1206}}, {{0.064, 0.11939999999999999`}, {0.064, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[33, 500], -0.0006}, {
          Rational[33, 500], 0.0006}}, {{0.066, -0.0006}, {0.066, 0.0006}}]], 
       LineBox[NCache[{{Rational[33, 500], 0.11939999999999999`}, {
          Rational[33, 500], 0.1206}}, {{0.066, 0.11939999999999999`}, {0.066,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[17, 250], -0.0006}, {
          Rational[17, 250], 0.0006}}, {{0.068, -0.0006}, {0.068, 0.0006}}]], 
       LineBox[NCache[{{Rational[17, 250], 0.11939999999999999`}, {
          Rational[17, 250], 0.1206}}, {{0.068, 0.11939999999999999`}, {0.068,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[7, 100], -0.0006}, {
          Rational[7, 100], 0.0006}}, {{0.07, -0.0006}, {0.07, 0.0006}}]], 
       LineBox[NCache[{{Rational[7, 100], 0.11939999999999999`}, {
          Rational[7, 100], 0.1206}}, {{0.07, 0.11939999999999999`}, {0.07, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[9, 125], -0.0006}, {
          Rational[9, 125], 0.0006}}, {{0.072, -0.0006}, {0.072, 0.0006}}]], 
       LineBox[NCache[{{Rational[9, 125], 0.11939999999999999`}, {
          Rational[9, 125], 0.1206}}, {{0.072, 0.11939999999999999`}, {0.072, 
         0.1206}}]]}, {
       LineBox[NCache[{{Rational[37, 500], -0.0006}, {
          Rational[37, 500], 0.0006}}, {{0.074, -0.0006}, {0.074, 0.0006}}]], 
       LineBox[NCache[{{Rational[37, 500], 0.11939999999999999`}, {
          Rational[37, 500], 0.1206}}, {{0.074, 0.11939999999999999`}, {0.074,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[19, 250], -0.0006}, {
          Rational[19, 250], 0.0006}}, {{0.076, -0.0006}, {0.076, 0.0006}}]], 
       LineBox[NCache[{{Rational[19, 250], 0.11939999999999999`}, {
          Rational[19, 250], 0.1206}}, {{0.076, 0.11939999999999999`}, {0.076,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[39, 500], -0.0006}, {
          Rational[39, 500], 0.0006}}, {{0.078, -0.0006}, {0.078, 0.0006}}]], 
       LineBox[NCache[{{Rational[39, 500], 0.11939999999999999`}, {
          Rational[39, 500], 0.1206}}, {{0.078, 0.11939999999999999`}, {0.078,
          0.1206}}]]}, {
       LineBox[NCache[{{Rational[2, 25], -0.0006}, {
          Rational[2, 25], 0.0006}}, {{0.08, -0.0006}, {0.08, 0.0006}}]], 
       LineBox[NCache[{{Rational[2, 25], 0.11939999999999999`}, {
          Rational[2, 25], 0.1206}}, {{0.08, 0.11939999999999999`}, {0.08, 
         0.1206}}]]}}, {{LineBox[{{-0.0806, 0}, {-0.0794, 0}}], 
       LineBox[{{0.0794, 0}, {0.0806, 0}}]}, {
       LineBox[NCache[{{-0.0806, Rational[1, 500]}, {-0.0794, Rational[
           1, 500]}}, {{-0.0806, 0.002}, {-0.0794, 0.002}}]], 
       LineBox[NCache[{{0.0794, Rational[1, 500]}, {
          0.0806, Rational[1, 500]}}, {{0.0794, 0.002}, {0.0806, 0.002}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[1, 250]}, {-0.0794, Rational[
           1, 250]}}, {{-0.0806, 0.004}, {-0.0794, 0.004}}]], 
       LineBox[NCache[{{0.0794, Rational[1, 250]}, {
          0.0806, Rational[1, 250]}}, {{0.0794, 0.004}, {0.0806, 0.004}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[3, 500]}, {-0.0794, Rational[
           3, 500]}}, {{-0.0806, 0.006}, {-0.0794, 0.006}}]], 
       LineBox[NCache[{{0.0794, Rational[3, 500]}, {
          0.0806, Rational[3, 500]}}, {{0.0794, 0.006}, {0.0806, 0.006}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[1, 125]}, {-0.0794, Rational[
           1, 125]}}, {{-0.0806, 0.008}, {-0.0794, 0.008}}]], 
       LineBox[NCache[{{0.0794, Rational[1, 125]}, {
          0.0806, Rational[1, 125]}}, {{0.0794, 0.008}, {0.0806, 0.008}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[1, 100]}, {-0.0794, Rational[
           1, 100]}}, {{-0.0806, 0.01}, {-0.0794, 0.01}}]], 
       LineBox[NCache[{{0.0794, Rational[1, 100]}, {
          0.0806, Rational[1, 100]}}, {{0.0794, 0.01}, {0.0806, 0.01}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[3, 250]}, {-0.0794, Rational[
           3, 250]}}, {{-0.0806, 0.012}, {-0.0794, 0.012}}]], 
       LineBox[NCache[{{0.0794, Rational[3, 250]}, {
          0.0806, Rational[3, 250]}}, {{0.0794, 0.012}, {0.0806, 0.012}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[7, 500]}, {-0.0794, Rational[
           7, 500]}}, {{-0.0806, 0.014}, {-0.0794, 0.014}}]], 
       LineBox[
        NCache[{{0.0794, Rational[7, 500]}, {0.0806, Rational[7, 500]}}, {{
         0.0794, 0.014}, {0.0806, 0.014}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[2, 125]}, {-0.0794, Rational[
           2, 125]}}, {{-0.0806, 0.016}, {-0.0794, 0.016}}]], 
       LineBox[NCache[{{0.0794, Rational[2, 125]}, {
          0.0806, Rational[2, 125]}}, {{0.0794, 0.016}, {0.0806, 0.016}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[9, 500]}, {-0.0794, Rational[
           9, 500]}}, {{-0.0806, 0.018}, {-0.0794, 0.018}}]], 
       LineBox[NCache[{{0.0794, Rational[9, 500]}, {
          0.0806, Rational[9, 500]}}, {{0.0794, 0.018}, {0.0806, 0.018}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[1, 50]}, {-0.0794, Rational[
           1, 50]}}, {{-0.0806, 0.02}, {-0.0794, 0.02}}]], 
       LineBox[NCache[{{0.0794, Rational[1, 50]}, {
          0.0806, Rational[1, 50]}}, {{0.0794, 0.02}, {0.0806, 0.02}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[11, 500]}, {-0.0794, Rational[
           11, 500]}}, {{-0.0806, 0.022}, {-0.0794, 0.022}}]], 
       LineBox[NCache[{{0.0794, Rational[11, 500]}, {
          0.0806, Rational[11, 500]}}, {{0.0794, 0.022}, {0.0806, 
         0.022}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[3, 125]}, {-0.0794, Rational[
           3, 125]}}, {{-0.0806, 0.024}, {-0.0794, 0.024}}]], 
       LineBox[NCache[{{0.0794, Rational[3, 125]}, {
          0.0806, Rational[3, 125]}}, {{0.0794, 0.024}, {0.0806, 0.024}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[13, 500]}, {-0.0794, Rational[
           13, 500]}}, {{-0.0806, 0.026}, {-0.0794, 0.026}}]], 
       LineBox[NCache[{{0.0794, Rational[13, 500]}, {
          0.0806, Rational[13, 500]}}, {{0.0794, 0.026}, {0.0806, 
         0.026}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[7, 250]}, {-0.0794, Rational[
           7, 250]}}, {{-0.0806, 0.028}, {-0.0794, 0.028}}]], 
       LineBox[NCache[{{0.0794, Rational[7, 250]}, {
          0.0806, Rational[7, 250]}}, {{0.0794, 0.028}, {0.0806, 0.028}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[3, 100]}, {-0.0794, Rational[
           3, 100]}}, {{-0.0806, 0.03}, {-0.0794, 0.03}}]], 
       LineBox[NCache[{{0.0794, Rational[3, 100]}, {
          0.0806, Rational[3, 100]}}, {{0.0794, 0.03}, {0.0806, 0.03}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[4, 125]}, {-0.0794, Rational[
           4, 125]}}, {{-0.0806, 0.032}, {-0.0794, 0.032}}]], 
       LineBox[NCache[{{0.0794, Rational[4, 125]}, {
          0.0806, Rational[4, 125]}}, {{0.0794, 0.032}, {0.0806, 0.032}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[17, 500]}, {-0.0794, Rational[
           17, 500]}}, {{-0.0806, 0.034}, {-0.0794, 0.034}}]], 
       LineBox[NCache[{{0.0794, Rational[17, 500]}, {
          0.0806, Rational[17, 500]}}, {{0.0794, 0.034}, {0.0806, 
         0.034}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[9, 250]}, {-0.0794, Rational[
           9, 250]}}, {{-0.0806, 0.036}, {-0.0794, 0.036}}]], 
       LineBox[NCache[{{0.0794, Rational[9, 250]}, {
          0.0806, Rational[9, 250]}}, {{0.0794, 0.036}, {0.0806, 0.036}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[19, 500]}, {-0.0794, Rational[
           19, 500]}}, {{-0.0806, 0.038}, {-0.0794, 0.038}}]], 
       LineBox[NCache[{{0.0794, Rational[19, 500]}, {
          0.0806, Rational[19, 500]}}, {{0.0794, 0.038}, {0.0806, 
         0.038}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[1, 25]}, {-0.0794, Rational[
           1, 25]}}, {{-0.0806, 0.04}, {-0.0794, 0.04}}]], 
       LineBox[NCache[{{0.0794, Rational[1, 25]}, {
          0.0806, Rational[1, 25]}}, {{0.0794, 0.04}, {0.0806, 0.04}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[21, 500]}, {-0.0794, Rational[
           21, 500]}}, {{-0.0806, 0.042}, {-0.0794, 0.042}}]], 
       LineBox[NCache[{{0.0794, Rational[21, 500]}, {
          0.0806, Rational[21, 500]}}, {{0.0794, 0.042}, {0.0806, 
         0.042}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[11, 250]}, {-0.0794, Rational[
           11, 250]}}, {{-0.0806, 0.044}, {-0.0794, 0.044}}]], 
       LineBox[NCache[{{0.0794, Rational[11, 250]}, {
          0.0806, Rational[11, 250]}}, {{0.0794, 0.044}, {0.0806, 
         0.044}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[23, 500]}, {-0.0794, Rational[
           23, 500]}}, {{-0.0806, 0.046}, {-0.0794, 0.046}}]], 
       LineBox[NCache[{{0.0794, Rational[23, 500]}, {
          0.0806, Rational[23, 500]}}, {{0.0794, 0.046}, {0.0806, 
         0.046}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[6, 125]}, {-0.0794, Rational[
           6, 125]}}, {{-0.0806, 0.048}, {-0.0794, 0.048}}]], 
       LineBox[NCache[{{0.0794, Rational[6, 125]}, {
          0.0806, Rational[6, 125]}}, {{0.0794, 0.048}, {0.0806, 0.048}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[1, 20]}, {-0.0794, Rational[
           1, 20]}}, {{-0.0806, 0.05}, {-0.0794, 0.05}}]], 
       LineBox[NCache[{{0.0794, Rational[1, 20]}, {
          0.0806, Rational[1, 20]}}, {{0.0794, 0.05}, {0.0806, 0.05}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[13, 250]}, {-0.0794, Rational[
           13, 250]}}, {{-0.0806, 0.052}, {-0.0794, 0.052}}]], 
       LineBox[NCache[{{0.0794, Rational[13, 250]}, {
          0.0806, Rational[13, 250]}}, {{0.0794, 0.052}, {0.0806, 
         0.052}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[27, 500]}, {-0.0794, Rational[
           27, 500]}}, {{-0.0806, 0.054}, {-0.0794, 0.054}}]], 
       LineBox[NCache[{{0.0794, Rational[27, 500]}, {
          0.0806, Rational[27, 500]}}, {{0.0794, 0.054}, {0.0806, 
         0.054}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[7, 125]}, {-0.0794, Rational[
           7, 125]}}, {{-0.0806, 0.056}, {-0.0794, 0.056}}]], 
       LineBox[NCache[{{0.0794, Rational[7, 125]}, {
          0.0806, Rational[7, 125]}}, {{0.0794, 0.056}, {0.0806, 0.056}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[29, 500]}, {-0.0794, Rational[
           29, 500]}}, {{-0.0806, 0.058}, {-0.0794, 0.058}}]], 
       LineBox[NCache[{{0.0794, Rational[29, 500]}, {
          0.0806, Rational[29, 500]}}, {{0.0794, 0.058}, {0.0806, 
         0.058}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[3, 50]}, {-0.0794, Rational[
           3, 50]}}, {{-0.0806, 0.06}, {-0.0794, 0.06}}]], 
       LineBox[NCache[{{0.0794, Rational[3, 50]}, {
          0.0806, Rational[3, 50]}}, {{0.0794, 0.06}, {0.0806, 0.06}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[31, 500]}, {-0.0794, Rational[
           31, 500]}}, {{-0.0806, 0.062}, {-0.0794, 0.062}}]], 
       LineBox[NCache[{{0.0794, Rational[31, 500]}, {
          0.0806, Rational[31, 500]}}, {{0.0794, 0.062}, {0.0806, 
         0.062}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[8, 125]}, {-0.0794, Rational[
           8, 125]}}, {{-0.0806, 0.064}, {-0.0794, 0.064}}]], 
       LineBox[NCache[{{0.0794, Rational[8, 125]}, {
          0.0806, Rational[8, 125]}}, {{0.0794, 0.064}, {0.0806, 0.064}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[33, 500]}, {-0.0794, Rational[
           33, 500]}}, {{-0.0806, 0.066}, {-0.0794, 0.066}}]], 
       LineBox[NCache[{{0.0794, Rational[33, 500]}, {
          0.0806, Rational[33, 500]}}, {{0.0794, 0.066}, {0.0806, 
         0.066}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[17, 250]}, {-0.0794, Rational[
           17, 250]}}, {{-0.0806, 0.068}, {-0.0794, 0.068}}]], 
       LineBox[NCache[{{0.0794, Rational[17, 250]}, {
          0.0806, Rational[17, 250]}}, {{0.0794, 0.068}, {0.0806, 
         0.068}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[7, 100]}, {-0.0794, Rational[
           7, 100]}}, {{-0.0806, 0.07}, {-0.0794, 0.07}}]], 
       LineBox[NCache[{{0.0794, Rational[7, 100]}, {
          0.0806, Rational[7, 100]}}, {{0.0794, 0.07}, {0.0806, 0.07}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[9, 125]}, {-0.0794, Rational[
           9, 125]}}, {{-0.0806, 0.072}, {-0.0794, 0.072}}]], 
       LineBox[NCache[{{0.0794, Rational[9, 125]}, {
          0.0806, Rational[9, 125]}}, {{0.0794, 0.072}, {0.0806, 0.072}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[37, 500]}, {-0.0794, Rational[
           37, 500]}}, {{-0.0806, 0.074}, {-0.0794, 0.074}}]], 
       LineBox[NCache[{{0.0794, Rational[37, 500]}, {
          0.0806, Rational[37, 500]}}, {{0.0794, 0.074}, {0.0806, 
         0.074}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[19, 250]}, {-0.0794, Rational[
           19, 250]}}, {{-0.0806, 0.076}, {-0.0794, 0.076}}]], 
       LineBox[NCache[{{0.0794, Rational[19, 250]}, {
          0.0806, Rational[19, 250]}}, {{0.0794, 0.076}, {0.0806, 
         0.076}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[39, 500]}, {-0.0794, Rational[
           39, 500]}}, {{-0.0806, 0.078}, {-0.0794, 0.078}}]], 
       LineBox[NCache[{{0.0794, Rational[39, 500]}, {
          0.0806, Rational[39, 500]}}, {{0.0794, 0.078}, {0.0806, 
         0.078}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[2, 25]}, {-0.0794, Rational[
           2, 25]}}, {{-0.0806, 0.08}, {-0.0794, 0.08}}]], 
       LineBox[
        NCache[{{0.0794, Rational[2, 25]}, {0.0806, Rational[2, 25]}}, {{
         0.0794, 0.08}, {0.0806, 0.08}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[41, 500]}, {-0.0794, Rational[
           41, 500]}}, {{-0.0806, 0.082}, {-0.0794, 0.082}}]], 
       LineBox[NCache[{{0.0794, Rational[41, 500]}, {
          0.0806, Rational[41, 500]}}, {{0.0794, 0.082}, {0.0806, 
         0.082}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[21, 250]}, {-0.0794, Rational[
           21, 250]}}, {{-0.0806, 0.084}, {-0.0794, 0.084}}]], 
       LineBox[NCache[{{0.0794, Rational[21, 250]}, {
          0.0806, Rational[21, 250]}}, {{0.0794, 0.084}, {0.0806, 
         0.084}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[43, 500]}, {-0.0794, Rational[
           43, 500]}}, {{-0.0806, 0.086}, {-0.0794, 0.086}}]], 
       LineBox[NCache[{{0.0794, Rational[43, 500]}, {
          0.0806, Rational[43, 500]}}, {{0.0794, 0.086}, {0.0806, 
         0.086}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[11, 125]}, {-0.0794, Rational[
           11, 125]}}, {{-0.0806, 0.088}, {-0.0794, 0.088}}]], 
       LineBox[NCache[{{0.0794, Rational[11, 125]}, {
          0.0806, Rational[11, 125]}}, {{0.0794, 0.088}, {0.0806, 
         0.088}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[9, 100]}, {-0.0794, Rational[
           9, 100]}}, {{-0.0806, 0.09}, {-0.0794, 0.09}}]], 
       LineBox[NCache[{{0.0794, Rational[9, 100]}, {
          0.0806, Rational[9, 100]}}, {{0.0794, 0.09}, {0.0806, 0.09}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[23, 250]}, {-0.0794, Rational[
           23, 250]}}, {{-0.0806, 0.092}, {-0.0794, 0.092}}]], 
       LineBox[NCache[{{0.0794, Rational[23, 250]}, {
          0.0806, Rational[23, 250]}}, {{0.0794, 0.092}, {0.0806, 
         0.092}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[47, 500]}, {-0.0794, Rational[
           47, 500]}}, {{-0.0806, 0.094}, {-0.0794, 0.094}}]], 
       LineBox[NCache[{{0.0794, Rational[47, 500]}, {
          0.0806, Rational[47, 500]}}, {{0.0794, 0.094}, {0.0806, 
         0.094}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[12, 125]}, {-0.0794, Rational[
           12, 125]}}, {{-0.0806, 0.096}, {-0.0794, 0.096}}]], 
       LineBox[NCache[{{0.0794, Rational[12, 125]}, {
          0.0806, Rational[12, 125]}}, {{0.0794, 0.096}, {0.0806, 
         0.096}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[49, 500]}, {-0.0794, Rational[
           49, 500]}}, {{-0.0806, 0.098}, {-0.0794, 0.098}}]], 
       LineBox[NCache[{{0.0794, Rational[49, 500]}, {
          0.0806, Rational[49, 500]}}, {{0.0794, 0.098}, {0.0806, 
         0.098}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[1, 10]}, {-0.0794, Rational[
           1, 10]}}, {{-0.0806, 0.1}, {-0.0794, 0.1}}]], 
       LineBox[NCache[{{0.0794, Rational[1, 10]}, {
          0.0806, Rational[1, 10]}}, {{0.0794, 0.1}, {0.0806, 0.1}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[51, 500]}, {-0.0794, Rational[
           51, 500]}}, {{-0.0806, 0.102}, {-0.0794, 0.102}}]], 
       LineBox[NCache[{{0.0794, Rational[51, 500]}, {
          0.0806, Rational[51, 500]}}, {{0.0794, 0.102}, {0.0806, 
         0.102}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[13, 125]}, {-0.0794, Rational[
           13, 125]}}, {{-0.0806, 0.104}, {-0.0794, 0.104}}]], 
       LineBox[NCache[{{0.0794, Rational[13, 125]}, {
          0.0806, Rational[13, 125]}}, {{0.0794, 0.104}, {0.0806, 
         0.104}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[53, 500]}, {-0.0794, Rational[
           53, 500]}}, {{-0.0806, 0.106}, {-0.0794, 0.106}}]], 
       LineBox[NCache[{{0.0794, Rational[53, 500]}, {
          0.0806, Rational[53, 500]}}, {{0.0794, 0.106}, {0.0806, 
         0.106}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[27, 250]}, {-0.0794, Rational[
           27, 250]}}, {{-0.0806, 0.108}, {-0.0794, 0.108}}]], 
       LineBox[NCache[{{0.0794, Rational[27, 250]}, {
          0.0806, Rational[27, 250]}}, {{0.0794, 0.108}, {0.0806, 
         0.108}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[11, 100]}, {-0.0794, Rational[
           11, 100]}}, {{-0.0806, 0.11}, {-0.0794, 0.11}}]], 
       LineBox[NCache[{{0.0794, Rational[11, 100]}, {
          0.0806, Rational[11, 100]}}, {{0.0794, 0.11}, {0.0806, 0.11}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[14, 125]}, {-0.0794, Rational[
           14, 125]}}, {{-0.0806, 0.112}, {-0.0794, 0.112}}]], 
       LineBox[NCache[{{0.0794, Rational[14, 125]}, {
          0.0806, Rational[14, 125]}}, {{0.0794, 0.112}, {0.0806, 
         0.112}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[57, 500]}, {-0.0794, Rational[
           57, 500]}}, {{-0.0806, 0.114}, {-0.0794, 0.114}}]], 
       LineBox[NCache[{{0.0794, Rational[57, 500]}, {
          0.0806, Rational[57, 500]}}, {{0.0794, 0.114}, {0.0806, 
         0.114}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[29, 250]}, {-0.0794, Rational[
           29, 250]}}, {{-0.0806, 0.116}, {-0.0794, 0.116}}]], 
       LineBox[NCache[{{0.0794, Rational[29, 250]}, {
          0.0806, Rational[29, 250]}}, {{0.0794, 0.116}, {0.0806, 
         0.116}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[59, 500]}, {-0.0794, Rational[
           59, 500]}}, {{-0.0806, 0.118}, {-0.0794, 0.118}}]], 
       LineBox[NCache[{{0.0794, Rational[59, 500]}, {
          0.0806, Rational[59, 500]}}, {{0.0794, 0.118}, {0.0806, 
         0.118}}]]}, {
       LineBox[NCache[{{-0.0806, Rational[3, 25]}, {-0.0794, Rational[
           3, 25]}}, {{-0.0806, 0.12}, {-0.0794, 0.12}}]], 
       LineBox[NCache[{{0.0794, Rational[3, 25]}, {
          0.0806, Rational[3, 25]}}, {{0.0794, 0.12}, {0.0806, 0.12}}]]}}}, 
    {Thickness[0.005], 
     LineBox[NCache[{{Rational[-2, 25], 0}, {Rational[2, 25], 0}, {
        Rational[2, 25], Rational[3, 25]}, {
        Rational[-2, 25], Rational[3, 25]}, {
        Rational[-2, 25], 0}}, {{-0.08, 0}, {0.08, 0}, {0.08, 0.12}, {-0.08, 
        0.12}, {-0.08, 0}}]], {{
       LineBox[NCache[{{Rational[-2, 25], -0.001}, {
          Rational[-2, 25], 0.001}}, {{-0.08, -0.001}, {-0.08, 0.001}}]], 
       LineBox[NCache[{{Rational[-2, 25], 0.119}, {
          Rational[-2, 25], 0.121}}, {{-0.08, 0.119}, {-0.08, 0.121}}]], 
       InsetBox["\<\"-8\[ThinSpace]cm\"\>", 
        NCache[{Rational[-2, 25], -0.005}, {-0.08, -0.005}]], 
       InsetBox["\<\"-8\[ThinSpace]cm\"\>", 
        NCache[{Rational[-2, 25], 0.125}, {-0.08, 0.125}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{Rational[-7, 100], -0.001}, {
          Rational[-7, 100], 0.001}}, {{-0.07, -0.001}, {-0.07, 0.001}}]], 
       LineBox[NCache[{{Rational[-7, 100], 0.119}, {
          Rational[-7, 100], 0.121}}, {{-0.07, 0.119}, {-0.07, 0.121}}]], 
       InsetBox["\<\"-7\[ThinSpace]cm\"\>", 
        NCache[{Rational[-7, 100], -0.005}, {-0.07, -0.005}]], 
       InsetBox["\<\"-7\[ThinSpace]cm\"\>", 
        NCache[{Rational[-7, 100], 0.125}, {-0.07, 0.125}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{Rational[-3, 50], -0.001}, {
          Rational[-3, 50], 0.001}}, {{-0.06, -0.001}, {-0.06, 0.001}}]], 
       LineBox[NCache[{{Rational[-3, 50], 0.119}, {
          Rational[-3, 50], 0.121}}, {{-0.06, 0.119}, {-0.06, 0.121}}]], 
       InsetBox["\<\"-6\[ThinSpace]cm\"\>", 
        NCache[{Rational[-3, 50], -0.005}, {-0.06, -0.005}]], 
       InsetBox["\<\"-6\[ThinSpace]cm\"\>", 
        NCache[{Rational[-3, 50], 0.125}, {-0.06, 0.125}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{Rational[-1, 20], -0.001}, {
          Rational[-1, 20], 0.001}}, {{-0.05, -0.001}, {-0.05, 0.001}}]], 
       LineBox[NCache[{{Rational[-1, 20], 0.119}, {
          Rational[-1, 20], 0.121}}, {{-0.05, 0.119}, {-0.05, 0.121}}]], 
       InsetBox["\<\"-5\[ThinSpace]cm\"\>", 
        NCache[{Rational[-1, 20], -0.005}, {-0.05, -0.005}]], 
       InsetBox["\<\"-5\[ThinSpace]cm\"\>", 
        NCache[{Rational[-1, 20], 0.125}, {-0.05, 0.125}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{Rational[-1, 25], -0.001}, {
          Rational[-1, 25], 0.001}}, {{-0.04, -0.001}, {-0.04, 0.001}}]], 
       LineBox[NCache[{{Rational[-1, 25], 0.119}, {
          Rational[-1, 25], 0.121}}, {{-0.04, 0.119}, {-0.04, 0.121}}]], 
       InsetBox["\<\"-4\[ThinSpace]cm\"\>", 
        NCache[{Rational[-1, 25], -0.005}, {-0.04, -0.005}]], 
       InsetBox["\<\"-4\[ThinSpace]cm\"\>", 
        NCache[{Rational[-1, 25], 0.125}, {-0.04, 0.125}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{Rational[-3, 100], -0.001}, {
          Rational[-3, 100], 0.001}}, {{-0.03, -0.001}, {-0.03, 0.001}}]], 
       LineBox[NCache[{{Rational[-3, 100], 0.119}, {
          Rational[-3, 100], 0.121}}, {{-0.03, 0.119}, {-0.03, 0.121}}]], 
       InsetBox["\<\"-3\[ThinSpace]cm\"\>", 
        NCache[{Rational[-3, 100], -0.005}, {-0.03, -0.005}]], 
       InsetBox["\<\"-3\[ThinSpace]cm\"\>", 
        NCache[{Rational[-3, 100], 0.125}, {-0.03, 0.125}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{Rational[-1, 50], -0.001}, {
          Rational[-1, 50], 0.001}}, {{-0.02, -0.001}, {-0.02, 0.001}}]], 
       LineBox[NCache[{{Rational[-1, 50], 0.119}, {
          Rational[-1, 50], 0.121}}, {{-0.02, 0.119}, {-0.02, 0.121}}]], 
       InsetBox["\<\"-2\[ThinSpace]cm\"\>", 
        NCache[{Rational[-1, 50], -0.005}, {-0.02, -0.005}]], 
       InsetBox["\<\"-2\[ThinSpace]cm\"\>", 
        NCache[{Rational[-1, 50], 0.125}, {-0.02, 0.125}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{Rational[-1, 100], -0.001}, {
          Rational[-1, 100], 0.001}}, {{-0.01, -0.001}, {-0.01, 0.001}}]], 
       LineBox[NCache[{{Rational[-1, 100], 0.119}, {
          Rational[-1, 100], 0.121}}, {{-0.01, 0.119}, {-0.01, 0.121}}]], 
       InsetBox["\<\"-1\[ThinSpace]cm\"\>", 
        NCache[{Rational[-1, 100], -0.005}, {-0.01, -0.005}]], 
       InsetBox["\<\"-1\[ThinSpace]cm\"\>", 
        NCache[{Rational[-1, 100], 0.125}, {-0.01, 0.125}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {LineBox[{{0, -0.001}, {0, 0.001}}], 
       LineBox[{{0, 0.119}, {0, 0.121}}], 
       InsetBox["\<\"0\[ThinSpace]cm\"\>", {0, -0.005}], 
       InsetBox["\<\"0\[ThinSpace]cm\"\>", {0, 0.125}], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{Rational[1, 100], -0.001}, {
          Rational[1, 100], 0.001}}, {{0.01, -0.001}, {0.01, 0.001}}]], 
       LineBox[NCache[{{Rational[1, 100], 0.119}, {
          Rational[1, 100], 0.121}}, {{0.01, 0.119}, {0.01, 0.121}}]], 
       InsetBox["\<\"1\[ThinSpace]cm\"\>", 
        NCache[{Rational[1, 100], -0.005}, {0.01, -0.005}]], 
       InsetBox["\<\"1\[ThinSpace]cm\"\>", 
        NCache[{Rational[1, 100], 0.125}, {0.01, 0.125}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[
        NCache[{{Rational[1, 50], -0.001}, {Rational[1, 50], 0.001}}, {{
         0.02, -0.001}, {0.02, 0.001}}]], 
       LineBox[NCache[{{Rational[1, 50], 0.119}, {Rational[1, 50], 0.121}}, {{
         0.02, 0.119}, {0.02, 0.121}}]], 
       InsetBox["\<\"2\[ThinSpace]cm\"\>", 
        NCache[{Rational[1, 50], -0.005}, {0.02, -0.005}]], 
       InsetBox["\<\"2\[ThinSpace]cm\"\>", 
        NCache[{Rational[1, 50], 0.125}, {0.02, 0.125}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{Rational[3, 100], -0.001}, {
          Rational[3, 100], 0.001}}, {{0.03, -0.001}, {0.03, 0.001}}]], 
       LineBox[NCache[{{Rational[3, 100], 0.119}, {
          Rational[3, 100], 0.121}}, {{0.03, 0.119}, {0.03, 0.121}}]], 
       InsetBox["\<\"3\[ThinSpace]cm\"\>", 
        NCache[{Rational[3, 100], -0.005}, {0.03, -0.005}]], 
       InsetBox["\<\"3\[ThinSpace]cm\"\>", 
        NCache[{Rational[3, 100], 0.125}, {0.03, 0.125}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{Rational[1, 25], -0.001}, {
          Rational[1, 25], 0.001}}, {{0.04, -0.001}, {0.04, 0.001}}]], 
       LineBox[NCache[{{Rational[1, 25], 0.119}, {Rational[1, 25], 0.121}}, {{
         0.04, 0.119}, {0.04, 0.121}}]], 
       InsetBox["\<\"4\[ThinSpace]cm\"\>", 
        NCache[{Rational[1, 25], -0.005}, {0.04, -0.005}]], 
       InsetBox["\<\"4\[ThinSpace]cm\"\>", 
        NCache[{Rational[1, 25], 0.125}, {0.04, 0.125}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{Rational[1, 20], -0.001}, {
          Rational[1, 20], 0.001}}, {{0.05, -0.001}, {0.05, 0.001}}]], 
       LineBox[NCache[{{Rational[1, 20], 0.119}, {Rational[1, 20], 0.121}}, {{
         0.05, 0.119}, {0.05, 0.121}}]], 
       InsetBox["\<\"5\[ThinSpace]cm\"\>", 
        NCache[{Rational[1, 20], -0.005}, {0.05, -0.005}]], 
       InsetBox["\<\"5\[ThinSpace]cm\"\>", 
        NCache[{Rational[1, 20], 0.125}, {0.05, 0.125}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{Rational[3, 50], -0.001}, {
          Rational[3, 50], 0.001}}, {{0.06, -0.001}, {0.06, 0.001}}]], 
       LineBox[
        NCache[{{Rational[3, 50], 0.119}, {Rational[3, 50], 0.121}}, {{0.06, 
         0.119}, {0.06, 0.121}}]], 
       InsetBox["\<\"6\[ThinSpace]cm\"\>", 
        NCache[{Rational[3, 50], -0.005}, {0.06, -0.005}]], 
       InsetBox["\<\"6\[ThinSpace]cm\"\>", 
        NCache[{Rational[3, 50], 0.125}, {0.06, 0.125}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{Rational[7, 100], -0.001}, {
          Rational[7, 100], 0.001}}, {{0.07, -0.001}, {0.07, 0.001}}]], 
       LineBox[NCache[{{Rational[7, 100], 0.119}, {
          Rational[7, 100], 0.121}}, {{0.07, 0.119}, {0.07, 0.121}}]], 
       InsetBox["\<\"7\[ThinSpace]cm\"\>", 
        NCache[{Rational[7, 100], -0.005}, {0.07, -0.005}]], 
       InsetBox["\<\"7\[ThinSpace]cm\"\>", 
        NCache[{Rational[7, 100], 0.125}, {0.07, 0.125}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{Rational[2, 25], -0.001}, {
          Rational[2, 25], 0.001}}, {{0.08, -0.001}, {0.08, 0.001}}]], 
       LineBox[NCache[{{Rational[2, 25], 0.119}, {Rational[2, 25], 0.121}}, {{
         0.08, 0.119}, {0.08, 0.121}}]], 
       InsetBox["\<\"8\[ThinSpace]cm\"\>", 
        NCache[{Rational[2, 25], -0.005}, {0.08, -0.005}]], 
       InsetBox["\<\"8\[ThinSpace]cm\"\>", 
        NCache[{Rational[2, 25], 0.125}, {0.08, 0.125}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}}, {{
       LineBox[{{-0.081, 0}, {-0.079, 0}}], LineBox[{{0.079, 0}, {0.081, 0}}],
        InsetBox["\<\"0\[ThinSpace]cm\"\>", {-0.08700000000000001, 0}], 
       InsetBox["\<\"0\[ThinSpace]cm\"\>", {0.08700000000000001, 0}], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{-0.081, Rational[1, 100]}, {-0.079, Rational[
           1, 100]}}, {{-0.081, 0.01}, {-0.079, 0.01}}]], 
       LineBox[NCache[{{0.079, Rational[1, 100]}, {
          0.081, Rational[1, 100]}}, {{0.079, 0.01}, {0.081, 0.01}}]], 
       InsetBox["\<\"1\[ThinSpace]cm\"\>", 
        NCache[{-0.08700000000000001, Rational[
          1, 100]}, {-0.08700000000000001, 0.01}]], 
       InsetBox["\<\"1\[ThinSpace]cm\"\>", 
        NCache[{0.08700000000000001, Rational[1, 100]}, {0.08700000000000001, 
         0.01}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{-0.081, Rational[1, 50]}, {-0.079, Rational[
           1, 50]}}, {{-0.081, 0.02}, {-0.079, 0.02}}]], 
       LineBox[NCache[{{0.079, Rational[1, 50]}, {0.081, Rational[1, 50]}}, {{
         0.079, 0.02}, {0.081, 0.02}}]], 
       InsetBox["\<\"2\[ThinSpace]cm\"\>", 
        NCache[{-0.08700000000000001, Rational[1, 50]}, {-0.08700000000000001,
          0.02}]], 
       InsetBox["\<\"2\[ThinSpace]cm\"\>", 
        NCache[{0.08700000000000001, Rational[1, 50]}, {0.08700000000000001, 
         0.02}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{-0.081, Rational[3, 100]}, {-0.079, Rational[
           3, 100]}}, {{-0.081, 0.03}, {-0.079, 0.03}}]], 
       LineBox[NCache[{{0.079, Rational[3, 100]}, {
          0.081, Rational[3, 100]}}, {{0.079, 0.03}, {0.081, 0.03}}]], 
       InsetBox["\<\"3\[ThinSpace]cm\"\>", 
        NCache[{-0.08700000000000001, Rational[
          3, 100]}, {-0.08700000000000001, 0.03}]], 
       InsetBox["\<\"3\[ThinSpace]cm\"\>", 
        NCache[{0.08700000000000001, Rational[3, 100]}, {0.08700000000000001, 
         0.03}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{-0.081, Rational[1, 25]}, {-0.079, Rational[
           1, 25]}}, {{-0.081, 0.04}, {-0.079, 0.04}}]], 
       LineBox[NCache[{{0.079, Rational[1, 25]}, {0.081, Rational[1, 25]}}, {{
         0.079, 0.04}, {0.081, 0.04}}]], 
       InsetBox["\<\"4\[ThinSpace]cm\"\>", 
        NCache[{-0.08700000000000001, Rational[1, 25]}, {-0.08700000000000001,
          0.04}]], 
       InsetBox["\<\"4\[ThinSpace]cm\"\>", 
        NCache[{0.08700000000000001, Rational[1, 25]}, {0.08700000000000001, 
         0.04}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{-0.081, Rational[1, 20]}, {-0.079, Rational[
           1, 20]}}, {{-0.081, 0.05}, {-0.079, 0.05}}]], 
       LineBox[NCache[{{0.079, Rational[1, 20]}, {0.081, Rational[1, 20]}}, {{
         0.079, 0.05}, {0.081, 0.05}}]], 
       InsetBox["\<\"5\[ThinSpace]cm\"\>", 
        NCache[{-0.08700000000000001, Rational[1, 20]}, {-0.08700000000000001,
          0.05}]], 
       InsetBox["\<\"5\[ThinSpace]cm\"\>", 
        NCache[{0.08700000000000001, Rational[1, 20]}, {0.08700000000000001, 
         0.05}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{-0.081, Rational[3, 50]}, {-0.079, Rational[
           3, 50]}}, {{-0.081, 0.06}, {-0.079, 0.06}}]], 
       LineBox[NCache[{{0.079, Rational[3, 50]}, {0.081, Rational[3, 50]}}, {{
         0.079, 0.06}, {0.081, 0.06}}]], 
       InsetBox["\<\"6\[ThinSpace]cm\"\>", 
        NCache[{-0.08700000000000001, Rational[3, 50]}, {-0.08700000000000001,
          0.06}]], 
       InsetBox["\<\"6\[ThinSpace]cm\"\>", 
        NCache[{0.08700000000000001, Rational[3, 50]}, {0.08700000000000001, 
         0.06}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{-0.081, Rational[7, 100]}, {-0.079, Rational[
           7, 100]}}, {{-0.081, 0.07}, {-0.079, 0.07}}]], 
       LineBox[NCache[{{0.079, Rational[7, 100]}, {
          0.081, Rational[7, 100]}}, {{0.079, 0.07}, {0.081, 0.07}}]], 
       InsetBox["\<\"7\[ThinSpace]cm\"\>", 
        NCache[{-0.08700000000000001, Rational[
          7, 100]}, {-0.08700000000000001, 0.07}]], 
       InsetBox["\<\"7\[ThinSpace]cm\"\>", 
        NCache[{0.08700000000000001, Rational[7, 100]}, {0.08700000000000001, 
         0.07}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{-0.081, Rational[2, 25]}, {-0.079, Rational[
           2, 25]}}, {{-0.081, 0.08}, {-0.079, 0.08}}]], 
       LineBox[NCache[{{0.079, Rational[2, 25]}, {0.081, Rational[2, 25]}}, {{
         0.079, 0.08}, {0.081, 0.08}}]], 
       InsetBox["\<\"8\[ThinSpace]cm\"\>", 
        NCache[{-0.08700000000000001, Rational[2, 25]}, {-0.08700000000000001,
          0.08}]], 
       InsetBox["\<\"8\[ThinSpace]cm\"\>", 
        NCache[{0.08700000000000001, Rational[2, 25]}, {0.08700000000000001, 
         0.08}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{-0.081, Rational[9, 100]}, {-0.079, Rational[
           9, 100]}}, {{-0.081, 0.09}, {-0.079, 0.09}}]], 
       LineBox[NCache[{{0.079, Rational[9, 100]}, {
          0.081, Rational[9, 100]}}, {{0.079, 0.09}, {0.081, 0.09}}]], 
       InsetBox["\<\"9\[ThinSpace]cm\"\>", 
        NCache[{-0.08700000000000001, Rational[
          9, 100]}, {-0.08700000000000001, 0.09}]], 
       InsetBox["\<\"9\[ThinSpace]cm\"\>", 
        NCache[{0.08700000000000001, Rational[9, 100]}, {0.08700000000000001, 
         0.09}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{-0.081, Rational[1, 10]}, {-0.079, Rational[
           1, 10]}}, {{-0.081, 0.1}, {-0.079, 0.1}}]], 
       LineBox[NCache[{{0.079, Rational[1, 10]}, {0.081, Rational[1, 10]}}, {{
         0.079, 0.1}, {0.081, 0.1}}]], 
       InsetBox["\<\"10\[ThinSpace]cm\"\>", 
        NCache[{-0.08700000000000001, Rational[1, 10]}, {-0.08700000000000001,
          0.1}]], 
       InsetBox["\<\"10\[ThinSpace]cm\"\>", 
        NCache[{0.08700000000000001, Rational[1, 10]}, {0.08700000000000001, 
         0.1}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{-0.081, Rational[11, 100]}, {-0.079, Rational[
           11, 100]}}, {{-0.081, 0.11}, {-0.079, 0.11}}]], 
       LineBox[NCache[{{0.079, Rational[11, 100]}, {
          0.081, Rational[11, 100]}}, {{0.079, 0.11}, {0.081, 0.11}}]], 
       InsetBox["\<\"11\[ThinSpace]cm\"\>", 
        NCache[{-0.08700000000000001, Rational[
          11, 100]}, {-0.08700000000000001, 0.11}]], 
       InsetBox["\<\"11\[ThinSpace]cm\"\>", 
        NCache[{0.08700000000000001, Rational[11, 100]}, {0.08700000000000001,
          0.11}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}, {
       LineBox[NCache[{{-0.081, Rational[3, 25]}, {-0.079, Rational[
           3, 25]}}, {{-0.081, 0.12}, {-0.079, 0.12}}]], 
       LineBox[NCache[{{0.079, Rational[3, 25]}, {0.081, Rational[3, 25]}}, {{
         0.079, 0.12}, {0.081, 0.12}}]], 
       InsetBox["\<\"12\[ThinSpace]cm\"\>", 
        NCache[{-0.08700000000000001, Rational[3, 25]}, {-0.08700000000000001,
          0.12}]], 
       InsetBox["\<\"12\[ThinSpace]cm\"\>", 
        NCache[{0.08700000000000001, Rational[3, 25]}, {0.08700000000000001, 
         0.12}]], 
       StyleBox[{},
        BaseStyle -> {FontSize -> 12}]}}}}, 
   {RGBColor[0, 0, 0.4], Thickness[0.006], 
    LineBox[{{0, 0}, {-0.00001831159575129636, 
      0.0004995525135494145}, {-0.0000731480821293974, 
      0.000996423712245777}, {-0.00016421512804257594`, 
      0.001487946673541241}, {-0.00029102393732048274`, 
      0.0019714831811818673`}, {-0.0004528938722934905, 
      0.002444437885655105}, {-0.0006489561070683329, 
      0.002904272234553039}, {-0.0008781582908913402, 
      0.0033485180980811265`}, {-0.0011392701965689909`, 
      0.0037747910165784157`}, {-0.0014308903236282774`, 
      0.0041808029989440785`}, {-0.001751453420774875, 
      0.004564374803275562}, {-0.0020992388872728388`, 
      0.004923447633802855}, {-0.0024723800081519953`, 
      0.005256094191336364}, {-0.0028688739736736925`, 
      0.005560529017915856}, {-0.0032865926292761014`, 
      0.005835118080136262}, {-0.003723293898299475, 
      0.0060783875397124685`}, {-0.004176633816180666, 
      0.006289031664207691}, {-0.004644179111524177, 
      0.006465919835465105}, {-0.005123420266521704, 
      0.006608102618125448}, {-0.005611784986619253, 
      0.006714816855658233}, {-0.0061066520071343085`, 
      0.0067854897665539406`}, {-0.006605365162716962, 
      0.006819742018691169}, {-0.007105247644138131, 
      0.006817389765377248}, {-0.007603616365882516, 
      0.0067784456321340215`}, {-0.008097796367429199, 
      0.006703118648932264}, {-0.008585135170922001, 
      0.006591813128238452}, {-0.009063017018165755, 
      0.006445126494895897}, {-0.009528876910532377, 
      0.0062638460794881825`}, {-0.009980214376418495, 
      0.006048944892396319}, {-0.010414606892358706`, 
      0.00580157640123205}, {-0.01082972288575757, 
      0.005523068339679092}, {-0.011223334249449018`, 
      0.005214915580972913}, {-0.011593328300912139`, 
      0.004878772114270173}, {-0.011937719121953025`, 
      0.004516442166974102}, {-0.012254658217987723`, 
      0.004129870520666129}, {-0.012542444439713283`, 
      0.003721132072622353}, {-0.012799533113912989`, 
      0.0032924206989426888`}, {-0.013024544334386799`, 
      0.0028460374790691}, {-0.01321627036850596, 
      0.002384378344897011}, {-0.013373682139637622`, 
      0.0019099212207724626`}, {-0.013495934750645434`, 
      0.0014252127234002008`}, {-0.013582372018819127`, 
      0.0009328544930500505}, {-0.013632529997892131`, 
      0.0004354892294278974}, {-0.013646139468243015`, \
-0.00006421349283720023}}]}},
  ImageSize->{600, 450},
  PlotRange->All]], "Output",
 CellChangeTimes->{3.400795261566826*^9, 3.400795324090159*^9, 
  3.400795362453773*^9, 3.40093609268106*^9}]
}, Open  ]],

Cell[TextData[{
 "Calculate the magnetic field needed to create the same trajectory with an \
alpha particle with the same speed.  Modify the preceding code to represent \
an alpha  particle.  (Note that an alpha particle is the nucleus of Helium-4, \
so it has two protons and two neutrons.)  To repeat the input code from above \
click the mouse cursor below this to give a horizontal line and then type ",
 StyleBox["Ctrl+L",
  FontSlant->"Italic"],
 "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["(B) Electric Field",
 FontSize->16]], "Section"],

Cell[TextData[{
 "Consider an electron shot with a speed of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"6", "\[Times]", 
    SuperscriptBox["10", "5"], " ", 
    StyleBox[
     RowBox[{"m", "/", "s"}],
     FontSlant->"Plain"]}], TraditionalForm]]],
 " into just an electric field (zero magnetic field).  Calculate the value of \
 ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["E", "x"], TraditionalForm]]],
 " needed to deflect the electron to the right by 3 cm.  Modify the code to \
reflect this and evaluate it.  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["E", "x"], TraditionalForm]]],
 " is to the right by convention; a negative value is to the left."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["(C) Electric and Magnetic Fields",
 FontSize->16]], "Section"],

Cell[TextData[{
 "Below, display the case an electron shot with a speed of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"6", "\[Times]", 
    SuperscriptBox["10", "5"], " ", 
    StyleBox[
     RowBox[{"m", "/", "s"}],
     FontSlant->"Plain"]}], TraditionalForm]]],
 " into an electric field of  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["E", "x"],
     TraditionalForm], " ", "=", " ", 
    RowBox[{"30", " ", 
     RowBox[{
      StyleBox["V",
       FontSlant->"Plain"], "/", 
      StyleBox["m",
       FontSlant->"Plain"]}]}]}], TraditionalForm]]],
 " and a magnetic field of.",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["B", "z"],
     TraditionalForm], "=", 
    RowBox[{
     RowBox[{"-", " ", "0.0002"}], " ", 
     StyleBox["T",
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "A ",
 StyleBox["velocity selector",
  FontSlant->"Italic"],
 " is a configuration with both fields to create no net force on any particle \
with one particular velocity."
}], "Text"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["F", "\[RightVector]"], "=", 
    RowBox[{
     RowBox[{"Q", " ", 
      RowBox[{"(", 
       RowBox[{
        OverscriptBox["E", "\[RightVector]"], "+", 
        RowBox[{
         OverscriptBox["v", "\[RightVector]"], "\[Cross]", 
         OverscriptBox["B", "\[RightVector]"]}]}], ")"}]}], "=", 
     OverscriptBox["0", "\[RightVector]"]}]}], TraditionalForm]]],
 "  \[DoubleLongRightArrow]  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["E", "\[RightVector]"], "=", 
    RowBox[{"-", 
     RowBox[{
      OverscriptBox["v", "\[RightVector]"], "\[Cross]", 
      OverscriptBox["B", "\[RightVector]"]}]}]}], TraditionalForm]]],
 "  \[DoubleLongRightArrow]  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", "=", 
    RowBox[{"v", " ", "B"}]}], TraditionalForm]]]
}], "Text",
 TextAlignment->Center,
 TextJustification->0],

Cell[TextData[{
 "Calculate the value of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["E", "x"], TraditionalForm]]],
 " needed to achieve this for any particle with speed ",
 Cell[BoxData[
  FormBox[
   RowBox[{"6", "\[Times]", 
    SuperscriptBox["10", "5"], " ", 
    StyleBox[
     RowBox[{"m", "/", "s"}],
     FontSlant->"Plain"]}], TraditionalForm]]],
 " in a magneitc field of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["B", "z"], "=", 
    RowBox[{".0005", " ", 
     StyleBox["T",
      FontSlant->"Plain"]}]}], TraditionalForm]]],
 ".  Make sure you get the sign of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["E", "x"], TraditionalForm]]],
 " correct to cancel the magnetic force with the electric force.  Demonstrate \
this for the three separate cases of an electron, proton and an alpha \
particle."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["(D) Three Dimensional ",
 FontSize->16]], "Section"],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Code for 3D Case"
}], "Subsection"],

Cell[TextData[{
 "This the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " code for the 3D calculation and to display the trajectory graphically.  \
The function ",
 StyleBox["CalculateAndDraw3d",
  FontWeight->"Bold"],
 " does all this.  The function ",
 StyleBox["Acceleration3D",
  FontWeight->"Bold"],
 " represents the 3D acceleration expression needed."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Acceleration3D", "[", 
   RowBox[{"r_", ",", "v_"}], "]"}], ":=", 
  RowBox[{
   FractionBox["Q", "m"], 
   RowBox[{"v", "\[Cross]", 
    RowBox[{"BField", "[", "r", "]"}]}]}]}], "\n", 
 RowBox[{
  RowBox[{"CalculateAndDraw3D", ":=", "\[IndentingNewLine]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"InRegionQ", "=", "InRegionQ3D"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Acceleration", "=", "Acceleration3D"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Show", "[", 
      RowBox[{"Graphics3D", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Thickness", "[", ".01", "]"}], ",", 
          RowBox[{"RGBColor", "[", 
           RowBox[{"0", ",", "0", ",", ".4"}], "]"}], ",", 
          RowBox[{"Line", "[", 
           RowBox[{"DataPointPosition", "/@", 
            RowBox[{"IterateData", "[", 
             RowBox[{"{", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"r0", ",", "v0"}], "}"}], ",", "0"}], "}"}], "}"}], 
             "]"}]}], "]"}]}], "}"}], ",", 
        RowBox[{"Axes", "\[Rule]", "True"}], ",", 
        RowBox[{"PlotRange", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", ".5"}], ",", ".5"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", ".5"}], ",", ".5"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", ".5"}], ",", ".5"}], "}"}]}], "}"}]}], ",", 
        RowBox[{"AxesLabel", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"\"\<x\>\"", ",", "\"\<y\>\"", ",", "\"\<z\>\""}], 
          "}"}]}]}], "\[IndentingNewLine]", "]"}], "]"}]}], ")"}]}], 
  "\[IndentingNewLine]"}], "\n", 
 RowBox[{
  RowBox[{"InRegionQ3D", "[", 
   RowBox[{"{", 
    RowBox[{"x_", ",", "y_", ",", "z_"}], "}"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", ".5"}], "\[LessEqual]", "x", "\[LessEqual]", ".5"}], ")"}], 
   "\[And]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", ".5"}], "\[LessEqual]", "y", "\[LessEqual]", ".5"}], ")"}], 
   "\[And]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", ".5"}], "\[LessEqual]", "z", "\[LessEqual]", ".5"}], 
    ")"}]}]}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"BField", "[", "r_", "]"}], ":=", "B"}], ";", 
  RowBox[{"m", "=", "mElectron"}], ";", 
  RowBox[{"Q", "=", 
   RowBox[{"-", "e"}]}], ";", 
  RowBox[{"r0", "=", 
   RowBox[{"{", 
    RowBox[{".25", ",", "0", ",", ".5"}], "}"}]}], ";", 
  RowBox[{"v0", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"0", ",", "1", ",", 
      RowBox[{"-", ".25"}]}], "}"}], " ", "6", "\[Times]", 
    SuperscriptBox["10", "5"]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B", "=", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", ".00002"}], "}"}]}], ";", 
  RowBox[{"\[CapitalDelta]s", "=", ".005"}], ";", 
  RowBox[{"iMax", "=", "2000"}], ";", "CalculateAndDraw3D"}]}], "Input",
 CellChangeTimes->{3.400936077298485*^9}],

Cell[BoxData[
 Graphics3DBox[
  {RGBColor[0, 0, 0.4], Thickness[0.01], 
   Line3DBox[{{0.25, 0, 0.5}, {0.24993103601276412`, 0.004850058758610588, 
     0.49878732187481833`}, {0.24972419981519844`, 0.00969619585746336, 
     0.4975746437496325}, {0.24937965865088224`, 0.014534492807794341`, 
     0.49636196562444246`}, {0.2488976911088549, 0.019361037460225557`, 
     0.49514928749924825`}, {0.24827868689835444`, 0.024171927168050167`, 
     0.4939366093740498}, {0.24752314653370644`, 0.028963271942833907`, 
     0.4927239312488472}, {0.24663168092961826`, 0.03373119759978132, 
     0.4915112531236404}, {0.2456050109072058, 0.038471848890323485`, 
     0.49029857499842944`}, {0.24444396661115195`, 0.04318139261939429, 
     0.48908589687321424`}, {0.24314948683846843`, 0.047856020744874696`, 
     0.4878732187479949}, {0.24172261827940345`, 0.052491953456698805`, 
     0.4866605406227713}, {0.240164514671109, 0.057085442233132104`, 
     0.48544786249754357`}, {0.23847643586475237`, 0.06163277287175058, 
     0.4842351843723116}, {0.2366597468068258, 0.06613026849266988, 
     0.4830225062470755}, {0.2347159164354783, 0.07057429251159635, 
     0.48180982812183515`}, {0.23264651649276183`, 0.07496125158029568, 
     0.48059714999659064`}, {0.23045322025375248`, 0.07928759849210192, 
     0.4793844718713419}, {0.22813780117357382`, 0.08354983505011726, 
     0.47817179374608904`}, {0.225702131453417, 0.08774451489578354, 
     0.47695911562083193`}, {0.22314818052671648`, 0.09186824629553826, 
     0.47574643749557066`}, {0.22047801346670592`, 0.09591769488330201, 
     0.4745337593703052}, {0.21769378931664157`, 0.09988958635657966, 
     0.4733210812450355}, {0.2147977593440435, 0.10378070912399531`, 
     0.47210840311976165`}, {0.2117922652203659, 0.1075879169021204, 
     0.4708957249944836}, {0.20867973712756893`, 0.11130813125949501`, 
     0.46968304686920137`}, {0.20546269179312246`, 0.11493834410578553`, 
     0.46847036874391496`}, {0.20214373045503106`, 0.1184756201240658, 
     0.4672576906186243}, {0.19872553675852528`, 0.12191709914425528`, 
     0.4660450124933295}, {0.19521087458612035`, 0.12525999845579483`, 
     0.4648323343680305}, {0.19160258582279638`, 0.1285016150576905, 
     0.46361965624272733`}, {0.1879035880581075, 0.1316393278441055, 
     0.46240697811741993`}, {0.18411687222707782`, 0.13467059972373366`, 
     0.46119429999210837`}, {0.18024550019179153`, 0.13759297967124023`, 
     0.4599816218667926}, {0.17629260226563312`, 0.14040410470911166`, 
     0.45876894374147265`}, {0.17226137468217892`, 0.14310170181831144`, 
     0.4575562656161485}, {0.1681550770107872, 0.14568358977619758`, 
     0.45634358749082016`}, {0.16397702952097593`, 0.14814768092021527`, 
     0.4551309093654876}, {0.15973061049771997`, 0.15049198283593868`, 
     0.4539182312401509}, {0.15541925350983785`, 0.1527145999680971, 
     0.45270555311481}, {0.15104644463367722`, 0.15481373515328284`, 
     0.4514928749894649}, {0.1466157196343438, 0.15678769107310117`, 
     0.4502801968641156}, {0.14213066110675307`, 0.15863487162658785`, 
     0.4490675187387621}, {0.13759489557881616`, 0.1603537832207842, 
     0.4478548406134044}, {0.1330120905791025, 0.16194303597842621`, 
     0.44664216248804256`}, {0.12838595167135017`, 0.16340134486177135`, 
     0.4454294843626765}, {0.1237202194582218, 0.1647275307116541, 
     0.44421680623730625`}, {0.11901866655672859`, 0.16592052120093034`, 
     0.4430041281119318}, {0.11428509454776829`, 0.16697935170153946`, 
     0.4417914499865532}, {0.10952333090224352`, 0.16790316606448316`, 
     0.4405787718611704}, {0.10473722588624594`, 0.16869121731209025`, 
     0.4393660937357834}, {0.09993064944780873, 0.16934286824200775`, 
     0.4381534156103922}, {0.09510748808774461, 0.16985759194242991`, 
     0.43694073748499684`}, {0.09027164171709962, 0.1702349722181484, 
     0.4357280593595973}, {0.08542702050376368, 0.17047470392707945`, 
     0.43451538123419353`}, {0.08057754171078767, 0.1705765932269955, 
     0.4333027031087856}, {0.07572712652896349, 0.17054055773226223`, 
     0.43209002498337346`}, {0.0708796969062282, 0.17036662658045382`, 
     0.43087734685795714`}, {0.06603917237645601, 0.17005494040879293`, 
     0.4296646687325366}, {0.06120946689020222, 0.16960575124043434`, 
     0.4284519906071119}, {0.056394485649961655`, 0.16901942228068398`, 
     0.427239312481683}, {0.051598121952500686`, 0.1682964276233186, 
     0.42602663435624993`}, {0.046824254040815944`, 0.16743735186724296`, 
     0.42481395623081264`}, {0.0420767419682652, 0.16644288964379506`, 
     0.4236012781053712}, {0.03735942447740605, 0.16531384505508118`, 
     0.4223885999799255}, {0.032676115896066046`, 0.1640511310237951, 
     0.42117592185447567`}, {0.02803060305315415, 0.16265576855504724`, 
     0.4199632437290216}, {0.023426642216707157`, 0.1611288859108006, 
     0.4187505656035634}, {0.018867956056647123`, 0.15947171769758076`, 
     0.41753788747810094`}, {0.0143582306347055, 0.157685603868198, 
     0.41632520935263434`}, {0.009901112423947919, 0.1557719886382884, 
     0.4151125312271635}, {0.005500205360309587, 0.1537324193185503, 
     0.41389985310168853`}, {0.0011590679285253327`, 0.15156854506362002`, 
     0.4126871749762093}, {-0.0031187897151894106`, 0.14928211553859863`, 
     0.41147449685072596`}, {-0.007329908581380168, 0.14687497950430803`, 
     0.41026181872523837`}, {-0.011470883644222562`, 0.14434908332242025`, 
     0.4090491405997466}, {-0.015538366594757554`, 0.14170646938166861`, 
     0.40783646247425065`}, {-0.019529068548266575`, 0.13894927444641342`, 
     0.4066237843487505}, {-0.023439762703597405`, 0.13607972792889733`, 
     0.40541110622324616`}, {-0.02726728695229055, 0.1331001500865876, 
     0.40419842809773765`}, {-0.03100854643539638, 0.13001295014606257`, 
     0.4029857499722249}, {-0.034660516045915704`, 0.12682062435495975`, 
     0.401773071846708}, {-0.0382202428748403, 0.12352575396356025`, 
     0.4005603937211869}, {-0.04168484859881562, 0.1201310031376418, 
     0.3993477155956616}, {-0.04505153180749502, 0.11663911680428808`, 
     0.3981350374701321}, {-0.048317570268703716`, 0.11305291843239597`, 
     0.39692235934459846`}, {-0.05148032312958083, 0.10937530774967551`, 
     0.3957096812190606}, {-0.054537233051919765`, 0.10560925839798857`, 
     0.39449700309351854`}, {-0.05748582827998036, 0.10175781552892198`, 
     0.3932843249679723}, {-0.06032372463910074, 0.0978240933415393, 
     0.39207164684242185`}, {-0.06304862746349288, 0.09381127256430229, 
     0.3908589687168672}, {-0.06565833345166311, 0.08972259788319796, 
     0.3896462905913084}, {-0.06815073244795729, 0.08556137531815092, 
     0.38843361246574537`}, {-0.07052380914879014, 0.08133096954984227, 
     0.3872209343401782}, {-0.07277564473217908, 0.0770348011990967, 
     0.38600825621460677`}, {-0.07490441840926504, 0.07267634406103736, 
     0.3847955780890312}, {-0.07690840889656558, 0.06825912229624524, 
     0.3835828999634514}, {-0.07878599580777007, 0.06378670758119391, 
     0.38237022183786745`}, {-0.08053566096395133, 0.05926271622026393, 
     0.38115754371227933`}, {-0.08215598962113459, 0.05469080622167204, 
     0.379944865586687}, {-0.08364567161423087, 0.05007467433967947, 
     0.3787321874610905}, {-0.08500350241641011, 0.045418053085470936`, 
     0.3775195093354898}, {-0.08622838411305729, 0.04072470770912136, 
     0.3763068312098849}, {-0.08731932628952403, 0.035998433155090555`, 
     0.3750941530842758}, {-0.08827544683195793, 0.031243050993707645`, 
     0.37388147495866253`}, {-0.08909597264056202, 0.02646240633112632, 
     0.37266879683304505`}, {-0.08978024025470771, 0.021660364700249526`, 
     0.3714561187074234}, {-0.09032769638939568, 0.01684080893513747, 
     0.37024344058179753`}, {-0.09073789838263095, 0.012007636031426287`, 
     0.3690307624561675}, {-0.09101051455335048, 0.007164753995295918, 
     0.36781808433053326`}, {-0.09114532446961375, 0.0023160786835350958`, 
     0.36660540620489485`}, {-0.09114221912683963, -0.002534469362741522, 
     0.3653927280792522}, {-0.09100120103594529, -0.007382968088163857, 
     0.3641800499536054}, {-0.09072238422131587, -0.012225497094401322`, 
     0.3629673718279544}, {-0.09030599412860672, -0.017058140810117885`, 
     0.36175469370229923`}, {-0.08975236744245257, -0.02187699165702412, 
     0.36054201557663984`}, {-0.08906195181423113, -0.026678153209466232`, 
     0.3593293374509763}, {-0.08823530550010117, -0.031457743344997305`, 
     0.3581166593253085}, {-0.0872730969096079, -0.03621189738338325, 
     0.35690398119963657`}, {-0.08617610406522047, -0.04093677121150533, 
     0.3556913030739604}, {-0.08494521397323869, -0.04562854439163256, 
     0.3544786249482801}, {-0.0835814219065776, -0.05028342325055061, 
     0.35326594682259554`}, {-0.08208583060000985, -0.054897643947049504`, 
     0.35205326869690684`}, {-0.08045964935851666, -0.05946747551528977, 
     0.3508405905712139}, {-0.0787041930794681, -0.06398922288158616, 
     0.34962791244551683`}, {-0.07682088118942368, -0.06845922985216982, 
     0.3484152343198155}, {-0.07481123649641254, -0.07287388206951279, 
     0.34720255619411006`}, {-0.07267688395862168, -0.0772296099348247, 
     0.34598987806840037`}, {-0.07041954937048746, -0.08152289149435833, 
     0.3447771999426865}, {-0.06804105796725303, -0.0857502552871905, 
     0.34356452181696845`}, {-0.06554333294911993, -0.08990828315217533, 
     0.3423518436912462}, {-0.06292839392618714, -0.09399361299180047, 
     0.34113916556551976`}, {-0.06019835528543511, -0.09800294149071133, 
     0.33992648743978915`}, {-0.05735542448107503, -0.10193302678670535`, 
     0.3387138093140543}, {-0.05440190024964588, -0.10578069109203637`, 
     0.3375011311883153}, {-0.05134017075130241, -0.10954282326290982`, 
     0.3362884530625721}, {-0.048172711638796946`, -0.11321638131509075`, 
     0.3350757749368247}, {-0.04490208405571653, -0.11679839488359112`, 
     0.3338630968110731}, {-0.04153093256559381, -0.1202859676244469, 
     0.33265041868531736`}, {-0.038061983013566295`, -0.12367627955664354`, 
     0.3314377405595574}, {-0.03449804032231292, -0.12696658934229568`, 
     0.33022506243379324`}, {-0.030841986224050117`, -0.1301542365032378, 
     0.3290123843080249}, {-0.02709677693042124, -0.13323664357223308`, 
     0.32779970618225235`}, {-0.02326544074216341, -0.13621131817706167`, 
     0.3265870280564756}, {-0.019351075600484602`, -0.13907585505580242`, 
     0.3253743499306947}, {-0.015356846582130825`, -0.1418279380016793, 
     0.32416167180490957`}, {-0.011285983340168872`, -0.1444653417358995, 
     0.3229489936791203}, {-0.007141777492553956, -0.14698593370696894`, 
     0.3217363155533268}, {-0.002927579960593788, -0.1493876758150305, 
     0.32052363742752915`}, {0.0013532017405388473`, -0.15166862605983047`, 
     0.3193109593017273}, {0.005697106257054164, -0.15382694011098075`, 
     0.31809828117592126`}, {0.010100621195326277`, -0.15586087279924726`, 
     0.31688560305011104`}, {0.014560185961943882`, -0.15776877952765864`, 
     0.3156729249242966}, {0.01907219464273431, -0.15954911760129414`, 
     0.314460246798478}, {0.023632998918433037`, -0.1612004474746757, 
     0.3132475686726552}, {0.0282389110146411, -0.16272143391575541`, 
     0.3120348905468282}, {0.03288620668368513, -0.16411084708555712`, 
     0.310822212420997}, {0.03757112821596896, -0.16536756353259957`, 
     0.30960953429516164`}, {0.042289887478381825`, -0.1664905671012966, 
     0.30839685616932205`}, {0.04703866897730648, -0.16747894975359998`, 
     0.3071841780434783}, {0.05181363294375036, -0.16833191230322078`, 
     0.30597149991763034`}, {0.05661091843810545, -0.16904876506183528`, 
     0.3047588217917782}, {0.061426646472026177`, -0.16962892839675295`, 
     0.30354614366592186`}, {0.06625692314490121, -0.17007193319959588`, 
     0.30233346554006135`}, {0.071097842792383, -0.17037742126561026`, 
     0.3011207874141966}, {0.07594549114442924, -0.17054514558330364`, 
     0.29990810928832773`}, {0.0807959484903027, -0.17057497053417334`, 
     0.2986954311624546}, {0.08564529284797032, -0.17046687200236488`, 
     0.29748275303657734`}, {0.09048960313533885, -0.1702209373941716, 
     0.29627007491069585`}, {0.09532496234076276, -0.16983736556735965`, 
     0.2950573967848102}, {0.1001474606902609, -0.16931646667037564`, 
     0.2938447186589203}, {0.10495319880888092`, -0.16865866189156684`, 
     0.29263204053302627`}, {0.10973829087365529`, -0.1678644831186168, 
     0.291419362407128}, {0.11449886775559937`, -0.16693457250847166`, 
     0.2902066842812256}, {0.11923108014821114`, -0.16586968196810506`, 
     0.28899400615531895`}, {0.12393110167994284`, -0.1646706725465412, 
     0.28778132802940815`}, {0.1285951320081279, -0.16333851373862807`, 
     0.2865686499034931}, {0.1332193998918613, -0.1618742827011234, 
     0.28535597177757394`}, {0.1378001662413491, -0.16027916338172743`, 
     0.28414329365165053`}, {0.142333727141261, -0.15855444556176673`, 
     0.28293061552572296`}, {0.14681641684564156`, -0.1567015238133029, 
     0.2817179373997912}, {0.15124461074195858`, -0.15472189637150985`, 
     0.2805052592738552}, {0.15561472828189168`, -0.1526171639232309, 
     0.27929258114791505`}, {0.15992323587649152`, -0.1503890283126958, 
     0.2780799030219707}, {0.16416664975336853`, -0.1480392911654437, 
     0.27686722489602217`}, {0.16834153877360117`, -0.14556985243156512`, 
     0.27565454677006945`}, {0.1724445272060855, -0.14298270884944056`, 
     0.2744418686441125}, {0.1764722974570835, -0.14027995233121812`, 
     0.2732291905181514}, {0.1804215927527624, -0.13746376827133552`, 
     0.2720165123921861}, {0.18428921977255633`, -0.1345364337794542, 
     0.2708038342662166}, {0.18807205123122098`, -0.13150031583923433`, 
     0.2695911561402429}, {0.19176702840749346`, -0.12835786939443958`, 
     0.26837847801426506`}, {0.1953711636173126, -0.12511163536391906`, 
     0.267165799888283}, {0.19888154262960006`, -0.12176423858707153`, 
     0.26595312176229674`}, {0.20229532702264882`, -0.11831838570145324`, 
     0.26474044363630633`}, {0.20560975647921387`, -0.11477686295424533`, 
     0.2635277655103117}, {0.2088221510184489, -0.1111425339493505, 
     0.2623150873843129}, {0.21192991316288495`, -0.10741833733194067`, 
     0.2611024092583099}, {0.2149305300386983, -0.1036072844123277, 
     0.25988973113230274`}, {0.21782157540756955`, -0.09971245673107855, 
     0.25867705300629135`}, {0.22060071162849101`, -0.0957370035673438, 
     0.2574643748802758}, {0.22326569154793616`, -0.0916841393924139, 
     0.256251696754256}, {0.22581436031686264`, -0.08755714127056255, 
     0.2550390186282321}, {0.2282446571330798, -0.08335934620927853, 
     0.25382634050220393`}, {0.2305546169075717, -0.07909414846102877, 
     0.2526136623761716}, {0.2327423718534287, -0.07476499677873423, 
     0.2514009842501351}, {0.23480615299610202`, -0.07037539162717776, 
     0.2501883061240944}, {0.23674429160376115`, -0.06592888235259887, 
     0.24897562799804945`}, {0.2385552205365966, -0.061429064312763904`, 
     0.24776294987200034`}, {0.2402374755139777, -0.05687957596983223, 
     0.24655027174594704`}, {0.24178969629844024`, -0.05228409594836909, 
     0.24533759361988955`}, {0.24321062779554725`, -0.047646340060883904`, 
     0.24412491549382787`}, {0.24449912106873292`, -0.04297005830329921, 
     0.242912237367762}, {0.24565413426830962`, -0.038259031822779545`, 
     0.24169955924169192`}, {0.24667473347388652`, -0.03351706986037205, 
     0.24048688111561767`}, {0.24756009344951885`, -0.028748006670930967`, 
     0.23927420298953922`}, {0.24830949831097696`, -0.023955698422816453`, 
     0.23806152486345658`}, {0.24892234210459593`, -0.019144020079874573`, 
     0.23684884673736975`}, {0.2493981292972376, -0.01431686226821964, 
     0.23563616861127876`}, {0.24973647517696843`, -0.009478128130352377, 
     0.23442349048518357`}, {0.24993710616413004`, -0.00463173016915756, 
     0.2332108123590842}, {0.2499998600325501, 0.00021841291566693406`, 
     0.23199813423298063`}, {0.24992468604071516`, 0.005068379396191718, 
     0.23078545610687287`}, {0.2497116449727993, 0.009914247687286045, 
     0.22957277798076092`}, {0.2493609090895151, 0.01475209951753249, 
     0.22836009985464478`}, {0.2488727619888273, 0.019578023097462233`, 
     0.22714742172852445`}, {0.248247598376641, 0.02438811628254918, 
     0.22593474360239993`}, {0.24748592374765066`, 0.02917848972840535, 
     0.22472206547627122`}, {0.24658835397660706`, 0.03394527003562636, 
     0.2235093873501383}, {0.2455556148203335, 0.03868460288174411, 
     0.22229670922400122`}, {0.24438854133089347`, 0.0433926561377542, 
     0.22108403109785993`}, {0.24308807718038436`, 0.04806562296669825, 
     0.21987135297171445`}, {0.24165527389790326`, 0.052699724901795496`, 
     0.21865867484556478`}, {0.24009129001930166`, 0.05729121490163485, 
     0.21744599671941092`}, {0.23839739015041678`, 0.061836380379957076`, 
     0.21623331859325287`}, {0.2365749439445368, 0.0663315462075772, 
     0.21502064046709063`}, {0.2346254249949267, 0.07077307768401987, 
     0.2138079623409242}, {0.2325504096433106, 0.07515738347646495, 
     0.21259528421475357`}, {0.2303515757052736, 0.07948091852362672, 
     0.21138260608857876`}, {0.22803070111361376`, 0.08374018690221913, 
     0.21016992796239975`}, {0.2255896624807417, 0.08793174465368878, 
     0.20895724983621655`}, {0.22303043358128946`, 0.09205220256893043, 
     0.20774457171002916`}, {0.22035508375615626`, 0.09609822892873318, 
     0.20653189358383758`}, {0.21756577623928106`, 0.10006655219774142`, 
     0.2053192154576418}, {0.21466476640849524`, 0.10395396366975242`, 
     0.20410653733144185`}, {0.2116543999618696, 0.10775732006221143`, 
     0.2028938592052377}, {0.2085371110210301, 0.11147354605780668`, 
     0.20168118107902935`}, {0.20531542016297621`, 0.11509963679110888`, 
     0.20046850295281682`}, {0.2019919323819932, 0.11863266027824486`, 
     0.1992558248266001}, {0.19856933498330623`, 0.12206975978764067`, 
     0.19804314670037917`}, {0.19505039541017954`, 0.1254081561499171, 
     0.19683046857415407`}, {0.1914379590062176, 0.12864515000507, 
     0.19561779044792477`}, {0.1877349467146776, 0.1317781239851185, 
     0.19440511232169128`}, {0.18394435271665371`, 0.1348045448304558, 
     0.1931924341954536}, {0.18006924201004249`, 0.13772196543819198`, 
     0.19197975606921172`}, {0.1761127479312474, 0.140528026840832, 
     0.19076707794296566`}, {0.17207806962162625`, 0.14322046011368927`, 
     0.1895543998167154}, {0.16796846944072985`, 0.1457970882094925, 
     0.18834172169046096`}, {0.1637872703284239, 0.14825582771870208`, 
     0.18712904356420232`}, {0.15953785311802687`, 0.15059469055411318`, 
     0.1859163654379395}, {0.15522365380263614`, 0.15281178555838296`, 
     0.18470368731167247`}, {0.15084816075685345`, 0.1549053200331822, 
     0.18349100918540126`}, {0.14641491191615538`, 0.1568736011887351, 
     0.18227833105912586`}, {0.14192749191619014`, 0.15871503751257504`, 
     0.18106565293284627`}, {0.13738952919431333`, 0.16042814005640932`, 
     0.17985297480656248`}, {0.13280469305570672`, 0.16201152364005283`, 
     0.1786402966802745}, {0.12817669070645205`, 0.16346390797145688`, 
     0.17742761855398237`}, {0.12350926425595886`, 0.1647841186819274, 
     0.17621494042768604`}, {0.1188061876911703, 0.16597108827569595`, 
     0.17500226230138552`}, {0.11407126382499336`, 0.16702385699307518`, 
     0.1737895841750808}, {0.109308321221421, 0.16794157358650116`, 
     0.1725769060487719}, {0.10452121109983249`, 0.16872349600883513`, 
     0.1713642279224588}, {0.09971380422097502, 0.1693689920133677, 
     0.17015154979614153`}, {0.09488998775714462, 0.16987753966504088`, 
     0.16893887166982005`}, {0.090053662149097, 0.17024872776247424`, 
     0.16772619354349438`}, {0.08520873795222977, 0.1704822561704541, 
     0.16651351541716453`}, {0.08035913267458616, 0.1705779360626167, 
     0.16530083729083048`}, {0.07550876760923701, 0.17053569007412955`, 
     0.16408815916449224`}, {0.0706615646636022, 0.17035555236424693`, 
     0.1628754810381498}, {0.06582144318827539, 0.17003766858868946`, 
     0.16166280291180318`}, {0.06099231680791615, 0.16958229578186987`, 
     0.16045012478545237`}, {0.05617809025677192, 0.1689898021490601, 
     0.15923744665909736`}, {0.05138265622138864, 0.16826066676866794`, 
     0.15802476853273817`}, {0.046609892193062843`, 0.16739547920486397`, 
     0.15681209040637478`}, {0.041863657332580365`, 0.16639493903087174`, 
     0.1555994122800072}, {0.03714778934977672, 0.16525985526330703`, 
     0.15438673415363544`}, {0.032466101400442315`, 0.16399114570802328`, 
     0.15317405602725948`}, {0.0278223790030815, 0.16258983621799242`, 
     0.15196137790087932`}, {0.02322037697801864, 0.16105705986382063`, 
     0.15074869977449498`}, {0.01866381641132607, 0.15939405601757042`, 
     0.14953602164810645`}, {0.014156381646028922`, 0.15760216935062935`, 
     0.14832334352171372`}, {0.009701717303019613, 0.15568284874643581`, 
     0.1471106653953168}, {0.0053034253340908544`, 0.15363764612894115`, 
     0.1458979872689157}, {0.0009650621094700416, 0.1514682152077552, 
     0.1446853091425104}, {-0.003309864457790022, 0.14917631014099, 
     0.14347263101610092`}, {-0.007517897748238914, 0.146763784116883, 
     0.14225995288968724`}, {-0.011655635230981405`, 0.14423258785534632`, 
     0.14104727476326936`}, {-0.01571973121489535, 0.14158476803065395`, 
     0.1398345966368473}, {-0.01970689955389009, 0.1388224656165423, 
     0.13862191851042105`}, {-0.02361391630401795, 0.13594791415506186`, 
     0.1374092403839906}, {-0.027437622330290318`, 0.13296343795058022`, 
     0.13619656225755597`}, {-0.031174925861090528`, 0.12987145019039634`, 
     0.13498388413111714`}, {-0.03482280498811806, 0.126674450993486, 
     0.13377120600467413`}, {-0.03837831010984267, 0.12337502538895598`, 
     0.13255852787822692`}, {-0.04183856631649276, 0.1199758412258416, 
     0.13134584975177552`}, {-0.045200775714649456`, 0.11647964701593781`, 
     0.13013317162531993`}, {-0.04846221968956681, 0.11288926971140796`, 
     0.12892049349886014`}, {-0.0516202611033889, 0.10920761241896722`, 
     0.12770781537239617`}, {-0.05467234642748637, 0.10543765205248903`, 
     0.126495137245928}, {-0.057616007807188147`, 0.1015824369259326, 
     0.12528245911945565`}, {-0.06044886505723905, 0.09764508428853763, 
     0.12406978099297912`}, {-0.06316862758636957, 0.09362877780427943, 
     0.1228571028664984}, {-0.06577309624942179, 0.0895367649776224, 
     0.12164442474001348`}, {-0.06826016512553389, 0.0853723545276534, 
     0.12043174661352438`}, {-0.07062782322094524, 0.08113891371271817, 
     0.11921906848703108`}, {-0.07287415609504545, 0.07683986560772413, 
     0.11800639036053359`}, {-0.07499734740835248, 0.072478686336311, 
     0.11679371223403191`}, {-0.07699568039116808, 0.06805890226012727, 
     0.11558103410752604`}, {-0.07886753923172322, 0.06358408712748534, 
     0.11436835598101598`}, {-0.0806114103826909, 0.05905785918370056, 
     0.11315567785450173`}, {-0.08222588378500997, 0.054483878245451076`, 
     0.11194299972798329`}, {-0.08370965400803053, 0.049865842741523815`, 
     0.11073032160146065`}, {-0.0850615213050588, 0.04520748672233953, 
     0.10951764347493383`}, {-0.08628039258344813, 0.04051257684067488, 
     0.10830496534840281`}, {-0.08736528228845168, 0.03578490930602294, 
     0.1070922872218676}, {-0.08831531320012212, 0.031028306815054717`, 
     0.1058796090953282}, {-0.08912971714261399, 0.026246615460663678`, 
     0.10466693096878463`}, {-0.08980783560531522, 0.021443701622092574`, 
     0.10345425284223686`}, {-0.09034912027530555, 0.016623448838657105`, 
     0.1022415747156849}, {-0.09075313348071129, 0.01178975466959432, 
     0.10102889658912875`}, {-0.09101954854459797, 0.00694652754257479, 
     0.09981621846256841}, {-0.09114815004911474, 0.002097683593426761, 
     0.09860354033600388}, {-0.09113883400967689, -0.002752856500372325, 
     0.09739086220943516}, {-0.09099160795904569, -0.007601170689876613, 
     0.09617818408286224}, {-0.0907065909412376, -0.012443338725961553`, 
     0.09496550595628514}, {-0.09028401341526762, -0.017275445329158323`, 
     0.09375282782970384}, {-0.0897242170688048, -0.02209358335546988, 
     0.09254014970311836}, {-0.08902765454189049, -0.026893856955608846`, 
     0.09132747157652868}, {-0.0881948890609427, -0.03167238472510271, 
     0.09011479344993481}, {-0.08722659398334259, -0.03642530284271924, 
     0.08890211532333675}, {-0.08612355225297132, -0.04114876819467445, 
     0.0876894371967345}, {-0.08488665576713729, -0.04583896148209696, 
     0.08647675907012806}, {-0.08351690465540607, -0.05049209030923603, 
     0.08526408094351742}, {-0.08201540647091574, -0.05510439224991637, 
     0.0840514028169026}, {-0.08038337529483176, -0.05967213788976007, 
     0.08283872469028358}, {-0.07862213075466544, -0.06419163384171592, 
     0.08162604656366038}, {-0.07673309695724968, -0.06865922573245771, 
     0.08041336843703298}, {-0.07471780133723493, -0.07307130115723685, 
     0.07920069031040139}, {-0.07257787342203634, -0.07742429260080003, 
     0.07798801218376561}, {-0.07031504351423067, -0.08171468032201014, 
     0.07677533405712564}, {-0.06793114129246859, -0.08593899519983803, 
     0.07556265593048149}, {-0.06542809433203338, -0.0900938215384238, 
     0.07434997780383315}, {-0.06280792654624237, -0.09417579982893967, 
     0.07313729967718062}, {-0.06007275654995149, -0.09818162946602108, 
     0.0719246215505239}, {-0.057224795946485954`, -0.10210807141656965`, 
     0.07071194342386299}, {-0.05426634753938243, -0.10595195083877007`, 
     0.06949926529719788}, {-0.05119980347038849, -0.10971015964920319`, 
     0.06828658717052859}, {-0.04802764328522503, -0.11337965903597963`, 
     0.0670739090438551}, {-0.044752431928675604`, -0.1169574819158618, 
     0.06586123091717742}, {-0.04137681767062372, -0.12044073533338764`, 
     0.06464855279049556}, {-0.037903529964715243`, -0.12382660280005603`, 
     0.0634358746638095}, {-0.03433537724137718, -0.1271123465716827, 
     0.062223196537119246`}, {-0.030675244636977447`, -0.1302953098620849, 
     0.061010518410424805`}, {-0.026926091660961757`, -0.13337291899130527`, 
     0.05979784028372617}, {-0.023090949802853946`, -0.13634268546663747`, 
     0.05858516215702335}, {-0.019172920081054645`, -0.1392022079947713, 
     0.057372484030316334`}, {-0.01517517053542032, -0.14194917442342997`, 
     0.05615980590360513}, {-0.011100933665650091`, -0.14458136361093, 
     0.05494712777688974}, {-0.006953503817551641, -0.1470966472221514, 
     0.05373444965017016}, {-0.002736234519299602, -0.14949299144946676`, 
     0.052521771523446385`}, {0.0015474642301597282`, -0.1517684586572369, 
     0.05130909339671842}, {0.0058941287183648, -0.15392120894854414`, 
     0.05009641526998627}, {0.010300244320026889`, -0.1559495016528958, 
     0.04888373714324992}, {0.014762248338885255`, -0.15785169673369534`, 
     0.047671059016509386`}, {0.01927653288843153, -0.15962625611434275`, 
     0.04645838088976466}, {0.023839447809174016`, -0.16127174492189247`, 
     0.04524570276301574}, {0.02844730362008308, -0.1627868326472626, 
     0.04403302463626263}, {0.03309637450183114, -0.16417029422105775`, 
     0.04282034650950533}, {0.03778290130941507, -0.16542101100413553`, 
     0.04160766838274384}, {0.04250309461172508, -0.16653797169211554`, 
     0.04039499025597816}, {0.04725313775560233, -0.16752027313309975`, 
     0.039182312129208294`}, {0.05202918995190773, -0.16836712105794288`, 
     0.037969634002434235`}, {0.056827389381106624`, -0.16907783072248248`, 
     0.036756955875655986`}, {0.061643856315858225`, -0.1696518274612092, 
     0.035544277748873546`}, {0.06647469625808494, -0.17008864715192976`, 
     0.034331599622086914`}, {0.07131600308798501, -0.17038793659104684`, 
     0.03311892149529609}, {0.07616386222244223, -0.17054945377915234`, 
     0.03190624336850108}, {0.08101435378027899, -0.17057306811670325`, 
     0.030693565241701873`}, {0.08586355575179315, -0.1704587605096218, 
     0.02948088711489848}, {0.09070754717001603, -0.17020662338473458`, 
     0.028268208988090897`}, {0.09554241128112728, -0.169816860615038, 
     0.027055530861279122`}, {0.10036423871146306`, -0.16928978735485078`, 
     0.025842852734463156`}, {0.1051691306285568, -0.16862582978498647`, 
     0.024630174607643}, {0.1099532018936565, -0.16782552476815224`, 
     0.02341749648081865}, {0.11471258420316965`, -0.16688951941485256`, 
     0.022204818353990116`}, {0.11944342921649556`, -0.16581857056014862`, 
     0.02099214022715739}, {0.12414191166771595`, -0.16461354415169685`, 
     0.019779462100320472`}, {0.12880423245862788`, -0.16327541454956102`, 
     0.018566783973479363`}, {0.133426621730618, -0.16180526373836435`, 
     0.017354105846634063`}, {0.13800534191289426`, -0.16020428045241858`, 
     0.016141427719784572`}, {0.14253669074461023`, -0.15847375921453738`, 
     0.014928749592930892`}, {0.14701700426843864`, -0.15661509928931128`, 
     0.013716071466073022`}, {0.15144265979317345`, -0.15462980355169048`, 
     0.012503393339210961`}, {0.1558100788229649, -0.15251947727179055`, 
     0.01129071521234471}, {0.16011572995081919`, -0.1502858268169033, 
     0.010078037085474266`}, {0.16435613171402283`, -0.14793065827176263`, 
     0.008865358958599634}, {0.1685278554091834, -0.1454558759781808, 
     0.0076526808317208105`}, {0.17262752786460994`, -0.14286348099523605`, 
     0.006440002704837797}, {0.17665183416779154`, -0.14015556948125651`, 
     0.005227324577950592}, {0.1805975203457688, -0.1373343309989089, 
     0.004014646451059197}, {0.18446139599623085`, -0.13440204674476225`, 
     0.002801968324163612}, {0.1882403368672102, -0.13136108770475835`, 
     0.001589290197263836}, {0.19193128738329007`, -0.12821391273708033`, 
     0.00037661207035986997`}, {
     0.195531263116281, -0.12496306658396947`, -0.000836066056548287}, {
     0.1990373531983693, -0.12161117781409791`, -0.0020487441834606346`}, {
     0.20244672267578614`, -0.11816095669716105`, -0.0032614223103771727`}, {
     0.20575661480109408`, -0.11461519301240805`, -0.004474100437297901}, {
     0.20896435326223733`, -0.11097675379288262`, -0.0056867785642228195`}, {
     0.21206734434655386`, -0.10724858100719785`, -0.006899456691151929}, {
     0.21506307903799912`, -0.10343368918071974`, -0.008112134818085228}, {
     0.21794913504588587`, -0.09953516295808268, -0.009324812945022718}, {
     0.2207231787634996, -0.09555615460900806, -0.0105374910719644}, {
     0.22338296715500586`, -0.09149988147944244, -0.011750169198910271`}, {
     0.22592634956912389`, -0.0873696233900765, -0.012962847325860333`}, {
     0.22835126947809978`, -0.0831687199843482, -0.014175525452814586`}, {
     0.23065576614057345`, -0.0789005680280743, -0.01538820357977303}, {
     0.23283797618699462`, -0.07456861866289406, -0.016600881706735664`}, {
     0.23489613512630578`, -0.07017637461574569, -0.01781355983370249}, {
     0.23682857877267413`, -0.06572738736663189, -0.019026237960673504`}, {
     0.23863374459111875`, -0.06122525427696477, -0.02023891608764871}, {
     0.24031017296094478`, -0.05667361568081188, -0.021451594214628104`}, {
     0.24185650835596345`, -0.0520761519413955, -0.02266427234161169}, {
     0.2432715004405431, -0.04743658047522507, -0.023876950468599466`}, {
     0.2445540050806054, -0.042758652746269186`, -0.025089628595591433`}, {
     0.24570298526874917`, -0.03804615123259743, -0.02630230672258759}, {
     0.24671751196275363`, -0.03330288636794484, -0.02751498484958794}, {
     0.24759676483678314`, -0.028532693460672005`, -0.02872766297659248}, {
     0.24834003294468626`, -0.023739429592612046`, -0.02994034110360121}, {
     0.2489467152948524, -0.018926970500311963`, -0.03115301923061413}, {
     0.24941632133616154`, -0.014099207441190212`, -0.032365697357631236`}, {
     0.249748471354634, -0.009260044047144372, -0.033578375484652534`}, {
     0.24994289678045944`, -0.004413393168153097, -0.03479105361167802}, {
     0.24999944040515718`, 0.00043682629157552194`, -0.0360037317387077}, {
     0.24991805650869153`, 0.005286692542350901, -0.037216409865741575`}, {
     0.2496988108964404, 0.010132284080080057`, -0.03842908799277964}, {
     0.2493418808459861, 0.014969682857116771`, -0.03964176611982189}, {
     0.2488475549637725, 0.01979497745031595, -0.040854444246868335`}, {
     0.24821623295174358`, 0.02460426622373152, -0.04206712237391897}, {
     0.24744842528415248`, 0.029393660483400627`, -0.0432798005009738}, {
     0.2465447527948023, 0.03415928762166319, -0.044492478628032815`}, {
     0.24550594617505253`, 0.038897294248474386`, -0.045705156755096024`}, {
     0.24433284538299657`, 0.04360384930717818, -0.04691783488216342}, {
     0.24302639896428868`, 0.048275147172222505`, -0.048130513009235}, {
     0.24158766328516898`, 0.052907410726311395`, -0.04934319113631078}, {
     0.24001780167830714`, 0.05749689441450587, -0.05055586926339074}, {
     0.23831808350215486`, 0.06203988727280418, -0.0517685473904749}, {
     0.2364898831145684, 0.0665327159287525, -0.05298122551756325}, {
     0.23453467876153034`, 0.07097174757165983, -0.05419390364465579}, {
     0.23245405138186984`, 0.07535339289001554, -0.05540658177175252}, {
     0.23024968332894727`, 0.07967410897373424, -0.05661925989885344}, {
     0.2279233570103372, 0.08393040217888154, -0.057831938025958554`}, {
     0.2254769534466096, 0.08811883095256406, -0.05904461615306786}, {
     0.22291245075037452`, 0.09223600861569978, -0.06025729428018135}, {
     0.2202319225268201, 0.09627860610141842, -0.06146997240729903}, {
     0.2174375361970372, 0.10024335464687785`, -0.0626826505344209}, {
     0.2145315512454864, 0.10412704843631984`, -0.06389532866154697}, {
     0.21151631739302437`, 0.10792654719322806`, -0.06510800678867723}, {
     0.20839427269696711`, 0.11163877871949235`, -0.06632068491581168}, {
     0.20516794157972573`, 0.11526074137952623`, -0.06753336304295031}, {
     0.20183993278760953`, 0.11878950652732889`, -0.06874604117009313}, {
     0.19841293728144627`, 0.12222222087452937`, -0.06995871929724014}, {
     0.19488972606072544`, 0.12555610879749798`, -0.07117139742439134}, {
     0.191273147923024, 0.12878847458165962`, -0.07238407555154673}, {
     0.18756612716052604`, 0.13191670460119428`, -0.07359675367870631}, {
     0.1837716611954991, 0.13493826943236217`, -0.07480943180587009}, {
     0.17989281815663888`, 0.1378507258987448, -0.07602210993303805}, {
     0.1759327343982423, 0.1406517190467482, -0.07723478806021021}, {
     0.1718946119642146, 0.14333898404977083`, -0.07844746618738656}, {
     0.16778171599896116`, 0.14591034803949687`, -0.0796601443145671}, {
     0.16359737210725755`, 0.14836373186283364`, -0.08087282244175183}, {
     0.15934496366523263`, 0.15069715176307288`, -0.08208550056894075}, {
     0.15502792908463864`, 0.15290872098391653`, -0.08329817869613386}, {
     0.15064975903262093`, 0.15499665129506976`, -0.08451085682333116}, {
     0.1462139936092348, 0.15695925443816797`, -0.08572353495053266}, {
     0.1417242194849919, 0.15879494349186846`, -0.08693621307773834}, {
     0.13718406700075086`, 0.16050223415500306`, -0.08814889120494822}, {
     0.1325972072322968, 0.16207974594675417`, -0.08936156933216229}, {
     0.12796734902198334`, 0.16352620332288367`, -0.09057424745938054}, {
     0.12329823597983765`, 0.16484043670711232`, -0.091786925586603}, {
     0.11859364345655263`, 0.16602138343681552`, -0.09299960371382963}, {
     0.11385737549081457`, 0.16706808862227096`, -0.09421228184106047}, {
     0.10909326173343409`, 0.1679797059187631, -0.09542495996829548}, {
     0.10430515435076769`, 0.1687554982109206, -0.09663763809553468}, {
     0.0994969249099337, 0.1693948382087328, -0.09785031622277807}, {
     0.09467246124834112, 0.1698972089547639, -0.09906299435002565}, {
     0.08983566433006268, 0.1702622042421542, -0.10027567247727742`}, {
     0.08499044509159391, 0.17048952894307096`, -0.10148835060453339`}, {
     0.08014072127954872, 0.17057899924734263`, -0.10270102873179354`}, {
     0.07529041428284845, 0.17053054281108415`, -0.10391370685905789`}, {
     0.07044344596196586, 0.17034419881519278`, -0.10512638498632643`}, {
     0.06560373547778774, 0.17002011793366714`, -0.10633906311359916`}, {
     0.060775196122660476`, 0.16955856221177532`, -0.10755174124087608`}, {
     0.055961732156180714`, 0.16895990485417028`, -0.10876441936815719`}, {
     0.05116723564828973, 0.16822462992312426`, -0.10997709749544249`}, {
     0.046395583332224155`, 0.16735333194712557`, -0.11118977562273198`}, {
     0.041650633469867596`, 0.166346715440155, -0.11240245375002567`}, {
     0.03693622273203785, 0.16520559433202991`, -0.11361513187732354`}, {
     0.03225616309623223, 0.1639308913102771, -0.11482781000462561`}, {
     0.027614238764339362`, 0.16252363707406617`, -0.11604048813193187`}, {
     0.02301420310280986, 0.16098496950080698`, -0.11725316625924231`}, {
     0.01845977560775982, 0.15931613272608497`, -0.11846584438655695`}, {
     0.013954638897461272`, 0.1575184761376781, -0.11967852251387578`}, {
     0.009502435734651244, 0.15559345328446925`, -0.1208912006411988}, {
     0.0051067660810672506`, 0.1535426207011357, -0.12210387876852602`}, {
     0.0007711841865907865, 
     0.1513676366495667, -0.12331655689585741`}, {-0.0035008042846474807`, 
     0.1490702597780263, -0.12452923502319299`}, {-0.0077057450888770715`, 
     0.14665234769914579`, -0.12574191315053276`}, {-0.011840238195719582`, 
     0.1441158554878956, -0.12695459127787673`}, {-0.015900940537384715`, 
     0.14146283410075108`, -0.12816726940522488`}, {-0.019884568711807514`, 
     0.13869542871733043`, -0.12937994753257723`}, {-0.023787901637541093`, 
     0.1358158770058457, -0.13059262565993376`}, {-0.02760778315825817, 
     0.13282650731376935`, -0.1318053037872945}, {-0.03134112459475546, 
     0.12972973678517952`, -0.1330179819146594}, {-0.03498490724239741, 
     0.1265280694063059, -0.13423066004202852`}, {-0.03853618481197995, 
     0.12322409398085707`, -0.13544333816940182`}, {-0.04199208581204054, 
     0.11982048203676592`, -0.13665601629677931`}, {-0.04534981587068827, 
     0.11631998566604604`, -0.137868694424161}, {-0.04860665999507671, 
     0.11272543529950561`, -0.13908137255154687`}, {-0.05175998476669232, 
     0.10903973741811808`, -0.14029405067893694`}, {-0.05480724047068366, 
     0.10526587220290022`, -0.1415067288063312}, {-0.057745963157509364`, 
     0.10140689112519788`, -0.14271940693372964`}, {-0.06057377663523813, 
     0.09746591447932769, -0.14393208506113228`}, {-0.06328839439088957, 
     0.09344612885957004, -0.1451447631885391}, {-0.06588762143926258, 
     0.08935078458355313, -0.14635744131595013`}, {-0.06836935609775606, 
     0.08518319306411166, -0.14757011944336534`}, {-0.07073159168574718, 
     0.0809467241317452, -0.14878279757078475`}, {-0.07297241814715283, 
     0.0766448033098411, -0.14999547569820834`}, {-0.0750900235948625, 
     0.07228090904486538, -0.15120815382563613`}, {-0.07708269577579366, 
     0.06785856989376102, -0.1524208319530681}, {-0.07894882345538512, 
     0.0633813616708279, -0.15363351008050427`}, {-0.08068689772040878, 
     0.058852904556391514`, -0.15484618820794463`}, {-0.0822955131990465, 
     0.05427686016959804, -0.15605886633538918`}, {-0.08377336919724539, 
     0.04965692860770296, -0.15727154446283792`}, {-0.08511927075043281, 
     0.044996845454246974`, -0.15848422259029085`}, {-0.08633212958974067, 
     0.040300378758538466`, -0.15969690071774797`}, {-0.08741096502195768, 
     0.03557132598888476, -0.1609095788452093}, {-0.0883549047224981, 
     0.030813510962035804`, -0.1621222569726748}, {-0.08916318544074586, 
     0.026030780751323018`, -0.1633349351001445}, {-0.0898351536172036, 
     0.02122700257599333, -0.16454761322761838`}, {-0.09037026591194769, 
     0.016406060674253667`, -0.16576029135509646`}, {-0.09076808964396199, 
     0.011571853162554196`, -0.1669729694825787}, {-0.09102830314099496, 
     0.006728288883649963, -0.16818564761006513`}, {-0.09115069599965736, 
     0.0018792842459894174`, -0.16939832573755575`}, {-0.09113516925555024, \
-0.002971239943014517, -0.17061100386505057`}, {-0.0909817354632855, \
-0.007819361647270293, -0.17182368199254958`}, {-0.09069051868633453, \
-0.012661160773287067`, -0.17303636012005277`}, {-0.09026175439671297, \
-0.01749272233988328, -0.17424903824756016`}, {-0.08969578928458276, \
-0.02231013964376153, -0.17546171637507174`}, {-0.08899308097792553, \
-0.027109517418390124`, -0.1766743945025875}, {-0.08815419767251378, \
-0.03188697498363707, -0.17788707263010747`}, {-0.08717981767247927, \
-0.03663864938360982, -0.17909975075763163`}, {-0.08607072884184995, \
-0.04136069851016354, -0.18031242888515997`}, {-0.08482782796749899, \
-0.04604930420955231, -0.1815251070126925}, {-0.08345212003402101, \
-0.05070067536971131, -0.18273778514022923`}, {-0.08194471741112179, \
-0.055311050985673585`, -0.18395046326777015`}, {-0.08030683895417853, \
-0.05987670320064287, -0.18516314139531526`}, {-0.07853980901869798, \
-0.06439394032026347, -0.18637581952286456`}, {-0.07664505638946925, \
-0.06885910979764988, -0.18758849765041805`}, {-0.0746241131252773, \
-0.07326860118676262, -0.18880117577797573`}, {-0.07247861332011096, \
-0.07761884906174209, -0.1900138539055376}, {-0.07021029178186762, \
-0.08190633589984005, -0.19122653203310366`}, {-0.0678209826296224, \
-0.08612759492561765, -0.19243921016067392`}, {-0.06531261781059651, \
-0.09027921291411019, -0.19365188828824836`}, {-0.06268722553802367, \
-0.09435783295069194, -0.194864566415827}, {-0.05994692865117772, \
-0.09836015714540976, -0.19607724454340983`}, {-0.05709394289888764, \
-0.10228294929959035`, -0.19728992267099685`}, {-0.054130575147927656`, \
-0.10612303752256534`, -0.19850260079858806`}, {-0.0510592215177313, \
-0.10987731679639812`, -0.19971527892618346`}, {-0.047882365442937495`, \
-0.11354275148653872`, -0.20092795705378305`}, {-0.0446025756653354, \
-0.11711637779637683`, -0.20214063518138684`}, {-0.041222504156831576`, \
-0.12059530616370791`, -0.2033533133089948}, {-0.037744883975118916`, \
-0.12397672359717495`, -0.20456599143660698`}, {-0.03417252705378133, \
-0.12725789595079662`, -0.20577866956422333`}, {-0.030508321928620897`, \
-0.13043617013474262`, -0.20699134769184388`}, {-0.02675523140204606, \
-0.1335089762605686, -0.20820402581946862`}, {-0.022916290147409302`, \
-0.13647382971917615`, -0.20941670394709755`}, {-0.018994602255231446`, \
-0.13932833318981772`, -0.21062938207473067`}, {-0.014993338723296626`, \
-0.14207017857852172`, -0.21184206020236798`}, {-0.010915734892647397`, \
-0.14469714888437094`, -0.21305473833000949`}, {-0.006765087831553175, \
-0.14720711999212466`, -0.21426741645765518`}, {-0.0025447536695672977`, \
-0.14959806238973555`, -0.21548009458530507`}, {
     0.001741855116171689, -0.15186804280937216`, -0.21669277271295914`}, {
     0.006091272460200265, -0.15401522579062027`, -0.2179054508406174}, {
     0.010499981511320604`, -0.15603787516459924`, -0.21911812896827987`}, {
     0.014964417476255567`, -0.1579343554577932, -0.22033080709594652`}, {
     0.019480970502068692`, -0.15970313321446186`, -0.22154348522361736`}, {
     0.02404598859501852, -0.1613427782365621, -0.2227561633512924}, {
     0.02865578057348713, -0.1628519647401771, -0.2239688414789716}, {
     0.03330661905259519, -0.1642294724275186, -0.22518151960665497`}, {
     0.03799474345809022, -0.16547418747363504`, -0.22639419773434255`}, {
     0.04271636306707119, -0.16658510342702806`, -0.22760687586203432`}, {
     0.04746766007309059, -0.16756132202344876`, -0.22881955398973028`}, {
     0.052244792673155814`, -0.16840205391221616`, -0.23003223211743043`}, {
     0.057043898174133606`, -0.16910661929447027`, -0.23124491024513477`}, {
     0.06186109611604587, -0.16967444847284366`, -0.2324575883728433}, {
     0.06669249140973141, -0.17010508231210736`, -0.23367026650055603`}, {
     0.0715341774863365, -0.1703981726104183, -0.23488294462827294`}, {
     0.07638223945608774, -0.17055348238086834`, -0.23609562275599405`}, {
     0.08123275727379298, -0.17057088604310705`, -0.23730830088371935`}, {
     0.08608180890851089, -0.17045036952488352`, -0.23852097901144884`}, {
     0.0909254735148261, -0.17019203027342486`, -0.23973365713918252`}, {
     0.09575983460316581, -0.16979607717664236`, -0.2409463352669204}, {
     0.10058098320659428`, -0.1692628303942289, -0.24215901339466245`}, {
     0.10538502104152472`, -0.16859272109878434`, -0.2433716915224087}, {
     0.11016806365979286`, -0.16778629112717797`, -0.24458436965015914`}, {
     0.11492624358954334`, -0.16684419254243021`, -0.24579704777791378`}, {
     0.11965571346238965`, -0.1657671871064676, -0.2470097259056726}, {
     0.12435264912431854`, -0.16455614566417737`, -0.24822240403343562`}, {
     0.1290132527278239, -0.1632120474392599, -0.24943508216120283`}, {
     0.1336337558027698, -0.16173597924244817`, -0.25064776028897423`}, {
     0.1382104223034993, -0.1601291345927345, -0.2518604384167498}, {
     0.14273955162972587`, -0.1583928127523151, -0.2530731165445296}, {
     0.14721748161876408`, -0.1565284176760329, -0.25428579467231355`}, {
     0.1516405915066805, -0.15453745687616802`, -0.2554984728001017}, {
     0.15600530485597056`, -0.15242154020349358`, -0.25671115092789404`}, {
     0.16030809244739372`, -0.15018237854558292`, -0.25792382905569056`}, {
     0.16454547513362908`, -0.1478217824434204, -0.2591365071834913}, {
     0.16871402665244375`, -0.1453416606274344, -0.2603491853112962}, {
     0.17281037639709948`, -0.14274401847413637`, -0.2615618634391053}, {
     0.17683121214175723`, -0.14003095638461388`, -0.2627745415669186}, {
     0.18077328271967633`, -0.13720466808618853`, -0.2639872196947361}, {
     0.1846334006520423, -0.13426743885861236`, -0.26519989782255776`}, {
     0.1884084447252981, -0.13122164368623662`, -0.26641257595038365`}, {
     0.19209536251489434`, -0.12806974533764734`, -0.2676252540782137}, {
     0.1956911728534183, -0.12481429237432029`, -0.26883793220604796`}, {
     0.19919296824110594`, -0.12145791708990553`, -0.2700506103338864}, {
     0.20259791719678738`, -0.1180033333818078, -0.27126328846172904`}, {
     0.20590326654736574`, -0.11445333455678383`, -0.27247596658957585`}, {
     0.20910634365397723`, -0.11081079107233077`, -0.2736886447174269}, {
     0.2122045585730332, -0.1070786482156921, -0.27490132284528207`}, {
     0.2151954061503962, -0.10325992372235776`, -0.2761140009731415}, {
     0.21807646804699726`, -0.09935770533598405, -0.27732667910100506`}, {
     0.22084541469425603`, -0.09537514831170632, -0.27853935722887285`}, {
     0.223500007177723, -0.09131547286486331, -0.2797520353567448}, {
     0.2260380990474206, -0.08718196156719592, -0.280964713484621}, {
     0.22845763805341937`, -0.0829779566926258, -0.2821773916125013}, {
     0.23075666780524598`, -0.07870685751476013, -0.2833900697403859}, {
     0.232933329353781, -0.07437211755830742, -0.2846027478682746}, {
     0.2349858626943678, -0.06997724180662707, -0.28581542599616755`}, {
     0.2369126081899168, -0.06552578386767045, -0.28702810412406465`}, {
     0.23871200791285457`, -0.06102134310060519, -0.288240782251966}, {
     0.24038260690483268`, -0.05646756170544582, -0.28945346037987146`}, {
     0.24192305435317765`, -0.05186812177804432, -0.29066613850778117`}, {
     0.24333210468313096`, -0.0472267423328216, -0.29187881663569504`}, {
     0.24460861856499558`, -0.042547176295647395`, -0.2930914947636131}, {
     0.24575156383537503`, -0.03783320746930007, -0.2943041728915354}, {
     0.24676001633175998`, -0.033088647473959946`, -0.29551685101946185`}, {
     0.24763316063978735`, -0.028317332665210037`, -0.2967295291473925}, {
     0.24837029075256792`, -0.023523121032036267`, -0.29794220727532733`}, {
     0.24897081064154933`, -0.018709889077335333`, -0.29915488540326635`}, {
     0.24943423473845272`, -0.013881528683452589`, -0.3003675635312096}, {
     0.24976018832789348`, -0.009041943965284429, -0.301580241659157}, {
     0.24994840785036854`, -0.004195048113489676, -0.3027929197871086}, {
     0.24999874111536524`, 0.0006552397696375214, -0.3040055979150644}, {
     0.2499111474244196, 0.00550499783907481, -0.3052182760430244}, {
     0.2496856976040242, 0.010350304678196462`, -0.30643095417098853`}, {
     0.24932257394835936`, 0.015187242469551837`, -0.3076436322989569}, {
     0.24882207007189366`, 0.02001190016273359, -0.30885631042692946`}, {
     0.24818459067197335`, 0.02482037663677421, -0.3100689885549062}, {
     0.24741065120159209`, 0.029608783854513807`, -0.31128166668288715`}, {
     0.2465008774526061, 0.03437325000638859, -0.31249434481087224`}, {
     0.2454560050497315, 0.03910992264109805, -0.31370702293886155`}, {
     0.24427687885573302`, 0.04381497178061942, -0.314919701066855}, {
     0.24296445228828487`, 0.04848459301705073, -0.3161323791948527}, {
     0.24151978654905656`, 0.05311501058877833, -0.31734505732285456`}, {
     0.2399440497656466, 0.05770248043348166, -0.31855773545086064`}, {
     0.23823851604705798`, 0.06224329321550656, -0.31977041357887087`}, {
     0.23640456445347943`, 0.06673377732515932, -0.3209830917068853}, {
     0.23444367788120507`, 0.0711703018474963, -0.32219576983490394`}, {
     0.23235744186359442`, 0.07554927949820851, -0.3234084479629268}, {
     0.230147543289042, 0.07986716952422747, -0.3246211260909538}, {
     0.22781576903699344`, 0.08412048056670672, -0.325833804218985}, {
     0.22536400453311076`, 0.08830577348406428, -0.3270464823470204}, {
     0.22279423222475517`, 0.09241966413280314, -0.32825916047506}, {
     0.22010852997802013`, 0.09645882610386156, -0.32947183860310375`}, {
     0.21730906939761066`, 0.1004199934122803, -0.33068451673115173`}, {
     0.21439811407092757`, 0.10429996313801235`, -0.3318971948592039}, {
     0.21137801773777634`, 0.10809559801573941`, -0.33310987298726025`}, {
     0.20825122238718052`, 0.11180382897160147`, -0.3343225511153208}, {
     0.2050202562828386, 0.11542165760478806`, -0.3355352292433855}, {
     0.201687731918821, 0.11894615861198471`, -0.33674790737145444`}, {
     0.19825634390716, 0.12237448215271421`, -0.33796058549952757`}, {
     0.19472886679904075`, 0.12570385615366025`, -0.33917326362760486`}, {
     0.19110815284135516`, 0.12893158855011, -0.34038594175568637`}, {
     0.18739712967043243`, 0.13205506946270335`, -0.34159861988377205`}, {
     0.18359879794481146`, 0.13507177330772882`, -0.34281129801186194`}, {
     0.1797162289189688, 0.13797926083925963`, -0.344023976139956}, {
     0.17575256195996408`, 0.14077518112147872`, -0.3452366542680543}, {
     0.17171100200901124`, 0.1434572734295982, -0.3464493323961567}, {
     0.1675948169900275, 0.1460233690778357, -0.3476620105242634}, {
     0.16340733516725614`, 0.14847139317297012`, -0.3488746886523742}, {
     0.15915194245409894`, 0.15079936629205842`, -0.35008736678048924`}, {
     0.1548320796753349, 0.15300540608295707`, -0.35130004490860844`}, {
     0.15045123978493877`, 0.1550877287863541, -0.35251272303673187`}, {
     0.1460129650417487, 0.15704465067808096`, -0.35372540116485945`}, {
     0.1415208441452673, 0.1588745894305379, -0.35493807929299126`}, {
     0.13697850933391145`, 0.16057606539213215`, -0.35615075742112723`}, {
     0.13238963344805757`, 0.16214770278369448`, -0.3573634355492674}, {
     0.1277579269602569, 0.1635882308109064, -0.35857611367741177`}, {
     0.12308713497502219`, 0.1648964846918389, -0.35978879180556034`}, {
     0.11838103420061169`, 0.16607140659877162`, -0.36100146993371307`}, {
     0.11364342989525894`, 0.16711204651353104`, -0.36221414806187}, {
     0.10887815279031769`, 0.168017562995656, -0.36342682619003114`}, {
     0.10408905599280972`, 0.16878722386276954`, -0.3646395043181965}, {
     0.09928001186988017, 0.16942040678260675`, -0.36585218244636597`}, {
     0.09445490891767949, 0.1699165997762202, -0.3670648605745397}, {
     0.0896176486172039, 0.1702754016319557, -0.36827753870271757`}, {
     0.08477214227963636, 0.170496522229864, -0.3694902168308996}, {
     0.07992230788373936, 0.17057978277628594`, -0.37070289495908587`}, {
     0.07507206690785607, 0.1705251159484214, -0.3719155730872763}, {
     0.07022534115908215, 0.17033256594876517`, -0.37312825121547094`}, {
     0.06538604960217145, 0.17000228846936566`, -0.37434092934366975`}, {
     0.0605581051907402, 0.1695345505659355, -0.3755536074718728}, {
     0.05574541170333152, 0.16892973044191567`, -0.37676628560007996`}, {
     0.05095186058689879, 0.16818831714266777`, -0.3779789637282914}, {
     0.046181327810260075`, 0.16731091016004188`, -0.37919164185650694`}, {
     0.041437670730067856`, 0.16629821894763946`, -0.38040431998472674`}, {
     0.0367247249718282, 0.16515106234716356`, -0.3816169981129507}, {
     0.03204630132849126, 0.1638703679263198, -0.38282967624117886`}, {
     0.027406182679120924`, 0.16245717122880388`, -0.3840423543694112}, {
     0.022808120930135026`, 0.1609126149369819, -0.38525503249764775`}, {
     0.01825583398158946, 0.1592379479479403, -0.38646771062588847`}, {
     0.013753002720959114`, 0.15743452436365293`, -0.3876803887541334}, {
     0.00930326804684644, 0.1555038023960815, -0.3888930668823825}, {
     0.004910227925024246, 0.15344734318809466`, -0.39010574501063583`}, {
     0.000577434479193074, 
     0.1512668095511595, -0.3913184231388933}, {-0.003691608881194429, 
     0.14896396462082565`, -0.392531101267155}, {-0.007893450293719783, 
     0.14654067043108931`, -0.3937437793954209}, {-0.012024692234105051`, 
     0.14399888640879033`, -0.39495645752369096`}, {-0.016081994263382506`, 
     0.14134066778925888`, -0.3961691356519652}, {-0.020062075728906233`, 
     0.13856816395449384`, -0.3973818137802437}, {-0.02396171841702176, 
     0.13568361669521564`, -0.3985944919085263}, {-0.02777776915524875, 
     0.13268935839819962`, -0.39980717003681315`}, {-0.0315071423618727, 
     0.12958781016035498`, -0.40101984816510416`}, {-0.03514682254088416, 
     0.12638147983107467`, -0.4022325262933994}, {-0.038693866720247994`, 
     0.12307295998443893`, -0.4034452044216988}, {-0.042145406831531285`, 
     0.1196649258229121, -0.4046578825500024}, {-0.045498652028965636`, 
     0.11616013301422788`, -0.4058705606783102}, {-0.04875089094606884, 
     0.11256141546321191`, -0.40708323880662217`}, {-0.05189949388800114, 
     0.10887168302034345`, -0.40829591693493833`}, {-0.0549419149578835, 
     0.10509391912890893`, -0.4095085950632587}, {-0.05787569411535851, 
     0.10123117841264986`, -0.41072127319158325`}, {-0.06069845916572942, 
     0.0972865842058556, -0.411933951319912}, {-0.06340792767806896, 
     0.09326332602789826, -0.41314662944824493`}, {-0.06600190883074704, 
     0.08916465700425158, -0.41435930757658207`}, {-0.06847830518288488, 
     0.08499389123607924, -0.4155719857049234}, {-0.0708351143703035, 
     0.0807544011205195, -0.4167846638332689}, {-0.07307043072459501, 
     0.07644961462383279, -0.4179973419616186}, {-0.0751824468140077, 
     0.0720830125096173, -0.4192100200899725}, {-0.07716945490489888, 
     0.06765812552433371, -0.42042269821833056`}, {-0.07902984834257394, 
     0.06317853154241476, -0.42163537634669285`}, {-0.08076212285039493, 
     0.05864785267326808, -0.4228480544750593}, {-0.08236487774610832, 
     0.054069752332511524`, -0.42406073260342997`}, {-0.08383681707440849, 
     0.04944793227980916, -0.4252734107318048}, {-0.085176750654821, 
     0.04478612962570302, -0.4264860888601838}, {-0.0863835950440586, 
     0.040088113809860784`, -0.427698766988567}, {-0.0874563744120716, 
     0.03535768355318291, -0.4289114451169544}, {-0.0883942213310844, 
     0.03059866378623343, -0.430124123245346}, {-0.0891963774769801, 
     0.0258149025564782, -0.43133680137374175`}, {-0.08986219424246608, 
     0.021010267916831277`, -0.43254947950214173`}, {-0.09039113326152484, 
     0.016188644798025297`, -0.4337621576305459}, {-0.09078276684472587, 
     0.01135393186733478, -0.43497483575895424`}, {-0.09103677832504671, 
     0.006510038376192347, -0.43618751388736676`}, {-0.09115296231392352, 
     0.001660880999246812, -0.4374001920157835}, {-0.09113122486732414, \
-0.003189619332581008, -0.4386128701442044}, {-0.09097158356170937, \
-0.008037540602484006, -0.43982554827262954`}, {-0.09067416747982096, \
-0.012878962879032526`, -0.44103822640105883`}, {-0.09023921710630795, \
-0.017709971485751916`, -0.44225090452949234`}, {-0.08966708413327555, \
-0.022526660166451024`, -0.44346358265793}, {-0.08895823117591412, \
-0.027325134243742215`, -0.4446762607863719}, {-0.08811323139843773, \
-0.03210151376819902, -0.44588893891481796`}, {-0.08713276805063522, \
-0.036851936655605065`, -0.44710161704326823`}, {-0.08601763391540812, \
-0.04157256180975746, -0.44831429517172267`}, {-0.08476873066774245, \
-0.046259572228299864`, -0.4495269733001813}, {-0.08338706814563243, \
-0.05090917808907358, -0.45073965142864414`}, {-0.0818737635335459, \
-0.05551761981449143, -0.4519523295571112}, {-0.08023004045909143, \
-0.06008117111145636, -0.4531650076855824}, {-0.07845722800361772, \
-0.064596141984367, -0.4543776858140578}, {-0.07655675962754518, \
-0.06905888171877368, -0.4555903639425374}, {-0.0745301720112988, \
-0.07346578183327261, -0.4568030420710212}, {-0.07237910381277923, \
-0.07781327899725116, -0.45801572019950915`}, {-0.07010529434237713, \
-0.08209785791212519, -0.45922839832800133`}, {-0.06771058215660175, \
-0.08631605415373869, -0.4604410764564977}, {-0.0651969035714613, \
-0.09046445697362726, -0.46165375458499824`}, {-0.06256629109679673, \
-0.09453971205688061, -0.462866432713503}, {-0.05982087179283538, \
-0.09853852423437397, -0.4640791108420119}, {-0.05696286555029282, \
-0.10245766014717546`, -0.46529178897052503`}, {-0.05399458329541404, \
-0.10629395086097494`, -0.46650446709904236`}, {-0.050918425121405035`, \
-0.11004429442842045`, -0.46771714522756386`}, {-0.04773687834776583, \
-0.11370565839729026`, -0.46892982335608957`}, {-0.04445251550909406, \
-0.11727508226247269`, -0.47014250148461945`}, {-0.04106799227498532, \
-0.12074967985977081`, -0.47135517961315354`}, {-0.0375860453027123, \
-0.12412664169959671`, -0.4725678577416918}, {-0.034009490024418845`, \
-0.12740323723866812`, -0.4737805358702343}, {-0.030341218370618265`, \
-0.13057681708787067`, -0.4749932139987809}, {-0.026584196431836623`, \
-0.13364481515450052`, -0.4762058921273318}, {-0.022741462060291713`, \
-0.13660475071715528`, -0.4774185702558868}, {-0.01881612241354698, \
-0.13945423043159522`, -0.47863124838444604`}, {-0.01481135144212654, \
-0.14219095026595327`, -0.47984392651300944`}, {-0.010730387323122774`, \
-0.14481269736372876`, -0.48105660464157707`}, {-0.00657652984187157, \
-0.14731735183305852`, -0.48226928277014885`}, {-0.0023531377238124266`, \
-0.1497028884608188, -0.4834819608987248}, {
     0.001936374081309253, -0.15196737835017174`, -0.484694639027305}, {
     0.0062885371607016875`, -0.1541089904802324, -0.4859073171558893}, {
     0.010699832443015405`, -0.15612599318659556`, -0.48711999528447786`}, {
     0.015166693043793401`, -0.15801675556152442`, -0.4883326734130706}, {
     0.019685507149582025`, -0.15977974877266984`, -0.4895453515416675}, {
     0.02425262093837055, -0.16141354729925345`, -0.4907580296702686}, {
     0.028864341533997993`, -0.1629168300847148, -0.4919707077988739}, {
     0.03351693999213836, -0.16428838160489131`, -0.4931833859274834}, {
     0.03820665431544987, -0.16552709285086648`, -0.4943960640560971}, {
     0.04292969249545018, -0.16663196222569213`, -0.49560874218471496`}, {
     0.04768223557865803, -0.1676020963542596, -0.49682142031333704`}, {
     0.05246044075452203, -0.16843671080566477`, -0.4980340984419633}, {
     0.057260444462639865`, -0.16913513072748315`, -0.49924677657059374`}, {
     0.062078365516755316`, -0.16969679139144192`, -0.5004594546992284}}]},
  Axes->True,
  AxesLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"y\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  PlotRange->{{-0.5, 0.5}, {-0.5, 0.5}, {-0.5, 0.5}}]], "Output",
 CellChangeTimes->{3.400936093612276*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", "\[IndentingNewLine]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"BField", "[", "r_", "]"}], ":=", 
  RowBox[{
   SuperscriptBox["10", 
    RowBox[{"-", "7"}]], " ", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"rMagnitude", "=", 
       RowBox[{"Norm", "[", "r", "]"}]}], "}"}], ",", 
     RowBox[{
      RowBox[{"3", " ", 
       FractionBox[
        RowBox[{
         RowBox[{"\[Mu]", ".", "r"}], " ", "r"}], 
        SuperscriptBox["rMagnitude", "5"]]}], "-", 
      FractionBox["\[Mu]", 
       SuperscriptBox["rMagnitude", "3"]]}]}], "]"}]}]}]], "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowToolbars->"EditBar",
WindowSize->{893, 653},
WindowMargins->{{Automatic, 174}, {101, Automatic}},
DockedCells->(FrontEndExecute[{
   FrontEnd`NotebookApply[
    FrontEnd`InputNotebook[], #, Placeholder]}]& ),
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
ShowSelection->True,
Magnification->1.25,
FrontEndVersion->"6.0 for Mac OS X x86 (32-bit) (June 19, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[590, 23, 134, 5, 89, "Subtitle"],
Cell[CellGroupData[{
Cell[749, 32, 31, 0, 83, "Section"],
Cell[CellGroupData[{
Cell[805, 36, 39, 0, 42, "Subsection"],
Cell[847, 38, 814, 24, 75, "Text"],
Cell[1664, 64, 1094, 36, 53, "Text"],
Cell[2761, 102, 508, 13, 53, "Text"],
Cell[3272, 117, 313, 9, 47, "Text"],
Cell[3588, 128, 536, 19, 47, "Text"],
Cell[4127, 149, 121, 3, 32, "Text"],
Cell[4251, 154, 363, 13, 34, "Text"],
Cell[4617, 169, 578, 19, 73, "Text"],
Cell[5198, 190, 879, 26, 47, "Text"],
Cell[6080, 218, 38, 0, 32, "Text"],
Cell[6121, 220, 1026, 34, 47, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7184, 259, 39, 0, 42, "Subsection"],
Cell[7226, 261, 221, 7, 32, "Text"],
Cell[7450, 270, 380, 11, 34, "Text"],
Cell[7833, 283, 564, 18, 47, "Text"],
Cell[8400, 303, 164, 6, 34, "Text"],
Cell[8567, 311, 482, 16, 38, "Text"],
Cell[9052, 329, 1040, 38, 59, "Text"],
Cell[10095, 369, 983, 39, 37, "Text"],
Cell[11081, 410, 59, 0, 32, "Text"],
Cell[11143, 412, 379, 12, 35, "Text"],
Cell[11525, 426, 187, 6, 34, "Text"],
Cell[11715, 434, 748, 22, 41, "Text"],
Cell[12466, 458, 1021, 28, 101, "Text"],
Cell[13490, 488, 872, 25, 41, "Text"],
Cell[14365, 515, 1470, 43, 110, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15872, 563, 43, 0, 42, "Subsection"],
Cell[15918, 565, 158, 3, 51, "Text"],
Cell[16079, 570, 411, 13, 40, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16527, 588, 104, 5, 42, "Subsection"],
Cell[16634, 595, 1493, 54, 102, "Text"],
Cell[18130, 651, 4579, 129, 625, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[22746, 785, 42, 0, 42, "Subsection"],
Cell[22791, 787, 266, 9, 51, "Text"],
Cell[23060, 798, 820, 32, 40, "Text"],
Cell[23883, 832, 75, 0, 32, "Text"],
Cell[23961, 834, 825, 27, 51, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[24823, 866, 103, 4, 42, "Subsection"],
Cell[24929, 872, 437, 13, 70, "Text"],
Cell[25369, 887, 12038, 326, 895, "Input"],
Cell[37410, 1215, 215, 6, 51, "Text"],
Cell[37628, 1223, 932, 23, 64, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[38609, 1252, 72, 1, 78, "Section"],
Cell[38684, 1255, 696, 20, 80, "Text"],
Cell[CellGroupData[{
Cell[39405, 1279, 505, 13, 79, "Input"],
Cell[39913, 1294, 1120, 22, 31, "Print"],
Cell[41036, 1318, 60824, 1090, 581, "Output"]
}, Open  ]],
Cell[101875, 2411, 469, 9, 89, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[102381, 2425, 72, 1, 78, "Section"],
Cell[102456, 2428, 668, 20, 76, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[103161, 2453, 86, 1, 78, "Section"],
Cell[103250, 2456, 859, 34, 59, "Text"],
Cell[104112, 2492, 200, 6, 32, "Text"],
Cell[104315, 2500, 902, 30, 41, "Text"],
Cell[105220, 2532, 841, 28, 78, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[106098, 2565, 76, 1, 78, "Section"],
Cell[CellGroupData[{
Cell[106199, 2570, 103, 4, 42, "Subsection"],
Cell[106305, 2576, 378, 12, 51, "Text"],
Cell[106686, 2590, 2339, 66, 248, "Input"],
Cell[CellGroupData[{
Cell[109050, 2660, 767, 22, 79, "Input"],
Cell[109820, 2684, 71353, 1015, 520, 60569, 836, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[181188, 3702, 80, 1, 72, "Input"],
Cell[181271, 3705, 556, 18, 58, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
